define([
        "dojo",
        "dijit"
    ], function(dojo, dijit) {

    return dojo.declare("support.base.NextGenUIMixin", null, {
        
        // these properties are intended to be overridden by any Dijit that
        //   includes this Mixin; they are used to define the title of the
        //   Dijit that is displayed in the Director NextGen UI title area.
        //   if non-null, the changeNextGenUITitle method of this Dijit will
        //   set the title programmatically.  if null, nothing will be done
        //   but the condition will be logged to the browser console.
        //
        // this is needed in the Director NextGen UI framework, otherwise the
        //   title of the Dijit will remain "Working..."
        cpProductName: null,
        cpPageContext: null,
        cpPageName: null,
   
        // this function actually performs the title set operation.  it must
        //   be called in the postCreate method of each Dijit that includes
        //   this Mixin
        changeNextGenUITitle: function() {
            this.inherited(arguments);

            var widgetTitle = "";
            
            // End any current polling pages on selector change
            dojo.publish("EndPollingOnSelectorChange");

            // the Configuration Patterns product name variable should be set
            //   in a subclass (namely Panel.js) but in case it was not set,
            //   reset it to a value (unfortunately untranslated) and log an
            //   error
            if(this.cpProductName == null) {
                this.cpProductName = "Configuration Patterns";
                console.log("Title for widget defined by: " + this.templateString + "; was undefined so will not be translated");
            }

            // set name (i.e. "Address Pools") of Dijit
            if(this.cpPageName != null) {
                widgetTitle = this.cpPageName;
                // if the context (i.e "Servers") of Dijit is non-null,
                //   prepend it to the title
                if(this.cpPageContext != null) {
                    widgetTitle = this.cpPageContext + " - " + widgetTitle;
                }
                
                console.log("Configuration Patterns widget title non-null; value: " + this.cpProductName + "-" + widgetTitle);

            }
            else {
                console.log("Widget defined by: " + this.templateString + " did not define a page title");
            }

            
            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                dojo.publish("updateTitle", {
                    resource : this.cpProductName,
                    view : widgetTitle,
                    loading : false
                });
                
                // publish event saying detailsSelector is no longer in charge of titles
                dojo.publish("TITLE_MASTER", false);
//                topic.publish("taskTitleChange", this.spWidgetTitle);
                                
            }            
        },
        
        // this function loads the Configure Chassis Components page in the
        //   FSM/ISC UI
        loadFSMHomePage: function() {
            // currently, hard-coding this ID is the best way to proceed
            launchInContext("com.ibm.itme.console.configChassisCompStep.navigationElement");
        },
        
        // this function reloads the work area of the NGUI with the specified
        //   Dojo widget.  the specified widget must be a path to that Widget,
        //   for example: support/panels/ServerProfileTable.  the second
        //   parameter is optional but if specified must be the OID of a
        //   server resource.  the third parameter is optional but if
        //   specified will be appended to the URL as a hash parameter with
        //   the key "cpTarget"
        loadNGUIWorkAreaWithCPWidget: function(cpWidgetPath, targetMOID, targetID) {
            var newParms = this.rebuildNGUIURLParms(cpWidgetPath, targetMOID, targetID);
            // invoke the NGUI utility method to launch the widget as a new
            //   URL
            launchNextGenPage(newParms);
        },
        
        //this function loads the work area with the given taskURI in Augusta
        //  the taskId can be what ever you would like to name the task
        //  the taskURI is the widget you would like to load in the workarea
        //  the launchParams are the data that you would like to send your new widget, must be in
        //  json type format - 
        //      var launchParams = {templateIDNum: this._selectedTemplate.id , serverID: "Frankie"};
        loadAugustaWorkAreaTask: function(taskId,taskURI,launchParams){
          
            require([ "augusta/augustaUICore/core/task/taskObject","augusta/augustaUICore/core/globals/globalConstants" ], dojo.hitch(this,function(taskObject, gConstants) {

                var workAreaTask = new taskObject({
                    taskId: taskId,
                    taskURI: taskURI,
                    type: gConstants.TASK_TYPE_WORKAREA_UI,
                    launchParams: launchParams
                  });
                  workAreaTask.invokeTask();
              
            }));
            
        },        
        // this function builds a new URL parameter string to load the
        //   specified (first parameter) widget path in the NGUI work area
        rebuildNGUIURLParms: function(cpWidgetPath, targetMOID, targetID) {
            var newParms = "";
            var taskPathParmValue = "";
            var taskPathParmKeyValue = "";
            // replace the "/" characters with the HTML equivalent: "%2F"
            var newCPWidgetPath = cpWidgetPath.replace(/\//g, "%2F");
            // get the current URL string
            var currentURL = window.location.href;
            var currentParms = currentURL.substr(currentURL.indexOf("#") + 1);
            // split the parms into pieces so we can modify them more easily
            var parmArr = currentParms.split("&");
            // iterate through the parm pieces, modify the ones we need to
            //   modify, and build the new parms
            for(var i=0; i<parmArr.length; i++) {
                var parm = parmArr[i];
                if(i == 0) {
                    newParms = newParms + parm;
                }
                else if(parm.indexOf("sourceToken") != -1 && targetMOID == null) {
                    // if we're processing the sourceToken parm and the page
                    //   we want to load is non-targeted, do nothing; this
                    //   prevents the sourceToken parm from being added
                }
                else if(parm.indexOf("selectorHome") != -1 && targetMOID != null) {
                    // if we're processing the selectorHome parm and either
                    //   the page we want to load is targeted or a
                    //   configuration patterns target ID was passed, do
                    //   nothing; this prevents the selectorHome parm from
                    //   being added
                }
                else if(parm.indexOf("cpParm") != -1) {
                    // omit processing the cpParm parameter; when reloading
                    //   another page we never want to load the Deploy Server
                    //   Template dialog automatically
                }
                else if(parm.indexOf("cpTarget") != -1 && targetMOID == null) {
                    // omit processing the cpTarget parameter; if it was part
                    //   of the URL but not included as a parm to this
                    //   function, we do not want to propogate it, but if it
                    //   was specified, we add it to the URL at the end of
                    //   this function
                }
                else if(parm.indexOf("taskSelection") != -1) {
                    // replace the value of the taskSelection parm with the
                    //   new value
                    newParms = newParms + "&taskSelection=" + newCPWidgetPath;
                }
                else {
                    // store away the value of the taskPath parm
                    if(parm.indexOf("taskPath") != -1 && parm.indexOf("taskPathKey") == -1) {
                        taskPathParmValue = parm.substr(parm.indexOf("=") + 1);
                    }
                    // store away the value of the taskPathKey parm
                    if(parm.indexOf("taskPathKey") != -1) {
                        taskPathParmKeyValue = parm.substr(parm.indexOf("=") + 1);
                    }
                    newParms = newParms + "&" + parm;
                }
            }
            // if the widget to load is not specific to a server, but is
            //   specific to another configuration patterns panel, construct
            //   the URL from scratch so it does not contain a selector
            if(targetMOID == null && targetID != null) {
                // the URL needs the taskPathParmKeyValue ("support")
                //   otherwise the NGUI code (main.js) will not be able to
                //   call dojo.registerModulePath(...) on the path, and there
                //   will be 404 errors when it tries to load the widget
                //   specified by "newCPWidgetPath"
                newParms = "workAreaDijit=" + newCPWidgetPath + "&path=" + taskPathParmKeyValue + ":" + taskPathParmValue;
            }
            // if we've finished building the parms, but have not added the
            //   sourceToken parm and one is needed, add the parm  (this will
            //   happen if the Dojo widget to launch is targeted but the page
            //   it was launched from is not targeted)
            if(targetMOID != null && newParms.indexOf("sourceToken") == -1) {
                newParms = newParms + "&sourceToken=" + targetMOID;
            }
            // append the cpTarget parm if it was specified
            if(targetID != null) {
                newParms = newParms + "&cpTarget=" + targetID;
            }
            console.log("Changing URL parameters from:\n    " + currentParms + "\nto:\n    " + newParms);
            return newParms;
        },
        
        // this function parses the OIDs that are in the sourceToken portion
        //   of the URL
        parseWidgetTargetOIDs: function() {
            var serverOIDArr = null;
            var currentURL = window.location.href;
            var tokenIndex = currentURL.indexOf("sourceToken");
            if(tokenIndex != -1) {
                var urlSubStr = currentURL.substring(tokenIndex);
                serverOIDStr = urlSubStr.substring(urlSubStr.indexOf("=") + 1, urlSubStr.indexOf("&"));
                console.log("Parsed server OIDs from URL: " + serverOIDStr);
                serverOIDArr = serverOIDStr.split(",");
            }
            return serverOIDArr;
        },
        
        // this function returns the server or chassis EndPoint for the OID on
        //   the URL string, if an OID was provided
        // note that regardless of how many OIDs are present on the URL
        //   string, this function will only return the UUID for the first OID
        getEndPointFromSourceToken: function() {
            var endpoint = null;
            
            // first, get the server or chassis OID from the URL
            var endpointOID = null;
            var endpointOIDArr = this.parseWidgetTargetOIDs();
            if(endpointOIDArr != null) {
                // this method only supports a single OID
                endpointOID = endpointOIDArr[0];
            }
            
            // next, obtain the endpoint
            if(endpointOID != null) {
                // this REST API will work on both servers and chasses
                var xhrGet = dojo.xhrGet({
                    url: this.REST_PATH + "endpoint/" + endpointOID,
                    handleAs: "json",
                    sync: true,
                    preventCache: true,
                    load: function(response) {
                        endpoint = response;
                        console.log("Retrieved endpoint: " + endpoint + " from services for OID: " + endpointOID);
                    },
                    error: function(error) {
                        console.error("Failed to retrieve endpoint from services for OID: " + endpointOID + "; error: " + error);
                    }
                });
            }
            
            return endpoint;
        },
        
        // this function returns an object containing several properties of
        //   the EndPoint with the OID on the URL string, if an OID was
        //   provided
        // currently, this function only supports the following properties:
        //   - UUID
        //   - name
        //   - server profile ID
        //   - server profile name (if and only if a server profile is
        //     assigned)
        //   - chassis profile ID
        //   - chassis profile name (if and only if a chassis profile is
        //     assigned)
        // support for more properties can be added by modifications to this
        //   function as well as the method
        //   ProfilesServiceHelper.getEndPointPropsJSONForOID(...)
        // note that regardless of how many OIDs are present on the URL
        //   string, this function will only return the UUID for the first OID
        getEndPointPropertiesFromSourceToken: function() {
            var epProps = new Object();
            
            // get the endpoint object for the OID
            var endPoint = this.getEndPointFromSourceToken();
            if(endPoint && endPoint != "") {
                epProps.uuid = endPoint.uuid;
                epProps.name = endPoint.name;
                epProps.serverProfileId = endPoint.serverProfileId;
                epProps.serverProfileName = endPoint.serverProfileName;
                epProps.chassisProfileId = endPoint.chassisProfileId;
                epProps.chassisProfileName = endPoint.chassisProfileName;
                console.log("Retrieved endpoint properties from services");
                console.log("  uuid: " + epProps.uuid);
                console.log("  name: " + epProps.name);
                console.log("  serverProfileId: " + epProps.serverProfileId);
                console.log("  serverProfileName: " + epProps.serverProfileName);
                console.log("  chassisProfileId: " + epProps.chassisProfileId);
                console.log("  chassisProfileName: " + epProps.chassisProfileName);
            }
            
            return epProps;
        },
        
        // this function subscribes to an topic that is published every time a
        //   node in the details selector is selected, and if the selected
        //   node is the widget that subscribed, the specified callback
        //   function will be executed
        subscribeToDetailsSelectorChanges: function(functionToExecute) {
            // subscribe to the topic "RETARGET_INFO" which is fired every
            //   time a link in the details selector changes; we do this so
            //   that the Available Table can reload its contents and
            //   immediately show updated data
            this.subscribe("RETARGET_INFO", function(retargetInfo) {
                // get the path and name of the widget that subscribed but
                //   also convert it from x.y.z to x/y/z format
                var widgetPathAndName = this.declaredClass;
                widgetPathAndName = widgetPathAndName.replace(/\./g, "/");
                // only execute the function if the node the user selected
                //   matches the subscribing widget
                if (retargetInfo.taskSelection === widgetPathAndName) {
                    functionToExecute.call();
                }
            });
        }

    });
});