define([
        "dojo",
        "dijit"
    ], function(dojo, dijit) {

    return dojo.declare("support.base.SelfStylingDijit", null, {
        
        // this property is intended to be overridden by all subclasses of
        //   class support.panels.Panel; it is used to define the path to the
        //   specific CSS stylesheet for a specific Dijit.  if non-null, the
        //   linkInCSS method of this Dijit will link the CSS stylesheet in
        //   programatically.  if null, nothing will be done but a message
        //   will be logged to the browser console.
        //
        // this is needed in the Director NextGen UI framework, because the
        //   framework does not provide a mechanism for an included Dijit to
        //   provide the path and name of its CSS stylesheet
        spCssPathAndName: null,
        
        // this function actually performs the CSS stylesheet loading.  it is
        //   called in the postCreate method of the support.panels.Panel Dijit
        loadCssForDijit: function() {
            if(this.spCssPathAndName != null) {
                // before loading the stylesheet, first check to ensure that
                //   it hasn't already been loaded
                var alreadyLoaded = this._isCssAlreadyLoaded();
                if(!alreadyLoaded) {
                    var cssPath = require.toUrl(this.spCssPathAndName);
                    var link = dojo.create("link", {
                       href:cssPath, 
                       type:"text/css", 
                       rel:"stylesheet", 
                       // AA120929: NOTE: This only works on IE, Opera, and FF >=9.
                       // Webkit added support for onload in <LINK> in their 
                       // mainline around Feb 2012.
                       onload: dojo.hitch(this, function() { 
                          console.log("CSS loaded", cssPath); 
                          if (this.cssArrived)
                             this.cssArrived();
                       })
                    });
                    document.getElementsByTagName("head")[0].appendChild(link);
                }
            }
            else {
                console.warn("Widget defined by: " + this.templateString + " did not define a CSS path name");
            }
        },
        
        // this function checks to see if the CSS stylesheet defined by the
        //   spCssPathAndName variable has already been loaded
        _isCssAlreadyLoaded: function() {
            var returnValue = false;
            // if the CSS path name is null, don't waste time doing anything
            if(this.spCssPathAndName != null) {
                var docElement = document.getElementsByTagName("head")[0];
                if(docElement != null) {
                    var docChildren = docElement.childNodes;
                    if(docChildren != null) {
                        for(var i=0; returnValue == false && i<docChildren.length; i++) {
                            var docChild = docChildren[i];
                            if(docChild != null && docChild.nodeName == "LINK") {
                                var linkURL = docChild.getAttribute("href");
                                if(linkURL != null) {
                                    var cssPath = require.toUrl(this.spCssPathAndName);
                                    if(linkURL == cssPath) {
                                        console.info("Stylesheet: " + cssPath + " for widget defined by: " + this.templateString + " already loaded.");
                                        // this CSS stylesheet is already loaded;
                                        //   this will also break out of the for
                                        //   { } loop
                                        returnValue = true;
                                    }
                                }
                            }
                        }
                    }
                }
            }
            
            return returnValue;
        }

    });
});