define([ 
        "dojo",
        "dijit",
        "dojo/query",
        "dojo/DeferredList",
        "gridx/Grid",
        "gridx/core/model/cache/Async",
        "gridx/core/model/extensions/Query",
        "gridx/modules/CellWidget",
        "gridx/modules/Tree",
        "gridx/modules/VirtualVScroller",
        "gridx/modules/ColumnResizer",
        "gridx/modules/ColumnWidth",
        "gridx/modules/SingleSort",
        "gridx/modules/select/Row",
        "gridx/modules/extendedSelect/Row",
        "gridx/modules/select/Column",
        "gridx/modules/IndirectSelect",
        "gridx/modules/RowHeader",
        "gridx/modules/Filter",
        "gridx/modules/ToolBar",
        "gridx/modules/filter/QuickFilter"
    ], function(dojo, dijit) {

    return dojo.declare("support.base.TableMixin", null, {        
        
        //Returns current table grid for CP        
        getTable: function(store, structure, noDataMessage, gridID, /*boolean?*/hideFilter, /*boolean?*/treeMode) {
            
            (!hideFilter) && (hideFilter = false);  // hide new table's filter controls?
            (!treeMode) && (treeMode = false);  // show new table's rows in tree mode?
            
            var table = new gridx.Grid({
                id: gridID + "_" + this.id,
                autoHeight: false,
                autoWidth: false,
                autoResize: true,
                cacheClass: "gridx/core/model/cache/Async",
                emptyInfo: noDataMessage,
                store: store,
                structure: structure,
                modules: [
                    "gridx/modules/CellWidget",
                    "gridx/modules/Tree",
                    { moduleClass: "gridx/modules/VirtualVScroller" , lazy: true, lazyTimeout: 200},                   
                    { moduleClass: "gridx/modules/extendedSelect/Row", triggerOnCell: true, treeMode: treeMode },
                    "gridx/modules/ColumnResizer",
                    "gridx/modules/select/Column",
                    "gridx/modules/SingleSort",
                    "gridx/modules/ColumnWidth",
                    "gridx/modules/Filter",
                    "gridx/modules/filter/QuickFilter",
                    "gridx/modules/ToolBar",
                    "gridx/modules/IndirectSelect",
                    "gridx/modules/RowHeader"
                ]
            });
            
            dojo.mixin(table, {
                _expandedItemIds: [],
                reexpandRows: dojo.hitch(table, function() {
                    // Expand all rows that were previously expanded in this table
                    for (var idx in this._expandedItemIds) {
                        this.tree.expand(this._expandedItemIds[idx]);
                    }
                }),
                collapseAll: dojo.hitch(table, function() {
                    // Collapse all rows in this table
                    for (var idx in this.store._arrayOfTopLevelItems) {
                        var identity = this.store.getIdentity(this.store._arrayOfTopLevelItems[idx]);
                        this.tree.collapseRecursive(identity);
                    }
                }),
                expandAll: dojo.hitch(table, function() {
                    // Expand all rows in this table
                    for (var idx in this.store._arrayOfTopLevelItems) {
                        var identity = this.store.getIdentity(this.store._arrayOfTopLevelItems[idx]);
                        this.tree.expandRecursive(identity);
                    }
                }),
                disableParentRows: dojo.hitch(table, function() {
                    // Disable selection of all top level rows
                    var topIds = new Array();
                    for (var idx in this.store._arrayOfTopLevelItems) {
                        var identity = this.store.getIdentity(this.store._arrayOfTopLevelItems[idx]);
                        topIds.push("\"" + identity + "\": true");
                    }
                    (topIds.length > 0) && (this.select.row.unselectable = dojo.fromJson("{" + topIds.join() + "}"));
                }),
                hideFilter: dojo.hitch(table, function() {
                    // Hide this table's filter controls
                    dojo.query(".gridxBarQuickFilter", this.domNode).style("display", "none");
                }),
                showFilter: dojo.hitch(table, function() {
                    // Reveal this table's filter controls
                    dojo.query(".gridxBarQuickFilter", this.domNode).style("display", "");
                })
            });
            
            // Connect handler to hide filter on initial table load
            (hideFilter) && dojo.connect(table, "onModulesLoaded", table.hideFilter);
            
            // Connect handler to column header node to collapse all before sorting
            dojo.connect(table.header.innerNode, "click", table.collapseAll);
            
            // Connect handlers to record expand/collapse states, to be re-applied when
            // table store is refreshed and/or replaced.
            dojo.connect(table.tree, "onExpand", table, function(rowId) {
                // Verify its possible to expand the row since "Expand All" function attempts to expand rows without children!
                if (this.tree.canExpand(rowId)) {
                    (dojo.indexOf(this._expandedItemIds, rowId) < 0) && this._expandedItemIds.push(rowId);
                }
            });
            dojo.connect(table.tree, "onCollapse", table, function(rowId) {
                var idx = dojo.indexOf(this._expandedItemIds, rowId);
                (idx >= 0) && this._expandedItemIds.splice(idx, 1);
            });
                        
            return table;
        }
        
    });
});