define([
        "dojo",
        "dijit",
        "support/dialogs/Dialog",
        "dojo/i18n!./nls/AddIOAdapter",
        "dojo/text!./templates/AddIOAdapter.html",
        "dojo/parser",
        "dijit/Tooltip",
        "support/widgets/Select",
        "support/widgets/OutlinePane",
        "dojox/grid/DataGrid",
        "dojox/data/AndOrReadStore",
        "dijit/layout/ContentPane",
        "support/widgets/ProgressButton",
        "dijit/TitlePane"
    ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.AddIOAdapter", [ support.dialogs.Dialog ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
             
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        baseClass: "AddIOAdapter",

        addType: "*", // all types
        nodeType: null,
        formFactor: null,
        cardNumber: null,
        dialogMode: "add",  // add or assign
        adapterType: null,
        adapterId: null, 
        adapterPatternId: null,
        initialPortPatternId1: null,
        initialPortPatternId2: null,
        initialPortExtendedPatternId1: null,
        initialPortExtendedPatternId2: null,
        numRows: null,
        
        _release: "GA1",   // No Port Physical Function templates for GA1

        _defaultselectorwidth: "400px",
        _defaultAdapterTemplateSelector: null,
        _defaultPortVirtualizationTemplateSelector: null,
        _defaultPortExtendedSettingsTemplateSelector: null,
        _defaultEthernetPortTemplateSelector: null,
        _defaultFibreChanneloverEthernetTemplateSelector: null,
        _defaultiSCSIFunctionTemplateSelector: null,
        _defaultFibreChannelPortTemplateSelector: null,
        _defaultPortVirtualizationTemplateSelector2: null,
        _defaultEthernetPortTemplateSelector2: null,
        _defaultFibreChanneloverEthernetTemplateSelector2: null,
        _defaultiSCSIFunctionTemplateSelector2: null,
        _defaultFibreChannelPortTemplateSelector2: null,
        
        _defaultPortVirtualizationSelectorArray: null,
        _adapterTable: null,
        _adapterModel: null,
        _adapterLayout: null,
        _adapterType: null,
        _adapterId: null,
        _controllerVendor: "*",   // any vendor
        _controllerChipset: "*",  // any chipset
        _protocol: null,
        _defaultAdapterTemplateStore: null,
        _defaultAdapterTemplateData: null,
        _defaultPortVirtualizationTemplateStore: null,
        _defaultPortPFTemplateStore: null,
        _defaultPortExtendedSettingsTemplateStore: null,
        _defaultEthernetAddressPoolStore: null,
        _defaultFCAddressPoolStore: null,
        _supportedIOAdaptersList: null,
        _numberOfControllers: null,
        _adapterPortConfigMode: null,
        _adapterPortConfigOptionCode: null,        
        _adapterControllerObj: null,
        
        _rowAttachPointList: null,
        _pendingCreateTemplateItems: null,
        
        //These are based on the values specified in the new adapter template
        //_adapterMode

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/AddIOAdapter.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);

            dojo.connect(this.dialogNode, "hide", this, function() {        
                this.destroy();
            });

            this._connectDataModels();
            
            var portVirtualizationAttachPointList = new Array();
            var portExtendedSettingsAttachPointList = new Array();
            var ethernetAttachPointList = new Array();
            var fcoeAttachPointList = new Array();
            var iscsiAttachPointList = new Array();
            var fcAttachPointList = new Array();
            var attachPointList = new Object();
            
            portVirtualizationAttachPointList.push(this._portVirtualizationRow);
            portVirtualizationAttachPointList.push(this._portVirtualizationRow2);
            portExtendedSettingsAttachPointList.push(this._portExtendedSettingsRow);
            portExtendedSettingsAttachPointList.push(this._portExtendedSettingsRow2);
            ethernetAttachPointList.push(this._ethernetPortRow);
            ethernetAttachPointList.push(this._ethernetPortRow2);
            fcoeAttachPointList.push(this._fibreChanneloverEthernetRow);
            fcoeAttachPointList.push(this._fibreChanneloverEthernetRow2);
            iscsiAttachPointList.push(this._iSCSIFunctionRow);
            iscsiAttachPointList.push(this._iSCSIFunctionRow2);
            fcAttachPointList.push(this._fibreChannelPortRow);
            fcAttachPointList.push(this._fibreChannelPortRow2);
            
            attachPointList.portVirtualizationList = portVirtualizationAttachPointList;
            attachPointList.portExtendedSettingsList = portExtendedSettingsAttachPointList;
            attachPointList.ethernetList = ethernetAttachPointList;
            attachPointList.fcoeList = fcoeAttachPointList;
            attachPointList.iscsiList = iscsiAttachPointList;
            attachPointList.fcList = fcAttachPointList;
            
            this._rowAttachPointList = attachPointList;
            
            if (this.dialogMode == "add") {
                this._initAdapLabel.style.display = "block";
                switch (this.addType) {
                    case "lom":
                        dojo.style(this._selectDefaultHeader, "display", "");
                        if (this.numRows == 1) {
                            this._adapterSelectMode(true);
                        } else {
                            this._adapterSelectMode(false);
                        }
                        if (this.formFactor == "1sd") {
                            this.set("title", this.locResources.LOM);
                        } else {
                            this.set("title", this.locResources.IOORLOM.replace("%1", ((this.cardNumber) ? this.cardNumber : "")));
                        }
                        break;
                    default:
                        dojo.style(this._selectDefaultHeader, "display", "");
                        if (this.numRows == 1) {
                            this._adapterSelectMode(true);
                        } else {
                            this._adapterSelectMode(false);
                        }
                        if (this.formFactor == "1sd") {
                            this.set("title", this.locResources.IO.replace("%1", ""));
                        } else {
                            this.set("title", this.locResources.IO.replace("%1", ((this.cardNumber) ? this.cardNumber : "")));
                        }
                        break;
                }
            } else if (this.dialogMode == "assign") {
                this._selectAdapterTypeSection.style.display = "none";
                this.set("title", this.locResources.ASSIGNPATTERN);
                this.okButton.set("label", this.resources.ASSIGN);
                this._adapLabel.style.display = "block";
                this._adapterType = this.adapterType;
                this._adapterId = this.adapterId;
                this._createSelectBoxes();
                this._setGlobalValues();
                this._checkSelectorDisplayRules();
                this._defaultAdapterTemplateSelector.set("value", this.adapterPatternId);
                this._defaultPortVirtualizationTemplateSelector.set("value", this.initialPortPatternId1);
                this._defaultPortExtendedSettingsTemplateSelector.set("value", this.initialPortExtendedPatternId1);
                if (this._numberOfControllers > 1) {
                    this._defaultPortVirtualizationTemplateSelector2.set("value", this.initialPortPatternId2);
                    this._defaultPortExtendedSettingsTemplateSelector2.set("value", this.initialPortExtendedPatternId2);
                }
            }
            
            this.subscribe("CreateAdapterTemplateComplete", function(data) {                
                var adapterTemplateData = null;
                var xhrAdapTemp = {
                    url: this.REST_PATH + "template/list/adapter",                    
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        adapterTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                };

                // Call the asynchronous xhrGet
                dojo.xhrGet(xhrAdapTemp);
                
                this._defaultAdapterTemplateData = adapterTemplateData;
                                
                
                this._defaultAdapterTemplateStore = new dojox.data.AndOrReadStore({
                    data: adapterTemplateData                
                });
                
                this._defaultAdapterTemplateSelector.setStore(this._defaultAdapterTemplateStore, null, {query: {
                    // Only templates for specified adapter.
                    complexQuery: "adapterType: '" + this._adapterId + "'"
                }});
                                
                this._defaultAdapterTemplateSelector.set("value", data.id);
            });
            
            this.subscribe("EditAdapterTemplateComplete", function(data) {                
                var adapterTemplateData = null;
                var xhrAdapTemp = {
                    url: this.REST_PATH + "template/list/adapter",                    
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        adapterTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                };

                // Call the asynchronous xhrGet
                dojo.xhrGet(xhrAdapTemp);
                
                this._defaultAdapterTemplateData = adapterTemplateData;
                                
                
                this._defaultAdapterTemplateStore = new dojox.data.AndOrReadStore({
                    data: adapterTemplateData                
                });
                
                this._defaultAdapterTemplateSelector.setStore(this._defaultAdapterTemplateStore, null, {query: {
                    // Only templates for specified adapter.
                    complexQuery: "adapterType: '" + this._adapterId + "'"
                }});
                
                this._defaultAdapterTemplateSelector.set("value", data.id);
            });            
            
            this.subscribe("CreatePortPhysicalFunctionTemplateComplete", function(data) {
                var ppfunctionTemplateData = null;
                var xhrAdapTemp = {
                    url: this.REST_PATH + "template/list/ppfunction",                    
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        ppfunctionTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                };

                // Call the asynchronous xhrGet
                dojo.xhrGet(xhrAdapTemp);

                this._defaultPortPFTemplateStore = new dojox.data.AndOrReadStore({
                    data: ppfunctionTemplateData                
                });
                
                if (this._pendingCreateTemplateItems == "ppfNic") {                     
                    this._defaultEthernetPortTemplateSelector.setStore(this._defaultPortPFTemplateStore, null, {query: {
                    // Only templates for specified adapter.
                        complexQuery: "physicalFunction: 'nic' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"                        
                    }});
                    this._defaultEthernetPortTemplateSelector.set("value", data.id);
                }
                if (this._pendingCreateTemplateItems == "ppfNic2") { 
                    this._defaultEthernetPortTemplateSelector2.setStore(this._defaultPortPFTemplateStore, null, {query: {
                        // Only templates for specified adapter.
                        complexQuery: "physicalFunction: 'nic' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"                    
                    }});                   
                    this._defaultEthernetPortTemplateSelector2.set("value", data.id);
                }
                if (this._pendingCreateTemplateItems == "ppfFcoe") { 
                    this._defaultFibreChanneloverEthernetTemplateSelector.setStore(this._defaultPortPFTemplateStore, null, {query: {
                        // Only templates for specified adapter.
                        complexQuery: "physicalFunction: 'fcoe' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"                            
                    }});             
                    this._defaultFibreChanneloverEthernetTemplateSelector.set("value", data.id);
                }
                if (this._pendingCreateTemplateItems == "ppfFcoe2") { 
                    this._defaultFibreChanneloverEthernetTemplateSelector2.setStore(this._defaultPortPFTemplateStore, null, {query: {
                        // Only templates for specified adapter.
                        complexQuery: "physicalFunction: 'fcoe' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"                            
                    }});
                    this._defaultFibreChanneloverEthernetTemplateSelector2.set("value", data.id);
                }
                if (this._pendingCreateTemplateItems == "ppfiscsi") { 
                    this._defaultiSCSIFunctionTemplateSelector.setStore(this._defaultPortPFTemplateStore, null, {query: {
                        // Only templates for specified adapter.                            
                        complexQuery: "physicalFunction: 'iscsi' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"
                    }});           
                    this._defaultiSCSIFunctionTemplateSelector.set("value", data.id);
                }
                if (this._pendingCreateTemplateItems == "ppfiscsi2") { 
                    this._defaultiSCSIFunctionTemplateSelector2.setStore(this._defaultPortPFTemplateStore, null, {query: {
                        // Only templates for specified adapter.
                        complexQuery: "physicalFunction: 'iscsi' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"                                                    
                    }});                    
                    this._defaultiSCSIFunctionTemplateSelector2.set("value", data.id);
                }
                if (this._pendingCreateTemplateItems == "ppfFc") { 
                    this._defaultFibreChannelPortTemplateSelector.setStore(this._defaultPortPFTemplateStore, null, {query: {
                        // Only templates for specified adapter.
                        complexQuery: "physicalFunction: 'fc' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"                        
                    }});                    
                    this._defaultFibreChannelPortTemplateSelector.set("value", data.id);
                }
                if (this._pendingCreateTemplateItems == "ppfFc2") { 
                    this._defaultFibreChannelPortTemplateSelector2.setStore(this._defaultPortPFTemplateStore, null, {query: {
                        // Only templates for specified adapter.
                        complexQuery: "physicalFunction: 'fc' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"                            
                    }});
                    this._defaultFibreChannelPortTemplateSelector2.set("value", data.id);
                }                               
            });            
            
            this.subscribe("CreatePortVirtualizationTemplateComplete", function(data) {
                var portvirtTemplateData = null;
                var xhrPortVirtTemp = {
                    url: this.REST_PATH + "template/list/portvirtual",                    
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        portvirtTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                };

                // Call the asynchronous xhrGet
                dojo.xhrGet(xhrPortVirtTemp);
                
                this._defaultPortVirtualizationTemplateStore = new dojox.data.AndOrReadStore({
                    data: portvirtTemplateData                
                });
                
                if (this._pendingCreateTemplateItems == "portvirt1") {
                    var portCfgMode = (this._adapterPortConfigMode[0]) ?  this._adapterPortConfigMode[0]: "";
                    if (portCfgMode && (typeof(portCfgMode) != "string")) {
                        portCfgMode = portCfgMode[0];
                    }
                    this._defaultPortVirtualizationTemplateSelector.setStore(this._defaultPortVirtualizationTemplateStore, null, {query: {
                        // Filter by mode and option
                        complexQuery: "(adapterType: '" + this._adapterId + "' OR adapterType: '*ANY') " +
                                        "AND portCfgMode: '" + portCfgMode + "' AND portCfgOptionCode: '" + this._adapterPortConfigOptionCode[0] + "'"
                    }});
                    
                    if (this._numberOfControllers > 1) {
                        var curValue = this._defaultPortVirtualizationTemplateSelector2.get("value");
                        this._defaultPortVirtualizationTemplateSelector2.setStore(this._defaultPortVirtualizationTemplateStore, curValue, {query: {
                            complexQuery: "(adapterType: '" + this._adapterId + "' OR adapterType: '*ANY') " +
                                        "AND portCfgMode: '" + portCfgMode + "' AND portCfgOptionCode: '" + this._adapterPortConfigOptionCode[1] + "'"
                        }});
                    }
                
                    this._defaultPortVirtualizationTemplateSelector.set("value", data.id);
                }
                if (this._pendingCreateTemplateItems == "portvirt2") {
                    this._defaultPortVirtualizationTemplateSelector2.setStore(this._defaultPortVirtualizationTemplateStore, null, {query: {
                        // Filter by mode and option
                        complexQuery: "(adapterType: '" + this._adapterId + "' OR adapterType: '*ANY') " +
                                        "AND portCfgMode: '" + this._adapterPortConfigMode[1] + "' AND portCfgOptionCode: '" + this._adapterPortConfigOptionCode[1] + "'"
                    }});
                    
                    var curValue = this._defaultPortVirtualizationTemplateSelector.get("value");
                    this._defaultPortVirtualizationTemplateSelector.setStore(this._defaultPortVirtualizationTemplateStore, curValue, {query: {
                        complexQuery: "(adapterType: '" + this._adapterId + "' OR adapterType: '*ANY') " +
                                        "AND portCfgMode: '" + this._adapterPortConfigMode[0] + "' AND portCfgOptionCode: '" + this._adapterPortConfigOptionCode[0] + "'"
                    }});
                
                    this._defaultPortVirtualizationTemplateSelector2.set("value", data.id);
                }
            });
            
            this.subscribe("ExPortTemplateComplete", function(data) {                
                var portExtendedSettingsTemplateData = null;
                var xhrPortExtTemp = {
                        url: this.REST_PATH + "template/list/ExtendedPort?Props=vendor,chipset,protocol",
                        handleAs: "json",
                        sync: true,
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        load: function(data) {
                            portExtendedSettingsTemplateData = data;
                        },
                        error: function(error) {
                            alert(error);
                        }
                };

                dojo.xhrGet(xhrPortExtTemp);
                
                this._defaultPortExtendedSettingsTemplateStore = new dojox.data.AndOrReadStore({
                    data: portExtendedSettingsTemplateData
                });                
                
                if (this._pendingCreateTemplateItems == "exPort1") {                
                    this._defaultPortExtendedSettingsTemplateSelector.setStore(this._defaultPortExtendedSettingsTemplateStore, null, {
                        query: {
                            complexQuery: "vendor: '" + this._controllerVendor[0][0] + "' AND chipset: '" + this._controllerChipset[0][0] + "' AND protocol: '" + this._protocol[0] + "'"
                        }
                    });
                    this._defaultPortExtendedSettingsTemplateSelector.addOption({
                        value: "*NONE",
                        label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
                    });
                    this._defaultPortExtendedSettingsTemplateSelector.set("value", data);
                    
                    if (this._numberOfControllers > 1) {
                        var curValue = this._defaultPortExtendedSettingsTemplateSelector2.get("value");
                        this._defaultPortExtendedSettingsTemplateSelector2.setStore(this._defaultPortExtendedSettingsTemplateStore, null, {
                            query: {
                                complexQuery: "vendor: '" + this._controllerVendor[1][0] + "' AND chipset: '" + this._controllerChipset[1][0] + "' AND protocol: '" + this._protocol[1] + "'"
                            }
                        });
                        this._defaultPortExtendedSettingsTemplateSelector2.addOption({
                            value: "*NONE",
                            label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
                        });
                        this._defaultPortExtendedSettingsTemplateSelector2.set("value", curValue);
                    }
                }
                if (this._pendingCreateTemplateItems == "exPort2") {
                    var curValue = this._defaultPortExtendedSettingsTemplateSelector.get("value");
                    this._defaultPortExtendedSettingsTemplateSelector.setStore(this._defaultPortExtendedSettingsTemplateStore, null, {
                        query: {
                            complexQuery: "vendor: '" + this._controllerVendor[0][0] + "' AND chipset: '" + this._controllerChipset[0][0] + "' AND protocol: '" + this._protocol[0] + "'"
                        }
                    });
                    this._defaultPortExtendedSettingsTemplateSelector.addOption({
                        value: "*NONE",
                        label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
                    });
                    this._defaultPortExtendedSettingsTemplateSelector2.setStore(this._defaultPortExtendedSettingsTemplateStore, null, {
                        query: {
                            complexQuery: "vendor: '" + this._controllerVendor[1][0] + "' AND chipset: '" + this._controllerChipset[1][0] + "' AND protocol: '" + this._protocol[1] + "'"
                        }
                    });
                    this._defaultPortExtendedSettingsTemplateSelector2.addOption({
                        value: "*NONE",
                        label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
                    });
                    this._defaultPortExtendedSettingsTemplateSelector.set("value", curValue);
                    this._defaultPortExtendedSettingsTemplateSelector2.set("value", data);
                }
            });            
        },

        _createSelectBoxes: function() {
            this._defaultPortVirtualizationSelectorArray = new Array();
            
            this._createDefaultAdapterTemplateSelector();
            
            this._setGlobalValues();
            
            for (var portGrpNum=1;portGrpNum<=this._numberOfControllers;++portGrpNum) {
                this._createConfigPortGroupSelectors(portGrpNum);                
            }
            
            this._adapterPatternRefresh();
        },
        
        _adapterPatternRefresh: function() {
            this._setGlobalValues();                    
            this._checkSelectorDisplayRules();
            var adapterPortConfigMode1 = (this._adapterPortConfigMode[0]) ?  this._adapterPortConfigMode[0]: "";
            var adapterPortConfigMode2 = (this._adapterPortConfigMode[1]) ?  this._adapterPortConfigMode[1]: "";
            var adapterPortConfigOptionCode1 = this._adapterPortConfigOptionCode;
            var adapterPortConfigOptionCode2 = adapterPortConfigOptionCode1;
            
            if (typeof(adapterPortConfigMode1) != "string") {                                                   
                if (typeof(adapterPortConfigMode1[0]) == "string") {
                    adapterPortConfigMode1 = adapterPortConfigMode1[0];
                } else {
                    adapterPortConfigMode1 = adapterPortConfigMode1[0][0];
                }
            }                       
            
            if (typeof(adapterPortConfigOptionCode1) != "string") {
                if (typeof(adapterPortConfigOptionCode1[0]) == "string") {
                    adapterPortConfigOptionCode1 = adapterPortConfigOptionCode1[0];
                } else {
                    adapterPortConfigOptionCode1 = adapterPortConfigOptionCode1[0][0];
                }
            }
            
            var curVal = this._defaultPortVirtualizationTemplateSelector.get("value");
            (curVal == "") && (this.dialogMode == "assign") && (curVal = this.initialPortPatternId1);
            this._defaultPortVirtualizationTemplateSelector.setStore(this._defaultPortVirtualizationTemplateStore, curVal, {query: {
                // Filter by mode and option
                complexQuery: "(adapterType: '" + this._adapterId + "' OR adapterType: '*ANY') " +
                                "AND portCfgMode: '" + adapterPortConfigMode1 + "' AND portCfgOptionCode: '" + adapterPortConfigOptionCode1 + "'"
            }});
            
            curVal = this._defaultPortExtendedSettingsTemplateSelector.get("value");
            (curVal == "") && (this.dialogMode == "assign") && (curVal = this.initialPortExtendedPatternId1);
            this._defaultPortExtendedSettingsTemplateSelector.setStore(this._defaultPortExtendedSettingsTemplateStore, curVal, {query: {
                complexQuery: "vendor: '" + this._controllerVendor[0][0] + "' AND chipset: '" + this._controllerChipset[0][0] + "' AND protocol: '" + this._protocol[0] + "'" 
            }});

            curVal = this._defaultPortExtendedSettingsTemplateSelector.get("value");
            if (curVal != "") {
                // Add "*NONE" option ...
                this._defaultPortExtendedSettingsTemplateSelector.addOption({
                    value: "*NONE",
                    label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
                });
                // If not in "assign" mode, or if initial value didn't take, then initialize to "*NONE".
                if ((this.dialogMode != "assign") || (curVal != this.initialPortExtendedPatternId1)) {
                    this._defaultPortExtendedSettingsTemplateSelector.set("value", "*NONE");
                }
            }

            if (this._numberOfControllers == 2) {
                if (typeof(adapterPortConfigMode2) != "string") {
                    if (typeof(adapterPortConfigMode2[1]) == "string") {
                        adapterPortConfigMode2 = adapterPortConfigMode2[0];
                    } else {
                        adapterPortConfigMode2 = adapterPortConfigMode2[0][0];
                    }
                }
                
                if (typeof(adapterPortConfigOptionCode2) != "string") {
                    if (typeof(adapterPortConfigOptionCode2[1]) == "string") {
                        adapterPortConfigOptionCode2 = adapterPortConfigOptionCode2[1];
                    } else {
                        adapterPortConfigOptionCode2 = adapterPortConfigOptionCode2[1][0];
                    }
                }
                
                var curVal2 = this._defaultPortVirtualizationTemplateSelector2.get("value");
                (curVal2 == "") && (this.dialogMode == "assign") && (curVal2 = this.initialPortPatternId2);
                this._defaultPortVirtualizationTemplateSelector2.setStore(this._defaultPortVirtualizationTemplateStore, curVal2, {query: {
                    // Filter by mode and option
                    complexQuery: "(adapterType: '" + this._adapterId + "' OR adapterType: '*ANY') " +
                                    "AND portCfgMode: '" + adapterPortConfigMode2 + "' AND portCfgOptionCode: '" + adapterPortConfigOptionCode2 + "'"
                }});

                curVal2 = this._defaultPortExtendedSettingsTemplateSelector2.get("value");
                (curVal2 == "") && (this.dialogMode == "assign") && (curVal2 = this.initialPortExtendedPatternId2);
                this._defaultPortExtendedSettingsTemplateSelector2.setStore(this._defaultPortExtendedSettingsTemplateStore, curVal2, {query: {
                    complexQuery: "vendor: '" + this._controllerVendor[1][0] + "' AND chipset: '" + this._controllerChipset[1][0] + "' AND protocol: '" + this._protocol[1] + "'" 
                }});

                curVal2 = this._defaultPortExtendedSettingsTemplateSelector2.get("value");
                if (curVal2 != "") {
                    // Add "*NONE" option ...
                    this._defaultPortExtendedSettingsTemplateSelector2.addOption({
                        value: "*NONE",
                        label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
                    });
                    // If not in "assign" mode, or if initial value didn't take, then initialize to "*NONE".
                    if ((this.dialogMode != "assign") || (curVal2 != this.initialPortExtendedPatternId2)) {
                        this._defaultPortExtendedSettingsTemplateSelector2.set("value", "*NONE");
                    }
                }
                
                this._validateSelectors();
            }

            if (this._defaultAdapterTemplateSelector.get("value") == "") {                    
                this._adapterError.style.display = "block";
                this._defaultAdapterTemplateSelector.set("disabled", true);
                dojo.replaceClass(this._defaultAdapterTemplateEditIcon, "spEditInactiveIcon_v2", "spEditIcon_v2");                   
            } else {                    
                this._adapterError.style.display = "none";
                this._defaultAdapterTemplateSelector.set("disabled", false);
                dojo.replaceClass(this._defaultAdapterTemplateEditIcon, "spEditIcon_v2", "spEditInactiveIcon_v2");                                                            
            }            
        },
        
        _createDefaultAdapterTemplateSelector: function() {
            if (this._defaultAdapterTemplateSelector == null) {
                this._defaultAdapterTemplateSelector = new support.widgets.Select({
                    id: this.get("id") + "_adapTemplateSelector",
                    name: "defaultAdapterTemplate",
                    store: this._defaultAdapterTemplateStore,
                    query: {
                        // Only templates for specified adapter.
                        complexQuery: "adapterType: '" + this._adapterId + "'"
                    },                                        
                    maxHeight: -1,
                    emptyLabel: this.resources.NO_TEMP_DEF_HYPHENS,
                    style: {
                        width: this._defaultselectorwidth
                    },
                    disabled: false
                }).placeAt(this._defaultAdapterTemplate);
                
                dojo.connect(this._defaultAdapterTemplateSelector, "onChange", this, this._adapterPatternRefresh);
                
                this._defaultAdapterTemplateSelector.startup();
            }
        },

        _validatePortPatternSelector: function(selector, errorIcon, editIcon) {
            if (selector.get("value") == "") {
                errorIcon.style.display = "block";
                selector.set("disabled", true);
                dojo.replaceClass(editIcon, "spEditInactiveIcon_v2", "spEditIcon_v2");
            } else {
                errorIcon.style.display = "none";
                selector.set("disabled", false);
                dojo.replaceClass(editIcon, "spEditIcon_v2", "spEditInactiveIcon_v2");
            }
        },
        
        _createPortPatternSelector: function(portGrp) {
            if (portGrp == 1) {
                this._defaultPortVirtualizationTemplateSelector = new support.widgets.Select({
                    id: this.get("id") + "_portVirtualizationTemplateSelector",
                    name: "defaultPortVirtualizationTemplate",
                    //store: this._defaultPortVirtualizationTemplateStore,
                    //query: {
                        // Filter by controller's vendor & chipset.
                    //    complexQuery: "adapterType: '" + this.adapterId + "'" //"vendor: '" + this._controllerVendor + "' AND chipset: '" + this._controllerChipset + "'"
                    //},                                      
                    maxHeight: -1,
                    emptyLabel: this.resources.NO_TEMP_DEF_HYPHENS,
                    style: {
                        width: this._defaultselectorwidth
                    },
                    disabled: false
                }).placeAt(this._defaultPortVirtualizationTemplate);
                
                dojo.connect(this._defaultPortVirtualizationTemplateSelector, "startup", this, function(evt) {
                    this._validatePortPatternSelector(this._defaultPortVirtualizationTemplateSelector, this._portVirtError, this._portVirtEdit);
                });
                
                dojo.connect(this._defaultPortVirtualizationTemplateSelector, "onChange", this, function() {
                    this._validatePortPatternSelector(this._defaultPortVirtualizationTemplateSelector, this._portVirtError, this._portVirtEdit);
                    this._validateSelectors();
                });
                
                dojo.connect(this._defaultPortVirtualizationTemplateSelector, "onSetStore", this, function() {
                    this._validatePortPatternSelector(this._defaultPortVirtualizationTemplateSelector, this._portVirtError, this._portVirtEdit);
                    this._validateSelectors();
                });
                
                // Connect the function to startup here since the port pattern selector must be instantiated first
                dojo.connect(this._defaultAdapterTemplateSelector, "startup", this, function() {
                    this._adapterConnectFunction();
                });
                
                this._defaultPortVirtualizationTemplateSelector.startup();
            }
            if (portGrp == 2) {
                this._defaultPortVirtualizationTemplateSelector2 = new support.widgets.Select({
                    id: this.get("id") + "_portVirtualizationTemplateSelector2",
                    name: "defaultPortVirtualizationTemplate2",
                    maxHeight: -1,
                    emptyLabel: this.resources.NO_TEMP_DEF_HYPHENS,
                    style: {
                        width: this._defaultselectorwidth
                    },
                    disabled: false
                }).placeAt(this._defaultPortVirtualizationTemplate2);

                dojo.connect(this._defaultPortVirtualizationTemplateSelector2, "startup", this, function() {
                    this._validatePortPatternSelector(this._defaultPortVirtualizationTemplateSelector2, this._portVirtError2, this._portVirtEdit2);
                });                
                
                dojo.connect(this._defaultPortVirtualizationTemplateSelector2, "onChange", this, function() {
                    this._validatePortPatternSelector(this._defaultPortVirtualizationTemplateSelector2, this._portVirtError2, this._portVirtEdit2);
                    this._validateSelectors();
                });

                dojo.connect(this._defaultPortVirtualizationTemplateSelector2, "onSetStore", this, function() {
                    this._validatePortPatternSelector(this._defaultPortVirtualizationTemplateSelector2, this._portVirtError2, this._portVirtEdit2);
                    this._validateSelectors();
                });
                
                this._defaultPortVirtualizationTemplateSelector2.startup();
            }
        },
        
        _createPortExtendedSettingsPatternSelector: function(portGrp) {
            if (portGrp == 1) {
                this._defaultPortExtendedSettingsTemplateSelector = new support.widgets.Select({
                    id: this.get("id") + "_portExtendedSettingsTemplateSelector",
                    name: "defaultPortExtendedSettingsTemplate",
                    store: this._defaultPortExtendedSettingsTemplateStore,
                    query: {
                        complexQuery: "vendor: '" + this._controllerVendor[0][0] + "' AND chipset: '" + this._controllerChipset[0][0] + "' AND protocol: '" + this._protocol[0] + "'"
                    },
                    maxHeight: -1,
                    emptyLabel: this.resources.NO_TEMP_DEF_HYPHENS,
                    style: {
                        width: this._defaultselectorwidth
                    },
                    disabled: false
                }).placeAt(this._defaultPortExtendedSettingsTemplate);
                this._defaultPortExtendedSettingsTemplateSelector.startup();

                (this.dialogMode == "assign") && (this._defaultPortExtendedSettingsTemplateSelector.set("value", this.initialPortExtendedPatternId1));
                var curVal = this._defaultPortExtendedSettingsTemplateSelector.get("value");
                if (curVal != "") {
                    // Add "*NONE" option ...
                    this._defaultPortExtendedSettingsTemplateSelector.addOption({
                        value: "*NONE",
                        label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
                    });
                    // If not in "assign" mode, or if initial value didn't take, then initialize to "*NONE".
                    if ((this.dialogMode != "assign") || (curVal != this.initialPortExtendedPatternId1)) {
                        this._defaultPortExtendedSettingsTemplateSelector.set("value", "*NONE");
                    }
                }
                
                dojo.connect(this._defaultPortExtendedSettingsTemplateSelector, "onChange", this, function() {            
                    if (this._defaultPortExtendedSettingsTemplateSelector.get("value") == "") {
                        this._defaultPortExtendedSettingsTemplateSelector.set("disabled", true);
                        dojo.replaceClass(this._portExtendedEditIcon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                    } else {
                        if (this._defaultPortExtendedSettingsTemplateSelector.get("value") == "*NONE") {
                            dojo.replaceClass(this._portExtendedEditIcon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                        } else {
                            dojo.replaceClass(this._portExtendedEditIcon, "spEditIcon_v2", "spEditInactiveIcon_v2");    
                        }
                        this._defaultPortExtendedSettingsTemplateSelector.set("disabled", false);
                    }
                    
                    this._validateSelectors();
                });
                
                if (this._defaultPortExtendedSettingsTemplateSelector.get("value") == "") {
                    this._defaultPortExtendedSettingsTemplateSelector.set("disabled", true);
                    dojo.replaceClass(this._portExtendedEditIcon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                } else {
                    if (this._defaultPortExtendedSettingsTemplateSelector.get("value") == "*NONE") {
                        dojo.replaceClass(this._portExtendedEditIcon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                    } else {
                        dojo.replaceClass(this._portExtendedEditIcon, "spEditIcon_v2", "spEditInactiveIcon_v2");    
                    }
                    this._defaultPortExtendedSettingsTemplateSelector.set("disabled", false);                
                }
            }
            if (portGrp == 2) {
                this._defaultPortExtendedSettingsTemplateSelector2 = new support.widgets.Select({
                    id: this.get("id") + "_portExtendedSettingsTemplateSelector2",
                    name: "defaultPortExtendedSettingsTemplate2",
                    store: this._defaultPortExtendedSettingsTemplateStore,
                    query: {
                        complexQuery: "vendor: '" + this._controllerVendor[1][0] + "' AND chipset: '" + this._controllerChipset[1][0] + "' AND protocol: '" + this._protocol[1] + "'"
                    },
                    maxHeight: -1,
                    emptyLabel: this.resources.NO_TEMP_DEF_HYPHENS,
                    style: {
                        width: this._defaultselectorwidth
                    },
                    disabled: false
                }).placeAt(this._defaultPortExtendedSettingsTemplate2);
                this._defaultPortExtendedSettingsTemplateSelector2.startup();
                
                (this.dialogMode == "assign") && (this._defaultPortExtendedSettingsTemplateSelector2.set("value", this.initialPortExtendedPatternId2));
                var curVal = this._defaultPortExtendedSettingsTemplateSelector2.get("value");
                if (curVal != "") {
                    // Add "*NONE" option ...
                    this._defaultPortExtendedSettingsTemplateSelector2.addOption({
                        value: "*NONE",
                        label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
                    });
                    // If not in "assign" mode, or if initial value didn't take, then initialize to "*NONE".
                    if ((this.dialogMode != "assign") || (curVal != this.initialPortExtendedPatternId2)) {
                        this._defaultPortExtendedSettingsTemplateSelector2.set("value", "*NONE");
                    }
                }
                
                dojo.connect(this._defaultPortExtendedSettingsTemplateSelector2, "onChange", this, function() {
                    // NOTE: This might not be necessary
                    if (this._defaultPortExtendedSettingsTemplateSelector2.get("value") == "") {
                        this._defaultPortExtendedSettingsTemplateSelector2.set("disabled", true);
                        dojo.replaceClass(this._portExtendedEditIcon2, "spEditInactiveIcon_v2", "spEditIcon_v2");
                    } else {
                        if (this._defaultPortExtendedSettingsTemplateSelector2.get("value") == "*NONE") {
                            dojo.replaceClass(this._portExtendedEditIcon2, "spEditInactiveIcon_v2", "spEditIcon_v2");
                        } else {
                            dojo.replaceClass(this._portExtendedEditIcon2, "spEditIcon_v2", "spEditInactiveIcon_v2");    
                        }
                        this._defaultPortExtendedSettingsTemplateSelector2.set("disabled", false);
                    }
                    this._validateSelectors();
                });
                
                if (this._defaultPortExtendedSettingsTemplateSelector2.get("value") == "") {
                    this._defaultPortExtendedSettingsTemplateSelector2.set("disabled", true);
                    dojo.replaceClass(this._portExtendedEditIcon2, "spEditInactiveIcon_v2", "spEditIcon_v2");
                } else {
                    if (this._defaultPortExtendedSettingsTemplateSelector2.get("value") == "*NONE") {
                        dojo.replaceClass(this._portExtendedEditIcon2, "spEditInactiveIcon_v2", "spEditIcon_v2");
                    } else {
                        dojo.replaceClass(this._portExtendedEditIcon2, "spEditIcon_v2", "spEditInactiveIcon_v2");    
                    }
                    this._defaultPortExtendedSettingsTemplateSelector2.set("disabled", false);
                }
            }
        },
        
        _createEthernetPortPatternSelector: function(portGrp) {
            if (portGrp == 1) {
                this._defaultEthernetPortTemplateSelector = new support.widgets.Select({
                    id: this.get("id") + "_ethernetPortTemplateSelector",
                    name: "defaultEthernetPortTemplate",
                    store: this._defaultPortPFTemplateStore,
                    query: {
                        // Filter by controller's vendor & chipset.
                        complexQuery: "physicalFunction: 'nic' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"
                    },
                    maxHeight: -1,
                    emptyLabel: this.resources.NO_TEMP_DEF_HYPHENS,
                    style: {
                        width: this._defaultselectorwidth
                    },
                    disabled: false
                }).placeAt(this._defaultEthernetPortTemplate);
                this._defaultEthernetPortTemplateSelector.startup();
                dojo.connect(this._defaultEthernetPortTemplateSelector, "onChange", this, function() {
                    if (this._defaultEthernetPortTemplateSelector.get("value") == "") {                    
                        this._ethernetPortError.style.display = "block";
                    } else {
                        this._ethernetPortError.style.display = "none";
                    }
                    
                    this._validateSelectors();
                });
            }
            if (portGrp == 2) {
                this._defaultEthernetPortTemplateSelector2 = new support.widgets.Select({
                    id: this.get("id") + "_ethernetPortTemplateSelector2",
                    name: "defaultEthernetPortTemplate2",
                    store: this._defaultPortPFTemplateStore,
                    query: {
                        // Filter by controller's vendor & chipset.
                        complexQuery: "physicalFunction: 'nic' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"
                    },
                    maxHeight: -1,
                    emptyLabel: this.resources.NO_TEMP_DEF_HYPHENS,
                    style: {
                        width: this._defaultselectorwidth
                    },
                    disabled: false
                }).placeAt(this._defaultEthernetPortTemplate2);
                this._defaultEthernetPortTemplateSelector2.startup();
                dojo.connect(this._defaultEthernetPortTemplateSelector2, "onChange", this, function() {
                    if (this._defaultEthernetPortTemplateSelector2.get("value") == "") {                    
                        this._ethernetPortError2.style.display = "block";
                    } else {
                        this._ethernetPortError2.style.display = "none";
                    }
                    
                    this._validateSelectors();
                });                
            }
        },
        
        _createFcoePatternSelector: function(portGrp) {
            if (portGrp == 1) {
                this._defaultFibreChanneloverEthernetTemplateSelector = new support.widgets.Select({
                    id: this.get("id") + "_fibreChanneloverEthernetTemplateSelector",
                    name: "defaultFibreChanneloverEthernetTemplate",
                    store: this._defaultPortPFTemplateStore,
                    query: {
                        complexQuery: "physicalFunction: 'fcoe' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"
                    },
                    maxHeight: -1,
                    emptyLabel: this.resources.NO_TEMP_DEF_HYPHENS,
                    style: {
                        width: this._defaultselectorwidth
                    },
                    disabled: false
                }).placeAt(this._defaultFibreChanneloverEthernetTemplate);
                this._defaultFibreChanneloverEthernetTemplateSelector.startup();
                dojo.connect(this._defaultFibreChanneloverEthernetTemplateSelector, "onChange", this, function() {
                    if (this._defaultFibreChanneloverEthernetTemplateSelector.get("value") == "") {                    
                        this._fcoeError.style.display = "block";
                    } else {
                        this._fcoeError.style.display = "none";
                    }
                    
                    this._validateSelectors();
                });
            }
            if (portGrp == 2) {
                this._defaultFibreChanneloverEthernetTemplateSelector2 = new support.widgets.Select({
                    id: this.get("id") + "_fibreChanneloverEthernetTemplateSelector2",
                    name: "defaultFibreChanneloverEthernetTemplate2",
                    store: this._defaultPortPFTemplateStore,
                    query: {
                        complexQuery: "physicalFunction: 'fcoe' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"
                    },
                    maxHeight: -1,
                    emptyLabel: this.resources.NO_TEMP_DEF_HYPHENS,
                    style: {
                        width: this._defaultselectorwidth
                    },
                    disabled: false
                }).placeAt(this._defaultFibreChanneloverEthernetTemplate2);
                this._defaultFibreChanneloverEthernetTemplateSelector2.startup();
                dojo.connect(this._defaultFibreChanneloverEthernetTemplateSelector2, "onChange", this, function() {
                    if (this._defaultFibreChanneloverEthernetTemplateSelector2.get("value") == "") {                    
                        this._fcoeError2.style.display = "block";
                    } else {
                        this._fcoeError2.style.display = "none";
                    }
                    
                    this._validateSelectors();
                });                
            }
        },
        
        _createiSCSIPatternSelector: function(portGrp) {
            if (portGrp == 1) {
                this._defaultiSCSIFunctionTemplateSelector = new support.widgets.Select({
                    id: this.get("id") + "_iSCSIFunctionTemplateSelector",
                    name: "defaultiSCSIFunctionTemplate",
                    store: this._defaultPortPFTemplateStore,
                    query: {
                        complexQuery: "physicalFunction: 'iscsi' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"
                    },
                    maxHeight: -1,
                    emptyLabel: this.resources.NO_TEMP_DEF_HYPHENS,
                    style: {
                        width: this._defaultselectorwidth
                    },
                    disabled: false
                }).placeAt(this._defaultiSCSIFunctionTemplate);
                this._defaultiSCSIFunctionTemplateSelector.startup();
                dojo.connect(this._defaultiSCSIFunctionTemplateSelector, "onChange", this, function() {
                    if (this._defaultiSCSIFunctionTemplateSelector.get("value") == "") {                    
                        this._iscsiError.style.display = "block";
                    } else {
                        this._iscsiError.style.display = "none";
                    }
                    
                    this._validateSelectors();
                });
            }
            if (portGrp == 2) {
                this._defaultiSCSIFunctionTemplateSelector2 = new support.widgets.Select({
                    id: this.get("id") + "_iSCSIFunctionTemplateSelector2",
                    name: "defaultiSCSIFunctionTemplate2",
                    store: this._defaultPortPFTemplateStore,
                    query: {
                        complexQuery: "physicalFunction: 'iscsi' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"
                    },
                    maxHeight: -1,
                    emptyLabel: this.resources.NO_TEMP_DEF_HYPHENS,
                    style: {
                        width: this._defaultselectorwidth
                    },
                    disabled: false
                }).placeAt(this._defaultiSCSIFunctionTemplate2);
                this._defaultiSCSIFunctionTemplateSelector2.startup();
                dojo.connect(this._defaultiSCSIFunctionTemplateSelector2, "onChange", this, function() {
                    if (this._defaultiSCSIFunctionTemplateSelector2.get("value") == "") {                    
                        this._iscsiError2.style.display = "block";
                    } else {
                        this._iscsiError2.style.display = "none";
                    }
                    
                    this._validateSelectors();
                });                
            }
        },
        
        _createFCPatternSelector: function(portGrp) {
            if (portGrp == 1) {
                this._defaultFibreChannelPortTemplateSelector = new support.widgets.Select({
                    id: this.get("id") + "_fibreChannelPortTemplateSelector",
                    name: "defaultFibreChannelPortTemplate",
                    store: this._defaultPortPFTemplateStore,
                    query: {
                        complexQuery: "physicalFunction: 'fc' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"
                    },
                    maxHeight: -1,
                    emptyLabel: this.resources.NO_TEMP_DEF_HYPHENS,
                    style: {
                        width: this._defaultselectorwidth
                    },
                    disabled: false
                }).placeAt(this._defaultFibreChannelPortTemplate);
                this._defaultFibreChannelPortTemplateSelector.startup();
                dojo.connect(this._defaultFibreChannelPortTemplateSelector, "onChange", this, function() {
                    if (this._defaultFibreChannelPortTemplateSelector.get("value") == "") {                    
                        this._fcError.style.display = "block";
                    } else {
                        this._fcError.style.display = "none";
                    }
                    
                    this._validateSelectors();
                });
            }
            if (portGrp == 2) {
                this._defaultFibreChannelPortTemplateSelector2 = new support.widgets.Select({
                    id: this.get("id") + "_fibreChannelPortTemplateSelector2",
                    name: "defaultFibreChannelPortTemplate2",
                    store: this._defaultPortPFTemplateStore,
                    query: {
                        complexQuery: "physicalFunction: 'fc' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"
                    },
                    maxHeight: -1,
                    emptyLabel: this.resources.NO_TEMP_DEF_HYPHENS,
                    style: {
                        width: this._defaultselectorwidth
                    },
                    disabled: false
                }).placeAt(this._defaultFibreChannelPortTemplate2);
                this._defaultFibreChannelPortTemplateSelector2.startup();
                dojo.connect(this._defaultFibreChannelPortTemplateSelector2, "onChange", this, function() {              
                    if (this._defaultFibreChannelPortTemplateSelector2.get("value") == "") {                    
                        this._fcError2.style.display = "block";
                    } else {
                        this._fcError2.style.display = "none";
                    }
                    
                    this._validateSelectors();
                });                
            }
        },
        
        _setSelectorStores: function(portGrp) {
            this._defaultAdapterTemplateSelector.setStore(this._defaultAdapterTemplateStore, null, {query: {
                // Only templates for specified adapter.
                complexQuery: "adapterType: '" + this._adapterId + "'"
            }});
            
            this._setGlobalValues();
            
            if (portGrp == 1) {
                // Port Pattern section start
                this._defaultPortVirtualizationTemplateSelector.setStore(this._defaultPortVirtualizationTemplateStore, null, {query: {
                    complexQuery: "(adapterType: '" + this._adapterId + "' OR adapterType: '*ANY') "
                }});
                // Port Pattern section end
                
                // Port Extended Settings section start
                var curVal = this._defaultPortExtendedSettingsTemplateSelector.get("value");
                if (((curVal == "") || (curVal == "*NONE")) && (this.dialogMode == "assign")) {
                    curVal = this.initialPortExtendedPatternId1;
                }
                this._defaultPortExtendedSettingsTemplateSelector.setStore(this._defaultPortExtendedSettingsTemplateStore, curVal, {query: {
                    complexQuery: "vendor: '" + this._controllerVendor[0][0] + "' AND chipset: '" + this._controllerChipset[0][0] + "' AND protocol: '" + this._protocol[0] + "'" 
                }});
                
                curVal = this._defaultPortExtendedSettingsTemplateSelector.get("value");
                if (curVal != "") {
                    // Add "*NONE" option ...
                    this._defaultPortExtendedSettingsTemplateSelector.addOption({
                        value: "*NONE",
                        label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
                    });
                    // If not in "assign" mode, or if initial value didn't take, then initialize to "*NONE".
                    if ((this.dialogMode != "assign") || (curVal != this.initialPortExtendedPatternId1)) {
                        this._defaultPortExtendedSettingsTemplateSelector.set("value", "*NONE");
                    }
                }
                
                if (this._defaultPortExtendedSettingsTemplateSelector.get("value") == "") {
                    this._defaultPortExtendedSettingsTemplateSelector.set("disabled", true);
                    dojo.replaceClass(this._portExtendedEditIcon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                } else {
                    if (this._defaultPortExtendedSettingsTemplateSelector.get("value") == "*NONE") {
                        dojo.replaceClass(this._portExtendedEditIcon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                    } else {
                        dojo.replaceClass(this._portExtendedEditIcon, "spEditIcon_v2", "spEditInactiveIcon_v2");    
                    }
                    this._defaultPortExtendedSettingsTemplateSelector.set("disabled", false);                
                }
                // Port Extended Settings section end
                
                // Ethernet Port section start
                this._defaultEthernetPortTemplateSelector.setStore(this._defaultPortPFTemplateStore, null, {query: {
                    complexQuery: "physicalFunction: 'nic' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"
                }});
                // Ethernet Port section end
                
                // FCOE section start
                this._defaultFibreChanneloverEthernetTemplateSelector.setStore(this._defaultPortPFTemplateStore, null, {query: {
                    complexQuery: "physicalFunction: 'fcoe' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"
                }});
                // FCOE section end
                
                // iSCSI section start
                this._defaultiSCSIFunctionTemplateSelector.setStore(this._defaultPortPFTemplateStore, null, {query: {
                    complexQuery: "physicalFunction: 'iscsi' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"
                }});
                // iSCSI section end
                
                // Fibre Channel section start
                this._defaultFibreChannelPortTemplateSelector.setStore(this._defaultPortPFTemplateStore, null, {query: {
                    complexQuery: "physicalFunction: 'fc' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"
                }});
                // Fibre Channel section end
            }
            if (portGrp == 2) {
                if (this._defaultPortVirtualizationTemplateSelector2 == null) {
                    this._createConfigPortGroupSelectors(2);
                    return;
                }
                // Port Pattern section start
                this._defaultPortVirtualizationTemplateSelector2.setStore(this._defaultPortVirtualizationTemplateStore, null, {query: {
                    complexQuery: "(adapterType: '" + this._adapterId + "' OR adapterType: '*ANY') "
                }});
                // Port Pattern section end
                
                // Port Extended Settings section start
                var curVal = this._defaultPortExtendedSettingsTemplateSelector2.get("value");
                if (((curVal == "") || (curVal == "*NONE")) && (this.dialogMode == "assign")) {
                    curVal = this.initialPortExtendedPatternId2;
                }
                this._defaultPortExtendedSettingsTemplateSelector2.setStore(this._defaultPortExtendedSettingsTemplateStore, curVal, {query: {
                    complexQuery: "vendor: '" + this._controllerVendor[1][0] + "' AND chipset: '" + this._controllerChipset[1][0] + "' AND protocol: '" + this._protocol[1] + "'"
                }});
                
                curVal = this._defaultPortExtendedSettingsTemplateSelector2.get("value");
                if (curVal != "") {
                    // Add "*NONE" option ...
                    this._defaultPortExtendedSettingsTemplateSelector2.addOption({
                        value: "*NONE",
                        label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
                    });
                    // If not in "assign" mode, or if initial value didn't take, then initialize to "*NONE".
                    if ((this.dialogMode != "assign") || (curVal != this.initialPortExtendedPatternId2)) {
                        this._defaultPortExtendedSettingsTemplateSelector2.set("value", "*NONE");
                    }
                }
                
                if (this._defaultPortExtendedSettingsTemplateSelector2.get("value") == "") {
                    this._defaultPortExtendedSettingsTemplateSelector2.set("disabled", true);
                    dojo.replaceClass(this._portExtendedEditIcon2, "spEditInactiveIcon_v2", "spEditIcon_v2");
                } else {
                    if (this._defaultPortExtendedSettingsTemplateSelector2.get("value") == "*NONE") {
                        dojo.replaceClass(this._portExtendedEditIcon2, "spEditInactiveIcon_v2", "spEditIcon_v2");
                    } else {
                        dojo.replaceClass(this._portExtendedEditIcon2, "spEditIcon_v2", "spEditInactiveIcon_v2");    
                    }
                    this._defaultPortExtendedSettingsTemplateSelector2.set("disabled", false);
                }
                // Port Extended Settings section end
                
                // Ethernet Port section start
                this._defaultEthernetPortTemplateSelector2.setStore(this._defaultPortPFTemplateStore, null, {query: {
                    complexQuery: "physicalFunction: 'nic' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"
                }});
                // Ethernet Port section end

                // FCOE section start
                this._defaultFibreChanneloverEthernetTemplateSelector2.setStore(this._defaultPortPFTemplateStore, null, {query: {
                    complexQuery: "physicalFunction: 'fcoe' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"
                }});
                // FCOE section end
                
                // iSCSI section start
                this._defaultiSCSIFunctionTemplateSelector2.setStore(this._defaultPortPFTemplateStore, null, {query: {
                    complexQuery: "physicalFunction: 'iscsi' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"
                }});
                // iSCSI section end
                
                // Fibre Channel section start
                this._defaultFibreChannelPortTemplateSelector2.setStore(this._defaultPortPFTemplateStore, null, {query: {
                    complexQuery: "physicalFunction: 'fc' AND (adapterType: '*ANY' OR adapterType: '" + this._adapterType  + "' OR adapterType: '" + this._adapterId + "')"
                }});
                // Fibre Channel section end
            }
        },
        
        _createConfigPortGroupSelectors: function(portGrp) {
            if (portGrp == 1) {
                if (this._defaultPortVirtualizationTemplateSelector == null) {
                    this._createPortPatternSelector(1);
                }

                if (this._defaultPortExtendedSettingsTemplateSelector == null) {                          
                    this._createPortExtendedSettingsPatternSelector(1);
                }

                if (this._defaultEthernetPortTemplateSelector == null) {
                    this._createEthernetPortPatternSelector(1);
                }
                            
                if (this._defaultFibreChanneloverEthernetTemplateSelector == null) {
                    this._createFcoePatternSelector(1);
                }
                            
                if (this._defaultiSCSIFunctionTemplateSelector == null) {
                    this._createiSCSIPatternSelector(1);
                }
                                       
                if (this._defaultFibreChannelPortTemplateSelector == null) {
                    this._createFCPatternSelector(1);
                }
                
                this._setSelectorOneArray();
            }
            if (portGrp == 2) {
                if (this._defaultPortVirtualizationTemplateSelector2 == null) {
                    this._createPortPatternSelector(2);
                }
                            
                if (this._defaultPortExtendedSettingsTemplateSelector2 == null) {
                    this._createPortExtendedSettingsPatternSelector(2);
                }
                
                if (this._defaultEthernetPortTemplateSelector2 == null) {
                    this._createEthernetPortPatternSelector(2);
                }
                                        
                if (this._defaultFibreChanneloverEthernetTemplateSelector2 == null) {
                    this._createFcoePatternSelector(2);
                }
                            
                if (this._defaultiSCSIFunctionTemplateSelector2 == null) {
                    this._createiSCSIPatternSelector(2);
                }
                                       
                if (this._defaultFibreChannelPortTemplateSelector2 == null) {
                    this._createFCPatternSelector(2);
                }
                            
                this._setSelectorTwoArray();                
            }
        },
        
        _setSelectorOneArray: function() {
            var selector1Array = new Object();
            selector1Array.portVirtualizationTemplateSelector = this._defaultPortVirtualizationTemplateSelector;
            selector1Array.portExtendedSettingsTemplateSelector = this._defaultPortExtendedSettingsTemplateSelector;
            selector1Array.ethernetPortTemplateSelector = this._defaultEthernetPortTemplateSelector;
            selector1Array.fibreChanneloverEthernetTemplateSelector = this._defaultFibreChanneloverEthernetTemplateSelector;
            selector1Array.iSCSIFunctionTemplateSelector = this._defaultiSCSIFunctionTemplateSelector;
            selector1Array.fibreChannelPortTemplateSelector = this._defaultFibreChannelPortTemplateSelector;
                                    
            this._defaultPortVirtualizationSelectorArray.push(selector1Array);                       
        },
        
        _setSelectorTwoArray: function() {        
            var selector2Array = new Object();
            selector2Array.portVirtualizationTemplateSelector = this._defaultPortVirtualizationTemplateSelector2;
            selector2Array.portExtendedSettingsTemplateSelector = this._defaultPortExtendedSettingsTemplateSelector2;
            selector2Array.ethernetPortTemplateSelector = this._defaultEthernetPortTemplateSelector2;
            selector2Array.fibreChanneloverEthernetTemplateSelector = this._defaultFibreChanneloverEthernetTemplateSelector2;
            selector2Array.iSCSIFunctionTemplateSelector = this._defaultiSCSIFunctionTemplateSelector2;
            selector2Array.fibreChannelPortTemplateSelector = this._defaultFibreChannelPortTemplateSelector2;
            
            this._defaultPortVirtualizationSelectorArray.push(selector2Array);
        },

        _adapterSelectCallback: function(event) {
            var item = event.grid.getItem(event.rowIndex);
            this._adapterType = item.adapterType[0];
            this._adapterId = item.id[0];                        
            //dojo.attr(this._adapterListNode, "innerHTML", item.name);
            this._adapterTitlePane.toggle();
            var title = this.locResources.SELECTED_ADAPTER + "&nbsp;&nbsp;" + item.name;
            this._adapterTitlePane.set("title", title);
            this._adapterSelectMode(true);
        },

        _adapterSelectMode: function(/* boolean */readOnly) {
            // If fired from onActionClick it'll pass the event object
            // We'll just consider that a false.
            if (readOnly != true && readOnly != false) {
                readOnly = false;
            }
            //dojo.style(this._adapterTableNode, "display", (readOnly) ? "none" : "");
            //dojo.style(this._adapterListNode, "display", (readOnly) ? "" : "none");
            dojo.style(this._addDialogSelectorsSection, "display", (readOnly) ? "" : "none");

            this.okButton.set("disabled", (readOnly) ? false : true);            

            if (readOnly) {
                if (this._defaultPortVirtualizationTemplateSelector == null) {
                    this._createSelectBoxes();
                } else {
                    for (var ctlNum=1;ctlNum<=this._numberOfControllers;++ctlNum) {
                        this._setSelectorStores(ctlNum);
                    }
                    this._adapterPatternRefresh();
                }
                
                this._setGlobalValues();
                
                this._checkSelectorDisplayRules();
            }

            // Show action icon only in "read-only" mode so user may switch back to "edit mode" if
            // they wish to do so.
            // --Only if more than one adapters are selectable.
            if (this.numRows == 1) {
                //this._adapterOutlinePane.showActionIcon(false);
            } else {
                //this._adapterOutlinePane.showActionIcon(readOnly);
            }            
        },

        _checkSelectorDisplayRules: function() {
            // Need to figure out number of controllers (this._defaultAdapterTemplateData.items[].controllers[])
            // If 1 controller, hide the Configuration port group heading and remove indentation
            // If 2 controllers, show the headings and add indentation
            if (this._adapterType == "vfa") {
                this._adapterTemplateRow.style.display = "table-row";                
            } else {
                this._adapterTemplateRow.style.display = "none";
            }
                        
            if (this._numberOfControllers == 1) {                
                this._configPortGroupOne.style.display = "none";
                this._configPortGroupTwo.style.display = "none";
                           
                // Hide config port group 2 headers
                this._portVirtualizationRow.style.display = "none";
                this._portVirtualizationRow2.style.display = "none";
                this._portExtendedSettingsRow.style.display = "none";
                this._portExtendedSettingsRow2.style.display = "none";                
                this._ethernetPortRow.style.display = "none";
                this._ethernetPortRow2.style.display = "none";
                this._fibreChanneloverEthernetRow.style.display = "none";
                this._fibreChanneloverEthernetRow2.style.display = "none";
                this._iSCSIFunctionRow.style.display = "none";
                this._iSCSIFunctionRow2.style.display = "none";
                this._fibreChannelPortRow.style.display = "none";
                this._fibreChannelPortRow2.style.display = "none";
                
                var portCfgMode = (this._adapterPortConfigMode[0]) ?  this._adapterPortConfigMode[0]: "";
                if (portCfgMode && (typeof(portCfgMode) != "string")) {
                    portCfgMode = portCfgMode[0];
                }
                
                if ((portCfgMode == "") || (portCfgMode == "pNIC")) {           
                    this._portVirtualizationRow.style.display = "none";
                    this._portVirtualizationCell.style.paddingLeft = "0px";
                    if (this._release == "GA1") {                        
                        this._portExtendedSettingsRow.style.display = "table-row";
                        this._portExtendedSettingsCell.style.paddingLeft = "0px";
                    } else {                    
                        if (typeof(this._adapterPortConfigOptionCode[0]) == "string") {
                            this._adapterPortConfigOptionCode = this._adapterPortConfigOptionCode[0];
                        } else {
                            this._adapterPortConfigOptionCode = this._adapterPortConfigOptionCode[0][0];
                        }
                    
                        switch (this._adapterPortConfigOptionCode) {
                            case "ethOnly":                            
                                this._ethernetPortRow.style.display = "table-row";
                                this._ethernetPortCell.style.paddingLeft = "0px";
                                this._fibreChanneloverEthernetRow.style.display = "none";
                                this._fibreChanneloverEthernetCell.style.paddingLeft = "0px";
                                this._iSCSIFunctionRow.style.display = "none";
                                this._iSCSIFunctionCell.style.paddingLeft = "0px";
                                this._fibreChannelPortRow.style.display = "none";
                                this._fibreChannelPortCell.style.paddingLeft = "0px";
                                break;
                            case "eth+fcoe":
                                this._ethernetPortRow.style.display = "table-row";
                                this._ethernetPortCell.style.paddingLeft = "0px";
                                this._fibreChanneloverEthernetRow.style.display = "table-row";
                                this._fibreChanneloverEthernetCell.style.paddingLeft = "0px";
                                this._iSCSIFunctionRow.style.display = "none";
                                this._iSCSIFunctionCell.style.paddingLeft = "0px";
                                this._fibreChannelPortRow.style.display = "none";
                                this._fibreChannelPortCell.style.paddingLeft = "0px";
                                break;                        
                            case "eth+iscsi":
                                this._ethernetPortRow.style.display = "table-row";
                                this._ethernetPortCell.style.paddingLeft = "0px";
                                this._fibreChanneloverEthernetRow.style.display = "none";
                                this._fibreChanneloverEthernetCell.style.paddingLeft = "0px";
                                this._iSCSIFunctionRow.style.display = "table-row";
                                this._iSCSIFunctionCell.style.paddingLeft = "0px";
                                this._fibreChannelPortRow.style.display = "none";
                                this._fibreChannelPortCell.style.paddingLeft = "0px";
                                break;                        
                            case "fcOnly":
                                this._ethernetPortRow.style.display = "none";
                                this._ethernetPortCell.style.paddingLeft = "0px";
                                this._fibreChanneloverEthernetRow.style.display = "none";
                                this._fibreChanneloverEthernetCell.style.paddingLeft = "0px";
                                this._iSCSIFunctionRow.style.display = "none";
                                this._iSCSIFunctionCell.style.paddingLeft = "0px";
                                this._fibreChannelPortRow.style.display = "table-row";
                                this._fibreChannelPortCell.style.paddingLeft = "0px";
                                break;                              
                            case "fcoeOnly":
                                this._ethernetPortRow.style.display = "none";
                                this._ethernetPortCell.style.paddingLeft = "0px";
                                this._fibreChanneloverEthernetRow.style.display = "table-row";
                                this._fibreChanneloverEthernetCell.style.paddingLeft = "0px";
                                this._iSCSIFunctionRow.style.display = "none";
                                this._iSCSIFunctionCell.style.paddingLeft = "0px";
                                this._fibreChannelPortRow.style.display = "none";
                                this._fibreChannelPortCell.style.paddingLeft = "0px";
                                break;                            
                        }
                    } // if ga1
                } else {  // vNIC mode
                    //alert("hide everything but port virtualization");                
                    this._portVirtualizationRow.style.display = "table-row";
                    this._portVirtualizationCell.style.paddingLeft = "0px";                         
                    this._portExtendedSettingsRow.style.display = "none";
                    this._portExtendedSettingsCell.style.paddingLeft = "0px";                    
                    this._ethernetPortRow.style.display = "none";
                    this._ethernetPortCell.style.paddingLeft = "0px";
                    this._fibreChanneloverEthernetRow.style.display = "none";
                    this._fibreChanneloverEthernetCell.style.paddingLeft = "0px";
                    this._iSCSIFunctionRow.style.display = "none";
                    this._iSCSIFunctionCell.style.paddingLeft = "0px";
                    this._fibreChannelPortRow.style.display = "none";
                    this._fibreChannelPortCell.style.paddingLeft = "0px";                
                }                
                return;                 
            }
            
            if (this._numberOfControllers == 2) {
                this._configPortGroupOne.style.display = "table-row";
                this._configPortGroupTwo.style.display = "table-row";
                
                this._portVirtualizationRow.style.display = "none";
                this._portVirtualizationRow2.style.display = "none";
                this._portExtendedSettingsRow.style.display = "none";
                this._portExtendedSettingsRow2.style.display = "none";                
                this._ethernetPortRow.style.display = "none";
                this._ethernetPortRow2.style.display = "none";
                this._fibreChanneloverEthernetRow.style.display = "none";
                this._fibreChanneloverEthernetRow2.style.display = "none";
                this._iSCSIFunctionRow.style.display = "none";
                this._iSCSIFunctionRow2.style.display = "none";
                this._fibreChannelPortRow.style.display = "none";
                this._fibreChannelPortRow2.style.display = "none";                
                
                // Set display values for controller 1
                var portCfgMode = (this._adapterPortConfigMode[0]) ?  this._adapterPortConfigMode[0]: "";
                if (portCfgMode && (typeof(portCfgMode) != "string")) {
                    portCfgMode = portCfgMode[0];
                }
                
                if ((portCfgMode == "") || (portCfgMode == "pNIC")) {
                    this._portVirtualizationRow.style.display = "none";
                    this._portVirtualizationCell.style.paddingLeft = "15px";
                    
                    if (this._release == "GA1") {
                        this._portExtendedSettingsRow.style.display = "table-row";
                        this._portExtendedSettingsCell.style.paddingLeft = "15px";
                    } else {                    
                        var adapterPortConfigOptionCode1 = this._adapterPortConfigOptionCode;
                        if (typeof(adapterPortConfigOptionCode1[0]) == "string") {
                            adapterPortConfigOptionCode1 = adapterPortConfigOptionCode1[0];
                        } else {
                            adapterPortConfigOptionCode1 = adapterPortConfigOptionCode1[0][0];
                        }                    
                    
                        switch (adapterPortConfigOptionCode1) {
                            case "ethOnly":                                  
                                this._ethernetPortRow.style.display = "table-row";
                                this._ethernetPortCell.style.paddingLeft = "15px";
                                this._fibreChanneloverEthernetRow.style.display = "none";
                                this._fibreChanneloverEthernetCell.style.paddingLeft = "15px";
                                this._iSCSIFunctionRow.style.display = "none";
                                this._iSCSIFunctionCell.style.paddingLeft = "15px";
                                this._fibreChannelPortRow.style.display = "none";
                                this._fibreChannelPortCell.style.paddingLeft = "15px";
                                break;
                            case "eth+fcoe":
                                this._ethernetPortRow.style.display = "table-row";
                                this._ethernetPortCell.style.paddingLeft = "15px";
                                this._fibreChanneloverEthernetRow.style.display = "table-row";
                                this._fibreChanneloverEthernetCell.style.paddingLeft = "15px";
                                this._iSCSIFunctionRow.style.display = "none";
                                this._iSCSIFunctionCell.style.paddingLeft = "15px";
                                this._fibreChannelPortRow.style.display = "none";
                                this._fibreChannelPortCell.style.paddingLeft = "15px";
                                break;                        
                            case "eth+iscsi":
                                this._ethernetPortRow.style.display = "table-row";
                                this._ethernetPortCell.style.paddingLeft = "15px";
                                this._fibreChanneloverEthernetRow.style.display = "none";
                                this._fibreChanneloverEthernetCell.style.paddingLeft = "15px";
                                this._iSCSIFunctionRow.style.display = "table-row";
                                this._iSCSIFunctionCell.style.paddingLeft = "15px";
                                this._fibreChannelPortRow.style.display = "none";
                                this._fibreChannelPortCell.style.paddingLeft = "15px";
                                break;                        
                            case "fcOnly":
                                this._ethernetPortRow.style.display = "none";
                                this._ethernetPortCell.style.paddingLeft = "15px";
                                this._fibreChanneloverEthernetRow.style.display = "none";
                                this._fibreChanneloverEthernetCell.style.paddingLeft = "15px";
                                this._iSCSIFunctionRow.style.display = "none";
                                this._iSCSIFunctionCell.style.paddingLeft = "15px";
                                this._fibreChannelPortRow.style.display = "table-row";
                                this._fibreChannelPortCell.style.paddingLeft = "15px";
                                break;                              
                            case "fcoeOnly":
                                this._ethernetPortRow.style.display = "none";
                                this._ethernetPortCell.style.paddingLeft = "15px";
                                this._fibreChanneloverEthernetRow.style.display = "table-row";
                                this._fibreChanneloverEthernetCell.style.paddingLeft = "15px";
                                this._iSCSIFunctionRow.style.display = "none";
                                this._iSCSIFunctionCell.style.paddingLeft = "15px";
                                this._fibreChannelPortRow.style.display = "none";
                                this._fibreChannelPortCell.style.paddingLeft = "15px";
                                break;                            
                        }    
                    }
                } else {
                    //vNIC for controller 1
                    this._portVirtualizationRow.style.display = "table-row";
                    this._portVirtualizationCell.style.paddingLeft = "15px";
                    this._portExtendedSettingsRow.style.display = "none";
                    this._portExtendedSettingsCell.style.paddingLeft = "15px";                    
                    this._ethernetPortRow.style.display = "none";
                    this._ethernetPortCell.style.paddingLeft = "15px";
                    this._fibreChanneloverEthernetRow.style.display = "none";
                    this._fibreChanneloverEthernetCell.style.paddingLeft = "15px";
                    this._iSCSIFunctionRow.style.display = "none";
                    this._iSCSIFunctionCell.style.paddingLeft = "15px";
                    this._fibreChannelPortRow.style.display = "none";
                    this._fibreChannelPortCell.style.paddingLeft = "15px";
                }
                // Set display values for controller 2
                var portCfgMode = (this._adapterPortConfigMode[1]) ?  this._adapterPortConfigMode[1]: "";
                if (portCfgMode && (typeof(portCfgMode) != "string")) {
                    portCfgMode = portCfgMode[1];
                }
                
                if ((portCfgMode == "") || (portCfgMode == "pNIC")) {
                    this._portVirtualizationRow2.style.display = "none";
                    this._portVirtualizationCell2.style.paddingLeft = "15px";

                    if (this._release == "GA1") {
                        this._portExtendedSettingsRow2.style.display = "table-row";
                        this._portExtendedSettingsCell2.style.paddingLeft = "15px";
                    } else { 
                        var adapterPortConfigOptionCode2 = this._adapterPortConfigOptionCode;
                        if (typeof(adapterPortConfigOptionCode2[1]) == "string") {
                            adapterPortConfigOptionCode2 = adapterPortConfigOptionCode2[1];
                        } else {
                            adapterPortConfigOptionCode2 = adapterPortConfigOptionCode2[1][0];
                        }                    
                    
                        switch (adapterPortConfigOptionCode2) {
                            case "ethOnly":                                  
                                this._ethernetPortRow2.style.display = "table-row";
                                this._ethernetPortCell2.style.paddingLeft = "15px";
                                this._fibreChanneloverEthernetRow2.style.display = "none";
                                this._fibreChanneloverEthernetCell2.style.paddingLeft = "15px";
                                this._iSCSIFunctionRow2.style.display = "none";
                                this._iSCSIFunctionCell2.style.paddingLeft = "15px";
                                this._fibreChannelPortRow2.style.display = "none";
                                this._fibreChannelPortCell2.style.paddingLeft = "15px";
                                break;
                            case "eth+fcoe":
                                this._ethernetPortRow2.style.display = "table-row";
                                this._ethernetPortCell2.style.paddingLeft = "15px";
                                this._fibreChanneloverEthernetRow2.style.display = "table-row";
                                this._fibreChanneloverEthernetCell2.style.paddingLeft = "15px";
                                this._iSCSIFunctionRow2.style.display = "none";
                                this._iSCSIFunctionCell2.style.paddingLeft = "15px";
                                this._fibreChannelPortRow2.style.display = "none";
                                this._fibreChannelPortCell2.style.paddingLeft = "15px";
                                break;                        
                            case "eth+iscsi":
                                this._ethernetPortRow2.style.display = "table-row";
                                this._ethernetPortCell2.style.paddingLeft = "15px";
                                this._fibreChanneloverEthernetRow2.style.display = "none";
                                this._fibreChanneloverEthernetCell2.style.paddingLeft = "15px";
                                this._iSCSIFunctionRow2.style.display = "table-row";
                                this._iSCSIFunctionCell2.style.paddingLeft = "15px";
                                this._fibreChannelPortRow2.style.display = "none";
                                this._fibreChannelPortCell2.style.paddingLeft = "15px";
                                break;                        
                            case "fcOnly":
                                this._ethernetPortRow2.style.display = "none";
                                this._ethernetPortCell2.style.paddingLeft = "15px";
                                this._fibreChanneloverEthernetRow2.style.display = "none";
                                this._fibreChanneloverEthernetCell2.style.paddingLeft = "15px";
                                this._iSCSIFunctionRow2.style.display = "none";
                                this._iSCSIFunctionCell2.style.paddingLeft = "15px";
                                this._fibreChannelPortRow2.style.display = "table-row";
                                this._fibreChannelPortCell2.style.paddingLeft = "15px";
                                break;                              
                            case "fcoeOnly":
                                this._ethernetPortRow2.style.display = "none";
                                this._ethernetPortCell2.style.paddingLeft = "15px";
                                this._fibreChanneloverEthernetRow2.style.display = "table-row";
                                this._fibreChanneloverEthernetCell2.style.paddingLeft = "15px";
                                this._iSCSIFunctionRow2.style.display = "none";
                                this._iSCSIFunctionCell2.style.paddingLeft = "15px";
                                this._fibreChannelPortRow2.style.display = "none";
                                this._fibreChannelPortCell2.style.paddingLeft = "15px";
                                break;                            
                        }
                    }
                } else {
                    //vNIC for controller 2
                    this._portVirtualizationRow2.style.display = "table-row";
                    this._portVirtualizationCell2.style.paddingLeft = "15px";
                    this._portExtendedSettingsRow2.style.display = "none";
                    this._portExtendedSettingsCell2.style.paddingLeft = "15px";
                    this._ethernetPortRow2.style.display = "none";
                    this._ethernetPortCell2.style.paddingLeft = "15px";
                    this._fibreChanneloverEthernetRow2.style.display = "none";
                    this._fibreChanneloverEthernetCell2.style.paddingLeft = "15px";
                    this._iSCSIFunctionRow2.style.display = "none";
                    this._iSCSIFunctionCell2.style.paddingLeft = "15px";
                    this._fibreChannelPortRow2.style.display = "none";
                    this._fibreChannelPortCell2.style.paddingLeft = "15px";
                }                
            }
        },        

        _connectDataModels: function() {
            if (this._adapterModel == null) {
                
                var portVirtualizationTemplateData = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "template/list/portvirtual",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        portVirtualizationTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                });
                this._defaultPortVirtualizationTemplateStore = new dojox.data.AndOrReadStore({
                    data: portVirtualizationTemplateData
                });

                /* Port Physical Function REST call not needed at this time

                var portPhysicalFunctionTemplateData = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "template/list/ppfunction",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        portPhysicalFunctionTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                });
                this._defaultPortPFTemplateStore = new dojox.data.AndOrReadStore({
                    data: portPhysicalFunctionTemplateData
                });
                
                */
                
                var portExtendedSettingsTemplateData = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "template/list/ExtendedPort?Props=vendor,chipset,protocol",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        portExtendedSettingsTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                });                
                this._defaultPortExtendedSettingsTemplateStore = new dojox.data.AndOrReadStore({
                    data: portExtendedSettingsTemplateData
                });

                var adapterData = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "supportedDevices/ioAdapter/list",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        adapterData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                });
                this._supportedIOAdaptersList = adapterData;              

                var preferences = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "config/default",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        preferences = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                });

                // Fix up data store items with displayable values.
                dojo.forEach(adapterData.items, dojo.hitch(this, function(adapter, adIdx) {

                    // Add displayable type field based on formatted values.
                    switch (adapter.adapterType) {
                        case "vfa":
                            switch (adapter.type) {
                                case "lom":
                                    adapter.typeForDisplay = this.resources.FABRIC_CONNECTOR;
                                    break;
                                case "adapter":
                                    adapter.typeForDisplay = this.resources.VIRTUAL_FABRIC;
                                    break;
                            }
                            break;
                        case "ethernet":
                            adapter.typeForDisplay = this.resources.ETHERNET;
                            break;
                        case "fc":
                            adapter.typeForDisplay = this.resources.FIBRE_CHANNEL;
                            break;
                        case "fcoe":
                            adapter.typeForDisplay = this.resources.FCOE;
                            break;
                    }
                    
                    // Add preferred field, as appropriate.
                    dojo.forEach(preferences.preferredAdapters, dojo.hitch(this, function(preferred, pIdx) {
                        if (adapter.id == preferred.id) {
                            adapter.preferred = "*";
                        }
                    }));
                    
                }));

                // Newports (2sc, 4sc, 8sc ff) will not support 2 ASIC adapters in slot 3 or 4
                if ((this.formFactor == "2sc" || this.formFactor == "4sc" || this.formFactor == "8sc") && (this.cardNumber == 3 || this.cardNumber == 4)) {
                    var removalArray = [];
                    for (var i=0;i < adapterData.items.length;++i) {
                        // If the adapter has more that 1 controller, mark it for removal
                        if (adapterData.items[i].controllers.length > 1) {
                            removalArray.push(i);
                        }
                    }
                    // Remove the unsupported adapters from the adapterData
                    for (var j=removalArray.length-1;j >= 0;--j) {
                        adapterData.items.splice(removalArray[j], 1);
                    }
                }
                
                var adapterStore = new dojox.data.AndOrReadStore({
                    data: adapterData
                });

                var adapterTemplateData = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "template/list/adapter",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        adapterTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                });
                this._defaultAdapterTemplateData = adapterTemplateData;
                
                this._defaultAdapterTemplateStore = new dojox.data.AndOrReadStore({
                    data: adapterTemplateData
                });

                var adapterLayout = [ {
                    name: this.resources.DEFAULT,
                    field: "preferred",
                    width: "50px",
                    editable: false,
                    defaultValue: "",
                    formatter: dojo.hitch(this, "_formatPreferred")
                }, {
                    name: this.locResources.ADAPTER,
                    field: "name",
                    width: "350px",
                    editable: false
                }, {
                    name: this.locResources.PORTS,
                    field: "portCount",
                    width: "75px",
                    editable: false
                }, {
                    name: this.resources.TYPE,
                    field: "typeForDisplay",
                    width: "auto",
                    editable: false
                } ];
                                
                var adapterType = this.addType;
                // formFactor 1sd = Caracara
                // Caracara has an Add LOM button only instead
                // of Add I/O adapter or LOM button
                if (this.addType == "lom" && this.formFactor != "1sd") {
                    adapterType = "*";
                }
                
                this._adapterModel = new dojox.grid.DataGrid({
                    store: adapterStore,
                    query: {
                        complexQuery: "type:\"" + adapterType + "\" AND supportedFormFactors:\"" + this.formFactor + "\" AND supportedNodeTypes:\"" + this.nodeType + "\"" + ((this.formFactor == "4s") ? " AND !id:\"lom:1\"" : "")
                        //complexQuery: "type:\"" + adapterType + "\" AND supportedNodeTypes:\"" + this.nodeType + "\"" + ((this.formFactor == "4s") ? " AND !id:\"lom:1\"" : "")
                    },
                    structure: adapterLayout,
                    autoWidth: false,
                    autoHeight: false,
                    columnReordering: false,
                    style: "width:675px; height:345px;",
                    noDataMessage: this.locResources.NO_ADAPTERS,
                    onRowClick: dojo.hitch(this, this._adapterSelectCallback),
                    selectionMode: "none",
                    sortFields: [{attribute: "preferred"}, {attribute: "adapterType"}]
                }).placeAt(this._adapterTableNode);
                
                this._adapterTitlePane.toggle();
                
                adapterStore.fetch({
                    query: {
                        complexQuery: "type:\"" + adapterType + "\" AND supportedFormFactors:\"" + this.formFactor + "\" AND supportedNodeTypes:\"" + this.nodeType + "\"" + ((this.formFactor == "4s") ? " AND !id:\"lom:1\"" : "")
                        //complexQuery: "type:\"" + adapterType + "\" AND supportedNodeTypes:\"" + this.nodeType + "\"" + ((this.formFactor == "4s") ? " AND !id:\"lom:1\"" : "")
                    },
                    onComplete: dojo.hitch(this, function(items, request) {
                        if (this.numRows == 1) {
                            this._adapterType = items[0].adapterType[0];
                            this._adapterId = items[0].id[0];
                            //dojo.attr(this._adapterListNode, "innerHTML", items[0].name);
                            this._adapterTitlePane.toggle();
                            var title = this.locResources.SELECTED_ADAPTER + "&nbsp;&nbsp;" + items[0].name;
                            this._adapterTitlePane.set("title", title);
                        }
                    }),                    
                    start: 0,
                    count: 0
                });
                
                this._adapterTitlePane.toggle();
            }
        },
        
        _setGlobalValues: function () {
            // Set values for this particular adapter
            var dftAdapterPatternFound = false;
            var controllerObjArray = new Array();            
            var controllerVendorArray = new Array();
            var controllerChipsetArray = new Array();
            var adapterPortConfigModeArray = new Array();
            var adapterPortConfigOptionCodeArray = new Array();
            var adapterPortProtocolArray = new Array();
            
            if (this._adapterType == "vfa") {
                // Retrieve the following settings for a vfa:
                // this._numberOfControllers
                // this._controllerVendor
                // this._adapterPortConfigMode pNIC/vNIC_IBMvf/vNIC_independent
                // this._adapterPortConfigOptionCode ethOnly/eth+fcoe/eth+iscsi
                
                /****************************************************************/
                /* This loop goes through the list of default adapter templates */
                /****************************************************************/
                for(var adaptercounter=0;adaptercounter < this._defaultAdapterTemplateData.items.length;++adaptercounter) {
                    // Find the adapter template data for the selected adapter template
                    if (this._defaultAdapterTemplateData.items[adaptercounter].id == this._defaultAdapterTemplateSelector.get("value")) {
                        dftAdapterPatternFound = true;
                        this._numberOfControllers = this._defaultAdapterTemplateData.items[adaptercounter].controllers.length;
                        /******************************************************************************/                        
                        /* This loop goes through the list of controllers in the default adapter data */
                        /******************************************************************************/
                        for (var controllercounter=0;controllercounter < this._defaultAdapterTemplateData.items[adaptercounter].controllers.length; ++controllercounter) {
                            var controllerObj = new Object();
                            /**********************************************************/
                            /* This loop goes through the list of supported IO adapters */
                            /**********************************************************/
                            for (var supportedadaptercounter=0;supportedadaptercounter < this._supportedIOAdaptersList.items.length; ++supportedadaptercounter) {
                                // Find the adapter that matches the one we're working on
                                if (this._adapterId == this._supportedIOAdaptersList.items[supportedadaptercounter].id) {
                                    // If the controller config dependency is mutual, all controllers are configured the same way
                                    if (this._supportedIOAdaptersList.items[supportedadaptercounter].controllerConfigDependency == "mutual") {
                                        controllerObj.vendor = this._supportedIOAdaptersList.items[supportedadaptercounter].controllers[controllercounter].vendor[0];
                                        controllerVendorArray.push(this._supportedIOAdaptersList.items[supportedadaptercounter].controllers[controllercounter].vendor);
                                        controllerObj.chipset = this._supportedIOAdaptersList.items[supportedadaptercounter].controllers[controllercounter].chipset[0];
                                        controllerChipsetArray.push(this._supportedIOAdaptersList.items[supportedadaptercounter].controllers[controllercounter].chipset);
                                        /**************************************************************************************************************/
                                        /* This loop goes through the list of configurations for a given controller in the supported IO adapters list */
                                        /**************************************************************************************************************/
                                        for (var supportedadapterconfigcounter=0;
                                             supportedadapterconfigcounter < this._supportedIOAdaptersList.items[supportedadaptercounter].controllers[controllercounter].configurations.length;
                                             ++supportedadapterconfigcounter) {                                                                                   
                                            if (this._supportedIOAdaptersList.items[supportedadaptercounter].controllers[controllercounter].configurations[supportedadapterconfigcounter].portConfigDependency == "mutual") {
                                                controllerObj.portConfigDependency = "mutual";
                                                // Since the portConfigDependency is mutual only need to grab the number of the first port                                                
                                                controllerObj.portNumber = 
                                                    this._supportedIOAdaptersList.items[supportedadaptercounter].controllers[controllercounter].configurations[supportedadapterconfigcounter].ports[0].portNumber[0];                                                
                                                /*************************************************************************************/
                                                /* This loop goes through the list of portConfigurations for this particular adapter */
                                                /*************************************************************************************/
                                                for (var portconfigcounter=0;
                                                     portconfigcounter < this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations.length;
                                                     ++portconfigcounter) {
                                                    if (dojo.toJson(this._defaultAdapterTemplateData.items[adaptercounter].controllers[controllercounter].ports[0].portCfgId) == 
                                                        dojo.toJson(this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].id)) {                                                           
                                                        var ctlNum = this._defaultAdapterTemplateData.items[adaptercounter].controllers[controllercounter].controllerNumber;
                                                        //alert(dojo.toJson(this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].mode) + " " + supportedadaptercounter + " " + portconfigcounter);
                                                        controllerObj.portConfigMode = this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].mode[0];
                                                        adapterPortConfigModeArray[ctlNum-1] = this._validateMode(this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].mode);                                                        
                                                        for (var portoptioncounter=0;
                                                            portoptioncounter < this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].options.length;
                                                            ++portoptioncounter) {
                                                            if (dojo.toJson(this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].options[portoptioncounter].id) ==
                                                                dojo.toJson(this._defaultAdapterTemplateData.items[adaptercounter].controllers[controllercounter].ports[0].portCfgOptionId)) {
                                                                controllerObj.optionCode = this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].options[portoptioncounter].optionCode[0];
                                                                adapterPortConfigOptionCodeArray[ctlNum-1] = this._validateMode(this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].options[portoptioncounter].optionCode);
                                                                //controllerObj.portprotocol = this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].options[portoptioncounter].protocols;
                                                                adapterPortProtocolArray[ctlNum-1] = this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].options[portoptioncounter].protocols;
                                                            }                                                        
                                                        }
                                                    }                                            
                                                }
                                            } else {
                                                //Future independent port config section
                                            }
                                        }
                                    } else {
                                    // Future independent controller config section
                                    }                                
                                } // Found the matching adapter
                            } // Loop through the supported IO adapters
                            controllerObjArray.push(controllerObj);
                        } // Loop through controllers in the default adapter list 
                    } // Found the adapter that matches the currently selected one
                } // Default adapter template list loop
            }
            
            if (!dftAdapterPatternFound) { // Or not a vfa
                // Retrieve settings for adapters that are not vfas
                // this._numberOfControllers
                // this._controllerVendor
                // this._adapterPortConfigMode pNIC
                // this._adapterPortConfigOptionCode ethOnly/eth+fcoe/eth+iscsi/fcOnly/fcoeOnly
                /************************************************************/
                /* This loop goes through the list of supported IO adapters */
                /************************************************************/
                for (var supportedadaptercounter=0;supportedadaptercounter < this._supportedIOAdaptersList.items.length; ++supportedadaptercounter) {                    
                    // Found a match in the supported IO adapters list
                    if (this._adapterId == this._supportedIOAdaptersList.items[supportedadaptercounter].id) {
                        if (this._supportedIOAdaptersList.items[supportedadaptercounter].controllerConfigDependency == "mutual") {
                            this._numberOfControllers = this._supportedIOAdaptersList.items[supportedadaptercounter].controllers.length;                            
                            /**********************************************************************/
                            /* This loop goes through the controllers for this particular adapter */
                            /**********************************************************************/
                            for (var controllercounter=0;controllercounter < this._numberOfControllers; ++controllercounter) {
                                var controllerObj = new Object();
                                //this._controllerVendor = this._supportedIOAdaptersList.items[supportedadaptercounter].controllers[0].vendor;
                                controllerObj.vendor = this._supportedIOAdaptersList.items[supportedadaptercounter].controllers[controllercounter].vendor[0];
                                controllerVendorArray.push(this._supportedIOAdaptersList.items[supportedadaptercounter].controllers[controllercounter].vendor);
                                controllerObj.chipset = this._supportedIOAdaptersList.items[supportedadaptercounter].controllers[controllercounter].chipset[0];
                                controllerChipsetArray.push(this._supportedIOAdaptersList.items[supportedadaptercounter].controllers[controllercounter].chipset);
                                /**************************************************************************************************************/
                                /* This loop goes through the list of configurations for a given controller in the supported IO adapters list */
                                /**************************************************************************************************************/
                                for (var supportedadapterconfigcounter=0;
                                     supportedadapterconfigcounter < this._supportedIOAdaptersList.items[supportedadaptercounter].controllers[controllercounter].configurations.length;
                                     ++supportedadapterconfigcounter) {
                                    if (this._supportedIOAdaptersList.items[supportedadaptercounter].controllers[controllercounter].configurations[supportedadapterconfigcounter].portConfigDependency == "mutual") {
                                        controllerObj.portConfigDependency = "mutual";
                                        controllerObj.portNumber = 
                                            this._supportedIOAdaptersList.items[supportedadaptercounter].controllers[controllercounter].configurations[supportedadapterconfigcounter].ports[0].portNumber[0];
                                        for (var supportedconfigcounter=0;
                                             supportedconfigcounter < this._supportedIOAdaptersList.items[supportedadaptercounter].controllers[controllercounter].configurations[supportedadapterconfigcounter].ports[0].supportedConfigurations.length;
                                             ++supportedconfigcounter) {
                                            for (var portconfigcounter=0;                                            
                                                 portconfigcounter < this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations.length;
                                                 ++portconfigcounter) {
                                                if (this._supportedIOAdaptersList.items[supportedadaptercounter].controllers[controllercounter].configurations[supportedadapterconfigcounter].ports[0].supportedConfigurations[supportedconfigcounter] ==
                                                    this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].id) {
                                                    if ( this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].mode != undefined) {
                                                        controllerObj.portConfigMode = this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].mode[0];
                                                        adapterPortConfigModeArray.push(this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].mode[0]);
                                                    } else {
                                                        controllerObj.portConfigMode = "pNIC";
                                                        adapterPortConfigModeArray.push("pNIC");
                                                    }                                                
                                                    for (var optioncounter=0;
                                                         optioncounter < this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].options.length;
                                                         ++optioncounter) {
                                                        controllerObj.optionCode = this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].options[optioncounter].optionCode[0];
                                                        //this._adapterPortConfigOptionCode = this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].options[optioncounter].optionCode[0];
                                                        adapterPortConfigOptionCodeArray.push(this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].options[optioncounter].optionCode[0]);
                                                        //alert("optionCode: " + this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].options[optioncounter].optionCode[0]);
                                                        adapterPortProtocolArray.push(this._supportedIOAdaptersList.items[supportedadaptercounter].portConfigurations[portconfigcounter].options[optioncounter].protocols);
                                                    }
                                                }                                        
                                            }
                                        }
                                    } else {
                                        // Future independent port config section
                                    }
                                }
                                controllerObjArray.push(controllerObj);
                            }
                        } else {
                            // Future independent controller config section
                        }
                    }
                }
            }
            this._controllerVendor = controllerVendorArray;
            this._controllerChipset = controllerChipsetArray;
            this._adapterPortConfigMode = adapterPortConfigModeArray;
            this._adapterPortConfigOptionCode = adapterPortConfigOptionCodeArray;
            this._protocol = new Array();
            console.log(this.declaredClass + ": Adapter Id: " + this._adapterId);
            console.log(this.declaredClass + ": Adapter Type: " + this._adapterType);
            console.log(this.declaredClass + ": Number of Controllers: " + this._numberOfControllers);
            console.log(this.declaredClass + ": Controller Vendor: " + this._controllerVendor);
            console.log(this.declaredClass + ": Controller Chipset: " + this._controllerChipset);
            console.log(this.declaredClass + ": Port Configuration Mode: " + this._adapterPortConfigMode);
            console.log(this.declaredClass + ": Port Configuration OptionCode: " + this._adapterPortConfigOptionCode);
            console.log(this.declaredClass + ": Port Protocol: " + adapterPortProtocolArray + " length: " + adapterPortProtocolArray.length);
            if (adapterPortProtocolArray[0] != undefined) {
                if (dojo.indexOf(adapterPortProtocolArray[0], "ethernet") != -1) {
                    this._protocol[0] = "ethernet"; 
                } else {
                    this._protocol[0] = "";
                }
            } else {
                this._protocol[0] = "";
            }            
            if (adapterPortProtocolArray[1] != undefined) {
                if (dojo.indexOf(adapterPortProtocolArray[1], "ethernet") != -1) {
                    this._protocol[1] = "ethernet"; 
                } else {
                    this._protocol[1] = "";
                }
            } else {
                this._protocol[1] = "";
            }
            this._adapterControllerObj = controllerObjArray;
        },
        
        _validateMode: function(mode) {
            if (typeof(mode) == "string") {
                return mode;
            }
            if (typeof(mode[0]) == "string") {
                return mode[0];
            }
            if (typeof(mode[0][0]) == "string") {
                return mode[0][0];
            }
            return mode[0].value;
        },
        
        _formatPreferred : function(value, rowIndex, cell) {
            var grid = cell.grid;
            var st = this;
            
            dojo.connect(grid, "onCellMouseOver", function(e) {
                if(e.cellIndex == 0 && e.rowIndex == rowIndex && value == "*") {
                    dijit.showTooltip("<div>" + st.locResources.IDENTIFIED + "</div>", e.cellNode);
                }
            });
            dojo.connect(grid, "onCellMouseOut", function(e) {
                if(e.cellIndex == 0 && e.rowIndex == rowIndex) {
                    dijit.hideTooltip(e.cellNode);
                }
            });
            dojo.connect(grid, "onBlur", function() {
                dijit.hideTooltip(cell.getNode());
            });
            
            return value;
        },

        _showNewAdapterDialog: function() {
           require(["support/dialogs/NewAdapterTemplate"], dojo.hitch(this, function() {
               var newAdapterDialog = new support.dialogs.NewAdapterTemplate({
                   adapterId: this._adapterId
               });
               newAdapterDialog.show();
           }));
        },
        
        _editNewAdapterDialog: function() {
            if (this._adapterError.style.display == "block") {
                // The error icon is displayed which means edit is disabled.
                return;
            } else {
                require(["support/dialogs/NewAdapterTemplate"], dojo.hitch(this, function() {
                    var newAdapterDialog = new support.dialogs.NewAdapterTemplate({
                        templateIDNum: this._defaultAdapterTemplateSelector.get("value")
                    });
                    newAdapterDialog.show();
                }));
            }
        },

        _showPortVirtualizationDialog: function() {
            // Need to pass the adapter type, operational mode and protocol
            var adapId = this._adapterId;
            var thismode = "";
            if (typeof(this._adapterPortConfigMode) == "string") {               
                thismode = this._adapterPortConfigMode;
            } else {
                thismode = this._adapterPortConfigMode[0];
            }
            var thisprotocol = this._adapterPortConfigOptionCode[0];
                      
            require(["support/dialogs/NewPortVirtualizationTemplate"], dojo.hitch(this, function() {
               var portVirtDialog = new support.dialogs.NewPortVirtualizationTemplate({
                   adapterId: adapId,
                   mode: thismode,
                   protocol: thisprotocol
               });            
               this._pendingCreateTemplateItems = "portvirt1";
               portVirtDialog.show();
            }));
        },
        
        _editPortVirtualizationDialog: function() {
            if (this._portVirtError.style.display == "block") {
                // The error icon is displayed which means edit is disabled.
                return;
            } else {
                require(["support/dialogs/NewPortVirtualizationTemplate"], dojo.hitch(this, function() {
                    var portVirtualizationDialog = new support.dialogs.NewPortVirtualizationTemplate({
                        templateIDNum: this._defaultPortVirtualizationTemplateSelector.get("value"),
                        templateMode: "edit"
                    });
                    this._pendingCreateTemplateItems = "portvirt1";
                    portVirtualizationDialog.show();            
                }));
            }
        },
        
        _showPortVirtualizationDialog2: function() {
            require(["support/dialogs/NewPortVirtualizationTemplate"], dojo.hitch(this, function() {
               // Need to pass the adapter type, operational mode and protocol            
               var portVirtDialog = new support.dialogs.NewPortVirtualizationTemplate({
                   adapterId: this._adapterId,
                   mode: this._adapterPortConfigMode[1],
                   protocol: this._adapterPortConfigOptionCode[1]                
               });
               this._pendingCreateTemplateItems = "portvirt2";
               portVirtDialog.show();
            }));
        },

        _editPortVirtualizationDialog2: function() {
            if (this._portVirtError2.style.display == "block") {
                // The error icon is displayed which means edit is disabled.
                return;
            } else {            
                require(["support/dialogs/NewPortVirtualizationTemplate"], dojo.hitch(this, function() {
                    var portVirtualizationDialog = new support.dialogs.NewPortVirtualizationTemplate({
                        templateIDNum: this._defaultPortVirtualizationTemplateSelector2.get("value"),
                        templateMode: "edit"
                    });
                    this._pendingCreateTemplateItems = "portvirt2";
                    portVirtualizationDialog.show();
                }));
            }
        },        
        
        _showPortPhysicalFunctionNicDialog: function() {            
            require(["support/dialogs/NewPortPhysicalFunctionTemplate"], dojo.hitch(this, function() {
               var portPhysicalDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                   nodeType: this.nodeType,
                   addType: this._adapterId,
                   physicalFunction: "nic"
                });
               this._pendingCreateTemplateItems = "ppfNic";
               portPhysicalDialog.show();
            }));
        },
        
        _showPortPhysicalFunctionNicDialog2: function() {            
           require(["support/dialogs/NewPortPhysicalFunctionTemplate"], dojo.hitch(this, function() {
               var portPhysicalDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                   nodeType: this.nodeType,
                   addType: this._adapterId,
                   physicalFunction: "nic"
                });
               this._pendingCreateTemplateItems = "ppfNic2";
               portPhysicalDialog.show();
           }));
        },
        
        _editPortExtendedSettingsDialog1: function() {
            if (this._defaultPortExtendedSettingsTemplateSelector.get("value") == "" || this._defaultPortExtendedSettingsTemplateSelector.get("value") == "*NONE") {
                return;
            }
            require(["support/dialogs/NewLearnedSettingsTemplate"], dojo.hitch(this, function() {
               var portExtendedSettingsDialog = new support.dialogs.NewLearnedSettingsTemplate({
                   templateIDNum: this._defaultPortExtendedSettingsTemplateSelector.get("value"),
                   type: "port",
                   templateMode: "edit"
               });
               this._pendingCreateTemplateItems = "exPort1";
               portExtendedSettingsDialog.show();            
            }));
        },        
        
        _editPortExtendedSettingsDialog2: function() {
            if (this._defaultPortExtendedSettingsTemplateSelector2.get("value") == "" || this._defaultPortExtendedSettingsTemplateSelector2.get("value") == "*NONE") {
                return;
            }            
            require(["support/dialogs/NewLearnedSettingsTemplate"], dojo.hitch(this, function() {
               var portExtendedSettingsDialog = new support.dialogs.NewLearnedSettingsTemplate({
                   templateIDNum: this._defaultPortExtendedSettingsTemplateSelector2.get("value"),
                   type: "port",
                   templateMode: "edit"
               });
               this._pendingCreateTemplateItems = "exPort2";
               portExtendedSettingsDialog.show();            
            }));
        },        
        
        _editPortPhysicalFunctionDialogNic1: function() {
            require(["support/dialogs/NewPortPhysicalFunctionTemplate"], dojo.hitch(this, function() {
               var portPhysicalFunctionDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                   templateIDNum: this._defaultEthernetPortTemplateSelector.get("value")
               });
               portPhysicalFunctionDialog.show();            
            }));
        },
        
        _showPortPhysicalFunctionFcoeDialog: function() {
            require(["support/dialogs/NewPortPhysicalFunctionTemplate"], dojo.hitch(this, function() {
               var portPhysicalDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                   nodeType: this.nodeType,
                   addType: this._adapterId,
                   physicalFunction: "fcoe"                
               });
               this._pendingCreateTemplateItems = "ppfFcoe";
               portPhysicalDialog.show();
            }));
        },
        
        _showPortPhysicalFunctionFcoeDialog2: function() {
            require(["support/dialogs/NewPortPhysicalFunctionTemplate"], dojo.hitch(this, function() {
               var portPhysicalDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                   nodeType: this.nodeType,
                   addType: this._adapterId,
                   physicalFunction: "fcoe"                
               });
               this._pendingCreateTemplateItems = "ppfFcoe2";
               portPhysicalDialog.show();
            }));
        },
        
        _editPortPhysicalFunctionDialogFcoe1: function() {
            require(["support/dialogs/NewPortPhysicalFunctionTemplate"], dojo.hitch(this, function() {
               var portPhysicalFunctionDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                   templateIDNum: this._defaultEthernetPortTemplateSelector.get("value")
               });
               portPhysicalFunctionDialog.show();            
            }));
        },
        
        _showPortPhysicalFunctioniSCSIDialog: function() {
            require(["support/dialogs/NewPortPhysicalFunctionTemplate"], dojo.hitch(this, function() {
               var portPhysicalDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                   nodeType: this.nodeType,
                   addType: this._adapterId,
                   physicalFunction: "iscsi"
               });
               this._pendingCreateTemplateItems = "ppfiscsi";
               portPhysicalDialog.show();
            }));
        },
        
        _showPortPhysicalFunctioniSCSIDialog2: function() {
            require(["support/dialogs/NewPortPhysicalFunctionTemplate"], dojo.hitch(this, function() {
               var portPhysicalDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                   nodeType: this.nodeType,
                   addType: this._adapterId,
                   physicalFunction: "iscsi"
               });
               this._pendingCreateTemplateItems = "ppfiscsi2";
               portPhysicalDialog.show();
            }));
        },
        
        _editPortPhysicalFunctionDialogiSCSI1: function() {
            require(["support/dialogs/NewPortPhysicalFunctionTemplate"], dojo.hitch(this, function() {
               var portPhysicalFunctionDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                   templateIDNum: this._defaultiSCSIFunctionTemplateSelector.get("value"),
                   addType: this._adapterId
               });
               portPhysicalFunctionDialog.show();            
            }));
        },
        
        _showPortPhysicalFunctionFCDialog: function() {
            require(["support/dialogs/NewPortPhysicalFunctionTemplate"], dojo.hitch(this, function() {
               var portPhysicalDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                   nodeType: this.nodeType,
                   addType: this._adapterId,
                   physicalFunction: "fc"
               });
               this._pendingCreateTemplateItems = "ppfFc";
               portPhysicalDialog.show();
            }));
        },
        
        _showPortPhysicalFunctionFCDialog2: function() {
            require(["support/dialogs/NewPortPhysicalFunctionTemplate"], dojo.hitch(this, function() {
               var portPhysicalDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                   nodeType: this.nodeType,
                   addType: this._adapterId,
                   physicalFunction: "fc"
               });
               this._pendingCreateTemplateItems = "ppfFc2";
               portPhysicalDialog.show();
            }));
        },
        
        _editPortPhysicalFunctionDialogFC1: function() {
            require(["support/dialogs/NewPortPhysicalFunctionTemplate"], dojo.hitch(this, function() {
               var portPhysicalFunctionDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                   templateIDNum: this._defaultFibreChannelPortTemplateSelector.get("value")
               });
               portPhysicalFunctionDialog.show();            
            }));
        },
                
        _editPortPhysicalFunctionDialogNic2: function() {
            require(["support/dialogs/NewPortPhysicalFunctionTemplate"], dojo.hitch(this, function() {
               var portPhysicalFunctionDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                   templateIDNum: this._defaultEthernetPortTemplateSelector2.get("value")
               });
               portPhysicalFunctionDialog.show();            
            }));
        },
        
        _editPortPhysicalFunctionDialogFcoe2: function() {
            require(["support/dialogs/NewPortPhysicalFunctionTemplate"], dojo.hitch(this, function() {
               var portPhysicalFunctionDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                   templateIDNum: this._defaultEthernetPortTemplateSelector2.get("value")
               });
               portPhysicalFunctionDialog.show();            
            }));
        },
        
        _editPortPhysicalFunctionDialogiSCSI2: function() {
           require(["support/dialogs/NewPortPhysicalFunctionTemplate"], dojo.hitch(this, function() {
              var portPhysicalFunctionDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                 templateIDNum: this._defaultiSCSIFunctionTemplateSelector2.get("value")
              });
              portPhysicalFunctionDialog.show();            
           }));
        },
        
        _editPortPhysicalFunctionDialogFC2: function() {
            require(["support/dialogs/NewPortPhysicalFunctionTemplate"], dojo.hitch(this, function() {
               var portPhysicalFunctionDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                   templateIDNum: this._defaultFibreChannelPortTemplateSelector2.get("value")
               });
               portPhysicalFunctionDialog.show();            
            }));
        },
        
        _saveandhideDialog: function() {
            var adapterJson = ({
                "adapterId": this._adapterId,
                "templates": []
            });

            // Initial adapter template
            if (this._adapterTemplateRow.style.display != "none") {
                var adapterObj = new Object();
                adapterObj.templateId = this._defaultAdapterTemplateSelector.get("value");
                adapterObj.type = "Adapter";
                adapterJson.templates.push(adapterObj);
            }

            for (var configportgroupcounter=0;configportgroupcounter < this._adapterControllerObj.length;++configportgroupcounter) { 
                // Initial port virtualization template
                if (this._rowAttachPointList.portVirtualizationList[configportgroupcounter].style.display != "none") {
                    var portvirtObj = new Object();
                    portvirtObj.templateId = this._defaultPortVirtualizationSelectorArray[configportgroupcounter].portVirtualizationTemplateSelector.get("value"); //this._defaultPortVirtualizationTemplateSelector.get("value");
                    portvirtObj.type = "PortVirtualization";
                    portvirtObj.portNumber = this._adapterControllerObj[configportgroupcounter].portNumber;
                    adapterJson.templates.push(portvirtObj);
                }
                
                // Initial port extended settings function template
                if (this._rowAttachPointList.portExtendedSettingsList[configportgroupcounter].style.display != "none") {
                    var pesObj = new Object();
                    pesObj.templateId = this._defaultPortVirtualizationSelectorArray[configportgroupcounter].portExtendedSettingsTemplateSelector.get("value");
                    pesObj.type = "PortExtendedSettings";
                    pesObj.portNumber = this._adapterControllerObj[configportgroupcounter].portNumber;
                    //pesObj.physicalFunction = "nic";
                    adapterJson.templates.push(pesObj);
                }                
            
                // Initial NIC function template
                if (this._rowAttachPointList.ethernetList[configportgroupcounter].style.display != "none") {
                    var nicObj = new Object();
                    nicObj.templateId = this._defaultPortVirtualizationSelectorArray[configportgroupcounter].ethernetPortTemplateSelector.get("value");
                    nicObj.type = "PortPhysicalFunction";
                    nicObj.portNumber = this._adapterControllerObj[configportgroupcounter].portNumber;
                    nicObj.physicalFunction = "nic";
                    adapterJson.templates.push(nicObj);
                }
            
                // Initial FCoE function template
                if (this._rowAttachPointList.fcoeList[configportgroupcounter].style.display != "none") {
                    var fcoeObj = new Object();
                    fcoeObj.templateId = this._defaultPortVirtualizationSelectorArray[configportgroupcounter].fibreChanneloverEthernetTemplateSelector.get("value");
                    fcoeObj.type = "PortPhysicalFunction";
                    fcoeObj.portNumber = this._adapterControllerObj[configportgroupcounter].portNumber;
                    fcoeObj.physicalFunction = "fcoe";
                    adapterJson.templates.push(fcoeObj);
                }

                // Initial iSCSI function template
                if (this._rowAttachPointList.iscsiList[configportgroupcounter].style.display != "none") {
                    var iSCSIObj = new Object();
                    iSCSIObj.templateId = this._defaultPortVirtualizationSelectorArray[configportgroupcounter].iSCSIFunctionTemplateSelector.get("value");
                    iSCSIObj.type = "PortPhysicalFunction";
                    iSCSIObj.portNumber = this._adapterControllerObj[configportgroupcounter].portNumber;
                    iSCSIObj.physicalFunction = "iscsi";
                    adapterJson.templates.push(iSCSIObj);
                }            
            
                // Initial FC function template
                if (this._rowAttachPointList.fcList[configportgroupcounter].style.display != "none") {
                    var fibreObj = new Object();
                    fibreObj.templateId = this._defaultPortVirtualizationSelectorArray[configportgroupcounter].fibreChannelPortTemplateSelector.get("value");
                    fibreObj.type = "PortPhysicalFunction";
                    fibreObj.portNumber = this._adapterControllerObj[configportgroupcounter].portNumber;
                    fibreObj.physicalFunction = "fc";
                    adapterJson.templates.push(fibreObj);
                }
            }
            
            console.log(this.declaredClass + ": ", dojo.toJson(adapterJson));
            
            if (this.dialogMode == "assign") {
                dojo.publish("ChooseAdapterComplete", adapterJson);
            }
            else {
                dojo.publish("IOAdapterTemplateComplete", adapterJson);
            }

            this.hide();
        },
        
        _validateSelectors: function() {
            if (this._adapterType == "vfa") {
                // Verify that an Adapter pattern is selected (required for VFAs).
                if ((this._adapterTemplateRow.style.display == "none") || (this._defaultAdapterTemplateSelector.get("value") == "")) {
                    this.okButton.set("disabled", true);
                    return;
                }
            }
            for (var configportgroupcounter=0;configportgroupcounter < this._adapterControllerObj.length;++configportgroupcounter) { 
                if (this._rowAttachPointList.portVirtualizationList[configportgroupcounter].style.display != "none") {
                    if (this._defaultPortVirtualizationSelectorArray[configportgroupcounter].portVirtualizationTemplateSelector.get("value") == "") {
                        this.okButton.set("disabled", true);
                        return;
                    }
                }
                if (this._rowAttachPointList.ethernetList[configportgroupcounter].style.display != "none") {              
                    if (this._defaultPortVirtualizationSelectorArray[configportgroupcounter].ethernetPortTemplateSelector.get("value") == "") {
                        this.okButton.set("disabled", true);
                        return;
                    }
                }
                if (this._rowAttachPointList.fcoeList[configportgroupcounter].style.display != "none") {                
                    if (this._defaultPortVirtualizationSelectorArray[configportgroupcounter].fibreChanneloverEthernetTemplateSelector.get("value") == "") {
                        this.okButton.set("disabled", true);
                        return;
                    }
                }
                if (this._rowAttachPointList.iscsiList[configportgroupcounter].style.display != "none") {
                    if (this._defaultPortVirtualizationSelectorArray[configportgroupcounter].iSCSIFunctionTemplateSelector.get("value") == "") {
                        this.okButton.set("disabled", true);
                        return;
                    }
                }            
                if (this._rowAttachPointList.fcList[configportgroupcounter].style.display != "none") {
                    if (this._defaultPortVirtualizationSelectorArray[configportgroupcounter].fibreChannelPortTemplateSelector.get("value") == "") {
                        this.okButton.set("disabled", true);
                        return;
                    }
                }                
            }
            this.okButton.set("disabled", false);
        },

        _cancelDialog: function() {
            this.hide();
        }
    });
});
