define([ 
        "dojo", 
        "dijit", 
        "support/dialogs/Dialog", 
        "dojo/i18n!./nls/AddPlaceholderChassis",
        "dojo/text!./templates/AddPlaceholderChassis.html", 
        "dojo/parser",
        "dijit/form/SimpleTextarea", 
        "dijit/form/ValidationTextBox", 
        "dojo/data/ItemFileReadStore",
        "support/dialogs/ProgressDialog",
        "support/widgets/ProgressButton"
    ], function(dojo, dijit, dialog, mri, template) {
    
    return dojo.declare("support.dialogs.AddPlaceholderChassis", [ support.dialogs.Dialog ], {
        
        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
             
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,
        
        editTemplate: false,
        templateIDNum: null,
        
        baseClass: "AddPlaceholderChassis",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/AddPlaceholderChassis.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            this.addphchasName.intermediateChanges = true;
            if (this.editTemplate == true){
                this.dialogNode.set("title", this.locResources.EDIT_PH);
                this.addButton.set("label", this.resources.SAVE);
                this.loadPrevSettings();
            }
            
            dojo.connect(this.dialogNode, "hide", this, function() {
                this.destroy();
            });

        },
        
        loadPrevSettings: function() {
            
            var store = null;
            
            console.log(this.declaredClass + ": ", this.templateIDNum);
            
            dojo.xhrGet({
                url: this.REST_PATH + "phchassis/" + this.templateIDNum,
                handleAs: "json",
                sync: true,
                preventCache: true,
                load: function(response) {
                    console.log(this.declaredClass + ": ", response);
                    store = response;
                }
            });
            
            this.addphchasName.set("value", store.name);
            this.addphchasDesc.set("value", store.description);
            
        },
        
        _nameChanged: function() {
            if (this.addphchasName.get("value").trim() != "") {
                this.addButton.set("disabled", false);
            }
            else {
                this.addButton.set("disabled", true);
            }
        },
        
        _createJSONForNewPHChassis: function() {
            var phChassisObject = new Object();
            phChassisObject.name = this.addphchasName.value;
            phChassisObject.description = this.encodeSpecialCharacters(this.addphchasDesc.value);
                       
            var phChassisJSONString = dojo.toJson(phChassisObject);
            
            return phChassisJSONString;
        },
        
        
        _postNewPHChassis: function() {
            var jsonStr = this._createJSONForNewPHChassis();

            dojo.xhrPost({
                url : this.REST_PATH + "phchassis/create",
                handleAs: "json",
                sync: true,
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                postData: jsonStr,
                load: dojo.hitch(this, function(response, ioArgs) {
                    dojo.publish("addPHChassis", []);
                }),
                error : dojo.hitch(this, function(error, ioArgs) {
                    
                    var response = dojo.fromJson(ioArgs.xhr.responseText);
                    if ((ioArgs.xhr.status >= 400) && (response.message == "service_error_name_in_use")) {
                        // Error Template name in use
                        this.getModalDialog("Error", this.resources.ERROR_400_PCNAME).show();
                    }
                    else if(ioArgs.xhr.status == 404) {
                        // Edit Template ID doesn't exist
                        this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                        this.hide();
                    }
                    else if(ioArgs.xhr.status >= 500) {
                        // Internal Programming Error
                        this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                        this.hide();
                    }
                    else {
                        this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                        this.hide();
                    }
                }),
                handle: function(response, ioArgs) {
                    /*
                    if (ioArgs.xhr.status >= 300) {
                        dojo.publish("ProgressFailed", response);
                    }
                    else {
                        dojo.publish("ProgressComplete", response);
                    }
                    */
                }
            });
        },
        
        _putPHChassis: function() {
            var jsonStr = this._createJSONForNewPHChassis();

            var xhrArgs = {
                    url: this.REST_PATH + "phchassis/" + this.templateIDNum,
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    postData: jsonStr,
                    load: dojo.hitch(this, function(response) {
                        dojo.publish("addPHChassis", []);
                    }),
                    error : dojo.hitch(this, function(error, ioArgs) {
                        
                        var response = dojo.fromJson(ioArgs.xhr.responseText);
                        if ((ioArgs.xhr.status >= 400) && (response.message == "service_error_name_in_use")) {
                            // Error Template name in use
                            this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                        }
                        else if(ioArgs.xhr.status == 404) {
                            // Edit Template ID doesn't exist
                            this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                            this.hide();
                        }
                        else if(ioArgs.xhr.status >= 500) {
                            // Internal Programming Error
                            this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                            this.hide();
                        }
                        else {
                            this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                            this.hide();
                        }
                    })
            };
            dojo.xhrPut(xhrArgs);
        },
        
        _finishAddPhChassis: function() {
            console.log(this.declaredClass + ": Creating new Placeholder Chassis with name: " + this.addphchasName.value);
            if (this.editTemplate == true){
                this._putPHChassis();
            }
            else {
                this._postNewPHChassis();
            }
            
            this.hide();
        },
        
        _cancelAddPhChassis: function() {
            this.hide();
        }
    });
});
