define([ 
        "dojo", 
        "dijit", 
        "support/dialogs/Dialog", 
        "dojo/i18n!./nls/AssignChassisProfile",
        "dojo/text!./templates/AssignChassisProfile.html", 
        "dijit/form/Button", 
        "dojo/data/ItemFileWriteStore",
        "support/widgets/OutlinePane",
        "support/base/TableMixin",
        "dijit/layout/BorderContainer", 
        "dijit/layout/ContentPane",
        "support/dialogs/ProgressDialog"
    ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.AssignChassisProfile", [ support.dialogs.Dialog, support.base.TableMixin ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "AssignChassisProfile",

        // profileId: String
        // Value of entry to pre-select in this widget's template selector.
        profileId: null,
        profileName: "",
        init: false,

        _availTable: null,
        
        // ID prefix used for table
        gridID: "AssignChassisProfileTable",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/AssignChassisProfile.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in
        // the template HTML file
        postMixInProperties: function() {

            this.inherited(arguments);

        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be
        // directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);
            this._setupProfile();
            this._setupAvailableTable();

        },

        onFinish: function() {

            var rowIds = this._availTable.select.row.getSelected();
            var chassisId = null;
            if (rowIds.length == 1) {
                var selectedId = this.profileId;
                this.getProgressDialog("activate").show(); // Show progress dialog for activate
                                                            // request
                
                // obtain the full row item
                var item = this._availTable.row(rowIds[0], true).item();
                // send the chassis UUID to the profile/assign API
                chassisId = item.uuid;
                
                var requestData = {
                    profileId: selectedId,
                    endPointType: "CMM",
                    endPointId: chassisId[0]
                };
                var jsonString = dojo.toJson(requestData);
                    
                dojo.xhrPost({
                    url: this.REST_PATH + "profile/assign",
                    handleAs: "json",
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    postData: jsonString,
                    handle: dojo.hitch(this, function(response, ioArgs) {
                        // Inform progress dialog...
                        if (ioArgs.xhr.status >= 300) {
                            dojo.publish("ProgressFailed", response);
                            this.getModalDialog("Error", this.resources.ERROR_ASSIGN_PROF).show();
                            console.error("Failed to deploy chassis pattern.");
                        }
                        else {
                            dojo.publish("ProgressComplete", response);
                            if (response && response.error && (response.error.length > 0)) {
                                this.getModalDialog("Error", response.error).show();
                                console.error("Failed to deploy chassis pattern.");
                            }
                            else {
                                dojo.publish("AssignChassisProfileComplete", response);
                                this._onClose();
                            }
                        }
                    })
                });
            }
            return false; // Don't allow close! Allow handlers (above) to determine when to close.

        },

        _setupProfile: function() {

            var title = this.locResources.PAGE_TITLE + " - " + this.profileName;
            this.set("title", title);
            this._assignButton.set("disabled", true);
            dojo.html.set(this._profName, this.profileName);

        },

        _changeProfileCallback: function(newVal) {

            var rows = this._availTable.select.row.getSelected().length;
            var si = (this._availTable != null) ? this._availTable.select.row.getSelected() : [];
            var rootFound = false;

            for (var idx in si) {
                var item = this._availTable.row(si[idx], true).item();
                if (item.children) {
                    rootFound = true;
                }
            }

            if (rows == 1 && !rootFound && this.profileId != null) {
                if (this._assignButton.get("disabled")) {
                    this._assignButton.set("disabled", false);
                }
            }
            else {
                // Disable Assign Button if more than 1 row selected
                this._assignButton.set("disabled", true);
            }

        },

        _setupAvailableTable: function() {

            // summary:
            // Set up available targets table widget...
            // tags:
            // protected
            if (this._availTable) {
                // Destroy old table and start from scratch.
                this._availTable.destroy();
            }
            
            // Setup table widget...
            var tableStore = this._retrieveStore();
            var tableStructure = this._getTableStructure();
            this._availTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.gridID, true);
            this._availTable.column(0).sort(false);
           
            setTimeout(dojo.hitch(this, function() {
                this._availTable.placeAt(this._availTableNode);
                this._availTable.startup();
            }), 500);
            
            // Connect handlers to reset table capabilities
            dojo.connect(this._availTable.select.row, "onSelectionChange", this, this._changeProfileCallback); // Fires on row selection
            dojo.connect(this._availTable, "setStore", this, this._changeProfileCallback); // Fires on store reset

        },     

        _accessFormatter: function(item) {
            // summary:
            // Formats "access" field value for display in TreeTable.
            // tags:
            // protected
            var access = item.access;
            var accessText = "";
            
            if (access == "ok") { // Access "OK"                
                accessText = this.resources.OK;
            }
            else if (access == "-Default-") { // Access "Default Access"            
                accessText = this.resources.ACCESS_DEFAULT;
            }
            else if (access == "offline") { // Access "Offline"
                accessText = this.resources.ACCESS_OFFLINE;
            }
            else if (access == "placeholder") { // Access "Placeholder"
                accessText = this.resources.PLACEHOLDER;
            }
            else if (access == "none") { // Access "No access"
                accessText = this.resources.ACCESS_NONE;
            }
            else if (access == "partial") { // Access "Partial access"
                accessText = this.resources.ACCESS_PARTIAL;
            }
            else /* (access == "unknown") */{ // Access "Unknown"
                accessText = this.resources.ACCESS_UNKNOWN;
            }
            
            return accessText;
        },

        _retrieveStore: function() {

            // summary:
            // Connect to data store and create models for TreeTable widgets.
            // tags:
            // protected

            var availStore = null;
            var availData = null;
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "chassis/unassigned",
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: dojo.hitch(this, function(response) {
                    // make identifier the "uuid" property instead of the "id"
                    //   property since the chassis/unassigned API sends the
                    //   OID as the "id"
                    availData = dojo.toJson({
                        "identifier": "uuid",
                        "label": "name",
                        "items": response.items
                    }, true);
                }),
                error: dojo.hitch(this, function(response) {
                    // if the REST API was unsuccessful in returning data,
                    // show an empty table
                    console.log(this.declaredClass + ": Unable to retrieve chassis from API: chassis/unassigned");                        

                    availData = dojo.toJson({
                        // make identifier the "uuid" property instead of the "id"
                        //   property since the chassis/unassigned API sends the
                        //   OID as the "id"
                        "identifier": "uuid",
                        "label": "name",
                        "items": []
                    }, true);                                                
                })
            });

            availStore = new dojo.data.ItemFileWriteStore({
                data: dojo.fromJson(availData)
            });                

            return availStore;
        },

        _getValidJsonData: function(jsonData) {
            // if any part of the file is not valid, return the default item
            if (jsonData == null || jsonData.length == 0) {
                return {
                    "items": []
                };
            }
            else {
                return {
                    "items": jsonData
                };
            }
        },
        
        _decorateNoWrap: function(item) {
            // tags:
            // protected
            var value = item;
            !value && (value = "");
            if (value !== "") {
                value = this.encoder(value);
                value = "<span title=\"" + value + "\" style=\"white-space: nowrap;\">" + value + "</span>";
            }
            return value;
        },

        // Columns to be displayed on table
        _getTableStructure: function() {
            var layout = new Array();
            
            layout.push({
                field: "name",
                name: this.resources.NAME,
                width: "250px",
                encoder: this.encodeForHtml, // pass value encoder to formatter function
                //formatter: dojo.hitch(this, "_formatName"),
                decorator: this._decorateNoWrap
            });
            layout.push({
                field: "ipaddresses",
                name: this.locResources.CMM_IP,
                width: "100px",
                encoder: this.encodeForHtml, // pass value encoder to formatter function
                //formatter: dojo.hitch(this, "_formatIPAddresses"),
                decorator: this._decorateNoWrap
            });
            layout.push({
                field: "access",
                name: this.resources.ACCESS,
                width: "auto",
                decorator: dojo.hitch(this, "_accessDecorator"),
                formatter: dojo.hitch(this, "_accessFormatter")
            });
            
            return layout;
        }

    });

});
