define([ 
        "dojo", 
        "dijit", 
        "support/dialogs/Dialog", 
        "dojo/i18n!./nls/AssignServerProfile",
        "dojo/text!./templates/AssignServerProfile.html", 
        "dijit/form/Select", 
        "dijit/form/Button", 
        "dojo/data/ItemFileWriteStore",
        "support/widgets/OutlinePane",
        "support/base/TableMixin",
        "support/temp/ModalDialog",
        "support/dialogs/ProgressDialog"
    ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.AssignServerProfile", [ support.dialogs.Dialog, support.base.TableMixin ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "AssignServerProfile",

        // profileId: String
        // Value of entry to pre-select in this widget's template selector.
        profileId: null,
        profileName: "",
        templateid:null,
        init: false,
        
        //FW Level State Names - Internal Usage
        FW_UNKNOWN: "unknown",
        FW_BACKLEVEL: "backlevel",
        FW_OK: "compatible",
        DEPLOY_OK: "ok",
        DEPLOY_NOTREADY: "notReady",
        DEPLOY_UNKNOWN: "unknown",
        DEPLOY_NOT_SUPPORTED: "notSupported",

        patternFormFactor: "",
        
        _availTable: null,
        
        // ID prefix used for table
        gridID: "AssignServerProfileTable",

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/AssignServerProfile.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in
        // the template HTML file
        postMixInProperties: function() {

            this.inherited(arguments);

        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be
        // directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);
            this._setupProfile();
            this._setupAvailableTable();

        },

        onFinish: function() {
            
            var selectedId = this.profileId;
            var rowIds = this._availTable.select.row.getSelected();
            var serverId = null;
            if (rowIds.length == 1) {
                var scheduleValue = null;
                if(this._schedule != null) {
                    scheduleValue = this._schedule.value;
                }
                
                // show the progress dialog
                this.getProgressDialog("activate").show(); // Show progress dialog for activate request
                
                var item = this._availTable.row(rowIds[0], true).item();
                serverId = item.location;
                
                var requestData = {
                    profileId: selectedId,
                    endPointType: "location",
                    endPointId: serverId[0],
                    schedule: scheduleValue
                };
                var jsonString = dojo.toJson(requestData);
                
                dojo.xhrPost({
                    url: this.REST_PATH + "profile/assign",
                    handleAs: "json",
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    sync: false,
                    postData: jsonString,
                    load: dojo.hitch(this,function(data) {
                        if(data && data.error && data.error.length > 0){
                            // regardless of the result, hide the progress
                            //   dialog
                            this.getProgressDialog("activate").hide();
                            var validationpopup = new support.temp.ModalDialog({
                                type: "Error",
                                text: data.error
                            });
                            validationpopup.show();
                        }
                        else{
                            dojo.publish("AssignServerProfileComplete");
                            // regardless of the result, hide the progress
                            //   dialog
                            this.getProgressDialog("activate").hide();
                            // must hide this dialog, since we're not relying
                            //   on the Dialog superclass to close it by
                            //   returning true from this function
                            this.hide();
                        }
                        
                    }),
                    error: dojo.hitch(this,function(data) {
                        // regardless of the result, hide the progress dialog
                        this.getProgressDialog("activate").hide();
                        var validationpopup = new support.temp.ModalDialog({
                            type: "Error",
                            text: this.resources.ERROR_ASSIGN_PROF
                        });
                        console.log(data);
                        validationpopup.show();
                    })
                });
            }
            
            // never close this dialog; the xhrGet(...) return functions will
            //   hide it
            return false;

        },

        _setupProfile: function() {

            var title = this.locResources.PAGE_TITLE + " - " + this.profileName;
            this.set("title", title);
            this._assignButton.set("disabled", true);
            dojo.html.set(this._profName, this.profileName);

        },

        _changeProfileCallback: function(newVal) {

            var rows = this._availTable.select.row.getSelected().length;
            var si = (this._availTable != null) ? this._availTable.select.row.getSelected() : [];
            var rootFound = false;

            for (var idx in si) {
                var item = this._availTable.row(si[idx], true).item();
                if (item.children) {
                    rootFound = true;
                }
            }

            if (rows == 1 && !rootFound && this.profileId != null) {
                if (this._assignButton.get("disabled")) {
                    this._assignButton.set("disabled", false);
                }
            }
            else {
                // Disable Assign Button if more than 1 row selected
                this._assignButton.set("disabled", true);
            }

        },

        _setupAvailableTable: function() {

            // summary:
            // Set up available targets table widget...
            // tags:
            // protected
            if (this._availTable) {
                // Destroy old table and start from scratch.
                this._availTable.destroy();
            }
            
            // Setup table widget...
            var tableStore = this._retrieveStore();
            var tableStructure = this._getTableStructure();
            this._availTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.gridID, true);
            
            // Set parent rows as "unselectable"
            this._availTable.disableParentRows();
            
            this._availTable.column(0).sort(false);
            this._availTable.select.row.multiple = false; // set table to single select
            
            setTimeout(dojo.hitch(this, function() {
                this._availTable.placeAt(this._availTableNode);
                this._availTable.startup();
            }), 500);            
            
            // Connect handlers to reset table capabilities
            dojo.connect(this._availTable.select.row, "onSelectionChange", this, this._changeProfileCallback); // Fires on row selection
            dojo.connect(this._availTable, "setStore", this, this._changeProfileCallback); // Fires on store reset

        },

        /**
         * Function used to assign correct icon for deploy status column based on deploy status
         */
        _deployDecorator: function(item) {
            var text = item;
            var iconClass = "";
            
            switch (text) {
                case this.resources.FW_COMPAT:
                    iconClass = "spSuccessIcon";
                    break;
                case this.resources.FW_BACK_LEVEL:    
                    iconClass = "spWarningIcon";
                    break;
                case this.locResources.NOT_SUPPORTED:
                    iconClass = "spWarningIcon";
                    break;
                case this.resources.FW_UNKNOWN:
                    iconClass = "spUnknownIcon";
                    break;
            }
            
            return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr><td>"
                    + "<img src=\"" + this._blankGif + "\" alt=\"\" class=\"" + iconClass + "\" title=\"" + text + "\"></img></td>"
                    + "<td><span style=\"margin-left:4px\" title=\"" + text + "\">" + text + "</span></td>"
                    + "</tr></tbody></table>";
        },
        
        _deployFormatter: function(item) {
            
            // summary:
            // Formats "deployStatus" field value for display.
            // tags:
            // protected
            if (!item.deployStatus || (item.deployStatus == "ignore")) {
                return "";
            }
            
            var fwLevel = item.deployStatus;
            var accessText = "";
            if(fwLevel == "ignore"){
                return [];
            }
            if (fwLevel == this.DEPLOY_OK) { // OK 
                accessText = this.resources.FW_COMPAT;
            }
            else if (fwLevel == this.DEPLOY_NOTREADY) { // NOT READY
                accessText = this.resources.FW_BACK_LEVEL;
            }
            else if (fwLevel == this.DEPLOY_NOT_SUPPORTED) { // NOT SUPPORTED
                accessText = this.locResources.NOT_SUPPORTED;
            }
            else /*(access == "unknown")*/ { // Unknown
                accessText = this.resources.FW_UNKNOWN;
            }            

            return accessText;
        },
        
        _bayFormatter: function(item) {
            var value = item.bays;
            if(value == null || value == undefined || value == ""){
                return "";
            }else{                
                return value;
            }
        },        
        
        _retrieveStore: function() {

            //Get server pattern form factor and other pattern information
            var svrTemplate = null;
            var availStore = null;
            if (this.templateId != null) {

                dojo.xhrGet({
                    url: this.REST_PATH + "template/" + this.templateId,
                    handleAs: "json",
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    sync: true,
                    preventCache: true,
                    load: function(response, ioArgs) {
                        if (response.items && (response.items.length > 0)) {
                            svrTemplate = response.items[0];
                        }
                    },
                    handle: function(response, ioArgs) {
                        console.log(this.declaredClass + ": Load template complete: ", response, ioArgs);
                    }
                });
                if (svrTemplate) {
                    this.patternFormFactor = svrTemplate.formFactor;
                }


            }

            // summary:
            // Connect to data store and create models for TreeTable widgets.
            // tags:
            // protected            
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "deploy/list/",
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: dojo.hitch(this, function(response) {
                    var rootArray = new Array();      //Holds chassis 
                    var childrenArray = new Array();  //Hold bays for UI display
                    
                    if (response && response.items) {
                        for (var ri = 0; ri < response.items.length; ++ri) { //Loop through Chassis
                            //Set deploy status fields and expand status fields if a child node.  Set root nodes to have deploy status ignored
                            var rootObject = response.items[ri];
                            rootObject.expandMe = "false";
                            rootObject.deployStatus = "ignore";
                            
                            if (rootObject.children) {   //Loop through Servers in Chassis
                                for (var ci = 0; ci < rootObject.children.length; ++ci) {                                        
                                    //If one bay has more then one compute node then loop through compute nodes (Caracara)
                                    if (rootObject.children[ci].ites.length > 1) {                                             
                                        for (var ii = 0; ii < rootObject.children[ci].ites.length; ++ii) {  //Loop compute nodes                                                
                                          //Set deploy status fields   
                                          rootObject.children[ci].ites[ii].deployStatus = this._determineDeployStatus(rootObject.children[ci].ites[ii]);
                                          rootObject.children[ci].ites[ii].deployMsg = this._getDeployMsg(rootObject.children[ci].ites[ii]);
                                          rootObject.children[ci].ites[ii].parentId = rootObject.id;
                                            
                                          //Check if pattern caracara, if so add this caracara node to list
                                          if (this.patternFormFactor == "1sd" && rootObject.children[ci].ites[ii].subBays.length != 0) {    
                                              //Set bay label with subnode indication
                                              rootObject.children[ci].ites[ii].bays[0] = rootObject.children[ci].ites[ii].bays[0] + ":" + rootObject.children[ci].ites[ii].subBays[0];                                                                                    
                                              childrenArray.push(rootObject.children[ci].ites[ii]);
                                          }
                                          
                                        }
                                        
                                    } else if(rootObject.children[ci].ites.length == 1) { //If bay has only one compute node (Everything except Caracara)
                                           //(Unless one node of the Caracara has already been deployed to)
                                        
                                        //Set deploy status fields                                               
                                        rootObject.children[ci].ites[0].deployStatus = this._determineDeployStatus(rootObject.children[ci].ites[0]);
                                        rootObject.children[ci].ites[0].deployMsg = this._getDeployMsg(rootObject.children[ci].ites[0]);
                                        rootObject.children[ci].ites[0].parentId = rootObject.id;                                                                                    

                                        if (this.patternFormFactor == rootObject.children[ci].ites[0].formFactor && rootObject.children[ci].ites[0].subBays.length == 0) {
                                          //Check if server takes up more than one slot (multi slot node)
                                            var slots = rootObject.children[ci].ites[0].bays.length;
                                            if(slots > 1) {
                                                rootObject.children[ci].ites[0].bays[0] = rootObject.children[ci].ites[0].bays[0] + "-" + rootObject.children[ci].ites[0].bays[slots-1];                                                                                    
                                            }
                                            childrenArray.push(rootObject.children[ci].ites[0]);
                                        }
                                        
                                        //If multi-slot server pattern selected then convert empty bays to multi-slot
                                        if (rootObject.children[ci].ites[0].access == "empty" && rootObject.children[ci].ites[0].subBays.length == 0) {
                                            switch (this.patternFormFactor) {
                                                case "4s":
                                                case "pxu":
                                                case "2sc":
                                                    if (rootObject.children[ci].ites[0].bays[0] % 2) {  //Check all odd bays first (logic here is that if the odd bay is not 
                                                        var oddBay = rootObject.children[ci].ites[0].bays[0]; //Get odd bay
                                                        var evenBay = parseInt(oddBay) + 1; //Get even bay
                                                        
                                                        for (var c=0;c <rootObject.children.length;c++) { //Loop through all bays to find oddbay + 1
                
                                                            if (rootObject.children[c].ites.length == 1) {  //Only search compute node 1 empty bays (which should be all thats there empty)
                                                                if (rootObject.children[c].ites[0].bays[0] == evenBay) { //We found a match
                                                                    if (rootObject.children[c].ites[0].access == "empty") { //If even bay after the odd bay is empty then add the odd bay to the list
                                                                        rootObject.children[ci].ites[0].bays[0] = rootObject.children[ci].ites[0].bays[0] + "-" + evenBay;
                                                                        rootObject.children[ci].ites[0].formFactor = this.patternFormFactor; // convert empty bay form factor value
                
                                                                        childrenArray.push(rootObject.children[ci].ites[0]);                                      
                                                                    }                                                                                                                            
                                                                }                                                            
                                                            }
                                                           
                                                        }
                                                        
                                                    }
                                                    break;
                                                case "4sc":
                                                    var bayToTest = parseInt(rootObject.children[ci].ites[0].bays[0]);
                                                    if (bayToTest % 2 && bayToTest < 12) {  //Check all odd bays first (logic here is that if the odd bay is not 
                                                        var lastBay = parseInt(bayToTest) + 3; //Get last bay number in possible complex
                                                        var possibleComplex = 0;
                                                        for (var c=0;c <rootObject.children.length;c++) { //Loop through all bays to find bays in complex
                                                            if(rootObject.children[c].ites.length == 1) {  //Only search compute node 1 empty bays (which should be all thats there empty)
                                                                if (parseInt(rootObject.children[c].ites[0].bays[0]) <= lastBay && 
                                                                        parseInt(rootObject.children[c].ites[0].bays[0]) > bayToTest) { //check to see if bay falls in possible complex
                                                                    if (rootObject.children[c].ites[0].access == "empty") { 
                                                                        possibleComplex++;
                                                                    }                                                                                                                            
                                                                }                                                            
                                                            }
                                                        }
                                                        if (possibleComplex == 3) { //If all bays in complex range are empty
                                                            rootObject.children[ci].ites[0].bays[0] = rootObject.children[ci].ites[0].bays[0] + "-" + lastBay;
                                                            rootObject.children[ci].ites[0].formFactor = this.patternFormFactor; // convert empty bay form factor value
                                                            childrenArray.push(rootObject.children[ci].ites[0]);                                      
                                                        }
                                                    }
                                                    break;
                                                case "8sc":
                                                    var bayToTest = parseInt(rootObject.children[ci].ites[0].bays[0]);
                                                    if (bayToTest % 2 && bayToTest < 12) {  //Check all odd bays first (logic here is that if the odd bay is not 
                                                        var lastBay = parseInt(bayToTest) + 7; //Get last bay number in possible complex
                                                        var possibleComplex = 0;
                                                        for (var c=0;c <rootObject.children.length;c++) { //Loop through all bays to find bays in complex
                                                            if(rootObject.children[c].ites.length == 1) {  //Only search compute node 1 empty bays (which should be all thats there empty)
                                                                if (parseInt(rootObject.children[c].ites[0].bays[0]) <= lastBay && 
                                                                        parseInt(rootObject.children[c].ites[0].bays[0]) > bayToTest) { //check to see if bay falls in possible complex
                                                                    if (rootObject.children[c].ites[0].access == "empty") { 
                                                                        possibleComplex++;
                                                                    }                                                                                                                            
                                                                }                                                            
                                                            }
                                                        }
                                                        if (possibleComplex == 7) { //If all bays in complex range are empty
                                                            rootObject.children[ci].ites[0].bays[0] = rootObject.children[ci].ites[0].bays[0] + "-" + lastBay;
                                                            rootObject.children[ci].ites[0].formFactor = this.patternFormFactor; // convert empty bay form factor value
                                                            childrenArray.push(rootObject.children[ci].ites[0]);                                      
                                                        }
                                                    }
                                                    break;
                                                default:
                                                    break;                                            
                                            }
                                        }
                                        
                                        //If one bay server has subbays then we can say that it is a Caracara with only one compute node available 
                                        //(meaning that the other compute node was already deployed to)
                                        if (this.patternFormFactor == "1sd" && rootObject.children[ci].ites[0].subBays.length != 0) {
                                            rootObject.children[ci].ites[0].bays[0] = rootObject.children[ci].ites[0].bays[0] + ":" + rootObject.children[ci].ites[0].subBays[0];
                                            childrenArray.push(rootObject.children[ci].ites[0]); //Add Caracara with only one compute node available                                      

                                        } else if(this.patternFormFactor == "1sd" && rootObject.children[ci].ites[0].access == "empty") {
                                            //If 1sd pattern selected (Caracara) and bay empty then show empty bays in 1 bay 2 compute node format (caracara format)

                                            //Get bay information
                                            var location = rootObject.children[ci].ites[0].location;
                                            var id = rootObject.children[ci].ites[0].id; 
                                            var name = rootObject.children[ci].ites[0].name; 
                                            var bays = rootObject.children[ci].ites[0].bays[0]; 
                                            
                                            //Clone bay information
                                            var secondServer = dojo.clone(rootObject.children[ci].ites[0]); 
                                            
                                            //Create empty compute node 1
                                            rootObject.children[ci].ites[0].location = location + ":1";
                                            rootObject.children[ci].ites[0].id = id + ":1";
                                            rootObject.children[ci].ites[0].name = name + ":1";
                                            rootObject.children[ci].ites[0].formFactor = "1sd";
                                            rootObject.children[ci].ites[0].bays[0] = bays + ":1"; 
                                            rootObject.children[ci].ites[0].subBays[0] = "1";

                                            childrenArray.push(rootObject.children[ci].ites[0]);                                      

                                            //Create empty compute node 2
                                            secondServer.location = location + ":2";
                                            secondServer.id = id + ":2";
                                            secondServer.name = name + ":2";
                                            secondServer.formFactor = "1sd";
                                            secondServer.bays[0] = bays + ":2";
                                            secondServer.subBays[0] = "2";

                                            childrenArray.push(secondServer);

                                        }

                                    }
                                }
                                
                                rootObject.children = childrenArray; //Add new children array with only pattern specific targets
                            }
                                                            
                            if (rootObject.children.length > 0) {
                                rootArray.push(rootObject);  //Add Chassis to json
                            }
                            childrenArray = new Array(); //Reset Bay info for next chassis
                        }
                    }

                    var patternData = dojo.toJson({
                        "identifier": "id",
                        "label": "name",
                        "items": rootArray
                    }, true);
                    
                    availStore = new dojo.data.ItemFileWriteStore({
                        data: dojo.fromJson(patternData)
                    });
                    availStore.comparatorMap = {};
                    availStore.comparatorMap["name"] = function (a, b) {                        
                        return a.localeCompare(b);
                    };
                }),
                error: dojo.hitch(this, function(response) {
                    // if the REST API was unsuccessful in returning data,
                    // show an empty table
                    console.log(this.declaredClass + ": Unable to retrieve server templates from API: deploy/list/");
                    
                    var templateData = dojo.toJson({
                        "identifier": "id",
                        "label": "name",
                        "items": []
                    }, true);
                    
                    availStore = new dojo.data.ItemFileWriteStore({
                        data: dojo.fromJson(templateData)
                    });
                })                    
            });
            
            if (!("hasChildren" in availStore)) {
                availStore.hasChildren = function(id, item) { 
                    return item && this.getValues(item, "children").length;
                };
            };
            
            if (!("getChildren" in availStore)) {
                availStore.getChildren = function(item) { 
                    return this.getValues(item, "children");
                };
            };
                        
            return availStore;

        },
        
        /**
         * Function used to determine deploy status of servers based on FW levels and access states.
         * 
         * item: Singular object representing a server
         */
        _determineDeployStatus: function(item, /*String (optional)*/checkFormFactor) {
            
            if (checkFormFactor && item.formFactor && (checkFormFactor != item.formFactor.toString())) {
                return this.DEPLOY_NOT_SUPPORTED;
            }
            
            var fwLevel = (item.firmwareLevel) ? item.firmwareLevel.toString() : "";
            if (fwLevel == this.FW_UNKNOWN) {
                return this.DEPLOY_UNKNOWN;   
            }
            if (fwLevel == this.FW_BACKLEVEL) {
                return this.DEPLOY_NOTREADY;   
            }
            
            var access = (item.access) ? item.access.toString() : "";
            if (access == "offline" || access == "partial" || access == "unknown" || access == "-Default-" ) {
                return this.DEPLOY_UNKNOWN;
            }
            if (access == "none") {
                return this.DEPLOY_NOTREADY;
            }
           
            return this.DEPLOY_OK;
        },
        
        /**
         * Function to generate deploy message tooltip content based on firmware levels and access states
         * 
         * item: Singular object representing a server
         */
        _getDeployMsg: function(item){

            if(item.firmwareLevel){
                if(item.firmwareLevel == this.FW_UNKNOWN){
                    return this.resources.FW_UNKOWN_MSG;   
                }
                if(item.firmwareLevel == this.FW_BACKLEVEL){
                    return this.resources.FW_BACKLEVEL_MSG;  
                }
            }
            if(item.access){
                if(item.access == "offline"){
                    return this.resources.ACCESS_OFFLINE + " - " + this.resources.ACCESS_CIM_CONFIRM;
                } 
                else if(item.access == "partial"){
                    return this.resources.ACCESS_PARTIAL + " - " + this.resources.ACCESS_CIM_CONFIRM;
                }
                else if(item.access == "unknown"){
                    return this.resources.ACCESS_UNKNOWN + " - " + this.resources.ACCESS_CIM_CONFIRM;
                } 
                else if(item.access == "-Default-"){
                    return this.resources.ACCESS_DEFAULT + " - " + this.resources.ACCESS_CIM_CONFIRM;
                } 
                else if(item.access == "none"){
                    return this.resources.ACCESS_NONE + " - " + this.resources.ACCESS_CIM_CONFIRM;
                }
            }
           
            return this.resources.FW_COMPAT;
        },

        _getValidJsonData: function(jsonData) {
            // if any part of the file is not valid, return the default item
            if (jsonData == null || jsonData.length == 0) {
                return {
                    "items": []
                };
            }
            else {
                return {
                    "items": jsonData
                };
            }
        },
        
        _decorateNoWrap: function(item) {
            // tags:
            // protected
            
            var value = item;
            !value && (value = "");
            if (value !== "") {
                value = this.encoder(value);
                value = "<span title=\"" + value + "\" style=\"white-space: nowrap;\">" + value + "</span>";
            }
            return value;
        },
        
        _formatName: function(item) {
            if (!item.name) {
                return "";
            }
            
            var text = "";
            if (item.access == "empty") {
                text = this.resources.ACCESS_EMPTY;
            } else {
                text = item.name;
            }
            return text;
        },

        // Columns to be displayed on table
        _getTableStructure: function() {
            var layout = new Array();
            
            layout.push({
                field: "name",
                name: this.resources.NAME,
                width: "250px",
                encoder: this.encodeForHtml, // pass value encoder to formatter function
                decorator: this._decorateNoWrap,
                formatter: dojo.hitch(this, "_formatName")
            });
            layout.push({
                field: "bays",
                name: this.resources.BAYS,
                width: "45px",
                formatter: dojo.hitch(this, "_bayFormatter")                
            });
            layout.push({
                field: "deployStatus",
                name: this.resources.FW_DEPLOY_STATUS,
                width: "auto",
                decorator: dojo.hitch(this, "_deployDecorator"),
                formatter: dojo.hitch(this, "_deployFormatter")
            });

            return layout;
        },

        _createChild: function(childObj, schedule) {
            return {
                "access": childObj.access,
                "schedule": (schedule) ? schedule : childObj.schedule,
                "type": childObj.type,
                "name": childObj.name,
                "id": childObj.id,
                "bays": childObj.bays,
                "parentId": childObj.parentId
            };
        },

        _createParent: function(parentObj) {
            return {
                "access": parentObj.access,
                "type": parentObj.type,
                "name": parentObj.name,
                "id": parentObj.id,
                "expandMe": parentObj.expandMe,
                "children": []
            };
        }
        
    });

});
