define(["dojo",
        "dijit",
        "support/dialogs/Dialog",
        "dojo/i18n!./nls/BuildAListPopup",
        "dojo/text!./templates/BuildAListPopup.html",
        "dojo/parser",
        "dijit/Tooltip",
        "dijit/form/Select",
        "support/widgets/OutlinePane",
        "dojo/data/ItemFileReadStore",
        "dojox/grid/LazyTreeGridStoreModel",
        "dijit/layout/ContentPane",
        "support/widgets/DoubleSelects",
        "support/widgets/ToggleButton",
        "support/widgets/ButtonBar",
        "dijit/Tooltip",
        "support/temp/ModalDialog"
        ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.BuildAListPopup", [ support.dialogs.Dialog ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
                 
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,
        
        baseClass: "BuildAListPopup",
        
        title: null,
        item: null,
        paths : new Array(),
        path: null,
        settings: null,
        

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/BuildAListPopup.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            
            this.paths = new Array();
            this.showSettings();
            this.set("settings", this.settings);
            
            dojo.connect(this.dialogNode, "hide", this, function() {
                this.dialogNode.destroy();
                this.dialogNode.destroyRecursive();
            });

            this.reposition();
            
          //  console.log(this.declaredClass + ": HERE SETTINGS");
          //  console.log(this.declaredClass + ": ", this.settings);
        },
        
        showSettings : function(){
            
            //expect only one level of settings
            if(this.hasChildren(this.item) > 0){
                var children = this.getChildren(this.item);
                for(var c=0; c<children.length; c++){ //returns child groups
                    var child = children[c];
                    for(var g=0; g<child.length; g++){ //iterate through each child in group
                        var item = child[g];
                        var itemPath = this.path + "__" + item.ID[0];
                        
                        if(item.ID[0] != "del"){
                        
                            if(item.text_data != null)
                                this.showOption(item, itemPath, "text");
                            else if(item.numeric_data != null)
                                this.showOption(item, itemPath, "numeric");
                            else if(item.enumerate_data != null)
                                this.showOption(item, itemPath, "enumerate");
                            else if(item.delim_data != null)
                                this.showOption(item, itemPath, "delim");
                            
                        }

                    }
                }
            }
        },
        
        _setSettingsAttr : function(s){
            if(s != null){
                for(var i=0; i<s.length; i++){
                    var item = s[i];
                    var path = item.path;
                    var value = item.value;
                    this.setValueFromDisplay(path, value);
                }
            }
        },
        
        showOption : function(item, path, type){
            this.paths.push(path);
            var label = item.name[0];
            var desc = item.desc[0];
            
            //overall node
            var node = document.createElement("div");
            node.setAttribute("id", path + "_node");
            
            //table layout
            var table = document.createElement("table");
            table.setAttribute("width", "100%");
            var tbody = document.createElement("tbody");
            var row1 = document.createElement("tr");
            var labelCell = document.createElement("td");
            var controlCell = document.createElement("td");
            table.appendChild(tbody);
            tbody.appendChild(row1);
            row1.appendChild(labelCell);
            row1.appendChild(controlCell);
            controlCell.setAttribute("style", "align:right");
            
            //label
            var labelNode = document.createElement("span");
            labelNode.innerHTML = label;
            labelNode.onmouseover = function(e){
                dijit.showTooltip("<div style=\"max-width:300px\">" + desc + "</div>", labelNode, ["above"]);
            };
            labelNode.onmouseout = function(e){
                dijit.hideTooltip(labelNode);
            };
            
            //option
            var optionNode = document.createElement("div");
            optionNode.setAttribute("id", path + "_optionNode");
            optionNode.setAttribute("style", "display:inline-block; float:right;");
            var option = this.getOptionInput(item, path, type);
            if(option != null){
                option.placeAt(optionNode);
                optionNode.onmouseover = function(e){
                    dijit.showTooltip("<div style=\"max-width:300px\">" + desc + "</div>", optionNode, ["above"]);
                };
                optionNode.onmouseout = function(e){
                    dijit.hideTooltip(optionNode);
                };
            }
            
            //append
            labelCell.appendChild(labelNode);
            controlCell.appendChild(optionNode);
            node.appendChild(table);
            this._node.appendChild(node);
            
            this.setValueFromDisplay(path, this.settings[path]);
            
        },
        
        getOptionInput : function(item, path, type){
            if(type == "text"){
                return this.getTextInput(item, path);
            }
            else if(type == "numeric"){
                return this.getNumericInput(item, path);
            }
            else if(type == "enumerate"){
                return this.getEnumerateInput(item, path);
            }else if(type == "delim"){
                var delim_data = item.delim_data[0];
                if(delim_data){
             
                    if(dijit.byId(path))   
                        dijit.byId(path).destroy();                                 

                    return new support.widgets.ToggleButton({
                        id: path,
                        value: [true, false]
                    });
                }
            }
        },
        
        getEnumerateInput : function(item, path){
            var isNested = false;
            var choice = null;
            var enumerate_data = item.enumerate_data[0];
            if(enumerate_data.choice != null)
                choice = enumerate_data.choice;
            if(choice != null){
                if(choice[0].label != null){                    
                    var options = new Array();
                    for(var i=0; i<choice.length; i++){
                        var option = choice[i];
                        options.push({name:option.label[0]+"", id:option.value[0]+""});
                    }
                    var store = new dojo.store.Memory({ data: options });
//                    var controlValue = options[0].id;
                    var controlValue = null;
                    var multiSelect = false;
                    
                    for(var i=0; i<choice.length; i++){
                        var option = choice[i];
                        if(option.enumerate_data != null){
                            isNested = true;
                            multiSelect = false;
                        }
                        if(option.list_data != null){
                            isNested = true;
                            multiSelect = true;
                        }
                    }
                    if(isNested){
                        return this.getDoubleEnumerateInput(item, path, multiSelect);
                    }
                    else{
                        //toggle button
                        if(choice.length == 2){
                            var label1 = choice[0].label[0];
                            var label2 = choice[1].label[0];
                            if( (label1 == this.resources.ENABLED && label2 == this.resources.DISABLED) || 
                                (label1 == this.resources.ENABLE && label2 == this.resources.DISABLE)){
                                var enabled = false;
                                if(choice[0]["default"] != null)
                                    enabled = true;
                                
                                if(dijit.byId(path))   
                                    dijit.byId(path).destroy();                                 

                                return new support.widgets.ToggleButton({
                                    id: path,
                                    value: [choice[0].value[0], choice[1].value[0]],
                                    enabled: enabled
                                });
                            } 
                            if( (label2 == this.resources.ENABLED && label1 == this.resources.DISABLED) ||
                                (label2 == this.resources.ENABLE && label1 == this.resources.DISABLE) ){
                                var enabled = false;
                                if(choice[1]["default"] != null)
                                    enabled = true;
                                
                                if(dijit.byId(path))   
                                    dijit.byId(path).destroy();                                 

                                return new support.widgets.ToggleButton({
                                    id: path,
                                    value: [choice[1].value[0], choice[0].value[0]],
                                    enabled: enabled
                                });
                            }
                        }
                        
                      //2 button bar
                        if(choice.length == 2 && (choice[0].label[0].length + choice[1].label[0].length < 10) ){
                            var values = choice[0].value[0] + "|" + choice[1].value[0];
                            var labels = choice[0].label[0] + "|" + choice[1].label[0];
                            
                            if(dijit.byId(path))   
                                dijit.byId(path).destroy();                                 

                            var twoButtons = new support.widgets.ButtonBar({
                                id: path,
                                labels: labels,
                                values: values,
                                _destroyOnRemove: true
                            });
                            for(var i=0; i<choice.length; i++){
                                if(choice[i]["default"] != null)
                                    twoButtons.set("selected", choice[i].value[0]);
                            }
                            return twoButtons;
                        }
                        //3 button bar
                        else if(choice.length == 3 &&
                                (choice[0].label[0].length + choice[1].label[0].length + choice[2].label[0].length < 15) ){
                            var values = choice[0].value[0] + "|" + choice[1].value[0] + "|" + choice[2].value[0];
                            var labels = choice[0].label[0] + "|" + choice[1].label[0] + "|" + choice[2].label[0];

                            if(dijit.byId(path))   
                                dijit.byId(path).destroy();                                 

                            var threeButtons = new support.widgets.ButtonBar({
                                id: path,
                                values: values,
                                labels: labels,
                                _destroyOnRemove: true
                            });
                            for(var i=0; i<choice.length; i++){
                                if(choice[i]["default"] != null)
                                    threeButtons.set("selected", choice[i].value[0]);
                            }
                            return threeButtons;
                        }
                        else{ //normal drop down
                            
                            if(dijit.byId(path))   
                                dijit.byId(path).destroy();                                 

                            
                            var dropDown = new dijit.form.FilteringSelect({
                                id: path,
                                required: false,
                                store: store,
                                searchAttr: "name",
                                value: controlValue,
                                _destroyOnRemove: true
                            });
                            for(var i=0; i<choice.length; i++){
                                if(choice[i]["default"] != null){
                                    dropDown.set("value", choice[i].value[0]);
                                }
                            }
                            
                            dropDown.textbox.readOnly = true;
                            return dropDown;
                        }
                    }
                }
            }
        },
        
        getDoubleEnumerateInput : function(item, path, multi){
            
            var choice = item.enumerate_data[0].choice;
            var controlValue = null;
            if(choice != null){
                var options = new Array();
                for(var i=0; i<choice.length; i++){
                    var option = choice[i];
                    options.push({name:option.label[0]+"", id:option.value[0]+""});
                }
                var store1 = new dojo.store.Memory({ data: options });
                /*if(controlValue == null){
                    controlValue = options[0].id;
                } */        
                
                if(dijit.byId(path))   
                    dijit.byId(path).destroy();                                 

                
                var dropDowns = new support.widgets.DoubleSelects({
                    id: path,
                    store1: store1,
                    multi: multi,
                    onChange: function(newValue){
                        var showOptions = false;
                        for(var i=0; i<choice.length; i++){
                            var option = choice[i];
                            if( option.value != null && newValue == option.value[0] && 
                               (option.enumerate_data != null || option.list_data != null) ){
                                showOptions = true;
                                var choice2 = null;
                                var options2 = new Array();
                                var store2 = null;
                                
                                if(option.enumerate_data != null){
                                    choice2 = option.enumerate_data[0].choice;
                                    for(var c=0; c<choice2.length; c++){
                                        var option2 = choice2[c];
                                        var object2 = { name: option2.label[0] + "", id: option2.value[0] + "" };
                                        options2.push(object2);
                                    }
                                    store2 = new dojo.store.Memory({
                                        data: options2
                                    });
                                }
                                else if(option.list_data != null){
                                    choice2 = option.list_data[0].choice;
                                    for(var c=0; c<choice2.length; c++){
                                        var option2 = choice2[c];
                                        var object2 = { label: option2.label[0] + "", value: option2.value[0] + "" };
                                        options2.push(object2);
                                    }
                                    store2 = options2;
                                }
                                
                                if(choice2 != null && store2 != null){
                                    this.set("store2", store2);
                                    this.set("value2", this.value2);
                                    this.set("disabled", false);
                                }
                            }
                        }
                        if(!showOptions){
                            this.set("store2", null); 
                            this.set("value2", null);
                            this.set("disabled", true);
                        }
                    }
                });
                dropDowns.set("value1", controlValue);
                return dropDowns;
            }            
            return null;
        },
        
        getNumericInput : function(item, path){
            var numeric_data = item.numeric_data[0];
            var min = 0, max = null, def = "", 
                step = null, type = "dec", controlValue = null;
            if(numeric_data.min != null)
                min = numeric_data.min[0];
            if(numeric_data.max != null)
                max = numeric_data.max[0];
            if(numeric_data.step != null)
                step = numeric_data.step[0];
            if(numeric_data["default"] != null)
                def = numeric_data["default"][0];
            if(numeric_data.type != null)
                type = numeric_data.type[0];
            if(controlValue == null)
                controlValue = def;
            
            if(dijit.byId(path))   
                dijit.byId(path).destroy();                                 

            
            return new dijit.form.ValidationTextBox({
                id: path,
                trim:true,
                value: controlValue,
                intermediateChanges: true,
                invalidMessage: this.resources["INVALID_BETWEEN"].replace("%1", min).replace("%2", max),
                validator: function(value, constraints){
                    if(value.length == 0)
                        return true;
                    if(isNaN(value))
                        return false;
                    return parseInt(value) >= min && parseInt(value) <= max;
                }
            });
        },
        
        getTextInput : function(item, path){
            var text_data = item.text_data[0];
            var minchars = 0, maxchars = null, pattern = ".*", 
                boxType = "text", def = "", controlValue = null;
            if(text_data.minchars != null)
                minchars = text_data.minchars[0];
            if(text_data.maxchars != null)
                maxchars = text_data.maxchars[0];
            if(text_data.pattern != null)
                pattern = text_data.pattern[0];
            if(text_data["default"] != null)
                def = text_data["default"][0];
            if(text_data.type != null){
                boxType = text_data.type[0];
                if(boxType == "encrypt")
                    boxType = "password";
                else
                    boxType = "text";
            }
            if(controlValue == null)
                controlValue = def;
            
            var widget = this;
            
            if(dijit.byId(path))   
                dijit.byId(path).destroy();                                 

            
            var textbox = new dijit.form.ValidationTextBox({
                id: path,
                trim:true,
                type: boxType,
                value: controlValue,
                intermediateChanges: true,
                regExp: pattern,
                _destroyOnRemove: true,
                validator: dojo.hitch(this, function(value, constraints){
                    if(value.length == 0){
                        return true;
                    }
                    if(minchars != null && maxchars != null){
                        if(value.length < minchars || value.length > maxchars){
                            this.invalidMessage = widget.resources["INVALID_BETWEEN"].replace("%1", minchars).replace("%2", maxchars);
                            return false;
                        }
                    }
                    if(minchars != null && maxchars == null){
                        if(value.length < minchars){
                            this.invalidMessage = widget.resources["INVALID_AT_LEAST"].replace("%1", minchars);
                            return false;
                        }
                    }
                    if(pattern != null && value != ""){
                        var reg = new RegExp(pattern);
                        if(!reg.test(value)){
                            this.invalidMessage = widget.resources.INVALID;
                            return false;
                        }
                    }
                    return true;
                })
            });
            if(maxchars != null)
                textbox.set("maxLength", maxchars);
            return textbox;
        },
        
        hasChildren : function(item){
            var count = 0;
            if(item.group != null)
                count++;
            if(item.command != null)
                count++;
            if(item.option != null)
                count++;
            if(item.Definitions != null)
                count++;
            return count;
        },
        
        getChildren : function(item){
            var children = new Array();
            if(item.Definitions != null)
                children.push(item.Definitions);
            if(item.group != null) 
                children.push(item.group);
            if(item.command != null) 
                children.push(item.command);
            if(item.option != null)
                children.push(item.option);
            //AT THE MOMENT, WILL ONLY RETURN ONE SET OF EACH CHILD
            return children;
        },
        
        reposition : function(){
            this.dialogNode._position();
        },
        checkValidation: function(){
            //Assume validation is true until proven false
            var check = true;
            
            for(var i=0; i<this.paths.length; i++){
                var path = this.paths[i];                
                var widget = dijit.byId(path);
                if(widget != null){
                    var declaredClass = widget.declaredClass;
                    if(declaredClass == "dijit.form.ValidationTextBox"){
                        if(widget.isValid() == false){
                            check = false;
                        }
                    }
                    
                    /*else if(declaredClass == "dijit.form.FilteringSelect"){
                        var value = this.getValueFromDisplay(path);
                        if(value == ""){
                            check = false;
                        }
                    }*/
                }

            }            
            
            return check;
        },
        getSettingsFromDialog : function(){
           // var settings = new Array();
            for(var i=0; i<this.paths.length; i++){
                var path = this.paths[i];
                var widget = dijit.byId(path);

                if(widget != null){
                    
                    var value = this.getValueFromDisplay(path);
                    var declaredClass = widget.declaredClass;
                    
                    //console.log(declaredClass);
                    //console.log(value);
                    
                    if((declaredClass == "dijit.form.FilteringSelect" || declaredClass == "support.widgets.DoubleSelects") && value == ""){
                        delete this.settings[path];
                    }else{
                        this.settings[path] = this.getValueFromDisplay(path);                        
                    }
                    
                }else{
                //var display = this.getDisplayValueFromDisplay(path);
                //var value = this.getValueFromDisplay(path);
                //settings.push({ path:path, display:display, value:value });
                    this.settings[path] = this.getValueFromDisplay(path);
                }
            }
            
            return this.settings;
            
        },
        
        setValueFromDisplay : function(path, value){
            var widget = dijit.byId(path);
            if(widget != null){
                var declaredClass = widget.declaredClass;
                if(declaredClass == "support.widgets.ButtonBar"){
                    widget.set("selected", value);
                }
                else if(declaredClass == "support.widgets.ToggleButton"){
                    widget.set("toggleValue", value);
                }
                else{
                    widget.set("value", value);
                }
            }
        },
        
        getValueFromDisplay : function(path){
            var widget = dijit.byId(path);
            //drop down, validation text box, number textbox
            if(widget != null){
                var declaredClass = widget.declaredClass;
                if(declaredClass == "support.widgets.ButtonBar"){
                    return widget.get("selected");
                }
                else if(declaredClass == "support.widgets.ToggleButton"){
                    return widget.get("toggleValue");
                }
                else{
                    return widget.get("value");
                }
            }
            return null;
        },
        
        getDisplayValueFromDisplay : function(path){
            var widget = dijit.byId(path);
            //drop down, validation text box, number textbox
            if(widget != null){
                var declaredClass = widget.declaredClass;
                if(declaredClass == "support.widgets.ButtonBar"){
                    return widget.get("selected");
                }
                else if(declaredClass == "support.widgets.ToggleButton"){
                    return widget.get("toggleValue");
                }
                else{
                    return widget.get("value");
                }
            }
            return null;
        },
        
        onSave : function(){
            //override if necessary
        },
        
        _saveandhideDialog: function() {
            if(this.checkValidation()){
                this.onSave();
                this.hide();
                this.dialogNode.destroyRecursive();              
            }else{                
                var validationpopup = new support.temp.ModalDialog({
                    type: "Error",
                    text: this.resources.VALIDCHECK
                });
                validationpopup.show();
            }
            
        },

        _cancelDialog: function() {
            this.hide();
            this.dialogNode.destroyRecursive();
        }
    });
});
