define(["dojo",
		"dijit",
		"support/dialogs/Dialog",
        "dojo/i18n!./nls/ChassisIOModuleTemplate",
        "dojo/text!./templates/ChassisNodeTemplate.html",
        "dijit/Tooltip",
        "dijit/layout/BorderContainer",
        "dijit/layout/ContentPane",
        "dijit/form/Button",
        "dijit/form/CheckBox",
        "dijit/form/Form",
        "dijit/form/NumberTextBox",
        "dijit/form/RadioButton",        
        "support/widgets/Select",
        "dijit/form/SimpleTextarea",
        "dojo/data/ItemFileReadStore",
        "support/panels/CheckLabelButton",
        "support/widgets/ButtonBar",
        "support/panels/SimpleTable",
        "support/panels/CheckButton",
        "support/widgets/OutlinePane",
        "support/dialogs/NewIPPool",
        "support/dialogs/ReDeployServerTemplate",
        "support/temp/ModalDialog",
        "support/panels/CustomNamingTable",
        "support/dialogs/SaveAs",
        "support/dialogs/ProgressDialog"
        ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.ChassisNodeTemplate", [ support.dialogs.Dialog ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        widgetsInTemplate: true,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "ChassisNodeTemplate",

        templateIDNum: null,

        _ipv4selectbox: null,
        _ipv6selectbox: null,
        _ipv4rangeselectbox: null,
        _ipv6rangeselectbox: null,
        _ipv4store: null,
        _ipv6store: null,
        _ipv4rangestore: null,
        _ipv6rangestore: null,
        maxPercent: .35,
        _ipv4rangedefault: "any",
        _ipv6rangedefault: "any",
        _customnametable: null,

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/ChassisNodeTemplate.css",

        // When this method is called, all variables inherited from superclasses
        // are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in
        // the template HTML file
        postMixInProperties: function() {

            this.inherited(arguments);

        },

        // postCreate() is called after buildRendering(). This is useful to
        // override when
        // you need to access and/or manipulate DOM nodes included with your
        // widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified
        // can now be
        // directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);

            //var availHeight = window.screen.availHeight;
            //var maxHeight = availHeight * this.maxPercent;
            //this.ethSettingsPane.setAttribute("style", "max-height:" + maxHeight + "px");

            //this.dialogNode.resize();

            this._connectCheckLabelButtonFunctions();
            this._getIPPooldata();
            this._createSelectBoxes();
            this._setDefaults();

            if (this.templateIDNum != null) {
                this._readSettingsFromService();

                // Setup for template new, edit, or copy mode
                this._setup();
                this._expandall();

            }
            else {

                // Setup for template new, edit, or copy mode
                this._setup();

                this._selectall();
            }

            dojo.connect(this.dialogNode, "hide", this, function() {
                this.destroy();
            });

            this.subscribe("ippoolComplete", function(data) {
                if (isNaN(data) == true) {
                    data = this._ipv4selectbox.get("value");
                }

                this._getIPPooldata();

                var responsedata;
                var xhrGet = dojo.xhrGet({
                    url: this.REST_PATH + "ippool/" + data,
                    handleAs: "json",
                    preventCache: true,
                    sync: true,
                    load: function(response) {
                        // alert(dojo.toJson(response));
                        responsedata = response;
                    },
                    error: function(error) {
                        console.error(this.declaredClass + ": " + this.locResources.ERRORSERVICE + " " + error);
                        // Probably need to mock up some dummy data here
                    }

                });

                if (responsedata.protocol == "IPv4") {
                    this._ipv4selectbox.setStore(this._ipv4store);
                    this._ipv4selectbox.addOption({
                        value: "noselected",
                        label: "&mdash; " + this.locResources.NOPOOLSELECTED + " &mdash;",
                        selected: true
                    });                    
                    this._ipv4selectbox.set("value", data);
                    this._ipv4selectbox.set("disabled", false);
                    this._ipv4rangeselectbox.set("disabled", false);
                }
                else {
                    this._ipv6selectbox.setStore(this._ipv6store);
                    this._ipv6selectbox.addOption({
                        value: "noselected",
                        label: "&mdash; " + this.locResources.NOPOOLSELECTED + " &mdash;",
                        selected: true
                    });
                    this._ipv6selectbox.set("value", data);
                    this._ipv6selectbox.set("disabled", false);
                    this._ipv6rangeselectbox.set("disabled", false);
                }
            });
        },

        _connectCheckLabelButtonFunctions: function() {
            
        },

        _createSelectBoxes: function() {
            this._ipv4selectbox = new support.widgets.Select({
                name: this.get("id") + "_ipv4Select",
                store: this._ipv4store,
                maxHeight: -1,
                emptyLabel: "&mdash; " + this.locResources.NOPOOLDEFINED + " &mdash;",
                disabled: true
            }, this.get("id") + "_ipv4Select");
            this._ipv4selectbox.startup();

            if (this._ipv4selectbox.get("value") != "") {
                this._ipv4selectbox.addOption({
                    value: "noselected",
                    label: "&mdash; " + this.locResources.NOPOOLSELECTED + " &mdash;",
                    selected: true
                });
            }
            this._ipv4selectbox.set("value", "noselected");

            dojo.connect(this._ipv4selectbox, "onChange", this, function() {
                this._setIPRangeSelect("IPv4", this._ipv4rangedefault);
                if (this._ipv4selectbox.get("value") == "noselected") {
                    this._ipv4rangeselectbox.set("disabled", true);
                    dojo.replaceClass(this.ipv4editicon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                }
                else {
                    this._ipv4rangeselectbox.set("disabled", false);
                    dojo.replaceClass(this.ipv4editicon, "spEditIcon_v2", "spEditInactiveIcon_v2");
                }
            });

            this._ipv4rangeselectbox = new support.widgets.Select({
                name: this.get("id") + "_ipv4RangeSelect",
                store: this._ipv4rangestore,
                maxHeight: -1,
                emptyLabel: "&mdash; " + this.locResources.NOPOOLSELECTED + " &mdash;",
                disabled: true
            }, this.get("id") + "_ipv4RangeSelect");
            this._ipv4rangeselectbox.startup();

            this._ipv6selectbox = new support.widgets.Select({
                name: this.get("id") + "_ipv6Select",
                store: this._ipv6store,
                maxHeight: -1,
                emptyLabel: "&mdash; " + this.locResources.NOPOOLDEFINED + " &mdash;",
                disabled: true
            }, this.get("id") + "_ipv6Select");
            this._ipv6selectbox.startup();

            if (this._ipv6selectbox.get("value") != "") {
                this._ipv6selectbox.addOption({
                    value: "noselected",
                    label: "&mdash; " + this.locResources.NOPOOLSELECTED + " &mdash;",
                    selected: true
                });
            }

            this._ipv6selectbox.set("value", "noselected");

            dojo.connect(this._ipv6selectbox, "onChange", this, function() {
                this._setIPRangeSelect("IPv6", this._ipv6rangedefault);

                if (this._ipv6selectbox.get("value") == "noselected") {
                    this._ipv6rangeselectbox.set("disabled", true);
                    dojo.replaceClass(this.ipv6editicon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                }
                else {
                    this._ipv6rangeselectbox.set("disabled", false);
                    dojo.replaceClass(this.ipv6editicon, "spEditIcon_v2", "spEditInactiveIcon_v2");
                }
            });

            this._ipv6rangeselectbox = new support.widgets.Select({
                name: this.get("id") + "_ipv6RangeSelect",
                store: this._ipv6rangestore,
                emptyLabel: "&mdash; " + this.locResources.NOPOOLSELECTED + " &mdash;",
                maxHeight: -1
            }, this.get("id") + "_ipv6RangeSelect");
            this._ipv6rangeselectbox.startup();

        },

        _setIPRangeSelect: function(type, rangedefault) {
            var ipv4selectvalue = this._ipv4selectbox.get("value");
            var ipv4rangedata = null;
            var ipv6selectvalue = this._ipv6selectbox.get("value");
            var ipv6rangedata = null;

            if (rangedefault == null) {
                rangedefault = "any";
            }

            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "ippool/list",
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: function(response) {
                    var count;
                    var ipv4data = {
                        "identifier": "id",
                        "label": "name",
                        "items": []
                    };
                    var ipv6data = {
                        "identifier": "id",
                        "label": "name",
                        "items": []
                    };
                    for (count = 0; count < response.items.length; ++count) {
                        if (response.items[count].protocol == "IPv4") {
                            ipv4data.items.push(response.items[count]);
                        }
                        if (response.items[count].protocol == "IPv6") {
                            ipv6data.items.push(response.items[count]);
                        }
                    }
                    var ippoolcount = 0;
                    var rangeheader = {
                        "identifier": "id",
                        "label": "name",
                        "items": []
                    };
                    for (ippoolcount = 0; ippoolcount < ipv4data.items.length; ++ippoolcount) {
                        var rangecount = 0;
                        if (ipv4data.items[ippoolcount].id == ipv4selectvalue) {
                            for (rangecount = 0; rangecount < ipv4data.items[ippoolcount].ranges.length; ++rangecount) {
                                rangeheader.items.push(ipv4data.items[ippoolcount].ranges[rangecount]);
                                rangeheader.items[rangecount].name = ipv4data.items[ippoolcount].ranges[rangecount].range;
                            }
                        }
                    }
                    ipv4rangedata = rangeheader;

                    var rangeheaderipv6 = {
                        "identifier": "id",
                        "label": "name",
                        "items": []
                    };

                    for (ippoolcount = 0; ippoolcount < ipv6data.items.length; ++ippoolcount) {
                        var rangecount = 0;
                        if (ipv6data.items[ippoolcount].id == ipv6selectvalue) {
                            for (rangecount = 0; rangecount < ipv6data.items[ippoolcount].ranges.length; ++rangecount) {
                                rangeheaderipv6.items.push(ipv6data.items[ippoolcount].ranges[rangecount]);
                                rangeheaderipv6.items[rangecount].name = ipv6data.items[ippoolcount].ranges[rangecount].range;
                            }
                        }
                    }
                    ipv6rangedata = rangeheaderipv6;
                }
            });
            if (type == "IPv4") {
                this._ipv4rangestore = new dojo.data.ItemFileReadStore({
                    data: ipv4rangedata
                });

                this._ipv4rangeselectbox.setStore(this._ipv4rangestore);

                /*this._ipv4rangeselectbox.addOption({
                    value: "any",
                    label: this.locResources.ANYRANGE,
                    selected: true
                });

                this._ipv4rangeselectbox.addOption({
                    value: "chassis",
                    label: this.resources.BY_CHASSIS,
                    selected: true
                });*/
                this._ipv4rangeselectbox.set("value", rangedefault);
            }

            if (type == "IPv6") {
                this._ipv6rangestore = new dojo.data.ItemFileReadStore({
                    data: ipv6rangedata
                });

                this._ipv6rangeselectbox.setStore(this._ipv6rangestore);

                /*this._ipv6rangeselectbox.addOption({
                    value: "any",
                    label: this.locResources.ANYRANGE,
                    selected: true
                });

                this._ipv6rangeselectbox.addOption({
                    value: "chassis",
                    label: this.resources.BY_CHASSIS,
                    selected: true
                });*/
                this._ipv6rangeselectbox.set("value", rangedefault);
            }
        },

        _readSettingsFromService: function() {
            var settings = null;
            var mode = this.templateMode;

            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "template/" + this.templateIDNum,
                handleAs: "json",
                sync: true,
                load: function(response) {
                    settings = response;
                },
                error: function(error) {
                    console.error(this.declaredClass + ": " + this.locResources.ERRORSERVICE + " " + error);
                }
            });

            // Set dialog title to reflect Editing and display the warning message
            // this.ethwarningMessage.style.display = "block";

            this.templateInUse = settings.items[0].inUse;

            if (mode == "copy") {
                this.dialogNode.set("title", this.locResources.COPYTITLE_NODE + " - " + settings.items[0].name + " - "
                        + this.resources.COPY);
                this.templateName.set("value", settings.items[0].name + " - " + this.resources.COPY);
            }
            else {
                this.dialogNode.set("title", this.locResources.EDITTITLE_NODE + " - " + settings.items[0].name);
                this.templateName.set("value", settings.items[0].name);
            }
            var descriptionvalue = settings.items[0].description;
            var description = this.decodeSpecialCharacters(descriptionvalue);
            this.ethadapDesc.set("value", description);
            if (settings.items[0].CMM__ifconfig__c == "dhcp") {
                this.ipv4dynamicradiobutton.set("checked", true);
                this._ipv4selectbox.set("disabled", true);
                this._ipv4rangeselectbox.set("disabled", true);
                this.ipv4addresspoolselections.style.display = "none";
            }
            else if (settings.items[0].CMM__ifconfig__c == "static") {
                this.ipv4staticradiobutton.set("checked", true);
                this._ipv4selectbox.set("disabled", false);
                this._ipv4rangeselectbox.set("disabled", false);
                this.ipv4addresspoolselections.style.display = "block";
            }
            else if (settings.items[0].CMM__ifconfig__c == "dthens") {
                this.ipv4firstdhcpradiobutton.set("checked", true);
                this._ipv4selectbox.set("disabled", false);
                this._ipv4rangeselectbox.set("disabled", false);
                this.ipv4addresspoolselections.style.display = "block";
            }
            this._ipv4selectbox.set("value", settings.items[0].ipv4settingsaddresspoolvalue);
            this._ipv4rangedefault = settings.items[0].ipv4settingsrangevalue;
            this._setIPRangeSelect("IPv4", settings.items[0].ipv4settingsrangevalue);
                if (settings.items[0].CMM__ifconfig__dhcp6 == "enabled") {
                    this.ipv6dynamicradiobutton.set("checked", true);
                    this._ipv6selectbox.set("disabled", true);
                    this.ipv6addresspoolselections.style.display = "none";
                } else{
                 this.ipv6dynamicradiobutton.set("checked", false);
                }
                if (settings.items[0].CMM__ifconfig__sa6 == "enabled") {
                    this.ipv6statelessradiobutton.set("checked", true);
                    this._ipv6selectbox.set("disabled", true);
                    this.ipv6addresspoolselections.style.display = "none";
                } else{
                 this.ipv6statelessradiobutton.set("checked", false);
                }
                if (settings.items[0].CMM__ifconfig__ipv6static == "enabled") {
                    this.ipv6staticradiobutton.set("checked", true);
                    this._ipv6selectbox.set("disabled", false);
                    this.ipv6addresspoolselections.style.display = "block";
                } else{
                 this.ipv6staticradiobutton.set("checked", false);
                }
                this._ipv6selectbox.set("value", settings.items[0].ipv6settingsaddresspoolvalue);
                this._ipv6rangedefault = settings.items[0].ipv6settingsrangevalue;
                this._setIPRangeSelect("IPv6", settings.items[0].ipv6settingsrangevalue);    
            
        },

        _saveandhideDialog: function(evt) {
            if (!this._validateFields()) {
                return;
            }
        
            var ethernetTemplateObj = this._generateSavedData();

            var argJSONString = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": [ ethernetTemplateObj ]
            }, true);


            this.getProgressDialog("save").show(); // Show progress dialog for save request.
        var xhrPost = dojo.xhrPost({
            url: this.REST_PATH + "template/create",
            handleAs: "json", // <<<< Required
            headers: {
                "Content-Type": "application/json; charset=utf-8" // <<<< Required
            },
            postData: argJSONString,     //Template Data to Post
            load: dojo.hitch(this, function(response, ioArgs) {
                // Happy path! We received a 2xx status code from the API.
                                        
                dojo.publish("ProgressComplete", response.id);
                dojo.publish("ChassisNodeTemplateComplete", response.id);
                this.hide();   
            }),
            error: dojo.hitch(this, function(error, ioArgs) {
                // Failure path! We received a non-2xx status code (ie: 3xx/4xx/5xx status code) from the API.

                dojo.publish("ProgressFailed", error);
                console.log(this.declaredClass + ": ", error, ioArgs);
                
                var response = dojo.fromJson(ioArgs.xhr.responseText);
                if ((ioArgs.xhr.status >= 400) && (response.message == "service_error_name_in_use")) {
                    // Error Template name in use
                    this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                }
                else if(ioArgs.xhr.status == 404) {
                    // Edit Template ID doesn't exist
                    this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                    this.hide();
                }
                else if(ioArgs.xhr.status >= 500) {
                    // Internal Programming Error
                    this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                    this.hide();
                }
                else {
                    this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                    this.hide();
                }
                
            }),
            handle: dojo.hitch(this, function(data, ioArgs) {
                // - The 1st param ("data") is either the response data (on success) or Error object (on failure).
                // - The 2nd param ("ioArgs") contains detail about the xhr request, including a reference to the
                // native xhr object. Check ioArgs.xhr.status to determine success or failure, and handle "data"
                // accordingly.
            })
        });

        
        
        },
        saveClicked: function() {

            if (!this._validateFields()) {
                return;
            }

            var ethernetTemplateObj = this._generateSavedData();
            ethernetTemplateObj.id = this.templateIDNum;

            var argJSONString = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": [ ethernetTemplateObj ]
            }, true);

            if (this.templateInUse.toString() == "true") {

                this.rdstDialog.set("templateID", this.templateIDNum);
                this.rdstDialog.set("templateType", "Category");
                this.rdstDialog.set("templateData", argJSONString);

                this.rdstDialog.show();

            }
            else {


                this.getProgressDialog("save").show(); // Show progress dialog for save request.
                var xhrPost = dojo.xhrPost({
                    url: this.REST_PATH + "template/create",
                    handleAs: "json", // <<<< Required
                    headers: {
                        "Content-Type": "application/json; charset=utf-8" // <<<< Required
                    },
                    postData: argJSONString,     //Template Data to Post
                    load: dojo.hitch(this, function(response, ioArgs) {
                        // Happy path! We received a 2xx status code from the API.
                                                
                        dojo.publish("ProgressComplete", response.id);
                        dojo.publish("ChassisNodeTemplateComplete", response.id);
                        this.hide();   
                    }),
                    error: dojo.hitch(this, function(error, ioArgs) {
                        // Failure path! We received a non-2xx status code (ie: 3xx/4xx/5xx status code) from the API.

                        dojo.publish("ProgressFailed", error);
                        console.log(this.declaredClass + ": ", error, ioArgs);
                        
                        var response = dojo.fromJson(ioArgs.xhr.responseText);
                        if ((ioArgs.xhr.status >= 400) && (response.message == "service_error_name_in_use")) {
                            // Error Template name in use
                            this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                        }
                        else if(ioArgs.xhr.status == 404) {
                            // Edit Template ID doesn't exist
                            this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                            this.hide();
                        }
                        else if(ioArgs.xhr.status >= 500) {
                            // Internal Programming Error
                            this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                            this.hide();
                        }
                        else {
                            this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                            this.hide();
                        }
                        
                    }),
                    handle: dojo.hitch(this, function(data, ioArgs) {
                        // - The 1st param ("data") is either the response data (on success) or Error object (on failure).
                        // - The 2nd param ("ioArgs") contains detail about the xhr request, including a reference to the
                        // native xhr object. Check ioArgs.xhr.status to determine success or failure, and handle "data"
                        // accordingly.
                    })
                });


            }

        },
        saveasClicked: function() {

            if (!this._validateFields()) {
                return;
            }
            var ethernetTemplateObj = this._generateSavedData();
            this.saveasDialog.set("templateObject", ethernetTemplateObj);
            this.saveasDialog.show();

        },
        _getIPPooldata: function() {
            var ipv4data = null;
            var ipv6data = null;
            var ipv4store = null;
            var ipv6store = null;

            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "ippool/list",
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: function(response) {
                    var count;
                    ipv4data = {
                        "identifier": "id",
                        "label": "name",
                        "items": []
                    };
                    ipv6data = {
                        "identifier": "id",
                        "label": "name",
                        "items": []
                    };
                    for (count = 0; count < response.items.length; ++count) {
                        if (response.items[count].protocol == "IPv4") {
                            ipv4data.items.push(response.items[count]);
                        }
                        if (response.items[count].protocol == "IPv6") {
                            ipv6data.items.push(response.items[count]);
                        }
                    }

                    ipv4store = new dojo.data.ItemFileReadStore({
                        data: ipv4data
                    });

                    ipv6store = new dojo.data.ItemFileReadStore({
                        data: ipv6data
                    });

                },
                error: function(error) {
                    console.error(this.declaredClass + ": error getting data from service");
                    // Probably need to mock up some dummy data here
                }

            });
            this._ipv4store = ipv4store;
            this._ipv6store = ipv6store;
        },

        _validateFields: function(evt) {
            var validationpopup = new support.temp.ModalDialog({
                type: "Error",
                text: this.locResources.ERRORSELECTADDRESSPOOL
            });

            if (this.ipv4staticradiobutton.get("checked") == true) {
                if (this._ipv4selectbox.get("value") == "noselected" || this._ipv4selectbox.get("disabled") == true) {
                    validationpopup.show();
                    return false;
                }
            }

                if (this.ipv6staticradiobutton.get("checked") == true) {
                    if (this._ipv6selectbox.get("value") == "noselected" || this._ipv6selectbox.get("disabled") == true) {
                        validationpopup.show();
                        return false;
                    }
                }
            

            return true;
        },

        _cancelDialog: function() {
            this.hide();
            // this.dialogNode.destroy();
        },

        _selectall: function() {
            this._toggleMgmtIPAddrSettings();

            this._expandall();
            this.layout();
           // this.dialogNode.resize();
        },

        _deselectall: function() {
            this._toggleMgmtIPAddrSettings();

            this.layout();
        },

        _expandall: function(evt) {
            this.mgmtipaddrCheckBoxId.set("checked", "on");

            this.expandall.style.display = "none";
            this.collapseall.style.display = "inline";
            this.mixedexpandcollapse.style.display = "none";
            this._toggleMgmtIPAddrSettings();
            this.layout();
            //this.dialogNode.resize();
        },

        _collapseall: function() {
            this.mgmtipaddrCheckBoxId.set("checked", false);

            this.expandall.style.display = "inline";
            this.collapseall.style.display = "none";
            this.mixedexpandcollapse.style.display = "none";
            this._toggleMgmtIPAddrSettings();
            // this._toggleInterfaceSpeedSettings();
            this.layout();
        },

        _checkExpandCollapseState: function() {
            var mgmtipaddrCheckBoxState = this.mgmtipaddrCheckBoxId.get("value");

            /* value on = checked, false = unchecked */

            if (mgmtipaddrCheckBoxState == "on"

            ) {
                return "allon";
            }
            if (mgmtipaddrCheckBoxState == false

            ) {
                return "alloff";
            }

            return "mixed";
        },

        _setExpandCollapseIcon: function() {
            if (this._checkExpandCollapseState() == "mixed") {
                this.expandall.style.display = "none";
                this.collapseall.style.display = "none";
                this.mixedexpandcollapse.style.display = "inline";
            }
            if (this._checkExpandCollapseState() == "allon") {
                this.expandall.style.display = "none";
                this.collapseall.style.display = "inline";
                this.mixedexpandcollapse.style.display = "none";
            }
            if (this._checkExpandCollapseState() == "alloff") {
                this.expandall.style.display = "inline";
                this.collapseall.style.display = "none";
                this.mixedexpandcollapse.style.display = "none";
            }
        },

        _displayGeneralSettings: function(evt) {
            this._setExpandCollapseIcon();
            if (this._gen1.style.display == "none") {
                dojo.style(this._gen1, "display", "");
                dojo.style(this._gen2, "display", "");
                dojo.style(this._gen3, "display", "");
                dojo.style(this._gen4, "display", "");
            }
            else {
                dojo.style(this._gen1, "display", "none");
                dojo.style(this._gen2, "display", "none");
                dojo.style(this._gen3, "display", "none");
                dojo.style(this._gen4, "display", "none");
            }

            this.layout();
        },

        _displayMgmtIPAddrSettings: function() {
            this._setExpandCollapseIcon();
            this._toggleMgmtIPAddrSettings();
            this.layout();
        },

        _ipv4radioClicked: function() {
            if (this.ipv4form.get("value").ipv4 == "dynamicip") {
                this._ipv4selectbox.set("disabled", true);
                this._ipv4rangeselectbox.set("disabled", true);
                this.ipv4addresspoolselections.style.display = "none";
            }
            if (this.ipv4form.get("value").ipv4 == "staticip" || this.ipv4form.get("value").ipv4 == "firstdynamicip") {
                if (this._ipv4selectbox.get("value") != "") {
                    this._ipv4selectbox.set("disabled", false);
                    this._ipv4rangeselectbox.set("disabled", false);
                    if (this._ipv4selectbox.get("value") == "noselected") {
                        this._ipv4rangeselectbox.set("disabled", true);
                        dojo.replaceClass(this.ipv4editicon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                    }
                    else {
                        dojo.replaceClass(this.ipv4editicon, "spEditIcon_v2", "spEditInactiveIcon_v2");
                    }
                }
                else {
                    this._ipv4selectbox.set("disabled", true);
                    this._ipv4rangeselectbox.set("disabled", true);
                    dojo.replaceClass(this.ipv4editicon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                }
                this.ipv4addresspoolselections.style.display = "block";
            }
            this.layout();
        },

        _ipv6radioClicked: function() {
            if (this.ipv6staticradiobutton.get("checked") == true) {
                if (this._ipv6selectbox.get("value") != "") {
                    this._ipv6selectbox.set("disabled", false);
                    //this._ipv6rangeselectbox.set("disabled", false);
                    
                    if (this._ipv6selectbox.get("value") == "noselected") {
                        this._ipv6rangeselectbox.set("disabled", true);
                        dojo.replaceClass(this.ipv6editicon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                    } else {
                        dojo.replaceClass(this.ipv6editicon, "spEditIcon_v2", "spEditInactiveIcon_v2");
                    }                   
                } else {
                    this._ipv6selectbox.set("disabled", true);
                    this._ipv6rangeselectbox.set("disabled", true);
                    dojo.replaceClass(this.ipv6editicon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                }
                this.ipv6addresspoolselections.style.display = "block";

            } else {
                this.ipv6addresspoolselections.style.display = "none";
            }
            this.layout();
        },

        _toggleMgmtIPAddrSettings: function() {
            if (this.hideuncheckedToggleButtonId.get("checked") == true) {
                this.mgmtipaddrtr.style.display = "block";
                if (this.mgmtipaddrCheckBoxId.get("value") == false) {
                    this.mgmtipaddripv4tr.style.display = "none";
                    this.mgmtipaddripv6tr.style.display = "none";
                    return;
                }
                else {
                    this.mgmtipaddripv4tr.style.display = "block";
                    this.mgmtipaddripv6tr.style.display = "block";
                }

                if (this.ipv4form.get("value").ipv4 != "dynamicip") {
                    this.mgmtipaddripv4tr.style.display = "block";

                }
                else {
                    this.mgmtipaddripv4tr.style.display = "none";
                }

                var defaultOps = (this.ipv6statelessradiobutton.get("checked") == true && this.ipv6dynamicradiobutton
                        .get("checked") == false && this.ipv6staticradiobutton.get("checked") == false);
                if (!defaultOps) {
                    this.mgmtipaddripv6tr.style.display = "block";
                }
                else {
                    this.mgmtipaddripv6tr.style.display = "none";
                }
                return;
            }
            else {
                this.ipv4outlinepane.style.display = "block";
                this.mgmtipaddrtr.style.display = "block";
                if (this.mgmtipaddrCheckBoxId.get("value") == false) {
                    this.mgmtipaddripv4tr.style.display = "none";
                    this.ipv4outlinepane.style.display = "block";
                    this.mgmtipaddripv6tr.style.display = "none";
                }
                else {
                    this.mgmtipaddripv4tr.style.display = "block";
                    this.mgmtipaddripv6tr.style.display = "block";
                    this.ipv6outlinepane.style.display = "block";
                }
                return;
            }

        },

        _hideUncheckedClicked: function(evt) {
            this._toggleMgmtIPAddrSettings();
            this.layout();
            return;
        },

        _showIPPoolDialog: function(evt) {
            var ippooldialog = new support.dialogs.NewIPPool({
                id: "cp_chassisnodetemplate_newippooldialog"
            });
            ippooldialog.show();
        },
        
        _showIPv6PoolDialog: function(evt) {
            var ipv6pooldialog = new support.dialogs.NewIPPool({
                id: "cp_chassisnodetemplate_newipv6pooldialog"                
            });
            ipv6pooldialog.set("version", "IPv6");
            ipv6pooldialog.show();
        },

        _editIPPoolDialog: function(evt) {
            var poolID = this._ipv4selectbox.get("value");
            console.log(this.declaredClass + ": ipv4pool:" + poolID);
            if(poolID == "noselected" || poolID == "") {
                return;
            }
            var ippooldialog = new support.dialogs.NewIPPool({
                id: "cp_chassisnodetemplate_editnewippooldialog",                
                poolIDNum: poolID
            });
            ippooldialog.show();
        },

        _editIPv6PoolDialog: function(evt) {
            var poolID = this._ipv6selectbox.get("value");
            console.log(this.declaredClass + ": ipv6pool:" + poolID);
            if(poolID == "noselected" || poolID == "") {
                return;
            }
            var ipv6pooldialog = new support.dialogs.NewIPPool({
                id: "cp_chassisnodetemplate_editnewipv6pooldialog",                
                poolIDNum: poolID
            });
            ipv6pooldialog.set("version", "IPv6");
            ipv6pooldialog.show();
        },

        exitParent: function() {
            var data = "ChassisManagement";

            dojo.publish("ChassisNodeTemplateComplete", data);

            if(this.rdstDialog != null){
                var deployClicked = this.rdstDialog.get("closeParent");
                if(deployClicked) {
                    this.hide();
                }
            }
        },

        _setDefaults: function() {

        },

        _nameChanged: function() {
            if (this.templateName.get("value").trim() != "") {
                this.okButton.set("disabled", false);
            }
            else {
                this.okButton.set("disabled", true);
            }
        },
        _setup: function() {

            // Set Template Field to catch intermediate changes
            this.templateName.intermediateChanges = true;

            // Set Description Label
            this._descLimit.innerHTML = this.resources.LIMIT.replace("%500", "500");

            // Set Information Messages and Load Previous Settings
            if (this.templateIDNum != null && this.templateMode == "edit") { // if dialog in edit
                                                                                // mode

                // this.loadprevSettings();

                // alert(this.templateInUse);

                if (this.templateInUse.toString() == "true") {

                    if (this.templateLaunchPoint == "table") {

                        this.warningMessage.style.display = "block";
                        this.warningMessageText.innerHTML = this.resources.INUSE;
                        this.saveBtnContainer.style.display = "inline";
                        this.saveasBtnContainer.style.display = "inline";

                    }
                    else if (this.templateLaunchPoint == "wizard") {

                        this.informationMessage.style.display = "block";
                        this.informationMessageText.innerHTML = this.resources.INUSEWIZARD;
                        this.saveasBtnContainer.style.display = "inline";

                    }
                    else {
                        this.informationMessage.style.display = "block";
                        this.informationMessageText.innerHTML = this.resources.INUSEWIZARD;
                        this.saveasBtnContainer.style.display = "inline";
                    }

                }
                else {

                    this.saveBtnContainer.style.display = "inline";
                    this.saveasBtnContainer.style.display = "inline";

                }

                this.templateName.set("disabled", true);

            }
            else if (this.templateIDNum != null && this.templateMode == "copy") { // if dialog in
                                                                                    // copy mode

                // this.loadprevSettings();
                this.templateIDNum = null; // set template ID to null in order to create new
                                            // template
                this.okBtnContainer.style.display = "inline";

            }
            else { // if dialog in create new mode

                this.okBtnContainer.style.display = "inline";

            }

        },

        _generateSavedData: function() {
            var ethernetTemplateObj = new Object();

            ethernetTemplateObj.name = this.templateName.get("value").trim();
            ethernetTemplateObj.type = "cmmNode";

            var description = this.ethadapDesc.get("value").trim();
            ethernetTemplateObj.description = this.encodeSpecialCharacters(description);

            // Needed for profile command generation
            //ethernetTemplateObj.CMM__ifconfig__eth = 1;

            // dynamic static
            var ipv4radiobutton = this.ipv4form.get("value").ipv4;
            // ethernetTemplateObj.ipv4settingsradiovalue = ipv4radiobutton;

            if (ipv4radiobutton == "staticip") {
                ethernetTemplateObj.CMM__ifconfig__c = "static";
                ethernetTemplateObj.ipv4settingsaddresspoolvalue = this._ipv4selectbox.get("value");
                var ipv4rangevalue = this._ipv4rangeselectbox.get("value");
                if (ipv4rangevalue == "any") {
                    ethernetTemplateObj.ipv4settingsrangevalue = null;
                }
                else {
                    ethernetTemplateObj.ipv4settingsrangevalue = ipv4rangevalue;
                }
            }
            else if (ipv4radiobutton == "firstdynamicip") {
                ethernetTemplateObj.CMM__ifconfig__c = "dthens";
                ethernetTemplateObj.ipv4settingsaddresspoolvalue = this._ipv4selectbox.get("value");
                var ipv4rangevalue = this._ipv4rangeselectbox.get("value");
                if (ipv4rangevalue == "any") {
                    ethernetTemplateObj.ipv4settingsrangevalue = null;
                }
                else {
                    ethernetTemplateObj.ipv4settingsrangevalue = ipv4rangevalue;
                }
            }
            else {
                ethernetTemplateObj.CMM__ifconfig__c = "dhcp";
            }

                    if (this.ipv6staticradiobutton.get("checked") == true) {
                        // stateless dynamic static
                        ethernetTemplateObj.ipv6settingsaddresspoolvalue = this._ipv6selectbox.get("value");
                        ethernetTemplateObj.CMM__ifconfig__ipv6static = "enabled";
                        var ipv6rangevalue = this._ipv6rangeselectbox.get("value");
                        if (ipv6rangevalue == "any") {
                            ethernetTemplateObj.ipv6settingsrangevalue = null;
                        } else {
                            ethernetTemplateObj.ipv6settingsrangevalue = ipv6rangevalue;
                        }
                    } else {
                        ethernetTemplateObj.ipv6settingsaddresspoolvalue = this._ipv6selectbox.get("value");
                        ethernetTemplateObj.CMM__ifconfig__ipv6static = "disabled";
                        var ipv6rangevalue = this._ipv6rangeselectbox.get("value");
                        if (ipv6rangevalue == "any") {
                            ethernetTemplateObj.ipv6settingsrangevalue = null;
                        } else {
                            ethernetTemplateObj.ipv6settingsrangevalue = ipv6rangevalue;
                        }
                        
                    }
                    
                    if (this.ipv6statelessradiobutton.get("checked") == true) {
                        ethernetTemplateObj.CMM__ifconfig__sa6 = "enabled";
                    } else{
                        ethernetTemplateObj.CMM__ifconfig__sa6 = "disabled";
                    }
                    if (this.ipv6dynamicradiobutton.get("checked") == true) {
                        ethernetTemplateObj.CMM__ifconfig__dhcp6 = "enabled";
                    } else{
                        ethernetTemplateObj.CMM__ifconfig__dhcp6 = "disabled";
                    }
            

            // If doing an edit, need to add the id
            if (this.templateIDNum != null && this.templateMode != "copy") {
                ethernetTemplateObj.id = this.templateIDNum;
            }
            else {
                this.templateIDNum = null;
            }

            return ethernetTemplateObj;
        }

    });
});