define([ 
        "dojo", 
        "dijit", 
        "dojo/aspect", 
        "support/dialogs/Dialog", 
        "dojo/i18n!./nls/ChassisTemplateWizard",
        "dojo/text!./templates/ChassisTemplateWizard.html",
        "dojo/dom-style",
        "dojo/dom-class",
        "dijit/layout/ContentPane", 
        "dijit/layout/TabContainer", 
        "support/wizard/pages/cmmGeneral", 
        "support/wizard/pages/cmmMgmtModules", 
        "support/wizard/pages/cmmIOModules",
        "support/wizard/pages/cmmBayNodes",
        "support/dialogs/ProgressDialog"
        ], function(dojo, dijit, aspect, dialog, mri, template, domStyle, domClass) {

    return dojo.declare("support.dialogs.ChassisTemplateWizard", [ support.dialogs.Dialog ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,
        
        // Number of Steps for wizard completion
        numberOfSteps: null,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "ChassisTemplateWizard",

        // templateIDNum: String
        // Specifies the ID of an existing template. If specified, the wizard will attempt to
        // retrieve settings from the specified template and use them to pre-populate wizard pages.
        //
        // NOTE: If both "templateIDNum" and "baseChassis" attributes are specified, the
        // "templateIDNum" attribute will be given preference.
        //
        templateIDNum: null,

        // templateInUse: String
        // Specifies whether an existing template is in use. If in use, this chassis template has
        // been
        // deployed to 1 or more chassis/locations, which means editing an in use template requires
        // re-deploying this template.
        //
        templateInUse: false,

        // baseChassis: Object
        // Specifies attributes used to identify an existing chassis. If specified, the wizard will
        // attempt to retrieve settings for the specified chassis and use them to pre-populate wizard
        // pages.
        //
        // For example:
        // baseChassis: {
        // chassisName: "r3-c2-ch3-mm.rtp.stglabs.ibm.com",
        // chassisUUID: "xxxx"
        // }
        //
        // NOTE: If both "templateIDNum" and "baseChassis" attributes are specified, the
        // "templateIDNum" attribute will be given preference.
        //
        baseChassis: null,

        // editMode: boolean
        // Specifies whether the wizard is to execute in edit mode. If "true", the "templateIDNum"
        // attribute is required.
        editMode: false,

        // copyMode: boolean
        // Specifies whether the wizard is to execute in copy mode. If "true", the "templateIDNum"
        // attribute is required.
        copyMode: false,

        // preGeneratedTemplate: boolean
        // Specifies whether the wizard is using a pre-generated(ie generated from existing chassis
        // settings.
        // If "true", the "templateIDNum"
        // attribute is required.
        preGeneratedTemplate: false,

        // templateType: String
        // Type of template created by this wizard. This name is only used internally
        templateType: "CMM",

        createFromExisting: false,

        constructor: function() {
            this.GENERAL_PAGE_TITLE = this.locResources.GEN_TITLE;
            this.MGMT_PAGE_TITLE = this.locResources.MGMT_TITLE;
            this.IO_PAGE_TITLE = this.locResources.IO_TITLE;
            this.NODE_PAGE_TITLE = this.locResources.NODE_TITLE;
        },

        //Holds user selected preferences
        userPreferences: null,
        
        //Holds wizard object, currently a tab container widget
        wizard: null,
        
        //Holds page objects
        generalPage: null,
        mgmtModulesPage: null,
        ioModulesPage: null,
        nodePage: null,

        //Used to compare saved pattern vs edit pattern, checks if any changes during edit
        _originalChassisPattern: null,

        //Holds learned chassis ID
        _learnedChassisUUID: null,

        // spCssPathAndName: String
        // The name of the CSS file for this widget.
        spCssPathAndName: "support/dialogs/themes/ChassisTemplateWizard.css",

        postCreate: function() {    
            this.inherited(arguments);

            // Set Dialog Title
            var title = this.copyMode ? this.locResources.COPY_TITLE : this.editMode ? this.locResources.EDIT_TITLE
                    : this.locResources.CREATE_TITLE;
            this.set("title", title);

            // Get preferences set by the user...
            this._getUserPreferences();

            // Set up wizard & pages ...
            this._setupWizard();

            // AA120928-- Workaround for this being a wrapper for Dialog vs a subclass.
            // We need to have the wizard go re-lay itself out now that we've resized it,
            // but it's fruitless to do this until it's showing on the screen. You
            // would think that when you resize the wizard it would automatically
            // re-lay itself out, but that is not the case.
          /*  aspect.after(this.dialogNode, "onShow", dojo.hitch(this, function() {
                this.wizard.updateUI();
            }));*/

            // BUGBUG- AA121017: SOMETIMES in FF, you don't get the onload event
            // for the CSS. So, we put in this ugly hack :(
          //  window.setTimeout(dojo.hitch(this, this.cssArrived), 10000);

            return this.inherited(arguments);
        }/*,

        cssArrived: function() {
            // It's possible for the CSS to arrive before we've even created the wizard.
            if (this.wizard) {
                this.wizard.updateUI();
                // This tells the wizard's BorderContainer to re-size the content div.
                this.resize();
            }

            this.inherited(arguments);
        }*/,

        _getUserPreferences: function() {
            // tags:
            // protected
            var preferences = null;
            dojo.xhrGet({
                url: this.REST_PATH + "config/default",
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                sync: true,
                load: function(response, ioArgs) {
                    preferences = response;
                },
                error: function(error, ioArgs) {
                    console.error(this.declaredClass + ": Error (", error, ") occurred when retrieving user preferences.");
                    alert(this.resources.ERROR + " - " + error);
                    preferences = {};
                    return false;
                }
            });
            this.userPreferences = preferences;
            console.log(this.declaredClass + ": user preferences: ", this.userPreferences);
        },

        _setupWizard: function() {
            // tags:
            // protected

            //Controls whether to enable the next button, can be called from wizard pages
            dojo.subscribe("enableNextButton", this, function(response) {
                                
                //If true then enable the next button else disable it and the save actions
                if(response){
                    this.nextButton.set("disabled", false);
                    
                    //Check if all pages enabled, if not then 
                     // don't enable to save, saveas, save deploy buttons
                    var canEnableSave = true;                    
                    dojo.forEach(this.wizard.getChildren(), dojo.hitch(this, function (childPage) {
                        if(childPage.cpDisabled){
                            canEnableSave = false;
                        }                        
                    }));                    
                    dojo.publish("enableSaveButton", canEnableSave);
                    dojo.publish("enableSaveDeployButton", canEnableSave);

                }else{  
                    //Disable next button and any save actions
                    this.nextButton.set("disabled", true);
                    dojo.publish("enableSaveButton", false);
                    dojo.publish("enableSaveDeployButton", false);
                }
                
            });

            //Controls whether to enable the save button, can be called from wizard pages
            dojo.subscribe("enableSaveButton", this, function(response) {
                                
                if(response){
                    this.saveButton.set("disabled", false);
                }else{                    
                    this.saveButton.set("disabled", true);
                }
                
            });

            //Controls whether to enable the saveas / savedeploy button, can be called from wizard pages
            dojo.subscribe("enableSaveDeployButton", this, function(response) {
                                
                if(response){
                    this.saveDeployButton.set("disabled", false);
                }else{                    
                    this.saveDeployButton.set("disabled", true);
                }
                
            });

            //Create wizard from layout tab container and place
            this.wizard = new dijit.layout.TabContainer({
                style: "height: 100%; width: 100%;"
            }, this._wizardNode);

            //Load wizard navigation button depending on wizard launch state {edit,copy,new}
            if(this.editMode){

                this.backButton = new dijit.form.Button({
                    disabled: true,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.BACK_BTN,
                    onClick: dojo.hitch(this, "onBack")
                }).placeAt(this._wizardButtonBar);

                this.nextButton = new dijit.form.Button({
                    disabled: false,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.NEXT_BTN,
                    onClick: dojo.hitch(this, "onNext")
                }).placeAt(this._wizardButtonBar);

                this.saveButton = new dijit.form.Button({
                    disabled: false,
                    showLabel: true,
                    role: "button",
                    label: this.resources.SAVE,
                    onClick: dojo.hitch(this, "onSave")
                }).placeAt(this._wizardButtonBar);

                this.saveDeployButton = new dijit.form.Button({
                    disabled: false,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.SAVEAS_BTN,
                    onClick: dojo.hitch(this, "onSaveDeploy")
                }).placeAt(this._wizardButtonBar);

                
            }else if(this.copyMode){

                this.backButton = new dijit.form.Button({
                    disabled: true,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.BACK_BTN,
                    onClick: dojo.hitch(this, "onBack")
                }).placeAt(this._wizardButtonBar);

                this.nextButton = new dijit.form.Button({
                    disabled: false,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.NEXT_BTN,
                    onClick: dojo.hitch(this, "onNext")
                }).placeAt(this._wizardButtonBar);

                this.saveButton = new dijit.form.Button({
                    disabled: false,
                    showLabel: true,
                    role: "button",
                    label: this.resources.SAVE,
                    onClick: dojo.hitch(this, "onSave")
                }).placeAt(this._wizardButtonBar);

                this.saveDeployButton = new dijit.form.Button({
                    disabled: false,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.BTN_LABEL,
                    onClick: dojo.hitch(this, "onSaveDeploy")
                }).placeAt(this._wizardButtonBar);

            }else{
            
                this.backButton = new dijit.form.Button({
                    disabled: true,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.BACK_BTN,
                    onClick: dojo.hitch(this, "onBack")
                }).placeAt(this._wizardButtonBar);

                this.nextButton = new dijit.form.Button({
                    disabled: true,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.NEXT_BTN,
                    onClick: dojo.hitch(this, "onNext")
                }).placeAt(this._wizardButtonBar);

                this.saveButton = new dijit.form.Button({
                    disabled: true,
                    showLabel: true,
                    role: "button",
                    label: this.resources.SAVE,
                    onClick: dojo.hitch(this, "onSave")
                }).placeAt(this._wizardButtonBar);

                this.saveDeployButton = new dijit.form.Button({
                    disabled: true,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.BTN_LABEL,
                    onClick: dojo.hitch(this, "onSaveDeploy")
                }).placeAt(this._wizardButtonBar);
                
            }
                      
            this.cancelButton = new dijit.form.Button({
                showLabel: true,
                role: "button",
                label: this.resources.CANCEL,
                onClick: dojo.hitch(this, "_onClose")
            }).placeAt(this._wizardButtonBar);

            //If edit or copy mode then load object with settings
            var chsTemplate = null;
            if (this.copyMode || this.editMode) {
                // Load in template data
                chsTemplate = this._getChassisPatternFromServices();

                // Remember Template's Name
                if (chsTemplate.name) {
                    this.templateName = chsTemplate.name;
                }
            }

            // Prepare General page and add into the wizard.
            var generalPageParams = new Object();
            if (chsTemplate) {
                // Pass current values...
                generalPageParams.nodeType = chsTemplate.nodeType;
                generalPageParams.formFactor = chsTemplate.formFactor;
                generalPageParams.templateName = chsTemplate.name;
                generalPageParams.templateDesc = this.decodeSpecialCharacters(chsTemplate.description);
            }
            else {
                // Pass default values...
                generalPageParams.nodeType = this.userPreferences.nodeType;
                generalPageParams.formFactor = this.userPreferences.formFactor;
            }
            this.generalPage = new support.wizard.pages.cmmGeneral({
                editMode:this.editMode,
                copyMode:this.copyMode,
                inUse:this.templateInUse,
                existing: this.createFromExisting,
                pageParams: generalPageParams
            });

            dojo.connect(this.generalPage, "onChangeNodeType", this, "_prepareForNodeType");
            dojo.connect(this.generalPage, "onChangeFormFactor", this, "_prepareForFormFactor");
            dojo.connect(this.generalPage, "onChangeStartingPoint", this, "_prepareForStartingPoint");
            dojo.connect(this.generalPage, "onChangeChassis", this, "_queueBaseChassisChange");

            // Management Modules Page
            var mgmtPageParams = new Object();
            if (chsTemplate) {
                // Pass current values...
                mgmtPageParams.templates = chsTemplate.mmSettings;
            }
            else {
                // Pass default values...
                // mgmtPageParams.templates = chsTemplate.mgmtModules.templates;
            }
            this.mgmtModulesPage = new support.wizard.pages.cmmMgmtModules({
                editMode:this.editMode,
                copyMode:this.copyMode,
                inUse:this.templateInUse,
                pageParams: mgmtPageParams
            });

            // IO Modules Page
            var ioPageParams = new Object();

            if (chsTemplate) {
                ioPageParams = chsTemplate.ioSettings;
            }
            else {
                // Pass default values...
                // mgmtPageParams.templates = chsTemplate.mgmtModules.templates;
            }
            this.ioModulesPage = new support.wizard.pages.cmmIOModules({
                editMode:this.editMode,
                copyMode:this.copyMode,
                inUse:this.templateInUse,
                pageParams: ioPageParams
            });

            // Bay nodes Page
            var nodePageParams = new Object();
            if (chsTemplate) {
                nodePageParams = chsTemplate.nodeSettings;
                // nodePageParams =
                // [{"index":"1","type":"cmmNode","id":"28","formFactor":"4"},{"index":"2","type":"cmmNode","id":"28","formFactor":"4"},{"index":"3","type":"cmmNode","id":"-1","formFactor":"-1"},{"index":"4","type":"cmmNode","id":"-1","formFactor":"-1"},{"index":"5","type":"cmmNode","id":"-1","formFactor":"-1"},{"index":"6","type":"cmmNode","id":"-1","formFactor":"-1"},{"index":"7","type":"cmmNode","id":"-1","formFactor":"-1"},{"index":"8","type":"cmmNode","id":"-1","formFactor":"-1"},{"index":"9","type":"cmmNode","id":"-1","formFactor":"-1"},{"index":"10","type":"cmmNode","id":"-1","formFactor":"-1"},{"index":"11","type":"cmmNode","id":"-1","formFactor":"-1"},{"index":"12","type":"cmmNode","id":"-1","formFactor":"-1"},{"index":"13","type":"cmmNode","id":"-1","formFactor":"-1"},{"index":"14","type":"cmmNode","id":"28","formFactor":"1"}];
            }
            else {
                // Pass default values...
                // mgmtPageParams.templates = chsTemplate.mgmtModules.templates;
            }
            this.nodePage = new support.wizard.pages.cmmBayNodes({
                editMode:this.editMode,
                copyMode:this.copyMode,
                inUse:this.templateInUse,
                pageParams: nodePageParams
            });

            //Build array of wizard pages and titles to pass to tab container
            var contents = [this.generalPage,this.mgmtModulesPage,this.ioModulesPage,this.nodePage];                    
            var titles = [this.GENERAL_PAGE_TITLE,this.MGMT_PAGE_TITLE,this.IO_PAGE_TITLE,this.NODE_PAGE_TITLE];

            var cpDisabled = false; //Holds if page should be disabled on launch

            //Loop through each page and add it to the wizard (tab container)           
            for(var count=0;count<contents.length;count++){

                var cp = new dijit.layout.ContentPane({
                    style: "overflow: auto;",
                    content: contents[count],
                    title: titles[count],
                    position: count+1,
                    cpDisabled: cpDisabled                    
                });
                
                this.wizard.addChild(cp);
                
                //If wizard launch is not edit or copy then disable all pages after first page
                if(this.editMode == false && this.copyMode == false)
                    cpDisabled = true;                
                
            }
            
            //This function is fired each time a page (tab) is selected
            this.wizard.watch("selectedChildWidget", dojo.hitch(this, function(name, oval, nval){

                //Get all wizard pages
                var childrenArray = this.wizard.getChildren();
                
                //Don't allow select of next page if previous page is still disabled or next button is not enabled
                if(nval.position != 1 && (childrenArray[nval.position-2].cpDisabled == true || this.nextButton.get("disabled") == true) && oval.position < nval.position){
                    //Reselect previous page instead of the newly selected page
                    this.wizard.selectChild(oval);
                }else{
                    //Set page isDisabled to false meaning the page has been visited properly
                    nval.cpDisabled = false;
                    //Remove custom style for disabled page
                    domStyle.set(nval.controlButton.domNode,"opacity",1);                                
                    domStyle.set(nval.controlButton.domNode,"filter","alpha(opacity=100)");

                    //Disable the next button and allow page AfterPageSelected funtion to properly set next button
                    dojo.publish("enableNextButton", false);

                    //Run special functions if newly selected page is the Boot page
    //                if (nval.title == "Boot") {
//                        this.bootPage.pageParams.fcAddressPool = this.ioAdaptersPage.getFcAddressPool();
  //                      this.bootPage.reloadData();
      //              }

                    //Run function linked to page that is called after a page is selected
                     // enable or disable next button from this function and also run page setups
                    dijit.byId(nval.content.id).AfterPageSelected();

                    //If at the last step of the wizard then enable save actions
                    if(this.numberOfSteps == this.wizard.selectedChildWidget.position){
                        dojo.publish("enableSaveButton", true);
                        dojo.publish("enableSaveDeployButton", true);                    
                    }

                    //If not on the first page of the wizard then allow then enable back button
                    if(this.wizard.selectedChildWidget.position > 1){
                        this.enableBackButton(true);
                    }else{
                        this.enableBackButton(false);
                    }

                }
                                  
             })); 
                           
            //Widget call to startup wizard with given information
            this.wizard.startup();
            
            //Get number of pages in the wizard after startup and disable all pages after first
             // if new wizard (meaning not edit or copy)
            var childrenArray = this.wizard.getChildren();
            this.numberOfSteps = childrenArray.length;
                            
            //Fade all pages expect first for new wizard if new wizard
            if(this.editMode == false && this.copyMode == false){
                var count=0;
                dojo.forEach(this.wizard.getChildren(), dojo.hitch(this, function (childPage) {
                    if(count!=0){
                        domStyle.set(childPage.controlButton.domNode,"opacity",.4);                                
                        domStyle.set(childPage.controlButton.domNode,"filter","alpha(opacity=40)");
                    }
                    count = count + 1;
                }));
            }
            
/*            dojo.connect(this.wizard, "OnPageSelected", this, "_pageChangeHandler");
            // Enable Save Button If All Pages Are Valid
            dojo.connect(this.wizard, "OnPageSelected", this, function() {
                // disable the Finish/OK button until all pages are valid
                var curr_page = this.wizard._pageContainer.selectedChildWidget;
                var valid = curr_page === this.nodePage || this.nodePage.visited;
                this.saveButton.set("disabled", !valid);
            });*/

            // Close Self If Save As Complete
            this.subscribe("CloseAfterSaveas", function(success) {
                if (success) {
                    this.hide();
                }
                else {
                    console.error(this.declaredClass + ": Failed To Save Chassis Template");
                }
            });
            // Destroy on hide
            dojo.connect(this.dialogNode, "hide", this, function() {
                this.dialogNode.destroy();
                this.nodePage.resetNodeTable();
                //Get rid of subscriptions from wizard pages
                if (this.nodePage.subHandle) {
                    dojo.unsubscribe(this.nodePage.subHandle);
                }
                this.mgmtModulesPage.unsubscribeSelectors();
                this.nodePage.unsubscribeSelectors();
                this.ioModulesPage.unsubscribeSelectors();
            });
        },
        enableBackButton: function (response){
            //Enable/disable wizard back button
            if(response){
                this.backButton.set("disabled", false);
            }else{                    
                this.backButton.set("disabled", true);
            }
        },
        onBack: function (){
            //Called when wizard back button clicked to go back to previous page            
            this.wizard.back();                    

            //Disable back button if newly selected page is first page
            if (this.wizard.selectedChildWidget.position == 1) {
                this.enableBackButton(false);                
            }
        },    
        onNext: function() {
            //Called when wizard next button clicked to go to next page            
 
            // If a base chassis was selected for "learn from existing chassis" path but has not
                // yet been successfully processed, attempt to process the selection now. If
                // successful allow user to proceed to next page.

            if (this.wizard.selectedChildWidget.position == 1 && this.baseChassis && (this.baseChassis.chassisUUID != this._learnedChassisUUID)) {

                if (this.baseChassis && (this.baseChassis.chassisUUID != this._learnedChassisUUID)) {

                    var proceedCallback = dojo.hitch(this, function() {
                        this._learnedChassisUUID = this.baseChassis.chassisUUID;
                        this.wizard.forward();
                    });

                    //Run the learning process of the selected base/chassis
                    this._prepareForChassis(this.baseChassis, proceedCallback);
                }
                
            }else{
                this.wizard.forward();
            }
                        
        },
        onClose: function() {
            // Function connected to the "Cancel" button
            var confirmedCancel = function() {
                this.hide();
                dojo.publish("CloseDialog");
            };

            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {
            
                    var confirmDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.OK,
                        type: "Warning",
                        text: this.resources.WIZARD_CANCEL_MESSAGE,
                        info: this.resources.WIZARD_CANCEL_INFO                    
                    });

        		    confirmDialog.startup();
		            confirmDialog.confirm(confirmedCancel, this);

                }));
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.OK,
                        type: "Warning",
                        text: this.resources.WIZARD_CANCEL_MESSAGE,
                        info: this.resources.WIZARD_CANCEL_INFO                    
                    });                    

        		    confirmDialog.startup();
		            confirmDialog.confirm(confirmedCancel, this);
                    
                }));
            }
            

            //console.log(this.declaredClass + ": onClose called");
            //return confirm(this.resources.CONFIRM_CANCEL);
        },

        onSaveDeploy: function() {
            // Function connected to the "Save and Deploy" and "Save As" buttons
            if (!this.editMode) {
                // Set up subscriber for _saveTemplateAsync() below.
                var handle = dojo.subscribe("ChassisTemplateComplete", this, function(response) {
                    if ((response.status >= 200) && (response.status < 300)) {
                        require([ "support/dialogs/DeployChassisTemplate" ], dojo.hitch(this, function() {
                            var dct = new support.dialogs.DeployChassisTemplate({
                                templateId: response.templateId
                            });
                            dojo.body().appendChild(dct.domNode);
                            dct.show();
                        }));
                        this.hide();
                    }
                    else {
                        console.error(this.declaredClass + ": Error Saving Chassis Template");

                        if ((response.status >= 400) && (response.response.message == "service_error_name_in_use")) {
                            // Error, pattern name in use
                            this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                        }
                        else if (response.status == 404) {
                            // Error, pattern does not exist. Use Save As.
                            this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                        }
                        else if (response.status >= 500) {
                            // Internal error. Call support.
                            this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                        }
                        else {
                            // Error occurred. Try again or call support.
                            this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                        }

                    }
                    handle && dojo.unsubscribe(handle);
                });

                // Save and Deploy
                this._saveTemplateAsync();
                return false; // Don't allow close! Must be handled by subscriber (above).
            }
            else {
                // Save As
                var template = this._generateTemplateData();
                
                require([ "support/dialogs/SaveAs" ], dojo.hitch(this, function() {
                    var saveAs = new support.dialogs.SaveAs({
                        templateObject: template,
                        originalName: this.templateName,
                        saveTemplateType: this.templateType
                    });
                   dojo.body().appendChild(saveAs.domNode);
                   saveAs.show();
                }));
            }

        },

        _prepareForNodeType: function(nodeType) {
            // tags:
            // protected

        },

        _prepareForFormFactor: function(formFactor) {
            // tags:
            // protected
        },

        _prepareForStartingPoint: function(startingPoint) {
            // tags:
            // protected
            // Enable or Disable page options based on starting point
            console.log(this.declaredClass + ": _prepareForStartingPoint called: ", startingPoint);
            if (!this.editMode) {
                if (startingPoint == this.generalPage.FROM_SCRATCH) {
                    this._queueBaseChassisChange(null); // clear base chassis, if any
                    this.ioModulesPage._selectAll.set("checked", true);
                    this.mgmtModulesPage.set("startingPoint", "FROM_SCRATCH");
                    this.ioModulesPage.set("startingPoint", "FROM_SCRATCH");
                    this.nodePage.set("startingPoint", "FROM_SCRATCH");
                }
                else if (startingPoint == this.generalPage.FROM_CHASSIS) {
                    // learned
                    this.ioModulesPage._selectAll.set("checked", false);
                    this.mgmtModulesPage.set("startingPoint", "FROM_CHASSIS");
                    this.ioModulesPage.set("startingPoint", "FROM_CHASSIS");
                    this.nodePage.set("startingPoint", "FROM_CHASSIS");
                }
            }
        },

        _queueBaseChassisChange: function(baseChassis) {
            // tags:
            // protected
            // Queues request to set base chassis for "learning" process. The user's selection is
            // processed when they click "Next" button to proceed.
            console.log(this.declaredClass + ": _queueBaseChassisChange called: ", (baseChassis) ? baseChassis.chassisUUID : null);
            this.baseChassis = baseChassis;
            if (!baseChassis || (baseChassis.chassisUUID != this._learnedChassisUUID)) {
                this._learnedChassisUUID = null;
            }

        },

        _prepareForChassis: function(baseChassis, onSuccessCallback) {
            // tags:
            // protected
            console.log(this.declaredClass + ": _prepareForChassis called: ", baseChassis, onSuccessCallback);
            this.mgmtModulesPage.set("chassis", baseChassis.chassisName);
            this.ioModulesPage.set("chassis", baseChassis.chassisName);
            this.getProgressDialog("learn").show(); // Show progress dialog for learn request.
            var templateName = this.generalPage._templateName.get("value").trim();
            var postData = dojo.toJson({
                "templateName": templateName
            });
            dojo.xhrPost({
                url: this.REST_PATH + "template/CMM/" + baseChassis.chassisUUID,
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                postData: postData,
                load: dojo.hitch(this, function(response, ioArgs) {
                    console.log(this.declaredClass + ": Generate learned template complete: ", response, ioArgs);
              
                    //Update Chassis Pattern Table after learning process
              		dojo.publish("RefreshTable");                
              
                    var chsTemplateId = response.id;
                    this.generalPage.warningMessage.style.display = "";
                    this._loadLearnedTemplates(chsTemplateId);
                }),
                error: dojo.hitch(this, function(error, ioArgs) {
                    console.log(this.declaredClass + ": Generate learned template complete: ", error, ioArgs);

                    var response = dojo.fromJson(ioArgs.xhr.responseText);
                    if (response.message == "service_error_name_in_use") {
                        // Error, pattern name in use
                        this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                    }
                    else if(response.message == "service_error_no_server_settings") {
                        // Error retrieving data from the chassis
                        this.getModalDialog("Error", this.resources.ERROR_CHASSIS_LEARN).show();
                    }
                    else if(response.status >= 500) {
                        // Internal error. Call support.
                        this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                    }
                    else {
                        // Error occurred. Try again or call support.
                        this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                    }
                }),
                handle: dojo.hitch(this, function(response, ioArgs) {
                    // Inform progress dialog...
                    if (ioArgs.xhr.status >= 300) {
                        dojo.publish("ProgressFailed", response);
                        // Display error on general page
                        this.generalPage.headerWarningMessage.style.display = "block";
                        this.generalPage.headerWarningMessageText.innerHTML = this.resources.ERROR_LEARNED;
                    }
                    else {
                        dojo.publish("ProgressComplete", response);

                        // Call onSuccess callback function, if any.
                        if (dojo.isFunction(onSuccessCallback)) {
                            onSuccessCallback();
                        }
                    }
                })
            });
        },

        _loadLearnedTemplates: function(chsTemplateId) {
            // tags:
            // protected
            if (chsTemplateId != null) {
                var chsTemplate = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "template/" + chsTemplateId,
                    handleAs: "json",
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    sync: true,
                    preventCache: true,
                    load: dojo.hitch(this, function(response, ioArgs) {
                        console.log(this.declaredClass + ": Get template complete: ", response, ioArgs);
                        if (response.items && (response.items.length > 0)) {
                            chsTemplate = response.items[0];
                        }
                    }),
                    error: dojo.hitch(this, function(error, ioArgs) {
                        console.log(this.declaredClass + ": Get template complete: ", error, ioArgs);

                        var response = dojo.fromJson(ioArgs.xhr.responseText);
                        if(response.status >= 500) {
                            // Internal error. Call support.
                            this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                        }
                        else {
                            // Error occurred. Try again or call support.
                            this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                        }
                    })
                });
                if (chsTemplate) {

                    this.templateIDNum = chsTemplateId;
                    this.preGeneratedTemplate = true;
                    
                    // General Page
                    this.generalPage.setNameAndDesc(chsTemplate.name, chsTemplate.description);

                    // MM Page
                    this.mgmtModulesPage.pageParams = chsTemplate.mmSettings;
                    this.mgmtModulesPage.refreshAllSelectors();

                    // IOPage
                    this.ioModulesPage.pageParams = chsTemplate.ioSettings;
                    this.ioModulesPage.refreshAllSelectors();

                    // Node Page
                    this.nodePage.pageParams = chsTemplate.nodeSettings;
                    this.nodePage.loadLearnedTemplates();
                }

            }

        },
        _generateTemplateData: function() {
            // tags:
            // protected
            // Generate Chassis Template object

            var chassisTemplate = new Object();

            // Append Standard information...
            if (this.editMode || this.preGeneratedTemplate) {
                chassisTemplate.id = this.templateIDNum;
            }
            chassisTemplate.type = this.templateType;
            chassisTemplate.userDefined = true;

            // Append General information...
            // chassisTemplate.nodeType = this.generalPage._nodeType.get("value").trim();
            // chassisTemplate.formFactor = this.generalPage._formFactor.get("value").trim();
            chassisTemplate.name = this.generalPage._templateName.get("value").trim();

            var description = this.generalPage._templateDescription.get("value").trim();
            chassisTemplate.description = this.encodeSpecialCharacters(description);

            // Mgmt Templates Info
            var templates = this.mgmtModulesPage._generateTemplateData();
            chassisTemplate.mmSettings = templates;

            // IO settings
            var ioSettings = this.ioModulesPage._generateTemplateData();
            chassisTemplate.ioSettings = ioSettings;

            // Node settings
            var nodeSettings = this.nodePage._generateTemplateData();
            chassisTemplate.nodeSettings = nodeSettings;

            return chassisTemplate;
        },

        onSave: function() {
            // Function connected to the "Save" button
            
            var chassisPatternChanged = false;
            
            if (this.editMode && this.templateInUse) {
                chassisPatternChanged = this._didChassisPatternSettingsChange();
            }

            if (this.editMode && this.templateInUse && chassisPatternChanged) {
                // If template is in use, trigger redeploy dialog
                var chsTemplate = this._generateTemplateData();
                var templateData = dojo.toJson({
                    "identifier": "name",
                    "label": "name",
                    "items": [ chsTemplate ]
                });
                require([ "support/dialogs/ReDeployServerTemplate" ], dojo.hitch(this, function() {
                    var rdstDialog = new support.dialogs.ReDeployServerTemplate({
                        "templateID": this.templateIDNum,
                        "templateType": this.templateType,
                        "templateData": templateData
                    });
                    dojo.connect(rdstDialog,"onClose",this,function(){
                        if(!rdstDialog){
                            return;
                        }
                        var deployClicked = rdstDialog.get("closeParent");
                        if(deployClicked) {
                            this.hide();
                        }
                    });
                    rdstDialog.show();
                }));
            }
            else {
                // Not in use, just save...

                // Set up subscriber for _saveTemplateAsync() below.
                var handle = dojo.subscribe("ChassisTemplateComplete", this, function(response) {
                    if ((response.status >= 200) && (response.status < 300)) {
                        this.hide();
                    }
                    else {
                        console.error(this.declaredClass + ": Error Saving Chassis Template");

                        if ((response.status >= 400) && (response.response.message == "service_error_name_in_use")) {
                            // Error, pattern name in use
                            this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                        }
                        else if (response.status == 404) {
                            // Error, pattern does not exist. Use Save As.
                            this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                        }
                        else if (response.status >= 500) {
                            // Internal error. Call support.
                            this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                        }
                        else {
                            // Error occurred. Try again or call support.
                            this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                        }

                    }
                    handle && dojo.unsubscribe(handle);
                });
                this._saveTemplateAsync();
            }
        },
        
        _getChassisPatternFromServices: function() {
            var chsTemplate = null;
            
            dojo.xhrGet({
                url: this.REST_PATH + "template/" + this.templateIDNum,
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                sync: true,
                preventCache: true,
                load: function(response, ioArgs) {
                    if (response.items && (response.items.length > 0)) {
                        chsTemplate = response.items[0];
                    }
                },
                handle: function(response, ioArgs) {
                    console.log(this.declaredClass + ": Load template complete: ", response, ioArgs);
                }
            });
            
            return chsTemplate;
        },
        
        _didChassisPatternSettingsChange: function() {
            this._originalChassisPattern = this._getChassisPatternFromServices();
            var currentChassisPattern = this._generateTemplateData();
            if (this._originalChassisPattern != null) {                
                if (this._didMgmtModuleSettingsChange()) {
                    return true;
                }
                if (this._didIOModuleSettingsChange(currentChassisPattern.ioSettings)) {
                    return true;
                }
                if (this._didNodeSettingsChange(currentChassisPattern.nodeSettings)) {
                    return true;
                }
            } else {
                // If this._originalChassisPattern is null force the redeploy
                return true;
            }
            return false;
        },

        _didMgmtModuleSettingsChange: function() {
            //  Check if any management module patterns have changed                
            var mgmtModuleSettingsVals = this.mgmtModulesPage._generateTemplateData();
            
            // Create an object with the original category pattern ids          
            var originalMgmtModuleSettings = new Object({
                chassisInfoTemplate: -1,
                chassisManagementTemplate: -1,
                performanceTemplate: -1,
                powerTemplate: -1
            });
            
            // Set the pattern ids
            for (var counter=0;counter < this._originalChassisPattern.mmSettings.length;++counter) {
                switch (this._originalChassisPattern.mmSettings[counter].type) {
                    case "cmmManagement":
                        originalMgmtModuleSettings.chassisManagementTemplate = this._originalChassisPattern.mmSettings[counter].id;
                        continue;
                    case "cmmInfo":
                        originalMgmtModuleSettings.chassisInfoTemplate = this._originalChassisPattern.mmSettings[counter].id;
                        continue;
                    case "cmmPower":
                        originalMgmtModuleSettings.powerTemplate = this._originalChassisPattern.mmSettings[counter].id;
                        continue;
                    case "cmmUsers":
                        originalMgmtModuleSettings.performanceTemplate = this._originalChassisPattern.mmSettings[counter].id;
                        continue;
                    default:
                        return true;
                }                
            }
            
            // Check if any category patterns have changed
            for (var counter=0;counter < mgmtModuleSettingsVals.length;++counter) {
                switch (mgmtModuleSettingsVals[counter].type) {
                    case "cmmManagement":
                        if(mgmtModuleSettingsVals[counter].id == null){mgmtModuleSettingsVals[counter].id = -1;}
                        if (originalMgmtModuleSettings.chassisManagementTemplate != mgmtModuleSettingsVals[counter].id) {
                            return true;
                        }
                        continue;
                    case "cmmInfo":
                        if(mgmtModuleSettingsVals[counter].id == null){mgmtModuleSettingsVals[counter].id = -1;}
                        if (originalMgmtModuleSettings.chassisInfoTemplate != mgmtModuleSettingsVals[counter].id) {
                            return true;
                        }
                        continue;
                    case "cmmPower":
                        if(mgmtModuleSettingsVals[counter].id == null){mgmtModuleSettingsVals[counter].id = -1;}
                        if (originalMgmtModuleSettings.powerTemplate != mgmtModuleSettingsVals[counter].id) {
                            return true;
                        }
                        continue;
                    case "cmmUsers":
                        if(mgmtModuleSettingsVals[counter].id == null){mgmtModuleSettingsVals[counter].id = -1;}
                        if (originalMgmtModuleSettings.performanceTemplate != mgmtModuleSettingsVals[counter].id) {
                            return true;
                        }
                        continue;
                    default:
                        return true;
                }                

            }
            return false;
        },
        
        _didIOModuleSettingsChange: function(ioSettings) {
            // Check if any io module settings changed
            if (this._originalChassisPattern.ioSettings.length > 0 && this._originalChassisPattern.ioSettings.length == ioSettings.length) {
                for (var index=0;index < this._originalChassisPattern.ioSettings.length;++index) {
                    if (this._originalChassisPattern.ioSettings[index].index == ioSettings[index].index && ioSettings[index].index == index+1) {
                        if (this._originalChassisPattern.ioSettings[index].id == ioSettings[index].id) {
                            continue;
                        } else {
                            return true;
                        }
                    } else {
                        return true;
                    }
                }
            } else {
                for (var counter=0;counter < ioSettings.length;++counter) {
                    if (ioSettings[counter].id != -1) {
                        return true;
                    }
                }                
            }
            return false;
        },
        
        _didNodeSettingsChange: function(nodeSettings) {
            // Check if any node settings changed
            
            // Create original node settings object
            var originalNodeSettings = new Array();

            for (var numNodes=0;numNodes < nodeSettings.length;++numNodes) {
                var nodeObject = new Object({
                    id: "-1",
                    index: "",
                    type: "cmmNode"
                });
                for (var origNodes=0;origNodes < this._originalChassisPattern.nodeSettings.length;++origNodes) {
                    if (this._originalChassisPattern.nodeSettings[origNodes].index == numNodes+1) {
                        nodeObject.id = this._originalChassisPattern.nodeSettings[origNodes].id;
                        break;
                    }
                }
                var indexString = numNodes+1;
                nodeObject.index = indexString.toString();
                originalNodeSettings.push(nodeObject);
            }
            
            if (this._originalChassisPattern.nodeSettings.length > 0) {
                for (var index=0;index < originalNodeSettings.length;++index) {
                    if (originalNodeSettings[index].index == nodeSettings[index].index && nodeSettings[index].index == index+1) {
                        if (originalNodeSettings[index].id == nodeSettings[index].id) {
                            continue;
                        } else {
                            return true;
                        }
                    } else {
                        return true;
                    }
                }
            } else {
                for (var counter=0;counter < nodeSettings.length;++counter) {
                    if (nodeSettings[counter].id != -1) {
                        return true;
                    }
                }                
            }
            return false;
        },
        
        setExisting: function() {
            this.createFromExisting = true;
        },

        _saveTemplateAsync: function() {
            // tags:
            // protected
            // Save Chassis template data using an asynchronous operation.
            // Subscribe to "ChassisTemplateComplete" to receive results.

            var chsTemplate = this._generateTemplateData();
            var postData = dojo.toJson({
                "identifier": "name",
                "label": "name",
                "items": [ chsTemplate ]
            });
            this.getProgressDialog("save").show(); // Show progress dialog for save request.
            dojo.xhrPost({
                url: this.REST_PATH + "template/create",
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                postData: postData,
                load: function(response, ioArgs) {
                    var templateId = response.id;
                    dojo.publish("ChassisTemplateComplete", {
                        status: ioArgs.xhr.status,
                        templateId: templateId,
                        response: response
                    }); // Inform caller(s) of success!
                },
                error: function(error, ioArgs) {
                    dojo.publish("ChassisTemplateComplete", {
                        status: ioArgs.xhr.status,
                        templateId: null,
                        response: dojo.fromJson(ioArgs.xhr.responseText)
                    }); // Inform caller(s) of failure!
                },
                handle: function(response, ioArgs) {
                    console.log(this.declaredClass + ": Save template complete: ", response, ioArgs);
                    if (ioArgs.xhr.status >= 300) {
                        dojo.publish("ProgressFailed", response);
                    }
                    else {
                        dojo.publish("ProgressComplete", response);
                    }
                }
            });
            // IMPORTANT! No return. Subscribe to "ChassisTemplateComplete" instead.

        }

    });

});