define([
        "dojo",
		"dijit",
		"support/dialogs/Dialog",
        "dojo/i18n!./nls/ChassisUserAndSecurityTemplate",
        "dojo/text!./templates/ChassisUserAndSecurityTemplate.html",
        "dijit/Tooltip",
        "dijit/layout/BorderContainer",
        "dijit/layout/ContentPane",
        "dijit/form/Button",
        "dijit/form/CheckBox",
        "dijit/form/Form",
        "dijit/form/NumberTextBox",
        "dijit/form/RadioButton",
        "dijit/form/Select",
        "dijit/form/SimpleTextarea",
        "dojo/data/ItemFileReadStore",
        "support/panels/CheckLabelButton",
        "support/widgets/ButtonBar",
        "support/panels/CheckButton",
        "support/widgets/OutlinePane",
        "support/dialogs/ReDeployServerTemplate",
        "support/dialogs/SaveAs",
        "dojox/grid/LazyTreeGrid",
        "support/dialogs/ProgressDialog"
    ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.ChassisUserAndSecurityTemplate", [ support.dialogs.Dialog], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        widgetsInTemplate: true,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "ChassisUserAndSecurityTemplate",

        templateIDNum: null,
        
        //Currently not used
        index: -1,
        
        /*
         * When the user authentication method is set to "Centralized user management via FSM", 
         * the overall security policy will be set the same as the FSM security policy when this pattern is deployed.
         * Thus, if some centralized mode is enabled, some settings get removed from the pattern
         */
        //Centralized Mgmt mode 
        centralized: false,
        
        //External Authentication mode
        externalAuth: false,
        
        _cliTimeoutRangeMessage: null,
        
        _securityLevelBar: null,
        _accSecurityLevelBar: null,
        
        initialData: null,
        
        // User login event values
        LOG_ALL: "LOG_ALL",
        LOG_LIMIT_DUPS: "LOG_LIMIT_DUPS",
        LOG_NONE: "LOG_NONE",

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/ChassisUserAndSecurityTemplate.css",

        // When this method is called, all variables inherited from superclasses
        // are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in
        // the template HTML file
        postMixInProperties: function() {
            this.inherited(arguments);
            this._cliTimeoutRangeMessage = this.locResources.CLI_TIMEOUT_MSG.replace("%1", 0).replace("%2", 4294967294);
        },

        constructor: function() {
            //Load additional MRI files
            this.ioResources = dojo.i18n.getLocalization("support.dialogs", "ChassisIOModuleTemplate", this.lang);
        },

        // postCreate() is called after buildRendering(). This is useful to
        // override when
        // you need to access and/or manipulate DOM nodes included with your
        // widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified
        // can now be
        // directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);
            
            // Create the button bar
            this._securityLevelBar = new support.widgets.ButtonBar({
                values: this.locResources.LEGACY + "|" + this.locResources.SECURE,
                selected: this.locResources.LEGACY,
                onClick: dojo.hitch(this, this._displaySecurityArea)
            }).placeAt(this._securityLevelBarNode);
            
            this._accSecurityLevelBar = new support.widgets.ButtonBar({
                values: this.resources.HIGH+ "|" + this.locResources.LEGACY,
                selected: this.resources.HIGH,
                onClick: dojo.hitch(this, this._setupAcctSecText)
            }).placeAt(this._accSecurityLevelBarNode);

            if (this.templateIDNum != null) {

                // Setup for template for edit or copy mode
                this._setup();
                this._expandall();
                this._readSettingsFromService();

            }
            else {

                // Setup for template new mode
                this._setup();
                this._selectall();
            }

            // Apply env specific message for Centralized security mode
            var message = new Array();
            message.push("<div style=\"width: 30em;\">");
            message.push(this.locResources.TT1_1.replace("%1", this.PRODUCT_NAME_ABBR));
            message.push("<br><br>" + this.locResources.TT1_2);
            message.push("<br><br>" + this.locResources.TT1_3);
            // NOTE: For dijit/Tooltip dijits, set the label, not innerHTML.
            this._userAuthtip.label = message.join("");

            message.length = 0; // Empty the array, effectively and efficiently.
            message.push(this.locResources.CENT_MSG1);
            message.push("<br><br>" + this.locResources.CENT_MSG2.replace("%1", this.PRODUCT_NAME_ABBR).replace("%2", this.PRODUCT_NAME_SHORT));
            message.push("<br><br>" + this.locResources.CENT_MSG3);
            this.securityModeCentralized.innerHTML = message.join("");
            
            message.length = 0; // Empty the array, effectively and efficiently.
            message.push(this.locResources.CENT_MSG1);
            message.push("<br><br>" + this.locResources.CENT_MSG4.replace("%1", this.PRODUCT_NAME_ABBR));
            message.push("<br><br>" + this.locResources.CENT_MSG3);
            this.accsecurityModeCentralized.innerHTML = message.join("");
            
            dojo.connect(this.dialogNode, "hide", this, function() {
                this.destroy();
            });
            
            dojo.connect(this._cliToUserInput, "onChange", this, function() {this._validate();});
            
        },
        
        //Load in saved pattern data from backend
        _readSettingsFromService: function() {
            var settings = null;
            var mode = this.templateMode;

            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "template/" + this.templateIDNum,
                handleAs: "json",
                sync: true,
                load: function(response) {
                    settings = response;
                },
                error: function(error) {
                    console.error(this.declaredClass + ": " + this.locResources.ERRORTEMPLATE);
                }
            });

            // Set dialog title to reflect Editing and display the warning message
            // this.ethwarningMessage.style.display = "block";

            this.initialData = settings.items;
            
            this.templateInUse = settings.items[0].inUse;

            if (mode == "copy") {
                this.dialogNode.set("title", this.resources.COPY + " " + this.locResources.TITLE + " - " + settings.items[0].name
                        + " - " + this.resources.COPY);
                this.templateName.set("value", settings.items[0].name + " - " + this.resources.COPY);
            }
            else {
                this.dialogNode.set("title", this.locResources.EDITTITLE + " - " + settings.items[0].name);
                this.templateName.set("value", settings.items[0].name);
            }
            var descriptionvalue = settings.items[0].description;
            var description = this.decodeSpecialCharacters(descriptionvalue);
            this.ethadapDesc.set("value", description);

            // Setup security policy
            if (settings.items[0]["Chassis Security Policy"] && settings.items[0]["Chassis Security Policy"].CMM__security__p == "secure") {
                this._securityLevelBar._setSelIndexAttr(1);
                this._displaySecurityArea();
            }
            // Authentication method and account security
            if (settings.items[0]["Authentication Method"].method != "Centralized") {
                this._authSelector.set("value", settings.items[0]["CMM Auth Method"].CMM__accseccfg__am);
                //External ldap authentication, setup same text as centralized mode
                if(settings.items[0]["CMM Auth Method"].CMM__accseccfg__am == "ldap"){
                    this.externalAuth = true;
                    //this._setupForCentralized();
                }
                else if (settings.items[0]["Account Security Config"].CMM__accseccfg__high) {
                    this._accSecurityLevelBar._setSelIndexAttr(0);
                    this._displaySecurityArea();
                }
                else if (settings.items[0]["Account Security Config"].CMM__accseccfg__legacy) {
                    this._accSecurityLevelBar._setSelIndexAttr(1);
                    this._displaySecurityArea();
                }
            }
            else {
                this.centralized = true;
                this._setupForCentralized();
                this._authSelector.set("value", "centralized");
            }
            
            //CLI, Web Inactivity, Logging timeouts
            this._cliToUserInput.set("value", settings.items[0]["CMM CLI Timeout"].CMM__accseccfg__ct);
            this._webInacSelector.set("value", settings.items[0]["CMM Web Timeout"].CMM__accseccfg__wt.toString());

            if (settings.items[0]["CMM Login Logging Timeout"]) {
                if(settings.items[0]["CMM Login Logging Timeout"].CMM__accseccfg__alt == ""){
                    this._userLogging._setSelIndexAttr(2); // a.k.a:  this.LOG_NONE
                }
                else if(settings.items[0]["CMM Login Logging Timeout"].CMM__accseccfg__alt != "0"){
                    this._delaySelector.set("value", settings.items[0]["CMM Login Logging Timeout"].CMM__accseccfg__alt);
                    this._userLogging._setSelIndexAttr(1); // a.k.a:  this.LOG_LIMIT_DUPS
                    this.ipCheckBox.set("checked", settings.items[0]["CMM Ignore Client IP for Logging"].CMM__accseccfg__ici == "on");
                }
            }
            
            //If pattern is inuse and displayed from the wizard, take away save btn and display warning msg
            if (this.templateInUse && this.templateInUse==true && this.templateLaunchPoint && this.templateLaunchPoint == "wizard") {
                this.informationMessage.style.display = "block";
                this.informationMessageText.innerHTML = this.resources.INUSEWIZARD;
                this.saveBtnContainer.style.display = "none";
            }

        },

        _saveandhideDialog: function(evt) {
                 
            var patternObj = this._generateSavedData();

            var argJSONString = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": [ patternObj ]
            }, true);

            this.getProgressDialog("save").show(); // Show progress dialog for save request.
            var xhrPost = dojo.xhrPost({
                url: this.REST_PATH + "template/create",
                handleAs: "json", // <<<< Required
                headers: {
                    "Content-Type": "application/json; charset=utf-8" // <<<< Required
                },
                postData: argJSONString,     //Template Data to Post
                load: dojo.hitch(this, function(response, ioArgs) {
                    // Happy path! We received a 2xx status code from the API.
                                            
                    dojo.publish("ProgressComplete", response.id);
                    dojo.publish("ChassisUserTemplateComplete", response.id);
                    this.hide();   
                }),
                error: dojo.hitch(this, function(error, ioArgs) {
                    // Failure path! We received a non-2xx status code (ie: 3xx/4xx/5xx status code) from the API.
    
                    dojo.publish("ProgressFailed", error);
                    console.log(this.declaredClass + ": ", error, ioArgs);
                    
                    var response = dojo.fromJson(ioArgs.xhr.responseText);
                    if ((ioArgs.xhr.status >= 400) && (response.message == "service_error_name_in_use")) {
                        // Error Template name in use
                        this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                    }
                    else if(ioArgs.xhr.status == 404) {
                        // Edit Template ID doesn't exist
                        this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                        this.hide();
                    }
                    else if(ioArgs.xhr.status >= 500) {
                        // Internal Programming Error
                        this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                        this.hide();
                    }
                    else {
                        this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                        this.hide();
                    }
                    
                }),
                handle: dojo.hitch(this, function(data, ioArgs) {
                    // - The 1st param ("data") is either the response data (on success) or Error object (on failure).
                    // - The 2nd param ("ioArgs") contains detail about the xhr request, including a reference to the
                    // native xhr object. Check ioArgs.xhr.status to determine success or failure, and handle "data"
                    // accordingly.
                })
            });

        },

        saveClicked: function() {

            var patternObj = this._generateSavedData();
            patternObj.id = this.templateIDNum;

            var argJSONString = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": [ patternObj ]
            }, true);
            
            var saveTemplate = true;

            if (this.templateInUse.toString() == "true") {
            //  Need to remove extra properties that aren't included in the dialog
                delete this.initialData[0].inUse;
                delete this.initialData[0].uri;
                delete this.initialData[0].referencedBy;
                delete this.initialData[0].useCount;
                delete this.initialData[0].userDefined;
                
                // Remove description since it shouldn't force a redeploy
                delete this.initialData[0].description;
                delete patternObj.description;
                
                var preData = this.sortAndFlatten(this.initialData[0]);
                var postData = this.sortAndFlatten(patternObj);
                
                if(preData != postData){
                    this.rdstDialog.set("templateID", this.templateIDNum);
                    this.rdstDialog.set("templateType", "Category");
                    this.rdstDialog.set("templateData", argJSONString);

                    this.rdstDialog.show();
                    
                    saveTemplate = false;
                }
            }
            if (saveTemplate){

                this.getProgressDialog("save").show(); // Show progress dialog for save request.
                var xhrPost = dojo.xhrPost({
                    url: this.REST_PATH + "template/create",
                    handleAs: "json", // <<<< Required
                    headers: {
                        "Content-Type": "application/json; charset=utf-8" // <<<< Required
                    },
                    postData: argJSONString,     //Template Data to Post
                    load: dojo.hitch(this, function(response, ioArgs) {
                        // Happy path! We received a 2xx status code from the API.
                                                
                        dojo.publish("ProgressComplete", response.id);
                        dojo.publish("ChassisUserTemplateComplete", response.id);
                        this.hide();   
                    }),
                    error: dojo.hitch(this, function(error, ioArgs) {
                        // Failure path! We received a non-2xx status code (ie: 3xx/4xx/5xx status code) from the API.

                        dojo.publish("ProgressFailed", error);
                        console.log(this.declaredClass + ": ", error, ioArgs);
                        
                        var response = dojo.fromJson(ioArgs.xhr.responseText);
                        if ((ioArgs.xhr.status >= 400) && (response.message == "service_error_name_in_use")) {
                            // Error Template name in use
                            this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                        }
                        else if(ioArgs.xhr.status == 404) {
                            // Edit Template ID doesn't exist
                            this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                            this.hide();
                        }
                        else if(ioArgs.xhr.status >= 500) {
                            // Internal Programming Error
                            this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                            this.hide();
                        }
                        else {
                            this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                            this.hide();
                        }
                        
                    }),
                    handle: dojo.hitch(this, function(data, ioArgs) {
                        // - The 1st param ("data") is either the response data (on success) or Error object (on failure).
                        // - The 2nd param ("ioArgs") contains detail about the xhr request, including a reference to the
                        // native xhr object. Check ioArgs.xhr.status to determine success or failure, and handle "data"
                        // accordingly.
                    })
                });
            }

        },

        saveasClicked: function() {

            var patternObj = this._generateSavedData();
            this.saveasDialog.set("templateObject", patternObj);
            this.saveasDialog.show();

        },

        _cancelDialog: function() {
            this.hide();
        },

        _selectall: function() {
            this._expandall();
            this.layout();
        },

        _deselectall: function() {
            this.layout();
        },

        _expandall: function(evt) {
            this.userCheck_loginCheck.set("checked", "on");
            this.securityAreaCheck.set("checked", "on");

            this.expandall.style.display = "none";
            this.collapseall.style.display = "inline";
            this.mixedexpandcollapse.style.display = "none";
            dojo.style(this.securityArea, "display", "");
            dojo.style(this.userPermArea, "display", "");
            dojo.style(this._authArea, "display", "");
            dojo.style(this._webInacArea, "display", "");
            dojo.style(this._cliToArea, "display", "");
            dojo.style(this._userLoggingArea, "display", "");
            dojo.style(this._accountSecurityArea, "display", "");
            dojo.style(this._accountSecurityMsgArea, "display", "");
            this.layout();
        },

        _collapseall: function() {
            this.userCheck_loginCheck.set("checked", false);
            this.securityAreaCheck.set("checked", false);

            this.expandall.style.display = "inline";
            this.collapseall.style.display = "none";
            this.mixedexpandcollapse.style.display = "none";
            dojo.style(this.securityArea, "display", "none");
            dojo.style(this.userPermArea, "display", "none");
            dojo.style(this._authArea, "display", "none");
            dojo.style(this._webInacArea, "display", "none");
            dojo.style(this._cliToArea, "display", "none");
            dojo.style(this._userLoggingArea, "display", "none");
            dojo.style(this._accountSecurityArea, "display", "none");
            dojo.style(this._accountSecurityMsgArea, "display", "none");
            this.layout();
        },

        _displaySecuritySettings: function(evt) {
            if (this.securityArea.style.display == "none") {
                dojo.style(this.securityArea, "display", "");
            }
            else {
                dojo.style(this.securityArea, "display", "none");
            }

            this.layout();
        },

        _displayGlobalLoginSettings: function() {
            if (this._authArea.style.display == "none") {
                dojo.style(this._authArea, "display", "");
                dojo.style(this._webInacArea, "display", "");
                dojo.style(this._cliToArea, "display", "");
                dojo.style(this._userLoggingArea, "display", "");
                this._displayLoggingArea();
                dojo.style(this._accountSecurityArea, "display", "");
                dojo.style(this._accountSecurityMsgArea, "display", "");
            }
            else {
                dojo.style(this._authArea, "display", "none");
                dojo.style(this._webInacArea, "display", "none");
                dojo.style(this._cliToArea, "display", "none");
                dojo.style(this._userLoggingArea, "display", "none");
                dojo.style(this._delayLoggingArea, "display", "none");
                dojo.style(this._accountSecurityArea, "display", "none");
                dojo.style(this._accountSecurityMsgArea, "display", "none");
            }
            this.layout();
        },

        _hideUncheckedClicked: function(evt) {
            
            if (this.hideuncheckedToggleButtonId.get("checked")) { // Hide info if defaults
                if (this._securityLevelBar.get("selected") == this.locResources.SECURE) {
                    this.securityArea.style.display = "none";                    
                }
                if (this._authSelector.get("value") == "local") {
                    this._authArea.style.display = "none";
                }
                if (this._webInacSelector.get("value") == "user") {
                    this._webInacArea.style.display = "none";
                }
                if (this._cliToUserInput.get("value") == 300) {
                    this._cliToArea.style.display = "none";
                }
                if (this._userLogging.get("selected") == this.LOG_ALL) {
                    this._userLoggingArea.style.display = "none";
                }
                if (this._securityLevelBar.get("selected") == this.locResources.SECURE
                        && (this._authSelector.get("value") == "local"
                            || this._authSelector.get("value") == "localldap"
                            || this._authSelector.get("value") == "ldaplocal")) {
                    this._accountSecurityArea.style.display = "none";
                    this._accountSecurityMsgArea.style.display = "none";
                }
            }
            else { // Re-display if had been previously hidden
                this.securityArea.style.display = "";
                this._authArea.style.display = "";
                this._webInacArea.style.display = "";
                this._cliToArea.style.display = "";
                this._userLoggingArea.style.display = "";
                this._accountSecurityArea.style.display = "";
                this._accountSecurityMsgArea.style.display = "";
            }

            this.layout();
            return;
        },

        exitParent: function() {
            var data = "ChassisManagement";
            dojo.publish("ChassisUserTemplateComplete", data);

            if(this.rdstDialog != null){
                var deployClicked = this.rdstDialog.get("closeParent");
                if(deployClicked) {
                    this.hide();
                }
            }
        },

        _validate: function() {
            //Name must be present and cli timeout input should be valid
            if (this.templateName.get("value").trim() != "" && this._cliToUserInput.isValid()) {
                this.okButton.set("disabled", false);
                this.saveBtn.set("disabled", false);
                this.saveasBtn.set("disabled", false);
            }
            else {
                this.okButton.set("disabled", true);
                this.saveBtn.set("disabled", true);
                this.saveasBtn.set("disabled", true);
            }
        },

        _setup: function() {

            // Set Template Field to catch intermediate changes
            this.templateName.intermediateChanges = true;

            // Set Description Label
            this._descLimit.innerHTML = this.resources.LIMIT.replace("%500", "500");

            // Set Information Messages and Load Previous Settings
            if (this.templateIDNum != null && this.templateMode == "edit") { // if dialog in edit
                                                                                // mode
                if (this.templateInUse.toString() == "true") {

                    if (this.templateLaunchPoint == "table") {
                        
                        this.warningMessage.style.display = "block";
                        this.warningMessageText.innerHTML = this.resources.INUSE;
                        this.saveBtnContainer.style.display = "inline";
                        this.saveasBtnContainer.style.display = "inline";

                    }
                    else if (this.templateLaunchPoint == "wizard") {
                        this.informationMessage.style.display = "block";
                        this.informationMessageText.innerHTML = this.resources.INUSEWIZARD;
                        this.saveasBtnContainer.style.display = "inline";

                    }
                    else {
                        this.informationMessage.style.display = "block";
                        this.informationMessageText.innerHTML = this.resources.INUSEWIZARD;
                        this.saveasBtnContainer.style.display = "inline";
                    }

                }
                else {

                    this.saveBtnContainer.style.display = "inline";
                    this.saveasBtnContainer.style.display = "inline";

                }

                this.templateName.set("disabled", true);

            }
            else if (this.templateIDNum != null && this.templateMode == "copy") { // if dialog in
                                                                                    // copy mode

                // this.loadprevSettings();
//                this.templateIDNum = null; // set template ID to null in order to create new
                                            // template
                this.okBtnContainer.style.display = "inline";

            }
            else { // if dialog in create new mode

                this.okBtnContainer.style.display = "inline";
                this._securityLevelBar._setSelIndexAttr(1);   // Set secure as default security setting

            }

            if (this.centralized || this.externalAuth) {
                // Setup security level settings
                this._setupForCentralized();

            }

            this._setupGlobalLoginArea();
            this._setupValidators();
        },

        /**
         * Function to create saved pattern data
         */
        _generateSavedData: function() {
            var patternObj = new Object();

            patternObj.name = this.templateName.get("value").trim();
            patternObj.type = "cmmUsers";
            patternObj.description = this.encodeSpecialCharacters(this.ethadapDesc.get("value").trim());
            patternObj["Authentication Method"] = {
                "method": (this.centralized) ? "Centralized" : "Local"
            };
            //Centralized and external auth override account security selections
            if (!this.centralized) {
                patternObj["CMM Auth Method"] = {
                    "CMM__accseccfg__am": this._authSelector.value,
                    "CMM__accseccfg__T": "mm[P]"
                };
                //External and Centralized dont take an account security setting
                if (!this.externalAuth) {
                    //Account security is legacy or high
                    if (this._accSecurityLevelBar.get("selected") == this.locResources.LEGACY) {
                        patternObj["Account Security Config"] = {
                            "CMM__accseccfg__legacy": true,
                            "CMM__accseccfg__T": "mm[P]"
                        };
                    }
                    else if (this._accSecurityLevelBar.get("selected") == this.resources.HIGH) {
                        patternObj["Account Security Config"] = {
                            "CMM__accseccfg__high": true,
                            "CMM__accseccfg__T": "mm[P]"
                        };
                    }
                }
            }
            patternObj["CMM CLI Timeout"] = {
                "CMM__accseccfg__ct": (this._cliToUserInput.get("value") == "" || !this._cliToUserInput.isValid()) ? "0" : this._cliToUserInput.get("value"),
                "CMM__accseccfg__T": "mm[P]"
            };
            patternObj["CMM Web Timeout"] = {
                "CMM__accseccfg__wt": this._webInacSelector.get("value"),
                "CMM__accseccfg__T": "mm[P]"
            };
            //Centralized mode ignores our sec policy choice
            if(!this.centralized){
                patternObj["Chassis Security Policy"] = {
                    "CMM__security__p": (this._securityLevelBar.get("selected")== this.locResources.LEGACY) ? "legacy" : "secure",
                    "CMM__security__T": "mm[P]"
                };
            }
            var logging = this._userLogging.get("selected");
            if (logging == this.LOG_LIMIT_DUPS) {
                patternObj["CMM Login Logging Timeout"] = {
                    "CMM__accseccfg__alt": this._delaySelector.get("value"),
                    "CMM__accseccfg__T": "mm[P]"
                };
                patternObj["CMM Ignore Client IP for Logging"] = {
                    "CMM__accseccfg__ici": (this.ipCheckBox.get("checked") == true) ? "on" : "off",
                    "CMM__accseccfg__T": "mm[P]"
                };
            }
            else if(logging == this.LOG_ALL){
                patternObj["CMM Login Logging Timeout"] = {
                    "CMM__accseccfg__alt": "0",
                    "CMM__accseccfg__T": "mm[P]"
                };
            }
            else if(logging == this.LOG_NONE){
                patternObj["CMM Login Logging Timeout"] = {
                    "CMM__accseccfg__alt": "",
                    "CMM__accseccfg__T": "mm[P]"
                };
            }

            // If doing an edit, need to add the id
            if (this.templateIDNum != null && this.templateMode != "copy") {
                patternObj.id = this.templateIDNum;
            }
            else {
                this.templateIDNum = null;
            }

            return patternObj;
        },

        _displaySecurityArea: function() {
            var level = this._securityLevelBar.get("selected");
            if (this.centralized) {
                dojo.style(this._centSecurity, "display", "");
                dojo.style(this._securityBarArea, "display", "none");
                dojo.style(this.securityModeCentralized, "display", "");
                dojo.style(this.securityModeSecure, "display", "none");
                dojo.style(this.securityModeLegacy, "display", "none");
                dojo.style(this._acctSecFixed, "display", "none");
            }
            else if (level == this.locResources.LEGACY) {
                dojo.style(this._centSecurity, "display", "none");
                dojo.style(this._securityBarArea, "display", "");
                dojo.style(this.securityModeCentralized, "display", "none");
                dojo.style(this.securityModeSecure, "display", "none");
                dojo.style(this.securityModeLegacy, "display", "");
                dojo.style(this._accSecurityLevelBarDiv, "display", "");
                dojo.style(this._accSecurityLevelLabel, "display", "none");
                dojo.style(this._acctSecFixed, "display", "none");
            }
            else { //Secure
                dojo.style(this._centSecurity, "display", "none");
                dojo.style(this._securityBarArea, "display", "");
                dojo.style(this.securityModeCentralized, "display", "none");
                dojo.style(this.securityModeSecure, "display", "");
                dojo.style(this.securityModeLegacy, "display", "none");
                dojo.style(this._accSecurityLevelBarDiv, "display", "none");
                dojo.style(this._accSecurityLevelLabel, "display", "");
                dojo.style(this._acctSecFixed, "display", "");
                this._accSecurityLevelBar.set("selIndex", 0);
            }
            
            //Change account security text as well
            this._setupAcctSecText();
        },

        /**
         * Function to display correct settings when centralized management is enabled
         */
        _setupForCentralized: function() {
            this._displaySecurityArea();
            this._setupAcctSecText();
            this.layout();
        },

        /**
         * Function to setup various selectors, grids...etc used on the pattern page
         */
        _setupGlobalLoginArea: function() {
            this._authSelector = new dijit.form.Select({
                options: [ {
                    label: this.locResources.USER_AUTH_OPT1.replace("%1", this.PRODUCT_NAME_ABBR),
                    value: "centralized"
                }, {
                    label: this.locResources.USER_AUTH_OPT2,
                    value: "local",
                    selected: true
                }, {
                    label: this.locResources.USER_AUTH_OPT3,
                    value: "ldap"
                }, {
                    label: this.locResources.USER_AUTH_OPT4,
                    value: "localldap"
                }, {
                    label: this.locResources.USER_AUTH_OPT5,
                    value: "ldaplocal"
                } ]
            }).placeAt(this._authSelector);
            this._authSelector.onChange = dojo.hitch(this, function() {
                // console.log(this.declaredClass + ": ", this.authSelector.value);
                if (this._authSelector.value == "centralized") {
                    this.centralized = true;
                    this.externalAuth = false;
                    this._setupForCentralized();
                }
                else if (this._authSelector.value == "ldap") {
                    this.externalAuth = true;
                    this.centralized = false;
                    this._setupForCentralized();
                }
                else {
                    this.centralized = false;
                    this.externalAuth = false;
                    this._setupForCentralized();
                }

            });

            this._webInacSelector = new dijit.form.Select({
                options: [ {
                    label: this.locResources.TIMEOUT_OPT1,
                    value: "user",
                    selected: true
                }, {
                    label: this.locResources.TIMEOUT_OPT2,
                    value: "none"
                }, {
                    label: this.resources.MINS.replace("%1", 1).replace("%s", ""),
                    value: "1"
                }, {
                    label: this.resources.MINS.replace("%1", 5).replace("%s", "s"),
                    value: "5"
                }, {
                    label: this.resources.MINS.replace("%1", 10).replace("%s", "s"),
                    value: "10"
                }, {
                    label: this.resources.MINS.replace("%1", 15).replace("%s", "s"),
                    value: "15"
                }, {
                    label: this.resources.MINS.replace("%1", 20).replace("%s", "s"),
                    value: "20"
                } ]
            }).placeAt(this._webInacSelector);

            // User login event button bar
            var values = this.LOG_ALL + "|" + this.LOG_LIMIT_DUPS + "|" + this.LOG_NONE;
            var labels = this.locResources.LOG_ALL + "|" + this.locResources.LOG_DELAY + "|" + this.locResources.LOG_NONE;
            this._userLogging = new support.widgets.ButtonBar({
                id: this.id + "_userLogging",
                values: values,
                labels: labels,
                selected: this.LOG_ALL
            }).placeAt(this._userLogging);
            this._userLogging.onClick = dojo.hitch(this, this._displayLoggingArea);

            //Delay selector, internal values are in seconds.  Displays more user friendly units of time
            this._delaySelector = new dijit.form.Select({
                options: [ {
                    label: this.resources2.SECONDS.replace("%1", 5),
                    value: "5",
                    selected: true
                }, {
                    label: this.resources2.SECONDS.replace("%1", 30),
                    value: "30"
                }, {
                    label: this.resources.MINS.replace("%1", 1).replace("%s", ""),
                    value: "60"
                }, {
                    label: this.resources.MINS.replace("%1", 5).replace("%s", "s"),
                    value: "300"
                }, {
                    label: this.resources.MINS.replace("%1", 10).replace("%s", "s"),
                    value: "600"
                }, {
                    label: this.resources.MINS.replace("%1", 30).replace("%s", "s"),
                    value: "1800"
                }, {
                    label: this.resources.HOURS.replace("%1", 1).replace("%s", ""),
                    value: "3600"
                }, {
                    label: this.resources.HOURS.replace("%1", 12).replace("%s", "s"),
                    value: "43200"
                }, {
                    label: this.resources.DAYS.replace("%1", 1).replace("%s", ""),
                    value: "86400"
                } ]
            }).placeAt(this._delaySelector);

        },

        /**
         * Function used to setup input validation functions.
         */
        _setupValidators: function() {
            // CLI Timeout bounds check
            this._cliToUserInput.validator = dojo.hitch(this, function() {
                var value = this._cliToUserInput.get("value");
                var i;
                for (i = 0; i < value.length; i++){
                    var code = value.charCodeAt(i);
                    //non number
                    if(code < 48 || code > 57){
                        return false;
                    }
                }
                //check range of number
                var num = parseInt(value);
                if (num > -1 && num < 4294967295) {
                    return true;
                }
                return false;
            });
        },

        _displayLoggingArea: function() {
           var value = this._userLogging.get("selected");
            if (value == this.LOG_LIMIT_DUPS) {
                dojo.style(this._delayLoggingArea, "display", "");
            }
            else {
                dojo.style(this._delayLoggingArea, "display", "none");
            }
        },

        /**
         * Function to setup text displayed depending on what account security level is chosen.
         * Content varies based on centralized managment state, security policy, and account security settings.
         */
        _setupAcctSecText: function() {
            var accSecLegacy = (this._accSecurityLevelBar.get("selected") == this.locResources.LEGACY);
            var secPolicyLegacy = (this._securityLevelBar.get("selected")== this.locResources.LEGACY);
            
            if (this.centralized || this.externalAuth) {
                if (this.centralized){
                    dojo.style(this._centAccSecurity, "display", "");
                    dojo.style(this._extAccSecurity, "display", "none");
                }
                if (this.externalAuth){
                    dojo.style(this._extAccSecurity, "display", "");
                    dojo.style(this._centAccSecurity, "display", "none");
                }
                dojo.style(this._normAccSecurity, "display", "none");
                dojo.style(this.accsecurityModeCentralized, "display", "");
                dojo.style(this.acctSecSettings, "display", "none");
                return;
            }
            //Legacy Security Policy
            if(secPolicyLegacy == true){
                //Legacy Acct Security
                if (accSecLegacy == true) {
                    dojo.style(this._normAccSecurity, "display", "");
                    dojo.style(this._centAccSecurity, "display", "none");
                    dojo.style(this._extAccSecurity, "display", "none");
                    dojo.style(this.accsecurityModeCentralized, "display", "none");
                    dojo.style(this.acctSecSettings, "display", "");
                    this.acctSecLegacy_pwdEx.innerHTML = this.ioResources.NONE;
                    
                    // Hide the "Force user to change password" message for Legacy
                    dojo.style(this.acctSecLegacy_forceUser, "display", "none");
                    dojo.style(this.acctSecLegacy_force, "display", "none");
                                        
                    this.acctSecLegacy_minPw.innerHTML = this.ioResources.NONE;
                    this.acctSecLegacy_minPw2.innerHTML = this.ioResources.NONE;
                    this.acctSecLegacy_lockTitle.innerHTML = this.locResources.LOCK.replace("%1", "2").replace("%2", "20");
                    this.acctSecLegacy_lock.innerHTML = this.resources.YES;
                    this.acctSecLegacy_pwRuleTitle.innerHTML = this.locResources.SIMPLE_PW;
                    this.acctSecLegacy_pwRule.innerHTML = /*"*" + */this.resources.YES;
                    dojo.style(this.acctSecLegacy_pwRule, "font-weight", "bold");
                    this.acctSecLegacy_actInacTitle.innerHTML = this.locResources.ACCT_INACT;
                    this.acctSecLegacy_actInac.innerHTML = this.ioResources.NONE;
                    dojo.style(this.acctSecLegacy_actInac2Title, "display", "none");
                    dojo.style(this.acctSecLegacy_actInac2, "display", "none");
                    
                    // Hide the "Factory default USERID account" message for Legacy
                    dojo.style(this.acctSecLegacy_factoryDef, "display", "none");
                    dojo.style(this.acctSecLegacy_userid, "display", "none");
                    
                    dojo.style(this.acctSecLegacy_msg, "display", "none");
                }
                //High Account Security
                else {
                    dojo.style(this._normAccSecurity, "display", "");
                    dojo.style(this._centAccSecurity, "display", "none");
                    dojo.style(this._extAccSecurity, "display", "none");
                    dojo.style(this.accsecurityModeCentralized, "display", "none");
                    dojo.style(this.acctSecSettings, "display", "");
                    this.acctSecLegacy_pwdEx.innerHTML = "90";
                    
                    // Show the "Force user to change password" message for High level
                    dojo.style(this.acctSecLegacy_forceUser, "display", "");
                    dojo.style(this.acctSecLegacy_force, "display", "");
                    this.acctSecLegacy_forceUser.innerHTML = this.locResources.FORCE_USER;                    
                    this.acctSecLegacy_force.innerHTML = this.resources.YES;                    
                    dojo.style(this.acctSecLegacy_force, "font-weight", "normal");

                    this.acctSecLegacy_minPw.innerHTML = "5";
                    this.acctSecLegacy_minPw2.innerHTML = "24";
                    this.acctSecLegacy_lockTitle.innerHTML = this.locResources.LOCK.replace("%1", "60").replace("%2", "20");
                    this.acctSecLegacy_lock.innerHTML = this.resources.YES;
                    this.acctSecLegacy_pwRuleTitle.innerHTML = this.locResources.COMPLEX_PW.replace("%1", "2");
                    this.acctSecLegacy_pwRule.innerHTML = this.resources.YES;
                    dojo.style(this.acctSecLegacy_pwRule, "font-weight", "normal");
                    this.acctSecLegacy_actInacTitle.innerHTML = this.locResources.ALERT.replace("%1", "120");
                    this.acctSecLegacy_actInac.innerHTML = this.resources.YES;
                    dojo.style(this.acctSecLegacy_actInac2Title, "display", "");
                    dojo.style(this.acctSecLegacy_actInac2, "display", "");
                    this.acctSecLegacy_actInac2Title.innerHTML = this.locResources.ACCT_DISABLE.replace("%1", "180");
                    this.acctSecLegacy_actInac2.innerHTML = this.resources.YES;
                    
                    // Show the "Factory default USERID account" message for High level
                    dojo.style(this.acctSecLegacy_factoryDef, "display", "");
                    this.acctSecLegacy_factoryDef.innerHTML = this.locResources.FACT_DEF;
                    dojo.style(this.acctSecLegacy_userid, "display", "");
                    this.acctSecLegacy_userid.innerHTML = this.resources.YES;
                    dojo.style(this.acctSecLegacy_userid, "font-weight", "normal");

                    dojo.style(this.acctSecLegacy_msg, "display", "none");
                }
            }
            //Legacy Security Policy
            else {
                //Legacy Account Security
                if (accSecLegacy == true) {
                    dojo.style(this._normAccSecurity, "display", "");
                    dojo.style(this._centAccSecurity, "display", "none");
                    dojo.style(this._extAccSecurity, "display", "none");
                    dojo.style(this.accsecurityModeCentralized, "display", "none");
                    dojo.style(this.acctSecSettings, "display", "");
                    this.acctSecLegacy_pwdEx.innerHTML = this.ioResources.NONE;
                    
                    // Hide the "Force user to change password" message for Legacy
                    dojo.style(this.acctSecLegacy_forceUser, "display", "none");
                    dojo.style(this.acctSecLegacy_force, "display", "none");
                    
                    this.acctSecLegacy_minPw.innerHTML = this.ioResources.NONE;
                    this.acctSecLegacy_minPw2.innerHTML = this.ioResources.NONE;
                    this.acctSecLegacy_lockTitle.innerHTML = this.locResources.LOCK.replace("%1", "2").replace("%2", "20");
                    this.acctSecLegacy_lock.innerHTML = this.resources.YES;
                    this.acctSecLegacy_pwRuleTitle.innerHTML = this.locResources.SIMPLE_PW;
                    this.acctSecLegacy_pwRule.innerHTML = "*" + this.resources.YES;
                    dojo.style(this.acctSecLegacy_pwRule, "font-weight", "bold");
                    this.acctSecLegacy_actInacTitle.innerHTML = this.locResources.ACCT_INACT;
                    this.acctSecLegacy_actInac.innerHTML = this.ioResources.NONE;
                    dojo.style(this.acctSecLegacy_actInac2Title, "display", "none");
                    dojo.style(this.acctSecLegacy_actInac2, "display", "none");
                    
                    // Hide the "Factory default USERID account" message for Legacy
                    dojo.style(this.acctSecLegacy_factoryDef, "display", "none");
                    dojo.style(this.acctSecLegacy_userid, "display", "none");
                    
                    dojo.style(this.acctSecLegacy_msg, "display", "");
                }
                //High Account Security
                else {
                    dojo.style(this._normAccSecurity, "display", "");
                    dojo.style(this._centAccSecurity, "display", "none");
                    dojo.style(this._extAccSecurity, "display", "none");
                    dojo.style(this.accsecurityModeCentralized, "display", "none");
                    dojo.style(this.acctSecSettings, "display", "");
                    this.acctSecLegacy_pwdEx.innerHTML = "90";
                    
                    // Show the "Force user to change password" message for High level
                    dojo.style(this.acctSecLegacy_forceUser, "display", "");
                    dojo.style(this.acctSecLegacy_force, "display", "");
                    this.acctSecLegacy_forceUser.innerHTML = this.locResources.FORCE_USER;
                    this.acctSecLegacy_force.innerHTML = this.resources.YES;
                    dojo.style(this.acctSecLegacy_force, "font-weight", "normal");

                    this.acctSecLegacy_minPw.innerHTML = "5";
                    this.acctSecLegacy_minPw2.innerHTML = "24";
                    this.acctSecLegacy_lockTitle.innerHTML = this.locResources.LOCK.replace("%1", "60").replace("%2", "20");
                    this.acctSecLegacy_lock.innerHTML = this.resources.YES;
                    this.acctSecLegacy_pwRuleTitle.innerHTML = this.locResources.COMPLEX_PW.replace("%1", "2");
                    this.acctSecLegacy_pwRule.innerHTML = this.resources.YES;
                    dojo.style(this.acctSecLegacy_pwRule, "font-weight", "normal");
                    this.acctSecLegacy_actInacTitle.innerHTML = this.locResources.ALERT.replace("%1", "120");
                    this.acctSecLegacy_actInac.innerHTML = this.resources.YES;
                    dojo.style(this.acctSecLegacy_actInac2Title, "display", "");
                    dojo.style(this.acctSecLegacy_actInac2, "display", "");
                    this.acctSecLegacy_actInac2Title.innerHTML = this.locResources.ACCT_DISABLE.replace("%1", "180");
                    this.acctSecLegacy_actInac2.innerHTML = this.resources.YES;
                    
                    // Show the "Factory USERID account" message for High level
                    dojo.style(this.acctSecLegacy_factoryDef, "display", "");
                    dojo.style(this.acctSecLegacy_userid, "display", "");
                    this.acctSecLegacy_factoryDef.innerHTML = this.locResources.FACT_DEF;
                    this.acctSecLegacy_userid.innerHTML = this.resources.YES;
                    dojo.style(this.acctSecLegacy_userid, "font-weight", "normal");

                    dojo.style(this.acctSecLegacy_msg, "display", "none");
                }
            }
        }

    });
});
