define([ "dojo", 
         "dijit", 
         "support/dialogs/Dialog", 
         "dojo/i18n!./nls/ChooseNewPattern", 
         "dojo/i18n!./nls/AddIOAdapter",
         "dojo/text!./templates/ChooseNewPattern.html",          
         "support/widgets/Select",
         "dijit/Tooltip", 
         "dijit/form/Button",
         "dojox/data/AndOrReadStore" 
         ], function(dojo, dijit, dialog, mri, mri2, template) {

    return dojo.declare("support.dialogs.ChooseNewPattern", [ support.dialogs.Dialog ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,
        // Resources from AddIOAdapter
        locResources2: mri2,

        baseClass: "ChooseNewPattern",

        templateIDNum: null,
        templateType: "adapter",
        adapterId: "*", // Used for Adapter, Port, and PortPhysicalFunction patterns
        physicalFunction: "*", // Used for PortPhysicalFunction patterns
        portCfgMode: "*", // Used for Port patterns
        portCfgOptionCode: "*", // Used for Port patterns
        vendor: "*", // Used for ExtendedPort patterns
        chipset: "*", // Used for ExtendedPort patterns
        protocol: "*", // Used for ExtendedPort patterns

        _createicon: null,
        _editicon: null,

        _defaultAdapterTemplateSelector: null,
        _defaultPortVirtualizationTemplateSelector: null,
        _defaultPortPhysicalFunctionTemplateSelector: null,
        _defaultPortExtendedSettingsTemplateSelector: null,
        _defaultAdapterTemplateStore: null,
        _defaultPortVirtualizationTemplateStore: null,
        _defaultPortPhysicalFunctionTemplateStore: null,
        _defaultPortExtendedSettingsTemplateStore: null,

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/ChooseNewPattern.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);

            dojo.connect(this.dialogNode, "hide", this, function() {        
                this.destroy();
            });

            // Connect port physical function tooltips to appropriate string
            var tooltipLabel = "";
            switch (this.physicalFunction) {
                case "nic":
                    tooltipLabel = this.locResources2.NIC_TIP;
                    break;
                case "fcoe":
                    tooltipLabel = this.locResources2.FCOE_TIP;
                    break;
                case "iscsi":
                    tooltipLabel = this.locResources2.ISCSI_TIP;
                    break;
                case "fc":
                    tooltipLabel = this.locResources2.FC_TIP;
                    break;
                default:
                    this._phyfunctooltip.style.display = "none";
            }
            new dijit.Tooltip({
                connectId: [ this.id + "_phyfunctip" ],
                position: [ "above" ],
                label: tooltipLabel
            });

            if (this.templateType == "adapter") {
                this._createicon = this._createIconAdapter;
                this._editicon = this._editIconAdapter;
            }
            else if (this.templateType == "portvirt") {
                this._createicon = this._createIconPortVirt;
                this._editicon = this._editIconPortVirt;
            }
            else if (this.templateType == "phyfunc") {
                this._createicon = this._createIconPhyFunc;
                this._editicon = this._editIconPhyFunc;
            }
            else if (this.templateType == "portextended") {
                this._editicon = this._editIconPortExtended;
            }

            this._connectDataModels();
            this._createSelectBoxes();

            this.subscribe("CreateAdapterTemplateComplete", function(data) {
                var adapterTemplateData = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "template/list/adapter",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        adapterTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                });
                this._defaultAdapterTemplateData = adapterTemplateData;
                this._defaultAdapterTemplateStore = new dojox.data.AndOrReadStore({
                    data: adapterTemplateData
                });
                this._defaultAdapterTemplateSelector.setStore(this._defaultAdapterTemplateStore, null, {
                    query: {
                        // Only templates for specified adapter.
                        complexQuery: "adapterType: '" + this.adapterId + "'"
                    }
                });
                this._defaultAdapterTemplateSelector.set("value", data.id);
            });

            this.subscribe("CreatePortPhysicalFunctionTemplateComplete", function(data) {
                var ppfunctionTemplateData = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "template/list/ppfunction",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        ppfunctionTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                });
                this._defaultPortPhysicalFunctionTemplateStore = new dojox.data.AndOrReadStore({
                    data: ppfunctionTemplateData
                });
                this._defaultPortPhysicalFunctionTemplateSelector.setStore(this._defaultPortPhysicalFunctionTemplateStore, null, {
                    query: {
                        // Only templates for specified adapter.
                        complexQuery: "physicalFunction: '" + this.physicalFunction + "' AND (adapterType: '" + this.adapterId
                                + "' OR adapterType: '*ANY')"
                    }
                });
                this._defaultPortPhysicalFunctionTemplateSelector.set("value", data.id);
            });

            this.subscribe("CreatePortVirtualizationTemplateComplete", function(data) {
                var portvirtTemplateData = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "template/list/portvirtual",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        portvirtTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                });
                this._defaultPortVirtualizationTemplateStore = new dojox.data.AndOrReadStore({
                    data: portvirtTemplateData
                });
                this._defaultPortVirtualizationTemplateSelector.setStore(this._defaultPortVirtualizationTemplateStore, null, {
                    query: {
                        complexQuery: "portCfgMode: '" + this.portCfgMode + "' AND portCfgOptionCode: '" + this.portCfgOptionCode
                                + "' AND (adapterType: '" + this.adapterId + "' OR adapterType: '*ANY')"
                    }
                });
                this._defaultPortVirtualizationTemplateSelector.set("value", data.id);
            });
            
            this.subscribe("ExPortTemplateComplete", function(data) {
                var portExtendedSettingsTemplateData = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "template/list/ExtendedPort?Props=vendor,chipset,protocol",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        portExtendedSettingsTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                });
                this._defaultPortExtendedSettingsTemplateStore = new dojox.data.AndOrReadStore({
                    data: portExtendedSettingsTemplateData
                });
                this._defaultPortExtendedSettingsTemplateSelector.setStore(this._defaultPortExtendedSettingsTemplateStore, null, {
                    query: {
                        complexQuery: "vendor: '" + this.vendor + "' AND chipset: '" + this.chipset + "' AND protocol: '"
                                + this.protocol + "'"
                    }
                });
                this._defaultPortExtendedSettingsTemplateSelector.set("value", data);
            });            
        },

        _connectDataModels: function() {
            if (this.templateType == "adapter") {
                this._adapterTemplateRow.style.display = "table-row";
                this.dialogNode.set("title", this.resources.ASSIGN_ADAPTER_TITLE);
                var adapterTemplateData = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "template/list/adapter",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        adapterTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                });
                this._defaultAdapterTemplateStore = new dojox.data.AndOrReadStore({
                    data: adapterTemplateData
                });
            }

            else if (this.templateType == "portvirt") {
                this._portTemplateRow.style.display = "table-row";
                this.dialogNode.set("title", this.resources.ASSIGN_PORTVIRT_TITLE);
                var portVirtualizationTemplateData = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "template/list/portvirtual",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        portVirtualizationTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                });
                this._defaultPortVirtualizationTemplateStore = new dojox.data.AndOrReadStore({
                    data: portVirtualizationTemplateData
                });
            }

            else if (this.templateType == "phyfunc") {
                this._physicalFunctionTemplateRow.style.display = "table-row";
                this.dialogNode.set("title", this.resources.ASSIGN_PHYFUNC_TITLE);
                var portPhysicalFunctionTemplateData = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "template/list/ppfunction",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        portPhysicalFunctionTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                });

                this._defaultPortPhysicalFunctionTemplateStore = new dojox.data.AndOrReadStore({
                    data: portPhysicalFunctionTemplateData
                });
            }

            else if (this.templateType == "portextended") {
                this._portExtendedSettingsTemplateRow.style.display = "table-row";
                this.dialogNode.set("title", this.resources.ASSIGN_EXTPORT_TITLE);
                var portExtendedSettingsTemplateData = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "template/list/ExtendedPort?Props=vendor,chipset,protocol",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        portExtendedSettingsTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                });
                this._defaultPortExtendedSettingsTemplateStore = new dojox.data.AndOrReadStore({
                    data: portExtendedSettingsTemplateData
                });
            }

        },

        _createSelectBoxes: function() {
            if (this.templateType == "adapter") {
                this._defaultAdapterTemplateSelector = new support.widgets.Select({
                    id: this.get("id") + "_adapTemplateSelector",
                    name: "defaultAdapterTemplate",
                    value: this.templateIDNum,
                    store: this._defaultAdapterTemplateStore,
                    maxHeight: -1,
                    emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                    query: {
                        complexQuery: "(adapterType: '" + this.adapterId + "' OR adapterType: '*ANY')"
                    },
                    style: {
                        width: "400px"
                    },
                    disabled: false
                }).placeAt(this._adapterTemplate);
                this._defaultAdapterTemplateSelector.startup();
            }

            else if (this.templateType == "portvirt") {
                this._defaultPortVirtualizationTemplateSelector = new support.widgets.Select({
                    id: this.get("id") + "_portVirtualizationTemplateSelector",
                    name: "defaultPortVirtualizationTemplate",
                    value: this.templateIDNum,
                    store: this._defaultPortVirtualizationTemplateStore,
                    maxHeight: -1,
                    emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                    query: {
                        complexQuery: "portCfgMode: '" + this.portCfgMode + "' AND portCfgOptionCode: '" + this.portCfgOptionCode
                                + "' AND (adapterType: '" + this.adapterId + "' OR adapterType: '*ANY')"
                    },
                    style: {
                        width: "400px"
                    },
                    disabled: false
                }).placeAt(this._portTemplate);
                this._defaultPortVirtualizationTemplateSelector.startup();
            }

            else if (this.templateType == "phyfunc") {
                this._defaultPortPhysicalFunctionTemplateSelector = new support.widgets.Select({
                    id: this.get("id") + "_portPhysicalFunctionTemplateSelector",
                    name: "defaultPortPhysicalFunctionTemplate",
                    value: this.templateIDNum,
                    store: this._defaultPortPhysicalFunctionTemplateStore,
                    maxHeight: -1,
                    emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                    query: {
                        complexQuery: "physicalFunction: '" + this.physicalFunction + "' AND (adapterType: '" + this.adapterId
                                + "' OR adapterType: '*ANY')"
                    },
                    style: {
                        width: "400px"
                    },
                    disabled: false
                }).placeAt(this._physicalFunctionTemplate);
                this._defaultPortPhysicalFunctionTemplateSelector.startup();
            }

            else if (this.templateType == "portextended") {
                this._defaultPortExtendedSettingsTemplateSelector = new support.widgets.Select({
                    id: this.get("id") + "_portExtendedSettingsTemplateSelector",
                    name: "defaultPortExtendedSettingsTemplate",
                    value: this.templateIDNum,
                    store: this._defaultPortExtendedSettingsTemplateStore,
                    maxHeight: -1,
                    emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                    query: {
                        complexQuery: "vendor: '" + this.vendor + "' AND chipset: '" + this.chipset + "' AND protocol: '"
                                + this.protocol + "'"
                    },
                    style: {
                        width: "400px"
                    },
                    onChange: dojo.hitch(this, function(newValue) {
                        if (newValue == "") {
                            dojo.replaceClass(this._editicon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                            this.okButton.set("disabled", true);
                        }
                        else if (newValue == "*NONE") {
                            dojo.replaceClass(this._editicon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                        }
                        else {
                            dojo.replaceClass(this._editicon, "spEditIcon_v2", "spEditInactiveIcon_v2");
                        }
                    })
                }).placeAt(this._portExtendedSettingsTemplate);
                this._defaultPortExtendedSettingsTemplateSelector.startup();
                var currentValue = this._defaultPortExtendedSettingsTemplateSelector.get("value");
                if (currentValue == "") {
                    this._defaultPortExtendedSettingsTemplateSelector.set("disabled", true);
                    dojo.replaceClass(this._editicon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                    this.okButton.set("disabled", true);
                }
                else {
                    this._defaultPortExtendedSettingsTemplateSelector.addOption({
                        value: "*NONE",
                        label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
                    });
                    (currentValue != this.templateIDNum) && this._defaultPortExtendedSettingsTemplateSelector.set("value", "*NONE");
                }
            }
        },

        _showNewAdapterDialog: function() {
            require([ "support/dialogs/NewAdapterTemplate" ], dojo.hitch(this, function() {
                var newAdapterDialog = new support.dialogs.NewAdapterTemplate({
                    adapterId: this.adapterId
                });
                newAdapterDialog.show();
            }));
        },

        _showNewPortVirtualizationDialog: function() {
            require([ "support/dialogs/NewPortVirtualizationTemplate" ], dojo.hitch(this, function() {
                var portVirtDialog = new support.dialogs.NewPortVirtualizationTemplate({
                    adapterId: this.adapterId,
                    mode: this.portCfgMode,
                    protocol: this.portCfgOptionCode
                });
                portVirtDialog.show();
            }));
        },

        _showNewPortPhysicalFunctionDialog: function() {
            require([ "support/dialogs/NewPortPhysicalFunctionTemplate" ], dojo.hitch(this, function() {
                var phyFuncDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                    addType: this.adapterId,
                    physicalFunction: this.physicalFunction
                });
                phyFuncDialog.show();
            }));
        },

        _editNewAdapterDialog: function() {

            require([ "support/dialogs/NewAdapterTemplate" ], dojo.hitch(this, function() {
                var adapterTemplateData = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "template/" + this._defaultAdapterTemplateSelector.get("value"),
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        adapterTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                });

                var tempmode = "copy";
                if (adapterTemplateData.items[0].userDefined == false) {
                    tempmode = "copy";
                }
                else {
                    tempmode = "edit";
                }

                var newAdapterDialog = new support.dialogs.NewAdapterTemplate({
                    templateIDNum: this._defaultAdapterTemplateSelector.get("value"),
                    templateMode: tempmode,
                    templateInUse: false
                });
                newAdapterDialog.show();
            }));

        },

        _editNewPortVirtualizationDialog: function() {

            require([ "support/dialogs/NewPortVirtualizationTemplate" ], dojo.hitch(this, function() {
                var portVirtualizationTemplateData = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "template/" + this._defaultPortVirtualizationTemplateSelector.get("value"),
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        portVirtualizationTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                });

                var tempmode = "copy";
                if (portVirtualizationTemplateData.items[0].userDefined == false) {
                    tempmode = "copy";
                }
                else {
                    tempmode = "edit";
                }

                var portVirtualizationDialog = new support.dialogs.NewPortVirtualizationTemplate({
                    templateIDNum: this._defaultPortVirtualizationTemplateSelector.get("value"),
                    templateMode: tempmode,
                    templateInUse: false
                });
                portVirtualizationDialog.show();
            }));

        },

        _editNewPortPhysicalFunctionDialog: function() {

            require([ "support/dialogs/NewPortPhysicalFunctionTemplate" ], dojo.hitch(this, function() {
                var portPhysicalFunctionTemplateData = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "template/" + this._defaultPortPhysicalFunctionTemplateSelector.get("value"),
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        portPhysicalFunctionTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                });

                var tempmode = "copy";
                if (portPhysicalFunctionTemplateData.items[0].userDefined == false) {
                    tempmode = "copy";
                }
                else {
                    tempmode = "edit";
                }

                var portPhysicalFunctionDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                    templateIDNum: this._defaultPortPhysicalFunctionTemplateSelector.get("value"),
                    templateMode: tempmode,
                    templateInUse: false,
                    addType: this.adapterId
                });
                portPhysicalFunctionDialog.show();
            }));

        },

        _editNewPortExtendedSettingsDialog: function() {

            var curVal = this._defaultPortExtendedSettingsTemplateSelector.get("value");
            if ((curVal == "") || (curVal == "*NONE")) {
                return;
            }
            require([ "support/dialogs/NewLearnedSettingsTemplate" ], dojo.hitch(this, function() {
                var portExtendedSettingsDialog = new support.dialogs.NewLearnedSettingsTemplate({
                    templateIDNum: curVal,
                    type: "port",
                    templateMode: "edit"
                });
                portExtendedSettingsDialog.show();
            }));

        },

        _saveandhideDialog: function() {

            switch (this.templateType) {
                case "adapter":
                    
                    if(this.CP_RUN_ENVIRONMENT == "fsm"){

                        require([ "support/widgets/ConfirmationDialog" ], dojo.hitch(this, function() {
                            var confirmChange = function() {
                                dojo.publish("ChooseAdapterComplete", {
                                    id: this._defaultAdapterTemplateSelector.get("value")
                                });
                                this.hide();
                            };
                            var prompt = new support.widgets.ConfirmationDialog({
                                buttonLabel: this.resources.ASSIGN,
                                type: "Warning",
                                text: this.locResources.CONFIRM_ASSIGN_TEMPLATE,
                                info: this.locResources.CONFIRM_ASSIGN_ADAPTER_TEMPLATE_MSG
                            });
                            dojo.body().appendChild(prompt.domNode);
                            prompt.startup();
                            prompt.confirm(confirmChange, this);
                        }));

                    
                    }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                        require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {
                            var confirmChange = function() {
                                dojo.publish("ChooseAdapterComplete", {
                                    id: this._defaultAdapterTemplateSelector.get("value")
                                });
                                this.hide();
                            };
                            var prompt = new idx.widget.ConfirmationDialog({
                                buttonLabel: this.resources.ASSIGN,
                                type: "Warning",
                                text: this.locResources.CONFIRM_ASSIGN_TEMPLATE,
                                info: this.locResources.CONFIRM_ASSIGN_ADAPTER_TEMPLATE_MSG
                            });
                            dojo.body().appendChild(prompt.domNode);
                            prompt.startup();
                            prompt.confirm(confirmChange, this);
                        }));

                    }
                    
                    break;
                case "portvirt":
                    dojo.publish("ChoosePortVirtualizationComplete", {
                        id: this._defaultPortVirtualizationTemplateSelector.get("value")
                    });
                    this.hide();
                    break;
                case "phyfunc":
                    dojo.publish("ChoosePortPhysicalFunctionTemplateComplete", {
                        id: this._defaultPortPhysicalFunctionTemplateSelector.get("value")
                    });
                    this.hide();
                    break;
                case "portextended":
                    dojo.publish("ChoosePortExtendedSettingsComplete", {
                        id: this._defaultPortExtendedSettingsTemplateSelector.get("value")
                    });
                    this.hide();
                    break;
            }
        },

        _createIconPressed: function() {
            dojo.replaceClass(this._createicon, "spCreateClickedIcon_v2", "spCreateHoverIcon_v2");
        },

        _createIconReleased: function() {
            dojo.replaceClass(this._createicon, "spCreateHoverIcon_v2", "spCreateClickedIcon_v2");
        },

        _createIconHover: function() {
            dojo.replaceClass(this._createicon, "spCreateHoverIcon_v2", "spCreateIcon_v2");
        },

        _createIconMouseOut: function() {
            dojo.replaceClass(this._createicon, "spCreateIcon_v2", "spCreateHoverIcon_v2");
        },

        _editIconPressed: function() {
            if(dojo.hasClass(this._editicon, "spEditHoverIcon_v2")){
                dojo.replaceClass(this._editicon, "spEditClickedIcon_v2", "spEditHoverIcon_v2");
            }
        },

        _editIconReleased: function() {
            if(dojo.hasClass(this._editicon, "spEditClickedIcon_v2")){
                dojo.replaceClass(this._editicon, "spEditHoverIcon_v2", "spEditClickedIcon_v2");
            }
        },

        _editIconHover: function() {
            if(dojo.hasClass(this._editicon, "spEditIcon_v2")){
                dojo.replaceClass(this._editicon, "spEditHoverIcon_v2", "spEditIcon_v2");
            }
        },

        _editIconMouseOut: function() {
            if(dojo.hasClass(this._editicon, "spEditHoverIcon_v2")){
                dojo.replaceClass(this._editicon, "spEditIcon_v2", "spEditHoverIcon_v2");
            }
        },

        _cancelDialog: function() {
            this.hide();
        }
    });
});
