define([
        "dojo",
        "dijit",
        "support/dialogs/Dialog",
        "dojo/i18n!./nls/ConfigPatterns",
        "dojo/text!./templates/ConfigPatterns.html",
        "support/widgets/Select",
        "dijit/Dialog",
        "dojo/parser",
        "dojo/store/Memory",
        "dijit/form/CheckBox",
        "dijit/layout/BorderContainer",
        "dijit/layout/ContentPane",
        "dojo/data/ItemFileWriteStore",
        "dojox/grid/DataGrid"
    ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.ConfigPatterns", [ support.dialogs.Dialog ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        baseClass: "ConfigPatterns",

        configID: null,
        configURI: null,
        computeNodeType: "sysx",
        computeNodeFormFactor: "2s",
        ioAdapterAddressing: "burned",
        virtualAddressPools: "ibm",
        sync: true,

        adapterGrid: null,
        adapterLayout: null,
        adapterStore: null,
        sysxAdapterStore: null,
        powerAdapterStore: null,

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/ConfigPatterns.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);

            this.setUpAdapterGrid();
            // this.xhrGetConfigPatterns();

            dojo.connect(this.dialogNode, "hide", this, function() {
                this.destroy();
            });
            
            var toolTip = new Array();
            toolTip.push("<div style=\"width: 50em;\">" + this.resources.FF_HELP_2S);
            toolTip.push("<br><br>" + this.resources.FF_HELP_PXE);
            toolTip.push("<br><br>" + this.resources.FF_HELP_1SD_1 + "<ul style=\"list-style-type: none; margin-top: 0px;\"><li>" + this.resources.FF_HELP_1SD_2 + "</li></ul>");
            toolTip.push(this.resources.FF_HELP_4S);
            toolTip.push("<br><br>" + this.resources.FF_HELP_SCAL + "<ul style=\"list-style-type: none; margin-top: 0px;\">" +
                "<li>" +this.resources.FF_HELP_2SC + "</li>" + 
                "<li>" + this.resources.FF_HELP_4SC + "</li>" +
                "<li>" + this.resources.FF_HELP_8SC + "</li>");
            toolTip.push("</ul></div>");
            // NOTE: For dijit/Tooltip dijits, set the label, not innerHTML.
            this._helpLink_FF.label = toolTip.join("");

            this._helpLink_IOA.label = "<div style=\"width: 30em;\">" + this.locResources.IOADDRESSING_TOOLTIP.replace("%1", this.PRODUCT_NAME_LONG) + "</div>";

        },

        setUpSelectors: function() {
            // Computer node type
            new support.widgets.Select({
                options: [{
                    label: this.resources.SYSX,
                    value: "sysx"
                },
                {
                    label: this.resources.POWER,
                    value: "power"
                }],
                value: this.computeNodeType,
                style: "width: 250px",
                onChange: dojo.hitch(this, function(newValue) {
                    this.computeNodeType = newValue;
                    this.adapterGrid.filter({
                        supportedNodeTypes: newValue
                    }, true);
                    this.adapterGrid.set("autoHeight", 5);
                    this.dialogNode._position();
                })
            }).placeAt(this._nodeTypeSelector);

            // Compute node form factor
            new support.widgets.Select({
                options: [{
                    label: this.resources.FF_BAY.replace("%1", "1"),
                    value: "2s"
                },
                {
                    label: this.resources.FF_BAY_DUAL.replace("%1", "1"),
                    value: "1sd"
                },
                {
                    label: this.resources.FF_BAY.replace("%1", "2"),
                    value: "4s"
                },
                {
                    label: this.resources.FF_PXU.replace("%1", "2"),
                    value: "pxu"
                },
                {
                    label: this.resources.FF_SCALABLE.replace("%1", "2"),
                    value: "2sc"
                },
                {
                    label: this.resources.FF_SCALABLE.replace("%1", "4"),
                    value: "4sc"
                },
                {
                    label: this.resources.FF_SCALABLE.replace("%1", "8"),
                    value: "8sc"
                }],
                value: this.computeNodeFormFactor,
                style: "width: 250px",
                onChange: dojo.hitch(this, function(newValue) {
                    this.computeNodeFormFactor = newValue;
                })
            }).placeAt(this._nodeFormSelector);

            // I/O adapter addressing
            new support.widgets.Select({
                options: [{
                    label: this.locResources.BURNED,
                    value: "burned"
                },
                {
                    label: this.locResources.VIRTUAL,
                    value: "virtual"
                }],
                value: this.ioAdapterAddressing,
                style: "width: 250px",
                onChange: dojo.hitch(this, function(newValue) {
                    this.ioAdapterAddressing = newValue;
                    this._hideVirtualAddressPoolsOption();
                })
            }).placeAt(this._ioAddressingSelector);

            // Virtual address pools
            new support.widgets.Select({
                options: [{
                    label: this.locResources.IBM,
                    value: "ibm"
                },
                {
                    label: this.locResources.VENDOR,
                    value: "vendor"
                }],
                value: this.virtualAddressPools,
                style: "width: 250px",
                onChange: dojo.hitch(this, function(newValue) {
                    this.virtualAddressPools = newValue;
                })
            }).placeAt(this._virtualAddressSelector);

            this._hideVirtualAddressPoolsOption();

        },

        setUpAdapterGrid: function() {

            this.adapterStore = new dojo.data.ItemFileWriteStore({
                url: this.REST_PATH + "supportedDevices/ioAdapter/list"
            });
            this.adapterStore.fetch({
                onComplete: dojo.hitch(this, function(items, request) {

                    // Fix up data store items with displayable values.
                    dojo.forEach(items, dojo.hitch(this, function(item, idx) {
                        item.check = [ false ];
                        // Add displayable type field based on formatted values.
                        switch (item.adapterType.toString()) {
                            case "vfa":
                                switch (item.type.toString()) {
                                    case "lom":
                                        item.typeForDisplay = [ this.resources.FABRIC_CONNECTOR ];
                                        break;
                                    case "adapter":
                                        item.typeForDisplay = [ this.resources.VIRTUAL_FABRIC ];
                                        break;
                                }
                                break;
                            case "ethernet":
                                item.typeForDisplay = [ this.resources.ETHERNET ];
                                break;
                            case "fc":
                                item.typeForDisplay = [ this.resources.FIBRE_CHANNEL ];
                                break;
                            case "fcoe":
                                item.typeForDisplay = [ this.resources.FCOE ];
                                break;
                        }
                    }));
                })
            });

            this.adapterLayout = [ {
                name: this.resources.DEFAULT,
                field: "check",
                cellType: "dojox.grid.cells.Bool",
                width: "50px",
                editable: true
            }, {
                name: this.locResources.ADAPTER_DESCRIPTION,
                field: "description",
                width: "400px",
                editable: false
            }, {
                name: this.locResources.PHYSICAL_PORTS,
                field: "portCount",
                width: "100px",
                editable: false,
                formatter: this._formatPortCount
            }, {
                name: this.resources.TYPE,
                field: "typeForDisplay",
                width: "100px",
                editable: false
            } ];
            this.adapterGrid = new dojox.grid.DataGrid({
                store: this.adapterStore,
                query: {
                    supportedNodeTypes: this.computeNodeType
                },
                structure: this.adapterLayout,
                autoWidth: true,
                autoHeight: true,                    
                columnReordering: false,
                noDataMessage: this.locResources.NO_ADAPTERS_AVAIL,
                selectionMode: "none"
            }).placeAt(this._adapterGrid);
            this.adapterGrid.startup();
            this.adapterGrid.render();

            this.dialogNode._position();
            this.xhrGetConfigPatterns();

        },

        _formatPortCount: function(value, rowIndex, cell) {
            if (value == "variable") {
                return "";
            }
            return value;
        },

        _formatType: function(fields) {
            var rv = fields[0];
            switch (fields[0]) {
                case "vfa":
                    switch (fields[1]) {
                        case "adapter":
                            rv = this.resources.VIRTUAL_FABRIC;
                            break;
                        case "lom":
                            rv = this.resources.FABRIC_CONNECTOR;
                            break;
                    }
                    break;
                case "ethernet":
                    rv = this.resources.ETHERNET;
                    break;
                case "fc":
                    rv = this.resources.FIBRE_CHANNEL;
                    break;
                case "fcoe":
                    rv = this.resources.FCOE;
                    break;
            }
            return rv;
        },

        _hideVirtualAddressPoolsOption: function() {
            if (this.ioAdapterAddressing == "virtual") {
                // 99291: Disabled on 9/29/2012. Control has been hidden in HTML template as well.
                // Will be considered for future support.
                // this._virtualAddressRow.setAttribute("style", "");
            }
            else {
                this._virtualAddressRow.setAttribute("style", "display:none");
            }
            this.dialogNode._position();
        },

        getSelectedAdaptersJsonObj: function() {
            var preferredAdapters = new Array();
            var store = this.adapterGrid.get("store");
            var items = store._arrayOfTopLevelItems;
            for ( var i = 0; i < items.length; i++) {
                var item = items[i];
                var checked = store.getValue(item, "check");
                if (checked) {
                    var id = store.getValue(item, "id");
                    var obj = {
                        id: id
                    };
                    preferredAdapters.push(obj);
                }
            }
            return preferredAdapters;
        },

        getConfigPatternsJsonObj: function() {
            var preferredAdapters = this.getSelectedAdaptersJsonObj();
            return dojo.toJson({
                "id": this.configID,
                "nodeType": this.computeNodeType,
                "formFactor": this.computeNodeFormFactor,
                "ioAddressingMode": this.ioAdapterAddressing,
                "pools": (this.ioAdapterAddressing == "virtual") ? this.virtualAddressPools : "",
                "preferredAdapters": preferredAdapters,
                "syncAddressPools": this.sync,
                "uri": this.configURI
            });

        },

        syncChanged: function() {
            this.set("sync", this._syncCheck.get("checked"));
        },

        _setSyncAttr: function(s) {
            if (s == undefined)
                s = true;
            this.sync = s;
            this._syncCheck.set("checked", this.sync);
        },

        okClicked: function() {
            console.log(this.declaredClass + ": ", this.getConfigPatternsJsonObj());
            var success = this.xhrPutConfigPatterns();
            if (success) {
                this.hide();
                dojo.destroy(this.id + "_underlay");
            }
        },

        cancelClicked: function() {
            var close = confirm(this.resources.CONFIRM_CANCEL);
            if (close) {
                this.hide();
                dojo.destroy(this.id + "_underlay");
            }
        },

        xhrGetConfigPatterns: function() {
            dojo.xhrGet({
                url: this.REST_PATH + "config/default",
                handleAs: "json",
                sync: true,
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                load: dojo.hitch(this, function(response) {
                    // compute node type                    
                    // Will be considered for future support of "power" systems.
                    // this.computeNodeType = response.nodeType;
                    this.computeNodeType = "sysx";

                    // compute form factor
                    this.computeNodeFormFactor = response.formFactor;

                    // io adapter addressing
                    this.ioAdapterAddressing = response.ioAddressingMode;

                    // virtual address pools
                    if (this.ioAdapterAddressing == "virtual") {
                        this.virtualAddressPools = response.pools;
                    }

                    // preferred adapters
                    var preferredAdapters = response.preferredAdapters;
                    for ( var i = 0; i < preferredAdapters.length; i++) {
                        var adapter = preferredAdapters[i];
                        var id = adapter.id;
                        var setSelected = dojo.hitch(this, function(items, request) {
                            for ( var j = 0; j < items.length; j++) {
                                var item = items[j];
                                if (this.adapterStore != null && item != null)
                                    this.adapterStore.setValue(item, "check", true);
                            }
                        });
                        this.adapterStore.fetch({
                            query: {
                                id: id
                            },
                            onComplete: setSelected
                        });
                    }

                    this.set("sync", response.syncAddressPools);
                    this.configID = response.id;
                    this.configURI = response.uri;

                    this.setUpSelectors();
                })
            });
        },

        xhrPutConfigPatterns: function() {
            var jsonString = this.getConfigPatternsJsonObj();
            var success = false;
            dojo.xhrPut({
                url: this.REST_PATH + "config/default/" + this.configID,
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                putData: jsonString,
                sync: true,
                load: dojo.hitch(this, function(data) {
                    dojo.publish("ConfigPatternsComplete", data);
                    success = true;
                }),
                error: dojo.hitch(this, function(error) {
                    alert(this.resources.UNEXPECTED_ERROR.replace("%1", error));
                    success = false;
                })
            });
            return success;
        }

    });
});