define([
        "dojo",
        "dijit",
        "support/dialogs/Dialog",
        "dojo/i18n!./nls/DeployChassisTemplate",
        "dojo/text!./templates/DeployChassisTemplate.html",
        "support/widgets/Select",
        "dojo/data/ItemFileWriteStore",
        "support/base/TableMixin",
        "dijit/form/Button",
        "dijit/Toolbar",
        "support/panels/DualTableSelector",
        "support/dialogs/ProgressDialog"
    ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.DeployChassisTemplate", [ support.dialogs.Dialog, support.base.TableMixin ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "DeployTemplate",

        // templateId: String
        // Value of entry to pre-select in this widget's template selector.
        templateId: null,
        init: false,

        _selectedChassis: new Array(),    //Holds selected chassis

        // UUID of chassis to deploy template to
        chassisUUID: null,
               
        //FW Level State Names - Internal Usage
        FW_UNKNOWN: "unknown",
        FW_BACKLEVEL: "backlevel",
        FW_OK: "compatible",
        DEPLOY_OK: "ok",
        DEPLOY_NOTREADY: "notReady",
        DEPLOY_UNKNOWN: "unknown",
        
        deployCellNum:0,
        
        // ID prefixes used for tables
        availGridID: "AvailableChassisDeployTable",
        selectGridID: "SelectedChassisDeployTable",

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/DeployTemplate.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in
        // the template HTML file
        postMixInProperties: function() {

            this.inherited(arguments);

        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be
        // directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);
            this._setupTemplateSelector();
            this._setupTables();

            // Need to wait until after _setupTables() is called to startup the template selector
            this._tmplSelector.startup();

            // subscribe to the topic "addPHChassis" so that the Available
            // Table can reload its contents and immediately show the new
            // placeholder chassis
            this.subscribe("addPHChassis", function() {
                console.log(this.declaredClass + ": Notified of new placeholder chassis; reloading available table");
                this._getSelected();
                this._refreshTree();
            });

            // subscribe to the topic "removePHChassis" so that the Available
            // Table can reload its contents and immediately show the
            // removal of the placeholder chassis
            this.subscribe("removePHChassis", function() {
                console.log(this.declaredClass + ": Notified of new placeholder chassis; reloading available table");
                this._getSelected();
                this._refreshTree();
            });

            dojo.connect(this._availTable.select.row, "onSelectionChange", this, this._availNodeClicked); // Fires on row selection
            
            // if showing this dialog for a specific chassis, pre-select it
            this._preselectTargetChassis();

        },
        
        _pushJobSuccessInfo: function(/*Object*/item, /*Array*/msgContent) {
            // Push job success information into the msgContent array.
            // Expect to see jobname w/ endPointIds/locationIds data.
            msgContent.push(this.resources.SUBMIT_JOB_SUCCESS.replace("%1", item.jobName));
            var names = new Array();
            for (var loopVar in this._selectTable.store._arrayOfAllItems) {
                var selected = this._selectTable.store._arrayOfAllItems[loopVar];
                if(!selected){
                    continue;
                }
                // get the name of the chassis that was deployed to
                if(item.endPointIds != null && selected.uuid != null &&
                    selected.uuid[0] != null &&
                    dojo.indexOf(item.endPointIds, selected.uuid[0]) >= 0) {
                    names.push(selected.name[0]);
                }
            }
            msgContent.push(this.resources.SUBMIT_JOB_SUCCESS_CHASSIS.replace("%1", names.join(", ")));
            msgContent.push("<p>" + this.resources.SUBMIT_JOB_DETAILS + "</p>");
        },
        
        _pushJobErrorInfo: function(/*Object*/item, /*Array*/msgContent) {
            // Push job failure information into the msgContent array.
            // No jobname, but endPointIds/locationIds are valid.
            msgContent.push(this.resources.SUBMIT_JOB_ERROR);
            var names = new Array();
            for (var loopVar in this._selectTable.store._arrayOfAllItems) {
                var selected = this._selectTable.store._arrayOfAllItems[loopVar];
                if(!selected){
                    continue;
                }
                // get the name of the chassis that was deployed to
                if(item.endPointIds != null && selected.uuid != null &&
                    selected.uuid[0] != null &&
                    dojo.indexOf(item.endPointIds, selected.uuid[0]) >= 0) {
                    names.push(selected.name[0]);
                }
            }
            msgContent.push(this.resources.SUBMIT_JOB_ERROR_CHASSIS.replace("%1", names.join(", ")));
        },

        onFinish: function() {

            var data = {
                templateId: this._tmplSelector.value,
                endPointIds: [],
                locationIds: []
            };
            for ( var loopVar in this._selectTable.store._arrayOfAllItems) {
                var item = this._selectTable.store._arrayOfAllItems[loopVar];
                if(!item){
                    continue;
                }
                item && data.endPointIds.push(item.uuid[0]);
            }
            var jsonString = dojo.toJson(data);
            this.getProgressDialog("deploy").show();
            dojo.xhrPost({
                url: this.REST_PATH + "profile/deploy",
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                postData: jsonString,
                handle: dojo.hitch(this, function(response, ioArgs) {
                    console.log(this.declaredClass + ": Deploy pattern complete: ", response, ioArgs);
                    if (ioArgs.xhr.status >= 300) {
                        dojo.publish("ProgressFailed", []);
                        this.getModalDialog("Error", this.locResources.DEPLOY_ERROR_MSG).show();
                        console.error("Failed to deploy chassis pattern.");
                    }
                    else {
                        // Examine response items to determine potential for multiple success/failure
                        var info = new Array();
                        var oneSuccess = false;
                        var oneFailure = false;
                        if (response.items) {
                            for (var loopVar = 0; loopVar < response.items.length; ++loopVar) {
                                var item = response.items[loopVar];
                                var title = this.resources.SUBMIT_JOB_TITLE.replace("%1", loopVar + 1);
                                var msgContent = new Array();
                                if (!item.message || (item.message.length == 0)) {
                                    this._pushJobSuccessInfo(item, msgContent);
                                    oneSuccess = true;
                                }
                                else {
                                    this._pushJobErrorInfo(item, msgContent);
                                    oneFailure = true;
                                }
                                info.push({
                                    title: title,
                                    content: msgContent.join(" ")
                                });
                            }
                        }
                        var msgType = null;
                        var msgText = null;
                        if (oneSuccess) {
                            if (oneFailure) {
                                msgType = "Warning";
                                msgText = this.resources.DEPLOY_WARNING_MSG;
                            }
                            else {
                                msgType = "Success";
                                msgText = this.resources.DEPLOY_SUCCESS_MSG;
                            }
                        }
                        else {
                            msgType = "Error";
                            msgText = this.resources.DEPLOY_ERROR_MSG;
                        }
                        dojo.publish("ProgressComplete", response.id);
                        this.getModalDialog(msgType, msgText, ((info.length == 1) ? info[0].content : info)).show();
                        dojo.publish("DeployChassisTemplateComplete", response);
                        this._onClose();
                    }
                })
            });
            return false; // Don't allow close! Allow handlers (above) to determine when to close.

        },

        _setupTemplateSelector: function() {

            // summary:
            // Set up template selector...
            // tags:
            // protected
            var tmplStore = null;
            dojo.xhrGet({
                url: this.REST_PATH + "template/list/CMM",
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: function(data) {
                    // Dont need to store all that server template info in the selector...
                    if (data.items) {
                        var i;
                        var items = [];
                        var newEntry = function(template) {
                            var temp = new Object();
                            temp.id = template.id;
                            temp.name = template.name;
                            temp.inUse = template.inUse;
                            temp.type = template.type;
                            temp.description = template.description;
                            temp.nodeType = template.nodeType;
                            temp.uri = template.uri;
                            return temp;
                        };
                        for (i = 0; i < data.items.length; i++) {
                            var temp = newEntry(data.items[i]);
                            items.push(temp);
                        }
                        var templateData = dojo.toJson({
                            "identifier": "id",
                            "label": "name",
                            "items": items
                        }, true);
                        tmplStore = new dojo.data.ItemFileWriteStore({
                            data: dojo.fromJson(templateData)
                        });
                    }
                }
            });

            // Add prompt option before load so its listed first.
            tmplStore.newItem({
                id: "-1",
                name: "&mdash; " + this.locResources.SELECT_TEMPLATE + " &mdash;"
            });

            this._tmplSelector = new support.widgets.Select({
                id: this.get("id") + "_tmplSelector",
                name: "template",
                required: true,
                store: tmplStore,
                value: (this.templateId) ? this.templateId : "-1", // Pre-select entry from store,
                // if it exists.
                emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                onChange: dojo.hitch(this, this._changeTemplateCallback)
            }).placeAt(this._tmplSelector);
            dojo.connect(this._tmplSelector, "onSetStore", this, "_presetTemplateCallback");
            dojo.connect(this._tmplSelector, "startup", this, function() {
                if (this.templateId) {
                    this._changeTemplateCallback(this.templateId);
                } else {
                    this._deployButton.set("disabled", true);
                    this.set("title", this.locResources.PAGE_TITLE);                    
                }
            });
        },

        _presetTemplateCallback: function() {

            // tags:
            // protected
            if (this.templateId) {
                if (this._tmplSelector.get("value") === this.templateId.toString()) {
                    // Pre-select entry was found, disable the selector.
                    this._tmplSelector.set("disabled", true);
                }
            }
        },

        _changeTemplateCallback: function(newVal) {

            // tags:
            // protected
            var title = this.locResources.PAGE_TITLE;
            var option = this._tmplSelector.getOptions(newVal);

            if (option && newVal != "-1") {
                this._validate();
                title = title + " - " + option.label;
            }
            else {
                // Disable Deploy Button w/ Invalid Template Selection
                this._deployButton.set("disabled", true);
            }
            this.set("title", title);

        },
        
        _setupTables: function() {
            
            this._setupAvailableTable();
            this._setupSelectedTable();
            
            this._setupToolbar(this._availTable);

            this._dualTableSelector = new support.panels.DualTableSelector({
                _availTable: this._availTable,
                _selectTable: this._selectTable,
                singleLevel: true
            }).placeAt(this._dualTableSelectorNode);

            dojo.connect(this._dualTableSelector, "_moveRight", this, this._validate);
            dojo.connect(this._dualTableSelector, "_moveAllRight", this, this._validate);
            dojo.connect(this._dualTableSelector, "_moveLeft", this, this._validate);
            dojo.connect(this._dualTableSelector, "_moveAllLeft", this, this._validate);
        },
        
        _setupToolbar: function(grid) {
            var toolbar = grid && grid.toolBar && grid.toolBar.widget;
            
            if (toolbar) {
                var phButton = new dijit.form.Button({
                    showLabel: true,
                    label: this.resources.ADD_PLACEHOLDER_CHASSIS,
                    title: this.resources.ADD_PLACEHOLDER_CHASSIS_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._addPlaceholderChassis();
                    })
                });
                toolbar.addChild(phButton);
                this._delButton = new dijit.form.Button({
                    showLabel: false,
                    iconClass: "spDeleteInactiveIcon",
                    title: this.resources.REMOVE_PLACEHOLDER_CHASSIS_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._removePlaceholderChassis();
                    })
                });
                toolbar.addChild(this._delButton);
            }
        },

        _setupAvailableTable: function() {

            // summary:
            // Set up available targets table widget...
            // tags:
            // protected
            if (this._availTable) {
                // Destroy old table and start from scratch.
                this._availTable.destroy();
            }
            
            // Setup table widget...
            var tableStore = this._retrieveStore();
            var tableStructure = this._getTableStructure();
            this._availTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.availGridID, true); 
            this._availTable.column(0).sort(false);
 
        },

        _setupSelectedTable: function() {

            // summary:
            // Set up selected targets table widget...
            // tags:
            // protected
            if (this._selectTable) {
                // Destroy old table and start from scratch.
                this._selectTable.destroy();
            }
            
            // Start with empty select table
            var emptyData = {
                    "items": [],
                    "label": "name",
                    "identifier": "id"
            };
            var selectedStore = new dojo.data.ItemFileWriteStore({
                data: emptyData
            });
            
            // Setup table widget...
            var tableStore = selectedStore;
            var tableStructure = this._getTableStructure();
            this._selectTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.selectGridID, true);
            this._selectTable.column(0).sort(false);

        },

        _getNumSelectedServers: function() {
            if (this._selectTable) {
                return this._selectTable.store._arrayOfTopLevelItems.length;
            }
            return -1;
        },

        // validate inputs in selected table and selector are filled
        _validate: function() {
            if (this._getNumSelectedServers() > 0 && this._tmplSelector.get("value") != "-1") {
                this._deployButton.set("disabled", false);
            }
            else {
                this._deployButton.set("disabled", true);
            }
        },

        _addPlaceholderChassis: function() {
            require([ "support/dialogs/AddPlaceholderChassis" ], dojo.hitch(this, function() {
                // show the Add Placeholder Chassis dialog
                var apcdialog = new support.dialogs.AddPlaceholderChassis();
                apcdialog.show();
                // no need to dismiss or delete this dialog; the Add Placeholder
                // Chassis dialog does this when the user clicks OK or Cancel
            }));
        },

        _removePlaceholderChassis: function() {
            // only process the remove if we have selected one Placeholder
            // Chassis, which is determined in the _availNodeClicked
            // function
            if (this._delButton.iconClass == "spDeleteIcon") {
                // if we get this far, we've already checked in the
                // _availNodeClicked() function that only one node was
                // clicked
                var msg = "";
                var phChassisObj = this._availTable.select.row.getSelected()[0];
                var item = this._availTable.row(phChassisObj, true).item();
                var chassisID = item.id;
                console.log(this.declaredClass + ": Removing new Placeholder Chassis with ID: "
                        + chassisID);
                var success = true;
                var xhrDelete = dojo.xhrDelete({
                    url: this.REST_PATH + "phchassis/" + chassisID,
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    error: dojo.hitch(this, function(error, ioArgs) {
                        console.error(this.declaredClass + ": Problem deleting Placeholder Chassis with ID: " + chassisID
                                + " error: " + error);
                        success = false;
                        if (ioArgs.xhr.status == 400) {
                            msg = this.locResources.DELETE_PH_ERROR; 
                        }
                        this.getModalDialog("Error", msg).show();
                        success = false;
                    })
                });
                
                // if an error occurs, return
                if (success == false) {
                    return success;
                }
                                
                // publish the topic "removePHChassis" so that the Available
                // Table can reload its contents and immediately show that
                // the placeholder chassis was removed
                dojo.publish("removePHChassis", []);
                
                this._delButton.set("iconClass", "spDeleteInactiveIcon");

                return success;
            }
        },

        _availNodeClicked: function() {
            var selectedRows = this._availTable.select.row.getSelected();
            if (selectedRows.length == 0) {
                return;
            }
            // "activate" the Remove Placeholer Chassis button (by changing
            // its icon from a gray "X" to a red "X") but only if the user
            // has selected exactly one Placeholder Chassis
            var item = this._availTable.row(selectedRows[0], true).item();
            if (selectedRows.length == 1 && item.access == "placeholder") {
                this._delButton.set("iconClass", "spDeleteIcon");
            }
            else {
                this._delButton.set("iconClass", "spDeleteInactiveIcon");
            }
        },
        
        _deployDecorator: function(item) {
            var accessText = item;
            var iconClass = "";
            
            switch (accessText) {
                case this.resources.FW_COMPAT:
                    iconClass = "spSuccessIcon";
                    break;
                case this.resources.FW_BACK_LEVEL:
                    iconClass = "spWarningIcon";
                    break;
                case this.resources.FW_UNKNOWN:
                    iconClass = "spUnknownIcon";
                    break;
            }            
            
            return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr><td>"
                    + "<img src=\"" + this._blankGif + "\" alt=\"\" class=\"" + iconClass + "\" title=\"" + accessText + "\"></img></td>"
                    + "<td><span style=\"margin-left:4px\" title=\"" + accessText + "\">" + accessText + "</span></td>"
                    + "</tr></tbody></table>";
        },

        //Function to format deploy status icons based on deploy status
        _deployFormatter: function(item) {

            // summary:
            // Formats "access" field value for display in TreeTable.
            // tags:
            // protected
            if (!item.firmwareLevel) {
                return "";
            }
            
            var fwLevel = item.firmwareLevel;
            var accessText = "";
            
            if (fwLevel == this.FW_OK) { // OK 
                accessText = this.resources.FW_COMPAT;
            }
            else if (fwLevel == this.FW_BACKLEVEL) { // NOT READY
                accessText = this.resources.FW_BACK_LEVEL;
            }
            else /*(access == "unknown")*/ { // Unknown
                accessText = this.resources.FW_UNKNOWN;
            }

            return accessText;
        },
        
        _refreshTree: function() {
            // tags:
            // protected
            // Refresh tree data
            var tableStore = this._retrieveStore();
            this._availTable.select.row.clear(); // clear selected rows
            this._availTable.setStore(tableStore);
            this._availTable.model.clearCache();
            this._availTable.body.refresh();
        },
                      
        _retrieveStore: function() {
            
            var availStore = null;
            var availData = null;
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "chassis/unassigned",
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: dojo.hitch(this, function(response) {
                    availData = dojo.toJson({
                        "identifier": "id",
                        "label": "name",
                        "items": response.items
                    }, true);
                }),
                error: dojo.hitch(this, function(response) {
                    // if the REST API was unsuccessful in returning data,
                    // show an empty table
                    console.log(this.declaredClass + ": Unable to retrieve chassis from API: chassis/unassigned");                        
                    
                    availData = dojo.toJson({
                        "identifier": "id",
                        "label": "name",
                        "items": []
                    }, true);                                                
                })
            });
            
            availStore = new dojo.data.ItemFileWriteStore({
                data: dojo.fromJson(availData)
            });                
            
            return availStore;
            
        },        
        
        _getTableStructure: function() {
            var layout = new Array();
            
            layout.push({
                field: "name",
                name: this.resources.NAME,
                width: "16em"
            });
            layout.push({
                field: "deployStatus",
                name: this.resources.FW_DEPLOY_STATUS,
                width: "auto",
                decorator: dojo.hitch(this, "_deployDecorator"),
                formatter: dojo.hitch(this, "_deployFormatter")
            });
            
            return layout;
        },
        
        //Get servers on selected side
        _getSelected: function(){  
            this._selectedChassis = new Array();
            var selectedObjs = this._selectTable.store._arrayOfAllItems;
            
            for ( var i = 0; i < selectedObjs.length; i++) {
                var selectedObj = selectedObjs[i];
                if (selectedObj != null) {
                    this._selectedChassis.push(selectedObj.id);
                }
            }  
        },

        // this function performs the selection manually on the items; it is
        // coded differently than FailoverMonitorTemplate.js (which instead
        // modifies the data store) because it is just easier
        _preselectTargetChassis: function() {
            if (this.chassisUUID != null) {
                console.log(this.declaredClass + ": Preselecting chassis with UUID: " + this.chassisUUID);
                var rowIDOfChassis = null;
                // all chasses will be in the _arrayOfTopLevelItems
                var availObjs = this._availTable.store._arrayOfTopLevelItems;
                for ( var i = 0; i < availObjs.length; i++) {
                    var availObj = availObjs[i];
                    if (availObj != null) {
                        var availChassisUUID = availObj.uuid;
                        if (availChassisUUID == this.chassisUUID) {
                            rowIDOfChassis = i;
                        }
                    }
                }
                console.log(this.declaredClass + ": Chassis row: " + rowIDOfChassis);
                if (rowIDOfChassis != null) {
                    // moving a chassis over simply involves performing the
                    // selection and calling the _moveRight() function;
                    // we do not need to expand or collapse the list
                    var id = this._availTable.store._arrayOfAllItems[rowIDOfChassis].id;
                    this._availTable.select.row.selectById(id);
                    this._dualTableSelector._moveRight();
                }
            }
        },
        
        /**
         * Function to analyze deploy status of inputted array of items, and returns a status object containing whether or not
         * there should be warnings or errors. 
         * 
         * items: Array of selected items containing deployStatus fields
         * @returns Object with two fields: {error:Boolean, warning:Boolean}
         */
        _checkDeployStatus: function(items){
            var i;
            var returnObj={error:false,warning:false};
            for(i=0;i<items.length;i++){
                if(items[i].deployStatus){
                    if(items[i].deployStatus == this.DEPLOY_NOTREADY){
                        returnObj.error = true;
                    }
                    if(items[i].deployStatus == this.DEPLOY_UNKNOWN){
                        returnObj.warning = true;
                    }
                }
            }
            return returnObj;
        },
        
        /**
         * Function used to determine deploy status of servers based on FW levels and access states.
         * 
         * item: Singular object representing a server
         */
        _determineDeployStatus: function(item){
            if(item.firmwareLevel){
                if(item.firmwareLevel == this.FW_UNKNOWN){
                    return this.DEPLOY_UNKNOWN;   
                }
                if(item.firmwareLevel == this.FW_BACKLEVEL){
                    return this.DEPLOY_NOTREADY;   
                }
            }
            if(item.access){
                if(item.access == "offline" || item.access == "partial" || 
                        item.access == "unknown" || item.access == "-Default-" ){
                    return this.DEPLOY_UNKNOWN;
                }
                if(item.access == "none"){
                    return this.DEPLOY_NOTREADY;
                }
            }
           
            return this.DEPLOY_OK;
        },
        
        /**
         * Function to generate deploy message tooltip content based on firmware levels and access states
         * 
         * item: Singular object representing a server
         */
        _getDeployMsg: function(item){
            if(item.firmwareLevel){
                if(item.firmwareLevel == this.FW_UNKNOWN){
                    return this.resources.FW_UNKOWN_MSG;   
                }
                if(item.firmwareLevel == this.FW_BACKLEVEL){
                    return this.resources.FW_BACKLEVEL_MSG;  
                }
            }
            if(item.access){
                if(item.access == "offline"){
                    return this.resources.ACCESS_OFFLINE+ " - "+this.resources.ACCESS_CIM_CONFIRM;
                } 
                else if(item.access == "partial"){
                    return this.resources.ACCESS_PARTIAL+ " - "+this.resources.ACCESS_CIM_CONFIRM;
                }
                else if(item.access == "unknown"){
                    return this.resources.ACCESS_UNKNOWN+ " - "+this.resources.ACCESS_CIM_CONFIRM;
                } 
                else if(item.access == "-Default-"){
                    return this.resources.ACCESS_DEFAULT+ " - "+this.resources.ACCESS_CIM_CONFIRM;
                } 
                else if(item.access == "none"){
                    return this.resources.ACCESS_NONE+ " - "+this.resources.ACCESS_CIM_CONFIRM;;
                }
            }
           
            return this.resources.FW_COMPAT;
        },
        
        /**
         * Function to return a deploy status message based on the row number.
         * 
         * rowIndex: Int - Row index
         * leftTable: Boolean - True if available table(left).  False if selected table(right).
         * 
         */
        _getCellMsg: function(rowIndex, leftTable){
            var items;
            var i;
            if(leftTable){
                items = this._availTable.store._arrayOfTopLevelItems;
            }
            else {
                items = this._selectTable.store._arrayOfTopLevelItems;
            }
            for(i=0;i<items.length;i++){
                if(i == rowIndex){
                    if(items[i].deployMsg){
                        return items[i].deployMsg;
                    }
                    return "";
                }
            }
            return "";
        },
        
        _deployClicked: function(){
            if(this.CP_RUN_ENVIRONMENT == "fsm"){
                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {
                    var prompt = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.DEPLOY,
                        type: "Warning",
                        text: this.locResources.DEPLOY_WARNING_HEADER,
                        info: this.locResources.DEPLOY_WARNIING_MSG.replace("%1", this.PRODUCT_NAME_LONG)
                    });
                    prompt.startup();
                    prompt.confirm(this._onFinish, this);
                }));
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){
                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {                
                    var prompt = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.DEPLOY,
                        type: "Warning",
                        text: this.locResources.DEPLOY_WARNING_HEADER,
                        info: this.locResources.DEPLOY_WARNIING_MSG.replace("%1", this.PRODUCT_NAME_LONG)
                    });
                    prompt.startup();
                    prompt.confirm(this._onFinish, this);                                
                }));
            }
        }

    });

});
