define([ "dojo", 
         "dijit",
         "support/dialogs/Dialog",
         "dojo/i18n!./nls/DeployPlaceholderChassis",
         "dojo/text!./templates/DeployPlaceholderChassis.html",
         "support/widgets/OutlinePane",
         "support/widgets/Icon",
         "dijit/layout/ContentPane",
         "dojo/data/ItemFileWriteStore",
         "dijit/form/Select",
         "dijit/form/Button",
         "support/base/TableMixin",
         "support/dialogs/ProgressDialog",
         "dijit/TitlePane"
    ], function(dojo, dijit, dialog, mri, template) {
    
    return dojo.declare("support.dialogs.DeployPlaceholderChassis", [ support.dialogs.Dialog, support.base.TableMixin ], {

        // Path to the template
        templateString: template,
        
        locResources: mri,
        
        baseClass: "DeployPlaceholderChassis",
        
        spCssPathAndName: "support/dialogs/themes/DeployPlaceholderChassis.css",
                
        placeholderChassisId: null,  // Pre-select option/value
        targetChassisUuid: null, // Pre-select option/value
        activationSchedule: null, // Pre-select option/value

        placeholderTable: null,       
        
        targetTable: null,        
        
        // ID prefixes used for tables
        placeholderGridID: "PlaceholderTable",
        targetGridID: "TargetTable",

        // Override this method to perform custom behavior during dijit construction.
        // Common operations for constructor:
        // 1) Initialize non-primitive types (i.e. objects and arrays)
        // 2) Add additional properties needed by succeeding lifecycle methods
        constructor: function() {
            
        },

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering().  This is useful to override when 
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this". 
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets 
        postCreate: function() {
            this.inherited(arguments);
            
            this._activation.addOption([{
                label: this.resources.IMMEDIATE_ACTIVATION,
                value: "imm"
                }, {
                label: this.resources.DEFER_ACTIVATION,
                value: "defer"
                } 
            ]);
            this.activationSchedule && this._activation.set("value", this.activationSchedule);
            dojo.style(this._activationContainer, "display", "none");
            
            // User tip
            dojo.style(this._step2, "display", "none");
            dojo.style(this._step3, "display", "none");
          
            // Placeholder chassis
            this._showPlaceholderTable();
            
            // Target chassis
            dojo.style(this._targetContainer, "display", "none");
            
            // Activation schedule
            dojo.style(this._activationContainer, "display", "none");
            
            this.okButton.set("disabled", this.validate());
            
            dojo.connect(this.dialogNode, "hide", this, function() {
                this.destroy();
            });
        },
        
        _showPlaceholderTable: function() {
            
            var tableStore = this._retrieveStore(false);
            var tableStructure = this._getTableStructure(false);
            
            this.placeholderTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.placeholderGridID, true);
            this.placeholderTable.column(0).sort(false);

            this.placeholderTable.select.row.multiple = false; // set table to single select
            
            setTimeout(dojo.hitch(this, function() {
                this.placeholderTable.placeAt(this._placeholderTable);
                this.placeholderTable.startup();
            }), 500);                       
            
            dojo.connect(this.placeholderTable.select.row, "onSelectionChange", this, this._onChangePlaceholder);

        },
         
        _setupTargetTable: function (phcId) {
            console.log(this.declaredClass + ": phcId: ", phcId);
      
            this.targetTable && this.targetTable.destroy(); // Destroy old table and start from scratch.
            
            var tableStore = this._retrieveStore(phcId);
            var tableStructure = this._getTableStructure(true);
            
            this.targetTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.targetGridID, true);
            this.targetTable.column(0).sort(false);
            
            this.targetTable.select.row.multiple = false; // set table to single select
            
            setTimeout(dojo.hitch(this, function() {
                this.targetTable.placeAt(this._targetTableNode);
                this.targetTable.startup();
            }), 500);                   
            
            dojo.connect(this.targetTable.select.row, "onSelectionChange", this, this._onChangeTarget);

        },
        
        _onChangePlaceholder: function() {
            var selection = this.placeholderTable.select.row.getSelected();
            
            if(this.placeholderTable.row(selection[0], true) != null){
                var item = this.placeholderTable.row(selection[0], true).item();
                var phcId = item.id.toString();
            
                if (this.placeholderChassisId != phcId) {
                    this.placeholderChassisId = phcId;
                    this._selectTargetChassis(phcId);                
                }
            
                this._placeholderPane.toggle();
                this._placeholderPane.set("title", this.locResources.SELECTED_PLACEHOLDER + ": <span style=\"font-weight:normal;\">" + item.name + "</span>");                        

            }
            
            this.okButton.set("disabled", this.validate());
            return;
        },
        
        _onChangeTarget: function() {
            var selection = this.targetTable.select.row.getSelected();
            
            if(this.targetTable.row(selection[0], true) != null){
                var item = this.targetTable.row(selection[0], true).item();
                var tgtUuid = item.uuid.toString();
                    
                if (this.targetChassisUuid != tgtUuid) {
                    this.targetChassisUuid = tgtUuid;
                    this._selectActivationSchedule();
                }
                
                this._targetPane.toggle();
                this._targetPane.set("title", this.locResources.SELECTED_TARGET + ": <span style=\"font-weight:normal;\">" + item.name + "</span>");
                this._targetSelectMode(true);

            }
            
            this.okButton.set("disabled", this.validate());
            return;
        },
        
        _retrieveStore: function(phcId) {
            
            if (phcId) {
                // Target Table
                var targetData = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "chassis/filter?deployableFromPlaceholderId=" + phcId,
                    handleAs: "json",
                    preventCache: true,
                    sync: true,
                    load: dojo.hitch(this, function(response) {
                        targetData = this._getTargets(response.items);
                    }),
                    handle: function() {
                        
                    }
                });
                
                var targetStore = new dojo.data.ItemFileWriteStore({
                    data: dojo.fromJson(targetData)
                });
                
                return targetStore;
            }
            else {
                // Placeholder Table
                var placeholderData = "";
                dojo.xhrGet({
                    url: this.REST_PATH + "phchassis?format=detailed",
                    handleAs: "json",
                    preventCache: true,
                    sync: true,
                    load: dojo.hitch(this, function(response) {
                        placeholderData = this._getPlaceholders(response.items);
                    }),
                    handle: function() {
                        
                    }
                });
                
                var placeholderStore = new dojo.data.ItemFileWriteStore({
                    data: dojo.fromJson(placeholderData)
                });
                
                return placeholderStore;
            }
        },

        _getTableStructure: function(target) {
            var layout = new Array();
            
            layout.push({
                field: "name",
                name: this.resources.NAME,
                width: "150px"
            });
            
            if (target) {       
                // Target Table
                layout.push({
                    field: "access",
                    name: this.resources.ACCESS,
                    decorator: dojo.hitch(this, this._decorateAccess),
                    formatter: dojo.hitch(this, this._formatAccess),
                    width: "50px"
                });
                layout.push({
                    field: "ipaddresses",
                    name: this.locResources.IP,
                    width: "auto"
                });
            }
            else {
                // Placeholder Table
                layout.push({
                    field: "description",
                    name: this.resources.DESCRIPTION,
                    width: "auto"
                });
            }
            
            return layout;
        },
        
        _decorateAccess: function(item) {
            // summary:
            // Formats "access" field value for display in TreeTable.
            // tags:
            // protected
            var access = item;
            if (access) {
                return "<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spSuccessIcon\" title=\"" + this.resources.ACCESS_OK + "\"></img>";
            }
            return "<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spOfflineIcon\" title=\"" + this.resources.ACCESS_OFFLINE + "\"></img>";

        },
        
        _formatAccess: function(item) {
            return item.access;
        },
        
        _getPlaceholders: function(response){
            var items = response;
            var newItems = new Array();
            
            for(var idx = 0; idx < items.length; ++idx){
                var hasProfile = (items[idx].profileId != "");
                for (var idx2 = 0; !hasProfile && (idx2 < items[idx].bays.length); ++idx2) {
                    hasProfile = (items[idx].bays[idx2].profileId != "");
                }
                // Add only PCH with profile(s).
                hasProfile && newItems.push({
                    "id": items[idx].id,
                    "name": items[idx].name,
                    "description": items[idx].description
                });
            }
            
            var newData = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": newItems
            }, true);
            
            return newData;
        },
        
        _getTargets: function(response){
            var items = response;
            var i;
            var newItems = new Array();
            
            for(i = 0; i < items.length; i++){
                var childObject = new Object();
                childObject.uuid = items[i].uuid;
                childObject.id = items[i].id;
                childObject.name = items[i].name;
                childObject.access = items[i].access;
                childObject.ipaddresses = items[i].ipaddresses;
                newItems.push(childObject);
            }
            
            var newData = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": newItems
            }, true);
            
            return newData;
        },
        
        _selectTargetChassis: function(phcId) {
            
            if (this._targetContainer.style.display == "none") {
                // User tip
                dojo.style(this._step1, "display", "none");
                dojo.style(this._step2, "display", "inline-block");
            }
            
            // Target Chassis Table
            this._setupTargetTable(phcId);
            this._targetSelectMode(false);
            
        },
        
        _selectActivationSchedule: function() {
            // User tip
            dojo.style(this._step2, "display", "none");
            dojo.style(this._step3, "display", "inline-block");
            
            // Activation Schedule
            dojo.style(this._activationContainer, "display", "inline-block");
            
        },
              
        _targetSelectMode: function(readOnly) {
            // Consider false if called from onActionClick
            if (readOnly != true && readOnly != false) {
                readOnly = false;
            }            
            if (readOnly == false) {
                this.targetTable.select.row.clear(); // deselect all rows
                this._targetPane.set("title", this.locResources.TARGET);
                if (!this._targetPane.open) {
                    this._targetPane.toggle();
                }
            }            
            
            dojo.style(this._targetContainer, "display", "inline-block");
            this.targetTable && this.targetTable.resize();            
        },
        
        validate: function() {
            var placeholderSelection = [];
            var targetSelection = [];
            
            if (this.placeholderTable) {
                placeholderSelection = this.placeholderTable.select.row.getSelected();
            }
            if (this.targetTable) {
                targetSelection = this.targetTable.select.row.getSelected();
            }            
            
            // No placeholder selected
            if (placeholderSelection.length < 1) {
                return true;
            }
            
            // No target selected
            if ((this._targetContainer.style.display == "none") || (targetSelection.length < 1)) {
                return true;
            }
            
            // No activation schedule set
            if ((this._activationContainer.style.display == "none") || (this._activation.get("value").toString().length <= 0)) {
                return true;
            }
            
            return false;
        },
        
        _pushJobSuccessInfo: function(/*Object*/item, /*Array*/msgContent) {
            // Push job success information into the msgContent array.
            // Expect to see jobname w/ endPointIds/locationIds data.
            msgContent.push(this.resources.SUBMIT_JOB_SUCCESS.replace("%1", item.jobName));
            if (item.endPointNames && (item.endPointNames.length > 0)) {
                // endPointNames indicate job was for Chassis profile, so add content accordingly.
                msgContent.push(this.resources.SUBMIT_JOB_SUCCESS_CHASSIS.replace("%1", item.endPointNames.join(", ")));                
            }
            else if (item.locationNames) {
                // else locationNames for Server profile is assumed, so add content accordingly.
                msgContent.push(this.resources.SUBMIT_JOB_SUCCESS_SERVERS.replace("%1", item.locationNames.join(", ")));                
            }
            msgContent.push("<p>" + this.resources.SUBMIT_JOB_DETAILS + "</p>");
        },
        
        _pushJobErrorInfo: function(/*Object*/item, /*Array*/msgContent) {
            // Push job failure information into the msgContent array.
            // No jobname, but endPointIds/locationIds are valid.
            msgContent.push(this.resources.SUBMIT_JOB_ERROR);
            if (item.endPointNames && (item.endPointNames.length > 0)) {
                // endPointNames indicate job was for Chassis profile, so add content accordingly.
                msgContent.push(this.resources.SUBMIT_JOB_ERROR_CHASSIS.replace("%1", item.endPointNames.join(", ")));                
            }
            else if (item.locationNames) {
                // else locationNames for Server profile is assumed, so add content accordingly.
                msgContent.push(this.resources.SUBMIT_JOB_ERROR_SERVERS.replace("%1", item.locationNames.join(", ")));                
            }
        },

        
        onFinish: function() {
            
            var deployAction = dojo.hitch(this, function() {
                var activationSchedule = this._activation.get("value").toString();
                var postData = dojo.toJson({
                    "uuid": this.targetChassisUuid,
                    "activation": activationSchedule
                }, true);
                this.getProgressDialog("deploy").show();
                dojo.xhrPost({
                    url: this.REST_PATH + "phchassis/deploy/" + this.placeholderChassisId,
                    handleAs: "json",
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    postData: postData,
                    handle: dojo.hitch(this, function(response, ioArgs) {
                        console.log(this.declaredClass + ": Deploy pch complete: ", response, ioArgs);
                        if (ioArgs.xhr.status >= 300) {
                            dojo.publish("ProgressFailed", []);
                            this.getModalDialog("Error", this.locResources.DEPLOY_ERROR_MSG).show();
                            console.error("Failed to deploy placeholder chassis.");
                        }
                        else {
                            // Examine response items to determine potential for multiple success/failure
                            var info = new Array();
                            var oneSuccess = false;
                            var oneFailure = false;
                            for (var idx = 0; idx < response.items.length; ++idx) {
                                var item = response.items[idx];
                                var title = this.resources.SUBMIT_JOB_TITLE.replace("%1", idx + 1);
                                var msgContent = new Array();
                                if (!item.message || (item.message.length == 0)) {
                                    this._pushJobSuccessInfo(item, msgContent);
                                    oneSuccess = true;
                                }
                                else {
                                    this._pushJobErrorInfo(item, msgContent);
                                    oneFailure = true;
                                }
                                info.push({
                                    title: title,
                                    content: msgContent.join(" ")
                                });
                            }
                            var msgType = null;
                            var msgText = null;
                            if (oneSuccess) {
                                if (oneFailure) {
                                    msgType = "Warning";
                                    msgText = this.resources.DEPLOY_WARNING_MSG;
                                }
                                else {
                                    msgType = "Success";
                                    if (info.length > 1) {
                                        msgText = this.resources.DEPLOY_SUCCESS_MULTIJOBS_MSG.replace("%1", info.length);
                                    }
                                    else {
                                        msgText = this.resources.DEPLOY_SUCCESS_MSG;
                                    }
                                }
                            }
                            else {
                                msgType = "Error";
                                msgText = this.locResources.DEPLOY_ERROR_MSG;
                            }
                            dojo.publish("ProgressComplete", response.id);
                            this.getModalDialog(msgType, msgText, ((info.length == 1) ? info[0].content : info)).show();                            
                            dojo.publish("deployPHChassis", response);
                            this._onClose();
                        }
                    })
                });
            });
            
            // Submit phc deploy request
            deployAction();
            return false; // Don't allow close! Allow handlers (above) to determine when to close.
            
        }                
        
    });
});

