define([
        "dojo", 
        "dijit", 
        "support/dialogs/Dialog", 
        "dojo/i18n!./nls/DeployServerTemplate",
        "dojo/text!./templates/DeployServerTemplate.html", 
        "support/widgets/Select",
        "dojo/data/ItemFileWriteStore", 
        "dijit/form/Button",
        "dijit/Toolbar",
        "support/base/TableMixin",
        "support/panels/DualTableSelector",
        "support/dialogs/ProgressDialog"
    ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.DeployServerTemplate", [ support.dialogs.Dialog, support.base.TableMixin ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "DeployTemplate",

        // templateId: String
        // Value of entry to pre-select in this widget's template selector.
        templateId: null,
        init: false,
        patternFormFactor: "2s", //default to 2s if pattern not selected
        _overlapErrorArea: null,
        
        _selectedServers: new Array(),    //Holds selected servers

        _toolbar: null,
        
        // UUID of endpoint to deploy template to; this isn't called
        // "serverUUID" because this widget can also be launched from a
        // chassis
        endpointUUID: null,
        
        //FW Level State Names - Internal Usage
        FW_UNKNOWN: "unknown",
        FW_BACKLEVEL: "backlevel",
        FW_OK: "compatible",
        DEPLOY_OK: "ok",
        DEPLOY_NOTREADY: "notReady",
        DEPLOY_UNKNOWN: "unknown",
        DEPLOY_NOT_SUPPORTED: "notSupported",
                
        // ID prefixes used for tables
        availGridID: "AvailableServerTable",
        selectGridID: "SelectServerTable",

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/DeployTemplate.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in
        // the template HTML file
        postMixInProperties: function() {

            this.inherited(arguments);

        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be
        // directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);
            this._setupTemplateSelector();
            this._setupTables(); // Must call after _setupTemplateSelector() to ensure template is selected.
            
            // Need to wait until after _setupTables() is called to startup the template selector
            this._tmplSelector.startup();

            // subscribe to the topic "addPHChassis" so that the Available
            // Table can reload its contents and immediately show the new
            // placeholder chassis
            this.subscribe("addPHChassis", function() {
                console.log(this.declaredClass + ": Notified of new placeholder chassis; reloading available table");
                this._getSelected();
                this._refreshTree();
            });

            // subscribe to the topic "removePHChassis" so that the Available
            // Table can reload its contents and immediately show the
            // removal of the placeholder chassis
            this.subscribe("removePHChassis", function() {
                console.log(this.declaredClass + ": Notified of placeholder chassis removal; reloading available table");
                this._getSelected();
                this._refreshTree();
            });

            dojo.connect(this._schedule, "onChange", this, this._setSelectedTableSchedules);
            
            dojo.connect(this._availTable.select.row, "onSelectionChange", this, this._availNodeClicked); // Fires on row selection

            // if showing this dialog for a specific server, pre-select it            
            this._preselectTargetServer(); 
            
            var msgDescription = this.locResources.OVERLAPPING_DETAILS;
            var msgTitle = this.locResources.OVERLAPPING_TITLE;
            
            if(this.CP_RUN_ENVIRONMENT == "fsm"){
                
                require([ "idx/oneui/messaging/SingleMessage" ], dojo.hitch(this, function() {

                    this._overlapErrorArea = new idx.oneui.messaging.SingleMessage({
                        description: msgDescription,
                        title: msgTitle,
                        type: "error",
                        dateFormat: { formatLength: "medium", selector: "time" },
                        showId: false,
                        showAction: false,
                        showRefresh: false,
                        showDetailsLink: false,
                        closable: false
                    }).placeAt(this.id + "_overlapErrorArea");
                }));

            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){
            
                require([ "idx/widget/SingleMessage" ], dojo.hitch(this, function() {

                    this._overlapErrorArea = new idx.widget.SingleMessage({
                        description: msgDescription,
                        title: msgTitle,
                        type: "error",
                        dateFormat: { formatLength: "medium", selector: "time" },
                        showId: false,
                        showAction: false,
                        showRefresh: false,
                        showDetailsLink: false,
                        closable: false
                    }).placeAt(this.id + "_overlapErrorArea");
                }));
            }

        },
        
        _pushJobSuccessInfo: function(/*Object*/item, /*Array*/msgContent) {
            // Push job success information into the msgContent array.
            // Expect to see jobname w/ endPointIds/locationIds data.
            msgContent.push(this.resources.SUBMIT_JOB_SUCCESS.replace("%1", item.jobName));
            var names = [];
            for (var loopVar in this._selectTable.store._arrayOfAllItems) {
                var selected = this._selectTable.store._arrayOfAllItems[loopVar];
                if (!selected) {
                    continue;
                }
                // get the name of the server that was deployed to
                if (item.locationIds != null && selected.location != null &&
                    selected.location[0] != null &&
                    dojo.indexOf(item.locationIds, selected.location[0]) >= 0) {
                    names.push(selected.name[0]);
                }
            }
            msgContent.push(this.resources.SUBMIT_JOB_SUCCESS_SERVERS.replace("%1", names.join(", ")));
            msgContent.push("<p>" + this.resources.SUBMIT_JOB_DETAILS + "</p>");
        },
        
        _pushJobErrorInfo: function(/*Object*/item, /*Array*/msgContent) {
            // Push job failure information into the msgContent array.
            // No jobname, but endPointIds/locationIds are valid.
            msgContent.push(this.resources.SUBMIT_JOB_ERROR);
            var names = [];
            for (var loopVar in this._selectTable.store._arrayOfAllItems) {
                var selected = this._selectTable.store._arrayOfAllItems[loopVar];
                if (!selected) {
                    continue;
                }
                // get the name of the server that was deployed to
                if (item.locationIds != null && selected.location != null &&
                    selected.location[0] != null &&
                    dojo.indexOf(item.locationIds, selected.location[0]) >= 0) {
                    names.push(selected.name[0]);
                }
            }
            msgContent.push(this.resources.SUBMIT_JOB_ERROR_SERVERS.replace("%1", names.join(", ")));
        },

        onFinish: function() {

            //Check If FW Levels OK For Deployment and make sure all targets supported
            var statusObj = this._checkDeployStatusBeforeDeploy(this._selectTable.store._arrayOfTopLevelItems);
                        
            //Display error or warning message if needed
            if (statusObj.error == true) {
                this.getModalDialog("Error", this.resources.DEPLOY_STATUS_ERROR_MSG).show();
                return; //Prevent from deploying with those selections
            }
            if (statusObj.unsupported == true) {
                this.getModalDialog("Error", this.locResources.DEPLOY_STATUS_ERROR_MSG_SUPPORTED).show();
                return; //Prevent from deploying with those selections
            }

            var data = {
                templateId: this._tmplSelector.value,
                endPointIds: [],
                locationIds: [],
                restart: this._schedule.get("value")
            };
            var deployAction = dojo.hitch(this, function() {
                var jsonString = dojo.toJson(data);
                this.getProgressDialog("deploy").show();
                dojo.xhrPost({
                    url: this.REST_PATH + "profile/deploy",
                    handleAs: "json",
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    postData: jsonString,
                    handle: dojo.hitch(this, function(response, ioArgs) {
                        console.log(this.declaredClass + ": Deploy pattern complete: ", response, ioArgs);
                        if (ioArgs.xhr.status >= 300) {
                            dojo.publish("ProgressFailed", []);
                            this.getModalDialog("Error", this.locResources.DEPLOY_ERROR_MSG).show();
                            console.error("Failed to deploy server pattern.");
                        }
                        else {
                            // Examine response items to determine potential for multiple success/failure
                            var info = new Array();
                            var oneSuccess = false;
                            var oneFailure = false;
                            for (var loopVar = 0; loopVar < response.items.length; ++loopVar) {
                                var item = response.items[loopVar];
                                var title = this.resources.SUBMIT_JOB_TITLE.replace("%1", loopVar + 1);
                                var msgContent = new Array();
                                if (!item.message || (item.message.length == 0)) {
                                    this._pushJobSuccessInfo(item, msgContent);
                                    oneSuccess = true;
                                }
                                else {
                                    this._pushJobErrorInfo(item, msgContent);
                                    oneFailure = true;
                                }
                                info.push({
                                    title: title,
                                    content: msgContent.join(" ")
                                });
                            }
                            var msgType = null;
                            var msgText = null;
                            if (oneSuccess) {
                                if (oneFailure) {
                                    msgType = "Warning";
                                    msgText = this.resources.DEPLOY_WARNING_MSG;
                                }
                                else {
                                    msgType = "Success";
                                    msgText = this.resources.DEPLOY_SUCCESS_MSG;
                                }
                            }
                            else {
                                msgType = "Error";
                                msgText = this.resources.DEPLOY_ERROR_MSG;
                            }
                            dojo.publish("ProgressComplete", response.id);
                            this.getModalDialog(msgType, msgText, ((info.length == 1) ? info[0].content : info)).show();                            
                            dojo.publish("DeployServerTemplateComplete", response);
                            this._onClose();
                        }
                    })
                });
            });
            var onlineServers = [];

            for (var loopVar in this._selectTable.store._arrayOfAllItems) {
                var item = this._selectTable.store._arrayOfAllItems[loopVar];
                if (item && item.type && item.location && (item.type != "chassis")) {
                    data.locationIds.push(item.location[0]);
                    // collect servers to warn the user
                    // these servers will be restarted
                    if ((item.powerStatus && item.powerStatus == "on") && (data.restart == "immediate")) {
                        onlineServers.push(item);
                    }
                }

            }
            if (onlineServers.length == 0) {
                deployAction();
            }
            else {
                // warn the users that online servers are about get restarted
                console.log(onlineServers);

                // extract server names from grid items to display in dialog
                var serverNames = this._getServerNamesFromGrid(onlineServers);
                if(this.CP_RUN_ENVIRONMENT == "fsm"){

                    require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                        var prompt = new support.temp.ConfirmationDialog({
                            buttonLabel: this.resources.DEPLOY,
                            type: "Warning",
                            text: this.locResources.ONLINE_SRVR_MSG.replace("%1", serverNames),
                            info: this.locResources.ONLINE_SRVR_CONFIRMATION_MSG
                        });
                        prompt.startup();
                        prompt.confirm(deployAction, this);
                
                    }));
                    
                }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                    require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                        var prompt = new idx.widget.ConfirmationDialog({
                            buttonLabel: this.resources.DEPLOY,
                            type: "Warning",
                            text: this.locResources.ONLINE_SRVR_MSG.replace("%1", serverNames),
                            info: this.locResources.ONLINE_SRVR_CONFIRMATION_MSG
                        });
                        prompt.startup();
                        prompt.confirm(deployAction, this);
                
                    }));

                }
            }
            return false; // Don't allow close! Allow handlers (above) to determine when to close.

        },

        _setupTemplateSelector: function() {

            // summary:
            // Set up template selector...
            // tags:
            // protected
            var patternStore = null;
            dojo.xhrGet({
                url: this.REST_PATH + "template/list/Server",
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: function(data) {
                    // Don't need to store all that server template info in the selector...
                    if (data.items) {
                        var items = [];
                        var newEntry = function(template) {
                            var temp = new Object();
                            temp.id = template.id;
                            temp.name = template.name;
                            temp.inUse = template.inUse;
                            temp.formFactor = template.formFactor;
                            temp.type = template.type;
                            temp.description = template.description;
                            temp.nodeType = template.nodeType;
                            temp.uri = template.uri;
                            return temp;
                        };
                        for (var idx = 0; idx < data.items.length; ++idx) {
                            var temp = newEntry(data.items[idx]);
                            items.push(temp);
                        }
                        var patternData = dojo.toJson({
                            "identifier": "id",
                            "label": "name",
                            "items": items
                        }, true);
                        patternStore = new dojo.data.ItemFileWriteStore({
                            data: dojo.fromJson(patternData)
                        });
                    }
                }
            });

            // Add prompt option before load so its listed first.
            patternStore.newItem({
                id: "-1",
                name: "&mdash; " + this.locResources.SELECT_TEMPLATE + " &mdash;"
            });

            this._tmplSelector = new support.widgets.Select({
                id: this.get("id") + "_tmplSelector",
                name: "template",
                required: true,
                store: patternStore,
                value: (this.templateId) ? this.templateId : "-1", // Pre-select entry from store,
                // if it exists.
                emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                onChange: dojo.hitch(this, this._changeTemplateCallback)
            }).placeAt(this._tmplSelector);
            dojo.connect(this._tmplSelector, "onSetStore", this, "_presetTemplateCallback");
            dojo.connect(this._tmplSelector, "startup", this, function() {
                if (this.templateId) {
                    this._changeTemplateCallback(this.templateId);
                } else {
                    this._deployButton.set("disabled", true);
                    this.set("title", this.locResources.PAGE_TITLE);                    
                }
            });
        },

        _presetTemplateCallback: function() {

            // tags:
            // protected
            if (this.templateId) {
                if (this._tmplSelector.get("value") === this.templateId.toString()) {
                    // Pre-select entry was found, disable the selector.
                    this._tmplSelector.set("disabled", true);
                }
            }
        },

        _changeTemplateCallback: function(newVal) {

            // tags:
            // protected

            var title = this.locResources.PAGE_TITLE;
            var option = this._tmplSelector.getOptions(newVal);
            
            if (option && newVal != "-1") {
                title = title + " - " + option.label;
                switch(option.item.formFactor[0]) {
                    case "2s":
                        this.formFactorLabel.innerHTML = this.locResources.BAY_1_CP;
                        break;
                    case "4s":
                        this.formFactorLabel.innerHTML = this.locResources.BAY_2_CP;
                        break;
                    case "1sd":
                        this.formFactorLabel.innerHTML = this.locResources.BAY_1_DCP;
                        break;
                    case "pxu":
                        this.formFactorLabel.innerHTML = this.locResources.BAY_2_CPE;
                        break;
                    case "2sc":
                        this.formFactorLabel.innerHTML = this.locResources.BAY_SCALE.replace("%1", "2");
                        break;
                    case "4sc":
                        this.formFactorLabel.innerHTML = this.locResources.BAY_SCALE.replace("%1", "4");
                        break;
                    case "8sc":
                        this.formFactorLabel.innerHTML = this.locResources.BAY_SCALE.replace("%1", "8");
                        break;
                    default:
                        this.formFactorLabel.innerHTML = this.locResources.BAY_1_CP;
                        break;
                }
                if (this.patternFormFactor != option.item.formFactor[0]) {
                    this.patternFormFactor = option.item.formFactor[0];
                    this._refreshTree();
                    this._setUnsupportedServers();
                }
                this._validate();
            }
               
            else {
                // Disable Deploy Button w/ Invalid Template Selection
                this._deployButton.set("disabled", true);
            }
            this.set("title", title);

        },
        
        //Get servers on selected side
        _getSelected: function() {
            this._selectedServers = new Array();
            var selectedObjs = this._selectTable.store._arrayOfAllItems;
            for ( var i = 0; i < selectedObjs.length; i++) {
                var selectedObj = selectedObjs[i];
                if (selectedObj != null) {
                    this._selectedServers.push(selectedObj.id);
                }
            }  
        },
        
        //Set server deploy status on all selected servers depending on what pattern is choosen
        _setUnsupportedServers: function() {
            
            var selected = this._selectTable.store._arrayOfTopLevelItems;
            
            for (var si = 0; si < selected.length; ++si) {
                if (selected[si].children) {
                    for (var ci = 0; ci < selected[si].children.length; ++ci) {
                        if (dojo.indexOf(this._selectedServers, selected[si].children[ci].id[0]) >= 0) {
                            selected.splice(ci, 1); // remove previously selected child
                            --ci; // backup 1 since child was removed
                            continue;
                        }
                        if (selected[si].children[ci].type[0] == "bay" || selected[si].children[ci].type[0] == "imm") {
                            selected[si].children[ci].deployStatus[0] = this._determineDeployStatus(selected[si].children[ci], this.patternFormFactor);
                            selected[si].children[ci].deployMsg[0] = this.locResources.TARGET_FORM_MSG;
                        }
                    }
                }
            }
            this._selectTable.model.clearCache();
            this._selectTable.body.refresh().then(this._selectTable.expandAll());

        },
        
        _checkOverlapping: function() {
            for (var chassis in this._selectTable.store._arrayOfTopLevelItems) {
                var servers = this._selectTable.store._arrayOfTopLevelItems[chassis].children;
                var offset = this.patternFormFactor == "4sc" ? 4 : 8;
                for (var si1 in servers) {
                    var bayToCheck = parseInt(servers[si1].bays[0].split("-")[0]);
                    for (var si2 in servers) {
                        var bay = parseInt(servers[si2].bays[0].split("-")[0]);
                        if (bay != bayToCheck && bay >= bayToCheck && bay < (bayToCheck + offset)) {
                            return true;
                        }
                    }
                }
            }
            
            return false;
        },
        
        _validate: function() {

            // Make sure there are selections in the selected table
            var num = this._selectTable.store._arrayOfTopLevelItems.length;
            var overlapping = false;
            if(this.patternFormFactor == "4sc" || this.patternFormFactor == "8sc") {
                overlapping = this._checkOverlapping();
            }
            
            if (num > 0 && this._tmplSelector.get("value") && this._tmplSelector.get("value") != "-1" && overlapping == false) {
                if (this._deployButton.get("disabled")) {
                    this._deployButton.set("disabled", false);
                }
            }
            else {
                this._deployButton.set("disabled", true);
            }
            
            if(overlapping == true) {
                this._overlapError.style.display = "inline";
                this._overlapErrorArea.resize();
            } else {
                this._overlapError.style.display = "none";
            }

        },
        
        _setupTables: function() {
            
            this._setupAvailableTable();
            this._setupSelectedTable();
            
            this._setupToolbar(this._availTable);

            this._dualTableSelector = new support.panels.DualTableSelector({
                _availTable: this._availTable,
                _selectTable: this._selectTable
            }).placeAt(this._dualTableSelectorNode);

            dojo.connect(this._dualTableSelector, "_moveRight", this, this._validate);
            dojo.connect(this._dualTableSelector, "_moveAllRight", this, this._validate);
            dojo.connect(this._dualTableSelector, "_moveLeft", this, this._validate);
            dojo.connect(this._dualTableSelector, "_moveAllLeft", this, this._validate);
        },
        
        _setupToolbar: function(grid) {
            var toolbar = grid && grid.toolBar && grid.toolBar.widget;
            
            if (toolbar) {
                var phButton = new dijit.form.Button({
                    showLabel: true,
                    label: this.resources.ADD_PLACEHOLDER_CHASSIS,
                    title: this.resources.ADD_PLACEHOLDER_CHASSIS_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._addPlaceholderChassis();
                    })
                });
                toolbar.addChild(phButton);
                this._delButton = new dijit.form.Button({
                    showLabel: false,
                    iconClass: "spDeleteInactiveIcon",
                    title: this.resources.REMOVE_PLACEHOLDER_CHASSIS_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._removePlaceholderChassis();
                    })
                });
                toolbar.addChild(this._delButton);
            }
        },

        _setupAvailableTable: function() {

            // summary:
            // Set up available targets table widget...
            // tags:
            // protected
            if (this._availTable) {
                // Destroy old table and start from scratch.
                this._availTable.destroy();
            }
            
            // Setup table widget...
            var tableStore = this._retrieveStore();
            var tableStructure = this._getTableStructure(false);
            this._availTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.availGridID, true, true);
            this._availTable.baseSort = [{colId: "bays", descending: false}];

            //Functions to handle tooltips for deploy status columns
            dojo.connect(this._availTable, "onCellMouseOver", dojo.hitch(this, function(e) {
                if (e.cellIndex == 2) {
                    //Get message content for tooltip
                    var msg = this._getCellMsg(e.rowIndex, true);
                    if (msg == "") {
                        return;
                    }
                    dijit.showTooltip("<div style=\"max-width:200px\">" + msg + "</div>", e.cellNode);
                }
            }));
            dojo.connect(this._availTable,"onCellMouseOut", function(e) {
              if (e.cellIndex == 2) {
                  dijit.hideTooltip(e.cellNode);                
              }
            });

        },

        _setupSelectedTable: function() {

            // summary:
            // Set up selected targets table widget...
            // tags:
            // protected
            if (this._selectTable) {
                // Destroy old table and start from scratch.
                this._selectTable.destroy();
            }
            
            // Start with empty select table
            var emptyData = {
                    items: [],
                    label: "name",
                    identifier: "id"
            };
            var selectedStore = new dojo.data.ItemFileWriteStore({
                data: emptyData
            });

            // Setup table widget...
            var tableStore = selectedStore;
            var tableStructure = this._getTableStructure(true);
            this._selectTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.selectGridID, true, true);
            this._selectTable.baseSort = [{colId: "bays", descending: false}];
            
            //Functions to handle tool tips for deploy status column
            dojo.connect(this._selectTable, "onCellMouseOver", dojo.hitch(this, function(e) {
                if (e.cellIndex == 2) {
                    //Get tooltip message for this cell
                    var msg = this._getCellMsg(e.rowIndex, false);
                    if (msg == "") {
                        return;
                    }
                    dijit.showTooltip("<div style=\"max-width:200px\">" + msg + "</div>", e.cellNode);
                }
            }));
            dojo.connect(this._selectTable, "onCellMouseOut", function(e) {
                if (e.cellIndex == 2) {
                    dijit.hideTooltip(e.cellNode);                
                }
            });

        },

        _addPlaceholderChassis: function() {
            require([ "support/dialogs/AddPlaceholderChassis" ], dojo.hitch(this, function() {
                // show the Add Placeholder Chassis dialog
                var apcdialog = new support.dialogs.AddPlaceholderChassis();
                apcdialog.show();
                // no need to dismiss or delete this dialog; the Add Placeholder
                // Chassis dialog does this when the user clicks OK or Cancel
            }));
        },

        _removePlaceholderChassis: function() {
            // only process the remove if we have selected one Placeholder
            // Chassis, which is determined in the _availNodeClicked
            // function
            if (this._delButton.iconClass == "spDeleteIcon") {
                // if we get this far, we've already checked in the
                // _availNodeClicked() function that only one node was
                // clicked
                var msg = "";
                var phChassisObj = this._availTable.select.row.getSelected()[0];
                var item = this._availTable.row(phChassisObj, true).item();
                var chassisID = item.id;
                console.log(this.declaredClass + ": Removing new Placeholder Chassis with name: " + item.name + " and ID: " + chassisID);
                var success = true;
                var xhrDelete = dojo.xhrDelete({
                    url: this.REST_PATH + "phchassis/" + chassisID,
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    error: dojo.hitch(this, function(error, ioArgs) {
                        console.error(this.declaredClass + ": Problem deleting Placeholder Chassis with name: " + item.name
                                + " error: " + error);
                        if (ioArgs.xhr.status == 400) {
                            msg = this.locResources.DELETE_PH_ERROR;
                        }
                        this.getModalDialog("Error", msg).show();
                        success = false;                       
                    })
                });
                
                // if an error occurs, return
                if (success == false) {
                    return success;
                }
                
                var selected = this._selectTable.store._arrayOfTopLevelItems;
                for (var si = 0; si < selected.length; ++si) {
                    if (selected[si].id.toString() == chassisID.toString()) {
                        this._selectTable.collapseAll();
                        this._selectTable.select.row.selectById(chassisID);
                        this._dualTableSelector._moveLeft();
                    }
                }
                
                // publish the topic "removePHChassis" so that the Available
                // Table can reload its contents and immediately show that
                // the placeholder chassis was removed
                dojo.publish("removePHChassis", []);
                
                this._delButton.set("iconClass", "spDeleteInactiveIcon");

                return success;
            }
        },

        _availNodeClicked: function() {
            var selectedRows = this._availTable.select.row.getSelected();
            if (selectedRows.length == 0) {
                return;
            }
            var item = this._availTable.row(selectedRows[0], true).item();
            // "activate" the Remove Placeholer Chassis button (by changing
            // its icon from a gray "X" to a red "X") but only if the user
            // has selected exactly one Placeholder Chassis
            if (selectedRows.length == 1 && item.access == "placeholder") {
                this._delButton.set("iconClass", "spDeleteIcon");
            }
            else {
                this._delButton.set("iconClass", "spDeleteInactiveIcon");
            }
        },
       
        _deployFormatter: function(item) {
            
            // summary:
            // Formats "deployStatus" field value for display.
            // tags:
            // protected
            if (!item.deployStatus || (item.deployStatus == "ignore")) {
            	return "";
           	}
           	
            var fwLevel = item.deployStatus;
            var accessText = "";
            if(fwLevel == "ignore"){
                return [];
            }
            if (fwLevel == this.DEPLOY_OK) { // OK 
                accessText = this.resources.FW_COMPAT;
            }
            else if (fwLevel == this.DEPLOY_NOTREADY) { // NOT READY
                accessText = this.resources.FW_BACK_LEVEL;
            }
            else if (fwLevel == this.DEPLOY_NOT_SUPPORTED) { // NOT SUPPORTED
                accessText = this.locResources.NOT_SUPPORTED;
            }
            else /*(access == "unknown")*/ { // Unknown
                accessText = this.resources.FW_UNKNOWN;
            }            

            return accessText;
        },
                
        /**
         * Function used to assign correct icon for deploy status column based on deploy status
         */
        _deployDecorator: function(item) {
            var text = item;
            var iconClass = "";
            
            switch (text) {
                case this.resources.FW_COMPAT:
                    iconClass = "spSuccessIcon";
                    break;
                case this.resources.FW_BACK_LEVEL:    
                    iconClass = "spWarningIcon";
                    break;
                case this.locResources.NOT_SUPPORTED:
                    iconClass = "spWarningIcon";
                    break;
                case this.resources.FW_UNKNOWN:
                    iconClass = "spUnknownIcon";
                    break;
            }
            
            return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr><td>"
            		+ "<img src=\"" + this._blankGif + "\" alt=\"\" class=\"" + iconClass + "\" title=\"" + text + "\"></img></td>"
					+ "<td><span style=\"margin-left:4px\" title=\"" + text + "\">" + text + "</span></td>"
					+ "</tr></tbody></table>";
        },
        
        _refreshTree: function() {
            // tags:
            // protected
            // Refresh tree data
            var tableStore = this._retrieveStore();
            this._availTable.select.row.clear(); // clear selected rows
            this._availTable.setStore(tableStore);
            this._availTable.model.clearCache();
            this._availTable.body.refresh().then(this._availTable.reexpandRows);
        },

        _retrieveStore: function() {          
            // Get server pattern form factor
            var patternId = this._tmplSelector.get("value");
            if (patternId != null && patternId != "") {
                dojo.xhrGet({
                    url: this.REST_PATH + "template/" + patternId,
                    handleAs: "json",
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    sync: true,
                    preventCache: true,
                    load: function(response, ioArgs) {
                        if (response.items && (response.items.length > 0)) {
                            this.patternFormFactor = response.items[0].formFactor;
                        }
                    },
                    handle: function(response, ioArgs) {
                        console.log(this.declaredClass + ": Load template complete: ", response, ioArgs);
                    }
                });
            }
            
            // Get table stores
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "deploy/list/",
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: dojo.hitch(this, function(response) {
                    this._loadTableStore(response);
                }),
                error: dojo.hitch(this, function(response) {
                    // if the REST API was unsuccessful in returning data,
                    // show an empty table
                    console.log(this.declaredClass + ": Unable to retrieve server patterns from API: deploy/list/");
                    var patternData = dojo.toJson({
                        "identifier": "id",
                        "label": "name",
                        "items": []
                    }, true);
                    this._availStore = new dojo.data.ItemFileWriteStore({
                        data: dojo.fromJson(patternData)
                    });
                })                                
            });
            
            if (!("hasChildren" in this._availStore)) {
                this._availStore.hasChildren = function(id, item) { 
                    return item && this.getValues(item, "children").length;
                };
            };
            
            if (!("getChildren" in this._availStore)) {
                this._availStore.getChildren = function(item) { 
                    return this.getValues(item, "children");
                };
            };
            
            return this._availStore;
        },
        
        _loadTableStore: function (response) {
            var rootArray = new Array();      //Holds chassis 
            var childrenArray = new Array();  //Hold bays for UI display
            
            if (response && response.items) {
                for (var ri = 0; ri < response.items.length; ++ri) { //Loop through Chassis
                    //Set deploy status fields and expand status fields if a child node.  Set root nodes to have deploy status ignored
                    var rootObject = response.items[ri];
                    rootObject.expandMe = "false";
                    rootObject.deployStatus = "ignore";
                    var index; //Holds index search from selected Servers search                               
                    
                    if (rootObject.children) {   //Loop through Servers in Chassis
                        for (var ci = 0; ci < rootObject.children.length; ++ci) {                                        
                            //If one bay has more then one compute node then loop through compute nodes (Caracara)
                            if (rootObject.children[ci].ites.length > 1) {                                             
                                for (var ii = 0; ii < rootObject.children[ci].ites.length; ++ii) {  //Loop compute nodes                                                
                                  //Set deploy status fields   
                                  rootObject.children[ci].ites[ii].deployStatus = this._determineDeployStatus(rootObject.children[ci].ites[ii]);
                                  rootObject.children[ci].ites[ii].deployMsg = this._getDeployMsg(rootObject.children[ci].ites[ii]);
                                  rootObject.children[ci].ites[ii].parentId = rootObject.id;
                                    
                                  //Check if pattern caracara, if so add this caracara node to list
                                  if (this.patternFormFactor == "1sd" && rootObject.children[ci].ites[ii].subBays.length != 0) {    
                                      //Set bay label with subnode indication
                                      rootObject.children[ci].ites[ii].bays[0] = rootObject.children[ci].ites[ii].bays[0] + ":" + rootObject.children[ci].ites[ii].subBays[0];                                                                                    
                                      index = dojo.indexOf(this._selectedServers, rootObject.children[ci].ites[ii].id);                                                  
                                      //Add node only if not already on selected side
                                      if (index == -1) {
                                          childrenArray.push(rootObject.children[ci].ites[ii]);
                                      } else { //pull item out of selectedServers array to note that the status of whats left in the 
                                             //selectedServers array was not found in the newly created list so its status will change
                                          this._selectedServers.splice(index,1);
                                      }
                                  }
                                  
                                }
                                
                            } else if(rootObject.children[ci].ites.length == 1) { //If bay has only one compute node (Everything except Caracara)
                                   //(Unless one node of the Caracara has already been deployed to)
                                
                                //Set deploy status fields                                               
                                rootObject.children[ci].ites[0].deployStatus = this._determineDeployStatus(rootObject.children[ci].ites[0]);
                                rootObject.children[ci].ites[0].deployMsg = this._getDeployMsg(rootObject.children[ci].ites[0]);
                                rootObject.children[ci].ites[0].parentId = rootObject.id;                                                                                    

                                if (this.patternFormFactor == rootObject.children[ci].ites[0].formFactor && rootObject.children[ci].ites[0].subBays.length == 0) {
                                  //Check if server takes up more than one slot (multi slot node)
                                    var slots = rootObject.children[ci].ites[0].bays.length;
                                    if(slots > 1) {
                                        rootObject.children[ci].ites[0].bays[0] = rootObject.children[ci].ites[0].bays[0] + "-" + rootObject.children[ci].ites[0].bays[slots-1];                                                                                    
                                    }
                                    index = dojo.indexOf(this._selectedServers, rootObject.children[ci].ites[0].id);
                                    //Add node only if not already on selected side
                                    if (index == -1) {
                                       childrenArray.push(rootObject.children[ci].ites[0]);
                                    } else { //pull item out of selectedServers array to note that the status of whats left in the 
                                       //selectedServers array was not found in the newly created list so its status will change
                                       this._selectedServers.splice(index,1);
                                    }
                                }
                                
                                //If multi-slot server pattern selected then convert empty bays to multi-slot
                                if (rootObject.children[ci].ites[0].access == "empty" && rootObject.children[ci].ites[0].subBays.length == 0) {
                                    console.log("form factor " + this.patternFormFactor);
                                    switch (this.patternFormFactor) {
                                        case "4s":
                                        case "pxu":
                                        case "2sc":
                                            if (rootObject.children[ci].ites[0].bays[0] % 2) {  //Check all odd bays first (logic here is that if the odd bay is not 
                                                var oddBay = rootObject.children[ci].ites[0].bays[0]; //Get odd bay
                                                var evenBay = parseInt(oddBay) + 1; //Get even bay
                                                
                                                for (var c=0;c <rootObject.children.length;c++) { //Loop through all bays to find oddbay + 1
        
                                                    if (rootObject.children[c].ites.length == 1) {  //Only search compute node 1 empty bays (which should be all thats there empty)
                                                        if (rootObject.children[c].ites[0].bays[0] == evenBay) { //We found a match
                                                            if (rootObject.children[c].ites[0].access == "empty") { //If even bay after the odd bay is empty then add the odd bay to the list
                                                                rootObject.children[ci].ites[0].bays[0] = rootObject.children[ci].ites[0].bays[0] + "-" + evenBay;
                                                                rootObject.children[ci].ites[0].formFactor = this.patternFormFactor; // convert empty bay form factor value
        
                                                                index = dojo.indexOf(this._selectedServers, rootObject.children[ci].ites[0].id);
                                                                //Add node only if not already on selected side
                                                                if (index == -1) {
                                                                    childrenArray.push(rootObject.children[ci].ites[0]);                                      
                                                                } else { //pull item out of selectedServers array to note that the status of whats left in the 
                                                                    //selectedServers array was not found in the newly created list so its status will change
                                                                    this._selectedServers.splice(index,1);
                                                                }                                                                    
                                                            }                                                                                                                            
                                                        }                                                            
                                                    }
                                                   
                                                }
                                                
                                            }
                                            break;
                                        case "4sc":
                                            var bayToTest = parseInt(rootObject.children[ci].ites[0].bays[0]);
                                            if (bayToTest % 2 && bayToTest < 12) {  //Check all odd bays first (logic here is that if the odd bay is not 
                                                var lastBay = parseInt(bayToTest) + 3; //Get last bay number in possible complex
                                                var possibleComplex = 0;
                                                for (var c=0;c <rootObject.children.length;c++) { //Loop through all bays to find bays in complex
                                                    if(rootObject.children[c].ites.length == 1) {  //Only search compute node 1 empty bays (which should be all thats there empty)
                                                        if (parseInt(rootObject.children[c].ites[0].bays[0]) <= lastBay && 
                                                                parseInt(rootObject.children[c].ites[0].bays[0]) > bayToTest) { //check to see if bay falls in possible complex
                                                            if (rootObject.children[c].ites[0].access == "empty") { 
                                                                possibleComplex++;
                                                            }                                                                                                                            
                                                        }                                                            
                                                    }
                                                }
                                                if (possibleComplex == 3) { //If all bays in complex range are empty
                                                    rootObject.children[ci].ites[0].bays[0] = rootObject.children[ci].ites[0].bays[0] + "-" + lastBay;
                                                    rootObject.children[ci].ites[0].formFactor = this.patternFormFactor; // convert empty bay form factor value
                                                    index = dojo.indexOf(this._selectedServers, rootObject.children[ci].ites[0].id);
                                                    //Add node only if not already on selected side
                                                    if (index == -1) {
                                                        childrenArray.push(rootObject.children[ci].ites[0]);                                      
                                                    } else { //pull item out of selectedServers array to note that the status of whats left in the 
                                                        //selectedServers array was not found in the newly created list so its status will change
                                                        this._selectedServers.splice(index,1);
                                                    }
                                                }
                                            }
                                            break;
                                        case "8sc":
                                            var bayToTest = parseInt(rootObject.children[ci].ites[0].bays[0]);
                                            if (bayToTest % 2 && bayToTest < 8) {  //Check all odd bays first (logic here is that if the odd bay is not 
                                                var lastBay = parseInt(bayToTest) + 7; //Get last bay number in possible complex
                                                var possibleComplex = 0;
                                                for (var c=0;c <rootObject.children.length;c++) { //Loop through all bays to find bays in complex
                                                    if(rootObject.children[c].ites.length == 1) {  //Only search compute node 1 empty bays (which should be all thats there empty)
                                                        if (parseInt(rootObject.children[c].ites[0].bays[0]) <= lastBay && 
                                                                parseInt(rootObject.children[c].ites[0].bays[0]) > bayToTest) { //check to see if bay falls in possible complex
                                                            if (rootObject.children[c].ites[0].access == "empty") { 
                                                                possibleComplex++;
                                                            }                                                                                                                            
                                                        }                                                            
                                                    }
                                                }
                                                if (possibleComplex == 7) { //If all bays in complex range are empty
                                                    rootObject.children[ci].ites[0].bays[0] = rootObject.children[ci].ites[0].bays[0] + "-" + lastBay;
                                                    rootObject.children[ci].ites[0].formFactor = this.patternFormFactor; // convert empty bay form factor value
                                                    index = dojo.indexOf(this._selectedServers, rootObject.children[ci].ites[0].id);
                                                    //Add node only if not already on selected side
                                                    if (index == -1) {
                                                        childrenArray.push(rootObject.children[ci].ites[0]);                                      
                                                    } else { //pull item out of selectedServers array to note that the status of whats left in the 
                                                        //selectedServers array was not found in the newly created list so its status will change
                                                        this._selectedServers.splice(index,1);
                                                    }
                                                }
                                            }
                                            break;
                                        default:
                                            break;                                            
                                    }
                                }
                                
                                //If one bay server has subbays then we can say that it is a Caracara with only one compute node available 
                                //(meaning that the other compute node was already deployed to)
                                if (this.patternFormFactor == "1sd" && rootObject.children[ci].ites[0].subBays.length != 0) {
                                    rootObject.children[ci].ites[0].bays[0] = rootObject.children[ci].ites[0].bays[0] + ":" + rootObject.children[ci].ites[0].subBays[0];
                                    index = dojo.indexOf(this._selectedServers, rootObject.children[ci].ites[0].id);
                                    //Add node only if not already on selected side
                                    if (index == -1) {
                                        childrenArray.push(rootObject.children[ci].ites[0]); //Add Caracara with only one compute node available                                      
                                    } else { //pull item out of selectedServers array to note that the status of whats left in the 
                                        //selectedServers array was not found in the newly created list so its status will change
                                        this._selectedServers.splice(index,1);
                                    }                                                                    

                                } else if(this.patternFormFactor == "1sd" && rootObject.children[ci].ites[0].access == "empty") {
                                    //If 1sd pattern selected (Caracara) and bay empty then show empty bays in 1 bay 2 compute node format (caracara format)

                                    //Get bay information
                                    var location = rootObject.children[ci].ites[0].location;
                                    var id = rootObject.children[ci].ites[0].id; 
                                    var name = rootObject.children[ci].ites[0].name; 
                                    var bays = rootObject.children[ci].ites[0].bays[0]; 
                                    
                                    //Clone bay information
                                    var secondServer = dojo.clone(rootObject.children[ci].ites[0]); 
                                    
                                    //Create empty compute node 1
                                    rootObject.children[ci].ites[0].location = location + ":1";
                                    rootObject.children[ci].ites[0].id = id + ":1";
                                    rootObject.children[ci].ites[0].name = name + ":1";
                                    rootObject.children[ci].ites[0].formFactor = "1sd";
                                    rootObject.children[ci].ites[0].bays[0] = bays + ":1"; 
                                    rootObject.children[ci].ites[0].subBays[0] = "1";

                                    index = dojo.indexOf(this._selectedServers, rootObject.children[ci].ites[0].id);
                                    //Add node only if not already on selected side
                                    if (index == -1) {
                                        childrenArray.push(rootObject.children[ci].ites[0]);                                      
                                    } else { //pull item out of selectedServers array to note that the status of whats left in the 
                                        //selectedServers array was not found in the newly created list so its status will change
                                        this._selectedServers.splice(index,1);
                                    }                                                                    

                                    //Create empty compute node 2
                                    secondServer.location = location + ":2";
                                    secondServer.id = id + ":2";
                                    secondServer.name = name + ":2";
                                    secondServer.formFactor = "1sd";
                                    secondServer.bays[0] = bays + ":2";
                                    secondServer.subBays[0] = "2";

                                    index = dojo.indexOf(this._selectedServers, secondServer.id);
                                    //Add node only if not already on selected side
                                    if (index == -1) {
                                        childrenArray.push(secondServer);
                                    } else { //pull item out of selectedServers array to note that the status of whats left in the 
                                        //selectedServers array was not found in the newly created list so its status will change
                                        this._selectedServers.splice(index, 1);
                                    }

                                }

                            }
                        }
                        
                        rootObject.children = childrenArray; //Add new children array with only pattern specific targets
                    }
                                                    
                    if (rootObject.children.length > 0) {
                        rootArray.push(rootObject);  //Add Chassis to json
                    }
                    childrenArray = new Array(); //Reset Bay info for next chassis
                }
            }

            var patternData = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": rootArray
            }, true);
            
            this._availStore = new dojo.data.ItemFileWriteStore({
                data: dojo.fromJson(patternData)
            });
            this._availStore.comparatorMap = {};
            this._availStore.comparatorMap["name"] = function (a, b) {                        
                return a.localeCompare(b);
            };
        },

        _nameFormatter: function(item) {
            
            // summary:
            // Formats "name" field value for display.
            // tags:
            // protected
            if (!item.name) {
                return "";
            }

            var text = "";
            if (item.access == "empty") {
                text = this.locResources.EMPTY_BAY;
            } else {
                text = item.name;
            }
            return "<span title=\"" + text + "\">" + text + "</span>";

        },

        _bayFormatter: function(item) {

            // summary:
            // Formats "bays" field value for display.
            // tags:
            // protected
            if (!item.bays || (item.bays == "")) {
                return "";
            }

            var value = item.bays;
            var title = "";
            var dashIdx = value.indexOf("-");
            if (dashIdx > 0) {
                title = this.resources.BAY_NUMS.replace("%1", value.substring(0, dashIdx)).replace("%2", value.substring(dashIdx + 1));
            }
            else {
                title = this.resources.BAY_NUM.replace("%1", value);
            }
            return "<span title=\"" + title + "\">" + value + "</span>";

        },        

        // Columns to be displayed on table
        _getTableStructure: function(selected) {
            var cols = new Array();
            cols[0] = {
               field: "name",
               name: this.resources.NAME,
               sortable: false,
               width: "14em",
               formatter: dojo.hitch(this, "_nameFormatter")
            };
            cols[1] = {
                field: "bays",
                name: this.resources.BAY,
                sortable: false,
                width: "3em",
                formatter: dojo.hitch(this, "_bayFormatter")                
            };          
            cols[2] = {
                field: "deployStatus",
                name: this.resources.FW_DEPLOY_STATUS,
                sortable: false,
                width: (selected) ? "7em" : "auto",
                decorator: dojo.hitch(this, "_deployDecorator"),
                formatter: dojo.hitch(this, "_deployFormatter")
            };
            if (selected) {
                cols[3] = {
                    field: "schedule",
                    name: this.resources.ACTIVATION,
                    sortable: false,
                    width: "auto",
                    decorator: dojo.hitch(this, "_scheduleDecorator"),
                    formatter: dojo.hitch(this, "_scheduleFormatter")
                };
            }
            
            return cols;
        },
        
        _scheduleDecorator: function(item) {
            var scheduleStr;
           
            if (item == "") {
                return "";
            }
            
            if (item == this.locResources.FULL) {
                scheduleStr = "<span title=\"" + this.resources.IMMEDIATE_ACTIVATION + "\">" + this.locResources.FULL + "</span>";
            }
            else if (item == this.locResources.PARTIAL) {
                scheduleStr = "<span title=\"" + this.resources.DEFER_ACTIVATION + "\">" + this.locResources.PARTIAL + "</span>";
            }
            else {
                scheduleStr = this.resources.UNASSIGNED;
            }
            
            return scheduleStr;
        },
        
        _scheduleFormatter: function(item) {
        
        	if (!item.type || ((item.type != "bay") && (item.type != "imm"))) {
        		return "";
        	}
            
        	var schedule = this._schedule.get("value");
            var value = "";
            switch (schedule) {
                case "immediate" :
                    value = this.locResources.FULL;
                    break;
                case "defer" :
                    value = this.locResources.PARTIAL;
                    break;
                default :
                    value = this.resources.UNASSIGNED;
	                break;
            }
            return value;
            
        },

        // this function performs the selection manually on the items; it is
        // coded differently than FailoverMonitorTemplate.js (which instead
        // modifies the data store) because it is just easier
        _preselectTargetServer: function() {
            if (this.endpointUUID != null) {
                console.log(this.declaredClass + ": Preselecting endpoint with UUID: " + this.endpointUUID);
                var rowIDOfParent = null;
                var rowIDOfChild = null;
                // note that we are assuming that the order of these items is
                // consistent and chassis-first, child servers next, next
                // chassis, next chassis child servers, etc.; if this is not
                // the case, the preselection will fail
                var availObjs = this._availTable.store._arrayOfAllItems;
                for ( var i = 0; i < availObjs.length; i++) {
                    var availObj = availObjs[i];
                    if (availObj != null) {
                        var availEndpointUUID = availObj.id;
                        if (availEndpointUUID == this.endpointUUID) {
                            rowIDOfChild = i;
                            break;
                        }
                    }
                }
                console.log(this.declaredClass + ": Parent row: " + rowIDOfParent + " endpoint " + rowIDOfChild);

                // the reason why we do two different things here is that
                // the Deploy Server Template dialog can be called from
                // a server context (where we need to expand its parent
                // chassis in order to move the server(s) over) or a
                // chassis context (in which case there is no single
                // server to move)
                if (rowIDOfChild != null) {
                    // in order for the _moveRight() function to actually
                    // detect which servers were selected, we need to have
                    // the tree expanded
                    var id = this._availTable.store._arrayOfAllItems[rowIDOfChild].id;
                    this._availTable.expandAll();

                    this._availTable.select.row.selectById(id);
                    this._dualTableSelector._moveRight();

                    // collapse the tree after the preselection
                    this._availTable.collapseAll();
                }

            }
        },

        /** Function to set activation schedule of servers in selected table to whatever is chosen in the
        * schedule selector.  Note: at this point, all selected servers will have the same activation schedule
        */
        _setSelectedTableSchedules: function(value) {

            var selected = this._selectTable.store._arrayOfTopLevelItems;

            for (var si = 0; si < selected.length; ++si) {
                if (selected[si].children) {
                    for (var ci = 0; ci < selected[si].children.length; ++ci) {
                        console.log(this.declaredClass + ": ", selected[si].children[ci]);
                        selected[si].children[ci].schedule[0] = value;
                    }
                }
            }
            this._selectTable.model.clearCache();
            this._selectTable.body.refresh();

        },

        _getServerNamesFromGrid: function(servers) {
            var num = servers.length;
            var str = "";
            var i;
            if (num > 0) {
                var rows = servers;
                for (i = 0; i < num; i++) {
                    if (str != "") {
                        str += ", ";
                    }

                    str += rows[i].name;
                }
            }
            return str;
        },
        /**
         * Function to analyze deploy status of inputted array of items, and returns a status object containing whether or not
         * there should be warnings or errors. 
         * 
         * items: Array of selected items containing deployStatus fields
         * @returns Object with two fields: {error:Boolean, warning:Boolean}
         */
        _checkDeployStatus: function(items){
            var i,j;
            var returnObj={error:false,warning:false};
            for(i=0;i<items.length;i++){
                if(items[i].type == "chassis"){
                    if(items[i].children){
                        for(j=0;j<items[i].children.length;j++){
                            if(items[i].children[j].deployStatus){
                                if(items[i].children[j].deployStatus == this.DEPLOY_NOTREADY){
                                    returnObj.error = true;
                                }   
                                if(items[i].children[j].deployStatus == this.DEPLOY_UNKNOWN){
                                    returnObj.warning = true;
                                }
                            }
                        }
                    }
                }
                else {
                    if(items[i].deployStatus){
                        if(items[i].deployStatus == this.DEPLOY_NOTREADY){
                            returnObj.error = true;
                        }
                        if(items[i].deployStatus == this.DEPLOY_UNKNOWN){
                            returnObj.warning = true;
                        }
                    }
                }
            }
            return returnObj;
        },
        _checkDeployStatusBeforeDeploy: function(items){
            var i,j;
            var returnObj={error:false,warning:false,unsupported:false};
                                    
            for(i=0;i<items.length;i++){
                if(items[i].type == "chassis"){
                    if(items[i].children){
                        for(j=0;j<items[i].children.length;j++){
                            if(items[i].children[j].deployStatus){
                                if(items[i].children[j].deployStatus[0] == this.DEPLOY_NOTREADY){
                                    returnObj.error = true;
                                }   
                                if(items[i].children[j].deployStatus[0] == this.DEPLOY_UNKNOWN){
                                    returnObj.warning = true;
                                }
                                if(items[i].children[j].deployStatus[0] == this.DEPLOY_NOT_SUPPORTED){
                                    returnObj.unsupported = true;
                                }
                            }
                        }
                    }
                }
                else {
                    if(items[i].deployStatus){
                        if(items[i].deployStatus[0] == this.DEPLOY_NOTREADY){
                            returnObj.error = true;
                        }
                        if(items[i].deployStatus[0] == this.DEPLOY_UNKNOWN){
                            returnObj.warning = true;
                        }
                        if(items[i].deployStatus[0] == this.DEPLOY_NOT_SUPPORTED){
                            returnObj.unsupported = true;
                        }
                    }
                }
            }
            return returnObj;
        },
        _getBayDisplay: function(item){
            var bayDisplay = "";
            
            if(this.patternFormFactor == "pxu" || this.patternFormFactor == "4s" || this.patternFormFactor == "2sc"){
                bayDisplay = item.bays[0].split(",");
                return bayDisplay[0] + "-" + (parseInt(bayDisplay[0])+1);
            }else if(this.patternFormFactor == "2s"){
                bayDisplay = item.bays[0].split(",");
                return bayDisplay[0];
            }
            
            
        },
        
        /**
         * Function used to determine deploy status of servers based on FW levels and access states.
         * 
         * item: Singular object representing a server
         */
        _determineDeployStatus: function(item, /*String (optional)*/checkFormFactor) {
            
            if (checkFormFactor && item.formFactor && (checkFormFactor != item.formFactor.toString())) {
                return this.DEPLOY_NOT_SUPPORTED;
            }
            
            var fwLevel = (item.firmwareLevel) ? item.firmwareLevel.toString() : "";
            if (fwLevel == this.FW_UNKNOWN) {
                return this.DEPLOY_UNKNOWN;   
            }
            if (fwLevel == this.FW_BACKLEVEL) {
                return this.DEPLOY_NOTREADY;   
            }
            
            var access = (item.access) ? item.access.toString() : "";
            if (access == "offline" || access == "partial" || access == "unknown" || access == "-Default-" ) {
                return this.DEPLOY_UNKNOWN;
            }
            if (access == "none") {
                return this.DEPLOY_NOTREADY;
            }
           
            return this.DEPLOY_OK;
        },
        
        /**
         * Function to generate deploy message tooltip content based on firmware levels and access states
         * 
         * item: Singular object representing a server
         */
        _getDeployMsg: function(item){

            if(item.firmwareLevel){
                if(item.firmwareLevel == this.FW_UNKNOWN){
                    return this.resources.FW_UNKOWN_MSG;   
                }
                if(item.firmwareLevel == this.FW_BACKLEVEL){
                    return this.resources.FW_BACKLEVEL_MSG;  
                }
            }
            if(item.access){
                if(item.access == "offline"){
                    return this.resources.ACCESS_OFFLINE + " - " + this.resources.ACCESS_CIM_CONFIRM;
                } 
                else if(item.access == "partial"){
                    return this.resources.ACCESS_PARTIAL + " - " + this.resources.ACCESS_CIM_CONFIRM;
                }
                else if(item.access == "unknown"){
                    return this.resources.ACCESS_UNKNOWN + " - " + this.resources.ACCESS_CIM_CONFIRM;
                } 
                else if(item.access == "-Default-"){
                    return this.resources.ACCESS_DEFAULT + " - " + this.resources.ACCESS_CIM_CONFIRM;
                } 
                else if(item.access == "none"){
                    return this.resources.ACCESS_NONE + " - " + this.resources.ACCESS_CIM_CONFIRM;
                }
            }
           
            return this.resources.FW_COMPAT;
        },
        
        /**
         * Function to return a deploy status message based on the row number.
         * 
         * rowIndex: Int - Row index
         * leftTable: Boolean - True if available table(left).  False if selected table(right).
         * 
         */
        _getCellMsg: function(rowIndex, leftTable) {

            var items;
            if (leftTable) {
                items = this._availTable.store._arrayOfTopLevelItems;
            }
            else {
                items = this._selectTable.store._arrayOfTopLevelItems;
            }
            var loopVar=0;
            for (var ii=0; ii<items.length; ++ii) {
                if (items[ii].type == "chassis" && items[ii].children && items[ii].expandMe == "true") {
                    loopVar++;
                    for (var ci=0; ci<items[ii].children.length; ++ci) {
                        if (rowIndex == loopVar) {
                            if(items[ii].children[ci].deployMsg){
                                return items[ii].children[ci].deployMsg;
                            }
                        }
                        loopVar++;
                    }
                }
                else {
                    loopVar++;
                }
            }
            return "";
        }

    });

});
