define([ "dojo", "dijit", "support/dialogs/Dialog", "dojo/i18n!./nls/EditEthernetAddressPool", "dojo/text!./templates/EditEthernetAddressPool.html", "dijit/form/Select",
        "dijit/form/Button", "dijit/form/SimpleTextarea", "dijit/form/ValidationTextBox", "support/widgets/ButtonBar",
        "support/widgets/HoverButton", "dojox/grid/DataGrid", "dojo/data/ItemFileWriteStore", "support/dialogs/EthernetSubrangePopup",
        "dijit/layout/ContentPane", "dijit/Dialog", "dojox/grid/DataGrid",
        "dijit/ProgressBar" ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.EditEthernetAddressPool", [ support.dialogs.Dialog ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
             
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 
        
        widgetsInTemplate: true, // This widget's template must be parsed for widgets.
        
        templateId: null,
        baseClass: "EditEthernetAddressPool",

        name: null,
        description: null,
        version: "IPv6",
        _invalidMessage : "This value is invalid",
        _regExp: ".*",
        _placeHolder: null,
        gateway: null,
        network: null,
        dropDown: null,
        rangeTable: null,
        rangeLayout: null,
        rangeStore: null,
        overallTable: null,
        overallLayout: null,
        overallStore: null,
        
        preloadedRanges: null,
        
        vendorfirstAddress: "",
        vendorlastAddress: "",
        vendorprefixFirst: "",
        vendorprefixLast: "",
        vendorRangeSize: "",
        vendor: "IBM",
        
        firstNoChar:"",
        lastNoChar:"",
        
        rangeCheckerArray: new Array,

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/EditEthernetAddressPool.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template
        // HTML file
        postMixInProperties: function() {
            this.inherited(arguments);
            this._setvendor();
            this.loadprevSettings();
            this._connectDataModels();
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be
        // directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments); 
            
            this.rangeCheckerArray = new Array; // reset range checker array
         
            dojo.connect(this.dialogNode, "show", this, function() {

                var availHeight = window.screen.availHeight;
                var maxHeight = availHeight * this.maxPercent;
                this.settingsContainer.setAttribute("style", "max-height:" + maxHeight + "px");

                this.layout(); 

             });

            dojo.connect(this.dialogNode, "hide", this, function() {        
                this.destroy();
            });

            
            this.showoverallTable();
            this.showRangeTable();
            this._addSubrangeButton.set("style", "display:block");         
            
            if(this.preloadedRanges.items){
                this.setPrevData();
            }
        },
        
        _setvendor : function(){
            
            var vendorStore = null;
            
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "addresspool/vendordefaults/" + "ETHERNET/" + this.vendor,
                handleAs: "json",
                sync: true,
                preventCache: true,
                load: function(response) {
                    vendorStore = response;                    
                },
                handle: function(){


                }
            });
            
            this.vendorfirstAddress = vendorStore.firstAddress;
            this.vendorlastAddress = vendorStore.lastAddress;
            
            this.vendorprefixFirst = vendorStore.vendorprefixFirst.substr(0,12);
            this.vendorprefixLast = vendorStore.vendorprefixLast.substr(0,12);
            this.vendorRangeSize = vendorStore.vendorRangeSize;
            
        },
        
	    _connectDataModels : function(){
            //rangeTable
            this.rangeStore = new dojo.data.ItemFileWriteStore({
                data: {"items" : []}                    
            });
            if (this.rangeLayout == null) {
                this.rangeLayout = [
                    { name: this.locResources.RANGE, field: "range", width:"50%", editable:false },
                    { name: this.locResources.FIRST_MAC, field: "first", width:"70%", editable:false },
                    { name: this.locResources.SIZE, field: "size", width:"60%", editable:false },
                    { name: this.locResources.LAST_MAC, field: "last", width:"70%", editable:false },
                    { name: this.resources.ALLOCATED, field: "allocated",formatter: this.displayAllocation, width: "30%", editable:false },
                    { name: " ",  field: "icons", width: "20%", editable:false, formatter: dojo.hitch(this, "_formatIcons")}
                ];
            }

           //overallTable                        
              this.overallStore = new dojo.data.ItemFileWriteStore({
                  data: {"items" : [ 
                          { firstaddress: this.vendorfirstAddress, rangesize: this.vendorRangeSize, lastaddress: this.vendorlastAddress, allocated: "0::"+this.vendorRangeSize }
                          ]}                    
              });
              if (this.overallLayout == null) {
                  this.overallLayout = this.getOverallLayout();
              }
        },
        
        cancelClicked: function() {      
            this.hide();
        },
        
        okClicked: function() {
            
            var grid = this.rangeTable;
            var store = grid.store;
            var topLevelItems = store._arrayOfTopLevelItems;
            var loopcount;
            
            var items = new Array;
            
            for (loopcount = 0; loopcount < topLevelItems.length; loopcount++) {
                var rangeObject = new Object();
                rangeObject.name = topLevelItems[loopcount].range[0];
                rangeObject.firstAddress = topLevelItems[loopcount].first[0];
                rangeObject.poolRangeSize = topLevelItems[loopcount].size[0];
                rangeObject.lastAddress = topLevelItems[loopcount].last[0];
                rangeObject.allocated = topLevelItems[loopcount].allocated[0];
                items.push(rangeObject);
            }
           
            var argJSONString = dojo.toJson({
                "vendorName":this.vendor,
                "identifier":this.vendor,
                  "vendorRangeSize":this.vendorRangeSize,
                  "items": items
            }, true);
            
//            alert(argJSONString);
            
            var xhrArgs = {
                    url:  this.REST_PATH + "addresspool/setpoolranges/" + "ETHERNET/" + this.vendor,
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    postData: argJSONString,
                    load : function(data) {
//                        alert(data);
                    },
                    error : function(error) {
                        alert("Post failed: " + error);
                    }
                };
                
            dojo.xhrPost(xhrArgs);
                        
            this.hide();

        },

        loadprevSettings: function() {  
            
            var preloadedRanges = "";
            
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "addresspool/getpoolranges/" + "ETHERNET/" + this.vendor,
                handleAs: "json",
                sync: true,
                preventCache: true,
                load: function(response) {
                    preloadedRanges = response;                    
                },
                handle: function(){

                }
            });
 
            this.preloadedRanges = preloadedRanges;
           
        },       
        
        setPrevData: function(){
            var pool = this;
            var i;
            
            var grid = pool.rangeTable;
            var store = grid.store;
            
            var firstAdd = "";
            var lastAdd = "";
            
            for (i = 0; i < this.preloadedRanges.items.length; i++){
                var range = store.newItem(
                        {range: this.preloadedRanges.items[i].range, 
                         first: this.preloadedRanges.items[i].first,
                         size: this.preloadedRanges.items[i].size,
                         last: this.preloadedRanges.items[i].last,
                         allocated: this.preloadedRanges.items[i].allocated
                        }
                );
                
                firstAdd = this.preloadedRanges.items[i].first;
                
                firstAdd = firstAdd.replace(/:/g, "");
                firstAdd = firstAdd.substr(-4);
                
                this.rangeCheckerArray.push(parseInt(firstAdd,16));
                
                lastAdd = this.preloadedRanges.items[i].last;
                
                lastAdd = lastAdd.replace(/:/g, "");
                lastAdd = lastAdd.substr(-4);
                
                this.rangeCheckerArray.push(parseInt(lastAdd,16));
                
            }
            
            store.save();
            grid.setStore(store);
            
            this._enableOk();
            this._setRangeIcons();
            
            this.layout();
        },
 
        showoverallTable : function(){
            if(this.overallTable == null){
                this.overallTable = new dojox.grid.DataGrid({
                    store: this.overallStore,
                    structure: this.getOverallLayout(),
                    autoWidth: false,
                    autoHeight: true,
                    singleClickEdit: true,
                    columnReordering: false,
                    style: "width: 100%; height: 50px;",
                    noDataMessage: this.locResources.NO_DATA
                }).placeAt(this._overallTableDiv);
                this.overallTable.startup();
                this.overallTable.canSort = function(col){ return false; };
                this.overallTable.update();
            }
            
            this.layout(); 

        },
        
        getOverallLayout : function(){
                return [
                        {name: this.locResources.FIRST_MAC, field: "firstaddress", width:"50%", editable:false },
                        {name: this.locResources.SIZE, field: "rangesize", width:"40%", editable:false },
                        {name: this.locResources.LAST_MAC, field: "lastaddress", width: "50%", editable:false },
                        {name: this.resources.ALLOCATED, field: "allocated",formatter: this.displayAllocation, width: "20%", editable:false }
                    ];
      },
      
      displayAllocation : function(value) {
          if (!value) {
              return "";
          }
          var strVal = new String(value);
          var sepIdx = strVal.indexOf("::");
          if (sepIdx > 0) {
              var alloc = strVal.substring(0, sepIdx);
              var total = strVal.substring(sepIdx + 2);
              var props = {
                  value: alloc / total * 100,
                  style: "width: 100%;"
                  //,title: alloc + " of " + total + " addresses allocated."
              };
              var progressBar = new dijit.ProgressBar(props);
              progressBar._destroyOnRemove = true;
              return progressBar;
          }
          return value;
      },
      
      displayExpand : function(value) {
          return "<button class=\"expandicon\"></button>";
      },
      
      showRangeTable : function(){
          if(this.rangeTable == null){
              this.rangeTable = new dojox.grid.DataGrid({
                  store: this.rangeStore,
                  structure: this.rangeLayout,
                  autoWidth: false,
                  autoHeight: 6,
                  singleClickEdit: true,
                  columnReordering: false,
                  style: "width: 100%;",
                  noDataMessage: this.locResources.NONE
              }).placeAt(this._subrangeTableDiv);
              this.rangeTable.startup();
              this.rangeTable.canSort = function(col){ return false; };
              this.rangeTable.update();
          }
          
          var availHeight = window.screen.availHeight;
          var maxHeight = availHeight * 0.25;
          var maxRows = maxHeight/35;
          this.rangeTable.set("autoHeight", maxRows);
          
          this.layout(); 

      },
      
      addRange : function(){
          var pool = this;
          var popup = new support.dialogs.EthernetSubrangePopup({
              mode: "add",
              newPool: pool,
              subrangeStore: this.rangeStore,
              overallStore: this.overallStore,
              dropDown: this.dropDown,
              version: this.version,
              network: this.network,
              gateway: this.gateway,
              vendorprefixFirst: this.vendorprefixFirst,
              vendorprefixLast: this.vendorprefixLast,
              vendorRangeSize: this. vendorRangeSize,
              rangeCheckerArray: this.rangeCheckerArray,
              addNewSubrange : function(){
                  var grid = pool.rangeTable;
                  var store = grid.store;
                  
                  var firstCombine = this.vendorprefixFirst+this.first;
                  var lastCombine = this.last;

                  var firstNoChar = this.first.replace(/:/g, "");
                  var lastNoChar = lastCombine.replace(/:/g, "");
                  lastNoChar = lastNoChar.substr(-4);
                  
                  firstNoChar = parseInt(firstNoChar,16);
                  lastNoChar = parseInt(lastNoChar,16);
                  
                  this.rangeCheckerArray.push(firstNoChar);
                  this.rangeCheckerArray.push(lastNoChar);
                  
                  store.newItem({range:this.range, first:firstCombine, size:this.size, last:this.last, allocated:this.allocated, remove:true });
                  store.save();
                  grid.setStore(store);
                  pool._enableOk();
                  pool._setRangeIcons();

                  this.layout(); 

              },
              _destroyOnRemove: true
          });  
          popup.show();
      },
      
      //Removes a row from the subrange table
      removeRow : function(row){
          var grid = this.rangeTable;
          var store = grid.store;
          var topLevelItems = store._arrayOfTopLevelItems;
          var item = topLevelItems[row];
          var range = store.getValue(item, "range");
          var remove = confirm(this.locResources.DELETE.replace("%1",range));
          if(remove){
              grid.store.deleteItem(item);
              grid.setItems(store._arrayOfTopLevelItems);
              
              var firstNoChar = item.first[0].replace(/:/g, "");
              var lastNoChar = item.last[0].replace(/:/g, "");
              
              firstNoChar = firstNoChar.substr(-4);
              lastNoChar = lastNoChar.substr(-4);              
              
              firstNoChar = parseInt(firstNoChar, 16);
              lastNoChar = parseInt(lastNoChar, 16);
                            
              var rangeCheckerCount = this.rangeCheckerArray.length;   //Remove Deleted Range from rangeCheckArray
              var counter;
              var counterPlusOne;
              
              for(counter = 0;counter < rangeCheckerCount;counter=counter+2){

                  counterPlusOne = counter + 1;

                  if(this.rangeCheckerArray[counter] == firstNoChar && this.rangeCheckerArray[counterPlusOne] == lastNoChar){                                            

                      this.rangeCheckerArray.splice(counter,2);
                      
                  }
                  
              }

              this._setRangeIcons();
             // this._enableOk();
              
              this.layout(); 

          } 
      },
      
      //Format for the "remove" column, icon
      _formatIcons : function(value, rowIndex, cell){            
         return "<div id=\"edit_" + rowIndex + "\" class=\"spEditLgIcon pointer\"" + //edit
              "onClick=\"dijit.byId('" + this.id + "').editRow(\"" + rowIndex + "\")\"" +
              "title=\"" + this.locResources.EDIT + "\" style=\"display:inline-block;\"></div>" +
              "<div style=\"display:inline-block; width:5px\"></div>" + //spacing
              "<div id=\"remove_" + rowIndex + "\" class=\"spDeleteIcon pointer\"" + //remove
              "onClick=\"dijit.byId('" + this.id + "').removeRow(\"" + rowIndex + "\")\"" +
              "title=\"" + this.locResources.REMOVE + "\" style=\"display:inline-block;\">";          
      },
      
      editRow : function(row){
          var pool = this;
          var store = this.rangeTable.store;
          var grid = this.rangeTable;
          var item = grid.getItem(row);
          var rangeCheckerArray = this.rangeCheckerArray;
                    
          var firstUncombined = item.first[0].replace(this.vendorprefixFirst, "");
          
          var rangeCheckerTempArray = dojo.clone(this.rangeCheckerArray);
          
          var firstNoChar = item.first[0].replace(/:/g, "");
          var lastNoChar = item.last[0].replace(/:/g, "");
          
          firstNoChar = firstNoChar.substr(-4);
          lastNoChar = lastNoChar.substr(-4);
          
          firstNoChar = parseInt(firstNoChar, 16);
          lastNoChar = parseInt(lastNoChar, 16);
          
          var firstNoCharPrev = firstNoChar;
          var lastNoCharPrev = lastNoChar;
                  
          var rangeCheckerCount = rangeCheckerTempArray.length;   //Remove Deleted Range from rangeCheckArray
          var counter;
          var counterPlusOne;

          for(counter = 0;counter < rangeCheckerCount;counter=counter+2){

              counterPlusOne = counter + 1;

              if(rangeCheckerTempArray[counter] == firstNoChar && rangeCheckerTempArray[counterPlusOne] == lastNoChar){                                            
                  rangeCheckerTempArray.splice(counter,2);
              }
              
          }
          
          var popup = new support.dialogs.EthernetSubrangePopup({
              mode: "Edit",
              newPool: pool,
              network: this.network,
              gateway: this.gateway,
              dropDown: this.dropDown,
              range: item.range[0],
              first: firstUncombined,
              size: item.size[0],
              index: row,
              subrangeStore: this.rangeStore,
              overallStore: this.overallStore,
              version: this.version,
              vendorprefixFirst: this.vendorprefixFirst,
              rangeCheckerArray: rangeCheckerTempArray,
              editSubrange : function(){
                  var firstCombine = this.vendorprefixFirst+this.first;
                  
                  store.setValue(item, "range", this.range);
                  store.setValue(item, "first", firstCombine);
                  store.setValue(item, "size", this.size);
                  store.setValue(item, "last", this.last);
                  store.setValue(item, "allocated", this.allocated);
          
                //Remove old Edited Range from rangeCheckArray if different                                    
                  var rangeCheckerCount = rangeCheckerArray.length;
                  var counter;
                  var counterPlusOne;
                  
                  for(counter = 0;counter < rangeCheckerCount;counter=counter+2){

                      counterPlusOne = counter + 1;

                      if(rangeCheckerArray[counter] == firstNoCharPrev && rangeCheckerArray[counterPlusOne] == lastNoCharPrev){                                            
                          rangeCheckerArray.splice(counter,2);
                      }
                      
                  }
                  
                  var firstNoChar = firstCombine;
                  var lastNoChar = this.last;
                  
                  firstNoChar = firstNoChar.replace(/:/g, "");
                  lastNoChar = lastNoChar.replace(/:/g, "");
                  
                  firstNoChar = firstNoChar.substr(-4);
                  lastNoChar = lastNoChar.substr(-4);
                  
                  firstNoChar = parseInt(firstNoChar, 16);
                  lastNoChar = parseInt(lastNoChar, 16);
                  
                  pool.rangeCheckerArray.push(firstNoChar);
                  pool.rangeCheckerArray.push(lastNoChar);
                  
                  store.save();
                  grid.setStore(store);
                  
                  pool._setRangeIcons();
                  
                  this.layout(); 

              },
              _destroyOnRemove: true
          });  
          popup.show(); 
      },
      
      _enableOk : function(){
          var hasItem = false;
          if(this.rangeTable != null){
                  var store = this.rangeTable.store;
                  var items = store._arrayOfTopLevelItems;
                  hasItem = (items.length > 0);
          }
          if(hasItem)
              this.okBtn.set("disabled", false);
          else
              this.okBtn.set("disabled", true);
      },
      
      checkForNetworkID : function(){
          var network = this.calculateNetworkID();
          this.set("network", network);
          this.updateOverallTable();
      },
      
      //Sets certain icons as enabled/disabled
      _setRangeIcons : function(){
          var min = 1;
          var grid = this.rangeTable;
          var store = grid.store;
          var topLevelItems = store._arrayOfTopLevelItems;
          for(var i=0; i<topLevelItems.length; i++){
              var item = topLevelItems[i];
              if(topLevelItems.length <= min) //disable remove
                  store.setValue(item, "remove", false);
              else //enable remove
                  store.setValue(item, "remove", true);
          }
          grid.setItems(topLevelItems);
      },      
      updateOverallTable : function(){
          var pool = this;
          //needs network, broadcast, first, host, highest
          var grid = this.overallTable;
          if(grid != null){
              var store = grid.store;
              var gotItems = function(items){ 
                  //pool.networkTable.setItems(items);
                  var item = items[0];
                  if(pool.version == "IPv4"){
                      var network = pool.calculateNetworkID();
                      var broadcast = pool.calculateBroadcastAddress(network);
                      var lowest = pool.calculateLowestAddress(network);
                      var host = pool.calculateHostAddress();
                      var highest = pool.calculateHighestAddress(network, broadcast, lowest, host);
                      store.setValue(item, "network", network);
                      store.setValue(item, "broadcast", broadcast);
                      store.setValue(item, "lowest", lowest);
                      store.setValue(item, "host", host);
                      store.setValue(item, "highest", highest);
                      grid.update();
                  }
              };
              this.layout(); 

              store.fetch({onComplete:gotItems}); 
          }
      }        
        
    });
 
});