define([ 
         "dojo", 
         "dijit", 
         "support/dialogs/Dialog", 
         "dojo/i18n!./nls/EditVirtualAddressing",
         "dojo/text!./templates/EditVirtualAddressing.html", 
         "support/widgets/Select",
         "dijit/Tooltip", 
         "dijit/form/Button",
         "dojox/data/AndOrReadStore", 
         "support/widgets/ToggleButton",
         "dojo/data/ItemFileWriteStore" 
         ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.EditVirtualAddressing", [ support.dialogs.Dialog ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,
        
        serverPatternInUse: false,
        ethernetPoolId: null,
        ethernetPoolSubrangeId: null,
        fibreChannelPoolId: null,
        fibreChannelPoolSubrangeId: null,

        _ethernetAddressPoolStore: null,
        _fibreChannelAddressPoolStore: null,
        _fibreChannelAddressPoolSelector: null,
        _addressSubrangeSelector: null,
        _addressSubrangeStore: null,
        _ethernetPoolId: null,
        _macAddressPoolRangeData: null,
        _wwnAddressPoolRangeData: null,

        baseClass: "EditVirtualAddressing",

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/EditVirtualAddressing.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            
            dojo.connect(this.dialogNode, "hide", this, function() {        
                this.destroy();
            });

            this._connectDataModels();
            this._createSelectBoxes();
            this._applyUserSettings();
            
            // Apply env specific tooltip for subrange selector
            // NOTE: For dijit/Tooltip dijits, set the label, not innerHTML.
            this._editvirtualtip.label = "<div style=\"width: 30em;\">" + this.locResources.SUBRANGE_TOOLTIP.replace("%1", this.PRODUCT_NAME_LONG).replace("%2", this.PRODUCT_NAME_SHORT) + "</div>";
            
            dojo.connect(this._ethVirtAddrToggleButton, "onMouseOut", this, function() {
                dijit.hideTooltip(this._ethVirtAddrToggleButton.domNode);
            });
            dojo.connect(this._fcVirtAddrToggleButton, "onMouseOut", this, function() {
                dijit.hideTooltip(this._fcVirtAddrToggleButton.domNode);
            });
        },

        _connectDataModels: function() {
            /**************************/
            /* Ethernet address pools */
            /**************************/
            var ethpooldata = null;
            var xhrEthPoolArgs = {
                    url: this.REST_PATH + "addresspool/vendordefaults/Ethernet/IBM",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        ethpooldata = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                };

            // Call the asynchronous xhrGet
            var deferred = dojo.xhrGet(xhrEthPoolArgs);
            
            this._macpoolname.innerHTML = ethpooldata.name;
            this._ethernetPoolId = ethpooldata.id;
            /*************************************/
            /* End Ethernet address pool section */
            /*************************************/

            /*******************************/
            /* Fibre Channel address pools */
            /*******************************/
            // Set up FC Address Pool selector. One store for "virtual" addressing, one for "burned" addressing.
            this._fibreChannelAddressPoolStore = new dojo.data.ItemFileReadStore({
                url: this.REST_PATH + "addresspool/listfcvendors"
            });
            this._fibreChannelAddressPoolStore.fetch({
                query: {},
                start: 0,
                count: 0,
                node: this._fcpoolselector
            });
            /******************************************/
            /* End Fibre Channel address pool section */
            /******************************************/
            
            /*********************/
            /* Address subranges */
            /*********************/
            var addresssubrangedata = {
                    "identifier": "id",
                    "label": "range",
                    "items": [
                              { "id": "1", "range":"1", "value":"1"},
                              { "id": "2", "range":"2", "value":"2"},
                              { "id": "3", "range":"3", "value":"3"},
                              { "id": "4", "range":"4", "value":"4"},
                              { "id": "5", "range":"5", "value":"5"},
                              { "id": "6", "range":"6", "value":"6"},
                              { "id": "7", "range":"7", "value":"7"},
                              { "id": "8", "range":"8", "value":"8"},
                              { "id": "9", "range":"9", "value":"9"}
                    ]
            };
            
            dojo.forEach(addresssubrangedata.items, dojo.hitch(this, function(subrange, subrangeIdx) {
                subrange.range = this.locResources.PREDEFINED_LABEL.replace("%1", subrange.value);
            }));
            
            this._addressSubrangeStore = new dojo.data.ItemFileReadStore({
                data: addresssubrangedata
            });            
            
            var macaddresssubrangedata = null;
            var xhrEthRangeArgs = {
                    url: this.REST_PATH + "addresspool/getpoolranges/ETHERNET/IBM",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        macaddresssubrangedata = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                };

            // Call the asynchronous xhrGet
            dojo.xhrGet(xhrEthRangeArgs);
                        
            this._macAddressPoolRangeData = macaddresssubrangedata;
            
            var wwnaddresssubrangedata = null;
            var xhrFibreRangeArgs = {
                    url: this.REST_PATH + "addresspool/getpoolranges/FIBRE/IBM",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        wwnaddresssubrangedata = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                };

            // Call the asynchronous xhrGet
            dojo.xhrGet(xhrFibreRangeArgs);
            
            this._wwnAddressPoolRangeData = wwnaddresssubrangedata;
        },

        _createSelectBoxes: function() {
            this._fibreChannelAddressPoolSelector = new support.widgets.Select({
                id: this.get("id") + "_fibreChannelAddressPoolSelector",
                name: "fibreChannelAddressPool",
                store: this._fibreChannelAddressPoolStore,
                maxHeight: -1,
                style: {
                    width: "200px"
                },
                disabled: false
            }).placeAt(this._fcpoolselector);
            this._fibreChannelAddressPoolSelector.startup();
            dojo.connect(this._fibreChannelAddressPoolSelector, "onChange", this, function(newValue) {                
                var wwnaddresssubrangedata = null;
                var xhrFibreRangeArgs = {
                        url: this.REST_PATH + "addresspool/getpoolranges/FIBRE/" + newValue,
                        handleAs: "json",
                        sync: true,
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        load: function(data) {                            
                            wwnaddresssubrangedata = data;                            
                        },
                        error: function(error) {
                            alert(error);                            
                        }
                    };

                // Call the asynchronous xhrGet
                dojo.xhrGet(xhrFibreRangeArgs);                
                
                this._wwnAddressPoolRangeData = wwnaddresssubrangedata;
                this._updateAddressesRemaining();
            });            
            this._fibreChannelAddressPoolSelector.set("value", "IBM");
            
            this._addressSubrangeSelector = new support.widgets.Select({
                id: this.get("id") + "_fibreChannelAddressPoolRangeSelector",
                name: "fibreChannelAddressPoolRange",
                store: this._addressSubrangeStore,
                maxHeight: -1,
                style: {
                    width: "200px"
                },
                disabled: false
            }).placeAt(this._subrangeSelector);
            dojo.connect(this._addressSubrangeSelector, "onChange", this, function() {
                this._updateAddressesRemaining();                
            });            
            this._addressSubrangeSelector.startup();
        },
        
        _applyUserSettings: function() {
            if (this.ethernetPoolId != null) {
                // If the id is *NONE, disable ethernet virtual addressing
                if (this.ethernetPoolId == "*NONE") {
                    this._ethVirtAddrToggleButton.set("enabled", false);
                    
                    // Setting enabled does not fire toggleClicked so set display here
                    this._ethpoolsection.style.display = "none";
                    this._remainingEthRow.style.display = "none";
                } else {
                    this._ethVirtAddrToggleButton.set("enabled", true);
                }
            }
            if (this.ethernetPoolSubrangeId != null) {
                if (this.ethernetPoolSubrangeId != "*NONE") {
                    this._addressSubrangeSelector.set("value", this.ethernetPoolSubrangeId);
                }
            }
            if (this.fibreChannelPoolId != null) {
                // If the id is *NONE, disable fibre channel virtual addressing                
                if (this.fibreChannelPoolId == "*NONE") {
                    this._fcVirtAddrToggleButton.set("enabled", false);
                    
                    // Setting enabled does not fire toggleClicked so set display here
                    this._fcpoolsection.style.display = "none";
                    this._remainingFcRow.style.display = "none";
                } else {
                    this._fcVirtAddrToggleButton.set("enabled", true);
                    this._fibreChannelAddressPoolSelector.set("value", this.fibreChannelPoolId);
                }
            }
            if (this.fibreChannelPoolSubrangeId != null) {
                if (this.fibreChannelPoolSubrangeId != "*NONE") {
                    this._addressSubrangeSelector.set("value", this.fibreChannelPoolSubrangeId);
                }
            }
            
            this._updateAddressesRemaining();
            
            if (this.serverPatternInUse) {
                this._fibreChannelAddressPoolSelector.set("disabled", true);
                this._addressSubrangeSelector.set("disabled", true);
                this.saveButton.set("disabled", true);
            }            
        },
        
        _getAllocated: function(poolType, domainId) {
            if (poolType == "mac") {            
                var macallocated = null;                
                dojo.forEach(this._macAddressPoolRangeData.items[0].domains.items, dojo.hitch(this, function(subrange, subrangeIdx) {                    
                    if (subrange.domain == domainId) {
                        macallocated = subrange.allocated.split("::");
                    }
                }));
                return macallocated;                
            }
            
            if (poolType == "wwn") {            
                var wwnallocated = null;                
                dojo.forEach(this._wwnAddressPoolRangeData.items[0].domains.items, dojo.hitch(this, function(subrange, subrangeIdx) {
                    if (subrange.domain == domainId) {
                        wwnallocated = subrange.allocated.split("::");                        
                    }
                }));
                return wwnallocated;                
            }
        },        
        
        _updateAddressesRemaining: function() {
            var subrangeSelected = this._addressSubrangeSelector.get("value");
            var macallocated = this._getAllocated("mac", subrangeSelected);
            var macAddressesRemaining = 0;
            var macPercentUsed = 0;            
            if (macallocated != undefined) {
                macAddressesRemaining = macallocated[1] - macallocated[0];
                macPercentUsed = 100 - (macallocated[0]/macallocated[1]*100);
                if (macallocated[0] > 0 && macPercentUsed > 99) {
                    macPercentUsed = 99;
                }
            }
            this._remainingEthernet.innerHTML = macPercentUsed.toFixed(0) + "% (" + this.locResources.ADDRESSES_REMAINING.replace("%1", macAddressesRemaining).replace("%2", macallocated[1]) + ")";
            
            var wwnallocated = this._getAllocated("wwn", subrangeSelected);
            var wwnAddressesRemaining = 0;
            var wwnPercentUsed = 0;
            if (wwnallocated != undefined) {
                wwnAddressesRemaining = wwnallocated[1] - wwnallocated[0];
                wwnPercentUsed = 100 - (wwnallocated[0]/wwnallocated[1]*100);
                if (wwnallocated[0] > 0 && wwnPercentUsed > 99) {
                    wwnPercentUsed = 99;
                }              
            }
            this._remainingFibre.innerHTML = wwnPercentUsed.toFixed(0) + "% (" + this.locResources.ADDRESSES_REMAINING.replace("%1", wwnAddressesRemaining).replace("%2", wwnallocated[1]) + ")";            
        },

        _saveandhideDialog: function() {

            var virtualAddressingObj = new Object();

            if (this._ethVirtAddrToggleButton.get("enabled")) {
                virtualAddressingObj.ethAddressPool = this._ethernetPoolId;
                virtualAddressingObj.ethAddressDomain = this._addressSubrangeSelector.get("value");
            } else {
                virtualAddressingObj.ethAddressPool = "*NONE";
                virtualAddressingObj.ethAddressDomain = "*NONE";
            }
            
            if (this._fcVirtAddrToggleButton.get("enabled")) {
                virtualAddressingObj.fcAddressPool = this._fibreChannelAddressPoolSelector.get("value");
                virtualAddressingObj.fcAddressDomain = this._addressSubrangeSelector.get("value");
            } else {
                virtualAddressingObj.fcAddressPool = "*NONE";
                virtualAddressingObj.fcAddressDomain = "*NONE";
            }
            
            console.log(this.declaredClass + ": ", dojo.toJson(virtualAddressingObj));
            
            dojo.publish("EditVirtualAddressingComplete", virtualAddressingObj);
            
            this.hide();
        },

        _cancelDialog: function() {
            this.hide();
        },
        
        _ethToggleClicked: function() {
            // Both mac and wwn addressing cannot both be disabled
            if (!this._fcVirtAddrToggleButton.get("enabled")) {
                if (!this._ethVirtAddrToggleButton.get("enabled")) {
                    this._ethVirtAddrToggleButton.set("enabled", true);
                    dijit.showTooltip(this.locResources.ONE_MUST_BE_ENABLED, this._ethVirtAddrToggleButton.domNode);
                }
            }
            
            if (this._ethVirtAddrToggleButton.get("enabled")) {
                this._ethpoolsection.style.display = "block";
                this._remainingEthRow.style.display = "table-row";
            } else {
                this._ethpoolsection.style.display = "none";
                this._remainingEthRow.style.display = "none";
            }
        },
        
        _fcToggleClicked: function() {
            // Both mac and wwn addressing cannot both be disabled
            if (!this._ethVirtAddrToggleButton.get("enabled")) {
                if (!this._fcVirtAddrToggleButton.get("enabled")) {
                    this._fcVirtAddrToggleButton.set("enabled", true);
                    dijit.showTooltip(this.locResources.ONE_MUST_BE_ENABLED, this._fcVirtAddrToggleButton.domNode);
                }
            }
            
            if (this._fcVirtAddrToggleButton.get("enabled")) {
                this._fcpoolsection.style.display = "block";
                this._remainingFcRow.style.display = "table-row";
            } else {
                this._fcpoolsection.style.display = "none";
                this._remainingFcRow.style.display = "none";
            }
        }
    });
});
