define("support/dialogs/EthernetSubrangePopup", [ "dojo", "dijit",  
        "support/dialogs/Dialog", 
        "dojo/i18n!./nls/EthernetSubrangePopup", 
        "dojo/text!./templates/EthernetSubrangePopup.html",
        "dojo/html",
        "dijit/form/Button",  
        "dijit/form/ValidationTextBox", 
        "dijit/form/RadioButton", 
        "dijit/form/FilteringSelect"],
        function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.EthernetSubrangePopup", [ support.dialogs.Dialog ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,
        
        widgetsInTemplate: true,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass : "EthernetSubrangePopup",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/EthernetSubrangePopup.css",
        
        //mode: "add" or "edit"
        mode: "add",
        
        //from NewIPPool dialog
        network: null,
        gateway: null,
        version: null,      //valid options: IPv4 | IPv6
        dropDown: null,     //subnet mask or routing prefix length
        _invalidMessage: "",
        _regExp: ".*",
        _placeHolder: "00:00",
        vendorprefixFirst: "",
        vendorprefixLast: "",
        vendorRangeSize: "",
        vendor: "",
        
        newPool: null,
        subrangeStore: null,
        overallStore: null,
        
        //initial properties
        range: null,
        first: null,
        size: null,
        last: null,
        index: null,        //if in edit mode
        
        type: null,         //valid options: single | custom 
        
        _rangeDropDown: null,
        _rangeTextBox: null,
        _rangeFirst: null,
        _rangeSize: null,
        _rangeLast: null,
        
        overlappingSubrange: false,

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in
        // the template HTML file
        postMixInProperties: function() {

            this.inherited(arguments);

        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be
        // directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            var popup = this;
                        
            //range
            this._rangeDropDown = new dijit.form.FilteringSelect({
                required: false,                
                style: "width:150px",
                store: new dojo.store.Memory({ data: this._getAvailableRangeNameOptions() }),
                labelAttr: "name",
                searchAttr: "name",
                onChange: function(newValue){
                    popup.set("range", newValue);
                },
                _destroyOnRemove: true
            }).placeAt(this._rangeDropDownDiv);
            
            this._rangeTextBox = new dijit.form.ValidationTextBox({
                required: true,
                trim: true,
                value: popup.range,
                placeHolder: popup.locResources.RANGE_NAME,
                style: "width:150px",
                intermediateChanges: true,
                onChange: function(newValue){
                    popup.set("range", newValue);
                },
                validator : function(value, constraints){
                    var usedNames = popup._getUsedRangeNames();
                    if(popup.containsWithIndex(value, usedNames, popup.index)){
                        this.invalidMessage = popup.locResources.INVALID_MESSAGE_UNIQUE_NAME + 
                            "<br/>" + popup.locResources["INVALID_MESSAGE_NAME_IN_USE"].replace("%1", value);
                        return false;
                    }
                    return true;
                }    
            }).placeAt(this._rangeTextBoxDiv);
            
            this._rangeFirstRegExp = "^([0-9A-Fa-f][0-9A-Fa-f]:[0-9A-Fa-f][0-9A-Fa-f])$";
            var rangeFirstRegExp = this._rangeFirstRegExp;
            
            //first
            this._rangeFirst = new dijit.form.ValidationTextBox({
                required: true,
                trim: true,
                value: this.first,
                placeHolder: this._placeHolder,
                invalidMessage: this._invalidMessage,
                style: "width: 65px",
                regExp: this._rangeFirstRegExp,
                intermediateChanges: true,
                onChange: function(newValue){
                    popup.set("first", newValue);
                },
                validator : function(value, constraints){
                    var checkExp = rangeFirstRegExp;
//                  console.log(this.declaredClass + ": ", rangeFirstRegExp);                            
                    // console.log(this.declaredClass + ": ", value.search(checkExp));
                    
                    console.log(this.declaredClass + ": overlapping: " + popup.overlappingSubrange);
              
                    if(value.search(checkExp) == -1){
                        this.invalidMessage = popup.locResources.INVALID_MESSAGE_FORM;
                        return false;
                    }
                    else if(popup.overlappingSubrange == true){
                        this.invalidMessage = popup.locResources.INVALID_MESSAGE_OVERLAPPING;
                        console.log(this.declaredClass + ": overlap");
                        return false;
                    }
                    return true;
                }
            }).placeAt(this._firstDiv);
            
            dojo.html.set(this._firstDivprefix, this.vendorprefixFirst);
            
            //size
            this._rangeSize = new dijit.form.ValidationTextBox({
                trim: true,
                required: true,
                value: this.size,
                placeHolder: "0",
                invalidMessage: popup.locResources.INVALID_MESSAGE_SIZE.replace("%1", "128"),
                style: "width:100px",
                regExp: "^([0-9]+)$",
                intermediateChanges: true,
                validator: function(value, contraints){
                    var checkExp = "^([0-9]+)$";
                     
                     if(value.search(checkExp) == -1){
                         return false;
                     }   
                     
                     if(parseInt(value) == 0){
                         return false;
                     }
                     
                     if(parseInt(value) > this.vendorRangeSize){
                         return false;
                     }
                     
                     if(parseInt(value) < 128){
                         return false;
                     }
                 
                    return true;
                },
                onChange: function(newValue) {
                    popup.set("size", newValue);
                }
            }).placeAt(this._sizeDiv);
            
            //last
            this._rangeLast = new dijit.form.ValidationTextBox({
                trim: true,
                required: true,
                disabled: true,
                value: this.last,
                placeHolder: "",
                invalidMessage: "",
                style: "width:175px",
                intermediateChanges: true,
                onChange: function(newValue){
                    popup.set("last", newValue);
                },
                validator : function(value, constraints){
                    var checkExp = "^([0-9A-Fa-f#][0-9A-Fa-f#]:[0-9A-Fa-f#][0-9A-Fa-f#]:[0-9A-Fa-f#][0-9A-Fa-f#]:[0-9A-Fa-f#][0-9A-Fa-f#]:[0-9A-Fa-f#][0-9A-Fa-f#]:[0-9A-Fa-f#][0-9A-Fa-f])$"; //|[0-9][0-9]|[0-1][0-9][0-9]|2[0-4][0-9]|25[0-5])([\.]([0-9]|[0-9][0-9]|[0-1][0-9][0-9]|2[0-4][0-9]|25[0-5])){3}$";

                    if(value.search(checkExp) == -1){
                         return false;
                     }            

                    return true;
                },
                //show invalid even when textbox is disabled
                validate : function(isFocused){
                    var message = "";
                    var isValid = this.isValid(isFocused);
                    if(isValid){ this._maskValidSubsetError = true; }
                    var isValidSubset = !isValid && isFocused && this._isValidSubset();
                    var isEmpty = this._isEmpty(this.textbox.value);
                    this.state = (isValid || (!this._hasBeenBlurred && isEmpty) || isValidSubset) ? "" : "Error";
                    if(this.state == "Error"){ this._maskValidSubsetError = false; }
                    this._setStateClass();
                    dijit.setWaiState(this.focusNode, "invalid", isValid ? "false" : "true");
                    if(isFocused){
                        if(isEmpty){
                            message = this.getPromptMessage(true);
                        }
                        if(!message && (this.state == "Error" || (isValidSubset && !this._maskValidSubsetError))){
                            message = this.getErrorMessage(true);
                        }
                    }
                    this.displayMessage(message);
                    return isValid;
                }
            }).placeAt(this._lastDiv);
            
            //setup
            if(this.mode == popup.resources.EDIT){
                var rangeName = this.range;
                var dropDownNames = this._getAllRangeNames();
                
                if(this.contains(this.range, dropDownNames)){
                    this._single.set("checked", true);
                    this.set("type", "single");
                    this._rangeTextBox.set("value", "");
                    this._rangeDropDown.set("displayedValue", rangeName);
                }
                
                else {
                    this._custom.set("checked", true);
                    this.set("type", "custom");
                    this._rangeDropDown.set("displayedValue", popup.locResources.NONE_SELECTED);
                    this._rangeTextBox.set("value", rangeName);
                }
                this.set("range", rangeName);
                
                this._okBtn.set("label", this.resources.SAVE);
            }
            else {
                this._single.set("checked", true);
                this.set("type", "single");
                this._rangeDropDown.set("displayedValue", popup.locResources.NONE_SELECTED);
            }
        },
        
        calculateNetworkID: function(base){
            if(base != null && base != "" && this._isValidAddress(base)){
                if(this.version == "IPv4" && this.dropDown != null){
                    var subnet = this.dropDown;
                    var splitSub = subnet.split(".");
                    var networkSplit = base.split(".");
                    var net = "";
                    for(var i=0; i<splitSub.length; i++){
                        if(splitSub[i] == 255)
                            net += networkSplit[i];
                        else
                            net += "0";
                        if(i != splitSub.length-1)
                            net += ".";
                    }
                    this.set("network", net);
                    return net;
                }
                else if(this.version == "IPv6"){
                    this.set("network", base);
                }
            }
            else{
                this.set("network", null);
            }
            return "0.0.0.0";
        },
       
        _getUsedRangeNames: function() {
            var usedNames = new Array();
            var store = this.subrangeStore;
            if(store != null){
                var items = store._arrayOfTopLevelItems;
                for(var i=0; i<items.length; i++){
                    var item = items[i];
                    var range = store.getValue(item, "range");
                    usedNames.push(range);
                }
            }
            return usedNames;
        },
        
        _getAllRangeNames : function(){
            var names = this._getAllRangeNameObjects();
            var allNames = new Array();
            for(var j=0; j<names.length; j++)
                allNames.push(names[j].name);
            return allNames;
        },
        
        _getAllRangeNameObjects: function(){
            return [
                    { name: "- None Selected -", id: null },
                    { name: "Chassis-0", id: "Chassis-0" },
                    { name: "Chassis-1", id: "Chassis-1" },
                    { name: "Chassis-2", id: "Chassis-2" }
                  ];
        },
        
        _getAvailableRangeNameOptions : function(){
            var popup = this;
            var allNameObjs = this._getAllRangeNameObjects();
            var allNames = this._getAllRangeNames();
            var usedNames = this._getUsedRangeNames();
            var availableNames = new Array();
            availableNames.push({ name: popup.locResources.NONE_SELECTED, id: null});
            //edit: include this.range (unless null)
            if(this.range != null){
                if(this.contains(this.range, allNames))
                    availableNames.push({ name:this.range, id: this.range });
            }
            //unused values
            for(var j=0; j<allNameObjs.length; j++){
                var value = allNameObjs[j].name;
                var id = allNameObjs[j].id;
                if(id != null && !this.contains(id, usedNames))
                    availableNames.push({ name:value, id:id });
            }
            return availableNames;
        },
        
        contains : function(value, array){
            if(array != null){
                for(var i=0; i<array.length; i++){
                    var arrayValue = array[i];
                    if(arrayValue == value)
                        return true;
                }
            }            
            return false;
        },
        
        containsWithIndex: function(value, array, index){
            if(array != null){
                for(var i=0; i<array.length; i++){
                    var arrayValue = array[i];
                    if(i != index && arrayValue == value)
                        return true;
                }
            }            
            return false;
        },
        
        _isValidAddress : function(ip){
            //if(this.version == "IPv4"){
               // var reg = new RegExp("^([0-9]|[0-9][0-9]|[0-1][0-9][0-9]|2[0-4][0-9]|25[0-5])(\.([0-9]|[0-9][0-9]|[0-1][0-9][0-9]|2[0-4][0-9]|25[0-5])){3}$");
             //   return reg.test(ip);
           // }
           if(this.version == "IPv6"){
                var reg = new RegExp("^\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*$");
                return reg.test(ip);
            }
            return true;
        },
  
        /*  The _isValidAddress function was duped.  Not sure which one was supposed to be used, but
 			since this dialog isn't in use yet, commenting out the second one.
        _isValidAddress : function(add){
            var reg = new RegExp(this._regExp);
            return reg.test(add);
        }, */

        _matchesGateway : function(value){
            return (this.gateway == value);
        },
        
        _sameAsNetworkID : function(value){
            return (this.network == value);
        },
        
        _matchesNetworkID : function(value){
            if(this.network != null && value != null){
                if(this._isValidAddress(this.network) && this._isValidAddress(value)){
                    if(this.version == "IPv4"){
                        var IDsplit = this.network.split(".");
                        var IPsplit = value.split(".");
                        for(var i=0; i<IDsplit.length; i++){
                            if(IDsplit[i] != "0"){
                                if(IDsplit[i] != IPsplit[i])
                                    return false;
                            }
                        }
                    }
                }
            }
            return true;
        },
        
        _isInBounds : function(value, first, size, last){
            if(this.version == "IPv4"){
                var network = this.overallStore._arrayOfTopLevelItems[0].network[0];
                if(this._isValidAddress(network) && this._isValidAddress(value) &&
                   this._isValidAddress(first) && this._isValidAddress(last) ){
                    var networkSplit = network.split(".");
                    var valueSplit = value.split(".");
                    var lowSplit = first.split(".");
                    var highSplit = last.split(".");
                    for(var i=0; i<4; i++){
                        var net = networkSplit[i];
                        if(net == "0"){
                            //can all be equal, except last one
                            var val = parseInt(valueSplit[i]);
                            var low = parseInt(lowSplit[i]);
                            var high = parseInt(highSplit[i]);
                            if(i != 3 && val > low && val < high){
                                return true;
                            }
                            if(val < low || val > high)
                                return false;
                        }
                    }
                }
            }
            
            return true;
        },
        
        calculateLowestAddress : function(network){
            var subnet = this.dropDown;
            var lowest = "";
            if(this.version == "IPv4"){
                if(subnet != null && network != null && subnet != "" && 
                   network != "" && network != "0.0.0.0" && this._isValidAddress(network)){
                    var splitSub = subnet.split(".");
                    var networkSplit = network.split(".");
                    for(var i=0; i<splitSub.length; i++){
                        if(splitSub[i] == 255)
                            lowest += networkSplit[i];
                        else if (i != splitSub.length-1)
                            lowest += "0";
                        else{
                            lowest += "1";
                        }
                        if(i != splitSub.length-1)
                            lowest += ".";
                    }
                }
                if(this.gateway == lowest){
                    var lowestSplit = lowest.split(".");
                    lowest = lowestSplit[0] + "." + lowestSplit[1] + "." + lowestSplit[2] + ".2";
                }
                if(lowest == "")
                    lowest = "0.0.0.0";
            }
            else if(this.version == "IPv6"){
                var split = network.split(":");
                var firstPart = "";
                for(var i=0; i<split.length-1; i++)
                    firstPart += split[i] + ":";
                lowest = firstPart + "0";
                if(lowest == "")
                    lowest = "0::0";
            }
            return lowest;  
        },
        
        _isInNetworkBounds : function(value, network, lowest, highest){
            if(this.version == "IPv4"){
                if(this._isValidAddress(network) && this._isValidAddress(value) &&
                   this._isValidAddress(lowest) && this._isValidAddress(highest) ){
                    var networkSplit = network.split(".");
                    var valueSplit = value.split(".");
                    var lowSplit = lowest.split(".");
                    var highSplit = highest.split(".");
                    for(var i=0; i<4; i++){
                        var net = networkSplit[i];
                        if(net == "0"){
                            //can all be equal, except last one
                            var val = parseInt(valueSplit[i]);
                            var low = parseInt(lowSplit[i]);
                            var high = parseInt(highSplit[i]);
                            if(i != 3 && val > low && val < high){
                                return true;
                            }
                            if(val < low || val > high)
                                return false;
                        }
                    }
                }
            }
            
            return true;
        },
      
        _isOverlapping : function(value, index){
            var store = this.subrangeStore;
            if(store != null){
                var items = store._arrayOfTopLevelItems;
                for(var i=0; i<items.length; i++){
                    if(i != index){
                        var item = items[i];
                        var first = store.getValue(item, "first");
                        var size = parseInt(store.getValue(item, "size"))+1;
                        var last = store.getValue(item, "last");
                        var reg = new RegExp("^[1-9][0-9]*$");
                        if(this._isValidAddress(first) && reg.test(size)){
                            if(this._isInBounds(value, first, size, last))
                                return true;
                        } 
                    }
                }
            }
            return false;
        },
        
        _setTypeAttr : function(t){
            this.type = t;
            if(t == "single"){
                //use drop down
                this._rangeDropDownDiv.style.display = "block";
                this._rangeTextBoxDiv.style.display = "none";
                this._singleMessage.style.display = "block";
                this._customMessage.style.display = "none";
                
                if(this._rangeSize != null && this._rangeTextBox != null){
                    if(this.mode == "add")
                        this._rangeSize.set("value", "128");
                    this.set("range", this._rangeDropDown.get("value"));
                }
            }
            else{
                //"custom", use textbox
                this._rangeDropDownDiv.style.display = "none";
                this._rangeTextBoxDiv.style.display = "block";
                this._singleMessage.style.display = "none";
                this._customMessage.style.display = "block";
               
                if(this._rangeSize != null && this._rangeDropDown != null
                        && this.subrangeStore != null && this.overallStore != null){
                    this.set("range", this._rangeTextBox.get("value"));
                    if(this.mode == "add" && this.subrangeStore._arrayOfTopLevelItems.length == 0){
                       // var overallSize = this.overallStore._arrayOfTopLevelItems[0].host[0];
                        //this._rangeSize.set("value", overallSize);
                    }
                    else if(this.mode == "add"){
                        var subrangeItems = this.subrangeStore._arrayOfTopLevelItems;
             //           var previousSize = subrangeItems[subrangeItems.length-1].size[0];
              //          this._rangeSize.set("value", previousSize);
                    }
                }
            }                           
        },
        
        _setVersionAttr : function(m){
            var popup = this;
            this.version = m;
            if(this.version == "IPv6"){
                //this._invalidMessage = "Address must have 8 groups of hexadecimal values separated by colons, each value in the range of 0-FFFF. Addresses can be abbreviated by omitting leading zeros within each group, and one group of consecutive zeroes may be replaced by a double colon. e.g. 2001:db8:0:0:1:0:0:1,  2001:0DB8:0:0:1::1";
                //this._regExp = "^\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*$";
                //this._placeHolder = "00:00:00";
            }
            else{
                this._invalidMessage = popup.resources.INVALID;
                this._regExp = ".*";
                this._placeHolder = null;
            }
        },
        
        _setRangeAttr : function(r){
            this.range = r;
            this._okEnabled();
        },
        
        _setFirstAttr : function(f){
            this.first = f;
            //this.calculateNetworkID();
            this.calExp();
            this.set("last", this._getLastAddress(this.first, this.size));
            this._okEnabled();
        },
        
        _setSizeAttr : function(s){
            this.size = s;
            this.set("last", this._getLastAddress(this.first, this.size));
            this._okEnabled();
        },
        
        _setLastAttr : function(l){
            this.last = l;
            if(this._rangeLast != null)
                this._rangeLast.set("value", this.last);
            this._okEnabled();
        },
        
        _getLastAddress : function(first, size){
            
            this.allocated = "0::100";
            
            var lasthexNumber = parseInt(this.hexNumber) + parseInt(size);
            
            var hexString = lasthexNumber.toString(16);
            
            var addCounter;
            
            if(hexString.length != 4){
                var addZeroCount = 4 - hexString.length;
                for(addCounter = 0; addCounter < addZeroCount; addCounter++){
                    hexString = "0"+hexString;
                }
            }
            
            hexString = hexString.toUpperCase();
            hexString = hexString.replace(/(\S{2})/g,"$1:");   // add : to hex string
            hexString = hexString.replace(/:$/,"");
            
            var checkFirstExp = this.rangeFirstRegExp;
            
            if(this.first == null || this.first == "")
                return "";
            
            if(this.first.search(checkFirstExp) == -1)
                return "";                
            
            var checkLastExp = "^([0-9A-Fa-f#][0-9A-Fa-f#]:[0-9A-Fa-f#][0-9A-Fa-f#]:[0-9A-Fa-f#][0-9A-Fa-f#]:[0-9A-Fa-f#][0-9A-Fa-f#]:[0-9A-Fa-f#][0-9A-Fa-f#]:[0-9A-Fa-f#][0-9A-Fa-f]:[0-9A-Fa-f][0-9A-Fa-f]:[0-9A-Fa-f][0-9A-Fa-f])$";
            
            return this.vendorprefixFirst+hexString;
        },
        
        _okEnabled : function(){            
            //var firstBox = this._rangeFirst;
            var sizeBox = this._rangeSize;
            var lastBox = this._rangeLast;
            var firstBoxIPcheck = this.calExp();
            
            if(firstBoxIPcheck == -1){
                this.overlappingSubrange = true;
            } else {
                this.overlappingSubrange = false;
            }
            
            if( this.range != null && this.range != "" &&
                this.first != null && this.first != "" &&
                this.size != null && this.size != "" && this.size > 127 &&
                this.last != null && this.last != "" && 
                firstBoxIPcheck != -1 && sizeBox != null && sizeBox.isValid() 
                && lastBox != null && lastBox.isValid()){
                console.log(this.declaredClass + ": show");

                this._okBtn.set("disabled", false);
            }else{
                
                console.log(this.declaredClass + ": dont show");

                this._okBtn.set("disabled", true);
            }
        },

        _okClicked : function(){                       
            this.hide();
            dojo.destroy(this.id + "_underlay");
            if(this.mode == "add")
                this.addNewSubrange();
            else
                this.editSubrange();
            this._sendAllOptions();
        },
        
        _sendAllOptions : function(){
            console.log(this.declaredClass + ": ", this.allOptionNames);
            this.sendAllOptions(this.allOptionNames);
        },
        
        sendAllOptions : function(options) {
            //override if necessary
        },
        
        _cancelClicked : function(){
            this.hide();
            dojo.destroy(this.id + "_underlay");
        },
        
        calExp : function(){
            var checkExp = "^([0-9A-Fa-f#][0-9A-Fa-f#]:[0-9A-Fa-f#][0-9A-Fa-f#]:[0-9A-Fa-f#][0-9A-Fa-f#]:[0-9A-Fa-f#][0-9A-Fa-f#]:[0-9A-Fa-f#][0-9A-Fa-f#]:[0-9A-Fa-f#][0-9A-Fa-f])$"; //|[0-9][0-9]|[0-1][0-9][0-9]|2[0-4][0-9]|25[0-5])([\.]([0-9]|[0-9][0-9]|[0-1][0-9][0-9]|2[0-4][0-9]|25[0-5])){3}$";
            
            var firstExp = this.vendorprefixFirst+this.first;  //combine vendor prefix and entered first addresses
                                    
            if(this.first != null){
                var firstExpNoChar = this.first.replace(/:/g, "");   //take out :
                           
                var intNumber = parseInt(firstExpNoChar,16);   //turn hex into decemical 
                
                this.hexNumber = intNumber;  //set global hexNumber var
                   
                var lastVendorExpNoChar = this.vendorprefixLast.replace(/:/g, "");   //take out :
                
                lastVendorExpNoChar = lastVendorExpNoChar.substr(-4);                
                   
                var lastvendorNumber = parseInt(lastVendorExpNoChar,16);                       
                
                if(this.last != null){
                    var lastNumberExpNoChar = this.last.replace(/:/g, "");   //take out :
                    
                    lastNumberExpNoChar = lastNumberExpNoChar.substr(-4);
                           
                    var lastNumber = parseInt(lastNumberExpNoChar,16);            
                }
               
                var hexString = intNumber.toString(16);
    
            }
            
            if(this.rangeCheckerArray.length != 0){

                var rangeCheckerCount = this.rangeCheckerArray.length;
                var counter;                
                for(counter = 0;counter < rangeCheckerCount;counter=counter+2){

                    if(lastNumber >= this.rangeCheckerArray[counter] && lastNumber <= this.rangeCheckerArray[counter+1]){

                        return -1;                        
                        
                    }

                    if(intNumber >= this.rangeCheckerArray[counter] && intNumber <= this.rangeCheckerArray[counter+1]){

                        return -1;
                        
                    }
                    
                    if(intNumber <= this.rangeCheckerArray[counter] && lastNumber >= this.rangeCheckerArray[counter+1]){
                        return -1;
                    }

                }
                
            }
            
            return firstExp.search(checkExp);            
        }
    });
});