define([
        "dojo",
        "dijit",
        "support/dialogs/Dialog",
        "dojo/i18n!./nls/ExportPatternResources",
        "dojo/text!./templates/ExportPatternResources.html",
        "dojo/store/Memory",
        "dijit/tree/ForestStoreModel",
        "dijit/Tree",
        "dijit/form/CheckBox", 
        "dojo/data/ItemFileWriteStore",
        "dojo/_base/window", 
        "support/dialogs/ProgressDialog",
        "support/panels/ExportTable"
    ], function(dojo, dijit, dialog, mri, template) {
    
    return dojo.declare("support.dialogs.ExportPatternResources", [ support.dialogs.Dialog ], {
        
        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
             
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,
        baseClass: "ExportPatternResources",
        
        // Checkbox variables
        _totalCheckboxes: 0,
        _totalItemsChecked: 0,
        _checkBoxes: new Array,
        
        // The available patterns
        _exportPatterns: new Array,
        
        // The data for the tree's store
        _treeData: null,
        
        // Patterns referenced by other patterns
        _referenced: new Array,
        
        // All IP pools
        _ipPools: new Array,
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/ExportPatternResources.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            this._connectDataModels();
            this._poolConnectDataModels();
            this.createTree();
            if(this._exportTree){
                this._exportTree.dndController.singular = true;
                this._exportTree.rootNode.getChildren().forEach(dojo.hitch(this, function(child) {
                    if(child.item.children) {
                        this._exportTree._expandNode(child);
                    }
                }));
            }
        },
        
        createTree: function() {
            // Create resource tree here
            if(this._itemsExist){ // If data exists, create tree
                this.store = new dojo.data.ItemFileWriteStore({
                    data: dojo.fromJson(this._treeData)});
            
                var treeModel = new dijit.tree.ForestStoreModel({
                    store: this.store,
                    query: {
                        type: "root"
                    },
                    deferItemLoadingUntilExpand: true,
                    rootId: "root",
                    rootLabel: "ExportPatterns",
                    childrenAttrs: [ "children" ]
                });
                this._exportTree = new dijit.Tree({
                    model: treeModel,
                    showRoot: false,
                    autoExpand: false,
                    selectionMode: "none",
                    _createTreeNode: dojo.hitch(this, function(args){// Must be overridden to add checkboxes
                    
                        var tnode = new dijit._TreeNode(args);
                        if(!args.isExpandable){ // Specific patterns, add checkbox
                            tnode.labelNode.innerHTML = args.label;
                            var cb = new dijit.form.CheckBox();
                            cb.placeAt(tnode.labelNode, "first");
                            this._checkBoxes.push(cb);
                            this._totalCheckboxes++;
                            dojo.connect(cb, "onChange", dojo.hitch(this, function(){
                                // If a box is checked, update all parent labels
                                this._updateNumberSelected(cb.getParent().getParent(), cb.checked);
                                //Adjust the total number of checked boxes
                                if(cb.checked){
                                    this._totalItemsChecked++;
                                }else{
                                    if(this._totalItemsChecked > 0){
                                        this._totalItemsChecked--;
                                    }
                                }
                                // Enable or disable export
                                this._enableExport();
                            }));
                        
                        }else if(args.item.children){
                            // Create label for parents when the tree is loaded
                            var totalBoxes = 0;
                            var createLabel = function(parent){
                                parent.children.forEach(
                                        function(child){
                                            if(child.children){
                                                createLabel(child);
                                            }else{
                                                totalBoxes++;
                                            }
                                        });
                            };
                            createLabel(args.item);
                            if(args.indent != 0){
                                tnode.labelNode.innerHTML = args.label + " (0 of " + totalBoxes + " Selected)";
                            }else{
                                tnode.labelNode.innerHTML = args.label;
                            }
                        }else{
                            tnode.labelNode.innerHTML = args.label;
                        }
                        return tnode;
                    }),
                    expandAll : function(top){
                        if(!top || !this.model.mayHaveChildren(top.item)) return;
                        top.getChildren().forEach(dojo.hitch(this, function(child) {
                            if(child.item.children) {
                                this._expandNode(child);
                                this.expandAll(child);
                            }
                        }));
                    },
                    collapseAll : function(top) {
                        if(!top || !this.model.mayHaveChildren(top.item)) return;
                            top.getChildren().forEach(dojo.hitch(this, function(child) {
                                if(child.item.children) {
                                    if(child.indent!=0){ // Don't collapse top-level nodes
                                        this._collapseNode(child);
                                    }
                                    this.collapseAll(child);
                                }
                            }));
                    },
                    selectAll : dojo.hitch(this, function() {
                        this._checkBoxes.forEach(function(box){
                            box.set("checked", true);
                        });
                     }),
                     deselectAll : dojo.hitch(this, function() {
                         this._checkBoxes.forEach(function(box){
                             box.set("checked", false);
                         });
                     }),
                     getIconClass : dojo.hitch(this, function(item, opened) {
                         // Must be overridden to create custom icons
                         // Id's refer to CSS classes
                         if(treeModel.getIdentity(item) == this.locResources.SERV){
                             return "server";
                         }else if(treeModel.getIdentity(item) == this.locResources.CHAS){
                             return "chassis";
                         }else if(treeModel.getIdentity(item) == this.locResources.AP){
                             return "addressPool";
                         }else if(treeModel.getIdentity(item) == this.locResources.SERV_PA || treeModel.getIdentity(item) == this.locResources.CHAS_PA){
                             return "patterns";
                         }else if(treeModel.getIdentity(item) == this.locResources.SERV_POL){
                             return "policies";
                         }
                     })
                }, this.exportTree);

            }else{
                // If no data exists, display no data message
                this.exportTree.innerHTML = "<div style=\"text-align: center;\">" + this.resources.NO_DATA + "</div>";
            }
        },
        
        _updateNumberSelected: function(parent, checked) {
            // tags:
            // protected
            // Update parent labels when a box is checked/unchecked
            if(!parent.item.root && !parent.getParent().item.root){
                var firstNum = parent.labelNode.innerHTML.match(/[0-9][0-9]*/);
                var firstNumIndex = parent.labelNode.innerHTML.indexOf(firstNum);
                var boxes = parseInt(firstNum);
                
                if(checked){
                    boxes++;
                }else{
                    if(boxes>0){
                        boxes--;
                    }
                }
                
                var secondIndex = parent.labelNode.innerHTML.indexOf(" of");
                
                parent.set("label", parent.labelNode.innerHTML.substr(0,firstNumIndex-2) +
                        " (" + boxes + parent.labelNode.innerHTML.substr(secondIndex,parent.labelNode.innerHTML.length-1));
                this._updateNumberSelected(parent.getParent(), checked);
            }
        },
        
        _expandAll: function() {
            // tags:
            // protected
            if(this._exportTree){
                this._exportTree.expandAll(this._exportTree.rootNode);
            }
        },
        
        _collapseAll: function() {
            // tags:
            // protected
            if(this._exportTree){
                this._exportTree.collapseAll(this._exportTree.rootNode);
            }
        },
        
        _selectAll: function() {
            // tags:
            // protected
            this._expandAll();
            if(this._exportTree && this._totalItemsChecked != this._totalCheckboxes){
                this._exportTree.selectAll();
            }
        },
        
        _deselectAll: function() {
            // tags:
            // protected
            if(this._exportTree && this._totalItemsChecked != 0){
                this._exportTree.deselectAll();
            }
        },
        
        _enableExport: function() {
            // tags:
            // protected
            if(this._totalItemsChecked==0){
                this.exportButton.set("disabled", true);
            }else if(this._totalItemsChecked == this._totalCheckboxes){
                this.exportButton.set("disabled", false);
            }else{
                this.exportButton.set("disabled", false);
            }
        },
        
        _connectDataModels: function(){
            // tags:
            // protected
            // Load failover policies, server-related patterns, and chassis-related patterns
            var policies = null;
            // Load failover policies
            dojo.xhrGet({
                url: this.REST_PATH + "policy?type=failover&list=true",
                handleAs: "json",
                sync: true,
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                preventCache: true,
                load: dojo.hitch(this, function(response) {
                    if(response && response.items && response.items.length>0){
                        this._itemsExist = true;
                        policies = response;
                    }
                }),
                error: dojo.hitch(this, function(error, ioArgs) {
                    // Failure path! We received a non-2xx status code (ie: 3xx/4xx/5xx status code) from the API.

                    dojo.publish("ProgressFailed", error);
                    console.log(this.declaredClass + ": ", error, ioArgs);
                    this.getModalDialog("Error", this.locResources.ERROR_FP).show();
                    this.hide();
                    
                })
            });
            
            // Load server- and chassis-related patterns
            dojo.xhrGet({
                url: this.REST_PATH + "template/list/all",
                handleAs: "json",
                preventCache: true,
                sync: true,
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                load: dojo.hitch(this, function(response) {
                    if (response && response.items && response.items.length>0) {
                        this._itemsExist = true;
                        this._flatToRoot(response.items, true, policies);
                    }else{
                        if(response.items){
                            this._flatToRoot(response.items, true, policies);
                        }else{
                            this._flatToRoot(response, true, policies);
                        }
                    }
                }),
                error: dojo.hitch(this, function(error, ioArgs) {
                    // Failure path! We received a non-2xx status code (ie: 3xx/4xx/5xx status code) from the API.
                    dojo.publish("ProgressFailed", error);
                    console.log(this.declaredClass + ": ", error, ioArgs);
                    this.getModalDialog("Error", this.locResources.ERROR_SC_PATTERNS).show();
                })
            });
            
        },
        
        _flatToRoot: function(dataItems, init, policies){
            // tags:
            // protected
            // Organize all patterns and policies into parent-child relationships
            var stChildren = new Array();
            var sitChildren = new Array();
            var mitChildren = new Array();
            var psctChildren = new Array();
            var partChildren = new Array();
            var daiptChildren = new Array();
            var fcbttChildren = new Array();
            var atChildren = new Array();
            var pvtChildren = new Array();
            //var eptChildren = new Array();
            var cmmInfoChildren = new Array();
            var cmmManagementChildren = new Array();
            var cmmPowerChildren = new Array();
            var cmmUsersChildren = new Array();
            var cmmNodeChildren = new Array();
            var cmmIOModuleChildren = new Array();
            var CMMChildren = new Array();
            var newChild = dojo.hitch(this, function(ref, prefix) {
                var child = {
                    id: ref.id,
                    name: ref.name,
                    type: (init) ? ref.type : ref.templateType,
                    referencedBy: ref.referencedBy,
                    description: ref.description
                };
                if(ref.referencedBy.length>0){
                    this._referenced.push(child);
                }
                return child;
            });
            
            var allFP = new Array();
            var newFPChild = function(ref) {
                var child = {
                        id: ref.id,
                        name: ref.name,
                        type: ref.type,
                        description: ref.description
                };
                return child;
            };
            
            if(policies){
                dojo.forEach(policies.items, function(item){
                    allFP.push(newFPChild(item));
                });
            }
            
            dojo.forEach(dataItems, function(item) {
                var typeValue = (init) ? item.type : item.templateType;

                if (typeValue == "Server") {
                    stChildren.push(newChild(item, (init) ? "st::" : ""));
                }
                else if (typeValue == "SystemInfo") {
                    sitChildren.push(newChild(item, (init) ? "sit::" : ""));
                }
                else if (typeValue == "Management") {
                    mitChildren.push(newChild(item, (init) ? "mit::" : ""));
                }
                else if (typeValue == "Power") {
                    psctChildren.push(newChild(item, (init) ? "psct::" : ""));
                }
                else if (typeValue == "PerformanceRecovery") {
                    partChildren.push(newChild(item, (init) ? "part::" : ""));
                }
                else if (typeValue == "DevicesAndIOPorts") {
                    daiptChildren.push(newChild(item, (init) ? "daipt::" : ""));
                }
                else if (typeValue == "FibreChannel") {
                    fcbttChildren.push(newChild(item, (init) ? "fcbtt::" : ""));
                }
                else if (typeValue == "Adapter") {
                    atChildren.push(newChild(item, (init) ? "at::" : ""));
                }
                else if (typeValue == "PortVirtualization") {
                    pvtChildren.push(newChild(item, (init) ? "pvt::" : ""));
                }
                /*else if (typeValue == "ExtendedPort") {
                    eptChildren.push(newChild(item, (init) ? "ept::" : ""));
                }*/
                else if(typeValue == "cmmInfo"){
                    cmmInfoChildren.push(newChild(item, (init) ? "ci::" : ""));
                }
                else if(typeValue == "cmmManagement"){
                    cmmManagementChildren.push(newChild(item, (init) ? "cm::" : ""));
                }
                else if(typeValue == "cmmPower"){
                    cmmPowerChildren.push(newChild(item, (init) ? "cp::" : ""));
                }
                else if(typeValue == "cmmUsers"){
                    cmmUsersChildren.push(newChild(item, (init) ? "cu::" : ""));
                }
                else if(typeValue == "cmmNode"){
                    cmmNodeChildren.push(newChild(item, (init) ? "cn::" : ""));
                }
                else if(typeValue == "CMM"){
                    CMMChildren.push(newChild(item, (init) ? "cmm::" : ""));
                }
                else if(typeValue == "cmmIOModule"){
                    cmmIOModuleChildren.push(newChild(item, (init) ? "cio::" : ""));
                }

            });
            
            var serverParent = new Array;
            var chassisParent = new Array;
            var fpParent = new Array;
            if (stChildren.length > 0) {
                // Server Templates Objects
                serverParent.push({
                    id: "st",
                    name: this.locResources.SERVER_PATTERNS,
                    type: "root",
                    children: stChildren
                });
            }
            if (sitChildren.length > 0) {
                // System Information Templates Objects
                serverParent.push({
                    id: "sit",
                    name: this.locResources.SYSTEM_INFO_PATTERNS,
                    type: "root",
                    children: sitChildren
                });
            }
            if (mitChildren.length > 0) {
                // Management Interface Templates Objects
                serverParent.push({
                    id: "mit",
                    name: this.locResources.MANAGEMENT_INTERFACE_PATTERNS,
                    type: "root",
                    children: mitChildren
                });
            }
            if (psctChildren.length > 0) {
                // Power Schedule and Capping Templates Objects
                serverParent.push({
                    id: "psct",
                    name: this.locResources.PSACP,
                    type: "root",
                    children: psctChildren
                });
            }
            if (partChildren.length > 0) {
                // Performance and Recovery Templates Objects
                serverParent.push({
                    id: "part",
                    name: this.locResources.PERF_RECOVERY_PATTERNS,
                    type: "root",
                    children: partChildren
                });
            }
            if (daiptChildren.length > 0) {
                // Devices And IO Ports Templates Objects
                serverParent.push({
                    id: "daipt",
                    name: this.locResources.DEV_IO_PATTERNS,
                    type: "root",
                    children: daiptChildren
                });
            }
            if (fcbttChildren.length > 0) {
                // Fibre Channel Boot Target Templates Objects
                serverParent.push({
                    id: "fcbtt",
                    name: this.locResources.FIBRE_BOOT_PATTERNS,
                    type: "root",
                    children: fcbttChildren
                });
            }
            if (atChildren.length > 0) {
                // Adapter Templates Objects
                serverParent.push({
                    id: "at",
                    name: this.locResources.ADAPTER_PATTERNS,
                    type: "root",
                    children: atChildren
                });
            }
            if (pvtChildren.length > 0) {
                // Port Virtualization Templates Objects
                serverParent.push({
                    id: "pvt",
                    name: this.locResources.PORT_PATTERNS,
                    type: "root",
                    children: pvtChildren
                });
            }
            /*if (eptChildren.length > 0) {
                // Extended Port Templates Objects
                serverParent.push({
                    id: "ept",
                    name: this.locResources.PORTTemplate,
                    type: "root",
                    children: eptChildren
                });
            }*/
            if (CMMChildren.length > 0) {
                // Chassis Templates Objects
                chassisParent.push({
                    id: "cmm",
                    name: this.locResources.CHASSIS_PATTERNS,
                    type: "root",
                    children: CMMChildren
                });
            }
            if (cmmInfoChildren.length > 0) {
                // Chassis Infomation Templates Objects
                chassisParent.push({
                    id: "ci",
                    name: this.locResources.CHASSIS_INFO_PATTERNS,
                    type: "root",
                    children: cmmInfoChildren
                });
            }
            if (cmmManagementChildren.length > 0) {
                // Management Interface Templates Objects
                chassisParent.push({
                    id: "cm",
                    name: this.locResources.MANAGEMENT_INTERFACE_PATTERNS,
                    type: "root",
                    children: cmmManagementChildren
                });
            }
            if (cmmPowerChildren.length > 0) {
                // Power and Acoustics Templates Objects
                chassisParent.push({
                    id: "cp",
                    name: this.locResources.POWER_ACOUSTICS_PATTERNS,
                    type: "root",
                    children: cmmPowerChildren
                });
            }
            if (cmmUsersChildren.length > 0) {
                // Security Templates Objects
                chassisParent.push({
                    id: "cu",
                    name: this.locResources.USERS_SEC_PATTERNS,
                    type: "root",
                    children: cmmUsersChildren
                });
            }
            if (cmmIOModuleChildren.length > 0) {
                // Chassis I/O Module Templates Objects
                chassisParent.push({
                    id: "cio",
                    name: this.locResources.CHASSIS_IO_MOD_PATTERNS,
                    type: "root",
                    children: cmmIOModuleChildren
                });
            }
            if (cmmNodeChildren.length > 0) {
                // Node Templates Objects
                chassisParent.push({
                    id: "cn",
                    name: this.locResources.NODE_PATTERNS,
                    type: "root",
                    children: cmmNodeChildren
                });
            }
            
            

            var serverPatterns = new Array;

            //ServerParent and allFP must be pushed separately to create the correct tree structure
            if(serverParent.length>0){
                serverPatterns.push({
                    id: this.locResources.SERV_PA,
                    name: this.resources.TEMPLATES,
                    type: "root",
                    children: serverParent
                    });
            }
            
            if(allFP.length > 0){
                fpParent.push({
                    id: "fp",
                    name: this.locResources.FAILOVER_POLICIES,
                    type: "root",
                    children: allFP
                });
            
                serverPatterns.push({
                    id: this.locResources.SERV_POL,
                    name: this.locResources.POLICIES,
                    type: "root",
                    children: fpParent
                });
            }
            
            if(serverPatterns.length > 0){
                this._exportPatterns.push({
                    id: this.locResources.SERV,
                    name: this.resources.SERVER,
                    type: "root",
                    children: serverPatterns
                });
            }
            
            var  chassisPatterns = new Array;
            
            if(chassisParent.length > 0){
                chassisPatterns.push({
                    id: this.locResources.CHAS_PA,
                    name: this.resources.TEMPLATES,
                    type: "root",
                    children: chassisParent
                });
                
                this._exportPatterns.push({
                    id: this.locResources.CHAS,
                    name: this.resources.CHASSIS,
                    type: "root",
                    children: chassisPatterns
                });
            }
            
        },
        
        _poolConnectDataModels: function() {
            // tags:
            // protected
            // Load IP Address Pools
            var loopcount = 0;
            var addressPools = new Array;
            dojo.xhrGet({
                url: this.REST_PATH + "ippool/list/",
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                preventCache: true,
                sync: true,
                load: dojo.hitch(this, function(response) {
                if(response && response.items && response.items.length>0){
                    this._itemsExist = true;
                    var numOfIPPools = response.items.length;
                    
                    for (loopcount = 0; loopcount < numOfIPPools; loopcount++) {
                        var childObject = new Object();
                        childObject.id = response.items[loopcount].id;
                        childObject.type = this.locResources.IP_ADDRESS_POOL;
                        childObject.poolType = this.locResources.IP_ADDRESS;
                        childObject.uri = response.items[loopcount].uri;
                        childObject.name = response.items[loopcount].name;
                        childObject.inUse = response.items[loopcount].inUse;
                        childObject.custom = response.items[loopcount].custom;
                        childObject.description = response.items[loopcount].description;
                        childObject.allocatedPct = response.items[loopcount].allocatedPct;
                        addressPools.push(childObject);
                        if(childObject.inUse == true){
                            this._ipPools.push(childObject);
                        }
                    }
                    
                    var ipAddressPools = new Array;
                    ipAddressPools.push({
                        id: this.locResources.AP_AP,
                        name: this.locResources.IP_ADDRESS_POOLS,
                        type: "root",
                        children: addressPools
                    });
                    this._exportPatterns.push({
                        id: this.locResources.AP,
                        name: this.resources.ADDRESS_POOLS,
                        type: "root",
                        children: ipAddressPools
                    });
                    
                    var tableData = dojo.toJson({
                        "identifier": "id",
                        "label": "name",
                        "items": this._exportPatterns
                    }, true);
                    
                    this._treeData = tableData;
                    
                }else{
                    var tableData = dojo.toJson({
                        "identifier": "id",
                        "label": "name",
                        "items": this._exportPatterns
                    }, true);
                    
                    this._treeData = tableData;
                }
            }),
            error: dojo.hitch(this, function(error, ioArgs) {
                // Failure path! We received a non-2xx status code (ie: 3xx/4xx/5xx status code) from the API.
                var tableData = dojo.toJson({
                    "identifier": "id",
                    "label": "name",
                    "items": this._exportPatterns
                }, true);
                
                this._treeData = tableData;
                dojo.publish("ProgressFailed", error);
                console.log(this.declaredClass + ": ", error, ioArgs);
                
                this.getModalDialog("Error", this.locResources.ERROR_POOL).show();
                
            })
        });

        },
        
        _getSelectedPatterns: function(){
            // tags:
            // protected
            // Find all selected patterns
            var selected = new Array;
            this._checkBoxes.forEach(
                function(box){
                    if(box.checked){
                        selected.push(box.getParent());
                    }
                });
            return selected;
        },
        
        _finishExport: function() {
            // tags:
            // protected
            // Get selected patterns and pass to ExportTable panel
            if(this._totalItemsChecked>0){
                var selected = this._getSelectedPatterns();
                var exportTable = new support.panels.ExportTable();
                
                // Pass in the selected resources as JSON to build the TreeTables
                exportTable.createTreeTable(selected, this._referenced, this._ipPools);
                var confirm = dojo.hitch(this, function(){
                    this._cancelExport(); //99288: Change this to a function that completes the export
                });
                
                if(this.CP_RUN_ENVIRONMENT == "fsm"){

                    require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                        var confirmDialog = new support.temp.ConfirmationDialog({
                            buttonLabel: this.resources.EXPORT,
                            type: "Warning",
                            style: "width: 800px !important; height:700px;",                            
                            text: this.resources.EXPORT_CONFIRM,
                            info: this.resources.INFO                    
                        });

                        confirmDialog.set("content", exportTable);
                        confirmDialog.startup();
                        confirmDialog.confirm(confirm, this);
                    
                    }));
     
                    
                }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                    require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                        var confirmDialog = new idx.widget.ConfirmationDialog({
                            buttonLabel: this.resources.EXPORT,
                            type: "Warning",
                            style: "width: 800px !important; height:700px;",                            
                            text: this.resources.EXPORT_CONFIRM,
                            info: this.resources.INFO                    
                        });

                        confirmDialog.set("content", exportTable);
                        confirmDialog.startup();
                        confirmDialog.confirm(confirm, this);
                    
                    }));
                    
                }

                /**this.hide();
                this.destroy();
                this.dialogNode.destroy();**/
            }
        },
        
        _cancelExport: function() {
            this.hide();
            this.dialogNode.destroyRecursive();            
        }
    });
});