define(["dojo", 
        "dijit", 
        "support/dialogs/Dialog", 
        "dojo/i18n!./nls/FailoverMonitorTemplate", 
        "dojo/text!./templates/FailoverMonitorTemplate.html",
        "dojo/parser", 
        "support/widgets/Select",
        "dojo/data/ItemFileReadStore",
        "dijit/form/Button", 
        "support/panels/DualTableSelector", 
        "support/base/TableMixin"
        ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.FailoverMonitorTemplate", [ support.dialogs.Dialog, support.base.TableMixin ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        baseClass: "FailoverMonitorTemplate",

        _availTable: null,
        _selectTable: null,
        
        _selectedStore: null,

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/FailoverMonitorTemplate.css",

        // Internal Selector Names
        POLTMPL_NAME: "policyTemplate",
        STBYPOOL_NAME: "standbyPool",

        // Type Names
        POL_NAME: "failover",

        SELECTOR_WIDTH: "300px",

        newPolicyId: "-1",
        newPoolId: "-1",
        newFormFactor: null,

        templateIDNum: null,

        // UUID of server to perform failover on
        serverUUID: null,
        
        // Label for Start/Save button
        buttonLabel: null,
        
        // Display table
        displayMode: null,
        
        // Max height of dialog
        maxHeight: null,
        
        // ID prefixes used for tables
        availGridID: "AvailableServersTable",
        selectGridID: "SelectedServersTable",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
            
            if (this.templateIDNum) {
                this.buttonLabel = this.resources.SAVE;  
            } else {
                this.buttonLabel = this.locResources.START;
            }
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            
            // Set Dialog Title
            var title = (this.templateIDNum == null) ? this.locResources.PAGE_TITLE : this.locResources.EDIT_TITLE;
            
            // warn user of system modification before starting failover monitor
            var msgDescription = this.locResources.FAILOVER_MONITOR_INFO_DETAILS;
            var msgTitle = this.locResources.FAILOVER_MONITOR_INFO;
            
            var msgDescriptionFF = this.locResources.FF_MISMATCH_DETAILS;
            var msgTitleFF = this.locResources.FF_MISMATCH;
            
            if(this.CP_RUN_ENVIRONMENT == "fsm"){
            
                require([ "idx/oneui/messaging/SingleMessage" ], dojo.hitch(this, function() {

                    this._ffMessageArea = new idx.oneui.messaging.SingleMessage({
                        description: msgDescriptionFF,
                        title: msgTitleFF,
                        type: "error",
                        dateFormat: { formatLength: "medium", selector: "time" },
                        showId: false,
                        showAction: false,
                        showRefresh: false,
                        showDetailsLink: false,
                        closable: false
                    }).placeAt(this.id + "_ffMessageArea");
                                          
                    this._fmMessageArea = new idx.oneui.messaging.SingleMessage({
                        description: msgDescription,
                        title: msgTitle,
                        type: "information",
                        dateFormat: { formatLength: "medium", selector: "time" },
                        showId: false,
                        showAction: false,
                        showRefresh: false,
                        showDetailsLink: false
                     }).placeAt(this.id + "_fmMessageArea");

                }));

            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){
            
                require([ "idx/widget/SingleMessage" ], dojo.hitch(this, function() {

                    this._ffMessageArea = new idx.widget.SingleMessage({
                        description: msgDescriptionFF,
                        title: msgTitleFF,
                        type: "error",
                        dateFormat: { formatLength: "medium", selector: "time" },
                        showId: false,
                        showAction: false,
                        showRefresh: false,
                        showDetailsLink: false,
                        closable: false
                    }).placeAt(this.id + "_ffMessageArea");
                                          
                    this._fmMessageArea = new idx.widget.SingleMessage({
                        description: msgDescription,
                        title: msgTitle,
                        type: "information",
                        dateFormat: { formatLength: "medium", selector: "time" },
                        showId: false,
                        showAction: false,
                        showRefresh: false,
                        showDetailsLink: false
                     }).placeAt(this.id + "_fmMessageArea");

                }));

                
            }
            
            
            this.set("title", title);

            // Setup Selectors
            this._setupPolicySelector();
            this._setupPoolSelector();
            this._setupButtons();

            if (this.templateIDNum == null) {
                // Create mode               
                this._setupTables();
                dojo.style(this._dualTableSelectorNode, "display", "");
            }
            else {
                // Edit mode
                this._tableArea.setAttribute("style", "display:none");
                this.contentNode.setAttribute("style", "max-height:325px");
                this.dialogNode.resize({
                    w: 650
                });
            }

            dojo.publish("FailoverMonitorComplete");
        },

        onFinish: function() {

            var serverUUIDS = [];
            var serverNames = [];

            // Start Failover
            if (this._selectTable != null) {
                // this should be the case for Create mode
                dojo.forEach(this._selectTable.store._arrayOfAllItems, function(item) {
                    if (item.type[0] != "chassis") {
                        serverUUIDS.push(item.id[0]);
                        serverNames.push(item.name[0]);
                    }
                });
                
                var jsonString = dojo.toJson({
                    "standbyPoolId": this._standbyPoolSelector.get("value"),
                    "failoverPolicyId": this._policyTmplSelector.get("value"),
                    "servers": serverUUIDS
                });
                
                var selectedServers = this._getSelectedServers(serverNames);
                
                var xhrArgs = {
                        url: this.REST_PATH + "failover/start",
                        handleAs: "json",
                        postData: jsonString,
                        sync: true,
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        load: dojo.hitch(this, function(data) {
                            console.log(this.declaredClass + ": Failover started successfully: " + data);
                            this.getModalDialog("Success", this.locResources.START_MONITORING_SUCCESS.replace("%1", selectedServers)).show();
                        }),
                        error: dojo.hitch(this, function(error) {
                            console.log(this.declaredClass + ": Failover could not be started: " + error);
                            this.getModalDialog("Error", this.locResources.START_MONITORING_FAIL.replace("%1", selectedServers)).show();
                        })
                };
                dojo.xhrPost(xhrArgs);
            }
            // Edit Failover
            else {
                // this should be the case for Edit mode, in which case this
                // dialog is only designed to be single-targeted
                serverUUIDS[0] = this.serverUUID;
                
                var jsonString = dojo.toJson({
                    "failover": "EDIT",
                    "standbyPoolId": this._standbyPoolSelector.get("value"),
                    "failoverPolicyId": this._policyTmplSelector.get("value")
                });
                
                var xhrArgs = {
                        url:  this.REST_PATH +"server/"+this.serverUUID,
                        handleAs: "json",
                        putData: jsonString,
                        headers: {
                             "Content-Type": "application/json; charset=utf-8"
                        }
                };
                dojo.xhrPut(xhrArgs);                
            }                        

            // publish successful failover start
            dojo.publish("AvailableFailoverServersModified");
            return true;
        },
        
        _getSelectedServers: function(names) {
            var num = names.length;
            var str = "";
            var i;
            if (num > 0) {
                for (i = 0; i < num; i++) {
                    if (str != "") {
                        str += ", ";
                    }
                    str += names[i];
                }
            }
            return str;
        },

        _cancelClicked: function() {
            this.hide();
            this.destroy();
        },

        _editPolicyTmpl: function() {
            require([ "support/dialogs/FailoverPolicyTemplate" ], dojo.hitch(this, function() {
                var fptDialog = new support.dialogs.FailoverPolicyTemplate({
                    templateIDNum: this._policyTmplSelector.get("value"),
                    templateMode: "edit"
                });
                fptDialog.show();

                return false;
            }));
        },

        _createPolicyTmpl: function() {
            require([ "support/dialogs/FailoverPolicyTemplate" ], dojo.hitch(this, function() {
                var fptDialog = new support.dialogs.FailoverPolicyTemplate();
                fptDialog.show();

                return false;
            }));
        },

        _setupPolicySelector: function() {
            // summary:
            // Setup Power State Template selector...
            // tags:
            // protected
            
            var tmplStore = null;
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "policy?type=failover&list=true&condensed=true",
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: dojo.hitch(this, function(response) {
                    var templateItems = [];
                    var i;
                    var label = this.locResources.NO_POLICY_SELECTED;
                    templateItems.push({
                        value: -1,
                        id: -1,
                        name: label
                    });
                    for (i = 0; i < response.items.length; i++) {
                        templateItems.push({
                            id: response.items[i].id,
                            name: response.items[i].name,
                            type: response.items[i].type,
                            inUse: response.items[i].inUse
                        });
                    }
                    var templateData = dojo.toJson({
                        "identifier": "id",
                        "label": "name",
                        "items": templateItems
                    }, true);
                    tmplStore = new dojo.data.ItemFileWriteStore({
                        data: dojo.fromJson(templateData)
                    });
                })
            });           

            this._policyTmplSelector = new support.widgets.Select({
                id: this.get("id") + "_polTmplSelector",
                name: this.POLTMPL_NAME,
                required: true,
                value: this.newPolicyId,
                store: tmplStore,
                emptyLabel: "&mdash; " + this.locResources.NO_POLICY + " &mdash;",
                style: {
                    width: this.SELECTOR_WIDTH
                },
                sortByLabel: false,
                onChange: dojo.hitch(this, this._changePolTmplCallback)
            }).placeAt(this._policyTmplSelectorArea);

            console.log("VALUE: ", this._policyTmplSelector.get("value"));

            // Also set up a subscriber function to be notified when a new Failover
            // policy is created.
            this.subscribe("FailoverTemplateComplete", function(data) {
                var tmplStore = null;
                var xhrGet = dojo.xhrGet({
                    url: this.REST_PATH + "policy?type=failover&list=true&condensed=true",
                    handleAs: "json",
                    preventCache: true,
                    sync: true,
                    load: dojo.hitch(this, function(response) {
                        var templateItems = [];
                        var i;
                        var label = this.locResources.NO_POLICY_SELECTED;
                        templateItems.push({
                            value: -1,
                            id: -1,
                            name: label
                        });
                        for (i = 0; i < response.items.length; i++) {
                            templateItems.push({
                                id: response.items[i].id,
                                name: response.items[i].name,
                                type: response.items[i].type,
                                inUse: response.items[i].inUse
                            });
                        }
                        var templateData = dojo.toJson({
                            "identifier": "id",
                            "label": "name",
                            "items": templateItems
                        }, true);
                        tmplStore = new dojo.data.ItemFileWriteStore({
                            data: dojo.fromJson(templateData)
                        });
                    })
                });
                this._policyTmplSelector.setStore(tmplStore);
                this._policyTmplSelector.set("value", data);
                this._policyTmplSelector.set("disabled", false);
            });

        },

        _setupPoolSelector: function() {
            // summary:
            // Setup Power State Template selector...
            // tags:
            // protected            
            var tmplStore = null;
            var xhrGet = dojo.xhrGet({              
                url: this.REST_PATH + "pool/standby/list",
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: dojo.hitch(this, function(response) {
                    var templateItems = [];
                    var i;
                    var label = this.locResources.NO_POOL_SELECTED;
                    templateItems.push({
                        value: -1,
                        id: -1,
                        name: label
                    });
                    for (i = 0; i < response.items.length; i++) {
                        templateItems.push({
                            id: response.items[i].id,
                            name: response.items[i].name,
                            type: response.items[i].type,
                            inUse: response.items[i].inUse
                        });
                    }
                    var templateData = dojo.toJson({
                        "identifier": "id",
                        "label": "name",
                        "items": templateItems
                    }, true);
                    tmplStore = new dojo.data.ItemFileWriteStore({
                        data: dojo.fromJson(templateData)
                    });
                })
            }); 

            this._standbyPoolSelector = new support.widgets.Select({
                id: this.get("id") + "_stdbyPoolSelector",
                name: this.STBYPOOL_NAME,
                required: true,
                value: this.newPoolId,
                store: tmplStore,
                emptyLabel: "&mdash; " + this.locResources.NO_POOL + " &mdash;",
                style: {
                    width: this.SELECTOR_WIDTH
                },
                onChange: dojo.hitch(this, this._changePoolCallback)
            }).placeAt(this._standbyPoolSelectorArea);

            console.log("VALUE: ", this._standbyPoolSelector.get("value"));

            this.subscribe("StandbyPoolComplete", function(data) {
                var tmplStore = new dojo.data.ItemFileReadStore({
                    url: this.REST_PATH + "pool/standby/list"
                });

                this._standbyPoolSelector.setStore(tmplStore);
                // Add option to select no template.
                var label = this.locResources.NO_POOL_SELECTED;
                this._standbyPoolSelector.addOption({
                    value: -1,
                    label: label
                });
                this._standbyPoolSelector.set("value", data);
                this._standbyPoolSelector.set("disabled", false);
            });

        },

        _setupButtons: function() {
            this._editTmplButton = new dijit.form.Button({
                showLabel: false,
                iconClass: "spEditInactiveLgIcon",
                disabled: true,
                title: this.locResources.EDIT_TEMP,
                onClick: dojo.hitch(this, function() {
                    this._editPolicyTmpl();
                })
            }).placeAt(this._tmplEditOption);
            this._createTmplButton = new dijit.form.Button({
                showLabel: false,
                iconClass: "spCreateLgIcon",
                disabled: false,
                title: this.locResources.NEW_TEMP,
                onClick: dojo.hitch(this, function() {
                    this._createPolicyTmpl();
                })
            }).placeAt(this._tmplCreateOption);

            this._editPoolButton = new dijit.form.Button({
                showLabel: false,
                iconClass: "spEditInactiveLgIcon",
                disabled: true,
                title: this.locResources.EDIT_POOL,
                onClick: dojo.hitch(this, function() {
                    this._editStandbyPool();
                })
            }).placeAt(this._poolEditOption);
            this._createPoolButton = new dijit.form.Button({
                showLabel: false,
                iconClass: "spCreateLgIcon",
                disabled: false,
                title: this.locResources.NEW_POOL,
                onClick: dojo.hitch(this, function() {
                    this._createStandbyPool();
                })
            }).placeAt(this._poolCreateOption);
            
        },

        _changePolTmplCallback: function() {
            var value = this._policyTmplSelector.get("value");
            
            if (value && value != "-1") {
                this._editTmplButton.set("disabled", false);
                this._editTmplButton.set("iconClass", "spEditLgIcon");
            }
            else {
                this._editTmplButton.set("disabled", true);
                this._editTmplButton.set("iconClass", "spEditInactiveLgIcon");
            }
            
            this._validate();
        },

        _changePoolCallback: function() {
            var value = this._standbyPoolSelector.get("value");
            
            if (value && value != "-1") {
                this._editPoolButton.set("disabled", false);
                this._editPoolButton.set("iconClass", "spEditLgIcon");
            }
            else {
                this._editPoolButton.set("disabled", true);
                this._editPoolButton.set("iconClass", "spEditInactiveLgIcon");
            }
            
            this._validate();
        },

        _editStandbyPool: function() {
            require([ "support/dialogs/StandbyPoolTemplate" ], dojo.hitch(this, function() {
                var fptDialog = new support.dialogs.StandbyPoolTemplate({
                    templateIDNum: this._standbyPoolSelector.get("value")
                });
                fptDialog.show();

                return false;
            }));
        },

        _createStandbyPool: function() {
            require([ "support/dialogs/StandbyPoolTemplate" ], dojo.hitch(this, function() {
                var sptDialog = new support.dialogs.StandbyPoolTemplate();
                sptDialog.show();

                return false;
            }));
        },

        _setupTables: function() {
                        
            this._setupAvailableTable();
            this._setupSelectedTable();            
            this._dualTableSelector = new support.panels.DualTableSelector({
                _availTable: this._availTable,
                _selectTable: this._selectTable
            }).placeAt(this._dualTableSelectorNode);

            dojo.connect(this._dualTableSelector, "_moveRight", this, this._validate);
            dojo.connect(this._dualTableSelector, "_moveAllRight", this, this._validate);
            dojo.connect(this._dualTableSelector, "_moveLeft", this, this._validate);
            dojo.connect(this._dualTableSelector, "_moveAllLeft", this, this._validate);
        },

        _setupAvailableTable: function() {

            // summary:
            // Set up available targets table widget...
            // tags:
            // protected
            if (this._availTable) {
                // Destroy old table and start from scratch.
                this._availTable.destroy();
            }
            
            var tableStore = this._retrieveStore();
            var tableStructure = this._getTableStructure();
            this._availTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.availGridID, true, true);
            this._availTable.column(0).sort(false);
        },
               
        _retrieveStore: function() {
            
            var availArray = new Array();
            var selectedArray = new Array();
            
            var availData = "";
            
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH
                        + "server?inclSrvrsWithProfile=true&numberOfNotMonitoredServers=false",
                handleAs: "json",
                sync: true,
                load: dojo.hitch(this, function(response) {
                    var i,j,k;
                    
                    if (response && response.items) {
                        for (i = 0; i < response.items.length; i++) {
                            var availParent = null;
                            var rootObject = response.items[i];                                
                            
                            if (rootObject.children) {
                                for (j = 0; j < rootObject.children.length; j++) {
                                    
                                    var selectedParent;
                                    
                                    if (rootObject.children[j].ites.length > 1) {
                                        for (k = 0; k < rootObject.children[j].ites.length; k++) {
                                            var ff = rootObject.children[j].ites[k].formFactor;

                                            //  Set bay label with subnode indication if 1sd or 2-bay indication if 4s or pxu.
                                            if (ff == "1sd")
                                                rootObject.children[j].ites[k].bays[0] = rootObject.children[j].ites[k].bays[0] + ":" + rootObject.children[j].ites[k].subBays[0];                                                                                    
                                            else if (ff == "4s" || ff == "pxu")
                                                rootObject.children[j].ites[k].bays[0] = rootObject.children[j].ites[k].bays[0] + ", " + rootObject.children[j].ites[k].bays[1];                                                                                    
                                            
                                            // Assign Arch Type
                                            var type = (rootObject.children[j].ites[k].architecture) ? 
                                                    this.determineType(rootObject.children[j].ites[k].architecture) : "";
                                                    
                                            rootObject.children[j].ites[k].archType = type;
                                            
                                            // Set parent ID
                                            rootObject.children[j].ites[k].parentId = rootObject.id;
                                            
                                            // Add to list
                                            if (rootObject.children[j].ites[k].id == this.serverUUID) {
                                                // add to "selected" array
                                                if (selectedParent == null) {
                                                    selectedParent = this._createParent(rootObject);
                                                }
                                                selectedParent.children.push(rootObject.children[j].ites[k]);
                                            }
                                            else {
                                                // add to "available" array
                                                if (availParent == null) {
                                                    availParent = this._createParent(rootObject);
                                                }
                                                availParent.children.push(rootObject.children[j].ites[k]);
                                            }                                            
                                        }
                                    }
                                    else {
                                        var ff = rootObject.children[j].ites[0].formFactor;

                                        //  Set bay label with subnode indication if 1sd or 2-bay indication if 4s or pxu.
                                        if (ff == "1sd")
                                            rootObject.children[j].ites[0].bays[0] = rootObject.children[j].ites[0].bays[0] + ":" + rootObject.children[j].ites[0].subBays[0];                                                                                    
                                        else if (ff == "4s" || ff == "pxu")
                                            rootObject.children[j].ites[0].bays[0] = rootObject.children[j].ites[0].bays[0] + ", " + rootObject.children[j].ites[0].bays[1];                                                                                    
                                        
                                        // Assign Arch Type
                                        var type = (rootObject.children[j].ites[0].architecture) ? 
                                                this.determineType(rootObject.children[j].ites[0].architecture) : "";
                                                
                                        rootObject.children[j].ites[0].archType = type;
                                        
                                        // Set parent ID
                                        rootObject.children[j].ites[0].parentId = rootObject.id;
                                        
                                        // Add to list
                                        if (rootObject.children[j].ites[0].id == this.serverUUID) {
                                            // add to "selected" array
                                            if (selectedParent == null) {
                                                selectedParent = this._createParent(rootObject);
                                            }
                                            selectedParent.children.push(rootObject.children[j].ites[0]);
                                        }
                                        else {
                                            // add to "available" array
                                            if (availParent == null) {
                                                availParent = this._createParent(rootObject);
                                            }
                                            availParent.children.push(rootObject.children[j].ites[0]);
                                        }                                        
                                    }
                                }
                            }
                            // if any servers are available, add them
                            // to the array object
                            if (availParent != null) {
                                availArray.push(availParent);
                            }
                            // if any servers were selected, add them
                            // to the array object
                            if (selectedParent != null) {
                                selectedArray.push(selectedParent);
                            }                            
                        }
                    }
                    
                    availData = dojo.toJson({
                        "identifier": "id",
                        "label": "name",
                        "items": availArray
                    }, true);
                })
            });
            
            // Create selected store
            
            // if items were pre-selected, set them into the
            // "selected" data store
            var selectedItems = [];
            if (selectedArray != null) {
                selectedItems = selectedArray;
            }
            var emptyData = dojo.toJson({
                    "items": selectedItems,
                    "label": "name",
                    "identifier": "id"
            }, true);
            this._selectedStore = new dojo.data.ItemFileWriteStore({
                data: dojo.fromJson(emptyData)
            });
            
            if (!("hasChildren" in this._selectedStore)) {
                this._selectedStore.hasChildren = function(id, item) { 
                    return item && this.getValues(item, "children").length;
                };
            };
            
            if (!("getChildren" in this._selectedStore)) {
                this._selectedStore.getChildren = function(item) { 
                    return this.getValues(item, "children");
                };
            };

            
            // Create available store
            var availStore = new dojo.data.ItemFileWriteStore({
                data: dojo.fromJson(availData)
            });
            
            if (!("hasChildren" in availStore)) {
                availStore.hasChildren = function(id, item) { 
                    return item && this.getValues(item, "children").length;
                };
            };
            
            if (!("getChildren" in availStore)) {
                availStore.getChildren = function(item) { 
                    return this.getValues(item, "children");
                };
            };
            
            return availStore;
            
        },
        
        // Columns to be displayed on table
        _getTableStructure: function() {
            var i;
            var layout = new Array();
            
            layout.push({
                field: "name",
                name: this.resources.NAME,
                width: "150px"
            });
            layout.push({
                field: "bays",
                name: this.resources.BAY,
                width: "35px"
            });
            layout.push({
                field: "access",
                name: this.resources.ACCESS,
                width: "50px",
                decorator: dojo.hitch(this, "_accessDecorator"),
                formatter: dojo.hitch(this, "_accessFormatter")
            });                      
            layout.push({
                field: "formFactor",
                name: this.resources.FORM_FACTOR,
                formatter: dojo.hitch(this, "_formatFormFactor"),
                width: "auto"
            });

            return layout;
        },
        
        _accessFormatter: function(item) {

            // summary:
            // Formats "access" field value for display in TreeTable.
            // tags:
            // protected

            var access = item.access;
            var accessText = "";
            
            if (access == "ok") { // Access "OK"
                accessText = this.resources.OK;
            }
            else if (access == "-Default-") { // Access "Default Access"
                accessText = this.resources.ACCESS_DEFAULT;
            }
            else if (access == "offline") { // Access "Offline"
                accessText = this.resources.ACCESS_OFFLINE;
            }
            else if (access == "placeholder") { // Access "Placeholder"
                accessText = this.resources.PLACEHOLDER;
            }
            else if (access == "none") { // Access "No access"
                accessText = this.resources.ACCESS_NONE;
            }
            else if (access == "partial") { // Access "Partial access"
                accessText = this.resources.ACCESS_PARTIAL;
            }
            else /*(access == "unknown")*/ { // Access "Unknown"
                accessText = this.resources.ACCESS_UNKNOWN;
            }
            
            return accessText;
        },

        _setupSelectedTable: function() {

            // summary:
            // Set up selected targets table widget...
            // tags:
            // protected
            if (this._selectTable) {
                // Destroy old table and start from scratch.
                this._selectTable.destroy();
            }
            
            // Setup table widget...
            var tableStore = this._selectedStore;
            var tableStructure = this._getTableStructure();
            this._selectTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.selectGridID, true, true);
            this._selectTable.column(0).sort(false);

        },

        _presetTemplateCallback: function(size, request) {
            // tags:
            // protected
            // No templates for this category?
            if (size <= 0) {
                // Disable selection and allow "emptyLabel" to display.
                request.selector.set("disabled", true);
                return;
            }
            
            request.selector.set("disabled", false);
            
            // Add option to select no template.
            var label = (request.selector.name == this.POLTMPL_NAME) ? this.locResources.NO_POLICY_SELECTED : this.locResources.NO_POOL_SELECTED;
            
            request.selector.addOption({
                value: -1,
                label: label
            });
            
            switch (request.selector.name) {
                case this.POLTMPL_NAME:                    
                    return request.selector.set("value", (this.newPolicyId) ? this.newPolicyId : -1);
                    break;
                case this.STBYPOOL_NAME:
                    return request.selector.set("value", (this.newPoolId) ? this.newPoolId : -1);
                    break;
            }
        },

        _getNumSelectedServers: function() {
            if (this._selectTable) {
                return this._selectTable.store._arrayOfTopLevelItems.length;
            }
            return -1;
        },

        _validate: function() {
            
            var poolID = this._standbyPoolSelector.get("value");
            var poolFormFactor;
            
            var formFactorMatch = this._checkFormFactor();
            
            console.log("Form Factor Matches: ", formFactorMatch);
            
            if (!formFactorMatch) {
                this._ffMismatchError.style.display = "inline";
                this._ffMessageArea.resize();
            } else {
                this._ffMismatchError.style.display = "none";
                this._ffMessageArea.resize();
            }
            
            if (this.templateIDNum == null) {
                if (this._getNumSelectedServers() > 0 && this._policyTmplSelector.get("disabled") == false
                        && this._policyTmplSelector.get("value") > -1 && this._standbyPoolSelector.get("disabled") == false
                        && this._standbyPoolSelector.get("value") > -1 && formFactorMatch) {
                    this._startBtn.set("disabled", false);
                    return;
                }
            } 
            else {
                if (this._policyTmplSelector.get("disabled") == false && this._policyTmplSelector.get("value") > -1
                        && this._standbyPoolSelector.get("disabled") == false
                        && this._standbyPoolSelector.get("value") > -1 && formFactorMatch) {
                    this._startBtn.set("disabled", false);
                    return;
                }
            }
            this._startBtn.set("disabled", true);
        },
              
        _checkFormFactor: function() {
            var poolFormFactor;
            var i,j;
            var selectedServers;
            var poolID = this._standbyPoolSelector.get("value");
                
            // If table exists, it is not edit mode...
            if (this._selectTable) {
                selectedServers = this._selectTable.store._arrayOfAllItems;            
                                        
                if (selectedServers[0] == null) { 
                    return true;
                }
            
            }
            
            if (poolID == "" || poolID < 0) {
                return true;
            }
            
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "pool/standby/" + poolID + "?showFormFactor=true", 
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: dojo.hitch(this, function(response) {
                    poolFormFactor = response.formFactor;
                }),
                error: function(error) {
                    console.log(this.declaredClass + ": Could not obtain form factor: " + error);
                }
            });
                               
            console.log("Pool Form Factor: ", poolFormFactor);
            
            // Check for selected servers that do not match the standby server pool's form factor
            if (this._selectTable) {
                for (i = 0; i < selectedServers.length; i++) {
                    var formFactor;
                    
                    if (selectedServers[i] == null) {
                        continue;
                    }
                                                   
                    if (selectedServers[i].type == "chassis") { 
                        var selected = selectedServers[i].children;
                        for (j = 0; j < selected.length; j++) {
                            console.log(selected[j]);
                            formFactor = selected[j].formFactor;
                            if (!this._checkForMatch(poolFormFactor, formFactor)) {
                                return false;
                            }
                        }
                    }
                    else {
                        console.log(selectedServers[i]);
                        formFactor = selectedServers[i].formFactor;
                        if (!this._checkForMatch(poolFormFactor, formFactor)) {
                            return false;
                        }
                    }                                
                }
            }
            // Edit Mode
            else {
                return this._checkForMatch(poolFormFactor, this.newFormFactor);
            }
            
            return true;
        },
        
        _checkForMatch: function(poolFormFactor, serverFormFactor) {
            var i;            
            for (i = 0; i < poolFormFactor.length; i++) {
                if (serverFormFactor == poolFormFactor[i]) {
                    return true;
                }
            }
            return false;
        },
        
        _createParent: function(parentObj) {
            return {
                "access": parentObj.access,
                "type": parentObj.type,
                "name": parentObj.name,
                "id": parentObj.id,
                "children": []
            };
        },

        // Determine internal usage type
        determineType: function(arch) {
            if (arch) {
                if (arch == "x86_32" || arch == "x86_64" || arch == "x86") {
                    return "sysx";
                }
                else if (arch == "ppc32" || arch == "ppc64") {
                    return "power";
                }
            }
            return "";
        },
        
        _formatFormFactor: function(item) {
            var value = item.formFactor;
            if (value == "2s") {
                return this.locResources["COMPUTE_NODE_OPT"].replace("%1", "1");
            }
            else if (value == "1sd") {
                return this.locResources["COMPUTE_NODE_DUAL_OPT"].replace("%1", "1");
            }
            else if (value == "4s") {
                return this.locResources["COMPUTE_NODE_OPT"].replace("%1", "2");
            }
            else if (value == "pxu") {
                return this.locResources["EXPANSION_NODE_OPT"].replace("%1", "2");
            }
            
            return null;
        }

    });
});
