define(["dojo", 
        "dijit", 
        "support/dialogs/Dialog", 
        "dojo/i18n!./nls/FailoverPolicyTemplate",
        "dojo/text!./templates/FailoverPolicyTemplate.html", 
        "dijit/form/SimpleTextarea", 
        "dijit/form/CheckBox",
        "support/dialogs/SaveAs",
        "dijit/form/ToggleButton", 
        "dijit/form/ValidationTextBox", 
        "dijit/Dialog", 
        "dojo/parser", 
        "support/widgets/ToggleButton", 
        "dijit/layout/BorderContainer",
        "dijit/layout/ContentPane", 
        "support/panels/Panel", 
        "support/panels/CheckButton",
        "support/dialogs/ReDeployServerTemplate",
        "dijit/Tooltip"
        ], function(dojo, dijit, dialog, mri, template) {
    
    return dojo.declare("support.dialogs.FailoverPolicyTemplate", [ support.dialogs.Dialog ], {
        
        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
        
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        maxPercent: .40,
        dialogRef: null,
        templateIDNum: null,
        intitialData: null,
        finalData: null,
        predefined: false,
        inUse: false,
        _foverServerType: "any",
        templateMode : "create",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/FailoverPolicyTemplate.css",
        
        POL_NAME: "failover",
        

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);
                        
            dojo.connect(this.dialogNode, "hide", this, function() {        
                this.destroy();
            });
            
          //Setup for template new, edit, or copy mode 
            this._setupSettingsArea();
            this._setup();
            
          //Controls dialog dynamic resizing max height 
            var availHeight = window.screen.availHeight;
            var maxHeight = availHeight * this.maxPercent;
            this.settingsContainer.setAttribute("style", "max-height:" + maxHeight + "px");
                                                                                              
            //Resizes dialog
            this.layout(); 
            
        },
        
        loadprevSettings: function() {
            var store = null;
            var dialogNode = this.dialogNode;
            //var warningMessage = this.warningMessage;

            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "policy/" + this.templateIDNum +"?type=failover",
                handleAs: "json",
                sync: true,
                preventCache: true,
                load: dojo.hitch(this, function(response) {
                    store = response; 
                })
            });
            
            this.initialData = store;
            
            // Set dialog title to reflect Editing and display the warning message
            //warningMessage.style.display = "block";
            dialogNode.set("title", this.locResources.TITLE_EDIT + " - " + store.name);
           
            var descriptionvalue = store.description;
            var description = this.decodeSpecialCharacters(descriptionvalue);
            this.templateDescription.set("value", description);
            
            if(this.templateMode == "copy"){
                dialogNode.set("title", this.locResources.TITLE_COPY + " - " + store.name);
                this.templateName.set("value", store.name + " - " + this.resources.COPY);
            }
            else{
                this.templateName.set("value", store.name);
            }
            this.templateName.set("disabled", this.inUse || this.predefined);
            
            //Failover Criteria
            this._powerCheck.set("value",store.conditions.pwrOff);
            this._powerCheck.set("disabled" ,this.predefined);
            this._CPUCheck.set("value", store.conditions.cpuFail);
            this._CPUCheck.set("disabled" ,this.predefined);
            this._BladeCommCheck.set("value", store.conditions.bladeCommErr);
            this._BladeCommCheck.set("disabled" ,this.predefined);
            this._BladeRemCheck.set("value",store.conditions.bladeRemove);
            this._BladeRemCheck.set("disabled" ,this.predefined);
            this._HDDCheck.set("value",store.conditions.HDDFail);
            this._HDDCheck.set("disabled" ,this.predefined);
            this._deniedCheck.set("value",store.conditions.deniedPwr);
            this._deniedCheck.set("disabled" ,this.predefined);
            this._memoryCheck.set("value",store.conditions.memFail);
            this._memoryCheck.set("disabled" ,this.predefined);
            this._voltageCheck.set("value",store.conditions.voltWarn);
            this._voltageCheck.set("disabled" ,this.predefined);
            this._PFACheck.set("value",store.conditions.PFAEvt);
            this._PFACheck.set("disabled" ,this.predefined);
            
            //Failover Server Attributes
            if(store.failoverServerAttr.serverCriteria){
//                this._FailoverMatch.set("value", true);
            }
//            this._FailoverMatch.set("disabled" ,this.predefined);
//            this._FailoverAnyBlade.set("disabled" ,this.predefined);
            this._FailoverAnyPowerStatus.set("value",store.failoverServerAttr.pwrStateDontCare);
            this._FailoverAnyPowerStatus.set("disabled" ,this.predefined);
            
            if (store.failoverServerAttr.resetToDefaults != null) {
	            this._ResetOption.set("value", store.failoverServerAttr.resetToDefaults);
    	        this._ResetOption.set("disabled" ,this.predefined);
    	    } else {
    	    	this._ResetOption.set("value", false);
    	        this._ResetOption.set("disabled" ,this.predefined);
    	    }
            
            //Additional Criteria (future release
//            this._CopyVLAN.set("value",store.additionalOpts.copyVLAN);
//            this._CopyVLAN.set("disabled" ,this.predefined);

        },
        
        cancelClicked: function() {
            this.hide();
        },

               
        exitParent: function() {
            var data = "";
            
            dojo.publish("FailoverTemplateComplete", data);

            if(this._rdstDialog != null){
                var deployClicked = this._rdstDialog.get("closeParent");
                if(deployClicked) {
                    this.hide();
                }
            }
        },
        
        _generateSaveData : function() {
            var templateObject = new Object();
            if(this.templateIDNum){
                //templateObject.uri = this.initialData[0].uri;
            }
            templateObject.name = this.templateName.get("value").trim();

            if(this.templateIDNum != null && this.templateMode != "copy"){                
                templateObject.id = this.templateIDNum;
            }else{
                this.templateIDNum = null;
            }
       
            templateObject.type = this.POL_NAME;
            
            var description = this.templateDescription.get("value").trim();
            templateObject.description = this.encodeSpecialCharacters(description);
            
            //Server Condition Criteria
            var condObj = new Object();
            condObj.pwrOff = this._powerCheck.get("value") == "on";
            condObj.cpuFail = this._CPUCheck.get("value")  == "on";
            condObj.bladeCommErr = this._BladeCommCheck.get("value")  == "on";
            condObj.bladeRemove = this._BladeRemCheck.get("value") == "on";
            condObj.HDDFail = this._HDDCheck.get("value") == "on";
            condObj.deniedPwr = this._deniedCheck.get("value") == "on";
            condObj.memFail = this._memoryCheck.get("value") == "on";
            condObj.voltWarn = this._voltageCheck.get("value") == "on";
            condObj.PFAEvt = this._PFACheck.get("value") == "on";
            templateObject.conditions = condObj;
            
            //Failover Server Attributes
            var failObj = new Object();
            failObj.serverCriteria = this._foverServerType;
            failObj.pwrStateDontCare = this._FailoverAnyPowerStatus.get("value") == "on";
            failObj.resetToDefaults = this._ResetOption.get("value") == "on";
            templateObject.failoverServerAttr = failObj;
            
            //Additional Criteria (future release)
//            var addit = new Object();
//            addit.copyVLAN = this._CopyVLAN.get("value") == "on";
//            templateObject.additionalOpts = addit;

            this.finalData = templateObject;
            
            return templateObject;
        },

        _saveTemplate: function(templateObject) {
 
            var argJSONString = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": (templateObject) ? [ templateObject ] : []
            }, true);

            
            var hideDialog = true;
            
            var xhrArgs = {
                url: this.REST_PATH + "policy?type=failover&action=create",
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                postData: argJSONString,
                sync: true,
                load: function(data) {
                    // alert("Template Create Successfully: ID " + data);
                    // alert("post data: " + data);
                    dojo.publish("FailoverTemplateComplete", data);
                },
                error: dojo.hitch(this, function(error, ioArgs) {
                    // Failure path! We received a non-2xx status code (ie: 3xx/4xx/5xx status code) from the API.

                    hideDialog = false;                    

//                    console.log(this.declaredClass + ": ", error, ioArgs);

                    var response = dojo.fromJson(ioArgs.xhr.responseText);
                    
                    if ((ioArgs.xhr.status >= 400) && (response && (response.message == "service_error_name_in_use"))) {
                        // Error Template name in use
                        this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                    }
                    else if (ioArgs.xhr.status == 404) {
                        // Edit Template ID doesn't exist
                        this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                    }
                    else if (ioArgs.xhr.status >= 500) {
                        // Internal Programming Error
                        this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                    }
                    else {
                        this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                    }                    
                })
            };

            dojo.xhrPost(xhrArgs);

            // this.clearprevSettings();
            if (hideDialog)
                this.hide();
        },
        
        _hasTemplateChanged: function(initial,end){
            if(initial.id != end.id){
                alert("ID CHANGED");
                return true;
            }
            if(initial.name != end.name){
                alert("NAME CHANGED");
                return true;
            }
            return false;
        },
        
        _selectAll : function(){
            
        },
        
        _setupSettingsArea : function(){

            //Failover Conditions
            this._powerCheck = new dijit.form.CheckBox({
                id: "cp_failoverpolicytemplate_powercheck",
                title : this.locResources.PWR_OFF,
                disabled: this.predefined,
                checked : false
            }).placeAt(this._powerCheck);
            
            this._CPUCheck = new dijit.form.CheckBox({
                id: "cp_failoverpolicytemplate_cpucheck",
                title : this.locResources.CPU_FAIL,
                disabled: this.predefined,
                checked : false
            }).placeAt(this._CPUCheck);
            
            this._BladeCommCheck = new dijit.form.CheckBox({
                id: "cp_failoverpolicytemplate_bladecommcheck",
                title : this.locResources.BLADE_COMM,
                disabled: this.predefined,
                checked : false
            }).placeAt(this._BladeCommCheck);
            
            this._BladeRemCheck = new dijit.form.CheckBox({
                id: "cp_failoverpolicytemplate_bladeremcheck",
                title : this.locResources.BLADE_REM,
                disabled: this.predefined,
                checked : false
            }).placeAt(this._BladeRemCheck);
            
            this._HDDCheck = new dijit.form.CheckBox({
                id: "cp_failoverpolicytemplate_hddcheck",
                title : this.locResources.HDD_FAIL,
                disabled: this.predefined,
                checked : false
            }).placeAt(this._HDDCheck);
            
            this._deniedCheck = new dijit.form.CheckBox({
                id: "cp_failoverpolicytemplate_deniedcheck",
                title : this.locResources.NO_PWR,
                disabled: this.predefined,
                checked : false
            }).placeAt(this._deniedCheck);
            
            this._memoryCheck = new dijit.form.CheckBox({
                id: "cp_failoverpolicytemplate_memorycheck",
                title : this.locResources.MEM_FAIL,
                disabled: this.predefined,
                checked : false
            }).placeAt(this._memoryCheck);
            
            this._voltageCheck = new dijit.form.CheckBox({
                id: "cp_failoverpolicytemplate_voltagecheck",
                title : this.locResources.VOLT_WARN,
                disabled: this.predefined,
                checked : false
            }).placeAt(this._voltageCheck);
            
            this._PFACheck = new dijit.form.CheckBox({
                id: "cp_failoverpolicytemplate_pfacheck",
                title : this.locResources.PVA_EVT,
                disabled: this.predefined,
                checked : false
            }).placeAt(this._PFACheck);
            
//  99289 enable for next release
//            //Failover Server Criteria
//            this._FailoverMatch = new dijit.form.RadioButton({
//                title : this.locResources.FOVER_MATCH,
//                disabled: this.predefined,
//                checked : true,
//                value : "match",
//                onClick : dojo.hitch(this, function(){
//                        this._foverServerType = "match";
//                })
//            }).placeAt(this._FailoverMatch);
//            
//            this._FailoverAnyBlade = new dijit.form.RadioButton({
//                title : this.locResources.FOVER_ANY,
//                disabled: this.predefined,
//                checked : false,
//                value : "any",
//                onClick : dojo.hitch(this, function(){
//                    this._foverServerType = "any";
//                })
//            }).placeAt(this._FailoverAnyBlade);
            
            this._FailoverAnyPowerStatus = new dijit.form.CheckBox({
                id: "cp_failoverpolicytemplate_failoveranypowerstatuscheck",
                title : this.locResources.FOVER_PWRSTATE,
                disabled: this.predefined,
                checked : true
            }).placeAt(this._FailoverAnyPowerStatus);
            
            this._ResetOption = new dijit.form.CheckBox({
                id: "cp_failoverpolicytemplate_resetoptioncheck",
                title : this.locResources.RESET_OPTION,
                disabled: this.predefined,
                checked : false,
                onClick: dojo.hitch(this, this._resetSelected)
            }).placeAt(this._ResetOption);
            
            //Additional Options (future release)
//            this._CopyVLAN = new dijit.form.CheckBox({
//                title : this.locResources.VLAN_COPY,
//                disabled: this.predefined,
//                checked : false,    
//            }).placeAt(this._CopyVLAN);
        },
        
        saveasClicked : function(){
            var data = this._generateSaveData();
            this.saveasDialog.set("templateObject",data);
            this.saveasDialog.set("saveTemplateType",this.POL_NAME);
            this.saveasDialog.show();
        },
        
        saveClicked : function(){
            this._saveTemplate(this._generateSaveData());
        },
        
        createClicked : function(){
            this._saveTemplate(this._generateSaveData());
        },
        
        _setup: function(){

            //Set Template Field to catch intermediate changes
            this.templateName.intermediateChanges = true;

            // Set Description Label
            this._descLimit.innerHTML = this.resources.LIMIT.replace("%500","500");

            //Set Information Messages and Load Previous Settings                      
            if(this.templateIDNum != null && this.templateMode == "edit"){     //if dialog in edit mode    
                                               
                if(this.templateInUse.toString() == "true"){
                
                    if(this.templateLaunchPoint == "table"){
                        
                        this.warningMessage.style.display = "block";
                        this.warningMessageText.innerHTML = this.resources.INUSE;
                        this.saveBtnContainer.style.display = "inline";
                        this.saveasBtnContainer.style.display = "inline";
                        
                    }else if(this.templateLaunchPoint == "wizard"){
                        
                        this.informationMessage.style.display = "block";
                        this.informationMessageText.innerHTML = this.resources.INUSEWIZARD;
                        this.saveasBtnContainer.style.display = "inline";
                        
                    }else{
                        this.informationMessage.style.display = "block";
                        this.informationMessageText.innerHTML = this.resources.INUSEWIZARD;
                        this.saveasBtnContainer.style.display = "inline";
                    }
                    
                }else{
                    
                    this.saveBtnContainer.style.display = "inline";
                    this.saveasBtnContainer.style.display = "inline";
                                        
                }    

                this.templateName.set("disabled",true);
                
                this.loadprevSettings();


             }else if(this.templateIDNum != null && this.templateMode == "copy"){    //if dialog in copy mode     
                 
                 this.loadprevSettings();
                 this.templateIDNum = null;  //set template ID to null in order to create new template
                 this.okBtnContainer.style.display = "inline";

             }else{     //if dialog in create new mode
                 
                 this.okBtnContainer.style.display = "inline";
                 
             } 

        },
        
        _nameChanged: function() {                        
            
            if (this.templateName.get("value").trim() != "") {
                this.okBtn.set("disabled", false);
            }
            else {
                this.okBtn.set("disabled", true);
            }
            
        },
        
        _resetSelected: function() {
            
            if (this._ResetOption.get("value") == "on") {
                this.confirmReset();
                this._ResetOption.set("checked", false);
            }

        },
        
        confirmReset: function() {
            
            var confirmedReset = function() {
                this._ResetOption.set("value", "on");
            };

            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.OK,
                        type: "Warning",
                        text: this.locResources.RESET_CONFIRM,
                        info: this.locResources.RESET_WARNING                    
                    });
                 
                    confirmDialog.startup();
                    confirmDialog.confirm(confirmedReset, this);
            
                }));
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.OK,
                        type: "Warning",
                        text: this.locResources.RESET_CONFIRM,
                        info: this.locResources.RESET_WARNING                    
                    });
                 
                    confirmDialog.startup();
                    confirmDialog.confirm(confirmedReset, this);
            
                }));

            }
        }                

    });
});
