define([
        "dojo",
        "dijit",
        "support/dialogs/Dialog",
        "dojo/i18n!./nls/NewAdapterTemplate",
        "dojo/text!./templates/NewAdapterTemplate.html",
        "dijit/form/SimpleTextarea",
        "dijit/form/ValidationTextBox",
        "dijit/layout/ContentPane",
        "dojo/data/ItemFileReadStore",
        "support/panels/CheckButton",
        "dijit/form/CheckBox",
        "support/widgets/Select",
        "support/dialogs/SaveAs",
        "support/dialogs/ReDeployServerTemplate",
        "support/dialogs/SaveAs",
        "support/dialogs/ProgressDialog"
    ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.NewAdapterTemplate", [ support.dialogs.Dialog], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        baseClass: "NewAdapterTemplate",
        
        templateIDNum: null,
        
        // UI spec test settings
        testAdapterData: null,
        runningSpecTest: false,

        _adapterTypeSelector: null,
        _adapterModeSelector: null,
        _adapterModeSelector2: null,
        _adapterModeSelector3: null,
        _adapterProtocolSelector: null,
        _adapterProtocolSelector2: null,
        _adapterProtocolSelector3: null,

        // _firstHalfSelector: null,
        // _secondHalfSelector: null,
        _adapterTypeStore: null,
        _adapterTypeData: null,
        _supportedIOAdaptersList: null,
        
        _modeAndOpcodeArray: null,
        _controllerObjectArray: null,
        _numberOfControllers: 1,
        _modeSelectorArray: null,
        _protocolSelectorArray: null,
        
        _groupheading1: null,
        _groupheading2: null,
        
        _pattern: null,
        
        _protocolSelectorValue: "ethOnly",
        _protocolSelectorValue2: "ethOnly",
        
        _defaultSelectorWidth: "250px",
        

        // 2 port = fabric connector, 4 port = expansion virtual fabric adapter
        adapterPorts: null,
        adapterId: "lom:2",
        adapterType: "vfa", //"Embedded 1 Gb Ethernet Controller (LOM) with Fabric Connector",

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/NewAdapterTemplate.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
            
            this._groupheading1 = this.locResources.CONFIGPORTGROUP.replace("%1", "1");
            this._groupheading2 = this.locResources.CONFIGPORTGROUP.replace("%1", "2");
            
            this._readAdapterList();
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);

            this._descLimit.innerHTML = this.resources.LIMIT.replace("%500","500");
            
            this.templateName.intermediateChanges = true;
            

            if (this.templateIDNum != null) {
                this._readSettingsFromService();                
            } 
            
            this._setGlobalValues();

            this._setDefaultDisplayState();
            this._createSelectBoxes();
            
            if (this.templateIDNum != null) {
                this._setSelectorsFromSettings("both");
            }

            dojo.connect(this.dialogNode, "hide", this, function() {
                this.destroy();
            });

            //Setup for template new, edit, or copy mode 
            this._setup();

            // Apply env specific tooltip for the page
            // NOTE: For dijit/Tooltip dijits, set the label, not innerHTML.
            this._protocolstip.label = "<div style=\"width: 30em;\">" + this.locResources.PROTOCOLSHOVER.replace("%1", this.PRODUCT_NAME_LONG) + "</div>";
            this._protocolstip2.label = "<div style=\"width: 30em;\">" + this.locResources.PROTOCOLSHOVER.replace("%1", this.PRODUCT_NAME_LONG) + "</div>";
            this._protocolstip3.label = "<div style=\"width: 30em;\">" + this.locResources.PROTOCOLSHOVER.replace("%1", this.PRODUCT_NAME_LONG) + "</div>";

        },

        _readAdapterList: function() {
            var adapterData = null;
            
            if(!this.runningSpecTest) {
                var xhrArgs = {
                    url: this.REST_PATH + "supportedDevices/ioAdapter/list",
                    handleAs: "json",
                    sync: true,
                    load: function(data) {
                        adapterData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
                };

                // Call the asynchronous xhrGet
                dojo.xhrGet(xhrArgs);
            } else {
                adapterData = this.testAdapterData;
            }

            var count;
            for (count = 0; count < adapterData.items.length; ++count) {         
                if (adapterData.items[count].id == this.adapterId) {
                    this.adapterPorts = adapterData.items[count].portCount;       
                }
            }
            this._adapterTypeData = adapterData;
            this._adapterTypeStore = new dojo.data.ItemFileReadStore({
                data: this._adapterTypeData
            });
        },
        
        _readSettingsFromService: function() {

            var pattern = null;
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "template/" + this.templateIDNum,
                handleAs: "json",
                sync: true,
                load: function(response) {
                    pattern = (response && response.items && (response.items.length > 0)) ? response.items[0] : null;
                },
                error: function(error) {
                    alert(this.locResources.ERRORTEMPLATE);
                }
            });
            if (pattern) {
                this._pattern = pattern;
                
                this.adapterId = this._pattern.adapterType;
                this._numberOfControllers = this._pattern.controllers.length;
                this.adapterDesc.set("value", this._pattern.description);
                this.templateInUse = this._pattern.inUse;
                if (this._pattern.userDefined) {
                    if (this.templateMode != "copy") {
                        this.templateMode = "edit";
                    } else {
                        this.templateInUse = false;
                    }
                } else {
                    this.templateMode = "copy";
                    this.templateInUse = false;
                }              
                
                if(this.templateMode == "copy"){
                    this.dialogNode.set("title", this.locResources.TITLE + " - " + this._pattern.name + " - " + this.resources.COPY);
                    this.templateName.set("value", this._pattern.name + " - " + this.resources.COPY);
                }else{
                    this.dialogNode.set("title", this.locResources.EDITTITILE + " - " + this._pattern.name);
                    this.templateName.set("value", this._pattern.name);
                }
            }

        },
        
        _setSelectorsFromSettings: function(selectortype) {
            for (var controllerCount=0;controllerCount < this._numberOfControllers;++controllerCount) {
                for (var adapterCount=0;adapterCount < this._adapterTypeData.items.length;++adapterCount) {
                    if (this.adapterId == this._adapterTypeData.items[adapterCount].id) {
                        for (var portconfigcounter=0;portconfigcounter < this._adapterTypeData.items[adapterCount].portConfigurations.length;++portconfigcounter) {
                            if (this._adapterTypeData.items[adapterCount].portConfigurations[portconfigcounter].id == 
                                this._pattern.controllers[controllerCount].ports[0].portCfgId) {
                                var ctlNum = this._pattern.controllers[controllerCount].controllerNumber;                                
                                if (selectortype == "both" || selectortype == "mode") {
                                    switch(this._adapterTypeData.items[adapterCount].portConfigurations[portconfigcounter].mode.toString()) {
                                        case "pNIC":
                                            this._modeSelectorArray[ctlNum-1].set("value", "pNIC");
                                            break;
                                        case "vNIC_independent":                                        
                                            this._modeSelectorArray[ctlNum-1].set("value", "vNIC_independent");
                                            break;
                                        case "vNIC_IBMvf":
                                            this._modeSelectorArray[ctlNum-1].set("value", "vNIC_IBMvf");
                                            break;
                                        case "vNIC_IBMufp":
                                            this._modeSelectorArray[ctlNum-1].set("value", "vNIC_IBMufp");
                                            break;
                                    }
                                }
                                for (var optionCount=0;
                                     optionCount < this._adapterTypeData.items[adapterCount].portConfigurations[portconfigcounter].options.length;
                                     ++optionCount) {
                                    if (this._adapterTypeData.items[adapterCount].portConfigurations[portconfigcounter].options[optionCount].id == 
                                        this._pattern.controllers[controllerCount].ports[0].portCfgOptionId) {
                                        if (selectortype == "both" || selectortype == "protocol") {
                                            switch(this._adapterTypeData.items[adapterCount].portConfigurations[portconfigcounter].options[optionCount].optionCode.toString()) {
                                                case "ethOnly":    
                                                    if ((ctlNum-1) == 0) {
                                                        this._protocolSelectorValue = "ethOnly";
                                                    } else {
                                                        this._protocolSelectorValue2 = "ethOnly";
                                                    }
                                                    this._protocolSelectorArray[ctlNum-1].set("value", "ethOnly");                                                
                                                    break;
                                                case "eth+fcoe":     
                                                    if ((ctlNum-1) == 0) {
                                                        this._protocolSelectorValue = "eth+fcoe";
                                                    } else {
                                                        this._protocolSelectorValue2 = "eth+fcoe";
                                                    }                                                     
                                                    this._protocolSelectorArray[ctlNum-1].set("value", "eth+fcoe");                                                
                                                    break;
                                                case "eth+roce":     
                                                    if ((ctlNum-1) == 0) {
                                                        this._protocolSelectorValue = "eth+roce";
                                                    } else {
                                                        this._protocolSelectorValue2 = "eth+roce";
                                                    }                                                     
                                                    this._protocolSelectorArray[ctlNum-1].set("value", "eth+roce");                                                
                                                    break;
                                                case "eth+iscsi":     
                                                    if ((ctlNum-1) == 0) {
                                                        this._protocolSelectorValue = "eth+iscsi";
                                                    } else {
                                                        this._protocolSelectorValue2 = "eth+iscsi";
                                                    }                                                     
                                                    this._protocolSelectorArray[ctlNum-1].set("value", "eth+iscsi");                                               
                                                    break;
                                            }
                                        }
                                    }
                                }
                            }                        
                        }
                    }
                }
            }            
        },

        _setDefaultDisplayState: function() {
            // How do we want things displayed when dialog is first launched:
            this._firstHalfRow.style.display = "none";
            this._secondHalfRow.style.display = "none";
            this._opModeRow.style.display = "none";
            this._opModeRow2.style.display = "none";
            this._opModeRow3.style.display = "none";
            this._protocolRow.style.display = "none";
            this._protocolRow2.style.display = "none";
            this._protocolRow3.style.display = "none";

            this._expandall();
            this._selectall();

            this._toggleAllSettings();
        },

        _createSelectBoxes: function() {
            var modeSelectorArray = new Array();
            var protocolSelectorArray = new Array();
            
            if (this.adapterType == "unknown") {
                if (this._adapterTypeSelector == null) {
                    this._adapterTypeSelector = new support.widgets.Select({
                        id: "cp_newadaptertemplate_adaptertypeselector",
                        name: "adapterType",
                        store: this._adapterTypeStore,
                        query: {                            
                            adapterType: "vfa"
                        },
                        disabled: this.templateInUse
                    }).placeAt(this._adapterType);
                    dojo.connect(this._adapterTypeSelector, "onChange", this, function() {  
                        this.adapterId = this._adapterTypeSelector.get("value");                        
                        this._setGlobalValues();
                        var count = 0;                        
                        for (count = 0; count < this._adapterTypeData.items.length; ++count) {
                            if (this._adapterTypeData.items[count].id == this.adapterId) {
                                this.adapterPorts = this._adapterTypeData.items[count].portCount;
                                this.adapterType = "vfa"; //this._adapterTypeData.items[count].name;                                
                                this._createSelectBoxes();
                            }
                        }

					if (this._adapterModeSelector) {
						var protocoloptions = this._adapterProtocolSelector.getOptions();
						this._adapterProtocolSelector.removeOption(protocoloptions);
				        this._setProtocolSelector(this._adapterModeSelector, this._adapterProtocolSelector);
				        this._adapterProtocolSelector.set("value", this._protocolSelectorValue);
					}

					if (this._adapterModeSelector2) {
						var protocoloptions = this._adapterProtocolSelector2.getOptions();
				        this._adapterProtocolSelector2.removeOption(protocoloptions);
				        this._setProtocolSelector(this._adapterModeSelector2, this._adapterProtocolSelector2);
				        this._adapterProtocolSelector2.set("value", this._protocolSelectorValue2);
					}

					if (this._adapterModeSelector3) {
						var protocoloptions = this._adapterProtocolSelector3.getOptions();
				        this._adapterProtocolSelector3.removeOption(protocoloptions);
				        this._setProtocolSelector(this._adapterModeSelector3, this._adapterProtocolSelector3);
				        this._adapterProtocolSelector3.set("value", this._protocolSelectorValue);
					}  

                        this._toggleAllSettings();
                    });
                }
            }
            else {
                if (this._adapterTypeSelector == null) {
                    this._adapterTypeSelector = new support.widgets.Select({
                        id: "cp_newadaptertemplate_adaptertypeselector2",                        
                        name: "adapterType",
                        store: this._adapterTypeStore,
                        query: {
                            id: this.adapterId,
                            adapterType: "vfa"
                        },
                        disabled: true
                    }).placeAt(this._adapterType);
                }
            }

            if (this._numberOfControllers == 1) {                
                if (this._adapterModeSelector3 == null) {                   
                    this._adapterModeSelector3 = new support.widgets.Select({
                        id: this.get("id") + "_adapterModeSelector3",
                        name: "opModeSelector3",
                        maxHeight: -1,
                        style: {
                            width: this._defaultSelectorWidth
                            //width: "200px"
                        },
                        disabled: this.templateInUse
                    }).placeAt(this._opModeSelector3);
                    
                    dojo.connect(this._adapterModeSelector3, "onChange", this, function() {
                        var protocoloptions = this._adapterProtocolSelector3.getOptions();
                        this._adapterProtocolSelector3.removeOption(protocoloptions);
                        this._setProtocolSelector(this._adapterModeSelector3, this._adapterProtocolSelector3);
                        this._adapterProtocolSelector3.set("value", this._protocolSelectorValue);
                    });
                    this._setModeSelector(this._adapterModeSelector3);
                    this._adapterModeSelector3.startup();                    
                    modeSelectorArray.push(this._adapterModeSelector3);
                } else {
                    modeSelectorArray.push(this._adapterModeSelector3);
                }               

                if (this._adapterProtocolSelector3 == null) {
                    this._adapterProtocolSelector3 = new support.widgets.Select({
                        id: this.get("id") + "_protocolSelector3",
                        name: "protolcolSelector3",
                        maxHeight: -1,
                        style: {
                            width: this._defaultSelectorWidth
                            //width: "200px"
                        },
                        disabled: this.templateInUse
                    }).placeAt(this._protocolSelector3);
                    this._setProtocolSelector(this._adapterModeSelector3, this._adapterProtocolSelector3);
                    this._adapterProtocolSelector3.set("value", this._protocolSelectorValue);
                    this._adapterProtocolSelector3.startup();
                    //this._adapterProtocolSelector3.set("value", "ap0");                    
                    protocolSelectorArray.push(this._adapterProtocolSelector3);
                } else {
                    protocolSelectorArray.push(this._adapterProtocolSelector3);
                }                
            }
            else {
                if (this._adapterModeSelector == null) {
                    this._adapterModeSelector = new support.widgets.Select({
                        id: this.get("id") + "_adapterModeSelector",
                        name: "opModeSelector",
                        maxHeight: -1,
                        style: {
                            width: this._defaultSelectorWidth
                            //width: "200px"
                        },
                        disabled: this.templateInUse
                    }).placeAt(this._opModeSelector);
                    
                    dojo.connect(this._adapterModeSelector, "onChange", this, function() {
                        var protocoloptions = this._adapterProtocolSelector.getOptions();
                        this._adapterProtocolSelector.removeOption(protocoloptions);
                        this._setProtocolSelector(this._adapterModeSelector, this._adapterProtocolSelector);
                        this._adapterProtocolSelector.set("value", this._protocolSelectorValue);
                    });
                    
                    this._setModeSelector(this._adapterModeSelector);
                    this._adapterModeSelector.startup();
                    modeSelectorArray.push(this._adapterModeSelector);
                } else {
                    modeSelectorArray.push(this._adapterModeSelector);
                }

                if (this._adapterProtocolSelector == null) {
                    this._adapterProtocolSelector = new support.widgets.Select({
                        id: this.get("id") + "_protocolSelector",
                        name: "protocolSelector",
                        maxHeight: -1,
                        style: {
                            width: this._defaultSelectorWidth
                            //width: "200px"
                        },
                        disabled: this.templateInUse
                    }).placeAt(this._protocolSelector);
                    
                    this._setProtocolSelector(this._adapterModeSelector, this._adapterProtocolSelector);
                    this._adapterProtocolSelector.set("value", this._protocolSelectorValue);
                    this._adapterProtocolSelector.startup();
                    protocolSelectorArray.push(this._adapterProtocolSelector);
                } else {
                    protocolSelectorArray.push(this._adapterProtocolSelector);
                }                                

                if (this._adapterModeSelector2 == null) {
                    this._adapterModeSelector2 = new support.widgets.Select({
                        id: this.get("id") + "_adapterModeSelector2",
                        name: "opModeSelector2",
                        maxHeight: -1,
                        style: {
                            width: this._defaultSelectorWidth
                            //width: "200px"
                        },
                        disabled: this.templateInUse
                    }).placeAt(this._opModeSelector2);
                    dojo.connect(this._adapterModeSelector2, "onChange", this, function() {
                        //this._setGlobalValues();
                        var protocoloptions = this._adapterProtocolSelector2.getOptions();
                        this._adapterProtocolSelector2.removeOption(protocoloptions);
                        this._setProtocolSelector(this._adapterModeSelector2, this._adapterProtocolSelector2);
                        this._adapterProtocolSelector2.set("value", this._protocolSelectorValue2);
                    });

                    this._setModeSelector(this._adapterModeSelector2);
                    this._adapterModeSelector2.startup();
                    modeSelectorArray.push(this._adapterModeSelector2);
                } else {
                    modeSelectorArray.push(this._adapterModeSelector2);
                }

                if (this._adapterProtocolSelector2 == null) {
                    this._adapterProtocolSelector2 = new support.widgets.Select({
                        id: this.get("id") + "_protocolSelector2",
                        name: "protocolSelector2",
                        maxHeight: -1,
                        style: {
                            width: this._defaultSelectorWidth
                            //width: "200px"
                        },
                        disabled: this.templateInUse
                    }).placeAt(this._protocolSelector2);
                    this._setProtocolSelector(this._adapterModeSelector2, this._adapterProtocolSelector2);
                    this._adapterProtocolSelector2.set("value", this._protocolSelectorValue2);
                    this._adapterProtocolSelector2.startup();

                    protocolSelectorArray.push(this._adapterProtocolSelector2);
                    //this._adapterProtocolSelector2.set("value", "ap0");
                } else {
                    protocolSelectorArray.push(this._adapterProtocolSelector2);
                }
            }            
            this._modeSelectorArray = modeSelectorArray;
            this._protocolSelectorArray = protocolSelectorArray;
        },
        
        _setModeSelector: function(selector) {            
            for (var modeandopcodecounter=0;modeandopcodecounter < this._modeAndOpcodeArray.length;++modeandopcodecounter) {
                var modestring = "";
                switch(this._modeAndOpcodeArray[modeandopcodecounter].mode) {
                    case "pNIC":
                        modestring = this.resources.PNICMODE;                                
                        break;
                    case "vNIC_independent":
                        modestring = this.resources.VNICINDEPENDENT;                                
                        break;
                    case "vNIC_IBMvf":
                        modestring = this.resources.VNICVF;                                
                        break;
                    case "vNIC_IBMufp":
                        modestring = this.resources.VNICUFP;                                
                        break;
                    default:
                        modestring = this._modeAndOpcodeArray[modeandopcodecounter].mode;
                        break;
                }                        
                selector.addOption({
                    value: this._modeAndOpcodeArray[modeandopcodecounter].mode,
                    label: modestring
                });                        
            }
            selector.set("value", "vNIC_independent");
        },
        
        _setProtocolSelector: function(modeselector, protocolselector) {            
            for (var modeandopcodecounter=0;modeandopcodecounter < this._modeAndOpcodeArray.length;++modeandopcodecounter) {                
                if (this._modeAndOpcodeArray[modeandopcodecounter].mode == modeselector.get("value")) {
                    for (var protocolcounter=0;protocolcounter < this._modeAndOpcodeArray[modeandopcodecounter].opcodes.length;++protocolcounter) {
                        var protocolstring = "";
                        switch(this._modeAndOpcodeArray[modeandopcodecounter].opcodes[protocolcounter].optionCode) {
                            case "ethOnly":
                                protocolstring = this.resources.ETHONLY;
                                break;
                            case "eth+fcoe":
                                protocolstring = this.resources.ETHFCOE;
                                break;
                            case "eth+roce":
                                protocolstring = this.resources.ETHROCE;
                                break;
                            case "eth+iscsi":
                                protocolstring = this.resources.ETHISCSI;
                                break;
                            default:
                                protocolstring = this._modeAndOpcodeArray[modeandopcodecounter].opcodes[protocolcounter].optionCode;
                                break;
                        }                        
                        protocolselector.addOption({
                            value: this._modeAndOpcodeArray[modeandopcodecounter].opcodes[protocolcounter].optionCode,
                            label: protocolstring
                        });
                    }
                }
            }    
        },        

        _nameChanged: function() {
            if (this.templateName.get("value").trim() != "") {
                this.okButton.set("disabled", false);
            }
            else {
                this.okButton.set("disabled", true);
            }
        },

        _selectall: function() {
            // this._generalSettings.set("checked", true);
            this._firstHalf.set("checked", true);
            //this._opModeCheckBox.set("checked", true);
            //this._protocolCheckBox.set("checked", true);
            this._secondHalf.set("checked", true);
            //this._opModeCheckBox2.set("checked", true);
            //this._protocolCheckBox2.set("checked", true);
            //this._opModeCheckBox3.set("checked", true);
            //this._protocolCheckBox3.set("checked", true);

            this._toggleAllSettings();

            this.layout();
        },

        _deselectall: function() {
            //this._opModeCheckBox.set("checked", false);
            //this._protocolCheckBox.set("checked", false);
            //this._opModeCheckBox2.set("checked", false);
            //this._protocolCheckBox2.set("checked", false);
            //this._opModeCheckBox3.set("checked", false);
            //this._protocolCheckBox3.set("checked", false);

            /*
             * this._toggleFirstHalfSettings(); this._toggleSecondHalfSettings();
             */
            this._toggleAllSettings();

            this.layout();

        },

        _expandall: function() {
            this._generalSettings.set("checked", true);
            this._firstHalf.set("checked", true);
            this._secondHalf.set("checked", true);

            this._toggleAllSettings();

            this.expandall.style.display = "none";
            this.collapseall.style.display = "inline";
            this.mixedexpandcollapse.style.display = "none";

        },

        _collapseall: function() {
            this._generalSettings.set("checked", false);
            this._firstHalf.set("checked", false);
            this._secondHalf.set("checked", false);

            this._toggleAllSettings();

            this.expandall.style.display = "inline";
            this.collapseall.style.display = "none";
            this.mixedexpandcollapse.style.display = "none";
        },

        _setExpandCollapseIcon: function() {
            if (this._numberOfControllers == 1) {
                this.expandall.style.display = "none";
                this.collapseall.style.display = "inline";
                this.mixedexpandcollapse.style.display = "none";
                return;
            }
            if (this._checkExpandCollapseState() == "mixed") {
                this.expandall.style.display = "none";
                this.collapseall.style.display = "none";
                this.mixedexpandcollapse.style.display = "inline";
            }
            if (this._checkExpandCollapseState() == "allon") {
                this.expandall.style.display = "none";
                this.collapseall.style.display = "inline";
                this.mixedexpandcollapse.style.display = "none";
            }
            if (this._checkExpandCollapseState() == "alloff") {
                this.expandall.style.display = "inline";
                this.collapseall.style.display = "none";
                this.mixedexpandcollapse.style.display = "none";
            }
        },

        _checkExpandCollapseState: function() {
            if (this._numberOfControllers == 1) {
                // if (this._generalSettings.get("checked") == false) {
                // return "alloff";
                // } else {
                return "allon";
                // }
            }

            if (this._generalSettings.get("checked") == false) {
                return "alloff";
            }
            
            /*if (this._firstHalf.get("checked") == false && this._secondHalf.get("checked") == false) {
                return "alloff";
            }*/

            if (this._firstHalf.get("checked") == true && this._secondHalf.get("checked") == true) {
                return "allon";
            }

            return "mixed";
        },

        _toggleAllSettings: function() {
            this._toggleFirstHalfSettings();
            this._toggleSecondHalfSettings();
            this._toggleFabricConnectorSettings();
            this._setExpandCollapseIcon();
        },

        _toggleFirstHalfSettings: function() {
            if (this._numberOfControllers == 1) {
                this._firstHalfRow.style.display = "none";
                this._opModeRow.style.display = "none";
                this._protocolRow.style.display = "none";
                return;
            }
            else {
                this._opModeRow3.style.display = "none";
                this._protocolRow3.style.display = "none";
            }
            this._setExpandCollapseIcon();

            var opModeCheckState = true; //this._opModeCheckBox.get("checked");
            var protocolCheckState = true; //this._protocolCheckBox.get("checked");

            if (this._generalSettings.get("checked") == false) {
                this._firstHalfRow.style.display = "none";
                this._opModeRow.style.display = "none";
                this._protocolRow.style.display = "none";
                return;
            } else {
                this._firstHalfRow.style.display = "table-row";
                this._opModeRow.style.display = "table-row";
                this._protocolRow.style.display = "table-row";
            }
            
            if (opModeCheckState) {
                this._opModeCell.style.display = "block";
            }
            else {
                this._opModeCell.style.display = "none";
            }

            if (protocolCheckState) {
                this._protocolCell.style.display = "block";
            }
            else {
                this._protocolCell.style.display = "none";
            }

            if (this.hideuncheckedToggleButtonId.get("checked") == true) {
                if (this._firstHalf.get("checked") == false) {
                    this._opModeRow.style.display = "none";
                    this._protocolRow.style.display = "none";
                }
                else {
                    if (false) { //this._opModeCheckBox.get("checked") == false) {
                        this._opModeRow.style.display = "none";
                    }
                    else {
                        this._opModeRow.style.display = "table-row";
                    }
                    if (false) { //this._protocolCheckBox.get("checked") == false) {
                        this._protocolRow.style.display = "none";
                    }
                    else {
                        this._protocolRow.style.display = "table-row";
                    }
                    if (this._adapterModeSelector.get("value") == "vNIC_independent") {
                        this._opModeRow.style.display = "none";
                    } else {
                        this._opModeRow.style.display = "table-row";
                    }
                    if (this._adapterProtocolSelector.get("value") == "ethOnly") {
                        this._protocolRow.style.display = "none";                        
                    } else {
                        this._protocolRow.style.display = "table-row";
                    }
                }
                if (false) { //this._opModeCheckBox.get("checked") == false && this._protocolCheckBox.get("checked") == false) {
                    this._firstHalfRow.style.display = "none";
                }
                else {
                    this._firstHalfRow.style.display = "block";
                }
                return;
            }
            else {
                this._firstHalfRow.style.display = "block";
                if (this._firstHalf.get("checked") == false) {
                    this._opModeRow.style.display = "none";
                    this._protocolRow.style.display = "none";
                }
                else {
                    this._opModeRow.style.display = "table-row";
                    this._protocolRow.style.display = "table-row";
                }
                return;
            }
        },

        _toggleSecondHalfSettings: function() {
            if (this._numberOfControllers == 1) {
                this._secondHalfRow.style.display = "none";
                this._opModeRow2.style.display = "none";
                this._protocolRow2.style.display = "none";
                return;
            }
            else {
                this._opModeRow3.style.display = "none";
                this._protocolRow3.style.display = "none";
            }

            this._setExpandCollapseIcon();

            var opModeCheckState = true;  //this._opModeCheckBox2.get("checked");
            var protocolCheckState = true; //this._protocolCheckBox2.get("checked");
            
            if (this._generalSettings.get("checked") == false) {
                this._secondHalfRow.style.display = "none";
                this._opModeRow2.style.display = "none";
                this._protocolRow2.style.display = "none";
                return;
            } else {
                this._secondHalfRow.style.display = "table-row";
                this._opModeRow2.style.display = "table-row";
                this._protocolRow2.style.display = "table-row";
            }

            if (opModeCheckState) {
                this._opModeCell2.style.display = "block";
            }
            else {
                this._opModeCell2.style.display = "none";
            }

            if (protocolCheckState) {
                this._protocolCell2.style.display = "block";
            }
            else {
                this._protocolCell2.style.display = "none";
            }

            if (this.hideuncheckedToggleButtonId.get("checked") == true) {
                if (this._secondHalf.get("checked") == false) {
                    this._opModeRow2.style.display = "none";
                    this._protocolRow2.style.display = "none";
                }
                else {
                    if (false) { //this._opModeCheckBox2.get("checked") == false) {
                        this._opModeRow2.style.display = "none";
                    }
                    else {
                        this._opModeRow2.style.display = "table-row";
                    }
                    if (false) { //this._protocolCheckBox2.get("checked") == false) {
                        this._protocolRow2.style.display = "none";
                    }
                    else {
                        this._protocolRow2.style.display = "table-row";
                    }
                    if (this._adapterModeSelector2.get("value") == "vNIC_independent") {
                        this._opModeRow2.style.display = "none";
                    } else {
                        this._opModeRow2.style.display = "table-row";
                    }
                    if (this._adapterProtocolSelector2.get("value") == "ethOnly") {
                        this._protocolRow2.style.display = "none";                        
                    } else {
                        this._protocolRow2.style.display = "table-row";
                    }
                }
                if (false) { //this._opModeCheckBox2.get("checked") == false && this._protocolCheckBox2.get("checked") == false) {
                    this._secondHalfRow.style.display = "none";
                }
                else {
                    this._secondHalfRow.style.display = "block";
                }

                return;
            }
            else {
                this._secondHalfRow.style.display = "block";
                if (this._secondHalf.get("checked") == false) {
                    this._opModeRow2.style.display = "none";
                    this._protocolRow2.style.display = "none";
                }
                else {
                    this._opModeRow2.style.display = "table-row";
                    this._protocolRow2.style.display = "table-row";
                }
                return;
            }
        },

        _toggleFabricConnectorSettings: function() {
            var opModeCheckState = true; //this._opModeCheckBox3.get("checked");
            var protocolCheckState = true; //this._protocolCheckBox3.get("checked");
            
            if (this._numberOfControllers == 2) {                
                return;
            }
            
            if (this._generalSettings.get("checked") == false) {
                this._opModeRow3.style.display = "none";
                this._protocolRow3.style.display = "none";
            } else {
                this._opModeRow3.style.display = "table-row";
                this._protocolRow3.style.display = "table-row";
            }
            
            if (this.hideuncheckedToggleButtonId.get("checked") == true) {
                if (this._adapterModeSelector3.get("value") == "vNIC_independent") {
                    this._opModeRow3.style.display = "none";
                } else {
                    this._opModeRow3.style.display = "true";
                }
                if (this._adapterProtocolSelector3.get("value") == "ethOnly") {
                    this._protocolRow3.style.display = "none";                        
                } else {
                    this._protocolRow3.style.display = "table-row";
                }
            }
            return;


            if (opModeCheckState) {
                this._opModeSelector3.style.display = "block";
            }
            else {
                this._opModeSelector3.style.display = "none";
            }

            if (protocolCheckState) {
                this._protocolSelector3.style.display = "block";
            }
            else {
                this._protocolSelector3.style.display = "none";
            }

            if (this.hideuncheckedToggleButtonId.get("checked") == true) {
                /*
                 * if (this._generalSettings.get("checked") == false) {
                 * this._opModeRow3.style.display = "none"; this._protocolRow3.style.display =
                 * "none"; return; } else {
                 */
                if (!opModeCheckState) {
                    this._opModeRow3.style.display = "none";
                }
                else {
                    this._opModeRow3.style.display = "table-row";
                }
                if (!protocolCheckState) {
                    this._protocolRow3.style.display = "none";
                }
                else {
                    this._protocolRow3.style.display = "table-row";
                }
                // }
                return;
            }
            else {
                // this._generalSettings.style.display = "block";
                /*
                 * if (this._generalSettings.get("checked") == false) {
                 * this._opModeRow3.style.display = "none"; this._protocolRow3.style.display =
                 * "none"; return; } else {
                 */
                this._opModeRow3.style.display = "table-row";
                this._protocolRow3.style.display = "table-row";
                // }
                return;
            }
        },
        
        _setGlobalValues: function () {
            // Set values for this particular adapter
            var controllerObjArray = new Array();
            var modeandopcodeArray = new Array();
                        
            /**************************************************/
            /* Loop through the list of supported IO adapters */
            /**************************************************/            
            for (var supportedadaptercounter=0;supportedadaptercounter < this._adapterTypeData.items.length; ++supportedadaptercounter) {
                // Found the adapter that matches the one currently selected
                if (this._adapterTypeData.items[supportedadaptercounter].id == this.adapterId) {
                    /*********************************************************/
                    /* Loop through the port configurations for this adapter */
                    /*********************************************************/
                    for (var portconfigcounter=0;portconfigcounter < this._adapterTypeData.items[supportedadaptercounter].portConfigurations.length;++portconfigcounter) {
                        var modeandopcodeObj = new Object();
                        var opcodeArray = new Array();
                        modeandopcodeObj.portCfgId = this._adapterTypeData.items[supportedadaptercounter].portConfigurations[portconfigcounter].id;
                        if (typeof(this._adapterTypeData.items[supportedadaptercounter].portConfigurations[portconfigcounter].mode) == "string") {
                            modeandopcodeObj.mode = this._adapterTypeData.items[supportedadaptercounter].portConfigurations[portconfigcounter].mode;
                        } else {
                            modeandopcodeObj.mode = this._adapterTypeData.items[supportedadaptercounter].portConfigurations[portconfigcounter].mode[0];
                        }
                        /*********************************************************/
                        /* Loop through the port config options for this adapter */
                        /*********************************************************/
                        for (var optioncounter=0;
                             optioncounter < this._adapterTypeData.items[supportedadaptercounter].portConfigurations[portconfigcounter].options.length;
                             ++optioncounter) {
                            var portoptionsObj = new Object();
                            portoptionsObj.portCfgOptionId = this._adapterTypeData.items[supportedadaptercounter].portConfigurations[portconfigcounter].options[optioncounter].id;
                            if (typeof(this._adapterTypeData.items[supportedadaptercounter].portConfigurations[portconfigcounter].options[optioncounter].optionCode) == "string") {
                                //opcodeArray.push(this._adapterTypeData.items[supportedadaptercounter].portConfigurations[portconfigcounter].options[optioncounter].optionCode);
                                portoptionsObj.optionCode = this._adapterTypeData.items[supportedadaptercounter].portConfigurations[portconfigcounter].options[optioncounter].optionCode;
                            } else {
                                //opcodeArray.push(this._adapterTypeData.items[supportedadaptercounter].portConfigurations[portconfigcounter].options[optioncounter].optionCode[0]);
                                portoptionsObj.optionCode = this._adapterTypeData.items[supportedadaptercounter].portConfigurations[portconfigcounter].options[optioncounter].optionCode[0];
                            }
                            opcodeArray.push(portoptionsObj);
                        }
                        modeandopcodeObj.opcodes = opcodeArray;
                        modeandopcodeArray.push(modeandopcodeObj);
                    }
                    /*********************************************************************/
                    /* Loop through the list of controllers for the specified IO adapter */
                    /*********************************************************************/
                    for (var controllercounter=0;controllercounter < this._adapterTypeData.items[supportedadaptercounter].controllers.length;++controllercounter) {
                        var controllerObj = new Object();
                        
                        this._numberOfControllers = this._adapterTypeData.items[supportedadaptercounter].controllers.length;
                        controllerObj.controllerId = this._adapterTypeData.items[supportedadaptercounter].controllers[controllercounter].id;
                        controllerObj.controllerNumber = this._adapterTypeData.items[supportedadaptercounter].controllers[controllercounter].controllerNumber;
                        // Dojo will convert object types from ints to arrays,
                        //   so need to handle both cases
                        if (typeof(controllerObj.controllerNumber) != "number") {
                            controllerObj.controllerNumber = controllerObj.controllerNumber[0];
                        }
                        /************************************************************************/
                        /* Loop through the list of configurations for the specified controller */
                        /************************************************************************/
                        for (var controllerconfigcounter=0;
                             controllerconfigcounter < this._adapterTypeData.items[supportedadaptercounter].controllers[controllercounter].configurations.length;
                             ++controllerconfigcounter) {
                            controllerObj.configId = this._adapterTypeData.items[supportedadaptercounter].controllers[controllercounter].configurations[controllerconfigcounter].id;
                            if (this._adapterTypeData.items[supportedadaptercounter].controllers[controllercounter].configurations[controllerconfigcounter].portConfigDependency == "mutual") {
                                var ports = new Object();
                                ports.portId = this._adapterTypeData.items[supportedadaptercounter].controllers[controllercounter].configurations[controllerconfigcounter].ports[0].id;
                                ports.portNumber = this._adapterTypeData.items[supportedadaptercounter].controllers[controllercounter].configurations[controllerconfigcounter].ports[0].portNumber;
                                // Dojo will convert object types from ints to
                                //   arrays, so need to handle both cases
                                if (typeof(ports.portNumber) != "number") {
                                    ports.portNumber = ports.portNumber[0];
                                }
                                
                                if (this._adapterModeSelector != null) {
              //                      alert(this._adapterModeSelector.get("value"));
                                } else {
                //                    alert("vNIC_independent gggg");
                                }
                                
                                /*******************************************************************************************/
                                /* Loop through the list of supported configurations for the first port on this controller */
                                /*******************************************************************************************/
                                for (var supportedconfigcounter=0;
                                     supportedconfigcounter < this._adapterTypeData.items[supportedadaptercounter].controllers[controllercounter].configurations[controllerconfigcounter].ports[0].supportedConfigurations.length;
                                     ++supportedconfigcounter) {
                                    
                                }
                                controllerObj.ports = ports;
                            } else {
                                // Future independent port config section
                            }
                        }
                        controllerObjArray.push(controllerObj);
                    }
                }
            }            
            this._modeAndOpcodeArray = modeandopcodeArray;
            this._controllerObjectArray = controllerObjArray;
            
            //alert(dojo.toJson(this._modeAndOpcodeArray));
            //alert(dojo.toJson(this._controllerObjectArray));
        },        

        _hideUncheckedClicked: function() {
            this._toggleAllSettings();
            this.layout();
        },

        _saveandhideDialog: function() {

            var adapterTemplateObj = new Object();
            adapterTemplateObj.id = "";  // Required for POST call
            adapterTemplateObj.name = this.templateName.get("value").trim();
            adapterTemplateObj.type = "Adapter";
            var description = this.adapterDesc.get("value").trim();
            adapterTemplateObj.description = this.encodeSpecialCharacters(description);
            adapterTemplateObj.adapterType = this._adapterTypeSelector.get("value");
            adapterTemplateObj.userDefined = true;
            var controllersArray = new Array();            
            for (var numControllers=0;numControllers < this._numberOfControllers;++numControllers) {
                var controllerObj = new Object();
                controllerObj.id = "";  // Required for POST call
                controllerObj.controllerId = this._controllerObjectArray[numControllers].controllerId.toString();
                controllerObj.controllerNumber = this._controllerObjectArray[numControllers].controllerNumber;
                controllerObj.configId = this._controllerObjectArray[numControllers].configId.toString();
                var portsArray = new Array();
                var portsObj = new Object(); // Just do one port for now                
                portsObj.id = "";  // Required for POST call
                portsObj.portId = this._controllerObjectArray[numControllers].ports.portId.toString();
                portsObj.portNumber = this._controllerObjectArray[numControllers].ports.portNumber;
                var currentControllerMode = this._modeSelectorArray[numControllers].get("value");
                var currentControllerProtocol = this._protocolSelectorArray[numControllers].get("value");                
                for (var modeandopcodecounter=0;modeandopcodecounter < this._modeAndOpcodeArray.length;++modeandopcodecounter) {                    
                    if (this._modeAndOpcodeArray[modeandopcodecounter].mode == currentControllerMode) {                        
                        portsObj.portCfgId = this._modeAndOpcodeArray[modeandopcodecounter].portCfgId.toString();
                        for (var portoptioncounter=0;portoptioncounter < this._modeAndOpcodeArray[modeandopcodecounter].opcodes.length;++portoptioncounter) {                           
                            if (this._modeAndOpcodeArray[modeandopcodecounter].opcodes[portoptioncounter].optionCode == currentControllerProtocol) {                                
                                portsObj.portCfgOptionId = this._modeAndOpcodeArray[modeandopcodecounter].opcodes[portoptioncounter].portCfgOptionId.toString();                                
                            }
                        }
                    }                                 
                }
                portsArray.push(portsObj);
                controllerObj.ports = portsArray;
                controllersArray.push(controllerObj);                
            }            
            adapterTemplateObj.controllers = controllersArray;
            
            console.log(this.declaredClass + ": ", dojo.toJson(adapterTemplateObj));
            
            this.getProgressDialog("save").show(); // Show progress dialog for save request.
            var xhrPost = dojo.xhrPost({
                url:  this.REST_PATH + "template/create?type=adapter",
                handleAs: "json", // <<<< Required
                headers: {
                    "Content-Type": "application/json; charset=utf-8" // <<<< Required
                },
                postData: dojo.toJson(adapterTemplateObj),     //Template Data to Post
                load: dojo.hitch(this, function(response, ioArgs) {
                    // Happy path! We received a 2xx status code from the API.
                                                               
                        dojo.publish("ProgressComplete", response);
                        dojo.publish("CreateAdapterTemplateComplete", response);
                        this.hide();

                }),
                error: dojo.hitch(this, function(error, ioArgs) {
                    // Failure path! We received a non-2xx status code (ie: 3xx/4xx/5xx status code) from the API.

                    dojo.publish("ProgressFailed", error);
                    console.log(this.declaredClass + ": ", error, ioArgs);
                    
                    var response = dojo.fromJson(ioArgs.xhr.responseText);
                    if ((ioArgs.xhr.status >= 400) && (response.message == "service_error_name_in_use")) {
                        // Error Template name in use
                        this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                    }
                    else if(ioArgs.xhr.status == 404) {
                        // Edit Template ID doesn't exist
                        this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                        this.hide();
                    }
                    else if(ioArgs.xhr.status >= 500) {
                        // Internal Programming Error
                        this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                        this.hide();
                    }
                    else {
                        this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                        this.hide();
                    }
                    
                }),
                handle: dojo.hitch(this, function(data, ioArgs) {
                    // - The 1st param ("data") is either the response data (on success) or Error object (on failure).
                    // - The 2nd param ("ioArgs") contains detail about the xhr request, including a reference to the
                    // native xhr object. Check ioArgs.xhr.status to determine success or failure, and handle "data"
                    // accordingly.
                })
            });

        },
        
        saveClicked: function() {

            var adapterTemplateObj = new Object();
            adapterTemplateObj.id = this.templateIDNum;  // Required for POST call
            adapterTemplateObj.name = this.templateName.get("value").trim();
            adapterTemplateObj.type = "Adapter";
            var description = this.adapterDesc.get("value").trim();
            adapterTemplateObj.description = this.encodeSpecialCharacters(description);
            adapterTemplateObj.adapterType = this._adapterTypeSelector.get("value");
            adapterTemplateObj.userDefined = true;
            var controllersArray = new Array();            
            for (var numControllers=0;numControllers < this._numberOfControllers;++numControllers) {
                var controllerObj = new Object();
                controllerObj.id = this._pattern.controllers[numControllers].id;
                controllerObj.controllerId = this._controllerObjectArray[numControllers].controllerId;
                controllerObj.controllerNumber = this._controllerObjectArray[numControllers].controllerNumber;
                controllerObj.configId = this._controllerObjectArray[numControllers].configId;
                var portsArray = new Array();
                var portsObj = new Object();  // Just do one port for now                
                portsObj.portId = this._controllerObjectArray[numControllers].ports.portId;
                portsObj.portNumber = this._controllerObjectArray[numControllers].ports.portNumber;
                var currentControllerMode = this._modeSelectorArray[numControllers].get("value");
                var currentControllerProtocol = this._protocolSelectorArray[numControllers].get("value");                
                for (var modeandopcodecounter=0;modeandopcodecounter < this._modeAndOpcodeArray.length;++modeandopcodecounter) {                    
                    if (this._modeAndOpcodeArray[modeandopcodecounter].mode == currentControllerMode) {                        
                        portsObj.portCfgId = this._modeAndOpcodeArray[modeandopcodecounter].portCfgId;
                        portsObj.id = this._pattern.controllers[numControllers].ports[0].id;
                        for (var portoptioncounter=0;portoptioncounter < this._modeAndOpcodeArray[modeandopcodecounter].opcodes.length;++portoptioncounter) {                           
                            if (this._modeAndOpcodeArray[modeandopcodecounter].opcodes[portoptioncounter].optionCode == currentControllerProtocol) {                                
                                portsObj.portCfgOptionId = this._modeAndOpcodeArray[modeandopcodecounter].opcodes[portoptioncounter].portCfgOptionId;                                
                            }
                        }
                    }                                 
                }
                portsArray.push(portsObj);
                controllerObj.ports = portsArray;
                controllersArray.push(controllerObj);                
            }            
            adapterTemplateObj.controllers = controllersArray;
            
            var argJSONString = dojo.toJson(adapterTemplateObj);
           
            var saveTemplate = true;

            if (this.templateInUse) {
                //  Need to remove extra properties that aren't included in the dialog
                delete this._pattern.inUse;
                delete this._pattern.uri;
                delete this._pattern.referencedBy;
                delete this._pattern.useCount;                
                
                // Remove description since it shouldn't force a redeploy
                delete this._pattern.description;
                delete adapterTemplateObj.description;
                
                var preData = this.sortAndFlatten(this._pattern);
                var postData = this.sortAndFlatten(adapterTemplateObj);
                
                if(preData != postData){
                    this.rdstDialog.set("templateID", this.templateIDNum);
                    this.rdstDialog.set("templateType", "Category");
                    this.rdstDialog.set("templateData", argJSONString);
                    this.rdstDialog.show();
                    
                    saveTemplate = false;
                }
            }
            if (saveTemplate) {

                this.getProgressDialog("save").show(); // Show progress dialog for save request.
                var xhrArgs = {
                        url:  this.REST_PATH + "template/" + this.templateIDNum,
                        handleAs: "json",
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        postData: argJSONString,
                        load: dojo.hitch(this, function(response, ioArgs) {
                            // Happy path! We received a 2xx status code from the API.
                            //alert("post data: " + data);
                            dojo.publish("ProgressComplete", response);
                            // Publish the ID of the pattern that was just updated for the benefit of all listeners.
                            dojo.publish("EditAdapterTemplateComplete", {
                                id: this.templateIDNum
                            });                          
                            this.hide();
                        }),
                        error: dojo.hitch(this, function(error, ioArgs) {
                            // Failure path! We received a non-2xx status code (ie: 3xx/4xx/5xx status code) from the API.

                            dojo.publish("ProgressFailed", error);
                            console.log(this.declaredClass + ": ", error, ioArgs);
                            
                            var response = dojo.fromJson(ioArgs.xhr.responseText);
                            if ((ioArgs.xhr.status >= 400) && (response.message == "service_error_name_in_use")) {
                                // Error Template name in use
                                this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                            }
                            else if(ioArgs.xhr.status == 404) {
                                // Edit Template ID doesn't exist
                                this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                                this.hide();
                            }
                            else if(ioArgs.xhr.status >= 500) {
                                // Internal Programming Error
                                this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                                this.hide();
                            }
                            else {
                                this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                                this.hide();
                            }
                            
                        })
                };
                dojo.xhrPut(xhrArgs);
                
            }
        },
        
        saveasClicked: function() {

            var adapterTemplateObj = new Object();
            adapterTemplateObj.id = "";  // Required for POST call
            adapterTemplateObj.name = this.templateName.get("value").trim();
            adapterTemplateObj.type = "Adapter";
            var description = this.adapterDesc.get("value").trim();
            adapterTemplateObj.description = this.encodeSpecialCharacters(description);
            adapterTemplateObj.adapterType = this._adapterTypeSelector.get("value");
            adapterTemplateObj.userDefined = true;
            var controllersArray = new Array();            
            for (var numControllers=0;numControllers < this._numberOfControllers;++numControllers) {
                var controllerObj = new Object();
                controllerObj.id = "";  // Required for POST call
                controllerObj.controllerId = this._controllerObjectArray[numControllers].controllerId;
                controllerObj.controllerNumber = this._controllerObjectArray[numControllers].controllerNumber;
                controllerObj.configId = this._controllerObjectArray[numControllers].configId;
                var portsArray = new Array();
                var portsObj = new Object();  // Just do one port for now
                portsObj.id = "";  // Required for POST call
                portsObj.portId = this._controllerObjectArray[numControllers].ports.portId;
                portsObj.portNumber = this._controllerObjectArray[numControllers].ports.portNumber;
                var currentControllerMode = this._modeSelectorArray[numControllers].get("value");
                var currentControllerProtocol = this._protocolSelectorArray[numControllers].get("value");                
                for (var modeandopcodecounter=0;modeandopcodecounter < this._modeAndOpcodeArray.length;++modeandopcodecounter) {                    
                    if (this._modeAndOpcodeArray[modeandopcodecounter].mode == currentControllerMode) {                        
                        portsObj.portCfgId = this._modeAndOpcodeArray[modeandopcodecounter].portCfgId;
                        for (var portoptioncounter=0;portoptioncounter < this._modeAndOpcodeArray[modeandopcodecounter].opcodes.length;++portoptioncounter) {                           
                            if (this._modeAndOpcodeArray[modeandopcodecounter].opcodes[portoptioncounter].optionCode == currentControllerProtocol) {                                
                                portsObj.portCfgOptionId = this._modeAndOpcodeArray[modeandopcodecounter].opcodes[portoptioncounter].portCfgOptionId;                                
                            }
                        }
                    }                                 
                }
                portsArray.push(portsObj);
                controllerObj.ports = portsArray;
                controllersArray.push(controllerObj);                
            }            
            adapterTemplateObj.controllers = controllersArray;
            
            this.saveasDialog.set("templateObject",adapterTemplateObj);
            this.saveasDialog.show();                                    
        },
        
        exitParent: function() {
            dojo.publish("EditAdapterTemplateComplete", {
                id: this.templateIDNum
            });
            if (this.rdstDialog.get("closeParent")) {
                this.hide();
            }
        },

        _cancelDialog: function() {
            this.hide();
        },
        
        _setup: function(){

            //Set Information Messages and Load Previous Settings          
            if (this.templateIDNum != null && this.templateMode == "edit") {

                //if dialog in edit mode  
                if (this.templateInUse) {
                    if (this.templateLaunchPoint == "table") {
                        this.warningMessage.style.display = "block";
                        this.warningMessageText.innerHTML = this.resources.INUSE;
                        this.saveBtnContainer.style.display = "inline";
                        this.saveasBtnContainer.style.display = "inline";
                    } else if(this.templateLaunchPoint == "wizard") {
                        this.informationMessage.style.display = "block";
                        this.informationMessageText.innerHTML = this.resources.INUSEWIZARD;
                        this.saveasBtnContainer.style.display = "inline";
                    } else {
                        this.informationMessage.style.display = "block";
                        this.informationMessageText.innerHTML = this.resources.INUSEWIZARD;
                        this.saveasBtnContainer.style.display = "inline";
                    }
                } else {
                    this.saveBtnContainer.style.display = "inline";
                    this.saveasBtnContainer.style.display = "inline";                  
                }
                this.templateName.set("disabled",true);
                
            }
            else if (this.templateIDNum != null && this.templateMode == "copy") {

                //if dialog in copy mode  
                this.templateIDNum = null;  //set template ID to null in order to create new template
                this.okBtnContainer.style.display = "inline";

            } else {

                //if dialog in create new mode
                this.okBtnContainer.style.display = "inline";

            }
        }
    });
});
