define([ 
        "dojo",
        "dijit",
        "support/dialogs/Dialog",
        "dojo/i18n!./nls/NewIPPool",
        "dojo/text!./templates/NewIPPool.html",
        "dijit/form/FilteringSelect",
        "dijit/form/Button",
        "dijit/form/SimpleTextarea",
        "dijit/form/ValidationTextBox",
        "support/widgets/ButtonBar",
        "dojo/store/Memory",
        "support/widgets/HoverButton",
        "dijit/layout/ContentPane",
        "dojox/grid/DataGrid",
        "dojo/data/ItemFileWriteStore",
        "support/dialogs/SubrangePopup",
        "support/dialogs/SaveAs",
        "dijit/layout/BorderContainer",
        "support/temp/ModalDialog",
        "support/dialogs/ProgressDialog"
    ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.NewIPPool", [ support.dialogs.Dialog ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 
        
        widgetsInTemplate: true, // This widget's template must be parsed for widgets.
        
        templateId: null,
        baseClass: "NewIPPool",
        
        poolIDNum: null,
        
        editMode: false,
        inUse: false,
        name: null,
        description: null,
        version: null,
        _version: null,
        invalidMessage: null,
        regExp: ".*",
        placeHolder: "",
        dropDown: null,
        gateway: null,
        _gateway: null,

        rangeTable: null,
        rangeStore: null,
        rangeLayout: null,
        _newRange: null,

        networkTable: null,
        networkStore: null,
        networkLayout: null,   
        networkTooltip: null,
        network: null,
        
        overlap: false,
        overlapMessage: null,
        
        IPPOOL_NAME: "IPPool",
        
        IPV4_VALUE: "IPv4",
        IPV6_VALUE: "IPv6",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/NewIPPool.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template
        // HTML file
        postMixInProperties: function() {
            this.inherited(arguments);
 
            // Initialize map to store if range is newly added
            this._newRange = {};
            
            this._connectDataModels();
        },

        _connectDataModels : function(){
          //rangeTable
            this.rangeStore = new dojo.data.ItemFileWriteStore({
                data: {"items" : []}                    
            });
            if (this.rangeLayout == null) {
                this.rangeLayout = [
                    { name: this.locResources.RANGE_NAME, field: "range", width:"23%", editable:false, noresize:true },
                    { name: this.locResources.FIRST_ADDRESS, field: "first", width:"24%", editable:false, noresize:true },
                    { name: this.locResources.RANGE_SIZE, field: "size", width:"15%", editable:false, noresize:true },
                    { name: this.locResources.LAST_ADDRESS, field: "last", width:"23%", editable:false, noresize:true },
                    { name: " ",  field: "icons", width: "15%", editable:false, formatter: dojo.hitch(this, "_formatIcons"), noresize:true }
                ];
            }
            
            //networkTable
            this.networkStore = new dojo.data.ItemFileWriteStore({
                data: {"items" : [ { network: null, broadcast: null, lowest: null, host: "0", highest: null } ]}                    
            });
            if (this.networkLayout == null) {
                this.networkLayout = this.getNetworkLayout();
            }
        },
        
        getNetworkLayout : function(){
            if(this.version == this.IPV6_VALUE){
                return [
                        {name: this.locResources.LOWEST_ADDRESS, field: "lowest", width:"50%", editable:false, formatter: dojo.hitch(this, "_formatAddress") },
                        {name: this.locResources.HIGHEST_ADDRESS, field: "highest", width: "50%", editable:false, formatter: dojo.hitch(this, "_formatAddress")}
                    ];
            }
            else{ //IPv4
                return [
                        {name: this.locResources.NETWORK_ID, field: "network", width:"20%", editable:false, formatter: dojo.hitch(this, "_formatAddress") },
                        {name: this.locResources.BROADCAST, field: "broadcast", width:"20%", editable:false, formatter: dojo.hitch(this, "_formatAddress") },
                        {name: this.locResources.FIRST_ADDRESS, field: "lowest", width:"20%", editable:false, formatter: dojo.hitch(this, "_formatAddress") },
                        {name: this.locResources.HOST_ADDRESSES, field: "host", width:"20%", editable:false, formatter: dojo.hitch(this, "_formatNumber") },
                        {name: this.locResources.LAST_ADDRESS, field: "highest", width: "20%", editable:false, formatter: dojo.hitch(this, "_formatAddress") }
                    ];
            }
        },
        
        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be
        // directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            
            // Create the button bar
            this._version = new support.widgets.ButtonBar({
                labels: this.resources.IPV4 + "|" + this.resources.IPV6,
                values: this.IPV4_VALUE + "|" + this.IPV6_VALUE,
                selected: this.resources.IPV4,
                onClick: dojo.hitch(this, function() {
                    this.set("version", this._version.get("selected"));               
                }),
                onKeyPress: dojo.hitch(this, function() {
                    this.set("version", this._version.get("selected"));
                })
            }).placeAt(this._versiondiv);
            
            this._descMsg.innerHTML = this.locResources.DESC.replace("%1", 500);           
            
            if(this.invalidMessage == null)
                this.invalidMessage = this.locResources.INVALID_MESSAGE;
            
            //overall network tooltip
            this.networkTooltip = new dijit.Tooltip({
                connectId: [this.id + "_networkhelp"]
             });
            if(this.version == null){
                this.set("version", this._version.get("selected"));  
            } else {
                this._version.set("selected", this.version);
            }
            this._createGatewayTextbox();   
            this._getDropDown();

            // Apply env specific warning text
            this._sameNetworkWarningMessageText.innerHTML = this.locResources.SAME_NETWORK_WARNING.replace("%1", this.PRODUCT_NAME_LONG).replace("%2", this.PRODUCT_NAME_SHORT);
            
            // NOTE: For dijit/Tooltip dijits, set the label, not innerHTML.
            this._subrangehelp.label = "<div style=\"width: 30em;\">" + this.locResources.SUBRANGE_TABLE_TOOLTIP.replace("%1", this.PRODUCT_NAME_LONG).replace("%2", this.PRODUCT_NAME_SHORT) + "</div>";

            dojo.connect(this.dialogNode, "show", this, function(){
                if(this.poolIDNum != null){
                    this.xhrGetNewIPPool();
                }
                this.dialogNode._position(); 
            });
            
            dojo.connect(this.dialogNode, "hide", this, function() {
                this.destroy();
            });

            this.dialogNode._position();
            
                        
        },
        
        _formatAddress : function(value, rowIndex, cell){
            if(value == null || value == ""){
                if(this.version == this.IPV4_VALUE)
                    return "0.0.0.0";
                if(this.version == this.IPV6_VALUE)
                    return "0::0";
            }
            return value;
        },
        
        _formatNumber : function(value, rowIndex, cell){
            if(value == null || value == "")
                return "0";
            return value;
        },
        
        //Displays subrange and network tables once drop down value is selected
        showTables : function(){
            this._subrangeTableTitle.setAttribute("style", "display:inline-block");
            this._networkTableTitle.setAttribute("style", "display:inline-block");
            this._addSubrangeButton.set("style", "display:block");
            this._subrangeHelp.setAttribute("style", "display:inline-block");
            this._networkHelp.setAttribute("style", "display:inline-block");
            this.showRangeTable();
            this.showNetworkTable();
            if(this.networkTable != null){
                this.networkLayout = this.getNetworkLayout();
                this.networkTable.set("structure", this.networkLayout);
                this.networkTable.setStore(this.networkStore);
            }
            this.reposition();
        },
        
        //Creates the subrange table, if null
        showRangeTable : function(){
            if(this.rangeTable == null){
                this.rangeTable = new dojox.grid.DataGrid({
                    store: this.rangeStore,
                    structure: this.rangeLayout,
                    autoWidth: false,
                    autoHeight: 3,
                    singleClickEdit: true,
                    columnReordering: false,
                    style: "width: 700px;",
                    noDataMessage: this.locResources.SUBRANGE_NO_DATA_MESSAGE
                }).placeAt(this._subrangeTableDiv);
                this.rangeTable.startup();
                this.rangeTable.canSort = function(col){ return false; };
                this.rangeTable.update();
            }
            
            var availHeight = window.screen.availHeight;
            var maxHeight = availHeight * 0.25;
            var maxRows = (maxHeight/35) - 1;
            if(maxRows < 1)
                maxRows = 1;
            this.rangeTable.set("autoHeight", maxRows);
        },
        
        //Creates the network table, if null
        showNetworkTable : function(){
            if(this.networkTable == null){
                this.networkTable = new dojox.grid.DataGrid({
                    store: this.networkStore,
                    structure: this.networkLayout,
                    autoWidth: false,
                    autoHeight: true,
                    singleClickEdit: true,
                    columnReordering: false,
                    style: "width: 700px; height: 50px;",
                    noDataMessage: this.locResources.NETWORK_NO_DATA_MESSAGE
                }).placeAt(this._networkTableDiv);
                this.networkTable.startup();
                this.networkTable.canSort = function(col){ return false; };
                this.networkTable.update();
            }
        },
        
        //Creates and places the text box for the gateway address in the dialog
        _createGatewayTextbox : function(){
            var pool = this;
            this._gateway = new dijit.form.ValidationTextBox({
                id: "cp_newippool_gatewayvalidationtextbox",
                value: this.gateway,
                required: true,
                trim: true,
                style:"width:100%; min-width:125px;",
                intermediateChanges: true,
                invalidMessage: this.invalidMessage,
                regExp: this.regExp,
                placeHolder: this.placeHolder,
                onChange : function(newValue){
                    // Both netmask and gateway have to be valid to show the tables.
                    pool.set("gateway", newValue);
                    if (this.isValid()) {
                        if (pool._dropDown != null) {
                            if (pool._dropDown.isValid()) {
                                if (pool.networkTable == null) {
                                    pool.showTables();
                                }
                                pool.updateNetworkTable();
                                pool.reposition();
                            }
                        }
                    } else {                 
                        // Only overlapping if the gateway is valid.
                        pool.overlap = false;
                    }
                },
                validator : function(value, constraints){
                 	// Disable the Add Range button if gateway is not valid
                    pool._addSubrangeButton._hoverButton.set("disabled", true);
                    if(pool.overlap) {
                        this.invalidMessage = pool.overlapMessage;
                        return false;
                    }     
                    if(value.length == 0){
                        return false;
                    }
                    if(!pool._isValidAddress(value)){
                        this.invalidMessage = pool.invalidMessage;
                        return false;
                    }
                    if(!pool._matchesNetworkID(value)){
                        this.invalidMessage = pool.locResources.INVALID_MESSAGE_GATEWAY_MATCH;
                        return false;
                    }
                    if(pool.networkTable != null){
                        var broadcast = pool.networkTable.store._arrayOfTopLevelItems[0].broadcast;
                        var network = pool.networkTable.store._arrayOfTopLevelItems[0].network;
                        if(broadcast != null && broadcast != ""){
                            if(value == broadcast){
                                this.invalidMessage = pool.locResources.INVALID_MESSAGE_GATEWAY_BROAD;
                                return false;
                            }
                        }
                        if(network != null && network != ""){
                            if(value == network){
                                this.invalidMessage = pool.locResources.INVALID_MESSAGE_GATEWAY_NET;
                                return false;
                            }
                        }
                    }
                    pool._addSubrangeButton._hoverButton.set("disabled", false);                    
                    return true;
                }
            }).placeAt(this._gatewayDiv);
        },
        
      //Determines if an ip address matches the network address
        _matchesNetworkID : function(value){
            if(this._isValidAddress(this.network) && this._isValidAddress(value)){
                if(this.version == this.IPV4_VALUE){
                    var subnet = this.dropDown;
                    var splitSub = subnet.split(".");
                    var IDsplit = this.network.split(".");
                    var IPsplit = value.split(".");
                    for(var i=0; i<IDsplit.length; i++){
                        if((IDsplit[i] != "0" || i > 0) && ((IPsplit[i] & splitSub[i]) != IDsplit[i])){
                            return false;
                        }
                    }
                }
                else if(this.version == this.IPV6_VALUE && this.dropDown != null){
                    var prefix = this.dropDown;
                    value = this._getIPv6FullNotation(value);
                    var network = this._getIPv6FullNotation(this.network);
                    
                    //full hex to binary prefix
                    if(value != null && network != null){
                        var hexValue = value.split(":");
                        var hexNetwork = network.split(":");
                        var binValue = new Array(8);
                        var binNetwork = new Array(8);
                        for(var i=0; i<8; i++){
                            binValue[i] = this.hex2bin(hexValue[i]);
                            binNetwork[i] = this.hex2bin(hexNetwork[i]);
                        }
                        var fullBinValue = this.getFullBinArray(binValue).join("").substring(0, prefix);
                        var fullBinNetwork = this.getFullBinArray(binNetwork).join("").substring(0, prefix);
                        return (fullBinValue == fullBinNetwork);
                    }  
                }
            }
            return true;
        },
        
        //Shows the drop down (subnet or routing prefix)
        _getDropDown : function(){
            var pool = this;
            var values = new Array();
            var options = new Array();
            if(this.version == this.IPV4_VALUE){
                values = ["128.0.0.0", "192.0.0.0", "224.0.0.0", 
                           "240.0.0.0", "248.0.0.0", "252.0.0.0",
                           "254.0.0.0", "255.0.0.0", "255.128.0.0",
                           "255.192.0.0", "255.224.0.0", "255.240.0.0",
                           "255.248.0.0", "255.252.0.0", "255.254.0.0", 
                           "255.255.0.0", "255.255.128.0", "255.255.192.0", 
                           "255.255.224.0", "255.255.240.0", "255.255.248.0", 
                           "255.255.252.0",  "255.255.254.0", "255.255.255.0",
                           "255.255.255.128", "255.255.255.192", "255.255.255.224",
                           "255.255.255.240", "255.255.255.248", "255.255.255.252"];
            }
            else if(this.version == this.IPV6_VALUE){
                values = ["48", "49", "50", "51", "52", "53", "54", "55", "56",
                           "57", "58", "59", "60", "61", "62", "63", "64"]; 
            }
            options.push({name: this.locResources.NONE_SELECTED, id: null});
            for(var i=0; i<values.length; i++){
                var option = values[i];
                var object = {name:option, id:option};
                options.push(object);
            }
            var store = new dojo.store.Memory({ data: options });
            if(this._dropDown == null){
                this._dropDown = new dijit.form.FilteringSelect({
                    required: true,
                    store: store,
                    labelAttr: "name",
                    searchAttr: "name",
                    _destroyOnRemove: true,
                    isValid: function() {                        
                        if (this.value == "") {
                            return false;
                        } else {
                            return true;
                        }                        
                    },
                    onChange: function(newValue){
						pool.set("dropDown", newValue);
                        // Both netmask and gateway have to be valid before showing the tables                        
                        if (pool._gateway != null) {
                            if (pool._gateway.isValid()) {
                                if (pool.networkTable == null) {
                                    pool.showTables();
                                }
                                pool.updateNetworkTable();
                                pool.reposition();                                    
                            } else {
                                pool.updateNetworkTable();
                            }
                        }                                                
                    },
                    style: "width:150px"
                }).placeAt(this._dropDownDiv);
                this._dropDown.set("displayedValue", this.locResources.NONE_SELECTED);
            }
            else{
                this._dropDown.set("store", store);
                this._dropDown.set("displayedValue", this.locResources.NONE_SELECTED);
            }
        },
        
        //Creates an "Add Subrange" popup and adds a new range to the table
        addRange : function(){
            var pool = this;
            var popup = new support.dialogs.SubrangePopup({
                id: "cp_newippool_subrangepopup",
                mode: "add",
                newPool: pool,
                subrangeStore: this.rangeStore,
                overallStore: this.networkStore,
                dropDown: this.dropDown,
                version: this.version,
                network: this.network,
                gateway: this.gateway,
                addNewSubrange : function(){
                    var item = { allocatedPct:null, range:this.range, chassisId:this.chassisId, first:this.first, size:this.size, last:this.last };
                    var index = pool.getInsertIndex(item);
                    pool.insertItemAtIndex(item, index);
                    pool._newRange[this.range] = true;
                    pool._enableOk();
                    pool.setRangesInTable();
                    pool.checkForNetworkID();
                    pool.reposition();
                },
                _destroyOnRemove: true
            });  
            popup.show();
        },
        
        //Removes a row from the subrange table
        removeRow : function(row){
            var grid = this.rangeTable;
            var store = grid.store;
            var topLevelItems = store._arrayOfTopLevelItems;
            var item = topLevelItems[row];
            var range = store.getValue(item, "range");
            var remove = confirm(this.locResources.DELETE_CONFIRMATION.replace("%1", range));
            if(remove){
                grid.store.deleteItem(item);
                grid.setItems(store._arrayOfTopLevelItems);
                if (this._newRange[range] == true) {
                    delete this._newRange[range];
                }
                this.setRangesInTable();
                this.checkForNetworkID();
                this._enableOk();
            } 
        },
        
        //Format for the "icon" column
        _formatIcons : function(value, rowIndex, cell){
            var editIconClass = "spEditIcon_v2";
            var editOnClick = "onClick=\"dijit.byId('" + this.id + "').editRow('" + rowIndex + "')\"";
            var removeIconClass = "spDeleteIcon_v2";
            var removeOnClick = "onClick=\"dijit.byId('" + this.id + "').removeRow('" + rowIndex + "')\"";
            var store = this.rangeTable.store;
            var allocPct = store._arrayOfTopLevelItems[rowIndex].allocatedPct;
            var rangeName = store._arrayOfTopLevelItems[rowIndex].range;
            var newRange = (this._newRange[rangeName] == true) ? true : false;
            if (allocPct != undefined) {
                var allocated = allocPct.toString().split("::");
                if (allocated[0] > 0) {
                    if(this.inUse && this.inUse == true){
                        editIconClass = "spEditInactiveIcon_v2";
                        editOnClick = "";
                        removeIconClass = "spDeleteInactiveIcon_v2";
                        removeOnClick = "";
                    }
                } else {
                    if(this.inUse && this.inUse == true){
                        if (!newRange) {
                            removeIconClass = "spDeleteInactiveIcon_v2";
                            removeOnClick = "";
                        }
                    }
                }
            }
            
            return "<div id=\"edit_" + rowIndex + "\" class=\""+editIconClass+"\"" + //edit
                editOnClick +
                "title=\"" + this.locResources.EDIT_TITLE + "\" style=\"display:inline-block;\"></div>" +
                "<div style=\"display:inline-block; width:5px\"></div>" + //spacing
                "<div id=\"remove_" + rowIndex + "\" class=\""+removeIconClass+"\"" + //remove
                removeOnClick +
                "title=\"" + this.locResources.REMOVE_TITLE + "\" style=\"display:inline-block;\" data-dojo-props=\"disabled:true\">";
        },
        
        //Creates and "Edit Subrange" popup
        editRow : function(row){
            var pool = this;
            var store = this.rangeTable.store;
            var item = store._arrayOfTopLevelItems[row];
            var originalRange = store.getValue(item, "range"); 
            var popup = new support.dialogs.SubrangePopup({
                id: "cp_newippool_editsubrangepopup",                
                mode: "edit",
                newPool: pool,
                network: this.network,
                gateway: this.gateway,
                dropDown: this.dropDown,
                range: originalRange,
                chassisId: store.getValue(item, "chassisId"),
                first: store.getValue(item, "first"),
                size: store.getValue(item, "size"),
                index: row,
                subrangeStore: this.rangeStore,
                overallStore: this.networkStore,
                version: this.version,
                editSubrange : function(){
                    if (originalRange != this.range) {
                        // Range name has changed during edit
                        // Was the original range a new one that was edited?
                        if (pool._newRange[originalRange]) {
                            // Mark the edited range name still new and clear out the original
                            pool._newRange[this.range] = true;
                            delete pool._newRange[originalRange];
                        }
                    }
                    store.setValue(item, "range", this.range);
                    store.setValue(item, "chassisId", this.chassisId);
                    store.setValue(item, "first", this.first);
                    store.setValue(item, "size", this.size);
                    store.setValue(item, "last", this.last);
                    pool.checkForNetworkID();
                    pool.setRangesInTable();
                },
                _destroyOnRemove: true
            });  
            popup.show(); 
        },
        
        //Checks to see if there is a network ID from the given information
        checkForNetworkID : function(){
            var network = this.calculateNetworkID();
            this.set("network", network);
            this._gateway.set("value", this._gateway.get("value"));
            this.updateNetworkTable();
        },
        
        _checkForOverlappingNetwork : function(network) {
            // 99286: One thing here....if this is an older pattern, it might actually overlap on edit.
            //       Maybe should validate here...
            if (!this.editMode) {
                var pool = this;
                var networkinfo = new Array();
                var networkIdList = this._getAllNetworkIDs();
                var overlappingNetwork = false;
                this.overlapMessage = "";
                if (this.version == this.IPV4_VALUE) {
                    dojo.forEach(networkIdList.items, function(networkid, idx) {
                        if (pool._matchesNetworkID(networkid.network) && networkid.protocol == pool.resources.IPV4) {
                            overlappingNetwork = true;                       
                            networkinfo.push(networkid);                        
                        }
                    });
                    if (overlappingNetwork) {
                        this.overlap = true;
                        var overlapHeading = "<div style=\"width:600px;\">" + pool.locResources.OVERLAPPING_NETWORK + "</div>" +
                                         "<div class=\"tooltipTable\">" +
                                         "<div class=\"tooltipRow\">" +
                                         "<div class=\"tooltipHeading\">" + pool.locResources.POOL_NAME + "</div>" +
                                         "<div class=\"tooltipHeading\">" + pool.locResources.SUBNET + "</div>" +
                                         "<div class=\"tooltipHeading\">" + pool.locResources.GATEWAY + "</div>" +
                                         "<div class=\"tooltipHeading\">" + pool.locResources.NETWORK_ID + "</div>" +
                                         "<div class=\"tooltipHeading\">" + pool.locResources.LAST_ADDRESS + "</div>" +                                          
                                         "</div>";
                        var overlapRow = new String();
                        dojo.forEach(networkinfo, function(poolitem, index) {
                            overlapRow += "<div class=\"tooltipRow\">" +
                                     "<div class=\"tooltipCell\">" + poolitem.name + "</div>" +
                                     "<div class=\"tooltipCell\">" + poolitem.netmask + "</div>" +
                                     "<div class=\"tooltipCell\">" + poolitem.gateway + "</div>" +
                                     "<div class=\"tooltipCell\">" + poolitem.network + "</div>" +
                                     "<div class=\"tooltipCell\">" + poolitem.highest + "</div>" +
                                     "</div>";
                        });
                        overlapRow += "</div>";
                        this.overlapMessage = overlapHeading + overlapRow;
                    } else {
                        this.overlap = false;
                        this.overlapMessage = "";
                    }
                } else if (this.version == this.IPV6_VALUE) {
                    dojo.forEach(networkIdList.items, function(networkid, idx) {
                        if (pool._matchesNetworkID(networkid.gateway) && networkid.protocol == pool.resources.IPV6) {
                            overlappingNetwork = true;                       
                            networkinfo.push(networkid);                        
                        }
                    });
                    if (overlappingNetwork) {
                        this.overlap = true;
                        var overlapHeading = "<div style=\"width:600px;\">" + pool.locResources.OVERLAPPING_NETWORK + "</div>" +
                                         "<div class=\"tooltipTable\">" +
                                         "<div class=\"tooltipRow\">" +
                                         "<div class=\"tooltipHeading\">" + pool.locResources.POOL_NAME + "</div>" +
                                         "<div class=\"tooltipHeading\">" + pool.locResources.SUBNET + "</div>" +
                                         "<div class=\"tooltipHeading\">" + pool.locResources.GATEWAY+ "</div>" +
                                         "<div class=\"tooltipHeading\">" + pool.locResources.LOWEST_ADDRESS + "</div>" +
                                         "<div class=\"tooltipHeading\">" + pool.locResources.HIGHEST_ADDRESS + "</div>" +                                          
                                         "</div>";
                        var overlapRow = new String();
                        dojo.forEach(networkinfo, function(poolitem, index) {
                            overlapRow += "<div class=\"tooltipRow\">" +
                                     "<div class=\"tooltipCell\">" + poolitem.name + "</div>" +
                                     "<div class=\"tooltipCell\">" + poolitem.netmask + "</div>" +
                                     "<div class=\"tooltipCell\">" + poolitem.gateway + "</div>" +
                                     "<div class=\"tooltipCell\">" + poolitem.lowest + "</div>" +
                                     "<div class=\"tooltipCellWide\">" + poolitem.highest + "</div>" +
                                     "</div>";
                        });
                        overlapRow += "</div>";
                        this.overlapMessage = overlapHeading + overlapRow;
                    } else {
                        this.overlap = false;
                        this.overlapMessage = "";
                    }                    
                }
                this._gateway.validate();
                return overlappingNetwork;
            }            
        },
        
        //Resets the table information so it shows in the table
        setRangesInTable : function(){
            this.rangeTable.setItems(this.rangeTable.store._arrayOfTopLevelItems);
        },
        
        //Reposition
        reposition : function(){
            this.dialogNode._position();
        },
        
        _setNameAttr : function(n){
            this.name = n;
            this._enableOk();
        },
        
        _setDescriptionAttr : function(d){
            this.description = d;
        },
        
        //Clears all data from the screen (when switching IPvs)
        _clearAllData : function(){
            this.dropDown = null;
            this.gateway = null;
            this.clearRangeTable();
        },
        
        //Clears all data from the subrange table
        clearRangeTable : function(){
            if(this.rangeTable != null){
                this.rangeStore = new dojo.data.ItemFileWriteStore({
                    data: { "items" : [] }
                });
                this.rangeTable.setStore(this.rangeStore);
                this.setRangesInTable();
            }
        },
        
        //Sets the new version and changes the dialog information
        _setVersionAttr : function(v){
            //if its not the same version, the dialog exists, and a grid exists
            if(this.version != v){
                var change = true;
                if(this.rangeTable != null && this.networkTable != null){
                    change = confirm(this.locResources.SWITCH_CONFIRMATION);
                }
                if(change){
                    this._version.set("selected", v);  // Set the button bar value
                    this.version = v;
                    this._clearAllData();
                    if(this.version == this.IPV4_VALUE){
                        this.invalidMessage = this.locResources.INVALID_MESSAGE_IPV4;
                        this.regExp = "^([0-9]|[0-9][0-9]|[0-1][0-9][0-9]|2[0-4][0-9]|25[0-5])([\.]([0-9]|[0-9][0-9]|[0-1][0-9][0-9]|2[0-4][0-9]|25[0-5])){3}$";
                        this.placeHolder = "0.0.0.0";
                        this._dropDownTitle.set("content", "<label id=\"NewIpPoolDropDownTitle_${id}\" class=\"required\">" + this.locResources.SUBNET + ": </label>");
                        if(this.networkTooltip != null){
                            var label = "<div style=\"width: 50em;\">" + this.locResources.NETWORK_TABLE_TOOLTIP_IPV4.replace("%1", this.PRODUCT_NAME_LONG).replace("%2", this.PRODUCT_NAME_SHORT) + "</div>";
                            this.networkTooltip.set("label", label);
                        }
                        this.networkStore = new dojo.data.ItemFileWriteStore({
                            data: {"items" : [ { network: null, broadcast: null, lowest: null, host: "0", highest: null } ]}                    
                        });
                    }
                    else if(this.version == this.IPV6_VALUE){
                        this.invalidMessage = this.locResources.INVALID_MESSAGE_IPV6;
                        this.regExp = "^\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*$";
                        this.placeHolder = "0::0";
                        this._dropDownTitle.set("content", "<label id=\"NewIpPoolDropDownTitle_${id}\" class=\"required\">" + this.locResources.ROUTING + ": </label>");
                        if(this.networkTooltip != null){
                            var label = "<div style=\"width: 50em;\">" + this.locResources.NETWORK_TABLE_TOOLTIP_IPV6.replace("%1", this.PRODUCT_NAME_LONG).replace("%2", this.PRODUCT_NAME_SHORT) + "</div>";
                            this.networkTooltip.set("label", label);
                            this.networkStore = new dojo.data.ItemFileWriteStore({
                                data: {"items" : [ { lowest: null,  highest: null } ]}                    
                            });
                        }
                    }
                    else{
                        this.invalidMessage = this.locResources.INVALID_MESSAGE;
                        this.regExp = ".*";
                        this.placeHolder = "";
                    }
                    if(this._gateway != null){
                        this._gateway.set("invalidMessage", this.invalidMessage);
                        this._gateway.set("regExp", this.regExp);
                        this._gateway.set("placeHolder", this.placeHolder);
                        this._gateway.set("value", "");
                    }                   
                    this._getDropDown();
                    this.set("network", null);
                    
                    if(this.networkTable != null){
                        this.networkLayout = this.getNetworkLayout();
                        this.networkTable.set("structure", this.networkLayout);
                        this.networkTable.setStore(this.networkStore);
                    }
                    this._enableOk();
                } else {
                    this._version.set("selected", this.version);  // Set the button bar value back to the previous
                }
               
            }
        },
        
        
        
        //127.127.255.254 - 127.0.0.1 = 8388605
        //two - one = difference
        _subtractAddresses : function(one, two){
            if(this._isValidAddress(one) && this._isValidAddress(two)){
                if(this.version == this.IPV4_VALUE){
                    var oneSplit = one.split(".");
                    var twoSplit = two.split(".");
                    var difference = 0;
                    for(var i=0; i<4; i++){
                        var partOne = parseInt(oneSplit[i]);
                        var partTwo = parseInt(twoSplit[i]);
                        difference += (partTwo - partOne) * Math.pow(256, 3-i);
                    }
                    return difference;
                }
                else if(this.version == this.IPV6_VALUE){
                    var oneSplit = one.split(":");
                    var twoSplit = two.split(":");
                    var difference = 0;
                    for(var i=0; i<8; i++){
                        //hex to dec
                        oneSplit[i] = parseInt(oneSplit[i], 16);
                        twoSplit[i] = parseInt(twoSplit[i], 16);
                        difference += (twoSplit[i] - oneSplit[i]) * Math.pow(65536, 7-i);
                    }
                    return difference;
                }
            }
            return null;
        },
        
        
        updateNetworkTable : function(){
            var pool = this;
            //needs network, broadcast, first, host, highest
            var grid = this.networkTable;
            if(grid != null){
                var store = grid.store;
                var gotItems = function(items){ 
                    var item = items[0];
                    var network = pool.calculateNetworkID();                        
                    pool._checkForOverlappingNetwork(network);                        
                    if(pool.version == pool.IPV4_VALUE){
                        var broadcast = pool.calculateBroadcastAddress(network);
                        var lowest = pool.calculateLowestAddress(network);
                        var host = pool.calculateHostAddress();
                        var highest = pool.calculateHighestAddress(network, broadcast, lowest, host);
                        store.setValue(item, "network", network);
                        store.setValue(item, "broadcast", broadcast);
                        store.setValue(item, "lowest", lowest);
                        store.setValue(item, "host", host);
                        store.setValue(item, "highest", highest);
                        grid.update();
                    } else if(pool.version == pool.IPV6_VALUE){
                        var network = pool.calculateNetworkID();
                        var lowest = pool.calculateLowestAddress(network);
                        var highest = pool.calculateHighestAddress(network, null, null, null);
                        store.setValue(item, "lowest", lowest);
                        store.setValue(item, "highest", highest);
                        grid.update();
                    }
                };
                store.fetch({onComplete:gotItems}); 
            }
        },
        
        _getAllNetworkIDs : function(){
            var networkIdList = null;
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "ippool/list",
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: function(response) {
                    networkIdList = response;
                }
            });
            return networkIdList;
        },        
        
        insertItemAtIndex : function(item, index){
            var store = this.rangeTable.store;
            var items = store._arrayOfTopLevelItems;
            var newStore = new dojo.data.ItemFileWriteStore({
                data: { "items" : [] }                    
            });
            for(var i=0; i<index; i++){
                var current = items[i];
                var currentItem = {id:store.getValue(current, "id"), allocatedPct:store.getValue(current, "allocatedPct"), range:store.getValue(current, "range"), chassisId:store.getValue(current, "chassisId"), first:store.getValue(current, "first"), size:store.getValue(current, "size"), last:store.getValue(current, "last") };
                newStore.newItem(currentItem);
            }
            newStore.newItem(item);
            for(var i=index; i<items.length; i++){
                var current = items[i];
                var currentItem = {id:store.getValue(current, "id"), allocatedPct:store.getValue(current, "allocatedPct"), range:store.getValue(current, "range"), chassisId:store.getValue(current, "chassisId"), first:store.getValue(current, "first"), size:store.getValue(current, "size"), last:store.getValue(current, "last") };
                newStore.newItem(currentItem);
            }
            this.rangeStore = newStore;
            this.rangeTable.setStore(newStore);
        },
        
        getInsertIndex : function(addressItem){
            var store = this.rangeTable.store;
            var items = store._arrayOfTopLevelItems;
            var addressFirst = addressItem.first;
            if(items.length == 0)
                return 0;
            else if(items.length == 1){
                var currentFirst = store.getValue(items[0], "first");
                var difference = this._subtractAddresses(currentFirst, addressFirst);
                if(difference > 0)
                    return 1;
                else
                    return 0;
            }
            else{
                for(var i=0; i<items.length-1; i++){
                    var current = items[i];
                    var next = items[i+1];
                    var currentFirst = store.getValue(current, "first");
                    var nextFirst = store.getValue(next, "first");
                    var firstDiff = this._subtractAddresses(currentFirst, addressFirst);
                    var secondDiff = this._subtractAddresses(addressFirst, nextFirst);
                    
                    if(i == 0 && firstDiff < 0 && secondDiff > 0)
                        return 0;
                    else if(i == items.length-2 && firstDiff > 0 && secondDiff < 0)
                        return items.length;
                    else if(firstDiff > 0 && secondDiff < 0){
                        //after, go to next
                    }
                    else{
                        //in between
                        if(firstDiff > 0 && secondDiff > 0)
                            return i+1;
                        //before first
                        else if(firstDiff < 0)
                            return i;
                    }                    
                }
            }
        },
       
        //Calculate broadcast address for IPv4
        calculateBroadcastAddress : function(network){
            var subnet = this.dropDown;
            if(this.version == this.IPV4_VALUE && subnet != null && this.network != null){
                var splitSub = subnet.split(".");
                var networkSplit = network.split(".");                
                var broadSplit = new Array(4);
                for(var i=0; i<splitSub.length; i++){
                    broadSplit[i] = parseInt(networkSplit[i]) + (255 - parseInt(splitSub[i]));
                }
                return broadSplit.join(".");
            }
            return null;     
        },
        
        //Calculate the lowest address (for overall network);
        calculateLowestAddress : function(network){
            if(this._isValidAddress(network) && this.dropDown != null){
                if(this.version == this.IPV4_VALUE){
                    var subnet = this.dropDown;
                    var lowest = new Array(4);
                    var splitSub = subnet.split(".");
                    var networkSplit = network.split(".");
                    for(var i=0; i<splitSub.length; i++){
                        if (i != splitSub.length-1)
                            lowest[i] = splitSub[i] & networkSplit[i];
                        else
                            lowest[i] = "1";
                    }
                    lowest = lowest.join(".");
                    if(this.gateway == lowest){
                        var lowestSplit = lowest.split(".");
                        lowest = lowestSplit[0] + "." + lowestSplit[1] + "." + lowestSplit[2] + ".2";
                    }
                    return lowest;
                }
                else if(this.version == this.IPV6_VALUE){
                    var prefix = this.dropDown;
                    //full notation
                    var fullIP = this._getIPv6FullNotation(network);
                    if(fullIP != null){
                        var hexArr = fullIP.split(":");
                        var binArr = new Array(8);
                        for(var i=0; i<8; i++){
                            binArr[i] = this.hex2bin(hexArr[i]);
                        }
                        var sub = this.getFullBinArray(binArr).join("").substring(0, prefix);
                        for(var i=prefix; i<128; i++){
                            sub += "0";
                        }
                        var newBinArr = sub.match(/.{1,16}/g);
                        for(var i=0; i<8; i++){
                            newBinArr[i] = this.bin2hex(newBinArr[i]);
                        }
                        return newBinArr.join(":").toUpperCase();
                    }
                }
            }
            return null;  
        },
        
        //Calculates the number of host addresses in an IPv4 network
        calculateHostAddress : function(){
            var subnet = this.dropDown; 
            var gateway = this.gateway;
            
            if(this.version == this.IPV4_VALUE && subnet != null){
                var bit = null;
                var host = "0";
                var subnets = ["128.0.0.0", "192.0.0.0", "224.0.0.0", 
                               "240.0.0.0", "248.0.0.0", "252.0.0.0",
                               "254.0.0.0", "255.0.0.0", "255.128.0.0",
                               "255.192.0.0", "255.224.0.0", "255.240.0.0",
                               "255.248.0.0", "255.252.0.0", "255.254.0.0", 
                               "255.255.0.0", "255.255.128.0", "255.255.192.0", 
                               "255.255.224.0", "255.255.240.0", "255.255.248.0", 
                               "255.255.252.0",  "255.255.254.0", "255.255.255.0",
                               "255.255.255.128", "255.255.255.192", "255.255.255.224",
                               "255.255.255.240", "255.255.255.248", "255.255.255.252"];
                    var bits = ["1", "2", "3", "4", "5", "6", "7", "8", "9",
                               "10", "11", "12", "13", "14", "15", "16", "17", "18", 
                               "19", "20", "21", "22",  "23", "24", "25", "26", "27",
                               "28", "29", "30"];
                    for(var i=0; i<subnets.length; i++){
                        if(subnet == subnets[i])
                            bit = bits[i];
                    }
                    bit = parseInt(bit);
                    if(this._isValidAddress(gateway))
                       host = (Math.pow(2, 32-bit) - 3) + "";
                    else
                        host = (Math.pow(2, 32-bit) - 2) + "";
                    return host;
            }
            return "0";
        },
        
        //Calculates the highest address (for overall network table)
        calculateHighestAddress : function(network, broadcast){
            if(this.dropDown != null && this._isValidAddress(network)){
                if(this.version == this.IPV4_VALUE && this._isValidAddress(broadcast)){
                    var splitFirst = broadcast.split(".");
                    splitFirst[3] = parseInt(splitFirst[3]) - 1;
                    var highest = splitFirst.join("."); 
                    if(this.gateway == highest){
                        var highestSplit = highest.split(".");
                        highest = highestSplit[0] + "." + highestSplit[1] + "." + highestSplit[2] + "." + (parseInt(highestSplit[3])-1);
                    }
                    return highest;
                }
                else if(this.version == this.IPV6_VALUE){
                    var fullIP = this._getIPv6FullNotation(network);
                    if(fullIP != null){
                        var prefix = this.dropDown;
                        var hexArr = fullIP.split(":");
                        var binArr = new Array(8);
                        for(var i=0; i<8; i++){
                            binArr[i] = this.hex2bin(hexArr[i]);
                        }
                        var sub = this.getFullBinArray(binArr).join("").substring(0, prefix);
                        for(var i=prefix; i<128; i++){
                            sub += "1"; //add 1's (binary)
                        }
                        var newBinArr = sub.match(/.{1,16}/g);
                        for(var i=0; i<8; i++){
                            newBinArr[i] = this.bin2hex(newBinArr[i]);
                        }
                        return newBinArr.join(":").toUpperCase();
                    }
                }
            }
            return null;
        },
       
        //Determines if any subranges exist in the table
        hasSubranges : function(){
            var store = this.rangeStore;
            var items = store._arrayOfTopLevelItems;
            return items.length > 0;
        },
        
        determineNetworkBase : function(){
            var base = null;
            var gateway = this.gateway;
            var hasSubranges = this.hasSubranges();
            var hasGateway = this._isValidAddress(gateway);
            //if it has a gateway and no subranges, use the gateway
            if( hasGateway && !hasSubranges )
                base = gateway;
            //if it has subranges and no gateway, use the subranges
            else if( hasSubranges && !hasGateway ){
                var items = this.rangeTable.store._arrayOfTopLevelItems;
                if(items.length > 0){
                    var item = items[0];
                    if(item.first != null)
                        base = item.first[0];
                }
            }
            //if it has both, use existing network
            else if( hasSubranges && hasGateway ){
                if(this.network != null && gateway == this.network)
                    base = this.network;
                else{
                    //use subrange in table
                    var items = this.rangeTable.store._arrayOfTopLevelItems;
                    if(items.length > 0 && items[0].first != null)
                        base = items[0].first[0];
                    else
                        base = gateway;
                }
            }
            //neither
            return base;
        },
        
        calculateNetworkID : function(){
            var base = this.determineNetworkBase();            
            if(this._isValidAddress(base) && this.dropDown != null){
                if(this.version == this.IPV4_VALUE){
                    var subnet = this.dropDown;
                    var splitSub = subnet.split(".");                    
                    var networkSplit = base.split(".");                    
                    var net = new Array(4);
                    for(var i=0; i<splitSub.length; i++){
                        net[i] = splitSub[i] & networkSplit[i];
                    }
                    net = net.join(".");
                    this.set("network", net);
                    return net;
                }
                else if(this.version == this.IPV6_VALUE && this._isValidNumber(this.dropDown)){
                    var prefix = this.dropDown;
                    var net = this.getIPv6NetworkID(base, prefix).toUpperCase();
                    this.set("network", net);
                    return net;
                }
            }
            this.set("network", null);
            return null;
        },
        
        _setDropDownAttr : function(d){
            this.dropDown = d;
            this.checkForNetworkID();
            this.reposition();
            this._enableOk();
        },
        
        _setGatewayAttr : function(g){
            this.gateway = g;
            this.calculateNetworkID();
            this._enableOk();
        },
          
        //Determines if an IP address is valid based on the version
        _isValidAddress : function(ip){
            if(ip == null || ip == "")
                return false;
            var reg = new RegExp(this.regExp);
            return reg.test(ip);
        },
        
        _isValidNumber : function(num){
            if(num == null || num == "")
                return false;
            var reg = new RegExp("^[1-9][0-9]*$");
            return reg.test(num);
        },
        
        _setNetworkAttr : function(id){
            this.network = id;
            this._enableOk();
        },
        
        _enableOk : function(){
            var hasItem = false;
            if(this.rangeTable != null){
                    var store = this.rangeTable.store;
                    var items = store._arrayOfTopLevelItems;
                    hasItem = (items.length > 0);
            }
            if(this.name != null && this.dropDown != null && this.dropDown != "" && 
                    hasItem && this._gateway.isValid()) {
                this._okBtn.set("disabled", false);
                this._saveasBtn.set("disabled", false);               
            }
            else {
                this._okBtn.set("disabled", true);
                this._saveasBtn.set("disabled", true);
            }
        },
        
        getRangesAsJson : function(){
            var grid = this.rangeTable;
            var store = grid.store;
            var topLevelItems = store._arrayOfTopLevelItems;
            var ranges = new Array();            
            for(var i=0; i<topLevelItems.length; i++){
                var item = topLevelItems[i];
                var rangeItem = new Object();
                rangeItem.id = store.getValue(item, "id");
                rangeItem.name = store.getValue(item, "range");
                rangeItem.chassisId = store.getValue(item, "chassisId");
                rangeItem.first = store.getValue(item, "first");
                rangeItem.size = store.getValue(item, "size");
                rangeItem.last = store.getValue(item, "last");
                ranges.push(rangeItem);
            }            
            return ranges;
        },
        
        getNetworkAsJson: function(){
            var grid = this.networkTable;
            var store = grid.store;
            var items = store._arrayOfTopLevelItems;
            var networkAsJson = new Object();
            for(var i=0; i<1; i++){
                var item = items[i];
                if(this.version == this.IPV4_VALUE){
                    networkAsJson.network = store.getValue(item, "network");
                    networkAsJson.broadcast = store.getValue(item, "broadcast");
                    networkAsJson.lowest = store.getValue(item, "lowest");
                    networkAsJson.host = store.getValue(item, "host");
                    networkAsJson.highest = store.getValue(item, "highest");
                }
                else if(this.version == this.IPV6_VALUE){
                    networkAsJson.lowest = store.getValue(item, "lowest");
                    networkAsJson.highest = store.getValue(item, "highest");
                }
            }
            return networkAsJson;     
        },
        
        _okClicked : function(){
            var success = false;
            if(this.editMode){
                success = this.xhrPutNewIPPool();
            }
            else{
                success = this.xhrPostNewIPPool();
            }
        },
        
        saveasClicked: function() {  
            var poolObject = new Object();
            
            poolObject.name = this.name;
            poolObject.description = this.description;
            poolObject.protocol = this.version;
            poolObject.netmask = this.dropDown;
            poolObject.gateway = this.gateway;          
            poolObject.ranges =  this.getRangesAsJson();
            
            var grid = this.networkTable;
            var store = grid.store;
            var items = store._arrayOfTopLevelItems;
            var item = items[0];
            if(this.version == this.IPV4_VALUE){
                poolObject.network = store.getValue(item, "network");
                poolObject.broadcast = store.getValue(item, "broadcast");
                poolObject.lowest = store.getValue(item, "lowest");
                poolObject.host = store.getValue(item, "host");
                poolObject.highest = store.getValue(item, "highest");
            }
            else if(this.version == this.IPV6_VALUE){
                poolObject.lowest = store.getValue(item, "lowest");
                poolObject.highest = store.getValue(item, "highest");
            }
            
            this.saveasDialog.set("templateObject",poolObject);
            this.saveasDialog.set("saveTemplateType",this.IPPOOL_NAME);
            this.saveasDialog.show();
        },
        
        _cancelClicked : function(){
            this.hide();
            dojo.destroy(this.id + "_underlay");
        },
        
        getFinalData : function(){
            var poolObject = new Object();
            
            poolObject.name = this.name;
            poolObject.description = this.description;
            poolObject.protocol = this.version;
            poolObject.netmask = this.dropDown;
            poolObject.gateway = this.gateway;          
            poolObject.ranges =  this.getRangesAsJson();
            
            var grid = this.networkTable;
            var store = grid.store;
            var items = store._arrayOfTopLevelItems;
            var item = items[0];
            if(this.version == this.IPV4_VALUE){
                poolObject.network = store.getValue(item, "network");
                poolObject.broadcast = store.getValue(item, "broadcast");
                poolObject.lowest = store.getValue(item, "lowest");
                poolObject.host = store.getValue(item, "host");
                poolObject.highest = store.getValue(item, "highest");
            }
            else if(this.version == this.IPV6_VALUE){
                poolObject.lowest = store.getValue(item, "lowest");
                poolObject.highest = store.getValue(item, "highest");
            }
                       
            var argJSONString = dojo.toJson(
                poolObject
            );
            return argJSONString;
        },
        
        getIPv6NetworkID : function(base, prefix){
            if(this.version == this.IPV6_VALUE && this._isValidAddress(base) && prefix != null){
                //full notation
                var fullIP = this._getIPv6FullNotation(base);
                if(fullIP != null){
                    var hexArr = fullIP.split(":");
                    
                    var binArr = new Array(8);
                    for(var i=0; i<8; i++){
                        binArr[i] = this.hex2bin(hexArr[i]);
                    }
                    
                    var fullBin = this.getFullBinArray(binArr);
                    
                    fullBin = fullBin.join("");
                    var sub = fullBin.substring(0, prefix);
                    //HERE: SUB IS BINARY PREFIX
                    
                    for(var i=prefix; i<128; i++){
                        sub += "0";
                    }
                    
                    var newBinArr = sub.match(/.{1,16}/g);
                    
                    for(var i=0; i<8; i++){
                        newBinArr[i] = this.bin2hex(newBinArr[i]);
                    }
                    return newBinArr.join(":");
                }
            }
            return null;
        },
        
       /*IPv6 CALCULATION*/
        
        hex2bin : function(hex){
            var dec = parseInt(hex, 16);
            return dec.toString(2);
        },
        
        //used
        bin2hex : function(bin){
            var dec = parseInt(bin, 2);
            return dec.toString(16);
        },
        
        //used
        getFullBinArray : function(binArr){
            for(var i=0; i<binArr.length; i++){
                var binItem = binArr[i];
                for(var j=binItem.length; j<16; j++){
                    //add leading 0s
                    binArr[i] = "0" + binArr[i];
                }
            }
            return binArr;
        },
        
        _getIPv6FullNotation : function(/*String*/ ipv6){
            if(this.version == this.IPV6_VALUE && this._isValidAddress(ipv6)){
              //convert to full notation
                var sep = ipv6.split(":");
                for(var i=0; i<sep.length; i++){
                    var part = sep[i];
                    if(part == ""){
                        sep.splice(i, 1);
                        var num = 8 - sep.length;
                        for(var j=0; j<num; j++){
                            sep.splice(i, 0, "0000");
                        }
                    }
                }
                return sep.join(":");
            }
            return null;            
        },
        
        /*PUT, POST, GET*/
        
        xhrPutNewIPPool : function(){
            var finalData = this.getFinalData();
            var success = false;
            
            this.getProgressDialog("pool").show(); // Show progress dialog for save request.
            dojo.xhrPut({
                url : this.REST_PATH + "ippool/" + this.poolIDNum,
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                putData: finalData,
                load : dojo.hitch(this, function(data) {
                    dojo.publish("ProgressComplete", data);
                    data = this._version.get("selected");
                    dojo.publish("ippoolComplete", data);
                    dojo.publish("RefreshTable", "");
                    this.hide();
                    dojo.destroy(this.id + "_underlay");
                    success = true;                    
                }),
                error : dojo.hitch(this, function(error) {
                    dojo.publish("ProgressFailed", error);
                    alert(this.locResources.ERROR_MESSAGE + ": " + error);
                    success = false;
                })
            });
            //show final info
            console.log(this.declaredClass + ": ", finalData);
            return success;
        },
        
        xhrPostNewIPPool : function(){
            var finalData = this.getFinalData();
            var success = false;
            
            this.getProgressDialog("save").show(); // Show progress dialog for save request.
            dojo.xhrPost({
                url : this.REST_PATH + "ippool/create",
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                postData: finalData,
                load : dojo.hitch(this, function(data) {
                    dojo.publish("ProgressComplete", data);
                    dojo.publish("ippoolComplete", data);
                    dojo.publish("RefreshTable", "");
                    this.hide();
                    dojo.destroy(this.id + "_underlay");
                    success = true;
                }),
                error : dojo.hitch(this, function(error) {
                    dojo.publish("ProgressFailed", error);
                                        
                    switch(error.status) {
                        case 400:
                            var errorMessage = dojo.fromJson(error.responseText);
                            if (errorMessage.message == "service_error_name_in_use") {
                                var errorDialog = new support.temp.ModalDialog({
                                    type: "Error",
                                    text: this.locResources.TEMPLATE_NAME_USED
                                });

                                errorDialog.show();                                
                            }
                            break;
                        default:                            
                            var errorDialog = new support.temp.ModalDialog({
                                type: "Error",
                                text: error
                            });

                            errorDialog.show();
                            break;
                    }                    
                    success = false;
                })
            });
            //show final info
            console.log(this.declaredClass + ": ", finalData);
            return success;
        },
        
        xhrGetNewIPPool : function(){
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "ippool/" + this.poolIDNum,
                handleAs: "json",
                sync: true,
                preventCache: true,
                load: dojo.hitch(this, function(response) {
                    this.poolIDNum = response.id;
                    if(response.name != null){
                        this.set("editMode", true);
                        this._okBtn.set("label", this.resources.SAVE);
                        this._saveasBtn.set("style", "display:inline");
                        this.inUse = response.inUse;
                        this.dialogNode.set("title", this.locResources.EDIT_DIALOG + " - " + response.name);
                        
                        this._name.set("value", response.name);
                        this._name.set("disabled", true);
                        this._desc.set("value", response.description);
                        this._setVersionAttr(response.protocol);
                        //this._version.set("selected", response.protocol);
                        this._dropDown.set("value", response.netmask);
                        this._gateway.set("value", response.gateway);
                        
                        this.showTables();
                        this.rangeStore = new dojo.data.ItemFileWriteStore({
                            data: {"items": response.ranges }
                        });
                        this.rangeTable.setStore(this.rangeStore);
                        this.setRangesInTable();
                        var networkData = new Object();
                        if(this.version == this.IPV4_VALUE){
                            networkData.network = response.network;
                            networkData.broadcast = response.broadcast;
                            networkData.lowest = response.lowest;
                            networkData.host = response.host;
                            networkData.highest = response.highest;
                        }
                        else if(this.version == this.IPV6_VALUE){
                            networkData.lowest = response.lowest;
                            networkData.highest = response.highest;
                        }
                        this.networkStore = new dojo.data.ItemFileWriteStore({
                            data: {"items": [networkData] }
                        });
                        this.networkTable.set("structure", this.getNetworkLayout());
                        this.networkTable.setStore(this.networkStore);
                        if(this.inUse && this.inUse == true){
                            this._setPageReadOnly(true);
                        }
                        
                    }
                })
            });
        },
        
        _setPageReadOnly : function(editInUse){
            //Get rid of save buttons
            this._saveasBtn.set("style", "display:none");
            //Disable subnet and gateway inputs
            this._dropDown.set("disabled", true);
            this._gateway.set("disabled", true);
            //Disable IP mode button bar
            var i;
            for(i=0;i<this._version._buttonNodes.length;i++){
                this._version._buttonNodes[i].set("disabled", true);
            }
            
            // Add this check if we want to display the read only page outside
            // of the context of editing an in use IP pool pattern
            if (!editInUse) {
				this._okBtn.set("style", "display:none");
				//Disable description - name input already disabled
            	this._desc.set("disabled",true);				
            	//Disable add row button
            	this._addSubrangeButton._hoverButton.set("disabled", true);
            }
        }
    });
});