define([
        "dojo",
        "dijit",
        "support/dialogs/Dialog",
        "dojo/i18n!./nls/NewPortVirtualizationTemplate",
         "dojo/text!./templates/NewPortVirtualizationTemplate.html",
         "dijit/layout/ContentPane",
         "support/widgets/Select",
         "support/dialogs/ReDeployServerTemplate",
         "support/panels/PortVirtualizationTable",
         "dijit/form/SimpleTextarea",        
         "dojox/data/AndOrReadStore",
         "support/dialogs/SaveAs",
         "support/dialogs/ProgressDialog"
    ], function(dojo, dijit, dialog, mri, template) {
   
    
    dojo.declare("support.dialogs.NewPortVirtualizationTemplate", [ support.dialogs.Dialog], {
        
        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,
        
        baseClass: "NewPortVirtualizationTemplate",
        
        templateIDNum: null, // Used for "edit" mode
        adapterId: null, // Used for "create" mode
        mode: null, // Used for "create" mode
        protocol: null, // Used for "create" mode
        
        numberOfVirtualFunctions: 4,
        
        _physicalFunctionModel: null,
        _physicalFunctionStore: null,
        _physicalFunctionLayout: null,
        _targetAdapterTypeSelector: null,
        _targetAdapterTypeStore: null,
        _targetPortOperationalModeSelector: null,
        _targetPortProtocolsSelector: null,
        _targetPortOperationalModeStore: null,
        _targetPortProtocolsStore: null,
        _defaultPortExtendedSettingsTemplateStore: null,
        _defaultPortExtendedSettingsTemplateSelector: null,
        _supportedIOAdaptersList: null,      
        
        _portVirtualizationTable: null,
        
        _pattern: null,
        
        _enablefunctionList: null,
        _slidervalueList: null,        
        _templateIdList: null,
        
        _isVlanValid: false,
        _isPctValid: false,
        
        _vendor: null,
        _chipset: null,
        _portExtendedTemplateID: "",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/NewPortVirtualizationTemplate.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
            
            // Initialize arrays
            this._enablefunctionList = [];
            this._slidervalueList = [];
            this._templateIdList = [];
            
            this.subscribe("portVirtTableComplete", function(data) {                
                if (this.templateIDNum != null) {
                    this._setSelectorsFromService();
                } else {
                    if (this._portVirtualizationTable != null) {
                        for (var count=0;count < this.numberOfVirtualFunctions;++count) {
                            this._portVirtualizationTable.setSliderValue(100/this.numberOfVirtualFunctions, count);
                        }
                    }
                }
                this._targetPortOperationalModeSelector.startup();
                this._targetPortProtocolsSelector.startup();
            });
            this.subscribe("InvalidPhysicalFunction", function(data) {               
                this._isVlanValid = data;
                if (!data) {
                    this.okButton.set("disabled", true);
                    this.saveBtn.set("disabled", true);
                    this.saveasBtn.set("disabled", true);                    
                } else {
                    var tgtOperationalMode = this._targetPortOperationalModeSelector.get("value");
                    if (this._isPctValid || (tgtOperationalMode == "vNIC_IBMvf") || (tgtOperationalMode == "vNIC_IBMufp")) {
                        this._nameChanged();
                    }
                }                
            });
            this.subscribe("ExPortTemplateComplete", function(data) {                
                var portExtendedSettingsTemplateData = null;
                var xhrPortExtTemp = {
                        url: this.REST_PATH + "template/list/ExtendedPort?Props=vendor,chipset,protocol",
                        handleAs: "json",
                        sync: true,
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        load: function(data) {
                            portExtendedSettingsTemplateData = data;
                        },
                        error: function(error) {
                            alert(error);
                        }
                };

                dojo.xhrGet(xhrPortExtTemp);
                
                this._defaultPortExtendedSettingsTemplateStore = new dojox.data.AndOrReadStore({
                    data: portExtendedSettingsTemplateData
                });                
                
                this._defaultPortExtendedSettingsTemplateSelector.setStore(this._defaultPortExtendedSettingsTemplateStore, null, {
                    query: {
                        complexQuery: "vendor: '" + this._vendor + "' AND chipset: '" + this._chipset + "' AND protocol: 'ethernet'"
                    }
                });
                
                this._defaultPortExtendedSettingsTemplateSelector.addOption({
                    value: "*NONE",
                    label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
                });
                
                this._defaultPortExtendedSettingsTemplateSelector.set("value", data);                
            });

            this._connectDataModels();            
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            
            this.layout();
            
            this._descLimit.innerHTML = this.resources.LIMIT.replace("%500","500");
            
            for(var count=0;count < this.numberOfVirtualFunctions;++count) {
                this._enablefunctionList.push(true);
            }
            
            if (this.templateIDNum != null) {
                this._readSettingsFromService();
            }
            
            this._setVendorandChipset();
            
            this._createSelectBoxes();
            
            if (this.templateIDNum != null) {
                this._setSelectorsFromService();
            }            

            dojo.connect(this.dialogNode, "hide", this, function() {
                this.destroy();
            });

            
            this._setup();
                    
            this.templateName.intermediateChanges = true; //Allow code to catch auto updates of field
            
            if (this.mode == "vNIC_independent") {
                if (this._portVirtualizationTable != null) {
                    this._portVirtualizationTable.toggleOverallPctSection("block");
                }                
            }
            else if ((this.mode == "vNIC_IBMvf") || (this.mode == "vNIC_IBMufp")) {
                if (this._portVirtualizationTable != null) {
                    this._portVirtualizationTable.toggleOverallPctSection("none");
                }
            }

        },

        _connectDataModels: function() {
            
            var adapterData = null;
            var xhrArgs = {
                url: this.REST_PATH + "supportedDevices/ioAdapter/list",
                handleAs: "json",
                sync: true,
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                load: function(data) {
                    adapterData = data;
                },
                error: function(error) {
                    alert(error);
                }
            };
            
            // Call the asynchronous xhrGet
            dojo.xhrGet(xhrArgs);
            
            this._supportedIOAdaptersList = adapterData;
            
            this._targetAdapterTypeStore = new dojox.data.AndOrReadStore({
                data: adapterData
            });
            
            /** Port Extended Settings rest call  ==NEW== **/                                           
            var portExtendedSettingsTemplateData = null;
            var xhrPortExtTemp = {
                    url: this.REST_PATH + "template/list/ExtendedPort?Props=vendor,chipset,protocol",
                    handleAs: "json",
                    sync: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: function(data) {
                        portExtendedSettingsTemplateData = data;
                    },
                    error: function(error) {
                        alert(error);
                    }
            };

            dojo.xhrGet(xhrPortExtTemp);
            
            //alert(dojo.toJson(portExtendedSettingsTemplateData));
            
            this._defaultPortExtendedSettingsTemplateStore = new dojox.data.AndOrReadStore({
                data: portExtendedSettingsTemplateData
            });                   
        },
        
        _readSettingsFromService: function() {

            var pattern = null;
            dojo.xhrGet({
                url: this.REST_PATH + "template/" + this.templateIDNum,
                handleAs: "json",
                sync: true,
                load: function(response) {
                    pattern = (response && response.items && (response.items.length > 0)) ? response.items[0] : null;
                },
                error: function(error) {
                    alert(this.locResources.ERRORTEMPLATE);
                }
            });
            if (pattern) {
                this._pattern = pattern;
                
                this.adapterId = this._pattern.adapterType;
                this.mode = this._pattern.portCfgMode;
                this.protocol = this._pattern.portCfgOptionCode;
                this.portVirtDesc.set("value", (this._pattern.description) ? this.decodeSpecialCharacters(this._pattern.description) : "");            
                this._portExtendedTemplateID = this._pattern.extendedPortTemplateId;
                this.templateInUse = this._pattern.inUse;
                if (this._pattern.userDefined) {

                } else {
                    this.templateMode = "copy";
                }

                this.saveBtn.set("disabled", false);
                this.saveasBtn.set("disabled", false);
                if (this.templateMode == "copy") {
                    this.dialogNode.set("title", this.locResources.TITLE + " - " + this._pattern.name + " - " + this.resources.COPY);
                    this.templateName.set("value", this._pattern.name + " - " + this.resources.COPY);
                } else {
                    this.dialogNode.set("title", this.locResources.EDITTITLE + " - " + this._pattern.name);
                    this.templateName.set("value", this._pattern.name);
                }
            }
        },
        
        _setVendorandChipset: function () { 
            if (this.adapterId != null && this.adapterId != "*ANY") {
                var adapter = this.adapterId;
            
                for (var adapterCount=0;adapterCount < this._supportedIOAdaptersList.items.length;++adapterCount) {
                    if (this._supportedIOAdaptersList.items[adapterCount].id == adapter) {
                        this._vendor = this._supportedIOAdaptersList.items[adapterCount].controllers[0].vendor.toString();
                        if (this._supportedIOAdaptersList.items[adapterCount].controllers[0].chipset != undefined) {
                            this._chipset = this._supportedIOAdaptersList.items[adapterCount].controllers[0].chipset.toString();
                        } else {
                            this._chipset = "*NONE";
                        }                       
                    }
                }            
            } else {
                this._vendor = "Emulex";
                this._chipset = "BE3";
            }
        },
              
        _setSelectorsFromService: function() {
            if (this._portVirtualizationTable == null) {                
                return;
            }
            
            for (var vportcount=0;vportcount < this._pattern.vports.length;++vportcount) {
                if (this._pattern.vports[vportcount].minSpeed != undefined) {
                    if(this._pattern.vports[vportcount].minSpeed == 0){
                        this._pattern.vports[vportcount].minSpeed = 1;
                    }
                    this._portVirtualizationTable.setSliderValue(this._pattern.vports[vportcount].minSpeed, vportcount);
                }                
                if (this.mode == "vNIC_independent") { 
                    if (this._pattern.vports[vportcount].enabled != undefined) {                                   
                        this._portVirtualizationTable.setEnableValue(this._pattern.vports[vportcount].enabled, vportcount);
                    } else {
                        this._portVirtualizationTable.setEnableValue(false, vportcount);
                        this._enablefunctionList[vportcount] = false;
                    }                    
                }
                if (this._pattern.vports[vportcount].vlanId != undefined) {
                    var vlanId = 0;
                    if (this._pattern.vports[vportcount].vlanId != "*NONE" && this._pattern.vports[vportcount].vlanId != "") {
                        vlanId = this._pattern.vports[vportcount].vlanId;
                    }
                    this._portVirtualizationTable.setVlanValue(vlanId, vportcount);
                }
            }
            this._defaultPortExtendedSettingsTemplateSelector.set("value", this._portExtendedTemplateID);
        },

        _targetPortOperationModeChange: function() {
            if (this.mode != null) {                   
                this._targetPortOperationalModeSelector.set("value", this.mode);
                this._targetPortOperationalModeSelector.set("disabled", true);
            }
            var tgtOperationalMode = this._targetPortOperationalModeSelector.get("value");
            this._portVirtualizationTable.modeChanged(tgtOperationalMode);
            if (tgtOperationalMode == "vNIC_independent") {
               this._portVirtualizationTable.toggleOverallPctSection("block");
            }
            else if ((tgtOperationalMode == "vNIC_IBMvf") || (tgtOperationalMode == "vNIC_IBMufp")) {
                this._portVirtualizationTable.toggleOverallPctSection("none");
                this.isPctValid = true;
            }            
        },
        
        _targetPortProtocolChange: function() {
            if (this.protocol != null) {                   
                this._targetPortProtocolsSelector.set("value", this.protocol);
                this._targetPortProtocolsSelector.set("disabled", true);
            }
            this._portVirtualizationTable.protocolChanged(this._targetPortProtocolsSelector.get("value"));

            this._setPortExtendedSelectorStore();            
        },
        
        _createSelectBoxes: function() {
            this._targetAdapterTypeSelector = new support.widgets.Select({
                id: this.get("id") + "_targetTypeSelector",
                name: "targetAdapterType",
                store: this._targetAdapterTypeStore,
                query: {
                    // Only templates for specified adapter.  
                    // Don't include the converged adapter (yet?)
                    complexQuery: "adapterType: 'vfa' AND NOT id: 'adapter:14'"
                },   
                maxHeight: -1,
                style: {
                    width: "250px"    //this._defaultselectorwidth
                },
                disabled: false
            }).placeAt(this._targetadaptertype);

            dojo.connect(this._targetAdapterTypeSelector, "onChange", this, function() {
                this._initializePortVirtualizationTable();
            });
            
            dojo.connect(this._targetAdapterTypeSelector, "startup", this, function() {
                this._initializePortVirtualizationTable();
            });            
            
            this._targetPortOperationalModeSelector = new support.widgets.Select({
                id: this.get("id") + "_targetPortOperationalModeSelector",
                name: "targetPortOperationalMode",
                maxHeight: -1,
                style: {
                    width: "250px"    //this._defaultselectorwidth
                },
                disabled: false
            }).placeAt(this._targetportoperationmode);
            this._targetPortOperationalModeSelector.addOption({
                value: "vNIC_independent",
                label: this.resources.VNICINDEPENDENT
            });
            this._targetPortOperationalModeSelector.addOption({
                value: "vNIC_IBMvf",
                label: this.resources.VNICVF
            });
            this._targetPortOperationalModeSelector.addOption({
                value: "vNIC_IBMufp",
                label: this.resources.VNICUFP
            });
            dojo.connect(this._targetPortOperationalModeSelector, "startup", this, this._targetPortOperationModeChange);
            dojo.connect(this._targetPortOperationalModeSelector, "onChange", this, this._targetPortOperationModeChange);

            this._targetPortProtocolsSelector = new support.widgets.Select({
                id: this.get("id") + "_targetPortProtocolsSelector",
                name: "targetPortProtocols",
                maxHeight: -1,
                style: {
                    width: "250px"    //this._defaultselectorwidth
                },
                disabled: false
            }).placeAt(this._targetportprotocols);
            this._targetPortProtocolsSelector.addOption({
                value: "ethOnly",
                label: this.resources.ETHONLY
            });
            this._targetPortProtocolsSelector.addOption({
                value: "eth+fcoe",
                label: this.resources.ETHFCOE
            });            
            this._targetPortProtocolsSelector.addOption({
                value: "eth+iscsi",
                label: this.resources.ETHISCSI
            });
            dojo.connect(this._targetPortProtocolsSelector, "startup", this, this._targetPortProtocolChange);
            dojo.connect(this._targetPortProtocolsSelector, "onChange", this, this._targetPortProtocolChange);

            var protocol = "ethernet";
            //if (this._targetPortProtocolsSelector.get("value") == "ethOnly") {
            //    protocol = "ethernet";
            //}
                               
            this._defaultPortExtendedSettingsTemplateSelector = new support.widgets.Select({
                id: this.get("id") + "_portExtendedSettingsTemplateSelector",
                name: "defaultPortExtendedSettingsTemplate",
                store: this._defaultPortExtendedSettingsTemplateStore,
                maxHeight: -1,
                emptyLabel: this.resources.NO_TEMP_DEF_HYPHENS,
                query: {
                    complexQuery: "vendor: '" + this._vendor + "' AND chipset: '" + this._chipset + "' AND protocol: '" + protocol + "'"
                },
                style: {
                    width: "250px"
                },
                disabled: false
            }).placeAt(this._portextendedsettings);
            if (this._defaultPortExtendedSettingsTemplateSelector.get("value") != "") {
                this._defaultPortExtendedSettingsTemplateSelector.addOption({
                    value: "*NONE",
                    label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
                });
                if (this._defaultPortExtendedSettingsTemplateSelector.get("value") == "*NONE") {                
                    dojo.replaceClass(this._editIconPortExtended, "spEditInactiveIcon_v2", "spEditIcon_v2");                
                }   
            } else {
                dojo.replaceClass(this._editIconPortExtended, "spEditInactiveIcon_v2", "spEditIcon_v2");
            }
            
            this._defaultPortExtendedSettingsTemplateSelector.startup();
            
            dojo.connect(this._defaultPortExtendedSettingsTemplateSelector, "onChange", this, function(newValue) {
                if (this._portExtendedTemplateID != "") {
                    this._defaultPortExtendedSettingsTemplateSelector.set("value", this._portExtendedTemplateID);
                    this._portExtendedTemplateID = "";
                } else {
                    this._defaultPortExtendedSettingsTemplateSelector.set("value", newValue);
                }
                
                if (this._defaultPortExtendedSettingsTemplateSelector.get("value") != "") { 
                    if (this._defaultPortExtendedSettingsTemplateSelector.get("value") == "*NONE") {                    
                        dojo.replaceClass(this._editIconPortExtended, "spEditInactiveIcon_v2", "spEditIcon_v2");                
                    } else {                    
                        dojo.replaceClass(this._editIconPortExtended, "spEditIcon_v2", "spEditInactiveIcon_v2");
                    }                
                    this._defaultPortExtendedSettingsTemplateSelector.set("disabled", false);
                } else {
                    this._defaultPortExtendedSettingsTemplateSelector.set("disabled", true);
                    dojo.replaceClass(this._editIconPortExtended, "spEditInactiveIcon_v2", "spEditIcon_v2");
                }
            });
            
            this._targetAdapterTypeSelector.startup();
            
            this._targetAdapterTypeSelector.addOption({
                value: "*ANY",
                label: this.resources.GENERICVFATITLE
            });
        },
        
        _initializePortVirtualizationTable: function() {
            this._targetAdapterTypeSelector.set("disabled", true);
            this._targetAdapterTypeSelector.set("value", "*ANY");
                   
            if (this._portVirtualizationTable == null) {
                this._portVirtualizationTable = new support.panels.PortVirtualizationTable({
                    id: "cp_newportvirtualizationtemplate_portvirtualizationtable",
                    adapterId: this._targetAdapterTypeSelector.get("value"),
                    mode: this.mode,
                    protocol: this.protocol,
                    numRows: this.numberOfVirtualFunctions
                }).placeAt(this._physicalFunctionGridNode);
                this._portVirtualizationTable.startup();
                                    
                dojo.publish("portVirtTableComplete", true);
                
                dojo.connect(this._portVirtualizationTable, "onSelectChange", this, function(templateId, idx) {
                    this._templateIdList[idx] = templateId;
                });

                dojo.connect(this._portVirtualizationTable, "onSliderChange", this, function(value, idx) {                        
                    if (this._targetPortOperationalModeSelector.get("value") == "vNIC_independent") {
                        this._slidervalueList[idx] = value;
                        var totalpct = 0;
                        for (var count=0;count < this.numberOfVirtualFunctions;++count) {
                            if (this._enablefunctionList[count] == true) {
                                totalpct += this._slidervalueList[count];
                            }
                        }
                        if (totalpct != 100) {             
                            this._portVirtualizationTable.displayOverallPctWarning("inline");
                            this._isPctValid = false;
                            this.okButton.set("disabled", true);
                            this.saveBtn.set("disabled", true);
                            this.saveasBtn.set("disabled", true);
                        } else {
                            this._portVirtualizationTable.displayOverallPctWarning("none");
                            this._isPctValid = true;
                            if (this._isVlanValid) {
                                this._nameChanged();
                            }
                        }
                        this._portVirtualizationTable.setOverallPct(totalpct);
                    }
                });
                dojo.connect(this._portVirtualizationTable, "onEnableChange", this, function(value, idx) {
                    this._enablefunctionList[idx] = value;
                    var totalpct = 0;
                    for (var count=0;count < this.numberOfVirtualFunctions;++count) {
                        if (this._enablefunctionList[count] == true) {
                            totalpct += this._slidervalueList[count];
                        }
                    }                    
                    if (totalpct != 100) {
                        this._portVirtualizationTable.displayOverallPctWarning("inline");
                    } else {
                        this._portVirtualizationTable.displayOverallPctWarning("none");
                    }
                    this._portVirtualizationTable.setOverallPct(totalpct);
                });                    
            } else {
                this._portVirtualizationTable.adapterChanged(this._targetAdapterTypeSelector.get("value"));
            }                
            this._setPortExtendedSelectorStore();            
        },
        
        _setPortExtendedSelectorStore: function() {
            var protocol = "ethernet";
            //if (this._targetPortProtocolsSelector.get("value") == "ethOnly") {
            //    protocol = "ethernet";
            //}

            this._defaultPortExtendedSettingsTemplateSelector.setStore(this._defaultPortExtendedSettingsTemplateStore, null, {query: {
                complexQuery: "vendor: '" + this._vendor + "' AND chipset: '" + this._chipset + "' AND protocol: '" + protocol + "'" 
            }});        
            
            if (this._defaultPortExtendedSettingsTemplateSelector.get("value") != "") {               
                this._defaultPortExtendedSettingsTemplateSelector.addOption({
                    value: "*NONE",
                    label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
                });
                this._defaultPortExtendedSettingsTemplateSelector.set("value", "*NONE");
                dojo.replaceClass(this._editIconPortExtended, "spEditInactiveIcon_v2", "spEditIcon_v2");
                this._defaultPortExtendedSettingsTemplateSelector.set("disabled", false);
            } else {
                this._defaultPortExtendedSettingsTemplateSelector.set("disabled", true);
                dojo.replaceClass(this._editIconPortExtended, "spEditInactiveIcon_v2", "spEditIcon_v2");                
            }
        },
                
        _editPortExtendedSettingsDialog1: function() {
            if (this._defaultPortExtendedSettingsTemplateSelector.get("value") == "*NONE" || this._defaultPortExtendedSettingsTemplateSelector.get("value") == "") {
                return;
            }
            require(["support/dialogs/NewLearnedSettingsTemplate"], dojo.hitch(this, function() {
               var portExtendedSettingsDialog = new support.dialogs.NewLearnedSettingsTemplate({
                   id: "cp_newportvirtualizationtemplate_newlearnedsettingsdialog",
                   templateIDNum: this._defaultPortExtendedSettingsTemplateSelector.get("value"),
                   type: "port",
                   templateMode: "edit"
               });
               portExtendedSettingsDialog.show();             
            }));
        },
        
        _nameChanged: function() {
            if (this.templateName.get("value").trim() != "") {
                var tgtOperationalMode = this._targetPortOperationalModeSelector.get("value");
                if ((tgtOperationalMode == "vNIC_IBMufp") || (this._isVlanValid && (this._isPctValid || (tgtOperationalMode == "vNIC_IBMvf")))) {
                    this.okButton.set("disabled", false);
                    this.saveBtn.set("disabled", false);
                    this.saveasBtn.set("disabled", false);
                } else {
                    this.okButton.set("disabled", true);
                    this.saveBtn.set("disabled", true);
                    this.saveasBtn.set("disabled", true);
                }                
            }
            else {
                this.okButton.set("disabled", true);
                this.saveBtn.set("disabled", true);
                this.saveasBtn.set("disabled", true);
            }
        },

        _setupPortVirtualizationTemplateObj: function(currentMode, currentOptionCode) {
            var portVirtObj = new Object();
            
            if (this.templateIDNum == null) {
                portVirtObj.id = "";
            } else {
                portVirtObj.id = this.templateIDNum;
            }
            portVirtObj.name = this.templateName.get("value").trim();
            portVirtObj.type = "PortVirtualization";
            portVirtObj.adapterType = this._targetAdapterTypeSelector.get("value");
            var description = this.portVirtDesc.get("value").trim();
            portVirtObj.description = this.encodeSpecialCharacters(description);
            portVirtObj.userDefined = true;
            portVirtObj.vendor = this._vendor; //"Emulex";
            portVirtObj.chipset = this._chipset; //"BE3";
            portVirtObj.portCfgMode = currentMode; // vNIC_independent, vNIC_IBMvf, or vNIC_IBMufp
            portVirtObj.portCfgOptionCode = currentOptionCode;
            portVirtObj.extendedPortTemplateId = 
                (this._defaultPortExtendedSettingsTemplateSelector.get("value") == "") ? "*NONE" : this._defaultPortExtendedSettingsTemplateSelector.get("value");
            
            return portVirtObj;
        },
        
        _setupVportObj: function(currentMode, currentOptionCode, vlanId, saveMode, idx) {            
            var vportObj = new Object();

			vportObj.id = (saveMode == "save") ? this._pattern.vports[idx].id : "";
            vportObj.portNumber = idx+1;
            vportObj.enabled = (currentMode == "vNIC_independent") ? this._enablefunctionList[idx] : true;
            if (vportObj.enabled && (currentMode == "vNIC_independent")) {
                vportObj.minSpeed = this._slidervalueList[idx];
                vportObj.maxSpeed = 100; // Hardcoded max value, required.
            }
            if (idx == 1) {
                vportObj.vlanId = (!vportObj.enabled || (currentMode == "vNIC_IBMufp") || (currentOptionCode != "ethOnly")) ? "*NONE" : vlanId.toString();
            } else {
                vportObj.vlanId = (!vportObj.enabled || (currentMode == "vNIC_IBMufp")) ? "*NONE" : vlanId.toString();
            }
            
            return vportObj;
        },

        saveClicked: function() {
            var vlanArray = this._portVirtualizationTable.getVlanValues();
            var currentMode = this._targetPortOperationalModeSelector.get("value");
            var currentOptionCode = this._targetPortProtocolsSelector.get("value");
            
            var portVirtualizationTemplateObj = this._setupPortVirtualizationTemplateObj(currentMode, currentOptionCode);
            
            var vportsArray = new Array();
            for(var count=0;count < this.numberOfVirtualFunctions;++count) {
                vportsArray.push(this._setupVportObj(currentMode, currentOptionCode, vlanArray[count], "save", count));
            }
            
            portVirtualizationTemplateObj.vports = vportsArray;
                      
            //console.log(this.declaredClass + ": ", dojo.toJson(portVirtualizationTemplateObj));
            
            var argJSONString = dojo.toJson(portVirtualizationTemplateObj);
            
            var saveTemplate = true;
            
            if(this.templateInUse){
                //  Need to remove extra properties that aren't included in the dialog
                delete this._pattern.inUse;
                delete this._pattern.uri;
                delete this._pattern.referencedBy;
                delete this._pattern.useCount;
                delete this._pattern.extSettingsTemplateId;
                
                // Remove description since it shouldn't force a redeploy
                delete this._pattern.description;
                delete portVirtualizationTemplateObj.description;
                
                for(var counter=0;counter<portVirtualizationTemplateObj.vports.length;counter++) {
                	if (portVirtualizationTemplateObj.vports[counter].vlanId == "*NONE") {
                		portVirtualizationTemplateObj.vports[counter].vlanId = "0";
                	}
                }
                
                var preData = this.sortAndFlatten(this._pattern);
                var postData = this.sortAndFlatten(portVirtualizationTemplateObj);
                
                if(preData != postData){
                    this.rdstDialog.set("templateID", this.templateIDNum);
                    this.rdstDialog.set("templateType", "Category");
                    this.rdstDialog.set("templateData", argJSONString);

                    this.rdstDialog.show();
                    
                    saveTemplate = false;
                }
            }
            if (saveTemplate) {
                this.getProgressDialog("save").show(); // Show progress dialog for save request.
                var xhrArgs = {
                        url:  this.REST_PATH + "template/" + this.templateIDNum, //create?type=ppfunction",
                        handleAs: "json",
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        postData: argJSONString,
                        load: dojo.hitch(this, function(response, ioArgs) {
                            // Happy path! We received a 2xx status code from the API.
                            //alert("post data: " + data);
                            dojo.publish("ProgressComplete", response);
                            // Publish the ID of the pattern that was just updated for the benefit of all listeners.
                            dojo.publish("EditPortVirtualizationTemplateComplete", {
                                id: this.templateIDNum
                            });
                            this.hide();
                        }),
                        error: dojo.hitch(this, function(error, ioArgs) {
                            // Failure path! We received a non-2xx status code (ie: 3xx/4xx/5xx status code) from the API.

                            dojo.publish("ProgressFailed", error);
                            console.log(this.declaredClass + ": ", error, ioArgs);
                            
                            var response = dojo.fromJson(ioArgs.xhr.responseText);
                            if ((ioArgs.xhr.status >= 400) && (response.message == "service_error_name_in_use")) {
                                // Error Template name in use
                                this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                            }
                            else if(ioArgs.xhr.status == 404) {
                                // Edit Template ID doesn't exist
                                this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                                this.hide();
                            }
                            else if(ioArgs.xhr.status >= 500) {
                                // Internal Programming Error
                                this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                                this.hide();
                            }
                            else {
                                this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                                this.hide();
                            }
                            
                        })
                };

                
                dojo.xhrPut(xhrArgs);
            }
            
        },

        saveasClicked: function() {            
            var vlanArray = this._portVirtualizationTable.getVlanValues();
            var currentMode = this._targetPortOperationalModeSelector.get("value");
            var currentOptionCode = this._targetPortProtocolsSelector.get("value");
            
            var portVirtualizationTemplateObj = this._setupPortVirtualizationTemplateObj(currentMode, currentOptionCode);
            
            var vportsArray = new Array();
            for(var count=0;count < this.numberOfVirtualFunctions;++count) {
                vportsArray.push(this._setupVportObj(currentMode, currentOptionCode, vlanArray[count], "saveas", count));
            }
            
            portVirtualizationTemplateObj.vports = vportsArray;
                      
            //console.log(this.declaredClass + ": ", dojo.toJson(portVirtualizationTemplateObj));            
            
            this.saveasDialog.set("templateObject", portVirtualizationTemplateObj);
            this.saveasDialog.show();
        },
        
        _saveandhideDialog: function() {
            var vlanArray = this._portVirtualizationTable.getVlanValues();
            var currentMode = this._targetPortOperationalModeSelector.get("value");
            var currentOptionCode = this._targetPortProtocolsSelector.get("value");
            
            var portVirtualizationTemplateObj = this._setupPortVirtualizationTemplateObj(currentMode, currentOptionCode);            
            
            var vportsArray = new Array();
            for(var count=0;count < this.numberOfVirtualFunctions;++count) {
                vportsArray.push(this._setupVportObj(currentMode, currentOptionCode, vlanArray[count], "create", count));
            }

            portVirtualizationTemplateObj.vports = vportsArray;
                                  
            //console.log(this.declaredClass + ": ", dojo.toJson(portVirtualizationTemplateObj));
            
            this.getProgressDialog("save").show(); // Show progress dialog for save request.
            var xhrPost = dojo.xhrPost({
                url:  this.REST_PATH + "template/create?type=portvirtual",
                handleAs: "json", // <<<< Required
                headers: {
                    "Content-Type": "application/json; charset=utf-8" // <<<< Required
                },
                postData: dojo.toJson(portVirtualizationTemplateObj),     //Template Data to Post
                load: dojo.hitch(this, function(response, ioArgs) {
                    // Happy path! We received a 2xx status code from the API.
                                            
                    dojo.publish("ProgressComplete", response);
                    dojo.publish("CreatePortVirtualizationTemplateComplete", response);
                    this.hide();   
                }),
                error: dojo.hitch(this, function(error, ioArgs) {
                    // Failure path! We received a non-2xx status code (ie: 3xx/4xx/5xx status code) from the API.

                    dojo.publish("ProgressFailed", error);
                    console.log(this.declaredClass + ": ", error, ioArgs);
                    
                    var response = dojo.fromJson(ioArgs.xhr.responseText);
                    if ((ioArgs.xhr.status >= 400) && (response.message == "service_error_name_in_use")) {
                        // Error Template name in use
                        this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                    }
                    else if(ioArgs.xhr.status == 404) {
                        // Edit Template ID doesn't exist
                        this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                        this.hide();
                    }
                    else if(ioArgs.xhr.status >= 500) {
                        // Internal Programming Error
                        this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                        this.hide();
                    }
                    else {
                        this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                        this.hide();
                    }
                    
                }),
                handle: dojo.hitch(this, function(data, ioArgs) {
                    // - The 1st param ("data") is either the response data (on success) or Error object (on failure).
                    // - The 2nd param ("ioArgs") contains detail about the xhr request, including a reference to the
                    // native xhr object. Check ioArgs.xhr.status to determine success or failure, and handle "data"
                    // accordingly.
                })
            });

        },
        
        _setup: function(){

            //Set Information Messages and Load Previous Settings          
            if (this.templateIDNum != null && this.templateMode == "edit") {

                //if dialog in edit mode    
                if (this.templateInUse) {
                    if (this.templateLaunchPoint == "table") {
                        this.warningMessage.style.display = "block";
                        this.warningMessageText.innerHTML = this.resources.INUSE;
                        this.saveBtnContainer.style.display = "inline";
                        this.saveasBtnContainer.style.display = "inline";
                        
                    } else if (this.templateLaunchPoint == "wizard") {
                        this.informationMessage.style.display = "block";
                        this.informationMessageText.innerHTML = this.resources.INUSEWIZARD;
                        this.saveasBtnContainer.style.display = "inline";
                    } else {
                        this.informationMessage.style.display = "block";
                        this.informationMessageText.innerHTML = this.resources.INUSEWIZARD;
                        this.saveasBtnContainer.style.display = "inline";
                    }
                } else {
                    this.saveBtnContainer.style.display = "inline";
                    this.saveasBtnContainer.style.display = "inline";
                }
                this.templateName.set("disabled",true);
                
             } else if (this.templateIDNum != null && this.templateMode == "copy") {

                 //if dialog in copy mode
                 this.okBtnContainer.style.display = "inline";

             } else {

                 //if dialog in create new mode
                 this.okBtnContainer.style.display = "inline";

             }

            this._nameChanged(); // Call to set proper button state now that selector values have been initialized.

        }, 
        
        exitParent: function() {
            dojo.publish("EditPortVirtualizationTemplateComplete", {
                id: this.templateIDNum
            });
            if (this.rdstDialog.get("closeParent")) {
                this.hide();
            }
        },          
        
        _cancelDialog: function() {
            this.hide();
        }           
    });
});
