define([
        "dojo",
        "dijit",
        "support/dialogs/Dialog",
        "dojo/i18n!./nls/ProgressDialog",
        "dojo/text!./templates/ProgressDialog.html",
        "dijit/layout/ContentPane",
        "dijit/layout/BorderContainer"
    ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.ProgressDialog", [ support.dialogs.Dialog ], {

        templateString: template, // Path to the template

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        title: null,
        mode: "load", // "load"|"save"|"learn"|"delete"
        nobuttons: false,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "progressDialog",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/ProgressDialog.css",

        // Override this method to perform custom behavior during dijit construction.
        // Common operations for constructor:
        // 1) Initialize non-primitive types (i.e. objects and arrays)
        // 2) Add additional properties needed by succeeding lifecycle methods
        constructor: function() {
        },

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);

            // NOTE: Defer initialization of dijit to onShowDialog() processing.

            this.subscribe("ProgressComplete", function(data) {
                this.hide();
            });
            this.subscribe("ProgressFailed", function(data) {
                this.hide();
            });
        },

        onShowDialog: function() {
            this.title && this.dialogNode.set("title", this.title);
            switch (this.mode) {
                case "load":
                    this._loadingSection.innerHTML = this.locResources.LOADING;
                    dojo.style(this._loadingSection, "display", "");
                    dojo.style(this._savingSection, "display", "none");
                    break;
                case "save":
                    this._loadingSection.innerHTML = this.locResources.SAVING;
                    dojo.style(this._loadingSection, "display", "none");
                    dojo.style(this._savingSection, "display", "");
                    break;
                case "learn":
                    this.set("title", this.locResources.LEARN_TITLE);
                    this._loadingSection.innerHTML = this.locResources.LEARNING;
                    dojo.style(this._loadingSection, "display", "");
                    dojo.style(this._savingSection, "display", "none");
                    break;
                case "delete":
                    this._loadingSection.innerHTML = this.locResources.DELETING;
                    dojo.style(this._loadingSection, "display", "");
                    dojo.style(this._savingSection, "display", "none");
                    break;
                case "placeholder":
                    this._loadingSection.innerHTML = this.locResources.PLACEHOLDER;
                    dojo.style(this._loadingSection, "display", "");
                    dojo.style(this._savingSection, "display", "none");
                    break;
                case "activate":
                    this._loadingSection.innerHTML = this.locResources.ACTIVATING;
                    dojo.style(this._loadingSection, "display", "");
                    dojo.style(this._savingSection, "display", "none");
                    break;
                case "pool":
                    this._loadingSection.innerHTML = this.locResources.POOL;
                    dojo.style(this._loadingSection, "display", "");
                    dojo.style(this._savingSection, "display", "none");
                    break;
                case "deploy":
                    this._loadingSection.innerHTML = this.locResources.DEPLOY;
                    dojo.style(this._loadingSection, "display", "");
                    dojo.style(this._savingSection, "display", "none");
                    break;
            }
            dojo.style(this._buttonSection, "display", (this.nobuttons) ? "none" : "");
            return true; // Thumbs up! Okay to show.
        },

        cancelClicked: function() {
            this.hide();
            this.dialogNode.destroyRecursive();
        },

        okClicked: function() {
            this.hide();
        }
    });
});