define([
        "dojo",
        "dijit",
        "support/dialogs/Dialog",
        "dojo/i18n!./nls/ReDeployServerTemplate",
        "dojo/text!./templates/ReDeployServerTemplate.html",
        "dijit/form/Button",
        "dojo/data/ItemFileWriteStore",
        "dijit/form/DropDownButton",
        "dijit/DropDownMenu",
        "dijit/layout/BorderContainer",
        "dijit/layout/ContentPane",
        "dijit/MenuItem",
        "dijit/Toolbar",
        "support/base/TableMixin",
        "support/dialogs/ProgressDialog"
    ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.ReDeployServerTemplate", [ support.dialogs.Dialog, support.base.TableMixin ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "ReDeployServerTemplate",

        // templateId: String
        // Info From Parent Dialog
        templateID: "", //ID of template we are going to save and redeploy
        templateType: "", //Type of template we are going to save and redeploy, category or server/chassis
        templateData: "", //Data of template we are going to save and redeploy
        
        templateName: "", //Name of template we are going to save and redeploy
        store: null,
        closeParent: false,
        chassisVersion: false,
        saveTemplateType : "", // Specific type of template provided (not generic "Category" type).
        viewMode: 2, //Current table view, by servers by default
        BY_CHASSIS: 1,
        BY_SERVERS: 2,

        _selectModel: null,
        _selectLayout: null,
        _selectedStore: null,
        _selectTable: null,
        _templateID: "template1",
        _argJSONString: null,
        _affectedTemplates: [],
        _affectedServers: [],
        
        // ID prefix used for table
        gridID: "RedeployServerTable",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/ReDeployServerTemplate.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in
        // the template HTML file
        postMixInProperties: function() {

            this.inherited(arguments);
            if (this.templateId) {
                // For some reason values passed from HTML contain "/r/n" chars. Must trim to
                // work properly.
                this.templateId = dojo.trim(this.templateId);
            }

        },

        onShowDialog: function() {
            if (this.templateData) {
                // Process right away, so these attributes are set!
                var data = dojo.fromJson(this.templateData);
                console.log(data);
                if (this.saveTemplateType == "") {
                    if (data && data.items && data.items[0].type) {
                        this.saveTemplateType = data.items[0].type;
                    }
                    else if (data && data.type) {
                        // Save APIs for some patterns don't use items array. (ie: Adapter, PortVirtualization, PortPhysicalFunction)
                        this.saveTemplateType = data.type;
                    }
                }
                if (this.templateName == "") {
                    if(data && data.items && data.items[0].name) {
                        this.templateName = data.items[0].name;
                    }
                    else if (data && data.name) {
                        // Save APIs for some patterns don't use items array. (ie: Adapter, PortVirtualization, PortPhysicalFunction)
                        this.templateName = data.name;
                    }
                }
            }
            console.log(this.saveTemplateType, this.templateName);
            this.closeParent = false;
            this.chassisVersion = this._isChassisType();
            this._setHeaderText();
            //Are we working with a chassis template? If so the table is setup differently
            this._setupSelectedTable();
            var title = this.resources2.SAVE_REDEPLOY;
            this.set("title", title);
            if (this._affectedServers.length < 1) {
                this._saveAndRedeployTemplate(true);
                return false;
            }
            return this.inherited(arguments);

        },
        
        _pushJobSuccessInfo: function(/*Object*/item, /*Array*/msgContent) {
            // Push job success information into the msgContent array.
            // Expect to see jobname w/ endPointIds/locationIds data.
            msgContent.push(this.resources.SUBMIT_JOB_SUCCESS.replace("%1", item.jobName));
            if (this.chassisVersion) {
                var names = new Array();
                for (var loopVar in this._selectTable.store._arrayOfAllItems) {
                    var selected = this._selectTable.store._arrayOfAllItems[loopVar];
                    // get the name of the chassis that was deployed to
                    if(item.endPointIds != null && selected.id != null &&
                        selected.id[0] != null &&
                        dojo.indexOf(item.endPointIds, selected.id[0]) >= 0) {
                        names.push(selected.name[0]);
                    }
                }
                msgContent.push(this.resources.SUBMIT_JOB_SUCCESS_CHASSIS.replace("%1", names.join(", ")));
            }
            else {
                var names = [];
                for (var loopVar in this._selectTable.store._arrayOfAllItems) {
                    var selected = this._selectTable.store._arrayOfAllItems[loopVar];
                    // get the name of the server that was deployed to
                    if(item.locationIds != null && selected.location != null &&
                        selected.location[0] != null &&
                        dojo.indexOf(item.locationIds, selected.location[0]) >= 0) {
                        names.push(selected.name[0]);
                    }
                }
                msgContent.push(this.resources.SUBMIT_JOB_SUCCESS_SERVERS.replace("%1", names.join(", ")));
            }
            msgContent.push("<p>" + this.resources.SUBMIT_JOB_DETAILS + "</p>");
        },
        
        _pushJobErrorInfo: function(/*Object*/item, /*Array*/msgContent) {
            // Push job failure information into the msgContent array.
            // No jobname, but endPointIds/locationIds are valid.
            msgContent.push(this.resources.SUBMIT_JOB_ERROR);
            if (this.chassisVersion) {
                var names = new Array();
                for (var loopVar in this._selectTable.store._arrayOfAllItems) {
                    var selected = this._selectTable.store._arrayOfAllItems[loopVar];
                    // get the name of the chassis that was deployed to
                    if(item.endPointIds != null && selected.id != null &&
                        selected.id[0] != null &&
                        dojo.indexOf(item.endPointIds, selected.id[0]) >= 0) {
                        names.push(selected.name[0]);
                    }
                }
                msgContent.push(this.resources.SUBMIT_JOB_ERROR_CHASSIS.replace("%1", names.join(", ")));
            }
            else {
                var names = [];
                for (var loopVar in this._selectTable.store._arrayOfAllItems) {
                    var selected = this._selectTable.store._arrayOfAllItems[loopVar];
                    // get the name of the server that was deployed to
                    if(item.locationIds != null && selected.location != null &&
                        selected.location[0] != null &&
                        dojo.indexOf(item.locationIds, selected.location[0]) >= 0) {
                        names.push(selected.name[0]);
                    }
                }
                msgContent.push(this.resources.SUBMIT_JOB_ERROR_SERVERS.replace("%1", names.join(", ")));
            }            
        },

        onFinish: function() {

            var data = {
                endPointIds: [],
                locationIds: []
            };
            for (var loopVar in this._selectTable.store._arrayOfAllItems) {
                var item = this._selectTable.store._arrayOfAllItems[loopVar];
                if (this.chassisVersion) {
                    item && data.endPointIds.push(item.id[0]);
                }
                else {
                    if(this.viewMode == this.BY_SERVERS){
                        item && item.location && data.locationIds.push(item.location[0]);
                    }
                    else if(this.viewMode == this.BY_CHASSIS){
                        item && (item.type != "chassis") && item.location && data.locationIds.push(item.location[0]);
                    }
                }
            }
            !this.chassisVersion && (data.restart = this._schedule);
            var jsonString = dojo.toJson(data);
            dojo.xhrPost({
                url: this.REST_PATH + "profile/redeploy",
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                postData: jsonString,
                handle: dojo.hitch(this, function(response, ioArgs) {
                    console.log(this.declaredClass + ": Redeploy pattern complete: ", response, ioArgs);
                    var xhrStatus = ioArgs.xhr.status;

                    // ** START MSIE bug fix! - Documented in defect XB243575 **
                    //  XMLHTTPRequest implementation in MSXML HTTP does not handle HTTP responses with
                    //  status code 204 (No Content) properly; the "status" property has the bogus value 1223. The following
                    //  is a workaournd to normalize xhr response settings for this MSIE bug.
                    (xhrStatus == 1223) && (xhrStatus = 204);
                    // ** END MSIE bug fix! - Documented in defect XB243575 **

                    if (xhrStatus >= 300) {
                        this.getModalDialog("Error", this.resources.DEPLOY_ERROR_MSG).show();
                        console.error("Failed to redeploy server pattern.");
                    }
                    else {
                        // Examine response items to determine potential for multiple success/failure
                        var info = new Array();
                        var oneSuccess = false;
                        var oneFailure = false;
                        for (var loopVar = 0; loopVar < response.items.length; ++loopVar) {
                            var item = response.items[loopVar];
                            var title = this.resources.SUBMIT_JOB_TITLE.replace("%1", loopVar + 1);
                            var msgContent = new Array();
                            if (!item.message || (item.message.length == 0)) {
                                this._pushJobSuccessInfo(item, msgContent);
                                oneSuccess = true;
                            }
                            else {
                                this._pushJobErrorInfo(item, msgContent);
                                oneFailure = true;
                            }
                            info.push({
                                title: title,
                                content: msgContent.join(" ")
                            });
                        }
                        var msgType = null;
                        var msgText = null;
                        if (oneSuccess) {
                            if (oneFailure) {
                                msgType = "Warning";
                                msgText = this.resources.DEPLOY_WARNING_MSG;
                            }
                            else {
                                msgType = "Success";
                                msgText = this.resources.DEPLOY_SUCCESS_MSG;
                            }
                        }
                        else {
                            msgType = "Error";
                            msgText = this.resources.DEPLOY_ERROR_MSG;
                        }
                        this.getModalDialog(msgType, msgText, ((info.length == 1) ? info[0].content : info)).show();
                        dojo.publish("ReDeployComplete", response);
                        this.closeParent = true;
                        this._onClose();
                    }
                    this._selectModel = null;
                })
            });
            return false; // Don't allow close! Allow handlers (above) to determine when to close.

        },

        _setupSelectedTable: function() {

            // summary:
            // Set up selected targets table widget...
            // tags:
            // protected
            if (this._selectTable) {
                // Destroy old table and start from scratch.
                this._selectTable.destroy();
            }
            
            var tableStore = this._retrieveStore();
            var tableStructure = this._getLayout(this.viewMode, this.chassisVersion);
            
            this._selectTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.gridID, true);
            this._selectTable.column(0).sort(false);
            
            setTimeout(dojo.hitch(this, function() {
                this._selectTable.placeAt(this._selectTableNode);
                this._selectTable.startup();
            }), 500);            
                                   
            var toolbar = this._selectTable && this._selectTable.toolBar && this._selectTable.toolBar.widget;
            
            this._addViewSelector(toolbar);
            if(!this.chassisVersion){
                toolbar.addChild(new dijit.ToolbarSeparator({}));
                this._loadToolBar(toolbar);
            }

        },

        _setHeaderText: function() {
            var templates = this._getAffectedTemplates();
            if (templates == "") {
                templates = this.locResources.NONE_AFFECTED;
            }
            // Grab type of template for display purposes
            var translatedType = this.translateTemplateType(this.saveTemplateType);
            var text = this.resources2.SAVING.replace("%1", " " + translatedType + " ").replace("%2",
                    " \"" + this.templateName + "\" ").replace("%3", templates)
                    + "<br></br>" + this.locResources.DEFAULT_MSG + "<br></br>" + this.locResources.CONF_MSG.replace("%1", " \"" + this.templateName + "\" ") + "<br></br>";
            this.rdst_headerText.innerHTML = text;

        },

        _getValidJsonData: function(jsonData) {

            // if any part of the file is not valid, return the default item
            if (jsonData == null || jsonData.length == 0) {
                return {
                    "items": []
                };
            }
            else {
                return {
                    "items": jsonData
                };
            }

        },

        _addViewSelector: function(toolbar) {

            // Add custom view selector button...
            var menu = new dijit.DropDownMenu({
                style: "display: none;"
            });
            var button = new dijit.form.DropDownButton({
                label: (this.chassisVersion) ? this.resources.BY_CHASSIS : this.locResources.BY_SERVERS,
                title: this.resources.TABLE_VIEW,
                dropDown: menu
            });
            var byChassisItem = new dijit.MenuItem({
                label: this.resources.BY_CHASSIS,
                onClick: dojo.hitch(this,function() {
                    if(this.viewMode == this.BY_CHASSIS){
                        return;
                    }
                    this._reRenderTable(this.BY_CHASSIS);
                    button.set("label", this.resources.BY_CHASSIS);
                })
            });
            menu.addChild(byChassisItem);
            if(!this.chassisVersion){
                var byServer = new dijit.MenuItem({
                    label: this.locResources.BY_SERVERS,
                    onClick: dojo.hitch(this,function() {
                        if(this.viewMode == this.BY_SERVERS){
                            return;
                        }
                        this._reRenderTable(this.BY_SERVERS);
                        button.set("label", this.locResources.BY_SERVERS);
                    })
                });
                menu.addChild(byServer);
            }
            toolbar.addChild(button);

        },

        _loadToolBar: function(toolbar) {
            //Activation options are server only
            if(!this.chassisVersion){
                var menu = new dijit.DropDownMenu({
                    style: "display: none;"
                });
                var button = new dijit.form.DropDownButton({
                    label: this.locResources.CHANGE_ACT,
                    title: this.locResources.CHANGE_ACT_TITLE,
                    dropDown: menu
                });
                //  Immediate Activation Menu Option
                var immediateOption = new dijit.MenuItem({
                    label: this.locResources.FULL_RESTART,
                    title: this.locResources.FULL_RESTART_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._immediateAction();
                        this._schedule = "immediate";
                    })
                });
                menu.addChild(immediateOption);
                //  Action Menu Option
                var deferOpt = new dijit.MenuItem({
                    label: this.locResources.PARTIAL_RESTART,
                    title: this.locResources.PARTIAL_RESTART_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._deferAction();
                        this._schedule = "defer";
                    })
                });
                menu.addChild(deferOpt);

                toolbar.addChild(button);

                this._schedule = "defer";
            }

        },

        _accessFormatter: function(item) {

            // summary:
            // Formats "access" field value for display in TreeTable.
            // tags:
            // protected

            var access = item.access;
            var accessText = "";
            if (access == "ok" || access == true) { // Access "OK"
                accessText = this.resources.ACCESS_OK;
            }
            else if (access == "-Default-") { // Access "Default Access"
                accessText = this.resources.ACCESS_DEFAULT;
            }
            else if (access == "offline" || access == false) { // Access "Offline"
                accessText = this.resources.ACCESS_OFFLINE;
            }
            else if (access == "EmptyBay" || access == "empty") { // Access Empty Bay Location
                accessText = this.resources.ACCESS_EMPTY;
            }
            else if (access == "placeholder") { // Access "Placeholder"
                accessText = this.resources.PLACEHOLDER;
            }
            else if (access == "none") { // Access "No access"
                accessText = this.resources.ACCESS_NONE;
            }
            else if (access == "partial") { // Access "Partial access"
                accessText = this.resources.ACCESS_PARTIAL;
            }
            else /*(access == "unknown")*/ { // Access "Unknown"
                accessText = this.resources.ACCESS_UNKNOWN;
            }
            
            return accessText;
        },

        _deferAction: function() {
            var i,j;
            var roots = this._selectTable.store._arrayOfTopLevelItems;
            if(this.viewMode == this.BY_CHASSIS){
                if (roots.length > 0) {
                    for (i = 0; i < roots.length; i++) {
                        if(roots[i].children){
                            for(j=0;j<roots[i].children.length;j++){
                                this._selectTable.store.setValue(roots[i].children[j], "schedule", this.locResources.PARTIAL_RESTART);
                            }
                        }
                        
                    }
                    roots = this._selectedStore_flat._arrayOfTopLevelItems;
                    for (i = 0; i < roots.length; i++) {
                        this._selectedStore_flat.setValue(roots[i], "schedule", this.locResources.PARTIAL_RESTART);
                    }
                }
            }
            else if(this.viewMode == this.BY_SERVERS) {
                if (roots.length > 0) {
                    for (i = 0; i < roots.length; i++) {
                        this._selectTable.store.setValue(roots[i], "schedule", this.locResources.PARTIAL_RESTART);
                    }
                    roots = this._selectedStore_root._arrayOfTopLevelItems;
                    for (i = 0; i < roots.length; i++) {
                        if(roots[i].children){
                            for(j=0;j<roots[i].children.length;j++){
                                this._selectedStore_root.setValue(roots[i].children[j], "schedule", this.locResources.PARTIAL_RESTART);
                            }
                        }
                    }
                }
            }
        },

        _immediateAction: function() {
            var i,j;
            var roots = this._selectTable.store._arrayOfTopLevelItems;
            if(this.viewMode == this.BY_CHASSIS){
                if (roots.length > 0) {
                    for (i = 0; i < roots.length; i++) {
                        if(roots[i].children){
                            for(j=0;j<roots[i].children.length;j++){
                                this._selectTable.store.setValue(roots[i].children[j], "schedule", this.locResources.FULL_RESTART);
                            }
                        }
                        
                    }
                    roots = this._selectedStore_flat._arrayOfTopLevelItems;
                    for (i = 0; i < roots.length; i++) {
                        this._selectedStore_flat.setValue(roots[i], "schedule", this.locResources.FULL_RESTART);
                    }
                }
            }
            else if(this.viewMode == this.BY_SERVERS) {
                if (roots.length > 0) {
                    for (i = 0; i < roots.length; i++) {
                        this._selectTable.store.setValue(roots[i], "schedule", this.locResources.FULL_RESTART);
                    }
                    roots = this._selectedStore_root._arrayOfTopLevelItems;
                    for (i = 0; i < roots.length; i++) {
                        if(roots[i].children){
                            for(j=0;j<roots[i].children.length;j++){
                                this._selectedStore_root.setValue(roots[i].children[j], "schedule", this.locResources.FULL_RESTART);
                            }
                        }
                    }
                }
            }
        },

        // Get number of selected rows in tree table
        _getSelectedRowCount: function() {

            return (this._selectTable != null) ? this._selectTable.select.row.getSelected().length : -1;

        },

        // Get row objects for selected rows
        _getSelectedRows: function() {
            
            var selectedRows = new Array();
            var selectedIds = this._selectTable.select.row.getSelected();
            var i;
            
            for (i = 0; i < this.selectedIds; i++) {
                var item = this._selectTable.row(selectedIds[i], true).item();
                selectedRows.push(item);
            }

            return selectedRows;

        },

        _getAffectedTemplates: function() {

            var str = "";
            if (this._affectedTemplates != null) {
                var i;
                for (i = 0; i < this._affectedTemplates.length; i++) {
                    if (str != "") {
                        str += ", ";
                    }
                    str += "\"" + this._affectedTemplates[i].name + "\"";
                }
            }
            return str;

        },

        // Returns an array of strings with each affected profile name listed once
        _generateProfileList: function() {

            // Look through selectModel store, get included templates
            var profileNames = [];
            dojo.forEach(this._selectTable.store._arrayOfAllItems, function(item) {
                var found = false;
                var i;
                if (item.profile != null) {
                    for (i = 0; i < profileNames.length; i++) {
                        if (profileNames[i] == item.profile.toString()) {
                            found = true;
                            break;
                        }
                    }
                    if (!found) {
                        profileNames.push(item.profile);
                    }
                }
            });
            return profileNames;

        },
        
        _retrieveStore: function() {
            var affectedServers = "";
            var affectedTemplates = "";

            var path = this.REST_PATH + "template/" + this.templateType + "/" + this.templateID + "/affects";
            var xhrGet = dojo.xhrGet({
                url: path,
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: function(response) {
                    affectedTemplates = response.affectedTemplates;
                    affectedServers = response.affectedEndPoints;
                },
                handle: function() {
                    
                }
            });
            
            this._affectedServers = affectedServers;
            this._affectedTemplates = affectedTemplates;
            var i;
            for(i=0;i<this._affectedServers.length;i++){
                if(this._affectedServers[i].schedule && this._affectedServers[i].schedule == "immediate"){
                    this._affectedServers[i].schedule = this.locResources.FULL_RESTART;
                }
                else {
                    this._affectedServers[i].schedule = this.locResources.PARTIAL_RESTART;
                }
                
                // concatenate the sub-bay ID onto the bay object so that
                //   the sub-bay is also shown
                if(this._affectedServers[i].subBayId && this._affectedServers[i].subBayId > 0) {
                    this._affectedServers[i].bay = this._affectedServers[i].bay + ":" + this._affectedServers[i].subBayId;
                }
            }
            
            var templateData_flat = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": this._affectedServers
            }, true); 
            
            this._selectedStore_flat = new dojo.data.ItemFileWriteStore({
                data: dojo.fromJson(templateData_flat)
            });

            if(!this.chassisVersion){
                this._selectedStore_root = this._createRootedStore(this._affectedServers);
            }
            
            return this._selectedStore_flat;
        },
        
        translateTemplateType: function(templateType){
            if(!templateType){
                return "";
            }
            switch(templateType){
                case "cmmInfo": return this.resources.CHASSIS_INFO;
                case "cmmManagement": return this.resources.CHASSIS_MGMT;
                case "cmmPower": return this.resources.CHASSIS_POWER;
                case "cmmIOModule": return this.resources.CHASSIS_IO;
                case "cmmNode": return this.resources.CHASSIS_NODE;
                case "cmmUsers": return this.resources.CHASSIS_USER;
                case "CMM": return this.resources.CHASSIS;
                
                case "SystemInfo": return this.resources.SERVER_INFO;
                case "Management": return this.resources.SERVER_MGMT;
                case "Power": return this.resources.SERVER_POWER;
                case "DevicesAndIOPorts": return this.resources.SERVER_DEVICE;
                case "PerformanceRecovery": return this.resources.SERVER_PERFORM;
                case "Adapter": return this.resources.SERVER_ADAPTER;
                case "PortVirtualization": return this.resources.SERVER_PORT;
                case "PortPhysicalFunction": return this.resources.SERVER_PORTPF;
                case "PXE": return this.resources.SERVER_PXE;
                case "FibreChannel": return this.resources.SERVER_FIBRE;
                case "SAS": return this.resources.SERVER_SAS;
                case "ExtendedIMM": return this.resources.SERVER_EXIMM;
                case "ExtendedUEFI": return this.resources.SERVER_EXUEFI;
                case "ExtendedPort": return this.resources.SERVER_EXPORT;
                case "Server": return this.resources.SERVER;
                
                default: return "";
            }
        },
        
        _createRootedStore: function(systems){
            var i;
            var id = 0;
            var roots = [];
            console.log(this.declaredClass + ": systems", systems);
            for(i=0;i<systems.length;i++){
                var j;
                var foundChassis = false;
                for(j=0;j<roots.length;j++){
                    if(roots[j].name == systems[i].chassisName){
                        foundChassis = true;
                        roots[j].children.push(systems[i]);
                        break;
                    }
                }
                if(!foundChassis){
                    roots.push({type: "chassis", id: id.toString(), name:systems[i].chassisName,
                        children:[systems[i]]});
                    id++;
                }
            }
            console.log(this.declaredClass + ": roots", roots);
            var data = dojo.toJson({
                identifier: "id",
                label: "name",
                items: roots
            }, true); 
            console.log(this.declaredClass + ": data", data);
            var store = new dojo.data.ItemFileWriteStore({
                data: dojo.fromJson(data)
            });
            if (!("hasChildren" in store)) {
                store.hasChildren = function(id, item) { 
                    return item && this.getValues(item, "children").length;
                };
            };
            
            if (!("getChildren" in store)) {
                store.getChildren = function(item) { 
                    return this.getValues(item, "children");
                };
            };
            console.log(this.declaredClass + ": ", store);
            return store;
        },
        
        _isChassisType: function() {
            switch(this.saveTemplateType){
                case "cmmInfo": return true;
                case "cmmManagement": return true;
                case "cmmPower": return true;
                case "cmmIOModule": return true;
                case "cmmNode": return true;
                case "cmmUsers": return true;
                case "CMM": return true;
                default: return false;
            }
        },
        
        _getLayout: function(view,chassisVersion){
            if(chassisVersion == true){
                return [ {
                    field: "name",
                    name: this.resources.NAME,
                    width: "210px"
                }, {
                    field: "access",
                    name: this.resources.ACCESS,
                    width: "75px",
                    decorator: dojo.hitch(this, "_accessDecorator"),
                    formatter: dojo.hitch(this, "_accessFormatter")
                }, {
                    field: "schedule",
                    name: this.resources.PROFILE_ACTIVATION,
                    width: "230px"
                }, {
                    field: "profileName",
                    name: this.resources.SERVER_PROFILE,
                    width: "auto"
                } ];
            }
            else{
                if(view == this.BY_CHASSIS){
                    return [ {
                        field: "name",
                        name: this.resources.NAME,
                        width: "180px"
                    }, {
                        field: "bay",
                        name: this.resources.BAY,
                        width: "35px"
                    }, {
                        field: "access",
                        name: this.resources.ACCESS,
                        width: "75px",
                        decorator: dojo.hitch(this, "_accessDecorator"),
                        formatter: dojo.hitch(this, "_accessFormatter")
                    }, {
                        field: "chassisName",
                        name: this.resources.CHASSIS,
                        width: "125px"
                    }, {
                        field: "schedule",
                        name: this.resources.PROFILE_ACTIVATION,
                        width: "200px"
                    }, {
                        field: "profileName",
                        name: this.resources.SERVER_PROFILE,
                        width: "auto"
                    } ];
                }
                else if(view == this.BY_SERVERS){
                    return [ {
                        field: "name",
                        name: this.resources.NAME,
                        width: "200px"
                    }, {
                        field: "bay",
                        name: this.resources.BAY,
                        width: "35px"
                    }, {
                        field: "access",
                        name: this.resources.ACCESS,
                        width: "75px",
                        decorator: dojo.hitch(this, "_accessDecorator"),
                        formatter: dojo.hitch(this, "_accessFormatter")
                    }, {
                        field: "schedule",
                        name: this.resources.PROFILE_ACTIVATION,
                        width: "225px"
                    }, {
                        field: "profileName",
                        name: this.resources.SERVER_PROFILE,
                        width: "auto"
                    } ];
                }
            }
        },
        
        _reRenderTable: function(newView){
            if(this.chassisVersion){
                return;
            }
            var layout = this._getLayout(newView, false);
            var spanVal = 0;
            var store = null;
            if(newView == this.BY_CHASSIS){
                spanVal = 5;
                store = this._selectedStore_root;
            }
            else if(newView == this.BY_SERVERS){
                store = this._selectedStore_flat;
            }
            
            var newColSpan = {
                    0: [ {
                        start: 0,
                        end: spanVal
                    } ]
            };
            if(store){
                this._selectTable.set("colSpans", newColSpan);
                this._selectTable.setColumns(layout);
                this._selectTable.setStore(store);
                if(newView == this.BY_CHASSIS){
                    this._selectTable.expandAll();
                }
                this.viewMode = newView;
            }
            
        },

        _saveAndRedeployTemplate: function(saveOnly){

            this.getProgressDialog("save").show(); // Show progress dialog for save request.
            var api = null;
            var doPUT = false;
            switch(this.saveTemplateType) {
                case "Adapter":
                case "PortVirtualization":
                case "PortPhysicalFunction":
                    api = "template/" + this.templateID;
                    doPUT = true;
                    break;
                case "Policy":
                    api = "policy/failover/create";
                    break;
                default:
                    api = "template/create";
                    break;
            }
            var xhrArgs = {
                url: this.REST_PATH + api,
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                postData: this.templateData,
                handle: dojo.hitch(this, function(response, ioArgs) {
                    console.log(this.declaredClass + ": ", response, ioArgs);
                    var xhrStatus = ioArgs.xhr.status;

                    // ** START MSIE bug fix! - Documented in defect XB243575 **
                    //  XMLHTTPRequest implementation in MSXML HTTP does not handle HTTP responses with
                    //  status code 204 (No Content) properly; the "status" property has the bogus value 1223. The following
                    //  is a workaournd to normalize xhr response settings for this MSIE bug.
                    (xhrStatus == 1223) && (xhrStatus = 204);
                    // ** END MSIE bug fix! - Documented in defect XB243575 **

                    if (xhrStatus < 300) {
                        (doPUT) ? dojo.publish("ProgressComplete", this.templateID) : dojo.publish("ProgressComplete", response.id);
                        if(saveOnly){
                            this.closeParent = true;
                            this._selectModel = null;
                            this._onClose();
                        }
                        else{
                            // Redeploy template too!
                            this._onFinish();
                        }
                        
                        // Refresh table data after ReDeploy
                        // Currently used in ServerConfiguration.js for detail pages
                        dojo.publish("RefreshAfterReDeploy","");

                    }
                    else {
                        dojo.publish("ProgressFailed", response);
                        var response = dojo.fromJson(ioArgs.xhr.responseText);
                        if ((xhrStatus >= 400) && (response.message == "service_error_name_in_use")) {
                            // Error Template name in use
                            this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                        }
                        else if(xhrStatus == 404) {
                            // Edit Template ID doesn't exist
                            this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                            this.hide();
                        }
                        else if(xhrStatus >= 500) {
                            // Internal Programming Error
                            this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                            this.hide();
                        }
                        else {
                            this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                            this.hide();
                        }
                    }
                })
            };
            console.log(xhrArgs.postData);
            (doPUT) ? dojo.xhrPut(xhrArgs) : dojo.xhrPost(xhrArgs);

        },

        _saveAndRedeployClicked: function(){
            this._saveAndRedeployTemplate(false);
        }

    });

});