define([
        "dojo",
        "dijit",
        "support/dialogs/Dialog",
        "dojo/i18n!./nls/Rename",
        "dojo/text!./templates/Rename.html",
        "dijit/layout/ContentPane",
        "dijit/form/ValidationTextBox",
        "support/widgets/ProgressButton",
        "support/dialogs/ProgressDialog"
    ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.Rename", [ support.dialogs.Dialog ], {

        templateString: template, // Path to the template

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        currentname: null, // template current name
        templateIDNum: null, // template ID
        templateStore: null, // holds template information for rename
        ippoolMode: false, // set to true if object to rename is IP Pool
        profileMode: false, // set to "server" if object to rename is Server Profile, or "chassis" if object to rename is a Chassis Profile
        putTemplateMode: false, //set to true if object rename Port Virtual or Adapter
        launchPoint: "",  //set Rename Dialog Launch Point
        
        saveTemplateType: "",
        // Template Naming Constants
        POL_NAME: "failover",
        
        // Profile Mode Constants
        PROFILE_MODE_SERVER: "server",
        PROFILE_MODE_CHASSIS: "chassis",

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/Rename.css",

        // Override this method to perform custom behavior during dijit construction.
        // Common operations for constructor:
        // 1) Initialize non-primitive types (i.e. objects and arrays)
        // 2) Add additional properties needed by succeeding lifecycle methods
        constructor: function() {
        },

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);

            dojo.connect(this.dialogNode, "show", this, function() {

                this.setCurrentName();

                var templatename = this.name; // Template Name
                var okButton = this.okBtn; // Ok Button Variable
                var currentname = this.currentname;

                templatename.intermediateChanges = true;

                dojo.connect(templatename, "onChange", function() {

                    if (templatename.get("value").trim() != "" && templatename.get("value").trim() != currentname) {
                        okButton.set("disabled", false);
                    }
                    else {
                        okButton.set("disabled", true);
                    }
                });

            });

            dojo.connect(this.dialogNode, "hide", this, function() {
                this.destroy();
            });

        },
        cancelClicked: function() {
            this.hide();
        },
        okClicked: function() {

            var launchPoint = this.launchPoint;
            // if (!this._validateFields()) {
            // return;
            // }

            var namevalue = this.name.get("value").trim();
            var restExt = "template/create";

            var argJSONString = "";
            
            // Handle Different Types Of Template Dialogs
            if (this.ippoolMode) {
                restExt = "ippool/" + this.templateIDNum;
                this.templateStore.name = namevalue;
                argJSONString = dojo.toJson(this.templateStore, true);
            }
            else if (this.profileMode) {
                if(this.profileMode == this.PROFILE_MODE_SERVER)
                {
                    restExt = "profile/" + this.templateIDNum;
                }
                else if(this.profileMode == this.PROFILE_MODE_CHASSIS)
                {
                    restExt = "cmmprofile/" + this.templateIDNum;
                }
                this.templateStore.items[0].profileName = namevalue;
                argJSONString = dojo.toJson(this.templateStore.items[0], true);
            }
            else if (this.saveTemplateType == this.POL_NAME) {
                restExt = "policy?type=failover&action=create";
                this.templateStore.name = namevalue;
                argJSONString = dojo.toJson({
                    "identifier": "id",
                    "label": "name",
                    "items": [ this.templateStore ]
                }, true);
            }
            else if (this.saveTemplateType == "standby") {
                restExt = "pool/standby/create";
                this.templateStore.name = namevalue;
                argJSONString = dojo.toJson({
                    "identifier": "id",
                    "label": "name",
                    "items": [ this.templateStore ]
                }, true);
            }
            else if(this.putTemplateMode){
                restExt = "template/" + this.templateIDNum;
                
                this.templateStore.items[0].name = namevalue;
                argJSONString = dojo.toJson(this.templateStore.items[0]);
                
            }
            else {
                this.templateStore.items[0].name = namevalue;
                argJSONString = dojo.toJson(this.templateStore);
            }


            if (this.ippoolMode) {
                
                var xhrArgs = {
                        url: this.REST_PATH + restExt,
                        handleAs: "json",
                        sync: true,
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        postData: argJSONString,
                        load: function(data) {
                            
                            if(launchPoint == "details"){
                                dojo.publish("RefreshNameDiv", namevalue);                           
                            }else{
                                dojo.publish("RefreshTable", data);                        
                            }
                            
                        },
                        error: function(error) {
                            alert("Post failed: " + error);
                        }
                    };

                dojo.xhrPut(xhrArgs);
            }else if(this.profileMode || this.putTemplateMode){
                
                var xhrArgs = {
                        url: this.REST_PATH + restExt,
                        handleAs: "json",
                        sync: true,
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        postData: argJSONString,
                        load: dojo.hitch(this, function(response, ioArgs) {
                            
                            if(launchPoint == "details"){
                                dojo.publish("RefreshNameDiv", namevalue);                           
                            }else{
                                dojo.publish("RefreshTable", null);                        
                            }
                            
                        }),
                        error: dojo.hitch(this, function(error, ioArgs) {
                            // Failure path! We received a non-2xx status code (ie: 3xx/4xx/5xx status code) from the API.

                            console.log(this.declaredClass + ": ", error, ioArgs);
                            
                            var response = (restExt.indexOf("template") == 0) ? dojo.fromJson(ioArgs.xhr.responseText) : null;
                            if ((ioArgs.xhr.status >= 400) && (response && (response.message == "service_error_name_in_use"))) {
                                // Error Template name in use
                                this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                            }
                            else if(ioArgs.xhr.status == 404) {
                                // Edit Template ID doesn't exist
                                this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                            }
                            else if(ioArgs.xhr.status >= 500) {
                                // Internal Programming Error
                                this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                            }
                            else {
                                this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                            }
                            
                        })
                    };

                dojo.xhrPut(xhrArgs);
                
            }else {
                
                var xhrArgs = {
                        url: this.REST_PATH + restExt,
                        handleAs: "json",
                        sync: true,
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        postData: argJSONString,
                        load: dojo.hitch(this, function(response, ioArgs) {
                                                      
                            if(launchPoint == "details"){
                                dojo.publish("RefreshNameDiv", namevalue);                           
                            }else{
                                dojo.publish("RefreshTable", response);                        
                            }
                                                        
                        }),
                        error: dojo.hitch(this, function(error, ioArgs) {
                            // Failure path! We received a non-2xx status code (ie: 3xx/4xx/5xx status code) from the API.

                            console.log(this.declaredClass + ": ", error, ioArgs);
                            
                            var response = dojo.fromJson(ioArgs.xhr.responseText);
                            
                            if ((ioArgs.xhr.status >= 400) && (response && (response.message == "service_error_name_in_use"))) {
                                // Error Template name in use
                                this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                            }
                            else if(ioArgs.xhr.status == 404) {
                                // Edit Template ID doesn't exist
                                this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                            }
                            else if(ioArgs.xhr.status >= 500) {
                                // Internal Programming Error
                                this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                            }
                            else {
                                this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                            }
                            
                        })
                    };

                
                dojo.xhrPost(xhrArgs);
            }

            this.hide();

        },
        setCurrentName: function() {

            var name = this.name;
            var templateStore = "";            
            var policyRestPath = "policy/" + this.templateIDNum +"?type=failover";
            var templateRestPath = "template/" + this.templateIDNum;
            var restExt = (this.saveTemplateType == this.POL_NAME) ? policyRestPath : templateRestPath;
            if (this.saveTemplateType == "standby") {
                restExt = "pool/standby/" + this.templateIDNum;
            }                
            if (this.ippoolMode) {
                restExt = "ippool/" + this.templateIDNum;
            }
            if (this.profileMode) {
                if(this.profileMode == this.PROFILE_MODE_SERVER)
                {
                    restExt = "profile/" + this.templateIDNum;
                }
                else if(this.profileMode == this.PROFILE_MODE_CHASSIS)
                {
                    restExt = "cmmprofile/" + this.templateIDNum;
                }
            }

            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + restExt,
                handleAs: "json",
                sync: true,
                preventCache: true,
                load: function(response) {
                    this.store = response;
                },
                handle: function() {
                    templateStore = this.store;
                }
            });
            
            if (this.ippoolMode || this.saveTemplateType == this.POL_NAME || this.saveTemplateType == "standby") {
                name.set("value", templateStore.name);
            }
            else if (this.profileMode) {
                name.set("value", templateStore.items[0].profileName);
            }
            else {
                name.set("value", templateStore.items[0].name);
                
                if(templateStore.items[0].type == "PortVirtualization" || templateStore.items[0].type == "Adapter"){
                    this.putTemplateMode = true;
                }
                
            }                      
            
            this.currentname = this.name.get("value").trim();
            var displayName = this.encodeForHtml(this.currentname);
            this.dialogNode.set("title", this.resources.RENAME + " - " + displayName);
            this.templateStore = templateStore;

        }
    });
});