define([
        "dojo",
        "dijit",
        "support/dialogs/Dialog",
        "dojo/i18n!./nls/SaveAs",
        "dojo/text!./templates/SaveAs.html",
        "dijit/layout/ContentPane",
        "dijit/form/ValidationTextBox",
        "support/widgets/ProgressButton",
        "support/dialogs/ProgressDialog"
    ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.SaveAs", [ support.dialogs.Dialog ], {

        templateString: template, // Path to the template

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        originalName: null, //holds original name of saved pattern, only used if passed 
                                //name is different from currently saved name
                                //currently only used with Server and Chassis template
                            
        currentname: null, // template current name
        templateIDNum: null, // template ID
        templateObject: null, // holds current template data
        saveTemplateType: "",

        // Template Naming Constants
        POL_NAME: "failover",
        SERV_NAME: "Server",
        IPPOOL_NAME: "IPPool",

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/SaveAs.css",

        // Override this method to perform custom behavior during dijit construction.
        // Common operations for constructor:
        // 1) Initialize non-primitive types (i.e. objects and arrays)
        // 2) Add additional properties needed by succeeding lifecycle methods
        constructor: function() {
        },

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);

            dojo.connect(this.dialogNode, "show", this, function() {

                this.loadCurrentData();

                var templatename = this.name; // Template Name
                var okButton = this.okBtn; // Ok Button Variable
                
                if(this.originalName == null){
                    var currentname = this.currentname;                    
                }else{
                    var currentname = this.originalName;                    
                }

                templatename.intermediateChanges = true;

                dojo.connect(templatename, "onChange", function() {

                    if (templatename.get("value").trim() != "" && templatename.get("value").trim() != currentname) {
                        okButton.set("disabled", false);
                    }
                    else {
                        okButton.set("disabled", true);
                    }
                });

            });

        },

        cancelClicked: function() {
            this.hide();
        },

        onFinish: function() {

            // if (!this._validateFields()) {
            // return;
            // }

            this.templateObject.name = this.name.get("value").trim();
            var argJSONString = "";
            // The Adapter/PortPhysicalFunction/PortVirtualization type template is a little
            // different than the others
            if (this.templateObject.type == "Adapter" || this.templateObject.type == "PortPhysicalFunction"
                    || this.templateObject.type == "PortVirtualization") {
                argJSONString = dojo.toJson(this.templateObject);
            }
            else {
                this.templateObject.id = null;
                argJSONString = dojo.toJson({
                    "identifier": "id",
                    "label": "name",
                    "items": [ this.templateObject ]
                }, true);
            }

            var tempType = ""; // Holds template type for save as publish call

            var url = "";
            if (this.saveTemplateType == this.POL_NAME) {
                url = "policy?type=failover&action=create";
            }
            else if (this.saveTemplateType == this.IPPOOL_NAME) {
                url = "ippool/create";
                argJSONString = dojo.toJson(this.templateObject);
            }
            else {
                tempType = this.templateObject.type;

                if (tempType == "Adapter") {
                    url = "template/create?type=adapter";
                }
                else if (tempType == "PortPhysicalFunction") {
                    url = "template/create?type=ppfunction";
                }
                else if (tempType == "PortVirtualization") {
                    url = "template/create?type=portvirtual";
                }
                else {
                    url = "template/create";
                }
            }

            var saveTemplateType = this.saveTemplateType;

            var xhrArgs = {
                url: this.REST_PATH + url,
                handleAs: "json",
                sync: true,
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                postData: argJSONString,
                load: dojo.hitch(this, function(response, ioArgs) {

                    if (saveTemplateType == "Server" || tempType == "CMM") {
                        dojo.publish("CloseAfterSaveas", response.id);
                    }
                    else if (saveTemplateType == this.IPPOOL_NAME) {
                        dojo.publish("ippoolComplete", response);
                    }

                    // Publish to reload drop down in boot tab of Server Wizard
                    if (tempType == "FibreChannel") {
                        dojo.publish("FibreBootTemplateComplete", response.id);
                    }
                    // Publish to reload drop down in boot tab of Server Wizard
                    else if (tempType == "PXE") {
                        dojo.publish("PXEBootTemplateComplete", response.id);
                    }

                    // Publish for the io adapter dialogs
                    else if (tempType == "Adapter") {
                        dojo.publish("CreateAdapterTemplateComplete", response);
                    }
                    else if (tempType == "PortPhysicalFunction") {
                        dojo.publish("CreatePortPhysicalFunctionTemplateComplete", response);
                    }
                    else if (tempType == "PortVirtualization") {
                        dojo.publish("CreatePortVirtualizationTemplateComplete", response);
                    }
                    // Category Templates - Server
                    else if (tempType == "SystemInfo") {
                        dojo.publish("SystemTemplateComplete", response.id);
                    }
                    else if (tempType == "Management") {
                        dojo.publish("EthTemplateComplete", response.id);
                    }
                    else if (tempType == "Power") {
                        dojo.publish("PowerTemplateComplete", response.id);
                    }
                    else if (tempType == "PerformanceRecovery") {
                        dojo.publish("PerformanceAndRecoveryComplete", response.id);
                    }
                    else if (tempType == "DevicesAndIOPorts") {
                        dojo.publish("DevicesAndIOPortsComplete", response.id);
                    }
                    // Category Templates - Chassis
                    else if (tempType == "cmmInfo") {
                        dojo.publish("ChassisInfoTemplateComplete", response.id);
                    }
                    else if (tempType == "cmmManagement") {
                        dojo.publish("ChassisManagementComplete", response.id);
                    }
                    else if (tempType == "cmmPower") {
                        dojo.publish("ChassisPowerComplete", response.id);
                    }
                    else if (tempType == "cmmUsers") {
                        dojo.publish("ChassisUserTemplateComplete", response.id);
                    }
                    else if (tempType == "cmmIOModule") {
                        dojo.publish("ChassisIOTemplateComplete", response.id);
                    }
                    else if (tempType == "cmmNode") {
                        dojo.publish("ChassisNodeTemplateComplete", response.id);
                    }
                    else if (tempType == "ExtendedIMM") {
                        dojo.publish("ExImmTemplateComplete", response.id);
                    }
                    else if (tempType == "ExtendedUEFI") {
                        dojo.publish("ExUefiTemplateComplete", response.id);
                    }
                    else if (tempType == "ExtendedPort") {
                        dojo.publish("ExPortTemplateComplete", response.id);
                    }

                    dojo.publish("RefreshTable", response.id);
                    this._onClose();
                }),
                error: dojo.hitch(this, function(error, ioArgs) {
                    // Failure path! We received a non-2xx status code (ie: 3xx/4xx/5xx status code)
                    // from the API.

                    console.log(this.declaredClass + ": ", error, ioArgs);

                    var response = (url.indexOf("template/create") == 0 || url.indexOf("policy?type=failover&action=create") == 0) ? dojo.fromJson(ioArgs.xhr.responseText) : null;

                    if ((ioArgs.xhr.status >= 400) && (response && (response.message == "service_error_name_in_use"))) {
                        // Error Template name in use
                        this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                    }
                    else if (ioArgs.xhr.status == 404) {
                        // Edit Template ID doesn't exist
                        this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                        this.hide();
                    }
                    else if (ioArgs.xhr.status >= 500) {
                        // Internal Programming Error
                        this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                        this.hide();
                    }
                    else {
                        this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                        this.hide();
                    }

                })
            };
            dojo.xhrPost(xhrArgs);
            return false; // Don't close now. Allow load/error handler to close/hide when appropriate.
            
        },

        loadCurrentData: function() {
            this.name.set("value", this.templateObject.name);
            this.currentname = this.templateObject.name.trim();
        }
    });
});