define([
        "dojo",
        "dijit",
        "support/dialogs/Dialog",
        "dojo/i18n!./nls/SelectExistingPattern",
        "dojo/text!./templates/SelectExistingPattern.html",
        "dijit/layout/ContentPane",
        "support/widgets/Select",
        "support/dialogs/ServerTemplateWizard",
        "support/dialogs/ChassisTemplateWizard"
    ], function(dojo, dijit, dialog, mri, template) {
    
    return dojo.declare("support.dialogs.SelectExistingPattern", [ support.dialogs.Dialog ], {
        
        templateString: template, // Path to the template

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,
        
        _nodeTitle: "", // Dialog title
        _pattern: "",   // Selector label
        
        //patternType: "",
        tmplStore: null,
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/SelectExistingPattern.css",

        // Override this method to perform custom behavior during dijit construction.
        // Common operations for constructor:
        // 1) Initialize non-primitive types (i.e. objects and arrays)
        // 2) Add additional properties needed by succeeding lifecycle methods
        constructor: function() {
            
        },

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);

            this.setType();
        },
        
        postCreate: function() {
            this.inherited(arguments);
            
            dojo.xhrGet({
                url: this.REST_PATH + "template/list/" + this.patternType,
                handleAs: "json",
                sync: true,
                load: dojo.hitch(this, function(data) {
                    // Dont need to store all that server template info in the selector...
                    if (data.items) {
                        var i;
                        var items = [];
                        var newEntry = function(template) {
                            var temp = new Object();
                            temp.id = template.id;
                            temp.name = template.name;
                            temp.inUse = template.inUse;
                            temp.formFactor = template.formFactor;
                            temp.type = template.type;
                            temp.description = template.description;
                            temp.nodeType = template.nodeType;
                            temp.uri = template.uri;
                            return temp;
                        };
                        for (i = 0; i < data.items.length; i++) {
                            var temp = newEntry(data.items[i]);
                            items.push(temp);
                        }
                        var templateData = dojo.toJson({
                            "identifier": "id",
                            "label": "name",
                            "items": items
                        }, true);
                        this.tmplStore = new dojo.data.ItemFileWriteStore({
                            data: dojo.fromJson(templateData)
                        });
                    }
                })
            });
            
            // Add prompt option before load so its listed first.
            this.tmplStore.newItem({
                id: "-1",
                name: "&mdash; " + this.locResources.SELECT_TEMPLATE + " &mdash;"
            });

            this.editPattern = new support.widgets.Select({
                id: this.get("id") + "_editPattern",
                name: "template",
                required: true,
                store: this.tmplStore,
                //value: (this.templateId) ? this.templateId : "-1", // Pre-select entry from store,
                // if it exists.
                emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                onChange: dojo.hitch(this, this._enableEdit)
            }).placeAt(this._editPattern);
            
            //dojo.connect(this.editPattern, "onSetStore", this, "_presetTemplateCallback");
            this.editPattern.startup();

        },
        
        _enableEdit: function() {
            var selected = this.editPattern.get("value");
            if (selected > 0) {
                this.okBtn.set("disabled", false);
            } else {
                this.okBtn.set("disabled", true);
            }
        },
        
        _onFinish: function() {
            var selectedPatternID = this.editPattern.get("value");
            if (this.patternType == "Server") {
                var dialog = new support.dialogs.ServerTemplateWizard({
                    templateIDNum: selectedPatternID,
                    editMode: true
                });
                dialog.show();
            }
            if (this.patternType == "CMM") {
                var dialog = new support.dialogs.ChassisTemplateWizard({
                    templateIDNum: selectedPatternID,
                    editMode: true
                });
                dialog.show();
            }
            this.hide();
        },
        
        _onCancel: function() {
            this.hide();
        },
        
        setServer: function() {
            this.patternType = "Server";
        },
        
        setChassis: function() {
            this.patternType = "CMM";
        },
        
        setType: function() {
            if (this.patternType == "Server") {
                this._nodeTitle = this.locResources.SELECT_SERVER;
                this._pattern = this.locResources.SERVER_PATTERN;
            }
            if (this.patternType == "CMM") {
                this._nodeTitle = this.locResources.SELECT_CHASSIS;
                this._pattern = this.locResources.CHASSIS_PATTERN;
            }
        }
        
    });
});