define([ 
        "dojo", 
        "dijit", 
        "dojo/aspect", 
        "support/dialogs/Dialog", 
        "dojo/i18n!./nls/ServerTemplateWizard",
        "dojo/text!./templates/ServerTemplateWizard.html", 
        "dojo/dom-style",
        "dojo/dom-class",
        "dijit/layout/ContentPane", 
        "dijit/layout/TabContainer", 
        "support/wizard/pages/nstGeneral", 
        "support/wizard/pages/nstStorage", 
        "support/wizard/pages/nstIOAdapters",
        "support/wizard/pages/nstBoot", 
        "support/wizard/pages/nstFwSettings",
        "support/dialogs/ProgressDialog"
        ], function(dojo, dijit, aspect, dialog, mri, template, domStyle, domClass) {

    return dojo.declare("support.dialogs.ServerTemplateWizard", [ support.dialogs.Dialog ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        // Number of Steps for wizard completion
        numberOfSteps: null,
        
        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "ServerTemplateWizard",

        // templateIDNum: String
        // Specifies the ID of an existing template. If specified, the wizard will attempt to
        // retrieve settings from the specified template and use them to pre-populate wizard pages.
        //
        // NOTE: If both "templateIDNum" and "baseServer" attributes are specified, the
        // "templateIDNum" attribute will be given preference.
        //
        templateIDNum: null,

        // templateInUse: String
        // Specifies whether an existing template is in use. If in use, this server template has
        // been
        // deployed to 1 or more servers/locations, which means editing an in use template requires
        // re-deploying this template.
        //
        templateInUse: false,

        // baseServer: Object
        // Specifies attributes used to identify an existing server. If specified, the wizard will
        // attempt to retrieve settings for the specified server and use them to pre-populate wizard
        // pages.
        //
        // For example:
        // baseServer: {
        //   chassisName: "r3-c2-ch3-mm.rtp.stglabs.ibm.com",
        //   serverName: "ip9-27-22-242.rtp.stglabs.ibm.com", 
        //   serverId: "958818DC5DB111E185C05CF3FC7F1758",
        //   type: "sysx",
        //   formFactor: "4s"
        // }
        //
        // NOTE: If both "templateIDNum" and "baseServer" attributes are specified, the
        // "templateIDNum" attribute will be given preference.
        //
        baseServer: null,

        // editMode: boolean
        // Specifies whether the wizard is to execute in edit mode. If "true", the "templateIDNum"
        // attribute is required.
        editMode: false,

        // copyMode: boolean
        // Specifies whether the wizard is to execute in copy mode. If "true", the "templateIDNum"
        // attribute is required.
        copyMode: false,

        // preGeneratedTemplate: boolean
        // Specifies whether the wizard is using a pre-generated(ie generated from existing server
        // settings.
        // If "true", the "templateIDNum"
        // attribute is required.
        preGeneratedTemplate: false,

        // templateType: String
        // Type of template created by this wizard. This name is only used internally
        templateType: "Server",
        
        // savedStorageStep : object
        // Holds the storage step saved if hidden/removed for 1sd support
        savedStorageStep: null,

        createFromExisting: false,

        constructor: function() {
            //Set page titles
            this.GENERAL_PAGE_TITLE = this.locResources.GEN_TITLE;
            this.STORAGE_PAGE_TITLE = this.locResources.STORAGE_TITLE;
            this.IOADAPTERS_PAGE_TITLE = this.locResources.IO_TITLE;
            this.BOOT_PAGE_TITLE = this.locResources.BOOT_TITLE;
            this.FWSETTINGS_PAGE_TITLE = this.locResources.FW_TITLE;
        },

        //Holds user selected preferences
        userPreferences: null,
        
        //Holds wizard object, currently a tab container widget
        wizard: null,
        
        //Holds page objects
        generalPage: null,
        storagePage: null,
        ioAdaptersPage: null,
        bootPage: null,
        fwSettingsPage: null,
              
        //Used to compare saved pattern vs edit pattern, checks if any changes during edit
        _originalServerPattern: null,

        //Holds learned server ID
        _learnedServerId: null,

        // spCssPathAndName: String
        // The name of the CSS file for this widget.
        spCssPathAndName: "support/dialogs/themes/ServerTemplateWizard.css",

        postCreate: function() {    
            this.inherited(arguments);
                        
            // Set Dialog Title
            var title = this.copyMode ? this.locResources.COPY_TITLE : this.editMode ? this.locResources.EDIT_TITLE
                    : this.locResources.CREATE_TITLE;
            this.set("title", title);

            // Get preferences set by the user...
            this._getUserPreferences();

            // Set up wizard & pages ...
            this._setupWizard();

            // AA120928-- Workaround for this being a wrapper for Dialog vs a subclass.
            // We need to have the wizard go re-lay itself out now that we've resized it,
            // but it's fruitless to do this until it's showing on the screen. You
            // would think that when you resize the wizard it would automatically
            // re-lay itself out, but that is not the case.
          
            /*aspect.after(this.dialogNode, "onShow", dojo.hitch(this, function() {
                this.wizard.updateUI();
              }));*/

            // BUGBUG- AA121017: SOMETIMES in FF, you don't get the onload event
            // for the CSS. So, we put in this ugly hack :(
            //window.setTimeout(dojo.hitch(this, this.cssArrived), 10000);

            return this.inherited(arguments);
        },

/*        cssArrived: function() {
            // It's possible for the CSS to arrive before we've even created the wizard.
            if (this.wizard) {
           //    this.wizard.resize();
                // This tells the wizard's BorderContainer to re-size the content div.
                //this.resize();
            }

            this.inherited(arguments);
        },*/

        _getUserPreferences: function() {
            // tags:
            // protected
            var preferences = null;
            dojo.xhrGet({
                url: this.REST_PATH + "config/default",
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                sync: true,
                load: function(response, ioArgs) {
                    preferences = response;
                },
                error: function(error, ioArgs) {
                    console.error(this.declaredClass + ": Error (", error, ") occurred when retrieving user preferences.");
                    alert(this.resources.ERROR + " - " + error);
                    preferences = {};
                    return false;
                }
            });
            this.userPreferences = preferences;
            console.log(this.declaredClass + ": user preferences: ", this.userPreferences);
        },

        _setupWizard: function() {
            // tags:
            // protected
            
            //Controls whether to enable the next button, can be called from wizard pages
            dojo.subscribe("enableNextButton", this, function(response) {
                                
                //If true then enable the next button else disable it and the save actions
                if(response){
                    this.nextButton.set("disabled", false);
                    
                    //Check if all pages enabled, if not then 
                     // don't enable to save, saveas, save deploy buttons
                    var canEnableSave = true;                    
                    dojo.forEach(this.wizard.getChildren(), dojo.hitch(this, function (childPage) {
                        if(childPage.cpDisabled){
                            canEnableSave = false;
                        }                        
                    }));                    
                    dojo.publish("enableSaveButton", canEnableSave);
                    dojo.publish("enableSaveDeployButton", canEnableSave);

                }else{  
                    //Disable next button and any save actions
                    this.nextButton.set("disabled", true);
                    dojo.publish("enableSaveButton", false);
                    dojo.publish("enableSaveDeployButton", false);
                }
                
            });

            //Controls whether to enable the save button, can be called from wizard pages
            dojo.subscribe("enableSaveButton", this, function(response) {
                                
                if(response){
                    this.saveButton.set("disabled", false);
                }else{                    
                    this.saveButton.set("disabled", true);
                }
                
            });

            //Controls whether to enable the saveas / savedeploy button, can be called from wizard pages
            dojo.subscribe("enableSaveDeployButton", this, function(response) {
                                
                if(response){
                    this.saveDeployButton.set("disabled", false);
                }else{                    
                    this.saveDeployButton.set("disabled", true);
                }
                
            });

            //Create wizard from layout tab container and place
            this.wizard = new dijit.layout.TabContainer({
                style: "height: 100%; width: 100%;"
            }, this._wizardNode);

            //Load wizard navigation button depending on wizard launch state {edit,copy,new}
            if(this.editMode){

                this.backButton = new dijit.form.Button({
                    disabled: true,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.BACK_BTN,
                    onClick: dojo.hitch(this, "onBack")
                }).placeAt(this._wizardButtonBar);

                this.nextButton = new dijit.form.Button({
                    disabled: false,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.NEXT_BTN,
                    onClick: dojo.hitch(this, "onNext")
                }).placeAt(this._wizardButtonBar);

                this.saveButton = new dijit.form.Button({
                    disabled: false,
                    showLabel: true,
                    role: "button",
                    label: this.resources.SAVE,
                    onClick: dojo.hitch(this, "onSave")
                }).placeAt(this._wizardButtonBar);

                this.saveDeployButton = new dijit.form.Button({
                    disabled: false,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.SAVEAS_BTN,
                    onClick: dojo.hitch(this, "onSaveDeploy")
                }).placeAt(this._wizardButtonBar);

                
            }else if(this.copyMode){

                this.backButton = new dijit.form.Button({
                    disabled: true,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.BACK_BTN,
                    onClick: dojo.hitch(this, "onBack")
                }).placeAt(this._wizardButtonBar);

                this.nextButton = new dijit.form.Button({
                    disabled: false,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.NEXT_BTN,
                    onClick: dojo.hitch(this, "onNext")
                }).placeAt(this._wizardButtonBar);

                this.saveButton = new dijit.form.Button({
                    disabled: false,
                    showLabel: true,
                    role: "button",
                    label: this.resources.SAVE,
                    onClick: dojo.hitch(this, "onSave")
                }).placeAt(this._wizardButtonBar);

                this.saveDeployButton = new dijit.form.Button({
                    disabled: false,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.BTN_LABEL,
                    onClick: dojo.hitch(this, "onSaveDeploy")
                }).placeAt(this._wizardButtonBar);

            }else{
            
                this.backButton = new dijit.form.Button({
                    disabled: true,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.BACK_BTN,
                    onClick: dojo.hitch(this, "onBack")
                }).placeAt(this._wizardButtonBar);

                this.nextButton = new dijit.form.Button({
                    disabled: true,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.NEXT_BTN,
                    onClick: dojo.hitch(this, "onNext")
                }).placeAt(this._wizardButtonBar);

                this.saveButton = new dijit.form.Button({
                    disabled: true,
                    showLabel: true,
                    role: "button",
                    label: this.resources.SAVE,
                    onClick: dojo.hitch(this, "onSave")
                }).placeAt(this._wizardButtonBar);

                this.saveDeployButton = new dijit.form.Button({
                    disabled: true,
                    showLabel: true,
                    role: "button",
                    label: this.locResources.BTN_LABEL,
                    onClick: dojo.hitch(this, "onSaveDeploy")
                }).placeAt(this._wizardButtonBar);
                
            }
                      
            this.cancelButton = new dijit.form.Button({
                id: this.id + "cp_servertemplatewizard_cancel",
                showLabel: true,
                role: "button",
                label: this.resources.CANCEL,
                onClick: dojo.hitch(this, "_onClose")
            }).placeAt(this._wizardButtonBar);

            //If edit or copy mode then load object with settings
            var svrTemplate = null;            
            if (this.copyMode || this.editMode) {
                svrTemplate = this._getServerPatternFromServices();
                // Remember Template's Name
                if (svrTemplate.name) {
                    this.templateName = svrTemplate.name;
                }
            }

            // Prepare General page and add into the wizard.
            var generalPageParams = new Object();
            if (svrTemplate) {
                // Pass current values...
                generalPageParams.nodeType = svrTemplate.nodeType;
                generalPageParams.formFactor = svrTemplate.formFactor;
                generalPageParams.templateName = svrTemplate.name;
                generalPageParams.templateDesc = this.decodeSpecialCharacters(svrTemplate.description);
            }
            else {
                // Pass default values...
                generalPageParams.nodeType = this.userPreferences.nodeType;
                generalPageParams.formFactor = this.userPreferences.formFactor;
            }
            this.generalPage = new support.wizard.pages.nstGeneral({
               editMode:this.editMode,
               copyMode:this.copyMode,
               inUse:this.templateInUse,
               existing: this.createFromExisting,
               pageParams: generalPageParams
            });

            dojo.connect(this.generalPage, "onChangeNodeType", this, "_prepareForNodeType");
            dojo.connect(this.generalPage, "onChangeFormFactor", this, "_prepareForFormFactor");
            dojo.connect(this.generalPage, "onChangeStartingPoint", this, "_prepareForStartingPoint");
            dojo.connect(this.generalPage, "onChangeServer", this, "_queueBaseServerChange");

            // Prepare Storage page and add into the wizard.
            var storagePageParams = new Object();
            if (svrTemplate) {
                // Pass current values...
                storagePageParams.storageID = svrTemplate.storageSettings.storageSelection;
                storagePageParams.storageAttrs = svrTemplate.storageSettings.storageAttr;
                storagePageParams.formFactor = svrTemplate.formFactor;
            }
            else {
                // Pass default values...
                storagePageParams.storageID = "Keep Existing";
                storagePageParams.storageAttrs = {};
                storagePageParams.formFactor = this.userPreferences.formFactor;
            }
            this.storagePage = new support.wizard.pages.nstStorage({
                editMode:this.editMode,
                copyMode:this.copyMode,
                inUse:this.templateInUse,
                pageParams: storagePageParams
            });


            // Prepare IOAdapters page and add into the wizard.
            var ioAdaptersPageParams = new Object();
            if (svrTemplate) {
                // Pass current values...
                ioAdaptersPageParams.nodeType = svrTemplate.nodeType;
                ioAdaptersPageParams.formFactor = svrTemplate.formFactor;
                ioAdaptersPageParams.adapterSettings = svrTemplate.adapterSettings;
                // Need to set the defaults if burned addressing otherwise, when toggling, the 
                // settings will be invalid.
                if (ioAdaptersPageParams.adapterSettings.ioAddressingMode == "burned") {
                    ioAdaptersPageParams.adapterSettings.ethAddressPool = "IBM";
                    ioAdaptersPageParams.adapterSettings.ethAddressDomain = "1";
                    ioAdaptersPageParams.adapterSettings.fcAddressPool = "IBM";
                    ioAdaptersPageParams.adapterSettings.fcAddressDomain = "1";                    
                }
            }
            else {
                // Pass default values...
                ioAdaptersPageParams.nodeType = this.userPreferences.nodeType;
                ioAdaptersPageParams.formFactor = this.userPreferences.formFactor;
                ioAdaptersPageParams.adapterSettings = new Object();
                ioAdaptersPageParams.adapterSettings.ioAddressingMode = this.userPreferences.ioAddressingMode;
                ioAdaptersPageParams.adapterSettings.ethAddressPool = "IBM";
                ioAdaptersPageParams.adapterSettings.ethAddressDomain = "1";
                ioAdaptersPageParams.adapterSettings.fcAddressPool = "IBM";
                ioAdaptersPageParams.adapterSettings.fcAddressDomain = "1";
                ioAdaptersPageParams.adapterSettings.adapters = new Array(); // None, by default.
            }
            this.ioAdaptersPage = new support.wizard.pages.nstIOAdapters({
                editMode:this.editMode,
                copyMode:this.copyMode,
                inUse:this.templateInUse,
                pageParams: ioAdaptersPageParams
            });

            dojo.connect(this.ioAdaptersPage, "onChangeIoAddressingMode", this, "_prepareForIoAddressingMode");
            dojo.connect(this.ioAdaptersPage, "onChangeIoBootDevices", this, "_prepareForIoBootDevices");

            // Prepare Boot page and add into the wizard.
            var bootPageParams = new Object();
            if (svrTemplate) {
                // Pass current values...                
                bootPageParams.bootOrder = svrTemplate.bootSettings.bootOrder;                    
                bootPageParams.bootOrderTable = svrTemplate.bootSettings.bootOrderTable;
                bootPageParams.bootWoLTable = svrTemplate.bootSettings.bootWoLTable;
                bootPageParams.learnedBootOptions = svrTemplate.bootSettings.learnedBootOptions;
                bootPageParams.learnedWoLBootOptions = svrTemplate.bootSettings.learnedWoLBootOptions;                
                bootPageParams.ioAddressingMode = this.ioAdaptersPage.getIoAddressingMode();
                bootPageParams.fcAddressPool = this.ioAdaptersPage.getFcAddressPool();
                bootPageParams.ioBootDevices = this.ioAdaptersPage.getIoBootDevices();
            }
            else {
                // Pass default values...
                bootPageParams.bootOrderTable = null;
                bootPageParams.bootWoLTable = null;
                bootPageParams.learnedBootOptions = null;
                bootPageParams.learnedWoLBootOptions = null;
                bootPageParams.bootOrder = "";  // "" is "Keep existing"
                bootPageParams.ioAddressingMode = this.ioAdaptersPage.getIoAddressingMode();
                bootPageParams.fcAddressPool = this.ioAdaptersPage.getFcAddressPool();
                bootPageParams.ioBootDevices = this.ioAdaptersPage.getIoBootDevices();
            }
            this.bootPage = new support.wizard.pages.nstBoot({
                editMode:this.editMode,
                copyMode:this.copyMode,
                inUse:this.templateInUse,
                pageParams: bootPageParams
            });

            // Make sure the boot mode is set based on the form factor 
            if (svrTemplate) {
                this.bootPage.setBootMode(svrTemplate.formFactor);
            }
            
            // Prepare Firmware Settings page and add into the wizard.
            var fwSettingsPageParams = new Object();
            if (svrTemplate) {
                // Pass current values...
                fwSettingsPageParams.templates = svrTemplate.templates;
                fwSettingsPageParams.formFactor = svrTemplate.formFactor;
            }
            else {
                // Pass empty array
                fwSettingsPageParams.templates = new Array();
                fwSettingsPageParams.formFactor = this.userPreferences.formFactor;
            }
            this.fwSettingsPage = new support.wizard.pages.nstFwSettings({
                editMode:this.editMode,
                copyMode:this.copyMode,
                inUse:this.templateInUse,
                pageParams: fwSettingsPageParams
            });

            
            //Build array of wizard pages and titles to pass to tab container
            var contents = [this.generalPage,this.storagePage,this.ioAdaptersPage,this.bootPage,
                            this.fwSettingsPage];                    
            var titles = [this.GENERAL_PAGE_TITLE,this.STORAGE_PAGE_TITLE,this.IOADAPTERS_PAGE_TITLE,this.BOOT_PAGE_TITLE,
                          this.FWSETTINGS_PAGE_TITLE
                         ];

            var cpDisabled = false; //Holds if page should be disabled on launch
            
            //Loop through each page and add it to the wizard (tab container)           
            for(var count=0;count<contents.length;count++){

                var cp = new dijit.layout.ContentPane({
                    style: "overflow: auto;",
                    content: contents[count],
                    title: titles[count],
                    position: count+1,
                    cpDisabled: cpDisabled                    
                });
                
                this.wizard.addChild(cp);
                
                //If wizard launch is not edit or copy then disable all pages after first page
                if(this.editMode == false && this.copyMode == false)
                    cpDisabled = true;                
                
            }

            //This function is fired each time a page (tab) is selected
            this.wizard.watch("selectedChildWidget", dojo.hitch(this, function(name, oval, nval){

                //Get all wizard pages
                var childrenArray = this.wizard.getChildren();
                
                //Don't allow select of next page if previous page is still disabled or next button is not enabled
                if(nval.position != 1 && (childrenArray[nval.position-2].cpDisabled == true || this.nextButton.get("disabled") == true) && oval.position < nval.position){
                    //Reselect previous page instead of the newly selected page
                    this.wizard.selectChild(oval);
                }else{
                    //Set page isDisabled to false meaning the page has been visited properly
                    nval.cpDisabled = false;
                    //Remove custom style for disabled page
                    domStyle.set(nval.controlButton.domNode,"opacity",1);                                
                    domStyle.set(nval.controlButton.domNode,"filter","alpha(opacity=100)");

                    //Disable the next button and allow page AfterPageSelected funtion to properly set next button
                    dojo.publish("enableNextButton", false);

                    //Run special functions if newly selected page is the Boot page
                    if (nval.title == "Boot") {
                        this.bootPage.pageParams.fcAddressPool = this.ioAdaptersPage.getFcAddressPool();
                        this.bootPage.reloadData();
                    }

                    //Run function linked to page that is called after a page is selected
                     // enable or disable next button from this function and also run page setups
                    dijit.byId(nval.content.id).AfterPageSelected();

                    //If at the last step of the wizard then enable save actions
                    if(this.numberOfSteps == this.wizard.selectedChildWidget.position){
                        dojo.publish("enableSaveButton", true);
                        dojo.publish("enableSaveDeployButton", true);                    
                    }

                    //If not on the first page of the wizard then allow then enable back button
                    if(this.wizard.selectedChildWidget.position > 1){
                        this.enableBackButton(true);
                    }else{
                        this.enableBackButton(false);
                    }


                }
                                  
             })); 
                           
            //Widget call to startup wizard with given information
            this.wizard.startup();
            
            //Get number of pages in the wizard after startup and disable all pages after first
             // if new wizard (meaning not edit or copy)
            var childrenArray = this.wizard.getChildren();
            this.numberOfSteps = childrenArray.length;
                            
            //Fade all pages expect first for new wizard if new wizard
            if(this.editMode == false && this.copyMode == false){
                var count=0;
                                
                dojo.forEach(this.wizard.getChildren(), dojo.hitch(this, function (childPage) {
                    if(count!=0){
                        domStyle.set(childPage.controlButton.domNode,"opacity",.4);                                
                        domStyle.set(childPage.controlButton.domNode,"filter","alpha(opacity=40)");
                    }
                    count = count + 1;
                }));
            }

            // Close Self If Save As Complete
            this.subscribe("CloseAfterSaveas", function(success) {
                if (success) {
                    this.hide();
                }
                else {
                    console.error(this.declaredClass + ": Failed To Save Server Template");
                }
            });

            
        },
        enableBackButton: function (response){
            //Enable/disable wizard back button
            if(response){
                this.backButton.set("disabled", false);
            }else{                    
                this.backButton.set("disabled", true);
            }
        },
        onBack: function (){
            //Called when wizard back button clicked to go back to previous page            
            this.wizard.back();                    

            //Disable back button if newly selected page is first page
            if (this.wizard.selectedChildWidget.position == 1) {
                this.enableBackButton(false);                
            }
        },    
        onNext: function () {
            //Called when wizard next button clicked to go to next page            

            // If a base server was selected for "learn from existing server" path but has not yet been
                // successfully processed, attempt to process the selection now. If successful allow user to
                // proceed to next page.
                if (this.wizard.selectedChildWidget.position == 1 && this.baseServer && (this.baseServer.serverId != this._learnedServerId)) {
                   
                    var proceedCallback = dojo.hitch(this, function() {
                        this._learnedServerId = this.baseServer.serverId;
                        this.wizard.forward();
                    });
                    
                    //Run the learning process of the selected base/server
                    this._prepareForServer(this.baseServer, proceedCallback);
          
                }else{
                    this.wizard.forward();
                }
                
        },
        onClose: function() {
            this.inherited(arguments);
            
            // Function connected to the "Cancel" button
            var confirmedCancel = function() {
                this.hide();
                dojo.publish("CloseDialog");
            };

            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.OK,
                        type: "Warning",
                        text: this.resources.WIZARD_CANCEL_MESSAGE,
                        info: this.resources.WIZARD_CANCEL_INFO                    
                    });

                    confirmDialog.startup();
                    confirmDialog.confirm(confirmedCancel, this);
                
                }));
 
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.OK,
                        type: "Warning",
                        text: this.resources.WIZARD_CANCEL_MESSAGE,
                        info: this.resources.WIZARD_CANCEL_INFO                    
                    });

                    confirmDialog.startup();
                    confirmDialog.confirm(confirmedCancel, this);
                
                }));
                
            }
            
        },

        onSaveDeploy: function() {
            // Function connected to the "Save and Deploy" and "Save As" buttons

            if (!this.editMode) {

                // Set up subscriber for _saveTemplateAsync() below.
                var handle = dojo.subscribe("ServerTemplateComplete", this, function(response) {
                    if ((response.status >= 200) && (response.status < 300)) {
                        require([ "support/dialogs/DeployServerTemplate" ], dojo.hitch(this, function() {
                            var dst = new support.dialogs.DeployServerTemplate({
                                templateId: response.templateId
                            });
                            dojo.body().appendChild(dst.domNode);
                            dst.show();
                        }));
                        this.hide();
                    }
                    else {
                        console.error(this.declaredClass + ": Error Saving Server Template");

                        if ((response.status >= 400) && (response.response.message == "service_error_name_in_use")) {
                            // Error, pattern name in use
                            this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                        }
                        else if (response.status == 404) {
                            // Error, pattern does not exist. Use Save As.
                            this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                        }
                        else if (response.status >= 500) {
                            // Internal error. Call support.
                            this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                        }
                        else {
                            // Error occurred. Try again or call support.
                            this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                        }

                    }
                    handle && dojo.unsubscribe(handle);
                });

                // Save and Deploy
                this._saveTemplateAsync();
                return false; // Don't allow close! Must be handled by subscriber (above).
            }
            else {
                // Save As
                var template = this._generateTemplateData();
                
                require([ "support/dialogs/SaveAs" ], dojo.hitch(this, function() {
                        var saveAs = new support.dialogs.SaveAs({
                            templateObject: template,
                            originalName: this.templateName,
                            saveTemplateType: this.templateType
                        });
                        dojo.body().appendChild(saveAs.domNode);
                        saveAs.show();
                }));                                
            }

        },

        _prepareForNodeType: function(nodeType) {
            // tags:
            // protected
            console.log(this.declaredClass + ": _prepareForNodeType called: ", nodeType);
            this.generalPage.pageParams.nodeType = nodeType;
            this.ioAdaptersPage.pageParams.nodeType = nodeType;
        },
        _prepareForFormFactor: function(formFactor) {
            // tags:
            // protected
            console.log(this.declaredClass + ": _prepareForFormFactor called: ", formFactor);
            this.generalPage.pageParams.formFactor = formFactor;
            this.storagePage.pageParams.formFactor = formFactor;
            this.ioAdaptersPage.pageParams.formFactor = formFactor;
            this.fwSettingsPage.pageParams.formFactor = formFactor;
            
            if(formFactor == "1sd"){
                
                //Get all wizard pages
                var childrenArray = this.wizard.getChildren();
                
                //Pull out the storage step and save it to a variable
                this.savedStorageStep = childrenArray[1];
                
                //Remove storage step as not needed with 1sd form factor
                this.wizard.removeChild(childrenArray[1]); 

                //Get all wizard pages again to set number of steps and proper position count                 
                childrenArray = this.wizard.getChildren();
                this.numberOfSteps = childrenArray.length;

                var count=0;
                dojo.forEach(this.wizard.getChildren(), dojo.hitch(this, function (childPage) {
                    childPage.position = count+1;
                    count = count + 1;
                }));

            } else {
            
                //If savedStorageStep not null (not removed) then re-add it to the wizard
                if(this.savedStorageStep != null){                    
                    this.wizard.addChild(this.savedStorageStep, 1);                    
                    this.savedStorageStep = null;
                    
                    //Get all wizard pages to set number of steps and proper position count                                     
                    var childrenArray = this.wizard.getChildren();
                    this.numberOfSteps = childrenArray.length;

                    //Apply disable style to storage page if disabled 
                    if(childrenArray[1].cpDisabled == true){
                        domStyle.set(childrenArray[1].controlButton.domNode,"opacity",.4);                                
                        domStyle.set(childrenArray[1].controlButton.domNode,"filter","alpha(opacity=40)");
                    }
                    
                    var count=0;
                    dojo.forEach(this.wizard.getChildren(), dojo.hitch(this, function (childPage) {
                        childPage.position = count+1;
                        count = count + 1;
                    }));

                }

            }
            
            this.storagePage.refreshRAIDOptions();
            this.ioAdaptersPage.reloadData();
            this.bootPage.setBootMode(formFactor);
            this.fwSettingsPage._validateChannelMode();
            this.fwSettingsPage._scalableWarning();
         },

        _prepareForStartingPoint: function(startingPoint) {
            // tags:
            // protected
            // Enable or Disable page options based on starting point
            console.log(this.declaredClass + ": _prepareForStartingPoint called: ", startingPoint);
            if (!this.editMode) {
                if (startingPoint == this.generalPage.FROM_SCRATCH) {
                    this._queueBaseServerChange(null); // clear base server, if any
                    this.fwSettingsPage.set("startingPoint", "FROM_SCRATCH");
                }
                else if (startingPoint == this.generalPage.FROM_SERVER) {
                    this.fwSettingsPage.set("startingPoint", "FROM_SERVER");
                }
            }
        },
        
        _queueBaseServerChange: function(baseServer) {
            // tags:
            // protected
            // Queues request to set base server for "learning" process. The user's selection is processed
            // when they click "Next" button to proceed.
            console.log(this.declaredClass + ": _queueBaseServerChange called: ", (baseServer) ? baseServer.serverId : null);
            this.baseServer = baseServer;
            if (!baseServer || (baseServer.serverId != this._learnedServerId)) {
                this._learnedServerId = null;
            }

        },

        _prepareForServer: function(baseServer, onSuccessCallback) {
            // tags:
            // protected
            console.log(this.declaredClass + ": _prepareForServer called: ", baseServer, onSuccessCallback);
            this.fwSettingsPage.set("chassis", baseServer.chassisName);
            this.fwSettingsPage.set("server", baseServer.serverName);
            this.getProgressDialog("learn").show(); // Show progress dialog for learn request.
            var templateName = this.generalPage._templateName.get("value").trim();
            var postData = dojo.toJson({
                "templateName": templateName
            });
            dojo.xhrPost({
                url: this.REST_PATH + "template/server/" + baseServer.serverId,
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                postData: postData,
                load: dojo.hitch(this, function(response, ioArgs) {
                    console.log(this.declaredClass + ": Generate learned template complete: ", response, ioArgs);

                    //Update Server Pattern Table after learning process
                    dojo.publish("RefreshTable");

                    if (response.id) {
                        var svrTemplateId = response.id;
                        dojo.xhrGet({
                            url: this.REST_PATH + "template/" + svrTemplateId,
                            handleAs: "json",
                            headers: {
                                "Content-Type": "application/json; charset=utf-8"
                            },
                            preventCache: true,
                            load: dojo.hitch(this, function(response, ioArgs) {
                                console.log(this.declaredClass + ": Get template complete: ", response, ioArgs);

                                if (response.items && (response.items.length > 0)) {

                                    var svrTemplate = response.items[0];

                                    this.templateIDNum = svrTemplateId;
                                    this.preGeneratedTemplate = true;

                                    // Call connect handlers first since they'll propagate a few of
                                    // these learned settings for us.
                                    this._prepareForNodeType(svrTemplate.nodeType);
                                    this._prepareForFormFactor(svrTemplate.formFactor);
                                    this._prepareForIoAddressingMode(svrTemplate.adapterSettings.ioAddressingMode);

                                    // General Page specific data
                                    this.generalPage.pageParams.templateName = svrTemplate.name;
                                    this.generalPage.pageParams.templateDesc = this.decodeSpecialCharacters(svrTemplate.description);
                                    // NOTE: Also need to set these directly since this process
                                    // initiated from the General Page. Setting page params alone is not
                                    // effect in this case since they've already been processed.
                                    this.generalPage.setNodeType(svrTemplate.nodeType, false);
                                    this.generalPage.setFormFactor(svrTemplate.formFactor, false);
                                    this.generalPage.setTemplateName(svrTemplate.name);
                                    this.generalPage.setTemplateDescription(this.decodeSpecialCharacters(svrTemplate.description));

                                    // Storage Page specific data
                                    this.copyMode = true;
                                    this.storagePage.pageParams.storageID = svrTemplate.storageSettings.storageSelection;
                                    this.storagePage.pageParams.storageAttrs = svrTemplate.storageSettings.storageAttr;
                                    this.storagePage._loadTemplateData();
                                    this.storagePage._specifyStorageConfigSettings;
                                    this.storagePage.pageParams.baseServerChanged = true;
                                    this.storagePage.pageParams.baseServer = baseServer;

                                    // IOAdapters Page specific data
                                    this.ioAdaptersPage.pageParams.adapterSettings = svrTemplate.adapterSettings;
                                    this.ioAdaptersPage.reloadData(); // force reload of new
                                                                    // adapterSettings!

                                    // Boot Page specific data
                                    this.bootPage.pageParams.bootOrderTable = svrTemplate.bootSettings.bootOrderTable;
                                    this.bootPage.pageParams.bootWoLTable = svrTemplate.bootSettings.bootWoLTable;
                                    this.bootPage.pageParams.learnedBootOptions = svrTemplate.bootSettings.learnedBootOptions;
                                    this.bootPage.pageParams.learnedWoLBootOptions = svrTemplate.bootSettings.learnedWoLBootOptions;
                                    this.bootPage.pageParams.bootOrder = svrTemplate.bootSettings.bootOrder;
                                    this.bootPage.reloadData();                                
                                
                                
                                    // FW Page specific data
                                    this.fwSettingsPage.pageParams.templates = svrTemplate.templates;
                                    this.fwSettingsPage.refreshAllSelectors();

                                    // IMPORTANT: Following must be done AFTER ioAdapterPage gets its
                                    // adapterSettings!
                                    this._prepareForIoBootDevices();
                                }
                            }),
                            error: dojo.hitch(this, function(error, ioArgs) {
                                console.log(this.declaredClass + ": Get template complete: ", error, ioArgs);

                                var response = dojo.fromJson(ioArgs.xhr.responseText);
                                if(response.status >= 500) {
                                    // Internal error. Call support.
                                    this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                                }
                                else {
                                    // Error occurred. Try again or call support.
                                    this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                                }
                            }),
                            handle: dojo.hitch(this, function(response, ioArgs) {
                                // Inform progress dialog...
                                if (ioArgs.xhr.status >= 300) {
                                    dojo.publish("ProgressFailed", response);
                                }
                                else {
                                    dojo.publish("ProgressComplete", response);

                                    // Call onSuccess callback function, if any.
                                    if (dojo.isFunction(onSuccessCallback)) {
                                        onSuccessCallback();
                                    }
                                }
                            })
                        });
                    } else {
                        dojo.publish("ProgressFailed", response);
                        this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                    }
                }),
                error: dojo.hitch(this, function(error, ioArgs) {
                    console.log(this.declaredClass + ": Generate learned template complete: ", error, ioArgs);

                    var response = dojo.fromJson(ioArgs.xhr.responseText);
                    if (response.message == "service_error_name_in_use") {
                        // Error, pattern name in use
                        this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                    }
                    else if(response.message == "service_error_no_server_settings") {
                        // Error retrieving data from the server
                        this.getModalDialog("Error", this.resources.ERROR_SERVER_LEARN).show();
                    }
                    else if(response.status >= 500) {
                        // Internal error. Call support.
                        this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                    }
                    else {
                        // Error occurred. Try again or call support.
                        this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                    }
                }),
                handle: dojo.hitch(this, function(response, ioArgs) {
                    // Inform progress dialog...
                    if (ioArgs.xhr.status >= 300) {
                        dojo.publish("ProgressFailed", response);
                    }
                })
            });

        },

        _prepareForIoAddressingMode: function(mode) {
            // tags:
            // protected
            console.log(this.declaredClass + ": _prepareForIoAddressingMode called: ", mode);
            this.bootPage.pageParams.ioAddressingMode = mode;
            var ioBootDevices = this.ioAdaptersPage.getIoBootDevices();
            this.bootPage.pageParams.ioBootDevices = ioBootDevices;
            this.bootPage.processBootUpdates();
        },

        _prepareForIoBootDevices: function() {
            // tags:
            // protected
            console.log(this.declaredClass + ": _prepareForIoBootDevices called");
            var ioBootDevices = this.ioAdaptersPage.getIoBootDevices();
            this.bootPage.pageParams.ioBootDevices = ioBootDevices;
            this.bootPage.processBootUpdates();
        },

        setExisting: function() {
            this.createFromExisting = true;
        },

        _generateTemplateData: function() {
            // tags:
            // protected
            // Generate Server Template object

            var serverTemplate = new Object();

            // Append Standard information...
            if (this.editMode || this.preGeneratedTemplate) {
                serverTemplate.id = this.templateIDNum;
            }
            serverTemplate.type = this.templateType;
            serverTemplate.userDefined = true;

            // Append General information...
            serverTemplate.nodeType = this.generalPage._nodeType.get("value").trim();
            serverTemplate.formFactor = this.generalPage._formFactor.get("value").trim();
            serverTemplate.name = this.generalPage._templateName.get("value").trim();

            var description = this.generalPage._templateDescription.get("value").trim();
            serverTemplate.description = this.encodeSpecialCharacters(description);

            // Append Storage information...
            var storageObject = new Object();
            if(serverTemplate.formFactor != "1sd"){
                storageObject.storageSelection = this.storagePage.curStorageID;
                
                if (this.storagePage.curStorageID == "Specify") {
                    storageObject.storageAttr = {
                        "numDrives": this.storagePage.numDrivesValue,
                        "diskType": this.storagePage.diskTypeValue,
                        "raidLevel": this.storagePage.raidLevelValue
                    };
                }
                else if (this.storagePage.curStorageID == "Clone") {
                    storageObject.storageAttr = {
                        "storageVolumes": this.storagePage.capturedGrid.getTreeAsJson()
                    };
                }
                else {
                    storageObject.storageAttr = {};
                }
            }
            serverTemplate.storageSettings = storageObject;

            // Append IO Adapter information...
            var adapterSettings = this.ioAdaptersPage.getAddapterSettings();
            serverTemplate.adapterSettings = adapterSettings;

            // Append Boot information...
            var bootObject = this.bootPage.getBootJsonObject();
            serverTemplate.bootSettings = bootObject;

            // Append FW Settings information...
            var templateObject = this.fwSettingsPage.getFWSettings();
            serverTemplate.templates = templateObject;

            return serverTemplate;
        },

        onSave: function() {
            // Function connected to the "Save" button

            var serverPatternChanged = false;
            
            if (this.editMode && this.templateInUse) {
                serverPatternChanged = this._didServerPatternSettingsChange();
            }
            
            if (this.editMode && this.templateInUse && serverPatternChanged) {                                
                // If template is in use, trigger redeploy dialog
                var svrTemplate = this._generateTemplateData();
                var templateData = dojo.toJson({
                    "identifier": "name",
                    "label": "name",
                    "items": [ svrTemplate ]
                });
                require([ "support/dialogs/ReDeployServerTemplate" ], dojo.hitch(this, function() {
                    var rdstDialog = new support.dialogs.ReDeployServerTemplate({
                        "templateID": this.templateIDNum,
                        "templateType": this.templateType,
                        "templateData": templateData
                    });
                    //Close wizard if redeploy successful
                    dojo.connect(rdstDialog,"onClose",this,function(){
                        if(!rdstDialog){
                            return;
                        }
                        var deployClicked = rdstDialog.get("closeParent");
                        if(deployClicked) {
                            this.hide();
                        }
                    });
                    rdstDialog.show();
                }));
            }
            else {
                // Not in use, just save...

                // Set up subscriber for _saveTemplateAsync() below.
                var handle = dojo.subscribe("ServerTemplateComplete", this, function(response) {
                    if ((response.status >= 200) && (response.status < 300)) {
                        this.hide();
                    }
                    else {
                        console.error(this.declaredClass + ": Error Saving Server Template");
                        
                        if ((response.status >= 400) && (response.response.message == "service_error_name_in_use")) {
                            // Error, pattern name in use
                            this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                            //this.wizard.jumpBackToPage(this.generalPage);
                        }
                        else if(response.status == 404) {
                            // Error, pattern does not exist. Use Save As.
                            this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                        }
                        else if(response.status >= 500) {
                            // Internal error. Call support.
                            this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                        }
                        else {
                            // Error occurred. Try again or call support.
                            this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                        }

                    }
                    handle && dojo.unsubscribe(handle);
                });
                this._saveTemplateAsync();
            }

        },
        
        _getServerPatternFromServices: function() {
            var svrTemplate = null;           
            // Load in template data
            dojo.xhrGet({
                url: this.REST_PATH + "template/" + this.templateIDNum,
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                sync: true,
                preventCache: true,
                load: function(response, ioArgs) {
                    if (response.items && (response.items.length > 0)) {
                        svrTemplate = response.items[0];
                    }
                },
                handle: function(response, ioArgs) {
                    console.log(this.declaredClass + ": Load template complete: ", response, ioArgs);
                }
            });

            return svrTemplate;        
        },
        
        _didServerPatternSettingsChange: function() {            
            this._originalServerPattern = this._getServerPatternFromServices();
            if (this._originalServerPattern != null) {                
                if (this._didStorageSettingsChange()) {
                    return true;
                }
                if (this._didAdapterSettingsChange()) {
                    return true;
                }
                if (this._didBootSettingsChange()) {
                    return true;
                }
                if (this._didFwSettingsChange()) {
                    return true;
                }
            } else {
                // If this._originalServerPattern is null force the redeploy
                return true;
            }
            return false;
        },
        
        _didStorageSettingsChange: function() {
            // Check if storage setting has changed.
            if (this._originalServerPattern.storageSettings.storageSelection != this.storagePage.curStorageID) {
                return true;
            }            
            return false;
        },
        
        _didAdapterSettingsChange: function() {
            // Check if any IO adapter settings have changed
            if (this.ioAdaptersPage.adapterPageChanged) {
                return true;
            }
            var adapterSettings = this.ioAdaptersPage.getAddapterSettings();
            // An adapter was either added or removed
            if (adapterSettings.adapters.length != this._originalServerPattern.adapterSettings.adapters.length) {
                return true;
            }
            return false;
        },
        
        _didBootSettingsChange: function() {
            // Check if any boot settings have changed
            var bootSettings = this.bootPage.getBootJsonObject();
            
            if (this._originalServerPattern.bootSettings.bootOrder != bootSettings.bootOrder) {
                return true;
            }
            
            // A row was added/removed
            if (this._originalServerPattern.bootSettings.bootOrderTable.length != bootSettings.bootOrderTable.length) {
                return true;
            }
            
            if (this._originalServerPattern.bootSettings.bootWoLTable.length != bootSettings.bootWoLTable.length) {
                return true;
            }

            if (this._originalServerPattern.bootSettings.learnedBootOptions != bootSettings.learnedBootOptions) {
                return true;
            }
            
            if (this._originalServerPattern.bootSettings.learnedWoLBootOptions != bootSettings.learnedWoLBootOptions) {
                return true;
            }
            
            if (this._didBootTableChange(this._originalServerPattern.bootSettings.bootOrderTable, bootSettings.bootOrderTable)) {
                return true;
            }
            
            if (this._didBootTableChange(this._originalServerPattern.bootSettings.bootWoLTable, bootSettings.bootWoLTable)) {
                return true;
            }
            return false;
        },
        
        _didBootTableChange: function(originalBootTable, currentBootSettings) {
            // Check the boot order table
            var bootOrderChange = false;
            dojo.some(originalBootTable, function(bootProperties, bootIdx) {
                if (bootProperties.order != currentBootSettings[bootIdx].order) {
                    bootOrderChange = true;
                }
                if (bootProperties.option != currentBootSettings[bootIdx].option) {
                    bootOrderChange = true;
                    return true;
                }
                if (bootProperties.device != currentBootSettings[bootIdx].device) {
                    if (bootProperties.device[0] == null && currentBootSettings[bootIdx].device == "*NONE") {
                        bootOrderChange = false;
                    } else {
                        bootOrderChange = true;
                        return true;
                    }
                }
                if (bootProperties.devicePort != currentBootSettings[bootIdx].devicePort) {
                    bootOrderChange = true;
                    return true;
                }
                if (bootProperties.deviceSlot != currentBootSettings[bootIdx].deviceSlot) {
                    bootOrderChange = true;
                    return true;
                }
                if (bootProperties.deviceType != currentBootSettings[bootIdx].deviceType) {
                    bootOrderChange = true;
                    return true;
                }
                if (bootProperties.functionNumber != currentBootSettings[bootIdx].functionNumber) {
                    bootOrderChange = true;
                    return true;
                }
                if (bootProperties.target != currentBootSettings[bootIdx].target) {
                    bootOrderChange = true;
                    return true;
                }
                if (bootProperties.template != currentBootSettings[bootIdx].template) {
                    if (bootProperties.template[0] == null && currentBootSettings[bootIdx].template == "*NONE") {
                        bootOrderChange = false;
                    } else {
                        bootOrderChange = true;
                        return true;
                    }
                } 
            });
            
            if (bootOrderChange) {
                return true;
            }
            return false;           
        },
        
        _didFwSettingsChange: function() {
            //  Check if any firmware patterns have changed                
            var fwSettingsVals = this.fwSettingsPage.getFWSettings();
            
            // Create an object with the original category pattern ids          
            var originalFwSettings = new Object({
                systemInfoTemplate: -1,
                managementTemplate: -1,
                deviceTemplate: -1,
                performanceTemplate: -1,
                powerTemplate: -1,
                extendedUEFI: -1, 
                extendedIMM: -1
            });
            
            // Set the pattern ids
            for (var counter=0;counter < this._originalServerPattern.templates.length;++counter) {
                switch (this._originalServerPattern.templates[counter].type) {
                    case "SystemInfo":
                        originalFwSettings.systemInfoTemplate = this._originalServerPattern.templates[counter].id;
                        continue;
                    case "Management":
                        originalFwSettings.managementTemplate = this._originalServerPattern.templates[counter].id;
                        continue;
                    case "DevicesAndIOPorts":
                        originalFwSettings.deviceTemplate = this._originalServerPattern.templates[counter].id;
                        continue;
                    case "PerformanceRecovery":
                        originalFwSettings.performanceTemplate = this._originalServerPattern.templates[counter].id;
                        continue;
                    case "Power":
                        originalFwSettings.powerTemplate = this._originalServerPattern.templates[counter].id;
                        continue;
                    case "ExtendedUEFI":
                        originalFwSettings.extendedUEFI = this._originalServerPattern.templates[counter].id;
                        continue;
                    case "ExtendedIMM":
                        originalFwSettings.extendedIMM = this._originalServerPattern.templates[counter].id;
                        continue;
                    default:
                        return true;
                }                
            }
            
            // Check if any category patterns have changed
            for (var counter=0;counter < fwSettingsVals.length;++counter) {
                switch (fwSettingsVals[counter].type) {
                    case "SystemInfo":
                        if(fwSettingsVals[counter].id == null){fwSettingsVals[counter].id = -1;}
                        if (originalFwSettings.systemInfoTemplate != fwSettingsVals[counter].id) {
                            return true;
                        }
                        continue;
                    case "Management":
                        if(fwSettingsVals[counter].id == null){fwSettingsVals[counter].id = -1;}
                        if (originalFwSettings.managementTemplate != fwSettingsVals[counter].id) {
                            return true;
                        }
                        continue;
                    case "DevicesAndIOPorts":
                        if(fwSettingsVals[counter].id == null){fwSettingsVals[counter].id = -1;}
                        if (originalFwSettings.deviceTemplate != fwSettingsVals[counter].id) {
                            return true;
                        }
                        continue;
                    case "PerformanceRecovery":
                        if(fwSettingsVals[counter].id == null){fwSettingsVals[counter].id = -1;}
                        if (originalFwSettings.performanceTemplate != fwSettingsVals[counter].id) {
                            return true;
                        }
                        continue;
                    case "Power":
                        if(fwSettingsVals[counter].id == null){fwSettingsVals[counter].id = -1;}
                        if (originalFwSettings.powerTemplate != fwSettingsVals[counter].id) {
                            return true;
                        }
                        continue;
                    case "ExtendedUEFI":
                        if(fwSettingsVals[counter].id == null){fwSettingsVals[counter].id = -1;}
                        if (originalFwSettings.extendedUEFI != fwSettingsVals[counter].id) {
                            return true;
                        }
                        continue;
                    case "ExtendedIMM":
                        if(fwSettingsVals[counter].id == null){fwSettingsVals[counter].id = -1;}
                        if (originalFwSettings.extendedIMM != fwSettingsVals[counter].id) {
                            return true;
                        }
                        continue;
                    default:
                        return true;
                }                

            }
            return false;
        },
                
        _saveTemplateAsync: function(data) {
            // tags:
            // protected
            // Save Server template data using an asynchronous operation.
            // Subscribe to "ServerTemplateComplete" to receive results.

            var svrTemplate = (data) ? data : this._generateTemplateData();
            var postData = dojo.toJson({
                "identifier": "name",
                "label": "name",
                "items": [ svrTemplate ]
            });
            this.getProgressDialog("save").show(); // Show progress dialog for save request.
            dojo.xhrPost({
                url: this.REST_PATH + "template/create",
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                postData: postData,
                load: function(response, ioArgs) {
                    var templateId = response.id;
                    dojo.publish("ServerTemplateComplete", {
                        status: ioArgs.xhr.status,
                        templateId: templateId,
                        response: response
                    }); // Inform caller(s) of success!
                },
                error: function(error, ioArgs) {
                    dojo.publish("ServerTemplateComplete", {
                        status: ioArgs.xhr.status,
                        templateId: null,
                        response: dojo.fromJson(ioArgs.xhr.responseText)
                    }); // Inform caller(s) of failure!
                },
                handle: function(response, ioArgs) {
                    console.log(this.declaredClass + ": Save template complete: ", response, ioArgs);
                    if (ioArgs.xhr.status >= 300) {
                        dojo.publish("ProgressFailed", response);
                    }
                    else {
                        dojo.publish("ProgressComplete", response);
                    }
                }
            });
            // IMPORTANT! No return. Subscribe to "ServerTemplateComplete" instead.

        }

    });

});