define([
        "dojo",
        "dijit",
        "support/dialogs/Dialog",
        "dojo/i18n!./nls/StandbyPoolTemplate",
        "dojo/text!./templates/StandbyPoolTemplate.html",
        "dojo/parser",
        "dijit/form/ValidationTextBox",
        "dijit/form/SimpleTextarea",
        "dijit/form/Button",
        "dijit/layout/BorderContainer",
        "dijit/layout/ContentPane",
        "dijit/Tooltip",
        "support/panels/DualTableSelector",
        "support/temp/ModalDialog",
        "support/base/TableMixin"
    ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.StandbyPoolTemplate", [ support.dialogs.Dialog, support.base.TableMixin ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        baseClass: "StandbyPoolTemplate",

        _availTable: null,
        _selectTable: null,
        
        _selectedStore: null,

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/StandbyPoolTemplate.css",

        // Internal Selector Names
        POLTMPL_NAME: "policyTemplate",
        STBYPOOL_NAME: "standbyPool",

        // Type Names
        STDBY_POOL: "standby",

        SELECTOR_WIDTH: "300px",
        launchPoint: "", // Dialog Launch from this point
        templateIDNum: null,
        
        // In use indicator
        inUse: false,
        
        dependentFormFactors: "",
        msgDescriptionFF: "",
        msgTitleFF: "",
        
        // ID prefixes used for tables
        availGridID: "AvailableStandbyTable",
        selectGridID: "SelectedStandbyTable",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);

            // Set Dialog Title
            var title = (this.templateIDNum == null) ? this.locResources.PAGE_TITLE : this.locResources.EDIT_TITLE;
            this.set("title", title);

            // Set Template Field to catch intermediate changes
            this.templateName.intermediateChanges = true;

            // Set Description Label
            this._descLimit.innerHTML = this.resources.LIMIT.replace("%500", "500");

            // Apply env specific tooltip for the page
            // NOTE: For dijit/Tooltip dijits, set the label, not innerHTML.
            this._settingstip.label = "<div style=\"width: 30em;\">" + this.locResources.TT1 + "<br><br>" + this.locResources.TT2.replace("%1", this.PRODUCT_NAME_LONG) + "</div>";

            // Setup Tables
            this._setupTables();

            if (this.templateIDNum == null) {
                this.okBtnContainer.style.display = "inline";
            }
            else {
                this.saveBtnContainer.style.display = "inline";
            }
            
            dojo.connect(this.dialogNode, "hide", this, function() {
                this.destroy();
            });

        },

        onFinish: function() {
            // Name and desc
            var success = false;
            var newStandbyPool = new Object();
            var name = this.templateName.get("value");
            var desc = this._templateDescription.get("value");
            newStandbyPool.name = name;
            newStandbyPool.description = desc;
            // Type and ID
            newStandbyPool.type = this.STDBY_POOL;
            if (this.templateIDNum != null) {
                newStandbyPool.id = this.templateIDNum;
            }
            // Grab selected endpoints
            var endPointIds = [];
            for (var i = 0; i < this._selectTable.store._arrayOfAllItems.length; i++) {
                var item = this._selectTable.store._arrayOfAllItems[i];
                if (item == null) continue;
                else (item && item.type && item.type == "imm" || item.type && item.type == "bay") && endPointIds.push(item.id[0]);
            }
            
            newStandbyPool.endpointIds = endPointIds;
            var argJSONString = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": [ newStandbyPool ]
            }, true);

            // Post ReDeploy Data To Backend
            dojo.xhrPost({
                url: this.REST_PATH + "pool/standby/create",
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                postData: argJSONString,
                sync: true,
                load: function(data) {
                    success = true;
                    dojo.publish("StandbyPoolComplete", data);
                },
                error: dojo.hitch(this, function(error, ioArgs) {
                  
                    var response = dojo.fromJson(ioArgs.xhr.responseText);
                    
                    if ((ioArgs.xhr.status >= 400 ) && (response && (response.message == "service_error_name_in_use"))) {
                        // Error Template name in use
                        this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                    }                        
                    else if (ioArgs.xhr.status == 404) {
                        // Edit Template ID doesn't exist
                        this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                    }
                    else if (ioArgs.xhr.status >= 500) {
                        // Internal Programming Error
                        this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                    }
                    else {
                        this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                    }
                })
            });

            if (this.launchPoint == "details") {
                dojo.publish("RefreshNameDiv", name);
            }

            return success;
        },

        _cancelClicked: function() {
            this.hide();
            dojo.publish("CloseDialog");
        },

        _setupTables: function() {
            this._setupAvailableTable();
            this._setupSelectedTable();
            this._dualTableSelector = new support.panels.DualTableSelector({
                id: "cp_standbypooltemplate_dualtableselector",
                _availTable: this._availTable,
                _selectTable: this._selectTable
            }).placeAt(this._dualTableSelectorNode);

            dojo.connect(this._dualTableSelector, "_moveRight", this, this._validate);
            dojo.connect(this._dualTableSelector, "_moveAllRight", this, this._validate);
            dojo.connect(this._dualTableSelector, "_moveLeft", this, this._validate);
            dojo.connect(this._dualTableSelector, "_moveAllLeft", this, this._validate);
        },

        _setupAvailableTable: function() {

            // summary:
            // Set up available targets table widget...
            // tags:
            // protected
            if (this._availTable) {
                // Destroy old table and start from scratch.
                this._availTable.destroy();
            }

            var tableStore = this._retrieveStore();
            var tableStructure = this._getTableStructure();
            this._availTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.availGridID, true, true);
            this._availTable.column(0).sort(false);
            
        },
        
        _retrieveStore: function() {
            var selectedIds = [];
            var selectedStore = null;
            var selectedList = new Array();
            
            // fill selected table if standby server pool is in edit mode
            if (this.templateIDNum) {
                var store = null;
                dojo.xhrGet({
                    url: this.REST_PATH + "pool/standby/" + this.templateIDNum,
                    handleAs: "json",
                    sync: true,
                    preventCache: true,
                    load: dojo.hitch(this, function(response) {
                        store = response;
                    })
                });
                if (store) {
                    this.templateName.set("value", (store.name) ? store.name : "");
                    this._templateDescription.set("value", (store.description) ? store.description : "");
                    selectedIds = (store.endpointIds) ? store.endpointIds : [];
                    this.inUse = store.inUse;
                }
                
                var deployList = null;
                var selectedData = null;
                                
                if (selectedIds.length > 0) {
                   
                    dojo.xhrGet({
                        url: this.REST_PATH + "learn/list",
                        handleAs: "json",
                        sync: true,
                        preventCache: true,
                        load: dojo.hitch(this, function(response) {
                            deployList = response;
                        })
                        
                    });
                                       
                    // build selected list for tree model from deploy list and 
                    // selected ids APIs respectively
                    if (deployList && deployList.items) {
                        for (var idx = 0; idx < deployList.items.length; idx++) {
                            var selectParent = null;
                            var rootObject = deployList.items[idx];
                            if (deployList.items[idx].children) {
                                // iterate through ITEs to remove non selected ites from standby server pool
                                for (var idx2 = 0; idx2 < deployList.items[idx].children.length; ++idx2) {
                                    // If one bay has more then one compute node then loop through compute nodes (Caracara)
                                    if(rootObject.children[idx2].ites.length > 1){
                                        for (var idx3 = 0; idx3 < deployList.items[idx].children[idx2].ites.length; idx3++) {
                                            var childId = deployList.items[idx].children[idx2].ites[idx3].id;
                                            var found = false;
                                            
                                            // compare current ITE from deploy list against selected ites in standby server pool
                                            for (var idx4 = 0; idx4 < selectedIds.length; idx4++) {
                                                if (childId == selectedIds[idx4]) {
                                                    found = true;
                                                }
                                            }
                                            // if we find the selected ITE we add it to the newly created list
                                            if (found) {
                                                var ff = rootObject.children[idx2].ites[idx3].formFactor;
                                                
                                                //  Set bay label with subnode indication if 1sd or 2-bay indication if 4s or pxu.
                                                if (ff == "1sd")
                                                    rootObject.children[idx2].ites[idx3].bays[0] = rootObject.children[idx2].ites[idx3].bays[0] + ":" + rootObject.children[idx2].ites[idx3].subBays[0];                                                                                    
                                                else if (ff == "4s" || ff == "pxu")
                                                    rootObject.children[idx2].ites[idx3].bays[0] = rootObject.children[idx2].ites[idx3].bays[0] + ", " + rootObject.children[idx2].ites[idx3].bays[1];                                                                                    
                                                
                                                rootObject.children[idx2].ites[idx3].parentId = rootObject.id;
                                                if (selectParent == null) {
                                                    selectParent = this._createParent(rootObject);
                                                }
                                                selectParent.children.push(deployList.items[idx].children[idx2].ites[idx3]);
                                            }
                                            
                                        }
                                        
                                    }
                                    else {
                                        var childId = deployList.items[idx].children[idx2].ites[0].id;
                                        var found = false;
                                        
                                        // compare current ITE from deploy list against selected ites in standby server pool
                                        for (var idx4 = 0; idx4 < selectedIds.length; idx4++) {
                                            if (childId == selectedIds[idx4]) {
                                                found = true;
                                            }
                                        }
                                        // if we find the selected ITE we add it to the newly created list
                                        if (found) {
                                            var ff = rootObject.children[idx2].ites[0].formFactor;
                                            
                                            //  Set bay label with subnode indication if 1sd or 2-bay indication if 4s or pxu.
                                            if (ff == "1sd")
                                                rootObject.children[idx2].ites[0].bays[0] = rootObject.children[idx2].ites[0].bays[0] + ":" + rootObject.children[idx2].ites[0].subBays[0];                                                                                    
                                            else if (ff == "4s" || ff == "pxu")
                                                rootObject.children[idx2].ites[0].bays[0] = rootObject.children[idx2].ites[0].bays[0] + ", " + rootObject.children[idx2].ites[0].bays[1];                                                                                    

                                            rootObject.children[idx2].ites[0].parentId = rootObject.id;
                                            if (selectParent == null) {
                                                selectParent = this._createParent(rootObject);
                                            }
                                            
                                            selectParent.children.push(deployList.items[idx].children[idx2].ites[0]);
                                        }
                                    }                                                              
                                }
                            }
                            if (selectParent != null) {
                                selectedList.push(selectParent);
                            }
                        }
                    }
                   
                }
                
            }
            
            selectedData = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": selectedList
            }, true);
            
            this._selectedStore = new dojo.data.ItemFileWriteStore({
                data: dojo.fromJson(selectedData)
            });
            
            if (!("hasChildren" in this._selectedStore)) {
                this._selectedStore.hasChildren = function(id, item) { 
                    return item && this.getValues(item, "children").length;
                };
            };
            
            if (!("getChildren" in this._selectedStore)) {
                this._selectedStore.getChildren = function(item) { 
                    return this.getValues(item, "children");
                };
            };
            
            // Fill available table
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH
                + "server?inclSrvrsWithProfile=false&numberOfNotMonitoredServers=false",
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: dojo.hitch(this, function(response) {

                    var availArray = new Array();
                    if (response && response.items) {
                        for ( var idx = 0; idx < response.items.length; ++idx) {
                            var availParent = null;
                            var rootObject = response.items[idx];

                            if (rootObject.children) {
                                for ( var idx2 = 0; idx2 < rootObject.children.length; ++idx2) {
                                    if(rootObject.children[idx2].ites.length > 1) {
                                        for (var idx3 = 0; idx3 < rootObject.children[idx2].ites.length; idx3++) {
                                            var ff = rootObject.children[idx2].ites[idx3].formFactor;

                                            //  Set bay label with subnode indication if 1sd or 2-bay indication if 4s or pxu.
                                            if (ff == "1sd")
                                                rootObject.children[idx2].ites[idx3].bays[0] = rootObject.children[idx2].ites[idx3].bays[0] + ":" + rootObject.children[idx2].ites[idx3].subBays[0];                                                                                    
                                            else if (ff == "4s" || ff == "pxu")
                                                rootObject.children[idx2].ites[idx3].bays[0] = rootObject.children[idx2].ites[idx3].bays[0] + ", " + rootObject.children[idx2].ites[idx3].bays[1];                                                                                    

                                            rootObject.children[idx2].ites[idx3].parentId = rootObject.id;

                                            if (availParent == null) {
                                                availParent = this._createParent(rootObject);
                                            }

                                            availParent.children.push(rootObject.children[idx2].ites[idx3]);
                                        }
                                    }
                                    else {
                                        var ff = rootObject.children[idx2].ites[0].formFactor;

                                        //  Set bay label with subnode indication if 1sd or 2-bay indication if 4s or pxu.
                                        if (ff == "1sd")
                                            rootObject.children[idx2].ites[0].bays[0] = rootObject.children[idx2].ites[0].bays[0] + ":" + rootObject.children[idx2].ites[0].subBays[0];                                                                                    
                                        else if (ff == "4s" || ff == "pxu")
                                            rootObject.children[idx2].ites[0].bays[0] = rootObject.children[idx2].ites[0].bays[0] + ", " + rootObject.children[idx2].ites[0].bays[1];                                                                                    

                                        rootObject.children[idx2].ites[0].parentId = rootObject.id;
                                        if (availParent == null) {
                                            availParent = this._createParent(rootObject);
                                        }
                                        availParent.children.push(rootObject.children[idx2].ites[0]);
                                    }
                                }
                            }
                            if (availParent != null) {
                                availArray.push(availParent);
                            }
                        }
                    }
                    var templateData = dojo.toJson({
                        "identifier": "id",
                        "label": "name",
                        "items": availArray
                    }, true);

                    this._availStore = new dojo.data.ItemFileWriteStore({
                        data: dojo.fromJson(templateData)
                    });
                    
                    if (!("hasChildren" in this._availStore)) {
                        this._availStore.hasChildren = function(id, item) { 
                            return item && this.getValues(item, "children").length;
                        };
                    };
                    
                    if (!("getChildren" in this._availStore)) {
                        this._availStore.getChildren = function(item) { 
                            return this.getValues(item, "children");
                        };
                    };

                }),
                error: dojo.hitch(this, function(response) {
                    // if the REST API was unsuccessful in returning data,
                    // show an empty table
                    console.log(this.declaredClass + ": Unable to retrieve server templates from API: deploy/list/");
                    var templateData = dojo.toJson({
                        "identifier": "id",
                        "label": "name",
                        "items": []
                    }, true);
                    this._availStore = new dojo.data.ItemFileWriteStore({
                        data: dojo.fromJson(templateData)
                    });
                    
                    if (!("hasChildren" in this._availStore)) {
                        this._availStore.hasChildren = function(id, item) { 
                            return item && this.getValues(item, "children").length;
                        };
                    };
                    
                    if (!("getChildren" in this._availStore)) {
                        this._availStore.getChildren = function(item) { 
                            return this.getValues(item, "children");
                        };
                    };
                })
            });
            
            return this._availStore;
        },

        // Columns to be displayed on table
        _getTableStructure: function() {
            var i;
            var layout = new Array();
            layout.push({
                field: "name",
                name: this.resources.NAME,
                width: "160px"
            });
            layout.push({
                field: "bays",
                name: this.resources.BAYS,
                width: "45px"
            });
            layout.push({
                field: "access",
                name: this.resources.ACCESS,
                width: "65px",
                decorator: dojo.hitch(this, "_accessDecorator"),
                formatter: dojo.hitch(this, "_accessFormatter")
            });         
            layout.push({
                field: "formFactor",
                name: this.resources.FORM_FACTOR,
                formatter: dojo.hitch(this, "_formatFormFactor"),
                width: "auto"
            });

            return layout;
        },

        _accessFormatter: function(item) {

            // summary:
            // Formats "access" field value for display in TreeTable.
            // tags:
            // protected

            var access = item.access;
            var accessText = "";
            
            if (access == "ok") { // Access "OK"
                accessText = this.resources.ACCESS_OK;
            }
            else if (access == "-Default-") { // Access "Default Access"
                accessText = this.resources.ACCESS_DEFAULT;
            }
            else if (access == "offline") { // Access "Offline"
                accessText = this.resources.ACCESS_OFFLINE;
            }
            else if (access == "placeholder") { // Access "Placeholder"
                accessText = this.resources.PLACEHOLDER;
            }
            else if (access == "none") { // Access "No access"
                accessText = this.resources.ACCESS_NONE;
            }
            else if (access == "partial") { // Access "Partial access"
                accessText = this.resources.ACCESS_PARTIAL;
            }
            else /*(access == "unknown")*/ { // Access "Unknown"
                accessText = this.resources.ACCESS_UNKNOWN;
            }
            
            return accessText;
        },

        _setupSelectedTable: function() {

            // summary:
            // Set up selected targets table widget...
            // tags:
            // protected
            if (this._selectTable) {
                // Destroy old table and start from scratch.
                this._selectTable.destroy();
            }

            var tableStore = this._selectedStore;
            var tableStructure = this._getTableStructure();
            this._selectTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.selectGridID, true, true);
            this._selectTable.column(0).sort(false);

        },

        _nameChanged: function() {
            if (this.templateName.get("value").trim() != "") {
                return true;
            }
            else {
                return false;
            }
        },

        _getNumSelectedServers: function() {
            if (this._selectTable) {
                return this._selectTable.store._arrayOfTopLevelItems.length;
            }
            return -1;
        },

        _validate: function() {
            
            if (this._ffMessageArea) {                
                this._ffMessageArea.destroyRecursive();
                this._ffMessageArea = null;
            }
            
            if (this.templateName.get("value").trim() == ""){
                this._createBtn.set("disabled", true);
                this.saveBtn.set("disabled", true);
                return;
            }
            
            if (this._nameChanged()) {
                if (this._getNumSelectedServers() == 0) {
                    this._createBtn.set("disabled", true);
                    this.saveBtn.set("disabled", true);
                    return;
                }
            }
            
            if (this.inUse) {                
                if (!this._formFactorsValid()) {
                    this._createBtn.set("disabled", true);
                    this.saveBtn.set("disabled", true);
                    this.msgDescriptionFF = this.locResources.FF_MISMATCH_DETAILS.replace("%1", this.dependentFormFactors);
                    this.msgTitleFF = this.locResources.FF_MISMATCH;
                    
                    if(this.CP_RUN_ENVIRONMENT == "fsm"){
                    
                        require([ "idx/oneui/messaging/SingleMessage" ], dojo.hitch(this, function() {

                            this._ffMessageArea = new idx.oneui.messaging.SingleMessage({
                                description: this.msgDescriptionFF,
                                title: this.msgTitleFF,
                                type: "error",
                                dateFormat: { formatLength: "medium", selector: "time" },
                                showId: false,
                                showAction: false,
                                showRefresh: false,
                                showDetailsLink: false,
                                closable: false
                            }).placeAt(this.id + "_ffMessageArea");
                        
                        }));
                    
                    }else if(this.CP_RUN_ENVIRONMENT == "augusta"){
                        
                        require([ "idx/widget/SingleMessage" ], dojo.hitch(this, function() {

                            this._ffMessageArea = new idx.widget.SingleMessage({
                                description: this.msgDescriptionFF,
                                title: this.msgTitleFF,
                                type: "error",
                                dateFormat: { formatLength: "medium", selector: "time" },
                                showId: false,
                                showAction: false,
                                showRefresh: false,
                                showDetailsLink: false,
                                closable: false
                            }).placeAt(this.id + "_ffMessageArea");
                        
                        }));

                        
                    }
                    
                    this._ffMismatchError.style.display = "inline";
                    this._ffMessageArea.resize();
                    return;
                }
            }
            this._createBtn.set("disabled", false);
            this.saveBtn.set("disabled", false);            
        },
        
        _formFactorsValid: function() {
            var poolFormFactor = [];
            var dependentFormFactor = [];
            var selected = this._selectTable.store._arrayOfAllItems;
            var i, j;
            var match = false;
            var store;
            
            this.dependentFormFactors = "";
            
            // Determine current form factor of standby server pool
            for (i = 0; i < selected.length; i++) {
                if (selected[i]) {
                    if (selected[i].formFactor) {
                        poolFormFactor.push(selected[i].formFactor[0]);
                    }
                }
            }
            
            // Determine form factor of dependent servers
            dojo.xhrGet({
                url: this.REST_PATH + "pool/standby/" + this.templateIDNum + "?showDependents=true",
                handleAs: "json",
                sync: true,
                preventCache: true,
                load: dojo.hitch(this, function(response) {
                    store = response;
                })
            });
            
            // Make list of dependent form factors
            for (i = 0; i < store.items.length; i++) {
                dependentFormFactor.push(store.items[i].formFactor);
            }
            this.dependentFormFactors = this._getFormFactors(dependentFormFactor);

            // Make sure form factors match
            for (i = 0; i < dependentFormFactor.length; i++) {
                for (j = 0; j < poolFormFactor.length; j++) {
                    if (dependentFormFactor[i] == poolFormFactor[j]) {
                        match = true;
                        break;
                    } else {
                        match = false;
                    }
                }
                if (match == false) {
                    return match;
                }
            }            
            
            return match;
        },
        
        _getFormFactors: function(formFactors) {
            var num = formFactors.length;
            var str = "";
            var i;
            if (num > 0) {
                for (i = 0; i < num; i++) {
                    if (str != "") {
                        str += ", ";
                    }
                    str += this._formatFormFactor(formFactors[i]);
                }
            }
            return str;
        },

        _formatFormFactor: function(value) {
            if (value.formFactor) { // called from table formatter
                value = value.formFactor; 
            }
            if (value == "2s") {
                return this.locResources["COMPUTE_NODE_OPT"].replace("%1", "1");
            }
            else if (value == "1sd") {
                return this.locResources["COMPUTE_NODE_DUAL_OPT"].replace("%1", "1");
            }
            else if (value == "4s") {
                return this.locResources["COMPUTE_NODE_OPT"].replace("%1", "2");
            }
            else if (value == "pxu") {
                return this.locResources["EXPANSION_NODE_OPT"].replace("%1", "2");
            }

            return null;
        },

        _createParent: function(parentObj) {
            return {
                "access": parentObj.access,
                "type": parentObj.type,
                "name": parentObj.name,
                "id": parentObj.id,
                "children": []
            };
        }
    });
});