define([
        "dojo",
        "dijit",
        "support/dialogs/Dialog",
        "dojo/i18n!./nls/SubrangePopup",
        "dojo/text!./templates/SubrangePopup.html",
        "dijit/form/Button",
        "dijit/form/ValidationTextBox",
        "dijit/Tooltip",
        "dijit/form/RadioButton",
        "dijit/form/FilteringSelect",
        "dijit/layout/ContentPane",
        "dijit/layout/BorderContainer",
        "support/widgets/ProgressButton"
    ], function(dojo, dijit, dialog, mri, template) {

    return dojo.declare("support.dialogs.SubrangePopup", [ support.dialogs.Dialog ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,
        
        widgetsInTemplate: true,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass : "SubrangePopup",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/SubrangePopup.css",
        
        //mode: "add" or "edit"
        mode: null,
        
        //from NewIPPool dialog
        network: null,
        gateway: null,
        version: null,      //valid options: IPv4 | IPv6
        dropDown: null,     //subnet mask or routing prefix length
        _invalidMessage: "This value is invalid",
        _regExp: ".*",
        _placeHolder: null,
        subrangeStore: null,
        overallStore: null,
        JAVASCRIPT_MAX_INT: 9007199254740992, // Javascript's Maximum Exact Integer value, 2^53 == 9007199254740992
        JAVA_MAX_INT: 2147483647, // Java's Maximum Signed Integer value, 2^31-1 == 2147483647
        
        //initial properties
        range: null,
        chassisId: null,
        first: null,
        size: null,
        last: null,
        index: null,        //if in edit mode
        
        type: null,         //valid options: single | custom 
        
        _rangeDropDown: null,
        _rangeTextBox: null,
        _rangeFirst: null,
        _rangeSize: null,
        _rangeLast: null,
        
        IPV4_VALUE: "IPv4",
        IPV6_VALUE: "IPv6",
        
        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in
        // the template HTML file
        postMixInProperties: function() {

            this.inherited(arguments);

        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be
        // directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            
            (this.mode == null) && (this.mode = "add");

            this.showInputControls();
            this.setupInitialView();
            
            dojo.connect(this._cancelBtn, "onMouseOver", this, function() {
                // Hide the tooltips
                dijit.hideTooltip(this._rangeFirst.domNode);
                dijit.hideTooltip(this._rangeSize.domNode);
            });

            dojo.connect(this._rangeFirst, "onMouseOut", this, function() {
                // Hide the tooltips
                dijit.hideTooltip(this._rangeFirst.domNode);
            });

            dojo.connect(this._rangeSize, "onMouseOut", this, function() {
                // Hide the tooltips
                dijit.hideTooltip(this._rangeSize.domNode);
            });

            dojo.connect(this.dialogNode, "hide", this, function() {
                this.destroy();
            });

        },
        
        //Creates and places input controls in the popup
        showInputControls : function(){
            var popup = this;
            
            //range
            this._rangeDropDown = new dijit.form.FilteringSelect({
                required: true,
                //style: "width:98%",
                store: new dojo.store.Memory({ data: this._getAvailableRangeNameOptions() }),
                labelAttr: "name",
                searchAttr: "name",               
                onChange: dojo.hitch(this, function(newValue){
                    if (newValue != "") {
                        this.set("range", this._rangeDropDown.get("displayedValue"));                        
                        this.set("chassisId", newValue);
                    }
                }),
                _destroyOnRemove: true
              }).placeAt(this._rangeDropDownDiv);
            
            this._rangeTextBox = new dijit.form.ValidationTextBox({
                id: "SubrangeRangeNameDropDown_" + this.id,
                required: true,
                trim: true,
                value: popup.range,
                placeHolder: popup.locResources.RANGE_NAME,
                //style: "width:98%",
                intermediateChanges: true,
                onChange: dojo.hitch(this, function(newValue){
                    this.set("range", newValue);
                }),
                validator : function(value, constraints){
                    var usedNames = popup._getUsedRangeNames();
                    if(popup.containsWithIndex(value, usedNames, popup.index)){
                        this.invalidMessage = popup.locResources.INVALID_MESSAGE_UNIQUE_NAME + 
                            "<br/>" + popup.locResources["INVALID_MESSAGE_NAME_IN_USE"].replace("%1", value);
                        return false;
                    }
                    if (value == "") {
                        return false;
                    }
                    return true;
                }
            }).placeAt(this._rangeTextBoxDiv);
                        
            //first            
            this._rangeFirst = new dijit.form.ValidationTextBox({
                id: "SubrangeFirstDropDown_" + this.id,
                required: true,
                trim: true,
                value: this.first,
                placeHolder: this._placeHolder,
                invalidMessage: this._invalidMessage,
                //style: "width:98%",
                regExp: this._regExp,
                intermediateChanges: true,
                tooltipPosition: ["above"],
                onChange: dojo.hitch(this, function(newValue){
                    this.set("first", newValue);
                    this._rangeSize.set("value", this._rangeSize.get("value"));
                }),
                validator : function(value, constraints){
                    if(!popup._isValidAddress(value)){
                        this.invalidMessage = popup._invalidMessage;
                        return false;
                    }
                    else if(popup._matchesGateway(value)){
                        this.invalidMessage = popup.locResources.INVALID_MESSAGE_GATEWAY;
                        return false;
                    }
                    else if(popup._sameAsNetworkID(value)){
                        this.invalidMessage = popup.locResources.INVALID_MESSAGE_NETWORK;
                        return false;
                    }
                    else if(!popup._matchesNetworkID(value)){
                        this.invalidMessage = popup.locResources.INVALID_MESSAGE_NETWORK_ID;
                        return false;
                    }
                    else if(popup._isGatewayInBounds(value, popup.last)){
                        var gateway = popup.gateway;
                        this.invalidMessage = popup.locResources["INVALID_MESSAGE_SUBRANGE"].replace("%1", gateway);
                        return false;
                    }
                    else if(popup._isOverlapping(value)){
                        this.invalidMessage = popup.locResources.INVALID_MESSAGE_OVERLAPPING;
                        return false;
                    }
                    else{
                        var lowest = null, highest = null, net = null;
                        if(popup.overallStore != null){
                            var items = popup.overallStore._arrayOfTopLevelItems;
                            lowest = items[0].lowest[0];
                            highest = items[0].highest[0];
                        }
                        if( (lowest == "0.0.0.0" && highest == "0.0.0.0") ||
                            (lowest == "0::0" && highest == "0::0") ){
                            net = popup.calculateNetworkID(value);
                            var broad = popup.calculateBroadcastAddress(net);
                            lowest = popup.calculateLowestAddress(net);
                            highest = popup.calculateHighestAddress(net, broad);
                        }
                        if(lowest != null && highest != null){
                            this.invalidMessage = popup.locResources["INVALID_MESSAGE_BETWEEN"].replace("%1", lowest).replace("%2", highest);
                            return popup._isInNetworkBounds(value, lowest, highest);
                        }
                    }
                    return true;
                }
            }).placeAt(this._firstDiv);
            
            dojo.connect(this._rangeFirst, "onMouseOver", this, function() {
               if(!this._rangeFirst.isValid()) {
                   dijit.showTooltip(this._rangeFirst.get("invalidMessage"), this._rangeFirst.domNode, this._rangeFirst.get("tooltipPosition"));
               } else {
                   dijit.hideTooltip(this._rangeFirst.domNode);
               }
            });
            
            //size
            this._rangeSize = new dijit.form.ValidationTextBox({
                id: "SubrangeSizeDropDown_" + this.id,
                trim: true,
                required: true,
                value: 1,   // Initialize to minimal size to avoid immediate display of error. ie: overlapping gateway, etc.
                maxLength: (""+this.JAVA_MAX_INT).length, // Number of digits in this.JAVA_MAX_INT
                placeHolder: "0",
                invalidMessage: this.locResources.INVALID_MESSAGE_SIZE,
                //style: "width:98%",
                intermediateChanges: true,
                tooltipPosition: ["above"],
                validator: function(value, constraints) {

                    if (popup.version == popup.resources.IPV6) {
                        if (isNaN(value) || (parseInt(value) <= 0)) {
                            this.invalidMessage = popup.locResources.INVALID_MESSAGE_SIZE;
                            return false;
                        }

                        // Precision is lost on numbers larger than 2^53 (Javascript's "MAX_INT" value), so the
                        // following routine is used to determine if user input exceeds that value.
                        // NOTE: We simply cannot support numbers larger than 2^53 even though IPv6 can.
                        // Defect XB250972 Note:
                        //   Due to current limitations in backend implementation of com.ibm.stg.sysx.profile.IPRange objectclass
                        //   we must enforce a maximum size limit based on Java's Integer.MAX_VALUE of 2^32-1, rather than
                        //   Javascript's maximum value of 2^52. Therefore the JAVA_MAX_INT constant is used below, as opposed to
                        //   JAVASCRIPT_MAX_INT constant. If/When com.ibm.stg.sysx.profile.IPRange objectclass is changed to
                        //   use Long object, rather than Integer object, the below may change to use JAVASCRIPT_MAX_INT to provide
                        //   the new (larger) max size limit for IP ranges.
                        if (popup._isGreater(value, ""+popup.JAVA_MAX_INT)) {
                            this.invalidMessage = popup.locResources["INVALID_MESSAGE_SIZE_BETWEEN"].replace("%1", 1).replace("%2", popup.JAVA_MAX_INT);
                            return false;
                        }

                    } else {
                        if(popup._isValidAddress(popup.first)){
                            var availSize = null;
                            if(popup.mode == "add"){
                                availSize = popup.getAvailableSize(popup.first);
                            }
                            else if(popup.mode == "edit"){
                                var nextIndex = parseInt(popup.index)+1;
                                var nextItem = null;
                                if(nextIndex >= popup.subrangeStore._arrayOfTopLevelItems.length){
                                    nextItem = popup.overallStore._arrayOfTopLevelItems[0].highest[0];
                                }
                                else{
                                    nextItem = popup.subrangeStore._arrayOfTopLevelItems[nextIndex].first[0];
                                }
                                availSize = popup._subtractAddresses(popup.first, nextItem) + 1;
                            }
                            if(availSize != null && !isNaN(availSize)){ //available size is a number
                                if(availSize == 0){
                                    this.invalidMessage = popup.locResources.INVALID_MESSAGE_SIZE_NONE;
                                    return false;
                                }
                                else{
                                    if(availSize == 1)
                                        this.invalidMessage = popup.locResources.INVALID_MESSAGE_SIZE_1;
                                    else {                                    
                                        this.invalidMessage = popup.locResources["INVALID_MESSAGE_SIZE_BETWEEN"].replace("%1", "1").replace("%2", availSize);
                                    }
                                    return !isNaN(value) && parseInt(value) > 0 && parseInt(value) <= availSize;
                                }
                            }
                            else{
                                this.invalidMessage = popup.locResources.INVALID_MESSAGE_SIZE;
                                return !isNaN(value) && parseInt(value) > 0;
                            }
                        }
                    }
                    return true;                    
                },
                onChange: dojo.hitch(this, function(newValue){
                    this.set("size", newValue);
                    if(!this._rangeSize.isValid()){
                        dijit.showTooltip(this._rangeSize.get("invalidMessage"), this._rangeSize.domNode, this._rangeSize.get("tooltipPosition"));
                    } else if(!this._rangeFirst.validate()){
                        dijit.showTooltip(this._rangeFirst.get("invalidMessage"), this._rangeFirst.domNode, this._rangeFirst.get("tooltipPosition"));
                    }
                })
            }).placeAt(this._sizeDiv);
            
            dojo.connect(this._rangeSize, "onMouseOver", this, function() {
                if(!this._rangeSize.isValid()) {
                    dijit.showTooltip(this._rangeSize.get("invalidMessage"), this._rangeSize.domNode, this._rangeSize.get("tooltipPosition"));
                } else {
                    dijit.hideTooltip(this._rangeSize.domNode);
                }
             });
            
            //last
            this._rangeLast = new dijit.form.ValidationTextBox({
                id: "SubrangeLastDropDown_" + this.id,
                trim: true,
                required: true,
                disabled: true,
                value: this.last,
                placeHolder: this._placeHolder,
                //style: "width:98%;",
                intermediateChanges: true,
                onChange: dojo.hitch(this, function(newValue){
                    this.set("last", newValue);
                }),
                validator : function(value, constraints){
                    var first = popup.get("first");
                    if(!popup._isValidAddress(value)){
                        this.invalidMessage = popup._invalidMessage;
                        return false;
                    }
                    else if(popup._matchesGateway(value)){
                        this.invalidMessage = popup.locResources.INVALID_MESSAGE_GATEWAY;
                        return false;
                    }
                    else if(popup._sameAsNetworkID(value)){
                        this.invalidMessage = popup.locResources.INVALID_MESSAGE_NETWORK;
                        return false;
                    }
                    else if(!popup._matchesNetworkID(value)){
                        this.invalidMessage = popup.locResources.INVALID_MESSAGE_NETWORK_ID;
                        return false;
                    }
                    else if(popup._isGatewayInBounds(first, value)){
                        var gateway = popup.gateway;
                        this.invalidMessage = popup.locResources["INVALID_MESSAGE_SUBRANGE"].replace("%1", gateway);
                        return false;
                    }
                    else if(popup._isOverlapping(value)){
                        this.invalidMessage = popup.locResources.INVALID_MESSAGE_OVERLAPPING;
                        return false;
                    }
                    return true;
                },
                //show invalid even when textbox is disabled
                validate : function(isFocused){
                    var message = "";
                    var isValid = this.isValid(isFocused);
                    if(isValid){ this._maskValidSubsetError = true; }
                    var isValidSubset = !isValid && isFocused && this._isValidSubset();
                    var isEmpty = this._isEmpty(this.textbox.value);
                    this.state = (isValid || (!this._hasBeenBlurred && isEmpty) || isValidSubset) ? "" : "Error";
                    if(this.state == "Error"){ this._maskValidSubsetError = false; }
                    this._setStateClass();
                    dijit.setWaiState(this.focusNode, "invalid", isValid ? "false" : "true");
                    if(isFocused){
                        if(isEmpty){
                            message = this.getPromptMessage(true);
                        }
                        if(!message && (this.state == "Error" || (isValidSubset && !this._maskValidSubsetError))){
                            message = this.getErrorMessage(true);
                        }
                    }
                    if (!isValid && isFocused) {                        
                        dijit.showTooltip(
                            message, 
                            this.domNode, 
                            this.get("tooltipPosition"),
                            !this.isLeftToRight()
                        );
                    }
                    return isValid;
                }
            }).placeAt(this._lastDiv);
            
            dojo.connect(this._rangeLast, "onMouseOver", this, function() {
                this._rangeLast.validate(true);
            });
            
            dojo.connect(this._rangeLast, "onMouseOut", this, function() {
                dijit.hideTooltip(this._rangeLast.domNode);
            });
        },        
       
        //Sets up the initial view based on "Add" or "Edit" mode
        setupInitialView : function(){
            //setup
            if(this.mode == "edit"){

                this.dialogNode.set("title", this.locResources.PAGE_TITLE_EDIT);

                var rangeName = this.range;
                var chassisIdValue = this.chassisId;
                var dropDownNames = this._getAllRangeNames();
                //if its from the drop down, select single and show name as displayed value
                if(0) {  // 99287 By chassis is not support for this release.  Need to enable next release.  this.contains(this.range, dropDownNames)){
                    this._single.set("checked", true);
                    this.set("type", "single");
                    this._rangeTextBox.set("value", "");                  
                    this._rangeDropDown.set("value", chassisIdValue);
                }
                //if its not, select custom and show name as value
                else{
                    this.chassisId = null;
                    this._custom.set("checked", true);
                    this.set("type", "custom");  
                    this._rangeSize.set("value", this.size);
                    this._rangeTextBox.set("value", rangeName);
                }
                this.set("range", rangeName);
            }
            else {

                this.dialogNode.set("title", this.locResources.PAGE_TITLE_ADD);

                var firstAvail = this.getFirstAvailableAddress();
                var sizeAvail = this.getAvailableSize(firstAvail);
                if(firstAvail != null && sizeAvail != 0){
                    this._rangeFirst.set("value", firstAvail);
                }
                else if( (this.hasSubranges() && firstAvail == null) || sizeAvail == 0){
                    this._errorMessage.setAttribute("style", "display:block");
                    this._errorMessage.innerHTML = "<b>&#42; " + this.locResources.NO_NEW_SUBRANGE + "</b><br/><br/>";
                }
                
                // 99287 By chassis support disabled for this release.  Need to enable next release.
                //var rangeNameList = this._getAvailableRangeNameOptions();              
               
                //if (rangeNameList.length > 0) {
                    if (1) {   // By chassis is not supported in this release...   rangeNameList[0].id == null){ 
                        this._custom.set("checked", true);
                        this.set("type", "custom");
                        this._rangeSize.set("value", 1); // Initialize to minimal size to avoid immediate display of error. ie: overlapping gateway, etc.
                    }
                    else{
                        this._single.set("checked", true);
                        this.set("type", "single");
                        //this._rangeDropDown.set("value", rangeNameList[0].id);
                    }
                //}
                
            }
            
            if( (this.dropDown == null || this.dropDown == "") && this.version == this.IPV4_VALUE){
                this._errorMessage.setAttribute("style", "display:block");
                this._errorMessage.innerHTML = "<b>&#42;" + this.locResources.NO_SUBNET + "</b><br/><br/>";
            }
            else if( (this.dropDown == null || this.dropDown == "") && this.version == this.IPV6_VALUE){
                this._errorMessage.setAttribute("style", "display:block");
                this._errorMessage.innerHTML = "<b>&#42;" + this.locResources.NO_ROUTING + "</b><br/><br/>";
            }
        },
        
        //Calculates the IPv6 netowork given a base address and a prefix
        getIPv6NetworkID : function(base, prefix){
            if(this.version == this.IPV6_VALUE && this._isValidAddress(base) && prefix != null){
                var fullIP = this._getIPv6FullNotation(base);
                if(fullIP != null){
                    var hexArr = fullIP.split(":");
                    var binArr = new Array(8);
                    for(var i=0; i<8; i++){
                        binArr[i] = this.hex2bin(hexArr[i]);
                    }
                    var fullBin = this.getFullBinArray(binArr);
                    fullBin = fullBin.join("");
                    var sub = fullBin.substring(0, prefix);
                    for(var i=prefix; i<128; i++){
                        sub += "0";
                    }
                    var newBinArr = sub.match(/.{1,16}/g);
                    for(var i=0; i<8; i++){
                        newBinArr[i] = this.bin2hex(newBinArr[i]);
                    }
                    return newBinArr.join(":");
                }                
            }
            return null;
        },
        
        //Calculates the network ID (for overall table)
        calculateNetworkID : function(base){
            if(this._isValidAddress(base) && this.dropDown != null){
                if(this.version == this.IPV4_VALUE){
                    var subnet = this.dropDown;
                    var splitSub = subnet.split(".");
                    var networkSplit = base.split(".");
                    var net = new Array(4);
                    for(var i=0; i<splitSub.length; i++){
                        net[i] = splitSub[i] & networkSplit[i];
                    }
                    net = net.join(".");
                    this.set("network", net);
                    return net;
                }
                else if(this.version == this.IPV6_VALUE){
                    var prefix = this.dropDown;
                    var net = this.getIPv6NetworkID(base, prefix).toUpperCase();
                    this.set("network", net);
                    return net;
                }
            }
            this.set("network", null);
            return null;
        },
        
        //Returns an array of existing range names
        _getUsedRangeNames : function(){
            var usedNames = new Array();
            var store = this.subrangeStore;
            if(store != null){
                var items = store._arrayOfTopLevelItems;
                for(var i=0; i<items.length; i++){
                    var item = items[i];
                    var range = store.getValue(item, "range");
                    usedNames.push(range);
                }
            }
            return usedNames;
        },
        
        //Returns an array of all "single" chassis range names as object
        _getAllRangeNameObjects : function(){
            var chassisList = null;
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "chassis",
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: function(response) {
                    chassisList = response;
                }
            });         
            
            if (chassisList.items.length > 0) {
                return(chassisList.items);
            } else {
                return [{name: this.locResources.NO_CHASSIS_AVAILABLE, id: null }]; 
            }
        },
        
        //Returns an array of all "single" chassis range names
        _getAllRangeNames : function(){
            var names = this._getAllRangeNameObjects();
            var allNames = new Array();
            for(var j=0; j<names.length; j++)
                allNames.push(names[j].name);
            return allNames;
        },
        
        //Returns an array of available range names
        _getAvailableRangeNameOptions : function(){
            var allNameObjs = this._getAllRangeNameObjects();
            var allNames = this._getAllRangeNames();
            var usedNames = this._getUsedRangeNames();           
            var availableNames = new Array();
            if(this.range != null){
                if(this.contains(this.range, allNames))
                    availableNames.push({ name:this.range, id: this.chassisId });
            }
            //unused values
            for(var j=0; j<allNameObjs.length; j++){
                var value = allNameObjs[j].name;
                var id = allNameObjs[j].id;
                if(id != null && !this.contains(value, usedNames))
                    availableNames.push({ name:value, id:id });
            }
            
            if (availableNames.length == 0) {
                availableNames.push({name: this.locResources.NO_CHASSIS_AVAILABLE, id: null});
            }
            return availableNames;
        },
        
        //Determines if a value is contained in an array
        contains : function(value, array){
            if(array != null){
                for(var i=0; i<array.length; i++){
                    var arrayValue = array[i];
                    if(arrayValue == value)
                        return true;
                }
            }            
            return false;
        },
        
        //Determines if a value is contained in an array (excluding the value at a certain index)
        containsWithIndex: function(value, array, index){
            if(array != null){
                for(var i=0; i<array.length; i++){
                    var arrayValue = array[i];
                    if(i != index && arrayValue == value)
                        return true;
                }
            }            
            return false;
        },
        
        //Determines if an IP address is valid based on the version
        _isValidAddress : function(ip){
            if(ip == null || ip == "")
                return false;
            var reg = new RegExp(this._regExp);
            return reg.test(ip);
        },
        
        //Determines if the address is the same as the gateway
        _matchesGateway : function(value){
            var difference = this._subtractAddresses(this.gateway, value);
            //console.log("_matchesGateway: difference=", difference);
            return ((difference != null) && (difference == 0));
        },
        
        //Determines if the address is the same as the network
        _sameAsNetworkID : function(value){
            return (this.version == this.IPV4_VALUE && this.network == value);
        },
        
        //Determines if an ip address matches the network address
        _matchesNetworkID : function(value){
            if(this._isValidAddress(this.network) && this._isValidAddress(value)){
                if(this.version == this.IPV4_VALUE){
                    var subnet = this.dropDown;
                    var splitSub = subnet.split(".");
                    var IDsplit = this.network.split(".");                    
                    var IPsplit = value.split(".");                    
                    for(var i=0; i<IDsplit.length; i++){
                        if(IDsplit[i] != "0" && (IDsplit[i] != (splitSub[i] & IPsplit[i]))){
                            return false;
                        }
                    }
                }
                else if(this.version == this.IPV6_VALUE && this.dropDown != null){
                    var prefix = this.dropDown;
                    value = this._getIPv6FullNotation(value);
                    var network = this._getIPv6FullNotation(this.network);
                    
                    //full hex to binary prefix
                    if(value != null && network != null){
                        var hexValue = value.split(":");
                        var hexNetwork = network.split(":");
                        var binValue = new Array(8);
                        var binNetwork = new Array(8);
                        for(var i=0; i<8; i++){
                            binValue[i] = this.hex2bin(hexValue[i]);
                            binNetwork[i] = this.hex2bin(hexNetwork[i]);
                        }
                        var fullBinValue = this.getFullBinArray(binValue).join("").substring(0, prefix);
                        var fullBinNetwork = this.getFullBinArray(binNetwork).join("").substring(0, prefix);
                        return (fullBinValue == fullBinNetwork);
                    }  
                }
            }
            return true;
        },

        //Returns true if gateway is in the bounds
        //gateway is 1.1.1.11, bounds are 1.1.1.1 and 1.1.1.10
        _isGatewayInBounds : function(first, last){
            var gateway = this.gateway;
            if(this._isValidAddress(first) && this._isValidAddress(gateway) && this._isValidAddress(last)){
                var inBounds = this._isInBounds(gateway, first, last);
                if(inBounds)
                    return true;
            }
            return false;
        },
        
        //Calculate the lowest address (for overall network);
        calculateLowestAddress : function(network){
            if(this._isValidAddress(network) && this.dropDown != null){
                if(this.version == this.IPV4_VALUE){
                    var subnet = this.dropDown;
                    var lowest = new Array(4);
                    var splitSub = subnet.split(".");
                    var networkSplit = network.split(".");
                    for(var i=0; i<splitSub.length; i++){
                        if(splitSub[i] == 255)
                            lowest[i] = networkSplit[i];
                        else if (i != splitSub.length-1)
                            lowest[i] = "0";
                        else
                            lowest[i] = "1";
                    }
                    lowest = lowest.join(".");
                    if(this.gateway == lowest){
                        var lowestSplit = lowest.split(".");
                        lowest = lowestSplit[0] + "." + lowestSplit[1] + "." + lowestSplit[2] + ".2";
                    }
                    return lowest;
                }
                else if(this.version == this.IPV6_VALUE){
                    var prefix = this.dropDown;
                    //full notation
                    var fullIP = this._getIPv6FullNotation(network);
                    if(fullIP != null){
                        var hexArr = fullIP.split(":");
                        var binArr = new Array(8);
                        for(var i=0; i<8; i++){
                            binArr[i] = this.hex2bin(hexArr[i]);
                        }
                        var sub = this.getFullBinArray(binArr).join("").substring(0, prefix);
                        for(var i=prefix; i<128; i++){
                            sub += "0";
                        }
                        var newBinArr = sub.match(/.{1,16}/g);
                        for(var i=0; i<8; i++){
                            newBinArr[i] = this.bin2hex(newBinArr[i]);
                        }
                        return newBinArr.join(":").toUpperCase();
                    }
                }
            }
            return null;  
        },
        
        //Determines if an IP address is within specified bounds
        //Ex: Is 1.1.1.10 in [1.1.1.1 - 1.1.1.2] with network 1.1.1.0?
        _isInBounds : function(value, first, last){
            if(this._isValidAddress(value) && this._isValidAddress(first) && this._isValidAddress(last) ){
                var firstDiff = this._subtractAddresses(first, value); // should be greater
                var secondDiff = this._subtractAddresses(value, last);
                return ( firstDiff >= 0 && secondDiff >= 0 );
            }
            return true;
        },
        
        //Determines if an IP address is within the network
        _isInNetworkBounds : function(value, lowest, highest){
            if(this._isValidAddress(value) && this._isValidAddress(lowest) && this._isValidAddress(highest) ){
                var firstDiff = this._subtractAddresses(lowest, value); // should be greater
                var secondDiff = this._subtractAddresses(value, highest);
                return ( firstDiff >= 0 && secondDiff >= 0 );
            }
            return true;
        },
       
        //Determines if any subranges are overlapping one another
        _isOverlapping : function(value){
            var store = this.subrangeStore;
            if(store != null){
                var items = store._arrayOfTopLevelItems;
                for(var i=0; i<items.length; i++){
                    if(this.mode == "add" || (this.mode == "edit" && i != this.index) ){
                        var item = items[i];
                        var first = store.getValue(item, "first");
                        var last = store.getValue(item, "last");
                        if(this._isValidAddress(first) && this._isValidAddress(last)){
                            if(value == first || value == last || this._isInBounds(value, first, last))
                                return true;
                        } 
                    }
                }
            }
            return false;
        }, 
        
        _isGreater: function(/*String*/x, /*String*/y) {

            // Determine if x is greater than y, where both x & y are String representations of numbers to compare.
            //
            // NOTE: This method does NOT use the normal mathematical approach since x & y may
            //       represent very large numbers that exceed Javascript's ability to store numerical
            //       values beyond 2^53 with precision. 
            var idx;
            //console.log(this.declaredClass + ": x=" + x, "y=" + y);
            
            // Initially, check which has more digits. Longest is greater.
            for (idx=x.length; idx<y.length; ++idx) {
                if (y[idx]) {
                    //console.log(this.declaredClass + ": false (y has more digits)");
                    return false;  // y has more digits
                }
            }
            for (idx=y.length; idx<x.length; ++idx) {
                if (x[idx]) {
                    //console.log(this.declaredClass + ": true (x has more digits)");
                    return true;  // x has more digits
                }
            }

            // Should be equal, but set stopIdx just in case to avoid walking off end of array.
            var stopIdx = (x.length < y.length) ? x.length : y.length;
            for (idx=0; idx < stopIdx; ++idx) {
                //console.log(this.declaredClass + ": x[" + idx + "]=" + x[idx], "y[" + idx + "]=" + y[idx]);
                if (x[idx]>y[idx]) {
                    //console.log(this.declaredClass + ": true (x[" + idx + "] is greater)");
                    return true;
                }
                else if (x[idx]<y[idx]) {
                    //console.log(this.declaredClass + ": false (y[" + idx + "] is greater)");
                    return false;
                }
            }

            //console.log(this.declaredClass + ": false (is equal)");
            return false;
        },

        //Determines the size available between a certain IP address and the next available address
        getAvailableSize : function(first){
            if(this._isValidAddress(first)){
                var store = this.subrangeStore;
                var items = store._arrayOfTopLevelItems;                
                var tempNet = this.calculateNetworkID(first);                
                var broad = this.calculateBroadcastAddress(tempNet);                
                var highestAddress = this.calculateHighestAddress(tempNet, broad);                
                
                if(items.length == 0){
                    return this._subtractAddresses(first, highestAddress) + 1;
                }
                else{
                    var item = {first: first};
                    var index = this.getInsertIndex(item);
                    if(index < items.length){
                      var firstAtIndex = items[index].first[0];                      
                      return this._subtractAddresses(first, firstAtIndex);
                    }
                    else{
                        return this._subtractAddresses(first, highestAddress) + 1;
                    }                   
                }
            }
            return null;
        },
        
        //Determines if any subranges exist in the table
        hasSubranges : function(){
            var store = this.subrangeStore;
            var items = store._arrayOfTopLevelItems;
            return items.length > 0;
        },
        
        //Returns the first available IP address
        getFirstAvailableAddress : function(){
            if (this.network == null) {
                return null;
            }
            var overallNetwork = this.overallStore._arrayOfTopLevelItems[0];
            var lowestAddress = this.overallStore.getValue(overallNetwork, "lowest");
            var highestAddress = this.overallStore.getValue(overallNetwork, "highest");
            if (!this._isOverlapping(lowestAddress)) {
                //if lowestAddress is yet used: --> return it
                return lowestAddress;
            }
            else{
                //get last address + 1 and see if it's overlapping
                //if not, return it
                var store = this.subrangeStore;
                var items = store._arrayOfTopLevelItems;
                for(var i=0; i<items.length; i++){
                    var item = items[i];
                    var last = store.getValue(item, "last");
                    var next = this._getLastAddress(last, 2); //size must be "2" to get to next address
                    if (this._isInBounds(next, lowestAddress, highestAddress) && !this._isOverlapping(next)) {
                        if (!this._matchesGateway(next)) {
                            //if lowestAddress is yet used: --> return it
                            return next;
                        }
                        else {
                            // if next matches gateway use next+1
                            next = this._getLastAddress(next, 2); //size must be "2" to get to next address
                            if (this._isInBounds(next, lowestAddress, highestAddress) && !this._isOverlapping(next)) {
                                return next;
                            }
                        }
                    }
                }
                //return highest network address if not overlapping
                if(!this._isOverlapping(highestAddress) && !this._matchesGateway(highestAddress)){
                    //if highestAddress is yet used: --> return it
                    return highestAddress;
                }
            }
            return null;
        },
        
        //Determines the index at which an IP address would be sorted in ascending order
        getInsertIndex : function(addressItem){
            var store = this.subrangeStore;
            var items = store._arrayOfTopLevelItems;
            var addressFirst = addressItem.first;
            if(items.length == 0)
                return 0;
            else if(items.length == 1){
                var currentFirst = store.getValue(items[0], "first");
                var difference = this._subtractAddresses(addressFirst, currentFirst);
                if(difference > 0)
                    return 0;
                else
                    return 1;
            }
            else{
                for(var i=0; i<items.length-1; i++){
                    var current = items[i];
                    var next = items[i+1];
                    var currentFirst = store.getValue(current, "first");
                    var nextFirst = store.getValue(next, "first");
                    var firstDiff = this._subtractAddresses(currentFirst, addressFirst);
                    var secondDiff = this._subtractAddresses(addressFirst, nextFirst);
                    //console.log(this.declaredClass + ": ", addressFirst + " - " + currentFirst + " = " + firstDiff);
                    //console.log(this.declaredClass + ": ", nextFirst + " - " + addressFirst + " = " + secondDiff);
                    
                    //first neg, second pos
                    if(firstDiff <= 0 && secondDiff >= 0){
                        return i;
                    }
                    //both pos
                    if(firstDiff >= 0 && secondDiff >= 0){
                        return i+1;
                    }          
                }
                return items.length;
            }
        },
        
        //127.127.255.254 - 127.0.0.1 = 8388605, two - one = difference
        _subtractAddresses : function(one, two){
            if(this._isValidAddress(one) && this._isValidAddress(two)){
                if(this.version == this.IPV4_VALUE){
                    var oneSplit = one.split(".");
                    var twoSplit = two.split(".");
                    var difference = 0;
                    
                    for(var i=0; i<4; i++){
                        var partOne = parseInt(oneSplit[i]);
                        var partTwo = parseInt(twoSplit[i]);
                        difference += (partTwo - partOne) * Math.pow(256, 3-i);
                    }
                    return difference;
                }
                else if(this.version == this.IPV6_VALUE){
                    one = this._getIPv6FullNotation(one);
                    two = this._getIPv6FullNotation(two);
                    var oneSplit = one.split(":");
                    var twoSplit = two.split(":");
                    var difference = 0;
                    for(var i=0; i<8; i++){
                        //hex to dec
                        oneSplit[i] = parseInt(oneSplit[i], 16);
                        twoSplit[i] = parseInt(twoSplit[i], 16);
                        difference += (twoSplit[i] - oneSplit[i]) * Math.pow(65536, 7-i);
                    }
                    return difference;
                }
            }
            return null;
        },
        
        _getIPv6FullNotation : function(/*String*/ ipv6){
            if(this.version == this.IPV6_VALUE && this._isValidAddress(ipv6)){
              //convert to full notation
                var sep = ipv6.split(":");
                for(var i=0; i<sep.length; i++){
                    var part = sep[i];
                    if(part == ""){
                        sep.splice(i, 1);
                        var num = 8 - sep.length;
                        for(var j=0; j<num; j++){
                            sep.splice(i, 0, "0000");
                        }
                    }
                }
                return sep.join(":");
            }
            return null;            
        },
        
        //Sets the type of range (single or custom)
        _setTypeAttr : function(t){
            this.type = t;
            if(t == "single"){
                //use drop down
                this._rangeDropDownDiv.setAttribute("style", "display:block");
                this._rangeTextBoxDiv.setAttribute("style", "display:none");
                this._singleMessage.setAttribute("style", "display:block");
                this._customMessage.setAttribute("style", "display:none");
                if(this._rangeSize != null && this._rangeTextBox != null){
                    if(this.mode == "add" && this._rangeSize.get("value") == ""){
                        this._rangeSize.set("value", "20");
                        var availSize = this.getAvailableSize(this.getFirstAvailableAddress());
                        if(availSize != null && !isNaN(availSize) &&  availSize < 20 && availSize > 0){
                            this._rangeSize.set("value", availSize);
                        }
                    }
                    this.set("range", this._rangeDropDown.get("displayedValue"));                    
                    this.set("chassisId", this._rangeDropDown.get("value"));                    
                }
            }
            else{
                //"custom", use textbox
                this._rangeDropDownDiv.setAttribute("style", "display:none");
                this._rangeTextBoxDiv.setAttribute("style", "display:block");
                this._singleMessage.setAttribute("style", "display:none");
                this._customMessage.setAttribute("style", "display:block");
                if(this._rangeSize != null && this._rangeDropDown != null
                        && this.subrangeStore != null && this.overallStore != null){
                    this.set("range", this._rangeTextBox.get("value"));
                    if(this.mode == "add" && this._rangeSize.get("value") == ""){
                        if(this.subrangeStore._arrayOfTopLevelItems.length == 0){
                            var overallSize = this.overallStore._arrayOfTopLevelItems[0].host[0];
                            this._rangeSize.set("value", overallSize);
                        }
                        else {
                            var subrangeItems = this.subrangeStore._arrayOfTopLevelItems;
                            var previousSize = subrangeItems[subrangeItems.length-1].size[0];
                            this._rangeSize.set("value", previousSize);
                        }  
                    }
                }
            }                           
        },
        
        //Sets the version and its corresponding attributes
        _setVersionAttr : function(m){
            this.version = m;
            if(this.version == this.IPV4_VALUE){
                this._invalidMessage = this.locResources.INVALID_MESSAGE_IPV4;
                this._regExp = "^([0-9]|[0-9][0-9]|[0-1][0-9][0-9]|2[0-4][0-9]|25[0-5])([\.]([0-9]|[0-9][0-9]|[0-1][0-9][0-9]|2[0-4][0-9]|25[0-5])){3}$";
                this._placeHolder = "0.0.0.0";
            }
            else if(this.version == this.IPV6_VALUE){
                this._invalidMessage = this.locResources.INVALID_MESSAGE_IPV6;
                this._regExp = "^\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*$";
                this._placeHolder = "0::0";
            }
            else{
                this._invalidMessage = this.locResources.INVALID_MESSAGE;
                this._regExp = ".*";
                this._placeHolder = null;
            }
        },
        
        //Set "range", enable ok?
        _setRangeAttr : function(r){
            this.range = r;
            this._okEnabled();
        },
        
        //Set "first", enable ok?
        _setFirstAttr : function(f){
            this.first = f.toUpperCase();
            this.calculateNetworkID(this.first);
            this.set("last", this._getLastAddress(this.first, this.size));
            this._okEnabled();
        },
        
        //Set "size", enable ok?
        _setSizeAttr : function(s){
            this.size = s;
            this.set("last", this._getLastAddress(this.first, this.size));
            this._okEnabled();
        },
        
        //Sets "last", enable ok?
        _setLastAttr : function(l){
            if(l != null)
                l = l.toUpperCase();
            this.last = l;
            if(this._rangeLast != null)
                this._rangeLast.set("value", this.last);
            this._okEnabled();
        },
        
        //Calculates the highest address (for overall network table)
        calculateHighestAddress : function(network, broadcast){
            if(this.dropDown != null && this._isValidAddress(network)){
                if(this.version == this.IPV4_VALUE && this._isValidAddress(broadcast)){
                    var splitFirst = broadcast.split(".");
                    splitFirst[3] = parseInt(splitFirst[3]) - 1;
                    var highest = splitFirst.join("."); 
                    if(this.gateway == highest){
                        var highestSplit = highest.split(".");
                        highest = highestSplit[0] + "." + highestSplit[1] + "." + highestSplit[2] + "." + (parseInt(highestSplit[3])-1);
                    }
                    return highest;
                }
                else if(this.version == this.IPV6_VALUE){
                    var fullIP = this._getIPv6FullNotation(network);
                    if(fullIP != null){
                        var prefix = this.dropDown;
                        var hexArr = fullIP.split(":");
                        var binArr = new Array(8);
                        for(var i=0; i<8; i++){
                            binArr[i] = this.hex2bin(hexArr[i]);
                        }
                        var sub = this.getFullBinArray(binArr).join("").substring(0, prefix);
                        for(var i=prefix; i<128; i++){
                            sub += "1"; //add 1's (binary)
                        }
                        var newBinArr = sub.match(/.{1,16}/g);
                        for(var i=0; i<8; i++){
                            newBinArr[i] = this.bin2hex(newBinArr[i]);
                        }
                        return newBinArr.join(":").toUpperCase();
                    }
                }
            }
            return null;
        },
        
        //Calculate broadcast address for IPv4
        calculateBroadcastAddress : function(network){
            var subnet = this.dropDown;
            if(this.version == this.IPV4_VALUE && subnet != null && this.network != null){
                var splitSub = subnet.split(".");
                var networkSplit = network.split(".");
                var broadSplit = new Array(4);
                for(var i=0; i<splitSub.length; i++){
                    broadSplit[i] = parseInt(networkSplit[i]) + (255 - parseInt(splitSub[i]));
                }
                return broadSplit.join(".");
            }
            return null;     
        },
        
        //Returns the last address depending on the first address, size, and version
        _getLastAddress : function(first, size){
            //is size a number greater than 0
            var reg = new RegExp("^[1-9][0-9]*$");
            if(first != null && size != null && reg.test(size) && this._isValidAddress(first)){
                if(this.version == this.IPV4_VALUE){
                    var splitFirst = first.split(".");
                    
                    //add
                    splitFirst[3] = parseInt(splitFirst[3]) + (parseInt(size)-1);
                    for(var i=3; i>=0; i--){
                        while(splitFirst[i] > 255){
                            splitFirst[i-1]++;
                            splitFirst[i] -= 256;
                        }
                    }
                    return splitFirst.join(".");
                }
                else if(this.version == this.IPV6_VALUE){ 
                    first = this._getIPv6FullNotation(first);
                    var sep = first.split(":");
                                        
                    //convert to decimal array
                    for(var i=0; i<sep.length; i++){
                       var part = sep[i];
                       var replace = parseInt(part, 16);
                       sep.splice(i, 1, replace);
                    }
                    
                    //add
                    sep[7] += (parseInt(size)-1);
                    for(var i=7; i>=0; i--){
                        if(sep[i] >= 65536){
                            var num = sep[i]/65536;
                            sep[i-1] = Math.floor(parseInt(num));
                            sep[i] = sep[i] % 65536;
                        }
                    }
                    
                    //convert to hex
                    for(var i=0; i<sep.length; i++){
                        var part = sep[i];
                           var replace = part.toString(16);
                           sep.splice(i, 1, replace);
                    }
                    
                    return sep.join(":").toUpperCase();
                }
            }
            return null;
        },
        
        //Overridable method to edit a subrange when the popup is closed
        editSubrange : function(){
            //override if necessary
        },
        
        //Overridable method to add a new subrange when the popup is closed
        addNewSubrange : function(){
            //override if necessary
        },
        
        hex2bin : function(hex){
            var dec = parseInt(hex, 16);
            return dec.toString(2);
        },
        
        //used
        bin2hex : function(bin){
            var dec = parseInt(bin, 2);
            return dec.toString(16);
        },
        
        //used
        getFullBinArray : function(binArr){
            for(var i=0; i<binArr.length; i++){
                var binItem = binArr[i];
                for(var j=binItem.length; j<16; j++){
                    //add leading 0s
                    binArr[i] = "0" + binArr[i];
                }
            }
            return binArr;
        },

        
        /*BUTTONS*/
        
        //Determines if the ok button should be enabled
        _okEnabled : function(){            
            var firstBox = this._rangeFirst;
            var sizeBox = this._rangeSize;
            var lastBox = this._rangeLast;
            if(this.range != null && this.range != "" &&
                this.first != null && this.first != "" &&
                this.size != null && this.size != "" &&
                this.last != null && this.last != "" &&
                firstBox != null && sizeBox != null && lastBox != null &&
                firstBox.isValid() && sizeBox.isValid() && lastBox.isValid() &&
                ( (this.type == "custom" && this._rangeTextBox.isValid()) ||
                  (this.type == "single" && this._rangeDropDown.get("value") != "" && this._rangeDropDown.isValid())
                )
               ){
                    this._okBtn.set("disabled", false);
            }
            //cannot click OK
            else
                this._okBtn.set("disabled", true);
        },
        
        //Action to take if ok is clicked
        _okClicked : function(){
            this.hide();
            dojo.destroy(this.id + "_underlay");
            if(this.mode == "add")
                this.addNewSubrange();
            else
                this.editSubrange();
        },
        
        //Action to take if cancel is clicked
        _cancelClicked : function(){
            // Make sure this tooltip isn't hanging around
            dijit.hideTooltip(this._rangeLast.domNode);
            
            this.hide();
            dojo.destroy(this.id + "_underlay");
        }
    });
});