define([
        "dojo",
        "dijit",
        "support/dialogs/Dialog",
        "dojo/i18n!./nls/SystemInfoTemplate",
        "dojo/text!./templates/SystemInfoTemplate.html",
        "dijit/form/SimpleTextarea",
        "dijit/form/CheckBox",
        "dijit/form/ToggleButton",
        "dijit/form/ValidationTextBox",
        "dijit/Dialog",
        "dojo/parser",
        "support/widgets/ButtonBar",
        "support/widgets/OutlinePane",
        "dijit/layout/BorderContainer",
        "dijit/layout/ContentPane",
        "support/panels/Panel",
        "support/panels/CheckButton",
        "support/panels/CheckLabelButton",
        "support/dialogs/ReDeployServerTemplate",
        "support/panels/CustomNamingTable",
        "dijit/Tooltip",
        "support/dialogs/SaveAs",
        "support/dialogs/ProgressDialog"
    ], function(dojo, dijit, dialog, mri, template) {
    
    return dojo.declare("support.dialogs.SystemInfoTemplate", [ support.dialogs.Dialog], {
        
        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
        
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        maxPercent: .40,
        dialogRef: null,
        templateIDNum: null,
        intitialData: null,
        finalData: null,
        _customnametable: null,
        default_button: 1,
        _isCustomNamePatternValid: null,
        _buttonbar1: null,
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/SystemInfoTemplate.css",


        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);
            
            this.subscribe("customNamePattern", function(data) {                
                if (data == "none") {
                    this._isCustomNamePatternValid = false;
                    this._nameChanged();
                } else {
                    this._isCustomNamePatternValid = true;
                    this._nameChanged();
                }
             });            
            
            dojo.connect(this.dialogNode, "hide", this, function() {
                this.destroy();
            });
            
            // Create the button bar
            this._buttonbar1 = new support.widgets.ButtonBar({
                values: this.resources.CUSTOM + "|" + this.resources.DISABLE,
                onClick: dojo.hitch(this, this.displayAutogenInfo)
            }).placeAt(this._buttonbar1div);
            
          //Setup for template new, edit, or copy mode 
            this._setup();
            
          //Controls dialog dynamic resizing max height 
//            var availHeight = window.screen.availHeight;
  //          var maxHeight = availHeight * this.maxPercent;
    //        this.settingsContainer.setAttribute("style", "max-height:" + maxHeight + "px");
            
            //Default button setting
            this._buttonbar1.set("selIndex", this.default_button);
                             
            
            // Warn user if Location field is blank
            if (this._locationValue.get("value").trim() == "") {
                dojo.style(this._blankLocationWarning, "visibility", "visible");
            } else {
                dojo.style(this._blankLocationWarning, "visibility", "hidden");
            }

            // Warn user if Contact field is blank
            if (this._contactValue.get("value").trim() == "") {
                dojo.style(this._blankContactWarning, "visibility", "visible");
            } else {
                dojo.style(this._blankContactWarning, "visibility", "hidden");
            }

            //Resizes dialog
            this.layout(); 
                               
        },
        
        loadprevSettings: function() {
            var mode = this.templateMode;

            var store = null;
            var dialogNode = this.dialogNode;

            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "template/" + this.templateIDNum,
                handleAs: "json",
                sync: true,
                preventCache: true,
                load: function(response) {
                    store = response; 
                    //alert(respnonse);
                }
            });
            
            this.initialData = store.items;
            
            this.templateInUse = store.items[0].inUse;

            
            // Set dialog title to reflect Editing and display the warning message                      
            if(mode == "copy"){
                dialogNode.set("title", this.locResources.NAME + " - " + store.items[0].name + " - " + this.resources.COPY);
                this.templateName.set("value", store.items[0].name + " - " + this.resources.COPY);
            }else{
                dialogNode.set("title", this.locResources.EDIT + " - " + store.items[0].name);
                this.templateName.set("value", store.items[0].name);
            }
           
            var descriptionvalue = store.items[0].description;
            var description = this.decodeSpecialCharacters(descriptionvalue);
            this._templateDescription.set("value", description);
            
            var buttonIndex = store.items[0].IMM__GeneralSettings__info__autogen.IMM__GeneralSettings__info__autogen == "Custom"?0:1;
            var patternData = store.items[0].IMM__GeneralSettings__info__autogen.pattern;
            this._customnametable = new support.panels.CustomNamingTable({
                id: "cp_systeminfotemplate_customnamingtable",
                templateType: "sys",
                namepattern: patternData    
            }, this.get("id") + "_customname");
            this._hyphenCheckBox.set("checked", store.items[0].IMM__GeneralSettings__info__autogen.hyphenChecked);
            
            console.log(this.declaredClass + ": namepattern");
            console.log(this.declaredClass + ": ", patternData);
            
            if(store.items[0].IMM__GeneralSettings__info__autogen){
                this.default_button = buttonIndex;
            }
            if(store.items[0].IMM__GeneralSettings__info__contact){
                this._contactValue.set("value", store.items[0].IMM__GeneralSettings__info__contact.IMM__GeneralSettings__info__contact);
            }
            if(store.items[0].IMM__GeneralSettings__info__location){
                this._locationValue.set("value", store.items[0].IMM__GeneralSettings__info__location.IMM__GeneralSettings__info__location);
            }
        },
        
        cancelClicked: function() {
            this.hide();
        },
        
        displayContactTextBox: function() {
            
            dojo.style(this._contactLabel, "display", "");
            this._contactValue.set("value", "");

            return;
        },

        displayLocationTextBox: function() {
            
            dojo.style(this._locationLabel, "display", "");
            this._locationValue.set("value", "");

            return;
        },

        displayAutogenButton: function() {

            this._autogenBoxArea.style.display = "inline-block";
            dojo.style(this._autogenLabel, "display", "");
            dojo.style(this._settingstip, "display", "");

            this._ButtonBarArea.style.display = "inline-block";

            this.displayAutogenInfo();    

            return;
        },
        
        displayAutogenInfo: function() {
            
            // Removed the predefined option for now per updated design.
            
            if (this._buttonbar1.get("selIndex") == 0) {
                this._customInfo.setAttribute("style", "display:inline-block");
                this._disabledInfo.setAttribute("style", "display:none");                
                this._predefinedInfo.setAttribute("style", "display:none");
                
                this._customArea.setAttribute("style", "display:inline-block");
                this._disabledArea.setAttribute("style", "display:none");                
                this._predefinedArea.setAttribute("style", "display:none");
                if (this._customnametable == null) {
                    this._customnametable = new support.panels.CustomNamingTable({
                        id: "cp_systeminfotemplate_customnamingtable",
                        templateType: "sys"
                    }, this.get("id") + "_customname");
                }
                this._nameChanged();
            }
            if (this._buttonbar1.get("selIndex") == 1) {
                this._predefinedInfo.setAttribute("style", "display:none");
                this._disabledInfo.setAttribute("style", "display:inline-block");
                this._customInfo.setAttribute("style", "display:none");
                
                this._predefinedArea.setAttribute("style", "display:none");
                this._disabledArea.setAttribute("style", "display:inline-block");
                this._customArea.setAttribute("style", "display:none");
                
                // If disabling autogen, then ok button rules for custom naming no longer apply                
                this._nameChanged();
            }
            // Just comment out for now to make adding it back later easier.
            /*if (this._buttonbar1.get("selIndex") == 2) {
                this._predefinedInfo.setAttribute("style", "display:none");
                this._customInfo.setAttribute("style", "display:none");
                this._disabledInfo.setAttribute("style", "display:inline-block");
                
                this._predefinedArea.setAttribute("style", "display:none");
                this._customArea.setAttribute("style", "display:none");
                this._disabledArea.setAttribute("style", "display:inline-block");                
            }*/
           
            return;
        },

        hideDefaultedSettings: function() {

            if (this._hideBox.get("checked")) { // Hide info if default
                if (this._buttonbar1.get("selIndex") == 1) {
                    this._autogenBoxArea.style.display = "none";
                    this._autogenSettingsBox.setAttribute("style", "display:none");
                    dojo.style(this._autogenLabel, "display", "none");
                    dojo.style(this._settingstip, "display", "none");
                    dojo.style(this._buttonbarArea, "display", "none");
                }
                if (this._contactValue.get("value") == "") {
                    dojo.style(this._contactValueArea, "display", "none");
                    dojo.style(this._contactLabel, "display", "none");
                    dojo.style(this._blankContactWarning, "visibility", "hidden");
                }
                if (this._locationValue.get("value") == "") {
                    dojo.style(this._locationValueArea, "display", "none");
                    dojo.style(this._locationLabel, "display", "none");
                    dojo.style(this._blankLocationWarning, "visibility", "hidden");
                }
            }
            else { // Re-display if had been hidden previously
                if (this._autogenBoxArea.style.display == "none") {
                    this._autogenBoxArea.style.display = "inline-block";
                    this._autogenSettingsBox.setAttribute("style", "display:inline-block");
                    dojo.style(this._autogenLabel, "display", "");
                    dojo.style(this._settingstip, "display", "");
                    dojo.style(this._buttonbarArea, "display", "");
                }
                if (this._contactValueArea.style.display == "none") {
                    dojo.style(this._contactValueArea, "display", "");
                    dojo.style(this._contactLabel, "display", "");
                    (this._contactValue.get("value").trim() == "") && dojo.style(this._blankContactWarning, "visibility", "visible");
                }
                if (this._locationValueArea.style.display == "none") {
                    dojo.style(this._locationValueArea, "display", "");
                    dojo.style(this._locationLabel, "display", "");
                    (this._locationValue.get("value").trim() == "") && dojo.style(this._blankLocationWarning, "visibility", "visible");
                }
            }

            return;

        },
        doNothing: function() {
        },
        
        exitParent: function() {
            var data = "systeminfo";
            
            dojo.publish("SystemTemplateComplete", data);

            if(this._rdstDialog != null){
                var deployClicked = this._rdstDialog.get("closeParent");
                if(deployClicked) {
                    this.hide();
                }
            }
        },

        createClicked: function() {

            var templateObject = new Object();
            if(this.templateIDNum){
                templateObject.uri = this.initialData[0].uri;
            }
            templateObject.name = this.templateName.get("value").trim();
       
            templateObject.type = "SystemInfo";
            
            var description = this._templateDescription.get("value").trim();
            templateObject.description = this.encodeSpecialCharacters(description);
   
            var contactObject = new Object();
            contactObject.IMM__GeneralSettings__info__contact = this._contactValue.get("value"); 
            contactObject.checked = true;
            templateObject.IMM__GeneralSettings__info__contact = contactObject;  
                            
            var autogenObject = new Object();  
//          autogenObject.IMM__GeneralSettings__info__autogen = "enabled";
            if(this._buttonbar1.get("selIndex") == 1){
                autogenObject.IMM__GeneralSettings__info__autogen = "Disable";
            }
            else{
                autogenObject.IMM__GeneralSettings__info__autogen = "Custom";
                if (this._customnametable){
                    autogenObject.pattern = this._customnametable.get("pattern");
                }
            }
            autogenObject.checked = true;
            autogenObject.hyphenChecked = this._hyphenCheckBox.checked;
            templateObject.IMM__GeneralSettings__info__autogen = autogenObject;

            var locationObject = new Object();
            locationObject.IMM__GeneralSettings__info__location = this._locationValue.get("value"); 
            locationObject.checked = true;
            templateObject.IMM__GeneralSettings__info__location = locationObject;

            // Autogenerate System Name Settings

            var argJSONString = dojo.toJson({ 
                "identifier":"id",
                  "label":"name",
                  "items":[ templateObject ]
            }, true);

            this.finalData = templateObject;

            this.getProgressDialog("save").show(); // Show progress dialog for save request.
                var xhrPost = dojo.xhrPost({
                    url: this.REST_PATH + "template/create",
                    handleAs: "json", // <<<< Required
                    headers: {
                        "Content-Type": "application/json; charset=utf-8" // <<<< Required
                    },
                    postData: argJSONString,     //Template Data to Post
                    load: dojo.hitch(this, function(response, ioArgs) {
                        // Happy path! We received a 2xx status code from the API.
                                                
                        dojo.publish("ProgressComplete", response.id);
                        dojo.publish("SystemTemplateComplete", response.id);
                        this.hide();   
                    }),
                    error: dojo.hitch(this, function(error, ioArgs) {
                        // Failure path! We received a non-2xx status code (ie: 3xx/4xx/5xx status code) from the API.

                        dojo.publish("ProgressFailed", error);
                        console.log(this.declaredClass + ": ", error, ioArgs);
                        
                        var response = dojo.fromJson(ioArgs.xhr.responseText);
                        if ((ioArgs.xhr.status >= 400) && (response.message == "service_error_name_in_use")) {
                            // Error Template name in use
                            this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                        }
                        else if(ioArgs.xhr.status == 404) {
                            // Edit Template ID doesn't exist
                            this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                            this.hide();
                        }
                        else if(ioArgs.xhr.status >= 500) {
                            // Internal Programming Error
                            this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                            this.hide();
                        }
                        else {
                            this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                            this.hide();
                        }
                        
                    }),
                    handle: dojo.hitch(this, function(data, ioArgs) {
                        // - The 1st param ("data") is either the response data (on success) or Error object (on failure).
                        // - The 2nd param ("ioArgs") contains detail about the xhr request, including a reference to the
                        // native xhr object. Check ioArgs.xhr.status to determine success or failure, and handle "data"
                        // accordingly.
                    })
                });

                
        },
        saveClicked: function() {      

            var templateObject = new Object();
            if(this.templateIDNum){
                templateObject.uri = this.initialData[0].uri;
            }
            templateObject.name = this.templateName.get("value").trim();

            templateObject.id = this.templateIDNum;
       
            templateObject.type = "SystemInfo";
            
            var description = this._templateDescription.get("value").trim();
            templateObject.description = this.encodeSpecialCharacters(description);
   
            var contactObject = new Object();
            contactObject.IMM__GeneralSettings__info__contact = this._contactValue.get("value"); 
            contactObject.checked = true;
            templateObject.IMM__GeneralSettings__info__contact = contactObject;  
                            
            var autogenObject = new Object();  
//          autogenObject.IMM__GeneralSettings__info__autogen = "enabled";
            if(this._buttonbar1.get("selIndex") == 1){
                autogenObject.IMM__GeneralSettings__info__autogen = "Disable";
            }
            else{
                autogenObject.IMM__GeneralSettings__info__autogen = "Custom";
                if (this._customnametable){
                    autogenObject.pattern = this._customnametable.get("pattern");
                }
            }
            autogenObject.checked = true;
            autogenObject.hyphenChecked = this._hyphenCheckBox.checked;
            templateObject.IMM__GeneralSettings__info__autogen = autogenObject;

            var locationObject = new Object();
            locationObject.IMM__GeneralSettings__info__location = this._locationValue.get("value"); 
            locationObject.checked = true;
            templateObject.IMM__GeneralSettings__info__location = locationObject;

            // Autogenerate System Name Settings

            var argJSONString = dojo.toJson({ 
                "identifier":"id",
                  "label":"name",
                  "items":[ templateObject ]
            }, true);

            this.finalData = templateObject;
            
            var saveTemplate = true;

            if(this.templateInUse.toString() == "true"){
                //  Need to remove extra properties that aren't included in the dialog
                delete this.initialData[0].inUse;
                delete this.initialData[0].referencedBy;
                delete this.initialData[0].useCount;
                delete this.initialData[0].userDefined;
                
                // Remove description since it shouldn't force a redeploy
                delete this.initialData[0].description;
                delete this.finalData.description;
                
                var preData = this.sortAndFlatten(this.initialData[0]);
                var postData = this.sortAndFlatten(this.finalData);
                
                if(preData != postData){
                    this._rdstDialog.set("templateID",this.templateIDNum);
                    this._rdstDialog.set("templateType",this.resources.CATEGORY);
                    this._rdstDialog.set("templateData",argJSONString);

                    this._rdstDialog.show();
                    
                    saveTemplate = false;
                }
            }
            if (saveTemplate){
                this.getProgressDialog("save").show(); // Show progress dialog for save request.
                var xhrPost = dojo.xhrPost({
                    url: this.REST_PATH + "template/create",
                    handleAs: "json", // <<<< Required
                    headers: {
                        "Content-Type": "application/json; charset=utf-8" // <<<< Required
                    },
                    postData: argJSONString,     //Template Data to Post
                    load: dojo.hitch(this, function(response, ioArgs) {
                        // Happy path! We received a 2xx status code from the API.
                                                
                        dojo.publish("ProgressComplete", response.id);
                        dojo.publish("SystemTemplateComplete", response.id);
                        this.hide();   
                    }),
                    error: dojo.hitch(this, function(error, ioArgs) {
                        // Failure path! We received a non-2xx status code (ie: 3xx/4xx/5xx status code) from the API.

                        dojo.publish("ProgressFailed", error);
                        console.log(this.declaredClass + ": ", error, ioArgs);
                        
                        var response = dojo.fromJson(ioArgs.xhr.responseText);
                        if ((ioArgs.xhr.status >= 400) && (response.message == "service_error_name_in_use")) {
                            // Error Template name in use
                            this.getModalDialog("Error", this.resources.ERROR_400_NAME).show();
                        }
                        else if(ioArgs.xhr.status == 404) {
                            // Edit Template ID doesn't exist
                            this.getModalDialog("Error", this.resources.ERROR_404_ID).show();
                            this.hide();
                        }
                        else if(ioArgs.xhr.status >= 500) {
                            // Internal Programming Error
                            this.getModalDialog("Error", this.resources.ERROR_500_INTERNAL).show();
                            this.hide();
                        }
                        else {
                            this.getModalDialog("Error", this.resources.ERROR_TEMPLATE_CREATE).show();
                            this.hide();
                        }
                        
                    }),
                    handle: dojo.hitch(this, function(data, ioArgs) {
                        // - The 1st param ("data") is either the response data (on success) or Error object (on failure).
                        // - The 2nd param ("ioArgs") contains detail about the xhr request, including a reference to the
                        // native xhr object. Check ioArgs.xhr.status to determine success or failure, and handle "data"
                        // accordingly.
                    })
                });
            }

        },        
        saveasClicked: function() {      

            var templateObject = new Object();
            if(this.templateIDNum){
                templateObject.uri = this.initialData[0].uri;
            }
            templateObject.name = this.templateName.get("value").trim();
       
            templateObject.type = "SystemInfo";
            
            var description = this._templateDescription.get("value").trim();
            templateObject.description = this.encodeSpecialCharacters(description);
   
            var contactObject = new Object();
            contactObject.IMM__GeneralSettings__info__contact = this._contactValue.get("value"); 
            contactObject.checked = true;
            templateObject.IMM__GeneralSettings__info__contact = contactObject;  
                            
            var autogenObject = new Object();  
//            autogenObject.IMM__GeneralSettings__info__autogen = "enabled";
            if(this._buttonbar1.get("selIndex") == 1){
                autogenObject.IMM__GeneralSettings__info__autogen = "Disable";
            }
            else{
                autogenObject.IMM__GeneralSettings__info__autogen = "Custom";
                if (this._customnametable){
                    autogenObject.pattern = this._customnametable.get("pattern");
                }
            }
            autogenObject.checked = true;
            autogenObject.hyphenChecked = this._hyphenCheckBox.checked;
            templateObject.IMM__GeneralSettings__info__autogen = autogenObject;

            var locationObject = new Object();
            locationObject.IMM__GeneralSettings__info__location = this._locationValue.get("value"); 
            locationObject.checked = true;
            templateObject.IMM__GeneralSettings__info__location = locationObject;

            this.saveasDialog.set("templateObject",templateObject);
            this.saveasDialog.show();

        },
        _hasTemplateChanged: function(initial,end){
            if(initial.id != end.id){
                //alert("ID CHANGED");
                return true;
            }
            if(initial.name != end.name){
                //alert("NAME CHANGED");
                return true;
            }
            return false;
        },        
        _nameChanged: function() {                        
            this._enableOK();
            
        },
        _contactpersonChanged: function(){
            
            // Warn user if Contact field is blank
            if (this._contactValue.get("value").trim() == "") {
                dojo.style(this._blankContactWarning, "visibility", "visible");
            } else {
                dojo.style(this._blankContactWarning, "visibility", "hidden");
            }

            this._enableOK();            
        },
        _locationChanged: function(newValue) {
            if (this._customnametable != null) {
                this._customnametable.changeLocation(newValue);
            }
            
            // Warn user if Location field is blank
            if (this._locationValue.get("value").trim() == "") {
                dojo.style(this._blankLocationWarning, "visibility", "visible");
            } else {
                dojo.style(this._blankLocationWarning, "visibility", "hidden");
            }

            
            this._enableOK();
        },
        
        _enableOK: function() {
            if (this._isCustomNamePatternValid == false && 
                    this._buttonbar1.get("selIndex") == 0) {
                this.okBtn.set("disabled", true);
                this.saveBtn.set("disabled", true);
                this.saveasBtn.set("disabled", true);
                return;
            } else {
                this.saveBtn.set("disabled", false);
                this.saveasBtn.set("disabled", false);
            }
            
            if (this.templateName.get("value").trim() != "" && this._contactValue.isValid() &&
                    this._locationValue.isValid()) {
                this.okBtn.set("disabled", false);
            }
            else {
                this.okBtn.set("disabled", true);
            }
                                                
        },
        
        _setup: function(){

            //Set Template Field to catch intermediate changes
            this.templateName.intermediateChanges = true;
            this._locationValue.intermediateChanges = true;
            
            this._contactValue.invalidMessage = this.resources.INVALID_CHARS.replace("%1", "& < >");
            this._locationValue.invalidMessage = this.resources.INVALID_CHARS.replace("%1", "! ~ ` @ # $ % ^ & * ( ) / : ; \" \' , " + 
                    "< > { } [ ] ? = | + \\");

            // Set Description Label
            this._descLimit.innerHTML = this.resources.LIMIT.replace("%500","500");

            //Set Information Messages and Load Previous Settings                      
            if(this.templateIDNum != null && this.templateMode == "edit"){     //if dialog in edit mode    
                
                this.loadprevSettings();

                if(this.templateInUse.toString() == "true"){
                
                    if(this.templateLaunchPoint == "table"){
                        
                        this.warningMessage.style.display = "block";
                        this.warningMessageText.innerHTML = this.resources.INUSE;
                        this.saveBtnContainer.style.display = "inline";
                        this.saveasBtnContainer.style.display = "inline";
                        
                    }else if(this.templateLaunchPoint == "wizard"){
                        
                        this.informationMessage.style.display = "block";
                        this.informationMessageText.innerHTML = this.resources.INUSEWIZARD;
                        this.saveasBtnContainer.style.display = "inline";
                        
                    }else{
                        this.informationMessage.style.display = "block";
                        this.informationMessageText.innerHTML = this.resources.INUSEWIZARD;
                        this.saveasBtnContainer.style.display = "inline";
                    }
                    
                }else{
                    
                    this.saveBtnContainer.style.display = "inline";
                    this.saveasBtnContainer.style.display = "inline";
                                        
                }    

                this.templateName.set("disabled",true);
                

             }else if(this.templateIDNum != null && this.templateMode == "copy"){    //if dialog in copy mode     
                 
                 this.loadprevSettings();
                 this.templateIDNum = null;  //set template ID to null in order to create new template
                 this.okBtnContainer.style.display = "inline";

             }else{     //if dialog in create new mode
                 
                 this.okBtnContainer.style.display = "inline";
                 
             } 

        }
   
    });
});
