define([
        "dojo",
        "dijit",
        "support/dialogs/Dialog",
        "dojo/i18n!./nls/UnassignProfile",
        "dojo/text!./templates/UnassignProfile.html",
        "dijit/form/CheckBox",
        "dijit/layout/BorderContainer", 
        "dijit/layout/ContentPane",
        "support/widgets/ProgressButton"
    ], function(dojo, dijit, dialog, mri, template) {
    
    return dojo.declare("support.dialogs.UnassignProfile", [ support.dialogs.Dialog ], {
        
        templateString: template, // Path to the template
        
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,
        
        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "UnassignProfile",
        
        profileName: "", // profile name to unassign
        externalLaunch: "", // unassign launched externally?
        message: "",
        selectedIDs: [],
        mgmtPatternPresent: false,
        addrPresent: false,
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/dialogs/themes/UnassignProfile.css",

        // Override this method to perform custom behavior during dijit construction.
        // Common operations for constructor:
        // 1) Initialize non-primitive types (i.e. objects and arrays)
        // 2) Add additional properties needed by succeeding lifecycle methods
        constructor: function() {
        },
        
        // When this method is called, all variables inherited from superclasses are 'mixed in'.
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);
            
            var title = this.locResources.UNASSIGN_CONFIRM;
            
            this.set("title", title);
            
            this.resetIMMCheckbox.set("checked", this.mgmtPatternPresent);
            this.powerOffCheckbox.set("checked", this.addrPresent);
            
            dojo.connect(this.dialogNode, "hide", this, function() {
                this.destroy();
            });

        },
        
        _onClose: function() {
            this.hide();
        },
        
        _onFinish: function() {
            this._cancelButton.set('disabled', true);
            for(var i = 0; i < this.selectedIDs.length; i++) {
                this._executeUnassign(this.selectedIDs[i].toString());
            }
            this.hide();
        },
        
        _executeUnassign: function(profileID) {
            
            var jsonData = this._createJSONforUnassign();
            
            dojo.xhrPost({
                url: this.REST_PATH + "profile/unassign/" + profileID,
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                postData: jsonData,
                sync: true, 
                load: dojo.hitch(this,function(data) {
                    if(data && data.error && data.error.length > 0){
                        var validationpopup = new support.temp.ModalDialog({
                            type: "Error",
                            text: data.error
                        });
                        validationpopup.show();
                    }
                    else{
                        dojo.publish("UnassignServerProfileComplete");
                    }
                    
                }),
                error: dojo.hitch(this,function(data) {
                    var validationpopup = new support.temp.ModalDialog({
                        type: "Error",
                        text: this.resources.ERROR_UNASSIGN_PROF
                    });
                    console.log(data);
                    validationpopup.show();
                })
            });
        },
        
        _createJSONforUnassign: function() {
            var unassignObject = new Object();
            unassignObject.powerDownITE = this.powerOffCheckbox.checked;
            unassignObject.resetIMM = this.resetIMMCheckbox.checked;
            
            return dojo.toJson(unassignObject);
        }

    });   
});