define(["dojo",
        "dijit",
        "support/panels/Panel", 
        "dojo/i18n!./nls/AddressPools",
        "dojo/text!./templates/AddressPools.html",
        "dijit/layout/LayoutContainer",
        "dijit/layout/ContentPane",
        "support/base/NextGenUIMixin",
        "support/base/TableMixin",
        "dojo/data/ItemFileWriteStore",
        "dijit/form/DropDownButton",
        "dijit/form/Button",
        "dijit/Menu", 
        "dijit/MenuItem",
        "dijit/MenuSeparator",
        "dojo/string", 
        "dijit/ProgressBar", 
        "support/dialogs/NewIPPool", 
        "support/dialogs/FibreChannelAddressPool",
        "support/dialogs/EditEthernetAddressPool",
        "dijit/Tooltip",
        "dijit/ToolbarSeparator"
        ], function(dojo, dijit, panel, mri, template) {

    return dojo.declare("support.panels.AddressPools", [ support.panels.Panel, support.base.NextGenUIMixin, support.base.TableMixin], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in managing address pools
        // for Configuration Patterns.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "AddressPools",

        _poolTable: null,

        createButton: null,
        editButton: null,
        deleteButton: null,

        actionMenu: null,
        _actionButton: null,

        _selectedPool: null,

        refreshRoot: null,

        // the properties used to construct the title of this page in the
        //   NextGen UI
        cpProductName: null,
        cpPageContext: null,
        cpPageName: null,
        
        // ID prefix used for table
        gridID: "AddressPoolTable",

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/AddressPools.css",
        
        // Views
        BY_ROOT: 1,
        FLAT: 2,
        viewMode: 1,

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
            this.cpProductName = this.resources.PRODUCT_NAME;
            this.cpPageContext = null;
            this.cpPageName = this.resources.ADDRESS_POOLS;
        },

        // postCreate() is called after
        // buildRendering(). This is useful to override
        // when
        // you need to access and/or manipulate DOM
        // nodes included with your widget.
        // DOM nodes and widgets with the
        // dojoAttachPoint attribute specified can now
        // be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in
        // buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            
            // Setup table widget...
            var poolStore = this._retrieveStore();
            var tableStructure = this._getTableStructure();
            this._poolTable = this.getTable(poolStore, tableStructure, this.resources.NO_DATA, this.gridID, true);
            
            // Set parent rows as unselectable
            this._poolTable.disableParentRows();
            this._refreshTree();
            
            setTimeout(dojo.hitch(this, function() {
                this._poolTable.placeAt(this.poolTable);
                this._poolTable.startup();
            }), 500);
            
            // Connect handlers to reset table capabilities
            dojo.connect(this._poolTable.select.row, "onSelectionChange", this, this.setTableCapabilities); // Fires on row selection
            dojo.connect(this._poolTable, "setStore", this, this.setTableCapabilities); // Fires on store reset

            // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
            this.changeNextGenUITitle();

            // reload contents of address pools table; by default, switching
            // between links in the details selector does not reload page
            this.subscribeToDetailsSelectorChanges(dojo.hitch(this, function() {
                this._refreshTree();
                this.changeNextGenUITitle();
            }));

            this._loadToolBar(this._poolTable); // load table toolbar

            // Register subscribers to topics we care about ...
            this.subscribe("RefreshTable", function(data) {
                this._refreshTree();
            });

            this.subscribe("CloseDialog", function() {
                if (this.viewMode == this.FLAT) {
                    this._poolTable.showFilter(); // Show filter controls on flat view.
                }
            });

        },

        _refreshTree: function(/*String?*/expandId) {
            // tags:
            // protected
            // Refresh tree data
            var poolStore = this._retrieveStore();
            this._poolTable.setStore(poolStore);
            this._poolTable.model.clearCache();
            var callback = function(treeView, table, /*String?*/expandId) {
                if (treeView) {
                    table.reexpandRows();
                    expandId && table.tree.expand(expandId);
                    table.disableParentRows();
                }
                else {
                    table.showFilter(); // Show filter controls on flat view.
                }
            };
            this._poolTable.body.refresh().then(callback((this.viewMode == this.BY_ROOT), this._poolTable, expandId));
        },

        // Connect to data source and create models for
        // TreeTable widgets.
        _retrieveStore: function() {

            var locResources = this.locResources;
            var poolData = "";
            var loopcount = 0;
            var items = new Array; // Array that holds only objects with children
            
            var ipObject = new Object();
  
            dojo.xhrGet({
                url: this.REST_PATH + "ippool/list/",
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                preventCache: true,
                sync: true,
                load: function(response) {
                    var numOfIPPools = response.items.length;

                    // IP Address Pool Objects
                    ipObject.id = "ip";
                    var ipchildrenArray = new Array();
                    var ipCount = 0;

                    for (loopcount = 0; loopcount < numOfIPPools; loopcount++) {
                        var childObject = new Object();
                        childObject.id = response.items[loopcount].id;
                        childObject.type = "ip";
                        childObject.poolType = "ip";
                        childObject.uri = response.items[loopcount].uri;
                        childObject.name = response.items[loopcount].name;
                        childObject.inUse = response.items[loopcount].inUse;
                        childObject.custom = response.items[loopcount].custom;
                        childObject.description = response.items[loopcount].description;
                        childObject.allocatedPct = response.items[loopcount].allocatedPct;
                        ipchildrenArray[ipCount] = childObject;
                        ipCount++;
                    }
                    if (ipCount != 0) {
                        ipObject.name = locResources.IP_POOLS;
                        ipObject.type = "root";
                        ipObject.children = ipchildrenArray;
                        items.push(ipObject);
                    }

                }
            });

            // Ethernet Address Pool Objects
            var ethObject = new Object();     //Eth Object to build json for table list
            ethObject.id = "eth";
            ethObject.name = locResources.ETH_POOLS;
            ethObject.type = "root";
            ethObject.children = new Array();

            var ethernetPools = null;       // Array of Ethernet Vendors
            dojo.xhrGet({     // Get list of Ethernet vendors for "children" array
                url: this.REST_PATH + "addresspool/listethernetvendors/",
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                preventCache: true,
                sync: true,
                load: function(response) {
                    ethernetPools = response.items;    //set vendors list
                },
                error: function(error) {
                    console.error("Error (", error, ") occurred when retrieving Ethernet address pool list.");
                }
            });
            dojo.forEach(ethernetPools, dojo.hitch(this, function(pool) {
                var childObject = new Object();                                       //to add to table
                childObject.name = pool.name;
                childObject.type = "eth";
                childObject.poolType = "eth";
                childObject.id = "eth::" + pool.id;
                childObject.description = pool.description;
                dojo.xhrGet({
                    url: this.REST_PATH + "addresspool/getpoolranges/ETHERNET/"+ pool.id,
                    handleAs: "json",
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    sync: true,
                    preventCache: true,
                    load: function(response) {
                        childObject.inUse = response.items[0].inUse;
                        childObject.allocatedPct = response.items[0].allocated;
                    },
                    error: function(error) {
                        console.error("Error (", error, ") occurred when retrieving Ethernet pool range data.");
                        childObject.inUse = false;
                        childObject.allocatedPct = "0::0";
                    }
                });
                ethObject.children.push(childObject);
            }));
            items.push(ethObject);    
            
            // Fibre Channel Address Pool Objects
            var fcObject = new Object();    //Fibre object to build json for table list
            fcObject.id = "fc";
            fcObject.name = locResources.FC_POOLS;
            fcObject.type = "root";
            fcObject.children = new Array();

            var fibrePools = null;            // Array of Fibre Channel Vendors
            dojo.xhrGet({      //Get list of fibre vendors
                url: this.REST_PATH + "addresspool/listfcvendors/",
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                preventCache: true,
                sync: true,
                load: function(response) {
                    fibrePools = response.items;
                },
                error: function(error) {
                    console.error("Error (", error, ") occurred when retrieving Fibre Channel address pool list.");
                }
            });
            dojo.forEach(fibrePools, dojo.hitch(this, function(pool) {
                var childObject = new Object();
                childObject.name = pool.name;
                childObject.type = "fc";
                childObject.poolType = "fc";
                childObject.id = pool.id;
                childObject.description = pool.description;
                dojo.xhrGet({
                    url: this.REST_PATH + "addresspool/getpoolranges/FIBRE/"+ pool.id,
                    handleAs: "json",
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    sync: true,
                    preventCache: true,
                    load: function(response) {
                        childObject.inUse = response.items[0].inUse;
                        childObject.allocatedPct = response.items[0].allocated; 
                    },
                    error: function(error) {
                        console.error("Error (", error, ") occurred when retrieving Fibre Channel pool range data.");
                        childObject.inUse = false;
                        childObject.allocatedPct = "0::0";
                    }
                });
                fcObject.children.push(childObject);
            }));
            items.push(fcObject);   

            if (this.viewMode == this.BY_ROOT) {
                poolData = dojo.toJson({    //Json to load into table list
                    "identifier": "id",
                    "label": "name",
                    "items": items
                }, true);
                
            }
            else if (this.viewMode == this.FLAT) {
                var flatItems = new Array();
                var i = 0;

                if(ipObject.children)
                for (i = 0; i < ipObject.children.length; i++) {
                    flatItems.push(ipObject.children[i]);
                }

                if(ethObject.children)
                for (i = 0; i < ethObject.children.length; i++) {
                    flatItems.push(ethObject.children[i]);
                }
                
                if(fcObject.children)
                for (i = 0; i < fcObject.children.length; i++) {
                    flatItems.push(fcObject.children[i]);
                }
                
                poolData = this._rootToFlat(flatItems);
            }
            
            var poolStore = new dojo.data.ItemFileWriteStore({
                data: dojo.fromJson(poolData)
            });

            if (!("hasChildren" in poolStore)) {
                poolStore.hasChildren = function(id, item) { // Need hasChildren function for TreeTable.
                    return item && this.getValues(item, "children").length;
                };
            };
            
            if (!("getChildren" in poolStore)) {
                poolStore.getChildren = function(item) { // Need getChildren function for TreeTable.
                    return this.getValues(item, "children");
                };
            };

            return poolStore;
            
        },

        // Load toolbar buttons/menus
        _loadToolBar: function(grid) {
            
            var toolbar = grid && grid.toolBar && grid.toolBar.widget;
            
            if (toolbar) {
                if (this._poolTable == null) {
                    // Profile table not created. Nothing to
                    // do but return.
                    return;
                }
    
                if (!this.actionMenu) {
                    this.actionMenu = new dijit.Menu({
                        style: "display: none;"
                    });
                }
    
                // Add custom Select View button
                var selectViewMenu = new dijit.Menu({
                    style: "display: none;"
                });
    
                var byTypeItem = new dijit.MenuItem({
                    //id: this.id + "cp_addresspools_bytypeitem",
                    label: this.resources.BY_TYPE,
                    title: this.locResources.POOL_BY_TYPE_DESC,
                    viewMode: this.BY_ROOT,
                    onClick: dojo.hitch(this, function() {
                        this._setViewByMode(this.BY_ROOT);
                    })
                });
                selectViewMenu.addChild(byTypeItem);
                
                var flatListItem = new dijit.MenuItem({
                    //id: this.id + "cp_addresspools_flatlistitem",
                    label: this.resources.FLAT,
                    title: this.locResources.POOL_FLAT_LIST_DESC,
                    viewMode: this.FLAT,
                    onClick: dojo.hitch(this, function() {
                        this._setViewByMode(this.FLAT);
                    })
                });
                selectViewMenu.addChild(flatListItem);
                
                this._selectViewButton = new dijit.form.DropDownButton({
                    //id: this.id + "cp_addresspools_selectviewbutton",
                    label: this.resources.BY_TYPE,
                    title: this.resources.TABLE_VIEW,
                    dropDown: selectViewMenu
                });
                toolbar.addChild(this._selectViewButton);
                
                toolbar.addChild(new dijit.ToolbarSeparator());
    
                // Add Create New Address Pool Button
    
                this.createButton = new dijit.form.Button({
                    //id: this.id + "cp_addresspools_createbutton",
                    showLabel: false,
                    iconClass: "spCreateLgIcon",
                    title: this.locResources.CREATE_IP,
                    onClick: dojo.hitch(this, this.createIPPool),
                    onKeypress: dojo.hitch(this, this.createIPPool)    
                });
                toolbar.addChild(this.createButton);
    
                // Add custom Edit button...
    
                this.editButton = new dijit.form.Button({
                    //id: this.id + "cp_addresspools_editbutton",
                    showLabel: false,
                    iconClass: "spEditInactiveLgIcon",
                    disabled: true,
                    title: this.resources.EDIT_SELECTED_ADDR_POOL,
                    onClick: dojo.hitch(this, this.editPool),
                    onKeypress: dojo.hitch(this, this.editPool)
                });
                toolbar.addChild(this.editButton);
    
                // Add custom Delete button...
    
                this.deleteButton = new dijit.form.Button({
                    //id: this.id + "cp_addresspools_deletebutton",
                    showLabel: false,
                    iconClass: "spDeleteInactiveLgIcon",
                    disabled: true,
                    title: this.locResources.DELETE_POOLS,
                    onClick: dojo.hitch(this, this.deletePools),
                    onKeypress: dojo.hitch(this, this.deletePools)
                });
                toolbar.addChild(this.deleteButton);
                
                toolbar.addChild(new dijit.ToolbarSeparator());
    
                // Add to Actions Menu
                this.expandAllItem = new dijit.MenuItem({
                    //id: this.id + "cp_addresspools_expandallitem",
                    label: this.resources.EXPAND_ALL,
                    iconClass: "spExpandAllLgIcon",
                    title: this.resources.EXPAND_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._poolTable.expandAll();
                    }),
                    onKeypress: dojo.hitch(this, function() {
                        this._poolTable.expandAll();
                    })
                });
                this.actionMenu.addChild(this.expandAllItem,0);
    
                this.collapseAllItem = new dijit.MenuItem({
                    //id: this.id + "cp_addresspools_collapseallitem",
                    label: this.resources.COLLAPSE_ALL,
                    iconClass: "spCollapseAllLgIcon",
                    title: this.resources.COLLAPSE_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._poolTable.collapseAll();
                    }),
                    onKeypress: dojo.hitch(this, function() {
                        this._poolTable.collapseAll();
                    })
                });
                this.actionMenu.addChild(this.collapseAllItem,1);                
                
                this.createItem = new dijit.MenuItem({
                    label: this.resources.CREATE,
                    iconClass: "spCreateLgIcon",
                    title: this.locResources.CREATE_IP,
                    onClick: dojo.hitch(this, this.createIPPool),
                    onKeypress: dojo.hitch(this, this.createIPPool)
                });
                this.actionMenu.addChild(this.createItem);
                
                var menuSeparator = new dijit.MenuSeparator();
                this.actionMenu.addChild(menuSeparator);
    
                this.renameItem = new dijit.MenuItem({
                    //id: this.id + "cp_addresspools_renameitem",
                    label: this.resources.RENAME,
                    title: this.resources.RENAME_SELECTED_ADDR_POOL,
                    onClick: dojo.hitch(this, this.renamePool),
                    onKeypress: dojo.hitch(this, this.renamePool)
                });                
    
                this.detailsItem = new dijit.MenuItem({
                    //id: this.id + "cp_addresspools_detailsitem",
                    label: this.resources.DETAILS,
                    title: this.locResources.POOL_DETAILS,
                    onClick: dojo.hitch(this, this.detailPool),
                    onKeypress: dojo.hitch(this, this.detailPool)
                });
                
                //Add action menu to action button and create action button if not already created
                if (this._actionButton) {
                    this._actionButton.dropDown = this.actionMenu;
                }
                else {
                    this._actionButton = new dijit.form.DropDownButton({
                        //id: this.id + "cp_addresspools_actionbutton",
                        showLabel: true,
                        label: this.resources.ALL_ACTIONS,
                        title: this.resources.ALL_ACTIONS_TITLE,
                        dropDown: this.actionMenu
                    });   
                    
                    toolbar.addChild(this._actionButton);
                }

            }
        },

        createIPPool: function() {
            var ipPooldialog = new support.dialogs.NewIPPool({
                id: this.id + "cp_addresspools_newippool"
            });
            ipPooldialog.show();
            return false;
        },

        detailPool: function() {
            var poolType = this._selectedPool.poolType.toString();
            var poolID = this._selectedPool.id.toString();
            var poolVendor = this._selectedPool.name.toString();
            
            if (poolType == "ip" || poolType == "root") {

           
                require(["support/panels/PoolDetails"], dojo.hitch(this,function() {
                    
                    if(this.CP_RUN_ENVIRONMENT == "fsm"){

                        this.loadNGUIWorkAreaWithCPWidget("support/panels/PoolDetails",null,poolID);
                    
                    }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                        var launchParams = {templateIDNum: poolID};
                        
                        this.loadAugustaWorkAreaTask("config.detail.page","support/panels/PoolDetails",launchParams);
                        
                    }
                
                }));

            }else if (poolType == "eth") {

                if (poolVendor == "QLogic WWN Addresses") {
                    poolVendor = "Qlogic";
                }

                if (poolVendor == "IBM MAC Addresses") {
                    poolVendor = "IBM";
                }

                require(["support/panels/EthPoolDetails"], dojo.hitch(this,function() {
                    
                    if(this.CP_RUN_ENVIRONMENT == "fsm"){

                        this.loadNGUIWorkAreaWithCPWidget("support/panels/EthPoolDetails",null,poolVendor);
                    
                    }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                        var launchParams = {templateIDNum: poolVendor};
                        
                        this.loadAugustaWorkAreaTask("config.detail.page","support/panels/EthPoolDetails",launchParams);
                        
                    }
                
                }));

            }       
            else if (poolType == "fc") {

                if (poolVendor == "QLogic WWN Addresses") {
                    poolVendor = "Qlogic";
                }

                if (poolVendor == "IBM WWN Addresses") {
                    poolVendor = "IBM";
                }

                if (poolVendor == "Emulex WWN Addresses") {
                    poolVendor = "Emulex";
                }

                if (poolVendor == "Brocade WWN Addresses") {
                    poolVendor = "Brocade";
                }
                
                require(["support/panels/FibrePoolDetails"], dojo.hitch(this,function() {
                    
                    if(this.CP_RUN_ENVIRONMENT == "fsm"){

                        this.loadNGUIWorkAreaWithCPWidget("support/panels/FibrePoolDetails",null,poolVendor);
                    
                    }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                        var launchParams = {templateIDNum: poolVendor};
                        
                        this.loadAugustaWorkAreaTask("config.detail.page","support/panels/FibrePoolDetails",launchParams);
                        
                    }
                
                }));

            }

        },

        // Edit Pool
        editPool: function() {
            var poolType = this._selectedPool.poolType.toString();
            var poolID = this._selectedPool.id.toString();
            var poolVendor = this._selectedPool.name.toString();

            if (poolType == "ip" || poolType == "root") {

                var ipeditdialog = new support.dialogs.NewIPPool({
                    id: this.id + "cp_addresspools_newippool",
                    poolIDNum: poolID,
                    editMode: true
                });
                ipeditdialog.show();

            }
            else if (poolType == "eth") {

                if (poolVendor == "QLogic WWN Addresses") {
                    poolVendor = "Qlogic";
                }

                if (poolVendor == "IBM MAC Addresses") {
                    poolVendor = "IBM";
                }

                var etheditdialog = new support.dialogs.EditEthernetAddressPool({
                    id: this.id + "cp_addresspools_ethernetaddresspool",
                    templateIDNum: poolID,
                    editMode: true
                });
                etheditdialog.show();

            }
            else if (poolType == "fc") {

                if (poolVendor == "QLogic WWN Addresses") {
                    poolVendor = "Qlogic";
                }

                if (poolVendor == "IBM WWN Addresses") {
                    poolVendor = "IBM";
                }

                if (poolVendor == "Emulex WWN Addresses") {
                    poolVendor = "Emulex";
                }

                if (poolVendor == "Brocade WWN Addresses") {
                    poolVendor = "Brocade";
                }

                var fceditdialog = new support.dialogs.FibreChannelAddressPool({
                    vendor: poolVendor
                });
                fceditdialog.show();

            }
        },

        // Delete Pools
        deletePools: function() {
            var deleteIds = new Array();
            var htmlNames = new Array();
            var ids = this._poolTable.select.row.getSelected();

            for (var loopVar = 0; loopVar < ids.length; loopVar++) {
                var item = this._poolTable.row(ids[loopVar], true).item();
                deleteIds.push(item.id);
                var htmlName = this.encodeForHtml(item.name.toString());
                htmlNames.push(htmlName);
            }

            var confirmedDelete = function() {

                dojo.forEach(deleteIds, dojo.hitch(this, function(id) {

                        var xhrDelete = dojo.xhrDelete({
                            url: this.REST_PATH + "ippool/" + id,
                            handleAs: "json",
                            headers: {
                                "Content-Type": "application/json; charset=utf-8"
                            },
                            sync: true,
                            preventCache: true,
                            load: function(response) {
                            },
                            error: dojo.hitch(this, function(error) {                                
                                switch(error.status) {
                                    case 400:
                                        var errorMessage = dojo.fromJson(error.responseText);                                        
                                        if (errorMessage.message == "service_error_pool_in_use") {                                            
                                            var errorDialog = new support.temp.ModalDialog({
                                                type: "Error",
                                                text: this.locResources.POOL_INUSE_ERROR
                                            });

                                            errorDialog.show();                                
                                        }
                                        break;
                                    default:                            
                                        var errorDialog = new support.temp.ModalDialog({
                                            type: "Error",
                                            text: error
                                        });

                                        errorDialog.show();
                                        break;
                                }                                 
                            })
                        });
                        
                }));

                this._refreshTree();
                
            };

            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.DELETE,
                        type: "Warning",
                        text: this.locResources.DELETE_TEXT,
                        info: this.locResources.DELETE_INFO                    
                    });

                    confirmDialog.set("content", "<p>" + this.locResources.DELETE_DESC + ":</p><p>" + htmlNames.join(", ") + "</p>");
                    confirmDialog.startup();
                    confirmDialog.confirm(confirmedDelete, this);
                
                }));
 
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.DELETE,
                        type: "Warning",
                        text: this.locResources.DELETE_TEXT,
                        info: this.locResources.DELETE_INFO                    
                    });

                    confirmDialog.set("content", "<p>" + this.locResources.DELETE_DESC + ":</p><p>" + htmlNames.join(", ") + "</p>");
                    confirmDialog.startup();
                    confirmDialog.confirm(confirmedDelete, this);
                
                }));
                
            }

            return false;
        },

        renamePool: function() {
            require([ "support/dialogs/Rename" ], dojo.hitch(this, function() {
                var renamedialog = new support.dialogs.Rename({
                    id: this.id + "cp_addresspools_rename",
                    templateIDNum: this._selectedPool.id.toString(),
                    ippoolMode: true
                });
                renamedialog.show();
            }));
        },
                
        _findPoolVendor: function(poolType, poolVendor) {
            if (poolType == "eth") {
                if (poolVendor == "QLogic WWN Addresses") {
                    return "Qlogic";
                }
                if (poolVendor == "IBM MAC Addresses") {
                    return "IBM";
                }
                
            } else if (poolType == "fc") {
                if (poolVendor == "QLogic WWN Addresses") {
                    return "Qlogic";
                }
                if (poolVendor == "IBM WWN Addresses") {
                    return "IBM";
                }
                if (poolVendor == "Emulex WWN Addresses") {
                    return "Emulex";
                }
                if (poolVendor == "Brocade WWN Addresses") {
                    return "Brocade";
                }
            }
        },
        
        _findPoolType: function(id) {
            if (id == "eth::IBM") {
                return "eth";
            } else if (id == "Brocade" || id == "Emulex" || id == "IBM" || id == "QLogic") {
                return "fc";
            }
        },

        // Set Table Formatters
        // Hard-coded path appended to URL, must be changed if path changes
        _decorateName: function(item, id) {
            // tags:
            // protected
            if (!item || !id) {
                return "";
            }
            
            var value = item;
            
            if(id == "ip" || id == "eth" || id == "fc"){
                return "<span title=\"" + value + "\" style=\"white-space:normal;\">" + value + "</span>";
            }
            if(!isNaN(id)){
                if(this.CP_RUN_ENVIRONMENT == "fsm"){
                    return "<a href=\"#workAreaDijit=support%2Fpanels%2FPoolDetails&path=support:%2Fcom.ibm.stg.sysx.profile." +
                        "navmodel%2Fjs%2Fsupport&cpTarget=" + id + "\">" + value + "</a>";
                } else if(this.CP_RUN_ENVIRONMENT == "augusta"){
                    return "<a href=\"#fwWidget=support%2Fpanels%2FPoolDetails&fwTaskId=config.detail.page&templateIDNum=" 
                        + id + "\">" + value + "</a>";
                }
            }
            var poolType = this._findPoolType(id);
            var poolVendor = this._findPoolVendor(poolType, value);
            if (poolType == "eth") {
                if(this.CP_RUN_ENVIRONMENT == "fsm"){
                    return "<a href=\"#workAreaDijit=support%2Fpanels%2FEthPoolDetails&path=support:%2Fcom.ibm.stg.sysx.profile." +
                    	"navmodel%2Fjs%2Fsupport&cpTarget=" + poolVendor + "\">" + value + "</a>";
                } else if(this.CP_RUN_ENVIRONMENT == "augusta"){
                    return "<a href=\"#fwWidget=support%2Fpanels%2FEthPoolDetails&fwTaskId=config.detail.page&templateIDNum=" 
                        + poolVendor + "\">" + value + "</a>";
                }
            } else if (poolType == "fc") {
                if(this.CP_RUN_ENVIRONMENT == "fsm"){
                    return "<a href=\"#workAreaDijit=support%2Fpanels%2FFibrePoolDetails&path=support:%2Fcom.ibm.stg.sysx.profile." +
                    	"navmodel%2Fjs%2Fsupport&cpTarget=" + poolVendor + "\">" + value + "</a>";
                } else if(this.CP_RUN_ENVIRONMENT == "augusta"){
                    return "<a href=\"#fwWidget=support%2Fpanels%2FFibrePoolDetails&fwTaskId=config.detail.page&templateIDNum=" 
                        + poolVendor + "\">" + value + "</a>";
                }
            }
        },
        
        _formatName: function(item) {
            
            if (!item || !item.name) {
                return "";
            }
            
            var value = this.decodeSpecialCharacters(item.name);
            value = this.encodeForHtml(value);
            
            return value;
        },
        
        _decorateInUse: function(item) {            
            if(item == "") {
                return "";
            }
                                    
            if (item == this.resources.IN_USE) {                
                return  "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" +
                            "<tbody>" +
                                "<tr title=\"" + this.locResources.POOL_INUSE + "\">" +
                                    "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spInUseIcon\"></img></td>" +
                                    "<td><span style=\"margin-left:4px; white-space: nowrap;\">" + this.resources.IN_USE + "</span></td>" +                                        
                                "</tr>" +
                            "</tbody>" +
                        "</table>";                                           
            }                        
            return  "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" +
                        "<tbody>" +
                		    "<tr title=\"" + this.locResources.POOL_NOT_INUSE + "\">" +
                		        "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spNotInUseIcon\"></img></td>" +
                		        "<td><span style=\"margin-left:4px; white-space: nowrap;\">" + this.resources.NOT_IN_USE + "</span></td>" +
                		    "</tr>" +
                		"</tbody>" +
                	"</table>";
        },
        
        _formatInUse: function(item) {
            var inUse = item.inUse;
            
            if (item.type == "root" && this.viewMode == this.BY_ROOT) {
                return "";
            }
            
            if (inUse) {
                return this.resources.IN_USE;
            }
            return this.resources.NOT_IN_USE;
        },

        _decorateCustom: function(item) {
            if (item == "") {
                return "";
            }
            
            if (item == this.resources.USER_DEFINED) {                                
                return  "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" +
                            "<tbody>" +
                                "<tr title=\"" + this.locResources.USERDEF_POOL + "\">" +
                                    "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spUserDefinedIcon\"></img></td>" +
                                    "<td><span style=\"margin-left:4px; white-space: nowrap;\">" + this.resources.USER_DEFINED + "</span></td>" +
                                "</tr>" +
                            "</tbody>" +
                        "</table>";
            }
            return  "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" +
                        "<tbody>" +
                            "<tr title=\"" + this.locResources.PREDEF_POOL + "\">" +
                                "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spPredefinedIcon\"></img></td>" +
                                "<td><span style=\"margin-left:4px; white-space: nowrap;\">" + this.resources.PREDEFINED + "</span></td>" +
                            "</tr>" +
                        "</tbody>" +
                    "</table>";
        },
        
        _formatCustom: function(item) {
            var isCustom = item.custom;
            
            if (item.type == "root" && this.viewMode == this.BY_ROOT) {
                return "";
            }
            
            if (isCustom) {
                return this.resources.USER_DEFINED;
            }
            return this.resources.PREDEFINED;
        },

        _decorateAllocation: function(item) {
            return "<span title=\"" + item + "\" style=\"white-space:nowrap;\">" + item + "</span>";
        },

        _formatAllocation: function(item) {
            
            if (item.type == "root" && (this.viewMode == this.BY_ROOT)) {
                return "";
            }

            var value = item.allocatedPct;
            var percentVal = 0;
            var alloc = 0;
            var total = 0;
            
            var strVal = new String(value);
            var sepIdx = strVal.indexOf("::");
            if (sepIdx > 0) {
                alloc = strVal.substring(0, sepIdx);
                total = strVal.substring(sepIdx + 2);
                percentVal = (alloc / total) * 100;
                if (percentVal < 1 && percentVal > 0) {  //If number smaller then 1% just show 1%
                    percentVal = 1;
                }
            }
            
            value = this.locResources.ALLOCATED_DESC.replace("%1", percentVal).replace("%2", alloc).replace("%3", total);
            value = this.encodeForHtml(value);
            return value;

        },
        
        _decorateDescription: function(item) {            
            return "<span title=\"" + item + "\" style=\"white-space:normal;\">" + item + "</span>";
        },

        _formatDescription: function(item) {
            var value = item.description;
            
            if (!value) {
                return "";
            }
            value = this.decodeSpecialCharacters(value);
            value = this.encodeForHtml(value);
            return value;
        },
        
        _decorateType: function(item) {
            return "<span title=\"" + item + "\" style=\"white-space:nowrap;\">" + item + "</span>";
        },
        
        _formatType: function(item) {
            
            var value = item.poolType;
            
            switch (value) {
                case "eth":
                    value = this.resources.POOL_ETHERNET;
                    break;
                case "fc":
                    value = this.resources.POOL_FC;
                    break;
                case "ip":
                    value = this.resources.POOL_IP;
                    break;
                default:
                    value = "";
                    break;
            }
            value = this.encodeForHtml(value);
            return value;
        },
        
        _setViewByMode: function(menuItem) {
            // tags:
            // protected
            if (this.viewMode == menuItem) {
                return;
            }           
            
            // Clear selected rows when switching views
            this._poolTable.select.row.clear();
            
            // Set Menu Button Label to Current View
            var label = "";
            if (menuItem == this.BY_ROOT) {
                label = this.resources.BY_TYPE;
                this.actionMenu.addChild(this.expandAllItem, 0);
                this.actionMenu.addChild(this.collapseAllItem, 1);
                this._poolTable.hideFilter(); // Hide filter controls on tree view.
            }
            else if (menuItem == this.FLAT) {
                label = this.resources.FLAT;
                this.actionMenu.removeChild(this.expandAllItem);
                this.actionMenu.removeChild(this.collapseAllItem);
                this._poolTable.showFilter(); // Show filter controls on flat view.
            }
            else { // Invalid view mode
                return;
            }

            // Disable Action Button if no children
            if (this.actionMenu.hasChildren()) {
                this._actionButton.set("disabled", false);
            } else {
                this._actionButton.set("disabled", true);                
            }

            // Reformat Table Data
            this._reRenderTable(menuItem);
            
            // Set Drop Down Menu Label
            this._selectViewButton.set("label", label);
            
            this.viewMode = menuItem;

        },
        
        _reRenderTable: function(nextViewMode) {
            // tags:
            // protected
            // Re-render Tree table with new columns, column spans, and data
            // presentation
            this.viewMode = nextViewMode;

            var poolStore = this._retrieveStore();
            var tableStructure = this._getTableStructure(); // Get structure for current view

            this._poolTable.setColumns(tableStructure);
            this._poolTable.setStore(poolStore);
            this._poolTable.model.clearCache();
            this._poolTable.body.refresh();

            if (this.viewMode == this.BY_ROOT) {
                this._poolTable.reexpandRows();
            }            
        },                       
        _rootToFlat : function(response) {
            var newItems = new Array();
            var loopcount = 0;
            
            for (loopcount = 0; loopcount < response.length; loopcount++) {
                if(!response[loopcount]) {
                    continue;
                }
                if(response[loopcount].type == "root") {
                    continue;
                }
                var childObject = new Object();
                childObject.id = response[loopcount].id;
                childObject.name = response[loopcount].name;
                childObject.type = "root";
                childObject.inUse = response[loopcount].inUse;
                childObject.description = response[loopcount].description;
                childObject.poolType = response[loopcount].poolType;
                childObject.allocatedPct = response[loopcount].allocatedPct;
                childObject.custom = response[loopcount].custom;
                newItems.push(childObject);                
            }
            
            var newData = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": newItems
            }, true);                      

            return newData;
        },
        
        _getTableStructure: function() {

            var includeType = (this.viewMode == this.FLAT);
            
            var layout = new Array();
            
            layout.push({
                field: "name",
                name: this.locResources.POOL_HEADING,
                width: "20em",
                formatter: dojo.hitch(this, this._formatName),
                decorator: dojo.hitch(this, this._decorateName),
                title: ""
            });
            layout.push({
                field: "inUse",
                name: this.resources.USAGE_STATUS,
                width: "8em",
                formatter: dojo.hitch(this, this._formatInUse),
                decorator: dojo.hitch(this, this._decorateInUse),
                title: ""
            });
            layout.push({
                field: "custom",
                name: this.resources.POOL_ORIGIN,
                width: "8em",
                formatter: dojo.hitch(this, this._formatCustom),
                decorator: dojo.hitch(this, this._decorateCustom),
                title: ""
            });
            layout.push({
                field: "allocatedPct",
                name: this.resources.ALLOCATED,
                width: "20em",
                formatter: dojo.hitch(this, this._formatAllocation),
                decorator: dojo.hitch(this, this._decorateAllocation),
                title: ""
            });
            (includeType) && layout.push({
                field: "poolType",
                name: this.locResources.POOL_TYPE,
                width: "8em",
                formatter: dojo.hitch(this, this._formatType),
                decorator: dojo.hitch(this, this._decorateType),
                title: this.locResources.POOL_TYPE
            });
            layout.push({
                field: "description",
                name: this.resources.DESCRIPTION,
                width: "auto",
                formatter: dojo.hitch(this, this._formatDescription),
                decorator: dojo.hitch(this, this._decorateDescription),
                title: ""
            });

            return layout;
            
        },

        // Set Table Capabilities
        setTableCapabilities: function() {

            if (!this._poolTable) {
                return; // nothing to do. return now
            }

            this.deleteButton.set("iconClass", "spDeleteInactiveLgIcon");
            this.deleteButton.set("disabled", true);            
            this.createButton.set("iconClass", "spCreateLgIcon");
            this.createButton.set("disabled", false); 

            var ids = this._poolTable.select.row.getSelected();
            
            var menuPosition = 2;
            
            var allowEdit = (ids.length == 1); // assume okay, disprove below
            var allowDelete = (ids.length > 0); // assume okay, disprove below

            var allowDetails = (ids.length == 1); // allow details menu item

            for (var loopVar = 0; loopVar < ids.length; loopVar++) {

                var item = this._poolTable.row(ids[loopVar], true).item();
                this._selectedPool = item;
                            
                if ((item.type == "root") && (this.viewMode != this.FLAT)) {
                    // Root selection not allowed.
                    allowEdit = allowDelete = false;
                }
                if ((item.type != "ip") && (this.viewMode != this.FLAT)) {
                    // Only IP pool selection allowed for edit.
                    allowEdit = allowDelete = false;
                }
                                
                if ((item.custom)&&(item.inUse)&&(item.custom[0] != true || item.inUse[0] == true)){
                   
                    allowDelete = false;
                }
                   
                if(item.custom == undefined){
                    allowEdit = allowDelete = false;                    
                }
                
            };
                        
                if (allowEdit) {

                    this.editButton.set("iconClass", "spEditLgIcon");
                    this.editButton.set("disabled", false);

                    if (!this.editItem) {

                        this.editItem = new dijit.MenuItem({
                           // id: this.id + "cp_addresspools_edititem",
                            label: this.resources.EDIT,
                            title: this.resources.EDIT_SELECTED_ADDR_POOL,
                            iconClass: "spEditLgIcon",
                            onClick: dojo.hitch(this, this.editPool),
                            onKeypress: dojo.hitch(this, this.editPool)
                        });

                    }

                    if (!this.deleteItem) {

                        this.deleteItem = new dijit.MenuItem({
                          //  id: this.id + "cp_addresspools_deleteitem",
                            label: this.resources.DELETE,
                            title: this.locResources.DELETE_POOLS,
                            iconClass: "spDeleteLgIcon",
                            onClick: dojo.hitch(this, this.deletePools),
                            onKeypress: dojo.hitch(this, this.deletePools)
                        });

                    }

                    if (!this.renameItem) {

                        this.renameItem = new dijit.MenuItem({
                           // id: this.id + "cp_addresspools_renameitem",
                            label: this.resources.RENAME,
                            title: this.resources.RENAME_SELECTED_ADDR_POOL,
                            onClick: dojo.hitch(this, this.renamePool),
                            onKeypress: dojo.hitch(this, this.renamePool)
                        });
                        
                    }

                    this.actionMenu.addChild(this.renameItem);
                    this.actionMenu.addChild(this.editItem, ((this.viewMode == this.FLAT)? menuPosition : menuPosition+2));
                    menuPosition++;
                    
                }else {
                    this.editButton.set("iconClass", "spEditInactiveLgIcon");
                    this.editButton.set("disabled", true);

                    this.actionMenu.removeChild(this.renameItem);
                    this.renameItem = null;
                    
                    this.actionMenu.removeChild(this.editItem);
                    this.editItem = null;
                    
                }                                  
                
                if (allowDelete) {
                    this.deleteButton.set("iconClass", "spDeleteLgIcon");
                    this.deleteButton.set("disabled", false);                                                                           
					
					this.actionMenu.addChild(this.deleteItem, ((this.viewMode == this.FLAT)? menuPosition : menuPosition+2));
					menuPosition++;

                }else {
                    this.deleteButton.set("iconClass", "spDeleteInactiveLgIcon");
                    this.deleteButton.set("disabled", true);
                    
                    this.actionMenu.removeChild(this.deleteItem);
                    this.deleteItem = null;
                }
                
                if(allowDetails){

                    if (!this.detailsItem) {

                        this.detailsItem = new dijit.MenuItem({
                          //  id: this.id + "cp_addresspools_detailsitem",
                            label: this.resources.DETAILS,
                            title: this.locResources.POOL_DETAILS,
                            onClick: dojo.hitch(this, this.detailPool),
                            onKeypress: dojo.hitch(this, this.detailPool)
                        });

                    }

                    this.actionMenu.addChild(this.detailsItem, ((this.viewMode == this.FLAT)? menuPosition : menuPosition+2));
                    menuPosition++;
                    
                }else{
                    
                    this.actionMenu.removeChild(this.detailsItem);
                    this.detailsItem = null;

                }

                //Disable Action Button if no children
                if(this.actionMenu.hasChildren()){
                    this._actionButton.set("disabled", false);
                }else{
                    this._actionButton.set("disabled", true);                
                }
            
        },        
        goForeground: function(params) { //Page base function that fires when the page is brought to focus
            
            this._refreshTree();

            return true;
        }
        
    });
});