define([ "dojo",
         "dijit",
         "support/panels/Panel",
         "dojo/i18n!./nls/BootOrderTable",
         "dojo/text!./templates/BootOrderTable.html",
         "dijit/form/ValidationTextBox",
         "dijit/form/NumberTextBox",
         "dojo/data/ItemFileWriteStore",
         "dojo/data/ItemFileReadStore",
         "support/base/TableMixin",
         "support/widgets/Icon",
         "dijit/form/Button",
         "dijit/form/Select",
         "support/widgets/Select",
         "dijit/Tooltip",
         "dijit/form/RadioButton"
    ], function (dojo, dijit, panel, mri, template) {

	dojo.declare("support.panels.BootOrderTable", [ support.panels.Panel, support.base.TableMixin ], {

	    // templateString: [protected] String
	    // This widget's HTML template, provided.
	    templateString: template,
	        
	    // locResources: [protected] Object
	    // This widget's National Language Source (NLS) object.
	    locResources: mri, 
	    
		widgetsInTemplate : true,
		baseClass: "BootOrderTable",
			
        // ID prefix used for table
        gridID: "BootOrderTable",

        SANboot: false,
        jsonData: null,
        pageParams: null,
        learnedOptions: null,

		_bootGrid: null,
		isHBA: false,
		_HBAOptionCache: new Object(),

		// SAN boot
		_bootTargetPatternSelector: null,
		_bootTargetPattern: null,
		DEVICE_NONE: "*NONE",
		DEVICE_CD_DVD: "CD/DVD Rom",
		DEVICE_DISKETTE: "Floppy Disk",
		DEVICE_FC: "Fibre Channel (SAN)",
		DEVICE_HD: "Hard disk",
        DEVICE_HYPERVISOR: "Embedded Hypervisor",
		DEVICE_ISCSI: "iSCSI",
		DEVICE_ISCSI_CRITICAL: "iSCSI Critical",
		DEVICE_LEGACY_ONLY: "Legacy Only",
		DEVICE_PXE: "PXE Network",
		DEVICE_USB: "USB",
        DEVICE_WBM: "Windows Boot Manager",
        BOOT_ORDER_KEEP_OPTION: "Keep",
        BOOT_ORDER_LEGACY_OPTION: "Legacy",
		_messageArea: null,
		bootOrderRadioValue: null,
		_editDefaultKeep: null,
		
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/BootOrderTable.css",

		postCreate : function(){
		    this.inherited(arguments);
		    
		    if (this.SANboot) {
                // This path is for the SAN boot tab
                // Need to verify that virtual addressing is enabled and that an FC device is configured
                if (this._isVAddressingEnabled()) {
                    if (this._getBootDeviceOptions(this.DEVICE_FC).length > 0) {
                        this._displaySANBootTable();
                        this._setupBootTargetSelector();
                    } else {
                        this._displayInformationMessage(this.locResources.VIRT_ADDR_NO_ADAP_DETAILS, this.locResources.VIRT_ADDR_NO_ADAP_TITLE);
                    }
                } else {
                    this._displayInformationMessage(this.locResources.VIRT_ADDR_DISABLED_DETAILS, this.locResources.VIRT_ADDR_DISABLED_TITLE);
                }                
                // Initialize cache data for "*NONE" device option.
                this._HBAOptionCache[this.DEVICE_NONE] = {
                        deviceType: this.DEVICE_NONE,
                        deviceSlot: this.DEVICE_NONE,
                        devicePort: this.DEVICE_NONE,
                        functionNumber: this.DEVICE_NONE
                };
            } else {
		        // This path is for the primary or wol tab
		        this.bootorder_keep.set("checked", true);
		        
		        if (this.jsonData != null) {
		            if (this.jsonData.length > 0) {
		                // SAN Boot options are commingled with Boot Order in pattern data,
		                // however they are handled separately in UI so filter them accordingly.		                
		                var primaryBootOrderItems = this._filterBootOrderItems(this.jsonData);
		                if (primaryBootOrderItems.length == 0) {
		                    this.bootorder_keep.set("checked", true);
		                    this.bootOrderRadioValue = "keep";
		                } else {
		                    this.bootorder_legacy.set("checked", true);
		                    this.bootOrderRadioValue = "legacy";
		                }		                
		            } else {
		                this.bootorder_keep.set("checked", true);
		                this.bootOrderRadioValue = "keep";
		            }
		        }
		        if (this.pageParams.bootOrder == "Legacy Only") { // NOTE: Must match value used by parent nstBoot.js dijit.
		            this._displayBootOrderTable();
		            if (this._bootGrid == null) {
		                this._createTable();
		            }
		        } else {
		            this._displayInformationMessage(this.locResources.BOOT_ORDER_MSG2, this.locResources.BOOT_ORDER_MSG1);
		        }
		    }
		},
	    
	    _getTableLayout: function() {
            // tags:
            // protected
	        
	        var layout = new Array();
    	    if (this.SANboot) {

                layout = [{
                    field: "device",
                    name: this.locResources.BOOT_DEVICE,
                    width: "auto",
                    sortable: false,
                    widgetsInCell: true,
                    decorator: function() {
                        // return cellWidget's HTML template
                        return "<div data-dojo-type=\"dijit.form.Select\" data-dojo-attach-point=\"devSelector\" style=\"width: 100%;\"></div>";
                    },
                    setCellValue: dojo.hitch(this, function(gridData, storeData, cellWidget) {
                        var rowItem = cellWidget.cell.row.item();
                        var bootOption = (rowItem.option) ? rowItem.option[0] : this.DEVICE_NONE;
                        if (bootOption == this.DEVICE_FC) {
                            var gridStore = cellWidget.cell.grid.store;
                            var bootDevices = this._getAvailableBootDevices(gridStore, bootOption, storeData, true);
                            var devStore = new dojo.data.ItemFileReadStore({
                                // Use a store so options are alphabetized (by the designated label)!
                                data: {
                                    identifier: "value",
                                    label: "label",
                                    items: bootDevices
                                }
                            });
                            var selectedValue = (storeData) ? storeData : ((bootDevices.length > 0) ? bootDevices[0].value : this.DEVICE_NONE);
                            cellWidget.devSelector.setStore(devStore, selectedValue);
                            cellWidget.devSelector.set("required", true);
                            cellWidget.devSelector.set("emptyLabel", "&mdash; " + this.locResources.NO_BOOT_DEVICE_AVAIL + " &mdash;");
                            cellWidget.devSelector.set("disabled", (bootDevices.length == 0));
                            dojo.style(cellWidget.devSelector.domNode, "display", "");  // Make visible, in case it was hidden previously.
                        }
                        else {
                            // No boot device support for selected option.
                            dojo.style(cellWidget.devSelector.domNode, "display", "none");
                        }
                    }),
                    getCellWidgetConnects: dojo.hitch(this, function(cellWidget, cell) {
                        // return an array of connection arguments
                        return [[cellWidget.devSelector, "onChange", dojo.hitch(this, function(newValue) {
                            var oldValue = cell.rawData();
                            console.log(this.declaredClass + ": onChange() device:", cell.row.index(), oldValue, newValue);
                            if (cellWidget.devSelector._cpIgnoreFirstChange === undefined) {
                                // 4/23/2014: The following is admittedly a hack, but it works!
                                // For some unknown reason the value of the cellWidget.devSelector dijit is changed to
                                // it's first option value when the dijit is initially displayed on screen. Two days
                                // have been spent trying to figure out how to keep the dijit set to it's initial 
                                // value as determined by the setCellValue() function but to no avail.
                                // So.... the following hack exists to detect this mysterious change and set it right again.
                                cellWidget.devSelector._cpIgnoreFirstChange = true;
                                cellWidget.devSelector.set("value", oldValue);
                                return;
                            }
                            if (oldValue != newValue) {
                                var deviceDetails = this._HBAOptionCache[newValue];
                                cell.row.setRawData({
                                    device: newValue,
                                    deviceType: deviceDetails.deviceType,
                                    devicePort: deviceDetails.devicePort,
                                    deviceSlot: deviceDetails.deviceSlot,
                                    functionNumber: deviceDetails.functionNumber,
                                });
                                cell.grid.body.refresh(); // Refresh entire grid to ensure other cellWidget's consider new values.
                            }
                        })]];
                    })
                }, {
                    field: "target",
                    name: this.locResources.BOOT_TARGET_PATTERN_TARGETS,
                    width: "auto",
                    sortable: false,
                    widgetsInCell: true,
                    decorator: function() {
                        // return cellWidget's HTML template
                        return "<div data-dojo-type=\"dijit.form.Select\" data-dojo-attach-point=\"tgtSelector\" style=\"width: 100%;\"></div>";
                    },
                    setCellValue: dojo.hitch(this, function(gridData, storeData, cellWidget) {
                        var rowItem = cellWidget.cell.row.item();
                        var targets = null;
                        if (this._bootTargetPattern.checkvalue) {
                            // Secondary Boot Targets are defined so include all options.
                            targets = [{
                                label: this.locResources.PRIMARY,
                                value: "primary"
                            }, 
                            {
                                label: this.locResources.SECONDARY,
                                value: "secondary"
                            },
                            {
                                label: this.locResources.ALL,
                                value: "all"
                            }];
                        } else {
                            // Primary Boot Targets is the only option.
                            targets = [{
                                label: this.locResources.PRIMARY,
                                value: "primary"
                            }];
                        }
                        var tgtStore = new dojo.data.ItemFileReadStore({
                            // Use a store so options are alphabetized (by the designated label)!
                            data: {
                                identifier: "value",
                                label: "label",
                                items: targets
                            }
                        });
                        var selectedValue = (rowItem.target) ? rowItem.target[0] : ((this._bootTargetPattern.checkvalue) ? "all" : "primary");
                        cellWidget.tgtSelector.setStore(tgtStore, selectedValue);
                        cellWidget.tgtSelector.set("emptyLabel", "&mdash; " + this.locResources.NO_TARGETS_DEFINED + " &mdash;");
                    }),
                    getCellWidgetConnects: dojo.hitch(this, function(cellWidget, cell) {
                        // return an array of connection arguments
                        return [[cellWidget.tgtSelector, "onChange", dojo.hitch(this, function(newValue) {
                            var oldValue = cell.rawData();
                            console.log(this.declaredClass + ": onChange() target:", cell.row.index(), oldValue, newValue);
                            if (oldValue != newValue) {
                                cell.row.setRawData({
                                    target: newValue
                                });
                            }
                        })]];
                    })
                }, {
                    field: "icons",
                    name: " ",
                    width: "140px",
                    sortable: false,
                    widgetsInCell: true,
                    decorator: function() {
                        // return cellWidget's HTML template
                        return "<table style=\"border:0;border-spacing:0;\"><tr>" +
                            "<td style=\"padding:0;\"><button data-dojo-type=\"support.widgets.Icon\" style=\"width:32px;height:30px;\" data-dojo-attach-point=\"upIcon\" iconClass=\"{ normal: 'spArrowUpIcon', disabled: 'spArrowUpInactiveIcon'}\"></button></td>" + 
                            "<td style=\"padding:0;\"><button data-dojo-type=\"support.widgets.Icon\" style=\"width:32px;height:30px;\" data-dojo-attach-point=\"downIcon\" iconClass=\"{ normal: 'spArrowDownIcon', disabled: 'spArrowDownInactiveIcon'}\"></button></td>" + 
                            "<td style=\"padding:0;\"><button data-dojo-type=\"support.widgets.Icon\" style=\"width:32px;height:30px;\" data-dojo-attach-point=\"addIcon\" iconClass=\"{ normal: 'spAddRowIcon_v2', disabled: 'spAddRowInactiveIcon_v2'}\"></button></td>" + 
                            "<td style=\"padding:0;\"><button data-dojo-type=\"support.widgets.Icon\" style=\"width:32px;height:30px;\" data-dojo-attach-point=\"removeIcon\" iconClass=\"{ normal: 'spDeleteIcon_v2', disabled: 'spDeleteInactiveIcon_v2'}\"></button></td>" +
                            "</tr></table>";
                    },
                    setCellValue: dojo.hitch(this, function(gridData, storeData, cellWidget) {
                        var rowIndex = cellWidget.cell.row.index();
                        var rowCount = cellWidget.cell.grid.rowCount();
                        var gridStore = cellWidget.cell.grid.store;
                        var disableUp = (rowIndex == 0);
                        var disableDown = (rowIndex == rowCount - 1);
                        var disableAdd = (this._getAvailableBootDevices(gridStore, this.DEVICE_FC).length == 0); // Disable adds when no devices are available!
                        var disableRemove = (rowCount <= 1); // Do not allow removal of last device!
                        cellWidget.upIcon.set("disabled", disableUp);
                        cellWidget.upIcon.set("title", this.locResources.MOVE_UP_TITLE);
                        cellWidget.downIcon.set("disabled", disableDown);
                        cellWidget.downIcon.set("title", this.locResources.MOVE_DOWN_TITLE);
                        cellWidget.addIcon.set("title", this.locResources.ADD_TITLE);
                        cellWidget.addIcon.set("disabled", disableAdd);
                        cellWidget.removeIcon.set("title", this.locResources.REMOVE_TITLE);
                        cellWidget.removeIcon.set("disabled", disableRemove);
                    }),
                    getCellWidgetConnects: dojo.hitch(this, function(cellWidget, cell) {
                        // return an array of connection arguments
                        return [[cellWidget.upIcon, "onClick", dojo.hitch(this, function() {
                            // Move row up one.
                            var rowIndex = cell.row.index();
                            console.log(this.declaredClass + ": onClick() upIcon:", rowIndex);
                            var newData = {
                                    items: []
                            };
                            var rows = cell.grid.rows();
                            for (var idx = 0; idx < rows.length; ++idx) {
                                var item = rows[idx].item();
                                var bump = 0;
                                if ((idx + 1) == rowIndex) {
                                    // Push next item first and decrement "order" value.
                                    ++idx, ++bump;
                                    var nextItem = rows[idx].item();
                                    newData.items.push({
                                        order: nextItem.order[0] - bump,
                                        option: nextItem.option[0],
                                        device: nextItem.device[0],
                                        deviceType: nextItem.deviceType[0],
                                        devicePort: nextItem.devicePort[0],
                                        deviceSlot: nextItem.deviceSlot[0],
                                        functionNumber: nextItem.functionNumber[0],
                                        template: nextItem.template[0],
                                        target: nextItem.target[0]
                                    });
                                }
                                newData.items.push({
                                    order: item.order[0] + bump,
                                    option: item.option[0],
                                    device: item.device[0],
                                    deviceType: item.deviceType[0],
                                    devicePort: item.devicePort[0],
                                    deviceSlot: item.deviceSlot[0],
                                    functionNumber: item.functionNumber[0],
                                    template: item.template[0],
                                    target: item.target[0]
                                });
                            }
                            var newStore = new dojo.data.ItemFileWriteStore({
                                data: newData
                            });
                            cell.grid.setStore(newStore); // cell.grid.body will refresh automatically
                        })],
                        [cellWidget.downIcon, "onClick", dojo.hitch(this, function() {
                            // Move row up one.
                            var rowIndex = cell.row.index();
                            console.log(this.declaredClass + ": onClick() downIcon:", rowIndex);
                            var newData = {
                                    items: []
                            };
                            var rows = cell.grid.rows();
                            for (var idx = 0; idx < rows.length; ++idx) {
                                var item = rows[idx].item();
                                var bump = 0;
                                if (idx == rowIndex) {
                                    // Push next item first.
                                    ++idx, ++bump;
                                    if (idx < rows.length) { // precautionary test
                                        var nextItem = rows[idx].item();
                                        newData.items.push({
                                            order: nextItem.order[0] - bump,
                                            option: nextItem.option[0],
                                            device: nextItem.device[0],
                                            deviceType: nextItem.deviceType[0],
                                            devicePort: nextItem.devicePort[0],
                                            deviceSlot: nextItem.deviceSlot[0],
                                            functionNumber: nextItem.functionNumber[0],
                                            template: nextItem.template[0],
                                            target: nextItem.target[0]
                                        });
                                    }
                                }
                                newData.items.push({
                                    order: item.order[0] + bump,
                                    option: item.option[0],
                                    device: item.device[0],
                                    deviceType: item.deviceType[0],
                                    devicePort: item.devicePort[0],
                                    deviceSlot: item.deviceSlot[0],
                                    functionNumber: item.functionNumber[0],
                                    template: item.template[0],
                                    target: item.target[0]
                                });
                            }
                            var newStore = new dojo.data.ItemFileWriteStore({
                                data: newData
                            });
                            cell.grid.setStore(newStore); // cell.grid.body will refresh automatically
                        })],
                        [cellWidget.addIcon, "onClick", dojo.hitch(this, function() {
                            // Insert a new row after the current one.
                            // Copy row items for a new data store with the new item in proper position.
                            var rowIndex = cell.row.index();
                            console.log(this.declaredClass + ": onClick() addIcon:", rowIndex);
                            var newData = {
                                    items: []
                            };
                            var rows = cell.grid.rows(), increase = 0;
                            for (var idx in rows) {
                                var item = rows[idx].item();
                                newData.items.push({
                                    order: item.order[0] + increase,
                                    option: item.option[0],
                                    device: item.device[0],
                                    deviceType: item.deviceType[0],
                                    devicePort: item.devicePort[0],
                                    deviceSlot: item.deviceSlot[0],
                                    functionNumber: item.functionNumber[0],
                                    template: item.template[0],
                                    target: item.target[0]
                                });
                                if (idx == rowIndex) {
                                    // Insert additional item for new row and increase for subsequent items.
                                    var newItem = this._nextAvailableSanBootDeviceItem(item.order[0] + 1);
                                    (newItem) && (newData.items.push(newItem));
                                    ++increase;
                                }
                            }
                            var newStore = new dojo.data.ItemFileWriteStore({
                                data: newData
                            });
                            cell.grid.setStore(newStore); // cell.grid.body will refresh automatically
                        })],
                        [cellWidget.removeIcon, "onClick", dojo.hitch(this, function() {
                            // Remove current row.
                            var rowIndex = cell.row.index();
                            console.log(this.declaredClass + ": onClick() removeIcon:", rowIndex);
                            var newData = {
                                    items: []
                            };
                            var rows = cell.grid.rows(), decrease = 0;
                            for (var idx in rows) {
                                var item = rows[idx].item();
                                if (idx == rowIndex) {
                                    ++decrease;
                                } else {
                                    newData.items.push({
                                        order: item.order[0] - decrease,
                                        option: item.option[0],
                                        device: item.device[0],
                                        deviceType: item.deviceType[0],
                                        devicePort: item.devicePort[0],
                                        deviceSlot: item.deviceSlot[0],
                                        functionNumber: item.functionNumber[0],
                                        template: item.template[0],
                                        target: item.target[0]
                                    });
                                }
                            }
                            var newStore = new dojo.data.ItemFileWriteStore({
                                data: newData
                            });
                            cell.grid.setStore(newStore); // cell.grid.body will refresh automatically
                        })]];
                    })
                }];                 
            } else {
    	        layout = [{
                    field: "order",
                    name: this.locResources.ORDER,
                    width: "5em",
                    sortable: false,
                    decorator: dojo.hitch(this, this._decorateOrder)
                }, {
                    field: "option",
                    name: this.locResources.BOOT_OPTION,
                    width: "auto",
                    sortable: false,
                    widgetsInCell: true,
                    decorator: function() {
                        // return cellWidget's HTML template
                        return "<div data-dojo-type=\"dijit.form.Select\" data-dojo-attach-point=\"optSelector\" style=\"width: 100%;\"></div>";
                    },
                    setCellValue: dojo.hitch(this, function(gridData, storeData, cellWidget) {
                        var store = cellWidget.cell.grid.store;
                        var rowItem = cellWidget.cell.row.item();
                        var bootOptions = this._getAvailableBootOptions(store, rowItem, this.learnedOptions);
                        var optStore = new dojo.data.ItemFileReadStore({
                            // Use a store so options are alphabetized (by the designated label)!
                            data: {
                                identifier: "value",
                                label: "label",
                                items: bootOptions
                            }
                        });
                        var selectedValue = (storeData) ? storeData : this.DEVICE_NONE;
                        cellWidget.optSelector.setStore(optStore, selectedValue);
                        cellWidget.optSelector.set("required", true);
                        cellWidget.optSelector.set("emptyLabel", "&mdash; " + this.locResources.NO_BOOT_OPTION_AVAIL + " &mdash;");
                        cellWidget.optSelector.set("disabled", (bootOptions.length == 0));
                    }),
                    getCellWidgetConnects: dojo.hitch(this, function(cellWidget, cell) {
                        // return an array of connection arguments
                        return [[cellWidget.optSelector, "onChange", dojo.hitch(this, function(newValue) {
                            var oldValue = cell.rawData();
                            console.log(this.declaredClass + ": onChange() option:", cell.row.index(), oldValue, newValue);
                            if (cellWidget.optSelector._cpIgnoreFirstChange === undefined) {
                                // 4/23/2014: The following is admittedly a hack, but it works!
                                // For some unknown reason the value of the cellWidget.optSelector dijit is changed to
                                // it's first option value when the dijit is initially displayed on screen. Two days
                                // have been spent trying to figure out how to keep the dijit set to it's initial 
                                // value as determined by the setCellValue() function but to no avail.
                                // So.... the following hack exists to detect this mysterious change and set it right again.
                                cellWidget.optSelector._cpIgnoreFirstChange = true;
                                cellWidget.optSelector.set("value", oldValue);
                                return;
                            }
                            if (oldValue != newValue) {
                                cell.row.setRawData({
                                    option: newValue,
                                    device: this.DEVICE_NONE // must discard "device" value too, since it is based on the "option" value!
                                });
                                cell.grid.body.refresh(); // Refresh entire grid to ensure other cellWidget's consider new values.
                            }
                        })]];
                    })
                }, {
                    field: "device",
                    name: this.locResources.BOOT_DEVICE,
                    width: "auto",
                    sortable: false,
                    widgetsInCell: true,
                    decorator: function() {
                        // return cellWidget's HTML template
                        return "<div data-dojo-type=\"dijit.form.Select\" data-dojo-attach-point=\"devSelector\" style=\"width: 100%;\"></div>";
                    },
                    setCellValue: dojo.hitch(this, function(gridData, storeData, cellWidget) {
                        var rowItem = cellWidget.cell.row.item();
                        var bootOption = (rowItem.option) ? rowItem.option[0] : this.DEVICE_NONE;
                        if ((bootOption == this.DEVICE_HD) || (bootOption == this.DEVICE_USB)) {
                            var gridStore = cellWidget.cell.grid.store;
                            var bootDevices = this._getAvailableBootDevices(gridStore, bootOption, storeData);
                            var devStore = new dojo.data.ItemFileReadStore({
                                // Use a store so options are alphabetized (by the designated label)!
                                data: {
                                    identifier: "value",
                                    label: "label",
                                    items: bootDevices
                                }
                            });
                            var selectedValue = (storeData) ? storeData : ((bootDevices.length > 0) ? bootDevices[0].value : this.DEVICE_NONE);
                            cellWidget.devSelector.setStore(devStore, selectedValue);
                            cellWidget.devSelector.set("required", true);
                            cellWidget.devSelector.set("emptyLabel", "&mdash; " + this.locResources.NO_BOOT_DEVICE_AVAIL + " &mdash;");
                            cellWidget.devSelector.set("disabled", (bootDevices.length == 0));
                            dojo.style(cellWidget.devSelector.domNode, "display", "");  // Make visible, in case it was hidden previously.
                        }
                        else {
                            // No boot devices for selected boot option is supported.
                            // IMPORTANT: Be sure to discard this cell's raw/store data to ensure what ever is currently there
                            // is not consider "used" as far as the other cellWidgets are concerned. In other words, make sure
                            // the data store is clean!
                            cellWidget.cell.setRawData(this.DEVICE_NONE); // Discard cell's current value.
                            dojo.style(cellWidget.devSelector.domNode, "display", "none");
                        }
                    }),
                    getCellWidgetConnects: dojo.hitch(this, function(cellWidget, cell) {
                        // return an array of connection arguments
                        return [[cellWidget.devSelector, "onChange", dojo.hitch(this, function(newValue) {
                            var oldValue = cell.rawData();
                            console.log(this.declaredClass + ": onChange() device:", cell.row.index(), oldValue, newValue);
                            if (oldValue != newValue) {
                                cell.row.setRawData({
                                    device: newValue
                                });
                                cell.grid.body.refresh(); // Refresh entire grid to ensure other cellWidget's consider new values.
                            }
                        })]];
                    })
                }, {
                    field: "icons",
                    name: " ",
                    width: "140px",
                    sortable: false,
                    widgetsInCell: true,
                    decorator: function() {
                        // return cellWidget's HTML template
                        return "<table style=\"border:0;border-spacing:0;\"><tr>" +
                            "<td style=\"padding:0;\"><button data-dojo-type=\"support.widgets.Icon\" style=\"width:32px;height:30px;\" data-dojo-attach-point=\"upIcon\" iconClass=\"{ normal: 'spArrowUpIcon', disabled: 'spArrowUpInactiveIcon'}\"></button></td>" + 
                            "<td style=\"padding:0;\"><button data-dojo-type=\"support.widgets.Icon\" style=\"width:32px;height:30px;\" data-dojo-attach-point=\"downIcon\" iconClass=\"{ normal: 'spArrowDownIcon', disabled: 'spArrowDownInactiveIcon'}\"></button></td>" + 
                            "<td style=\"padding:0;\"><button data-dojo-type=\"support.widgets.Icon\" style=\"width:32px;height:30px;\" data-dojo-attach-point=\"addIcon\" iconClass=\"{ normal: 'spAddRowIcon_v2', disabled: 'spAddRowInactiveIcon_v2'}\"></button></td>" + 
                            "<td style=\"padding:0;\"><button data-dojo-type=\"support.widgets.Icon\" style=\"width:32px;height:30px;\" data-dojo-attach-point=\"removeIcon\" iconClass=\"{ normal: 'spDeleteIcon_v2', disabled: 'spDeleteInactiveIcon_v2'}\"></button></td>" +
                            "</tr></table>";
                    },
                    setCellValue: dojo.hitch(this, function(gridData, storeData, cellWidget) {
                        var rowIndex = cellWidget.cell.row.index();
                        var rowCount = cellWidget.cell.grid.rowCount();
                        var disableUp = (rowIndex == 0);
                        var disableDown = (rowIndex == rowCount - 1);
                        var disableRemove = (rowCount <= 1); // Do not allow removal of last device!
                        cellWidget.upIcon.set("disabled", disableUp);
                        cellWidget.upIcon.set("title", this.locResources.MOVE_UP_TITLE);
                        cellWidget.downIcon.set("disabled", disableDown);
                        cellWidget.downIcon.set("title", this.locResources.MOVE_DOWN_TITLE);
                        cellWidget.addIcon.set("title", this.locResources.ADD_TITLE);
                        cellWidget.removeIcon.set("title", this.locResources.REMOVE_TITLE);
                        cellWidget.removeIcon.set("disabled", disableRemove);
                    }),
                    getCellWidgetConnects: dojo.hitch(this, function(cellWidget, cell) {
                        // return an array of connection arguments
                        return [[cellWidget.upIcon, "onClick", dojo.hitch(this, function() {
                            // Move row up one.
                            var rowIndex = cell.row.index();
                            console.log(this.declaredClass + ": onClick() upIcon:", rowIndex);
                            var newData = {
                                    items: []
                            };
                            var rows = cell.grid.rows();
                            for (var idx = 0; idx < rows.length; ++idx) {
                                var item = rows[idx].item();
                                var bump = 0;
                                if ((idx + 1) == rowIndex) {
                                    // Push next item first and decrement "order" value.
                                    ++idx, ++bump;
                                    var nextItem = rows[idx].item();
                                    newData.items.push({
                                        order: nextItem.order[0] - bump,
                                        option: nextItem.option[0],
                                        device: nextItem.device[0]
                                    });
                                }
                                newData.items.push({
                                    order: item.order[0] + bump,
                                    option: item.option[0],
                                    device: item.device[0]
                                });
                            }
                            var newStore = new dojo.data.ItemFileWriteStore({
                                data: newData
                            });
                            cell.grid.setStore(newStore); // cell.grid.body will refresh automatically
                        })],
                        [cellWidget.downIcon, "onClick", dojo.hitch(this, function() {
                            // Move row up one.
                            var rowIndex = cell.row.index();
                            console.log(this.declaredClass + ": onClick() downIcon:", rowIndex);
                            var newData = {
                                    items: []
                            };
                            var rows = cell.grid.rows();
                            for (var idx = 0; idx < rows.length; ++idx) {
                                var item = rows[idx].item();
                                var bump = 0;
                                if (idx == rowIndex) {
                                    // Push next item first.
                                    ++idx, ++bump;
                                    if (idx < rows.length) { // precautionary test
                                        var nextItem = rows[idx].item();
                                        newData.items.push({
                                            order: nextItem.order[0] - bump,
                                            option: nextItem.option[0],
                                            device: nextItem.device[0]
                                        });
                                    }
                                }
                                newData.items.push({
                                    order: item.order[0] + bump,
                                    option: item.option[0],
                                    device: item.device[0]
                                });
                            }
                            var newStore = new dojo.data.ItemFileWriteStore({
                                data: newData
                            });
                            cell.grid.setStore(newStore); // cell.grid.body will refresh automatically
                        })],
                        [cellWidget.addIcon, "onClick", dojo.hitch(this, function() {
                            // Insert a new row after the current one.
                            // Copy row items for a new data store with the new item in proper position.
                            var rowIndex = cell.row.index();
                            console.log(this.declaredClass + ": onClick() addIcon:", rowIndex);
                            var newData = {
                                    items: []
                            };
                            var rows = cell.grid.rows(), increase = 0;
                            for (var idx in rows) {
                                var item = rows[idx].item();
                                newData.items.push({
                                    order: item.order[0] + increase,
                                    option: item.option[0],
                                    device: item.device[0]
                                });
                                if (idx == rowIndex) {
                                    // Insert additional item for new row and increase for subsequent items.
                                    newData.items.push({
                                        order: item.order[0] + 1,
                                        option: this.DEVICE_NONE,
                                        device: this.DEVICE_NONE
                                    });
                                    ++increase;
                                }
                            }
                            var newStore = new dojo.data.ItemFileWriteStore({
                                data: newData
                            });
                            cell.grid.setStore(newStore); // cell.grid.body will refresh automatically
                        })],
                        [cellWidget.removeIcon, "onClick", dojo.hitch(this, function() {
                            // Remove current row.
                            var rowIndex = cell.row.index();
                            console.log(this.declaredClass + ": onClick() removeIcon:", rowIndex);
                            var newData = {
                                    items: []
                            };
                            var rows = cell.grid.rows(), decrease = 0;
                            for (var idx in rows) {
                                var item = rows[idx].item();
                                if (idx == rowIndex) {
                                    ++decrease;
                                } else {
                                    newData.items.push({
                                        order: item.order[0] - decrease,
                                        option: item.option[0],
                                        device: item.device[0]
                                    });
                                }
                            }
                            var newStore = new dojo.data.ItemFileWriteStore({
                                data: newData
                            });
                            cell.grid.setStore(newStore); // cell.grid.body will refresh automatically
                        })]];
                    })
                }];

    	    }
    	    return layout;
        	        
        },
       
        _decorateOrder : function(value, id, idx) {
            // tags:
            // protected
            return value + 1;
        },
        
        _getBootDeviceOptions : function(deviceType) {
            // tags:
            // protected
            // Get boot device options for specified device type.
            var FABRIC_CONNECTOR_VPORT = "Fabric Connector - Port %1 - Function %2";
            var FABRIC_CONNECTOR_PORT = "Fabric Connector - Port %1";
            var IOADAPTER_VPORT = "I/O Adapter %1 - Port %2 - Function %3";
            var IOADAPTER_PORT = "I/O Adapter %1 - Port %2";
            var deviceOptions = [];

        	if (deviceType == this.DEVICE_FC) {
        	    for (var idx in this.pageParams.ioBootDevices) {
        	        // Loop through Boot Devices to pick out "fc" and "fcoe" HBA (SAN) ports
        	        var device = this.pageParams.ioBootDevices[idx];
        	        if (device.physicalFunction == "fc" || device.physicalFunction == "fcoe") {
        	            var label = null;
        	            var value = null;
        	            if (device.type == "lom") {
        	                if (device.vportNumber > 0) {
        	                    label = this.locResources.FABRIC_CONNECTOR_VPORT.replace("%1", device.portLabel).replace("%2", device.physicalFunctionLabel);
        	                    value = FABRIC_CONNECTOR_VPORT.replace("%1", device.portLabel).replace("%2", device.physicalFunctionLabel);
        	                }
        	                else {
        	                    label = this.locResources.FABRIC_CONNECTOR_PORT.replace("%1", device.portLabel);
        	                    value = FABRIC_CONNECTOR_PORT.replace("%1", device.portLabel);
        	                }
        	            }
        	            else if (device.type == "adapter") {
        	                if (device.vportNumber > 0) {
        	                    label = this.locResources.IOADAPTER_VPORT.replace("%1", device.cardNumber).replace("%2", device.portLabel).replace("%3", device.physicalFunctionLabel);
        	                    value = IOADAPTER_VPORT.replace("%1", device.cardNumber).replace("%2", device.portLabel).replace("%3", device.physicalFunctionLabel);
        	                }
        	                else {
        	                    label = this.locResources.IOADAPTER_PORT.replace("%1", device.cardNumber).replace("%2", device.portLabel);
        	                    value = IOADAPTER_PORT.replace("%1", device.cardNumber).replace("%2", device.portLabel);
        	                }
        	            }
        	            if (label && value) {
            	            deviceOptions.push({
            	                label: label,
            	                value: value
            	            });
            	            this._HBAOptionCache[value] = {
            	                    deviceType: device.type,
            	                    deviceSlot: device.slotNumber,
            	                    devicePort: device.portNumber,
            	                    functionNumber: device.vportNumber
            	            };
        	            }
        	        }
        	    }
        	    this.isHBA = (deviceOptions.length > 0);
        	}
        	else if(deviceType == this.DEVICE_HD) {
        	    deviceOptions = [{
        	        label: this.locResources.LOCAL_DISK_OPT.replace("%1", "0"),
        	        value: "Hard Disk 0"
        	    }, {
        	        label: this.locResources.LOCAL_DISK_OPT.replace("%1", "1"),
        	        value: "Hard Disk 1"
        	    }, {
        	        label: this.locResources.LOCAL_DISK_OPT.replace("%1", "2"),
        	        value: "Hard Disk 2"
        	    }, {
        	        label: this.locResources.LOCAL_DISK_OPT.replace("%1", "3"),
        	        value: "Hard Disk 3"
        	    }, {
        	        label: this.locResources.LOCAL_DISK_OPT.replace("%1", "4"),
        	        value: "Hard Disk 4"
        	    }];
        	}
        	else if(deviceType == this.DEVICE_USB) {
        	    deviceOptions = [{
        	        label: this.locResources.USB_LABEL_FF,
        	        value: "First found"
        	    }, {
        	        label: this.locResources.USB_LABEL.replace("%1", "0"),
        	        value: "USB0"
        	    }, {
        	        label: this.locResources.USB_LABEL.replace("%1", "1"),
        	        value: "USB1"
        	    }, {
        	        label: this.locResources.USB_LABEL.replace("%1", "2"),
        	        value: "USB2"
        	    }, {
        	        label: this.locResources.USB_LABEL.replace("%1", "3"),
        	        value: "USB3"
        	    }, {
        	        label: this.locResources.USB_LABEL.replace("%1", "4"),
        	        value: "USB4"
        	    }];
        	}
        	return deviceOptions;

        },

        _isVAddressingEnabled: function() {
            // tags:
            // protected
            return (this.pageParams.ioAddressingMode == "virtual" && this.pageParams.fcAddressPool != "*NONE");
        },

        _getAvailableBootOptions: function(/**/store, includeItem, learnedOptions) {
            // tags:
            // protected

            // Start by compiling all known options...
            var allOptions = [{
                label: this.locResources.CD_DVD,
                value: this.DEVICE_CD_DVD
            }, {
                label: this.locResources.DISKETTE,
                value: this.DEVICE_DISKETTE
            }, {
                label: this.locResources.PXE,
                value: this.DEVICE_PXE
            }, {
                label: this.locResources.ISCSI,
                value: this.DEVICE_ISCSI
            }, {
                label: this.locResources.ISCSI_CRITICAL,
                value: this.DEVICE_ISCSI_CRITICAL
            }, {
                label: this.locResources.HYPER,
                value: this.DEVICE_HYPERVISOR
            }, {
                label: this.locResources.LEGACY_ONLY,
                value: this.DEVICE_LEGACY_ONLY
            }, {
                label: this.locResources.MANAGER,
                value: this.DEVICE_WBM
            }];
            var includeDevice = (includeItem.device) ? includeItem.device[0] : null;
            if (this._getAvailableBootDevices(store, this.DEVICE_HD, includeDevice).length > 0) {
                // At least one HD boot device is available.
                allOptions.push({
                    label: this.locResources.LOCAL_DISK,
                    value: this.DEVICE_HD
                });
            }
            if (this._getAvailableBootDevices(store, this.DEVICE_USB, includeDevice).length > 0) {
                // At least one USB boot device is available.
                allOptions.push({
                    label: this.locResources.USB,
                    value: this.DEVICE_USB
                });
            }
            
            // Add learned options, if any...
            if (learnedOptions) {
                var unique = {}; //Use this unique object to verify no dup's are added to allOptions array
                                                
                // Loop through learned options and add them to allOptions, but only if they haven't been added already.
                // This is necessary because sometimes duplicate options appear in the returned json. That is valid
                // because of the way config manager has some components with the same label or value even though they
                // are different components.                 
                for (var idx in learnedOptions) {
                    var option = learnedOptions[idx];
                    if (!unique[option.value]) {
                        allOptions.push({
                            label: option.label,
                            value: option.value
                        });
                        unique[option.value] = true;                        
                    }    
                }                
            }

            // Exclude those that are used, returning all that remain.
            var excludeOption = (includeItem.option) ? includeItem.option[0] : null;
            var usedOptions = this._getUsedOptions(store, excludeOption);
            var availOptions = new Array();
            for (var idx in allOptions) {
                var option = allOptions[idx];
                // NOTE: Check for FC devices too in case one slipped in through learned options!
                if ((option.value != this.DEVICE_FC) && !this._arrayContains(usedOptions, option.value)) {
                    availOptions.push(option);
                }
            }
            // The "*NONE" option is only desirable when other options are available.
            if (availOptions.length > 0) {
                availOptions.push({
                    label: "&mdash; " + this.locResources.NO_BOOT_OPTION_SELECTED + " &mdash;",
                    value: this.DEVICE_NONE
                });
            }
            return availOptions;

        }, 
       
       _getUsedOptions : function(/**/store, /*Object?*/excludeOption) {
           // tags:
           // protected
           var usedOptions = new Array();
           if (store) {
               for (var idx in store._arrayOfTopLevelItems) {
                   var option = store._arrayOfTopLevelItems[idx].option[0];
                   if (option && (!excludeOption || (option != excludeOption)) && (option != this.DEVICE_FC) && (option != this.DEVICE_HD) && (option != this.DEVICE_USB)) {
                       usedOptions.push(option);
                   }
               }
           }
           return usedOptions;
       },

       _getAvailableBootDevices: function(/**/store, /*String*/deviceType, /*String?*/includeDevice, /*boolean?*/includeNoneOption) {
           // tags:
           // protected
           var deviceOptions = this._getBootDeviceOptions(deviceType);
           var usedDevices = this._getUsedDevices(store, includeDevice);
           var availOptions = new Array();
           for (var idx in deviceOptions) {
               var option = deviceOptions[idx];
               if (!this._arrayContains(usedDevices, option.value)) {
                   availOptions.push(option);
                }
           }
           // The "*NONE" option is desirable for SAN Boot tab, but not for Primary Boot or WoL Boot tabs
           // so special consideration is applied here.
           if (includeNoneOption && (availOptions.length > 0)) {
               availOptions.push({
                   label: "&mdash; " + this.locResources.NO_BOOT_DEVICE_SELECTED + " &mdash;",
                   value: this.DEVICE_NONE
               });
           }
           return availOptions;            
      },
       
      _getUsedDevices : function(/**/store, /*Object?*/excludeDevice) {
          // tags:
          // protected
          var usedDevices = new Array();
          if (store) {
              for (var idx in store._arrayOfTopLevelItems) {
                  var device = store._arrayOfTopLevelItems[idx].device[0];
                  if (device && (device != this.DEVICE_NONE) && (!excludeDevice || (device != excludeDevice))) {
                      usedDevices.push(device);
                  }
              }
          }
          return usedDevices;
      },

       _arrayContains : function(array, object) {
           // tags:
           // protected
           // Determines if an object is contained in an array.
           if (array != null) {
               for (var idx in array) {
                   if (array[idx] === object) {
                       return true;
                   }
               }
           }            
           return false;
       },
        
        getTableItems: function() {
            // Returns array of items obtained from the Boot Order Table.
            var tableItems = [];
            if (this._bootGrid) {
                var store = this._bootGrid.store;
                var gridItems = store._arrayOfTopLevelItems;
                for (var idx in gridItems) {
                    var item = gridItems[idx];
                    var order = store.getValue(item, "order");
                    var option = store.getValue(item, "option");
                    var device = store.getValue(item, "device");
                    var deviceType = store.getValue(item, "deviceType");
                    var devicePort = store.getValue(item, "devicePort");
                    var deviceSlot = store.getValue(item, "deviceSlot");
                    var functionNumber = store.getValue(item, "functionNumber");
                    var template = store.getValue(item, "template");
                    var target = store.getValue(item, "target");
                    if (this.SANboot && ((option != this.DEVICE_FC) || (device == this.DEVICE_NONE))) {
                        // Skip "*NONE" option data for SAN Boot tables.
                        continue;
                    }
                    if (!this.SANboot && (option == this.DEVICE_NONE)) {
                        // Skip "*NONE" option data for Primary and WoL Boot tables.
                        continue;
                    }
                    tableItems.push({
                        order: order,
                        option: option,
                        device: device,
                        deviceType: deviceType,
                        devicePort: devicePort,
                        deviceSlot: deviceSlot,
                        functionNumber: functionNumber,
                        template: template,
                        target: target
                    });
                }
            }
            return tableItems;
        },
        
        _filterBootOrderItems : function(patternData) {
            // tags:
            // protected
            
            var bootOrderItems = null;
            if (this.SANboot) {
                // Strip out non-SAN Boot options for SAN Boot table since they are handled separately now.
                // NOTE: dojo.filter() handles null & empty arrays too!
                bootOrderItems = dojo.filter(patternData, function(item) {
                    return item && (item.option == this.DEVICE_FC);
                }, this);
                if (bootOrderItems.length == 0) {
                    // Create path bootOrderTable will be null.  Edit path it might just be empty.  Either way need to add an initial row to the store.
                    var newItem = this._nextAvailableSanBootDeviceItem(0);
                    (newItem) && bootOrderItems.push(newItem);
                }

            }
            else {
                // Strip out SAN Boot options for Boot Order tables since they are handled separately now.
                // NOTE: dojo.filter() handles null & empty arrays too!
                bootOrderItems = dojo.filter(patternData, function(item) {
                    return item && (item.option != this.DEVICE_FC);
                }, this);
                // Fix up 'order' values for Boot Order tables
                for (var idx = 0; idx < bootOrderItems.length; ++idx) {
                    bootOrderItems[idx].order = idx;
                }
                // If none remain ititialize to default list for Boot Order tables
                if (bootOrderItems.length == 0) {
                    bootOrderItems = [{
                        order: 0,
                        option: this.DEVICE_CD_DVD,
                        device: this.DEVICE_NONE
                    },
                    {
                        order: 1,
                        option: this.DEVICE_HD,
                        device: "Hard Disk 0"
                    },
                    {
                        order: 2,
                        option: this.DEVICE_PXE,
                        device: this.DEVICE_NONE
            	    }];
            	}
            }
        	return bootOrderItems;
        },

        updateGridStoreData: function(){                        

            if (this._bootGrid == null) {
                return;
            }
            
            if (this.SANboot) {
                var store = this._bootGrid.store;
                var gridItems = store._arrayOfTopLevelItems;
                var newData = {
                        items: []
                };
                if ((this.pageParams.ioAddressingMode == "virtual") && (this.pageParams.fcAddressPool != "*NONE")) {
                    // SAN Boot is still allowed. Check FC devices and omit any that are no longer options.
                    var deviceOptions = this._getBootDeviceOptions(this.DEVICE_FC);
                    var optionValues = [];
                    for (var idx in deviceOptions) {
                        var option = deviceOptions[idx];
                        option && optionValues.push(option.value);
                    }
                    for (var idx in gridItems) {
                        var item = gridItems[idx];
                        var device = store.getValue(item, "device");
                        if (dojo.indexOf(optionValues, device) >= 0) {
                            // FC Device still an option, so keep it.
                            console.log(this.declaredClass + ": updateGridStoreData() keeping SAN Boot device ", device);
                            newData.items.push({
                                order: item.order[0],
                                option: item.option[0],
                                device: item.device[0],
                                deviceType: item.deviceType[0],
                                devicePort: item.devicePort[0],
                                deviceSlot: item.deviceSlot[0],
                                functionNumber: item.functionNumber[0],
                                template: item.template[0],
                                target: item.target[0]
                            });
                        } else {
                            console.log(this.declaredClass + ": updateGridStoreData() removing SAN Boot device ", device);
                        }
                    }
                    if ((newData.items.length == 0) && (deviceOptions.length > 0)) {
                        // All previous FC devices were removed from the pattern however other
                        // options exist so a blank item must be added.
                        var newItem = this._nextAvailableSanBootDeviceItem(0, true);
                        (newItem) && (newData.items.push(newItem));
                    }
                }
                var newStore = new dojo.data.ItemFileWriteStore({
                    data: newData
                });
                this._bootGrid.setStore(newStore);
            }
            this._bootGrid.model.clearCache();
            this._bootGrid.body.refresh();

        },
        
        systemBootModeChanged: function(bootMode) {
            if (this.SANboot) {                
                this._displaySANBootTable();
            } else {                
                if (bootMode == "Legacy Only") {  // NOTE: Must match value used by parent nstBoot.js dijit.
                    this._bootOrderRadioSection.style.display = "block";
                    this._selectBootOrder(this.bootorder_legacy);
                    this._selectBootOrder(this.bootorder_keep);
                    this._bootOrderMessage.style.display = "none";
                } else {
                    this._bootOrderRadioSection.style.display = "none";
                    this._hideTables();
                    // Need to reset the message text.  On some occasions the text will disappear for some reason.
                    this._displayInformationMessage(this.locResources.BOOT_ORDER_MSG2, this.locResources.BOOT_ORDER_MSG1);
                    this._bootOrderMessage.style.display = "block";                    
                }
            }
        },
        
        _selectBootOrder: function(button) {
            // tags:
            // protected
            // Need to connect the tooltip to the icon
            // Doesn't work in postCreate for some reason, but works here
            this._patternconfig.set("connectId", this.id + "_patternconfigure");
            this._tablehelpone.set("connectId", this.id + "_tablehelp");
            
            if (this.SANboot) {                
                // Show the SAN boot table
                this._displaySANBootTable();
            } else if (button.checked) {
                // Show the Boot Order table
                if (button.value == this.BOOT_ORDER_LEGACY_OPTION) {
                    if (this.bootorder_legacy.get("checked") == true) {
                        this.bootOrderRadioValue = "legacy";
                        this._displayBootOrderTable();
                        this._bootTableOnlySection.style.display = "block";
                        if (this._bootGrid == null) {
                            this._createTable();
                        } else {
                            this._bootGrid.model.clearCache();
                            this._bootGrid.body.refresh();
                        }
                    } else {
                        this.bootOrderRadioValue = "keep";
                        this._hideTables();
                    }
                } else {
                    this.bootOrderRadioValue = "keep";
                    this._hideTables();
                }
            }
        },
        
        _displayBootOrderTable: function() {
            // tags:
            // protected
            this._bootOrderTableSection.style.display = "block";
            this._bootOrderSection.style.display = "block";
            this._bootTableOnlySection.style.display = "block";
            this._SANbootSection.style.display = "none";
        },
        
        _displaySANBootTable: function() {
            // tags:
            // protected
            if (this._isVAddressingEnabled()) {
                if (this._getBootDeviceOptions(this.DEVICE_FC).length > 0) {
                    this._bootOrderTableSection.style.display = "block";
                    this._bootOrderSection.style.display = "none";
                    this._SANbootSection.style.display = "block";
                    this._bootOrderMessage.style.display = "none";
                    this._bootTableOnlySection.style.display = "block";
                    
                    if (this._bootTargetPatternSelector == null) {
                        this._setupBootTargetSelector();
                    }
                    var patternId = this._bootTargetPatternSelector.get("value");
                    if ((patternId != "") && (patternId != "*NONE")) {
                        this._loadBootTargetPattern(patternId);
                        if (this._bootGrid == null) {
                            this._createTable();
                        } else {
                            this._bootGrid.model.clearCache();
                            this._bootGrid.body.refresh();
                        }
                    }
                    return;
                } else {
                    this._displayInformationMessage(this.locResources.VIRT_ADDR_NO_ADAP_DETAILS, this.locResources.VIRT_ADDR_NO_ADAP_TITLE);       
                }
            } else {                        
                this._displayInformationMessage(this.locResources.VIRT_ADDR_DISABLED_DETAILS, this.locResources.VIRT_ADDR_DISABLED_TITLE);            
            }
            this._hideTables();
        },
        
        _displayInformationMessage: function(msgDesc, msgTitle) {
            // tags:
            // protected
            this._bootOrderMessage.style.display = "block";
            this._hideTables();
            if (this._messageArea == null) {
                
                if(this.CP_RUN_ENVIRONMENT == "fsm"){
                    
                    require([ "idx/oneui/messaging/SingleMessage" ], dojo.hitch(this, function() {
                        this._messageArea = new idx.oneui.messaging.SingleMessage({
                            description: msgDesc,
                            title: msgTitle,
                            type: "information",
                            dateFormat: { formatLength: "medium", selector: "time" },
                            showId: false,
                            showAction: false,
                            showRefresh: false,
                            showDetailsLink: false,
                            closable: false
                        }).placeAt(this._bootOrderMessage);                               
                    }));
                    
                }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                    require([ "idx/widget/SingleMessage" ], dojo.hitch(this, function() {
                        this._messageArea = new idx.widget.SingleMessage({
                            description: msgDesc,
                            title: msgTitle,
                            type: "information",
                            dateFormat: { formatLength: "medium", selector: "time" },
                            showId: false,
                            showAction: false,
                            showRefresh: false,
                            showDetailsLink: false,
                            closable: false
                        }).placeAt(this._bootOrderMessage);                               
                    }));                    

                }
                
            } else {
                this._messageArea.set("description", msgDesc);
                this._messageArea.set("title", msgTitle);
            }
        },
        
        _hideTables: function() {
            // tags:
            // protected
            this._bootOrderTableSection.style.display = "none";
            this._bootOrderSection.style.display = "none";
            this._SANbootSection.style.display = "none";
            this._bootTableOnlySection.style.display = "none";
        },
        
        _createTable: function() {
            // tags:
            // protected
            var bootStore = new dojo.data.ItemFileWriteStore({
                data: {
                    items: []
                }
            });
            var layout = this._getTableLayout();
            
            this._bootGrid = this.getTable(bootStore, layout, this.resources.NO_DATA, this.gridID, true);
            this._bootGrid.column(0).sort(false);
            this._bootGrid.placeAt(this._bootOrderTableNode);
            this._bootGrid.startup();

             var bootOrderItems = this._filterBootOrderItems(this.jsonData);

             bootStore = new dojo.data.ItemFileWriteStore({
                    data: {
                        items: bootOrderItems
                    }
             });
             
             this._bootGrid.setStore(bootStore);
             this._bootGrid.model.clearCache();
             this._bootGrid.body.refresh();
                
        },
        
        _editSANBootTargetPattern: function() {
            // tags:
            // protected
            var patternId = this._bootTargetPatternSelector.get("value");
            if (patternId >= 0) {
                patternId = patternId.toString();
                require(["support/dialogs/FibreChannelBootTargetTemplate"], dojo.hitch(this, function() {
                    var dialog = new support.dialogs.FibreChannelBootTargetTemplate({
                        templateIDNum: patternId,
                        templateMode: "edit"
                    });
                    dialog.show();
                }));
            }
        },
        
        _createSANBootTargetPattern: function() {
            // tags:
            // protected
            require(["support/dialogs/FibreChannelBootTargetTemplate"], dojo.hitch(this, function() {
                var dialog = new support.dialogs.FibreChannelBootTargetTemplate();
                dialog.show();
            }));
        },
        
        _getBootTargetPatterns: function() {
            // tags:
            // protected
            var bootTargetTemplateData = null;
            dojo.xhrGet({
                url: this.REST_PATH + "template/list/FibreChannel",                    
                handleAs: "json",
                sync: true,
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                load: function(data) {
                    bootTargetTemplateData = data;
                },
                error: function(error) {
                    alert(error);
                }
            });            
            return bootTargetTemplateData;
        },
        
        _setupBootTargetSelector: function() {
            // tags:
            // protected
            if (this._bootTargetPatternSelector == null) {
                var bootTargetPatterns = this._getBootTargetPatterns();
                var patternStore = new dojo.data.ItemFileReadStore({
                    data: bootTargetPatterns
                });
                this._bootTargetPatternSelector = new support.widgets.Select({
                    id: this.id + "cp_bootordertable_boottargetpatternselector",
                    name: "bootTargetPattern",
                    store: patternStore,
                    maxHeight: -1,
                    emptyLabel: this.resources.NO_TEMP_DEF_HYPHENS,
                    style: {
                        width: "300px"
                    },
                    onChange: dojo.hitch(this, function(newValue) {
                        if (newValue == "") {
                            this._bootTargetPatternSelector.set("disabled", true);
                            dojo.replaceClass(this._bootTargetEditIcon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                            this._bootTableOnlySection.style.display = "none";                        
                        } else if (newValue == "*NONE") {
                            dojo.replaceClass(this._bootTargetEditIcon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                            this._bootTableOnlySection.style.display = "none";                           
                        } else {
                            this._loadBootTargetPattern(newValue);
                            dojo.replaceClass(this._bootTargetEditIcon, "spEditIcon_v2", "spEditInactiveIcon_v2");
                            this._bootTargetPatternSelector.set("disabled", false);
                            this._displaySANBootTable();
                        }
                    })
                }).placeAt(this._bootTargetPatternSelectorNode);
                
                this._bootTargetPatternSelector.startup();
                
                if (this._bootTargetPatternSelector.get("value") != "") {
                    this._bootTargetPatternSelector.addOption({
                        value: "*NONE",
                        label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
                    });
                    this._bootTargetPatternSelector.set("value", "*NONE");
                }
                
                if (this._bootTargetPatternSelector.get("value") == "") {
                    this._bootTargetPatternSelector.set("disabled", true);
                    dojo.replaceClass(this._bootTargetEditIcon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                } else {
                    if (this._bootTargetPatternSelector.get("value") == "*NONE") {
                        dojo.replaceClass(this._bootTargetEditIcon, "spEditInactiveIcon_v2", "spEditIcon_v2");
                    } else {
                        dojo.replaceClass(this._bootTargetEditIcon, "spEditIcon_v2", "spEditInactiveIcon_v2");    
                    }
                    this._bootTargetPatternSelector.set("disabled", false);
                }

                // Subscribe to events for Boot Target pattern changes.
                this.subscribe("FibreBootTemplateComplete", function(patternId) {

                    var curId = this._bootTargetPatternSelector.get("value"); // Grab current value before it changes below.
                    
                    console.log(this.declaredClass + ": FibreBootTemplateComplete fired!", curId, patternId);

                    // Ensure the Boot Target pattern selector has current list
                    var bootTargetPatterns = this._getBootTargetPatterns();
                    if (bootTargetPatterns.items && (bootTargetPatterns.items.length > 0)) {
                        var tmplStore = new dojo.data.ItemFileReadStore({
                            data: bootTargetPatterns
                        });
                        this._bootTargetPatternSelector.set("disabled", false);
                        this._bootTargetPatternSelector.setStore(tmplStore, patternId); // Triggers onChange() only if new value is different than current value.
                    }
                    if (curId == patternId) {
                        // Selected Boot Target pattern must have been edited so reload the table
                        // to pick up potential changes that affect content.
                        this._displaySANBootTable();
                    }
                });

            }
            if (this.pageParams.bootOrderTable != null) {
                for (var counter=0;counter < this.pageParams.bootOrderTable.length;++counter) {
                    if (this.pageParams.bootOrderTable[counter].option == this.DEVICE_FC) {
                        this._bootTargetPatternSelector.set("value", this.pageParams.bootOrderTable[counter].template);
                        break;
                    }
                }
            }
        },
        
        _nextAvailableSanBootDeviceItem: function(/*integer*/order, /*boolean?*/initForNone) {
            // tags:
            // protected
            // Return a table item for the next available SAN Boot Device, with "order" set to specified value.
            // If "initForNone" is true the item will be initialized to blank/default values.
            // Return null if no device is available.
            var deviceItem = null;
            var store = (this._bootGrid) ? this._bootGrid.store : null; // NOTE: The grid may not be created yet so consider cases where there's no store.
            var availableDevices = this._getAvailableBootDevices(store, this.DEVICE_FC);
            if (availableDevices.length > 0) {
                var device = (initForNone) ? this.DEVICE_NONE : availableDevices[0].value;
                var deviceDetails = this._HBAOptionCache[device];
                if (deviceDetails != undefined) {
                    deviceItem = {
                        order: order,
                        option: this.DEVICE_FC,
                        device: device,
                        deviceType: deviceDetails.deviceType,
                        devicePort: deviceDetails.devicePort,
                        deviceSlot: deviceDetails.deviceSlot,
                        functionNumber: deviceDetails.functionNumber,
                        template: this._bootTargetPatternSelector.get("value"),
                        target: ((this._bootTargetPattern.checkvalue) ? "all" : "primary")
                    };
                }
            }
            return deviceItem;
        },
        
        _loadBootTargetPattern: function(/*String*/patternId) {
            // tags:
            // protected
            var pattern = null;
            dojo.xhrGet({
                url: this.REST_PATH + "template/" + patternId,
                handleAs: "json",
                sync: true,
                preventCache: true,
                load: function(response) {
                    if (response.items.length > 0) {
                        pattern = response.items[0];
                    }
                }
            });
            this._bootTargetPattern = pattern;
        },
        
        reloadBootTableData: function(pageParams) {            
            this.pageParams = pageParams;
            this.learnedOptions = pageParams.learnedBootOptions;
            this.systemBootModeChanged(pageParams.bootOrder);
        }
	});
});