define([
        "dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/BootTargetsTable",
        "dojo/text!./templates/BootTargetsTable.html",
        "dojox/grid/DataGrid",
        "dijit/form/ValidationTextBox",
        "dojo/data/ItemFileWriteStore",
        "dojo/data/ItemFileReadStore",
        "dojox/grid/cells/_base",
        "dijit/form/Button"
    ], function (dojo, dijit, panel, mri, template) {

	dojo.declare("support.panels.BootTargetsTable", [ support.panels.Panel ], {
	    // templateString: [protected] String
	    // This widget's HTML template, provided.
	    templateString: template,
	        
	    // locResources: [protected] Object
	    // This widget's National Language Source (NLS) object.
	    locResources: mri, 
	    
		widgetsInTemplate : false,
		baseClass: "BootTargetsTable",
			
		lunDisabled: true,
		jsonData: null,
		_bootStore: null,
		_bootLayout: null,
		_bootGrid: null,
		_bootLauncher: null,
		
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/BootTargetsTable.css",

		postMixInProperties : function() {
			this._connectDataModels();
			this.inherited(arguments);
		},

		postCreate : function(){	
			this.inherited(arguments);
			this._bootGrid = new dojox.grid.DataGrid({
			    id: this.id + "cp_boottargetstable_datagrid",
        		store: this._bootStore,
        		structure: this._bootLayout,
        		autoWidth: true,
        		autoHeight: true,
        		singleClickEdit: true,
        		columnReordering: false,
        		canEdit: dojo.hitch(this, function(inCell, inRowIndex) {
        		    // Edit rules:
        		    // First row, boxes are editable
        		    // The storage target cell (1) is always editable
        		    // The lun cell (2) for rows > the first one are only editable if the allow multiple box is selected
        		    if (inRowIndex == 0 || inCell.index == 1 || (inCell.index == 2 && !this.lunDisabled)) {
        		        return true;
        		    } else {
        		        return false;
        		    }
        		}),
        		onApplyCellEdit: this._doApplyCellEdit,
        		noDataMessage: this.locResources.NO_ITEMS
        	}, this.bootTargetsTable);			
			this._bootGrid.startup();
			        	
        	var grid = this._bootGrid;
        	grid.canSort = function(col){
        		return false;
        	};
        	grid.update();
        	this._setIcons();
		},
		
		startup: function(){
            this._resetItems();
            this.inherited(arguments);
        },
        
        _resetItems : function(){
            dojo.style(this.bootTargetsTable, {display: "block"});
            var grid = this._bootGrid;
            if(grid != null){
                var store = grid.store;
                var topLevelItems = store._arrayOfTopLevelItems;
                grid.setItems(topLevelItems);
            }            
        },
        
        resetTable : function(jsonData){
        	this.clearTable();
        	var grid = this._bootGrid;
        	var store = grid.store;
        	var btt = this;
        	
        	var resetItems = function(tableitems){
        		var data = btt._getValidJsonData(jsonData);
            	var items = data.items;
            	for(var i=0; i<items.length; i++){
            		var item = items[i];
            		var item_obj = {order: i, wwpn: item.wwpn, lun: item.lun, drag: true, add: true, remove: true};
            		store.newItem(item_obj);
            		grid.update(); 
            	}
        	};
        	store.fetch({onComplete:resetItems});
        	this._setIcons();
        	
        },
        
        clearTable : function(){
        	var grid = this._bootGrid;
        	var store = grid.store;
        	var deleteItems = function(items, request){
        		for(var i=0; i<items.length; i++){
        			var item = items[i];
        			store.deleteItem(item);
        			grid.setItems(store._arrayOfTopLevelItems);
        		}
        	};
        	store.fetch({onComplete:deleteItems});
        }, 
		
		reposition : function(){
		    //override if necessary
		},
		
		
	    _doApplyCellEdit : function(){
	    	this.update();
	    },
	    
	    
	    
	    _connectDataModels : function(){
        	this._bootStore = new dojo.data.ItemFileWriteStore({
        		data: this._getValidJsonData(this.jsonData)
        	});
        	if (this._bootLayout == null) {
                
        	    if(this._bootLauncher == "dialog"){
        	    
        	    this._bootLayout = [ 
        	    {
        	        field: "order",
                    name: this.locResources.ORDER,
                    width: "45px",
                    title: "This is a test.",
                    noresize: true,
                    formatter: this._formatOrder                        
                }, {
                    field: "wwpn",
                    name: this.locResources.STORAGE,
                    width: "204px",
                    title: "This is a test.",
                    editable: this.lunDisabled,
                    noresize: true,
                    formatter: this._formatWWPN                     
                }, {
                    field: "lun",
                    name: this.locResources.LUN,
                    width: "120px",
                    title: "This is a test.",
                    editable: true,
                    noresize: true,
                    formatter: this._formatLUN
                },{
                    field: "drag",
                    name: " ",
                    width: "100px",
                    title: "This is a test.",
                    draggable: true,
                    noresize: true,
                    formatter: this._formatIcons
                }];        	                            
        	    
        	    }else{
        	        
        	    
        	    this._bootLayout = [ 
                {
                    field: "order",
                    name: this.locResources.ORDER,
                    width: "5px",
                    title: "This is a test.",
                    noresize: true,
                    formatter: this._formatOrder                    	
                }, {
                    field: "wwpn",
                    name: this.locResources.STORAGE,
                    width: "70px",
                    title: "This is a test.",
                    editable: this.lunDisabled,
                    noresize: true,
                    formatter: this._formatWWPN                    	
                }, {
                    field: "lun",
                    name: this.locResources.LUN,
                    width: "20px",
                    title: "",
                    editable: true,
                    noresize: true,
                    formatter: this._formatLUN
                },{
                	field: "drag",
                	name: " ",
                	width: "100px",
                	title: "",
                	draggable: true,
                	noresize: true,
                	formatter: this._formatIcons
                }];
        	    
        	    }
            }
        },
        
        _formatIcons : function(value, rowIndex, cell){
        	var store = this.grid.store;
        	var item = store._arrayOfTopLevelItems[rowIndex];
        	var add = this.grid.getParent()._getAddIcon(store.getValue(item, "add"), rowIndex);
        	var remove = this.grid.getParent()._getRemoveIcon(store.getValue(item, "remove"), rowIndex);
        	var icons = "<div style=\"width:5px;display:inline-block;\"></div>" + 
        				add + "<div style=\"width:5px;display:inline-block;\"></div>" + remove;
        	return icons;
        },
        
        _setLunDisabledAttr : function(disabled){
        	this.lunDisabled = disabled;
	    	if(this._bootGrid != null){
	    		if(disabled){
	    			var grid = this._bootGrid;
	            	var store = grid.store;
	            	var topLevelItems = store._arrayOfTopLevelItems;
	            	var firstID = store.getValue(topLevelItems[0], "lun");
	            	for(var i=0; i<topLevelItems.length; i++){
	            		store.setValue(topLevelItems[i], "lun", firstID);
	            	}
	    		}
	    		this._bootGrid.update(); 
	    	}
        },
        
        //Format "order": rowIndex+1
        _formatOrder : function(value, rowIndex, cell){
        	return parseInt(rowIndex) + 1;
        },
        
        //Format "wwpn": IPv6 format
        _formatWWPN : function(value, rowIndex, cell){
            var grid = this.grid;
        	var widget = this.grid.getParent();
        	var store = this.grid.store;
        	var item = store._arrayOfTopLevelItems[rowIndex]; 
        	var errorMessage = null;
        	var control = dijit.form.ValidationTextBox({
        		value: value,
        		required: true,
        		intermediateChanges: true,
        		trim: true,
        		style:"width:100%",
        		onChange: function(newValue){
        			widget._setStoreValue(item, "wwpn", newValue);
        		},
        		onchange: this.grid.getParent()._setStoreValue(item, "wwpn", value),
        		//regExp: "^([0-9A-F]{2}[:-]){7}([0-9A-F]{2})$",
    			//invalidMessage: "To boot from 1st discovered target, enter 00:00:00:00:00:00:00:00",
    			placeHolder: "00:00:00:00:00:00:00:00",
    			validator : function(value){
    			    if(value.length == 0){
    			        errorMessage = widget.resources.REQUIRED;
    			        return false;
    			    }
    			    else{
    			    	var reg = new RegExp("^([0-9A-Fa-f]{2}[:-]){7}([0-9A-Fa-f]{2})$");
        			    if(!reg.test(value)){
        			        errorMessage = widget.locResources.ERROR1;
        			        return false;
        			    }
        			    else{
	        			    errorMessage = null;
	        			    return true;
        			    }
    			    }
    			}
        	});
        	control._destroyOnRemove = true;
        	
        	//sets value in store as cell is being edited
        	dojo.connect(cell, "formatEditing", function(inDatum, inRowIndex){
        		var tableID = this.grid.getParent().id;
        		var gridID = this.grid.id;
				this.needFormatNode(inDatum, inRowIndex);
				return  "<input class=\"dojoxGridInput\" type=\"text\" " +
						"onkeyup=\"dijit.byId('" + tableID + "')._setStoreValue(dijit.byId('" + gridID + "').store._arrayOfTopLevelItems[" + inRowIndex + "] , \"wwpn\", value)\" " +
						"value=\"" + inDatum + "\">";
			});
        	
        	dojo.connect(grid,"onCellMouseOver", function(e){
        		var controlValue = item.wwpn;
                if(e.cellIndex == 1 && e.rowIndex == rowIndex && !control.validator(controlValue)){
                    dijit.showTooltip("<div style=\"max-width:200px\">" + errorMessage + "</div>", e.cellNode);
                }
            });
            dojo.connect(grid,"onCellMouseOut", function(e){
                if(e.cellIndex == 1 && e.rowIndex == rowIndex){
                    dijit.hideTooltip(e.cellNode);                
                }

				// Check if cell is in edit mode.
				// If so, apply the edit.
                if (grid.edit.isEditing()) {
                    grid.edit.apply();
               }
            });
            return control;
        },
        
        _isValidWWPN : function(wwpn){
        	var reg = new RegExp("^([0-9A-Fa-f]{2}[:-]){7}([0-9A-Fa-f]{2})$");
        	return (wwpn != null) && (wwpn != "") && (reg.test(wwpn));
        },
        
        
        _setStoreValue : function(item, type, value){
        	if(item != null){
        		this._bootGrid.store.setValue(item, type, value);
        		this._bootGrid.getParent()._getDisabledLUNs();
        		//this._bootGrid.update();
        	}
        },
        
        //Format "lun" identifier: 0-255
        _formatLUN : function(value, rowIndex, cell){
            var grid = this.grid;
        	var widget = this.grid.getParent();
        	var store = this.grid.store;
        	var item = store._arrayOfTopLevelItems[rowIndex];
        	var errorMessage = null;
        	var control = dijit.form.ValidationTextBox({
        		value: value,
        		intermediateChanges: true,
        		//required: true,
        		trim: true,
        		disabled: (this.grid.getParent().get("lunDisabled") && rowIndex > 0),
        		style:"width:100%",
        		//regExp: "^([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])$",
        		onChange: function(newValue){
        			widget._setStoreValue(item, "lun", newValue);
        		},
        		onchange: this.grid.getParent()._setStoreValue(item, "lun", value),
    			//invalidMessage: "To input LUN identifiers in hexadecimal format, include \"0x\" as a prefix.",
    			placeHolder: "0",
    			validator: function(value){
    			    if(value.length == 0){
    			        errorMessage = widget.resources.REQUIRED;
    			        return false;
    			    }
    			    else{
    			    	var reg = new RegExp("^([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])$");
    		            var reg2 = new RegExp("^(0x[0-9A-Fa-f][0-9A-Fa-f])$");
                        var reg3 = new RegExp("^(0x[0-9A-Fa-f])$");

	    			    if(reg.test(value) || reg2.test(value) || reg3.test(value)){
	    		           errorMessage = null;
	                       return true;	          
	    			    }
	    			    else{
	    	               errorMessage = widget.locResources.ERROR2;
	                       return false;
	    			    }
    			    }
    			}
        	});
        	control._destroyOnRemove = true;
        	
        	//sets value in store as cell is being edited
        	dojo.connect(cell, "formatEditing", function(inDatum, inRowIndex){
        		var tableID = this.grid.getParent().id;
        		var gridID = this.grid.id;
				this.needFormatNode(inDatum, inRowIndex);
				return  "<input class=\"dojoxGridInput\" type=\"text\" " +
						"onkeyup=\"dijit.byId('" + tableID + "')._setStoreValue(dijit.byId('" + gridID + "').store._arrayOfTopLevelItems[" + inRowIndex + "] , \"lun\", value)\" " +
						"value=\"" + inDatum + "\">";
			});
        	dojo.connect(grid,"onCellMouseOver", function(e){
        		var controlValue = item.lun;
                if(e.cellIndex == 2 && e.rowIndex == rowIndex && !control.validator(controlValue)) {
                    dijit.showTooltip("<div style=\"max-width:200px\">" + errorMessage + "</div>", e.cellNode);
                }
            });
            dojo.connect(grid,"onCellMouseOut", function(e){
                if(e.cellIndex == 2 && e.rowIndex == rowIndex){
                    dijit.hideTooltip(e.cellNode);                
                }
            });
        	return control;
        },
        
        _isValidLUN : function(lun){
            var reg = new RegExp("^([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])$");
            var reg2 = new RegExp("^(0x[0-9A-Fa-f][0-9A-Fa-f])$");
            var reg3 = new RegExp("^(0x[0-9A-Fa-f])$");
            
            if(reg.test(lun) || reg2.test(lun) || reg3.test(lun)){
                return (lun != null) && (lun != "");
            }else{
                return false;
            }    
                
        },

        _getAddIcon : function(enabled, rowIndex){
        	if(enabled){
        		return "<div class=\"spAddRowIcon_v2\" " +
				"onClick=\"dijit.byId('" + this.id + "')._addRow('" + rowIndex + "')\"" +
				"title=\"" + this.locResources.CLICK2 + "\" style=\"display:inline-block;\"></div>";
        	}
        	return "<div class=\"spAddRowInactiveIcon_v2\" style=\"display:inline-block;\"></div>";
        },
        
      
        _getRemoveIcon : function(enabled, rowIndex){
        	if(enabled){
        		return "<div class=\"spDeleteIcon_v2\"" +
		        "onClick=\"dijit.byId('" + this.id + "')._removeRow('" + rowIndex + "')\"" +
				"title=\"" + this.locResources.CLICK3 + "\" style=\"display:inline-block;\"></div>";
        	}
        	return "<div class=\"spDeleteInactiveIcon_v2\" style=\"display:inline-block;\"></div>";
        },
        
        _getDisabledLUNs : function(){
        	if(this.lunDisabled){
        		var grid = this._bootGrid;
            	var store = grid.store;
            	var topLevelItems = store._arrayOfTopLevelItems;
            	var firstID = store.getValue(topLevelItems[0], "lun");
            	for(var i=0; i<topLevelItems.length; i++){
            		store.setValue(topLevelItems[i], "lun", firstID);
            	}
            	//grid.update();
        	}
        },
                
        _addRow : function(prevIndex){
        	var grid = this._bootGrid;
        	var store = grid.store;
        	var topLevelItems = store._arrayOfTopLevelItems;
        	var nextIndex = (parseInt(prevIndex) + 1) + "";
        	var item = {order: nextIndex, wwpn: "", lun: "0", drag: true, add: true, remove: true};
        	for(var i=nextIndex; i<topLevelItems.length; i++){
        		var foo = topLevelItems[i];
        		var currentOrder = parseInt(store.getValue(foo, "order"));
        		var newOrder = currentOrder + 1;
        		store.setValue(foo, "order", newOrder);
        	}
        	
        	store.newItem(item);
        	grid.setItems(topLevelItems);
        	
        	this._sortAscending();
        	
        	this._setIcons();
        	this._getDisabledLUNs();
        	this.reposition();
        	//alert("Row added...");
        	
        },
        
        _removeRow : function(row){
        	var remove = confirm(this.resources.DELETE_CONFIRM.replace("%1",parseInt(row)+1));
        	if(remove){
        		var grid = this._bootGrid;
        		var store = grid.store;
        		var topLevelItems = store._arrayOfTopLevelItems;
        		var item = topLevelItems[row];
        		grid.store.deleteItem(item);
        		grid.setItems(store._arrayOfTopLevelItems);
        		this._sortItemsAfterRemove();
        		this._setIcons();
        		this.reposition();
        		//alert("Row removed...");
        	}
        },
        
        _sortItemsAfterRemove : function(){
        	var grid = this._bootGrid;
        	var store = grid.store;
        	var topLevelItems = store._arrayOfTopLevelItems;
        	for(var i=0; i<topLevelItems.length; i++){
        		var item = topLevelItems[i];
        		store.setValue(item, "order", i);
        	}
        	grid.setItems(topLevelItems);
        },
        
        _sortAscending : function(){
        	var grid = this._bootGrid;
        	var store = grid.store;
        	var topLevelItems = store._arrayOfTopLevelItems;
        	grid.setItems(topLevelItems);
        	var pos = 0;
        	var positions = new Array();
        	while(positions.length < topLevelItems.length){
        		for(var i=0; i<topLevelItems.length; i++){
            		var order = parseInt(store.getValue(topLevelItems[i], "order"));
            		if(order == pos)
            			positions.push(topLevelItems[i]);
            	}
            	pos++;            	
        	}
        	//grid.setItems(positions);  
        	for(var i=0; i<store._arrayOfTopLevelItems.length; i++){
        		store._arrayOfTopLevelItems[i] = positions[i];        		
        	}        	      	        	
        	this._sortItemsAfterRemove();
        },
        
        
        _setIcons : function(){
        	var max = 4;
        	var min = 1;
        	var grid = this._bootGrid;
        	var store = grid.store;
        	var topLevelItems = store._arrayOfTopLevelItems;
        	for(var i=0; i<topLevelItems.length; i++){
        		var item = topLevelItems[i];
        		if(topLevelItems.length <= min){
        			//disable remove and drag
        			store.setValue(item, "drag", false);
        			store.setValue(item, "remove", false);
        		}
        		else if(topLevelItems.length >= max){
        			//disable add
        			store.setValue(item, "add", false);
        		}
        		else{
        			//enable drag, add, remove
        			store.setValue(item, "drag", true);
        			store.setValue(item, "remove", true);
        			store.setValue(item, "add", true);
        		}
        	}
        	grid.setItems(topLevelItems);
        },
        
        getStoreAsJson : function(){
        	var grid = this._bootGrid;
        	var store = grid.store;
        	var topLevelItems = store._arrayOfTopLevelItems;
        	var storeAsJson = "{\"items\":[";
        	for(var i=0; i<topLevelItems.length; i++){
        		var item = topLevelItems[i];
        		var order = store.getValue(item, "order");
        		var wwpn = store.getValue(item, "wwpn");
        		var lun = store.getValue(item, "lun");
        		storeAsJson +="{\"order\":\""+order+"\",\"wwpn\":\""+wwpn+"\",\"lun\":\""+lun+"\",\"drag\":false,\"add\":true,\"remove\":false}";        		        	        		 
        		if(i != topLevelItems.length-1)
					storeAsJson += ",";
//        		storeAsJson += "\n";
			}
        	storeAsJson +="]}";
        	return storeAsJson;
        },
        
        _getValidJsonData : function(jsonData){
        	//if any part of the file is not valid, return the default item
        	if(jsonData == null ||
        	   jsonData.items == null ||
        	   jsonData.items[0].order == null ||
        	   jsonData.items[0].wwpn == null ||
        	   jsonData.items[0].lun == null ||
        	   jsonData.items[0].drag == null ||
        	   jsonData.items[0].add == null ||
        	   jsonData.items[0].remove == null){
        	   jsonData = {
		    			"items": [
		    			{
		    				"order" : "0",
		    				"wwpn" : "",
		    				"lun" : "0",
		    				"drag" : false,
		    				"add" : true,
		    				"remove" : false
		    			}]
		    		};
        	}
        	return jsonData;
        },
        
        isValid : function(){
        	var widget = this;
        	var grid = this._bootGrid;
    		var store = grid.store;
        	//var items = store._arrayOfTopLevelItems; 
        	var valid = true;
        	//at least one item
        	//all textboxes valid
        	var gotItems = function(items, request){
        		if(items == null || items.length == 0)
        			valid = false;
        		else{
        			for(var i=0; i<items.length; i++){
        				var item = items[i];
        				var wwpn = item.wwpn[0];
        				var lun = item.lun[0];
        				if( !widget._isValidWWPN(wwpn) || !widget._isValidLUN(lun) ){
        					valid  = false;
        				}
        			}
        		}
        	};
        	store.fetch({onComplete:gotItems});
        	return valid;
        }
	});
});     	