define(["dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/BuildAList", 
        "dojo/text!./templates/BuildAList.html",
        "dojo/_base/lang",
        "dijit/form/ValidationTextBox",
        "dijit/form/NumberTextBox", 
		"dojo/data/ItemFileWriteStore",
		"dojo/data/ItemFileReadStore",
		"dojox/grid/cells/_base", 
		"dijit/form/Button",
		"dojox/grid/EnhancedGrid",
		"dojox/grid/enhanced/plugins/DnD",
		"support/widgets/OutlinePane",
		"support/dialogs/BuildAListPopup"
		], function (dojo, dijit, panel, mri, template, lang) {

	dojo.declare("support.panels.BuildAList", [ support.panels.Panel ], {
	    // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/BuildAList.css",
        
        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 
        
		widgetsInTemplate : true,
		baseClass: "BuildAList",		
			
		jsonData: null,
		store: null,
		layout: null,
		grid: null,
		
		min: 0, 
		max: null,
		settings: new Array(),

		item: null,
		allItems: new Array(),
		addTitle: null,
		removeTitle: null,
		editTitle: null,
		dragTitle: null,

		postMixInProperties : function() {
		    this.inherited(arguments);
		    
		    this._connectDataModels();
		},

		postCreate : function(){	
			this.inherited(arguments);
								
			if(this.addTitle == null)
			    this.addTitle = this.resources.ADD;
			if(this.removeTitle == null)
			    this.removeTitle = this.resources.REMOVE;
			if(this.editTitle == null)
			    this.editTitle = this.resources.EDIT;
			if(this.dragTitle == null)
			    this.dragTitle = this.resources.CLICK_AND_DRAG;
			
			this.grid = new dojox.grid.EnhancedGrid({
        		store: this.store,
        		structure: this.layout,
        		autoWidth: false,
        		autoHeight: true,
        		singleClickEdit: true,
        		columnReordering: false,
        		onApplyCellEdit: this._doApplyCellEdit,
        		selectionMode: "single",
                canSort: function(){return false;}
        	}).placeAt(this.table);			
			this.grid.startup();
			this.grid.store.save();
            this.grid.render();
        	
        	if(this.grid.store._arrayOfTopLevelItems.length == 0){
              //  this.table.setAttribute("style", "display:none");
        	}       	
            
            this._setIcons();
            this.reposition();

		},
		
		//must have function for Learned Settings
		_show : function(){
            if(this.grid != null){
                this.grid.store.save();
                this.grid.render();
            }
        },
		
		startup: function(){
            this.inherited(arguments);
        },
		
		reposition : function(){
		    //override if necessary
		},
		
		copyItem : function(obj) {
		    if (null == obj || "object" != typeof obj) return obj;
		    var copy = obj.constructor();
		    for (var attr in obj) {
		        if (obj.hasOwnProperty(attr)) copy[attr] = obj[attr];
		    }
		    return copy;
		},
		
		getDefaultSettings : function(){
		    var copyItem = this.copyItem(this.item);
		    //var defaultSettings = this.settings;
		    var defaultSettings = new Array();
		    //expect only one level of settings
            if(this.hasChildren(copyItem) > 0){
                var children = this.getChildren(copyItem);
                for(var c=0; c<children.length; c++){ //returns child groups
                    var child = children[c];
                    for(var g=0; g<child.length; g++){ //iterate through each child in group
                        var item = child[g];
                        var itemPath = this.path + "__" + item.ID[0];
                        var value = null;
                        if(item.text_data != null && item.text_data[0]["default"]){
                            value = item.text_data[0]["default"][0];
                        }
                        else if(item.numeric_data != null && item.numeric_data[0]["default"]){
                            value = item.numeric_data[0]["default"][0];
                        }
                        else if(item.enumerate_data != null && item.enumerate_data[0]["default"]){
                            value = item.enumerate_data[0]["default"][0];
                        }
                        defaultSettings.push({ path: itemPath, value: value });
                    }
                }
            }
            return defaultSettings;
		},
		
		
	    _connectDataModels : function(){
	        this.defaultSettings = new Array();
	        this.getDefaultSettings();
	        this.allItems = new Array();
            
            var data = new Object();
            var displayItems = new Array();
                        
            data.items = displayItems;

            /*if(this.min != 0 && this.max != null){
                for(var i=0; i<this.min; i++){
                    displayItems.push(this.getItemToAdd(i));
                }
            }*/
            
        	this.store = new dojo.data.ItemFileWriteStore({
        		data: data
        	});
        	        	        	
        	if (this.layout == null) {
                this.layout = [ 
                     { field: "label", name: " ", width: "150px", noresize: true, formatter: dojo.hitch(this, this._formatLabel) },
                     { field: "attr1", name: " ", width: "150px", noresize: true },
                     { field: "icons", name: " ", width: "90px", noresize: true, formatter: dojo.hitch(this, this._formatIcons) }
                ];
            }
        },
        
        _formatLabel : function(value, rowIndex, cell){
            return value + " " + (parseInt(rowIndex)+parseInt(this.min));
        },
        
        getItemToAdd : function(index){
            var copyItem = this.copyItem(this.item);
            var label = copyItem.name[0];
            var attr1 = null;
                    
            this.allItems.push(copyItem);
            var item = this.allItems[this.allItems.length-1];
                                                        
            if(item.settings == undefined)
                item.settings = this.getDefaultSettings();
            if(item.settings != null && item.settings[0] != null)
                attr1 = item.settings[0].value;           
           
            if(this.settings != undefined){
            
                var settings = this.settings[index];

                for(var j=0; j<item.option.length; j++){
                    var item2 = item.option[j];                                        
                    var path = this.path + "__" + item2.ID[0];
                                               
                    if(item2.ID[0] == "n" && settings != undefined && settings[path] != undefined){
                        attr1 = settings[path];
                    }else if(item2.ID[0] == "i6" && settings != undefined && settings[path] != undefined){
                        attr1 = settings[path];                    
                    }
                }
                
            }

            return { label: label, attr1: attr1, icons: true };
        },
        
        //Returns the children of an item
        getChildren : function(item){
            var children = new Array();
            
           /* if(item.Definitions != null)
                children.push(item.Definitions);           
            if(item.group != null) 
                children.push(item.group);
            if(item.command != null) 
                children.push(item.command);
            if(item.option != null)
                children.push(item.option);*/
            //AT THE MOMENT, WILL ONLY RETURN ONE SET OF EACH CHILD
            return children;
        },
        
        //Determines if the item has children
        hasChildren : function(item){
            var count = 0;
            if(item.group != null)
                count++;
            if(item.command != null)
                count++;
            if(item.option != null)
                count++;
            if(item.Definitions != null)
                count++;
            return count;
        },
        
        _doApplyCellEdit : function(){
            this.render();
        },
        
        //Formats the icons (enabled/disabled)
        _formatIcons : function(value, rowIndex, cell){
            //drag   edit   remove
        	var store = this.grid.store;
        	var item = store._arrayOfTopLevelItems[rowIndex];
        	var drag = "";
        	var edit = this._getEditIcon(rowIndex);
        	var remove = this._getRemoveIcon(store.getValue(item, "icons"), rowIndex);
        	var icons = drag + "<div style=\"width:5px;display:inline-block;\"></div>" + 
        	            edit + "<div style=\"width:5px;display:inline-block;\"></div>" + remove;
        	return icons;
        },
        
        //Returns an enabled/disabled remove icon
        _getRemoveIcon : function(enabled, rowIndex){
            if(enabled){
                return "<div class=\"spDeleteIcon_v2\"" +
                "onClick=\"dijit.byId('" + this.id + "')._removeRow('" + rowIndex + "')\"" +
                "title=\"" + this.removeTitle + "\" style=\"display:inline-block;\"></div>";
            }
            return "<div class=\"spDeleteInactiveIcon_v2\" style=\"display:inline-block;\"></div>";
        },
        
       //Returns an enabled edit icon
        _getEditIcon : function(rowIndex){
            return "<div class=\"spEditIcon_v2\"" +
                "onClick=\"dijit.byId('" + this.id + "')._editRow('" + rowIndex + "')\"" +
                "title=\"" + this.editTitle + "\" style=\"display:inline-block;\"></div>";
        },
        
        //Adds a row to the end of the grid    
        _addRow : function(){

            this.table.setAttribute("class", "table");
            this.table.setAttribute("style", "display:block");
        	var length = this.grid.store._arrayOfTopLevelItems.length;
        	this.grid.store.newItem(this.getItemToAdd(length));
        	this.grid.store.save();
        	this.grid.render(); 
        	this._setIcons();
        	this.reposition();        
        	//alert("Row added...");
        },
      //Adds a row to the end of the grid    
        _addRowButton : function(){
                        
            this.table.setAttribute("class", "table");
            this.table.setAttribute("style", "display:block");
            var length = this.grid.store._arrayOfTopLevelItems.length;
            this.grid.store.newItem(this.getItemToAdd(length));
            
            var lengthIndex = (parseInt(length)+parseInt(this.min));
            
            if(this.settings == undefined){
                this.settings = new Array();
            }
            
            this.settings[length] = { index: lengthIndex };

            this.grid.store.save();
            this.grid.render(); 
            this._setIcons();
            this.reposition();
        
            //alert("Row added...");
        },
        //Removes a row from the grid
        _removeRow : function(rowIndex){
            
            var itemCount = this.grid.store._arrayOfTopLevelItems.length;            
            var item = this.grid.store._arrayOfTopLevelItems[rowIndex];            
            var label = item.label[0] + " " + (parseInt(rowIndex) + parseInt(this.min));
            var remove = confirm(this.resources["DELETE_CONFIRM"].replace("%1", "\"" + label + "\""));
                                   
            if(remove){
                
                this.allItems.splice(rowIndex,1);
                this.settings.splice(rowIndex,1);
                                
                this.grid.store.deleteItem(item);
                this.grid.store.save();
                this.grid.render();
        		
        		if(this.grid.store._arrayOfTopLevelItems == 0){
        			//this.table.setAttribute("style", "display:none");
        		}
        		this._setIcons();
        		this.reposition();
        		//alert("Row removed...");
        	}
        },
        
        _removeWithoutConfirm : function(rowIndex){
            var item = this.grid.store._arrayOfTopLevelItems[rowIndex];
            this.grid.store.deleteItem(item);
            this.grid.store.save();
            this.grid.render();
            
            if(this.grid.store._arrayOfTopLevelItems == 0){
                //this.table.setAttribute("style", "display:none");
            }
            this._setIcons();
            this.reposition();
            //alert("Row removed...");
        },
        
        //Removes a row from the grid
        _editRow : function(rowIndex){
                        
            var item = this.allItems[rowIndex];
//            var item = this.grid.store._arrayOfTopLevelItems[rowIndex];            
                        
            var title = this.locResources.EDIT_CONFIRMATION.replace("%1", item.name[0]).replace("%2", (parseInt(rowIndex)+parseInt(this.min)));
                        
            //edit item through dialog
            var list = this;
            var editDialog = new support.dialogs.BuildAListPopup({
                title: title,
                item: item,
                path: this.path,
                settings: this.settings[rowIndex],
                onSave: function(){
                    this.settings[rowIndex] = this.getSettingsFromDialog(); //object
                                        
                    if(this.settings[rowIndex] != null){
                        var displayItem = list.grid.store._arrayOfTopLevelItems[rowIndex];
                                                
                        var settings = this.settings[rowIndex];
                        var displayPath = this.path+"__n";                        
                        var attr1 = settings[displayPath];                        
                        if(attr1 == undefined || attr1 == null){
                             displayPath = this.path+"__i6";                        
                             attr1 = settings[displayPath];                        
                             if(attr1 == undefined || attr1 == null){}else{
                                 list.grid.store.setValue(displayItem, "attr1", attr1);
                             }
                        }else{                        
                            list.grid.store.setValue(displayItem, "attr1", attr1);
                        }
                    }
                }             
            });
            editDialog.show();
            
            this.grid.store.save();
            this.grid.render();
        },
       
        _getValueAttr : function(){
            //return array of JSON objects
            
            var values = new Array();
            var delExist = false;
            var delpath = "";
                        
            for(var i=0; i<this.grid.store._arrayOfTopLevelItems.length; i++){
                                
                var obj  = new Object();
                obj["index"] = parseInt(this.min) + i;
                
                var settings = this.settings[i];
                                
                for(var j=0; j<this.item.option.length; j++){
                    var item = this.item.option[j];                                        
                    var path = this.path + "__" + item.ID[0];
                    
                    if(item.ID[0] == "del"){
                        delExist = true;
                        delpath = path;
                    }
                    
                    var value = settings[path]; // item.settings[j].value;
                                        
                    obj[path] = value;
                }
                                                
                values.push(obj);
            }
            //add empty entries for indexed values
            if(this.max != null && delExist == true){
                for(var i=this.grid.store._arrayOfTopLevelItems.length; i<this.max; i++){
                    var obj  = new Object();
                    obj["index"] = parseInt(this.min) + i;
                    obj[delpath] = true;
                    //console.log(obj);
                    values.push(obj);
                }  
            }
            
            return values;
        },
        
        _setValueAttr : function(value){
                               
            if(value != null && value != undefined){
                for(var i=0; i<value.length; i++){
                    var obj = value[i];
                    var deleteItem = false;
                    for(var attr in obj){
                        var split = attr.split("__");
                        var ID = split[split.length-1];
                                         
                        if(ID == "del" && obj[attr] == true){
                            deleteItem = true;
                        }
                    }
                    if(!deleteItem){
                        this._addRow();                                                    
                    }
                } 
            }
        },
                
        _setIcons : function(){
            var max = this.max;
            var items = this.grid.store._arrayOfTopLevelItems;
            
            
            if(max != null && items.length < max && this._addIcon != null){
                //enable drag, add, remove
                if(this._addIcon != null){
                    this._addIcon.setAttribute("title", this.addTitle);
                    this._addIcon.setAttribute("class", "spAddRowIcon_v2");
                    this._addIcon.onclick = dojo.hitch(this, function(e){
                        this._addRowButton();
                    });
                }
            }
            else{
                //disable add
                this._addIcon.setAttribute("class", "spAddRowInactiveIcon_v2");
                this._addIcon.onclick = function(e){};
                this._addIcon.setAttribute("title", "");
            }

            //disable remove
          /*  if(items.length == 0){                
                this._deleteIcon.setAttribute("class", "spDeleteInactiveIcon");
                this._deleteIcon.onclick = function(e){};
            }else{
                this._deleteIcon.setAttribute("class", "spDeleteIcon pointer");
                this._deleteIcon.onclick = dojo.hitch(this, function(e){
                    this._removeRow(0);
                });                
            }*/
        }
	});
});     	