define([ 
        "dojo", 
        "dijit", 
        "support/panels/Panel", 
        "dojo/i18n!./nls/DetailsDependentTable", 
        "dojo/text!./templates/DetailsDependentTable.html",
        "dijit/layout/LayoutContainer",
        "dijit/layout/ContentPane",
        "support/base/TableMixin",        
        "dojo/data/ItemFileWriteStore"
        ], function(dojo, dijit, panel, mri, template) {

    return dojo.declare("support.panels.CategoryDetailsDependent", [ support.panels.Panel , support.base.TableMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in managing server profiles
        // for Configuration Patterns.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "DetailsDependentTable",
        
        mode: "", // Current Context Mode
        
        _detailsTable: null,
        
        profileLayout: {            /* Table View Options */
            flatView: true, // Flat Table View
            chassisView: true, // By Chassis View
            groupView: true, // By Group View
            templateView: true // By Template View
            /* Toolbar Buttons and Menus */

            /* "More" menu options */

            /* Table Columns To Display */
        },
        
        templateIDNum:"",   //Category Template ID   
        templateType: "",   //Template Type
        
        // ID prefix used for table
        gridID: "CategoryDetailsTable",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/DetailsDependentTable.css",

        // When this method is called, all variables
        // inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget
        // property variables defined in the template
        // HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after
        // buildRendering(). This is useful to override
        // when
        // you need to access and/or manipulate DOM
        // nodes included with your widget.
        // DOM nodes and widgets with the
        // dojoAttachPoint attribute specified can now
        // be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in
        // buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);

            this._setupLayoutTemplate(this.mode);
            
            // Setup table widget...
            var tableStore = this._retrieveStore(this.mode);
            var tableStructure = this._getTableStructure(this.mode); // Get structure for current view

            this._detailsTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.gridID, true);  
            this._detailsTable.column(0).sort(false);

            setTimeout(dojo.hitch(this, function() {
    	        this._detailsTable.placeAt(this.detailsTable);
	            this._detailsTable.startup();
            }), 500);
            
            // Connect handlers to reset table capabilities
            dojo.connect(this._detailsTable.select.row, "onSelectionChange", this, this.setTableCapabilities); // Fires on row selection
            dojo.connect(this._detailsTable, "setStore", this, this.setTableCapabilities); // Fires on store reset            
        },
        
        // Connect to data source and create models for
        // TreeTable widgets.
        _retrieveStore: function(mode) {
                                               
            var tableStore = new dojo.data.ItemFileWriteStore({
                url: this.REST_PATH + "template/"+this.templateIDNum+"/dependents"
            });
      
            return tableStore;            
        },
        
        // Re-render Tree table with new columns/data
        // presentation
        _reRenderTable: function() {
            // 99275: Get function from tree table widget
        },

        // Disable or enable options for custom view modes
        _setupLayoutTemplate: function(mode) {            
            switch (mode) {
                case ("categoryTemplateDetails"):
                    this.profileLayout.portCol = true; //Port Column
                    this.profileLayout.templateCol = true; //Server Pattern
                    this.profileLayout.consumersCol= true; // Server Profile Column
                    this.profileLayout.serverCol= true; // Server Column
                    this.profileLayout.chassisCol= true; // Chassis Column
                    this.profileLayout.bayCol = true;   //Bay Column

                    break;
                case ("chassiscategoryTemplateDetails"):
                    this.profileLayout.templateCol = true; //Chassis Pattern
                    this.profileLayout.consumersCol= true; // Chassis Profile Column
                    this.profileLayout.chassisCol= true; // Chassis Column
                    this.profileLayout.bayCol = false;   //Bay Column
                    this.profileLayout.systemCol = false;   //System Column
                    
                    break;
            }
        },
        
        // Columns to be displayed on table
        _getTableStructure: function(mode) {
            var cols = new Array();

            if(mode == "categoryTemplateDetails"){

                if(this.templateType == "ExtendedPort"){

                    cols.push({
                        field: "portTemplateName",
                        name: this.locResources.PORT_TEMP,
                        width: "15em",
                        title: ""
                    });

                } 

                cols.push({
                    field: "templateName",
                    name: this.resources.SERVER_TEMP,
                    width: "21em",
                    title: ""
                });

                cols.push({
                    field: "profileName",
                    name: this.resources.SERVER_PROFILE,
                    width: "21em",
                    title: ""
                });

                cols.push({
                    field: "serverName",
                    name: this.resources.SERVER,
                    width: "21em",
                    title: ""
                });

                cols.push({
                    field: "chassisName",
                    name: this.locResources.CHAS,
                    width: "21em",
                    title: ""
                });

                cols.push({
                    field: "bayId",
                    name: this.resources.BAY,
                    formatter: dojo.hitch(this, this._formatBay),
                    title: ""
                });


            }else if(mode == "chassiscategoryTemplateDetails"){

                cols.push({
                    field: "templateName",
                    name: this.resources.CHASSIS_TEMP,
                    width: "33em",
                    title: ""
                });

                cols.push({
                    field: "profileName",
                    name: this.resources.CHASSIS_PROF,
                    width: "33em",
                    title: ""
                });

                cols.push({
                    field: "chassisName",
                    name: this.locResources.CHAS,
                    title: ""
                });
            }

            return cols;
        },
        
        _formatBay: function(item) {
            if(item.subBayId != null){                
                return item.bayId+":"+item.subBayId;
            }else{
                var bays = item.bayId.split(" , ");
                if(bays.length == 1) {
                    return item.bayId;
                } else {
                    return bays[0] + "&ndash;" + bays[bays.length-1];
                }
            }
        },

        // Get number of selected rows in tree table
        _getSelectedRowCount: function() {
            return this._detailsTable.select.row.getSelected().length;
        },

        // Get row objects for selected rows
        _getSelectedRows: function() {
            return this._detailsTable.select.row.getSelected();
        },
        // Set Table Capabilities
        setTableCapabilities: function() {
        }
    });
});