define(["dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/CategoryTemplateDetails", 
        "dojo/text!./templates/CategoryTemplateDetails.html",
        "support/base/NextGenUIMixin", 
        "support/widgets/OutlinePane", 
        "dijit/layout/TabContainer",
        "support/panels/CategoryDetailsDependent",
        "dijit/Toolbar",
        "dijit/ToolbarSeparator",
        "dijit/Menu",
        "dojo/dom"
        ], function(dojo, dijit, panel, mri, template) {
    
    return dojo.declare("support.panels.ChassisCategoryTemplateDetails", [ support.panels.Panel, support.base.NextGenUIMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in displaying information
        // associated with a server.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "ServerDetails",
        
        // the properties used to construct the title of this page in the
        //   NextGen UI
        cpPageContext: null,
        cpPageName: null,
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/ChassisCategoryTemplateDetails.css",

        configurationTabdata: "",
        ioAddressesTabdata: "",
        eventsTabdata: "",
        dependenciesTabdata: "",
        
        templateInformation: "",
        
        templateIDNum: "",
        templateName: "",
        templateDesc: "",
        templateInUse: false,
        templateType: "",
        
        postMixInProperties: function() {
            this.inherited(arguments);
            this.cpPageContext = "";
            this.cpPageName = this.resources.DETAILS;
        },

        postCreate: function() {
            this.inherited(arguments);

            if(this.CP_RUN_ENVIRONMENT == "augusta"){
                
                //Add page base in order to use the setTitle() function
                require([ "augusta/augustaUICore/core/augustaPageBase" ], dojo.hitch(this, function(PageBase) {                
                    var CP_AUGUSTA_PAGE_BASE = new augusta.augustaUICore.core.augustaPageBase();
                    dojo.safeMixin(this, CP_AUGUSTA_PAGE_BASE);            
                }));

                this.bwIsCacheable = false;
            }    

            this.subscribe("RefreshNameDiv", function(data) {
                // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
                this.cpPageContext = data;
                                
                if(this.CP_RUN_ENVIRONMENT == "fsm"){
                    //Set title in FSM
                    this.changeNextGenUITitle();
                }
                
                if(this.CP_RUN_ENVIRONMENT == "augusta"){
                    //Set Title in xHMC, only can be used if augustaPageBase mixed in
                    this.setTitle(this.cpPageContext + " - " + this.cpPageName);
                }
            });

            this.subscribe("ChassisPowerComplete", function(data) {
                // Update details page with Server Pattern new information
                this.getServerTemplateID();
                dojo.publish("RefreshNameDiv",this.templateName);
            });            
            
            this.subscribe("ChassisManagementComplete", function(data) {
                // Update details page with Server Pattern new information
                this.getServerTemplateID();
                dojo.publish("RefreshNameDiv",this.templateName);
            });
            
            this.subscribe("ChassisUserTemplateComplete", function(data) {
                // Update details page with Server Pattern new information
                this.getServerTemplateID();
                dojo.publish("RefreshNameDiv",this.templateName);
            });
            
            this.subscribe("ChassisInfoTemplateComplete", function(data) {
                // Update details page with Server Pattern new information
                this.getServerTemplateID();
                dojo.publish("RefreshNameDiv",this.templateName);
            });
            
            this.subscribe("ChassisIOTemplateComplete", function(data) {
                // Update details page with Server Pattern new information
                this.getServerTemplateID();
                dojo.publish("RefreshNameDiv",this.templateName);
            });
            
            this.subscribe("ChassisNodeTemplateComplete", function(data) {
                // Update details page with Server Pattern new information
                this.getServerTemplateID();
                dojo.publish("RefreshNameDiv",this.templateName);
            });
            
            this.getServerTemplateID();
                        
            // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
            this.cpPageContext = this.templateName;

            if(this.CP_RUN_ENVIRONMENT == "fsm"){
                //Set title in FSM
                this.changeNextGenUITitle();
            }
            
            if(this.CP_RUN_ENVIRONMENT == "augusta"){
                //Set Title in xHMC
                this.setTitle(this.cpPageContext + " - " + this.cpPageName);
            }

            this.toolbarSetup();

            // Dependencies Tab
            this.dependenciesTabdata = new support.panels.CategoryDetailsDependent({
                id: this.id + "cp_chassiscategorytemplatedetails_categorydetailsdependent",
                mode: "chassiscategoryTemplateDetails",
                templateIDNum:this.templateIDNum,
                tableWidth: "99%",
                title:this.locResources.SVRTMPL_DEPENDENTS_TAB_TITLE
            }).placeAt(this._dependenciesTab);  
                                   
            // Events Tab
            /*this.eventsTabdata = new support.panels.DetailsDependentTable({
                mode: "detailsTab", 
                tableWidth: "99%",
                title:this.locResources.EVENTS_TAB_TITLE
            }).placeAt(this._eventsTab);*/
                                                                                                                                                                                             
        },
        //Get Server Template ID
        getServerTemplateID: function(){
            var store ="";
          
            if(this.templateIDNum == ""){  //If this templateID not provided then parse url for it
                var url = window.location.href;
                var splitUrl = url.split("cpTarget=");
                var IDarray = new Array();
                var ID = "";
                
                if(splitUrl[1]){
                    IDarray = splitUrl[1].split("&");
                }else{
                    ID = "";
                }
                
                if(IDarray[0]){
                    ID=IDarray[0];
                    
                    var xhrGet = dojo.xhrGet({
                        url: this.REST_PATH + "template/" + ID,
                        handleAs: "json",
                        sync: true,
                        preventCache: true,
                        load: function(response) {
                            console.log(response);
                            store = response;
                        }
                    });
                                 
                    this.templateInformation = store;
                    
                    this.templateIDNum = ID;
                    
                }else{  //If no ID found then ...
                                                
                    
                }
    
                                                                 
            }else{ //IF ID passed then use passed ID

                var xhrGet = dojo.xhrGet({
                    url: this.REST_PATH + "template/" + this.templateIDNum,
                    handleAs: "json",
                    sync: true,
                    preventCache: true,
                    load: function(response) {
          //              console.log(response);
                        store = response;
                    }
                });
                
                this.templateInformation = store;
            }            
            
            
         //Set Template Name and Description and InUse ... etc ....            
            //Check to see if server template info was gained
            if(this.templateInformation != "" && this.templateInformation.items != ""){  
             
                //if(this.templateInformation.items[0].name){
                    this.templateName = this.templateInformation.items[0].name;
                //}
                
                if(this.templateInformation.items[0].description){
                    this.DetailsTemplateDescription.innerHTML = this.templateInformation.items[0].description;
                }
                
                //if(this.templateInformation.items[0].inUse){
                    this.templateInUse = this.templateInformation.items[0].inUse;
                //}
                  
                //if(this.templateInformation.items[0].type){
                    this.templateType = this.templateInformation.items[0].type;
                    this.DetailsTemplateType.innerHTML = this.typeTitle(this.templateType);
                //}
                    
            }

            
        },
        //ToolBar Setup
        toolbarSetup: function(){
          
            this.moreMenu = new dijit.Menu({          // Add to More/Actions Menu
                style: "display: none;"
            });
      
            this.editButton = new dijit.form.Button({     //Add Edit Button to toolbar
                id: this.id + "cp_chassiscategorytemplatedetails_editbutton",
                showLabel: false,
                iconClass: "spEditLgIcon",
                disabled: false,
                title: this.resources.EDIT_PATTERN_TITLE,
                onClick: dojo.hitch(this, this.editTemplate),
                onKeypress: dojo.hitch(this, this.editTemplate)
            });
            this.detailsToolBar.addChild(this.editButton);
            
            // Add to More/Actions Menu                    
            this.detailsSeparator = new dijit.ToolbarSeparator();
            this.detailsToolBar.addChild(this.detailsSeparator);
            
            this.copyItem = new dijit.MenuItem({
             //   id: this.id + "cp_chassiscategorytemplatedetails_copyitem",
                label: this.resources.COPY,
                title: this.resources.COPY,
                onClick: dojo.hitch(this, this.copyTemplate),
                onKeypress: dojo.hitch(this, this.copyTemplate)
            });
            this.moreMenu.addChild(this.copyItem);

            this.renameItem = new dijit.MenuItem({
            //    id: this.id + "cp_chassiscategorytemplatedetails_renameitem",
                label: this.resources.RENAME,
                title: this.resources.RENAME_PATTERN_TITLE,
                onClick: dojo.hitch(this, this.renameTemplate),
                onKeypress: dojo.hitch(this, this.renameTemplate)
            });
            this.moreMenu.addChild(this.renameItem);
            
            this.editItem = new dijit.MenuItem({
          //      id: this.id + "cp_chassiscategorytemplatedetails_edititem",
                label: this.resources.EDIT,
                title: this.resources.EDIT_PATTERN_TITLE,
                iconClass: "spEditLgIcon",
                onClick: dojo.hitch(this, this.editTemplate),
                onKeypress: dojo.hitch(this, this.editTemplate)
            });
            this.moreMenu.addChild(this.editItem,0);
            
            this.moreButton = new dijit.form.DropDownButton({
                id: this.id + "cp_chassiscategorytemplatedetails_morebutton",
                showLabel: true,
                label: this.resources.ALL_ACTIONS,
                title: this.resources.ALL_ACTIONS_TITLE,
                dropDown: this.moreMenu
            });

            this.detailsToolBar.addChild(this.moreButton);

//            if(dojo.byId("toolbarDiv")){
  //              var toolbarContainer = dojo.byId("toolbarDiv");
    //            this.detailsToolBar.placeAt(toolbarContainer);  
      //      }
            
        },        
        //Return type title
        typeTitle: function(templateType){
          
            if (templateType == "cmmPower") {
                templateType = this.resources.CHASSIS_POWER;
            }
            else if (templateType == "cmmInfo") {
                templateType = this.resources.CHASSIS_INFO;
            }
            else if (templateType == "cmmUsers") {
                templateType = this.resources.CHASSIS_USER;
            }
            else if (templateType == "cmmManagement") {
                templateType = this.resources.CHASSIS_MGMT;
            }
            else if (templateType == "cmmIOModule") {
                templateType = this.resources.CHASSIS_IO;
            }
            else if (templateType == "cmmNode") {
                templateType = this.resources.CHASSIS_NODE;
            }

            return templateType;
            
        },
        // Edit Template
        editTemplate: function() {
        
            if (this.templateType == "cmmPower") {

                require([ "support/dialogs/ChassisPowerAndAcousticsTemplate" ], dojo.hitch(this, function() {
                    var psceditdialog = new support.dialogs.ChassisPowerAndAcousticsTemplate({
                        id: this.id + "cp_chassiscategorytemplatedetails_chassispowerandacoustics",
                        templateIDNum: this.templateIDNum,
                        templateMode: "edit",
                        templateInUse: this.templateInUse,
                        templateLaunchPoint : "table"
                    });
                    psceditdialog.show();
                }));

            }
            else if (this.templateType == "cmmInfo") {

                require([ "support/dialogs/ChassisInfoTemplate" ], dojo.hitch(this, function() {
                    var sitdialog = new support.dialogs.ChassisInfoTemplate({
                        id: this.id + "cp_chassiscategorytemplatedetails_chassisinfo",
                        templateIDNum: this.templateIDNum,
                        templateMode: "edit",
                        templateInUse: this.templateInUse,
                        templateLaunchPoint : "table"
                    });
                    sitdialog.show();
                }));

            }
            else if (this.templateType == "cmmUsers") {

                require([ "support/dialogs/ChassisUserAndSecurityTemplate" ], dojo.hitch(this, function() {
                    var partdialog = new support.dialogs.ChassisUserAndSecurityTemplate({
                        id: this.id + "cp_chassiscategorytemplatedetails_chassisuserandsecurity",
                        templateIDNum: this.templateIDNum,
                        templateMode: "edit",
                        templateInUse: this.templateInUse,
                        templateLaunchPoint : "table"
                    });
                    partdialog.show();
                }));

            }
            else if (this.templateType == "cmmManagement") {

                require([ "support/dialogs/ChassisManagementInterfaceTemplate" ], dojo.hitch(this, function() {
                    var miteditdialog = new support.dialogs.ChassisManagementInterfaceTemplate({
                        id: this.id + "cp_chassiscategorytemplatedetails_chassismanagementinterface",
                        templateIDNum: this.templateIDNum,
                        templateMode: "edit",
                        templateInUse: this.templateInUse,
                        templateLaunchPoint : "table"
                    });
                    miteditdialog.show();

                }));

            }
            else if (this.templateType == "cmmIOModule") {

                require([ "support/dialogs/ChassisIOModuleTemplate" ], dojo.hitch(this, function() {
                    var miteditdialog = new support.dialogs.ChassisIOModuleTemplate({
                        id: this.id + "cp_chassiscategorytemplatedetails_chassisiomodule",
                        templateIDNum: this.templateIDNum,
                        templateMode: "edit",
                        templateInUse: this.templateInUse,
                        templateLaunchPoint : "table"
                    });
                    miteditdialog.show();

                }));


            }
            else if (this.templateType == "cmmNode") {

                require([ "support/dialogs/ChassisNodeTemplate" ], dojo.hitch(this, function() {
                    var miteditdialog = new support.dialogs.ChassisNodeTemplate({
                        id: this.id + "cp_chassiscategorytemplatedetails_chassisnode",
                        templateIDNum: this.templateIDNum,
                        templateMode: "edit",
                        templateInUse: this.templateInUse,
                        templateLaunchPoint : "table"
                    });
                    miteditdialog.show();

                }));


            }

        },
        // Copy Template
        copyTemplate: function() {

            if (this.templateType == "cmmPower") {

                require([ "support/dialogs/ChassisPowerAndAcousticsTemplate" ], dojo.hitch(this, function() {
                    var psceditdialog = new support.dialogs.ChassisPowerAndAcousticsTemplate({
                        id: this.id + "cp_chassiscategorytemplatedetails_chassispowerandacoustic",
                        templateIDNum: this.templateIDNum,
                        templateMode: "copy"
                    });
                    psceditdialog.show();

                }));


            }
            else if (this.templateType == "cmmInfo") {

                require([ "support/dialogs/ChassisInfoTemplate" ], dojo.hitch(this, function() {
                    var sitdialog = new support.dialogs.ChassisInfoTemplate({
                        id: this.id + "cp_chassiscategorytemplatedetails_chassisinfo",
                        templateIDNum: this.templateIDNum,
                        templateMode: "copy"
                    });
                    sitdialog.show();
                }));

               

            }
            else if (this.templateType == "cmmUsers") {

                require([ "support/dialogs/ChassisUsersTemplate" ], dojo.hitch(this, function() {
                    var partdialog = new support.dialogs.ChassisUserAndSecurityTemplate({
                        id: this.id + "cp_chassiscategorytemplatedetails_chassisuserandsecurity",
                        templateIDNum: this.templateIDNum,
                        templateMode: "copy"
                    });
                    partdialog.show();

                }));


            }
            else if (this.templateType == "cmmManagement") {

                require([ "support/dialogs/ChassisManagementInterfaceTemplate" ], dojo.hitch(this, function() {
                    var miteditdialog = new support.dialogs.ChassisManagementInterfaceTemplate({
                        id: this.id + "cp_chassiscategorytemplatedetails_chassismanagementinterface",
                        templateIDNum: this.templateIDNum,
                        templateMode: "copy"
                    });
                    miteditdialog.show();

                }));


            }
            else if (this.templateType == "cmmIOModule") {

                require([ "support/dialogs/ChassisIOModuleTemplate" ], dojo.hitch(this, function() {
                    var miteditdialog = new support.dialogs.ChassisIOModuleTemplate({
                        id: this.id + "cp_chassiscategorytemplatedetails_chassisiomodule",
                        templateIDNum: this.templateIDNum,
                        templateMode: "copy"
                    });
                    miteditdialog.show();

                }));


            }
            else if (this.templateType == "cmmNode") {

                require([ "support/dialogs/ChassisNodeTemplate" ], dojo.hitch(this, function() {
                    var miteditdialog = new support.dialogs.ChassisNodeTemplate({
                        id: this.id + "cp_chassiscategorytemplatedetails_chassisnode",
                        templateIDNum: this.templateIDNum,
                        templateMode: "copy"
                    });
                    miteditdialog.show();

                }));


            }

        
        },
        // Rename Template
        renameTemplate: function() {

            require([ "support/dialogs/Rename" ], dojo.hitch(this, function() {
                var renamedialog = new support.dialogs.Rename({
                    id: this.id + "cp_chassiscategorytemplatedetails_rename",
                    templateIDNum: this.templateIDNum,
                    launchPoint: "details"
                });
                renamedialog.show();

            }));

            
        }
    });

});