define(["dojo",
        "dijit",
        "support/ConfigPatternsAll",
        "support/panels/Panel",
        "dojo/i18n!./nls/ChassisDetails",
        "dojo/text!./templates/ChassisDetails.html",
        "support/base/NextGenUIMixin",
        "support/widgets/OutlinePane",
        "dijit/layout/TabContainer",
        "support/panels/ServerConfiguration",
        "support/temp/ModalDialog", 
        "dijit/Toolbar",
        "dijit/ToolbarSeparator",
        "dijit/Menu",
        "support/dialogs/ChassisTemplateWizard",
        "dojo/dom"
        ], function(dojo, dijit, layer, panel, mri, template) {

    
    return dojo.declare("support.panels.ChassisDetails", [ support.panels.Panel, support.base.NextGenUIMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in displaying information
        // associated with a server.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "ChassisDetails",
        
        // the properties used to construct the title of this page in the
        //   NextGen UI
        cpProductName: null,
        cpPageContext: null,
        cpPageName: null,
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/ChassisDetails.css",

        configurationTabdata: "",
        ioAddressesTabdata: "",
        eventsTabdata: "",
        dependenciesTabdata: "",
        
        detailsMode: "",
        templateIDNum: "",
        templateName: "",
        chassisUUID: "",

        servertemplateID:"",
        serverprofileID:"",

        serverprofileStatustxt:"",
        servertemplateStatustxt:"",
        profileStatustxt:"",
        failoverStatustxt:"",
        
        serverUUID:"",
        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
            this.cpProductName = this.resources.PRODUCT_NAME;
            this.cpPageContext = this.resources.CHASSIS;
            this.cpPageName = this.resources.DETAILS;
        },

        // postCreate() is called after buildRendering(). This is useful to override when you need
        // to access and/or manipulate DOM nodes included with your widget. DOM nodes and widgets
        // with the dojoAttachPoint attribute specified can now be directly accessed as fields on
        // "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            
            if(this.CP_RUN_ENVIRONMENT == "augusta"){
                
                //Add page base in order to use the setTitle() function
                require([ "augusta/augustaUICore/core/augustaPageBase" ], dojo.hitch(this, function(PageBase) {                
                    var CP_AUGUSTA_PAGE_BASE = new augusta.augustaUICore.core.augustaPageBase();
                    dojo.safeMixin(this, CP_AUGUSTA_PAGE_BASE);            
                }));

                this.bwIsCacheable = false;
            }    

            if(this.CP_RUN_ENVIRONMENT == "fsm"){
                //Set title in FSM
                this.changeNextGenUITitle();
            }
            
            if(this.CP_RUN_ENVIRONMENT == "augusta"){
                //Set Title in xHMC, only can be used if augustaPageBase mixed in
                this.setTitle(this.cpPageContext + " - " + this.cpPageName);
            }
            
            // obtain the chassis UUID; must obtain this from the URL string,
            //   and also this must be done before _setTableData() can be
            //   called
            var epObj = this.getEndPointFromSourceToken();
            if(epObj != null) {
                this.chassisUUID = epObj.uuid;
            }
            
            this.moreMenu = new dijit.Menu({          // Add to More/Actions Menu
                style: "display: none;"
            });
            
            this.createButton = new dijit.form.Button({     //Add Create Button to toolbar
                showLabel: false,
                iconClass: "spCreateLgIcon",
                disabled: false,
                title: this.resources.CREATE_CHASSIS_PATTERN_TITLE,
                onClick: dojo.hitch(this, this.createServerTemplate),
                onKeypress: dojo.hitch(this, this.createServerTemplate)
            });
            
            this.editButton = new dijit.form.Button({     //Add Edit Button to toolbar
                showLabel: false,
                iconClass: "spEditLgIcon",
                disabled: false,
                title: this.resources.EDIT_CHASSIS_PATTERN_TITLE,
                onClick: dojo.hitch(this, this.editTemplate),
                onKeypress: dojo.hitch(this, this.editTemplate)
            });
            
            this.deployButton = new dijit.form.Button({     //Add Deploy Button to toolbar
                showLabel: false,
                iconClass: "spDeployLgIcon",
                disabled: false,
                title: this.resources.DEPLOY_CHASSIS_PATTERN_TITLE,
                onClick: dojo.hitch(this, this.deployTemplate),
                onKeypress: dojo.hitch(this, this.deployTemplate)
            });
            
            this.restartButton = new dijit.form.Button({     //Add Restart Button to toolbar
                showLabel: false,
                iconClass: "spRestartLgIcon",
                disabled: false,
                title: this.locResources.ACTIVATE,
                onClick: dojo.hitch(this, this.activateProfile),
                onKeypress: dojo.hitch(this, this.unassignProfile)
            });
            
            this.detailsToolBar.addChild(this.createButton);
            this.detailsToolBar.addChild(this.editButton);
            this.detailsToolBar.addChild(this.deployButton);
            //         this.detailsToolBar.addChild(this.restartButton);
            
            // Add to More/Actions Menu
            this.detailsSeparator = new dijit.ToolbarSeparator();
            this.detailsToolBar.addChild(this.detailsSeparator);
            
            this.copyItem = new dijit.MenuItem({
                label: this.resources.COPY_CHASSIS_PATTERN,
                title: this.resources.COPY_CHASSIS_PATTERN_TITLE,
                onClick: dojo.hitch(this, this.copyTemplate),
                onKeypress: dojo.hitch(this, this.copyTemplate)
            });
            
            this.assignItem = new dijit.MenuItem({
                label: this.resources.DEPLOY_CHASSIS_PROFILE,
                title: this.resources.DEPLOY_CHASSIS_PROFILE_TITLE,
                onClick: dojo.hitch(this, this.assignProfile),
                onKeypress: dojo.hitch(this, this.assignProfile)
            });
            
            this.unassignItem = new dijit.MenuItem({
                label: this.resources.UNASSIGN_CHASSIS_PROFILE,
                title: this.resources.UNASSIGN_CHASSIS_PROFILE_TITLE,
                onClick: dojo.hitch(this, this.unassignProfile),
                onKeypress: dojo.hitch(this, this.unassignProfile)
            });
            
            this.moreButton = new dijit.form.DropDownButton({
                showLabel: true,
                label: this.resources.MORE,
                dropDown: this.moreMenu
            });
            
            this.detailsToolBar.addChild(this.moreButton);
            
            this.detailsSeparator = new dijit.ToolbarSeparator();
            this.detailsToolBar.addChild(this.detailsSeparator);
            
            this.refreshButton = new dijit.form.Button({     //Add Deploy Button to toolbar
                showLabel: false,
                iconClass: "spRefreshInactiveIcon",
                disabled: true,
                title: this.resources.REFRESH_PAGE,
                onClick: dojo.hitch(this, this.refreshPage),
                onKeypress: dojo.hitch(this, this.refreshPage)
            });
            
            this.detailsToolBar.addChild(this.refreshButton);
            
            //        this.DetailsTemplateName.innerHTML = "Server Name";

/*                if(dojo.byId("toolbarDiv")){
                    var toolbarContainer = dojo.byId("toolbarDiv");
                    this.detailsToolBar.placeAt(toolbarContainer);  
                }*/
            
            // Configurations Tab
            this.configurationTabdata = new support.panels.ServerConfiguration({
                endpointUUID: this.chassisUUID,
                mode: "chassisDetails",
                title:this.locResources.CONFIG_TAB_TITLE
            }).placeAt(this._configurationTab);
            
            // setup variables; must be done before
            //   showChassisTemplateWizardDialog() can be called
            this._setTableData();

            // (possibly) render the Chassis Template Wizard dialog on top of
            //   this panel
            this.showChassisTemplateWizardDialog();
            
          //Refresh status bar data after Chassis Pattern Wizard changed and saved
            this.subscribe("ChassisTemplateComplete", function(data) {
                this.refreshPage();
            });
            
          //Refresh status bar data after Server or Chassis Pattern Redeploy
            this.subscribe("RefreshAfterReDeploy", function(data) {
                this.refreshPage();
            });
            
          //Refresh status bar data after Chassis Pattern Deploy
            this.subscribe("DeployChassisTemplateComplete", function(data) {
                this.refreshPage();
            });

            //Refresh Complete
            this.subscribe("RefreshPageComplete", function() {
                this.refreshButton.set("iconClass", "spRefreshIcon");
                this.refreshButton.set("disabled", false);
            });

        },
        
        showChassisTemplateWizardDialog: function() {
            var currentURL = window.location.href;
            
            // if this widget was launched by the Edit Chassis Template ITask,
            //   it will contain a special URL parameter; if this parameter is
            //   present, we know we must launch the Chassis Template Wizard
            //   dialog
            if(currentURL.indexOf("cpParm") != -1 &&
                currentURL.indexOf("ectTask") != -1) {
                // show the Server Template Wizard dialog, but nest these
                //   calls in a try/catch; if there is an error instantiating
                //   the dialog, we still want this panel to be displayed
                var chassisUUID = this.getEndPointPropertiesFromSourceToken().uuid;
                var chassisStr = " ";
                try {
                    var ctwdialog = null;
                    if(this.servertemplateID != null && this.servertemplateID != "") {
                        ctwdialog = new support.dialogs.ChassisTemplateWizard({
                            templateIDNum: this.servertemplateID,
                            templateInUse: true,
                            editMode: true
                        });
                    }
                    else {
                        ctwdialog = new support.dialogs.ChassisTemplateWizard({
                            templateIDNum: null,
                            editMode: true
                        });
                    }
                    chassisStr = " (for chassis with UUID: " + chassisUUID + ") ";
                    ctwdialog.show();
                    // no need to dismiss or delete this dialog; the Chassis
                    //   Template Wizard dialog does this when the user clicks
                    //   OK or Cancel
                    console.log("Showing Chassis Template Wizard dialog" + chassisStr + "from Chassis Details page");
                }
                catch(err) {
                    console.error("Failed to show Chassis Template Wizard dialog" + chassisStr + "from Chassis Details page; error: " + err.message);
                }
            }
        },

        _setTableData: function() {

           var xhrGet = dojo.xhrGet({
               url: this.REST_PATH + "chassis/"+this.chassisUUID+"/status",
               handleAs: "json",
               sync: true,
               preventCache: true,
               load: dojo.hitch(this, function(response, ioArgs) {
                   var store = response;
                   
                   
                   if(store.items){
                       
                       if(store.items == ""){

                           
                       }else{

//                              store.items[0].cmmProfileID = "12";
                           this.serverUUID =  store.items[0].uuid;

                           if(store.items[0].cmmProfileID == null || store.items[0].cmmProfileID == ""){  //Set Server Profile Status

                               this.serverprofileStatus.innerHTML = this.resources.UNASSIGNED;
//                               this.moreMenu.addChild(this.assignItem,0);

                           }else{
                               this.moreMenu.addChild(this.assignItem,0);
                               
                               this.serverprofileStatus.innerHTML = store.items[0].cmmProfileName;

                               this.moreMenu.addChild(this.unassignItem,0);
                           }
                           
                           this.serverprofileStatustxt = store.items[0].cmmProfileName;

                           this.serverprofileID = store.items[0].cmmProfileID;

                  //         store.items[0].cmmTemplateId = "17";
                           
                           if(store.items[0].cmmTemplateId == null || store.items[0].cmmTemplateId == ""){

                               this.servertemplateStatus.innerHTML = this.resources.UNASSIGNED;                            

                               this.createButton.set("iconClass", "spCreateLgIcon");
                               this.createButton.set("disabled", false);

                               this.deployButton.set("iconClass", "spDeployLgIcon");
                               this.deployButton.set("disabled", false);
                               
                               this.editButton.set("iconClass", "spEditInactiveLgIcon");
                               this.editButton.set("disabled", true);

                           }else{
                               
                               this.servertemplateStatus.innerHTML = store.items[0].cmmTemplateName;

                               this.createButton.set("iconClass", "spCreateInactiveLgIcon");
                               this.createButton.set("disabled", true);
                               
                               this.deployButton.set("iconClass", "spDeployInactiveLgIcon");
                               this.deployButton.set("disabled", true);
                                                           
                               this.editButton.set("iconClass", "spEditLgIcon");
                               this.editButton.set("disabled", false);

                           }

                           this.servertemplateStatustxt = store.items[0].cmmTemplateName;
                           
                           this.servertemplateID = store.items[0].cmmTemplateId;


                           this.serverTitle.innerHTML = store.items[0].chassisName;
                           
//                           store.items[0].configStatus = "MATCHES_PROFILE";
                           
                          /* if(store.items[0].cmmTemplateId == null){

                               this.moreMenu.addChild(this.assignItem,0);

                               this.profileStatus.innerHTML = "No template assigned<br/>Action: Deploy Server Template";
                               
                           }else*/ if(store.items[0].configStatus == "NO_PROFILE"){

//                               this.moreMenu.addChild(this.assignItem,0);

                               this.profileStatus.innerHTML = this.resources.NOTEMPASSIGNED+"<br/>"+this.resources.ACTION_DEPLOYC;

                               this.OfflineIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "STANDBY"){
                               
                               this.profileStatus.innerHTML = this.resources.STANDBYSTATUS;

                               this.NormalIcon.style.display = "block";

                             // this.moreMenu.addChild(this.failbackItem);

                           }else if(store.items[0].configStatus == "BUILDING_PROFILE"){
                               
                               this.profileStatus.innerHTML = this.resources.BUILDINGSTATUS;

                               this.ProcessingIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "SAVING_PROFILE"){

                               this.profileStatus.innerHTML = this.resources.SAVINGSTATUS;

                               this.ProcessingIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "PENDING_PROFILE"){

                               this.profileStatus.innerHTML = this.resources.PENDING_ACTIVATION+"<br/>"+this.resources.ACTION_DEPLOYC;

                            //   this.moreMenu.addChild(this.unassignItem,0);

                               this.WarningIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "ACTIVATING_PROFILE"){

                               this.profileStatus.innerHTML = this.resources.ACTIVATINGSTATUS;
                               
                               //this.moreMenu.addChild(this.unassignItem,0);

                               this.ProcessingIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "ERROR_CREATING_PROFILE"){

                               this.profileStatus.innerHTML = this.resources.PROFILECREATEFAILED;

                             //  this.moreMenu.addChild(this.assignItem,0);

                               this.WarningIcon.style.display = "block";


                           }else if(store.items[0].configStatus == "ERROR_ACTIVATING_PROFILE"){

                               this.profileStatus.innerHTML = this.resources.PROFILEACTIVATIONFAILED;

                               // Default to standard fail, but override in the case that we know more
                               if(store.items[0].activationStatus == "LDAPERROR"){
                                   this.profileStatus.innerHTML = this.resources.PROFILEACTIVATIONLOAD;
                               } 
                               else if(store.items[0].activationStatus == "CONFIGERROR"){
                                   this.profileStatus.innerHTML = this.resources.PROFILEACTIVATIONERRORS;
                               }

                              // this.moreMenu.addChild(this.assignItem,0);

                               this.WarningIcon.style.display = "block";


                           }else if(store.items[0].configStatus == "RESTARTING"){
                               
                               this.profileStatus.innerHTML = this.resources.RESTARTINGSERVER;

                               this.ProcessingIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "TEMPLATE_DEPLOYED"){
                               
                               this.profileStatus.innerHTML = this.resources.PROFILEASSIGNED;

                               this.moreMenu.addChild(this.copyItem);
                     //          this.moreMenu.addChild(this.unassignItem,0);

                               this.SuccessIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "PROFILE_ACTIVATED"){

                               this.profileStatus.innerHTML = this.resources.PROFILEASSIGNED;

                               this.moreMenu.addChild(this.copyItem);
                          //     this.moreMenu.addChild(this.unassignItem,0);
                               this.moreMenu.addChild(this.assignItem,0);
                               
                               this.SuccessIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "MATCHES_PROFILE"){

                               this.profileStatus.innerHTML = this.resources.SERVERMATCHES;

                               this.moreMenu.addChild(this.copyItem);
                         //      this.moreMenu.addChild(this.unassignItem,0);
                               this.moreMenu.addChild(this.assignItem,0);
                               
                               this.SuccessIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "OUT_OF_COMPLIANCE"){

                               this.profileStatus.innerHTML = this.resources.OUTOFCOMPLIANCE+"<br/>"+this.resources.ACTION_DEPLOYC;

                               this.moreMenu.addChild(this.copyItem);
                           //    this.moreMenu.addChild(this.unassignItem,0);
                               
                               this.WarningIcon.style.display = "block";

                           }

                           this.profileStatustxt = store.items[0].configStatus;
                                                                                       
                       }
                    } 
               })
           });
                        
        },
        assignProfile: function(){
            var confirmedDeploy = function() {

                dojo.xhrPost({
                    url: this.REST_PATH + "profile/assign/" + this.serverprofileID + "/CMM/",
                    handleAs: "json",
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    sync: true,
                    load: function(data) {
                        if(data && data.error && data.error.length > 0){
                            var validationpopup = new support.temp.ModalDialog({
                                type: "Error",
                                text: data.error
                            });
                            validationpopup.show();
                        }
                        else{
                            dojo.publish("AssignChassisProfileComplete");
                        }
                        
                    },
                    error: dojo.hitch(this,function(data) {
                        var validationpopup = new support.temp.ModalDialog({
                            type: "Error",
                            text: this.resources.ERROR_ASSIGN_PROF
                        });
                        console.log(data);
                        validationpopup.show();
                    })                
                });

                this.refreshPage();

            };
            

            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.DEPLOY,
                        type: "Warning",
                        text: this.locResources.DEPLOY_CONF,
                        info: this.locResources.DEPLOY_CONF_LIST.replace("%1", this.serverName)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedDeploy, this);                                       

                }));
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                
                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.DEPLOY,
                        type: "Warning",
                        text: this.locResources.DEPLOY_CONF,
                        info: this.locResources.DEPLOY_CONF_LIST.replace("%1", this.serverName)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedDeploy, this);                                       

                }));

                                
            }

        },
        createServerTemplate: function() {

            var stwdialog = new support.dialogs.ChassisTemplateWizard({
                templateIDNum: null,
                editMode: false
            });
            
            stwdialog.show();

            this.refreshPage();
        },       
        // Deploy Template
        deployTemplate: function() {

            require([ "support/dialogs/DeployChassisTemplate" ], dojo.hitch(this, function() {

                var dst = new support.dialogs.DeployChassisTemplate({
                    templateId: this.servertemplateID
                });
                dst.show();
                
            }));

            

            this.refreshPage();
        },
        // Edit Template
        editTemplate: function() {
        
            var stweditdialog = new support.dialogs.ChassisTemplateWizard({
                templateIDNum: this.servertemplateID,
                templateInUse: true,
                editMode: true
            });
            stweditdialog.show();

            this.refreshPage();
        },
        // Copy Template
        copyTemplate: function() {

            var stweditdialog = new support.dialogs.ChassisTemplateWizard({
                templateIDNum: this.servertemplateID.toString(),
                copyMode: true
            });
            stweditdialog.show();
            
            this.refreshPage();

        }, // Unassign Profile Menu Option
        unassignProfile: function() {
            //99292 get serverId from REST call when available and unassign for every selected row/profile

         var confirmedUnassign = function() {
             dojo.xhrPost({
                 url: this.REST_PATH + "profile/unassign/" + this.serverprofileID,
                 handleAs: "json",
                 headers: {
                     "Content-Type": "application/json; charset=utf-8"
                 },
                 sync: true,
                 load: function(data) {
                     if(data && data.error && data.error.length > 0){
                         var validationpopup = new support.temp.ModalDialog({
                             type: "Error",
                             text: data.error
                         });
                         validationpopup.show();
                     }
                     else{
                         dojo.publish("UnassignChassisProfileComplete");
                     }
                     
                 },
                 error: dojo.hitch(this,function(data) {
                     var validationpopup = new support.temp.ModalDialog({
                         type: "Error",
                         text: this.resources.ERROR_UNASSIGN_PROF
                     });
                     console.log(data);
                     validationpopup.show();
                 })
             });
             
             this.refreshPage();
            };
            
            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.UNASSIGN,
                        type: "Warning",
                        text: this.resources.CONFIRM_UNASSIGN,
                        info: this.resources.UNASSIGN_CHASSIS.replace("%1", this.serverprofileStatustxt)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedUnassign, this);
            
                }));
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.UNASSIGN,
                        type: "Warning",
                        text: this.resources.CONFIRM_UNASSIGN,
                        info: this.resources.UNASSIGN_CHASSIS.replace("%1", this.serverprofileStatustxt)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedUnassign, this);
            
                }));
                
            }
            
        },
        //Refresh Page
        refreshPage: function(){
            
            this.refreshButton.set("iconClass", "spRefreshInactiveIcon");
            this.refreshButton.set("disabled", true);

            this.SuccessIcon.style.display = "none";
            this.OfflineIcon.style.display = "none";
            this.ProcessingIcon.style.display = "none";
            this.WarningIcon.style.display = "none";
            this.NormalIcon.style.display = "none";

            this._setTableData();  //Refresh Status Bar
            
            dojo.publish("ServerDetailRefreshButtonClicked"); //Refresh Chassis Details Table


        }

    });

});