define(["dojo",
        "dijit",
        "support/ConfigPatternsAll",
        "support/panels/Panel",
        "dojo/i18n!./nls/ChassisProfileDetails",
        "dojo/text!./templates/ChassisProfileDetails.html",
        "support/base/NextGenUIMixin",
        "support/widgets/OutlinePane", 
        "dijit/layout/TabContainer",
        "support/panels/ServerConfiguration",
        "dijit/Toolbar","dijit/ToolbarSeparator",
        "dijit/Menu",
        "support/dialogs/ChassisTemplateWizard",
        "support/temp/ModalDialog",
        "dojo/dom",
        "support/dialogs/ProgressDialog"
        ], function(dojo, dijit, layer, panel, mri, template) {

    
    return dojo.declare("support.panels.ChassisProfileDetails", [ support.panels.Panel, support.base.NextGenUIMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in displaying information
        // associated with a server.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "ChassisDetails",
        
        // the properties used to construct the title of this page in the
        //   NextGen UI
        cpPageContext: null,
        cpPageName: null,
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/ChassisProfileDetails.css",

        configurationTabdata: "",
        ioAddressesTabdata: "",
        eventsTabdata: "",
        dependenciesTabdata: "",
                
        templateIDNum: "", //ID pulled from URL or passed in when unit testing
        
        templateID:"",      //Pattern ID
        templateName: "",   //Pattern Name

        profileID:"",       //Profile ID
        profileName: "",    //Profile Name
        profileStatus: "",  //Profile Status

        chassisName: "",    //Chassis Name
        serverName:"",       //Server Name
        UUID:"",            //Server UUID

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
            this.cpPageContext = "";
            this.cpPageName = this.resources.DETAILS;
        },

        // postCreate() is called after buildRendering(). This is useful to override when you need
        // to access and/or manipulate DOM nodes included with your widget. DOM nodes and widgets
        // with the dojoAttachPoint attribute specified can now be directly accessed as fields on
        // "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);

            if(this.CP_RUN_ENVIRONMENT == "augusta"){
                
                //Add page base in order to use the setTitle() function
                require([ "augusta/augustaUICore/core/augustaPageBase" ], dojo.hitch(this, function(PageBase) {                
                    var CP_AUGUSTA_PAGE_BASE = new augusta.augustaUICore.core.augustaPageBase();
                    dojo.safeMixin(this, CP_AUGUSTA_PAGE_BASE);            
                }));

                this.bwIsCacheable = false;
            }    

            this._getProfileID();

            this.toolbarSetup();

            this._getProfileInfo();
            
            this.subscribe("RefreshNameDiv", function(data) {
                // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
                this.cpPageContext = data;

                if(this.CP_RUN_ENVIRONMENT == "fsm"){
                    //Set title in FSM
                    this.changeNextGenUITitle();
                }
                
                if(this.CP_RUN_ENVIRONMENT == "augusta"){
                    //Set Title in xHMC, only can be used if augustaPageBase mixed in
                    this.setTitle(this.cpPageContext + " - " + this.cpPageName);
                }

            });

            // Configurations Tab
            this.configurationTabdata = new support.panels.ServerConfiguration({
                id: this.id + "cp_chassisprofiledetails_serverconfiguration",
                mode: "chassisProfileDetails",
                profileIDNum: this.profileID,
                title:this.locResources.CONFIG_TAB_TITLE
            }).placeAt(this._configurationTab); 
               
            
            // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
            this.cpPageContext = this.profileName;

            if(this.CP_RUN_ENVIRONMENT == "fsm"){
                //Set title in FSM
                this.changeNextGenUITitle();
            }
            
            if(this.CP_RUN_ENVIRONMENT == "augusta"){
                //Set Title in xHMC
                this.setTitle(this.cpPageContext + " - " + this.cpPageName);
            }

            
          //Refresh status bar data after Chassis Pattern Wizard changed and saved
            this.subscribe("ChassisTemplateComplete", function(data) {
                this.refreshPage();
            });
            
          //Refresh status bar data after Server or Chassis Pattern Redeploy
            this.subscribe("RefreshAfterReDeploy", function(data) {
                this.refreshPage();
            });
            
            //Refresh Complete
            this.subscribe("RefreshPageComplete", function() {

                this.refreshButton.set("iconClass", "spRefreshIcon");
                this.refreshButton.set("disabled", false);

            });

        },                
        //ToolBar Setup
        toolbarSetup: function(){
            
            this.moreMenu = new dijit.Menu({          // Add to More/Actions Menu
                style: "display: none;"
            });
      
            this.editButton = new dijit.form.Button({     //Add Edit Button to toolbar
                id: this.id + "cp_chassisprofiledetails_editbutton",
                showLabel: false,
                iconClass: "spEditLgIcon",
                disabled: false,
                title: this.resources.EDIT_CHASSIS_PATTERN_TITLE,
                onClick: dojo.hitch(this, this.editTemplate),
                onKeypress: dojo.hitch(this, this.editTemplate)
            });
                    
            this.deployButton = new dijit.form.Button({     //Add Deploy Button to toolbar
                id: this.id + "cp_chassisprofiledetails_deploybutton",
                showLabel: false,
                iconClass: "spDeployProfile",
                disabled: false,
                title: this.resources.DEPLOY_CHASSIS_PROFILE_TITLE,
                onClick: dojo.hitch(this, this.deployProfile),
                onKeypress: dojo.hitch(this, this.deployProfile)
            });

            this.detailsToolBar.addChild(this.editButton);
            this.detailsToolBar.addChild(this.deployButton);

                    
            // Add to More/Actions Menu                    
            this.detailsSeparator = new dijit.ToolbarSeparator();
            this.detailsToolBar.addChild(this.detailsSeparator);

            this.copyItem = new dijit.MenuItem({
     //           id: this.id + "cp_chassisprofiledetails_copyitem",
                label: this.resources.COPY_CHASSIS_PATTERN,
                title: this.resources.COPY_CHASSIS_PATTERN_TITLE,
                onClick: dojo.hitch(this, this.copyTemplate),
                onKeypress: dojo.hitch(this, this.copyTemplate)
            });

            this.renameItem = new dijit.MenuItem({
       //         id: this.id + "cp_chassisprofiledetails_renameitem",
                label: this.resources.RENAME,
                title: this.resources.RENAME_PROF,
                onClick: dojo.hitch(this, this.renameProfile),
                onKeypress: dojo.hitch(this, this.renameProfile)
            });
            
            this.unassignItem = new dijit.MenuItem({
       //         id: this.id + "cp_chassisprofiledetails_unassignitem",
                label: this.resources.UNASSIGN_CHASSIS_PROFILE,
                title: this.resources.UNASSIGN_CHASSIS_PROFILE_TITLE,
                onClick: dojo.hitch(this, this.unassignProfile),
                onKeypress: dojo.hitch(this, this.unassignProfile)
            });
            
            this.editItem = new dijit.MenuItem({
       //         id: this.id + "cp_chassisprofiledetails_edititem",
                label: this.resources.EDIT_CHASSIS_PATTERN,
                title: this.resources.EDIT_CHASSIS_PATTERN_TITLE,
                iconClass: "spEditLgIcon",
                onClick: dojo.hitch(this, this.editTemplate),
                onKeypress: dojo.hitch(this, this.editTemplate)
            });
            
            this.deployItem = new dijit.MenuItem({
         //       id: this.id + "cp_chassisprofiledetails_deployitem",
                label: this.resources.DEPLOY_CHASSIS_PROFILE,
                title: this.resources.DEPLOY_CHASSIS_PROFILE_TITLE,
                iconClass: "spDeployLgIcon",
                onClick: dojo.hitch(this, this.deployProfile),
                onKeypress: dojo.hitch(this, this.deployProfile)
            });
            
            this.moreMenu.addChild(this.deployItem);
            
            this.moreButton = new dijit.form.DropDownButton({
                id: this.id + "cp_chassisprofiledetails_morebutton",
                showLabel: true,
                label: this.resources.ALL_ACTIONS,
                title: this.resources.ALL_ACTIONS_TITLE,
                dropDown: this.moreMenu
            });

            this.detailsToolBar.addChild(this.moreButton);                        

            this.detailsSeparator = new dijit.ToolbarSeparator();
            this.detailsToolBar.addChild(this.detailsSeparator);
            
            this.refreshButton = new dijit.form.Button({     //Add Deploy Button to toolbar
                id: this.id + "cp_chassisprofiledetails_refreshbutton",
                showLabel: false,
                iconClass: "spRefreshInactiveIcon",
                disabled: true,
                title: this.resources.REFRESH_PAGE,
                onClick: dojo.hitch(this, this.refreshPage),
                onKeypress: dojo.hitch(this, this.refreshPage)
            });
            

             this.detailsToolBar.addChild(this.refreshButton);
        },        
        _getProfileInfo: function(){
          
            var store = "";

            dojo.xhrGet({
                url: this.REST_PATH + "cmmprofile/"+this.profileID,                        
                handleAs: "json",
                sync: true,
                preventCache: true,
                load: function(response) {
                    store = response; 
                }
            });
            
            //Check if items array empty , if not load variables 
            if(store.items){

                if(store.items == ""){}else{

                    this.profileName = store.items[0].profileName;  //Set Profile Name
                    this.profileID = store.items[0].id;  //Set Profile ID
                    this.profileStatus = store.items[0].profileStatus;  //Set Profile Status
                                        
                    this.templateName = store.items[0].templateName; //Set Template Name                    
                    this.templateID = store.items[0].templateId; //Set Template ID

                    this.serverName = store.items[0].serverName; //Set Server Name
                    this.chassisName = store.items[0].chassisName; //Set Chassis Name
                    this.UUID = store.items[0].uuid;

                }
                
                
            }
            
            //If templateID not empty then set edit enable and add copy option
            if(this.templateID == "" || this.templateID == null){

                //Disable Edit button if no Template ID exist
                this.editButton.set("iconClass", "spEditInactiveLgIcon");
                this.editButton.set("disabled", true);
                this.moreMenu.removeChild(this.editItem);

            }else{

                // Add Copy Template to More Menu
                this.moreMenu.addChild(this.copyItem);
            
                //Enabled Template Edit Button
                this.editButton.set("iconClass", "spEditLgIcon");
                this.editButton.set("disabled", false);
                this.moreMenu.addChild(this.editItem,0);
                
            }

            //Set Server Profile Name
            this.serverprofileStatus.innerHTML = this.profileName;
            //Set Server Pattern Name
            this.servertemplateStatus.innerHTML = this.templateName;
            
            
            //Set Pattern, Profile, and Server Status in top Status Bar            
            if (this.profileStatus == "ASSIGNED" || this.profileStatus == "assigned" || this.profileStatus == "Assigned") {

                //Set Server name
                this.serverTitle.innerHTML = this.serverName;                

                // Set Profile Status Column
                this.profileStatusDiv.innerHTML = this.resources.PROFILEASSIGNED; 
                this.SuccessIcon.style.display = "block";

                //Add Available Actions
                this.moreMenu.addChild(this.renameItem);
                this.moreMenu.addChild(this.unassignItem);

            }else if (this.profileStatus == "UNASSIGNED" || this.profileStatus == "unassigned" || this.profileStatus == "Unassigned") {

                //Set Server name
                if(this.serverName == null || this.servername == ""){
                    this.serverTitle.innerHTML = this.resources.UNASSIGNED;
                }else{
                    this.serverTitle.innerHTML = this.serverName;                
                }

                // Set Profile Status Column
                this.profileStatusDiv.innerHTML = this.resources.UNASSIGNED; 
                this.OfflineIcon.style.display = "block";

                //Add Available Actions
                this.moreMenu.addChild(this.renameItem);

           }else if (this.profileStatus == "ACTIVATING" || this.profileStatus == "activating" || this.profileStatus == "Activating") {

               //Set Server name
               if(this.serverName == null || this.servername == ""){
                   this.serverTitle.innerHTML = this.resources.UNASSIGNED;
               }else{
                   this.serverTitle.innerHTML = this.serverName;                
               }

               // Set Profile Status Column
               this.profileStatusDiv.innerHTML = this.resources.ACTIVATINGSTATUS; 
               this.ProcessingIcon.style.display = "block";

               //Add Available Actions
               this.moreMenu.addChild(this.renameItem);

           }else if (this.profileStatus == "ERROR_ACTIVATING" || this.profileStatus == "error_activating" || this.profileStatus == "Error_activating") {

               //Set Server name
               if(this.serverName == null || this.servername == ""){
                   this.serverTitle.innerHTML = this.resources.UNASSIGNED;
               }else{
                   this.serverTitle.innerHTML = this.serverName;                
               }

               // Set Profile Status Column
               this.profileStatusDiv.innerHTML = this.resources.PROFILEACTIVATIONFAILED; 
               this.WarningIcon.style.display = "block";

               //Add Available Actions
               this.moreMenu.addChild(this.renameItem);

           }else if (this.profileStatus == "PLACEHOLDER" || this.profileStatus == "placeholder" || this.profileStatus == "Placeholder") {

               //Set Server name
               this.serverTitle.innerHTML = this.serverName;                

               // Set Profile Status Column
               this.profileStatusDiv.innerHTML = this.resources.STANDBYSTATUS;
               this.NormalIcon.style.display = "block";

               //Add Available Actions
               this.moreMenu.addChild(this.renameItem);               
           }
            
            
        },
        //Get Profile ID
        _getProfileID: function(){
          
            if(this.templateIDNum == ""){  //If this Profile ID not provided then parse url for it
                var url = window.location.href;
                var splitUrl = url.split("cpTarget=");
                var IDarray = new Array();
                var ID = "";
                
                if(splitUrl[1]){
                    IDarray = splitUrl[1].split("&");
                }else{
                    ID = "";
                }
                
                if(IDarray[0]){
                    ID=IDarray[0];

                    this.profileID = ID;
                                        
                }
    
                                                                 
            }else{ //IF ID passed then use passed ID

                this.profileID = this.templateIDNum;
                
            }            
                        
        },
        _getMoreInfo: function() {
           var store = "";

           var xhrGet = dojo.xhrGet({
               url: this.REST_PATH + "server/"+this.UUID+"/status2",                        
               handleAs: "json",
               sync: true,
               preventCache: true,
               load: function(response) {
                   store = response; 
               }
           });

           //Check if items array empty , if not load variables             
           if(store.items){               
               if(store.items == ""){
                   
                   //Disable Edit button if no Template ID exist
                   this.editButton.set("iconClass", "spEditInactiveLgIcon");
                   this.editButton.set("disabled", true);
                   this.moreMenu.removeChild(this.editItem);
                   
               }else{
                   
                   //Check if template ID exist
                   if(store.items[0].cmmTemplateId == null || store.items[0].cmmTemplateId == ""){
                       //Disable Edit button if no Template ID exist                       
                       this.editButton.set("iconClass", "spEditInactiveLgIcon");
                       this.editButton.set("disabled", true);
                       this.moreMenu.removeChild(this.editItem);
                   }else{
                       // Set template ID var
                       this.templateID = store.items[0].cmmTemplateId;
                       
                       // Add Copy Template to More Menu
                       this.moreMenu.addChild(this.copyItem);
                       
                       //Enabled Template Edit Button                
                       this.editButton.set("iconClass", "spEditLgIcon");
                       this.editButton.set("disabled", false);
                       this.moreMenu.addChild(this.editItem);
                   }
                                                               
               }
            }   
        },
        createServerTemplate: function() {

            var stwdialog = new support.dialogs.ChassisTemplateWizard({
                templateIDNum: null,
                editMode: false
            });
            
            stwdialog.show();
   
        },       
        // Deploy Profile
        deployProfile: function() {
                  
            if (this.profileStatus == "ASSIGNED" || this.profileStatus == "assigned" || this.profileStatus == "Assigned") {
                
                var confirmedDeploy = function() {
                    // show the progress dialog
                    this.getProgressDialog("activate").show(); // Show progress dialog for activate request
                    dojo.xhrPost({
                        url: this.REST_PATH + "profile/assign/" + this.profileID + "/CMM/",
                        handleAs: "json",
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        sync: false,
                        load: dojo.hitch(this,function(data) {
                            if(data && data.error && data.error.length > 0){
                                // regardless of the result, hide the
                                //   progress dialog
                                this.getProgressDialog("activate").hide();
                                var validationpopup = new support.temp.ModalDialog({
                                    type: "Error",
                                    text: data.error
                                });
                                validationpopup.show();
                            }
                            else{
                                dojo.publish("AssignChassisProfileComplete");
                                // regardless of the result, hide the
                                //   progress dialog
                                this.getProgressDialog("activate").hide();
                            }
                            this.refreshPage();
                        }),
                        error: dojo.hitch(this,function(data) {
                            // regardless of the result, hide the progress
                            //   dialog
                            this.getProgressDialog("activate").hide();
                            var validationpopup = new support.temp.ModalDialog({
                                type: "Error",
                                text: this.resources.ERROR_ASSIGN_PROF
                            });
                            console.log(data);
                            validationpopup.show();
                        })
                    });
                    
                };
                
                if(this.CP_RUN_ENVIRONMENT == "fsm"){

                    require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                        var confDialog = new support.temp.ConfirmationDialog({
                            buttonLabel: this.resources.DEPLOY,
                            type: "Warning",
                            text: this.locResources.DEPLOY_CONF,
                            info: this.locResources.DEPLOY_CONF_LIST.replace("%1", this.profileName)
                        });

                        confDialog.startup();
                        confDialog.confirm(confirmedDeploy, this);

                    }));
                }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                    require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                        var confDialog = new idx.widget.ConfirmationDialog({
                            buttonLabel: this.resources.DEPLOY,
                            type: "Warning",
                            text: this.locResources.DEPLOY_CONF,
                            info: this.locResources.DEPLOY_CONF_LIST.replace("%1", this.profileName)
                        });

                        confDialog.startup();
                        confDialog.confirm(confirmedDeploy, this);

                    }));

                }
                
            }else{
                
                require([ "support/dialogs/AssignChassisProfile" ], dojo.hitch(this, function() {
                    
                    var assigndialog = new support.dialogs.AssignChassisProfile({
                        profileId: this.profileID,
                        profileName: this.profileName
                    });
                    assigndialog.show();
                    
                }));

                
                
            }
        },
        // Edit Template
        editTemplate: function() {
        
            var stweditdialog = new support.dialogs.ChassisTemplateWizard({
                templateIDNum: this.templateID,
                templateInUse: true,
                editMode: true
            });
            stweditdialog.show();
            
        },
        // Copy Template
        copyTemplate: function() {

            var stweditdialog = new support.dialogs.ChassisTemplateWizard({
                templateIDNum: this.templateID,
                copyMode: true
            });
            stweditdialog.show();

        }, 
        // Unassign Profile Menu Option
        unassignProfile: function() {
            
         var confirmedUnassign = function() {
             dojo.xhrPost({
                 url: this.REST_PATH + "profile/unassign/" + this.profileID,
                 handleAs: "json",
                 headers: {
                     "Content-Type": "application/json; charset=utf-8"
                 },
                 sync: true,
                 load: dojo.hitch(this,function(data) {
                     if(data && data.error && data.error.length > 0){
                         var validationpopup = new support.temp.ModalDialog({
                             type: "Error",
                             text: data.error
                         });
                         validationpopup.show();
                     }
                     else{
                         this.refreshPage();
                         dojo.publish("UnassignChassisProfileComplete");
                     }
                 }),
                 error: dojo.hitch(this,function(data) {
                     var validationpopup = new support.temp.ModalDialog({
                         type: "Error",
                         text: this.resources.ERROR_UNASSIGN_PROF
                     });
                     console.log(data);
                     validationpopup.show();
                 })
             });
             
             
            };

            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.UNASSIGN,
                        type: "Warning",
                        text: this.resources.CONFIRM_UNASSIGN,
                        info: this.resources.UNASSIGN_CHASSIS.replace("%1", this.profileName)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedUnassign, this);
            
                }));
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.UNASSIGN,
                        type: "Warning",
                        text: this.resources.CONFIRM_UNASSIGN,
                        info: this.resources.UNASSIGN_CHASSIS.replace("%1", this.profileName)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedUnassign, this);
            
                }));
                
            }
                                                            
        },
        // Rename Profile Menu Option
        renameProfile: function() {
            require([ "support/dialogs/Rename" ], dojo.hitch(this, function() {
            
                var renamedialog = new support.dialogs.Rename({
                    id: this.id + "cp_chassisprofiledetails_rename",
                    templateIDNum: this.profileID,
                    profileMode: "chassis",
                    launchPoint: "details"
                });
                renamedialog.show();
                
            }));

            
        },
        //Refresh Page
        refreshPage: function(){
            this.refreshButton.set("iconClass", "spRefreshInactiveIcon");
            this.refreshButton.set("disabled", true);

            this.SuccessIcon.style.display = "none";
            this.OfflineIcon.style.display = "none";
            this.ProcessingIcon.style.display = "none";
            this.WarningIcon.style.display = "none";
            this.NormalIcon.style.display = "none";

            this._getProfileInfo();        
            
            dojo.publish("ServerDetailRefreshButtonClicked"); //Refresh Server Details Table

        }
    });

});