define([
        "dojo",
        "dijit",
        "support/ConfigPatternsAll",
        "support/panels/Panel",
        "dojo/i18n!./nls/TemplatesTable",
        "dojo/text!./templates/TemplatesTable.html",
        "dijit/layout/LayoutContainer",
        "dijit/layout/ContentPane",
        "support/base/NextGenUIMixin",
        "support/base/TableMixin",
        "dojo/data/ItemFileWriteStore",
        "dijit/form/DropDownButton",
        "dijit/form/Button",
        "dijit/Menu",
        "dijit/MenuItem",
        "dijit/MenuSeparator",
        "dijit/PopupMenuItem",
        "dojo/string",
        "dijit/Toolbar",
        "dijit/ToolbarSeparator",
        "support/dialogs/ProgressDialog"
    ], function(dojo, dijit, layer, panel, mri, template) {

    return dojo.declare("support.panels.ChassisTemplatesTable", [ support.panels.Panel, support.base.NextGenUIMixin, support.base.TableMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in managing templates for
        // Configuration Patterns.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "TemplatesTable",

        // a reference to the function that is responsible for automatically
        pollingFunc: null,

        // a reference to the current Date instance; used to turn off the
        //   automatic refresh mechanism if it runs too long
        pollingStartDate: null,

        _patternTable: null,
        patternStore: null,
        editButton: null,
        deleteButton: null,
        _selectedTemplate: null,
        actionMenu: null,
        _actionButton: null,
        refreshRoot: null,
        
        _iconTitle: "",
        _helpText: "",
        
        // the properties used to construct the title of this page in the
        //   NextGen UI
        cpProductName: null,
        cpPageContext: null,
        cpPageName: null,
        
        // ID prefix used for table
        gridID: "ChassisPatternsTable",

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/TemplatesTable.css",

        //Views
        BY_ROOT: 1,
        FLAT: 2,
        //Default View Mode
        viewMode : 1,

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
            this.cpProductName = this.resources.PRODUCT_NAME;
            // this.cpPageContext = this.resources.CHASSIS; Must remain null (unspecified), per XB216902
            this.cpPageName = this.resources.CHASSIS_PATTERNS;
            this._iconTitle = this.locResources.CHASSIS_TOOLTIP_TEXT;
            this._helpText = this.locResources.CHASSIS_HELP_TEXT;            
        },

        // postCreate() is called after
        // buildRendering(). This is useful to override
        // when
        // you need to access and/or manipulate DOM
        // nodes included with your widget.
        // DOM nodes and widgets with the
        // dojoAttachPoint attribute specified can now
        // be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in
        // buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);

            // Setup table widget...
            var patternStore = this._retrieveStore();
            var tableStructure = this._getTableStructure();       
            this._patternTable = this.getTable(patternStore, tableStructure, this.resources.NO_DATA, this.gridID, true);
            
            // Set parent rows as "unselectable"
            this._patternTable.disableParentRows();
            this._refreshTree();
            
            // Hide Config Patterns Preferences Link
            this._preferencesLink.style.display = "none";

            setTimeout(dojo.hitch(this, function() {
                this._patternTable.placeAt(this.templateTable);
                this._patternTable.startup();
            }), 500);
            
            // Connect handlers to reset table capabilities
            dojo.connect(this._patternTable.select.row, "onSelectionChange", this, this.setTableCapabilities); // Fires on row selection
            dojo.connect(this._patternTable, "setStore", this, this.setTableCapabilities); // Fires on store reset                      
            
            // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
            this.changeNextGenUITitle();
            
            // reload contents of templates table; by default, switching
            // between links in the details selector does not reload page
            this.subscribeToDetailsSelectorChanges(dojo.hitch(this, function() {
                this._refreshTree();
                this.changeNextGenUITitle();
                this._pollForStatusUpdates();                
            }));
            
            this._loadToolBar(this._patternTable); // load table toolbar

            // Register subscribers to topics we care about ...
            this.subscribe("ChassisTemplateComplete", function(data) {
                this._refreshTree("ct");
            });
            this.subscribe("ChassisPowerComplete", function(data) {
                this._refreshTree("cpat");
            });
            this.subscribe("ChassisManagementComplete", function(data) {
                this._refreshTree("cmit");
            });
            this.subscribe("ChassisUserTemplateComplete", function(data) {
                this._refreshTree("cusr");
            });
            this.subscribe("ChassisInfoTemplateComplete", function(data) {
                this._refreshTree("cit");
            });
            this.subscribe("ChassisIOTemplateComplete", function(data) {
                this._refreshTree("cio");
            });
            this.subscribe("ChassisNodeTemplateComplete", function(data) {
                this._refreshTree("cnod");
            });
            this.subscribe("RefreshTable", function(data) {
                this._refreshTree();
            });
            this.subscribe("ReDeployComplete", function(data) {
                this._refreshTree();
            });
            this.subscribe("DeployChassisTemplateComplete", function(data) {
                this._refreshTree();
            });
            
            this.subscribe("CloseDialog", function() {
                if (this.viewMode == this.FLAT) {
                    this._patternTable.showFilter(); // Show filter controls on flat view.
                }
            });

            this.subscribe("EndPollingOnSelectorChange", function() {
                // Used to end polling when ever the CP selector changes                
                this._stopPolling();
             });

            //Start polling on page load 
            this._pollForStatusUpdates();

        },

        _refreshTree: function(/*String?*/expandId) {
            // tags:
            // protected
            // Refresh tree data
            var patternStore = this._retrieveStore();
            this._patternTable.setStore(patternStore);
            this._patternTable.model.clearCache();
            var callback = function(treeView, table, /*String?*/expandId) {
                if (treeView) {
                    table.reexpandRows();
                    expandId && table.tree.expand(expandId);
                    table.disableParentRows();
                }
                else {
                    table.showFilter(); // Show filter controls on flat view.
                }
            };
            this._patternTable.body.refresh().then(callback((this.viewMode == this.BY_ROOT), this._patternTable, expandId));
        },

        _retrieveStore: function() {
            // tags:
            // protected
            // Connect to data source and create store
            var patternData = "";
            
            dojo.xhrGet({
                url: this.REST_PATH + "template/list/all",
                handleAs: "json",
                preventCache: true,
                sync: true,
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                load: dojo.hitch(this, function(response) {
                    if(response && response.items){
                        if(this.viewMode == this.BY_ROOT){
                            patternData = this._flatToRoot(response.items);
                        }
                        else if(this.viewMode == this.FLAT){
                            patternData = this._rootToFlat(response.items);
                        }
                    }
                })
            });                

            var patternStore = new dojo.data.ItemFileWriteStore({
                 data: dojo.fromJson(patternData)
            });
            
            if (!("hasChildren" in patternStore)) {
                patternStore.hasChildren = function(id, item) { 
                    return item && this.getValues(item, "children").length;
                };
            };
            
            if (!("getChildren" in patternStore)) {
                patternStore.getChildren = function(item) { 
                    return this.getValues(item, "children");
                };
            };

            return patternStore;
        },

        _showConfigPatternsPrefs: function(evt) {
            // tags:
            // protected
            require([ "support/dialogs/ConfigPatterns" ], dojo.hitch(this, function() {
                var dialog = new support.dialogs.ConfigPatterns();
                dialog.show();
            }));

        },

        _loadToolBar: function(grid) {
            // tags:
            // protected
            var toolbar = grid && grid.toolBar && grid.toolBar.widget;
            
            if (toolbar) {
                // Load toolbar buttons/menus
                if (this._patternTable == null) {
                    // Table not created. Nothing to
                    // do but return.
                    return;
                }
    
                if (!this.actionMenu) {
                    this.actionMenu = new dijit.Menu({
                        style: "display: none;"
                    });
                }
                // Add custom display config menu
                var displayMenu = new dijit.Menu({
                    style: "display: none;"
                });
                
                // By Type View
                var displayItem1 = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_displayitem1",
                    label: this.resources.BY_TYPE,
                    title: this.locResources.ViewGroup,
                    viewMode: this.BY_ROOT,
                    onClick: dojo.hitch(this, function() {
                        this._setViewByMode(this.BY_ROOT);
                    })
                });
                displayMenu.addChild(displayItem1);
                            
                // Flat List View
                var displayItem2 = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_displayitem2",
                    label: this.resources.FLAT,
                    title: this.locResources.ViewList,
                    viewMode: this.FLAT,
                    onClick: dojo.hitch(this, function() {
                        this._setViewByMode(this.FLAT);
                    })
                });
                displayMenu.addChild(displayItem2);
    
                this._displayButton = new dijit.form.DropDownButton({
                    //id: this.id + "cp_chassistemplatestable_displaybutton",
                    showLabel: true,
                    label: this.resources.BY_TYPE,
                    title: this.resources.TABLE_VIEW,
                    dropDown: displayMenu
                });
                toolbar.addChild(this._displayButton);
                
                toolbar.addChild(new dijit.ToolbarSeparator());
                
                // Add create menu...
                var createMenu = new dijit.Menu({
                    style: "display: none;"
                });
    
                var newServerTemplate = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_newserver",
                    label: this.locResources.NEW_CHASSIS_PATTERN,
                    title: this.locResources.CREATE_CHASSIS_PATTERN,
                    onClick: dojo.hitch(this, this.createChassisTemplate),
                    onKeypress: dojo.hitch(this, this.createChassisTemplate)
                });
                createMenu.addChild(newServerTemplate);
    
                var newSystemInformationTemplate = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_newsysteminfo",
                    label: this.locResources.NEW_CHASSIS_PATTERN_INFO,
                    title: this.locResources.CREATE_CHASSIS_PATTERN_INFO,
                    onClick: dojo.hitch(this, this.createChassisInfoTemplate),
                    onKeypress: dojo.hitch(this, this.createChassisInfoTemplate)
                });
                createMenu.addChild(newSystemInformationTemplate);
    
                var newManagementInterfaceTemplate = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_newmanagementinterface",
                    label: this.locResources.NEW_CHASSIS_PATTERN_MGMT,
                    title: this.locResources.CREATE_CHASSIS_PATTERN_MGMT,
                    onClick: dojo.hitch(this, this.createChassisManagementInterfaceTemplate),
                    onKeypress: dojo.hitch(this, this.createChassisManagementInterfaceTemplate)
                });
                createMenu.addChild(newManagementInterfaceTemplate);
    
                var newPowerStateAndCappingTemplate = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_newpowerstateandcapping",
                    label: this.locResources.NEW_CHASSIS_PATTERN_PWR,
                    title: this.locResources.CREATE_CHASSIS_PATTERN_PWR,
                    onClick: dojo.hitch(this, this.createChassisPowerTemplate),
                    onKeypress: dojo.hitch(this, this.createChassisPowerTemplate)
                });
                createMenu.addChild(newPowerStateAndCappingTemplate);
    
                var newPerformanceAndRecoveryTemplate = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_newperformanceandrecovery",
                    label: this.locResources.NEW_CHASSIS_PATTERN_USER,
                    title: this.locResources.CREATE_CHASSIS_PATTERN_USER,
                    onClick: dojo.hitch(this, this.createChassisUsersTemplate),
                    onKeypress: dojo.hitch(this, this.createChassisUsersTemplate)
                });
                createMenu.addChild(newPerformanceAndRecoveryTemplate);
                
                var newIOModuleTemplate = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_newiomodule",
                    label: this.locResources.NEW_CHASSIS_PATTERN_IO,
                    title: this.locResources.CREATE_CHASSIS_PATTERN_IO,
                    onClick: dojo.hitch(this, this.createIOTemplate),
                    onKeypress: dojo.hitch(this, this.createIOTemplate)
                });
                createMenu.addChild(newIOModuleTemplate);
                
                var newNodeTemplate = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_newnode",
                    label: this.locResources.NEW_CHASSIS_PATTERN_NODE,
                    title: this.locResources.CREATE_CHASSIS_PATTERN_NODE,
                    onClick: dojo.hitch(this, this.createNodeTemplate),
                    onKeypress: dojo.hitch(this, this.createNodeTemplate)
                });
                createMenu.addChild(newNodeTemplate);
    
                var createButton = new dijit.form.DropDownButton({
                    //id: this.id + "cp_chassistemplatestable_createbutton",
                    showLabel: false,
                    iconClass: "spCreateLgIcon",
                    title: this.resources.CREATE_PATTERN_TITLE,
                    dropDown: createMenu
                });
    
                toolbar.addChild(createButton);
                
                // Add create menu...
                var createMenu2 = new dijit.Menu({
                    style: "display: none;"
                });
    
                var newServerTemplate2 = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_newserver2",
                    label: this.locResources.NEW_CHASSIS_PATTERN,
                    title: this.locResources.CREATE_CHASSIS_PATTERN,
                    onClick: dojo.hitch(this, this.createChassisTemplate),
                    onKeypress: dojo.hitch(this, this.createChassisTemplate)
                });
                createMenu2.addChild(newServerTemplate2);
    
                var newSystemInformationTemplate2 = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_newsysteminfo2",
                    label: this.locResources.NEW_CHASSIS_PATTERN_INFO,
                    title: this.locResources.CREATE_CHASSIS_PATTERN_INFO,
                    onClick: dojo.hitch(this, this.createChassisInfoTemplate),
                    onKeypress: dojo.hitch(this, this.createChassisInfoTemplate)
                });
                createMenu2.addChild(newSystemInformationTemplate2);
    
                var newManagementInterfaceTemplate2 = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_newmanagementinterface2",
                    label: this.locResources.NEW_CHASSIS_PATTERN_MGMT,
                    title: this.locResources.CREATE_CHASSIS_PATTERN_MGMT,
                    onClick: dojo.hitch(this, this.createChassisManagementInterfaceTemplate),
                    onKeypress: dojo.hitch(this, this.createChassisManagementInterfaceTemplate)
                });
                createMenu2.addChild(newManagementInterfaceTemplate2);
    
                var newPowerStateAndCappingTemplate2 = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_newpowerstateandcapping2",
                    label: this.locResources.NEW_CHASSIS_PATTERN_PWR,
                    title: this.locResources.CREATE_CHASSIS_PATTERN_PWR,
                    onClick: dojo.hitch(this, this.createChassisPowerTemplate),
                    onKeypress: dojo.hitch(this, this.createChassisPowerTemplate)
                });
                createMenu2.addChild(newPowerStateAndCappingTemplate2);
    
                var newPerformanceAndRecoveryTemplate2 = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_newperformanceandrecovery2",
                    label: this.locResources.NEW_CHASSIS_PATTERN_USER,
                    title: this.locResources.CREATE_CHASSIS_PATTERN_USER,
                    onClick: dojo.hitch(this, this.createChassisUsersTemplate),
                    onKeypress: dojo.hitch(this, this.createChassisUsersTemplate)
                });
                createMenu2.addChild(newPerformanceAndRecoveryTemplate2);
                
                var newIOModuleTemplate2 = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_newiomodule2",
                    label: this.locResources.NEW_CHASSIS_PATTERN_IO,
                    title: this.locResources.CREATE_CHASSIS_PATTERN_IO,
                    onClick: dojo.hitch(this, this.createIOTemplate),
                    onKeypress: dojo.hitch(this, this.createIOTemplate)
                });
                createMenu2.addChild(newIOModuleTemplate2);
                
                var newNodeTemplate2 = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_newnode2",
                    label: this.locResources.NEW_CHASSIS_PATTERN_NODE,
                    title: this.locResources.CREATE_CHASSIS_PATTERN_NODE,
                    onClick: dojo.hitch(this, this.createNodeTemplate),
                    onKeypress: dojo.hitch(this, this.createNodeTemplate)
                });
                createMenu2.addChild(newNodeTemplate2);
    
                // Add custom Edit button...
                this.editButton = new dijit.form.Button({
                    //id: this.id + "cp_chassistemplatestable_editbutton",
                    showLabel: false,
                    iconClass: "spEditInactiveLgIcon",
                    disabled: true,
                    title: this.resources.EDIT_SELECTED_PATTERN,
                    onClick: dojo.hitch(this, this.editTemplate),
                    onKeypress: dojo.hitch(this, this.editTemplate)
                });
                toolbar.addChild(this.editButton);
    
    
                // Add custom Delete button...
                this.deleteButton = new dijit.form.Button({
                    //id: this.id + "cp_chassistemplatestable_deletebutton",
                    showLabel: false,
                    iconClass: "spDeleteInactiveLgIcon",
                    disabled: true,
                    title: this.resources.DELETE_SELECTED_PATTERNS,
                    onClick: dojo.hitch(this, this.deleteTemplates),
                    onKeypress: dojo.hitch(this, this.deleteTemplates)
                });
                toolbar.addChild(this.deleteButton);
                
                toolbar.addChild(new dijit.ToolbarSeparator());
    
                // Add to Actions Menu...
                this.expandAllItem = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_expandallitem",
                    label: this.resources.EXPAND_ALL,
                    iconClass: "spExpandAllLgIcon",
                    title: this.resources.EXPAND_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._patternTable.expandAll();
                    }),
                    onKeypress: dojo.hitch(this, function() {
                        this._patternTable.expandAll();
                    })
                });
                this.actionMenu.addChild(this.expandAllItem,0);
                
                this.collapseAllItem = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_collapseallitem",
                    label: this.resources.COLLAPSE_ALL,
                    iconClass: "spCollapseAllLgIcon",
                    title: this.resources.COLLAPSE_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._patternTable.collapseAll();
                    }),
                    onKeypress: dojo.hitch(this, function() {
                        this._patternTable.collapseAll();
                    })
                });
                this.actionMenu.addChild(this.collapseAllItem,1);
                
                this.createItem = new dijit.PopupMenuItem({
                    label: this.resources.CREATE,
                    iconClass: "spCreateLgIcon",
                    title: this.resources.CREATE_PATTERN_TITLE,
                    popup: createMenu2
                });
                this.actionMenu.addChild(this.createItem);
                
                var menuSeparator = new dijit.MenuSeparator();
                this.actionMenu.addChild(menuSeparator);
    
                this.detailsItem = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_detailsitem",
                    label: this.resources.DETAILS,
                    title: this.resources.DETAILS_SELECTED_PATTERN,
                    onClick: dojo.hitch(this, this.detailTemplate),
                    onKeypress: dojo.hitch(this, this.detailTemplate)
                });                
    
                this.renameItem = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_renameitem",
                    label: this.resources.RENAME,
                    title: this.resources.RENAME_SELECTED_PATTERN,
                    onClick: dojo.hitch(this, this.renameTemplate),
                    onKeypress: dojo.hitch(this, this.renameTemplate)
                });
    
                this.copyItem = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_copyitem",
                    label: this.resources.COPY,
                    title: this.resources.COPY_SELECTED_PATTERN,
                    onClick: dojo.hitch(this, this.copyTemplate),
                    onKeypress: dojo.hitch(this, this.copyTemplate)
                });
    
                this.deployItem = new dijit.MenuItem({
                    //id: this.id + "cp_chassistemplatestable_deployitem",
                    label: this.resources.DEPLOY,
                    title: this.resources.DEPLOY_SELECTED_CHASSIS_PATTERN,
                    iconClass: "spDeployLgIcon",
                    onClick: dojo.hitch(this, this.deployTemplate),
                    onKeypress: dojo.hitch(this, this.deployTemplate)
                });
                    
                //Add action menu to action button and create action button if not already created
                if (this._actionButton) {
                    this._actionButton.dropDown = this.actionMenu;
                }
                else {
                    this._actionButton = new dijit.form.DropDownButton({
                        //id: this.id + "cp_chassistemplatestable_actionbutton",
                        label: this.resources.ALL_ACTIONS,
                        title: this.resources.ALL_ACTIONS_TITLE,
                        showLabel: true,
                        dropDown: this.actionMenu
                    });   
                    
                    toolbar.addChild(this._actionButton);
                }
                
            }
        },

        // Functions to open dialogs
        createChassisPowerTemplate: function() {

            require([ "support/dialogs/ChassisPowerAndAcousticsTemplate" ], dojo.hitch(this, function() {
                var pscdialog = new support.dialogs.ChassisPowerAndAcousticsTemplate({
                    id: this.id + "cp_chassistemplatestable_chassispowerandacoustic"
                });
                pscdialog.show();

            }));

            return false;
        },

        createChassisInfoTemplate: function() {

            require([ "support/dialogs/ChassisInfoTemplate" ], dojo.hitch(this, function() {
                var sitdialog = new support.dialogs.ChassisInfoTemplate({
                    id: this.id + "cp_chassistemplatestable_chassisinfo"
                });
                sitdialog.show();

            }));

            return false;
        },

        createChassisUsersTemplate: function() {

            require([ "support/dialogs/ChassisUserAndSecurityTemplate" ], dojo.hitch(this, function() {
                var partdialog = new support.dialogs.ChassisUserAndSecurityTemplate({
                    id: this.id + "cp_chassistemplatestable_chassisuserandsecurity"
                });
                partdialog.show();
            }));

            return false;
        },

        createChassisManagementInterfaceTemplate: function() {

            require([ "support/dialogs/ChassisManagementInterfaceTemplate" ], dojo.hitch(this, function() {
                var mitdialog = new support.dialogs.ChassisManagementInterfaceTemplate({
                    id: this.id + "cp_chassistemplatestable_chassismanagementinterface"
                });
                mitdialog.show();
            }));

            return false;
        },

        createChassisTemplate: function() {

            require([ "support/dialogs/ChassisTemplateWizard" ], dojo.hitch(this, function() {
                var natdialog = new support.dialogs.ChassisTemplateWizard();
                natdialog.show();
            }));
            
            return false;
        },

        createIOTemplate: function() {

            require([ "support/dialogs/ChassisIOModuleTemplate" ], dojo.hitch(this, function() {
                var natdialog = new support.dialogs.ChassisIOModuleTemplate({
                    id: this.id + "cp_chassistemplatestable_chassisiomodule"
                });
                natdialog.show();
            }));
                     
            return false;
        },

        createNodeTemplate: function() {

            require([ "support/dialogs/ChassisNodeTemplate" ], dojo.hitch(this, function() {
                var natdialog = new support.dialogs.ChassisNodeTemplate({
                    id: this.id + "cp_chassistemplatestable_chassisnode"
                });
                natdialog.show();
            }));
            
            return false;
        },
        
        // Deploy Template
        deployTemplate: function() {

            if (this._selectedTemplate.id.toString() != null && this._selectedTemplate.id.toString() != "") {
                require([ "support/dialogs/DeployChassisTemplate" ], dojo.hitch(this, function() {
                    var dst = new support.dialogs.DeployChassisTemplate({
                        templateId: this._selectedTemplate.id.toString()
                    });
                    dst.show();
                }));
            }

        },

        // Details Template
        detailTemplate: function() {

            if (this._selectedTemplate.templateType.toString() == "CMM") { 
                
                require([ "support/panels/ChassisTemplateDetails" ], dojo.hitch(this,function() {
                
                    if(this.CP_RUN_ENVIRONMENT == "fsm"){

                        this.loadNGUIWorkAreaWithCPWidget("support/panels/ChassisTemplateDetails",null,this._selectedTemplate.id.toString());
                    
                    }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                        var launchParams = {templateIDNum: this._selectedTemplate.id.toString()};
                        
                        this.loadAugustaWorkAreaTask("config.detail.page","support/panels/ChassisTemplateDetails",launchParams);
                        
                    }
                
                }));

            }
            else{
                
                require([ "support/panels/ChassisCategoryTemplateDetails" ], dojo.hitch(this,function() {
                    
                    if(this.CP_RUN_ENVIRONMENT == "fsm"){

                        this.loadNGUIWorkAreaWithCPWidget("support/panels/ChassisCategoryTemplateDetails",null,this._selectedTemplate.id.toString());
                    
                    }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                        var launchParams = {templateIDNum: this._selectedTemplate.id.toString()};
                        
                        this.loadAugustaWorkAreaTask("config.detail.page","support/panels/ChassisCategoryTemplateDetails",launchParams);
                        
                    }
                
                }));

            }            
            
        },

        editTemplate: function() {

           if (this._selectedTemplate.templateType.toString() == "CMM") {             
               require([ "support/dialogs/ChassisTemplateWizard" ], dojo.hitch(this, function() {
                   var stweditdialog = new support.dialogs.ChassisTemplateWizard({
                       templateIDNum: this._selectedTemplate.id.toString(),
                       templateInUse: (this._selectedTemplate.inUse.toString() == "true") ? true : false,
                       editMode: true
                   });
                   stweditdialog.show();
               }));
            }
            else if (this._selectedTemplate.templateType.toString() == "cmmPower") {
                require([ "support/dialogs/ChassisPowerAndAcousticsTemplate" ], dojo.hitch(this, function() {
                    var psceditdialog = new support.dialogs.ChassisPowerAndAcousticsTemplate({
                        id: this.id + "cp_chassistemplatestable_chassispowerandacoustics",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit",
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint : "table"
                    });
                    psceditdialog.show();
                }));            
            }
            else if (this._selectedTemplate.templateType.toString() == "cmmInfo") {
                require([ "support/dialogs/ChassisInfoTemplate" ], dojo.hitch(this, function() {
                    var sitdialog = new support.dialogs.ChassisInfoTemplate({
                        id: this.id + "cp_chassistemplatestable_chassisinfo",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit",
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint : "table"
                    });
                    sitdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "cmmUsers") {
                require([ "support/dialogs/ChassisUserAndSecurityTemplate" ], dojo.hitch(this, function() {
                    var partdialog = new support.dialogs.ChassisUserAndSecurityTemplate({
                        id: this.id + "cp_chassistemplatestable_chassisuserandsecurity",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit",
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint : "table"
                    });
                    partdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "cmmManagement") {
                require([ "support/dialogs/ChassisManagementInterfaceTemplate" ], dojo.hitch(this, function() {
                    var miteditdialog = new support.dialogs.ChassisManagementInterfaceTemplate({
                        id: this.id + "cp_chassistemplatestable_chassismanagementinterface",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit",
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint : "table"
                    });
                    miteditdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "cmmIOModule") {
                require([ "support/dialogs/ChassisIOModuleTemplate" ], dojo.hitch(this, function() {
                    var miteditdialog = new support.dialogs.ChassisIOModuleTemplate({
                        id: this.id + "cp_chassistemplatestable_chassisiomodule",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit",
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint : "table"
                    });
                    miteditdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "cmmNode") {
                require([ "support/dialogs/ChassisNodeTemplate" ], dojo.hitch(this, function() {
                    var miteditdialog = new support.dialogs.ChassisNodeTemplate({
                        id: this.id + "cp_chassistemplatestable_chassisnode",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit",
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint : "table"
                    });
                    miteditdialog.show();
                }));
            }

            return false;
        },

        // Copy Template
        copyTemplate: function() {

            if (this._selectedTemplate.templateType.toString() == "CMM") {
                require([ "support/dialogs/ChassisTemplateWizard" ], dojo.hitch(this, function() {
                    var stweditdialog = new support.dialogs.ChassisTemplateWizard({
                        templateIDNum: this._selectedTemplate.id.toString(),
                        copyMode: true
                    });
                    stweditdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "cmmPower") {
                require([ "support/dialogs/ChassisPowerAndAcousticsTemplate" ], dojo.hitch(this, function() {
                    var psceditdialog = new support.dialogs.ChassisPowerAndAcousticsTemplate({
                        id: this.id + "cp_chassistemplatestable_chassispowerandacoustics",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    psceditdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "cmmInfo") {
                require([ "support/dialogs/ChassisInfoTemplate" ], dojo.hitch(this, function() {
                    var sitdialog = new support.dialogs.ChassisInfoTemplate({
                        id: this.id + "cp_chassistemplatestable_chassisinfo",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    sitdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "cmmUsers") {
                require([ "support/dialogs/ChassisUserAndSecurityTemplate" ], dojo.hitch(this, function() {
                    var partdialog = new support.dialogs.ChassisUserAndSecurityTemplate({
                        id: this.id + "cp_chassistemplatestable_chassisuserandsecurity",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    partdialog.show();                
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "cmmManagement") {
                require([ "support/dialogs/ChassisManagementInterfaceTemplate" ], dojo.hitch(this, function() {
                    var miteditdialog = new support.dialogs.ChassisManagementInterfaceTemplate({
                        id: this.id + "cp_chassistemplatestable_chassismanagementinterface",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    miteditdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "cmmIOModule") {
                require([ "support/dialogs/ChassisIOModuleTemplate" ], dojo.hitch(this, function() {
                    var miteditdialog = new support.dialogs.ChassisIOModuleTemplate({
                        id: this.id + "cp_chassistemplatestable_chassisiomodule",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    miteditdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "cmmNode") {
                require([ "support/dialogs/ChassisNodeTemplate" ], dojo.hitch(this, function() {
                    var miteditdialog = new support.dialogs.ChassisNodeTemplate({
                        id: this.id + "cp_chassistemplatestable_chassisnode",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    miteditdialog.show();
                }));
            }
           
            return false;
        },
        
        // Rename Template
        renameTemplate: function() {

            require([ "support/dialogs/Rename" ], dojo.hitch(this, function() {
                var renamedialog = new support.dialogs.Rename({
                    id: this.id + "cp_chassistemplatestable_rename",
                    templateIDNum: this._selectedTemplate.id.toString()
                });
                renamedialog.show();

            }));
            
        },
        
        // Delete Templates
        deleteTemplates: function() {
            
            var deleteIds = new Array();
            var deleteTypes = new Array();
            var htmlNames = new Array();
            
            var ids = this._patternTable.select.row.getSelected();
            
            for (var loopVar = 0; loopVar < ids.length; loopVar++) {
                var item = this._patternTable.row(ids[loopVar], true).item();
                deleteIds.push(item.id);
                (dojo.indexOf(deleteTypes, item.type) < 0) && deleteTypes.push(item.type);
                var htmlName = this.encodeForHtml(item.name.toString());
                htmlNames.push(htmlName);
            }
            
            var processDeleteRequest = function() {
                var responses = new Array();
                this.getProgressDialog("delete", false).show(); // show progress dialog for delete
                
                dojo.forEach(deleteIds, dojo.hitch(this, function(id) {
                    dojo.xhrDelete({
                        url: this.REST_PATH + "template/" + id,
                        handleAs: "json",
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        error: dojo.hitch(this, function(response, ioArgs) {
                            var msg = "";
                            if (ioArgs.xhr.status == 400) {
                                msg = this.resources.ERROR_PATTERN_DELETE_INUSE;
                            }
                            else {
                                msg = this.resources.ERROR_PATTERN_DELETE;
                            }
                            this.getModalDialog("Error", msg).show();
                        }),
                        handle: dojo.hitch(this, function(response, ioArgs) {
                            if (responses.push(response) < deleteIds.length) {
                                // Not the final request. Keep the user waiting.
                                return;
                            }
                            dojo.publish("ProgressComplete", response);
                            
                            this._refreshTree();                            
                        })
                    });
                }));
            };
            

            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.DELETE,
                        type: "Warning",
                        text: this.resources.CONFIRM_DELETE_TEMP,
                        info: this.locResources.DELETEINFO                    
                    });

                    confirmDialog.set("content", "<p>" + this.locResources.DeleteUnused.replace("%1", htmlNames.join(", ")) + "</p>");
                    confirmDialog.startup();
                    confirmDialog.confirm(processDeleteRequest, this);
                
                }));
 
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.DELETE,
                        type: "Warning",
                        text: this.resources.CONFIRM_DELETE_TEMP,
                        info: this.locResources.DELETEINFO                    
                    });

                    confirmDialog.set("content", "<p>" + this.locResources.DeleteUnused.replace("%1", htmlNames.join(", ")) + "</p>");
                    confirmDialog.startup();
                    confirmDialog.confirm(processDeleteRequest, this);
                
                }));
                
            }

            
            return false;
        },

        // Set Table Formatters
        // Hard-coded path appended to URL, must be changed if path changes
        _decorateName: function(item, id) {
            // tags:
            // protected
            if (!item || !id) {
                return "";
            }
            
            var value = item;
            var rowItem = this._patternTable.row(id, true).item();
            
            if(id == "cusr" || id == "cpat" || id == "cmit" || id == "ct" 
                || id == "cit" || id == "cio" || id == "cnod"){
                return "<span title=\"" + value + "\" style=\"white-space:normal;\">" + value + "</span>";
            } 
            if(this.CP_RUN_ENVIRONMENT == "fsm"){
                if (rowItem.type=="CMM"){
                    return "<a href=\"#workAreaDijit=support%2Fpanels%2FChassisTemplateDetails&path=support:%2Fcom.ibm.stg.sysx.profile." +
                    		"navmodel%2Fjs%2Fsupport&cpTarget=" + id + "\" title=\"" + this.resources.LAUNCH_DETAILS + "\">" + value + "</a>";
                } else {
                    return "<a href=\"#workAreaDijit=support%2Fpanels%2FChassisCategoryTemplateDetails&path=support:%2Fcom.ibm.stg.sysx." +
                    		"profile.navmodel%2Fjs%2Fsupport&cpTarget=" + id + "\" title=\"" + this.resources.LAUNCH_DETAILS + "\">" + value + "</a>";
                }
            } else if(this.CP_RUN_ENVIRONMENT == "augusta"){
                if (rowItem.type=="CMM"){
                    return "<a href=\"#fwWidget=support%2Fpanels%2FChassisTemplateDetails&fwTaskId=config.detail.page&templateIDNum=" 
                        + id + "\" title=\"" + this.resources.LAUNCH_DETAILS + "\">" + value + "</a>";
                } else {
                    return "<a href=\"#fwWidget=support%2Fpanels%2FChassisCategoryTemplateDetails&fwTaskId=config.detail.page&templateIDNum=" 
                        + id + "\" title=\"" + this.resources.LAUNCH_DETAILS + "\">" + value + "</a>";
                }
            }
            
        },
        
        _formatName: function(item) {
            
            if (!item || !item.name) {
                return "";
            }
            
            var value = this.decodeSpecialCharacters(item.name);
            value = this.encodeForHtml(value);

            return value;
        },
        
        _decorateInUse: function(item) {
            // tags:
            // protected      
            if (item == "") {
                return "";
            }

            if (item == this.resources.DEPLOYED) {
                return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr title=\"" + this.locResources.TemplateInUseText + "\">" +
                    "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spDeployedIcon\"></img></td>" +
                    "<td><span style=\"margin-left:4px; white-space: nowrap;\">" + 
                    this.resources.DEPLOYED + "</span></td></tr></tbody></table>";
            }else if(item == this.resources.REFERENCED){
                return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr title=\"" + this.locResources.TemplateReferenced + "\">" +
                    "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spReferencedIcon\"></img></td>" +
                    "<td><span style=\"margin-left:4px; white-space: nowrap;\">" + 
                    this.resources.REFERENCED + "</span></td></tr></tbody></table>";
            }
            return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr title=\"" + this.locResources.TemplateNotInUseText + "\">" +
                "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spNotInUseIcon\"></img></td>" +
                "<td><span style=\"margin-left:4px; white-space: nowrap;\">" + 
                this.resources.NOT_IN_USE + "</span></td></tr></tbody></table>";
        },
        
        _formatInUse: function(item) {
            var inUseValue = (item && item.inUse) ? item.inUse : false;
            var refValue = (item && item.referencedBy && item.referencedBy > 0) ? true : false;
            
            if (item.type == "root" && this.viewMode == this.BY_ROOT) {
                return "";
            }
            
            if (inUseValue) {
                return this.resources.DEPLOYED;
            } else if (refValue) {
                return this.resources.REFERENCED;
            }
            return this.resources.NOT_IN_USE;
        },

        _decorateUserDefined: function(item) {
            // tags:
            // protected
            if (item == "") {
                return "";
            }
            
            if (item == this.resources.USER_DEFINED) {
                return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr title=\"" + this.resources.USER_DEFINED_TEMPLATE + "\">" +
                    "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spUserDefinedIcon\"></img></td>" +
                    "<td><span style=\"margin-left:4px; white-space: nowrap;\">" + 
                    this.resources.USER_DEFINED + "</span></td></tr></tbody></table>";
            }
            return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr title=\"" + this.resources.PREDEFINED_TEMPLATE+ "\">" +
                "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spPredefinedIcon\"></img></td>" +
                "<td><span style=\"margin-left:4px; white-space: nowrap;\">" + 
                this.resources.PREDEFINED + "</span></td></tr></tbody></table>";
        },
        
        _formatUserDefined: function(item) {
            var value = (item && item.userDefined) ? item.userDefined : false;
            
            if (item.type == "root" && this.viewMode == this.BY_ROOT) {
                return "";
            }
            
            if (value) {
                return this.resources.USER_DEFINED;
            }
            return this.resources.PREDEFINED;
        },

        _decorateDescription: function(item) {
            return "<span title=\"" + item + "\" style=\"white-space: normal;\">" + item + "</span>";
        },

        _formatDescription: function(item) {
            // tags:
            // protected
            if (!item || !item.description) {
                return "";
            }
            var value = this.decodeSpecialCharacters(item.description);
            value = this.encodeForHtml(value);
            return value;
        },
        
        _decorateTemplateType: function(item) {
            return "<span title=\"" + item + "\" style=\"white-space: nowrap;\">" + item + "</span>";
        },

        _formatTemplateType: function(item) {
            // tags:
            // protected
            if (!item || !item.templateType) {
                return "";
            }
            var value = item.templateType;
            switch (value) {
                case "CMM":
                    value = this.resources.CHASSIS;
                    break;
                case "cmmInfo":
                    value = this.resources.CHASSIS_INFO;
                    break;
                case "cmmManagement":
                    value = this.resources.CHASSIS_MGMT;
                    break;
                case "cmmPower":
                    value = this.resources.CHASSIS_POWER;
                    break;
                case "cmmUsers":
                    value = this.resources.CHASSIS_USER;
                    break;
                case "cmmIOModule":
                    value = this.resources.CHASSIS_IO;
                    break;
                case "cmmNode":
                    value = this.resources.CHASSIS_NODE;
                    break;
                default:
                    value = "";
                    break;
            }
            return value;
        },

        // Set Table Capabilities
        setTableCapabilities: function() {

            if (!this._patternTable) {
                return; // nothing to do. return now
            }           

            this.editButton.set("iconClass", "spEditInactiveLgIcon");
            this.editButton.set("disabled", true);
            this.deleteButton.set("iconClass", "spDeleteInactiveLgIcon");
            this.deleteButton.set("disabled", true);

            var ids = this._patternTable.select.row.getSelected();
            
            var menuPosition = 2;
            
            var allowEdit = (ids.length == 1); // assume okay, disprove below
            var allowDelete = (ids.length > 0); // assume okay, disprove below
            var allowDeploy = (ids.length == 1); // assume okay, disprove below
            var allowRename = (ids.length == 1); // assume okay, disprove below
            for (var loopVar = 0; loopVar < ids.length; ++loopVar) {

                var item = this._patternTable.row(ids[loopVar], true).item();
                this._selectedTemplate = item;
                
                if ((item.type == "root") && (this.viewMode != this.FLAT)) {
                    // Root selection not allowed.
                    allowEdit = allowDelete = allowDeploy = allowRename = false;
                }
                if (item.userDefined != "true") {
                    allowDelete = allowRename = false;
                }
                if (item.inUse == "true" || item.referencedBy == null 
                        || item.referencedBy.length > 0) {
                    allowDelete = false;
                }
            };

            if (allowEdit) {
                this.editButton.set("iconClass", "spEditLgIcon");
                this.editButton.set("disabled", false);

                if (!this.detailsItem) {
                    this.detailsItem = new dijit.MenuItem({
                 //       id: this.id + "cp_chassistemplatestable_detailsitem",
                        label: this.resources.DETAILS,
                        title: this.resources.DETAILS_SELECTED_PATTERN,
                        onClick: dojo.hitch(this, this.detailTemplate),
                        onKeypress: dojo.hitch(this, this.detailTemplate)
                    });
                }

                if (!this.copyItem) {
                    this.copyItem = new dijit.MenuItem({
                     //   id: this.id + "cp_chassistemplatestable_copyitem",
                        label: this.resources.COPY,
                        title: this.resources.COPY_SELECTED_PATTERN,
                        onClick: dojo.hitch(this, this.copyTemplate),
                        onKeypress: dojo.hitch(this, this.copyTemplate)
                    });
                }

                if (allowRename && !this.renameItem) {
                    this.renameItem = new dijit.MenuItem({
                      //  id: this.id + "cp_chassistemplatestable_renameitem",
                        label: this.resources.RENAME,
                        title: this.resources.RENAME_SELECTED_PATTERN,
                        onClick: dojo.hitch(this, this.renameTemplate),
                        onKeypress: dojo.hitch(this, this.renameTemplate)
                    });
                }

                if (!this.editItem) {
                    this.editItem = new dijit.MenuItem({
                      //  id: this.id + "cp_chassistemplatestable_edititem",
                        label: this.resources.EDIT,
                        title: this.resources.EDIT_SELECTED_PATTERN,
                        iconClass: "spEditLgIcon",
                        onClick: dojo.hitch(this, this.editTemplate),
                        onKeypress: dojo.hitch(this, this.editTemplate)
                    });                    
                }

                if (allowDelete && !this.deleteItem) {
                    this.deleteItem = new dijit.MenuItem({
                      //  id: this.id + "cp_chassistemplatestable_deleteitem",
                        label: this.resources.DELETE,
                        title: this.resources.DELETE_SELECTED_PATTERNS,
                        iconClass: "spDeleteLgIcon",
                        onClick: dojo.hitch(this, this.deleteTemplates),
                        onKeypress: dojo.hitch(this, this.deleteTemplates)
                    });                    
                }

                this.actionMenu.addChild(this.editItem, ((this.viewMode == this.FLAT)? menuPosition : menuPosition+2));
                this.actionMenu.addChild(this.copyItem);
                this.actionMenu.addChild(this.detailsItem);
                menuPosition++;
                if (allowDelete) {
                    this.actionMenu.addChild(this.deleteItem, ((this.viewMode == this.FLAT)? menuPosition : menuPosition+2));
                    menuPosition++;
                }
                else if (this.deleteItem) {
                    this.actionMenu.removeChild(this.deleteItem);
                    this.deleteItem = null;
                }                
                if (allowRename) {
                    this.actionMenu.addChild(this.renameItem);
                }
                else if (this.renameItem) {
                    this.actionMenu.removeChild(this.renameItem);
                    this.renameItem = null;
                }                

            }
            else {

                this.actionMenu.removeChild(this.detailsItem);
                this.detailsItem = null;

                this.actionMenu.removeChild(this.copyItem);
                this.copyItem = null;

                this.actionMenu.removeChild(this.renameItem);
                this.renameItem = null;
                
                this.actionMenu.removeChild(this.editItem);
                this.editItem = null;
                
                if (!allowDelete && this.deleteItem) {
                    this.actionMenu.removeChild(this.deleteItem);
                    this.deleteItem = null;
                }

            }

            if (allowDelete) {
                this.deleteButton.set("iconClass", "spDeleteLgIcon");
                this.deleteButton.set("disabled", false);
            }

            if (allowDeploy && (this._selectedTemplate.templateType.toString() === "CMM")) {

                if (!this.deployItem) {

                    this.deployItem = new dijit.MenuItem({
                      //  id: this.id + "cp_chassistemplatestable_deployitem",
                        label: this.resources.DEPLOY,
                        title: this.resources.DEPLOY_SELECTED_CHASSIS_PATTERN,
                        iconClass: "spDeployLgIcon",
                        onClick: dojo.hitch(this, this.deployTemplate),
                        onKeypress: dojo.hitch(this, this.deployTemplate)
                    });

                }
                
                this.actionMenu.addChild(this.deployItem, ((this.viewMode == this.FLAT)? menuPosition : menuPosition+2));
                menuPosition++;

            }
            else {
                this.actionMenu.removeChild(this.deployItem);
                this.deployItem = null;
            }

            //Disable Action Button if no children
            if(this.actionMenu.hasChildren()){
                this._actionButton.set("disabled", false);
            }else{
                this._actionButton.set("disabled", true);                
            }
        },
        
        _setViewByMode : function(menuItem) {
            // tags:
            // protected
            if (this.viewMode == menuItem) {
                return;
            }
            
            // Clear selected rows when switching views
            this._patternTable.select.row.clear();
            
            // Set Menu Button Label To Current View
            var label = "";
            if (menuItem == this.BY_ROOT) {
                label = this.resources.BY_TYPE;
                this.actionMenu.addChild(this.expandAllItem, 0);
                this.actionMenu.addChild(this.collapseAllItem, 1);
                this._patternTable.hideFilter(); // Hide filter controls on tree view.
            }
            else if (menuItem == this.FLAT) {
                label = this.resources.FLAT;                
                this.actionMenu.removeChild(this.expandAllItem);
                this.actionMenu.removeChild(this.collapseAllItem);
                this._patternTable.showFilter(); // Show filter controls on flat view.
            }
            else { //Invalid view mode
                return;
            }
            
            //Disable Action Button if no children
            if (this.actionMenu.hasChildren()) {
                this._actionButton.set("disabled", false);
            } else {
                this._actionButton.set("disabled", true);                
            }

            //Reformat Table Data
            this._reRenderTable(menuItem);
            
            //Set Drop Down Menu Label
            this._displayButton.set("label", label);
            
            this.viewMode = menuItem;

        },
        
        _reRenderTable: function(nextViewMode) {
            // tags:
            // protected
            // Re-render Tree table with new columns, column spans, and data
            // presentation
            this.viewMode = nextViewMode;
            
            var patternStore = this._retrieveStore();
            var tableStructure = this._getTableStructure(); // Get structure for current view
            
            this._patternTable.setColumns(tableStructure);
            this._patternTable.setStore(patternStore);
            this._patternTable.model.clearCache();
            this._patternTable.body.refresh();

            if (this.viewMode == this.BY_ROOT) {
                this._patternTable.reexpandRows();
            }
            
        },
        
        _getTableStructure: function() {
            // tags:
            // protected            
            var includeType = (this.viewMode == this.FLAT);
        
            var layout = new Array();
            layout.push({
                field: "name",
                name: this.resources.NAME,
                width: "25em",
                formatter: dojo.hitch(this, this._formatName),
                decorator: dojo.hitch(this, this._decorateName),
                title: this.resources.TEMP_NAME
            });
            layout.push({
                fields: "inUse",
                name: this.resources.USAGE_STATUS,
                width: "8em",
                formatter: dojo.hitch(this, this._formatInUse),
                decorator: dojo.hitch(this, this._decorateInUse),
                title: this.locResources.TemplateInUse
            });
            layout.push({
                field: "userDefined",
                name: this.resources.PATTERN_ORIGIN,
                width: "8em",
                formatter: dojo.hitch(this, this._formatUserDefined),
                decorator: dojo.hitch(this, this._decorateUserDefined),
                title: this.locResources.TemplateCC
            });
            (includeType) && layout.push({
                field: "templateType",
                name: this.locResources.TemplateT,
                width: "9em",
                formatter: dojo.hitch(this, this._formatTemplateType),
                decorator: dojo.hitch(this, this._decorateTemplateType),
                title: this.locResources.TemplateT    
            });
            layout.push({
                field: "description",
                name: this.resources.DESCRIPTION,
                width: "auto",
                formatter: dojo.hitch(this, this._formatDescription),
                decorator: dojo.hitch(this, this._decorateDescription),
                title: this.locResources.TemplateD
            });
            return layout;
        },

        _flatToRoot: function(dataItems) {
            // tags:
            // protected
            var ctChildren = new Array();
            var citChildren = new Array();
            var cmitChildren = new Array();
            var cpaChildren = new Array();
            var cusrChildren = new Array();         
            var cioChildren = new Array();
            var cnodChildren = new Array();
            var newChild = function(item) {
                var child = {
                        id: item.id,
                        name: item.name,
                        type: item.type,
                        templateType: item.type,
                        uri: item.uri,
                        inUse: item.inUse,
                        referencedBy: item.referencedBy,                      
                        userDefined: !(item.userDefined === false),
                        description: item.description
                };
                return child;
            };

            dojo.forEach(dataItems, function(item) {
                // console.log(item);
                
                if (item.type == "CMM") {
                    ctChildren.push(newChild(item));
                }
                else if (item.type == "cmmInfo") {
                    citChildren.push(newChild(item));
                }
                else if (item.type == "cmmManagement") {
                    cmitChildren.push(newChild(item));
                }
                else if (item.type == "cmmPower") {
                    cpaChildren.push(newChild(item));
                }
                else if (item.type == "cmmUsers") {
                    cusrChildren.push(newChild(item));
                }
                else if (item.type == "cmmIOModule") {
                    cioChildren.push(newChild(item));
                }
                else if (item.type == "cmmNode") {
                    cnodChildren.push(newChild(item));
                }
               
            });

            var items = new Array; // Array that holds only objects with children
            if (ctChildren.length > 0) {
                // Chassis Templates Objects
                items.push({
                    id: "ct",
                    name: this.locResources.CT,
                    type: "root",
                    children: ctChildren
                });
            }
            if (citChildren.length > 0) {
                // Chassis Information Templates Objects
                items.push({
                    id: "cit",
                    name: this.locResources.CIT,
                    type: "root",
                    children: citChildren
                });
            }
            if (cmitChildren.length > 0) {
                // Chassis Management Interface Templates Objects
                items.push({
                    id: "cmit",
                    name: this.locResources.CMIT,
                    type: "root",
                    children: cmitChildren
                });
            }
            if (cpaChildren.length > 0) {
                // Chassis Power and Acoustics Templates Objects
                items.push({
                    id: "cpat",
                    name: this.locResources.CPT,
                    type: "root",
                    children: cpaChildren
                });
            }
            if (cusrChildren.length > 0) {
                // Chassis User Templates Objects
                items.push({
                    id: "cusr",
                    name: this.locResources.CUSR,
                    type: "root",
                    children: cusrChildren
                });
            }
            if (cioChildren.length > 0) {
                // Chassis IO Module Templates Objects
                items.push({
                    id: "cio",
                    name: this.locResources.CIO,
                    type: "root",
                    children: cioChildren
                });
            }
            if (cnodChildren.length > 0) {
                // Chassis IO Module Templates Objects
                items.push({
                    id: "cnod",
                    name: this.locResources.CNODE,
                    type: "root",
                    children: cnodChildren
                });
            }

            var tableData = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": items
            }, true);
            
            return tableData;
            
        },

        _rootToFlat: function(dataItems) {
            // tags:
            // protected
            var items = new Array();
            for (var loopVar = 0; loopVar < dataItems.length; ++loopVar) {
                var item = dataItems[loopVar];
                if (!item) {
                    continue;
                };
                switch (item.type) {
                    case "root":
                        // Store expand info since we "lose" the root nodes
                        if (item.id == this.POL_NAME) {
                            this._expandMe.policy = item.expandMe;
                        }
                        else if (item.id == this.SCH_NAME) {
                            this._expandMe.schedule = item.expandMe;
                        }
                        break;
                    case "CMM":
                    case "cmmInfo":
                    case "cmmManagement":
                    case "cmmPower":
                    case "cmmUsers":
                    case "cmmIOModule":
                    case "cmmNode":
                        items.push({
                            id: item.id,
                            name: item.name,
                            uri: item.uri,
                            type: "root",
                            templateType: item.type,
                            inUse: item.inUse,
                            referencedBy: item.referencedBy,
                            userDefined: item.userDefined,
                            description: item.description
                        });
                        break;
                    default:
                        // Skip unwanted pattern types...
                        break;
                }
            };

            var tableData = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": items
            }, true);

            return tableData;

        },                
        scopedRTFunc : function(){
            if(this._patternTable != null){
                var rowCount = this._patternTable.select.row.getSelected();
                if(rowCount.length == 0){
                    this._refreshTree();
                }
            }else{
                if(this.pollingFunc != null) {
                    this._stopPolling();
                }
            }
        },
        // this function is responsible for continually polling the services
        //   layer for pattern information in order to dynamically update
        _pollForStatusUpdates: function() {
            console.log("Initiating polling for chassis pattern status updates (Chassis Patterns page)");
            // call the _retrieveStore(...) function every 10 seconds in
            //   order to obtain the most recent Chassis pattern data
            
            // first, stop the existing polling mechanism (needed in case user
            //   does multiple assignments/deployments in succession)
                console.log("Stopping Chassis pattern status updates before starting again (Chassis Patterns page)");
                this._stopPolling();
                
            // next, create a bound function so that when
            //   _refreshTree(...) executes, it will execute with the correct
            //   context for "this"
            //var scopedRTFunc = dojo.hitch(this, this._refreshTree);
            
            // next, create a bound function so that when
            //   _checkStopPolling(...) executes, it wille execute with the
            //   correct context for "this"
            var scopedCSPFunc = dojo.hitch(this, this._checkStopPolling);
            
            // set the current date
            this.pollingStartDate = (new Date()).getTime();
            
            // now, simply call the setInterval(...) function to call
            //   _refreshTree(...) followed by _checkStopPolling(...) every 60
            //   seconds
            this.pollingFunc = window.setInterval(dojo.hitch(this, function(){ this.scopedRTFunc();scopedCSPFunc(); }), 60000);
        }, // _pollForStatusUpdates
        
        // this function is responsible for determining whether to stop the
        //   polling for status updates
        _checkStopPolling: function() {
            // if we have been polling for too long, stop the polling
            //   mechanism
            var curDate = (new Date()).getTime();
            var dateDiff = curDate - this.pollingStartDate;
            // if we've been polling for longer than 10 minutes, stop
            if(dateDiff > 600000) {
                console.log("Polling for Chassis pattern status updates has occurred for longer than 10 minutes; stopping polling (Chassis Patterns page)");
                window.clearInterval(this.pollingFunc);
                this.pollingFunc = null;
            }
        }, // _checkStopPolling
        // this function is responsible for stopping the polling
        _stopPolling: function() {

            if(this.pollingFunc != null) {
                console.log("Stopping Chassis pattern status polling (Chassis Patterns page)");
                window.clearInterval(this.pollingFunc);
                this.pollingFunc = null;
            }

        },//_stopPolling
        
        goForeground: function(params) { //Page base function that fires when the page is brought to focus
            
            this._refreshTree();
                    
            this._pollForStatusUpdates();

            return true;
        },
        
        goBackground: function(){ //Page base function that fires when the page losses to focus
            
            this._stopPolling();
            
        }
    });

});