define([
        "dojo", 
        "dijit", 
        "support/panels/Panel", 
        "dojo/i18n!./nls/CheckLabelButton", 
        "dojo/text!./templates/CheckLabelButton.html", 
        "support/widgets/ToggleButton", 
        "support/widgets/ButtonBar", 
        "dijit/form/NumberTextBox", 
        "dijit/form/CheckBox"
	], function (dojo, dijit, panel, mri, template) {

	return dojo.declare("support.panels.CheckLabelButton", [ support.panels.Panel ], {
	    
	    // templateString: [protected] String
	    // This widget's HTML template, provided.
	    templateString: template,
	        
	    // locResources: [protected] Object
	    // This widget's National Language Source (NLS) object.
	    locResources: mri, 
	    
		widgetsInTemplate : true,
		baseClass : "CheckLabelButton", // This widget's base CSS classname.
		labelValue : "",
		buttonType : "support.widgets.ToggleButton",
		buttonProps : null,
		buttonVisibility : "display:none;",
		labelColumnWidth : "340px;",
		checkedState : "false",
		showCheckBox : "false",
		toolTip : null,
		infoTip : null,
		widgetTitle: "",
		hideButton: false,
		buttondivId: null,
		selected: null,
		
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/CheckLabelButton.css",

		// When this method is called, all variables inherited from superclasses are "mixed in".
		// Common operations for postMixInProperties
		// 1) Modify or assign values for widget property variables defined in the template
		// HTML file
		postMixInProperties : function () {

			this.inherited(arguments);
			
			// Set default button properties
			if (!this.buttonProps) {
				this.buttonProps = {};
				this.buttonProps["enabled"] = true;
			}

		},

		// postCreate() is called after buildRendering(). This is useful to override when
		// you need to access and/or manipulate DOM nodes included with your widget.
		// DOM nodes and widgets with the dojoAttachPoint attribute specified can now be
		// directly
		// accessed as fields on "this".
		// Common operations for postCreate
		// 1) Access and manipulate DOM nodes created in buildRendering()
		// 2) Add new DOM nodes or widgets
		postCreate : function () {

			this.inherited(arguments);

			this._createButton();
			
			if (this.checkedState == "false") {
				this.buttonId.style.display = "none";
				this.checkboxId.set("checked", false);
			} else {
				this.buttonId.style.display = "block";
				this.checkboxId.set("checked", true);
			}
			
			if (this.showCheckBox == "false") {			    
			    this.checkboxCell.style.display = "none";			       
			} else {			    
			    this.checkboxCell.style.display = "block";
			}
			
			if (this.toolTip != null) {
				this.toolTipIcon.style.display = "inline";
				this.labelId.style.padding = "0px 10px 0px 0px";
			}	
			
			if (this.infoTip != null) {
                this.infoTipIcon.style.display = "inline";
                this.labelId.style.padding = "0px 10px 0px 0px";
            }   
			
			if (this.hideButton == true) {			    
			    this.hideButtonDiv.style.display = "none";
			}
		},
		
		_createButton: function() {
        	var propKeys = Object.keys(this.buttonProps);
		    switch(this.buttonType) {
		        case "support.widgets.ButtonBar":
		            this.buttondivId = new support.widgets.ButtonBar({
		                title: this.widgetTitle,
		                values: this.buttonProps["values"],
		                titles: this.buttonProps["titles"],
		                selected: this.selected
		            }).placeAt(this.buttondivIddiv);
		            break;
		        case "dijit.form.NumberTextBox":
		        	this.buttondivId = new dijit.form.NumberTextBox();
		        	for(var propCount=0;propCount<propKeys.length;++propCount) {
						this.buttondivId.set(propKeys[propCount], this.buttonProps[propKeys[propCount]]);
					}
					this.buttondivId.placeAt(this.buttondivIddiv);
		        	break;
		        case "support.widgets.ToggleButton":
		        	this.buttondivId = new support.widgets.ToggleButton();
		        	for(var propCount=0;propCount<propKeys.length;++propCount) {
						this.buttondivId.set(propKeys[propCount], this.buttonProps[propKeys[propCount]]);
					}
					this.buttondivId.placeAt(this.buttondivIddiv);
					break;
		        default:
		    }
		    if (this.buttondivId) {
		        dojo.connect(this.buttondivId, "onClick", this, this._onButtonClick);
		        dojo.connect(this.buttondivId, "onChange", this, this._onChange);
		    }
		},

		onClick : function (evt) {
			// Does nothing.
			// Override to specify custom handler.        
		},

		_onClick : function (evt) {
			var checkBoxState = this.checkboxId.get("value");
			if (checkBoxState == "on") {
				this.buttonId.style.display = "block";
			} else {
				this.buttonId.style.display = "none";
			}
			this.onClick(evt);
			this._setToolTip();

		},
		
        onChange : function (value) {
            // Does nothing.
            // Override to specify custom handler.        
        },
        
        _onChange: function(value) {
            this.onChange(value);
        },      

        onButtonClick: function(evt) {
            // Does nothing.
            // Override to specify custom handler.        
        },
        
        _onButtonClick: function(evt) {
    		this.onClick(evt);
        
        },		
		
		_getCheckBoxAttr : function () {
			return this.checkboxId.get("value");
		},

		_setCheckBoxAttr : function (state) {
			if (state == "on") {
				this.checkboxId.set("value", "on");
				this.buttonId.style.display = "block";
			} else {
				this.checkboxId.set("value", false);
				this.buttonId.style.display = "none";
			}
		},

		_getButtonStateAttr : function () {
	      if (this.buttonType == "support.widgets.ToggleButton"){
			   return this.buttondivId.get("enabled");
        	} else if (this.buttonType == "support.widgets.ButtonBar") {
        		return this.buttondivId.get("selIndex");
        	} else if (this.buttonType == "dijit.form.NumberTextBox") {
        		return this.buttondivId.get("value");
        	}
		},
		
		_setButtonStateAttr : function (value) {
	       	if (this.buttonType == "support.widgets.ToggleButton"){
        		this.buttondivId.set("enabled",  value);
        	} else if (this.buttonType == "support.widgets.ButtonBar") {
        		this.buttondivId.set("selIndex", value);
        	} else if (this.buttonType == "dijit.form.NumberTextBox") {
        		this.buttondivId.set("value", value);
        	}
		},
		
		_setToolTip : function() {
			var uncheckedText = this.locResources.UNCHECKED_TEXT;
			var checkedText = this.locResources.CHECKED_TEXT;			
			if (this.checkboxId.get("value") == "on") {
				this.checkboxId.set("title", checkedText);
			} else {
				this.checkboxId.set("title", uncheckedText);
			}
		},
		
		isValid : function() {
		    if (this.buttonType == "dijit.form.NumberTextBox") {
		        return this.buttondivId.isValid();
		    }
		    return false;
		},
		
		setNumberTextBoxConstraints : function(constraints) {
            if (this.buttonType == "dijit.form.NumberTextBox" && constraints){
                this.buttondivId.set("constraints", constraints);
            }
        },
        
        _setCheckedStateAttr: function(value) {
            this.checkedState = value;
            if (value == "false") {
                this.buttonId.style.display = "none";
                this.checkboxId.set("checked", false);
            } else {
                this.buttonId.style.display = "block";
                this.checkboxId.set("checked", true);
            }
        },
        
        _setLabelColumnWidthAttr: function(value) {
            this.labelColumnWidth = value;
            this.columnlabel.style.width = value;
        }
	});
});
