define([ 
        "dojo", 
        "dijit", 
        "support/ConfigPatternsAll", 
        "support/panels/Panel", 
        "dojo/i18n!./nls/DeployPlaceholderChassis",
        "dojo/text!./templates/DeployPlaceholderChassis.html", 
        "support/base/NextGenUIMixin", 
        "support/base/TableMixin",
        "dojo/data/ItemFileWriteStore",
        "dojox/grid/LazyTreeGridStoreModel",        
        "dijit/form/DropDownButton", 
        "dijit/form/Button",
        "dijit/Menu", 
        "dijit/MenuItem",
        "dijit/PopupMenuItem",
        "dijit/Toolbar",
        "dijit/ToolbarSeparator",
        "dojo/string", 
        "support/dialogs/AddPlaceholderChassis", 
        "support/dialogs/DeployPlaceholderChassis",
        "support/dialogs/DeployServerTemplate", 
        "support/dialogs/DeployChassisTemplate",
        "support/dialogs/ProgressDialog"
        ], function(dojo, dijit, layer, panel, mri, template) {

    return dojo.declare("support.panels.DeployPlaceholderChassis", [ support.panels.Panel, support.base.NextGenUIMixin, support.base.TableMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in managing templates for
        // Configuration Patterns.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "DeployPlaceholderChassis",

        _placeholderTable: null,
        _tableToolbar: null,
                        
        editButton: null,
        deleteButton: null,
        
        refreshRoot: null,        
        
        // the properties used to construct the title of this page in the
        //   NextGen UI
        cpProductName: null,
        cpPageContext: null,
        cpPageName: null,
        
        // ID prefix used for table
        gridID: "PlaceholderChassisTable",

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/DeployPlaceholderChassis.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
            this.cpProductName = this.resources.PRODUCT_NAME;
            // this.cpPageContext = this.resources.CHASSIS; Must remain null (unspecified), per XB216902
            this.cpPageName = this.locResources.PLACEHOLDER;
        },

        // postCreate() is called after
        // buildRendering(). This is useful to override
        // when
        // you need to access and/or manipulate DOM
        // nodes included with your widget.
        // DOM nodes and widgets with the
        // dojoAttachPoint attribute specified can now
        // be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in
        // buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            
            this.inherited(arguments);

            this.subscribe("RefreshTable", function(data) {
                this._refreshTree();
            });
            this.subscribe("ReDeployComplete", function(data) {
                this._refreshTree();
            });
            this.subscribe("DeployServerTemplateComplete", function(data) {
                this._refreshTree();
            });
            this.subscribe("DeployChassisTemplateComplete", function(data) {
                this._refreshTree();
            });
            this.subscribe("addPHChassis", function() {
                this._refreshTree();
            });
            
            // subscribe to the topic "removePHChassis" so that the Available
            //   Table can reload its contents and immediately show the
            //   removal of the placeholder chassis
            this.subscribe("removePHChassis", function() {
                this._refreshTree();
            });
            
            this.subscribe("deployPHChassis", function() {
                this._refreshTree();
            });           
            
            // Setup table widget...
            var placeholderStore = this._retrieveStore();
            var tableStructure = this._getTableStructure();
            
            this._placeholderTable = this.getTable(placeholderStore, tableStructure, this.resources.NO_DATA, this.gridID, true);
            this._placeholderTable.column(0).sort(false);
            
            // Set timeout to ensure table data has enough time to load
            setTimeout(dojo.hitch(this, function() {
                this._placeholderTable.placeAt(this.placeholderTable);
                this._placeholderTable.startup();
            }), 1000);  
                        
            // Connect handlers to reset table capabilities
            dojo.connect(this._placeholderTable.select.row, "onSelectionChange", this, this.setTableCapabilities); // Fires on row selection
            dojo.connect(this._placeholderTable, "setStore", this, this.setTableCapabilities); // Fires on store reset
            
            // Connect double click handler, to launch the selected pattern's Details page
            dojo.connect(this._placeholderTable, "onCellDblClick", this, function() {
                var ids = this._placeholderTable.select.row.getSelected();
                if (ids.length > 0) {
                    var item = this._placeholderTable.row(ids[0], true).item();
                    if (!item.children) {
                        this._selectedTemplate = item;
                        this.detailTemplate();
                    }
                }
            });

            // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
            this.changeNextGenUITitle();

            // reload contents of templates table; by default, switching
            //   between links in the details selector does not reload page
            this.subscribeToDetailsSelectorChanges(dojo.hitch(this, function() {
                this._refreshTree();
                this.changeNextGenUITitle();

            }));
            
            this._loadToolBar(this._placeholderTable); // load table toolbar

        },

        _refreshTree: function() {
            // tags:
            // protected
            // Refresh tree data
            var patternStore = this._retrieveStore();
            this._placeholderTable.setStore(patternStore);
            this._placeholderTable.model.clearCache();
            this._placeholderTable.body.refresh().then(this._placeholderTable.reexpandRows);
            
        },

        _retrieveStore: function() {
            // Connect to data source and create store
            var placeholderData = "";
           
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "phchassis?format=detailed",
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: dojo.hitch(this, function(response) {
                    placeholderData = this._getPlaceholders(response.items);
                }),
                handle: function() {
                    
                }
            });
                      
            var placeholderStore = new dojo.data.ItemFileWriteStore({
                data: dojo.fromJson(placeholderData)
            });
            
            if (!("hasChildren" in placeholderStore)) {
                placeholderStore.hasChildren = function(id, item) { 
                    return item && this.getValues(item, "bays").length;
                };
            };
            
            if (!("getChildren" in placeholderStore)) {
                placeholderStore.getChildren = function(item) { 
                    return this.getValues(item, "bays");
                };
            };
            
            return placeholderStore;
        },
        
        _getPlaceholders: function(response){
            var items = response?response:[];
            var i,j;
            
            //Create sort-friendly formatted names for displaying on table
            for(i = 0; i < items.length; i++){
                items[i].formattedName = items[i].name;
                items[i].description = items[i].description;
                items[i].formFactor = "";

                if(items[i].bays){
                    for(j = 0; j < items[i].bays.length; j++){
                        if(items[i].bays[j].formFactor == "1sd"){ //Assume Caracara 
                            items[i].bays[j].formattedName = this.resources.BAY_NUM.replace("%1", items[i].bays[j].bayId+":"+items[i].bays[j].subBayId);
                            
                            var searchResults = "notFound";   //Set initial search for other node 
                            var currentNodeID = items[i].bays[j].id; //Get current node ID
                            var nodeArray = currentNodeID.split(":"); //Split node ID to get subbay
                            var nodeNumber = nodeArray[1]; //Set subbay var of current node
                            var nodeSearch = ""; //Node to search
                            var newSubbayId = ""; //Node subbay to search
                            
                            if(nodeNumber == "1"){ //If current node 1 then search for node 2
                                nodeSearch = nodeArray[0]+":2";
                                newSubbayId = "2";
                            }
                            
                            if(nodeNumber == "2"){ //If current node 2 then search for node 1
                                nodeSearch = nodeArray[0]+":1";
                                newSubbayId = "1";
                            }

                            for(var y = 0; y < items[i].bays.length; y++){ //Loop all bays on current chassis to search for other node
                                if(items[i].bays[y].id == nodeSearch){
                                    searchResults = "found";    //If node found then set searchResults to found
                                } 
                            }
                            
                            if(searchResults == "notFound"){ //If other node not found then create it as empty bay
                                //Clone node information
                                var secondNode = dojo.clone(items[i].bays[j]); //clone current node
                                secondNode.profileId = "";
                                secondNode.templateId = "";
                                secondNode.templateName = "--Unassigned--";
                                secondNode.location = nodeSearch;
                                secondNode.id = nodeSearch;
                                secondNode.profileName = "--Unassigned--";
                                secondNode.subBayId = newSubbayId;
                                items[i].bays.push(secondNode);
                            }
                            
                        }else if(items[i].bays[j].formFactor == "4s" || items[i].bays[j].formFactor == "pxu"){ //Assume 2 bay
                            var currentBay = items[i].bays[j].bayId;
                            var partnerBay = parseInt(items[i].bays[j].bayId) + 1;
                            
                            items[i].bays[j].formattedName = this.resources.BAY_NUM.replace("%1", currentBay+" , "+partnerBay);

                            for(var y = 0; y < items[i].bays.length; y++){ //Loop all bays on current chassis to remove partnerBay
                                if(items[i].bays[y].bayId == partnerBay){
                                    items[i].bays.splice(y,1);    //If bay found then remove it
                                } 
                            }

                            
                        }else{ //Assume 1 bay
                            items[i].bays[j].formattedName = this.resources.BAY_NUM.replace("%1", items[i].bays[j].bayId);                                                        
                        }
                    }
                }
                
                //items[i].bays.sort(function(a,b){return a-b;});
                items[i].bays.sort(function (a, b) {
                    return a.bayId-b.bayId;
                });
            } 
            
            var newData = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": items
            }, true);
            
            return newData;
        },

        // Load toolbar buttons/menus
        _loadToolBar: function(grid) {
            
            var toolbar = grid && grid.toolBar && grid.toolBar.widget;
            
            if (toolbar) {
                if (this._placeholderTable == null) {
                    // Profile table not created. Nothing to
                    // do but return.
                    return;
                }
                
                if (!this.actionMenu) {
                    this.actionMenu = new dijit.Menu({
                        style: "display: none;"
                    });
                }
    
                // Add Placeholder Chassis
                var createButton = new dijit.form.Button({
                    //id: this.id + "cp_placeholdertable_createbutton",
                    showLabel: false,
                    iconClass: "spCreateLgIcon",
                    disabled: false,
                    title: this.locResources.CREATE_PLACEHOLDER_CHASSIS_TITLE,
                    onClick: dojo.hitch(this, this.createNewPlaceholderChassis),
                    onKeypress: dojo.hitch(this, this.createNewPlaceholderChassis)
                });
    
                toolbar.addChild(createButton);
                
                // Add custom Edit button...                
                this.editButton = new dijit.form.Button({
                    //id: this.id + "cp_placeholdertable_editbutton",
                    showLabel: false,
                    iconClass: "spEditInactiveLgIcon",
                    disabled: true,
                    title: this.locResources.EDIT_PLACEHOLDER_CHASSIS_TITLE,
                    onClick: dojo.hitch(this, this.editPlaceholderChassis),
                    onKeypress: dojo.hitch(this, this.editPlaceholderChassis)
                });
                toolbar.addChild(this.editButton);
                
                // Add custom Delete button...                
                this.deleteButton = new dijit.form.Button({
                    //id: this.id + "cp_placeholdertable_deletebutton",
                    showLabel: false,
                    iconClass: "spDeleteInactiveLgIcon",
                    disabled: true,
                    title: this.locResources.DELETE_PLACEHOLDER_CHASSIS_TITLE,
                    onClick: dojo.hitch(this, this.deletePlaceholderChassis),
                    onKeypress: dojo.hitch(this, this.deletePlaceholderChassis)
                });
                toolbar.addChild(this.deleteButton);
               
                // Add Deploy Template button...    
                var deployMenu = new dijit.Menu({
                    style: "display: none;"
                });
                
                this.deployServerTemp = new dijit.MenuItem({
                    //id: this.id + "cp_placeholdertable_deployserver",
                    label: this.locResources.DEPLOY_SERVER_TEMP,
                    title: this.locResources.DEPLOY_SERVER_TEMP_TITLE,
                    onClick: dojo.hitch(this, this.deployServerTemplate),
                    onKeypress: dojo.hitch(this, this.deployServerTemplate),
                    disabled: true
                });
                deployMenu.addChild(this.deployServerTemp);
                
                this.deployServerTemp3 = new dijit.MenuItem({
                    //id: this.id + "cp_placeholdertable_deployserver3",
                    label: this.locResources.DEPLOY_SERVER_TEMP,
                    title: this.locResources.DEPLOY_SERVER_TEMP_TITLE,
                    iconClass: "spDeployLgIcon",
                    onClick: dojo.hitch(this, this.deployServerTemplate),
                    onKeypress: dojo.hitch(this, this.deployServerTemplate)
                    //disabled: true
                });

				//Only show chassis deploy if not Augusta.
				 // All chassis related actions removed from Augusta
                if(this.CP_RUN_ENVIRONMENT != "augusta"){
                
                    this.deployChassisTemp = new dijit.MenuItem({
                        //id: this.id + "cp_placeholdertable_deploychassis",
                        label: this.locResources.DEPLOY_CHASSIS_TEMP,
                        title: this.locResources.DEPLOY_CHASSIS_TEMP_TITLE,
                        onClick: dojo.hitch(this, this.deployChassisTemplate),
                        onKeypress: dojo.hitch(this, this.deployChassisTemplate),
                        disabled: true
                    });
                    deployMenu.addChild(this.deployChassisTemp);
                    
                    this.deployChassisTemp3 = new dijit.MenuItem({
                        //id: this.id + "cp_placeholdertable_deploychassis3",
                        label: this.locResources.DEPLOY_CHASSIS_TEMP,
                        title: this.locResources.DEPLOY_CHASSIS_TEMP_TITLE,
                        iconClass: "spDeployLgIcon",
                        onClick: dojo.hitch(this, this.deployChassisTemplate),
                        onKeypress: dojo.hitch(this, this.deployChassisTemplate)
                        //disabled: true
                    });
                
                }
                
                this.deployButton = new dijit.form.DropDownButton({
                    //id: this.id + "cp_placeholdertable_deploybutton",
                    showLabel: false,
                    iconClass: "spDeployInactiveLgIcon",
                    title: this.locResources.DEPLOY_PATTERN,
                    dropDown: deployMenu,
                    disabled: true
                });
                
                toolbar.addChild(this.deployButton);
                
                // Add Deploy Placeholder button...            

				//Only show chassis placeholder deploy if not Augusta.
				 // All chassis related actions removed from Augusta
                if(this.CP_RUN_ENVIRONMENT != "augusta"){

                    this.deployPlaceholderButton = new dijit.form.Button({
                        showLabel: false,
                        iconClass: "spDeployPlaceholderInactive",
                        disabled: true,
                        title: this.locResources.DEPLOY_PLACEHOLDER_CHASSIS_TITLE,
                        onClick: dojo.hitch(this, this.deployPlaceholderChassis),
                        onKeypress: dojo.hitch(this, this.deployPlaceholderChassis)
                    });
                    toolbar.addChild(this.deployPlaceholderButton);
                    
                    toolbar.addChild(new dijit.ToolbarSeparator());
                
                    this.deployPlaceholderItem = new dijit.MenuItem({
                        label: this.locResources.DEPLOY_PLACEHOLDER_CHASSIS,
                        title: this.locResources.DEPLOY_PLACEHOLDER_CHASSIS_TITLE,
                        iconClass: "spDeployPlaceholder",
                        onClick: dojo.hitch(this, this.deployPlaceholderChassis),
                        onKeypress: dojo.hitch(this, this.deployPlaceholderChassis)
                    });
                    
                    if (this._placeholderTable.store._arrayOfAllItems.length > 0){
                        this.deployPlaceholderButton.set("iconClass", "spDeployPlaceholder");
                        this.deployPlaceholderButton.set("disabled", false);
                        this.actionMenu.addChild(this.deployPlaceholderItem);
                    }
                
                }else{
                    
                    toolbar.addChild(new dijit.ToolbarSeparator());

                }
                
                // Add to Actions Menu
                var expandAllItem = new dijit.MenuItem({
                    label: this.resources.EXPAND_ALL,
                    iconClass: "spExpandAllLgIcon",
                    title: this.resources.EXPAND_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._placeholderTable.expandAll();
                    }),
                    onKeypress: dojo.hitch(this, function() {
                        this._placeholderTable.expandAll();
                    })
                });
                this.actionMenu.addChild(expandAllItem,0);
                
                this.collapseAllItem = new dijit.MenuItem({
                    //id: this.id + "cp_placeholdertable_collapseallitem",
                    label: this.resources.COLLAPSE_ALL,
                    iconClass: "spCollapseAllLgIcon",
                    title: this.resources.COLLAPSE_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._placeholderTable.collapseAll();
                    }),
                    onKeypress: dojo.hitch(this, function() {
                        this._placeholderTable.collapseAll();
                    })
                });
                this.actionMenu.addChild(this.collapseAllItem,1);
                
                this.createItem = new dijit.MenuItem({
                    label: this.resources.CREATE,
                    iconClass: "spCreateLgIcon",
                    title: this.locResources.CREATE_PLACEHOLDER_CHASSIS_TITLE,
                    onClick: dojo.hitch(this, this.createNewPlaceholderChassis),
                    onKeypress: dojo.hitch(this, this.createNewPlaceholderChassis)
                });
                this.actionMenu.addChild(this.createItem,2);
                
                var menuSeparator = new dijit.MenuSeparator();
                this.actionMenu.addChild(menuSeparator);
                  
                // Add action menu to action button and create action button if not already created
                if (this._actionButton) {
                    this._actionButton.dropDown = this.actionMenu;
                }
                else {
                    this._actionButton = new dijit.form.DropDownButton({
                        //id: this.id + "cp_placeholdertable_actionbutton",
                        label: this.resources.ALL_ACTIONS,
                        title: this.resources.ALL_ACTIONS_TITLE,
                        showLabel: true,
                        dropDown: this.actionMenu
                    });   
                    
                    toolbar.addChild(this._actionButton);
                }
                
            }            
        },
        
        // Add Placeholder Chassis
        createNewPlaceholderChassis: function() {
            var apcdialog = new support.dialogs.AddPlaceholderChassis();
            apcdialog.show();
        },
        
        // Delete Placeholder Chassis
        deletePlaceholderChassis: function() {
            var selectedIds = this._placeholderTable.select.row.getSelected();
            
            var deleteItems = new Array();
            var names = new Array();
            var htmlNames = new Array();
            dojo.forEach(selectedIds, dojo.hitch(this, function(id) {
                var item = this._placeholderTable.row(id, true).item();
                if (item) {
                    deleteItems.push(item);
                    var name = item.name.toString();
                    names.push(name);
                    var htmlName = name.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;");
                    htmlNames.push(htmlName);
                }
            }));
            
            var processDeleteRequest = function() {
                console.log("deleteItems=", deleteItems);
                var responses = new Array();
                this.getProgressDialog("delete", false).show();
                dojo.forEach(deleteItems, dojo.hitch(this, function(item) {
                    //console.log("item", idx, item.id);
                    dojo.xhrDelete({
                        url: this.REST_PATH + "phchassis/" + item.id,
                        handleAs: "json",
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        error: dojo.hitch(this, function(response, ioArgs) {
                            var msg = "";
                            if (ioArgs.xhr.status == 400) {
                                msg = this.locResources.ERROR_PLACEHOLDER_DELETE_INUSE;
                            }
                            else {
                                msg = this.locResources.ERROR_PLACEHOLDER_DELETE;
                            }
                            this.getModalDialog("Error", msg).show();

                        }),
                        handle: dojo.hitch(this, function(response, ioArgs) {
                            dojo.publish("ProgressComplete", response);
                            // publish the topic "removePHChassis" so that the Available
                            //   Table can reload its contents and immediately show that
                            //   the placeholder chassis was removed
                            dojo.publish("removePHChassis", []);
                        })
                    });
                }));                
                
            };
            

            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.DELETE,
                        type: "Warning",
                        text: this.locResources.CONFIRM_DELETE_PH,
                        info: this.locResources.DELETEINFO                    
                    });

                    confirmDialog.set("content", "<p>" + this.locResources.DELETE_PLACEHOLDERS.replace("%1", htmlNames.join(", ")) + "</p>");
                    confirmDialog.startup();
                    confirmDialog.confirm(processDeleteRequest, this);                        
                
                }));
 
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.DELETE,
                        type: "Warning",
                        text: this.locResources.CONFIRM_DELETE_PH,
                        info: this.locResources.DELETEINFO                    
                    });

                    confirmDialog.set("content", "<p>" + this.locResources.DELETE_PLACEHOLDERS.replace("%1", htmlNames.join(", ")) + "</p>");
                    confirmDialog.startup();
                    confirmDialog.confirm(processDeleteRequest, this);                        
                
                }));
                
            }

            
            return false;
        },
        
        // Edit Placeholder Chassis
        editPlaceholderChassis: function() {
            var id = this._placeholderTable.select.row.getSelected()[0];
            var apcdialog = new support.dialogs.AddPlaceholderChassis({
                templateIDNum: id,
                editTemplate: true
            });
            apcdialog.show();
        },
        
        // Deploy Placeholder Chassis
        deployPlaceholderChassis: function() {
            var dpcdialog = new support.dialogs.DeployPlaceholderChassis();
            dpcdialog.show();
        },

        // Deploy Server Template
        deployServerTemplate: function() {
            var id = this._placeholderTable.select.row.getSelected()[0];
            var item = this._placeholderTable.row(id, true).item();
            var formFactor = item.formFactor[0];
            var dst;
            
            if(formFactor == ""){

                dst = new support.dialogs.DeployServerTemplate({
                    endpointUUID: id
                });

            }else{

                dst = new support.dialogs.DeployServerTemplate({
                    endpointUUID: id,
                    patternFormFactor: formFactor
                });

            }
            dst.show();
        },
        
        // Deploy Chassis Template
        deployChassisTemplate: function() {
            var id = this._placeholderTable.select.row.getSelected()[0];
            var item = this._placeholderTable.row(id, true).item();
            var uuid = item.uuid;
            
            var dct = new support.dialogs.DeployChassisTemplate({
                chassisUUID: uuid[0]
            });
            dct.show();
        },

        // Set Table Formatters
        _decorateName: function(item) {
            return "<span title=\"" + item + "\">" + item + "</span>";
        },
        
        _formatName: function(item) {
            // tags:
            // protected
            var value = (item && item.formattedName) ? item.formattedName : "";
            value = this.encodeForHtml(value);
            return value;
        },
        
        _decorateDescription: function(item) {
            return "<span title=\"" + item + "\" style=\"white-space:normal;\">" + item + "</span>";
        },
        
        _formatDescription: function(item) {
            // tags:
            // protected
            if (!item || !item.description) {
                return "";
            }
            var value = this.decodeSpecialCharacters(item.description);
            value = this.encodeForHtml(value);
            return value;
        },

        // Set Table Capabilities
        setTableCapabilities: function() {
            
            if (!this._placeholderTable) {
                return; // nothing to do. return now
            }
            
            this.editButton.set("iconClass", "spEditInactiveLgIcon");
            this.editButton.set("disabled", true);
            this.deleteButton.set("iconClass", "spDeleteInactiveLgIcon");
            this.deleteButton.set("disabled", true);
            this.deployButton.set("iconClass", "spDeployInactiveLgIcon");
            this.deployButton.set("disabled", true);
            
            var ids = this._placeholderTable.select.row.getSelected();
            
            menuPosition = 4;
            
            // Deploy Placeholder Button

				//Only show chassis deploy if not Augusta.
				 // All chassis related actions removed from Augusta
            if(this.CP_RUN_ENVIRONMENT != "augusta"){

                if (this._placeholderTable.store._arrayOfAllItems.length > 0){
                    this.deployPlaceholderButton.set("iconClass", "spDeployPlaceholder");
                    this.deployPlaceholderButton.set("disabled", false);
                    this.actionMenu.addChild(this.deployPlaceholderItem);
                }
                else {
                    this.deployPlaceholderButton.set("iconClass", "spDeployPlaceholderInactive");
                    this.deployPlaceholderButton.set("disabled", true);
                    this.deployButton.set("iconClass", "spDeployInactiveLgIcon");
                    this.deployButton.set("disabled", true);
                    this.actionMenu.removeChild(this.deployPlaceholderItem);
                }
            
            }
                                   
            
            // Action Menu
            if (ids.length > 0){
                
                item = this._placeholderTable.row(ids[0], true).item();
                
                // Edit Placeholder enabled if one placeholder chassis is selected
                if (!item.bayId && ids.length == 1){
                    if (this.editPlaceholder == null){
                        this.editPlaceholder = new dijit.MenuItem({
                    //        id: this.id + "cp_placeholdertable_editplaceholder",
                            label: this.resources.EDIT,
                            title: this.locResources.EDIT_PLACEHOLDER_CHASSIS_TITLE,
                            iconClass: "spEditLgIcon",
                            onClick: dojo.hitch(this, this.editPlaceholderChassis),
                            onKeypress: dojo.hitch(this, this.editPlaceholderChassis)
                        });
                        this.actionMenu.addChild(this.editPlaceholder, menuPosition++);
                    }
                    this.editButton.set("iconClass", "spEditLgIcon");
                    this.editButton.set("disabled", false);
                }
                else {
                    this.actionMenu.removeChild(this.editPlaceholder);
                    this.editPlaceholder = null;
                    this.editButton.set("iconClass", "spEditInactiveLgIcon");
                    this.editButton.set("disabled", true);
                }
                
                // Delete Placeholder enabled if one or more chassis are selected
                var placeholdersSelected = true;
                for (var i = 0; i < ids.length; i++){
                    item = this._placeholderTable.row(ids[i], true).item();
                    if ((item != null) && (item.bayId)){
                        placeholdersSelected = false;
                        break;
                    } 
                }
                if (placeholdersSelected){
                    if (this.deletePlaceholder == null){
                        this.deletePlaceholder = new dijit.MenuItem({
                          //  id: this.id + "cp_placeholdertable_deleteplaceholder",
                            label: this.resources.DELETE,
                            title: this.locResources.DELETE_PLACEHOLDER_CHASSIS_TITLE,
                            iconClass: "spDeleteLgIcon",
                            onClick: dojo.hitch(this, this.deletePlaceholderChassis),
                            onKeypress: dojo.hitch(this, this.deletePlaceholderChassis)
                        });
                        this.actionMenu.addChild(this.deletePlaceholder, menuPosition++);
                    }
                    this.deleteButton.set("iconClass", "spDeleteLgIcon");
                    this.deleteButton.set("disabled", false);
                }
                else {
                    this.actionMenu.removeChild(this.deletePlaceholder);
                    this.deletePlaceholder = null;
                    this.deleteButton.set("iconClass", "spDeleteInactiveLgIcon");
                    this.deleteButton.set("disabled", true);
                }
                
                // Deploy Patterns Button
                if (ids.length == 1){
                    var item = this._placeholderTable.row(ids[0], true).item();
                    
                    this.deployButton.set("iconClass", "spDeployLgIcon");
                    this.deployButton.set("disabled", false);
                    
                    if(item != null){
                        if (item.profileId == "" && item.bayId) {
                            this.deployServerTemp.set("disabled", false);
                            this.actionMenu.addChild(this.deployServerTemp3, menuPosition++);
                        } else {
                            this.deployServerTemp.set("disabled", true);
                            this.actionMenu.removeChild(this.deployServerTemp3);
                        }
                    }
                    
                    
                    //If not augusta then perform actions on deploy chassis else
                     //just perform actions with deploy server
                    if(this.CP_RUN_ENVIRONMENT != "augusta"){
                        
                        if (item == null){
                            this.deployButton.set("iconClass", "spDeployInactiveLgIcon");
                            this.deployButton.set("disabled", true);
                        }
                        else if (!item.bayId && item.profileId == ""){
                            this.deployChassisTemp.set("disabled", false);
                            this.actionMenu.addChild(this.deployChassisTemp3, menuPosition++);
                        }
                        else {
                            this.deployChassisTemp.set("disabled", true);
                            this.actionMenu.removeChild(this.deployChassisTemp3);                        
                        }
                        
                        // Disable deploy button if both deploy options are disabled
                        if (this.deployChassisTemp.disabled && this.deployServerTemp.disabled) {
                            this.deployButton.set("iconClass", "spDeployInactiveLgIcon");
                            this.deployButton.set("disabled", true);
                        }
                    
                    }else{

                        if (item == null){
                            this.deployButton.set("iconClass", "spDeployInactiveLgIcon");
                            this.deployButton.set("disabled", true);
                        }

                        // Disable deploy button if both deploy options are disabled
                        if (this.deployServerTemp.disabled) {
                            this.deployButton.set("iconClass", "spDeployInactiveLgIcon");
                            this.deployButton.set("disabled", true);
                        }
                        
                    }
                    
                }
                else {
                    this.deployButton.set("iconClass", "spDeployInactiveLgIcon");
                    this.deployButton.set("disabled", true);
                    this.deployServerTemp.set("disabled", true);
                    this.actionMenu.removeChild(this.deployServerTemp3);

                    if(this.CP_RUN_ENVIRONMENT != "augusta"){
                        this.deployChassisTemp.set("disabled", true);
                        this.actionMenu.removeChild(this.deployChassisTemp3);
                    }
                    
                }
                
                // Delete Server Profile enabled if one or more bays with assign profiles are selected
                var enableDeleteServer = true;
                
                for (var i = 0; i < ids.length; i++) {
                    item = this._placeholderTable.row(ids[i], true).item();
                    if (!item.bayId || item.profileName == "--Unassigned--"){
                        enableDeleteServer = false;
                        break;
                    }
                }
                
                if (enableDeleteServer){
                    if (this.deleteServer == null){
                        this.deleteServer = new dijit.MenuItem({
                      //      id: this.id + "cp_placeholdertable_deleteserverprofile",
                            label: this.resources.UNASSIGN_SERVER_PROFILE,
                            title: this.locResources.UNASSIGN_SERVER_PROFILE_TITLE,
                            onClick: dojo.hitch(this, this.unassignServerProfile),
                            onKeypress: dojo.hitch(this, this.unassignServerProfile)
                        });
                        this.actionMenu.addChild(this.deleteServer);
                    }
                }
                else {
                    this.actionMenu.removeChild(this.deleteServer);
                    this.deleteServer = null;
                }
                
                // Delete Chassis Profile enabled if one or more chassis with assign profiles are selected
                var enableDeleteChassis = true;

                for (var i = 0; i < ids.length; i++){
                    item = this._placeholderTable.row(ids[i], true).item();
                    if (item.bayId || item.profileName == "--Unassigned--"){
                        enableDeleteChassis = false;
                        break;
                    }
                }

                if (enableDeleteChassis){
                    if (this.deleteChassis == null){
                        this.deleteChassis = new dijit.MenuItem({
                        //    id: this.id + "cp_placeholdertable_deletechassisprofile",
                            label: this.resources.UNASSIGN_CHASSIS_PROFILE,
                            title: this.locResources.UNASSIGN_CHASSIS_PROFILE_TITLE,
                            onClick: dojo.hitch(this, this.unassignChassisProfile),
                            onKeypress: dojo.hitch(this, this.unassignChassisProfile)
                        });
                        this.actionMenu.addChild(this.deleteChassis);
                    }
                }
                else {
                    this.actionMenu.removeChild(this.deleteChassis);
                    this.deleteChassis = null;
                }
            } 
            else {
                this.actionMenu.removeChild(this.editPlaceholder);
                this.editPlaceholder = null;
                
                this.actionMenu.removeChild(this.deletePlaceholder);
                this.deletePlaceholder = null;
                
                this.actionMenu.removeChild(this.deleteServer);
                this.deleteServer = null;
                
                this.actionMenu.removeChild(this.deleteChassis);
                this.deleteChassis = null;
            }
                       
        },
        
        unassignServerProfile: function() {
            
            var selectedCount = this._placeholderTable.select.row.getSelected().length;
            var selectedIds = this._placeholderTable.select.row.getSelected();
            
            var deleteItems = new Array();
            var names = new Array();
            var htmlNames = new Array();
            dojo.forEach(selectedIds, dojo.hitch(this, function(id) {
                var item = this._placeholderTable.row(id, true).item();
                if (item) {
                    deleteItems.push(item);
                    var name = item.profileName.toString();
                    names.push(name);
                    var htmlName = name.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;");
                    htmlNames.push(htmlName);
                }
            }));
            
            var confirmedUnassign = function() {
                for (var i = 0; i < selectedCount; i++) {
                                                    
                    var selectedItem = this._placeholderTable.row(selectedIds[i], true).item();
                    var profID = selectedItem.profileId;
                    
                    var xhrPost = dojo.xhrPost({
                        url: this.REST_PATH + "profile/unassign/" + profID,
                        handleAs: "json",
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        sync: true,
                        preventCache: true,
                        load: dojo.hitch(this, function(response) {
                            // also refresh the table after the delete
                            dojo.publish("RefreshTable");
                        })
                    });
                    
                }                
                
            };
            

            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.UNASSIGN,
                        type: "Warning",
                        text: this.resources.CONFIRM_UNASSIGN,
                        info: this.locResources.DELETEINFO                    
                    });

                    confirmDialog.set("content", "<p>" + this.resources.UNASSIGN_PROFILE_FROM_SERVER.replace("%1", htmlNames.join(", ")) + "</p>" + this.resources.UNASSIGN_MSG);
                    confirmDialog.startup();
                    confirmDialog.confirm(confirmedUnassign, this);
                
                }));
 
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.UNASSIGN,
                        type: "Warning",
                        text: this.resources.CONFIRM_UNASSIGN,
                        info: this.locResources.DELETEINFO                    
                    });

                    confirmDialog.set("content", "<p>" + this.resources.UNASSIGN_PROFILE_FROM_SERVER.replace("%1", htmlNames.join(", ")) + "</p>" + this.resources.UNASSIGN_MSG);
                    confirmDialog.startup();
                    confirmDialog.confirm(confirmedUnassign, this);
                
                }));
                
            }

        },
        
        unassignChassisProfile: function() {
            
            var selectedCount = this._placeholderTable.select.row.getSelected().length;
            var selectedIds = this._placeholderTable.select.row.getSelected();
            
            var deleteItems = new Array();
            var names = new Array();
            var htmlNames = new Array();
            dojo.forEach(selectedIds, dojo.hitch(this, function(id) {
                var item = this._placeholderTable.row(id, true).item();
                if (item) {
                    deleteItems.push(item);
                    var name = item.profileName.toString();
                    names.push(name);
                    var htmlName = name.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;");
                    htmlNames.push(htmlName);
                }
            }));
            
            var confirmedUnassign = function() {
                for (var i = 0; i < selectedCount; i++) {
                                
                    var selectedItem = this._placeholderTable.row(selectedIds[i], true).item();
                    var profID = selectedItem.profileId;
                    
                    var xhrPost = dojo.xhrPost({
                        url: this.REST_PATH + "profile/unassign/" + profID,
                        handleAs: "json",
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        sync: true,
                        preventCache: true,
                        load: dojo.hitch(this, function(response) {
                            dojo.publish("RefreshTable");
                        })
                    });
                    
                }                
                
            };

            
            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.UNASSIGN,
                        type: "Warning",
                        text: this.resources.CONFIRM_UNASSIGN,
                        info: this.locResources.DELETEINFO                    
                    });

                    confirmDialog.set("content", "<p>" + this.resources.UNASSIGN_PROFILE_FROM_SERVER.replace("%1", htmlNames.join(", ")) + "</p>" + this.resources.UNASSIGN_MSG);
                    confirmDialog.startup();
                    confirmDialog.confirm(confirmedUnassign, this);
                
                }));
 
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.UNASSIGN,
                        type: "Warning",
                        text: this.resources.CONFIRM_UNASSIGN,
                        info: this.locResources.DELETEINFO                    
                    });

                    confirmDialog.set("content", "<p>" + this.resources.UNASSIGN_PROFILE_FROM_SERVER.replace("%1", htmlNames.join(", ")) + "</p>" + this.resources.UNASSIGN_MSG);
                    confirmDialog.startup();
                    confirmDialog.confirm(confirmedUnassign, this);
                
                }));
                
            }

        },
                    
        _getTableStructure : function() {
            var layout = new Array();
            
            layout.push({
                field: "formattedName",
                name: this.locResources.PLACEHOLDER,
                width: "20em",
                formatter: dojo.hitch(this, this._formatName),
                decorator: dojo.hitch(this, this._decorateName)
            });
            layout.push({
                field: "templateName",
                name: this.resources.TEMPLATE,
                width: "20em"
            });
            layout.push({
                field: "profileName",
                name:  this.resources.PROF,
                width: "20em"
            });
            layout.push({
                field: "description",
                name: this.resources.DESCRIPTION,
                formatter: dojo.hitch(this, this._formatDescription),
                decorator: dojo.hitch(this, this._decorateDescription)
            });
            
            return layout;
        },        
        goForeground: function(params) { //Page base function that fires when the page is brought to focus
            
            this._refreshTree();

            return true;
        }
    });
});