define(["dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/DualTableSelector",
        "dojo/text!./templates/DualTableSelector.html",
        "support/widgets/OutlinePane",
        "dojo/data/ItemFileWriteStore",
        "dijit/form/DropDownButton",
        "dijit/DropDownMenu",
        "dijit/MenuItem",
        "support/base/TableMixin"
        ], function(dojo, dijit, panel, mri, template) {

    return dojo.declare("support.panels.DualTableSelector", [ support.panels.Panel, support.base.TableMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in managing server profiles
        // for Configuration Patterns.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "DualTableSelector",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/DualTableSelector.css",
        
        // templateId: String
        // Value of entry to pre-select in this widget's template selector.
        templateId: null,
        rootNodeType: "chassis",
        init: false,
        
        singleLevel: false,
        
        _availTable : null,
        _selectTable : null,
        _toolbar: null,
        
        DEPLOY_NOT_SUPPORTED: "notSupported",

        // When this method is called, all variables
        // inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget
        // property variables defined in the template
        // HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);            
        },

        // postCreate() is called after
        // buildRendering(). This is useful to override
        // when
        // you need to access and/or manipulate DOM
        // nodes included with your widget.
        // DOM nodes and widgets with the
        // dojoAttachPoint attribute specified can now
        // be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in
        // buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            this._setupAvailableTable();
            this._setupSelectedTable();
            if (this._toolbar != null) {
                this._toolbar.placeAt(this._tableToolbar);
                this._toolbar.startup();
            }
        },

        /**
         * Function to move all selected table data to left side.  Used for the move all left button.
         * 
         */
        _moveAllLeft: function() {
            // Grab all selected(right) table data and "clean" it of unneeded fields
            var itemsToShift = this._cleanTableData(this._selectTable.store._arrayOfTopLevelItems);

            // Merge with table data on the other side and update with new table store
            var mergedData = this._mergeDataModels(itemsToShift, this._availTable.store._arrayOfTopLevelItems);
            var newStore = new dojo.data.ItemFileWriteStore({
                data: this._getValidJsonData(mergedData)
            });
            if (!("hasChildren" in newStore)) {
                newStore.hasChildren = function(id, item) {
                    return item && this.getValues(item, "children").length;
                };
            };            
            if (!("getChildren" in newStore)) {
                newStore.getChildren = function(item) {
                    return this.getValues(item, "children");
                };
            };
            this._availTable.setStore(newStore);
            this._availTable.model.clearCache();
            this._availTable.body.refresh().then(this._availTable.reexpandRows);
            
            // Explicitly replace selected table with an empty data store
            var emptyData = dojo.toJson({
                    "items": [],
                    "label": "name",
                    "identifier": "id"
            });
            var emptyStore = new dojo.data.ItemFileWriteStore({
                data: dojo.fromJson(emptyData)
            });
            this._selectTable.setStore(emptyStore);
            this._selectTable.model.clearCache();
            this._selectTable.body.refresh();
            // NOTE: No need to reexpand since _selectTable is empty now.

        },

        /**
         * Function to move all available table data to right side.  Used for the move all right button.
         * 
         */
        _moveAllRight: function() {
            // Grab all available(left) table data and "clean" it of unneeded fields
            var itemsToShift = this._cleanTableData(this._availTable.store._arrayOfTopLevelItems);

            // Merge with table data on the other side and update with new table store
            var mergedData = this._mergeDataModels(itemsToShift, this._selectTable.store._arrayOfTopLevelItems);
            var newStore = new dojo.data.ItemFileWriteStore({
                data: this._getValidJsonData(mergedData)
            });
            if (!("hasChildren" in newStore)) {
                newStore.hasChildren = function(id, item) { 
                    return item && this.getValues(item, "children").length;
                };
            };            
            if (!("getChildren" in newStore)) {
                newStore.getChildren = function(item) { 
                    return this.getValues(item, "children");
                };
            };
            this._selectTable.setStore(newStore);
            this._selectTable.model.clearCache();
            this._selectTable.body.refresh().then(this._selectTable.reexpandRows);
            
            // Explicitly replace available table with an empty data store.
            var emptyData = dojo.toJson({
                    "items": [],
                    "label": "name",
                    "identifier": "id"
            });
            var emptyStore = new dojo.data.ItemFileWriteStore({
                data: dojo.fromJson(emptyData)
            });
            this._availTable.setStore(emptyStore);
            this._availTable.model.clearCache();
            this._availTable.body.refresh();
            // NOTE: No need to reexpand since _availTable is empty now.
            
            // Denote first move has been made
            this.init = true;

        },

        /**
         * Move selected rows from selected(right) table to the available(left) table.  Used for the move left button.
         * 
         */
        _moveLeft: function() {
            //Grab selected rows from selected table
            var itemsToShift = this._generateOutgoingNodes(false);
            
            //Merge selected rows into other table, update table data store
            if(itemsToShift.length > 0){
                var mergedData = this._mergeDataModels(itemsToShift, this._availTable.store._arrayOfTopLevelItems);
                
                var tableData = dojo.toJson({
                    "identifier": "id",
                    "label": "name",
                    "items": mergedData
                }, true); 
                
                var newStore = new dojo.data.ItemFileWriteStore({
                    data: dojo.fromJson(tableData)
                });
                
                if (!("hasChildren" in newStore)) {
                    newStore.hasChildren = function(id, item) { 
                        return item && this.getValues(item, "children").length;
                    };
                };                
                if (!("getChildren" in newStore)) {
                    newStore.getChildren = function(item) { 
                        return this.getValues(item, "children");
                    };
                };
                
                this._availTable.setStore(newStore);
                this._availTable.model.clearCache();
                this._availTable.body.refresh().then(this._availTable.reexpandRows);
            }
            
            //Denote that first move has been made
            this.init = true;

        },

        /**
         * Move selected rows from available(left) table to the selected(right) table.  Used for the move right button.
         * 
         */
        _moveRight: function() {
            //Grab selected rows from available table
            var itemsToShift = this._generateOutgoingNodes(true);

            //Merge selected rows into other table, update table data store
            if(itemsToShift.length > 0){
                var mergedData = this._mergeDataModels(itemsToShift, this._selectTable.store._arrayOfTopLevelItems);
                
                var tableData = dojo.toJson({
                    "identifier": "id",
                    "label": "name",
                    "items": mergedData
                }, true);
                
                var newStore = new dojo.data.ItemFileWriteStore({
                    data: dojo.fromJson(tableData)
                });
                
                if (!("hasChildren" in newStore)) {
                    newStore.hasChildren = function(id, item) { 
                        return item && this.getValues(item, "children").length;
                    };
                };                
                if (!("getChildren" in newStore)) {
                    newStore.getChildren = function(item) { 
                        return this.getValues(item, "children");
                    };
                };
                
                this._selectTable.setStore(newStore);
                this._selectTable.model.clearCache();
                this._selectTable.body.refresh().then(this._selectTable.reexpandRows);
            }
        },

        _setupAvailableTable: function() {

            // summary:
            // Set up available targets table widget...
            // tags:
            // protected
            if (this._availTable) {               
               // Set timeout to ensure table data has enough time to load
               setTimeout(dojo.hitch(this, function() {
                   this._availTable.placeAt(this._availTableNode);
                   this._availTable.startup();
               }), 500);
            }

        },
        
        _setupSelectedTable: function() {

            // summary:
            // Set up selected targets table widget...
            // tags:
            // protected
            if (this._selectTable) {                
                // Set timeout to ensure table data has enough time to load
                setTimeout(dojo.hitch(this, function() {
                    this._selectTable.placeAt(this._selectTableNode);
                    this._selectTable.startup();
                }), 500);
            }

        },
        
        /**
         * Function to merge parent nodes and children from the one array into
         * root nodes and children of another array.  This is used to move selected entries
         * into the opposite table's data
         * 
         * newData - Array of root nodes to merge into oldData array
         * oldData - Array of existing root nodes to merge with
         * avail - Boolean: True if destination is available table(left side), false if selected table(right side)
         * 
         * @returns Array of merged root and children nodes
         */
        _mergeDataModels: function(newData, oldData){
            var i,j,k;
            var mergedData = this._cleanTableData(oldData); //Begin with old(current) table data
            
            for(i = 0; i < newData.length; i++){
                if(newData[i] != null && newData[i].type == this.rootNodeType){
                    var foundSameRoot = false;
                    for(j = 0; j < mergedData.length; j++){
                        //Found Same Root Node?
                        if(mergedData[j] && mergedData[j].children && mergedData[j].id.toString() == newData[i].id.toString()){
                            foundSameRoot = true;
                            //Merge Children To Existing Root
                            for(k = 0; k < newData[i].children.length; k++){
                                if(newData[i].children[k] != null && (!newData[i].children[k].deployStatus || newData[i].children[k].deployStatus.toString() != this.DEPLOY_NOT_SUPPORTED)){
                                    mergedData[j].children.push(newData[i].children[k]);
                                }
                            }
                            mergedData[j].children.sort(function(a, b) {
                                // Following converts various forms of "bays" values to numerical values
                                // so they may be computed and compared for sorting purposes.
                                // Example 1 - "6:1" and "6:2" will be converted to "6.1" and "6.2" for sorting purposes
                                // Example 2 - "9-10" and "1-2" will be converted to "9.10" and "1.2" for sorting purposes
                                // Example 3 - "5" and "4" will be left as is, for sorting purposes.
                                var aBay = a.bays[0].replace(":", ".").replace("-", ".");
                                var bBay = b.bays[0].replace(":", ".").replace("-", ".");
                                return aBay - bBay;
                            });                         
                        }
                    }
                    //New Root Node
                    if(!foundSameRoot){
                        mergedData.push(newData[i]);
                    }
                }
            }

            return mergedData;                        
        },
        
        /**
         * Filter out item data for a given list of root nodes.  This is necessary
         * because the TreeTable(and underlying Enhanced Grid) gives our root and child
         * nodes data it doesn't need.  This unnecessary data really adds up and affects table performance.
         * 
         * roots - Array of root nodes to "clean"
         * @return Array of same inputed root nodes, without extraneous data attached
         */
        _cleanTableData : function(roots){
            var cleanRoots = [];
            var i,j;

            if(roots){
                for(i = 0; i < roots.length; i++){
                    var newRoot = this._createParent(roots[i]);
                    if(roots[i].children){
                        for(j = 0; j < roots[i].children.length; j++){
                            var newChild = this._createChild(roots[i].children[j]);
                            newRoot.children.push(newChild);
                        }
                    }
                    cleanRoots.push(newRoot);
                }
            }
            return cleanRoots;
            
        },
        
        /**
         * Function to gather selected rows from desired table and generates an array of root nodes(w/ children).
         * 
         * moveRight - Boolean:  True gathering selected rows from available table(intended left to right movement).  False
         * False if gathering selected rows from selected table(intended right to left movement).
         * 
         * @returns Array of root nodes(w/ children) that were selected in desired table 
         */
        _generateOutgoingNodes: function(moveRight) {
            var moveRows,store;
            var numMoveRows = -1;
            var allRoots = []; //Array of all root nodes on beginning side
            var outgoingItems = []; //Array of root nodes to move to the otherside
            var deleteDeferred = []; //Array of root nodes to delete
            var i,j;
            var parent = null;
            var rootAdded = false;
            
            if(moveRight){
                numMoveRows = this._availTable.select.row.getSelected().length;
            }
            else{
                numMoveRows = this._selectTable.select.row.getSelected().length;
            }
            
            if(numMoveRows > 0){
                //Gather All Root Nodes
                if(moveRight){
                    moveRows = this._availTable.select.row.getSelected();
                    allRoots = this._availTable.store._arrayOfTopLevelItems;
                    store = this._availTable.store;
                }
                else{
                    moveRows = this._selectTable.select.row.getSelected();
                    allRoots = this._selectTable.store._arrayOfTopLevelItems;
                    store = this._selectTable.store;
                }
                                
                // Check for root nodes
                for(i = 0; i < numMoveRows; i++) {
                    var item;
                    var rootSelected;
                    if (moveRight) {
                        item = this._availTable.row(moveRows[i], true).item();
                        rootSelected = (item.parentId) ? false : true;
                        
                        // Add to expanded row array                        
                        (dojo.indexOf(this._selectTable._expandedItemIds, rowId) < 0) && this._selectTable._expandedItemIds.push(rowId);
                    
                    } else {
                        item = this._selectTable.row(moveRows[i], true).item();
                        rootSelected = (item.parentId) ? false : true;

                        // Add to expanded row array
                        (dojo.indexOf(this._availTable._expandedItemIds, rowId) < 0) && this._availTable._expandedItemIds.push(rowId);
                    }                    
                    // If root is selected, add it to the outgoing items
                    if (rootSelected) {
                        var newParentObj = this._cleanTableData([item])[0];
                        outgoingItems.push(newParentObj);
                        deleteDeferred.push(item);
                        rootAdded = true;
                    }
                }
                
                //Iterate Through Selected Rows, if no root nodes have already been added
                if (!rootAdded) {
                    for(i = 0; i < numMoveRows; i++) {
                        var item;
                        var rowId;
                        if (moveRight) {
                            item = this._availTable.row(moveRows[i], true).item();
                            rowId = (item.parentId) ? item.parentId[0] : item.id[0];
                            
                            // Add to expanded row array                        
                            (dojo.indexOf(this._selectTable._expandedItemIds, rowId) < 0) && this._selectTable._expandedItemIds.push(rowId);
                        
                        } else {
                            item = this._selectTable.row(moveRows[i], true).item();
                            rowId = (item.parentId) ? item.parentId[0] : item.id[0];
    
                            // Add to expanded row array
                            (dojo.indexOf(this._availTable._expandedItemIds, rowId) < 0) && this._availTable._expandedItemIds.push(rowId);
                        }
                        
                        if(!item){
                            continue;
                        }                        

                        for(j = 0; j < allRoots.length; j++){
                            if(allRoots[j].id.toString() == item.parentId.toString()){
                                parent = allRoots[j];
                                break;
                            }
                        }
                        
                        var parentAlreadyOutgoing = false;
                        var newChildObj = this._createChild(item);
                        for(j = 0; j < outgoingItems.length; j++){
                            //Parent object already outgoing
                            if(item.parentId.toString() == outgoingItems[j].id.toString()){
                                outgoingItems[j].children.push(newChildObj); 
                                parentAlreadyOutgoing = true;
                                break;
                            }
                        }
                        //Parent not yet outgoing
                        if(!parentAlreadyOutgoing){
                            var newRootObj = this._createParent(parent);
                            newRootObj.children.push(newChildObj);
                            outgoingItems.push(newRootObj);
                        }
                        store.deleteItem(item);
                    }
                }
                
                // Delete deferred parent nodes
                for(i = 0; i < deleteDeferred.length; i++) {
                    store.deleteItem(deleteDeferred[i]);
                }
                
                // Delete empty parent nodes
                if (!this.singleLevel) {
                    for(i = 0; i < store._arrayOfTopLevelItems.length; i++) {
                        var item = store._arrayOfTopLevelItems[i];                    
                        if (!item.children) {
                            store.deleteItem(item);
                        }
                    }
                }
                               
                // Re-expand source table rows after deleting items from store.
                if (moveRight) {
                    this._availTable.model.clearCache();
                    this._availTable.body.refresh().then(this._availTable.reexpandRows);
                }
                else {
                    this._selectTable.model.clearCache();
                    this._selectTable.body.refresh().then(this._selectTable.reexpandRows);
                }
            }
        
            return outgoingItems;
        },
        
        /**
         * Function to create generic child object from existing child object.  Used to get rid of unnecessary data fields 
         * in original child object.
         * childObj: Existing child object, must have fields listed below.
         * schedule: Field used to populate new child object's schedule field(immediate/defer).  If null, takes existing
         * childObj schedule field.
         */
        _createChild : function(childObj, schedule){
            return {
                "access": childObj.access,
                "schedule": (schedule) ? schedule : childObj.schedule,
                "type": childObj.type,
                "name": childObj.name,
                "id": childObj.id,
                "bays": childObj.bays,
                "parentId": childObj.parentId,
                "formFactor": childObj.formFactor,
                "location": childObj.location,
                "uuid": childObj.uuid,
                "firmwareLevel": childObj.firmwareLevel,
                "deployStatus": childObj.deployStatus,
                "deployMsg": childObj.deployMsg
            };
        },
        
        /**
         * Function to create generic parent(root) object from existing parent object.  Used to get rid of unnecessary data fields 
         * in original parent object.
         * parentObj: Existing child object, must have fields listed below.
         * 
         */
        _createParent : function(parentObj){
            return {
                "access": parentObj.access,
                "type": parentObj.type,
                "name": parentObj.name,
                "id": parentObj.id,
                "uuid": parentObj.uuid,
                "firmwareLevel": parentObj.firmwareLevel,
                "deployStatus": parentObj.deployStatus,
                //"expandMe": parentObj.expandMe,
                "children": []
            };                   
        },
        
        _getValidJsonData: function(jsonData) {
            //if any part of the file is not valid, return the default item
            if(jsonData == null || jsonData.length == 0){
                return {"items": []};
            }
            else {
                return {"items": jsonData};
            }
        }
        
    });
});