define([
        "dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/EnhancedTable",
        "dojo/text!./templates/EnhancedTable.html",
        "dijit/form/ValidationTextBox",
        "dijit/form/NumberTextBox",
        "dojo/data/ItemFileWriteStore",
        "dojo/data/ItemFileReadStore",
        "dojox/grid/cells/_base",
        "dijit/form/Button",
        "dojox/grid/EnhancedGrid",
        "dojox/grid/enhanced/plugins/DnD"
    ], function (dojo, dijit, panel, mri, template) {

	dojo.declare("support.panels.EnhancedTable", [ support.panels.Panel ], {
	    // templateString: [protected] String
	    // This widget's HTML template, provided.
	    templateString: template,
	        
	    // locResources: [protected] Object
	    // This widget's National Language Source (NLS) object.
	    locResources: mri, 
	    
		widgetsInTemplate : true,
		baseClass: "EnhancedTable",
			
		jsonData: null,
		simpleStore: null,
		simpleLayout: null,
		simpleGrid: null,
		
		min: 0, 
		max: null,
		minchars: 0,
		maxchars: null,
		label: null,
		addTitle: null,
		removeTitle: null,
		addPosition: "left", //left or below
		regExp: ".*",
		invalidMessage: "",
		placeHolder: "",
		
		 // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/EnhancedTable.css",

		postMixInProperties : function() {
			this._connectDataModels();
			this.inherited(arguments);
			this.addTitle = this.resources.ADD;
			this.removeTitle = this.resources.REMOVE;
		},

		postCreate : function(){	
			this.inherited(arguments);
			this.simpleGrid = new dojox.grid.EnhancedGrid({
        		store: this.simpleStore,
        		structure: this.simpleLayout,
        		autoWidth: false,
        		//autoHeight: 5,
        		//height: "150px",
        		singleClickEdit: true,
        		columnReordering: false,
        		//rowSelector: "20px",
        		selectionMode: "single",
                canSort: function(){return false;},
                plugins: {
                    selector: {
                        cell: false,
                        col: false
                    }
                    // do not enable dnd plugin; drag-and-drop was
                    //   intentionally disabled due to issues combining mouse
                    //   out handler with data store refactoring
                }        		
        	}, this._simpleTable);
            this.simpleStore.fetch({
                onBegin: dojo.hitch(this, function(size, request) {                    
                    var adjustedHeight = size * 50;
                    var height = adjustedHeight + "px";
                    this.simpleGrid.set("height", height);                    
                }),
                start: 0,
                count: 0
            });			
			this.simpleGrid.startup();
			
        	var addIcon = 
        		"<div id=\"" + this.id + "_addIcon\" " +
        		"class=\"spAddRowIcon_v2\" " +
				"onClick=\"dijit.byId('" + this.id + "')._addRow()\"" +
				"title=\"" + this.addTitle + "\"" +
				"style=\"display:inline-block;\"></div>";
		
        	if(this.addPosition == "below")
        		this.addIconBelow.innerHTML = addIcon;
        	else //left by default
        		this.addIconLeft.innerHTML = addIcon;
        	
        	if(this._getValidJsonData(this.jsonData).items.length == 0){
        		this.table.setAttribute("style", "display:none");
        	}
        	this._setIcons();
		},
		
		startup: function(){
            //this._resetItems();
            this.inherited(arguments);
        },
        
        _resetItems : function(){
            var grid = this.simpleGrid;
            if(grid != null){
                var store = grid.store;
                var topLevelItems = store._arrayOfTopLevelItems;
                grid.setItems(topLevelItems);
                if(topLevelItems.length < this.min){
                    for(var i=topLevelItems.length; i<this.min; i++){
                        this._addRow();
                    }
                }
            }           
        },
		
		//Sets the label attribute and shows next to the table
		_setLabelAttr : function(l){
		 	this.label = l;
		    this.labelSpan.innerHTML = this.label;
		},
	    
	    _connectDataModels : function(){
        	this.simpleStore = new dojo.data.ItemFileWriteStore({
        		data: this._getValidJsonData(this.jsonData)
        	});
        	if (this.simpleLayout == null) {
                this.simpleLayout = [ 
                {
                    field: "field",
                    name: " ",
                    width: "100%",
                    title: "This is a test.",
                    editable: true,
                    noresize: true,
                    formatter: this._formatField                   	
                },{
                    field: "icons",
                    name: " ",
                    width: "30px",
                    title: "This is a test.",
                    noresize: true,
                    formatter: this._formatIcons
                }];
            }
        },
        
        //Formats the icons (enabled/disabled)
        _formatIcons : function(value, rowIndex, cell){
        	var st = this.grid.getParent();
        	var store = this.grid.store;
        	var item = store._arrayOfTopLevelItems[rowIndex];
        	// there used to be a drag and drop icon here too but that was
        	//   removed when we intentionally removed drag-and-drop
        	//   functionality
        	return st._getRemoveIcon(store.getValue(item, "icons"), rowIndex);
        },
        
        //Returns an enabled/disabled remove icon
        _getRemoveIcon : function(enabled, rowIndex){
            if(enabled){
                return "<div class=\"spDeleteIcon_v2\"" +
                "onClick=\"dijit.byId('" + this.id + "')._removeRow('" + rowIndex + "')\"" +
                "title=\"" + this.removeTitle + "\" style=\"display:inline-block;\"></div>";
            }
            return "<div class=\"spDeleteInactiveIcon_v2\" style=\"display:inline-block;\"></div>";
        },
        
        //Format "field": given format
        _formatField : function(value, rowIndex, cell){
            var grid = this.grid;
            var gridID = grid.id != null ? grid.id : "";
            var textboxID = gridID + "_textField_" + rowIndex;
        	var st = this.grid.getParent();
        	var store = this.grid.store;
        	var item = store._arrayOfTopLevelItems[rowIndex];
        	var errorMessage = null;
        	var control = dijit.form.ValidationTextBox({
        	    id: textboxID,
        		value: value,
        		required: true,
        		trim: true,
        		style:"width:100%",
        		onChange: function(newValue){
        		    // note that IE will not fire this event when characters
        		    //   are added to the field; we need to add the value to
        		    //   the store explicitly from an onCellMouseOut event
        			st._setStoreValue(item, "field", newValue);
        		},
        		//regExp: "^([0-9A-F]{2}[:-]){7}([0-9A-F]{2})$",
    			//invalidMessage: "To boot from 1st discovered target, enter 00:00:00:00:00:00:00:00",
    			placeHolder: st.placeHolder,
    			validator : function(value){
    			    if(value.length == 0){
//    			        errorMessage = "Error";
    			        errorMessage = st.resources.REQUIRED;
    			        return false;
    			    }
    			    if(st.minchars != null && st.maxchars != null){
                        if(value.length < parseInt(st.minchars) || value.length > parseInt(st.maxchars)){
//                            errorMessage = "Error";
                            errorMessage = st.resources.INVALID_BETWEEN.replace("%1", st.minchars).replace("%2", st.maxchars);
//                            errorMessage = st.locResources.BETWEEN + " " + st.minchars + " " + st.locResources.AND 
                                        + st.maxchars + " " + st.locResources.CHAR;
                            return false;
                        }
                    }
    			    if(st.minchars != null && st.maxchars == null){
                        if(value.length < parseInt(st.minchars)){
  //                          errorMessage = "Error";
                            errorMessage = st.locResources.GREATER.replace("%1", st.minchars);
//                            errorMessage = st.locResources.GREATER + " " + st.minchars + " " + st.locResources.CHAR;
                            return false;
                        }
                    }    			    
			    	errorMessage = st.resources.INVALID;
			    	var reg = new RegExp(st.regExp);
			    	return reg.test(value);
    			}
        	});
        	control._destroyOnRemove = true;
        	
        	// show (and hide) tooltip text for error messages
        	dojo.connect(grid,"onCellMouseOver", function(e){
        	    var controlValue = store.getValue(item, "field");
        	    if(e.cellIndex == 0 && e.rowIndex == rowIndex && !control.validator(controlValue)){
        	        require(["dijit/Tooltip", "dojo/dom", "dojo/on", "dojo/mouse", "dojo/domReady!"], function(Tooltip, dom, on, mouse) {
        	            // show the tooltip
        	            Tooltip.show(errorMessage, e.cellNode);
        	            // when the mouse leaves the text field, hide the
        	            //   tooltip
        	            on(e.cellNode, mouse.leave, function(){
        	                Tooltip.hide(e.cellNode);
        	            });
        	            // equivalent to blur; handles dismissing tooltip when
        	            //   user hits TAB key to exit text field
        	            on(e.cellNode, "focusout", function(){
                            Tooltip.hide(e.cellNode);
                        });
        	        });
        	    }
        	});
          
            // update the data store when the user moves the mouse out of the
            //   text field; this is needed for IE because the onChange
            //   handler will not fire until the user hits TAB, ENTER, or
            //   clicks on the button area to the right of the field
            dojo.connect(grid,"onCellMouseOut", function(e){
                if(e.rowIndex == rowIndex) { // only do this for the row the event corresponds to
                    // e.cell.getValue works for IE but not Firefox; this is
                    //   ok because Firefox saves the value in the onChange
                    //   handler
                    var value = e.cell.getValue(rowIndex);
                    if(value != null) {
                        var itemToSet = grid.store._arrayOfTopLevelItems[rowIndex];
                        st._setStoreValue(itemToSet, "field", value);
                    }
                }
            });
          
//           //sets value in store as cell is being edited
//        	/*dojo.connect(cell, "formatEditing", function(inDatum, inRowIndex){
//        		var tableID = this.grid.getParent().id;
//        		var gridID = this.grid.id;
//				this.needFormatNode(inDatum, inRowIndex);
//				return  "<input class=\"dojoxGridInput\" type=\"text\" " +
//						"onkeyup=\"dijit.byId('" + tableID + "')._setStoreValue(dijit.byId('" + gridID + "').store._arrayOfTopLevelItems[" + inRowIndex + "] , \"field\", value)\" " +
//						"value=\"" + inDatum + "\">";
//			});*/
        	return control;
        },
        
        //Determines if a value is a valid field
        _isValidField : function(value){
        	if(value.length == 0)
        		return false;
        	var reg = new RegExp(this.regExp);
        	return reg.test(value);
        },
        
        //Sets the value in the store
        _setStoreValue : function(item, type, value){
        	if(item != null){
        		this.simpleGrid.store.setValue(item, type, value);
        		dojo.publish("rowadded", "");
        	}
        },
        
        //Adds a row to the end of the grid    
        _addRow : function(){
        	this.table.setAttribute("style", "display:block; width:290px; overflow-y:auto; overflow-x:none;");
        	this.simpleGrid.store.newItem({field: "", id: this.simpleGrid.rowCount, icons: true});
            this.simpleStore.fetch({
                onBegin: dojo.hitch(this, function(size, request) {    
                    this.simpleGrid.set("autoHeight", 5);
                }),
                start: 0,
                count: 0
            });            
        	this._setIcons();
        	//alert("Row added...");
        },
        
        //Removes a row from the grid
        _removeRow : function(row){
        	var remove = confirm(this.resources.DELETE_CONFIRM.replace("%1", parseInt(row)+1));
        	if(remove){
        		var grid = this.simpleGrid;
        		var store = grid.store;
        		var items = grid.selection.getSelected();
        		
        		dojo.forEach(items, function(selectedItem){
        		    grid.store.deleteItem(selectedItem);
        		});
                this.simpleStore.fetch({
                    onBegin: dojo.hitch(this, function(size, request) {                    
                        this.simpleGrid.set("autoHeight", 5);                        
                    }),
                    start: 0,
                    count: 0
                });
        		
        		if(store._arrayOfTopLevelItems == 0){
        			this.table.setAttribute("style", "display:none");
        		}
        		this._setIcons();
        	}
        },
        
        getStoreAsJson : function(){
        	var grid = this.simpleGrid;
        	var store = grid.store;
        	var topLevelItems = store._arrayOfTopLevelItems;        	
        	
        	var storeAsJson = 
        		"{\n" +
        		"	\"items\" : [\n";
        	for(var i=0; i<topLevelItems.length; i++){
        		var item = topLevelItems[i];
        		var field = store.getValue(item, "field");
        		storeAsJson +=
        		"		{\n" +
        		"			\"field\": \"" + field + "\"\n" +
        		"		}";
        		if(i != topLevelItems.length-1)
					storeAsJson += ",";
        		storeAsJson += "\n";
			}
        	storeAsJson += 
				"	]\n" +
				"}";
        	return storeAsJson;
        },
        
        _setIcons : function(){
            var max = this.max;
            var min = this.min;
            var grid = this.simpleGrid;
            var store = grid.store;
            var addIcon = document.getElementById(this.id + "_addIcon");
            var st = this;
            var topLevelItems = store._arrayOfTopLevelItems;
            for(var i=0; i<topLevelItems.length; i++){
                var item = topLevelItems[i];
                if(min != null && topLevelItems.length <= min){
                    //disable remove and drag
                    store.setValue(item, "icons", false);
                }
                else if(max != null && topLevelItems.length >= max && addIcon != null){
                    //disable add
                    addIcon.setAttribute("class", "spAddRowInactiveIcon_v2");
                    addIcon.onclick = function(e){};
                    addIcon.setAttribute("title", "");
                }
                else{
                    //enable drag, add, remove
                    store.setValue(item, "icons", true);
                    if(addIcon != null){
                    	addIcon.setAttribute("class", "spAddRowIcon_v2");
                        addIcon.onclick = function(e){
                            st._addRow();
                        };
                        addIcon.setAttribute("title", this.addTitle);
                    }
                }
            }
        },
        
        _getValidJsonData : function(jsonData){
        	//if any part of the file is not valid, return the default item
        	if(jsonData == null ||
        	   jsonData.items == null ||
        	   jsonData.items[0].field == null ||
        	   jsonData.items[0].icons == null){
        	   jsonData = {
		    			"items": [
		    			]
		    		};
        	}
        	return jsonData;
        },
        
        isValid : function(){
        	var st = this;
        	var grid = this.simpleGrid;
    		var store = grid.store;
        	var valid = true;
        	//at least one item
        	//all textboxes valid
        	var gotItems = function(items, request){
        		if(items == null || items.length == 0) {
        			valid = false;
        		}
        		else{
        			for(var i=0; i<items.length; i++){
        				var item = items[i];
        				var field = item.field[0];
        				if( !st._isValidField(field)){
        					valid  = false;
        				}
        			}
        		}
        	};
        	store.fetch({onComplete:gotItems});
        	return valid;
        }
	});
});     	