define([ 
        "dojo", 
        "dijit", 
        "support/panels/Panel", 
        "dojo/i18n!./nls/DetailsDependentTable", 
        "dojo/text!./templates/DetailsDependentTable.html",
        "dojo/data/ItemFileWriteStore",
        "support/base/TableMixin"        
        ],function(dojo, dijit, panel, mri, template) {

    return dojo.declare("support.panels.EthPoolDetailsDependent", [ support.panels.Panel , support.base.TableMixin], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in managing server profiles
        // for Configuration Patterns.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "DetailsDependentTable",
        
        mode: "", // Current Context Mode
        
        _detailsTable: null,
        
        profileLayout: {            /* Table View Options */
            flatView: true, // Flat Table View
            chassisView: true, // By Chassis View
            groupView: true, // By Group View
            templateView: true // By Template View
            /* Toolbar Buttons and Menus */

            /* "More" menu options */

            /* Table Columns To Display */
        },
        
        vendor:"", //Eth or Fibre pool vendor
        templateIDNum:"",   //Category Template ID   
        templateType: "",   //Template Type
        
        // ID prefix used for table
        gridID: "EthPoolDetailsDependentTable",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/DetailsDependentTable.css",

        // When this method is called, all variables
        // inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget
        // property variables defined in the template
        // HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after
        // buildRendering(). This is useful to override
        // when
        // you need to access and/or manipulate DOM
        // nodes included with your widget.
        // DOM nodes and widgets with the
        // dojoAttachPoint attribute specified can now
        // be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in
        // buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);

            // Setup table widget...
            var tableStore = this._retrieveStore();
            var tableStructure = this._getTableStructure(); // Get structure for current view

            this._detailsTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.gridID, true);
  
            setTimeout(dojo.hitch(this, function() {
    	        this._detailsTable.placeAt(this.detailsTable);
	            this._detailsTable.startup();
            }), 500);
  
                        
            // Connect handlers to reset table capabilities
            dojo.connect(this._detailsTable.select.row, "onSelectionChange", this, this.setTableCapabilities); // Fires on row selection
            dojo.connect(this._detailsTable, "setStore", this, this.setTableCapabilities); // Fires on store reset            
            
        },
        
        // Connect to data source and create models for
        // TreeTable widgets.
        _retrieveStore: function() {
                                                            
                    var ranges = "";
                    
                    var xhrGet = dojo.xhrGet({
                        url: this.REST_PATH + "addresspool/getdetails/ETHERNET/"+this.vendor,
                        handleAs: "json",
                        sync: true,
                        preventCache: true,
                        load: function(response) {                            
                            ranges = dojo.toJson(response.items);
                        }
                    });
    
                    var tableStore = "";
                    
                    if(ranges == "" || ranges == null){

                        tableStore = new dojo.data.ItemFileWriteStore({
                            data: dojo.fromJson("{ \"items\" : [], \"label\" : \"id\", \"identifier\" : \"id\" }")
                        });                           

                    }else{
                        
                        tableStore = new dojo.data.ItemFileWriteStore({
                            data: dojo.fromJson("{ \"items\" : " + ranges + ", \"label\" : \"id\", \"identifier\" : \"id\" }")
                        });
                    }                                        
                            
                    return tableStore;
                    
        },
        // Re-render Tree table with new columns/data
        // presentation
        _reRenderTable: function() {
            // 99275: Get function from tree table widget
        },
        
        // Columns to be displayed on table
        _getTableStructure: function() {
            var cols = new Array();

               cols.push({
                   field: "range",
                   name: this.locResources.ADDRESS,
                   width: "18em",
                   title: ""});
               
               
               cols.push({
               field: "pattern",
               name: this.resources.SERVER_TEMP,
               width: "20em",
               title: ""});
                   
               cols.push({
               field: "profile",
               name: this.resources.SERVER_PROFILE,
               width: "20em",
               title: ""});
               
               cols.push({
               field: "chassis",
               name: this.resources.CHASSIS,
               width: "12em",
               title: ""});
                   
               
               cols.push({
               field: "bay",
               name: this.resources.BAY,
               width: "8em",
               title: ""});
                   
               cols.push({
                   field: "system",
                   name: this.locResources.SYSTEM,
                   title: ""});

       
               return cols;
        },

        // Get number of selected rows in tree table
        _getSelectedRowCount: function() {
            return this._detailsTable.select.row.getSelected().length;
        },

        // Get row objects for selected rows
        _getSelectedRows: function() {
            return this._detailsTable.select.row.getSelected();
        },
        // Set Table Capabilities
        setTableCapabilities: function() {
        }
    });
});