define([ "dojo",
         "dijit", 
         "support/panels/Panel", 
         "dojo/i18n!./nls/ExportTable", 
         "dojo/text!./templates/ExportTable.html",
         "dijit/layout/LayoutContainer", 
         "dijit/layout/ContentPane", 
         "support/base/NextGenUIMixin", 
         "dojo/data/ItemFileWriteStore",
         "support/base/TableMixin",
         "dijit/form/Button",
         "dijit/Menu",
         "dijit/MenuItem" ], function(
        dojo, dijit, panel, mri, template) {

    return dojo.declare("support.panels.ExportTable", [ support.panels.Panel, support.base.NextGenUIMixin, support.base.TableMixin ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "ExportTable",

        // ID prefix used for tables
        selectGridID: "ExportTable",
        select2GridID: "Export2Table",
        
        // Patterns referenced by other patterns
        _referenced: new Array,
        
        // All IP pools
        _ipPools: new Array,
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/ExportTable.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
        },
        
        createTreeTable: function(selectedPatterns, referenced, ipPools) {
            this._referenced = referenced;
            this._ipPools = ipPools;
            var autoRun = false;
            
            var selected = this._formatTableData(selectedPatterns, autoRun); //Format selected resources
            var selectedStore = this._retrieveSelectedStore(selected);
            var selectedStructure = this._getTableStructure();
            this._selectedTable = this.getTable(selectedStore, selectedStructure, this.resources.NO_DATA, this.selectGridID, true);
            
            autoRun = true;
            var auto = this._formatTableData(this.autoInclude, autoRun); //Format automatically included resources
            var autoSelectedStore = this._retrieveSelectedStore(auto);
            this._autoIncludedTable = this.getTable(autoSelectedStore, selectedStructure, this.resources.NO_DATA, this.select2GridID, true);
            
            setTimeout(dojo.hitch(this, function(){
                this._startTables();
            }),500);
        },
        
        _startTables: function(){
            // tags:
            // protected
            this._selectedTable.placeAt(this.exportTable);
            this._selectedTable.startup();
            
            this._autoIncludedTable.placeAt(this.exportTable2);
            this._autoIncludedTable.startup();
        },
        
        _selExpandAll: function(){
            // tags:
            // protected
            this._selectedTable.expandAll();
        },
        
        _autoExpandAll: function(){
            // tags:
            // protected
            this._autoIncludedTable.expandAll();
        },
        
        _selCollapseAll: function(){
            // tags:
            // protected
            this._selectedTable.collapseAll();
        },
        
        _autoCollapseAll: function(){
            // tags:
            // protected
            this._autoIncludedTable._collapseAll();
        },
        
        _retrieveSelectedStore: function(patterns) {
            // tags:
            // protected
            // Connect to data source and create store
            var selectedStore = "";
            if(patterns){
                selectedStore = new dojo.data.ItemFileWriteStore({
                    data: dojo.fromJson(patterns)
                });
                if (!("hasChildren" in selectedStore)) {
                    selectedStore.hasChildren = function(id, item) { // Need hasChildren function for
                                                                  // TreeTable.
                        return item && this.getValues(item, "children").length;
                    };
                }
                
                if (!("getChildren" in selectedStore)) {
                    selectedStore.getChildren = function(item) { // Need getChildren function for
                                                              // TreeTable.
                        return this.getValues(item, "children");
                    };
                }
                
            }
            return selectedStore;
        },

        _getTableStructure: function() {
            // tags:
            // protected           
            
            var layout = new Array();
            layout.push({
                field: "name",
                name: this.resources.NAME,
                width: "35%",
                decorator: dojo.hitch(this, this._decorator),
                formatter: dojo.hitch(this, this._formatName),
                title: this.resources.TEMP_NAME,
                comparator: function(v1, v2){
                     return v1.localeCompare(v2);
                }
            });
            layout.push({
                field: "type",
                name: this.resources.TYPE,
                width: "10%",
                formatter: dojo.hitch(this, this._formatType),
                title: this.resources.TYPE
            });
            layout.push({
                field: "description",
                name: this.resources.DESCRIPTION,
                width: "auto",
                decorator: dojo.hitch(this, this._decorator),
                formatter: dojo.hitch(this, this._formatDescription),
                title: this.resources.DESCRIPTION
            });
            
            
            return layout;
        },
        
        // Decorate name/description
        _decorator: function(item) {
            return "<span title=\"" + item + "\">" + item + "</span>";
        },
        
        // Set Table Formatters
        _formatName: function(item) {
            // tags:
            // protected            
            var value = (item && item.name) ? item.name : "";
            value = this.encodeForHtml(value);
            return value;
        },
        
        _formatType: function(item) {
            // tags:
            // protected
            if(item.type.toString() == "root") {
                return "";
            } else {
                return item.type.toString();
            }
        },
        
        _formatDescription: function(item) {
            // tags:
            // protected
            if (!item || !item.description) {
                return "";
            }
            var value = this.decodeSpecialCharacters(item.description);
            value = this.encodeForHtml(value);
            return value;
        },
        
        _formatTableData: function(selectedNodes, autoRun) {
            // tags:
            // protected
            // Create JSON for the resources the user selects from the tree
            
            //type: Server
            var ServerPatterns = new Array;
            var SystemInfoPatterns = new Array;
            var SERVManagementIntPatterns = new Array;
            var PowerSchedCappingPatterns = new Array;
            var PerformanceRecoveryPatterns = new Array;
            var DevicesIOPortsPatterns = new Array;
            var FibreChannelBootTargetPatterns = new Array;
            var AdapterPatterns = new Array;
            var PortPatterns = new Array;
            //var ExtendedPort = new Array;
            var FailoverPolicies = new Array;
            
            //type: Chassis
            var ChassisPatterns = new Array;
            var ChassisInfoPatterns = new Array;
            var CHASManagementIntPatterns = new Array;
            var PowerAcousticsPatterns = new Array;
            var UsersSecPatterns = new Array;
            var ChassisIOModulePatterns = new Array;
            var NodePatterns = new Array;
            
            //type: Pool
            var IPAddressPools = new Array;
            
            var newServerChild = dojo.hitch(this, function(res) {
                if(res.item){
                    var child = {
                            id: res.item.id,
                            name: res.item.name,
                            type: this.resources.SERVER,
                            description: res.item.description
                    };
                    return child;
                }else{
                    var child = {
                            id: res.id,
                            name: res.name,
                            type: this.resources.SERVER,
                            description: res.description
                    };
                    return child;
                }
                
            });
            
            var newChassisChild = dojo.hitch(this, function(res) {
                if(res.item){
                    var child = {
                            id: res.item.id[0],
                            name: res.item.name[0],
                            type: this.resources.CHASSIS,
                            description: res.item.description[0]
                    };
                    return child;
                }else{
                    var child = {
                            id: res.id,
                            name: res.name,
                            type: this.resources.CHASSIS,
                            description: res.description
                    };
                    return child;
                }
                
            });
            this.autoInclude = new Array();

            selectedNodes.forEach(dojo.hitch(this, function(node) {
                
                if(node.item){
                    typeValue = node.item.type[0];
                }else{
                    typeValue = node.type;
                }
                if (typeValue == "Server") {
                    this._getDependentPatterns(node.item.id[0], selectedNodes);
                    ServerPatterns.push(newServerChild(node));
                }
                else if (typeValue == "SystemInfo") {
                    SystemInfoPatterns.push(newServerChild(node));
                }
                else if (typeValue == "Management") {
                    if(!autoRun){
                        this._getRequiredIPPools(node.item.id[0], selectedNodes);
                    }
                    SERVManagementIntPatterns.push(newServerChild(node));
                }
                else if (typeValue == "Power") {
                    PowerSchedCappingPatterns.push(newServerChild(node));
                }
                else if (typeValue == "PerformanceRecovery") {
                    PerformanceRecoveryPatterns.push(newServerChild(node));
                }
                else if (typeValue == "DevicesAndIOPorts") {
                    DevicesIOPortsPatterns.push(newServerChild(node));
                }
                else if (typeValue == "FibreChannel") {
                    FibreChannelBootTargetPatterns.push(newServerChild(node));
                }
                else if (typeValue == "Adapter") {
                    AdapterPatterns.push(newServerChild(node));
                }
                else if (typeValue == "PortVirtualization") {
                    PortPatterns.push(newServerChild(node));
                }
                /*else if (typeValue == "ExtendedPort") {
                    ExtendedPort.push(newServerChild(node));
                }*/
                else if (typeValue == "failover") {
                    FailoverPolicies.push(newServerChild(node));
                }
                else if(typeValue == "CMM"){
                    this._getDependentPatterns(node.item.id[0], selectedNodes);
                    ChassisPatterns.push(newChassisChild(node));
                }
                else if(typeValue == "cmmInfo"){
                    ChassisInfoPatterns.push(newChassisChild(node));
                }
                else if(typeValue == "cmmManagement"){
                    if(!autoRun){
                        this._getRequiredIPPools(node.item.id[0], selectedNodes); //May have associated IP pool
                    }
                    CHASManagementIntPatterns.push(newChassisChild(node));
                }
                else if(typeValue == "cmmPower"){
                    PowerAcousticsPatterns.push(newChassisChild(node));
                }
                else if(typeValue == "cmmUsers"){
                    UsersSecPatterns.push(newChassisChild(node));
                }
                else if(typeValue == "cmmIOModule"){
                    if(!autoRun){
                        this._getRequiredIPPools(node.item.id[0], selectedNodes); //May have associated IP pool
                    }
                    ChassisIOModulePatterns.push(newChassisChild(node));
                }
                else if(typeValue == "cmmNode"){
                    if(!autoRun){
                        this._getRequiredIPPools(node.item.id[0], selectedNodes); //May have associated IP pool
                    }
                    NodePatterns.push(newChassisChild(node));
                }
                else if(typeValue == "IPAddressPool"){
                    if(node.item){
                        IPAddressPools.push({
                            id: node.item.id[0],
                            name: node.item.name[0],
                            type: "Pool",
                            description: node.item.description[0]
                        });
                    }else{
                        IPAddressPools.push({
                            id: node.id,
                            name: node.name,
                            type: "Pool",
                            description: node.description
                        });
                    }
                    
                }
            }));
            
            //Push selected resources onto array so the table will create the correct format
            var allSelected = new Array;
            if(ServerPatterns.length>0){
                allSelected.push({
                    id: "st",
                    name: this.locResources.SERVER_PATTERNS,
                    type: this.resources.SERVER,
                    children: ServerPatterns
                });
            }
            if(SystemInfoPatterns.length>0){
                allSelected.push({
                    id: "sit",
                    name: this.locResources.SYSTEM_INFO_PATTERNS,
                    type: this.resources.SERVER,
                    children: SystemInfoPatterns
                });
            }
            if(SERVManagementIntPatterns.length>0){
                allSelected.push({
                    id: "mit",
                    name: this.locResources.MANAGEMENT_INTERFACE_PATTERNS,
                    type: this.resources.SERVER,
                    children: SERVManagementIntPatterns
                });
            }
            if(PowerSchedCappingPatterns.length>0){
                allSelected.push({
                    id: "psct",
                    name: this.locResources.PSACP,
                    type: this.resources.SERVER,
                    children: PowerSchedCappingPatterns
                });
            }
            if(PerformanceRecoveryPatterns.length>0){
                allSelected.push({
                    id: "part",
                    name: this.locResources.PERF_RECOVERY_PATTERNS,
                    type: this.resources.SERVER,
                    children: PerformanceRecoveryPatterns
                });
            }
            if(DevicesIOPortsPatterns.length>0){
                allSelected.push({
                    id: "daipt",
                    name: this.locResources.DEV_IO_PATTERNS,
                    type: this.resources.SERVER,
                    children: DevicesIOPortsPatterns
                });
            }
            if(FibreChannelBootTargetPatterns.length>0){
                allSelected.push({
                    id: "fcbtt",
                    name: this.locResources.FIBRE_BOOT_PATTERNS,
                    type: this.resources.SERVER,
                    children: FibreChannelBootTargetPatterns
                });
            }
            if(AdapterPatterns.length>0){
                allSelected.push({
                    id: "at",
                    name: this.locResources.ADAPTER_PATTERNS,
                    type: this.resources.SERVER,
                    children: AdapterPatterns
                });
            }
            if(PortPatterns.length>0){
                allSelected.push({
                    id: "pvt",
                    name: this.locResources.PORT_PATTERNS,
                    type: this.resources.SERVER,
                    children: PortPatterns
                });
            }
            /*if(ExtendedPort.length>0){
                allSelected.push({
                    id: "ept",
                    name: this.locResources.PORTTemplate,
                    type: this.resources.SERVER,
                    children: ExtendedPort
                });
            }*/
            if(FailoverPolicies.length>0){
                allSelected.push({
                    id: "fp",
                    name: this.locResources.FAILOVER_POLICIES,
                    type: this.resources.SERVER,
                    children: FailoverPolicies
                });
            }
            if(ChassisPatterns.length>0){
                allSelected.push({
                    id: "cmm",
                    name: this.locResources.CHASSIS_PATTERNS,
                    type: this.resources.CHASSIS,
                    children: ChassisPatterns
                });
            }
            if(ChassisInfoPatterns.length>0){
                allSelected.push({
                    id: "ci",
                    name: this.locResources.CHASSIS_INFO_PATTERNS,
                    type: this.resources.CHASSIS,
                    children: ChassisInfoPatterns
                });
            }
            if(CHASManagementIntPatterns.length>0){
                allSelected.push({
                    id: "cm",
                    name: this.locResources.MANAGEMENT_INTERFACE_PATTERNS,
                    type: this.resources.CHASSIS,
                    children: CHASManagementIntPatterns
                });
            }
            if(PowerAcousticsPatterns.length>0){
                allSelected.push({
                    id: "cp",
                    name: this.locResources.POWER_ACOUSTICS_PATTERNS,
                    type: this.resources.CHASSIS,
                    children: PowerAcousticsPatterns
                });
            }
            if(UsersSecPatterns.length>0){
                allSelected.push({
                    id: "cu",
                    name: this.locResources.USERS_SEC_PATTERNS,
                    type: this.resources.CHASSIS,
                    children: UsersSecPatterns
                });
            }
            if(ChassisIOModulePatterns.length>0){
                allSelected.push({
                    id: "cio",
                    name: this.locResources.CHASSIS_IO_MOD_PATTERNS,
                    type: this.resources.CHASSIS,
                    children: ChassisIOModulePatterns
                });
            }
            if(NodePatterns.length>0){
                allSelected.push({
                    id: "cn",
                    name: this.locResources.NODE_PATTERNS,
                    type: this.resources.CHASSIS,
                    children: NodePatterns
                });
            }
            if(IPAddressPools.length>0){
                allSelected.push({
                    id: this.locResources.AP_AP,
                    name: this.locResources.IP_ADDRESS_POOLS,
                    type: "Pool",
                    children: IPAddressPools
                });
            }
            
            var exportData = dojo.toJson({
                "identifier": "id",
                "label": this.locResources.EXPORT_PATTERNS,
                "items": allSelected
            }, true);
            return exportData;
        },
        
        _getDependentPatterns: function(mainId, selected) {
            // tags:
            // protected
            // Find the patterns that need to be automatically included
            this._referenced.forEach(dojo.hitch(this, function(ref){
                var include = false;
                var id = ref.id;
                ref.referencedBy.forEach(function(n){
                    if(n == mainId){
                        include = true;
                    }
                });
                var alreadySelected = false;
                selected.forEach(function(sel){ //Make sure the user didn't already select this pattern
                    if(sel.item.id[0] == id){
                        alreadySelected = true;
                        return;
                    }
                });
                var alreadyIncluded = false;
                this.autoInclude.forEach(function(auto){ //Make sure the pattern isn't already in the autoInclude array
                    if(auto.id == id){
                        alreadyIncluded = true;
                        return;
                    }
                });
                if(include && !alreadySelected && !alreadyIncluded){
                    this.autoInclude.push(ref);
                    if(ref.type=="cmmManagement" || ref.type=="cmmIOModule" || ref.type=="cmmNode" 
                        || ref.type=="Management"){
                        this._getRequiredIPPools(ref.id, selected);
                    }
                }
            }));
        },
        
        _getRequiredIPPools: function(patternId, selected){
            // tags:
            // protected
            // Need a separate function for IP pools as they have no referencedBy property
            dojo.xhrGet({ // Make REST call to determine the dependent IP pools
                url: this.REST_PATH + "template/" + patternId,
                handleAs: "json",
                sync: true,
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                preventCache: true,
                load: dojo.hitch(this, function(response) {
                    var ipv4 = "";
                    var ipv6 = "";
                    if(response && response.items && response.items.length>0){
                        if(response.items[0].ipv4settingsaddresspoolvalue 
                                && response.items[0].ipv4settingsaddresspoolvalue!=""){
                            // Pattern references IPv4 pool
                            ipv4 = response.items[0].ipv4settingsaddresspoolvalue;
                        }
                        if(response.items[0].ipv6settingsaddresspoolvalue 
                                && response.items[0].ipv6settingsaddresspoolvalue!=""){
                            // Pattern references IPv6 pool
                            ipv6 = response.items[0].ipv6settingsaddresspoolvalue;
                        }
                        if(ipv4 != ""){
                            this._ipPools.forEach(dojo.hitch(this, function(pool){
                                var include = false;
                                if(pool.id == ipv4){
                                    include = true;
                                }
                                var alreadySelected = false;
                                selected.forEach(function(sel){ // Make sure the user didn't already select this pattern
                                    if(sel.item.id[0] == ipv4){
                                        alreadySelected = true;
                                        return;
                                    }
                                });
                                var alreadyIncluded = false;
                                this.autoInclude.forEach(function(auto){ // Make sure the pattern isn't already in the autoInclude array
                                    if(auto.id == ipv4){
                                        alreadyIncluded = true;
                                        return;
                                    }
                                });
                                if(include && !alreadySelected && !alreadyIncluded){
                                    this.autoInclude.push(pool);
                                }
                            }));
                        }
                        if(ipv6 != ""){
                            this._ipPools.forEach(dojo.hitch(this, function(pool){
                                var include = false;
                                if(pool.id == ipv6){
                                    include = true;
                                }
                                var alreadySelected = false;
                                selected.forEach(function(sel){ //Make sure the user didn't already select this pattern
                                    if(sel.item.id[0] == ipv6){
                                        alreadySelected = true;
                                        return;
                                    }
                                });
                                var alreadyIncluded = false;
                                this.autoInclude.forEach(function(auto){ //Make sure the pattern isn't already in the autoInclude array
                                    if(auto.id == ipv6){
                                        alreadyIncluded = true;
                                        return;
                                    }
                                });
                                if(include && !alreadySelected && !alreadyIncluded){
                                    this.autoInclude.push(pool);
                                }
                            }));
                        }
                    }
                })
            });
        }
        
    });
});