define([
        "dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/PoolDetails",
        "dojo/text!./templates/PoolDetails.html",
        "support/base/NextGenUIMixin",
        "support/widgets/OutlinePane",
        "dijit/layout/TabContainer",
        "support/panels/FibrePoolDetailsDependent",
        "dijit/Toolbar",
        "dijit/ToolbarSeparator",
        "dijit/Menu",
        "dojo/dom"
    ], function(dojo, dijit, panel, mri, template) {

    return dojo.declare("support.panels.FibrePoolDetails", [ support.panels.Panel, support.base.NextGenUIMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in displaying information
        // associated with a server.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "ServerDetails",
        
        // the properties used to construct the title of this page in the
        //   NextGen UI
        cpPageContext: null,
        cpPageName: null,
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/PoolDetails.css",

        dependentsTabWidget: null,
        
        templateInformation: "",
        
        templateIDNum: "",
        templateName: "",
        templateDesc: "",
        templateInUse: false,
       // templateType: "",
        
        postMixInProperties: function() {
            this.inherited(arguments);
            this.cpPageContext = this.resources.PRODUCT_NAME;
            this.cpPageName = this.resources.DETAILS;
        },

        postCreate: function() {
            this.inherited(arguments);
            
            if(this.CP_RUN_ENVIRONMENT == "augusta"){
                
                //Add page base in order to use the setTitle() function
                require([ "augusta/augustaUICore/core/augustaPageBase" ], dojo.hitch(this, function(PageBase) {                
                    var CP_AUGUSTA_PAGE_BASE = new augusta.augustaUICore.core.augustaPageBase();
                    dojo.safeMixin(this, CP_AUGUSTA_PAGE_BASE);            
                }));

                this.bwIsCacheable = false;
            }    

            this.subscribe("RefreshNameDiv", function(data) {
                // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
                this.cpPageContext = data;

                if(this.CP_RUN_ENVIRONMENT == "fsm"){
                    //Set title in FSM
                    this.changeNextGenUITitle();
                }
                
                if(this.CP_RUN_ENVIRONMENT == "augusta"){
                    //Set Title in xHMC, only can be used if augustaPageBase mixed in
                    this.setTitle(this.cpPageContext + " - " + this.cpPageName);
                }
            });
                        
            this.getServerTemplateID();
                        
            // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
            this.cpPageContext = this.templateName;

            if(this.CP_RUN_ENVIRONMENT == "fsm"){
                //Set title in FSM
                this.changeNextGenUITitle();
            }
            
            if(this.CP_RUN_ENVIRONMENT == "augusta"){
                //Set Title in xHMC
                this.setTitle(this.cpPageContext + " - " + this.cpPageName);
            }

            this.toolbarSetup();

            // Dependents Tab
            this.dependentsTabWidget = new support.panels.FibrePoolDetailsDependent({
                id: this.id + "cp_fibrepooldetails_fibrepooldetailsdependent",
                mode: "fibrepoolDetails",
                vendor: this.templateIDNum,
                tableWidth: "99%",
                title:this.locResources.SVRTMPL_DEPENDENTS_TAB_TITLE
            });
            this._tabContainer.addChild(this.dependentsTabWidget);
                                   
        },

        //Get Server Template ID
        getServerTemplateID: function(){

                    if(this.templateIDNum == ""){  //If this templateID not provided then parse url for it
                        var url = window.location.href;
                        var splitUrl = url.split("cpTarget=");
                        var IDarray = new Array();
                        var ID = "";
                        
                        if(splitUrl[1]){
                            IDarray = splitUrl[1].split("&");
                        }else{
                            ID = "";
                        }
                        
                        if(IDarray[0]){
                            ID=IDarray[0];
                            
                            this.templateIDNum = ID;
                            
                            if(this.templateIDNum == "Emulex"){
                                this.templateName = this.locResources.EMULEX_TITLE;;
                            }else if(this.templateIDNum == "IBM"){
                                this.templateName  = this.locResources.IBM_TITLE;                                                                
                            }else if(this.templateIDNum == "Brocade"){
                                this.templateName  = this.locResources.BROCADE_TITLE;                                                                
                            }else if(this.templateIDNum == "Qlogic"){
                                this.templateName  = this.locResources.QLOGIC_TITLE;                                                                
                            }
                            
                            // this.DetailsTemplateDescription.innerHTML = (this.templateInformation.description != null) ? this.templateInformation.description : "";
                            
                        }else{  //If no ID found then ...

                            this.templateName = this.locResources.FIBRECHANNELADDRESS+" - "+this.resources.DETAILS;
                            
                        }
                                                                                             
                    }else{
                        
                        if(this.templateIDNum == "Emulex"){
                            this.templateName = this.locResources.EMULEX_TITLE;;
                        }else if(this.templateIDNum == "IBM"){
                            this.templateName  = this.locResources.IBM_TITLE;                                                                
                        }else if(this.templateIDNum == "Brocade"){
                            this.templateName  = this.locResources.BROCADE_TITLE;                                                                
                        }else if(this.templateIDNum == "Qlogic"){
                            this.templateName  = this.locResources.QLOGIC_TITLE;                                                                
                        }

                        
                    }

                    this.DetailsTemplateType.innerHTML = this.locResources.TYPE_FIBRE;
        },
        //ToolBar Setup
        toolbarSetup: function(){
          
            this.moreMenu = new dijit.Menu({          // Add to More/Actions Menu
                style: "display: none;"
            });
      
      }
        
    });

});