define([ 
        "dojo", 
        "dijit", 
        "support/panels/Panel", 
        "dojo/i18n!./nls/DetailsDependentTable", 
        "dojo/text!./templates/DetailsDependentTable.html",
        "dojo/data/ItemFileWriteStore",
        "support/base/TableMixin"        
        ],function(dojo, dijit, panel, mri, template) {

    return dojo.declare("support.panels.FibrePoolDetailsDependent", [ support.panels.Panel , support.base.TableMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in managing server profiles
        // for Configuration Patterns.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "DetailsDependentTable",
        
        mode: "", // Current Context Mode
        
        _detailsTable: null,
        
        profileLayout: {            /* Table View Options */
            flatView: true, // Flat Table View
            chassisView: true, // By Chassis View
            groupView: true, // By Group View
            templateView: true // By Template View
            /* Toolbar Buttons and Menus */

            /* "More" menu options */

            /* Table Columns To Display */
        },

        vendor:"", //Eth or Fibre pool vendor
        templateIDNum:"",   //Category Template ID   
        templateType: "",   //Template Type
        
        // ID prefix used for table
        gridID: "FibrePoolDetailsDependentTable",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/DetailsDependentTable.css",

        // When this method is called, all variables
        // inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget
        // property variables defined in the template
        // HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after
        // buildRendering(). This is useful to override
        // when
        // you need to access and/or manipulate DOM
        // nodes included with your widget.
        // DOM nodes and widgets with the
        // dojoAttachPoint attribute specified can now
        // be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in
        // buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);

            // Setup table widget...
            var tableStore = this._retrieveStore();
            var tableStructure = this._getTableStructure(); // Get structure for current view

            this._detailsTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.gridID, true);

            setTimeout(dojo.hitch(this, function() {
    	        this._detailsTable.placeAt(this.detailsTable);
	            this._detailsTable.startup();
            }), 500);
                        
            // Connect handlers to reset table capabilities
            dojo.connect(this._detailsTable.select.row, "onSelectionChange", this, this.setTableCapabilities); // Fires on row selection
            dojo.connect(this._detailsTable, "setStore", this, this.setTableCapabilities); // Fires on store reset            

        },
        
        // Connect to data source and create models for
        // TreeTable widgets.
        _retrieveStore: function() {
                                                            
                    var ranges = "";
                    
                    var xhrGet = dojo.xhrGet({
                        url: this.REST_PATH + "addresspool/getdetails/FIBRE/"+this.vendor,
                        handleAs: "json",
                        sync: true,
                        preventCache: true,
                        load: function(response) {                            
                            ranges = dojo.toJson(response.items);
                        }
                    });

                    var tableStore = "";
                                                                                 
                    if(ranges == "" || ranges == null){
                        tableStore = new dojo.data.ItemFileWriteStore({
                            data: dojo.fromJson("{ \"items\" : [], \"label\" : \"id\", \"identifier\" : \"id\" }")
                        });
                    }else{
                        
                        ranges = dojo.fromJson(ranges);
                                                
                        for(var x=0;x<ranges.length;x++){
                            
                            //Profile
                            ranges[x].consumers = ranges[x].profile;    
                            ranges[x].range = ""; 

                            var childrenArray = new Array();

                            //Fabric A and children
                            var childObject = new Object();
                            childObject.id = "AFAB" + ranges[x].id;
                            childObject.consumers = "Fabric A";
                            childObject.range = "";
                            childObject.chassis = "";
                            childObject.bay = "";
                            childObject.system = "";

                            //Fabric A Childern Array
                            var childrenArray2 = new Array();
                            
                            //WWNNA Object
                            var childObject2 = new Object();
                            childObject2.id = "A2WWNN" + ranges[x].id;
                            childObject2.consumers = "WWNN";
                            childObject2.range = ranges[x].wwnnA_range;
                            childObject2.chassis = "";
                            childObject2.bay = "";
                            childObject2.system = "";
                            childrenArray2.push(childObject2);                            

                            //WWPNA Object
                            var childObject2 = new Object();
                            childObject2.id = "A3WWPN" + ranges[x].id;
                            childObject2.consumers = "WWPN";
                            childObject2.range = ranges[x].wwpnA_range;
                            childObject2.chassis = "";
                            childObject2.bay = "";
                            childObject2.system = "";
                            childrenArray2.push(childObject2);                            
                            
                            childObject.children = childrenArray2;
                            
                            childrenArray.push(childObject);                            

                            //Fabric B and children
                            var childObject = new Object();
                            childObject.id = "BFAB" + ranges[x].id;
                            childObject.consumers = "Fabric B";
                            childObject.range = "";
                            childObject.chassis = "";
                            childObject.bay = "";
                            childObject.system = "";

                            var childrenArray2 = new Array();

                            //WWNNB Object
                            var childObject2 = new Object();
                            childObject2.id = "B2WWNN" + ranges[x].id;
                            childObject2.consumers = "WWNN";
                            childObject2.range = ranges[x].wwnnB_range;
                            childObject2.chassis = "";
                            childObject2.bay = "";
                            childObject2.system = "";
                            childrenArray2.push(childObject2);                            

                            //WWPNB Object
                            var childObject2 = new Object();
                            childObject2.id = "B3WWPN" + ranges[x].id;
                            childObject2.consumers = "WWPN";
                            childObject2.range = ranges[x].wwpnB_range;
                            childObject2.chassis = "";
                            childObject2.bay = "";
                            childObject2.system = "";
                            childrenArray2.push(childObject2);                            
                            
                            childObject.children = childrenArray2;

                            childrenArray.push(childObject);                            

                            ranges[x].children = childrenArray;                                                 
                        }

                        ranges = dojo.toJson(ranges);

                        tableStore = new dojo.data.ItemFileWriteStore({
                            data: dojo.fromJson("{ \"items\" : " + ranges + ", \"label\" : \"id\", \"identifier\" : \"id\" }")
                        });
                    }
                    
                    return tableStore;

        },
        
        // Re-render Tree table with new columns/data
        // presentation
        _reRenderTable: function() {
            // 99275: Get function from tree table widget
        },
        
        // Columns to be displayed on table
        _getTableStructure: function() {
            var cols = new Array();

            cols.push({
                field: "consumers",               
                name: this.locResources.CONSUMERS,
                width: "20em",
                decorator: dojo.hitch(this, this._decorateConsumer),
                formatter: dojo.hitch(this, this._formatConsumer),
                title: ""
            });

            cols.push({
                field: "range",
                name: this.locResources.ADDRESS,
                width: "18em",
                title: ""
            });

            cols.push({
                field: "chassis",
                name: this.resources.CHASSIS,
                width: "12em",
                title: ""
            });

            cols.push({
                field: "bay",
                name: this.resources.BAY,
                width: "8em",
                title: ""
            });

            cols.push({
                field: "system",
                name: this.locResources.SYSTEM,
                title: ""
            });

            return cols;
        },
        
        // Get number of selected rows in tree table
        _getSelectedRowCount: function() {
            return this._detailsTable.select.row.getSelected().length;
        },

        // Get row objects for selected rows
        _getSelectedRows: function() {
            return this._detailsTable.select.row.getSelected();
        },
        
        // Set Table Capabilities
        setTableCapabilities: function() {
        },
        
        // Decorate consumers
        _decorateConsumer: function(item) {
            if(item == "Fabric A" || item == "Fabric B" || item == "WWNN" || item == "WWPN"){
                return item;                
            }else{                
                return "<span title=\"" + item + " - " + this.resources.SERVER_PROFILE + "\" style=\"white-space: nowrap;\">" + item + "</span>";
            } 
        },
        
        // Set Table Formatter for Consumer Fibre Address Pool
        _formatConsumer: function(item) {
            // tags:
            // protected
            return item.consumers;                        
        }
    });
});