define([ "dojo", 
         "dijit", 
         "support/panels/Panel", 
         "dojo/i18n!./nls/IOAddresses", 
         "dojo/text!./templates/IOAddresses.html", 
         "support/widgets/OutlinePane",
         "dijit/form/Button",
         "dojo/data/ItemFileReadStore", 
         "support/base/TableMixin",
         "dijit/Toolbar"
         ], function(dojo, dijit, panel, mri, template) {

    return dojo.declare("support.panels.IOAddresses", [ support.panels.Panel, support.base.TableMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in displaying I/O Addresses
        // associated
        // with a server.
        //
        // example:
        // | <div data-dojo-type="support.panels.IOAddresses" title="I/O Addresses Summary"></div>

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
        
        serverUUID: "", // Server UUID
        profileID: "", //Profile ID

        mode: "", // launch point server or profile 
        
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "IOAddresses",

        _addressTable: null,
        
        _tableToolbar: null,
        
        // ID prefix used for table
        gridID: "IOAddressTable",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/IOAddresses.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {

            this.inherited(arguments);

        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);

            // Setup table widget...
            var addressStore = this._retrieveStore();       // Get table store information
            var tableStructure = this._generateDetailsLayout();  // Get structure for current view
            
            this._addressTable = this.getTable(addressStore, tableStructure, this.resources.NO_DATA, this.gridID, true);

            this._addressTable.placeAt(this.addressTable);

            this._addressTable.startup();

            this._addToolbarButtons(this._addressTable); //load table toolbar

        },

        startup: function() {

            if (this._graphicView) {
                // 99296: Set graphic view to default state and content. For now just set a
                // placeholder.
               // this._graphicView.set("content", "BladeCenter H");
            }

            // Start all our children widgets too.
            this.inherited(arguments);

        },

        // Connect to data source and create models for TreeTable widgets.
        _retrieveStore: function() {

            var addressData = "";
            var addressStore = null;

            if(this.mode == "profile"){

                var xhrGet = dojo.xhrGet({
                    url: this.REST_PATH + "profile/"+this.profileID+"/ioAddressSummary",
                    handleAs: "json",
                    preventCache: true,
                    sync: true,
                    load: dojo.hitch(this, function(response) {
                        addressData = response;                
                    }),
                    handle: function() {

                    }
                });

            }else{
                var xhrGet = dojo.xhrGet({
                    url: this.REST_PATH + "server/"+this.serverUUID+"/ioAddressSummary",
                    handleAs: "json",
                    preventCache: true,
                    sync: true,
                    load: dojo.hitch(this, function(response) {
                        addressData = response;                
                    }),
                    handle: function() {

                    }
                });
            }

//           addressData = {"identifier":"id","label":"name","items":[{"id":"17","type":"root","nodeType":"node","children":[{"id":"adapter:1","type":"adapter","children":[{"id":"adapter:1:port:1","type":"port","portLabel":"1","physicalFunction":"nic","physicalFunctionLabel":"","moduleBay":1,"mac":"00:1A:64:76:00:00","wwnn":"","wwpn":"","children":[]},{"id":"adapter:1:port:2","type":"port","portLabel":"2","physicalFunction":"nic","physicalFunctionLabel":"","moduleBay":2,"mac":"00:1A:64:76:00:02","wwnn":"","wwpn":"","children":[]},{"id":"adapter:1:port:3","type":"port","portLabel":"3","physicalFunction":"nic","physicalFunctionLabel":"","moduleBay":1,"mac":"00:1A:64:76:00:04","wwnn":"","wwpn":"","children":[]},{"id":"adapter:1:port:4","type":"port","portLabel":"4","physicalFunction":"nic","physicalFunctionLabel":"","moduleBay":2,"mac":"00:1A:64:76:00:06","wwnn":"","wwpn":"","children":[]}],"slotNumber":1,"adapterType":"ethernet","description":"","adapterPackage":"adapter","startModuleBay":1,"endModuleBay":2},{"id":"adapter:2","type":"adapter","children":[{"id":"adapter:2:port:1","type":"port","portLabel":"1","physicalFunction":"nic","physicalFunctionLabel":"","moduleBay":3,"mac":"00:1A:64:76:00:10","wwnn":"","wwpn":"","children":[]},{"id":"adapter:2:port:2","type":"port","portLabel":"2","physicalFunction":"virtual","physicalFunctionLabel":"","moduleBay":4,"mac":"","wwnn":"","wwpn":"","children":[{"id":"adapter:2:port:2:vPort:1","type":"vport","portLabel":"1","physicalFunction":"nic","physicalFunctionLabel":"","moduleBay":4,"mac":"00:1A:64:76:00:10","wwnn":"","wwpn":"","children":[]},{"id":"adapter:2:port:2:vPort:2","type":"vport","portLabel":"2","physicalFunction":"fcoe","physicalFunctionLabel":"4","moduleBay":4,"mac":"00:1A:64:76:00:11","wwnn":"20:81:00:50:76:20:00:04","wwpn":"21:81:00:50:76:20:00:04","children":[]},{"id":"adapter:2:port:2:vPort:3","type":"vport","portLabel":"3","physicalFunction":"nic","physicalFunctionLabel":"6","moduleBay":4,"mac":"00:1A:64:76:00:12","wwnn":"","wwpn":"","children":[]},{"id":"adapter:2:port:2:vPort:4","type":"vport","portLabel":"4","physicalFunction":"nic","physicalFunctionLabel":"8","moduleBay":4,"mac":"00:1A:64:76:00:13","wwnn":"","wwpn":"","children":[]}]},{"id":"adapter:2:port:3","type":"port","portLabel":"3","physicalFunction":"nic","physicalFunctionLabel":"","moduleBay":3,"mac":"00:1A:64:76:00:14","wwnn":"","wwpn":"","children":[]},{"id":"adapter:2:port:4","type":"port","portLabel":"4","physicalFunction":"virtual","physicalFunctionLabel":"","moduleBay":4,"mac":"","wwnn":"","wwpn":"","children":[{"id":"adapter:2:port:4:vPort:1","type":"vport","portLabel":"1","physicalFunction":"nic","physicalFunctionLabel":"2","moduleBay":4,"mac":"00:1A:64:76:00:10","wwnn":"","wwpn":"","children":[]},{"id":"adapter:2:port:4:vPort:2","type":"vport","portLabel":"2","physicalFunction":"fcoe","physicalFunctionLabel":"4","moduleBay":4,"mac":"00:1A:64:76:00:11","wwnn":"20:81:00:50:76:20:00:05","wwpn":"21:81:00:50:76:20:00:05","children":[]},{"id":"adapter:2:port:4:vPort:3","type":"vport","portLabel":"3","physicalFunction":"nic","physicalFunctionLabel":"6","moduleBay":4,"mac":"00:1A:64:76:00:12","wwnn":"","wwpn":"","children":[]},{"id":"adapter:2:port:4:vPort:4","type":"vport","portLabel":"4","physicalFunction":"nic","physicalFunctionLabel":"8","moduleBay":4,"mac":"00:1A:64:76:00:13","wwnn":"","wwpn":"","children":[]}]}],"slotNumber":2,"adapterType":"vfa","description":"","adapterPackage":"adapter","startModuleBay":3,"endModuleBay":4},{"id":"adapter:3","type":"adapter","children":[{"id":"adapter:3:port:1","type":"port","portLabel":"1","physicalFunction":"fc","physicalFunctionLabel":"","moduleBay":1,"mac":"","wwnn":"20:80:00:50:76:00:00:08","wwpn":"21:80:00:50:76:00:00:08","children":[]},{"id":"adapter:3:port:2","type":"port","portLabel":"2","physicalFunction":"fc","physicalFunctionLabel":"","moduleBay":2,"mac":"","wwnn":"20:81:00:50:76:20:00:08","wwpn":"21:81:00:50:76:20:00:08","children":[]}],"slotNumber":3,"adapterType":"fc","description":"","adapterPackage":"adapter","startModuleBay":1,"endModuleBay":2},{"id":"adapter:4","type":"adapter","children":[{"id":"adapter:4:port:1","type":"port","portLabel":"1","physicalFunction":"virtual","physicalFunctionLabel":"","moduleBay":3,"mac":"","wwnn":"","wwpn":"","children":[{"id":"adapter:4:port:1:vPort:1","type":"vport","portLabel":"1","physicalFunction":"nic","physicalFunctionLabel":"10","moduleBay":3,"mac":"00:1A:64:76:00:30","wwnn":"","wwpn":"","children":[]},{"id":"adapter:4:port:1:vPort:2","type":"vport","portLabel":"2","physicalFunction":"nic","physicalFunctionLabel":"12","moduleBay":3,"mac":"00:1A:64:76:00:31","wwnn":"","wwpn":"","children":[]},{"id":"adapter:4:port:1:vPort:3","type":"vport","portLabel":"3","physicalFunction":"nic","physicalFunctionLabel":"14","moduleBay":3,"mac":"00:1A:64:76:00:32","wwnn":"","wwpn":"","children":[]},{"id":"adapter:4:port:1:vPort:4","type":"vport","portLabel":"4","physicalFunction":"nic","physicalFunctionLabel":"16","moduleBay":3,"mac":"00:1A:64:76:00:33","wwnn":"","wwpn":"","children":[]}]},{"id":"adapter:4:port:2","type":"port","portLabel":"2","physicalFunction":"virtual","physicalFunctionLabel":"","moduleBay":4,"mac":"","wwnn":"","wwpn":"","children":[{"id":"adapter:4:port:2:vPort:1","type":"vport","portLabel":"1","physicalFunction":"nic","physicalFunctionLabel":"10","moduleBay":4,"mac":"00:1A:64:76:00:30","wwnn":"","wwpn":"","children":[]},{"id":"adapter:4:port:2:vPort:2","type":"vport","portLabel":"2","physicalFunction":"fcoe","physicalFunctionLabel":"12","moduleBay":4,"mac":"00:1A:64:76:00:31","wwnn":"20:81:00:50:76:20:00:0C","wwpn":"21:81:00:50:76:20:00:0C","children":[]},{"id":"adapter:4:port:2:vPort:3","type":"vport","portLabel":"3","physicalFunction":"nic","physicalFunctionLabel":"14","moduleBay":4,"mac":"00:1A:64:76:00:32","wwnn":"","wwpn":"","children":[]},{"id":"adapter:4:port:2:vPort:4","type":"vport","portLabel":"4","physicalFunction":"nic","physicalFunctionLabel":"16","moduleBay":4,"mac":"00:1A:64:76:00:33","wwnn":"","wwpn":"","children":[]}]},{"id":"adapter:4:port:3","type":"port","portLabel":"3","physicalFunction":"virtual","physicalFunctionLabel":"","moduleBay":3,"mac":"","wwnn":"","wwpn":"","children":[{"id":"adapter:4:port:3:vPort:1","type":"vport","portLabel":"1","physicalFunction":"nic","physicalFunctionLabel":"10","moduleBay":3,"mac":"00:1A:64:76:00:30","wwnn":"","wwpn":"","children":[]},{"id":"adapter:4:port:3:vPort:2","type":"vport","portLabel":"2","physicalFunction":"nic","physicalFunctionLabel":"12","moduleBay":3,"mac":"00:1A:64:76:00:31","wwnn":"","wwpn":"","children":[]},{"id":"adapter:4:port:3:vPort:3","type":"vport","portLabel":"3","physicalFunction":"nic","physicalFunctionLabel":"14","moduleBay":3,"mac":"00:1A:64:76:00:32","wwnn":"","wwpn":"","children":[]},{"id":"adapter:4:port:3:vPort:4","type":"vport","portLabel":"4","physicalFunction":"nic","physicalFunctionLabel":"16","moduleBay":3,"mac":"00:1A:64:76:00:33","wwnn":"","wwpn":"","children":[]}]},{"id":"adapter:4:port:4","type":"port","portLabel":"4","physicalFunction":"virtual","physicalFunctionLabel":"","moduleBay":4,"mac":"","wwnn":"","wwpn":"","children":[{"id":"adapter:4:port:4:vPort:1","type":"vport","portLabel":"1","physicalFunction":"nic","physicalFunctionLabel":"10","moduleBay":4,"mac":"00:1A:64:76:00:30","wwnn":"","wwpn":"","children":[]},{"id":"adapter:4:port:4:vPort:2","type":"vport","portLabel":"2","physicalFunction":"fcoe","physicalFunctionLabel":"12","moduleBay":4,"mac":"00:1A:64:76:00:31","wwnn":"20:81:00:50:76:20:00:0D","wwpn":"21:81:00:50:76:20:00:0D","children":[]},{"id":"adapter:4:port:4:vPort:3","type":"vport","portLabel":"3","physicalFunction":"nic","physicalFunctionLabel":"14","moduleBay":4,"mac":"00:1A:64:76:00:32","wwnn":"","wwpn":"","children":[]},{"id":"adapter:4:port:4:vPort:4","type":"vport","portLabel":"4","physicalFunction":"nic","physicalFunctionLabel":"16","moduleBay":4,"mac":"00:1A:64:76:00:33","wwnn":"","wwpn":"","children":[]}]}],"slotNumber":4,"adapterType":"vfa","description":"","adapterPackage":"adapter","startModuleBay":3,"endModuleBay":4}]}]};

            if(addressData.items){
            
                var numOflocations = addressData.items.length;   
                var numOfportloc = 0;
                var numOfport = 0;
                var numOfvport = 0;
                     
                var loopcount;
                var loopcount2;
                var loopcount3;
                var loopcount4;
           
                for (loopcount = 0; loopcount < numOflocations; loopcount++) {

//               console.log(this._formatName(addressData.items[loopcount].nodeType,""));
                    addressData.items[loopcount].name = this._formatName(addressData.items[loopcount].nodeType,"");
               
                    if(addressData.items[loopcount].children.length)
                        numOfportloc = addressData.items[loopcount].children.length;

                    for (loopcount2 = 0; loopcount2 < numOfportloc; loopcount2++) {

  //                 console.log(this._formatName(addressData.items[loopcount].children[loopcount2].adapterPackage,addressData.items[loopcount].children[loopcount2].slotNumber));
                        addressData.items[loopcount].children[loopcount2].name = this._formatName(addressData.items[loopcount].children[loopcount2].adapterPackage,addressData.items[loopcount].children[loopcount2].slotNumber);

                        addressData.items[loopcount].children[loopcount2].moduleBay = addressData.items[loopcount].children[loopcount2].startModuleBay + "-" + addressData.items[loopcount].children[loopcount2].endModuleBay ;
                                        
                        addressData.items[loopcount].children[loopcount2].packageType = this._formatType(addressData.items[loopcount].children[loopcount2].adapterPackage,addressData.items[loopcount].children[loopcount2].adapterType);
                   
                        if(addressData.items[loopcount].children[loopcount2].children.length)
                            numOfport = addressData.items[loopcount].children[loopcount2].children.length;

                        for (loopcount3 = 0; loopcount3 < numOfport; loopcount3++) {

    //                   console.log(this._formatName(addressData.items[loopcount].children[loopcount2].children[loopcount3].type,addressData.items[loopcount].children[loopcount2].children[loopcount3].portLabel));
                            addressData.items[loopcount].children[loopcount2].children[loopcount3].name = this._formatName(addressData.items[loopcount].children[loopcount2].children[loopcount3].type,addressData.items[loopcount].children[loopcount2].children[loopcount3].portLabel);

                            addressData.items[loopcount].children[loopcount2].children[loopcount3].packageType = this._formatPortType(addressData.items[loopcount].children[loopcount2].children[loopcount3].physicalFunction,"");
                       
                            if(addressData.items[loopcount].children[loopcount2].children[loopcount3].children)
                                numOfvport = addressData.items[loopcount].children[loopcount2].children[loopcount3].children.length;

                            for (loopcount4 = 0; loopcount4 < numOfvport; loopcount4++) {
      //                     console.log(this._formatName(addressData.items[loopcount].children[loopcount2].children[loopcount3].children[loopcount4].physicalFunction,addressData.items[loopcount].children[loopcount2].children[loopcount3].children[loopcount4].physicalFunctionLabel));
                                addressData.items[loopcount].children[loopcount2].children[loopcount3].children[loopcount4].name = this._formatName(addressData.items[loopcount].children[loopcount2].children[loopcount3].children[loopcount4].physicalFunction,addressData.items[loopcount].children[loopcount2].children[loopcount3].children[loopcount4].physicalFunctionLabel);
                           
                                addressData.items[loopcount].children[loopcount2].children[loopcount3].children[loopcount4].packageType = this._formatType(addressData.items[loopcount].children[loopcount2].children[loopcount3].children[loopcount4].physicalFunction,"");
                           }
                       }
                   }    
               }
           }
            
            
            
           //console.log(addressData);
           
            if (this.addressDataModel == null) {
                var addressStore = new dojo.data.ItemFileReadStore({
                    data: addressData
                });
            }else{            
                addressStore = new dojo.data.ItemFileReadStore({
                    data: { "items" : [] }
                });
            } 
            
            if (!("hasChildren" in addressStore)) {
                addressStore.hasChildren = function(id, item) { 
                    return item && this.getValues(item, "children").length;
                };
            };
            
            if (!("getChildren" in addressStore)) {
                addressStore.getChildren = function(item) { 
                    return this.getValues(item, "children");
                };
            };
            
            return addressStore;
        },
        
        _generateDetailsLayout: function() {
            var addressDataLayout = [ {
                field: "name",
                name: this.locResources.LOCATION,
                width: "13%",
                formatter: this._formatNoWrap
            }, {
                field: "macA",
                name: this.locResources.MAC_A,
                width: "14%"
            }, {
                field: "macAB",
                name: this.locResources.MAC_B,
                width: "14%",
                formatter: this._formatNoWrap
            },  {
                field: "wwnn",
                name: this.locResources.WWNN,
                width: "19%",
                formatter: this._formatNoWrap
            }, {
                field: "wwpn",
                name: this.locResources.WWPN,
                width: "19%",
                formatter: this._formatNoWrap
            }, {
                field: "functionmac",
                name: this.locResources.FUNCTION_MAC,
                width: "16%",
                formatter: this._formatNoWrap
            }];
                        
            return addressDataLayout;
        },        
        
        _addToolbarButtons: function(grid) {
            var toolbar = grid && grid.toolBar && grid.toolBar.widget;
            
            if (toolbar) {
                var collapsebutton = new dijit.form.Button({
                    id: this.id + "cp_ioaddresses_collapsebutton",
                    showLabel: false,
                    iconClass: "spCollapseAllLgIcon",
                    disabled: false,
                    title: this.resources.COLLAPSE_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._addressTable.collapseAll();
                    }),
                    onKeypress: dojo.hitch(this, function() {
                        this._addressTable.collapseAll();
                    })
                });
                
                toolbar.addChild(collapsebutton);
    
                // Add custom Expand All button...
                var expandbutton = new dijit.form.Button({
                    id: this.id + "cp_ioaddresses_expandbutton",
                    showLabel: false,
                    iconClass: "spExpandAllLgIcon",
                    disabled: false,
                    title: this.resources.EXPAND_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {
                        window.setTimeout(dojo.hitch(this, function(){    // !important for IE8
                            this._addressTable.expandAll();
                        }), 1000);
                    }),
                    onKeypress: dojo.hitch(this, function() {
                        window.setTimeout(dojo.hitch(this, function(){    // !important for IE8
                            this._addressTable.expandAll();
                        }), 1000);
                    })
                });
                
                toolbar.addChild(expandbutton);
            }
        },

        _formatPackageType: function(value) {

            var rv = [ "" ];
            switch (value) {
                case 0:
                    rv.push("<span title=\"" + this.locResources.ON_BOARD + "\" style=\"white-space: nowrap;\">");
                    rv.push("<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spMotherboardIcon\" align=\"left\">");
                    rv.push(this.locResources.ON_BOARD);
                    rv.push("</span>");
                    break;
                case 5:
                    rv.push("<span title=\"" + this.resources.ETHERNET + "\" style=\"white-space: nowrap;\">");
                    rv.push("<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spEthPortIcon\" align=\"left\">");
                    rv.push(this.resources.ETHERNET);
                    rv.push("</span>");
                    break;
                case 8:
                    rv.push("<span title=\"" + this.resources.VIRTUAL_FABRIC + "\" style=\"white-space: nowrap;\">");
                    rv.push("<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spAdapterIcon\" align=\"left\">");
                    rv.push(this.resources.VIRTUAL_FABRIC);
                    rv.push("</span>");
                    break;
                case 16:
                    rv.push("<span title=\"" + this.locResources.VNIC + "\" style=\"white-space: nowrap;\">");
                    rv.push("<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spFcoePortIcon\" align=\"left\">");
                    rv.push(this.locResources.VNIC);
                    rv.push("</span>");
                    break;
                case 25:
                    rv.push("<span title=\"" + this.locResources.VIRTUAL_ETHERNET + "\"style=\"white-space: nowrap;\">");
                    rv.push("<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spEthPortIcon\" align=\"left\">");
                    rv.push(this.resources.ETHERNET);
                    rv.push("</span>");
                    break;
                case 26:
                    rv.push("<span title=\"" + this.locResources.FIBRE + "\" style=\"white-space: nowrap;\">");
                    rv.push("<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spFcoePortIcon\" align=\"left\">");
                    rv.push(this.resources.FCOE);
                    rv.push("</span>");
                    break;
            }
            return rv.join("");

        },

        _formatNoWrap: function(value) {

            var rv = [ "" ];

            if ((value != null) && (value !== "")) {
                rv.push("<span title=\"" + value + "\" style=\"white-space: nowrap;\">" + value + "</span>");
            }
            return rv.join("");

        },
        
        _formatName: function(value,slotnumber) {
            var name;
            
            if(value == "node"){

                name = this.locResources.COMPUTENODE;

            }else if(value == "expansion"){
            
                name = this.locResources.IO_EXPANSION;
                
            }else if(value == "lom"){
            
                name = this.resources.FABRIC_CONNECTOR;
                
            }
            else if(value == "adapter"){
                
                name = this.resources.IO_ADAPTER.replace("%1", slotnumber);
                
            }
            else if(value == "port"){
                
                name = this.locResources.PORT_NAME.replace("%1", slotnumber);
                
            }
            else if(value == "nic"){
                
                if(slotnumber == ""){
                    name = this.locResources.VPORT_NIC.replace("%1", slotnumber);
                }else{
                    name = this.locResources.VPORT_NAME_NIC.replace("%1", slotnumber);                    
                }
                
            }
            else if(value == "fc"){
                
                if(slotnumber == ""){
                    name = this.locResources.VPORT_HBA.replace("%1", slotnumber);
                }else{
                    name = this.locResources.VPORT_NAME_HBA.replace("%1", slotnumber);
                }

            }
            else if(value == "fcoe"){
                
                if(slotnumber == ""){
                    name = this.locResources.VPORT_HBA.replace("%1", slotnumber);
                }else{
                    name = this.locResources.VPORT_NAME_HBA.replace("%1", slotnumber);
                }

            }
            else if(value == "iscsi"){
                
                if(slotnumber == ""){
                    name = this.locResources.VPORT_HBA.replace("%1", slotnumber);
                }else{
                    name = this.locResources.VPORT_NAME_HBA.replace("%1", slotnumber);
                }

            }
            else{
                name = "";
            }
            
            
            return name;
        },
        _formatType: function(value,adaptertype) {

            var rv = [ "" ];
            
            if(value == "lom"){
                
                return this.resources.VIRTUAL_FABRIC;
                
            }else if(value == "adapter"){
                
                if(adaptertype == "vfa"){
                    rv.push("<span title=\"" + this.resources.VIRTUAL_FABRIC + "\" style=\"white-space: nowrap;\">" + this.resources.VIRTUAL_FABRIC + "</span>");
                    return rv.join("");
                }else if(adaptertype == "ethernet"){
                    rv.push("<span title=\"" + this.resources.ETHERNET + "\" style=\"white-space: nowrap;\">" + this.resources.ETHERNET + "</span>");
                    return rv.join("");
                }else if(adaptertype == "fc"){
                    rv.push("<span title=\"" + this.resources.FIBRE_CHANNEL + "\" style=\"white-space: nowrap;\">" + this.resources.FIBRE_CHANNEL + "</span>");
                    return rv.join("");                    
                }else if(adaptertype == "fcoe"){
                    rv.push("<span title=\"" + this.resources.FCOE_TITLE + "\" style=\"white-space: nowrap;\">" + this.resources.FCOE + "</span>");
                    return rv.join("");
                }
            
            }else if(value == "nic"){
                
                rv.push("<span title=\"" + this.resources.ETHERNET + "\" style=\"white-space: nowrap;\">");
                rv.push("<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spEthPortIcon\" align=\"left\" style=\"vertical-align: text-top; margin-right: 4px;\">");
                rv.push(this.resources.ETHERNET);
                rv.push("</span>");
                return rv.join("");
                
            }
            else if(value == "fc"){
                
                rv.push("<span title=\"" + this.resources["FIBRE_CHANNEL"] + "\" style=\"white-space: nowrap;\">");
                rv.push("<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spFcPortIcon\" align=\"left\" style=\"vertical-align: text-top; margin-right: 4px;\">");
                rv.push(this.resources["FIBRE_CHANNEL"]);
                rv.push("</span>");
                return rv.join("");
                
            }
            else if(value == "fcoe"){
                
                rv.push("<span title=\"" + this.resources["FCOE_TITLE"] + "\" style=\"white-space: nowrap;\">");
                rv.push("<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spFcoePortIcon\" align=\"left\" style=\"vertical-align: text-top; margin-right: 4px;\">");
                rv.push(this.resources["FCOE"]);
                rv.push("</span>");
                return rv.join("");
                
            }
            else if(value == "iscsi"){
                
                rv.push("<span title=\"" + this.resources["ISCSI_TITLE"] + "\" style=\"white-space: nowrap;\">");
                rv.push("<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spEthPortIcon\" align=\"left\" style=\"vertical-align: text-top; margin-right: 4px;\">");
                rv.push(this.resources["ISCSI"]);
                rv.push("</span>");
                return rv.join("");
    
            }
            
            return value;
            
        },        
        _formatPortType: function(value,adaptertype) {
            
            if(value == "virtual"){
                
                return this.resources.VIRTUAL_FABRIC;
                
            }else if(value == "nic"){
                
                return this.resources.ETHERNET;
                
            }
            else if(value == "fc"){
                
                return this.resources.FIBRE_CHANNEL;
                
            }
            else if(value == "fcoe"){
                
                return this.resources.FCOE;
                
            }
            else if(value == "iscsi"){
                
                return this.resources.ISCSI;
    
            }
            
            return value;
            
        }

    });

});