define([ "dojo", 
         "dijit", 
         "support/panels/Panel", 
         "dojo/i18n!./nls/IOAddresses", 
         "dojo/text!./templates/IOAddresses2.html", 
         "support/widgets/OutlinePane",
         "dijit/form/Button",
         "dojo/data/ItemFileReadStore", 
         "support/base/TableMixin",
         "dijit/Toolbar"
         ], function(dojo, dijit, panel, mri, template) {

    return dojo.declare("support.panels.IOAddresses2", [ support.panels.Panel, support.base.TableMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in displaying I/O Addresses
        // associated
        // with a server.
        //
        // example:
        // | <div data-dojo-type="support.panels.IOAddresses" title="I/O Addresses Summary"></div>

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
        
        serverUUID: "", //Server ID
        profileID: "", //Profile ID

        mode: "", // launch point server or profile 

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "IOAddresses",

        _addressTable: null,
        
        _tableToolbar: null,
        
        // ID prefix used for table
        gridID: "IOAddress2Table",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/IOAddresses.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {

            this.inherited(arguments);

        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);

            // Setup table widget...
            var addressStore = this._retrieveStore();       // Get table store information
            var tableStructure = this._generateDetailsLayout();  // Get structure for current view
            
            this._addressTable = this.getTable(addressStore, tableStructure, this.resources.NO_DATA, this.gridID, true);

            this._addressTable.placeAt(this.addressTable);

            this._addressTable.startup();

            this._addToolbarButtons(this._addressTable); //load table toolbar

        },

        startup: function() {

            if (this._graphicView) {
                // 99296: Set graphic view to default state and content. For now just set a
                // placeholder.
               // this._graphicView.set("content", "BladeCenter H");
            }

            // Start all our children widgets too.
            this.inherited(arguments);

        },

        // Connect to data source and create models for TreeTable widgets.
        _retrieveStore: function() {

            var addressData = "";
            var addressStore = null;

            if(this.mode == "profile"){
                var xhrGet = dojo.xhrGet({
                    url: this.REST_PATH + "profile/"+this.profileID+"/ioAddressAllReserved",
                    handleAs: "json",
                    preventCache: true,
                    sync: true,
                    load: dojo.hitch(this, function(response) {
                        addressData = response;                
                    }),
                    handle: function() {

                    }
                });
            }else{
                var xhrGet = dojo.xhrGet({
                    url: this.REST_PATH + "server/"+this.serverUUID+"/ioAddressAllReserved",
                    handleAs: "json",
                    preventCache: true,
                    sync: true,
                    load: dojo.hitch(this, function(response) {
                        addressData = response;                
                    }),
                    handle: function() {

                    }
                });
            }
            
            if(addressData.items){
                var numOfAdapters = addressData.items.length;
                var numOfPorts = 0;
                var numOfFunction = 0;
            
                console.log(numOfAdapters);
        
                for (var loopcount = 0; loopcount < numOfAdapters; loopcount++) {
                    addressData.items[loopcount].name = this._formatName(addressData.items[loopcount].type,addressData.items[loopcount].label);

                    if(addressData.items[loopcount].children.length)
                        numOfPorts = addressData.items[loopcount].children.length;
                    
                        for (var loopcount2 = 0; loopcount2 < numOfPorts; loopcount2++) {
                            addressData.items[loopcount].children[loopcount2].name = this._formatName(addressData.items[loopcount].children[loopcount2].type,addressData.items[loopcount].children[loopcount2].label);
                        
                            if(addressData.items[loopcount].children[loopcount2].children.length)
                                numOfFunction = addressData.items[loopcount].children[loopcount2].children.length;
        
                            for (var loopcount3 = 0; loopcount3 < numOfFunction; loopcount3++) {
                                if(addressData.items[loopcount].children[loopcount2].children[loopcount3])
                                    addressData.items[loopcount].children[loopcount2].children[loopcount3].name = this._formatName(addressData.items[loopcount].children[loopcount2].children[loopcount3].type,addressData.items[loopcount].children[loopcount2].children[loopcount3].label);
                            }
                        }
                }
                addressStore = new dojo.data.ItemFileReadStore({
                    data: addressData
                });
            }else{            
                addressStore = new dojo.data.ItemFileReadStore({
                    data: { "items" : [] }
                });
            } 
            
            if (!("hasChildren" in addressStore)) {
                addressStore.hasChildren = function(id, item) { 
                    return item && this.getValues(item, "children").length;
                };
            };
            
            if (!("getChildren" in addressStore)) {
                addressStore.getChildren = function(item) { 
                    return this.getValues(item, "children");
                };
            };
            
            return addressStore;
        },
        
        _generateDetailsLayout: function() {
            var addressDataLayout = [ {
                field: "name",
                name: this.locResources.LOCATION,
                width: "13%",
                formatter: this._formatNoWrap
            }, {
                field: "macA",
                name: this.locResources.MAC_A,
                width: "14%"
            }, {
                field: "macAB",
                name: this.locResources.MAC_B,
                width: "14%",
                formatter: this._formatNoWrap
            },  {
                field: "wwnn",
                name: this.locResources.WWNN,
                width: "19%",
                formatter: this._formatNoWrap
            }, {
                field: "wwpn",
                name: this.locResources.WWPN,
                width: "19%",
                formatter: this._formatNoWrap
            }, {
                field: "functionmac",
                name: this.locResources.FUNCTION_MAC,
                width: "16%",
                formatter: this._formatNoWrap
            }];
                        
            return addressDataLayout;
        },        
        
        _addToolbarButtons: function(grid) {
            var toolbar = grid && grid.toolBar && grid.toolBar.widget;
            
            if (toolbar) {
                var collapsebutton = new dijit.form.Button({
                    id: this.id + "cp_ioaddresses2_collapsebutton",
                    showLabel: false,
                    iconClass: "spCollapseAllLgIcon",
                    disabled: false,
                    title: this.resources.COLLAPSE_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._addressTable.collapseAll();
                    }),
                    onKeypress: dojo.hitch(this, function() {
                        this._addressTable.collapseAll();
                    })
                });
                
                toolbar.addChild(collapsebutton);
    
                // Add custom Expand All button...
                var expandbutton = new dijit.form.Button({
                    id: this.id + "cp_ioaddresses2_expandbutton",
                    showLabel: false,
                    iconClass: "spExpandAllLgIcon",
                    disabled: false,
                    title: this.resources.EXPAND_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {
                        window.setTimeout(dojo.hitch(this, function(){    // !important for IE8
                            this._addressTable.expandAll();
                        }), 1000);
                    }),
                    onKeypress: dojo.hitch(this, function() {
                        window.setTimeout(dojo.hitch(this, function(){    // !important for IE8
                            this._addressTable.expandAll();
                        }), 1000);
                    })
                });
                
                toolbar.addChild(expandbutton);
            }
        },

        _formatNoWrap: function(item) {
            var rv = [ "" ];
            
            switch(this.field) {
                case "name":
                    rv.push("<span title=\"" + item.name + "\" style=\"white-space: nowrap;\">" + item.name + "</span>");
                    break;
                case "macA":
                    rv.push("<span title=\"" + item.macA + "\" style=\"white-space: nowrap;\">" + item.macA + "</span>");
                    break;
                case "macAB":
                    rv.push("<span title=\"" + item.macAB + "\" style=\"white-space: nowrap;\">" + item.macAB + "</span>");
                    break;
                case "wwnn":
                    rv.push("<span title=\"" + item.wwnn + "\" style=\"white-space: nowrap;\">" + item.wwnn + "</span>");
                    break;
                case "wwpn":
                    rv.push("<span title=\"" + item.wwpn + "\" style=\"white-space: nowrap;\">" + item.wwpn + "</span>");
                    break;
                case "functionmac":
                    rv.push("<span title=\"" + item.functionmac + "\" style=\"white-space: nowrap;\">" + item.functionmac + "</span>");
                    break;                    
                default:                    
            }

            return rv.join("");
        },
        
        _formatName: function(type,label) {
            var name;
            
            if(type == "root"){

                name = this.locResources.IO_ADAPTER.replace("%1",label);

            }else if(type == "port"){
            
                name = this.locResources.PORT_NAME.replace("%1",label);
                
            }else if(type == "function"){
            
                name = this.locResources.FUNCTION.replace("%1",label);
                
            }else{
                name = "";
            }
            
            return name;
        }

    });

});