define(["dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/LearnedSettings",        
        "dojo/text!./templates/LearnedSettings.html",
        "dojo/data/ItemFileWriteStore",        
        "dojox/grid/LazyTreeGridStoreModel",
        "dojo/data/ItemFileReadStore",
        "dijit/form/CheckBox", 
        "support/widgets/ToggleButton", 
        "dijit/form/Select",
        "dijit/form/ValidationTextBox", 
        "dijit/form/NumberTextBox", 
        "dijit/Tooltip", 
        "support/panels/CheckButton",
        "support/widgets/ButtonBar", 
        "dijit/form/DateTextBox",
        "dijit/form/TimeTextBox",
        "dijit/layout/ContentPane", 
        "support/panels/BuildAList",
        "dijit/form/FilteringSelect", 
        "dojo/store/Memory", 
        "support/panels/SimpleTable",
        "support/widgets/DoubleSelects", 
        "support/panels/OrderedList",
        "dojo/_base/array",
        "support/dialogs/SaveAs",
        "support/temp/ModalDialog",
        "dojo/string",
        "support/dialogs/ProgressDialog"
        ], function (dojo, dijit, panel, mri, template) {

	return dojo.declare("support.panels.LearnedSettings", [ support.panels.Panel ], {
	    
	    // templateString: [protected] String
	    // This widget's HTML template, provided.
	    templateString: template,
	        
	    // locResources: [protected] Object
	    // This widget's National Language Source (NLS) object.
	    locResources: mri, 
	    
		widgetsInTemplate : true,
		baseClass : "LearnedSettings", // This widget's base CSS classname.
		
		type: null,
		
		store: null,
		maxPercent: .75,
		_twisties: new Array(),
		paths: new Array(),
		settings: new Object(),
		level: 0,
		
		configDef:null,
		configSettings:null,
		      
        tablePaths: new Array(), //Holds all path options
        idToPath: new Object(), //Holds ID to path value
        
        idToSuppress: new Object(),  //Holds ID to suppress-if value
        pathWithSuppress: new Object(), //Holds path to suppress-if value

        idToGray: new Object(), //Holds ID to gray-if value
        pathWithGray: new Object(), //Holds path to gray-if value

        suppressMe: new Array(), //Holds setting not to create cli for

        excludeMe: new Array(), //Holds setting not to create cli for
        pathReset: new Array(), //holds value of path incase cancel button hit
        checkReset: new Array(), //holds path values incase cancel button hit

        numOfIncludedSettings: 0, //Holds number of included settings
        numOfExcludedSettings: 0, //Holds number of excluded settings
        numOfSettings: 0, //Holds number of settings
        
        //Performance and Recovery Pattern Related
         //Currently only has a value when during Server Pattern Firmware page
         //when a performance recovery template is selected on that page
        performanceRelated: null,
        
        //Holds the Operating Mode of the related Performance and Recovery Pattern
        performanceOpMode: "",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/LearnedSettings.css",

		// postCreate() is called after buildRendering(). This is useful to override when
		// you need to access and/or manipulate DOM nodes included with your widget.
		// DOM nodes and widgets with the dojoAttachPoint attribute specified can now be
		// directly
		// accessed as fields on "this".
		// Common operations for postCreate
		// 1) Access and manipulate DOM nodes created in buildRendering()
		// 2) Add new DOM nodes or widgets
		postCreate : function () {
			this.inherited(arguments);

			//reset array and objects
			 this.paths = new Array();
             this.suppressMe = new Array(); 
			 this.excludeMe = new Array(); 
	         this.checkReset = new Array();

	         this.tablePaths = new Array();
	         this.idToPath = new Object();
	         this.idToSuppress = new Object();
	         this.pathWithSuppress = new Object();
             this.idToGray = new Object();
             this.pathWithGray = new Object();

             this.numOfSettings = 0;
	         this.numOfExcludedSettings = 0;
             this.numOfIncludedSettings = 0;
	         
            if(this.type == "uefi" || this.type == "imm" || this.type == "port"){
			    this.store = new dojo.data.ItemFileWriteStore({
                    data: this.configDef
			    });	
	            this.getProgressDialog("load").show(); // Show progress dialog for save request.
			}
			else{
			    this.store = new dojo.data.ItemFileWriteStore({
	                data: { "items" : [] }
	            });
			}

			//Find and set operating mode of related performance and recovery pattern
            if(this.performanceRelated != null && this.performanceRelated > 0){

                var returnedData;
                
                var xhrGet = dojo.xhrGet({
                    url: this.REST_PATH + "template/"+this.performanceRelated,
                    handleAs: "json",
                    sync: true,
                    preventCache: true,
                    load: dojo.hitch(this, function(response) {
                        returnedData = response; 
                    }),
                    handle: dojo.hitch(this, function() {
                        //console.log("HERE IT IS");
                        if(returnedData.items[0].operating_mode.buttonBar1_1 == "Custom"){
                            this.performanceOpMode = "Custom Mode";                                                    
                        }else{
                            if(returnedData.items[0].operating_mode.buttonBar1_1 == "Predefined"){
                                
                                if(returnedData.items[0].operating_mode.opModeSelection == "Maximum Performance"){
                                    this.performanceOpMode = "Maximum Performance";                                                                                                             
                                }else if(returnedData.items[0].operating_mode.opModeSelection == "Efficiency - Favor Performance"){
                                    this.performanceOpMode = "Efficiency - Favor Performance";                                                                                                             
                                }else if(returnedData.items[0].operating_mode.opModeSelection == "Efficiency - Favor Power"){
                                    this.performanceOpMode = "Efficiency - Favor Power";                                                                                                             
                                }else if(returnedData.items[0].operating_mode.opModeSelection == "Minimal Power"){
                                    this.performanceOpMode = "Minimal Power";                                                                                                             
                                }
                             
                                //console.log(this.performanceOpMode);

                            }                            
                        }
                            
                    })
                });
                
            }
                                                
            window.setTimeout(dojo.hitch(this, function(){    // !important for IE8                 
                this._showAllSettings();  

                //Set number of Settings Available
              //  this.totalContainer.innerHTML = this.numOfSettings;

                //Set number of Included Settings
               // this.includedContainer.style.display = "inline-block";
                
                this.hideParentChangeSettings();
                this._expandAll();
                
                //this.countSettings(); //Count settings for display
                
                dojo.publish("ProgressComplete", "");
            }),1000);
                  
		},
		//Count initial settings
        countSettings : function(){
            this.numOfIncludedSettings = 0;

            for(var i=0; i<this.paths.length; i++){  //Loop path settings
                var path = this.paths[i];
                
                var check = document.getElementById(path + "_check");
                var optionNode = document.getElementById(path + "_optionNode");

                if(check != null){
                    if(check.checked){  //If setting checked show option Node and don't hide node 
                        
                        if(optionNode != null){
                            this.numOfIncludedSettings++;
                        }

                    }
                }

            }
            //this.includedContainer.innerHTML = this.numOfIncludedSettings;
        },
		_widgetsShow : function(){

            var tablePaths = this.tablePaths;

            for(var i=0; i<tablePaths.length; i++){
                var path = tablePaths[i];
                var widget = dijit.byId(path);
                    //need to initially display custom table widgets
                    if(widget != null){
                        var declaredClass = widget.declaredClass;
                        if(declaredClass == "support.panels.OrderedList" ||
                           declaredClass == "support.panels.BuildAList" ||
                           declaredClass == "support.panels.SimpleTable" ){
                            widget._show();
                        }  
                    }
            }

		},
		//items
		//	group
		//		command
		//			option
		//			_settings
		//				_setting
		
		//Helper method to set up initial state of hierarchy
		_setUpCollapseGroups : function(){
			//show 2nd hierarchy
			var level = 2;
			for(var i=0; i<this.paths.length; i++){
				var path = this.paths[i];
				var node = document.getElementById(path + "_node");
				if(node != null){
					var twistie = document.getElementById(path + "_twistie");
					if(twistie != null){
						var pathSplit = path.split("__");
						if(pathSplit.length >= level)
							this._collapseItem(path);
					}
				}			
			}
		},
	
		//Reposition
		reposition : function(){
			//leave empty so user can override
		},
		
		//Sets the width of the dialog based on the number of levels in the hierarchy
		_setDialogWidth : function(){
			this.setDialogWidth(this.level);
			this.reposition();
		},
		
		//Connects to the parent dialog
		setDialogWidth : function(level){
			//leave empty so user can override
		},
		
		//Returns the value of an object with a given mriName
		_getValue : function(mri){		    
		    //Code here taken out because it sets the value twice with this old code
		    
		    //var store = this.store;
			//var value = null;
			/*var gotItems = function(items, request){
				for(var i=0; i<items.length; i++){
					var item = items[i];
					if(item.value != null){
						value = item.value[0];
					}
				}
			};*/
			//store.fetch({query: {mriName:mri}, queryOptions: {deep:true}, onComplete: gotItems});
			
			return null;
		},
		
		//Determines if the item has children
		_hasChildren : function(item){
		    var count = 0;
		    if(item.group != null)
		        count++;
		    if(item.command != null)
		        count++;
		    if(item.option != null)
		        count++;
		    /*if(item.Definitions != null)
		        count++;*/
		    return count;
		    //return (item.group != null) || (item.command != null) || (item.option != null || (item.Definitions != null));
		},
		
		//Returns the children of an item
		_getChildren : function(item){
		    var children = new Array();
		    /*if(item.Definitions != null)
		        children.push(item.Definitions);*/
		    if(item.group != null) 
                children.push(item.group);
		    if(item.command != null) 
                children.push(item.command);
            if(item.option != null)
                children.push(item.option);
		    //AT THE MOMENT, WILL ONLY RETURN ONE SET OF EACH CHILD
		    return children;
		},
		
        //Helper method to show all of the learned settings
        _showAllSettings : function(){
            var lst = this;
            var store = this.store;
                              
            var gotItems = function(items){
            
                for(var i=0; i<items.length; i++){                    
                    var item = items[i];
                    lst._showSetting(item, 0, "", false);
                }            
                lst._setUpCollapseGroups();
                lst._setDialogWidth();
                lst._widgetsShow();
                
                //Run suppress and gray if
                lst.setSuppressIf(); 
                lst.setGrayIf();
            };

            
            store.fetch({onComplete:gotItems});
                                                
        },
        
        findSettings : function(item, path){
            if(item != null){
                path += item.ID[0];
                if(this._hasChildren(item) > 0){
                    var children = this._getChildren(item);
                    path += "__";
                                        
                    for(var c=0; c<children.length; c++){ //iterate through multiple "group", "command"
                        var someGroup = children[c];
                        for(var g = 0; g<someGroup.length; g++){
                            var someGroupItem = someGroup[g];
                            if(someGroupItem.settings != null){
                                var settings = someGroupItem.settings;
                                var p = path + someGroupItem.ID + "__";
                                if(this.isIndexedOptions(settings)){
                                    this.assignIndexedOptions(settings, p);
                                }
                                else if(this.hasSuboptions(settings)){
                                    this.assignSuboptions(settings, p);
                                }
                                else{
                                    for(var i=0; i<settings.length; i++){
                                        var set = settings[i];
                                        if(set.setting != null){
                                            for(var j=0; j<set.setting.length; j++){
                                                var setting = set.setting[j];
                                                if(setting.length == undefined){
                                                    this.settings[p + setting.ID] = setting.value[0];
                                                }
                                                else{
                                                    this.getIndexedArray(setting, p);
                                                }
                                            }
                                        } 
                                    }
                                }
                                
                            }
                            this.findSettings(someGroupItem, path);
                        }
                    }
                }
            }
        },
        
        hasSuboptions : function(settings){
            if(settings.length > 1){
                for(var i=0; i<settings.length; i++){
                    var setting = settings[i].setting;
                    if(setting.length != 2){
                        return false;
                    }
                }
                return true;
            }
            return false;
        },
        
        assignSuboptions : function(settings, path){
            if(settings.length > 1){
                for(var i=0; i<settings.length; i++){
                    var setting = settings[i].setting;
                    if(setting.length == 2){
                        var p1 = setting[1].ID[0];
                        var p2 = setting[0].ID[0];
                        var p = path + p1 + "__" + p2;
                        var value = setting[0].value[0];
                        this.settings[p] = value;
                    }
                }
            }
        },
        
        isIndexedOptions : function(settings){
            for(var i=0; i<settings.length; i++){
                var setting = settings[i].setting;
                var hasOne = this.hasOneIndex(setting);
                if(!hasOne)
                    return false;
            }
            return true;
        },
        
        assignIndexedOptions : function(settings, path){
            for(var s=0; s<settings.length; s++){
                var setting = settings[s].setting;
                var initialPath = path;
                var objIndex = null;
                for(var i=0; i<setting.length; i++){
                    var setIndex = setting[i].index;
                    if(setIndex != null){
                        objIndex = setIndex[0];
                        initialPath += setting[i].ID;
                        break;
                    }
                }                
                var obj = new Object;
                obj["index"] = objIndex;
                for(var i=0; i<setting.length; i++){
                    var set = setting[i];
                    if(set.index == null){
                        var setPath = initialPath + "__" + set.ID[0];
                        var setValue = set.value[0];
                        obj[setPath] = setValue;
                    }
                }
                if(this.settings[initialPath] == null){
                    var arr = new Array();
                    arr.push(obj);
                    this.settings[initialPath] = arr;
                }
                else{
                    var arr = this.settings[initialPath];
                    arr.push(obj);
                    this.settings[initialPath] = arr;
                }
            }
        },

        // each object in array has index
        hasEachIndex: function(setting) {
            for ( var i = 0; i < setting.length; i++) {
                var item = setting[i];
                if (item.index == null)
                    return false;
            }
            return true;
        },

        // one object in array has index
        hasOneIndex: function(setting) {
            for ( var i = 0; i < setting.length; i++) {
                var item = setting[i];
                if (item.index != null)
                    return true;
            }
            return false;
        },

        //each has an index --> USB Foward Port
        getIndexedArray : function(setting, path){
            if(this.hasEachIndex(setting)){
                var p = path + setting[0].ID;
                var value = new Array();
                for(var index=1; index<=setting.length; index++){
                    for(var i=0; i<setting.length; i++){
                        var setIndex = setting[i].index;
                        if(index == setIndex){
                            value.push({index:index, value:setting[i].value});
                        }
                    }
                }
                this.settings[p] = value;
            }
        },
        
        displayFoundSettings : function(){
            var obj = this.settings;
            for (var attr in obj) {
                this.setValueFromDisplay(attr, obj[attr]);
            }
        },
        
        //Shows all of the settings: (label/twistie, label/no twistie, options)
        _showSetting : function(item, level, path, parentIsCommand){
                        
            var lst = this;
            if(item != null){
                
                if(parseInt(level) > parseInt(this.level)){
                    this.level = level;
                }
                
                if(item.name != null){ //not a "setting"

                    var ID = item.ID[0];
                    path += ID;

                    if(item.data_index != null && item.multi_data == null){

                        lst._showOption(item, level, path, "buildalist");
                        
                    }                   
                    else if(lst._hasChildren(item) > 0){

                        var children = lst._getChildren(item); //returns array of children groups
                        var childLevel = level;
                        
                        //If suppress if, then add it's value to proper objects to execute later
                        var suppressif = "";
                        if(item["suppress-if"]){
                            suppressif = item["suppress-if"][0];           
                            lst.idToSuppress[path] = suppressif;
                            lst.pathWithSuppress[item.ID[0]] = path;
                        }

                        //If gray if, then add it's value to proper objects to execute later                        
                        var grayif = "";
                        if(item["gray-if"]){               
                            grayif = item["gray-if"][0];  
                            lst.idToGray[path] = grayif;
                            lst.pathWithGray[item.ID[0]] = path;
                        }

                        //if it has a child, show the twistie
                        if(item.display == null){

                            childLevel = childLevel + 1;
                            lst._showLabelWithTwistie(item, level, path, parentIsCommand);
                        }
                        else if(ID == "set"){
                            //checkbox
                          if(dijit.byId(path))   
                          dijit.byId(path).destroy();                                 

                            this.paths.push(path);
                            new dijit.form.CheckBox({
                                id: path,
                                checked: true
                            });
                        }
                        
                        path += "__";
                        for(var c=0; c<children.length; c++){ //iterate through multiple "group", "command"
                            var someGroup = children[c];
                                                                                    
                            for(var g = 0; g<someGroup.length; g++){
                                var someGroupItem = someGroup[g];
                                var parentIsCommand = false;
                                if(item.option != null && someGroupItem.option != null){
                                    parentIsCommand = true;
                                }
                                                                                                
                                lst._showSetting(someGroupItem, childLevel, path, parentIsCommand);                                
                                                                
                            }
                        }
                    }
                    else{ //has no children                        
                        if(item.date_data != null)
                            lst._showOption(item, level, path, "date");
                        else if(item.time_data != null)
                            lst._showOption(item, level, path, "time");
                        else if(item.multi_data != null)
                            lst._showOption(item, level, path, "multi");
                        else if(item.enumerate_data != null)
                            lst._showOption(item, level, path, "enumerate");
                        else if(item.text_data != null)
                            lst._showOption(item, level, path, "text");
                        else if(item.numeric_data != null)
                            lst._showOption(item, level, path, "numeric");
                        else if(item.boolean_data != null)
                            lst._showOption(item, level, path, "bool");
                        else if(item.list_data != null)
                            lst._showOption(item, level, path, "list");
                        else if(item.delim_data != null)
                            lst._showOption(item, level, path, "delim");
                        else
                            lst._showLabelWithoutTwistie(item, level, path);
                    }
                }
            }
        },
		
		//Shows each option
		//Node containing space (tabbing), checkbox, label, input
		_showOption : function(item, level, path, type){
		    		  		    
			this.paths.push(path);
			var desc = null;
			if(item.desc != null)
			    desc = item.desc[0];
			var label = item.name[0];

            this.idToPath[item.ID[0]] = path;
            
			//Add suppress-if and gray-if to tooltip description									
			if(item["suppress-if"]){

			    //Only show suppress-if if not blank string
             if(item["suppress-if"][0] != ""){

	             var suppressif = item["suppress-if"][0];  //Suppress If object

                    this.idToSuppress[path] = suppressif;
                    this.pathWithSuppress[item.ID[0]] = path;

	                var suppressifArray = suppressif.split(" ");                        

                    for(var e=0;e < suppressifArray.length;e++){
                        
                        suppressifArray[e] = suppressifArray[e].split("(").join(" ");
                        suppressifArray[e] = suppressifArray[e].split(")").join(" ");

                        suppressifArray[e] = suppressifArray[e].replace(" ","");
                                                
                        if(dojo.indexOf(suppressifArray[e],"!") !== -1){
                            suppressifArray[e] = suppressifArray[e].replace("!","");
                            suppressifArray[e] = suppressifArray[e] + " "+this.locResources.DOES_NOT;
                        }

                        suppressifArray[e] = dojo.trim(suppressifArray[e]);

                        if(dojo.indexOf(suppressifArray[e],".") !== -1)
                        {
                            var suppressIDArray = suppressifArray[e].split(".");                        
                            suppressifArray[e] = suppressIDArray[1];
                        }

                        if(suppressifArray[e] == "=="){
                            suppressifArray[e] = this.locResources.EQUAL;
                        }else if(suppressifArray[e] == "||"){
                            suppressifArray[e] = this.locResources.OR_IF;                            
                        }
                        
                    }
                    
                    //console.log(suppressifArray);

                    suppressif = suppressifArray.join(" ");

                    //console.log(suppressif);

                    desc = desc + "<br/><br/>"+this.locResources.SUPPRESS_IF+" "+suppressif;
                    
              }    
			}

	         if(item["gray-if"]){           
	             
                 var grayif = item["gray-if"][0];  //Gray If object  

                 this.idToGray[path] = grayif;
                 this.pathWithGray[item.ID[0]] = path;

                 var grayifArray = grayif.split(" ");                        

                 var operatingModeDesc = false;
                 
                 for(var e=0;e < grayifArray.length;e++){
                     
                     grayifArray[e] = grayifArray[e].split("(").join(" ");
                     grayifArray[e] = grayifArray[e].split(")").join(" ");

                     grayifArray[e] = grayifArray[e].replace(" ","");

                     
                     if(dojo.indexOf(grayifArray[e],"!") !== -1){
                         grayifArray[e] = grayifArray[e].replace("!","");                        
                         grayifArray[e] = grayifArray[e] + " "+this.locResources.DOES_NOT;
                     }

                     grayifArray[e] = dojo.trim(grayifArray[e]);

                     if(grayifArray[e] == "OperatingModes.ChooseOperatingMode does not" && this.performanceOpMode != ""){
                         operatingModeDesc = true;
                     }

                     //if the gray array inside the joint then get the path, if not undefined then say this the text for it
                     
                     if(grayifArray[e] == "=="){
                         grayifArray[e] = this.locResources.EQUAL;
                     }else if(grayifArray[e] == "||"){
                         grayifArray[e] = this.locResources.OR_IF;                            
                     }

                 }
                 
                 //console.log(grayifArray);

                 grayif = grayifArray.join(" ");

                 //console.log(grayif);
                 desc = desc + "<br/><br/>"+this.locResources.GRAY_IF+" "+grayif;
                 
                 if(operatingModeDesc){
                     desc = desc + "<br/><br/>"+this.locResources.GRAY_OPMODE_TOOLTIP;                     
                 }

	         }

            
			var table = document.createElement("table");
			table.setAttribute("width", "100%");
			var tbody = document.createElement("tbody");
			var row1 = document.createElement("tr");
			var labelCell = document.createElement("td");
			var controlCell = document.createElement("td");
			table.appendChild(tbody);
			tbody.appendChild(row1);
			row1.appendChild(labelCell);
			row1.appendChild(controlCell);
			controlCell.setAttribute("style", "align:right");
			labelCell.setAttribute("style", "vertical-align:top");

			var access = false; //controls whether option is readonly and disabled 
			
			if(item.access){
			    if(item.access[0] == "readonly")
			    access = true;			    
			}
			

			//////////////////////////////////////////////
			var base = path;
			var lst = this;
			var node = document.createElement("div");
			node.setAttribute("id", base + "_node");

			
			//spacing
			var space = document.createElement("div");
			var width = parseInt(level) * 25;
			space.setAttribute("style", "width:" + width + "px; display:inline-block;");
			
			
			//checkbox	        
	       if(dijit.byId(base +  "_check"))   
	       dijit.byId(base +  "_check").destroy();	          	          	       
	          
			var checkboxNode = document.createElement("div");
			checkboxNode.setAttribute("id", base + "_checkboxNode");
			checkboxNode.setAttribute("style", "display:none;");						
			
			if(this.configSettings[path] === undefined){
	             var check = new dijit.form.CheckBox({
	                    id: base +  "_check",
	                    checked: false,
	                    _destroyOnRemove: true,
	                    onChange: dojo.hitch(this, function(newValue) {
	                        if(newValue){
	                            this.numOfIncludedSettings++;	                            
	                        }else{
                                this.numOfIncludedSettings = this.numOfIncludedSettings - 1;                                                       	                            
	                        }

	                        if(this.numOfIncludedSettings > this.numOfSettings){
	                            this.numOfIncludedSettings = this.numOfSettings;
	                        }
	                        
                            //this.includedContainer.innerHTML = this.numOfIncludedSettings;                              

	                    })
	                }).placeAt(checkboxNode);
			}else{
			    var check = new dijit.form.CheckBox({
			        id: base +  "_check",
			        checked: true,
			        _destroyOnRemove: true,
                    onChange: dojo.hitch(this, function(newValue) {
                        if(newValue){
                            this.numOfIncludedSettings++;                               
                        }else{
                            this.numOfIncludedSettings = this.numOfIncludedSettings - 1;                                                                                    
                        }

                        if(this.numOfIncludedSettings > this.numOfSettings){
                            this.numOfIncludedSettings = this.numOfSettings;
                        }
                        
                        //this.includedContainer.innerHTML = this.numOfIncludedSettings;                              
                    })
			    }).placeAt(checkboxNode);
			}

//           if(dijit.byId(base +  "_optionNode"))   
  //         dijit.byId(base +  "_optionNode").destroy();                                 
			
			//option
			var optionNode = document.createElement("div");
			optionNode.setAttribute("id", base + "_optionNode");
			if(check.checked){
				optionNode.setAttribute("style", "display:inline-block; float:right;");
			}else{
                this.excludeMe.push(base);
                this.numOfExcludedSettings++;
//				optionNode.setAttribute("style", "display:none;");
                node.setAttribute("style", "display:none;");
			}	
			var option = this._getOptionInput(item, path, type, access);
			if(option != null){
				option.placeAt(optionNode);
				if(type != "multi" && type != "list" && type != "buildalist" && desc != null){
				    optionNode.onmouseover = function(e){
	                    dijit.showTooltip("<div style=\"max-width:500px\">" + desc + "</div>", optionNode, ["above"]);
	                };
	                optionNode.onmouseout = function(e){
	                    dijit.hideTooltip(optionNode);
	                };
				}
			}
			

			//line break
			var linebreak = document.createElement("br");
			
			//append
			labelCell.appendChild(space);
            labelCell.appendChild(checkboxNode);

            this.numOfSettings++;  //Increase Number of Settings by 1

            
			if(type != "buildalist"){
			    //label
			    var labelNode = document.createElement("span");
	            labelNode.innerHTML = label;
	            labelNode.onmouseover = function(e){
	                dijit.showTooltip("<div style=\"max-width:500px\">" + desc + "</div>", labelNode, ["above"]);
	            };
	            labelNode.onmouseout = function(e){
	                dijit.hideTooltip(labelNode);
	            };
	            labelCell.appendChild(labelNode);
			}
			    
			if(type == "multi" || type == "list" || type == "buildalist"){
			    labelCell.appendChild(linebreak);
			    labelCell.appendChild(optionNode);
			    optionNode.setAttribute("style", "padding-left:" + (width+25) + "px;");
			}
			else
			    controlCell.appendChild(optionNode);
			
			node.appendChild(table);
			this._learnedTableNode.appendChild(node);

			if(this.configSettings[path] === undefined){}else{
	            this.setValueFromDisplay(path, this.configSettings[path]);			    
			}

						
		},
									
		//Returns a specific input control (toggle, text box, number box)
        _getOptionInput : function(item, path, type, access){
            
            if(dijit.byId(path))   
            dijit.byId(path).destroy();                                 
         
            
			var lst = this;
			var mri = item.mriName[0];
			var controlValue = this._getValue(mri);

			if(type == "buildalist"){
			    return this._getBuildAList(item, path);
			}
			else if(type == "enumerate"){
			    return this._getEnumerateData(item, path, controlValue);
			}
			else if(type == "date"){
			    return new dijit.form.DateTextBox({
                    id: path,
                    disabled: access
                });
			}
			else if(type == "time"){
			    return new dijit.form.TimeTextBox({
                    id: path,
                    disabled: access
                });
			}
			
			else if(type == "text"){
			    var text_data = item.text_data[0];
			    var minchars = 0, maxchars = null, pattern = ".*", boxType = "text", def = "";
			    if(text_data.minchars != null)
			        minchars = text_data.minchars[0];
			    if(text_data.maxchars != null)
			        maxchars = text_data.maxchars[0];
			    if(text_data.pattern != null)
			        pattern = text_data.pattern[0];
			    if(text_data["default"] != null)
			        def = text_data["default"][0];
			    if(text_data.type != null){
			        boxType = text_data.type[0];
			        if(boxType == "encrypt")
			            boxType = "password";
			        else
			            boxType = "text";
			    }
			    if(controlValue == null)
			        controlValue = def;
			    
                var textbox = new dijit.form.ValidationTextBox({
                    id: path,
                    trim:true,
                    type: boxType,
                    disabled: access,
                    //placeHolder: defaultValue,
                    value: controlValue,
                    intermediateChanges: true,
                    regExp: pattern,
                    validator: function(value, constraints){
                        //if(value.length == 0)
                          //  return true;
                        if(minchars != null && maxchars != null){
                            if(value.length < minchars || value.length > maxchars){
                                this.invalidMessage = lst.locResources["INVALID_BETWEEN"].replace("%1", minchars).replace("%2", maxchars);
                                return false;
                            }
                        }
                        if(minchars != null && maxchars == null){
                            if(value.length < minchars){
                                this.invalidMessage = lst.locResources["INVALID_AT_LEAST"].replace("%1", minchars);
                                return false;
                            }
                        }
                        if(pattern != null && value != ""){
                            var reg = new RegExp(pattern);
                            if(!reg.test(value)){
                                this.invalidMessage = lst.locResources.INVALID;
                                return false;
                            }
                        }
                        return true;
                    }
                });
                if(maxchars != null)
                    textbox.set("maxLength", maxchars);
                return textbox;
			}
			else if(type == "numeric"){
			    var numeric_data = item.numeric_data[0];
                var min = 0, max = null, def = "", step = null, type = "dec";
                if(numeric_data.min != null)
                    min = numeric_data.min[0];
                if(numeric_data.max != null)
                    max = numeric_data.max[0];
                if(numeric_data.step != null)
                    step = numeric_data.step[0];
                if(numeric_data["default"] != null)
                    def = numeric_data["default"][0];
                if(numeric_data.type != null)
                    type = numeric_data.type[0];
                if(controlValue == null)
                    controlValue = def;
                                
				return new dijit.form.ValidationTextBox({
					id: path,
                    disabled: access,
					trim:true,
					//placeHolder: def,
					value: controlValue,
					intermediateChanges: true,
					invalidMessage: lst.locResources["INVALID_BETWEEN_NUM"].replace("%1", min).replace("%2", max),
					validator: function(value, constraints){
					    //if(value.length == 0)
					      //  return true;					    
					 
					    if(type == "hex"){  //If type hex then convert hex to int and then compare
					        var valueNum = parseInt(value,16);
                            return valueNum >= parseInt(min,16) && valueNum <= parseInt(max,16);                            					        
					    }else{
	                       if(isNaN(value))
                            return false;
	                       
	                        return parseInt(value) >= min && parseInt(value) <= max;					        
					    }
					}
				});
			}
			else if(type == "delim"){
			    var delim_data = item.delim_data[0];
			    if(delim_data){
			        return new support.widgets.ToggleButton({
	                    id: path,
	                    value: [true, false]
	                });
			    }
			}
			else if(type == "bool"){
                return new dijit.form.CheckBox({
                    id: path,
                    disabled: access,
                    checked: true,
                    style: "display:none",
                    _destroyOnRemove: true
                });
            }
			else if (type == "list"){
			    var list_data = item.list_data[0];
			    
	            var settings = this._getAllSettingsPaths(path);

			    this.tablePaths.push(path);
			    
			    return new support.panels.OrderedList({
			        id: path,
			        max: list_data.max[0],
			        choices: list_data.choice,
	                 settings: settings,
			        _destroyOnRemove: true
			    });
			}
			else if(type == "multi"){
			    var numeric_data = item.numeric_data[0];
			    var text_data = item.text_data[0];
			    var min = 0, max = null, minchars = 0, maxchars = null, pattern = ".*", def = "";
			    if(numeric_data.min != null)
			        min = numeric_data.min[0];
			    if(numeric_data.max != null)
			        max = numeric_data.max[0];
			    if(text_data.minchars != null)
			        minchars = text_data.minchars[0];
			    if(text_data.maxchars != null)
			        maxchars = text_data.maxchars[0];
			    if(text_data.pattern != null)
			        pattern = text_data.pattern[0];
			    if(text_data["default"] != null)
			        def = text_data["default"][0];
                if(controlValue == null)
                    controlValue = [def];
                
                this.tablePaths.push(path);
                
                return new support.panels.SimpleTable({
                    id: path,
                    minchars: minchars, 
                    maxchars: maxchars,
                    min: min,
                    max: max,
                    regExp: pattern,
                    _destroyOnRemove: true
                });
                
			}
			else{
//				console.log("Uh-oh...something went wrong!");
				return null;
			}				
		},	
		
		_getBuildAList : function(item, path){
		     		    		    
	            var min = 0;
	            var max = null;
	            var settings = this._getAllSettingsPaths(path);
	            if(item.numeric_data != null){
	                min = item.numeric_data[0].min[0];
	                max = item.numeric_data[0].max[0];
	            }
	            
	            if(dijit.byId(path))   
	            dijit.byId(path).destroy(); 
	    	            	            
	            this.tablePaths.push(path);
	            
	            return new support.panels.BuildAList({
	                id: path,
	                path: path,
	                item: item,
	                min: min,
	                max: max,
	                settings: settings,
	                _destroyOnRemove: true
	            }); 
		    
		},
		
		_getDoubleEnumData : function(item, path, controlValue, multi){		
	                                          

		    var lst = this;
            var choice = item.enumerate_data[0].choice;
            if(choice != null){
                var options = new Array();
                for(var i=0; i<choice.length; i++){
                    var option = choice[i];
                    options.push({name:option.label[0]+"", id:option.value[0]+""});
                }
                var store1 = new dojo.store.Memory({ data: options });
                /*if(controlValue == null){
                    controlValue = options[0].id;
                }*/         
                
                if(dijit.byId(path))   
                    dijit.byId(path).destroy(); 
                
                var dropDowns = new support.widgets.DoubleSelects({
                    id: path,
                    store1: store1,
                    multi: multi,
                    onChange: function(newValue){
                        var showOptions = false;
                        for(var i=0; i<choice.length; i++){
                            var option = choice[i];
                            if( option.value != null && newValue == option.value[0] && 
                               (option.enumerate_data != null || option.list_data != null) ){
                                showOptions = true;
                                var choice2 = null;
                                var options2 = new Array();
                                var store2 = null;
                                
                                if(option.enumerate_data != null){
                                    choice2 = option.enumerate_data[0].choice;
                                    for(var c=0; c<choice2.length; c++){
                                        var option2 = choice2[c];
                                        var object2 = { name: option2.label[0] + "", id: option2.value[0] + "" };
                                        options2.push(object2);
                                    }
                                    store2 = new dojo.store.Memory({
                                        data: options2
                                    });
                                }
                                else if(option.list_data != null){
                                    choice2 = option.list_data[0].choice;
                                    for(var c=0; c<choice2.length; c++){
                                        var option2 = choice2[c];
                                        var object2 = { label: option2.label[0] + "", value: option2.value[0] + "" };
                                        options2.push(object2);
                                    }
                                    store2 = options2;
                                }
                                
                                if(choice2 != null && store2 != null){
                                    this.set("store2", store2);
                                    this.set("value2", this.value2);
                                    this.set("disabled", false);
                                }
                            }
                        }
                        if(!showOptions){
                            this.set("store2", null); 
                            this.set("value2", null);
                            this.set("disabled", true);
                        }
                    },
                    _destroyOnRemove: true
                });
                dropDowns.set("value1", controlValue);
                return dropDowns;
            }            
            return null;
		},
		
		//Returns a toggle button or drop down with appropriate settings
		_getEnumerateData : function(item, path, controlValue){
		    		  		    
			var lst = this;
			var choice = null;
			var enumerate_data = item.enumerate_data[0]; //first enumerate data
			if(enumerate_data.choice != null)
			    choice = enumerate_data.choice;
			if(choice != null){
				if(choice[0].label != null){	
	                var isNested = false;
	                var multiSelect = true;
					var options = new Array();
					
					for(var i=0; i<choice.length; i++){
						var option = choice[i];
						
						//Execute suppress-if
						if(option["suppress-if"]){
						    if(option["suppress-if"][0] == "false"){
		                        options.push({name:option.label[0]+"", id:option.value[0]+""});						        
						    }
						}else{
	                        options.push({name:option.label[0]+"", id:option.value[0]+""});						    
						}
						
					}
					var store = new dojo.store.Memory({ data: options });
					if(controlValue == null){
					 //   controlValue = options[0].id;
					}

					
					for(var i=0; i<choice.length; i++){
					    var option = choice[i];
                        if(option.enumerate_data != null){
                            isNested = true;
                            multiSelect = false;
                        }
                        if(option.list_data != null){
                            isNested = true;
                            multiSelect = true;
                        }
					}

					if(isNested){
					    return this._getDoubleEnumData(item, path, controlValue, multiSelect);
					}
					else{
					    //toggle button
					    if(choice.length == 2){
					        var label1 = choice[0].label[0];
	                        var label2 = choice[1].label[0];
	                        
	                        if( (label1 == this.resources.ENABLED && label2 == this.resources.DISABLED) || 
	                            (label1 == this.resources.ENABLE && label2 == this.resources.DISABLE)){
	                            
	                            var enabled = false;
	                            if(choice[0]["default"] != null)
	                                enabled = true;
	                            
	                            if(dijit.byId(path))   
	                                dijit.byId(path).destroy(); 
	                            
	                            return new support.widgets.ToggleButton({
                                    id: path,
                                    value: [choice[0].value[0], choice[1].value[0]],
                                    enabled: enabled,
                                    toggleValue: controlValue,
                                    _destroyOnRemove: true
                                });
	                        } 
	                        if( (label2 == this.resources.ENABLED && label1 == this.resources.DISABLED) ||
	                            (label2 == this.resources.ENABLE && label1 == this.resources.DISABLE)){

	                            var enabled = false;
                                if(choice[1]["default"] != null)
                                    enabled = true;
                                
                                if(dijit.byId(path))   
                                    dijit.byId(path).destroy(); 
                                
	                            return new support.widgets.ToggleButton({
                                    id: path,
                                    value: [choice[1].value[0], choice[0].value[0]],
                                    enabled: enabled,
                                    toggleValue: controlValue,
                                    _destroyOnRemove: true
                                });
	                        }
					    }
					    
	                    
	                    //2 button bar
	                    if(choice.length == 2 && (choice[0].label[0].length + choice[1].label[0].length < 10) ){
	                        var values = choice[0].value[0] + "|" + choice[1].value[0];
	                        var labels = choice[0].label[0] + "|" + choice[1].label[0];
	                        
	                        if(dijit.byId(path))   
	                            dijit.byId(path).destroy(); 
	                        
	                        var twoButtons = new support.widgets.ButtonBar({
	                            id: path,
	                            labels: labels,
	                            values: values,
	                            _destroyOnRemove: true
	                        });
	                        for(var i=0; i<choice.length; i++){
                                if(choice[i]["default"] != null)
                                    twoButtons.set("selected", choice[i].value[0]);
                            }
	                        return twoButtons;
	                    }
	                    //3 button bar
	                    else if(choice.length == 3 &&
	                            (choice[0].label[0].length + choice[1].label[0].length + choice[2].label[0].length < 15) ){
	                        var values = choice[0].value[0] + "|" + choice[1].value[0] + "|" + choice[2].value[0];
	                        var labels = choice[0].label[0] + "|" + choice[1].label[0] + "|" + choice[2].label[0];
	                        
	                        if(dijit.byId(path))   
	                            dijit.byId(path).destroy(); 
	                        
	                        var threeButtons = new support.widgets.ButtonBar({
	                            id: path,
	                            values: values,
	                            labels: labels,
	                            _destroyOnRemove: true
	                        });
	                        for(var i=0; i<choice.length; i++){
                                if(choice[i]["default"] != null)
                                    threeButtons.set("selected", choice[i].value[0]);
                            }
                            return threeButtons;
	                    }
	                    else{ //normal drop down
	                        
	                        if(dijit.byId(path))   
	                            dijit.byId(path).destroy(); 
	                        	                        
	                        var dropDown = new dijit.form.FilteringSelect({
                                id: path,
                                required: false,
                                store: store,
                                searchAttr: "name",
                                value: controlValue,
                                _destroyOnRemove: true
                            });
                            for(var i=0; i<choice.length; i++){
                                if(choice[i]["default"] != null){
                                    dropDown.set("value", choice[i].value[0]);
                                }
                            }
                            
                            dropDown.textbox.readOnly = true;
	                        return dropDown;
	                    }
					}
				}
			}
		},
		
		//Shows a label only if object has no children
		_showLabelWithoutTwistie : function(item, level, path){
			this.paths.push(path);
			var label = item.name[0];
			var lst = this;
			var space = document.createElement("div");
			var width = (parseInt(level) * 25) + 5;
			space.setAttribute("style", "width:" + width + "px; display:inline-block;");
			var node = document.createElement("div");
			node.setAttribute("id", path + "_node");
			var twistie = document.createElement("div");
			twistie.setAttribute("style", "width:15px; margin-right:10px; display:inline-block;");
			var labelNode = document.createElement("span");
			labelNode.innerHTML = label;
			var desc = null;
			if(item.desc != null)
			    desc = item.desc[0];
			if(desc != null){
			    labelNode.onmouseover = function(e){
	                dijit.showTooltip("<div style=\"max-width:500px\">" + desc + "</div>", labelNode, ["above"]);
	            };
	            labelNode.onmouseout = function(e){
	                dijit.hideTooltip(labelNode);
	            };
			}
			var heightPadding = document.createElement("div");
			heightPadding.setAttribute("style", "height:7px");
			
			node.appendChild(space);
			node.appendChild(twistie);
			node.appendChild(labelNode);
			node.appendChild(heightPadding);
			this._learnedTableNode.appendChild(node);
		},
		
		//Shows label with twistie if object has children
		_showLabelWithTwistie : function(item, level, path, bool){
			this.paths.push(path);
			var label = item.name[0];
			var lst = this;
			
			
			//table
			var table = document.createElement("table");
            table.setAttribute("width", "100%");
            var tbody = document.createElement("tbody");
            var row1 = document.createElement("tr");
            var labelCell = document.createElement("td");
            var controlCell = document.createElement("td");
            table.appendChild(tbody);
            tbody.appendChild(row1);
            row1.appendChild(labelCell);
            row1.appendChild(controlCell);
            controlCell.setAttribute("style", "align:right");
         
            if(dijit.byId(path + "_node"))   
            dijit.byId(path + "_node").destroy(); 
            
            //if(dijit.byId(path + "_twistie"))   
            //dijit.byId(path + "_twistie").destroy(); 
                
			var space = document.createElement("div");
			var width = parseInt(level) * 25;
			space.setAttribute("style", "width:" + width + "px; display:inline-block;");
			var node = document.createElement("div");
			node.setAttribute("id", path + "_node");
			var twistie = document.createElement("input");
			twistie.setAttribute("id", path + "_twistie");
			twistie.setAttribute("type", "image");
			twistie.setAttribute("value", "");
			twistie.setAttribute("class", "collapseIcon");
			twistie.setAttribute("style", "margin-right:10px;");
			twistie.onclick = function(){
				lst._toggleTwistie(path);
			//	lst._hideUnchecked(lst._checkbox.checked);
				lst._setTriStateIcon();
				lst.reposition();
			};
			var labelNode = document.createElement("span");
			labelNode.innerHTML = label;
			var heightPadding = document.createElement("div");
			heightPadding.setAttribute("style", "height:7px");
			
			
	         //checkbox           
            if(dijit.byId(path +  "_check"))   
            dijit.byId(path +  "_check").destroy();                                 
               
             var checkboxNode = document.createElement("div");
             checkboxNode.setAttribute("id", path + "_checkboxNode");
             checkboxNode.setAttribute("style", "display:none;");                        

              var check = new dijit.form.CheckBox({
                     id: path +  "_check",
                     checked: true,
                     _destroyOnRemove: true
                 }).placeAt(checkboxNode);

                  			
			labelCell.appendChild(space);
			labelCell.appendChild(twistie);
            labelCell.appendChild(checkboxNode);
			labelCell.appendChild(labelNode);			
			
			if(bool){
			    //checkboxes for "options" under "commands"
			    var checkNode = document.createElement("DIV");
			    checkNode.setAttribute("style", "display:inline-block; float:right;");
			    
			    if(dijit.byId(path))   
	                  dijit.byId(path).destroy(); 
			    
			    var check = new dijit.form.CheckBox({
                    id: path,
                    checked: true,
                    style: "display:none",
                    _destroyOnRemove: true
                }).placeAt(checkNode);
			    controlCell.appendChild(checkNode);			    
            }
			
			node.appendChild(table);
			this._learnedTableNode.appendChild(node);
		},
				
		//Toggles the twistie image when clicked and performs the appropriate twistie action
		_toggleTwistie : function(path){
			var twistie = document.getElementById(path + "_twistie");
			if(twistie.getAttribute("class") == "collapseIcon")
				this._collapseItem(path);
			else if(twistie.getAttribute("class") == "expandIcon")
				this._expandItem(path);
		},
		
		//Collapse an item when clicked
		_collapseItem : function(path){
         //   console.log(path + "_twistie");

			var twistie = document.getElementById(path + "_twistie");
			if(twistie != null){
				twistie.setAttribute("class", "expandIcon");
				this._collapseDivs(path);
			}
		},
		
		//Expand an item when clicked
		_expandItem : function(path){
			var twistie = document.getElementById(path + "_twistie");
			if(twistie != null){
				twistie.setAttribute("class", "collapseIcon");
				this._expandDivs(path);
			}
		},		
				
		//Determines if a path is a match
		_isPathMatch : function(actual, compare){				    		    
		    var actualSplit = actual.split("__");
			var compareSplit = compare.split("__");
			if(compareSplit == actualSplit || compareSplit.length <= actualSplit.length)
				return false;
			else{
				for(var i=0; i<actualSplit.length; i++){
					if(actualSplit[i] != null && compareSplit[i] != null){
						if(actualSplit[i] != compareSplit[i]){
							return false;
						}
					}
					
				}
				return true;
			}			
		},
		
		//Returns all paths (IDs) that should be used for a task
		_getAllPaths : function(path){
			var paths = new Array();
			for(var i=0; i<this.paths.length; i++){
				var somePath = this.paths[i];
				//if the start of both paths match, use that path
				if(this._isPathMatch(path, somePath)){
					paths.push(somePath);
				}
			}
			return paths;
		},
		
		_getAllSettingsPaths : function(path){		    		    
		   return this.configSettings[path];		    
		},
		
		//Removes collapsed paths from list of paths
		_removeCollapsedPaths : function(paths){
			for(var i=0; i<paths.length; i++){
				var path = paths[i];
				var twistie = document.getElementById(path + "_twistie");
				if(twistie != null){
					var twistieClass = twistie.getAttribute("class");
					if(twistieClass == "expandIcon"){
						for(var j=i; j<paths.length; j++){
							var somePath = paths[j];
							//if(this._isPathMatch(path, somePath)){
								//paths[j] = null;
							//}
						}
					}
				}
			}
			return paths;
		},
		
		//Collapses all of the div under a certain path when a twistie is clicked		
		_collapseDivs : function(somepath){
			var paths = this._getAllPaths(somepath);
			for(var i=0; i<paths.length; i++){
				var path = paths[i];
				var node = document.getElementById(path + "_node");
				if(node != null){
					node.setAttribute("style", "display:none");
				}
			}	
			this.reposition();
		},
		
		//Expands all of the divs under a certain path when a twistie is clicked
		_expandDivs : function(somepath){
			var paths = this._getAllPaths(somepath);
			paths = this._removeCollapsedPaths(paths);
			for(var i=0; i<paths.length; i++){
				var path = paths[i];
				var widget = dijit.byId(path);
				var node = document.getElementById(path + "_node");
                var check = document.getElementById(path + "_check");

				if(node != null && check != null && check.checked){  //Only expand checked/included settings
					node.setAttribute("style", "display:block");
					//need to initially display custom table widgets
					if(widget != null){
					    var declaredClass = widget.declaredClass;
                        if(declaredClass == "support.panels.OrderedList" ||
                           declaredClass == "support.panels.BuildAList" ||
                           declaredClass == "support.panels.SimpleTable" ){ 
                            widget._show();
                        }  
					}
				}
			}
			this.reposition();
		},
		
		//Determines if settings should be shown or not
		_hideUnchecked : function(checked){
			if(checked)
				this._hideAllDefaultedSettings();
			else
				this._unhideAllDefaultedSettings();
			this.reposition();
		},
		
		//Get name of twisite given a path and an index (concatination of IDs in path name)
		_getTwistieName : function(path, index){
			var pathSplit = path.split("__");
			var somePath = "";
			for(var i=0; i<index+1; i++){
				somePath += pathSplit[i];
				if(i != index)
					somePath += "__";
			}
			return somePath;
		},
		
		//Get all twisties that come before a certain path
		_getPreviousTwisties : function(path){
			var twisties = new Array();
			var pathSplit = path.split("__");
			for(var i=0; i<pathSplit.length; i++){
				var name = this._getTwistieName(path, i);
				var twistie = document.getElementById(name + "_twistie");
				if(twistie != null)
					twisties.push(twistie);
			}
			return twisties;
		},
		
		
		//Helper method to include all settings
		_unhideAllDefaultedSettings : function(){
			for(var i=0; i<this.paths.length; i++){
				var path = this.paths[i];
				var checkbox = dijit.byId(path + "_check");
				//if you are on the path of a setting
				if(checkbox != null){
				    checkbox.set("checked", true);
					//find all of the twisties before it
					var twisties = this._getPreviousTwisties(path);
					var show = true;
					for(var t=0; t<twisties.length; t++){
						var twistie = twisties[t];
						if(twistie != null){
							var twistieClass = twistie.getAttribute("class");
							//do not show if any of the twisties before it are expandIcons
							if(twistieClass == "expandIcon")
								show = false;
						}
					}
					if(show){
						var node = document.getElementById(path + "_node");
						if(node != null)
							node.setAttribute("style", "display:block");
					}
				}
			}
		},

		//Hides all excluded (unchecked) settings from view
		_hideAllDefaultedSettings : function(){
			for(var i=0; i<this.paths.length; i++){
				var path = this.paths[i];
				var widget = dijit.byId(path);
				var checkbox =  dijit.byId(path + "_check");
				var node = document.getElementById(path + "_node");
				if(widget != null && checkbox != null && node != null){
				    var declaredClass = widget.declaredClass;
				    if(declaredClass != "dijit.form.CheckBox"){
				        var defaultValue = this.settings[path];
	                    var currentValue = this.getValueFromDisplay(path);
	                    if(defaultValue == currentValue){
	                        checkbox.set("checked", false);
	                        node.setAttribute("style", "display:none");
	                    }   
				    }
				}
			}
		},
		
		//Displays all of the values chosen (excluding null and "")
		getLearnedSettings : function(){
		    var settings = new Object();
		    for(var i=0; i<this.paths.length; i++){
		        var path = this.paths[i] + "";
		        var widget = dijit.byId(path);
		        if(widget != null){
	                
		            var value = this.getValueFromDisplay(path);
                    var declaredClass = widget.declaredClass;
                    
		            if(value != null && declaredClass != "dijit.form.FilteringSelect"){		                
                        var index = dojo.indexOf(this.excludeMe, path);  //Find if path is in excluded list
                        var index2 = dojo.indexOf(this.suppressMe, path);  //Find if path is in suppressed array
                        //If path is a selected option then add setting , if not then don't add setting
                        if(index == -1 && index2 == -1) 
                        settings[path] = value;
		            }else if(declaredClass == "dijit.form.FilteringSelect" && value != null && value != ""){
                        var index = dojo.indexOf(this.excludeMe, path);  //Find if path is in excluded list
                        var index2 = dojo.indexOf(this.suppressMe, path);  //Find if path is in suppressed array
                        //If path is a selected option then add setting , if not then don't add setting
                        if(index == -1 && index2 == -1) 
                          settings[path] = value;		                
		            }    
		        }		        
		    }
//		    console.log(this.excludeMe);
		    return settings;		    
		},
		
		getValueFromDisplay : function(path){
            var widget = dijit.byId(path);
            //drop down, validation text box, number textbox
            if(widget != null){
                var declaredClass = widget.declaredClass;
                if(declaredClass == "support.widgets.ButtonBar"){
                    return widget.get("selected");
                }
                else if(declaredClass == "support.widgets.ToggleButton"){
                  //  return widget.get("enabled");
                    return widget.get("toggleValue");
                }                
                else if(declaredClass == "dijit.form.CheckBox"){
                    var paths = this._getAllPaths(path);
                    for(var i=0; i<paths.length; i++){
                        var p = paths[i];
                        var val = this.getValueFromDisplay(p);
                        if(val != null && val != ""){
                            return true;
                        }
                    }
                    return null;
                }
                else{
                    return widget.get("value");
                }
            }
            return null;
        },
        
        setValueFromDisplay : function(path, value){
            var widget = dijit.byId(path);
            if(widget != null){
                var declaredClass = widget.declaredClass;
                if(declaredClass == "support.widgets.ButtonBar"){
                    widget.set("selected", value);
                }
                else if(declaredClass == "support.widgets.ToggleButton"){
                    widget.set("toggleValue", value);                    
                 //   widget.set("enabled", value);                                                                    
                }
                else{
                    widget.set("value", value);
                }
            }
        },
		
		//Collapse all of the items
		_collapseAll : function(){
			for(var i=0; i<this.paths.length; i++){
				var path = this.paths[i];
				this._collapseItem(path);
			}
			this._setTriStateIcon();
			//this._hideUnchecked(this._checkbox.checked);
			this.reposition();
		},
		
		//Expand all of the items
		_expandAll : function(){
			for(var i=0; i<this.paths.length; i++){
				var path = this.paths[i];
				this._expandItem(path);
			}
			this._setTriStateIcon();
			//this._hideUnchecked(this._checkbox.checked);
			this.reposition();
		},
				
		//Gets each twistie class that is currently displayed
		_getTwisties : function(){
			for(var i=0; i<this.paths.length; i++){
				var path = this.paths[i];
				var node = document.getElementById(path + "_node");
				if(node != null){
					var visible = node.getAttribute("style");
					if(visible != "display:none"){
						var twistie = document.getElementById(path + "_twistie");
						if(twistie != null)
							this._twisties.push(twistie.getAttribute("class"));
					}
				}
			}
		},
		
		//Returns which tri-state icon should be shown
		_getTriStateIcon : function(){
			this._twisties = new Array();
			this._getTwisties();
			var first = this._twisties[this._twisties.length-1];
			for(var i=0; i<this._twisties.length-1; i++){
				var current = this._twisties[i];
				if(current != null && current != first)
					return "mixedIcon";
			}
			return first;
		},
		
		//Displays the correct tri-state icon
		_setTriStateIcon : function(){
			var iconClass = this._getTriStateIcon();
			if(iconClass == "collapseIcon"){
				this._collapseAllState.setAttribute("style", "display:inline-block");
				this._mixedState.setAttribute("style", "display:none");
				this._expandAllState.setAttribute("style", "display:none");
			}
			else if(iconClass == "expandIcon"){
				this._collapseAllState.setAttribute("style", "display:none");
				this._mixedState.setAttribute("style", "display:none");
				this._expandAllState.setAttribute("style", "display:inline-block");
			}
			else{
				this._collapseAllState.setAttribute("style", "display:none");
				this._mixedState.setAttribute("style", "display:inline-block");
				this._expandAllState.setAttribute("style", "display:none");
			}
		},
		
		//Shows Change Settings Panel
		changeSettings : function(){
		    		    
		   this.Node_1.style.display = "none";   //Hide Extended Edit
           this.Node_2.style.display = "inline"; //Show Extended Customize Settings

           this.pathReset = new Array(); //holds value of path incase cancel button hit
           this.checkReset = new Array(); //holds path values incase cancel button hit

	            for(var i=0; i<this.paths.length; i++){   //Loop thru all settings path
	                var path = this.paths[i];
	    	                
                    var index = dojo.indexOf(this.suppressMe, path);  

                    
                    if(index == -1){
                    
                        var check = document.getElementById(path + "_check");
                        if(check != null){
                            this.pathReset.push(path);
                            this.checkReset.push(check.checked);                               
                        }
                        
                        var node = document.getElementById(path + "_node");   
                        var optionNode = document.getElementById(path + "_optionNode");
                        var checkboxNode = document.getElementById(path + "_checkboxNode");

                        if(node != null){   //show all nodes 
                            node.setAttribute("style", "display:block");
                        }
                        
                        if(optionNode != null){ //hide all options
                            optionNode.setAttribute("style", "display:none");
                        }

                        var paths = this._getAllPaths(path); //Does path have children
                        
                        if(checkboxNode != null  && paths.length == 0){ //show only children check boxes , not parent
                            checkboxNode.setAttribute("style", "display:inline-block; margin-right:10px;");
                        }
	                
                    }
	            }

	  //          console.log(this.excludeMe);
		},
		//Cancel changing settings so revert back to previous settings list
        cancelChangeSettings : function(){
            this.Node_2.style.display = "none";   //Hide Extended Customize Settings
            this.Node_1.style.display = "inline"; //Show Extended Edit

            for(var i=0; i<this.pathReset.length; i++){  //Loop path settings
                var path = this.pathReset[i];

                var node = document.getElementById(path + "_node");
                var optionNode = document.getElementById(path + "_optionNode");
                var checkboxNode = document.getElementById(path + "_checkboxNode");
                var check = document.getElementById(path + "_check");
                
                if(check != null){
                    dijit.byId(path + "_check").set("checked", this.checkReset[i]);
                    
                    if(this.checkReset[i]){  //If setting prev checked show option Node 
                        
                        if(optionNode != null){
                            optionNode.setAttribute("style", "display:inline-block;float:right");
                        }

                    }else{  //If setting not checked hide its label

                        if(node != null){
                            node.setAttribute("style", "display:none");
                        }
                        
                    }


                }
                
//                console.log(path);
//                console.log(this.checkReset[i]);
                
                if(checkboxNode != null){ //hide all check boxes
                    checkboxNode.setAttribute("style", "display:none");
                }

            }
            
            
//            this.pathReset = path;
//            this.checkReset = check.checked;                               

            
        },
		//Done changing settings, save selection and hide unselected
		saveChangeSettings : function(){
		    this.Node_2.style.display = "none";   //Hide Extended Customize Settings
		    this.Node_1.style.display = "inline"; //Show Extended Edit
		    
		    this.excludeMe = new Array(); //reset suppress array
            this.numOfIncludedSettings = 0;
		    
            this.hideParentNodes(); //auto hide all parents and only show if one child is checked

	          for(var i=0; i<this.paths.length; i++){  //Loop path settings
	              var path = this.paths[i];

                  var node = document.getElementById(path + "_node");
	              var optionNode = document.getElementById(path + "_optionNode");
                  var checkboxNode = document.getElementById(path + "_checkboxNode");
                  var check = document.getElementById(path + "_check");
                  var suppressAsuSet = "yes";
                  
                  var paths = this._getAllPaths(path); //Does path have children
                  
                  if(node != null && paths.length != 0){ //If parent path then loop children to set parent display

                      dijit.byId(path + "_check").set("checked", false); //Auto set parent to hide
                      
                      for(var y=0; y<paths.length; y++){  //Loop children and only show parent if atleast one child checked
                          var childpath = paths[y];
                          var childcheck = document.getElementById(childpath + "_check");
                          if(childcheck != null){
                              if(childcheck.checked){ //If one child checked then parent is visible                               
                                  dijit.byId(path + "_check").set("checked", true); //Set parent visible to true
                                  suppressAsuSet = "no";
                              }    
                          }
                          
                      }
                      
                      if(suppressAsuSet == "yes")
                      this.excludeMe.push(path+"__asu__set");

                  }

                  
                  
                  if(check != null){
                      if(check.checked){  //If setting checked show option Node and don't hide node 
                          
                          if(optionNode != null){
                              optionNode.setAttribute("style", "display:inline-block;float:right");
                              this.numOfIncludedSettings++;
                          }

                      }else{  //If setting not checked hide it and add to suppress array

                          if(node != null){
                              this.excludeMe.push(path);
                              node.setAttribute("style", "display:none");
                          }
                          
                      }
                  }
	              
	              if(checkboxNode != null){ //hide all check boxes
	                    checkboxNode.setAttribute("style", "display:none");
                  }

	          }

	 //           console.log(this.numOfIncludedSettings);

	            //Set number of Included Settings
	           // this.includedContainer.innerHTML = this.numOfIncludedSettings;

    //          console.log(this.excludeMe);

	            this._widgetsShow();
		},
        //Hide Parent Nodes
        hideParentNodes : function(){

            for(var i=0; i<this.paths.length; i++){  //Loop path settings to set all parents as false
                
                var path = this.paths[i];                  
                var node = document.getElementById(path + "_node");
                
                var paths = this._getAllPaths(path); //Does path have children
                                  
                if(node != null && paths.length != 0){ //If parent path then loop children to set parent display
                                      
                    dijit.byId(path + "_check").set("checked", false); //Auto set parent to hide
                }
            }

        },
		//Hide Parent Titles on page load that don't have children settings selected
        hideParentChangeSettings : function(){

            this.hideParentNodes(); //auto hide all parents and only show if one child is checked
            
              for(var i=0; i<this.paths.length; i++){  //Loop path settings
                  var path = this.paths[i];                  
                  var node = document.getElementById(path + "_node");
                  var optionNode = document.getElementById(path + "_optionNode");
                  var checkboxNode = document.getElementById(path + "_checkboxNode");
                  var check = document.getElementById(path + "_check");
                  
                  var paths = this._getAllPaths(path); //Does path have children
                                    
                  if(node != null && paths.length != 0){ //If parent path then loop children to set parent display
                                        
                      dijit.byId(path + "_check").set("checked", false); //Auto set parent to hide

                      for(var y=0; y<paths.length; y++){  //Loop children and only show parent if atleast one child checked
                          var childpath = paths[y];
                          var childcheck = document.getElementById(childpath + "_check");
                          if(childcheck != null){
                              if(childcheck.checked){ //If one child checked then parent is visible 
                                  dijit.byId(path + "_check").set("checked", true); //Set parent visible to true
                              }    
                          }
                          
                      }
                      
                      if(check != null){
                          if(check.checked){  //If setting checked show option Node and don't hide node 
                              
                              if(optionNode != null){
                                  optionNode.setAttribute("style", "display:inline-block;float:right");
                              }

                          }else{  //If setting not checked hide it and add to suppress array

                              if(node != null){
                                  this.excludeMe.push(path);
                                  this.excludeMe.push(path+"__asu__set");
                                  node.setAttribute("style", "display:none");
                              }
                              
                          }
                      }

                  }

              }

        },

		//Select all setting options to add to pattern 
		_selectAll : function(){
            for(var i=0; i<this.paths.length; i++){
                var path = this.paths[i];

                var index = dojo.indexOf(this.suppressMe, path);  

                if(index == -1){
                
                    var check = document.getElementById(path + "_check");
                    if(check != null && check.disabled != true){
                        dijit.byId(path + "_check").set("checked", true);
                    }        
                
                }
            }		    
		},
        //Deselect all setting options to remove from pattern 
        _deselectAll : function(){
            for(var i=0; i<this.paths.length; i++){
                var path = this.paths[i];

                var index = dojo.indexOf(this.suppressMe, path);  

                if(index == -1){
                    var check = document.getElementById(path + "_check");

                    if(check != null){
                        dijit.byId(path + "_check").set("checked", false);
                    }
                
                }
            }           
        },       
        checkValidation: function(){
            //Assume validation is true until proven false
            var validationCheck = true;

            
            this.excludeMe = new Array(); //reset suppress array
            //this.numOfIncludedSettings = 0;
            
            this.hideParentNodes(); //auto hide all parents and only show if one child is checked

              for(var i=0; i<this.paths.length; i++){  //Loop path settings
                  var path = this.paths[i];

                  var index = dojo.indexOf(this.suppressMe, path);  

                  if(index == -1){

                      var node = document.getElementById(path + "_node");
                      var optionNode = document.getElementById(path + "_optionNode");
                      var checkboxNode = document.getElementById(path + "_checkboxNode");
                      var check = document.getElementById(path + "_check");
                      var suppressAsuSet = "yes";
                      
                      var paths = this._getAllPaths(path); //Does path have children
                      
                      if(node != null && paths.length != 0){ //If parent path then loop children to set parent display

                          dijit.byId(path + "_check").set("checked", false); //Auto set parent to hide
                          
                          for(var y=0; y<paths.length; y++){  //Loop children and only show parent if atleast one child checked
                              var childpath = paths[y];
                              var childcheck = document.getElementById(childpath + "_check");
                              if(childcheck != null){
                                  if(childcheck.checked){ //If one child checked then parent is visible                               
                                      dijit.byId(path + "_check").set("checked", true); //Set parent visible to true
                                      suppressAsuSet = "no";
                                  }    
                              }
                              
                          }
                          
                          if(suppressAsuSet == "yes")
                            this.excludeMe.push(path+"__asu__set");

                      }

                                                          
                      if(check != null){
                          if(check.checked){  //If setting checked show option Node and don't hide node 
                      
                              if(optionNode != null){
                                  var widget = dijit.byId(path);
                                  if(widget != null){
                                      var declaredClass = widget.declaredClass;
                                      if(declaredClass == "dijit.form.ValidationTextBox"){
                                          if(widget.isValid() == false){
                                              widget.focus();
                                              validationCheck = false;
                                          }
                                      }else if(declaredClass == "support.panels.SimpleTable"){
                                          if(widget.isValid() == false){                                
                                              validationCheck = false;
                                          }
                                      }

                                  }

                              }

                          }else{  //If setting not checked hide it and add to suppress array

                              if(node != null){
                                  this.excludeMe.push(path);
                              }
                              
                          }
                      }

                  

                  }
              }

                        
            return validationCheck;
        },
          setSuppressIf: function(){
            var path = "";  
            var suppressif = "";  
			
		  //Loop through and execute suppress-if              
           for(var p in this.pathWithSuppress){
           
            path = this.pathWithSuppress[p];  
            suppressif = this.idToSuppress[path];  
            
            if(suppressif == "false" || suppressif == "true"){

                if(suppressif == "true"){
                    
                    var suppressSearch = dojo.indexOf(this.suppressMe, path);  
                    if(suppressSearch == -1)
                        this.suppressMe.push(path);
                    
                    if(dojo.byId(path + "_optionNode"))
                    dojo.byId(path + "_optionNode").style.display = "none";

                    if(dojo.byId(path + "_node")){
                        dojo.byId(path + "_node").style.display = "none";

                        //if(dojo.byId(path + "_check"))
                          //  dijit.byId(path + "_check").set("checked", false); //show parent

                            var paths = this._getAllPaths(path); //Does path have children
                              
                            for(var y=0; y<paths.length; y++){  //Loop children and show

                                if(dojo.byId(paths[y] + "_optionNode"))
                                    dojo.byId(paths[y] + "_optionNode").style.display = "none";
                                    
                                if(dojo.byId(paths[y] + "_node"))
                                    dojo.byId(paths[y] + "_node").style.display = "none";

                           //     if(dojo.byId(paths[y] + "_check"))
                             //       dijit.byId(paths[y] + "_check").set("checked", false);

                                var suppressSearch = dojo.indexOf(this.suppressMe, paths[y]);  

                                if(suppressSearch == -1)
                                    this.suppressMe.push(paths[y]);

                            }

                    }

                    if(dijit.byId(path + "_check")){

                        if(dijit.byId(path + "_check").checked)
                        this.numOfExcludedSettings++;

                       // dijit.byId(path + "_check").set("checked", false);
                    }

                }
                                               
            }else{

                //Split the Suppress If to find the ID of the widget its related to,
                //to find the correct value of that suppress if
                var suppressData = suppressif.split("||"); 
                var suppressReplaced;
                var suppressArray = new Array();
                var pathArray = new Array();

                //Loop suppress if object to find the ID's                    
                for(var x=0;x<suppressData.length;x++){
                    suppressReplaced = suppressData[x].replace("!","");
                    suppressReplaced = suppressData[x].replace("(","");
                    //console.log(suppressReplaced);
                
                    var suppressIDArray = suppressReplaced.split(" ");                        
                    // console.log("IDArray:"+suppressIDArray);

                    if(x==0){
                        var suppressID = suppressIDArray[0].split("(").join(" ");
                    }else{
                        var suppressID = suppressIDArray[1].split("(").join(" ");
                    }

                    suppressID = suppressID.replace(" ","");

                    suppressID = dojo.trim(suppressID);
                
                    suppressArray.push(suppressID);
              //      console.log("ID:"+suppressID);

                    if(dojo.indexOf(suppressID,".") !== -1)
                    {
                        var suppressIDArray2 = suppressID.split(".");                        
                        suppressID = suppressIDArray2[1];
                    }
                         
                    pathArray.push(this.idToPath[suppressID]);

                }

                //Loop through found ID paths
                var groupPath = path;

                
                for(var z=0;z<pathArray.length;z++){

            
                    //console.log("ID With Path:"+pathArray[z]);
            
                    var newsuppressif="";

                    //Find related value of found ID paths
                    for(var y=0;y<suppressArray.length;y++){
                      //console.log(value);
                      var compareValue = this.getValueFromDisplay(pathArray[y]);
                      //console.log(compareValue);

                      if(compareValue != null){

                          if(y==0){
                              newsuppressif = suppressif.replace(suppressArray[y],"'"+compareValue+"'");
                          }else{
                              newsuppressif = newsuppressif.replace(suppressArray[y],"'"+compareValue+"'");                                    
                          }
                      
                      }
                      //console.log("New Sup If: "+newsuppressif);
                    }

                    //Evaluate suppress if with found values                        
                    if(eval(newsuppressif)){
                      //Hide if suppress if true
                        //console.log("HIDE ME");
                        //console.log(path);

                        var suppressSearch = dojo.indexOf(this.suppressMe, path);  

                        if(suppressSearch == -1)
                            this.suppressMe.push(path);
                   
                        if(dojo.byId(path + "_optionNode"))
                        dojo.byId(path + "_optionNode").style.display = "none";

                        if(dojo.byId(path + "_node"))
                        dojo.byId(path + "_node").style.display = "none";

                        if(dojo.byId(groupPath + "_node")){
                            dojo.byId(groupPath + "_node").style.display = "none";

                            //if(dojo.byId(groupPath + "_check"))
                              //  dijit.byId(groupPath + "_check").set("checked", false); //show parent

                                var paths = this._getAllPaths(groupPath); //Does path have children
                                  
                                for(var y=0; y<paths.length; y++){  //Loop children and show

                                    if(dojo.byId(paths[y] + "_optionNode"))
                                        dojo.byId(paths[y] + "_optionNode").style.display = "none";
                                        
                                    if(dojo.byId(paths[y] + "_node"))
                                        dojo.byId(paths[y] + "_node").style.display = "none";

                          //          if(dojo.byId(paths[y] + "_check"))
                        //                dijit.byId(paths[y] + "_check").set("checked", false);

                                    var suppressSearch = dojo.indexOf(this.suppressMe, paths[y]);  
                                    if(suppressSearch == -1)
                                        this.suppressMe.push(paths[y]);

                                }

                        }

                        if(dijit.byId(path + "_check")){

                            if(dijit.byId(path + "_check").checked)
                            this.numOfExcludedSettings++;

                            //dijit.byId(path + "_check").set("checked", false);
                        }


                    }else{
                      //Show if suppress-if not true
                       //console.log("SHOW ME");
                       //console.log(path);

                        var index2 = dojo.indexOf(this.suppressMe, path);  
                        this.suppressMe.splice(index2, 1);                              

                        var index = dojo.indexOf(this.excludeMe, path);  
                        
                        if(index == -1){

                            if(dojo.byId(path + "_optionNode"))
                                dojo.byId(path + "_optionNode").setAttribute("style", "display:inline-block;float:right");
                                
                                if(dojo.byId(path + "_node"))
                                dojo.byId(path + "_node").style.display = "block";

                                if(dojo.byId(groupPath + "_node")){
                                    dojo.byId(groupPath + "_node").style.display = "block";

                              //      if(dojo.byId(groupPath + "_check"))
                                //        dijit.byId(groupPath + "_check").set("checked", true); //show parent

                                        var paths = this._getAllPaths(groupPath); //Does path have children
                                          
                                        for(var y=0; y<paths.length; y++){  //Loop children and show

                                            var index4 = dojo.indexOf(this.excludeMe, paths[y]);  

                                            if(index4 == -1){

                                                var index3 = dojo.indexOf(this.suppressMe, paths[y]);  

                                                this.suppressMe.splice(index3, 1);                              

                                                if(dojo.byId(paths[y] + "_optionNode"))
                                                    dojo.byId(paths[y] + "_optionNode").setAttribute("style", "display:inline-block;float:right");
                                                
                                                if(dojo.byId(paths[y] + "_node"))
                                                    dojo.byId(paths[y] + "_node").style.display = "block";

                                  //          if(dojo.byId(paths[y] + "_check"))
                                    //            dijit.byId(paths[y] + "_check").set("checked", true);
                                            }
                                        }

                                }

                                if(dijit.byId(path + "_check")){

                                    if(!dijit.byId(path + "_check").checked)                             
                                    this.numOfExcludedSettings--;

                                   // dijit.byId(path + "_check").set("checked", true);
                                }

                            }

                           //console.log(this.excludeMe);
                    }
                    this.setSuppressIfConnect(pathArray[z],path,groupPath,suppressArray,suppressif,pathArray);
                  
                }
            }// End true false if
            
            
           }
        },
        setSuppressIfConnect: function(connectPath,path,groupPath,suppressArray,suppressif,pathArray){

            var isConnectButtonBarWidget = false;            
            var connectWidget = dijit.byId(connectPath);
            if(connectWidget != null){
                if(connectWidget.declaredClass == "support.widgets.ButtonBar"){
                    isConnectButtonBarWidget = true;
                }
            }
            
            if(isConnectButtonBarWidget){

                //Connect related path to onClick event to execute suppress-if if value changes during runtime
                dojo.connect(dijit.byId(connectPath), "onClick", this, function(value){
                    
                    //console.log(path);
                                         
                    var newsuppressif="";
                    
                    for(var y=0;y<suppressArray.length;y++){
//                        console.log(value);
                        var compareValue = this.getValueFromDisplay(pathArray[y]);
                        //console.log(compareValue);

                        if(compareValue != null){

                            if(y==0){
                                newsuppressif = suppressif.replace(suppressArray[y],"'"+compareValue+"'");
                            }else{
                                newsuppressif = newsuppressif.replace(suppressArray[y],"'"+compareValue+"'");                                    
                            }
                        
                        }
                        //console.log(newsuppressif);
                    }


                    //console.log("HERE IS THE PATH: "+path);
                    
                    if(eval(newsuppressif)){
                          //console.log("HIDE ME");
                          //console.log(path);
                          //console.log(groupPath);
                          
                        var suppressSearch = dojo.indexOf(this.suppressMe, path);  

                        if(suppressSearch == -1)
                          this.suppressMe.push(path);

                          if(dojo.byId(path + "_optionNode"))
                          dojo.byId(path + "_optionNode").style.display = "none";
                          
                          if(dojo.byId(path + "_node"))
                          dojo.byId(path + "_node").style.display = "none";

                          if(dojo.byId(groupPath + "_node")){
                              
                              dojo.byId(groupPath + "_node").style.display = "none";

                              //if(dojo.byId(groupPath + "_check"))
                              //dijit.byId(groupPath + "_check").set("checked", false); //Auto set parent to hide

                              var paths = this._getAllPaths(groupPath); //Does path have children
                                
                              for(var y=0; y<paths.length; y++){  //Loop children and hide

                                  if(dojo.byId(paths[y] + "_optionNode"))
                                      dojo.byId(paths[y] + "_optionNode").style.display = "none";
                                      
                                  if(dojo.byId(paths[y] + "_node"))
                                      dojo.byId(paths[y] + "_node").style.display = "none";

                                //  if(dojo.byId(paths[y] + "_check"))
                                  //    dijit.byId(paths[y] + "_check").set("checked", false);

                                  var suppressSearch = dojo.indexOf(this.suppressMe, paths[y]);  
                                  
                                  if(suppressSearch == -1)
                                      this.suppressMe.push(paths[y]);
                                  
                              }

                          }

                          //if(dojo.byId(path + "_check"))
                          //dijit.byId(path + "_check").set("checked", false);

                     }else{
                         //console.log("SHOW ME");
                         //console.log(path);
                         //console.log(groupPath);
                         
                         if(dojo.byId(path + "_check")){
                             if(!dijit.byId(path + "_check").disabled){  //If option disabled then don't run suppress, this currently can
                                                                         //only happen if this is a setting dependent on operating modes
  //                               var index = this.excludeMe.indexOf(path);
    //                             this.excludeMe.splice(index, 1);                              

                                 var index2 = dojo.indexOf(this.suppressMe, path);  

                                 this.suppressMe.splice(index2, 1);                              

                                 var index = dojo.indexOf(this.excludeMe, path);  

                                 
                                 if(index == -1){

                                     if(dojo.byId(path + "_optionNode"))
                                         dojo.byId(path + "_optionNode").setAttribute("style", "display:inline-block;float:right");
                                     
                                         if(dojo.byId(path + "_node"))
                                         dojo.byId(path + "_node").style.display = "block";

                                         if(dojo.byId(groupPath + "_node")){
                                             dojo.byId(groupPath + "_node").style.display = "block";

                            //                 if(dojo.byId(groupPath + "_check"))
                              //                   dijit.byId(groupPath + "_check").set("checked", true); //show parent

                                                 var paths = this._getAllPaths(groupPath); //Does path have children
                                                   
                                                 for(var y=0; y<paths.length; y++){  //Loop children and show

                                                     var index3 = dojo.indexOf(this.suppressMe, paths[y]);  

                                                     this.suppressMe.splice(index3, 1);                              

                                                     var index4 = dojo.indexOf(this.excludeMe, paths[y]);  

                                                     
                                                     if(index4 == -1){

                                                         if(dojo.byId(paths[y] + "_optionNode"))
                                                             dojo.byId(paths[y] + "_optionNode").setAttribute("style", "display:inline-block;float:right");
                                                         
                                                         if(dojo.byId(paths[y] + "_node"))
                                                             dojo.byId(paths[y] + "_node").style.display = "block";

                                //                     if(dojo.byId(paths[y] + "_check"))
                                  //                       dijit.byId(paths[y] + "_check").set("checked", true);
                                                     }
                                                 }

                                         }

                                    //     if(dojo.byId(path + "_check"))
                                     //    dijit.byId(path + "_check").set("checked", true);

                                     
                                 }else{

                                     if(dojo.byId(groupPath + "_node")){

                                             var paths = this._getAllPaths(groupPath); //Does path have children
                                               
                                             for(var y=0; y<paths.length; y++){  //Loop children and show

                                                 var index3 = dojo.indexOf(this.suppressMe, paths[y]);  

                                                 this.suppressMe.splice(index3, 1);                              

                                             }

                                     }
                                 }

                             }
                         }else{

//                             var index = this.excludeMe.indexOf(path);
  //                           this.excludeMe.splice(index, 1);                              

                             var index2 = dojo.indexOf(this.suppressMe, path);  

                             this.suppressMe.splice(index2, 1);                              

                             var index = dojo.indexOf(this.excludeMe, path);  
                             
                             if(index == -1){
                             
                                 if(dojo.byId(path + "_optionNode"))
                                     dojo.byId(path + "_optionNode").setAttribute("style", "display:inline-block;float:right");
                                 
                                     if(dojo.byId(path + "_node"))
                                     dojo.byId(path + "_node").style.display = "block";

                                     if(dojo.byId(groupPath + "_node")){
                                         dojo.byId(groupPath + "_node").style.display = "block";

                                       //  if(dojo.byId(groupPath + "_check"))
                                        //     dijit.byId(groupPath + "_check").set("checked", true); //show parent

                                             var paths = this._getAllPaths(groupPath); //Does path have children
                                               
                                             for(var y=0; y<paths.length; y++){  //Loop children and show

                                                 var index3 = dojo.indexOf(this.suppressMe, paths[y]);  

                                                 this.suppressMe.splice(index3, 1);                              

                                                 var index4 = dojo.indexOf(this.excludeMe, paths[y]);  

                                                 
                                                 if(index4 == -1){

                                                     if(dojo.byId(paths[y] + "_optionNode"))
                                                         dojo.byId(paths[y] + "_optionNode").setAttribute("style", "display:inline-block;float:right");
                                                     
                                                     if(dojo.byId(paths[y] + "_node"))
                                                         dojo.byId(paths[y] + "_node").style.display = "block";

                                                     //if(dojo.byId(paths[y] + "_check"))
                                                         // dijit.byId(paths[y] + "_check").set("checked", true);
                                                 }

                                             }

                                     }

                                    // if(dojo.byId(path + "_check"))
                                    // dijit.byId(path + "_check").set("checked", true);

                                 
                             }else{
                                 

                                 if(dojo.byId(groupPath + "_node")){
                                         var paths = this._getAllPaths(groupPath); //Does path have children
                                           
                                         for(var y=0; y<paths.length; y++){  //Loop children and show

                                             var index3 = dojo.indexOf(this.suppressMe, paths[y]);  

                                             this.suppressMe.splice(index3, 1);                              

                                         }

                                 }
                             }
                         }
                         
                     }

                    //console.log(this.excludeMe);

                });

                
            }else{

                //Connect related path to onClick event to execute suppress-if if value changes during runtime
                dojo.connect(dijit.byId(connectPath), "onChange", this, function(value){
                    
                    //console.log(path);
                                         
                    var newsuppressif="";
                    
                    for(var y=0;y<suppressArray.length;y++){
//                        console.log(value);
                        var compareValue = this.getValueFromDisplay(pathArray[y]);
                        //console.log(compareValue);

                        if(compareValue != null){

                            if(y==0){
                                newsuppressif = suppressif.replace(suppressArray[y],"'"+compareValue+"'");
                            }else{
                                newsuppressif = newsuppressif.replace(suppressArray[y],"'"+compareValue+"'");                                    
                            }
                        
                        }
                        //console.log(newsuppressif);
                    }


                    //console.log("HERE IS THE PATH: "+path);
                    
                    if(eval(newsuppressif)){
                          //console.log("HIDE ME");
                          //console.log(path);
                          //console.log(groupPath);
                          
//                          this.excludeMe.push(path);
                        var suppressSearch = dojo.indexOf(this.suppressMe, path);  

                        if(suppressSearch == -1)
                          this.suppressMe.push(path);

                          if(dojo.byId(path + "_optionNode"))
                          dojo.byId(path + "_optionNode").style.display = "none";
                          
                          if(dojo.byId(path + "_node"))
                          dojo.byId(path + "_node").style.display = "none";

                          if(dojo.byId(groupPath + "_node")){
                              
                              dojo.byId(groupPath + "_node").style.display = "none";

                              //if(dojo.byId(groupPath + "_check"))
                              //dijit.byId(groupPath + "_check").set("checked", false); //Auto set parent to hide

                              var paths = this._getAllPaths(groupPath); //Does path have children
                                
                              for(var y=0; y<paths.length; y++){  //Loop children and hide

                                  if(dojo.byId(paths[y] + "_optionNode"))
                                      dojo.byId(paths[y] + "_optionNode").style.display = "none";
                                      
                                  if(dojo.byId(paths[y] + "_node"))
                                      dojo.byId(paths[y] + "_node").style.display = "none";

                                //  if(dojo.byId(paths[y] + "_check"))
                                  //    dijit.byId(paths[y] + "_check").set("checked", false);

                                  var suppressSearch = dojo.indexOf(this.suppressMe, paths[y]);  

                                  if(suppressSearch == -1)
                                      this.suppressMe.push(paths[y]);

                              }

                          }

                          //if(dojo.byId(path + "_check"))
                          //dijit.byId(path + "_check").set("checked", false);

                     }else{
                         //console.log("SHOW ME");
                         //console.log(path);
                         //console.log(groupPath);
                         
                         if(dojo.byId(path + "_check")){
                             if(!dijit.byId(path + "_check").disabled){  //If option disabled then don't run suppress, this currently can
                                                                         //only happen if this is a setting dependent on operating modes
  //                               var index = this.excludeMe.indexOf(path);
    //                             this.excludeMe.splice(index, 1);                              

                                 var index2 = dojo.indexOf(this.suppressMe, path);  

                                 this.suppressMe.splice(index2, 1);                              

                                 var index = dojo.indexOf(this.excludeMe, path);  
                                 
                                 if(index == -1){
                                 
                                     if(dojo.byId(path + "_optionNode"))
                                         dojo.byId(path + "_optionNode").setAttribute("style", "display:inline-block;float:right");
                                     
                                         if(dojo.byId(path + "_node"))
                                         dojo.byId(path + "_node").style.display = "block";

                                         if(dojo.byId(groupPath + "_node")){
                                             dojo.byId(groupPath + "_node").style.display = "block";

                            //                 if(dojo.byId(groupPath + "_check"))
                              //                   dijit.byId(groupPath + "_check").set("checked", true); //show parent

                                                 var paths = this._getAllPaths(groupPath); //Does path have children
                                                   
                                                 for(var y=0; y<paths.length; y++){  //Loop children and show

                                                     var index3 = dojo.indexOf(this.suppressMe, paths[y]);  

                                                     this.suppressMe.splice(index3, 1);                              

                                                     var index4 = dojo.indexOf(this.excludeMe, paths[y]);  

                                                     
                                                     if(index4 == -1){

                                                         if(dojo.byId(paths[y] + "_optionNode"))
                                                             dojo.byId(paths[y] + "_optionNode").setAttribute("style", "display:inline-block;float:right");
                                                         
                                                         if(dojo.byId(paths[y] + "_node"))
                                                             dojo.byId(paths[y] + "_node").style.display = "block";

                                //                     if(dojo.byId(paths[y] + "_check"))
                                  //                       dijit.byId(paths[y] + "_check").set("checked", true);
                                                     }
                                                     
                                                 }

                                         }

                                   //      if(dojo.byId(path + "_check"))
                                     //    dijit.byId(path + "_check").set("checked", true);

                                     
                                 }else{
                                     
                                     
                                     if(dojo.byId(groupPath + "_node")){
                                         
                                         var paths = this._getAllPaths(groupPath); //Does path have children
                                           
                                         for(var y=0; y<paths.length; y++){  //Loop children and show

                                             var index3 = dojo.indexOf(this.suppressMe, paths[y]);  

                                             this.suppressMe.splice(index3, 1);                              
                                         }

                                     }

                                     
                                 }
                                 
                             }
                         }else{

   //                          var index = this.excludeMe.indexOf(path);
     //                        this.excludeMe.splice(index, 1);                              

                             var index2 = dojo.indexOf(this.suppressMe, path);  

                             this.suppressMe.splice(index2, 1);                              

                             var index = dojo.indexOf(this.excludeMe, path);  
                             
                             if(index == -1){

                                 if(dojo.byId(path + "_optionNode"))
                                     dojo.byId(path + "_optionNode").setAttribute("style", "display:inline-block;float:right");
                                 
                                     if(dojo.byId(path + "_node"))
                                     dojo.byId(path + "_node").style.display = "block";

                                     if(dojo.byId(groupPath + "_node")){
                                         dojo.byId(groupPath + "_node").style.display = "block";

                                    //     if(dojo.byId(groupPath + "_check"))
                                      //       dijit.byId(groupPath + "_check").set("checked", true); //show parent

                                             var paths = this._getAllPaths(groupPath); //Does path have children
                                               
                                             for(var y=0; y<paths.length; y++){  //Loop children and show

                                                 var index3 = dojo.indexOf(this.suppressMe, paths[y]);  

                                                 this.suppressMe.splice(index3, 1);                              

                                                 var index4 = dojo.indexOf(this.excludeMe, paths[y]);  

                                                 if(index4 == -1){

                                                     if(dojo.byId(paths[y] + "_optionNode"))
                                                         dojo.byId(paths[y] + "_optionNode").setAttribute("style", "display:inline-block;float:right");
                                                     
                                                     if(dojo.byId(paths[y] + "_node"))
                                                         dojo.byId(paths[y] + "_node").style.display = "block";

                                        //         if(dojo.byId(paths[y] + "_check"))
                                          //           dijit.byId(paths[y] + "_check").set("checked", true);
                                                 }

                                             }

                                     }

                                     //if(dojo.byId(path + "_check"))
                                     //dijit.byId(path + "_check").set("checked", true);

                                 
                             }else{
                                 
                                 if(dojo.byId(groupPath + "_node")){
                                         var paths = this._getAllPaths(groupPath); //Does path have children
                                           
                                         for(var y=0; y<paths.length; y++){  //Loop children and show

                                             var index3 = dojo.indexOf(this.suppressMe, paths[y]);  

                                             this.suppressMe.splice(index3, 1);                              


                                         }

                                 }
                             
                             }  
                         }
                         
                     }

                    //console.log(this.excludeMe);

                });

                
            }

        },
        setGrayIf: function(){

            var path = "";
            var grayif = "";

		  //Loop through and execute gray-if                          
           for(var w in this.pathWithGray){

                 path = this.pathWithGray[w];  
                 grayif = this.idToGray[path];  
                             
            //Split the Gray If to find the ID of the widget its related to,
            //to find the correct value of that gray if
            var grayData = grayif.split("||");
            var grayReplaced;
            var grayArray = new Array();
            var pathArray = new Array();

            //Loop gray if object to find the ID's                    
            for(var x=0;x<grayData.length;x++){
                grayReplaced = grayData[x].replace("!","");
                grayReplaced = grayData[x].replace("(","");
               // console.log(grayReplaced);
                
                var grayIDArray = grayReplaced.split(" ");                        
               // console.log("IDArray:"+grayIDArray);

                if(x==0){
                    var grayID = grayIDArray[0].split("(").join(" ");
                }else{
                    var grayID = grayIDArray[1].split("(").join(" ");                          
                }

                grayID = grayID.replace(" ","");

                grayID = dojo.trim(grayID);
  
                grayArray.push(grayID);
                  
                if(dojo.indexOf(grayID,".") !== -1)
                {
                    var grayIDArray2 = grayID.split(".");                        
                    grayID = grayIDArray2[1];
                }
                                       
                pathArray.push(this.idToPath[grayID]);
  //              console.log(this.idToPath);
                
            }

            //Loop through found ID paths
             var groupPath = path;
            
            //console.log(grayArray);
            //console.log(pathArray2);
            
            
            //console.log("**********************************************************************");

             //Loop through found ID paths
             for(var z=0;z<pathArray.length;z++){

                 var newgrayif="";
                 
                 var operatingMode = false;

                 //Find related value of found ID paths
                 for(var y=0;y<grayArray.length;y++){
//                     console.log(value);
                     var compareValue = this.getValueFromDisplay(pathArray[y]);

                     if(compareValue != null){

                         //console.log("GRAY ARRAY");
                         //console.log(grayArray[y]);

                         grayArray[y] = grayArray[y].replace("!","");

                         if(y==0){
                             newgrayif = grayif.replace(grayArray[y],"'"+compareValue+"'");
                         }else{
                             newgrayif = newgrayif.replace(grayArray[y],"'"+compareValue+"'");                                    
                         }

                     }else if(grayArray[y] == "!OperatingModes.ChooseOperatingMode" && this.performanceOpMode != "" && this.performanceRelated != null && this.performanceRelated > 0){
                         //console.log(grayArray[y]);                              
                         //console.log(this.performanceRelated);
                         
                         compareValue = this.performanceOpMode;
                         
                         grayArray[y] = grayArray[y].replace("!","");

                         if(y==0){
                             newgrayif = grayif.replace(grayArray[y],"'"+compareValue+"'");
                         }else{
                             newgrayif = newgrayif.replace(grayArray[y],"'"+compareValue+"'");                                    
                         }
                         
                         operatingMode = true;
                     }
                     
                 //   console.log("New Gray If: "+newgrayif);

                 }


                 //Evaluate suppress if with found values                        
                 if(eval(newgrayif)){

                     if(operatingMode){

                      //   console.log("HIDE ME");
                      //   console.log(path);

                         this.excludeMe.push(path);
                         if(dojo.byId(path + "_optionNode"))
                         dojo.byId(path + "_optionNode").style.display = "none";

                         if(dojo.byId(path + "_node"))
                         dojo.byId(path + "_node").style.display = "none";

                         if(dijit.byId(path + "_check")){

                             if(dijit.byId(path + "_check").checked)
                                 this.numOfExcludedSettings++;

                             dijit.byId(path + "_check").set("checked", false);
                             dijit.byId(path + "_check").set("disabled",true);
                             
                         }
                         
                     }else{

                      //   console.log("Gray ME");
                       //  console.log(path);

                         if(dijit.byId(path)){
                             
                             var widget = dijit.byId(path);
                             if(widget != null){
                                 widget.set("disabled",true);                                 
                             }

                             
                         }

                     }
                       
                  }else{
                    //  console.log("Dont Gray ME");
                    //  console.log(path);

                      if(dijit.byId(path)){
                          
                          var widget = dijit.byId(path);
                          if(widget != null){
                              widget.set("disabled",false);                         
                          }

                          
                      }

                  }
                 
                 this.setGrayIfConnect(pathArray[z],path,groupPath,grayArray,grayif,pathArray);

             }     
           }
        },
        setGrayIfConnect: function(connectPath,path,groupPath,grayArray,grayif,pathArray){

            var isConnectButtonBarWidget = false;            
            var connectWidget = dijit.byId(connectPath);
            if(connectWidget != null){
                if(connectWidget.declaredClass == "support.widgets.ButtonBar"){
                    isConnectButtonBarWidget = true;
                }
            }
            
            if(isConnectButtonBarWidget){

                //Connect related path to onClick event to execute gray-if if value changes during runtime                        
                dojo.connect(dijit.byId(connectPath), "onClick", this, function(value){
                    
                    var newgrayif="";
                    
                    for(var y=0;y<grayArray.length;y++){
                        //console.log(value);
                        var compareValue = this.getValueFromDisplay(pathArray[y]);
                    //    console.log("COMPARE VALUE");
                      //  console.log(compareValue);

                        if(compareValue != null){

                            if(y==0){
                                newgrayif = grayif.replace(grayArray[y],"'"+compareValue+"'");
                            }else{
                                newgrayif = newgrayif.replace(grayArray[y],"'"+compareValue+"'");                                    
                            }
                        
                        }
                     //   console.log("New Gray If: "+newgrayif);
                    }

                    
                    if(eval(newgrayif)){

        //                console.log("Gray ME");
          //              console.log(path);

                        if(dijit.byId(path)){
                            
                            var widget = dijit.byId(path);
                            if(widget != null){
                                widget.set("disabled",true);
                            }

                            
                        }

                     }else{
            //            console.log("Dont Gray ME");
              //          console.log(path);

                         if(dijit.byId(path)){
                             
                             var widget = dijit.byId(path);
                             if(widget != null){
                                 widget.set("disabled",false);                                 
                             }

                             
                         }
                     }

                });           

                
            }else{
                
                //Connect related path to onClick event to execute gray-if if value changes during runtime                        
                dojo.connect(dijit.byId(connectPath), "onChange", this, function(value){
                    
                    var newgrayif="";
                    
                    for(var y=0;y<grayArray.length;y++){
                        //console.log(value);
                        var compareValue = this.getValueFromDisplay(pathArray[y]);
                    //    console.log("COMPARE VALUE");
                      //  console.log(compareValue);

                        if(compareValue != null){

                            if(y==0){
                                newgrayif = grayif.replace(grayArray[y],"'"+compareValue+"'");
                            }else{
                                newgrayif = newgrayif.replace(grayArray[y],"'"+compareValue+"'");                                    
                            }
                        
                        }
                     //   console.log("New Gray If: "+newgrayif);
                    }

                    
                    if(eval(newgrayif)){

        //                console.log("Gray ME");
          //              console.log(path);

                        if(dijit.byId(path)){
                            
                            var widget = dijit.byId(path);
                            if(widget != null){
                                widget.set("disabled",true);                                
                            }

                            
                        }

                     }else{
            //            console.log("Dont Gray ME");
              //          console.log(path);

                         if(dijit.byId(path)){
                             
                             var widget = dijit.byId(path);
                             if(widget != null){
                                 widget.set("disabled",false);                                 
                             }

                             
                         }
                     }

                });           

                
            }
            
        }
	});
});
