define([
        "dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/OrderedList",
        "dojo/text!./templates/OrderedList.html",
        "dojox/grid/DataGrid",
        "dojo/data/ItemFileWriteStore",
        "dojox/grid/cells/_base",
        "support/widgets/OutlinePane",
		"support/widgets/Icon"
    ], function (dojo, dijit, panel, mri, template) {
    
	dojo.declare("support.panels.OrderedList", [ support.panels.Panel ], {

	 // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/OrderedList.css",
        
	    // templateString: [protected] String
	    // This widget's HTML template, provided.
	    templateString: template,
	        
	    // locResources: [protected] Object
	    // This widget's National Language Source (NLS) object.
	    locResources: mri, 
	    
		widgetsInTemplate : true,
		baseClass: "OrderedList",		
		max: 0,
        choices: new Array(),
        options: new Array(),        
        selectedList: new Array(),
        command: "",
        format: "=",
        grid: null,
        store: null,
        layout: null,
        data: null,
        settings: null,
		        
        postMixInProperties : function() {
			this.inherited(arguments);
			this.buildOptions();
			this._connectDataModels();
		},

		postCreate : function(){
		    this.inherited(arguments);		
		    		  		    
		    if(this.choices.length < this.max){
		        this.max = this.choices.length;
		    }
		    
		    //grid
            this.grid = new dojox.grid.DataGrid({
                store: this.store, 
                structure: this.layout,
                autoHeight: true,
                autoWidth: false,
                columnReordering: false,
//                height: "690px",
                canSort: function(col){return false;}
            }).placeAt(this._table);

            this.grid.startup();
            this.grid.store.save();
            this.grid.render();

            this._setIcons();
		},
		
		startup : function(){
		    this.inherited(arguments);
		},
		
		//must have function for Learned Settings
		_show : function(){
		    if(this.grid != null){
		        this.grid.store.save();
	            this.grid.render();
		    }
		},
	   buildOptions: function(){
	       
	       this.options = new Array();
	       
           for(var c=0; c<this.choices.length; c++){
               this.options.push({ label: this.choices[c].label[0], value: this.choices[c].value[0] }); 
           }           
	   },
	    //Sets up the data, store, and layout for the grid
	    _connectDataModels : function(){
	        var newData = new Array();
	       
            this.format = this.choices[0]["format"];
	        
	        if(this.settings == "" || this.settings == null || this.settings === undefined){
	            
	                for(var i=1; i<=this.max; i++){
	                    for(var c=0; c<this.choices.length; c++){
	                        var choice = this.choices[c];
	                        var defaultValue = choice["default"];
	                        var label = choice.value;
	                        if(i == defaultValue){
	                            newData.push({order: i, label: label});
	                            this.selectedList[c] = label;
	                        }
	                    }
	                }

	        }else{

	            this.selectedList=this.settings.split(this.format); 

                for(var i=0; i < this.selectedList.length; i++){
                    newData.push({order: parseInt(i+1), label: this.selectedList[i]});                    
                }

	        }
	        
	        	      
	        //store
            this.store = new dojo.data.ItemFileWriteStore({
                data: {"items" : newData}
            });

            //layout
            this.layout = [
                { field: "order",  name: " ", width: "5%",  noresize: true, editable: false, formatter: this._formatOrder },
                { field: "label",  name: " ", width: "70%",  noresize: true, editable: false, formatter: this._formatOption }
            ];
            
            this.buildCommand();
        },
        
        _formatOrder : function(value, rowIndex, cell){
            var item = this.grid.store._arrayOfTopLevelItems[rowIndex];
            this.grid.store.setValue(item, "order", rowIndex);
            return parseInt(rowIndex) + 1;
        },
        buildCommand : function(){
            this.command = "";
            
            for(var c=0; c<this.selectedList.length; c++){
                if(c==0){
                    this.command = this.selectedList[c];                    
                }else{
                    this.command = this.command + this.format + this.selectedList[c];
                }
            }
            //console.log(this.command);
        },
        //Sets the value in the store
        setValue : function(value, index){
            
            this.selectedList[index] = value;
            
            this.buildCommand();

            this.grid.store._arrayOfTopLevelItems[index].option = value;
                           
            for(var c=0; c<this.options.length; c++){            
                if(this.options[c].value == value){                
                    this.grid.store._arrayOfTopLevelItems[index].label[0] = this.options[c].label;                    
                    break;                    
                }else{                
                    this.grid.store._arrayOfTopLevelItems[index].label[0] = value;                    
                }               
            }          
            
            //console.log(this.grid.store._arrayOfTopLevelItems);
            
            this.grid.store.save();            
            this.grid._refresh(); 
   
            //console.log(this.selectedList);
        },
        _setIcons : function(){
            var index = this.grid.store._arrayOfTopLevelItems.length;

            
            if(index >= this.max){
                this._addIcon.setAttribute("class", "spAddRowInactiveIcon_v2");
            }else{
                this._addIcon.setAttribute("class", "spAddRowIcon_v2");
            }
            
            if(index < 1){
                this._deleteIcon.setAttribute("class", "spDeleteInactiveIcon_v2");
            }else{
                this._deleteIcon.setAttribute("class", "spDeleteIcon_v2");                
            }

        },
        _formatOption : function(value, rowIndex, cell){
            
            var bot = this.grid.getParent();
            
            var availOptions = new Array();                 

            for(var i=0; i<bot.options.length; i++){
                var inArray = dojo.indexOf(bot.selectedList, bot.options[i].value);
                               
                if(inArray == -1 || value == bot.options[i].value)
                availOptions.push({label: bot.options[i].label, value: bot.options[i].value});                                
            }                
            
            var control = dijit.form.Select({
                value: value,
                required: true,
                trim: true,
                style:"width:80%;",
                emptyLabel: "&mdash; " + bot.locResources.SELECT_OPTION + " &mdash;",
                options: availOptions,
                intermediateChanges: true,
                onChange: function(newValue){
                    bot.setValue(newValue, rowIndex);
                }
            });
            control._destroyOnRemove = true;
                    
            
            if(value == ""){
                bot.selectedList[rowIndex] = control.get("value");
                bot.setValue(control.get("value"), rowIndex);
            }else{
                bot.selectedList[rowIndex] = value;
//                bot.setValue(control.get("value"), rowIndex);
            }

            bot._setIcons();

           // console.log(bot.selectedList);

            return control;
        },               
        getItemToAdd : function(index){
                       
            return {order: parseInt(index+1), label: ""};
        },
        //Adds a row to the end of the grid    
        _addRow : function(){

            var length = this.grid.store._arrayOfTopLevelItems.length;
            
            if(length >= this.max){}else{
                this.grid.store.newItem(this.getItemToAdd(length));
                this.grid.store.save();
                this.grid._refresh(); //Refresh Table
            }
              
            this._setIcons();
          
        },
        //Removes a row from the grid
        _removeRow : function(rowIndex){

            var selectedListLength = this.selectedList.length;
            this.selectedList.splice(parseInt(selectedListLength-1),1);    

            if(selectedListLength < 1){}else{

                var item = this.grid.store._arrayOfTopLevelItems[parseInt(selectedListLength-1)];            

                this.grid.store.deleteItem(item);
                this.grid.store.save();            
                this.grid._refresh(); //Refresh Table

            }
            
            this._setIcons();

            this.buildCommand();

          },
        _getValueAttr : function(){
            return this.command;
        }
        
	});
});     	