define([ 
         "dojo", 
         "dijit", 
         "dojo/topic", 
         "support/panels/Panel", 
         "support/base/NextGenUIMixin", 
         "dojo/i18n!./nls/Overview",
         "dojo/text!./templates/Overview.html", 
         "support/widgets/Icon", 
         "dijit/layout/StackContainer",
         "dijit/layout/BorderContainer", 
         "dijit/layout/ContentPane", 
         "support/widgets/HoverButton"
         ], function(dojo, dijit, topic, Panel, NextGenUIMixin, mri, template) {

    return dojo.declare("support.panels.Overview", [ support.panels.Panel, support.base.NextGenUIMixin ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "Overview",

        // the properties used to construct the title of this page in the
        //   NextGen UI
        cpProductName: null,
        cpPageName: null,
        
        serversWelcomeButton: null,
        
        fsmInitialSetup: false,

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/Overview.css",

        // Override this method to perform custom behavior during dijit construction.
        // Common operations for constructor:
        // 1) Initialize non-primitive types (i.e. objects and arrays)
        // 2) Add additional properties needed by succeeding lifecycle methods
        constructor: function() {
            // var cssPath = require.toUrl("support/panels/themes/Overview.css");
            // var link = dojo.create("link", {href:cssPath, type:"text/css", rel:"stylesheet"});
            // document.getElementsByTagName("head")[0].appendChild(link);
        },

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
            this.cpProductName = this.resources.PRODUCT_NAME;
            this.cpPageName = this.locResources.OVERVIEW;
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            
            var currentURL = window.location.hash;
            
            if(currentURL.indexOf("initialsetup=true") != -1){
                this.fsmInitialSetup = true;
            }

            // Prepare Create Server Template icon handlers...
            dojo.attr(this._cstIcon, "expanderTitle", {
                expand: this.locResources.EXPAND,
                collapse: this.locResources.COLLAPSE
            });
            this._cstIcon.setExpanderNode(true, true);
            dojo.connect(this._cstIcon, "onExpanderClick", this, "_showCreateSvrTmplDetails");
            dojo.connect(this._cstCollapseDetailIcon, "onClick", this, function() {
                // NOTE: Need custom handler for this icon since it must also change the state
                // of the Create Server Template expand action node.
                this._showCreateSvrTmplDetails(false);
                this._cstIcon.setExpanderNode(true, true);
            });
            this._showCreateSvrTmplDetails(false);

            // Prepare Server Profiles icon handlers...
            dojo.attr(this._spsIcon, "expanderTitle", {
                expand: this.locResources.EXPAND,
                collapse: this.locResources.COLLAPSE
            });
            this._spsIcon.setExpanderNode(true, true);
            dojo.connect(this._spsIcon, "onExpanderClick", this, "_showSvrPrfsDetails");
            dojo.connect(this._spsCollapseDetailIcon, "onClick", this, function() {
                // NOTE: Need custom handler for this icon since it must also change the state
                // of the Create Server Template expand action node.
                this._showSvrPrfsDetails(false);
                this._spsIcon.setExpanderNode(true, true);
            });
            this._showSvrPrfsDetails(false);

            // Prepare Suggested Tasks switcher...
            if (this._suggestedTasksExist()) {
                // Enable Suggested Tasks link 1
                dojo.addClass(this._suggestedTasksLink1, "linkText");
                dojo.attr(this._suggestedTasksLink1, "title", this.locResources.GO_TO);
                dojo.connect(this._suggestedTasksLink1, "onclick", this, "onGotoSuggestedTasks");
                this._toGetStartedSection.style.display = "inline";
                // Enable Suggested Tasks link 2
                dojo.addClass(this._suggestedTasksLink2, "linkText");
                dojo.attr(this._suggestedTasksLink2, "title", this.locResources.GO_TO);
                dojo.connect(this._suggestedTasksLink2, "onclick", this, "onGotoSuggestedTasks");
                this._selectorMessage.style.display = "inline-block";
                this._selectorMessage2.style.display = "none";
            }
            
            this.createCollapsibleSections();
            
            if(this.fsmInitialSetup){
                this.fromFSMInitialSetup();
            }
            
            this.inherited(arguments);
            
            // finally, invoke the NextGenUIMixin to change the title of the
            // NextGen UI work area
            this.changeNextGenUITitle();

			//If in Augusta framework then remove any reference for chassis from overview
            if(this.CP_RUN_ENVIRONMENT == "augusta"){
                this._chassisWelcome.remove();
                this._chassisTemplate.remove();
                this._chassisDeploy.remove();
                this._chassisProfile.remove();
                this._chassis.remove();
                this._chassisCategory.remove();
                this._createChassisTemplateQuickStart.remove();
            }   
        },
        
        createCollapsibleSections: function() {
            
            // Server Welcome
            this.serversWelcomeButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_serverswelcomebutton",
                iconClass: "spExpandIcon",
                label: this.resources2.SERVERS,
                title: this.resources2.SERVERS,
                onClick: dojo.hitch(this, function(){
                    this._serversWelcome.style.display = "none";
                    this._serversWelcomeText.style.display = "";
                    this._serversWelcomeExpanded.style.display = "";                    
                })
            }).placeAt(this._serversWelcome);

            this.serversWelcomeButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_serverswelcomebutton2",
                iconClass: "spCollapseIcon",
                label: this.resources2.SERVERS,
                title: this.resources2.SERVERS,
                onClick: dojo.hitch(this, function(){
                    this._serversWelcome.style.display = "";
                    this._serversWelcomeText.style.display = "none";
                    this._serversWelcomeExpanded.style.display = "none";
                })
            }).placeAt(this._serversWelcomeExpanded);
            
            // Chassis Welcome
            this.chassisWelcomeButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_chassiswelcomebutton",
                iconClass: "spExpandIcon",
                label: this.resources2.CHASSIS,
                title: this.resources2.CHASSIS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                   this._chassisWelcome.style.display = "none";
                   this._chassisWelcomeText.style.display = "";
                   this._chassisWelcomeExpanded.style.display = "";
                })
            }).placeAt(this._chassisWelcome);
            
            this.chassisWelcomeButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_chassiswelcomebutton2",
                iconClass: "spCollapseIcon",
                label: this.resources2.CHASSIS,
                title: this.resources2.CHASSIS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                   this._chassisWelcome.style.display = "";
                   this._chassisWelcomeText.style.display = "none";
                   this._chassisWelcomeExpanded.style.display = "none";
                })
            }).placeAt(this._chassisWelcomeExpanded);

            // Server Template
            this.serversTemplateButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_serverstemplatebutton",
                iconClass: "spExpandIcon",
                label: this.resources2.SERVERS,
                title: this.resources2.SERVERS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                    this._serversTemplate.style.display = "none";
                    this._serversTemplateText.style.display = "";
                    this._serversTemplateExpanded.style.display = "";
                })
            }).placeAt(this._serversTemplate);

            this.serversTemplateButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_serverstemplatebutton2",
                iconClass: "spCollapseIcon",
                label: this.resources2.SERVERS,
                title: this.resources2.SERVERS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                    this._serversTemplate.style.display = "";
                    this._serversTemplateText.style.display = "none";
                    this._serversTemplateExpanded.style.display = "none";
                })
            }).placeAt(this._serversTemplateExpanded);

            // Chassis Template
            this.chassisTemplateButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_chassistemplatebutton",
                iconClass: "spExpandIcon",
                label: this.resources2.CHASSIS,
                title: this.resources2.CHASSIS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                   this._chassisTemplate.style.display = "none";
                   this._chassisTemplateText.style.display = "";
                   this._chassisTemplateExpanded.style.display = "";
                })
            }).placeAt(this._chassisTemplate);

            this.chassisTemplateButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_chassistemplatebutton2",
                iconClass: "spCollapseIcon",
                label: this.resources2.CHASSIS,
                title: this.resources2.CHASSIS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                   this._chassisTemplate.style.display = "";
                   this._chassisTemplateText.style.display = "none";
                   this._chassisTemplateExpanded.style.display = "none";
                })
            }).placeAt(this._chassisTemplateExpanded);
            
            // Server Deploy Template
            this.serversDeployButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_serversdeploybutton",
                iconClass: "spExpandIcon",
                label: this.resources2.SERVERS,
                title: this.resources2.SERVERS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                   this._serversDeploy.style.display = "none";
                   this._serversDeployText.style.display = "";
                   this._serversDeployExpanded.style.display = "";
                })
            }).placeAt(this._serversDeploy);
            
            this.serversDeployButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_serversdeploybutton2",
                iconClass: "spCollapseIcon",
                label: this.resources2.SERVERS,
                title: this.resources2.SERVERS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                   this._serversDeploy.style.display = "";
                   this._serversDeployText.style.display = "none";
                   this._serversDeployExpanded.style.display = "none";
                })
            }).placeAt(this._serversDeployExpanded);

            // Chassis Deploy Template
            this.chassisDeployButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_chassisdeploybutton",
                iconClass: "spExpandIcon",
                label: this.resources2.CHASSIS,
                title: this.resources2.CHASSIS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                   this._chassisDeploy.style.display = "none";
                   this._chassisDeployText.style.display = "";
                   this._chassisDeployExpanded.style.display = "";
                })
            }).placeAt(this._chassisDeploy);
            
            this.chassisDeployButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_chassisdeploybutton2",
                iconClass: "spCollapseIcon",
                label: this.resources2.CHASSIS,
                title: this.resources2.CHASSIS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                   this._chassisDeploy.style.display = "";
                   this._chassisDeployText.style.display = "none";
                   this._chassisDeployExpanded.style.display = "none";
                })
            }).placeAt(this._chassisDeployExpanded);
            
            // Servers
            this.serversButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_serversbutton",
                iconClass: "spExpandIcon",
                label: this.resources2.SERVERS,
                title: this.resources2.SERVERS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                   this._servers.style.display = "none";
                   this._serversText.style.display = "";
                   this._serversExpanded.style.display = "";
                })
            }).placeAt(this._servers);
            
            this.serversButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_serversbutton2",
                iconClass: "spCollapseIcon",
                label: this.resources2.SERVERS,
                title: this.resources2.SERVERS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                   this._servers.style.display = "";
                   this._serversText.style.display = "none";
                   this._serversExpanded.style.display = "none";
                })
            }).placeAt(this._serversExpanded);

            // Chassis
            this.chassisButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_chassisbutton",
                iconClass: "spExpandIcon",
                label: this.resources2.CHASSIS,
                title: this.resources2.CHASSIS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                    this._chassis.style.display = "none";
                    this._chassisText.style.display = "";
                    this._chassisExpanded.style.display = "";
                })                    
            }).placeAt(this._chassis);
            
            this.chassisButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_chassisbutton2",
                iconClass: "spCollapseIcon",
                label: this.resources2.CHASSIS,
                title: this.resources2.CHASSIS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                    this._chassis.style.display = "";
                    this._chassisText.style.display = "none";
                    this._chassisExpanded.style.display = "none";
                })                    
            }).placeAt(this._chassisExpanded);

            // Servers Profile
            this.serversProfileButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_serversprofilebutton",
                iconClass: "spExpandIcon",
                label: this.resources2.SERVERS,
                title: this.resources2.SERVERS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                    this._serversProfile.style.display = "none";
                    this._serversProfileText.style.display = "";
                    this._serversProfileExpanded.style.display = "";
                })
            }).placeAt(this._serversProfile);
            
            this.serversProfileButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_serversprofilebutton2",
                iconClass: "spCollapseIcon",
                label: this.resources2.SERVERS,
                title: this.resources2.SERVERS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                    this._serversProfile.style.display = "";
                    this._serversProfileText.style.display = "none";
                    this._serversProfileExpanded.style.display = "none";
                })
            }).placeAt(this._serversProfileExpanded);

            // Chassis Profile
            this.chassisProfileButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_chassisprofilebutton",
                iconClass: "spExpandIcon",
                label: this.resources2.CHASSIS,
                title: this.resources2.CHASSIS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                   this._chassisProfile.style.display = "none";
                   this._chassisProfileText.style.display = "";
                   this._chassisProfileExpanded.style.display = "";
                })
            }).placeAt(this._chassisProfile);
            
            this.chassisProfileButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_chassisprofilebutton2",
                iconClass: "spCollapseIcon",
                label: this.resources2.CHASSIS,
                title: this.resources2.CHASSIS,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                   this._chassisProfile.style.display = "";
                   this._chassisProfileText.style.display = "none";
                   this._chassisProfileExpanded.style.display = "none";
                })
            }).placeAt(this._chassisProfileExpanded);

            // Servers Category Templates
            this.serversCategoryButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_serverscategorybutton",
                iconClass: "spExpandIcon",
                label: this.resources2.SERVER_CATEGORY,
                title: this.resources2.SERVER_CATEGORY,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                   this._serversCategory.style.display = "none";
                   this._serversCategoryText.style.display = "";
                   this._serversCategoryExpanded.style.display = "";
                })
            }).placeAt(this._serversCategory);
            
            this.serversCategoryButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_serverscategorybutton2",
                iconClass: "spCollapseIcon",
                label: this.resources2.SERVER_CATEGORY,
                title: this.resources2.SERVER_CATEGORY,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                   this._serversCategory.style.display = "";
                   this._serversCategoryText.style.display = "none";
                   this._serversCategoryExpanded.style.display = "none";
                })
            }).placeAt(this._serversCategoryExpanded);

            // Chassis Category Templates
            this.chassisCategoryButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_chassiscategorybutton",
                iconClass: "spExpandIcon",
                label: this.resources2.CHASSIS_CATEGORY,
                title: this.resources2.CHASSIS_CATEGORY,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                   this._chassisCategory.style.display = "none";
                   this._chassisCategoryText.style.display = "";
                   this._chassisCategoryExpanded.style.display = "";
                })
            }).placeAt(this._chassisCategory);
            
            this.chassisCategoryButton = new support.widgets.HoverButton({
                id: this.id + "cp_overview_chassiscategorybutton2",
                iconClass: "spCollapseIcon",
                label: this.resources2.CHASSIS_CATEGORY,
                title: this.resources2.CHASSIS_CATEGORY,
                disabled: false,
                onClick: dojo.hitch(this, function(){
                   this._chassisCategory.style.display = "";
                   this._chassisCategoryText.style.display = "none";
                   this._chassisCategoryExpanded.style.display = "none";
                })
            }).placeAt(this._chassisCategoryExpanded);

        },

        onGotoSuggestedTasks: function() {
            // summary:
            // Override or hitch custom handler.
            // Called when user requests to goto the Suggested Tasks tab.
        },

        _suggestedTasksExist: function() {
            // tags:
            // protected

            var tasksExist = false;

            // Check to see if defaults already exist
            dojo.xhrGet({
                url: this.REST_PATH + "tasks",
                handleAs: "json",
                sync: true,
                load: function(response) {
                    tasksExist = ((response.tasks.deployTemplate.status == "uncompleted") ||
                            (response.tasks.createTemplate.status == "uncompleted") ||
                            (response.tasks.chooseConfig.status == "uncompleted"));
                },
                error: function(error) {
                    alert(this.resources.UNEXPECTED_ERROR.replace("%1", error));
                }
            });
            return tasksExist;

        },

        _selectDefaultView: function(evt) {
            // tags:
            // protected
            this._dftIcon.setSelected(true); // switches to default view *and* de-selects all icons
        },

        _showCreateSvrTmplDetails: function(/* boolean */expand) {
            // tags:
            // protected

            // Hide/Display icons in the details sub-section.
            dojo.style(this._cstCollapseDetailIcon.domNode, "display", expand ? "" : "none");
            dojo.style(this._catIcon.domNode, "display", expand ? "" : "none");
            dojo.style(this._apsIcon.domNode, "display", expand ? "" : "none");

            if (expand) {

                // Expand details sub-section.
                var styles = this._spsIcon.get("expanderState") ? "diagramContainer cstExpandedBackground"
                        : "diagramContainer bothExpandedBackground";
                dojo.replaceClass(this._diagramContainer.domNode, styles);

            }
            else {

                // Collapse details sub-section.
                var styles = this._spsIcon.get("expanderState") ? "diagramContainer noneExpandedBackground"
                        : "diagramContainer spsExpandedBackground";
                dojo.replaceClass(this._diagramContainer.domNode, styles);

                // If one of the icons in the details sub-section (now collapsed) is currently
                // selected, we must also change the selection to the "Create Server Template" icon
                // itself that is still visible.
                if (this._catIcon.get("selected") || this._apsIcon.get("selected")) {
                    this._cstIcon.setSelected(true);
                }

            }

        },

        _showSvrPrfsDetails: function(/* boolean */expand) {
            // tags:
            // protected

            // Hide/Display icons in the details sub-section.
            dojo.style(this._spsCollapseDetailIcon.domNode, "display", expand ? "" : "none");
            dojo.style(this._stIcon.domNode, "display", expand ? "" : "none");
            dojo.style(this._sp1Icon.domNode, "display", expand ? "" : "none");
            dojo.style(this._sp2Icon.domNode, "display", expand ? "" : "none");

            if (expand) {

                // Expand details sub-section.
                var styles = this._cstIcon.get("expanderState") ? "diagramContainer spsExpandedBackground"
                        : "diagramContainer bothExpandedBackground";
                dojo.replaceClass(this._diagramContainer.domNode, styles);

            }
            else {

                // Collapse details sub-section.
                var styles = this._cstIcon.get("expanderState") ? "diagramContainer noneExpandedBackground"
                        : "diagramContainer cstExpandedBackground";
                dojo.replaceClass(this._diagramContainer.domNode, styles);

                // If one of the icons in the details sub-section (now collapsed) is currently
                // selected, we must change the selection to the "Create Server Template" icon
                // itself that is still visible.
                if (this._stIcon.get("selected") || this._sp1Icon.get("selected") || this._sp2Icon.get("selected")) {
                    this._spsIcon.setSelected(true);
                }

            }

        },

        _showServerTemplateWizard: function() {
            // tags:
            // protected
            require(["support/dialogs/ServerTemplateWizard"], dojo.hitch(this, function() {
                var stwdialog = new support.dialogs.ServerTemplateWizard({
                    createFromExisting: true,
                    templateIDNum: null,
                    editMode: false
                });
                stwdialog.show();
            }));
        },
        
        _showChassisTemplateWizard: function() {
            require(["support/dialogs/ChassisTemplateWizard"], dojo.hitch(this, function() {
                var chdialog = new support.dialogs.ChassisTemplateWizard({
                    createFromExisting: true,
                    templateIDNum: null,
                    editMode: false
                });
                chdialog.show();
            }));
        },
        
        fromFSMInitialSetup: function () {
            
            // Welcome
            this._selectorMessage.style.display = "none";
            this._beginMessage.style.display = "inline-block";
            
            // Address Pools
            this._addrPoolsSummary.style.display = "none";
            
            // System Templates
            this._templatesSummary.style.display = "none";
            
            // Edit Template
            this._editTemplateSummary.style.display = "none";
            
            // Deploy Template
            this._deploySummary.style.display = "none";
            
            // System Template
            this._templateSummary.style.display = "none";
            
            // System Profile
            this._profileSummary.style.display = "none";
                        
        }

    });

});