define(["dojo", 
        "dijit", 
        "dijit/_Widget", 
        "dijit/_TemplatedMixin", 
        "dijit/_WidgetsInTemplateMixin",
        "dijit/layout/_ContentPaneResizeMixin", 
        "support/base/SelfStylingDijit",
        "dojo/i18n", 
        "dojo/string", 
        "dojo/i18n!./nls/Panel", 
        "dojo/i18n!./nls/Panel_II",
        "dojo/cookie",
        "support/temp/ModalDialog"
         ], function(dojo, dijit, _Widget, _TemplatedMixin,
        _WidgetsInTemplateMixin, _ContentPaneResizeMixin, SelfStylingDijit, i18n, string, mri, mri2) {

    return dojo.declare("support.panels.Panel", [ dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin,
            dijit.layout._ContentPaneResizeMixin, support.base.SelfStylingDijit], {

        // loadingMessage: String
        //      Message and graphic for display during downloads
        loadingMessage: "<span class=\"dijitContentPaneLoading\"><span class=\"dijitInline dijitIconLoading\"></span>${loadingState}</span>",

        // errorMessage: String
        //      Message and graphic for display on error
        errorMessage: "<span class=\"dijitContentPaneError\"><span class=\"dijitInline dijitIconError\"></span>${errorState}</span>",

        // Path to IBM RESTful Web Service
        // REST_PATH: "/ibm/director/rest/profiles/services/",
        REST_PATH: "",

        // Var holding current runtime environment
        CP_RUN_ENVIRONMENT: "",

        // Var holding long product name for current runtime environment
        // e.g. PRODUCT_NAME_LONG = "IBM Flex System Manager"
        PRODUCT_NAME_LONG: "",
        
        // Var holding short product name for current runtime environment
        // e.g. PRODUCT_NAME_SHORT = "Flex System Manager"
        PRODUCT_NAME_SHORT: "",

        // Var holding abbreviation of product name for current runtime environment
        // e.g. PRODUCT_NAME_ABBR = "FSM"
        PRODUCT_NAME_ABBR: "",

        // Var holding augusta page base functions
        //CP_AUGUSTA_PAGE_BASE: "",
        
        // Path to help plug-in and Configuration Patterns' content for Augusta.
        HELPDOC_AUGUSTA_PATH: "/help/?content=/com.ibm.acc.xhmc.doc",

        // Path to help plug-in and Configuration Patterns' content for FSM.
        HELPDOC_FSM_PATH: "/help/topic/com.ibm.director.configuration_ngui.helps.doc",
        
        // Name of Configuration Patterns' help window
        HELPDOC_WINDOW: "com_ibm_stg_sysx_profile_cpHelpWindow",
        
        // Specs for Configuration Patterns' help window
        HELPDOC_WINDOW_SPECS: "height=600px,width=800px,menubar=no,scrollbars=yes",
        
        // resources: [protected] Object
        // This widget's National Language Source (NLS) object.
        resources: mri, 
        
        // resources2: [protected] Object
        // This widget's National Language Source (NLS) object.
        resources2: mri2, 
        
        // _reuseableModalDialog: [protected] Object
        // Re-useable modal dialog. See getModalDialog().
        _reuseableModalDialog: null,

        // _reuseableProgressDialog: [protected] Object
        // Re-useable progress dialog. See getProgressDialog().
        _reuseableProgressDialog: null,
        
        constructor: function() {
            // must define String.trim() method at the earliest possible point
            //   as it is used in many places in CP code
            if(String.prototype.trim == null) {
                String.prototype.trim = function() {
                    return this.replace(/^\s+|\s+$/g, "");
                };
            }            
        },

        postMixInProperties: function(){
            dojo.xhrGet({
                url: "/com.ibm.stg.sysx.profile.navmodel/js/support/restpath.json",
                sync: true,
                handleAs: "json",
                load: dojo.hitch(this, function(obj) {
                    if (!obj.restpath) {
                        dojo.xhrGet({
                            url: "/ibm/augusta/rest/profiles/js/support/restpath.json",
                            sync: true,
                            handleAs: "json",
                            load: dojo.hitch(this, function(obj) {
                                this.REST_PATH = obj.restpath;
                            }),
                            error: dojo.hitch(this, function(err) {
                                this.REST_PATH = "/ibm/director/rest/profiles/services/";                   
                            })
                        });
                    } else {
                        this.REST_PATH = obj.restpath;
                    }
                }),
                error: dojo.hitch(this, function(err) {
                    dojo.xhrGet({
                        url: "/ibm/augusta/rest/profiles/js/support/restpath.json",
                        sync: true,
                        handleAs: "json",
                        load: dojo.hitch(this, function(obj) {
                            this.REST_PATH = obj.restpath;
                        }),
                        error: dojo.hitch(this, function(err) {
                            this.REST_PATH = "/ibm/director/rest/profiles/services/";                   
                        })
                    });                   
                })
            });
            
            //Set CP environment var
            this.setEnvironment();

            this.inherited(arguments);
                        
            var msgs = i18n.getLocalization("dijit", "loading", this.lang);
            this.loadingMessage = string.substitute(this.loadingMessage, msgs);
            this.errorMessage = string.substitute(this.errorMessage, msgs);
        },
        
        buildRendering: function() {
            this.inherited(arguments);

            // Set this.containerNode to make getChildren() work for us, and allow automatic
            // propagation of onShow(), resize(), layout(), etc. to children (if any).
            if (!this.containerNode) {
                this.containerNode = this.domNode;
            }
        },
        
        postCreate: function() {
            this.inherited(arguments);
            // invoke the SelfStylingDijit superclass to load the CSS
            //   stylesheet for this class
            this.loadCssForDijit();
        },
        
        decodeSpecialCharacters: function(/*String*/ value) {
            // summary: Convert encoded characters back to their orignal form
            // value: string to decode
            // returns: decoded string
            return value.replace(/x38/g, "&").replace(/x37/g, "%").replace(/x36/g, "x");
        },
        
        encodeSpecialCharacters: function(/*String*/ value) {
            // summary: Encode special characters to avoid data storage errors
            // value: string to encode
            // returns: encoded string
            return value.replace(/x/g, "x36").replace(/&/g, "x38").replace(/%/g, "x37");
        },
        
        encodeForHtml: function(/*String*/ value) {
            // summary: Encode special characters for HTML
            // value: string to encode
            // returns: encoded string
            return value.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;");
        },

        getModalDialog: function(/*String*/type, /*String*/text, /*String | Array[{title, content}]*/info) {
            // Return an instance of ModalDialog auto-attached to the dijit's DOM node.
            if (this._reuseableModalDialog == null) {
                this._reuseableModalDialog = new support.temp.ModalDialog();
                dojo.body().appendChild(this._reuseableModalDialog.domNode);
            }
            this._reuseableModalDialog.set("type", type);
            this._reuseableModalDialog.set("text", text);
            this._reuseableModalDialog.set("info", info);
            return this._reuseableModalDialog;
        },

        getProgressDialog: function(/*String*/mode, /*boolean*/showButtons) {
            // Return an instance of ProgressDialog that may be used to show progress spinner and
            // message during asynchronous operations.
            if (this._reuseableProgressDialog == null) {
                require([ "support/dialogs/ProgressDialog" ], dojo.hitch(this, function() {
                    this._reuseableProgressDialog = new support.dialogs.ProgressDialog();
                }));
                dojo.body().appendChild(this._reuseableProgressDialog.domNode);
            }
            this._reuseableProgressDialog.set("mode", mode);
            this._reuseableProgressDialog.set("nobuttons", !showButtons);
            return this._reuseableProgressDialog;
        },
        
        genHelpDocLaunchScript: function(/*String*/topic) {
            // Generate JavaScript to open Help Doc topic provided by help plug-in.
            // Example:
            //   var script = genHelpDocLaunchScript("provisioning_with_configuration_patterns.html");
            var script = "window.open(\"%1\", \"%2\", \"%3\");";
            var helpPath = (this.CP_RUN_ENVIRONMENT == "augusta") ? this.HELPDOC_AUGUSTA_PATH : this.HELPDOC_FSM_PATH;
            script = script.replace("%1", (helpPath + "/" + topic + "?noframes=true"));
            script = script.replace("%2", this.HELPDOC_WINDOW);
            script = script.replace("%3", this.HELPDOC_WINDOW_SPECS);
            return script;
        },
        
        setEnvironment: function(){

            if(this.REST_PATH == "/ibm/director/rest/profiles/services/"){
                
                this.CP_RUN_ENVIRONMENT = "fsm";
                this.PRODUCT_NAME_LONG = this.resources.PRODUCT_FSM_LONG;
                this.PRODUCT_NAME_SHORT = this.resources.PRODUCT_FSM_SHORT;
                this.PRODUCT_NAME_ABBR = this.resources.PRODUCT_FSM_ABBR;
                
            }else{

                this.CP_RUN_ENVIRONMENT = "augusta";
                this.PRODUCT_NAME_LONG = this.resources.PRODUCT_HMC_LONG;
                this.PRODUCT_NAME_SHORT = this.resources.PRODUCT_HMC_SHORT;
                this.PRODUCT_NAME_ABBR = this.resources.PRODUCT_HMC_ABBR;
            }
            
        },
        
        _accessDecorator: function(item) {
            var accessText = item;
            var iconClass = "";
            
            switch (accessText) {
                case this.resources.OK:
                    iconClass = "spSuccessIcon";
                    break;
                case this.resources.ACCESS_DEFAULT:
                    iconClass = "spSuccessIcon";
                    break;
                case this.resources.ACCESS_OFFLINE:
                    iconClass = "spOfflineIcon";
                    break;
                case this.resources.ACCESS_EMPTY:
                    iconClass = "spOfflineIcon";
                    break;
                case this.resources.PLACEHOLDER:
                    iconClass = "spBlankIcon";
                    break;
                case this.resources.ACCESS_NONE:
                    iconClass = "spOnlineNoAccessIcon";
                    break;
                case this.resources.ACCESS_PARTIAL:
                    iconClass = "spOnlinePartialAccessIcon";
                    break;
                case this.resources.ACCESS_UNKNOWN:
                    iconClass = "spUnknownIcon";
                    break;
            }
            
            return [ "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr><td><img src=\"" + this._blankGif + "\" alt=\""
                     + accessText + "\" class=\"" + iconClass + "\" title=\"" + accessText
                     + "\"></img></td><td><span style=\"margin-left:4px\" title=\"" + accessText + "\">" + accessText + "</span></td></tr></tbody></table>" ]
                     .join("");
        },

    });
});