define([
        "dojo",
        "dijit",
        "support/ConfigPatternsAll",
        "support/panels/Panel",
        "support/base/NextGenUIMixin",
        "dojo/i18n!./nls/PolicyListTable",
        "dojo/text!./templates/PolicyListTable.html",
        "support/widgets/OutlinePane",
        "support/base/TableMixin",
        "dojo/data/ItemFileWriteStore",
        "support/dialogs/FailoverPolicyTemplate",
        "dijit/form/DropDownButton",
        "dijit/form/Button",
        "dijit/Menu",
        "dijit/MenuItem",
        "dijit/MenuSeparator",
        "dijit/PopupMenuItem",
        "dojo/string",
        "support/dialogs/Rename",
        "dijit/Toolbar",
        "dijit/ToolbarSeparator"
    ], function(dojo, dijit, layer, panel, NextGenUIMixin, mri, template) {

    return dojo.declare("support.panels.PolicyListTable", [ support.panels.Panel, support.base.NextGenUIMixin, support.base.TableMixin ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,
        
        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "PolicyListTable",
        
        mode: "", // Current Context Mode        
        
        _policyTable: null,
        patternStore: null,
        editButton: null,
        deleteButton: null,
        _selectedTemplate: null,

        actionMenu: null,
        _actionButton: null,
        
        //Internal Naming Constants
        POL_NAME: "failover",
        SCH_NAME: "Schedule",
        
        //Views
        BY_ROOT: 1,
        FLAT: 2,
        viewMode: 1, // Current View, Flat List is by default

        policyLayout: {
            /* Table View Options */
            flatView: true, // Flat Table View
            typeView: true, // By Type View
            /* Toolbar Buttons and Menus */
            newMenuBtn: true, // New Policy Menu Button
            newFailoverOpt: true, //New Failover Policy Menu Option
            newActSchedOpt: true, //New Activation Schedule Option
            deleteBtn: true, // Delete Policy Button
            editBtn: true, // Edit Policy Button
            copyBtn: true, // Copy Policy Button
            /* "More" menu options */
            detailsOpt: true,
            renameOpt: true,
            /* Table Columns To Display */
            nameCol: true, // Name Column
            useCol: true, // In Use Column
            descCol: true, // Description Column
            typeCol: false //Type Column
        },
        
        // the properties used to construct the title of this page in the
        //   NextGen UI
        cpProductName: null,
        cpPageContext: null,
        cpPageName: null,
        
        // ID prefix used for table
        gridID: "PolicyListTable",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/PolicyListTable.css",

        // Override this method to perform custom
        // behavior during dijit construction.
        // Common operations for constructor:
        // 1) Initialize non-primitive types (i.e.
        // objects and arrays)
        // 2) Add additional properties needed by
        // succeeding lifecycle methods
        constructor: function(){
            this.locResources = dojo.i18n.getLocalization("support.panels", "PolicyListTable", this.lang);
        },

        // When this method is called, all variables
        // inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget
        // property variables defined in the template
        // HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
            this.cpProductName = this.resources.PRODUCT_NAME;
            // this.cpPageContext = this.resources.SERVERS; Must remain null (unspecified), per XB216902
            this.cpPageName = this.locResources.PAGE_NAME;
        },

        // postCreate() is called after
        // buildRendering(). This is useful to override
        // when
        // you need to access and/or manipulate DOM
        // nodes included with your widget.
        // DOM nodes and widgets with the
        // dojoAttachPoint attribute specified can now
        // be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in
        // buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            
            // Apply env specific tooltip for the page
            var policyTip = new Array();
            policyTip.push("<div style=\"width: 30em;\">");
            policyTip.push(this.locResources.TOOLTIP_TEXT1);
            policyTip.push("<br><br>" + this.locResources.TOOLTIP_TEXT2);
            policyTip.push("<br><br>" + this.locResources.TOOLTIP_TEXT3);
            policyTip.push("<br><br>" + this.locResources.TOOLTIP_TEXT4.replace("%1", this.PRODUCT_NAME_LONG));
            policyTip.push("</div>");
            // NOTE: For dijit/Tooltip dijits, set the label, not innerHTML.
            this._policyTooltip.label = policyTip.join("");

            // Setup table widget...
            var patternStore = this._retrieveStore();
            var tableStructure = this._getTableStructure();       
            this._policyTable = this.getTable(patternStore, tableStructure, this.resources.NO_DATA, this.gridID, true);
            
            // Set parent rows as "unselectable"
            this._policyTable.disableParentRows();
            this._refreshTree();
            
            setTimeout(dojo.hitch(this, function() {
                this._policyTable.placeAt(this.policyTable);
                this._policyTable.startup();
            }), 500);
            
            // Connect handlers to reset table capabilities
            dojo.connect(this._policyTable.select.row, "onSelectionChange", this, this.setTableCapabilities); // Fires on row selection
            dojo.connect(this._policyTable, "setStore", this, this.setTableCapabilities); // Fires on store reset

            // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
            this.changeNextGenUITitle();

            // reload contents of policy table; by default, switching between
            // links in the details selector does not reload page
            this.subscribeToDetailsSelectorChanges(dojo.hitch(this, function() {
                this._refreshTree();
                this.changeNextGenUITitle();
            }));
            
            this._loadToolBar(this._policyTable); // load table toolbar

            // Register subscribers to topics we care about ...
            this.subscribe("FailoverTemplateComplete", function(data) {
                this._refreshTree("failover");                
            });

            this.subscribe("RefreshTable", function(data) {
                this._refreshTree();
            });
            
            this.subscribe("CloseDialog", function() {
                if (this.viewMode == this.FLAT) {
                    this._policyTable.showFilter(); // Show filter controls on flat view.
                }
            });

        },

        _refreshTree: function(/*String?*/expandId) {
            // tags:
            // protected
            // Refresh tree data
            var policyStore = this._retrieveStore();
            this._policyTable.setStore(policyStore);
            this._policyTable.model.clearCache();
            var callback = function(treeView, table, /*String?*/expandId) {
                if (treeView) {
                    table.reexpandRows();
                    expandId && table.tree.expand(expandId);
                    table.disableParentRows();
                }
                else {
                    table.showFilter(); // Show filter controls on flat view.
                }
            };
            this._policyTable.body.refresh().then(callback((this.viewMode == this.BY_ROOT), this._policyTable, expandId));           
         },
        
        _retrieveStore: function() {
            // tags:
            // protected
            // Connect to data source and create store
            var patternData = "";
            dojo.xhrGet({
                url: this.REST_PATH + "policy?type=failover&list=true",
                handleAs: "json",
                preventCache: true,
                sync: true,
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                load: dojo.hitch(this, function(response) {
                    if (response && response.items) {

                    if (this.viewMode == this.BY_ROOT) {
                            patternData = this._flatToRoot(response.items);
                        }
                        else if (this.viewMode == this.FLAT) {
                            patternData = this._rootToFlat(response.items);
                        }                        
                    }

                })
            });

            var patternStore = new dojo.data.ItemFileWriteStore({
                data: dojo.fromJson(patternData)
            });

            if (!("hasChildren" in patternStore)) {
                patternStore.hasChildren = function(id, item) { // Need hasChildren function for TreeTable.
                    return item && this.getValues(item, "children").length;
                };
            };
            
            if (!("getChildren" in patternStore)) {
                patternStore.getChildren = function(item) { // Need getChildren function for TreeTable.
                    return this.getValues(item, "children");
                };
            };
            
            return patternStore;
        },
        
        _loadToolBar: function(grid) {
            // tags:
            // protected
            var toolbar = grid && grid.toolBar && grid.toolBar.widget;
            
            if (toolbar) {
                // Load toolbar buttons/menus
                if (this._policyTable == null) {
                    // Profile table not created. Nothing to
                    // do but return.
                    return;
                }
                if (!this.actionMenu) {
                    this.actionMenu = new dijit.Menu({
                        style: "display: none;"
                    });
                }
                // Add custom display config menu
                var displayMenu = new dijit.Menu({
                    style: "display: none;"
                });
                
                // By Type View
                if (this.policyLayout.typeView) {
                    var displayItem1 = new dijit.MenuItem({
                        //id: this.id + "cp_policylisttable_displayitem1",
                        label: this.resources.BY_TYPE,
                        title: this.locResources.DISPLAY_TYPE,
                        viewMode: this.BY_ROOT,
                        onClick: dojo.hitch(this, function() {
                            this._setViewByMode(this.BY_ROOT);
                        })
                    });
                    displayMenu.addChild(displayItem1);
                }
                
                // Flat List View
                var displayItem2 = new dijit.MenuItem({
                    //id: this.id + "cp_policylisttable_displayitem2",
                    label: this.resources.FLAT,
                    title: this.locResources.DISPLAY_FLAT,
                    viewMode: this.FLAT,
                    onClick: dojo.hitch(this, function() {
                        this._setViewByMode(this.FLAT);
                    })
                });
                displayMenu.addChild(displayItem2);
    
                this._displayButton = new dijit.form.DropDownButton({
                    //id: this.id + "cp_policylisttable_displaybutton",
                    label: this.resources.BY_TYPE,
                    title: this.resources.TABLE_VIEW,
                    showLabel: true,
                    dropDown: displayMenu
                });
                toolbar.addChild(this._displayButton);
                //this.viewMode = this.BY_ROOT;
                
                toolbar.addChild(new dijit.ToolbarSeparator());
                
    			// Create Menu            
                var createMenu = new dijit.Menu({
                    style: "display: none;"
                });
    
                var newFailoverPolicy = new dijit.MenuItem({
                    //id: this.id + "cp_policylisttable_newfailoverpolicy",
                    label: this.locResources.FAIL_POLICY,
                    title: this.locResources.CREATE_FAIL,
                    onClick: dojo.hitch(this, this._newFailover),
                    onKeypress: dojo.hitch(this, this._newFailover)
                });
                createMenu.addChild(newFailoverPolicy);
    
                var newActSchedule = new dijit.MenuItem({
                    //id: this.id + "cp_policylisttable_newactschedule",
                    label: this.locResources.ACT_SCHED,
                    title: this.locResources.ACT_SCHED,
                    onClick: dojo.hitch(this, this._newActivationSchedule),
                    onKeypress: dojo.hitch(this, this._newActivationSchedule)
                });
                //createMenu.addChild(newActSchedule);
    
                var createButton = new dijit.form.DropDownButton({
                    //id: this.id + "cp_policylisttable_createbutton",
                    showLabel: false,
                    iconClass: "spCreateLgIcon",
                    title: this.locResources.NEW_POLICY,
                    dropDown: createMenu
                });
    
                toolbar.addChild(createButton);
                
                // Create Menu            
                var createMenu2 = new dijit.Menu({
                    style: "display: none;"
                });
    
                var newFailoverPolicy2 = new dijit.MenuItem({
                    //id: this.id + "cp_policylisttable_newfailoverpolicy2",
                    label: this.locResources.FAIL_POLICY,
                    title: this.locResources.CREATE_FAIL,
                    onClick: dojo.hitch(this, this._newFailover),
                    onKeypress: dojo.hitch(this, this._newFailover)
                });
                createMenu2.addChild(newFailoverPolicy2);
    
                var newActSchedule2 = new dijit.MenuItem({
                    //id: this.id + "cp_policylisttable_newactschedule2",
                    label: this.locResources.ACT_SCHED,
                    title: this.locResources.ACT_SCHED,
                    onClick: dojo.hitch(this, this._newActivationSchedule),
                    onKeypress: dojo.hitch(this, this._newActivationSchedule)
                });
                //createMenu2.addChild(newActSchedule2);
                                
                // Edit Button
                if (this.policyLayout.editBtn) {
                    this.editButton = new dijit.form.Button({
                        //id: this.id + "cp_policylisttable_editbutton",
                        showLabel: false,
                        iconClass: "spEditInactiveLgIcon",
                        disabled: true,
                        title: this.locResources.EDIT_SEL,
                        onClick: dojo.hitch(this, function() {
                            this.editSelection();
                        })
                    });
                    toolbar.addChild(this.editButton);
                }
                            
                //Delete Button
                if (this.policyLayout.deleteBtn) {
                    // Add custom delete button...
                    this.deleteButton = new dijit.form.Button({
                        //id: this.id + "cp_policylisttable_deletebutton",
                        showLabel: false,
                        iconClass: "spDeleteInactiveLgIcon",
                        disabled: true,
                        title: this.locResources.DELETE_SEL,
                        onClick: dojo.hitch(this, function() {
                            this.deletePolicy();
                        })
                    });
                    toolbar.addChild(this.deleteButton);
                }
                
                toolbar.addChild(new dijit.ToolbarSeparator());
                
                // Add to Actions Menu...            
                this.expandAllItem = new dijit.MenuItem({
                    //id: this.id + "cp_policylisttable_expandallitem",
                    label: this.resources.EXPAND_ALL,
                    iconClass: "spExpandAllLgIcon",
                    title: this.resources.EXPAND_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._policyTable.expandAll();
                    }),
                    onKeypress: dojo.hitch(this, function() {
                        this._policyTable.expandAll();
                    })
                });
                this.actionMenu.addChild(this.expandAllItem,0);
    
                this.collapseAllItem = new dijit.MenuItem({
                    //id: this.id + "cp_policylisttable_collapseallitem",
                    label: this.resources.COLLAPSE_ALL,
                    iconClass: "spCollapseAllLgIcon",
                    title: this.resources.COLLAPSE_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._policyTable.collapseAll();
                    }),
                    onKeypress: dojo.hitch(this, function() {
                        this._policyTable.collapseAll();
                    })
                });
                this.actionMenu.addChild(this.collapseAllItem,1);
                
                this.createItem = new dijit.PopupMenuItem({
                    label: this.resources.CREATE,
                    iconClass: "spCreateLgIcon",
                    title: this.locResources.NEW_POLICY,
                    popup: createMenu2
                });
                this.actionMenu.addChild(this.createItem);
                
                var menuSeparator = new dijit.MenuSeparator();
                this.actionMenu.addChild(menuSeparator);
    
                this.detailsItem = new dijit.MenuItem({
                    //id: this.id + "cp_policylisttable_detailsitem",
                    label: this.resources.DETAILS,
                    title: this.locResources.DETAILS_SELECTED_POLICY,
                    onClick: dojo.hitch(this, this.detailPolicy),
                    onKeypress: dojo.hitch(this, this.detailPolicy)
                });
    
                this.renameItem = new dijit.MenuItem({
                    //id: this.id + "cp_policylisttable_renameitem",
                    label: this.resources.RENAME,
                    title: this.locResources.RENAME_SEL,
                    onClick: dojo.hitch(this, this.renamePolicy),
                    onKeypress: dojo.hitch(this, this.renamePolicy)
                });
                
                this.copyItem = new dijit.MenuItem({
                    //id: this.id + "cp_policylisttable_copyitem",
                    label: this.resources.COPY,
                    title: this.locResources.COPY_SEL,
                    onClick: dojo.hitch(this, this.copySelection),
                    onKeypress: dojo.hitch(this, this.copySelection)
                });
                
                if (this._actionButton) {
                    this._actionButton.dropDown = this.actionMenu;
                }
                else {
                    this._actionButton = new dijit.form.DropDownButton({
                        //id: this.id + "cp_policylisttable_actionbutton",
                        label: this.resources.ALL_ACTIONS,
                        title: this.resources.ALL_ACTIONS_TITLE,
                        showLabel: true,
                        dropDown: this.actionMenu
                    });   
                    
                    toolbar.addChild(this._actionButton);
                }
                
            }            
        },
        
        _newFailover: function(){
            var fptDialog = new support.dialogs.FailoverPolicyTemplate();
            fptDialog.show();
        },
        
        _newActivationSchedule: function(){
            alert(this.locResources.ACT_SCHED);
        },
        
        // Delete Profile Menu Option
        deletePolicy: function() {
            if (this.policyLayout.deleteBtn) {
                var str = this._getSelectedPolicyInfo();
                var rows = this._policyTable.select.row.getSelected();
                var i;
                             
                var confirmedDelete = function() {
  
                    if (rows.length > 0 && str != "") {
                        for(i = 0; i < rows.length; i++){
                            var xhrDelete = dojo.xhrDelete({
                                url: this.REST_PATH + "policy/" + rows[i] + "?type=failover",                                
                                handleAs: "json",
                                headers: {
                                    "Content-Type": "application/json; charset=utf-8"
                                },
                                sync: true,
                                preventCache: true,
                                load: function(response) {
                                }
                            });
                        }                        
                        this._refreshTree();                    
                    }                
                };
                
                str = this.encodeForHtml(str.toString());

                if(this.CP_RUN_ENVIRONMENT == "fsm"){

                    require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {
                
                        var confDialog = new support.temp.ConfirmationDialog({
                            buttonLabel: this.resources.DELETE,
                            type: "Warning",
                            text: this.locResources.CONFIRM_DEL,
                            info: this.locResources.DELETE.replace("%1", str)
                        });

                        confDialog.startup();
                        confDialog.confirm(confirmedDelete, this);
                
                    }));
                    
                }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                    require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {
                        
                        var confDialog = new idx.widget.ConfirmationDialog({
                            buttonLabel: this.resources.DELETE,
                            type: "Warning",
                            text: this.locResources.CONFIRM_DEL,
                            info: this.locResources.DELETE.replace("%1", str)
                        });

                        confDialog.startup();
                        confDialog.confirm(confirmedDelete, this);
                
                    }));
                    
                }
                                                
            }
        },

        // Edit Menu Option
        editSelection: function() {
            if (this.policyLayout.editBtn) {
                if(this._selectedTemplate.templateType.toString() == this.POL_NAME){
                    var fptDialog = new support.dialogs.FailoverPolicyTemplate({
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit"
                    });
                    fptDialog.show();
                }
                else if(row.templateType == this.SCH_NAME){
                    
                }
            }
        },
        
        // Copy Menu Option
        copySelection: function() {
            if (this.policyLayout.copyBtn) {
                if(this._selectedTemplate.templateType.toString() == this.POL_NAME){
                    var fptDialog = new support.dialogs.FailoverPolicyTemplate({
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    fptDialog.show();
                }
                else if(row.templateType == this.SCH_NAME){
                    
                }
            }
        },
        
        renamePolicy : function(){
            var renamedialog = new support.dialogs.Rename({
                id: this.id + "cp_policylisttable_rename",
                templateIDNum: this._selectedTemplate.id.toString(),
                saveTemplateType: this.POL_NAME
            });
            renamedialog.show();
        },
        
        // Details Template
        detailPolicy: function() {
                     
            require([ "support/panels/PolicyDetails" ], dojo.hitch(this,function() {
                
                if(this.CP_RUN_ENVIRONMENT == "fsm"){

                    this.loadNGUIWorkAreaWithCPWidget("support/panels/PolicyDetails", null, this._selectedTemplate.id.toString());
                
                }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                    var launchParams = {templateIDNum: this._selectedTemplate.id.toString()};
                    
                    this.loadAugustaWorkAreaTask("config.detail.page","support/panels/PolicyDetails",launchParams);
                    
                }
            
            }));

        }, 
        
        // Set Table Formatters
        // Hard-coded path appended to URL, must be changed if path changes
        _decorateName: function(item, id) {
            // tags:
            // protected
            if (!item || !id) {
                return "";
            }
            
            var value = item;
            
            if(id == "failover"){
                return "<span title=\"" + value + "\" style=\"white-space:normal;\">" + value + "</span>";
            } 
            if(this.CP_RUN_ENVIRONMENT == "fsm"){
                return "<a href=\"#workAreaDijit=support%2Fpanels%2FPolicyDetails&path=support:%2Fcom.ibm.stg.sysx.profile." +
                    "navmodel%2Fjs%2Fsupport&cpTarget=" + id + "\">" + value + "</a>";
            } else if(this.CP_RUN_ENVIRONMENT == "augusta"){
                return "<a href=\"#fwWidget=support%2Fpanels%2FPolicyDetails&fwTaskId=config.detail.page&templateIDNum=" 
                    + id + "\">" + value + "</a>";
            }
        },
        
        _formatName: function(item) {           
            
            if (!item || !item.name) {
                return "";
            }
            
            var value = this.decodeSpecialCharacters(item.name);
            value = this.encodeForHtml(value);
            
            return value;
        },
        
        _decorateInUse: function(item) {
            // tags:
            // protected      
            if (item == "") {
                return "";
            }

            if (item == this.resources.DEPLOYED) {
                return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr title=\"" + this.locResources.PIN_USE + "\">" +
                    "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spDeployedIcon\"></img></td>" +
                    "<td><span style=\"margin-left:4px; white-space: nowrap;\">" + 
                    this.resources.DEPLOYED + "</span></td></tr></tbody></table>";
            }else if(item == this.resources.REFERENCED){
                return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr title=\"" + this.locResources.PIN_USE + "\">" +
                    "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spReferencedIcon\"></img></td>" +
                    "<td><span style=\"margin-left:4px; white-space: nowrap;\">" + 
                    this.resources.REFERENCED + "</span></td></tr></tbody></table>";
            }
            return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr title=\"" + this.locResources.PN_USE + "\">" +
                "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spNotInUseIcon\"></img></td>" +
                "<td><span style=\"margin-left:4px; white-space: nowrap;\">" + 
                this.resources.NOT_IN_USE + "</span></td></tr></tbody></table>";                  
        },
        
        _formatInUse: function(item) {
            var inUseValue = (item && item.inUse) ? item.inUse : false;
            var refValue = (item && item.referencedBy && item.referencedBy > 0) ? true : false;
            
            if (item.type == "root" && this.viewMode == this.BY_ROOT) {
                return "";
            }
            
            if (inUseValue) {
                return this.resources.DEPLOYED;
            } else if (refValue) {
                return this.resources.REFERENCED;               
            }
            return this.resources.NOT_IN_USE;
        },
        
        _decorateDescription: function(item) {
            return "<span title=\"" + item + "\" style=\"white-space:normal;\">" + item + "</span>";
        },
        
        _formatDescription: function(item) {
            // tags:
            // protected
            if (!item || !item.description) {
                return "";
            }
            var value = this.decodeSpecialCharacters(item.description);
            value = this.encodeForHtml(value);
            return value;
        },
        
        _decorateTemplateType: function(item) {
            return "<span title=\"" + item + "\" style=\"white-space:nowrap;\">" + item + "</span>";
        },
        
        _formatTemplateType: function(item) {
            // tags:
            // protected
            if (!item || !item.templateType) {
                return "";
            }
            var value = item.templateType;

            switch (value) {
                case "failover":
                    value = this.resources.POLICY_FAILOVER;
                    break;
                default:
                    value = "";
                    break;
            }
            return value;
        },
        
        // Set Table Capabilities
        setTableCapabilities: function() {

            if (!this._policyTable) {
                return; // nothing to do. return now
            }
            
            this.editButton.set("iconClass", "spEditInactiveLgIcon");
            this.editButton.set("disabled", true);
            this.deleteButton.set("iconClass", "spDeleteInactiveLgIcon");
            this.deleteButton.set("disabled", true);
            
            var ids = this._policyTable.select.row.getSelected();
            
            var menuPosition = 2;
            
            var allowEdit = (ids.length == 1); // assume okay, disprove below
            var allowDelete = (ids.length > 0); // assume okay, disprove below            
            var allowRename = (ids.length == 1); // assume okay, disprove below
            for (var loopVar = 0; loopVar < ids.length; ++loopVar) {

                var item = this._policyTable.row(ids[loopVar], true).item();
                this._selectedTemplate = item;
                
                if ((item.type == "root") && (this.viewMode != this.FLAT)) {
                    // Root selection not allowed.
                    allowEdit = allowDelete = allowDeploy = allowRename = false;
                }                
                if (item.inUse == "true") {
                    allowDelete = false;
                }                                
                
            };

            if (allowEdit) {
                this.editButton.set("iconClass", "spEditLgIcon");
                this.editButton.set("disabled", false);

                if (!this.detailsItem) {
                    this.detailsItem = new dijit.MenuItem({
               //         id: this.id + "cp_policylisttable_detailsitem",
                        label: this.resources.DETAILS,
                        title: this.locResources.DETAILS_SELECTED_POLICY,
                        onClick: dojo.hitch(this, this.detailPolicy),
                        onKeypress: dojo.hitch(this, this.detailPolicy)
                    });
                }

                if (!this.copyItem) {
                    this.copyItem = new dijit.MenuItem({
                   //     id: this.id + "cp_policylisttable_copyitem",
                        label: this.resources.COPY,
                        title: this.locResources.COPY_SEL,
                        onClick: dojo.hitch(this, this.copySelection),
                        onKeypress: dojo.hitch(this, this.copySelection)
                    });
                }

                if (allowRename && !this.renameItem) {
                    this.renameItem = new dijit.MenuItem({
                       // id: this.id + "cp_policylisttable_renameitem",
                        label: this.resources.RENAME,
                        title: this.locResources.RENAME_SEL,
                        onClick: dojo.hitch(this, this.renamePolicy),
                        onKeypress: dojo.hitch(this, this.renamePolicy)
                    });
                }

                if (!this.editItem) {
                    this.editItem = new dijit.MenuItem({
                       // id: this.id + "cp_policylisttable_edititem",
                        label: this.resources.EDIT,
                        title: this.locResources.EDIT_SEL,
                        iconClass: "spEditLgIcon",
                        onClick: dojo.hitch(this, this.editSelection),
                        onKeypress: dojo.hitch(this, this.editSelection)
                    });
                }              

                this.actionMenu.addChild(this.editItem, ((this.viewMode == this.FLAT)? menuPosition : menuPosition+2));
                this.actionMenu.addChild(this.copyItem);
                this.actionMenu.addChild(this.detailsItem);
                menuPosition++;

                if (allowRename) {
                    this.actionMenu.addChild(this.renameItem);
                }
                else if (this.renameItem) {
                    this.actionMenu.removeChild(this.renameItem);
                    this.renameItem = null;
                }                

            }
            else {

                this.actionMenu.removeChild(this.detailsItem);
                this.detailsItem = null;

                this.actionMenu.removeChild(this.copyItem);
                this.copyItem = null;

                this.actionMenu.removeChild(this.renameItem);
                this.renameItem = null;
                
                this.actionMenu.removeChild(this.editItem);
                this.editItem = null;

            }

            if (allowDelete && !this.deleteItem) {
                this.deleteItem = new dijit.MenuItem({
                  //  id: this.id + "cp_policylisttable_deleteitem",
                    label: this.resources.DELETE,
                    title: this.locResources.DELETE_SEL,
                    iconClass: "spDeleteLgIcon",
                    onClick: dojo.hitch(this, this.deletePolicy),
                    onKeypress: dojo.hitch(this, this.deletePolicy)
                });
            }

            if (allowDelete) {

                this.deleteButton.set("iconClass", "spDeleteLgIcon");
                this.deleteButton.set("disabled", false);
                
                this.actionMenu.addChild(this.deleteItem, ((this.viewMode == this.FLAT)? menuPosition : menuPosition+2));
                menuPosition++;
            
            }else if (this.deleteItem) {
                    this.actionMenu.removeChild(this.deleteItem);
                    this.deleteItem = null;
            }

            //Disable Action Button if no children
            if(this.actionMenu.hasChildren()){
                this._actionButton.set("disabled", false);
            }else{
                this._actionButton.set("disabled", true);                
            }


        },
        
        _setViewByMode: function(menuItem) {
            // tags:
            // protected
            if (this.viewMode == menuItem) {
                return;
            }
            
            // Clear selected rows when switching views
            this._policyTable.select.row.clear();
			
            // Set Menu Button Label to Current View
            var label = "";
			if (menuItem == this.BY_ROOT) {
				label = this.resources.BY_TYPE;
				this.actionMenu.addChild(this.expandAllItem, 0);
				this.actionMenu.addChild(this.collapseAllItem, 1);
                this._policyTable.hideFilter(); // Hide filter controls on tree view.
			}
			else if (menuItem == this.FLAT) {
				label = this.resources.FLAT;
				this.actionMenu.removeChild(this.expandAllItem);
				this.actionMenu.removeChild(this.collapseAllItem);
                this._policyTable.showFilter(); // Show filter controls on flat view.
			}
			else { // Invalid view mode
			    return;
			}
            
			//Disable Action Button if no children
            if (this.actionMenu.hasChildren()) {
                this._actionButton.set("disabled", false);
            } else {
                this._actionButton.set("disabled", true);                
            }
			
            //Reformat Table Data
            this._reRenderTable(menuItem);
            
            //Set Drop Down Menu Label
            this._displayButton.set("label", label);
            
            this.viewMode = menuItem;

        },

        _reRenderTable: function(nextViewMode) {
            // tags:
            // protected
            // Re-render Tree table with new columns, column spans, and data
            // presentation
            this.viewMode = nextViewMode;
            
            var patternStore = this._retrieveStore();
            var tableStructure = this._getTableStructure(); // Get structure for current view
            
            this._policyTable.setColumns(tableStructure);
            this._policyTable.setStore(patternStore);
            this._policyTable.model.clearCache();
            this._policyTable.body.refresh();

            if (this.viewMode == this.BY_ROOT) {
                this._policyTable.reexpandRows();
            }

        },
        
        _getTableStructure: function() {
            // tags:
            // protected            
            var includeType = (this.viewMode == this.FLAT);
            
            var layout = new Array();
            layout.push({
                field: "name",
                name: this.resources.NAME,
                width: "25em",
                formatter: dojo.hitch(this, this._formatName),
                decorator: dojo.hitch(this, this._decorateName),
                title: this.resources.TEMP_NAME
            });
            layout.push({
                fields: "inUse",
                name: this.resources.USAGE_STATUS,
                width: "8em",
                formatter: dojo.hitch(this, this._formatInUse),
                decorator: dojo.hitch(this, this._decorateInUse),
                title: this.locResources.POLICY_COLUMN
            });
            (includeType) && layout.push({
                field: "templateType",
                name: this.locResources.POLICY_TYPE,
                width: "8em",
                formatter: dojo.hitch(this, this._formatTemplateType),
                decorator: dojo.hitch(this, this._decorateTemplateType),
                title: this.locResources.POLICY_TYPE
            });
            layout.push({
                field: "description",
                name: this.resources.DESCRIPTION,
                width: "auto",
                formatter: dojo.hitch(this, this._formatDescription),
                decorator: dojo.hitch(this, this._decorateDescription),
                title: this.resources.DESCRIPTION
            });

            return layout;
        },
        
        _flatToRoot : function(dataItems) {
            var failoverChildren = new Array();
            
            var newChild = function(item) {
                var child = {
                        id: item.id,
                        name: item.name,
                        type: item.type,
                        templateType: item.type,
                        uri: item.uri,
                        inUse: item.inUse,
                        description: item.description
                };
                return child;
            };
            
            dojo.forEach(dataItems, function(item) {
                if (item.type == "failover") {
                    failoverChildren.push(newChild(item));
                }
            });
            
            var items = new Array;
            if (failoverChildren.length > 0) {
                items.push({
                    id: "failover",
                    name: this.locResources.FAILOVER,
                    type: "root",
                    children: failoverChildren
                });
            }
            
            var tableData = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": items
            }, true);
            
            return tableData;
        },                
        
        _rootToFlat: function(dataItems) {
            var items = Array();
            for (var loopVar = 0; loopVar < dataItems.length; loopVar++) {
                var item = dataItems[loopVar];
                if (!item) {
                    continue;
                }
                switch (item.type) {
                    case "root":
                        // Store expand info since we "lose" the root nodes
                        if (item.id == this.POL_NAME) {
                            this._expandMe.policy = item.expandMe;
                        }
                        else if (item.id == this.SCH_NAME) {
                            this._expandMe.schedule = item.expandMe;
                        }
                        break;
                    case "failover":
                        items.push({
                            id: item.id,
                            name: item.name,
                            uri: item.uri,
                            type: "root",
                            templateType: item.type,
                            inUse: item.inUse,
                            description: item.description
                        });
                        break;
                    default:
                        break;
                }
            }
            
            var tableData = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": items
            }, true);
            
            return tableData;            
        },
                
        // Disable or enable options for custom view modes
        // Only one mode for now
        _setupLayoutTemplate: function(mode) {
            switch (mode) {
                default:
                    break;
            }
        },        

        // Return basic info about rows selected
        _getSelectedPolicyInfo: function () {
            var num = this._policyTable.select.row.getSelected().length;
            var str = "";
            var ids;
            var i;
            if (num > 0) {
                ids = this._policyTable.select.row.getSelected();                
                for (i = 0; i < num; i++) {
                    var item = this._policyTable.row(ids[i], true).item();
                    if(str != "") {
                        str += ", ";
                    }
                    str += item.name;
                }
            }
            return str;
        },        
        goForeground: function(params) { //Page base function that fires when the page is brought to focus
            
            this._refreshTree();

            return true;
        }
        

    });
});