define([
        "dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/PoolDetails", 
        "dojo/text!./templates/PoolDetails.html",
        "support/base/NextGenUIMixin", 
        "support/widgets/OutlinePane", 
        "dijit/layout/TabContainer",
        "support/panels/PoolDetailsDependent",
        "dijit/Toolbar",
        "dijit/ToolbarSeparator",
        "dijit/Menu",
        "dojo/dom",
        "support/dialogs/Rename",
        "support/dialogs/NewIPPool"
    ], function(dojo, dijit, panel, mri, template) {

    return dojo.declare("support.panels.PoolDetails", [ support.panels.Panel, support.base.NextGenUIMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in displaying information
        // associated with a server.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "ServerDetails",
        
        // the properties used to construct the title of this page in the
        //   NextGen UI
        cpPageContext: null,
        cpPageName: null,
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/PoolDetails.css",

        dependentsTabWidget: null,
        
        templateInformation: "",
        
        templateIDNum: "",
        templateName: "",
        templateDesc: "",
        templateInUse: false,
       // templateType: "",
        
        postMixInProperties: function() {
            this.inherited(arguments);
            this.cpPageContext = "";
            this.cpPageName = this.resources.DETAILS;
        },

        postCreate: function() {
            this.inherited(arguments);

            if(this.CP_RUN_ENVIRONMENT == "augusta"){
                
                //Add page base in order to use the setTitle() function
                require([ "augusta/augustaUICore/core/augustaPageBase" ], dojo.hitch(this, function(PageBase) {                
                    var CP_AUGUSTA_PAGE_BASE = new augusta.augustaUICore.core.augustaPageBase();
                    dojo.safeMixin(this, CP_AUGUSTA_PAGE_BASE);            
                }));

                this.bwIsCacheable = false;
            }    

            this.subscribe("RefreshNameDiv", function(data) {
                // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
                this.cpPageContext = data;

                if(this.CP_RUN_ENVIRONMENT == "fsm"){
                    //Set title in FSM
                    this.changeNextGenUITitle();
                }
                
                if(this.CP_RUN_ENVIRONMENT == "augusta"){
                    //Set Title in xHMC, only can be used if augustaPageBase mixed in
                    this.setTitle(this.cpPageContext + " - " + this.cpPageName);
                }
            });

            this.subscribe("ippoolComplete", function(data) {
                // Update details page with new IP Pool information
                this.getServerTemplateID();
                this.updatePoolDetails();
                dojo.publish("RefreshNameDiv", this.templateName);
            });
                        
            this.getServerTemplateID();

            // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
            this.cpPageContext = this.templateName;

            if(this.CP_RUN_ENVIRONMENT == "fsm"){
                //Set title in FSM
                this.changeNextGenUITitle();
            }
            
            if(this.CP_RUN_ENVIRONMENT == "augusta"){
                //Set Title in xHMC
                this.setTitle(this.cpPageContext + " - " + this.cpPageName);
            }

            this.toolbarSetup();
            this.updatePoolDetails();

        },
        
        updatePoolDetails: function() {

            // Dependents Tab
            if (this.dependentsTabWidget) {
                // Must remove & destroy to ensure updates are reflected when rebuilt below.
                this._tabContainer.removeChild(this.dependentsTabWidget);
                this.dependentsTabWidget.destroyRecursive();
            }
            this.dependentsTabWidget = new support.panels.PoolDetailsDependent({
                id: this.id + "cp_pooldetails_pooldetailsdependent",
                mode: "poolDetails",
                templateIDNum: this.templateIDNum,
                tableWidth: "99%",
                title: this.locResources.SVRTMPL_DEPENDENTS_TAB_TITLE
            });
            this._tabContainer.addChild(this.dependentsTabWidget);
                                   
        },

        //Get Server Template ID
        getServerTemplateID: function(){
            var store ="";
          
            if (this.templateIDNum == "") {
                //If this templateID not provided then parse url for it
                var url = window.location.href;
                var splitUrl = url.split("cpTarget=");
                var IDarray = new Array();
                var ID = "";
                
                if (splitUrl[1]) {
                    IDarray = splitUrl[1].split("&");
                } else {
                    ID = "";
                }                
                if (IDarray[0]) {
                    ID=IDarray[0];
                    dojo.xhrGet({
                        url: this.REST_PATH + "ippool/" + ID,
                        handleAs: "json",
                        sync: true,
                        preventCache: true,
                        load: function(response) {
                            console.log(response);
                            store = response;
                        }
                    });         
                    this.templateInformation = store;
                    this.templateIDNum = ID;
                }

            } else {
                //IF ID passed then use passed ID
                dojo.xhrGet({
                    url: this.REST_PATH + "ippool/" + this.templateIDNum,
                    handleAs: "json",
                    sync: true,
                    preventCache: true,
                    load: function(response) {
          //              console.log(response);
                        store = response;
                    }
                });
                
                this.templateInformation = store;
            }

            //Set Template Name and Description and InUse ... etc ....
            //Check to see if server template info was gained
            if(this.templateInformation != "" && this.templateInformation.items != "") {
                this.templateName = this.templateInformation.name;
                this.DetailsTemplateDescription.innerHTML = (this.templateInformation.description != null) ? this.templateInformation.description : "";
                this.DetailsTemplateType.innerHTML = this.locResources.TYPE_IP;
                this.templateInUse = this.templateInformation.inUse;
            }
            
        },

        //ToolBar Setup
        toolbarSetup: function(){

            this.detailsSeparator = new dijit.ToolbarSeparator();

            this.moreMenu = new dijit.Menu({          // Add to More/Actions Menu
                style: "display: none;"
            });
      
            this.editButton = new dijit.form.Button({     //Add Edit Button to toolbar
                id: this.id + "cp_pooldetails_editbutton",
                showLabel: false,
                iconClass: "spEditLgIcon",
                disabled: false,
                title: this.resources.EDIT_ADDR_POOL,
                onClick: dojo.hitch(this, this.editTemplate),
                onKeypress: dojo.hitch(this, this.editTemplate)
            });
            this.detailsToolBar.addChild(this.editButton);

            // Add to More/Actions Menu                    
            this.detailsSeparator = new dijit.ToolbarSeparator();
            this.detailsToolBar.addChild(this.detailsSeparator);
            
            this.renameItem = new dijit.MenuItem({
         //       id: this.id + "cp_pooldetails_renameitem",
                label: this.resources.RENAME,
                title: this.resources.RENAME_ADDR_POOL,
                onClick: dojo.hitch(this, this.renameTemplate),
                onKeypress: dojo.hitch(this, this.renameTemplate)
            });
            this.moreMenu.addChild(this.renameItem);
            
            this.editItem = new dijit.MenuItem({
                label: this.resources.EDIT,
                title: this.resources.EDIT_ADDR_POOL,
                iconClass: "spEditLgIcon",
                onClick: dojo.hitch(this, this.editTemplate),
                onKeyPress: dojo.hitch(this, this.editTemplate),
            });
            this.moreMenu.addChild(this.editItem,0);
            
            this.moreButton = new dijit.form.DropDownButton({
                id: this.id + "cp_pooldetails_morebutton",
                showLabel: true,
                label: this.resources.ALL_ACTIONS,
                title: this.resources.ALL_ACTIONS_TITLE,
                dropDown: this.moreMenu
            });

            this.detailsToolBar.addChild(this.moreButton);

//            if(dojo.byId("toolbarDiv")){
  //              var toolbarContainer = dojo.byId("toolbarDiv");
    //            this.detailsToolBar.placeAt(toolbarContainer);  
      //      }
            
        },
        // Edit Template
        editTemplate: function() {
        
            var ipeditdialog = new support.dialogs.NewIPPool({
                id: this.id + "cp_pooldetails_newippool",
                poolIDNum: this.templateIDNum.toString(),
                editMode: true,
                templateInUse: this.templateInUse,
                templateLaunchPoint : "table"
            });
            ipeditdialog.show();

        },
        // Rename Template
        renameTemplate: function() {
            
            var renamedialog = new support.dialogs.Rename({
                id: this.id + "cp_pooldetails_rename",
                templateIDNum: this.templateIDNum,
                ippoolMode: true,
                launchPoint: "details"
            });
            renamedialog.show();

            
        }
    });

});