define([ 
        "dojo", 
        "dijit", 
        "support/panels/Panel", 
        "dojo/i18n!./nls/DetailsDependentTable", 
        "dojo/text!./templates/DetailsDependentTable.html",
        "dojo/data/ItemFileWriteStore",
        "support/base/TableMixin"        
        ],function(dojo, dijit, panel, mri, template) {

    return dojo.declare("support.panels.PoolDetailsDependent", [ support.panels.Panel , support.base.TableMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in managing server profiles
        // for Configuration Patterns.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "DetailsDependentTable",
        
        mode: "", // Current Context Mode
        
        _detailsTable: null,
        
        profileLayout: {            /* Table View Options */
            flatView: true, // Flat Table View
            chassisView: true, // By Chassis View
            groupView: true, // By Group View
            templateView: true // By Template View
            /* Toolbar Buttons and Menus */

            /* "More" menu options */

            /* Table Columns To Display */
        },
        
        templateIDNum:"",   //Category Template ID   
        templateType: "",   //Template Type
        
        // ID prefix used for table
        gridID: "PoolDetailsDependentTable",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/DetailsDependentTable.css",

        // When this method is called, all variables
        // inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget
        // property variables defined in the template
        // HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after
        // buildRendering(). This is useful to override
        // when
        // you need to access and/or manipulate DOM
        // nodes included with your widget.
        // DOM nodes and widgets with the
        // dojoAttachPoint attribute specified can now
        // be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in
        // buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);

            // Setup table widget...
            var tableStore = this._retrieveStore();
            var tableStructure = this._getTableStructure(); // Get structure for current view

            this._detailsTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.gridID, true);

            setTimeout(dojo.hitch(this, function() {
    	        this._detailsTable.placeAt(this.detailsTable);
	            this._detailsTable.startup();
            }), 500);
                        
            // Connect handlers to reset table capabilities
            dojo.connect(this._detailsTable.select.row, "onSelectionChange", this, this.setTableCapabilities); // Fires on row selection
            dojo.connect(this._detailsTable, "setStore", this, this.setTableCapabilities); // Fires on store reset                                    
        },
        
        // Connect to data source and create models for
        // TreeTable widgets.
        _retrieveStore: function() {
                                               
                    var ranges = "";
                    var allocated = "";
                    
                    var xhrGet = dojo.xhrGet({
                        url: this.REST_PATH + "ippool/"+this.templateIDNum,
                        handleAs: "json",
                        sync: true,
                        preventCache: true,
                        load: function(response) {                            
                            ranges = response.ranges;
                            allocated = response.allocations;
                        }
                    });

                    var numOfranges = ranges.length;
                    var numOfallocated = allocated.length;
                                                                              
                    var loopcount;
                    var loopcount2;
                    
                    for (loopcount = 0; loopcount < numOfranges; loopcount++) {


                        ranges[loopcount].allocated = ranges[loopcount].allocatedPct;
                        ranges[loopcount].categoryPatternName = " ";
                        ranges[loopcount].systemPatternName = " ";
                        ranges[loopcount].chassis = " ";
                        ranges[loopcount].bay = " ";
                        ranges[loopcount].server = " ";
                        ranges[loopcount].adapter = " ";

                        var childrenArray = new Array();

                        for (loopcount2 = 0; loopcount2 < numOfallocated; loopcount2++) {
                        
                            if(allocated[loopcount2].rangeName == ranges[loopcount].name){
                                var childObject = new Object();
                                childObject.name = allocated[loopcount2].address;
                                childObject.allocated = allocated[loopcount2].profileName;
                                childObject.categoryPatternName = allocated[loopcount2].categoryPatternName;
                                childObject.systemPatternName = allocated[loopcount2].systemPatternName;
                                childObject.chassis = allocated[loopcount2].chassisName;                                
                                childObject.bay = allocated[loopcount2].bay;
                                childObject.server = allocated[loopcount2].serverName;
                                childObject.adapter = allocated[loopcount2].adapter;
                                childObject.id = allocated[loopcount2].id;
                                childrenArray.push(childObject);
                            }
          
                        }
                        
                        ranges[loopcount].children = childrenArray;                                                 
                    }
                                        
                    ranges = dojo.toJson(ranges);
                    
                    
                    var tableStore = new dojo.data.ItemFileWriteStore({
                          data: dojo.fromJson("{ \"items\" : " + ranges + ", \"label\" : \"id\", \"identifier\" : \"id\" }")
                        });

                    return tableStore;
                        
        },
        // Re-render Tree table with new columns/data
        // presentation
        _reRenderTable: function() {
            // 99275: Get function from tree table widget
        },        
        // Columns to be displayed on table
        _getTableStructure: function() {
            var cols = new Array();

               cols.push({
               field: "name",
               name: this.locResources.ADDRESSESBYRANGE,
               width: "15%",
               title: this.locResources.ADDRESSESBYRANGE});
               
               cols.push({
               field: "allocated",
               name: this.resources.ALLOCATED,
               formatter: dojo.hitch(this, this.displayAllocation),
               width: "15%",
               title: this.resources.ALLOCATED});
                   
               cols.push({
               field: "categoryPatternName",
               name: this.locResources.CATEGORY_PATTERN,
               width: "15%",
               title: this.locResources.CATEGORY_PATTERN});
                   
               cols.push({
               field: "systemPatternName",
               name: this.locResources.SYSTEM_PATTERN,
               width: "15%",
               title: this.locResources.SYSTEM_PATTERN});
                   
               cols.push({
               field: "server",
               name: this.locResources.SYSTEM,
                width: "10%",
                title: this.locResources.SYSTEM});
                   
               cols.push({
               field: "chassis",
               name: this.resources.CHASSIS,
               width: "10%",
               title: this.resources.CHASSIS});
                   
               cols.push({
               field: "bay",
               name: this.resources.BAY,
               width: "5%",
               title: this.resources.BAY});
                   
               cols.push({
               field: "adapter",
               name: this.resources.INTERFACE,
               width: "15%",
               title: this.resources.INTERFACE});
                   
               return cols;
        },
        // Get number of selected rows in tree table
        _getSelectedRowCount: function() {
            return this._detailsTable.select.row.getSelected().length;
        },

        // Get row objects for selected rows
        _getSelectedRows: function() {
            return this._detailsTable.select.row.getSelected();
        },
        // Set Table Capabilities
        setTableCapabilities: function() {
        },
        
        displayAllocation : function(item) {
            
            var value = item.allocated;
            
            if (!value) {
                return "";
            }
                                               
            var percentVal = 0;
            
            var strVal = new String(value);

            var sepIdx = strVal.indexOf("::");
            if (sepIdx > 0) {

	            var alloc = strVal.substring(0, sepIdx);            
    	        var total = strVal.substring(sepIdx + 2);            
        	    percentVal = (alloc / total) * 100;
                            
	            if(percentVal < 1 && percentVal > 0){  //If number smaller then 1% just show 1%            
    	            percentVal = 1;                
        	    }
            
            }
            var displayString = percentVal + "%  (" + this.locResources.ALLOCATED_DESC.replace("%1", alloc).replace("%2", total) + ")";
            
            return displayString;

            
        }
 });
});