define([
        "dojo", 
        "dijit", 
        "support/panels/Panel", 
        "dojo/i18n!./nls/PortVirtualizationTable", 
        "dojo/text!./templates/PortVirtualizationTable.html", 
        "dojox/grid/DataGrid", 
        "dijit/form/ValidationTextBox",  
        "dijit/form/NumberTextBox", 
        "dojo/data/ItemFileWriteStore",
        "dojo/data/ItemFileReadStore",
        "dojox/grid/cells/_base",
        "dijit/form/Button",
        "dijit/form/Select",
        "support/widgets/LabelledSlider",
        "support/widgets/SelectWithButtons" 
    ], function (dojo, dijit, panel, mri, template) {

	dojo.declare("support.panels.PortVirtualizationTable", [ support.panels.Panel ], {

	    // templateString: [protected] String
	    // This widget's HTML template, provided.
	    templateString: template,
	        
	    // locResources: [protected] Object
	    // This widget's National Language Source (NLS) object.
	    locResources: mri, 
	    
		widgetsInTemplate : false,
		baseClass: "PortVirtualizationTable",
			
		adapterId: null,
		mode: null,
		protocol: null,
		numRows: 4,
		
		_release: "GA1",
		
		_physicalFunctionGrid: null,
		_physicalFunctionStore: null,
		_physicalFunctionLayoutIndependent: null,
		_physicalFunctionLayoutVirtualFabric: null,
		_physicalFunctionLayoutUFP: null,
		
		_selectWithButtons: null,
		
		_adapterTypeData: null,
		_adapterTypeStore: null,
		
		_enableArray: null,
		_saveEnableArray: null,
        
        _physicalFunctionArray: null,
        
        _vlanArray: null,
        _vlanValidArray: null,
        _vlanMsg: null,
        _vlanErrorMsg: null,
                
        _setSliderValueArray: null,
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/PortVirtualizationTable.css",
        
        constructor: function() {
            this.npvtResources = dojo.i18n.getLocalization("support.dialogs", "NewPortVirtualizationTemplate", this.lang);
        },

		postMixInProperties : function() {
	        this.inherited(arguments);
	        this._physicalFunctionArray = new Array();
	        this._setSliderValueArray = new Array();
	        this._vlanArray = new Array();
	        this._vlanValidArray = new Array();
	        this._selectWithButtons = new Array();
	        if (this.mode == null) {
	            this.mode = "vNIC_independent";
	        }
	        if (this.protocol == null) {
	            this.protocol = "ethOnly";
	        }
	        this._setDefaults();
			this._connectDataModels();
		},

		postCreate : function(){
		    this.inherited(arguments);
		    
		    this._vlanMsg = this.locResources.VLANMESSAGE.replace("%1", 2).replace("%2", 4094);
		    this._vlanErrorMsg = this.locResources.VLANINVALIDMESSAGE.replace("%1", 2).replace("%2", 4094);
		    
		    if (this.mode == "vNIC_IBMvf") {
		        this._physicalFunctionGrid = new dojox.grid.DataGrid({
		            id: this.id + "cp_portvirtualizationtable_physicalfunctiongrid",
		            store: this._physicalFunctionStore,
	                structure: this._physicalFunctionLayoutVirtualFabric,
	                autoWidth: false,
	                autoHeight: true,
	                singleClickEdit: true,
	                columnReordering: false,
                    canEdit: dojo.hitch(this, function(inCell, inRowIndex) {
                        var vportFunction = this._getVportFunction(inRowIndex);
                        if (this._enableArray[inRowIndex]) {  // Is the row enabled? (vf mode should always be enabled)
                            if (this.protocol != "ethOnly" && vportFunction != "nic") {  // Vlan id is not set for non-ethOnly
                                return false;
                            } else {
                                return true;
                            }
                        } else {
                            return false;
                        }
                    }),
	                canSort: function(col){
	                    return false;
	                },
	                style: "width: 495px;",
	                noDataMessage: this.locResources.NO_ADAPTERS,
	                onApplyCellEdit: function() {
                        this.update();
                    },
                    onCellFocus: function(inCell, rowIdx) {
                    	// Override this event to allow better keyboard control for the slider.
                    },
	                selectionMode: "none"                
		        }, this._physicalFunctionGridNode);
		        this._physicalFunctionGrid.startup();
		    } else if (this.mode == "vNIC_IBMufp") {
                this._physicalFunctionGrid = new dojox.grid.DataGrid({
                    id: this.id + "cp_portvirtualizationtable_physicalfunctiongrid",
                    store: this._physicalFunctionStore,
                    structure: this._physicalFunctionLayoutUFP,
                    autoWidth: false,
                    autoHeight: true,
                    singleClickEdit: true,
                    columnReordering: false,
                    canEdit: dojo.hitch(this, function(inCell, inRowIndex) {
                        var vportFunction = this._getVportFunction(inRowIndex);
                        if (this._enableArray[inRowIndex]) {  // Is the row enabled? (vf mode should always be enabled)
                            if (this.protocol != "ethOnly" && vportFunction != "nic") {  // Vlan id is not set for non-ethOnly 
                                return false;
                            } else {
                                return true;
                            }
                        } else {
                            return false;
                        }
                    }),
                    canSort: function(col){
                        return false;
                    },
                    style: "width: 495px;",
                    noDataMessage: this.locResources.NO_ADAPTERS,
                    onApplyCellEdit: function() {
                        this.update();
                    },
                    selectionMode: "none"                
                }, this._physicalFunctionGridNode);
                this._physicalFunctionGrid.startup();
            } else {		  
		        this._physicalFunctionGrid = new dojox.grid.DataGrid({
		            id: this.id + "cp_portvirtualizationtable_physicalfunctiongrid",
		            store: this._physicalFunctionStore,
                    structure: this._physicalFunctionLayoutIndependent,
                    autoWidth: false,
                    autoHeight: true,
                    singleClickEdit: true,
                    columnReordering: false,
                    canEdit: dojo.hitch(this, function(inCell, inRowIndex) {
                        var vportFunction = this._getVportFunction(inRowIndex);
                        if (this._enableArray[inRowIndex]) {  // Is the row enabled?
                            if (this.protocol != "ethOnly" && vportFunction != "nic") {  // Vlan id is not set for non-ethOnly 
                                return false;
                            } else {
                                return true;
                            }
                        } else {
                            return false;
                        }
                    }),
                    canSort: function(col){
                        return false;
                    },
                    style: "width: 545px;",
                    noDataMessage: this.locResources.NO_ADAPTERS,                    
                    onApplyCellEdit: function() {
                        this.update();
                    },
                    onCellFocus: function(inCell, rowIdx) {
                    	// Override this event to allow better keyboard control for the slider.
                    },
                    selectionMode: "none"                
                }, this._physicalFunctionGridNode);
                this._physicalFunctionGrid.startup();
		    }		    
            this._physicalFunctionGrid.update();
            for (var rowCount=0;rowCount < this.numRows;++rowCount) {
                this._addRow();
            }
		},
				
	    //Sets up the data, store, and layout for the grid
	    _connectDataModels : function(){
        	this._physicalFunctionStore = new dojo.data.ItemFileWriteStore({        	
        	    data: { "items" : [] }
        	});
        	if (this._physicalFunctionLayoutIndependent == null) {
        	    if (this._release == "GA1") {
        	        // No physical function template support
                    this._physicalFunctionLayoutIndependent = [ 
                        { field: "portNumber", name: this.locResources.FUNCTION, editable: false, width: "75px", formatter: dojo.hitch(this, this._formatFunction) }, 
                        { field: "enabled", name: this.locResources.ENABLE, width: "50px", editable: false, formatter: dojo.hitch(this, this._formatEnable) },
                        { field: "porttype", name: this.resources.TYPE, width: "100px", editable: false, formatter: dojo.hitch(this, this._formatPortType) }, 
                        { field: "bandwidth", name: this.locResources.BANDWIDTH, width: "175px", editable: false, formatter: this._formatBandwidth },
                        { field: "vlan", name: this.locResources.VLAN, formatter: this._formatVlan, editable: true, width: "auto" }
                    ];        	        
        	    } else {
        	        this._physicalFunctionLayoutIndependent = [ 
        	            { field: "portNumber", name: this.locResources.FUNCTION, editable: false, width: "75px", formatter: dojo.hitch(this, this._formatFunction) }, 
        	            { field: "enabled", name: this.locResources.ENABLE, width: "50px", editable: false, formatter: dojo.hitch(this, this._formatEnable)},
        	            { field: "porttype", name: this.resources.TYPE, width: "100px", editable: false, formatter: dojo.hitch(this, this._formatPortType) }, 
        	            { field: "bandwidth", name: this.locResources.BANDWIDTH, width: "140px", editable: false, formatter: this._formatBandwidth },
        	            { field: "physicalfunction", name: this.locResources.PHYSICALFUNCTION, editable: false, formatter: this._formatPhysicalFunction, width: "280px" },
        	            { field: "vlan", name: this.locResources.VLAN, formatter: dojo.hitch(this, this._formatVlan), editable: false, width: "140px" }
        	        ];
        	    }
        	}
        	if (this._physicalFunctionLayoutVirtualFabric == null) {
        	    if (this._release == "GA1") {
                    this._physicalFunctionLayoutVirtualFabric = [ 
                        { field: "portNumber", name: this.locResources.FUNCTION, width: "75px", formatter: dojo.hitch(this, this._formatFunction) }, 
                        { field: "porttype", name: this.resources.TYPE, width: "100px", formatter: dojo.hitch(this, this._formatPortType) }, 
                        { field: "vlan", name: this.locResources.VLAN, formatter: this._formatVlan, editable: true, width: "auto" }
                    ];        	        
        	    } else {
        	        this._physicalFunctionLayoutVirtualFabric = [ 
        	            { field: "portNumber", name: this.locResources.FUNCTION, width: "75px", formatter: dojo.hitch(this, this._formatFunction) }, 
        	            { field: "porttype", name: this.resources.TYPE, width: "100px", formatter: dojo.hitch(this, this._formatPortType) }, 
        	            { field: "physicalfunction", name: this.locResources.PHYSICALFUNCTION, formatter: this._formatPhysicalFunction, width: "280px" },
        	            { field: "vlan", name: this.locResources.VLAN, formatter: dojo.hitch(this, this._formatVlan), width: "auto" }
        	        ];     
        	    }
        	}
            if (this._physicalFunctionLayoutUFP == null) {
                if (this._release == "GA1") {
                    this._physicalFunctionLayoutUFP = [ 
                        { field: "portNumber", name: this.locResources.FUNCTION, width: "75px", formatter: dojo.hitch(this, this._formatFunction) }, 
                        { field: "porttype", name: this.resources.TYPE, formatter: dojo.hitch(this, this._formatPortType), width: "auto" }
                    ];                  
                } else {
                    this._physicalFunctionLayoutUFP = [ 
                        { field: "portNumber", name: this.locResources.FUNCTION, width: "75px", formatter: dojo.hitch(this, this._formatFunction) }, 
                        { field: "porttype", name: this.resources.TYPE, width: "100px", formatter: dojo.hitch(this, this._formatPortType) }, 
                        { field: "physicalfunction", name: this.locResources.PHYSICALFUNCTION, formatter: this._formatPhysicalFunction, width: "auto" }
                    ];     
                }
            }
        	
            var adapterData = null;
            var xhrArgs = {
                url: this.REST_PATH + "supportedDevices/ioAdapter/list",
                handleAs: "json",
                sync: true,
                load: function(data) {
                    adapterData = data;
                },
                error: function(error) {
                    alert(error);
                }
            };

            // Call the asynchronous xhrGet
            dojo.xhrGet(xhrArgs);

            this._adapterTypeData = adapterData;

            this._adapterTypeStore = new dojo.data.ItemFileReadStore({
                data: this._adapterTypeData
            });        	
        },
        
        _setDefaults: function() {
            var enableArray = new Array();
            for (var rowCount=0; rowCount < this.numRows; ++rowCount) {
                enableArray[rowCount] = true;
            }
            this._enableArray = enableArray;
            this._saveEnableArray = enableArray;
        },
        
        /* Formatters */
        _formatFunction: function(value, idx) {            
            var rv = "";
            // Currently have MRI for [FUNCTION_1 .. FUNCTION_16] only.
            if ((idx >= 0) && (idx < 16)) {
                var mriTag = "FUNCTION_" + (idx+1);
                rv = this.locResources[mriTag];
            }
            return rv;
            
        },
        
        _formatEnable: function(value, idx) {
            var item = this._physicalFunctionStore._arrayOfTopLevelItems[idx];            
            var checkbox = new dijit.form.CheckBox({ 
                checked: this._enableArray[idx]
            });
            dojo.connect(checkbox, "onChange", this, function(newValue) {
                if (newValue == false) {
                    this._vlanValidArray[idx] = "disabled";
                    this._physicalFunctionGrid.update();
                } else {
                    this._vlanValidArray[idx] = "enabled";
                    this._physicalFunctionGrid.update();
                }                
                if (this._setSliderValueArray[idx] == undefined) {
                    this._setSliderValueArray[idx] = 25;
                }
                this._physicalFunctionStore.setValue(item, "bandwidth", this._setSliderValueArray[idx]);
                this._enableArray[idx] = newValue;
                this._onEnableChange(newValue, idx);                
            });

            return checkbox;
        },
        
        _formatPortType: function(value, idx) {
            var vportFunction = this._getVportFunction(idx);
            var rv = [""];
            
            if (typeof(this.protocol) != "string") {
                this.protocol = this.protocol[0];
            }
            var isDisabled = false;
            if (this._enableArray[idx] == "disabled" || this._enableArray[idx] == false) {
                isDisabled = true;
            }
            if (this.protocol == "eth+fcoe" && vportFunction == "fcoe") {
                rv.push("<span title=\"" + (isDisabled ? this.resources["DISABLED"] : this.resources["FCOE_TITLE"]) + "\" style=\"white-space: nowrap;\">");
                rv.push("<img src=\"" + this._blankGif + "\" alt=\"\" class=\"" + (isDisabled ? "spInactiveIcon" : "spFcoePortIcon") + "\" align=\"left\" style=\"vertical-align: text-top; margin-right: 4px;\">");
                rv.push(this.resources["FCOE"]);
                rv.push("</span>");                
            } else if (this.protocol == "eth+iscsi" && vportFunction == "iscsi") {
                rv.push("<span title=\"" + (isDisabled ? this.resources["DISABLED"] : this.resources["ISCSI_TITLE"]) + "\" style=\"white-space: nowrap;\">");
                rv.push("<img src=\"" + this._blankGif + "\" alt=\"\" class=\"" + (isDisabled ? "spInactiveIcon" : "spEthPortIcon") + "\" align=\"left\" style=\"vertical-align: text-top; margin-right: 4px;\">");
                rv.push(this.resources["ISCSI"]);
                rv.push("</span>");
            } else {            
                rv.push("<span title=\"" + (isDisabled ? this.resources["DISABLED"] : this.resources["ETHERNET"]) + "\" style=\"white-space: nowrap;\">");
                rv.push("<img src=\"" + this._blankGif + "\" alt=\"\" class=\"" + (isDisabled ? "spInactiveIcon" : "spEthPortIcon") + "\" align=\"left\" style=\"vertical-align: text-top; margin-right: 4px;\">");
                rv.push(this.resources["ETHERNET"]);
                rv.push("</span>");
            }
            return rv.join("");
        },
        
        _formatBandwidth: function (value, idx) {
            var bot = this.grid.getParent();
            
            if (bot._enableArray[idx] == false) {
                return null;
            }
           
            if (bot._setSliderValueArray[idx] == undefined) {
                return null;
            }
            var slider = new support.widgets.LabelledSlider({
                initialValue: bot._setSliderValueArray[idx], //store.getValue(item, "bandwidth"),
                onChange: function(newValue) {
                    bot._setSliderValueArray[idx] = newValue;                    
                    bot._onSliderChange(newValue, idx);                    
                }
            });

            if (bot.CP_RUN_ENVIRONMENT == "fsm") {
                /*  The DataGrid has an annoying habit of taking focus away
                from the widget within the cell on keyboard events (when
                not in edit mode).  Had to play with a few events to keep 
                the focus on the slider while using the left and right 
                arrow keys. */
                dojo.connect(this.grid, "onCellFocus", function(c, rowIdx) {
                    // If any cell in the row is focused, give the slider focus
                    if (rowIdx == idx) {                
                        slider._slider.focus();
                    }
                });
                
                dojo.connect(this.grid, "onCellMouseOver", function(e) {
                    // If the bandwidth cell is moused over, give the cell and
                    // the slider focus.
                    if (e.rowIndex == idx && e.cellIndex == 3) {
                        this.focus.setFocusCell(e.cell, e.rowIndex);
                        slider._slider.focus();
                    }
                });                
            }
            
            dojo.connect(this.grid, "onCellClick", function(e) {
                if (bot.CP_RUN_ENVIRONMENT == "fsm") {
                    //  If the bandwidth cell is clicked, give the slider focus
                    if (e.rowIndex == idx && e.cellIndex == 3) {
                        slider._slider.focus();
                    }
                }
                // If the vlan tag cell is clicked, set edit mode
                // This is needed because the onCellFocus event is overidden
                if (e.rowIndex == idx && e.cellIndex == 4) {
                    this.edit.setEditCell(this.focus.cell, e.rowIndex);
                }
            });
            
            slider._destroyOnRemove = true;
            return slider;
        },        

        _formatPhysicalFunction: function (value, idx) {            
            var bot = this.grid.getParent();            
            
            if (bot._enableArray[idx] == false) {
                return null;
            }
            var phyfunc = "";
            if (bot.protocol == "eth+fcoe") {
                if (idx == 1) {
                    phyfunc = "fcoe";
                } else {
                    phyfunc = "nic";
                }
            } 
            if (bot.protocol == "eth+iscsi") {
                if (idx == 1) {
                    phyfunc = "iscsi";
                } else {
                    phyfunc = "nic";
                }
            }
            if (bot.protocol == "ethOnly") {
                phyfunc = "nic";
            }
             
            var selector = new support.widgets.SelectWithButtons({
                addType: bot.adapterId,
                physicalFunction: phyfunc,
                restpath: bot.REST_PATH,
                templateValue: bot._physicalFunctionArray[idx],
                onChange: function(value) {
                    var portPhysicalFunctionTemplateData = null;
                    var xhrAdapTemp = {
                        url: bot.REST_PATH + "template/" + value,
                        handleAs: "json",
                        sync: true,
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        load: function(data) {
                            portPhysicalFunctionTemplateData = data;
                        },
                        error: function(error) {
                            alert(error);
                        }
                    };

                    // Call the asynchronous xhrGet
                    dojo.xhrGet(xhrAdapTemp);
                    
                    if (portPhysicalFunctionTemplateData.items[0].vlanId == "*NONE") {
                        portPhysicalFunctionTemplateData.items[0].vlanId = "0";                                
                    }
                    
                    bot._vlanArray[idx] = portPhysicalFunctionTemplateData.items[0].vlanId;
                    
                    if (dojo.byId(bot.id + "_vlan" + idx) != null) {
                        if (bot._validateVlan(idx)) {
                            dojo.byId(bot.id + "_vlan" + idx).innerHTML = portPhysicalFunctionTemplateData.items[0].vlanId;
                            bot._vlanValidArray[idx] = "valid";
                        } else {
                            dojo.byId(bot.id + "_vlan" + idx).innerHTML = "<div title=\"" + bot.npvtResources.VLANERROR +"\" style=\"float:left;\" class=\"spWarningIcon\"></div>&nbsp;" + portPhysicalFunctionTemplateData.items[0].vlanId;
                            bot._vlanValidArray[idx] = "invalid";                            
                        }
                        dojo.publish("InvalidPhysicalFunction", bot._validateVlan(idx));
                    }
                    bot._onSelectChange(value, idx);
                }
            });
            
            selector._destroyOnRemove = true;
            return selector;
        },        

        _formatVlan: function (value, idx, cell) {
            var bot = this.grid.getParent();
            var grid = this.grid;
            var store = this.grid.store;
            var topLevelItems = store._arrayOfTopLevelItems;  
            var item = store._arrayOfTopLevelItems[idx];
            var vportFunction = bot._getVportFunction(idx);
            
            dojo.connect(this.grid,"onBlur", function(){
                dijit.hideTooltip(cell.getNode());                
            });

            
            if (bot._release == "GA1") {
                if (bot.protocol != "ethOnly" && vportFunction != "nic") {
                    bot._vlanValidArray[idx] = true; 
                    // Still need to flag that this row is valid
                    dojo.publish("InvalidPhysicalFunction", true);
                    return " ";
                }
                var textbox = new dijit.form.NumberTextBox({
                   //id: "cp_portvirtualizationtable_numbertextbox_" + this.id,
                   value: value,  
                   trim: true,
                   title: bot._vlanMsg,  
                   invalidMessage: null,
                   intermediateChanges: true,
                   constraints: {pattern:"#"},
                   style: "width:60px;",
                   onChange: function(newValue) {                       
                       bot._setStoreValue(item, "vlan", newValue);
                   },
                   onFocus: function() {
                       window.setTimeout(function() {
                           grid.edit.setEditCell(cell, idx);
                       },10);
                   },
                   isValid: function() {
                       if (bot._vlanValidArray[idx] == "disabled") {
                           //dojo.publish("InvalidPhysicalFunction", true);
                           return true;
                       }
                       if (isNaN(this.get("value")) == true) {
                           this.set("value", 0);
                           bot._setStoreValue(item, "vlan", 0);
                       }
                       var num=this.get("value");
                       if (!bot._enableArray[idx]) {
                           bot._vlanValidArray[idx] = true;                           
                           //dojo.publish("InvalidPhysicalFunction", true);
                           return true;
                       }
                       if(bot.mode == "vNIC_independent") {
                           if (num == 0) {
                               this.set("title", bot._vlanErrorMsg);
                               bot._vlanValidArray[idx] = false;
                               dojo.publish("InvalidPhysicalFunction", false);
                               return false;                               
                           }
                       }                   
                       if(num == 0 || (num >= 2 && num <= 4094)) {
                           for (var numItems=0;numItems < topLevelItems.length;++numItems) {
                               if (value != 0) {
                                   if (numItems == idx) {
                                       continue;
                                   }
                                   if (store.getValue(topLevelItems[numItems], "vlan") == value) {
                                       if (bot._vlanValidArray[numItems] == "disabled") {
                                           continue;
                                       }
                                       this.set("title", bot._vlanErrorMsg);
                                       bot._vlanValidArray[idx] = false;
                                       dojo.publish("InvalidPhysicalFunction", false);
                                       return false;
                                   }
                               }
                           }                                      
                           bot._vlanValidArray[idx] = true;
                           for (var invalidCount=0;invalidCount < bot._vlanValidArray.length;++invalidCount) {
                               if (bot._vlanValidArray[invalidCount] == false) {
                                   dojo.publish("InvalidPhysicalFunction", false);
                                   return true;
                               } 
                           }
                           // check if any of the other cells are invalid                           
                           dojo.publish("InvalidPhysicalFunction", true);
                           return true;
                       } else {                                             
                           this.set("title", bot._vlanErrorMsg);
                           bot._vlanValidArray[idx] = false;
                           dojo.publish("InvalidPhysicalFunction", false);
                           return false;
                       }
                   }
                });
                textbox._destroyOnRemove = true;
                if (bot._enableArray[idx] == false) {                
                    return null;
                }       
                
             // update the data store when the user moves the mouse out of the
                //   text field; this is needed for IE because the onChange
                //   handler will not fire until the user hits TAB, ENTER, or
                //   clicks on the button area to the right of the field
                dojo.connect(this.grid,"onMouseLeave", function(e){
                    if(e.rowIndex == idx) { // only do this for the row the event corresponds to
                        // e.cell.getValue works for IE but not Firefox; this is
                        //   ok because Firefox saves the value in the onChange
                        //   handler
                        var value = e.cell.getValue(idx);
                        if(value != null) {
                            bot._setStoreValue(item, "vlan", value);
                        }
                    }
                    // Check if cell is in edit mode.
                    // If so, apply the edit.
                    if (grid.edit.isEditing()) {
                        grid.edit.apply();
                   }
                });
                return textbox;
                 
            } else {
                var vlanstring = "<div id=\"" + bot.id + "_vlan" + idx + "\"></div>";
            
                if (bot.protocol != "ethOnly" && idx == 1) {
                    return " ";
                }
                if (bot._enableArray[idx] == false) {                
                    return null;
                }            
                return vlanstring;
            }
        },        
        
        /* Events */
        _onEnableChange: function(newValue, idx) {
            this._physicalFunctionGrid.updateRow(idx);
            this.onEnableChange(newValue, idx);
        },

        onEnableChange: function(newValue, idx) {

        },        
                
        _onSliderChange: function(sliderval, idx) {
            this.onSliderChange(sliderval, idx);
        },
        
        onSliderChange: function(sliderval, idx) {
            
        },
        
        _onSelectChange: function(templateId, idx) {            
            this.onSelectChange(templateId, idx);
        },
        
        onSelectChange: function(templateId, idx) {

        },
        
        _onTextBoxChange: function(newValue, idx) {
            this.onTextBoxChange(newValue, idx);
        },
        
        onTextBoxChange: function(newValue, idx) {
            
        },
                
        adapterChanged: function(adapterId) {
            this.adapterId = adapterId;
            this._physicalFunctionGrid.update();
        },
        
        modeChanged: function(mode) {
            console.log("Mode changing to: " + mode);
            var previousMode = this.mode;
            this.mode = mode;
            if (this.mode == "vNIC_independent") {
                // Reset the enabled state
                for (var rowcount=0; rowcount < this.numRows; ++rowcount) {                    
                    this._enableArray[rowcount] = this._saveEnableArray[rowcount];
                    // Need to make sure the vlan validator knows that the row is disabled
                    if (!this._saveEnableArray[rowcount]) {
                        this._vlanValidArray[rowcount] = "disabled";
                    }
                    this._onEnableChange(this._saveEnableArray[rowcount], rowcount);
                }
                this._physicalFunctionGrid.set("structure", this._physicalFunctionLayoutIndependent);
            }
            else if (this.mode == "vNIC_IBMvf") {
                if (previousMode == "vNIC_independent") {
                    // Save the previous enabled state                
                    this._saveEnableArray = this._enableArray.slice();
                }
                // Reenable any rows
                for (var rowcount=0;rowcount < this.numRows;++rowcount) {   
                    this._enableArray[rowcount] = true;
                    // Need to make sure the vlan validator knows that the row is enabled
                    this._vlanValidArray[rowcount] = "enabled";
                    this._onEnableChange(true, rowcount);
                }
                this._physicalFunctionGrid.set("structure", this._physicalFunctionLayoutVirtualFabric);
            }
            else if (this.mode == "vNIC_IBMufp") {
                if (previousMode == "vNIC_independent") {
                    // Save the previous enabled state                
                    this._saveEnableArray = this._enableArray.slice();
                }            
                // Reenable any rows
                for (var rowcount=0;rowcount < this.numRows;++rowcount) {   
                    this._enableArray[rowcount] = true;
                    // Need to make sure the vlan validator knows that the row is enabled
                    this._vlanValidArray[rowcount] = "enabled";
                    this._onEnableChange(true, rowcount);
                }
                this._physicalFunctionGrid.set("structure", this._physicalFunctionLayoutUFP);
            }
        },
        
        protocolChanged: function(protocol) {
            this.protocol = protocol;
            this._physicalFunctionGrid.update();
        },
        
        setSliderValue: function(value, idx) {            
            if (value != null) {
                this._setSliderValueArray[idx] = value;            
            }
            this._physicalFunctionGrid.update();
        },
                
        setEnableValue: function(value, idx) {            
            if (value != null) {
                this._enableArray[idx] = value;
            }
            
            this._physicalFunctionGrid.update();
        },
        
        setFunctionValue: function(value, idx) {
            if (value != null) {
                this._physicalFunctionArray[idx] = value;
            }
            
            this._physicalFunctionGrid.update();
        },
        
        setVlanValue: function(value, idx) {
            var topLevelItems = this._physicalFunctionStore._arrayOfTopLevelItems;
            if (value != null) {
                this._physicalFunctionGrid.store.setValue(topLevelItems[idx], "vlan", value);
            }
        },
        
        setOverallPct: function(value) {
            dojo.byId(this.id + "_overallpct").innerHTML = value + "%";
        },
        
        displayOverallPctWarning: function(displayvalue) {
            this._overallPctWarning.style.display = displayvalue;
        },
        
        toggleOverallPctSection: function(value) {
            this._labelSection.style.display = value;
        },
        
        _setStoreValue: function(item, type, value) {
            if (item != null) {
                this._physicalFunctionGrid.store.setValue(item, type, value);
            }
        },
        
        _validateVlan: function(idx) {
            for (var vlancount=0;vlancount < this._vlanArray.length;++vlancount) {
                if (this._vlanArray[vlancount] == 0) {
                    continue;
                }
                if (idx == vlancount) {
                    continue;
                }
                if (this._vlanArray[idx] == this._vlanArray[vlancount]) {
                    return false;
                }
            }
            return true;
        },
        
        getVlanValues: function() {
            var topLevelItems = this._physicalFunctionStore._arrayOfTopLevelItems;
            var vlanArray = new Array();
            for (var itemCount=0;itemCount < topLevelItems.length;++itemCount) {
                vlanArray.push(this._physicalFunctionStore.getValue(topLevelItems[itemCount], "vlan"));
            }
            
            return vlanArray;
        },
        
        //Adds a row to the end of the grid    
        _addRow : function(){
            var grid = this._physicalFunctionGrid;
            var item = null;
            if (this.mode == "vNIC_independent") {
                item = {portNumber: null, enabled: true, porttype: null, bandwidth: null, vlan: 0};
            }
            else if (this.mode == "vNIC_IBMvf") {
                item = {portNumber: true, porttype: null, vlan: 0};
            }
            else if (this.mode == "vNIC_IBMufp") {
                item = {portNumber: true, porttype: null};
            }
            grid.store.newItem(item);
            grid.store.save();
            grid.render();
        },
        
        _getVportFunction: function(vportNumber) {
            for (var adapterCount=0;adapterCount<this._adapterTypeData.items.length;++adapterCount) {
                if (this.adapterId == "*ANY") {
                    this.adapterId = "adapter:1";  // 4054 will be the default for now
                }
                // Find the adapter data that corresponds to the current adapter
                if (this._adapterTypeData.items[adapterCount].id == this.adapterId) {
                    for (var modeCount=0;modeCount<this._adapterTypeData.items[adapterCount].portConfigurations.length;++modeCount) {
                        // Find the current mode
                        if (this._adapterTypeData.items[adapterCount].portConfigurations[modeCount].mode == this.mode) {
                            for (var optionCount=0;optionCount<this._adapterTypeData.items[adapterCount].portConfigurations[modeCount].options.length;++optionCount) {
                                // Find the current option
                                if (this._adapterTypeData.items[adapterCount].portConfigurations[modeCount].options[optionCount].optionCode == this.protocol) {
                                    return this._adapterTypeData.items[adapterCount].portConfigurations[modeCount].options[optionCount].vports[vportNumber].physicalFunction;
                                }
                            }
                        }
                    }
                }
            }
            return "nic";
        }
	});
});     	
