define(["dojo",
        "dijit",
        "support/panels/Panel", 
        "dojo/i18n!./nls/ServerConfiguration",
        "dojo/text!./templates/ServerConfiguration.html",
        "dojo/data/ItemFileReadStore",
        "support/base/TableMixin",
        "dijit/form/Button",
        "dijit/ToolbarSeparator",
        "dijit/Menu",
        "dijit/MenuItem", 
        "dijit/form/DropDownButton",
        "support/panels/CheckButton",
        "support/temp/ModalDialog",
        "dijit/Toolbar"
        ], function(dojo, dijit, panel, mri, template, lang) {

    return dojo.declare("support.panels.ServerConfiguration", [ support.panels.Panel, support.base.TableMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in displaying Configuration
        // settings associated with a server.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "ServerConfiguration",

        // _detailsTable: [protected] Object
        // This widget's details TreeTable.
        _detailsTable: null,
        _detailsStoreData: null,
        
        _tableToolbar: null,
        
        // ID prefix used for table
        gridID: "ServerConfigTable",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/ServerConfiguration.css",

        mode: "", // Current Context Mode
        templateIDNum: "", //Pattern ID
        templateName:"",   //Pattern Name
        endpointUUID:"",   //Server or CMM UUID
        profileIDNum: "",  //Profile ID
        
        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);

            // Setup table widget...
            var detailsStore = this._retrieveStore();       // Get table store information
            var tableStructure = this._generateDetailsLayout(this._detailsStoreData);  //_getTableStructure(); // Get structure for current view
            
            this._detailsTable = this.getTable(detailsStore, tableStructure, this.resources.NO_DATA, this.gridID, true);

            this._detailsTable.placeAt(this.detailsTable);

            this._detailsTable.startup();

            //Set emptyInfo to loading icon and message for server details
            if(this.mode == "serverDetails" && this.CP_RUN_ENVIRONMENT == "augusta"){
                var emptyHtml = "<div class='dijitContentPaneLoading '><div class='dijitInline dijitIconLoading'></div>"+this.locResources.LOADING_MESSAGE+"</div>";
                this._detailsTable.body.emptyInfo = emptyHtml;
            }else if(this.mode == "serverDetails" && this.CP_RUN_ENVIRONMENT == "fsm"){
                var emptyHtml = "<div class='dijitInline dijitIconLoading'></div>"+this.locResources.LOADING_MESSAGE;
                this._detailsTable.body.emptyInfo = emptyHtml;
            }

            //Set emptyInfo to loading icon and message for chassis details
            if(this.mode == "chassisDetails" && this.CP_RUN_ENVIRONMENT == "augusta"){
                var emptyHtml = "<div class='dijitContentPaneLoading '><div class='dijitInline dijitIconLoading'></div>"+this.locResources.LOADING_MESSAGE+"</div>";
                this._detailsTable.body.emptyInfo = emptyHtml;
            }else if(this.mode == "chassisDetails" && this.CP_RUN_ENVIRONMENT == "fsm"){
                var emptyHtml = "<div class='dijitInline dijitIconLoading'></div>"+this.locResources.LOADING_MESSAGE;
                this._detailsTable.body.emptyInfo = emptyHtml;
            }

            
            this._addToolbarButtons(this._detailsTable); //load table toolbar
            
            //Refresh table data after Server Pattern Wizard changed and saved
            this.subscribe("ServerTemplateComplete", function(data) {
                this._refreshTree();
            });
            
            //Refresh table data after Chassis Pattern Wizard changed and saved
            this.subscribe("ChassisTemplateComplete", function(data) {
                this._refreshTree();
            });

			//Refresh table data after Server or Chassis Pattern Redeploy
            this.subscribe("RefreshAfterReDeploy", function(data) {
                this._refreshTree();
            });
            
            //Refresh status bar data after Server Pattern Deploy
            this.subscribe("DeployServerTemplateComplete", function(data) {
                this._refreshTree();
            });

            //Refresh status bar data after Chassis Pattern Deploy
            this.subscribe("DeployChassisTemplateComplete", function(data) {
                this._refreshTree();
            });

            //Refresh table data
            this.subscribe("ServerDetailRefreshButtonClicked", function(data) {
                this._refreshTree();
            });
        },
        
        // Connect to data source and create models for TreeTable widgets.
        _retrieveStore: function() {
                                
			// setup the URL to be requested
			var requestURL = "";
			if(this.mode == "chassisDetails" &&
			        this.endpointUUID != "") {  // If Chassis Details page
			    requestURL = this.REST_PATH + "details/cmm/" + this.endpointUUID;
			}
			else if(this.mode == "serverTemplateDetails" &&
			        this.templateIDNum != "" &&
			        this.templateIDNum != null) {  // If Server Template Details page
			    requestURL = this.REST_PATH + "details/template/" + this.templateIDNum;
			}
			else if(this.mode == "chassisTemplateDetails" &&
			        this.templateIDNum != "" &&
                    this.templateIDNum != null) {  // If Chassis Template Details page
			    requestURL = this.REST_PATH + "details/cmmtemplate/" + this.templateIDNum;
			}
			else if(this.mode == "serverDetails" &&
	                this.endpointUUID != "") {  // If Server Details page
			    requestURL = this.REST_PATH + "details/server/" + this.endpointUUID;
			}
			else if(this.mode == "serverProfileDetails" &&
			        this.profileIDNum != "" &&
			        this.profileIDNum != null) {  // If Server Profile Details page
			    requestURL = this.REST_PATH + "details/profile/" + this.profileIDNum;
			}
			else if(this.mode == "chassisProfileDetails" &&
			        this.profileIDNum != "" &&
			        this.profileIDNum != null) {  // If Chassis Profile Details page
			    requestURL = this.REST_PATH + "details/cmmprofile/" + this.profileIDNum;
			}

			// if we constructed a request URL properly, make the
			//   request - but make it asynchronously, because some of
			//   these services APIs (i.e. the details/server API)
			//   take a long time to return
			if(requestURL != "") {
			    console.log("In ServerConfiguration; sending services request to URL: " + requestURL);
                 
			    var xhrGet = dojo.xhrGet({
			        url: requestURL,
			        handleAs: "json",
			        preventCache: true,
			        load: dojo.hitch(this, function(response, ioArgs) {
			            console.log("In ServerConfiguration; parsing response from URL: " + requestURL);
			            if(this.mode == "serverDetails") {
			                var i = 0;	
			                while(i < response.items.length && response.items[i].id.indexOf(".computeNode.") != -1 
			                        && !response.items[i].children){
                                response.items[i].type = [];
                                response.items[i].type[i] = "root";
                                var secondaryUUID = response.items[i].id.split(".");
                                secondaryUUID = secondaryUUID[secondaryUUID.length-1];
                                var secondaryInfo = this._getSecondaryNodeDetails(secondaryUUID);
                                if(secondaryInfo != null) {
                                    response.items[i].children = secondaryInfo;
                                }
                                i++;
                            }
			            }
                        
                        
			            var newStore = new dojo.data.ItemFileWriteStore({
                            data: response
                        });
			            
                        if (!("hasChildren" in newStore)) {
                            newStore.hasChildren = function(id, item) { 
                                return item && this.getValues(item, "children").length;
                            };
                        };
                        
                        if (!("getChildren" in newStore)) {
                            newStore.getChildren = function(item) { 
                                return this.getValues(item, "children");
                            };
                        };
                        
                        var tableStructure = this._generateDetailsLayout(response);
                                       
                        this._detailsTable.setColumns(tableStructure);
                        this._detailsTable.setStore(newStore);
                        this._detailsTable.model.clearCache();
                        this._detailsTable.body.refresh().then(this._detailsTable.reexpandRows);
                        
                        dojo.publish("RefreshPageComplete");
                        
			        }),
			        handle: dojo.hitch(this, function(data, ioArgs) {
			        }),
			        error: dojo.hitch(this, function(error, ioArgs) {
		                // No Server Settings                             
		                // show the error message in the table
		                this._detailsTable.body.emptyInfo = this.resources.ERROR_SETTINGS_DETAILS.replace("%1", this.PRODUCT_NAME_LONG).replace("%2", this.PRODUCT_NAME_SHORT);
                            		                
			            dojo.publish("RefreshPageComplete");
			        })
			    });
			}
                
			// return emptyStore as real data will be set when rest call is returned
			    var emptyStore = new dojo.data.ItemFileReadStore({
			        data: {"items" : []}
			    });
                    			    
			    return emptyStore;
        },
        
        _getSecondaryNodeDetails: function(secondaryUUID) {
            var responseInfo = null;
            if(secondaryUUID != "") {
                requestURL = this.REST_PATH + "details/server/" + secondaryUUID;
                
                console.log("In ServerConfiguration; sending services request to URL: " + requestURL);
                     
                var xhrGet = dojo.xhrGet({
                    url: requestURL,
                    handleAs: "json",
                    sync: true,
                    preventCache: true,
                    load: dojo.hitch(this, function(response, ioArgs) {
                        console.log("In ServerConfiguration; parsing response from URL: " + requestURL);
                        responseInfo = response.items;
                    }),
                    handle: dojo.hitch(this, function(data, ioArgs) {
                    }),
                    error: dojo.hitch(this, function(error, ioArgs) {
                        // No Server Settings                             
                        // show the error message in the table
                        this._detailsTable.body.emptyInfo = this.resources.ERROR_SETTINGS_DETAILS.replace("%1", this.PRODUCT_NAME_LONG).replace("%2", this.PRODUCT_NAME_SHORT);
                                                     
                        dojo.publish("RefreshPageComplete");
                    })
                });
            }
            return responseInfo;
        },
        
        // This function is a helper function used by _connectDataModels() to
        //   build the details layout; the reason it is not part of
        //   _connectDataModels() is that it in addition to being called
        //   initially whe the page loads, it must be also called after
        //   asynchronous data requests succeed
        _generateDetailsLayout: function(store) {
            var detailsLayout = new Array();

            if(this.mode == "serverTemplateDetails") {
                detailsLayout = [ {
                    field: "name",
                    name: this.locResources.TEMPLATE_SETTINGS+" "+this.templateName,
                    sortable: false
                } ];
            }
            else if(this.mode == "chassisTemplateDetails") {
                detailsLayout = [ {
                    field: "name",
                    name: this.resources.CHASSIS_TEMPLATE_SETTINGS+" "+this.templateName,
                    sortable: false
                } ];
            }
            else {

                if(store != null){

                    if(store != "") {  //Check to see if server template has been applied to server, Check
                        //for compliance label
                        var complianceCheck = store.items[0].children ? store.items[0].children[0].compliance : store.items[0].compliance;
                        if(complianceCheck) {
                            if(this.mode == "chassisDetails" || this.mode == "chassisProfileDetails") {
                                detailsLayout = [ {
                                    field: "name",
                                    name: this.locResources.CHASSIS_PROFILE_SETTINGS,
                                    width: "45%"
                                }, {
                                    field: "compliance",
                                    name: "&nbsp;",
                                    width: "10%",
                                    decorator: dojo.hitch(this, this._decorateCompliance),
                                    formatter: dojo.hitch(this, this._formatCompliance)
                                }, {
                                    field: "compare",
                                    name: this.locResources.CHASSIS_SETTINGS
                                } ];
                            }
                            else {  //serverDetails or serverProfileDetails
                                detailsLayout = [ {
                                    field: "name",
                                    name: this.locResources.PROFILE_SETTINGS,
                                    width: "45%"
                                }, {
                                    field: "compliance",
                                    name: "&nbsp;",
                                    width: "10%",
                                    decorator: dojo.hitch(this, this._decorateCompliance),
                                    formatter: dojo.hitch(this, this._formatCompliance)
                                }, {
                                    field: "compare",
                                    name: this.locResources.SERVER_SETTINGS
                                } ];
                            }
                        } else {  // If no compliance label then only show server details*/
                            if(this.mode == "chassisDetails") {
                                detailsLayout = [  {
                                    field: "compare",
                                    name: this.locResources.CHASSIS_SETTINGS
                                } ];
                            }else if(this.mode == "chassisProfileDetails") {
                                detailsLayout = [  {
                                    field: "name",
                                    name: this.locResources.CHASSIS_PROFILE_SETTINGS
                                } ];
                            }else if(this.mode == "serverProfileDetails") {
                                detailsLayout = [  {
                                    field: "name",
                                    name: this.locResources.PROFILE_SETTINGS
                                } ];
                            }else{  //Server Details (show sever settings)
                                detailsLayout = [  {
                                    field: "compare",
                                    name: this.locResources.SERVER_SETTINGS
                                } ];
                            }
                        }
                    }
                }
            }

            return detailsLayout;
        },
        
        _decorateCompliance: function(item) {
            var compliance = item;
            if (compliance == this.locResources.COMPLIANCE_MATCH) {
                return "<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spSuccessIcon\" title=\"" + this.locResources.COMPLIANCE_MATCH + "\"></img>";
            }
            else if (compliance == this.locResources.COMPLIANCE_MISMATCH) {
                return "<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spWarningIcon\" title=\"" + this.locResources.COMPLIANCE_MISMATCH + "\"></img>";
            }
            else if (compliance == this.locResources.COMPLIANCE_NOT_APPLICABLE) {              
                return "<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spInformationIcon\" title=\"" + this.locResources.COMPLIANCE_NOT_APPLICABLE + "\"></img>";
            }
            else if (compliance == this.locResources.COMPLIANCE_UNKNOWN) {
                return "<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spUnknownIcon\" title=\"" + this.locResources.COMPLIANCE_UNKNOWN + "\"></img>";
            }
            return "";
        },

        // Format compliance icon in table
        _formatCompliance: function(item) {
            var compliance = item.compliance;
            if (compliance == "Valid") {
                return this.locResources.COMPLIANCE_MATCH;
            }
            else if (compliance == "Invalid") {
                return this.locResources.COMPLIANCE_MISMATCH;
            }
            else if (compliance == "Info") {
                return this.locResources.COMPLIANCE_NOT_APPLICABLE;
            }
            else if (compliance == "None") {
                return this.locResources.COMPLIANCE_UNKNOWN;
            }
            return "";
        },
        
        // Create table with custom toolbar buttons when page is loaded
        _addToolbarButtons: function(grid) {

            var toolbar = grid && grid.toolBar && grid.toolBar.widget;
            
            if (toolbar) {
                // Add custom Collapse All button...
                var collapsebutton = new dijit.form.Button({
                    showLabel: false,
                    iconClass: "spCollapseAllLgIcon",
                    disabled: false,
                    title: this.resources.COLLAPSE_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._collapseAll();
                    }),
                    onKeypress: dojo.hitch(this, function() {
                        this._collapseAll();
                    })
                });
                
                toolbar.addChild(collapsebutton);
    
                // Add custom Expand All button...
                var expandbutton = new dijit.form.Button({
                    showLabel: false,
                    iconClass: "spExpandAllLgIcon",
                    disabled: false,
                    title: this.resources.EXPAND_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {
                        window.setTimeout(dojo.hitch(this, function(){    // !important for IE8
                            this._expandAll();
                        }), 1000);
                    }),
                    onKeypress: dojo.hitch(this, function() {
                        window.setTimeout(dojo.hitch(this, function(){    // !important for IE8
                            this._expandAll();
                        }), 1000);
                    })
                });
                
                toolbar.addChild(expandbutton);
                
            }

        },

        _refreshTree: function() {
            // tags:
            // protected
            // Refresh tree data

            var detailsStore = this._retrieveStore();
            this._detailsTable.setStore(detailsStore);
            this._detailsTable.model.clearCache();
            this._detailsTable.body.refresh().then(this._detailsTable.reexpandRows);
            
            //Set emptyInfo to loading icon and message for server details
            if(this.mode == "serverDetails" && this.CP_RUN_ENVIRONMENT == "augusta"){
                var emptyHtml = "<div class='dijitContentPaneLoading '><div class='dijitInline dijitIconLoading'></div>"+this.locResources.LOADING_MESSAGE+"</div>";
                this._detailsTable.body.emptyInfo = emptyHtml;
            }else if(this.mode == "serverDetails" && this.CP_RUN_ENVIRONMENT == "fsm"){
                var emptyHtml = "<div class='dijitInline dijitIconLoading'></div>"+this.locResources.LOADING_MESSAGE;
                this._detailsTable.body.emptyInfo = emptyHtml;
            }

            //Set emptyInfo to loading icon and message for chassis details
            if(this.mode == "chassisDetails" && this.CP_RUN_ENVIRONMENT == "augusta"){
                var emptyHtml = "<div class='dijitContentPaneLoading '><div class='dijitInline dijitIconLoading'></div>"+this.locResources.LOADING_MESSAGE+"</div>";
                this._detailsTable.body.emptyInfo = emptyHtml;
            }else if(this.mode == "chassisDetails" && this.CP_RUN_ENVIRONMENT == "fsm"){
                var emptyHtml = "<div class='dijitInline dijitIconLoading'></div>"+this.locResources.LOADING_MESSAGE;
                this._detailsTable.body.emptyInfo = emptyHtml;
            }

        },
        
        _expandAll: function() {
            // tags:
            // protected
            // summary:
            // Expands all rows in the details table
            
            for (var idx = 0; idx < this._detailsTable.store._arrayOfTopLevelItems.length; ++idx) {
                this._detailsTable.tree.expandRecursive(this._detailsTable.store._arrayOfTopLevelItems[idx].id[0][0]);
            };
        },

        _collapseAll: function() {
            // tags:
            // protected
            // summary:
            // Collapses all rows in the details table
            
            for (var idx = 0; idx < this._detailsTable.store._arrayOfTopLevelItems.length; ++idx) {
                this._detailsTable.tree.collapseRecursive(this._detailsTable.store._arrayOfTopLevelItems[idx].id[0][0]);
            };
        },


    });
});
