define(["dojo", 
        "dijit", 
        "support/ConfigPatternsAll", 
        "support/panels/Panel",
        "dojo/i18n!./nls/ServerDetails", 
        "dojo/text!./templates/ServerDetails.html",
        "support/base/NextGenUIMixin", 
        "dijit/layout/TabContainer",
        "support/panels/ServerConfiguration", 
        "dijit/Toolbar",
        "dijit/ToolbarSeparator",
        "dijit/Menu",
        "dojo/dom",
        "support/dialogs/FailoverMonitorTemplate"
        ], function(dojo, dijit, layer, panel, mri, template) {
    
    return dojo.declare("support.panels.ServerDetails", [ support.panels.Panel, support.base.NextGenUIMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in displaying information
        // associated with a server.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "ServerDetails",
        
        // the properties used to construct the title of this page in the
        //   NextGen UI
        cpProductName: null,
        cpPageContext: null,
        cpPageName: null,
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/ServerDetails.css",

        configurationTabdata: "",
        ioAddressesTabdata: "",
        ioAddressesTabdata2: "",
        eventsTabdata: "",
        dependenciesTabdata: "",
        
        detailsMode: "",
        templateIDNum: "",
        templateName: "",
        serverUUID: "",

        servertemplateID:"",
        serverprofileID:"",
        serverName:"",

        serverprofileStatustxt:"",
        servertemplateStatustxt:"",
        profileStatustxt:"",
        failoverStatustxt:"",
        
        profileConfigStatus:"",
        
        poolId:"", 
        policyId:"",
        serverFormFactor:"",
        serversArray:"",
        
        addressMode:"",     //Only show I/O tabs if addressing mode is virtual 

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
            this.cpProductName = this.resources.PRODUCT_NAME;
            this.cpPageContext = this.resources.SERVERS;
            this.cpPageName = this.resources.DETAILS;
        },

        // postCreate() is called after buildRendering(). This is useful to override when you need
        // to access and/or manipulate DOM nodes included with your widget. DOM nodes and widgets
        // with the dojoAttachPoint attribute specified can now be directly accessed as fields on
        // "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            
            if(this.CP_RUN_ENVIRONMENT == "augusta"){
                
                //Add page base in order to use the setTitle() function
                require([ "augusta/augustaUICore/core/augustaPageBase" ], dojo.hitch(this, function(PageBase) {                
                    var CP_AUGUSTA_PAGE_BASE = new augusta.augustaUICore.core.augustaPageBase();
                    dojo.safeMixin(this, CP_AUGUSTA_PAGE_BASE);            
                }));

                this.bwIsCacheable = false;
            }    

            if(this.CP_RUN_ENVIRONMENT == "fsm"){
                //Set title in FSM
                this.changeNextGenUITitle();
            }
            
            if(this.CP_RUN_ENVIRONMENT == "augusta"){
                //Set Title in xHMC, only can be used if augustaPageBase mixed in
                this.setTitle(this.cpPageContext + " - " + this.cpPageName);
            }
            
            // obtain the server UUID; must obtain this from the URL string,
            //   and also this must be done before _setTableData() can be
            //   called
            var epObj = this.getEndPointFromSourceToken();
            if(epObj != null) {
                this.serverUUID = epObj.uuid;
            }
            
            this.moreMenu = new dijit.Menu({          // Add to More/Actions Menu
                style: "display: none;"
            });
            
            this.createButton = new dijit.form.Button({     //Add Create Button to toolbar
                showLabel: false,
                iconClass: "spCreateLgIcon",
                disabled: false,
                title: this.resources.CREATE_SERVER_PATTERN_TITLE,
                onClick: dojo.hitch(this, this.createServerTemplate),
                onKeypress: dojo.hitch(this, this.createServerTemplate)
            });
            
            this.editButton = new dijit.form.Button({     //Add Edit Button to toolbar
                showLabel: false,
                iconClass: "spEditLgIcon",
                disabled: false,
                title: this.resources.EDIT_SERVER_PATTERN_TITLE,
                onClick: dojo.hitch(this, this.editTemplate),
                onKeypress: dojo.hitch(this, this.editTemplate)
            });
            
            this.deployButton = new dijit.form.Button({     //Add Deploy Button to toolbar
                showLabel: false,
                iconClass: "spDeployLgIcon",
                disabled: false,
                title: this.resources.DEPLOY_SERVER_PATTERN_TITLE,
                onClick: dojo.hitch(this, this.deployTemplate),
                onKeypress: dojo.hitch(this, this.deployTemplate)
            });
            
            this.restartButton = new dijit.form.Button({     //Add Restart Button to toolbar
                showLabel: false,
                iconClass: "spRestartLgIcon",
                disabled: false,
                title: this.locResources.RESTARTSERVER,
                onClick: dojo.hitch(this, this.restartServer),
                onKeypress: dojo.hitch(this, this.restartServer)
            });
            
            this.detailsToolBar.addChild(this.createButton);
            this.detailsToolBar.addChild(this.editButton);
            this.detailsToolBar.addChild(this.deployButton);
            //this.detailsToolBar.addChild(this.restartButton);
            
            // Add to More/Actions Menu
            this.detailsSeparator = new dijit.ToolbarSeparator();
            this.detailsToolBar.addChild(this.detailsSeparator);
            
            this.copyItem = new dijit.MenuItem({
                label: this.resources.COPY_SERVER_PATTERN,
                title: this.resources.COPY_SERVER_PATTERN_TITLE,
                onClick: dojo.hitch(this, this.copyTemplate),
                onKeypress: dojo.hitch(this, this.copyTemplate)
            });
            
            this.assignItem = new dijit.MenuItem({
                label: this.resources.DEPLOY_SERVER_PROFILE,
                title: this.resources.DEPLOY_SERVER_PROFILE_TITLE,
                onClick: dojo.hitch(this, this.assignProfile),
                onKeypress: dojo.hitch(this, this.assignProfile)
            });
            
            this.unassignItem = new dijit.MenuItem({
                label: this.resources.UNASSIGN_SERVER_PROFILE,
                title: this.resources.UNASSIGN_SERVER_PROFILE_TITLE,
                onClick: dojo.hitch(this, this.unassignProfile),
                onKeypress: dojo.hitch(this, this.unassignProfile)
            });
            
            this.startFailoverMenuItem = new dijit.MenuItem({
                label: this.resources.START_FAILOVER_MONITORING,
                title: this.resources.START_FAILOVER_MONITORING_TITLE,
                onClick: dojo.hitch(this, this.startfailover),
                onKeypress: dojo.hitch(this, this.startfailover)
            });
            
            this.stopFailoverMenuItem = new dijit.MenuItem({
                label: this.resources.STOP_FAILOVER_MONITORING,
                title: this.resources.STOP_FAILOVER_MONITORING_TITLE,
                onClick: dojo.hitch(this, this.stopfailover),
                onKeypress: dojo.hitch(this, this.stopfailover)
            });
            
            this.restartFailoverMenuItem = new dijit.MenuItem({
                label: this.resources.RESTART_FAILOVER_MONITORING,
                title: this.resources.RESTART_FAILOVER_MONITORING_TITLE,
                onClick: dojo.hitch(this, this.restartfailover),
                onKeypress: dojo.hitch(this, this.restartfailover)
            });
            
            this.removeFailoverMenuItem = new dijit.MenuItem({
                label: this.resources.REMOVE_FAILOVER_MONITORING,
                title: this.resources.REMOVE_FAILOVER_MONITORING_TITLE,
                onClick: dojo.hitch(this, this.removefailover),
                onKeypress: dojo.hitch(this, this.removefailover)
            });
            
            this.editFailoverMenuItem = new dijit.MenuItem({
                label: this.resources.EDIT_FAILOVER_MONITORING,
                title: this.resources.EDIT_FAILOVER_MONITORING_TITLE,
                onClick: dojo.hitch(this, this.editfailover),
                onKeypress: dojo.hitch(this, this.editfailover)
            });
            
            
                      /*  this.failbackItem = new dijit.MenuItem({
                            label: this.locResources.FAILBACK_TO,
                            title: this.locResources.FAILBACK_TO,
                            onClick: dojo.hitch(this, this.failbackTo),
                            onKeypress: dojo.hitch(this, this.failbackTo)
                        });*/
            
            this.failoverMenuItem = new dijit.MenuItem({
                label: this.resources.FAILOVER_TO,
                title: this.resources.FAILOVER_TO_TITLE,
                onClick: dojo.hitch(this, this.failover),
                onKeypress: dojo.hitch(this, this.failover)
            });
            
            this.resetItem = new dijit.MenuItem({
                label: this.resources.RESET_LOC_STORAGE,
                title: this.resources.RESET_LOC_STORAGE_TITLE,
                onClick: dojo.hitch(this, this.resetStorage),
                onKeypress: dojo.hitch(this, this.resetStorage)
            });
            
            this.moreMenu.addChild(this.startFailoverMenuItem);
            this.moreMenu.addChild(this.stopFailoverMenuItem);
            this.moreMenu.addChild(this.restartFailoverMenuItem);
            this.moreMenu.addChild(this.removeFailoverMenuItem);
            this.moreMenu.addChild(this.editFailoverMenuItem);
            //                        this.moreMenu.addChild(this.failbackItem);
            this.moreMenu.addChild(this.failoverMenuItem);
            this.moreMenu.addChild(this.resetItem);
            
            this.moreButton = new dijit.form.DropDownButton({
                showLabel: true,
                label: this.resources.MORE,
                dropDown: this.moreMenu
            });
            
            this.detailsToolBar.addChild(this.moreButton);
            this.detailsSeparator = new dijit.ToolbarSeparator();
            this.detailsToolBar.addChild(this.detailsSeparator);
            
            this.refreshButton = new dijit.form.Button({     //Add Deploy Button to toolbar
                showLabel: false,
                iconClass: "spRefreshInactiveIcon",
                disabled: true,
                title: this.resources.REFRESH_PAGE,
                onClick: dojo.hitch(this, this.refreshPage),
                onKeypress: dojo.hitch(this, this.refreshPage)
            });
            
            this.detailsToolBar.addChild(this.refreshButton);
                        
                //        this.DetailsTemplateName.innerHTML = "Server Name";

/*                if(dojo.byId("toolbarDiv")){
                    var toolbarContainer = dojo.byId("toolbarDiv");
                    this.detailsToolBar.placeAt(toolbarContainer);  
                }*/
                                            
                    // Events Tab
                 /*   this.eventsTabdata = new support.panels.ServerProfileTable({
                        //mode: "detailsTab", 
                        tableWidth: "99%",
                        title:this.locResources.EVENTS_TAB_TITLE
                    }).placeAt(this._eventsTab);  */
            
            // setup variables; must be done before
            //   showDialog() can be called
            this._setTableData();
                    
            // Configurations Tab
            this.configurationTabdata = new support.panels.ServerConfiguration({
                endpointUUID: this.serverUUID,
                mode: "serverDetails",
                title:this.locResources.CONFIG_TAB_TITLE
            }).placeAt(this._configurationTab);
                 
            
            //Check if IO address mode is set to virtual
            if(this.servertemplateID == null || this.servertemplateID == ""){}else{
                this._getIOAddressMode();
            }

                                                
            if(this.profileConfigStatus == "TEMPLATE_DEPLOYED" || this.profileConfigStatus == "PROFILE_ACTIVATED" ||
                    this.profileConfigStatus == "MATCHES_PROFILE" || this.profileConfigStatus == "OUT_OF_COMPLIANCE" ||
                    this.profileConfigStatus == "ERROR_ACTIVATING_PROFILE"){
                   
                if(this.addressMode == "virtual" || this.addressMode == "Virtual"){

                    
                    // I/O Addresses Tab
                    /*require([ "support/panels/IOAddresses2" ], dojo.hitch(this, function() {

                    this.ioAddressesTabdata = new support.panels.IOAddresses({
                        serverUUID: this.serverUUID,
                        title:this.locResources.IOADDR_TAB_TITLE
                    }).placeAt(this._ioAddressesTab);

                        return false;
                        
                    }));*/

                    // I/O Addresses Tab 2
                    require([ "support/panels/IOAddresses2" ], dojo.hitch(this, function() {

                        this.ioAddressesTabdata2 = new support.panels.IOAddresses2({
                            serverUUID: this.serverUUID,
                            title:this.locResources.IOADDR_TAB_TITLE2
                        }).placeAt(this._ioAddressesTab2);
                        
                        return false;
                        
                    }));


                    
                }
            }
            
            // (possibly) render a dialog on top of this panel
            this.showDialog();
          
          //Refresh status bar data after Server Pattern Wizard changed and saved
            this.subscribe("ServerTemplateComplete", function(data) {
                this.refreshPage();
            });

          //Refresh status bar data after Server or Chassis Pattern Redeploy
            this.subscribe("RefreshAfterReDeploy", function(data) {
                this.refreshPage();
            });
            
          //Refresh status bar data after Server Pattern Deploy
            this.subscribe("DeployServerTemplateComplete", function(data) {
                this.refreshPage();
            });
            
          //Refresh status bar data after FailoverMonitoring is started
            this.subscribe("AvailableFailoverServersModified", function(data) {
                this.refreshPage();
            });

          //Refresh status bar data after FailoverMonitoring is started
            this.subscribe("UnassignServerProfileComplete", function(data) {
                this.refreshPage();
            });
            
            //Refresh Complete
            this.subscribe("RefreshPageComplete", function() {
                this.refreshButton.set("iconClass", "spRefreshIcon");
                this.refreshButton.set("disabled", false);
            });

        },
        
        showDialog: function() {
            var currentURL = window.location.href;
            
            // if this widget was launched by one of the Configuration
            //   Patterns Java ITasks, it will contain a special URL
            //   parameter; if this parameter is present, we know we must
            //   launch a dialog
            if(currentURL.indexOf("cpParm") != -1) {
                var serverUUID = this.getEndPointPropertiesFromSourceToken().uuid;
                
                if(currentURL.indexOf("estTask") != -1) {
                    // show the Server Template Wizard dialog, but nest these
                    //   calls in a try/catch; if there is an error
                    //   instantiating the dialog, we still want this panel to
                    //   be displayed
                    var serverStr = " ";
                    try {
                        var stwdialog = null;
                        if(this.servertemplateID != null && this.servertemplateID != "") {

                            require([ "support/dialogs/ServerTemplateWizard" ], dojo.hitch(this, function() {
                                stwdialog = new support.dialogs.ServerTemplateWizard({
                                    templateIDNum: this.servertemplateID,
                                    templateInUse: true,
                                    editMode: true
                                });
                            }));
                        }
                        else {
                            
                            require([ "support/dialogs/ServerTemplateWizard" ], dojo.hitch(this, function() {                                                            
                                stwdialog = new support.dialogs.ServerTemplateWizard({
                                    templateIDNum: null,
                                    editMode: true
                                });
                            }));
                        }
                        serverStr = " (for server with UUID: " + serverUUID + ") ";
                        stwdialog.show();
                        // no need to dismiss or delete this dialog; the Server
                        //   Template Wizard dialog does this when the user clicks
                        //   OK or Cancel
                        console.log("Showing Server Template Wizard dialog" + serverStr + "from Server Details page");
                    }
                    catch(err) {
                        console.error("Failed to show Server Template Wizard dialog" + serverStr + "from Server Details page; error: " + err.message);
                    }
                }
                else if(currentURL.indexOf("editFMTask") != -1) {
                    // show the Failover Monitor Template dialog, but nest
                    //   these calls in a try/catch; if there is an error
                    //   instantiating the dialog, we still want this panel to
                    //   be displayed
                    var serverStr = " ";
                    try {
                        // to show the Edit Failover Monitoring dialog, must
                        //   pass a template ID parameter to the constructor
                        var editFMdialog = null;
                        if(serverUUID != null) {
                            editFMdialog = new support.dialogs.FailoverMonitorTemplate({
                                // 99297 update this with the correct template number?
                                // pass any template number to ensure the
                                //   dialog renders itself in "Edit" mode
                                templateIDNum:5555,
                                serverUUID:serverUUID
                            });
                            serverStr = " (for server with UUID: " + this.serverUUID + ") ";
                        }
                        else {
                            editFMdialog = new support.dialogs.FailoverMonitorTemplate();
                        }
                        editFMdialog.show();
                        // no need to dismiss or delete this dialog; the
                        //   Failover Monitor Template dialog does this when
                        //   the user clicks OK or Cancel
                        console.log("Showing (Edit) Failover Monitor Template dialog" + serverStr + "from Server Details page");
                    }
                    catch(err) {
                        console.error("Failed to show (Edit) Failover Monitor Template dialog" + serverStr + "from Server Details page; error: " + err.message);
                    }
                }
            }
        },              
        _getIOAddressMode: function() {
            
            dojo.xhrGet({
                url: this.REST_PATH + "template/"+this.servertemplateID,                        
                handleAs: "json",
                sync: true,
                preventCache: true,
                load: dojo.hitch(this,function(response) {
                   this.addressMode = response.items[0].adapterSettings.ioAddressingMode;
                })
            });

         },
        _setTableData: function() {

           dojo.xhrGet({
               url: this.REST_PATH + "server/"+this.serverUUID+"/status2",
               handleAs: "json",
               sync: true,
               preventCache: true,
               load: dojo.hitch(this, function(response, ioArgs) {
                  var store = response;
                   
                   if(store.items){

                      if(store.items != ""){
                               
                           this.serverUUID =  store.items[0].uuid;
                           
                           if(store.items[0].serverProfileId == null || store.items[0].serverProfileId == ""){  //Set Server Profile Status

                               this.serverprofileStatus.innerHTML = this.resources.UNASSIGNED;
//                               this.moreMenu.addChild(this.assignItem,0);

                           }else{

                               this.moreMenu.addChild(this.assignItem,0);
                               
                               this.serverprofileStatus.innerHTML = store.items[0].serverProfileName;

                               this.moreMenu.addChild(this.unassignItem,0);
                           }

                           this.serverprofileStatustxt = store.items[0].serverProfileName;

                           this.serverprofileID = store.items[0].serverProfileId;

                  //         store.items[0].serverTemplate = "17";

                           if(store.items[0].serverTemplateId == null || store.items[0].serverTemplateId == ""){

                               this.servertemplateStatus.innerHTML = this.resources.UNASSIGNED;                            

                               this.createButton.set("iconClass", "spCreateLgIcon");
                               this.createButton.set("disabled", false);

                               this.deployButton.set("iconClass", "spDeployLgIcon");
                               this.deployButton.set("disabled", false);

                               this.editButton.set("iconClass", "spEditInactiveLgIcon");
                               this.editButton.set("disabled", true);

                           }else{

                               this.servertemplateStatus.innerHTML = store.items[0].serverTemplateName;

                               this.createButton.set("iconClass", "spCreateInactiveLgIcon");
                               this.createButton.set("disabled", true);

                               this.deployButton.set("iconClass", "spDeployInactiveLgIcon");
                               this.deployButton.set("disabled", true);

                               this.editButton.set("iconClass", "spEditLgIcon");
                               this.editButton.set("disabled", false);
                               
                           }

                           this.servertemplateStatustxt = store.items[0].serverTemplateName;

                           this.servertemplateID = store.items[0].serverTemplateId;


                           this.serverTitle.innerHTML = store.items[0].serverName;
                           this.serverName = store.items[0].serverName;

                           this.profileConfigStatus = store.items[0].configStatus;
                           
//                                 store.items[0].configStatus = "MATCHES_PROFILE";

                          /* if(store.items[0].serverTemplate == null){

                               this.moreMenu.addChild(this.assignItem,0);

                               this.profileStatus.innerHTML = "No template assigned<br/>Action: Deploy Server Template";

                           }else*/ if(store.items[0].configStatus == "NO_PROFILE"){

//                               this.moreMenu.addChild(this.assignItem,0);

                               this.profileStatus.innerHTML = this.resources.NOTEMPASSIGNED+"<br/>"+this.resources.ACTION_DEPLOY;

                               this.OfflineIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "STANDBY"){

                               this.profileStatus.innerHTML = this.resources.STANDBYSTATUS;

                               this.NormalIcon.style.display = "block";

                             // this.moreMenu.addChild(this.failbackItem);

                           }else if(store.items[0].configStatus == "BUILDING_PROFILE"){

                               this.profileStatus.innerHTML = this.resources.BUILDINGSTATUS;

                               this.ProcessingIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "SAVING_PROFILE"){

                               this.profileStatus.innerHTML = this.resources.SAVINGSTATUS;

                               this.ProcessingIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "PENDING_PROFILE"){

                               this.profileStatus.innerHTML = this.resources.PENDING_ACTIVATION+"<br/>"+this.resources.ACTION_RESTART;

                             //  this.moreMenu.addChild(this.unassignItem,0);

                               this.WarningIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "ACTIVATING_PROFILE"){

                               this.profileStatus.innerHTML = this.resources.ACTIVATINGSTATUS;

                               //this.moreMenu.addChild(this.unassignItem,0);

                               this.ProcessingIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "ERROR_CREATING_PROFILE"){

                               this.profileStatus.innerHTML = this.resources.PROFILECREATEFAILED;

                             //  this.moreMenu.addChild(this.assignItem,0);

                               this.WarningIcon.style.display = "block";


                           }else if(store.items[0].configStatus == "ERROR_ACTIVATING_PROFILE"){

                               this.profileStatus.innerHTML = this.resources.PROFILEACTIVATIONFAILED;

                               // Default to standard fail, but override in the case that we know more
                               if(store.items[0].activationStatus == "LDAPERROR"){
                                   this.profileStatus.innerHTML = this.resources.PROFILEACTIVATIONLOAD;
                               } 
                               else if(store.items[0].activationStatus == "CONFIGERROR"){
                                   this.profileStatus.innerHTML = this.resources.PROFILEACTIVATIONERRORS;
                               }

                              // this.moreMenu.addChild(this.assignItem,0);

                             //  this.moreMenu.addChild(this.unassignItem,0);

                               this.WarningIcon.style.display = "block";


                           }else if(store.items[0].configStatus == "RESTARTING"){

                               this.profileStatus.innerHTML = this.resources.RESTARTINGSERVER;

                               this.ProcessingIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "TEMPLATE_DEPLOYED"){

                               this.profileStatus.innerHTML = this.resources.PROFILEASSIGNED;

                               this.moreMenu.addChild(this.copyItem);
                            //   this.moreMenu.addChild(this.unassignItem,0);

                               this.SuccessIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "PROFILE_DEPLOYED"){

                               this.profileStatus.innerHTML = this.resources.NOT_ACTIVATED+"<br/>"+this.resources.ACTION_REDEPLOY_PROFILE;

                               this.moreMenu.addChild(this.copyItem);
                           //    this.moreMenu.addChild(this.unassignItem,0);
                               this.moreMenu.addChild(this.assignItem,0);
                               
                               this.WarningIcon.style.display = "block";
                               
                           }else if(store.items[0].configStatus == "PROFILE_ACTIVATED"){

                               this.profileStatus.innerHTML = this.resources.PROFILEASSIGNED;

                               this.moreMenu.addChild(this.copyItem);
                           //    this.moreMenu.addChild(this.unassignItem,0);
                               this.moreMenu.addChild(this.assignItem,0);
                               
                               this.SuccessIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "MATCHES_PROFILE"){

                               this.profileStatus.innerHTML = this.resources.SERVERMATCHES;

                               this.moreMenu.addChild(this.copyItem);
                            //   this.moreMenu.addChild(this.unassignItem,0);
                               this.moreMenu.addChild(this.assignItem,0);
                               
                               this.SuccessIcon.style.display = "block";

                           }else if(store.items[0].configStatus == "OUT_OF_COMPLIANCE"){

                               this.profileStatus.innerHTML = this.resources.OUTOFCOMPLIANCE+"<br/>"+this.resources.ACTION_RESTART;

                               this.moreMenu.addChild(this.copyItem);
                         //      this.moreMenu.addChild(this.unassignItem,0);

                               this.WarningIcon.style.display = "block";

                           }

                           this.profileStatustxt = store.items[0].configStatus;

                           var state = store.items[0].failoverMonitorStatus;
                           
                           this._setFailoverMenuItems(state);
                           
                           this.failoverStatustxt = state;


                           if(store.items[0].serverProfileId == null || store.items[0].serverProfileId == ""){

                               this.moreMenu.removeChild(this.startFailoverMenuItem);
                               this.moreMenu.removeChild(this.failoverMenuItem);
                               this.moreMenu.removeChild(this.editFailoverMenuItem);
                               
                               // Remove failover monitoring when profile is unassigned
                               if (store.items[0].failoverMonitorStatus == "ACTIVE" ||
                                       store.items[0].failoverMonitorStatus == "STOPPED") {
                                   var jsonString = dojo.toJson({"failover":"remove"});
                                   
                                   var xhrArgs = {
                                           url:  this.REST_PATH +"server/"+this.serverUUID,
                                           handleAs: "json",
                                           sync: true,
                                           putData: jsonString,
                                           headers: {
                                               "Content-Type": "application/json; charset=utf-8"
                                           },
                                           load : function(data) {
                                               //alert("Template Create Successfully: ID " + data);
                                               //dojo.publish("PowerTemplateComplete", data);
                                           },
                                           error : function(error) {
                                               //  alert("Put failed: " + error);
                                           }
                                   };
                                   dojo.xhrPut(xhrArgs);
                               }
                              
                           }

                                             
                       }
                    } 
               })
           });
  
        },

        restartServer: function(){

            var xhrArgs = {
                   url: this.REST_PATH +this.serverUUID+"/restart",
                   sync: true,
                   handleAs: "json",
                   headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
//                    postData: argJSONString,
                    load : function(data) {
                        //alert("Template Create Successfully: ID " + data);
                        //dojo.publish("PowerTemplateComplete", data);
                    }
                };
                dojo.xhrPost(xhrArgs);
            
        },
        resetStorage: function(){
            
           var confirmedReset = function() {

               var xhrArgs = {
                  url: this.REST_PATH + "resetstorage/" + this.serverUUID,
                  handleAs: "json",
                  sync: true,
                  headers: {
                      "Content-Type": "application/json; charset=utf-8"
                  },
                  load : function(data) {
                      //alert("Template Create Successfully: ID " + data);
                      //dojo.publish("PowerTemplateComplete", data);
                  },
                  error : function(error) {
                     //alert("Post failed: " + error);
                  }
              };
              dojo.xhrPost(xhrArgs);

           };
           
           if(this.CP_RUN_ENVIRONMENT == "fsm"){

               require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                   var confDialog = new support.temp.ConfirmationDialog({
                       buttonLabel: this.resources2.RESET_BTN_LABEL,
                       type: "Warning",
                       text: this.resources.CONFIRM_STORAGE,
                       info: this.resources.CONFIRM_STORAGE_ACTION + "<br><br>" + this.serverName,
                       validationMode: true,
                       validationLabel: this.resources.CONFIRM_STORAGE_Verify,
                       validationRegExp: "1",
                       validationInvalidMsg: this.resources2.RESET_STORAGE_ERR
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedReset, this);
           
               }));
               
           }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

               require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                   var confDialog = new idx.widget.ConfirmationDialog({
                       buttonLabel: this.resources2.RESET_BTN_LABEL,
                       type: "Warning",
                       text: this.resources.CONFIRM_STORAGE,
                       info: this.resources.CONFIRM_STORAGE_ACTION + "<br><br>" + this.serverName,
                       validationMode: true,
                       validationLabel: this.resources.CONFIRM_STORAGE_Verify,
                       validationRegExp: "1",
                       validationInvalidMsg: this.resources2.RESET_STORAGE_ERR
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedReset, this);
           
               }));

           }
            
        },
        assignProfile: function(){
            var confirmedDeploy = function() {

                dojo.xhrPost({
                    url: this.REST_PATH + "profile/assign/" + this.serverprofileID + "/location/",
                    handleAs: "json",
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    sync: true,
                    load: function(data) {
                        if(data && data.error && data.error.length > 0){
                            var validationpopup = new support.temp.ModalDialog({
                                type: "Error",
                                text: data.error
                            });
                            validationpopup.show();
                        }
                        else{
                            dojo.publish("AssignServerProfileComplete");
                        }
                        
                    },
                    error: dojo.hitch(this,function(data) {
                        var validationpopup = new support.temp.ModalDialog({
                            type: "Error",
                            text: this.resources.ERROR_ASSIGN_PROF
                        });
                        console.log(data);
                        validationpopup.show();
                    })
                    
                });

                this.refreshPage();

            };
            
            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.DEPLOY,
                        type: "Warning",
                        text: this.locResources.DEPLOY_CONF,
                        info: this.locResources.DEPLOY_CONF_LIST.replace("%1", this.serverName)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedDeploy, this);
            
                }));
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.DEPLOY,
                        type: "Warning",
                        text: this.locResources.DEPLOY_CONF,
                        info: this.locResources.DEPLOY_CONF_LIST.replace("%1", this.serverName)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedDeploy, this);
            
                }));

            }
        },
        createServerTemplate: function() {

            require([ "support/dialogs/ServerTemplateWizard" ], dojo.hitch(this, function() {
                var stwdialog = new support.dialogs.ServerTemplateWizard({
                    templateIDNum: null,
                    editMode: false
                });
                        
                stwdialog.show();
            
                this.refreshPage();
            
            }));

        },       
        // Deploy Template
        deployTemplate: function() {

            require([ "support/dialogs/DeployServerTemplate" ], dojo.hitch(this, function() {

                var dst = new support.dialogs.DeployServerTemplate({
                    templateId: this.servertemplateID
                });
                dst.show();

                this.refreshPage();
                
            }));
        },
        // Edit Template
        editTemplate: function() {

            require([ "support/dialogs/ServerTemplateWizard" ], dojo.hitch(this, function() {
                var stweditdialog = new support.dialogs.ServerTemplateWizard({
                    templateIDNum: this.servertemplateID,
                    templateInUse: true,
                    editMode: true
                });
                stweditdialog.show();

                this.refreshPage();
            }));    
        },
        // Copy Template
        copyTemplate: function() {

            require([ "support/dialogs/ServerTemplateWizard" ], dojo.hitch(this, function() {
                var stweditdialog = new support.dialogs.ServerTemplateWizard({
                    templateIDNum: this.servertemplateID.toString(),
                    copyMode: true
                });
                stweditdialog.show();
            
                this.refreshPage();
            }));

        },
        //Fail Back To Original Server
       // failbackTo: function(){
            
       // },
        //Failover Original Server
        failover: function(){
            var confirmedFailover = function() {

                var jsonString = dojo.toJson({"failover":"failover"});

                var xhrArgs = {
                        url:  this.REST_PATH +"server/"+this.serverUUID,
                        handleAs: "json",
                        putData: jsonString,
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        load : dojo.hitch(this, function(data) {
                            //alert("Template Create Successfully: ID " + data);
                            //dojo.publish("PowerTemplateComplete", data);
                            this.getModalDialog("Success", this.resources.FAILOVER_COMPLETE_MSG).show();
                        }),
                        error : dojo.hitch(this, function(error) {
                            this.getModalDialog("Error", this.resources.FAILOVER_ERROR_MSG).show();
                        })
                };
                
                dojo.xhrPut(xhrArgs);
                this.getModalDialog("Success", this.resources.FAILOVER_SUCCESS_MSG).show();
                this.refreshPage();

            };
            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {
            
                    var confDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.FAILOVER_MONITORING_FAILOVER_ACTION,
                        type: "Warning",
                        text: this.resources.FAILOVER_MONITORING_FAILOVER_CONFIRMATION_MESSAGE.replace("%1",this.serverName),
                        info: this.resources.FAILOVER_MONITORING_FAILOVER_DETAILED_MESSAGE.replace("%1", this.serverName).replace("%2", this.serverName).replace("%3", this.serverName).replace("%4", this.serverName).replace("%5", this.serverName).replace("%6", this.serverName).replace("%7", this.serverName).replace("%8", this.PRODUCT_NAME_LONG)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedFailover, this);

                }));
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {
                    
                    var confDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.FAILOVER_MONITORING_FAILOVER_ACTION,
                        type: "Warning",
                        text: this.resources.FAILOVER_MONITORING_FAILOVER_CONFIRMATION_MESSAGE.replace("%1",this.serverName),
                        info: this.resources.FAILOVER_MONITORING_FAILOVER_DETAILED_MESSAGE.replace("%1", this.serverName).replace("%2", this.serverName).replace("%3", this.serverName).replace("%4", this.serverName).replace("%5", this.serverName).replace("%6", this.serverName).replace("%7", this.serverName).replace("%8", this.PRODUCT_NAME_LONG)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedFailover, this);

                }));

            }

        },
        //Edit Failover
        editfailover: function(){
            
            dojo.xhrGet({
                url: this.REST_PATH + "server/" + this.serverUUID + "?showFailover=true",                        
                handleAs: "json",
                sync: true,
                preventCache: true,
                load: dojo.hitch(this,function(response) {
                    this.policyId = response.failoverPolicyId;
                    this.poolId = response.standbyPoolId;
                    this.serverFormFactor = response.formFactor;
                })
            });
            
            var dialog = new support.dialogs.FailoverMonitorTemplate({
                // 99297 template ID num
                // pass any template number to ensure the dialog renders
                //   itself in "Edit" mode
                templateIDNum: "1",
                serverUUID: this.serverUUID,
                newPolicyId: this.policyId,
                newPoolId: this.poolId,
                newFormFactor: this.serverFormFactor
            });
            dialog.show();
            
            this.refreshPage();

        },
        //Start Failover
        startfailover: function(){

            var dialog = new support.dialogs.FailoverMonitorTemplate({
                serverUUID: this.serverUUID
            });
            dialog.show();
            
            this.refreshPage();

        },
        //Stop Failover
        stopfailover: function(){

            var confirmedStop = function() {

                var jsonString = dojo.toJson({"failover":"stop"});
                var xhrArgs = {
                        url:  this.REST_PATH +"server/"+this.serverUUID,
                        handleAs: "json",
                        sync: true,
                        putData: jsonString,
                        headers: {
                             "Content-Type": "application/json; charset=utf-8"
                        },
                        load : function(data) {
                            //alert("Template Create Successfully: ID " + data);
                            //dojo.publish("PowerTemplateComplete", data);
                        },
                        error : function(error) {
                            //    alert("Put failed: " + error);
                        }
                };
                dojo.xhrPut(xhrArgs);

                this.refreshPage();

            };
            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.FAILOVER_MONITORING_STOP_ACTION,
                        type: "Warning",
                        text: this.resources.FAILOVER_MONITORING_STOP_CONFIRMATION_MESSAGE,
                        info: this.resources.FAILOVER_MONITORING_STOP_DETAILED_MESSAGE.replace("%1",this.serverName)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedStop, this);
            
                }));
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.FAILOVER_MONITORING_STOP_ACTION,
                        type: "Warning",
                        text: this.resources.FAILOVER_MONITORING_STOP_CONFIRMATION_MESSAGE,
                        info: this.resources.FAILOVER_MONITORING_STOP_DETAILED_MESSAGE.replace("%1",this.serverName)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedStop, this);
            
                }));
                
            }

        },
        //Restart Failover
        restartfailover: function(){
            var confirmedRestart = function() {

                var jsonString = dojo.toJson({"failover":"restart"});
                var xhrArgs = {
                        url:  this.REST_PATH +"server/"+this.serverUUID,
                        handleAs: "json",
                        sync: true,
                        putData: jsonString,
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        load : function(data) {
                            //alert("Template Create Successfully: ID " + data);
                            //dojo.publish("PowerTemplateComplete", data);
                        },
                        error : function(error) {
                            //  alert("Put failed: " + error);
                        }
                };
                dojo.xhrPut(xhrArgs);

                this.refreshPage();

            };

            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.FAILOVER_MONITORING_RESTART_ACTION,
                        type: "Warning",
                        text: this.resources.FAILOVER_MONITORING_RESTART_CONFIRMATION_MESSAGE,
                        info: this.resources.FAILOVER_MONITORING_RESTART_DETAILED_MESSAGE.replace("%1",this.serverName)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedRestart, this);
            
                }));
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.FAILOVER_MONITORING_RESTART_ACTION,
                        type: "Warning",
                        text: this.resources.FAILOVER_MONITORING_RESTART_CONFIRMATION_MESSAGE,
                        info: this.resources.FAILOVER_MONITORING_RESTART_DETAILED_MESSAGE.replace("%1",this.serverName)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedRestart, this);
            
                }));

            }

        },
        //Remove Failover
        removefailover: function(){
            var confirmedRemove = function() {

                var jsonString = dojo.toJson({"failover":"remove"});
                var xhrArgs = {
                        url:  this.REST_PATH +"server/"+this.serverUUID,
                        handleAs: "json",
                        sync: true,
                        putData: jsonString,
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        load : function(data) {
                            //alert("Template Create Successfully: ID " + data);
                            //dojo.publish("PowerTemplateComplete", data);
                        },
                        error : function(error) {
                            //  alert("Put failed: " + error);
                        }
                };
                dojo.xhrPut(xhrArgs);

                this.refreshPage();

            };

            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.FAILOVER_MONITORING_REMOVE_ACTION,
                        type: "Warning",
                        text: this.resources.FAILOVER_MONITORING_REMOVE_CONFIRMATION_MESSAGE,
                        info: this.resources.FAILOVER_MONITORING_REMOVE_DETAILED_MESSAGE.replace("%1",this.serverName)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedRemove, this);
            
                }));
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.FAILOVER_MONITORING_REMOVE_ACTION,
                        type: "Warning",
                        text: this.resources.FAILOVER_MONITORING_REMOVE_CONFIRMATION_MESSAGE,
                        info: this.resources.FAILOVER_MONITORING_REMOVE_DETAILED_MESSAGE.replace("%1",this.serverName)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedRemove, this);
            
                }));
                
            }

        },
        // Unassign Profile Menu Option
        unassignProfile: function() {
            //99292 get serverId from REST call when available and unassign for every selected row/profile

         var confirmedUnassign = function() {
             dojo.xhrPost({
                 url: this.REST_PATH + "profile/unassign/" + this.serverprofileID,
                 handleAs: "json",
                 headers: {
                     "Content-Type": "application/json; charset=utf-8"
                 },
                 sync: true,
                 load: function(data) {
                     if(data && data.error && data.error.length > 0){
                         var validationpopup = new support.temp.ModalDialog({
                             type: "Error",
                             text: data.error
                         });
                         validationpopup.show();
                     }
                     else{
                         dojo.publish("UnassignServerProfileComplete");
                     }
                     
                 },
                 error: dojo.hitch(this,function(data) {
                     var validationpopup = new support.temp.ModalDialog({
                         type: "Error",
                         text: this.resources.ERROR_UNASSIGN_PROF
                     });
                     console.log(data);
                     validationpopup.show();
                 })
             });
             
             this.refreshPage();

            };

            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.UNASSIGN,
                        type: "Warning",
                        text: this.resources.CONFIRM_UNASSIGN,
                        info: this.resources.UNASSIGN_PROFILE_FROM_SERVER.replace("%1", this.serverprofileStatustxt) + "<br><br>" + this.resources.UNASSIGN_MSG
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedUnassign, this);
            
                }));
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.UNASSIGN,
                        type: "Warning",
                        text: this.resources.CONFIRM_UNASSIGN,
                        info: this.resources.UNASSIGN_PROFILE_FROM_SERVER.replace("%1", this.serverprofileStatustxt) + "<br><br>" + this.resources.UNASSIGN_MSG
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedUnassign, this);
            
                }));
                
            }
                                                            
        },
        //Refresh Page
        refreshPage: function(){
            
            this.refreshButton.set("iconClass", "spRefreshInactiveIcon");
            this.refreshButton.set("disabled", true);

            this.SuccessIcon.style.display = "none";
            this.OfflineIcon.style.display = "none";
            this.ProcessingIcon.style.display = "none";
            this.WarningIcon.style.display = "none";
            this.NormalIcon.style.display = "none";

            this._setTableData();  //Refresh Status Bar
            
            dojo.publish("ServerDetailRefreshButtonClicked"); //Refresh Server Details Table
        
        },
        
        /*
         *  setup failover menu items, icons and failover state text
         */
        _setFailoverMenuItems: function(state)
        {
            
            if ( this.moreMenu.hasChildren())
            {
                if ( state == "ACTIVE" )
                {
                    if (this.moreMenu.getIndexOfChild(this.startFailoverMenuItem) > 0)
                        this.moreMenu.removeChild(this.startFailoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.stopFailoverMenuItem) < 0)
                        this.moreMenu.addChild(this.stopFailoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.restartFailoverMenuItem) > 0)
                        this.moreMenu.removeChild(this.restartFailoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.removeFailoverMenuItem) < 0)
                        this.moreMenu.addChild(this.removeFailoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.failoverMenuItem) < 0 )
                        this.moreMenu.addChild(this.failoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.editFailoverMenuItem) < 0 )
                        this.moreMenu.addChild(this.editFailoverMenuItem);
                    
                    this.failoverStatus.innerHTML = this.resources.FAILOVERACTIVE;

                    this.failoverActiveIcon.style.display = "block";
                    this.failoverStopIcon.style.display = "none";
                    this.failoverStartingIcon.style.display = "none";
                                        
                }
                else if ( state == "NOTSTARTED" )
                {
                    if (this.moreMenu.getIndexOfChild(this.startFailoverMenuItem) < 0)
                        this.moreMenu.addChild(this.startFailoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.stopFailoverMenuItem) > 0)
                        this.moreMenu.removeChild(this.stopFailoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.restartFailoverMenuItem) > 0)
                        this.moreMenu.removeChild(this.restartFailoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.removeFailoverMenuItem) > 0)
                        this.moreMenu.removeChild(this.removeFailoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.failoverMenuItem) > 0 )
                        this.moreMenu.removeChild(this.failoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.editFailoverMenuItem) > 0 )
                        this.moreMenu.removeChild(this.editFailoverMenuItem);
                    
                    this.failoverActiveIcon.style.display = "none";
                    this.failoverStopIcon.style.display = "block";
                    this.failoverStartingIcon.style.display = "none";
                    
                    this.failoverStatus.innerHTML = this.resources.FAILOVERNOTSTARTED;
                }
                else if ( state == "STOPPED" || state == "FAILED")
                {
                    if (this.moreMenu.getIndexOfChild(this.startFailoverMenuItem) > 0)
                        this.moreMenu.removeChild(this.startFailoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.stopFailoverMenuItem) > 0)
                        this.moreMenu.removeChild(this.stopFailoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.restartFailoverMenuItem) < 0)
                        this.moreMenu.addChild(this.restartFailoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.removeFailoverMenuItem) < 0)
                        this.moreMenu.addChild(this.removeFailoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.failoverMenuItem) > 0 )
                        this.moreMenu.removeChild(this.failoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.editFailoverMenuItem) > 0 )
                        this.moreMenu.removeChild(this.editFailoverMenuItem);
                                      
                    this.failoverActiveIcon.style.display = "none";
                    this.failoverStopIcon.style.display = "block";
                    this.failoverStartingIcon.style.display = "none";
                    
                    if (state == "STOPPED")
                        this.failoverStatus.innerHTML = this.resources.FAILOVERSTOPPED;
                    else if (state == "FAILED")
                        this.failoverStatus.innerHTML = this.resources.FAILOVERFAILED;
                    
                
                }
                else if ( state == "STARTING")
                {
                    if (this.moreMenu.getIndexOfChild(this.startFailoverMenuItem) > 0 )
                        this.moreMenu.removeChild(this.startFailoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.stopFailoverMenuItem) < 0 )
                        this.moreMenu.addChild(this.stopFailoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.restartFailoverMenuItem) < 0 )
                        this.moreMenu.addChild(this.restartFailoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.removeFailoverMenuItem) < 0)
                        this.moreMenu.addChild(this.removeFailoverMenuItem);
                    
                    if (this.moreMenu.getIndexOfChild(this.failoverMenuItem) > 0)
                        this.moreMenu.removeChild(this.failoverMenuItem);
                    
                    this.failoverStatus.innerHTML = this.resources.FAILOVERSTARTING;

                    this.failoverActiveIcon.style.display = "none";
                    this.failoverStopIcon.style.display = "none";
                    this.failoverStartingIcon.style.display = "block";

                
                }
                else
                {
                    console.warning("An unknown failover monitoring state was detected. State: " + state + " Check the system logs for more details.");
                    this.moreMenu.removeChild(this.startFailoverMenuItem);
                    this.moreMenu.removeChild(this.stopFailoverMenuItem);
                    this.moreMenu.removeChild(this.editFailoverMenuItem);
                    this.moreMenu.removeChild(this.restartFailoverMenuItem);
                    this.moreMenu.removeChild(this.removeFailoverMenuItem);                                        
                    this.moreMenu.removeChild(this.failoverMenuItem);
                    
                }

            }
              
        }
        

    });

});