define([
        "dojo",
        "dijit",
        "support/ConfigPatternsAll",
        "support/panels/Panel",
        "dojo/i18n!./nls/ServerProfileTable",
        "dojo/text!./templates/ServerProfileTable.html",
        "dijit/layout/LayoutContainer",
        "dijit/layout/ContentPane",
        "support/base/NextGenUIMixin",
        "support/widgets/OutlinePane",
        "support/base/TableMixin",
        "dojo/data/ItemFileWriteStore",
        "support/dialogs/DeployServerTemplate",
        "dijit/Toolbar",
        "dijit/ToolbarSeparator",
        "dijit/form/CheckBox",
        "support/dialogs/ProgressDialog",
        "support/dialogs/UnassignProfile"
    ], function(dojo, dijit, layer, panel, mri, template) {

    return dojo.declare("support.panels.ServerProfileTable", [ support.panels.Panel, support.base.NextGenUIMixin, support.base.TableMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in managing server profiles
        // for Configuration Patterns.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "ServerProfileTable",
        
        // a reference to the function that is responsible for automatically
        //   refreshing the panel after profile deployments
        pollingFunc: null,
        
        // a reference to the current Date instance; used to turn off the
        //   automatic refresh mechanism if it runs too long
        pollingStartDate: null,
        
        // Saving this code below incase we switch to checking profile status to poll
        /* pollingAtLoad: [protected] Boolean
         holds if polling should start on page load or not, only time it should be true is
         if any profile is currently in Activating mode when page is first loaded
        pollingAtLoad: false,*/
        
        mode: "", // Current Context Mode
        tooltip: "",
        _profileTable: null,
        _deleteButton: null,
        _deployButton: null,
        _detailsOpt: null,
        _assignOpt: null,
        _unassignOpt: null,
        _renameOpt: null,

        actionMenu: null,
        _actionButton: null,

        _deployType: "",  //Used when deploying to know if deploying unassign or assigned
        profileLayout: {
            /* Toolbar Buttons and Menus */
            assignProfOpt: false, // Assign Server
            // Profile Option
            unassignProfOpt: true, // Unassign Server
            // Profile Option
            renameProfOpt: true, // Rename Server
            // Profile Option
            detailsOpt: true, // Server Profile
            // Details Option
            templateIDNum:""            
        },
        
        
        // the properties used to construct the title of this page in the
        //   NextGen UI
        cpProductName: null,
        cpPageContext: null,
        cpPageName: null,
        
        hideProfileTooltip: false,
        
        // ID prefix used for table
        gridID: "ServerProfileTable",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/ServerProfileTable.css",

        // When this method is called, all variables
        // inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget
        // property variables defined in the template
        // HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
            
            if(this.mode != "serverTemplateDetails") {
                this.cpProductName = this.resources.PRODUCT_NAME;
                // this.cpPageContext = this.resources.SERVERS; Must remain null (unspecified), per XB216902
                this.cpPageName = this.resources.SERVER_PROFILES;
                this.tooltip = "<br>" + this.resources.ALSO_SEE_TEXT;
            }
            
        },

        // postCreate() is called after
        // buildRendering(). This is useful to override
        // when
        // you need to access and/or manipulate DOM
        // nodes included with your widget.
        // DOM nodes and widgets with the
        // dojoAttachPoint attribute specified can now
        // be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in
        // buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            
            if(this.hideProfileTooltip){                
                this.profileToolTipArea.style.display = "none";
            }
            
            this.subscribe("RefreshTable", function(data) {            
                this._refreshTree();
            });
            
            this.subscribe("AssignServerProfileComplete", function(data) {       
                this._refreshTree();
                this._profileTable.select.row.clear();
                
                // since the profile assignment is now asynchronous, we must
                //   occasionally poll the services layer to see if profile
                //   activation states change
                this._pollForProfileStatusUpdates();
            });
            
            this.subscribe("DeployServerTemplateComplete", function(data) {       
                // no need to refresh the tree explicitly, as deployment
                //   occurs from a different page and navigating to this page
                //   will cause an explicit refresh
                
                // since the profile assignment is now asynchronous, we must
                //   occasionally poll the services layer to see if profile
                //   activation states change
                //this._pollForProfileStatusUpdates();
            });
            
            this.subscribe("UnassignServerProfileComplete", function(data) {
                // The refresh is already being done in the load of the xhr call
                // Refreshing again here causes issues with the store and can cause the table
                // to fail to load.  Just commenting out for now as a reference.
                this._refreshTree();
            });

            this.subscribe("EndPollingOnSelectorChange", function() {
                // Used to end polling when ever the CP selector changes                
                this._stopPolling();
             });

            this._setupLayoutTemplate(this.mode);

            // Setup table widget...
            var profileStore = this._retrieveStore(this.mode);
            var tableStructure = this._getTableStructure(this.mode);

            this._profileTable = this.getTable(profileStore, tableStructure, this.resources.NO_DATA, this.gridID);
            this._profileTable.column(0).sort(false);
            
            setTimeout(dojo.hitch(this, function() {
                this._profileTable.placeAt(this.profileTable);
                this._profileTable.startup();
            }), 500);

            // Connect handlers to reset table capabilities
            dojo.connect(this._profileTable.select.row, "onSelectionChange", this, this.setTableCapabilities); // Fires on row selection
            dojo.connect(this._profileTable, "setStore", this, this.setTableCapabilities); // Fires on store reset
    
            // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
            this.changeNextGenUITitle();
            
            // (possibly) render a dialog on top of this panel
            this.showDialog();
            
            // reload contents of profiles table; by default, switching
            //   between links in the details selector does not reload page
            this.subscribeToDetailsSelectorChanges(dojo.hitch(this, this._refreshTree));
            
            // switch selector title and start polling
            this.subscribeToDetailsSelectorChanges(dojo.hitch(this, this._refreshAfterSelectorChange));
            
            //Start polling on page load only if a profile is in Activating Status
            if(this.pollingAtLoad == true){
                this._pollForProfileStatusUpdates();    
            }

            //Start polling on page load 
            this._pollForProfileStatusUpdates();
            
            this._loadToolBar(this._profileTable); // load table toolbar
            
        },
        
        showDialog: function() {
            var currentURL = window.location.href;
            
            // if this widget was launched by one of the Configuration
            //   Patterns Java ITasks, it will contain a special URL
            //   parameter; if this parameter is present, we know we must
            //   launch a dialog
            if(currentURL.indexOf("cpParm") != -1) {
                if(currentURL.indexOf("dstTask") != -1) {
                    // show the Deploy Server Template dialog, but nest these
                    //   calls in a try/catch; if there is an error
                    //   instantiating the dialog, we still want this panel to
                    //   be displayed
                    var serverUUID = this.getEndPointPropertiesFromSourceToken().uuid;
                    var serverStr = " ";
                    try {
                        var dstdialog = new support.dialogs.DeployServerTemplate({
                            endpointUUID:serverUUID
                        });
                        serverStr = " (for server with UUID: " + serverUUID + ") ";
                        dstdialog.show();
                        // no need to dismiss or delete this dialog; the Deploy
                        //   Server Template dialog does this when the user clicks
                        //   OK or Cancel
                        console.log("Showing Deploy Server Template dialog" + serverStr + "from Server Profiles page");
                    }
                    catch(err) {
                        console.error("Failed to show Deploy Server Template dialog" + serverStr + "from Server Profiles page; error: " + err.message);
                    }
                }
                else if(currentURL.indexOf("uspTask") != -1) {
                    // show a confirmation dialog asking the user whether they
                    //   wish to unassign the server profile from the server,
                    //   but nest these calls in a try/catch; if there is an
                    //   error instantiating the dialog, we still want this
                    //   panel to be displayed
                    try {
                        this._unassignProfile(true);
                        console.log("Showing Unassign Server Profile confirmation dialog from Server Profiles page");
                    }
                    catch(err) {
                        console.error("Failed to show Unassign Server Profile confirmation dialog from Server Profiles page; error: " + err.message);
                    }
                }
            }
        },

        // Connect to data source and create models for
        // TreeTable widgets.
        _retrieveStore: function(mode) {

            // first, get the URL for the type of panel we are showing
                var servicesURL = "";
                if(mode == "serverTemplateDetails") {
                    servicesURL = this.REST_PATH + "template/"+this.templateIDNum+"/dependents";
                }
                else {
                    servicesURL = this.REST_PATH + "profile";
                }
                
                // invoke the services and store the data in a temporary
                var profileDataArr = new Array();
                var xhrGet = dojo.xhrGet({
                    url: servicesURL,
                    handleAs: "json",
                    preventCache: true,
                    sync: true,
                    load: dojo.hitch(this, function(response) {
                        if(response != null && response.items != null) {
                            for(var i=0; i<response.items.length; i++) {
                                profileDataArr.push(response.items[i]);
                               //Saving code below for when we switch to using profile status to control polling
                               /* if(response.items[i].profileStatus == "ACTIVATING" || response.items[i].profileStatus == "activating" || response.items[i].profileStatus == "Activating"){
                                    this.pollingAtLoad = true;
                                }*/
                            }
                        }
                        else {
                            console.error("Failed to load server profile data");
                        }
                    })
                });
                
                // setup the data store for the TreeTable
                var profileData = dojo.toJson({
                    "identifier": "id",
                    "label": "name",
                    "items": profileDataArr
                }, true);
                
                
                //profileData = dojo.toJson({"identifier":"id","label":"id","items":[{"profileName":"nsp 1-profile1","serverName":"","uuid":"","chassisName":"","bayId":"","subBayId":null,"profileStatus":"UNASSIGNED","templateName":"nsp 1","templateId":"41","type":"root","id":"85","externalId":null},{"profileName":"nsp 1-profile4","serverName":"","uuid":"","chassisName":"","bayId":"","subBayId":null,"profileStatus":"UNASSIGNED","templateName":"nsp 1","templateId":"41","type":"root","id":"94","externalId":null},{"profileName":"1BayDual-profile1","serverName":"cp-node-7.boss.stglabs.ibm.com","uuid":"C0D40A14F0A3B6018E70EEA17FA01E44","chassisName":"CFGPTRN-CMM.boss.stglabs.ibm.com","bayId":"6","subBayId":"2","profileStatus":"ASSIGNED","templateName":"1BayDual","templateId":"3","type":"root","id":"4","externalId":"1B54B9AEFCE04D5E820C0B6310D03590_bay6:2"},{"profileName":"nsp 1-profile3","serverName":"","uuid":"","chassisName":"","bayId":"","subBayId":null,"profileStatus":"UNASSIGNED","templateName":"nsp 1","templateId":"41","type":"root","id":"93","externalId":null},{"profileName":"nsp 1-profile2","serverName":"","uuid":"","chassisName":"","bayId":"","subBayId":null,"profileStatus":"UNASSIGNED","templateName":"nsp 1","templateId":"41","type":"root","id":"86","externalId":null}]}, true);
                
                //profileData = dojo.toJson({"identifier":"id","label":"id","items":[{"profileName":"terst2-profile1","serverName":"","uuid":"","chassisName":"","bayId":"","subBayId":null,"profileStatus":"UNASSIGNED","templateName":"-rename-terst2","templateId":"55","type":"root","id":"56","externalId":null}]}, true);            

                                
                var profileDataStore = new dojo.data.ItemFileWriteStore({
                    data: dojo.fromJson(profileData)
                });
                
                // Customize the sort function to be case insensitive
                profileDataStore.comparatorMap = {};
                profileDataStore.comparatorMap["profileName"] = function (a, b) {                            
                    return a.localeCompare(b);
                };
                profileDataStore.comparatorMap["templateName"] = function (a, b) {                            
                    return a.localeCompare(b);
                };
                profileDataStore.comparatorMap["bayId"] = function (a, b) {
                    //Return bayId's in correct order
                    //Split used here for those bayId's like "8 , 9"
                    var aArray = a.split(","); 
                    var bArray = b.split(","); 
                    
                    return aArray[0] - bArray[0];                                        
                };
                
                return profileDataStore;                        
        },

        // Load toolbar buttons/menus
        _loadToolBar: function(grid) {
            // tags:
            // protected
            var toolbar = grid && grid.toolBar && grid.toolBar.widget;
            
            if (toolbar) {
                // Load toolbar buttons/menus
                if (this._profileTable == null) {
                    // Profile table not created. Nothing to
                    // do but return.
                    return;
                }
                
                if (!this.actionMenu) {
                    this.actionMenu = new dijit.Menu({
                        style: "display: none;"
                    });
                }           
                                        
                // Add deploy button...
                this._deployButton = new dijit.form.Button({
                    //id: this.id + "cp_serverprofiletable_deploybutton",
                    showLabel: false,
                    iconClass: "spDeployProfileInactive",
                    disabled: true,
                    title: this.resources.DEPLOY_SELECTED_SERVER_PROFILE,
                    onClick: dojo.hitch(this, this._assignProfile),
                    onKeypress: dojo.hitch(this, this._assignProfile)                                                
                });
                toolbar.addChild(this._deployButton);
                
                // Add delete button...
                this._deleteButton = new dijit.form.Button({
                    //id: this.id + "cp_serverprofiletable_deletebutton",
                    showLabel: false,
                    iconClass: "spDeleteInactiveLgIcon",
                    disabled: true,
                    title: this.resources.DELETE_SELECTED_SERVER_PROFILES,
                    onClick: dojo.hitch(this, this._deleteProfile),
                    onKeypress: dojo.hitch(this, this._deleteProfile)                                                    
                });
                toolbar.addChild(this._deleteButton);
                
                toolbar.addChild(new dijit.ToolbarSeparator({}));
                
                //Add action menu to action button and create action button if not already created
                if (this._actionButton) {
                    this._actionButton.dropDown = this.actionMenu;
                }
                else {
                    this._actionButton = new dijit.form.DropDownButton({
                        //id: this.id + "cp_serverprofiletable_actionbutton",
                        label: this.resources.ALL_ACTIONS,
                        title: this.resources.ALL_ACTIONS_TITLE,
                        showLabel: true,
                        dropDown: this.actionMenu
                    });   
                    
                    toolbar.addChild(this._actionButton);
                }
                
            }
        },

        // Re-render Tree table with new columns/data
        // presentation
        _reRenderTable: function() {
            // 99275: Get function from tree table widget
        },
        
        _refreshTree: function() {
            // tags:
            // protected
            // Refresh tree data
            var patternStore = this._retrieveStore(this.mode);
            this._profileTable.setStore(patternStore);
            this._profileTable.model.clearCache();
            this._profileTable.body.refresh();
        },

        _refreshAfterSelectorChange: function() {            
            this.changeNextGenUITitle();
            this._pollForProfileStatusUpdates();
        },

        // Assign Profile Menu Option
        _assignProfile: function() {            
            
            if(this._getSelectedRowCount() == 1){
            
                var rowIds = this._getSelectedRowIds();
                var profileName = this._getRowsData(rowIds,"profileName","string");
                var profileId = this._getRowsData(rowIds,"id","string");            
                var templateId = this._getRowsData(rowIds,"templateId","string");            

                
                if(this._deployType == "unassign"){
                    // if the profile the user clicked on was unassigned
                    
                    require([ "support/dialogs/AssignServerProfile" ], dojo.hitch(this, function() {

                        var assigndialog = new support.dialogs.AssignServerProfile({
                            profileId: profileId,
                            profileName: profileName,
                            templateId: templateId
                        });
                        assigndialog.show();


                    }));

                }
                else if(this._deployType == "assign" || this._deployType == "failed"){
                    // if the profile the user clicked on is assigned
                    
                    var confirmedDeploy = function() {
                        // show the progress dialog
                        this.getProgressDialog("activate").show(); // Show progress dialog for activate request
                        dojo.xhrPost({
                            url: this.REST_PATH + "profile/assign/" + profileId + "/location/",
                            handleAs: "json",
                            headers: {
                                "Content-Type": "application/json; charset=utf-8"
                            },
                            sync: false,
                            load: dojo.hitch(this,function(data) {
                                if(data && data.error && data.error.length > 0){
                                    // regardless of the result, hide the
                                    //   progress dialog
                                    this.getProgressDialog("activate").hide();
                                    var validationpopup = new support.temp.ModalDialog({
                                        type: "Error",
                                        text: data.error
                                    });
                                    validationpopup.show();
                                }
                                else{
                                    dojo.publish("AssignServerProfileComplete");
                                    // regardless of the result, hide the
                                    //   progress dialog
                                    this.getProgressDialog("activate").hide();                      
                                }
                                
                            }),
                            error: dojo.hitch(this,function(data) {
                                // regardless of the result, hide the progress
                                //   dialog
                                this.getProgressDialog("activate").hide();
                                var validationpopup = new support.temp.ModalDialog({
                                    type: "Error",
                                    text: this.resources.ERROR_ASSIGN_PROF
                                });
                                console.log(data);
                                validationpopup.show();
                            })
                        });
                        
                    };
                    

                    if(this.CP_RUN_ENVIRONMENT == "fsm"){

                        require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                            var confDialog = new support.temp.ConfirmationDialog({
                                buttonLabel: this.resources.DEPLOY,
                                type: "Warning",
                                text: this.locResources.DEPLOY_CONF,
                                info: this.locResources.DEPLOY_CONF_LIST.replace("%1", profileName)
                            });

                            confDialog.startup();
                            confDialog.confirm(confirmedDeploy, this);
                    
                        }));
                        
                    }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                        require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                            var confDialog = new idx.widget.ConfirmationDialog({
                                buttonLabel: this.resources.DEPLOY,
                                type: "Warning",
                                text: this.locResources.DEPLOY_CONF,
                                info: this.locResources.DEPLOY_CONF_LIST.replace("%1", profileName)
                            });

                            confDialog.startup();
                            confDialog.confirm(confirmedDeploy, this);
                    
                        }));                        
                        
                    }
                    
                }
            
            }
        },
        
        // Unassign Profile Menu Option
        _unassignProfile: function(externallyLaunched) {

            var profNameToUnassign = "";
            var selectedIDArray = [];
            var managementPatternsPresent = [];
            var addressesPresent = [];
            // if this function was called from the showDialog(...) function,
            //   then the ServerProfileTable was launched externally (e.g. via
            //   the FSM Chassis Map or Resource Explorer); in this case there
            //   will be no profile selected, so we must obtain the profile to
            //   unassign from the URL
            if(externallyLaunched) {
                profNameToUnassign = this.getEndPointPropertiesFromSourceToken().serverProfileName;
                selectedIDArray.push(this.getEndPointPropertiesFromSourceToken().serverProfileId);                
            }
            else {
                profNameToUnassign = this._getRowsData("","profileName","string");
                selectedIDArray = this._getRowsData("","id","array");
                managementPatternsPresent = this._getRowsData("","managementPatternPresent","array");
                addressesPresent = this._getRowsData("", "addressPresent", "array");
            }
                        
            var unassignMsg = this.resources.UNASSIGN_PROFILE_FROM_SERVER.replace("%1", profNameToUnassign);
            
            // Determine if resetIMM should be selected by default
            var resetIMM = false;
            for (var i = 0; i < managementPatternsPresent.length; i++) {
                if (managementPatternsPresent[i].toString() == "false") {
                    resetIMM = false;
                    break;
                } else {
                    resetIMM = true;
                }
            }
            
            // Determine if powerOff should be selected by default
            var powerOff = false;
            for (var i = 0; i < addressesPresent.length; i++) {
                if (addressesPresent[i].toString() == "false") {
                    powerOff = false;
                    break;
                } else {
                    powerOff = true;
                }
            }

            var unassignDialog = new support.dialogs.UnassignProfile({
                profileName: profNameToUnassign,
                externalLaunch: externallyLaunched,
                message: unassignMsg,
                selectedIDs: selectedIDArray,
                mgmtPatternPresent: resetIMM,
                addrPresent: powerOff
            });
            
            unassignDialog.show();
        },
        
        // Rename Profile Menu Option
        _renameProfile: function() {

            var rowIds = this._getSelectedRowIds();
            var profileName = this._getRowsData(rowIds,"profileName","string");
            var selectedID = this._getRowsData(rowIds,"id","string");            
                                  
            if (profileName != "") {
                
                require([ "support/dialogs/Rename" ], dojo.hitch(this, function() {

                    var renamedialog = new support.dialogs.Rename({
                        id: this.id + "cp_serverprofiletable_rename",
                        templateIDNum: selectedID,
                        profileMode: "server"
                    });
                    renamedialog.show();
                
                }));

            }
            else {
                alert(this.resources.NO_PROFILES);
            }
        },
        
        // Server Details Menu Option
        _detailsServer: function(profileId) {
            
            var selectedID = profileId.toString();
            
            if(profileId == ""){
                selectedID = this._getRowsData("","id","string"); 
            }
                       
            require([ "support/panels/ServerProfileDetails" ], dojo.hitch(this,function() {
                
                if(this.CP_RUN_ENVIRONMENT == "fsm"){

                    this.loadNGUIWorkAreaWithCPWidget("support/panels/ServerProfileDetails", null, selectedID);                                                 
                
                }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                    var launchParams = {templateIDNum: selectedID};
                    
                    this.loadAugustaWorkAreaTask("config.detail.page","support/panels/ServerProfileDetails",launchParams);
                    
                }
            
            }));

        },
        _getRowsData: function(rowIds,dataField,returnType){

            if(rowIds == "") {
                rowIds = this._getSelectedRowIds();
            }

            var dataReturn;

            if(returnType == "array"){

                dataReturn = new Array();
                
                for (var loopVar = 0; loopVar < rowIds.length; ++loopVar) {
                    var item = this._profileTable.row(rowIds[loopVar], true).item();
                    dataReturn.push(item[dataField]);
                }
            
            }else if(returnType == "string"){
            
                dataReturn = "";
            
                for (var loopVar = 0; loopVar < rowIds.length; ++loopVar) {
                    var item = this._profileTable.row(rowIds[loopVar], true).item();

                    if(dataReturn != "") {
                        dataReturn += ", ";
                    }
                    
                    dataReturn += item[dataField];
                }
            
            }else{

                dataReturn = "";
                
            }
            
            return dataReturn;
        },                        
        // Delete Profile Menu Option
        _deleteProfile: function() {

                var rowIds = this._getSelectedRowIds();
                var profileName = this._getRowsData(rowIds,"profileName","string");
                
                var confirmedDelete = function() {

                    var selectedIDArray = this._getRowsData(rowIds,"id","array");
                    
                    for(var loopVar = 0; loopVar < rowIds.length; ++loopVar){
                        var xhrDelete = dojo.xhrDelete({
                            url: this.REST_PATH + "profile/" + selectedIDArray[loopVar],
                            handleAs: "json",
                            sync: true,
                            preventCache: true,
                            load: dojo.hitch(this, function(response) {
                                // also refresh the table after the delete
                                this._refreshTree();
                            })
                        });

                    }
                                       
                };
                
                if(this.CP_RUN_ENVIRONMENT == "fsm"){

                    require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {
                
                        var confDialog = new support.temp.ConfirmationDialog({
                            buttonLabel: this.resources.DELETE,
                            type: "Warning",
                            text: this.locResources.CONFIRM5,
                            info: this.locResources.DELETE.replace("%1", profileName)
                        });

                        confDialog.startup();
                        confDialog.confirm(confirmedDelete, this);
                
                    }));
                    
                }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                    require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {
                        
                        var confDialog = new idx.widget.ConfirmationDialog({
                            buttonLabel: this.resources.DELETE,
                            type: "Warning",
                            text: this.locResources.CONFIRM5,
                            info: this.locResources.DELETE.replace("%1", profileName)
                        });

                        confDialog.startup();
                        confDialog.confirm(confirmedDelete, this);
                
                    }));

                }
                
            
        },

        // Disable or enable options for custom modes
        _setupLayoutTemplate: function(mode) {
            switch (mode) {
                case ("serverTemplateDetails"):
                    this.profileLayout.assignProfOpt = false;
                    this.profileLayout.detailsOpt = false;
                    break;               
                default:
                    this.profileLayout.detailsOpt = true;
                    break;
            }
        },
        
        // Hard-coded path appended to URL, must be changed if path changes
        _decorateName: function(item, id) {
            // tags:
            // protected
            if (!item || !id) {
                return "";
            }
            
            var value = item;
            
            if (this.CP_RUN_ENVIRONMENT == "fsm") {
                return "<a href=\"#workAreaDijit=support%2Fpanels%2FServerProfileDetails&path=support:%2Fcom.ibm.stg.sysx.profile." +
                	"navmodel%2Fjs%2Fsupport&cpTarget=" + id + "\">" + value + "</a>";
            } else if(this.CP_RUN_ENVIRONMENT == "augusta") {
                return "<a href=\"#fwWidget=support%2Fpanels%2FServerProfileDetails&fwTaskId=config.detail.page&templateIDNum=" 
                    + id + "\">" + value + "</a>";
            }
        },
        
        _formatName: function(item) {

            var value = item.profileName;
            value = this.encodeForHtml(value);
            
            return value;
        },
        
        _formatBay: function(item){
                        
            if(item.subBayId != null){                
                return item.bayId+":"+item.subBayId;
            }else{
                var bays = item.bayId.split(" , ");
                if(bays.length == 1) {
                    return item.bayId;
                } else {
                    return bays[0] + "&ndash;" + bays[bays.length-1];
                }
            }
            
        },

       	_decorateNoWrap: function(item) {
            // tags:
            // protected
            var value = item;
            !value && (value = "");
            if (value !== "") {
                value = this.encoder(value);
                value = "<span title=\"" + value + "\" style=\"white-space: nowrap;\">" + value + "</span>";
            }
            return value;
        },
        
        _decorateWrap: function(item) {
            // tags:
            // protected
            var value = item;
            !value && (value = "");
            if (value !== "") {
                value = this.encoder(value);
                value = "<span title=\"" + value + "\" style=\"white-space: normal;\">" + value + "</span>";
            }
            return value;
        },        

        _decorateProfileStatus: function(item) {
            // tags:
            // protected            
            var text = item;
            var iconClass = "";
            
            switch (text) {
                case this.resources.PROFILEASSIGNED:
                    iconClass = "spSuccessIcon";
                    break;
                case this.resources.UNASSIGNED:
                    iconClass = "spOfflineIcon";
                    break;
                case this.resources.ACTIVATINGSTATUS:
                    iconClass = "spProcessingIcon";
                    break;
                case this.resources.DEPLOYFAILED:
                    iconClass = "spWarningIcon";
                    break;
                case this.resources.PLACEHOLDER:
                    iconClass = "spNormalIcon";
                    break;
                case this.resources.NOT_ACTIVATED:
                    iconClass = "spWarningIcon";
                    break;
            }
            
            if(text != ""){
                return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr>" +
                    "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"" + iconClass + "\" title=\"" + text + "\"></img></td>" +
                    "<td><span style=\"margin-left:4px; white-space: nowrap;\" title=\"" + text + "\">" + text + "</span></td></tr></tbody></table>";
            }else{
                return "";
            }    

            return status;
       },
       
       _formatProfileStatus: function(item) {
           var status = item.profileStatus.toUpperCase();
           var text = "";
           
           switch (status) {
               case "ASSIGNED":
                   text = this.resources.PROFILEASSIGNED;
                   break;
               case "UNASSIGNED":
                   text = this.resources.UNASSIGNED;
                   break;
               case "ACTIVATING":
                   text = this.resources.ACTIVATINGSTATUS;
                   break;
               case "ERROR_ACTIVATING":
                   text = this.resources.DEPLOYFAILED;
                   break;
               case "PLACEHOLDER":
                   text = this.resources.PLACEHOLDER;
                   break;
               case "ASSIGNED_NEEDING_ACTIVATION":
                   text = this.resources.NOT_ACTIVATED;
                   break;
           }
           return text;
       },

        // Columns to be displayed on table
        _getTableStructure: function(mode) {

            var cols = new Array();

            cols.push({
                field: "profileName",
                name: this.resources.PROF,
                width: "200px",
                formatter: dojo.hitch(this, this._formatName),
                decorator: dojo.hitch(this, this._decorateName)
            });
            cols.push({
                field: "serverName",
                name: this.resources.SERVER,
                width: "140px",
                encoder: this.encodeForHtml, // pass value encoder to formatter function
                decorator: this._decorateNoWrap
            });
            cols.push({
                field: "chassisName",
                name: this.locResources.CHAS,
                width: "140px",
                encoder: this.encodeForHtml, // pass value encoder to formatter function
                decorator: this._decorateNoWrap
            });
            cols.push({
                field: "bayId",                
                name: this.resources.BAY,
                width: "50px",
                formatter: dojo.hitch(this, this._formatBay)
            });
            cols.push({
                field: "profileStatus",
                name: this.resources.PROFILE_STATUS,
                width: "120px",
                formatter: dojo.hitch(this, this._formatProfileStatus),
                decorator: dojo.hitch(this, this._decorateProfileStatus)
            });
            cols.push({
                field: "templateName",
                name: this.resources.SERVER_TEMP,
                width: "auto",
                encoder: this.encodeForHtml, // pass value encoder to formatter function
                decorator: this._decorateWrap
            });      
             
            return cols;
        },

        // Return number of selected rows
        _getSelectedRowCount: function() {
            return this._profileTable.select.row.getSelected().length;
        },

        // Return array of IDs for selected rows
        _getSelectedRowIds: function() {
            return this._profileTable.select.row.getSelected();
        },

        // Return array of row objects for selected rows
        _getSelectedRows: function() {
            var rows = [];
            var rowIds = this._getSelectedRowIds();
            for (var loopVar = 0; loopVar < rowIds.length; ++loopVar) {
                rows.push(this._profileTable.row(rowIds[loopVar], true).item());
            }
            return rows;
        },

        // Enable or disable toolbar buttons depending on rows selected
        setTableCapabilities: function() {            

            if (this._deleteButton) {
                this._deleteButton.set("iconClass", "spDeleteInactiveLgIcon");
                this._deleteButton.set("disabled", true);
            }
            
            if (this._deployButton) {
                this._deployButton.set("disabled", true);
                this._deployButton.set("iconClass", "spDeployProfileInactive");
            }

            this._setDynamicMenus();

        },

        _selectedUnassigned : function() {
            // Return true if ALL selected profiles (rows) are unassigned, otherwise false.
            var selectedRows = this._getSelectedRows();
            return ((selectedRows.length > 0) && dojo.every(selectedRows, function(item) {
                return (item && item.profileStatus && (item.profileStatus[0].toUpperCase() == "UNASSIGNED"));
            }));
            return false;
        },

        _selectedAssigned : function () {
            // Return true if ALL selected profiles (rows) are assigned, otherwise false.
            var selectedRows = this._getSelectedRows();
            return ((selectedRows.length > 0) && dojo.every(selectedRows, function(item) {
                return (item && item.profileStatus && (item.profileStatus[0].toUpperCase() == "ASSIGNED"));
            }));
            return false;
        },

        _selectedFailed : function () {
            // Return true if ALL selected profiles (rows) are in failed activation state, otherwise false.
            var selectedRows = this._getSelectedRows();
            return ((selectedRows.length > 0) && dojo.every(selectedRows, function(item) {
                return (item && item.profileStatus && (item.profileStatus[0].toUpperCase() == "ERROR_ACTIVATING"));
            }));
        }, 

        //Set Context and Action menus baes on rows currently clicked in table
        _setDynamicMenus : function () {
            /*There is different logic the same menu items since the context menu is created
            on every click while the action menu is created once and changed only if transition 
            between states is detected, i.e. No need to add/remove all menu options on every row click
            */
            var numSelected = this._getSelectedRowCount();
            var allAssigned = this._selectedAssigned();
            var allUnassigned = this._selectedUnassigned();
            var allFailedActivation = this._selectedFailed();

            console.log(numSelected, "allAssigned: " + allAssigned, ", allUnassigned: " + allUnassigned, ", allFailedActivation: " + allFailedActivation);
            //alert(numSelected + " Assign:" + allAssigned + " Unassign:" + allUnassigned);

            if(numSelected > 0){
                
                this._deployButton.set("disabled", true);
                this._deployButton.set("iconClass", "spDeployProfileInactive");

                if(numSelected === 1 && allAssigned){
                    this._deployButton.set("disabled", false);
                    this._deployButton.set("iconClass", "spDeployProfile");
                    this._deployType = "assign";
                }
                
                if(numSelected === 1 && allFailedActivation){
                    this._deployButton.set("disabled", false);
                    this._deployButton.set("iconClass", "spDeployProfile");
                    this._deployType = "failed";
                }
                
                if(numSelected === 1 && allUnassigned){
                    this._deployButton.set("disabled", false);
                    this._deployButton.set("iconClass", "spDeployProfile");
                    this._deployType = "unassign";
                }
                
                if (numSelected === 1) {
                    if (!this._deployActionItem) {
                        
                        this._deployActionItem = new dijit.MenuItem({
                         //   id: this.id + "cp_serverprofiletable_deployactionitem",
                            label: this.locResources.DEPLOY,
                            title: this.resources.DEPLOY_SELECTED_SERVER_PROFILE,
                            iconClass: "spDeployProfile",
                            onClick: dojo.hitch(this, this._assignProfile),
                            onKeypress: dojo.hitch(this, this._assignProfile)                                
                        });
                    }
                    
                    this.actionMenu.addChild(this._deployActionItem,0);                    
                } else {
                    this.actionMenu.removeChild(this._deployActionItem);
                    this._deployActionItem = null;
                }

                if(allUnassigned){
                    
                    if (!this._deleteActionItem) {
                        
                        // Delete Action Item
                        this._deleteActionItem = new dijit.MenuItem({
                         //   id: this.id + "cp_serverprofiletable_deleteactionitem",
                            label: this.resources.DELETE,
                            title: this.resources.DELETE_SELECTED_SERVER_PROFILES,
                            iconClass: "spDeleteLgIcon",
                            onClick: dojo.hitch(this, this._deleteProfile),
                            onKeypress: dojo.hitch(this, this._deleteProfile)                                                    
                        });
                    }
                    
                    this._deleteButton.set("iconClass", "spDeleteLgIcon");
                    this._deleteButton.set("disabled", false);   
                    
                    this.actionMenu.addChild(this._deleteActionItem);                    
                }else{
                    this._deleteButton.set("iconClass", "spDeleteInactiveLgIcon");
                    this._deleteButton.set("disabled", true);
                    
                    this.actionMenu.removeChild(this._deleteActionItem);
                    this._deleteActionItem = null;
                }                               
                
                // Details Option
                if(numSelected === 1){

                	//Action Menu Option
                    if(this._detailsOpt == null){
                    	this._detailsOpt = new dijit.MenuItem({
                        //   id: this.id + "cp_serverprofiletable_detailsoption",
                        	label: this.resources.DETAILS,
                            title: this.resources.DETAILS_SELECTED_PROFILE,
                            onClick: dojo.hitch(this, function() {
                            	this._detailsServer("");
                            })
                        });
                        this.actionMenu.addChild(this._detailsOpt);
                    }
                    else if(numSelected != 1 && this._detailsOpt != null){
                    	this.actionMenu.removeChild(this._detailsOpt);
                        this._detailsOpt = null;
                    }
                }else{
                    this.actionMenu.removeChild(this._detailsOpt);
                    this._detailsOpt = null;                        
                }                
                
                //Rename Profile 
                if (this.profileLayout.renameProfOpt) {
                    if(numSelected === 1){
                        //Action Menu
                        if(this._renameOpt == null ){
                            this._renameOpt = new dijit.MenuItem({
                             //   id: this.id + "cp_serverprofiletable_renameoption",
                                label: this.resources.RENAME,
                                title: this.resources.RENAME_SELECTED_PROFILE,
                                onClick: dojo.hitch(this, function() {
                                    this._renameProfile();
                                })
                            });
                            this.actionMenu.addChild(this._renameOpt);
                        }

                    }
                    else if(this._renameOpt != null){
                        this.actionMenu.removeChild(this._renameOpt);
                        this._renameOpt = null;
                    }
                }

                //Unassign Profile Option
                if (this.profileLayout.unassignProfOpt) {
                    if((allAssigned || allFailedActivation) && !allUnassigned){                        

                        //Action Menu
                        if(this._unassignOpt == null ){
                            this._unassignOpt = new dijit.MenuItem({
                              //  id: this.id + "cp_serverprofiletable_unassignoption",
                                label: this.resources.UNASSIGN,
                                title: this.resources.UNASSIGN_SELECTED_SERVER_PROFILE,
                                onClick: dojo.hitch(this, function() {
                                    this._unassignProfile();
                                })
                            });
                            this.actionMenu.addChild(this._unassignOpt);
                        }

                    }
                    else if(this._unassignOpt != null){
                        this.actionMenu.removeChild(this._unassignOpt);
                        this._unassignOpt = null;
                    }
                }
                
                //Assign Profile Option
                if (this.profileLayout.assignProfOpt) {
                    if(numSelected === 1 && allUnassigned){
                        
                        //Action Menu
                        if(this._assignOpt == null) {
                            this._assignOpt = new dijit.MenuItem({
                               // id: this.id + "cp_serverprofiletable_assignoption",
                                label: this.locResources.DEPLOY,
                                title: this.resources.DEPLOY_SELECTED_SERVER_PROFILE,
                                onClick: dojo.hitch(this, function() {
                                    this._assignProfile();
                                })
                            });
                            this.actionMenu.addChild(this._assignOpt);
                        }

                    }
                    else if(this._assignOpt != null){
                        this.actionMenu.removeChild(this._assignOpt);
                        this._assignOpt = null;
                    }
                }                
                
            }
            else{ //Remove Menu Items If Nothing Is Selected
                if(this._deployActionItem != null){
                    this.actionMenu.removeChild(this._deployActionItem);
                    this._deployActionItem = null;
                }
                if(this._deleteActionItem != null){
                    this.actionMenu.removeChild(this._deleteActionItem);
                    this._deleteActionItem = null;
                }
                if(this._assignOpt != null){
                    this.actionMenu.removeChild(this._assignOpt);
                    this._assignOpt = null;
                }
                if(this._unassignOpt != null){
                    this.actionMenu.removeChild(this._unassignOpt);
                    this._unassignOpt = null;
                }
                if(this._renameOpt != null){
                    this.actionMenu.removeChild(this._renameOpt);
                    this._renameOpt = null;
                }
                if(numSelected != 1 && this._detailsOpt != null){
                    this.actionMenu.removeChild(this._detailsOpt);
                    this._detailsOpt = null;
                }
            }
        },

        scopedRTFunc : function(){
            if(this._profileTable != null){
                var rowCount = this._getSelectedRowCount();
                if(rowCount == 0){
                    this._refreshTree();
                }
            }else{
                if(this.pollingFunc != null) {
                    this._stopPolling();
                }
            }
        },
        // this function is responsible for continually polling the services
        //   layer for profile information in order to dynamically update the
        //   table as profile activation states change
        _pollForProfileStatusUpdates: function() {
            console.log("Initiating polling for server profile status updates (Server Profiles page)");
            // call the _retrieveStore(...) function every 10 seconds in
            //   order to obtain the most recent server profile data
            
            // first, stop the existing polling mechanism (needed in case user
            //   does multiple assignments/deployments in succession)
                console.log("Stopping server profile status updates before starting again (Server Profiles page)");
                this._stopPolling();
                
            // next, create a bound function so that when
            //   _refreshTree(...) executes, it will execute with the correct
            //   context for "this"
            //var scopedRTFunc = dojo.hitch(this, this._refreshTree);
            
            // next, create a bound function so that when
            //   _checkStopPolling(...) executes, it wille execute with the
            //   correct context for "this"
            var scopedCSPFunc = dojo.hitch(this, this._checkStopPolling);
            
            // set the current date
            this.pollingStartDate = (new Date()).getTime();
            
            // now, simply call the setInterval(...) function to call
            //   _refreshTree(...) followed by _checkStopPolling(...) every 30
            //   seconds
            this.pollingFunc = window.setInterval(dojo.hitch(this, function(){ this.scopedRTFunc();scopedCSPFunc(); }), 30000);
        }, // _pollForProfileStatusUpdates
        
        // this function is responsible for determining whether to stop the
        //   polling for status updates
        _checkStopPolling: function() {
            // if we have been polling for too long, stop the polling
            //   mechanism
            var curDate = (new Date()).getTime();
            var dateDiff = curDate - this.pollingStartDate;
            // if we've been polling for longer than 10 minutes, stop
            if(dateDiff > 600000) {
                console.log("Polling for server profile status updates has occurred for longer than 10 minutes; stopping polling (Server Profiles page)");
                window.clearInterval(this.pollingFunc);
                this.pollingFunc = null;
            }
        }, // _checkStopPolling
        // this function is responsible for stopping the polling
        _stopPolling: function() {

            if(this.pollingFunc != null) {
                console.log("Stopping server profile status polling (Server Profiles page)");
                window.clearInterval(this.pollingFunc);
                this.pollingFunc = null;
            }

        },//_stopPolling
        
        goForeground: function(params) { //Page base function that fires when the page is brought to focus
            
            this._refreshTree();

            //this._profileTable.select.row.clear();
                    
            this._pollForProfileStatusUpdates();

            return true;
        },
        
        goBackground: function(){ //Page base function that fires when the page losses to focus
            
            this._stopPolling();
            
        }
    });
});