define(["dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/ServerTemplateDetails",
        "dojo/text!./templates/ServerTemplateDetails.html",
        "support/base/NextGenUIMixin", 
        "support/widgets/OutlinePane", 
        "dijit/layout/TabContainer",
        "support/panels/ServerConfiguration",
        "support/panels/ServerProfileTable",
        "dijit/Toolbar",
        "dijit/ToolbarSeparator",
        "dijit/Menu",
        "support/dialogs/ServerTemplateWizard",
        "support/dialogs/DeployServerTemplate",
        "dojo/dom"
        ], function(dojo, dijit, panel, mri, template) {

    
    return dojo.declare("support.panels.ServerTemplateDetails", [ support.panels.Panel, support.base.NextGenUIMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in displaying information
        // associated with a server.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "ServerDetails",
        
        // the properties used to construct the title of this page in the
        //   NextGen UI
        cpPageContext: null,
        cpPageName: null,
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/ServerTemplateDetails.css",

        configurationTabdata: "",
        ioAddressesTabdata: "",
        eventsTabdata: "",
        dependenciesTabdata: "",
        
        templateInformation: "",
        
        templateIDNum: "",
        templateName: "",
        templateDesc: "",
        templateInUse: false,

        postMixInProperties: function() {
            this.inherited(arguments);
            this.cpPageContext = "";
            this.cpPageName = this.resources.DETAILS;
        },

        postCreate: function() {
            this.inherited(arguments);

            if(this.CP_RUN_ENVIRONMENT == "augusta"){

                //Add page base in order to use the setTitle() function
                require([ "augusta/augustaUICore/core/augustaPageBase" ], dojo.hitch(this, function(PageBase) {                
                    var CP_AUGUSTA_PAGE_BASE = new augusta.augustaUICore.core.augustaPageBase();
                    dojo.safeMixin(this, CP_AUGUSTA_PAGE_BASE);            
                }));

                this.bwIsCacheable = false;
            }    
            
            this.subscribe("RefreshNameDiv", function(data) {
                // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
                this.cpPageContext = data;

                if(this.CP_RUN_ENVIRONMENT == "fsm"){
                    //Set title in FSM
                    this.changeNextGenUITitle();
                }
                
                if(this.CP_RUN_ENVIRONMENT == "augusta"){
                    //Set Title in xHMC, only can be used if augustaPageBase mixed in
                    this.setTitle(this.cpPageContext + " - " + this.cpPageName);
                }

            });

            this.subscribe("ServerTemplateComplete", function(data) {
                // Update details page with Server Pattern new information
                this.getServerTemplateID();
                dojo.publish("RefreshNameDiv",this.templateName);
            });

            this.getServerTemplateID();
                        
            // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
            this.cpPageContext = this.templateName;

            if(this.CP_RUN_ENVIRONMENT == "fsm"){
                //Set title in FSM
                this.changeNextGenUITitle();
            }
            
            if(this.CP_RUN_ENVIRONMENT == "augusta"){
                //Set Title in xHMC
                this.setTitle(this.cpPageContext + " - " + this.cpPageName);
            }

            this.toolbarSetup();

                // Configurations Tab
                this.configurationTabdata = new support.panels.ServerConfiguration({
                    id: this.id + "cp_servertemplatedetails_serverconfiguration",
                    mode: "serverTemplateDetails",
                    templateIDNum:this.templateIDNum,
                    templateName: this.templateName,
                    title:this.locResources.CONFIG_TAB_TITLE
                }).placeAt(this._configurationTab); 
                              
                // Dependencies Tab
                this.dependenciesTabdata = new support.panels.ServerProfileTable({
                    id: this.id + "cp_serverprofiletable_serverprofiletable",
                    mode: "serverTemplateDetails",
                    templateIDNum:this.templateIDNum,
                    title:this.locResources.SVRTMPL_DEPENDENTS_TAB_TITLE,
                    hideProfileTooltip: true
                }).placeAt(this._dependenciesTab);  
                
                // Events Tab
                /*this.eventsTabdata = new support.panels.ServerProfileTable({
                    mode: "detailsTab", 
                    tableWidth: "99%",
                    title:this.locResources.EVENTS_TAB_TITLE
                }).placeAt(this._eventsTab);*/
                                                                                                                                                                                             
        },
        //Get Server Template ID
        getServerTemplateID: function(){
            var store ="";
          
            if(this.templateIDNum == ""){  //If this templateID not provided then parse url for it
                var url = window.location.href;
                var splitUrl = url.split("cpTarget=");
                var IDarray = new Array();
                var ID = "";
                
                if(splitUrl[1]){
                    IDarray = splitUrl[1].split("&");
                }else{
                    ID = "";
                }
                
                if(IDarray[0]){
                    ID=IDarray[0];
                    
                    var xhrGet = dojo.xhrGet({
                        url: this.REST_PATH + "template/" + ID,
                        handleAs: "json",
                        sync: true,
                        preventCache: true,
                        load: function(response) {
          //                  console.log(response);
                            store = response;
                        }
                    });
                                 
                    this.templateInformation = store;
                    
                    this.templateIDNum = ID;
                    
                }else{  //If no ID found then ...
                                                
                    
                }
    
                                                                 
            }else{ //IF ID passed then use passed ID

                var xhrGet = dojo.xhrGet({
                    url: this.REST_PATH + "template/" + this.templateIDNum,
                    handleAs: "json",
                    sync: true,
                    preventCache: true,
                    load: function(response) {
                        console.log(response);
                        store = response;
                    }
                });
                
                this.templateInformation = store;
            }            
            
            
         //Set Template Name and Description and InUse ... etc ....            
            //Check to see if server template info was gained
            if(this.templateInformation != "" && this.templateInformation.items != ""){  
             
                //if(this.templateInformation.items[0].name){
                    this.templateName = this.templateInformation.items[0].name;
                //}
                
                //if(this.templateInformation.items[0].description){
                    this.DetailsTemplateDescription.innerHTML = this.templateInformation.items[0].description;
                //}

                    this.DetailsTemplateType.innerHTML = this.resources.SERVER;

                //if(this.templateInformation.items[0].inUse){
                    this.templateInUse = this.templateInformation.items[0].inUse;
                //}
            }

            
        },
        //ToolBar Setup
        toolbarSetup: function(){
          
            this.moreMenu = new dijit.Menu({          // Add to More/Actions Menu
                style: "display: none;"
            });
      
            this.editButton = new dijit.form.Button({     //Add Edit Button to toolbar
                showLabel: false,
                iconClass: "spEditLgIcon",
                disabled: false,
                title: this.resources.EDIT_PATTERN_TITLE,
                onClick: dojo.hitch(this, this.editTemplate),
                onKeypress: dojo.hitch(this, this.editTemplate)
            });
            this.detailsToolBar.addChild(this.editButton);

            this.deployButton = new dijit.form.Button({     //Add Deploy Button to toolbar
                showLabel: false,
                iconClass: "spDeployLgIcon",
                disabled: false,
                title: this.resources.DEPLOY_SERVER_PATTERN_TITLE,
                onClick: dojo.hitch(this, this.deployTemplate),
                onKeypress: dojo.hitch(this, this.deployTemplate)
            });
            this.detailsToolBar.addChild(this.deployButton);

            // Add to More/Actions Menu                    
            this.detailsSeparator = new dijit.ToolbarSeparator();
            this.detailsToolBar.addChild(this.detailsSeparator);
            
            this.copyItem = new dijit.MenuItem({
                label: this.resources.COPY,
                title: this.resources.COPY_PATTERN_TITLE,
                onClick: dojo.hitch(this, this.copyTemplate),
                onKeypress: dojo.hitch(this, this.copyTemplate)
            });
            this.moreMenu.addChild(this.copyItem);

            this.renameItem = new dijit.MenuItem({
                label: this.resources.RENAME,
                title: this.resources.RENAME_PATTERN_TITLE,
                onClick: dojo.hitch(this, this.renameTemplate),
                onKeypress: dojo.hitch(this, this.renameTemplate)
            });
            this.moreMenu.addChild(this.renameItem);
            
            this.editItem = new dijit.MenuItem({
                label: this.resources.EDIT,
                title: this.resources.EDIT_PATTERN_TITLE,
                iconClass: "spEditLgIcon",
                onClick: dojo.hitch(this, this.editTemplate),
                onKeypress: dojo.hitch(this, this.editTemplate)
            });
            this.moreMenu.addChild(this.editItem,0);
            
            this.deployItem = new dijit.MenuItem({
                label: this.resources.DEPLOY,
                title: this.resources.DEPLOY_SERVER_PATTERN_TITLE,
                iconClass: "spDeployLgIcon",
                onClick: dojo.hitch(this, this.deployTemplate),
                onKeypress: dojo.hitch(this, this.deployTemplate)
            });
            this.moreMenu.addChild(this.deployItem,1);
            
            this.moreButton = new dijit.form.DropDownButton({
                showLabel: true,
                label: this.resources.ALL_ACTIONS,
                title: this.resources.ALL_ACTIONS_TITLE,
                dropDown: this.moreMenu
            });

            this.detailsToolBar.addChild(this.moreButton);

//            if(dojo.byId("toolbarDiv")){
  //              var toolbarContainer = dojo.byId("toolbarDiv");
    //            this.detailsToolBar.placeAt(toolbarContainer);  
      //      }
            
        },
        // Edit Template
        editTemplate: function() {
        
                var stweditdialog = new support.dialogs.ServerTemplateWizard({
                    templateIDNum: this.templateIDNum,
                    templateInUse: this.templateInUse,
                    editMode: true
                });
                stweditdialog.show();

        },       
        // Deploy Template
        deployTemplate: function() {

            if (this.templateIDNum != null && this.templateIDNum != "") {
                var dst = new support.dialogs.DeployServerTemplate({
                    templateId: this.templateIDNum
                });
                dst.show();
            }

        },
        // Copy Template
        copyTemplate: function() {

            var stweditdialog = new support.dialogs.ServerTemplateWizard({
                templateIDNum: this.templateIDNum,
                copyMode: true
            });
            stweditdialog.show();

        },
        // Rename Template
        renameTemplate: function() {
            
            require([ "support/dialogs/Rename" ], dojo.hitch(this, function() {
                var renamedialog = new support.dialogs.Rename({
                    templateIDNum: this.templateIDNum,
                    launchPoint: "details"
                });
                renamedialog.show();
            }));

        }
    });

});