define([
        "dojo", 
        "dijit", 
        "support/panels/Panel", 
        "dojo/i18n!./nls/SimpleTable", 
        "dojo/text!./templates/SimpleTable.html",
		"dojox/grid/DataGrid", 
		"dijit/form/ValidationTextBox", 
		"dojo/data/ItemFileWriteStore", 
		"dojox/grid/cells/_base", 
		"dijit/form/Button"], 
		function (dojo, dijit, panel, mri, template) {

	dojo.declare("support.panels.SimpleTable", [ support.panels.Panel ], {
	    // templateString: [protected] String
	    // This widget's HTML template, provided.
	    templateString: template,
	        
	    // locResources: [protected] Object
	    // This widget's National Language Source (NLS) object.
	    locResources: mri, 
	    
		widgetsInTemplate : true,
		baseClass: "SimpleTable",
			
		jsonData: null,
		simpleStore: null,
		simpleLayout: null,
		simpleGrid: null,
		
		min: 0, 
		max: null,
		minchars: 0,
		maxchars: null,
		label: null,
		addTitle: null,
		removeTitle: null,
		dragDropTitle: null,
		addPosition: "left", //left or below
		regExp: ".*",
		invalidMessage: null,
		placeHolder: "",
		
		 // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/SimpleTable.css",

		postMixInProperties : function() {
			this._connectDataModels();
			this.inherited(arguments);
		},

		postCreate : function(){	
			this.inherited(arguments);
			
			if(this.addTitle == null)
			    this.addTitle = this.locResources.ADD_TITLE;
			if(this.removeTitle == null)
			    this.removeTitle = this.locResources.REMOVE_TITLE;
			if(this.dragDropTitle == null)
			    this.dragDropTitle = this.locResources.DRAG_DROP_TITLE;
			if(this.invalidMessage == null)
			    this.invalidMessage = this.locResources.INVALID;
			
			this.simpleGrid = new dojox.grid.DataGrid({
        		store: this.simpleStore,
        		structure: this.simpleLayout,
        		autoWidth: false,
        		autoHeight: true,
        		singleClickEdit: true,
        		selectionMode: "single",
        		columnReordering: false,
        		onApplyCellEdit: this._doApplyCellEdit
        	}).placeAt(this._simpleTable);			
			this.simpleGrid.startup();
            this.simpleGrid.store.save();
            this.simpleGrid.render();
        	
        	var addIcon = 
        		"<div id=\"" + this.id + "_addIcon\" " +
        		"class=\"spAddRowIcon_v2\" " +
				"onClick=\"dijit.byId('" + this.id + "')._addRow()\"" +
				"title=\"" + this.addTitle + "\"" +
				"style=\"display:inline-block;\"></div>";
		
        	if(this.addPosition == "below")
        		this.addIconBelow.innerHTML = addIcon;
        	else //left by default
        		this.addIconLeft.innerHTML = addIcon;
       
       
        	this._setIcons();
        	        	
		},
		
		_show : function(){
            if(this.simpleGrid != null){
                this.simpleGrid.store.save();
                this.simpleGrid.render();
            }
        },
        
        _setValueAttr : function(value){
            var items = new Array();
            if(value != null){
                for(var i=0; i<value.length; i++){
                    items.push({field: value[i].value, icons: true});
                }
            }
            this.simpleStore = new dojo.data.ItemFileWriteStore({
                data: { "items" : items }
            });
            this.value = value;
            if(this.simpleGrid != null){
                this.simpleGrid.setStore(this.simpleStore);
                this.simpleGrid.store.save();
                this.simpleGrid.render();
            }
            this._setIcons();
        },
		
        _getValueAttr : function(){
            var value = null;
            var grid = this.simpleGrid;
            if(grid != null){
                var store = grid.store;
                var items = store._arrayOfTopLevelItems;
                value = new Array();
                for(var i=0; i<items.length; i++){
                    value.push({ index:i+1, value:items[i].field[0] });
                }
            }
            return value;
        },
		
		startup: function(){
            this.inherited(arguments);
        },        		
		//Sets the label attribute and shows next to the table
		_setLabelAttr : function(l){
		 	this.label = l;
		    this.labelSpan.innerHTML = this.label;
		},
		
		
	    _doApplyCellEdit : function(){
	    	this.update();
	    },
	    
	    _connectDataModels : function(){
        	this.simpleStore = new dojo.data.ItemFileWriteStore({
        		data: this._getValidJsonData(this.jsonData)
        	});
        	if (this.simpleLayout == null) {
                this.simpleLayout = [ 
                {
                    field: "field",
                    name: " ",
                    width: "75%",
                    editable: true,
                    noresize: true,
                    formatter: this._formatField                   	
                },{
                    field: "icons",
                    name: " ",
                    width: "25%",
                    noresize: true,
                    formatter: this._formatIcons
                }];
            }
        },
        
        //Formats the icons (enabled/disabled)
        _formatIcons : function(value, rowIndex, cell){
        	var st = this.grid.getParent();
        	var store = this.grid.store;
        	var item = store._arrayOfTopLevelItems[rowIndex];
        	var drag = "";
        	var remove = st._getRemoveIcon(store.getValue(item, "icons"), rowIndex);
        	var icons = drag + "<div style=\"width:5px;display:inline-block;\"></div>" + remove;
        	return icons;
        },
        
        //Returns an enabled/disabled remove icon
        _getRemoveIcon : function(enabled, rowIndex){
            if(enabled){
                return "<div class=\"spDeleteIcon_v2\"" +
                "onClick=\"dijit.byId('" + this.id + "')._removeRow('" + rowIndex + "')\"" +
                "title=\"" + this.removeTitle +"\" style=\"display:inline-block;\"></div>";
            }
            return "<div class=\"spDeleteInactiveIcon_v2\" style=\"display:inline-block;\"></div>";
        },
        
        //Format "field": given format
        _formatField : function(value, rowIndex, cell){
            var grid = this.grid;
        	var st = this.grid.getParent();
        	var store = this.grid.store;
        	var item = store._arrayOfTopLevelItems[rowIndex]; 
        	var errorMessage = null;
        	var control = dijit.form.ValidationTextBox({  
        	    id: "support_panels_simpleTable_"+rowIndex,
        		value: value,
        		required: true,
        		trim: true,
        		style:"width:100%",
        		onChange: function(newValue){
        			st._setStoreValue(item, "field", newValue);
        		},
        		//regExp: "^([0-9A-F]{2}[:-]){7}([0-9A-F]{2})$",
    			//invalidMessage: "To boot from 1st discovered target, enter 00:00:00:00:00:00:00:00",
    			placeHolder: st.placeHolder,
    			validator : function(value){
    			    if(value.length == 0){
     			      errorMessage = st.locResources.REQUIRED;
    			      return false;
    			    }
    			    if(st.minchars != null && st.maxchars != null){
                        if(value.length < parseInt(st.minchars) || value.length > parseInt(st.maxchars)){
                            errorMessage = st.locResources["INVALID_BETWEEN"].replace("%1", st.minchars).replace("%2", st.maxchars);
                            return false;
                        }
                    }
    			    if(st.minchars != null && st.maxchars == null){
                        if(value.length < parseInt(st.minchars)){
                            errorMessage = st.locResources["INVALID_GREATER"].replace("%1", st.minchars);
                            return false;
                        }
                    }
			    	errorMessage = st.invalidMessage;
			    	var reg = new RegExp(st.regExp);
			    	return reg.test(value);
    			}
        	});
        	control._destroyOnRemove = true;
        	dojo.connect(grid,"onCellMouseOver", function(e){
        		var controlValue = store.getValue(item, "field");
                if(e.cellIndex == 0 && e.rowIndex == rowIndex && !control.validator(controlValue)){
                    dijit.showTooltip("<div style=\"max-width:200px\">" + errorMessage + "</div>", e.cellNode);
                }
            });
            dojo.connect(grid,"onCellMouseOut", function(e){
                if(e.cellIndex == 0 && e.rowIndex == rowIndex){
                    dijit.hideTooltip(e.cellNode);                
                }
            });
            dojo.connect(grid,"onBlur", function(){
            	dijit.hideTooltip(cell.getNode());                
            });
           //sets value in store as cell is being edited
        	/*dojo.connect(cell, "formatEditing", function(inDatum, inRowIndex){
        		var tableID = this.grid.getParent().id;
        		var gridID = this.grid.id;
				this.needFormatNode(inDatum, inRowIndex);
				return  "<input class=\"dojoxGridInput\" type=\"text\" " +
						"onkeyup=\"dijit.byId('" + tableID + "')._setStoreValue(dijit.byId('" + gridID + "').store._arrayOfTopLevelItems[" + inRowIndex + "] , \"field\", value)\" " +
						"value=\"" + inDatum + "\">";
			});*/
        	return control;
        },
        
        //Determines if a value is a valid field
        _isValidField : function(value){
        	if(value.length == 0)
        		return false;  //required
        	var reg = new RegExp(this.regExp);
        	return reg.test(value);
        },
        
        //Sets the value in the store
        _setStoreValue : function(item, type, value){
            if(item != null){
                this.simpleGrid.store.setValue(item, type, value);
            }
        },
        
        
        //Adds a row to the end of the grid    
        _addRow : function(){
        	var grid = this.simpleGrid;
            var store = grid.store;
        	var topLevelItems = store._arrayOfTopLevelItems;

        	if(topLevelItems.length < this.max){
                grid.store.newItem({field: "", icons: true});
            }
            
        	grid.store.save();
            grid.render();
        	this._setIcons();
        	
        },
        
        //Removes a row from the grid
        _removeRow : function(row){
        	var remove = confirm(this.locResources["DELETE_CONFIRM"].replace("%1", parseInt(row)+1));
        	if(remove){
        		var grid = this.simpleGrid;
        		var item = grid.store._arrayOfTopLevelItems[row];
        		grid.store.deleteItem(item);
                grid.store.save();
                grid.render();
   
        		this._setIcons();
        	}
        },
        
        getStoreAsJson : function(){
        	var grid = this.simpleGrid;
        	var store = grid.store;
        	var topLevelItems = store._arrayOfTopLevelItems;
        	var storeAsJson = 
        		"{\n" +
        		"	\"items\" : [\n";
        	for(var i=0; i<topLevelItems.length; i++){
        		var item = topLevelItems[i];
        		var field = store.getValue(item, "field");
        		storeAsJson +=
        		"		{\n" +
        		"			\"field\": \"" + field + "\"\n" +
        		"		}";
        		if(i != topLevelItems.length-1)
					storeAsJson += ",";
        		storeAsJson += "\n";
			}
        	storeAsJson += 
				"	]\n" +
				"}";
        	return storeAsJson;
        },
        
        _setIcons : function(){
            var max = this.max;
            var min = this.min;
            var grid = this.simpleGrid;
            var store = grid.store;
            var addIcon = document.getElementById(this.id + "_addIcon");
            var st = this;
            var topLevelItems = store._arrayOfTopLevelItems;
            for(var i=0; i<topLevelItems.length; i++){
                var item = topLevelItems[i];
                if(min != null && topLevelItems.length <= min){
                    //disable remove and drag
                    store.setValue(item, "icons", false);
                }
                else if(max != null && topLevelItems.length >= max && addIcon != null){
                    //disable add
                    addIcon.setAttribute("class", "spAddRowInactiveIcon_v2");
                    addIcon.onclick = function(e){};
                    addIcon.setAttribute("title", "");
                }
                else{
                    //enable drag, add, remove
                    store.setValue(item, "icons", true);
                    if(addIcon != null){
                    	addIcon.setAttribute("class", "spAddRowIcon_v2");
                        addIcon.onclick = function(e){
                            st._addRow();
                        };
                        addIcon.setAttribute("title", this.addTitle);
                    }
                }
            }
            grid.store.save();
            grid.render();
        },
        
        _getValidJsonData : function(jsonData){
        	//if any part of the file is not valid, return the default item
        	if(jsonData == null ||
        	   jsonData.items == null ||
        	   jsonData.items[0].field == null ||
        	   jsonData.items[0].icons == null){
        	    return { "items" : [] };
        	}
        	return jsonData;
        },
        
        isValid : function(){
        	var st = this;
        	var grid = this.simpleGrid;
    		var store = grid.store;
        	//var items = store._arrayOfTopLevelItems; 
        	var valid = true;
        	//at least one item
        	//all textboxes valid
        	var gotItems = function(items, request){
        		if(items == null || items.length == 0){}else{
        			for(var i=0; i<items.length; i++){
        				var item = items[i];
        				var field = item.field[0];
        				if( !st._isValidField(field)){
        				    var widget = dijit.byId("support_panels_simpleTable_"+i);
        		            if(widget != null){
                                var declaredClass = widget.declaredClass;
                                if(declaredClass == "dijit.form.ValidationTextBox"){
                                    widget.focus();
                                }
        		            }
        					valid  = false;
        				}
        			}
        		}
        	};
        	store.fetch({onComplete:gotItems});
        	return valid;
        }
	});
});     	