define(["dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/StandbyDetails", 
        "dojo/text!./templates/StandbyDetails.html",
        "support/base/NextGenUIMixin", 
        "support/widgets/OutlinePane",
        "dijit/layout/TabContainer",
        "support/panels/StandbyDetailsDependent",
        "dijit/Toolbar",
        "dijit/ToolbarSeparator",
        "dijit/Menu",
        "dojo/dom"
        ], function(dojo, dijit, panel, mri, template) {

    
    return dojo.declare("support.panels.StandbyDetails", [ support.panels.Panel, support.base.NextGenUIMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in displaying information
        // associated with a server.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "ServerDetails",
        
        // the properties used to construct the title of this page in the
        //   NextGen UI
        cpPageContext: null,
        cpPageName: null,
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/StandbyDetails.css",

        configurationTabdata: "",
        ioAddressesTabdata: "",
        eventsTabdata: "",
        dependenciesTabdata: "",
        
        templateInformation: "",
        
        templateIDNum: "",
        templateName: "",
        templateDesc: "",
        templateType: "",
        
        postMixInProperties: function() {
            this.inherited(arguments);
            this.cpPageContext = "";
            this.cpPageName = this.resources.DETAILS;
        },

        postCreate: function() {
            this.inherited(arguments);
            
            if(this.CP_RUN_ENVIRONMENT == "augusta"){

                //Add page base in order to use the setTitle() function
                require([ "augusta/augustaUICore/core/augustaPageBase" ], dojo.hitch(this, function(PageBase) {                
                    var CP_AUGUSTA_PAGE_BASE = new augusta.augustaUICore.core.augustaPageBase();
                    dojo.safeMixin(this, CP_AUGUSTA_PAGE_BASE);            
                }));

                this.bwIsCacheable = false;
            }    

            this.subscribe("RefreshNameDiv", function(data) {
                // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
                this.cpPageContext = data;

                if(this.CP_RUN_ENVIRONMENT == "fsm"){
                    //Set title in FSM
                    this.changeNextGenUITitle();
                }
                
                if(this.CP_RUN_ENVIRONMENT == "augusta"){
                    //Set Title in xHMC, only can be used if augustaPageBase mixed in
                    this.setTitle(this.cpPageContext + " - " + this.cpPageName);
                }

            });

            this.subscribe("StandbyPoolComplete", function(data) {
                this.getServerTemplateID();
            });
                        
            this.getServerTemplateID();

            // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
            this.cpPageContext = this.templateName;

            if(this.CP_RUN_ENVIRONMENT == "fsm"){
                //Set title in FSM
                this.changeNextGenUITitle();
            }
            
            if(this.CP_RUN_ENVIRONMENT == "augusta"){
                //Set Title in xHMC
                this.setTitle(this.cpPageContext + " - " + this.cpPageName);
            }

            this.toolbarSetup();

            // Pool Memebers Tab
            this.dependenciesTabdata = new support.panels.StandbyDetailsDependent({
                id: this.id + "cp_standbydetails_standbydetailsdependent",
                mode: "standByPoolMembers",
                templateIDNum: this.templateIDNum,
                tableWidth: "99%",
                title:this.locResources.POOLMEM
            }).placeAt(this._dependenciesTab);  

            // Dependencies Tab
            this.dependenciesTabdata = new support.panels.StandbyDetailsDependent({
                id: this.id + "cp_standbydetails_standbydetailsdependent2",
                mode: "standByServerDetails",
                templateIDNum: this.templateIDNum,
                tableWidth: "99%",
                title:this.locResources.SVRTMPL_DEPENDENTS_TAB_TITLE
            }).placeAt(this._dependenciesTab);  
                                   
            // Events Tab
            /*this.eventsTabdata = new support.panels.DetailsDependentTable({
                //mode: "detailsTab", 
                tableWidth: "99%",
                title:this.locResources.EVENTS_TAB_TITLE
            }).placeAt(this._eventsTab);*/

                                                                                                                                                                                             
        },
        //Get Server Template ID
        getServerTemplateID: function(){
            var store ="";
          
            if(this.templateIDNum == ""){  //If this templateID not provided then parse url for it
                var url = window.location.href;
                var splitUrl = url.split("cpTarget=");
                var IDarray = new Array();
                var ID = "";
                
                if(splitUrl[1]){
                    IDarray = splitUrl[1].split("&");
                }else{
                    ID = "";
                }
                
                if(IDarray[0]){
                    ID=IDarray[0];
                    
                    var xhrGet = dojo.xhrGet({
                        url: this.REST_PATH + "pool/standby/" + ID,
                        handleAs: "json",
                        sync: true,
                        preventCache: true,
                        load: function(response) {
                            console.log(response);
                            store = response;
                        }
                    });
                                 
                    this.templateInformation = store;
                    
                    this.templateIDNum = ID;
                    
                }else{  //If no ID found then ...
                                                
                    
                }
    
                                                                 
            }else{ //IF ID passed then use passed ID

                var xhrGet = dojo.xhrGet({
                    url: this.REST_PATH + "pool/standby/" + this.templateIDNum,
                    handleAs: "json",
                    sync: true,
                    preventCache: true,
                    load: function(response) {
          //              console.log(response);
                        store = response;
                    }
                });
                
                this.templateInformation = store;
            }            
            
            
         //Set Template Name and Description and InUse ... etc ....            
            //Check to see if server template info was gained
            if(this.templateInformation != "" ){  
             
                //if(this.templateInformation.items[0].name){
                    this.templateName = this.templateInformation.name;
                //}
                
                //if(this.templateInformation.items[0].description){
                    this.DetailsTemplateDescription.innerHTML = this.templateInformation.description;
                //}
                
                //if(this.templateInformation.items[0].type){
                    this.templateType = this.templateInformation.type;
                    this.DetailsTemplateType.innerHTML = this.resources.STDBY_SERVER_POOL_DESC;
                //}
                    
            }

            
        },
        //ToolBar Setup
        toolbarSetup: function(){
          
            this.moreMenu = new dijit.Menu({          // Add to More/Actions Menu
                style: "display: none;"
            });
      
            this.editButton = new dijit.form.Button({     //Add Edit Button to toolbar
                id: this.id + "cp_standbydetails_editbutton",
                showLabel: false,
                iconClass: "spEditLgIcon",
                disabled: false,
                title: this.locResources.EDIT,
                onClick: dojo.hitch(this, this.editTemplate),
                onKeypress: dojo.hitch(this, this.editTemplate)
            });
            this.detailsToolBar.addChild(this.editButton);
            
        },
        // Edit Template
        editTemplate: function() {
        
            require([ "support/dialogs/StandbyPoolTemplate" ], dojo.hitch(this, function() {
            var editdialog = new support.dialogs.StandbyPoolTemplate({
                templateIDNum: this.templateIDNum,
                launchPoint: "details"
            });
            editdialog.show();
            }));
        },
        // Rename Template
        renameTemplate: function() {
            
            require([ "support/dialogs/Rename" ], dojo.hitch(this, function() {
            var renamedialog = new support.dialogs.Rename({
                id: this.id + "cp_standbydetails_rename",
                templateIDNum: this.templateIDNum
            });
            renamedialog.show();
            }));
        }
    });

});