define(["dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/DetailsDependentTable", 
        "dojo/text!./templates/DetailsDependentTable.html",
        "dojo/data/ItemFileWriteStore",
        "support/base/TableMixin",        
        "support/dialogs/FailoverMonitorTemplate",
        "dijit/Toolbar"
        ], function(dojo, dijit, panel, mri, template) {

    return dojo.declare("support.panels.StandbyDetailsDependent", [ support.panels.Panel , support.base.TableMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in managing server profiles
        // for Configuration Patterns.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
            
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "DetailsDependentTable",
        
        mode: "", // Current Context Mode
        view: null, // Current View, Flat List is by default
        tableWidth: "99%", // Tree Table Width
        _detailsTable: null,
        _displayButton: null,

        actionMenu: null,
        _actionButton: null,
        
        profileLayout: {            /* Table View Options */
            flatView: true, // Flat Table View
            chassisView: true, // By Chassis View
            groupView: true, // By Group View
            templateView: true // By Template View
            /* Toolbar Buttons and Menus */

            /* "More" menu options */

            /* Table Columns To Display */
        },
        
        templateIDNum:"",   //Category Template ID   
        templateType: "",   //Template Type
        serverUUID:"", //Used for Failover
        serverName:"",
        
        // ID prefix used for table
        gridID: "StandbyDetailsDependentTable",
        
        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/DetailsDependentTable.css",

        // When this method is called, all variables
        // inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget
        // property variables defined in the template
        // HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after
        // buildRendering(). This is useful to override
        // when
        // you need to access and/or manipulate DOM
        // nodes included with your widget.
        // DOM nodes and widgets with the
        // dojoAttachPoint attribute specified can now
        // be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in
        // buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
                        
            this._setupLayoutTemplate(this.mode);
                        
            // Setup table widget...
            var tableStore = this._retrieveStore(this.mode);
            var tableStructure = this._getTableStructure(this.mode); // Get structure for current view
            
            this._detailsTable = this.getTable(tableStore, tableStructure, this.resources.NO_DATA, this.gridID, true);
            this._detailsTable.column(0).sort(false);

            setTimeout(dojo.hitch(this, function() {
    	        this._detailsTable.placeAt(this.detailsTable);
	            this._detailsTable.startup();
            }), 500);

            
            if(this.mode == "standByServerDetails"){
                /*this._tableToolbar = new dijit.Toolbar({
                    id: "toolbarDetailsTable_"+this.id
                }, "_tableToolbar").placeAt(this._tableToolbar); // Define toolbar            
                this._tableToolbar.startup();*/            
                this._loadToolBar(this._detailsTable); //load table toolbar
            }
            
            // Connect handlers to reset table capabilities
            dojo.connect(this._detailsTable.select.row, "onSelectionChange", this, this.setTableCapabilities); // Fires on row selection
            dojo.connect(this._detailsTable, "setStore", this, this.setTableCapabilities); // Fires on store reset            
                                               
            this.subscribe("RefreshTable", function(data) {
                this._reloadTable(this.mode);
            });
            
            this.subscribe("StandbyPoolComplete", function(data) {
                this._reloadTable(this.mode);
            });
            
        },
        
        // Connect to data source and create models for
        // TreeTable widgets.
        _retrieveStore: function(mode) {
                                               
                if(mode == "standByServerDetails"){
                    
                   var tableStore = new dojo.data.ItemFileWriteStore({
                        url: this.REST_PATH + "pool/standby/"+this.templateIDNum+"?showDependents=true"
                   });
                    
                }else if(mode == "standByPoolMembers"){
                  
                    var tableStore = new dojo.data.ItemFileWriteStore({
                        url: this.REST_PATH + "pool/standby/"+this.templateIDNum+"?showMembers=true"
                    });

                }
                
                return tableStore;
        },
        
        _reloadTable: function(mode) {

            var tableStore = this._retrieveStore(mode);
            this._detailsTable.setStore(tableStore);
            this._detailsTable.model.clearCache();
            this._detailsTable.body.refresh();
                
        },

        // Load toolbar buttons/menus
        _loadToolBar: function(grid) {
            var toolbar = grid && grid.toolBar && grid.toolBar.widget;
            
            if (toolbar) {
                if (this._detailsTable == null) {
                    // Profile table not created. Nothing to
                    // do but return.
                    return;
                }
                
                if (!this.actionMenu) {
                    this.actionMenu = new dijit.Menu({
                        style: "display: none;"
                    });
                }
    
                //Add action menu to action button and create action button if not already created
                if(this._actionButton) {
                    this._actionButton.dropDown = this.actionMenu;
                }else {
                    this._actionButton = new dijit.form.DropDownButton({
                        id: this.id + "cp_standbydetailsdependent_actionbutton",
                        label: this.resources.ALL_ACTIONS,
                        title: this.resources.ALL_ACTIONS_TITLE,
                        showLabel: true,
                        dropDown: this.actionMenu
                    });   
                    toolbar.addChild(this._actionButton);
                }
            }
        },
        // Re-render Tree table with new columns/data
        // presentation
        _reRenderTable: function() {
            // 99275: Get function from tree table widget
        },

        // Disable or enable options for custom view modes
        _setupLayoutTemplate: function(mode) {
            
            switch (mode) {
                case ("standByServerDetails"):
                    this.profileLayout.serverCol= true; // Server Column
                    this.profileLayout.chassisCol= true; // Chassis Column
                    this.profileLayout.bayCol = true;   //Bay Column
                    this.profileLayout.accessCol=true;  //Access Column
                    this.profileLayout.failoverCol= true; // Failover Monitoring Column
                    this.profileLayout.profileCol= true; // Server Profile Column                    

                    break;

                case ("standByPoolMembers"):
                    this.profileLayout.serverCol= true; // Server Column
                    this.profileLayout.chassisCol= true; // Chassis Column
                    this.profileLayout.bayCol = true;   //Bay Column
                    this.profileLayout.accessCol=true;  //Access Column
                    
                    break;

                default:
                    break;
            }
        },
        
        // Columns to be displayed on table
        _getTableStructure: function(mode) {
            var cols = new Array();

            if(mode == "standByServerDetails"){

                if(this.profileLayout.serverCol){

                    cols.push({
                        field: "name",
                        name: this.resources.SERVER,
                        width: "20em",
                        title: ""});

                } 

                if(this.profileLayout.chassisCol){

                    cols.push({
                        field: "chassis",
                        name: this.resources.CHASSIS,
                        width: "20em",
                        title: ""});

                } 

                if(this.profileLayout.bayCol){

                    cols.push({
                        field: "bay",
                        name: this.resources.BAY,
                        width: "8em",
                        title: ""});

                } 

                if(this.profileLayout.accessCol){

                    cols.push({               
                        field: "access",
                        name: this.resources.ACCESS,
                        width: "10em",
                        title: ""});

                } 

                if(this.profileLayout.failoverCol){

                    cols.push({
                        field: "failoverMonitoring",
                        name: this.resources.FAILOVER_MONITORING,
                        width: "18em",
                        title: ""});

                }

                if(this.profileLayout.profileCol){

                    cols.push({
                        field: "profile",
                        name: this.resources.PROF,
                        title: ""});

                }

            }else if(mode == "standByPoolMembers"){

                if(this.profileLayout.serverCol){

                    cols.push({
                        field: "name",
                        name: this.resources.SERVER,
                        width: "20em",
                        title: ""});

                } 

                if(this.profileLayout.chassisCol){

                    cols.push({
                        field: "chassis",
                        name: this.resources.CHASSIS,
                        width: "40em",
                        title: ""});

                } 

                if(this.profileLayout.bayCol){

                    cols.push({
                        field: "bay",
                        name: this.resources.BAY,
                        width: "8em",
                        title: ""});

                } 

                if(this.profileLayout.accessCol){

                    cols.push({               
                        field: "access",
                        name: this.resources.ACCESS,
                        title: ""});

                } 


            }

            return cols;
        },
        // Get number of selected rows in tree table
        _getSelectedRowCount: function() {
            return this._detailsTable.select.row.getSelected().length;
        },

        // Get row objects for selected rows
        _getSelectedRows: function() {
            return this._detailsTable.select.row.getSelected();
        },

        // Set Table Capabilities
        setTableCapabilities: function() {
            
            if (!this._detailsTable) {
                return; // nothing to do. return now
            }

            if(this.mode != "standByServerDetails"){
                return; // nothing to do. return now                
            }
                       
             var srCount = this._getSelectedRowCount();
             // console.log(srCount);
             var ids = this._getSelectedRows();
            // console.log(si);
         
             var failoverstatus = "";

             for (var loopVar = 0; loopVar < ids.length; ++loopVar) {

                 var item = this._detailsTable.row(ids[loopVar], true).item();

                 this.serverUUID = item.id;
                 this.serverName = item.name;
                 if (failoverstatus == "" || failoverstatus[0] == item.failoverMonitoring[0]) {
                     failoverstatus = item.failoverMonitoring;
                 } else {
                     failoverstatus = "MISMATCH";
                 }
                        
             }
     
                if (!this.stopfailover) {

                    this.stopfailover = new dijit.MenuItem({
                //        id: this.id + "cp_standbydetailsdependent_stopfailover",
                        label: this.resources.STOP_FAILOVER_MONITORING,
                        title: this.resources.STOP_FAILOVER_MONITORING_TITLE,
                        onClick: dojo.hitch(this, this.stopfailoverFunc),
                        onKeypress: dojo.hitch(this, this.stopfailoverFunc)
                    });

                }

                if (!this.restartfailover) {

                    this.restartfailover = new dijit.MenuItem({
                //        id: this.id + "cp_standbydetailsdependent_restartfailover",
                        label: this.resources.RESTART_FAILOVER_MONITORING,
                        title: this.resources.RESTART_FAILOVER_MONITORING_TITLE,
                        onClick: dojo.hitch(this, this.restartfailoverFunc),
                        onKeypress: dojo.hitch(this, this.restartfailoverFunc)
                    });
                
                }
                
                if (!this.editfailover) {

                    this.editfailover = new dijit.MenuItem({
                  //      id: this.id + "cp_standbydetailsdependent_editfailover",
                        label: this.resources.EDIT_FAILOVER_MONITORING,
                        title: this.resources.EDIT_FAILOVER_MONITORING_TITLE,
                        onClick: dojo.hitch(this, this.editfailoverFunc),
                        onKeypress: dojo.hitch(this, this.editfailoverFunc)
                    });

                }
                
                if (!this.failoverItem) {

                    this.failoverItem = new dijit.MenuItem({
                   //     id: this.id + "cp_standbydetailsdependent_failoveritem",
                        label: this.resources.FAILOVER_TO,
                        title: this.resources.FAILOVER_TO_TITLE,
                        onClick: dojo.hitch(this, this.failover),
                        onKeypress: dojo.hitch(this, this.failover)
                    });

                }
                
                if (!this.removefailoverItem) {

                    this.removefailoverItem = new dijit.MenuItem({
                  //      id: this.id + "cp_standbydetailsdependent_removefailoveritem",
                        label: this.resources.REMOVE_FAILOVER_MONITORING,
                        title: this.resources.REMOVE_FAILOVER_MONITORING_TITLE,
                        onClick: dojo.hitch(this, this.removefailoverFunc),
                        onKeypress: dojo.hitch(this, this.removefailoverFunc)
                    });

                }
                                    
                if(failoverstatus == "ACTIVE"){

                    //this.actionMenu.addChild(this.startfailover);
                    this.actionMenu.addChild(this.stopfailover);
                    this.actionMenu.removeChild(this.restartfailover);
                    this.restartfailover = null;

                }else if(failoverstatus == "STOPPED"){

                    this.actionMenu.addChild(this.restartfailover);
                    this.actionMenu.removeChild(this.stopfailover);
                    this.stopfailover = null;

                }else if(failoverstatus == "STARTING"){

                }else if(failoverstatus == "FAILED"){

                    this.actionMenu.addChild(this.restartfailover);
                    this.actionMenu.removeChild(this.stopfailover);
                    this.stopfailover = null;

                }else if(failoverstatus == "NOTSTARTED"){

                    this.actionMenu.addChild(this.restartfailover);
                    this.actionMenu.removeChild(this.stopfailover);
                    this.stopfailover = null;

                }

                if ((srCount === 1)) {
                    this.actionMenu.addChild(this.failoverItem);
                    this.actionMenu.addChild(this.editfailover);
                }
                // Multiple servers selected, mismatched failover status
                // Remove failover monitoring is the only valid option
                else if (failoverstatus == "MISMATCH") {
                    this.actionMenu.removeChild(this.failoverItem);
                    this.actionMenu.removeChild(this.editfailover);
                    if (this.stopfailover) {
                        this.actionMenu.removeChild(this.stopfailover);
                        this.stopfailover = null;
                    }
                    if (this.restartfailover) {
                        this.actionMenu.removeChild(this.restartfailover);
                        this.restartfailover = null;
                    }
                    this.failoverItem = null;
                    this.editfailover = null;                
                }else{
                    this.actionMenu.removeChild(this.failoverItem);
                    this.actionMenu.removeChild(this.editfailover);
                    this.failoverItem = null;
                    this.editfailover = null;
                }   
                    
                this.actionMenu.addChild(this.removefailoverItem);
                
                // Clear all actions if there are no selected rows
                if (srCount == 0) {
                    this.actionMenu.removeChild(this.removefailoverItem);
                    this.removefailoverItem = null;
                    if (this.stopfailover) {
                        this.actionMenu.removeChild(this.stopfailover);
                        this.stopfailover = null;
                    }
                    if (this.restartfailover) {
                        this.actionMenu.removeChild(this.restartfailover);
                        this.restartfailover = null;
                    }
                }

            //Disable Action Button if no children
            if(this.actionMenu.hasChildren()){
                this._actionButton.set("disabled", false);
            }else{
                this._actionButton.set("disabled", true);                
            }

            
        },

        editfailoverFunc: function(){
            
            dojo.xhrGet({
                url: this.REST_PATH + "server/" + this.serverUUID + "?showFailover=true",                        
                handleAs: "json",
                sync: true,
                preventCache: true,
                load: dojo.hitch(this,function(response) {
                    this.policyId = response.failoverPolicyId;
                    this.poolId = response.standbyPoolId;
                    this.serverFormFactor = response.formFactor;
                })
            });

            var dialog = new support.dialogs.FailoverMonitorTemplate({
                templateIDNum: "1",
                serverUUID: this.serverUUID,
                newPolicyId: this.policyId,
                newPoolId: this.poolId, 
                newFormFactor: this.serverFormFactor
            });
            dialog.show();
            
        },
        removefailoverFunc: function(){

            var ids = this._getSelectedRows();
            
            var confirmedRemove = function() {
                
                var jsonString = dojo.toJson({"failover":"remove"});

                for (var loopVar = 0; loopVar < ids.length; ++loopVar) {

                    var item = this._detailsTable.row(ids[loopVar], true).item();

                    var xhrArgs = {
                            url:  this.REST_PATH +"server/"+item.id,
                            handleAs: "json",
                            sync: true,
                            putData: jsonString,
                            headers: {
                                "Content-Type": "application/json; charset=utf-8"
                            },
                            load : function(data) {
                            },
                            error : function(error) {
                            }
                         };
                         dojo.xhrPut(xhrArgs);

                }

                dojo.publish("RefreshTable");
            };

            var tempName;
            var tempNameDisplay ="";

            for (var loopVar = 0; loopVar < ids.length; ++loopVar) {
                var item = this._detailsTable.row(ids[loopVar], true).item();
                tempName = item.name;
                tempNameDisplay = tempName.toString();
                tempNameDisplay = "  " + tempNameDisplay;
            }

            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.FAILOVER_MONITORING_REMOVE_ACTION,
                        type: "Warning",
                        text: this.resources.FAILOVER_MONITORING_REMOVE_CONFIRMATION_MESSAGE,
                        info: this.resources.FAILOVER_MONITORING_REMOVE_DETAILED_MESSAGE.replace("%1",tempNameDisplay)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedRemove, this);
            
                }));
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){
                
                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.FAILOVER_MONITORING_REMOVE_ACTION,
                        type: "Warning",
                        text: this.resources.FAILOVER_MONITORING_REMOVE_CONFIRMATION_MESSAGE,
                        info: this.resources.FAILOVER_MONITORING_REMOVE_DETAILED_MESSAGE.replace("%1",tempNameDisplay)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedRemove, this);
            
                }));

            }
        },
        restartfailoverFunc: function(){

            var ids = this._getSelectedRows();
            
            var confirmedRestart = function() {
                
                var jsonString = dojo.toJson({"failover":"restart"});

                for (var loopVar = 0; loopVar < ids.length; ++loopVar) {

                    var item = this._detailsTable.row(ids[loopVar], true).item();

                        var xhrArgs = {
                                url:  this.REST_PATH +"server/"+item.id,
                                handleAs: "json",
                                sync: true,
                                putData: jsonString,
                                headers: {
                                     "Content-Type": "application/json; charset=utf-8"
                                },
                                load : function(data) {
                                },
                                error : function(error) {
                                }
                             };
                             dojo.xhrPut(xhrArgs);                                                

                }
                
                dojo.publish("RefreshTable");

            };
                        
            var tempName;
            var tempNameDisplay = "";

            for (var loopVar = 0; loopVar < ids.length; ++loopVar) {
                var item = this._detailsTable.row(ids[loopVar], true).item();
                tempName = item.name;
                tempNameDisplay = tempName.toString();
                tempNameDisplay = "  " + tempNameDisplay;
            }

            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.FAILOVER_MONITORING_RESTART_ACTION,
                        type: "Warning",
                        text: this.resources.FAILOVER_MONITORING_RESTART_CONFIRMATION_MESSAGE,
                        info: this.resources.FAILOVER_MONITORING_RESTART_DETAILED_MESSAGE.replace("%1",tempNameDisplay)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedRestart, this);
            
                }));
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.FAILOVER_MONITORING_RESTART_ACTION,
                        type: "Warning",
                        text: this.resources.FAILOVER_MONITORING_RESTART_CONFIRMATION_MESSAGE,
                        info: this.resources.FAILOVER_MONITORING_RESTART_DETAILED_MESSAGE.replace("%1",tempNameDisplay)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedRestart, this);
            
                }));

            }
        },
        failover: function(){
            var confirmedFailover = function() {
                
                var jsonString = dojo.toJson({"failover":"failover"});

                var xhrArgs = {
                        url:  this.REST_PATH +"server/"+this.serverUUID,
                        handleAs: "json",
                        putData: jsonString,
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        load : dojo.hitch(this, function(data) {
                            this.getModalDialog("Success", this.resources.FAILOVER_COMPLETE_MSG).show();
                        }),
                        error : dojo.hitch(this, function(error) {
                            this.getModalDialog("Error", this.resources.FAILOVER_ERROR_MSG).show();
                        })
                     };
                     dojo.xhrPut(xhrArgs);
                     this.getModalDialog("Success", this.resources.FAILOVER_SUCCESS_MSG).show();
                     dojo.publish("RefreshTable");

            };
            
            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.FAILOVER_MONITORING_FAILOVER_ACTION,
                        type: "Warning",
                        text: this.resources.FAILOVER_MONITORING_FAILOVER_CONFIRMATION_MESSAGE.replace("%1",this.serverName),
                        info: this.resources.FAILOVER_MONITORING_FAILOVER_DETAILED_MESSAGE.replace("%1", this.serverName).replace("%2", this.serverName).replace("%3", this.serverName).replace("%4", this.serverName).replace("%5", this.serverName).replace("%6", this.serverName).replace("%7", this.serverName).replace("%8", this.PRODUCT_NAME_LONG)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedFailover, this);
            
                }));
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.FAILOVER_MONITORING_FAILOVER_ACTION,
                        type: "Warning",
                        text: this.resources.FAILOVER_MONITORING_FAILOVER_CONFIRMATION_MESSAGE.replace("%1",this.serverName),
                        info: this.resources.FAILOVER_MONITORING_FAILOVER_DETAILED_MESSAGE.replace("%1", this.serverName).replace("%2", this.serverName).replace("%3", this.serverName).replace("%4", this.serverName).replace("%5", this.serverName).replace("%6", this.serverName).replace("%7", this.serverName).replace("%8", this.PRODUCT_NAME_LONG)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedFailover, this);
            
                }));

                
            }
        },
        stopfailoverFunc: function(){
            var ids = this._getSelectedRows();

            var confirmedStop = function() {

                var jsonString = dojo.toJson({"failover":"stop"});
                
                for (var loopVar = 0; loopVar < ids.length; ++loopVar) {

                    var item = this._detailsTable.row(ids[loopVar], true).item();

                        var xhrArgs = {
                                url:  this.REST_PATH +"server/"+item.id,
                                handleAs: "json",
                                sync: true,
                                putData: jsonString,
                                headers: {
                                    "Content-Type": "application/json; charset=utf-8"
                                },
                                load : function(data) {
                                },
                                error : function(error) {
                                }
                        };
                        dojo.xhrPut(xhrArgs);
                }

                dojo.publish("RefreshTable");
                
            };

            var tempName;
            var tempNameDisplay = "";

            for (var loopVar = 0; loopVar < ids.length; ++loopVar) {
                var item = this._detailsTable.row(ids[loopVar], true).item();
                tempName = item.name;
                tempNameDisplay = tempName.toString();
                tempNameDisplay = "  " + tempNameDisplay;
            }

            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.FAILOVER_MONITORING_STOP_ACTION,
                        type: "Warning",
                        text: this.resources.FAILOVER_MONITORING_STOP_CONFIRMATION_MESSAGE,
                        info: this.resources.FAILOVER_MONITORING_STOP_DETAILED_MESSAGE.replace("%1",tempNameDisplay)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedStop, this);
            
                }));
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.FAILOVER_MONITORING_STOP_ACTION,
                        type: "Warning",
                        text: this.resources.FAILOVER_MONITORING_STOP_CONFIRMATION_MESSAGE,
                        info: this.resources.FAILOVER_MONITORING_STOP_DETAILED_MESSAGE.replace("%1",tempNameDisplay)
                    });

                    confDialog.startup();
                    confDialog.confirm(confirmedStop, this);
            
                }));

            }
        }
    });
});