define([ 
        "dojo", 
        "dijit", 
        "support/ConfigPatternsAll", 
        "support/panels/Panel",
        "dojo/i18n!./nls/StandbyListTable", 
        "dojo/text!./templates/StandbyListTable.html",
        "support/base/NextGenUIMixin", 
        "support/base/TableMixin",
        "support/dialogs/ConfigPatterns", 
        "dojo/data/ItemFileWriteStore",
        "dijit/form/DropDownButton", 
        "dijit/form/Button",
        "dijit/Menu", 
        "dijit/MenuItem",
        "dijit/MenuSeparator",
        "dijit/Toolbar",
        "dijit/ToolbarSeparator",
        "dojo/string", 
        "dijit/ProgressBar", 
        "support/dialogs/Rename",
        "support/dialogs/StandbyPoolTemplate"
    ], function(dojo, dijit, layer, panel, mri, template) {

    return dojo.declare("support.panels.StandbyListTable", [ support.panels.Panel, support.base.NextGenUIMixin, support.base.TableMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in managing address pools
        // for Configuration Patterns.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "StandbyListTable",

        _standbyTable: null,
        _tableToolbar: null,
        
        editButton: null,
        deleteButton: null,
            
        refreshRoot: null,
        
        actionMenu: null,
        _actionButton: null,

        // the properties used to construct the title of this page in the
        //   NextGen UI
        cpProductName: null,
        cpPageContext: null,
        cpPageName: null,
        
        // ID prefix used for table
        gridID: "StandbyListTable",

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/StandbyListTable.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
            this.cpProductName = this.resources.PRODUCT_NAME;
            // this.cpPageContext = this.resources.SERVERS; Must remain null (unspecified), per XB216902
            this.cpPageName = this.locResources.PAGE_NAME;
        },

        // postCreate() is called after
        // buildRendering(). This is useful to override
        // when
        // you need to access and/or manipulate DOM
        // nodes included with your widget.
        // DOM nodes and widgets with the
        // dojoAttachPoint attribute specified can now
        // be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in
        // buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            
            // Apply env specific tooltip for the page
            var standbyTip = new Array();
            standbyTip.push("<div style=\"width: 30em;\">");
            standbyTip.push(this.locResources.TOOLTIP_TEXT1);
            standbyTip.push("<br><br>" + this.locResources.TOOLTIP_TEXT2);
            standbyTip.push("<br><br>" + this.locResources.TOOLTIP_TEXT3);
            standbyTip.push("<br><br>" + this.locResources.TOOLTIP_TEXT4.replace("%1", this.PRODUCT_NAME_LONG));
            standbyTip.push("</div>");
            // NOTE: For dijit/Tooltip dijits, set the label, not innerHTML.
            this._standbyTooltip.label = standbyTip.join("");

            // Setup table widget...
            var poolStore = this._retrieveStore();
            var tableStructure = this._getTableStructure();            
            this._standbyTable = this.getTable(poolStore, tableStructure, this.resources.NO_DATA, this.gridID);
            setTimeout(dojo.hitch(this, function() {
                this._standbyTable.placeAt(this.standbyTable);
                this._standbyTable.startup();
            }), 500);                                   
            
            // Connect handlers to reset table capabilities
            dojo.connect(this._standbyTable.select.row, "onSelectionChange", this, this.setTableCapabilities); // Fires on row selection
            dojo.connect(this._standbyTable, "setStore", this, this.setTableCapabilities); // Fires on store reset                      

            // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
            this.changeNextGenUITitle();

            // reload contents of templates table; by default, switching
            //   between links in the details selector does not reload page
            this.subscribeToDetailsSelectorChanges(dojo.hitch(this, function() {
                this._refreshTree();
                this.changeNextGenUITitle();
            }));
            
            this._loadToolBar(this._standbyTable); // load table toolbar

            // Register subscribers to topics we care about ...
            this.subscribe("StandbyPoolComplete", function(data) {
                this._refreshTree();
            });
            
            this.subscribe("RefreshTable", function(data) {
                this._refreshTree();
            });

        },
        
        _refreshTree: function() {
            // tags:
            // protected
            // Refresh tree data
            var poolStore = this._retrieveStore();
            this._standbyTable.setStore(poolStore);
            this._standbyTable.model.clearCache();
            this._standbyTable.body.refresh();
        },
        
        // Connect to data source and create store
        _retrieveStore: function() {
            var standbyData = "";
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "pool/standby?table=true",
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: function(response) {
                   standbyData = dojo.toJson({
                        "identifier": "id",
                        "label": "name",
                        "items": response.items
                    }, true);
                },
                handle: function() {

                }
            });

            var poolStore = new dojo.data.ItemFileWriteStore({
                data: dojo.fromJson(standbyData)
            });            
            
            return poolStore;
        },
        
        // Load toolbar buttons/menus
        _loadToolBar: function(grid) {
            
            var toolbar = grid && grid.toolBar && grid.toolBar.widget;
            
            if (toolbar) {
                if (this._standbyTable == null) {
                    // Standby pool table not created. Nothing to
                    // do but return.
                    return;
                }
                
                if (!this.actionMenu) {
                    this.actionMenu = new dijit.Menu({
                        style: "display: none;"
                    });
                }
                
                // Add Create New Address Pool Button
                this.createButton = new dijit.form.Button({
                    //id: this.id + "cp_standbylisttable_createbutton",
                    showLabel: false,
                    iconClass: "spCreateLgIcon",
                    title: this.locResources.NEW_TEMP,
                    onClick: dojo.hitch(this, this.createStandbyPool),
                    onKeypress: dojo.hitch(this, this.createStandbyPool)    
                });
                toolbar.addChild(this.createButton);
    
                // Add custom Edit button...
                this.editButton = new dijit.form.Button({
                    //id: this.id + "cp_standbylisttable_editbutton",
                    showLabel: false,
                    iconClass: "spEditInactiveLgIcon",
                    disabled: true,
                    title: this.locResources.EDIT_SEL,
                    onClick: dojo.hitch(this, this.editStandbyPool),
                    onKeypress: dojo.hitch(this, this.editStandbyPool)
                });
                toolbar.addChild(this.editButton);
    
                // Add custom Delete button...
                this.deleteButton = new dijit.form.Button({
                    //id: this.id + "cp_standbylisttable_deletebutton",
                    showLabel: false,
                    iconClass: "spDeleteInactiveLgIcon",
                    disabled: true,
                    title: this.locResources.DELETE_SEL,
                    onClick: dojo.hitch(this, this.deleteStandbyPools),
                    onKeypress: dojo.hitch(this, this.deleteStandbyPools)
                });
                toolbar.addChild(this.deleteButton);
                
                toolbar.addChild(new dijit.ToolbarSeparator());
                
                this.createItem = new dijit.MenuItem({
                    label: this.resources.CREATE,
                    title: this.locResources.NEW_TEMP,
                    iconClass: "spCreateLgIcon",
                    onClick: dojo.hitch(this, this.createStandbyPool),
                    onKeypress: dojo.hitch(this, this.createStandbyPool)
                });
                this.actionMenu.addChild(this.createItem,0);
                
                var menuSeparator = new dijit.MenuSeparator();
                this.actionMenu.addChild(menuSeparator,1);    
                                
                // Add action menu to action button and create action button if not already created
                if (this._actionButton) {
                    this._actionButton.dropDown = this.actionMenu;
                }
                else {
                    this._actionButton = new dijit.form.DropDownButton({
                        //id: this.id + "cp_standbylisttable_actionbutton",
                        label: this.resources.ALL_ACTIONS,
                        title: this.resources.ALL_ACTIONS_TITLE,
                        showLabel: true,
                        dropDown: this.actionMenu
                    });   
                    
                    toolbar.addChild(this._actionButton);
                }
                
                // Disable Action Button if no children
                if(this.actionMenu.hasChildren()){
                    this._actionButton.set("disabled", false);
                }else{
                    this._actionButton.set("disabled", true);                
                }
            }
        },
        
        showDialog: function() {
            var currentURL = window.location.href;
            
            // if this widget was launched by one of the Configuration
            //   Patterns Java ITasks, it will contain a special URL
            //   parameter; if this parameter is present, we know we must
            //   launch a dialog
            if(currentURL.indexOf("cpParm") != -1 ) {
                var epProps = this.getEndPointPropertiesFromSourceToken();
                if (epProps == null || epProps == "") {
                    console.error("Could not retrieve endpoint properties for server or chassis.");
                    return;
                }
                
                var serverUUID = epProps.uuid;
                var serverName = epProps.name;
                
                // do not proceed if serverUUID is null
                if (serverUUID != null )
                {
                    if(currentURL.indexOf("startFMTask") != -1 && currentURL.indexOf("restartFMTask") == -1) {
                        // show the Failover Monitor Template dialog, but nest
                        //   these calls in a try/catch; if there is an error
                        //   instantiating the dialog, we still want this panel to
                        //   be displayed
                        var serverStr = " ";
                        try {
                            var startFMdialog = null;
                            if(serverUUID != null) {
                                startFMdialog = new support.dialogs.FailoverMonitorTemplate({
                                    serverUUID:serverUUID
                                });
                                serverStr = " (for server with UUID: " + serverUUID + ") ";
                            }
                            else {
                                startFMdialog = new support.dialogs.FailoverMonitorTemplate();
                            }
                            startFMdialog.show();
                            // no need to dismiss or delete this dialog; the
                            //   Failover Monitor Template dialog does this when
                            //   the user clicks OK or Cancel
                            console.log("Showing (Start) Failover Monitor Template dialog" + serverStr + "from Standby Server Pools page");
                        }
                        catch(err) {
                            console.error("Failed to show (Start) Failover Monitor Template dialog" + serverStr + "from Standby Server Pools page; error: " + err.message);
                        }
                    }
                    else if(currentURL.indexOf("stopFMTask") != -1) {
                        // show a confirmation dialog asking the user whether they
                        //   wish to stop failover monitoring for the server, but
                        //   nest these calls in a try/catch; if there is an error
                        //   instantiating the dialog, we still want this panel to
                        //   be displayed
                        try {
                            var stopFailoverAction = function() {
                                this.executeFailoverOperation("stop", serverUUID);
                            };
                            
                            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                            
                                    var prompt = new support.temp.ConfirmationDialog({
                                        buttonLabel: this.resources.FAILOVER_MONITORING_STOP_ACTION,
                                        type: "Warning",
                                        text: this.resources.FAILOVER_MONITORING_STOP_CONFIRMATION_MESSAGE,
                                        info: this.resources.FAILOVER_MONITORING_STOP_DETAILED_MESSAGE.replace("%1", serverName)
                                    });
                                    
                                    prompt.startup();
                                    prompt.confirm(stopFailoverAction, this); 
                                    console.log("Showing Stop Failover Monitoring confirmation dialog from Standby Server Pools page");
                            
                                }));
                                                                
                            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                                    
                                    var prompt = new idx.widget.ConfirmationDialog({
                                        buttonLabel: this.resources.FAILOVER_MONITORING_STOP_ACTION,
                                        type: "Warning",
                                        text: this.resources.FAILOVER_MONITORING_STOP_CONFIRMATION_MESSAGE,
                                        info: this.resources.FAILOVER_MONITORING_STOP_DETAILED_MESSAGE.replace("%1", serverName)
                                    });
                                    
                                    prompt.startup();
                                    prompt.confirm(stopFailoverAction, this); 
                                    console.log("Showing Stop Failover Monitoring confirmation dialog from Standby Server Pools page");
                            
                                }));

                            }
                        }
                        catch(err) {
                            console.error("Failed to show Stop Failover Monitoring confirmation dialog from Standby Server Pools page; error: " + err.message);
                        }
                    }
                    else if(currentURL.indexOf("restartFMTask") != -1) {
                        // show a confirmation dialog asking the user whether they
                        //   wish to restart failover monitoring for the server,
                        //   but nest these calls in a try/catch; if there is an
                        //   error instantiating the dialog, we still want this
                        //   panel to be displayed
                        try {
                            var restartFailoverAction = function() {
                                this.executeFailoverOperation("restart", serverUUID);
                            };
                            
                            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                                    var prompt = new support.temp.ConfirmationDialog({
                                        buttonLabel: this.resources.FAILOVER_MONITORING_RESTART_ACTION,
                                        type: "Warning",
                                        text: this.resources.FAILOVER_MONITORING_RESTART_CONFIRMATION_MESSAGE,
                                        info: this.resources.FAILOVER_MONITORING_RESTART_DETAILED_MESSAGE.replace("%1", serverName)
                                    });
                                    
                                    prompt.startup();
                                    prompt.confirm(restartFailoverAction, this); 
                                    console.log("Showing Restart Failover Monitoring confirmation dialog from Standby Server Pools page");
                            
                                }));
                            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){
                                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                                    var prompt = new idx.widget.ConfirmationDialog({
                                        buttonLabel: this.resources.FAILOVER_MONITORING_RESTART_ACTION,
                                        type: "Warning",
                                        text: this.resources.FAILOVER_MONITORING_RESTART_CONFIRMATION_MESSAGE,
                                        info: this.resources.FAILOVER_MONITORING_RESTART_DETAILED_MESSAGE.replace("%1", serverName)
                                    });
                                    
                                    prompt.startup();
                                    prompt.confirm(restartFailoverAction, this); 
                                    console.log("Showing Restart Failover Monitoring confirmation dialog from Standby Server Pools page");
                            
                                }));
                                
                            }
                                   
                        }
                        catch(err) {
                            console.error("Failed to show Restart Failover Monitoring confirmation dialog from Standby Server Pools page; error: " + err.message);
                        }
                    }
                    else if(currentURL.indexOf("removeFMTask") != -1) {
                        // show a confirmation dialog asking the user whether they
                        //   wish to remove failover monitoring for the server,
                        //   but nest these calls in a try/catch; if there is an
                        //   error instantiating the dialog, we still want this
                        //   panel to be displayed
                        try {
                            var removeFailoverAction = function() {
                                this.executeFailoverOperation("remove", serverUUID);
                            };
                            
                            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                                    var prompt = new support.temp.ConfirmationDialog({
                                        buttonLabel: this.resources.FAILOVER_MONITORING_REMOVE_ACTION,
                                        type: "Warning",
                                        text: this.resources.FAILOVER_MONITORING_REMOVE_CONFIRMATION_MESSAGE,
                                        info: this.resources.FAILOVER_MONITORING_REMOVE_DETAILED_MESSAGE.replace("%1", serverName)
                                    });
                                    
                                    prompt.startup();
                                    prompt.confirm(removeFailoverAction, this); 
                                    console.log("Showing Remove Failover Monitoring confirmation dialog from Standby Server Pools page");
                                }));
                            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){
                                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                                    var prompt = new idx.widget.ConfirmationDialog({
                                        buttonLabel: this.resources.FAILOVER_MONITORING_REMOVE_ACTION,
                                        type: "Warning",
                                        text: this.resources.FAILOVER_MONITORING_REMOVE_CONFIRMATION_MESSAGE,
                                        info: this.resources.FAILOVER_MONITORING_REMOVE_DETAILED_MESSAGE.replace("%1", serverName)
                                    });
                                    
                                    prompt.startup();
                                    prompt.confirm(removeFailoverAction, this); 
                                    console.log("Showing Remove Failover Monitoring confirmation dialog from Standby Server Pools page");
                                }));
                                
                            }
                        }
                        catch(err) {
                            console.error("Failed to show Remove Failover Monitoring confirmation dialog from Standby Server Pools page; error: " + err.message);
                        }
                    }
                    else if(currentURL.indexOf("ftsspTask") != -1) {
                        // show a confirmation dialog asking the user whether they
                        //   wish to failover from the server to the standby
                        //   server pool, but nest these calls in a try/catch; if
                        //   there is an error instantiating the dialog, we still
                        //   want this panel to be displayed
                        try {
                            var failoverToStandbyServerPoolAction = function() {
                                this.executeFailoverOperation("failover", serverUUID);
                            };
                            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                                    var prompt = new support.temp.ConfirmationDialog({
                                        buttonLabel: this.resources.FAILOVER_MONITORING_FAILOVER_ACTION,
                                        type: "Warning",
                                        text: this.resources.FAILOVER_MONITORING_FAILOVER_CONFIRMATION_MESSAGE.replace("%1", serverName),
                                        info: this.resources.FAILOVER_MONITORING_FAILOVER_DETAILED_MESSAGE.replace("%1", this.serverName).replace("%2", this.serverName).replace("%3", this.serverName).replace("%4", this.serverName).replace("%5", this.serverName).replace("%6", this.serverName).replace("%7", this.serverName).replace("%8", this.PRODUCT_NAME_LONG)
                                    });
                                    
                                    prompt.startup();
                                    prompt.confirm(failoverToStandbyServerPoolAction, this); 
                                    console.log("Showing Failover to Standby Server Pool confirmation dialog from Standby Server Pools page");
                                }));
                            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                                    var prompt = new idx.widget.ConfirmationDialog({
                                        buttonLabel: this.resources.FAILOVER_MONITORING_FAILOVER_ACTION,
                                        type: "Warning",
                                        text: this.resources.FAILOVER_MONITORING_FAILOVER_CONFIRMATION_MESSAGE.replace("%1", serverName),
                                        info: this.resources.FAILOVER_MONITORING_FAILOVER_DETAILED_MESSAGE.replace("%1", this.serverName).replace("%2", this.serverName).replace("%3", this.serverName).replace("%4", this.serverName).replace("%5", this.serverName).replace("%6", this.serverName).replace("%7", this.serverName).replace("%8", this.PRODUCT_NAME_LONG)
                                    });
                                    
                                    prompt.startup();
                                    prompt.confirm(failoverToStandbyServerPoolAction, this); 
                                    console.log("Showing Failover to Standby Server Pool confirmation dialog from Standby Server Pools page");
                                }));

                            }
                        }
                        catch(err) {
                            console.error("Failed to show Failover to Standby Server Pool confirmation dialog from Standby Server Pools page; error: " + err.message);
                        }
                    }
                
                }
                else
                {
                    console.error("Failed to retrieve uuid for target oid");
                }
            }
        },
        
        executeFailoverOperation: function(operation, serverUUID) {
            var jsonString = dojo.toJson({"failover":operation});
            var xhrArgs = {
                url:  this.REST_PATH +"server/"+serverUUID,
                handleAs: "json",
                sync: true,
                putData: jsonString,
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                load : function(data) {
                    console.log("Successfully submitted failover operation: " + operation);
                },
                error : function(error) {
                    console.error("Failed to execute failover operation: " + operation + "; error: " + err.message);
                }
            };
            dojo.xhrPut(xhrArgs);
        },

        createStandbyPool: function() {
            var createPooldialog = new support.dialogs.StandbyPoolTemplate();
            createPooldialog.show();
            
            this._refreshTree();
            return false;
        },

        detailStandbyPool: function() {
            
          require([ "support/panels/StandbyDetails" ], dojo.hitch(this,function() {
              
              if(this.CP_RUN_ENVIRONMENT == "fsm"){

                  this.loadNGUIWorkAreaWithCPWidget("support/panels/StandbyDetails",null,this._selectedPool.id.toString());          
              
              }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                  var launchParams = {templateIDNum: this._selectedPool.id.toString()};
                  
                  this.loadAugustaWorkAreaTask("config.detail.page","support/panels/StandbyDetails",launchParams);
                  
              }
          
          }));

        },

        // Edit Pool
        editStandbyPool: function() {
            
            var editdialog = new support.dialogs.StandbyPoolTemplate({
                templateIDNum: this._selectedPool.id[0]
            });
            editdialog.show();

        },
        
        // Rename Pool
        renamePool: function() {
            var renameDialog = new support.dialogs.Rename({
                id: this.id + "cp_standbylisttable_rename",
                templateIDNum: this._selectedPool.id.toString(),
                saveTemplateType: "standby"
            });
            renameDialog.show();
        },

        // Delete Pools
        deleteStandbyPools: function() {
            var confirmedDelete = function() {

                var si = (this._standbyTable != null) ? this._standbyTable.select.row.getSelected() : [];

                for (var loopVar in si) {

                    if (si[loopVar] != null) {
                        var xhrDelete = dojo.xhrDelete({
                            url: this.REST_PATH + "pool/standby/" + si[loopVar],
                            handleAs: "json",
                            headers: {
                                "Content-Type": "application/json; charset=utf-8"
                            },
                            sync: true,
                            preventCache: true,
                            load: function(response) {
                            }
                        });
                    }

                }                
                
                this._refreshTree();                
                dojo.publish("AvailableFailoverServersModified");
            };

            var si = (this._standbyTable != null) ? this._standbyTable.select.row.getSelected() : [];

            for (var loopVar in si) {
                var item = this._standbyTable.row(si[loopVar], true).item();
                var tempName = item.name;
                tempName = this.encodeForHtml(tempName.toString());
                si[loopVar] = "  " + tempName;
            }
            
            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.DELETE,
                        type: "Warning",
                        text: this.locResources.DELETE_TEXT,
                        info: this.locResources.DELETE_INFO                    
                    });

                    confirmDialog.set("content", "<p>" + this.locResources.DELETE_INFO + ":</p><p>" + si + "</p>");

                    confirmDialog.startup();
                    confirmDialog.confirm(confirmedDelete, this);
                
                }));
 
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.DELETE,
                        type: "Warning",
                        text: this.locResources.DELETE_TEXT,
                        info: this.locResources.DELETE_INFO                    
                    });

                    confirmDialog.set("content", "<p>" + this.locResources.DELETE_INFO + ":</p><p>" + si + "</p>");

                    confirmDialog.startup();
                    confirmDialog.confirm(confirmedDelete, this);
                
                }));
                
            }

            
            return false;
        },

        // Set Table Formatters
        // Hard-coded path appended to URL, must be changed if path changes
        _decorateName: function(item, id) {
            // tags:
            // protected
            if (!item || !id) {
                return "";
            }
            
            var value = item;
            
            if (this.CP_RUN_ENVIRONMENT == "fsm") {
                return "<a href=\"#workAreaDijit=support%2Fpanels%2FStandbyDetails&path=support:%2Fcom.ibm.stg.sysx.profile." +
                	"navmodel%2Fjs%2Fsupport&cpTarget=" + id + "\">" + value + "</a>";
            } else if(this.CP_RUN_ENVIRONMENT == "augusta") {
                return "<a href=\"#fwWidget=support%2Fpanels%2FStandbyDetails&fwTaskId=config.detail.page&templateIDNum=" 
                    + id + "\">" + value + "</a>";
            }

        },
        
        _formatName: function(item) {                        
            var value = this.encodeForHtml(item.name);                        
            return value;
        },

        _decorateInUse: function(item) {            
            
            if (item == this.resources.IN_USE) {
                return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr>" +
                    "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spInUseIcon\" title=\"" + this.locResources.SPIN_USE + "\"></img></td>" +
                    "<td><span style=\"margin-left:4px; white-space: nowrap;\" title=\"" + this.locResources.SPIN_USE + "\">" + 
                    this.resources.IN_USE + "</span></td></tr></tbody></table>";
            }
            return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr>" +
                "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spNotInUseIcon\" title=\"" + this.locResources.PPN_USE + "\"></img></td>" +
                "<td><span style=\"margin-left:4px; white-space: nowrap;\" title=\"" + this.locResources.PPN_USE + "\">" + 
                this.resources.NOT_IN_USE + "</span></td></tr></tbody></table>";
        },
        
        _formatInUse: function(item) {
            var inUse = (item && item.inUse) ? item.inUse : false;
            
            if (inUse) {
                return this.resources.IN_USE;
            } 
            return this.resources.NOT_IN_USE;
        },   
        
        _decorateDescription: function(item) {
            return "<span title=\"" + item + "\" style=\"white-space:normal;\">" + item + "</span>";
        },

        _formatDescription: function(item) {
            // tags:
            // protected
            if (!item || !item.description) {
                return "";
            }
            var value = this.decodeSpecialCharacters(item.description);
            value = this.encodeForHtml(value);
            return value;
        },

        // Set Table Capabilities
        setTableCapabilities: function() {
            
            if (!this._standbyTable) {
                return; // nothing to do. return now
            }
            
            if (this._standbyTable != null && this._standbyTable.select.row.getSelected().length > 0 && 
                    this._standbyTable.select.row.getSelected()[0] != null) {

                this.deleteButton.set("iconClass", "spDeleteInactiveLgIcon");
                this.deleteButton.set("disabled", true);

                var srCount = this._standbyTable.select.row.getSelected().length;
                var si = (this._standbyTable != null) ? this._standbyTable.select.row.getSelected() : [];
                var inUseFound = false;

                for (var loopVar in si) {
                    var item = this._standbyTable.row(si[loopVar], true).item();
                    this._selectedPool = item;
                    
                    if (item != null) {
                        var inUseFound = item.inUse != null ? item.inUse[0] : false;                        
                    }
                    
                    if (inUseFound) {
                        break; // found an "in-use" pool
                    }
                }

                if ((this.editButton != null) && (srCount === 1)) {
                    this.editButton.set("iconClass", "spEditLgIcon");
                    this.editButton.set("disabled", false);

                    if (!this.editActionItem) {
                        this.editActionItem = new dijit.MenuItem({
                            //id: this.id + "cp_standbydetailsdependent_editactionitem",
                            label: this.resources.EDIT,
                            title: this.locResources.EDIT_SEL,
                            iconClass: "spEditLgIcon",                                
                            onClick: dojo.hitch(this, this.editStandbyPool),
                            onKeypress: dojo.hitch(this, this.editStandbyPool)
                        });     
                    }
                    this.actionMenu.addChild(this.editActionItem);
                    
                    if (!this.deleteActionItem) {
                        this.deleteActionItem = new dijit.MenuItem({
                            //id: this.id + "cp_standbydetailsdependent_deleteactionitem",
                            label: this.resources.DELETE,
                            title: this.locResources.DELETE_SEL,
                            iconClass: "spDeleteLgIcon",
                            onClick: dojo.hitch(this, this.deleteStandbyPools),
                            onKeypress: dojo.hitch(this, this.deleteStandbyPools)
                        });
                    }                   
                    // only add if pool not in use
                    if (!inUseFound) {
                        this.actionMenu.addChild(this.deleteActionItem);
                    }
                    
                    if (!this.detailsActionItem) {
                        this.detailsActionItem = new dijit.MenuItem({
                            //id: this.id + "cp_standbylisttable_detailsactionitem",
                            label: this.resources.DETAILS,
                            title: this.locResources.DETAILS_SELECTED_POOL,
                            onClick: dojo.hitch(this, this.detailStandbyPool),
                            onKeypress: dojo.hitch(this, this.detailStandbyPool)
                        });                       
                    }                    
                    this.actionMenu.addChild(this.detailsActionItem); // add to toolbar action menu
                    
                    if (!this.renameActionItem) {                        
                        this.renameActionItem = new dijit.MenuItem({
                            label: this.resources.RENAME,
                            title: this.locResources.RENAME_SELECTED_POOL,
                            onClick: dojo.hitch(this, this.renamePool),
                            onKeypress: dojo.hitch(this, this.renamePool)
                        });
                    }                    
                    this.actionMenu.addChild(this.renameActionItem); // add to toolbar action menu

                }
                
                else {
                    this.editButton.set("iconClass", "spEditInactiveLgIcon");
                    this.editButton.set("disabled", true);
                    
                    this.actionMenu.removeChild(this.editActionItem);
                    this.editActionItem = null;                    

                    this.actionMenu.removeChild(this.detailsActionItem);
                    this.detailsActionItem = null;    
                    
                    this.actionMenu.removeChild(this.renameActionItem);
                    this.renameActionItem = null;
                }
                
                if ((this.deleteButton != null) && (srCount > 0) && inUseFound == false) {
                    this.deleteButton.set("iconClass", "spDeleteLgIcon");
                    this.deleteButton.set("disabled", false);
                    
                    this.createButton.set("iconClass", "spCreateLgIcon");
                    this.createButton.set("disabled", false);
                }
                
                else {
                    this.deleteButton.set("iconClass", "spDeleteInactiveLgIcon");
                    this.deleteButton.set("disabled", true);
                    
                    this.actionMenu.removeChild(this.deleteActionItem);
                    this.deleteActionItem = null;
                    
                    this.createButton.set("iconClass", "spCreateLgIcon");
                    this.createButton.set("disabled", false);                                       
                }
                
            }
            
            else {
                // no rows available, disable edit/delete buttons
                
                if (this.deleteButton != null) {
                    this.deleteButton.set("iconClass", "spDeleteInactiveLgIcon");
                    this.deleteButton.set("disabled", true);
                }
                
                if (this.editButton != null) {
                    this.editButton.set("iconClass", "spEditInactiveLgIcon");
                    this.editButton.set("disabled", true);
                }
                
                if (this.detailsActionItem) {
                    this.actionMenu.removeChild(this.detailsActionItem);
                    this.detailsActionItem = null;
                }
                
                if (this.editActionItem) {
                    this.actionMenu.removeChild(this.editActionItem);
                    this.editActionItem = null;    
                }
                
                if (this.deleteActionItem) {
                    this.actionMenu.removeChild(this.deleteActionItem);
                    this.deleteActionItem = null;    
                }
                
                if (this.renameActionItem) {
                    this.actionMenu.removeChild(this.renameActionItem);
                    this.renameActionItem = null;    
                }
            }
            
            // Disable Action Button if no children
            if(this.actionMenu.hasChildren()) {
                this._actionButton.set("disabled", false);
            } else {
                this._actionButton.set("disabled", true);                
            }
        },
        
        _getTableStructure: function() {
            
            var layout = new Array();
            
            layout.push({
                field: "name",
                name: this.locResources.STANDBY_SERVER_POOL,
                width: "25em",
                formatter: dojo.hitch(this, this._formatName),
                decorator: dojo.hitch(this, this._decorateName),
                title: ""
            });
            layout.push({
                fields: "inUse",
                name: this.resources.USAGE_STATUS,
                width: "8em",
                formatter: dojo.hitch(this, this._formatInUse),
                decorator: dojo.hitch(this, this._decorateInUse),
                title: ""
            });
            layout.push({
                field: "description",
                name: this.resources.DESCRIPTION,
                formatter: dojo.hitch(this, this._formatDescription),
                decorator: dojo.hitch(this, this._decorateDescription),
                title: ""
            });
            
            return layout;
        },
        
        goForeground: function(params) { //Page base function that fires when the page is brought to focus
            
            this._refreshTree();

            return true;
        }               
    });
});