define([ 
        "dojo",
        "dijit",
        "support/panels/Panel",
        "support/base/NextGenUIMixin",
        "dojo/i18n!./nls/SuggestedTasks",
        "dojo/text!./templates/SuggestedTasks.html",
        "dijit/Dialog",
        "dijit/layout/ContentPane",
        "dijit/form/Button",
        "support/dialogs/DeployServerTemplate",
        "support/dialogs/ConfigPatterns",
        "support/dialogs/DeployChassisTemplate"
    ], function(dojo, dijit, Panel, NextGenUIMixin, mri, template) {

    return dojo.declare("support.panels.SuggestedTasks", [ support.panels.Panel, support.base.NextGenUIMixin ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "SuggestedTasks",

        // Globals
        handles: [],
        taskCount: 0,

        chooseConfigComplete: false,
        createTemplateComplete: false,
        deployTemplateComplete: false,
        createChassisTemplateComplete: false,
        deployChassisTemplateComplete: false,
        standbyPoolComplete: false,
        failoverMonitoringComplete: false,

        standbyServerPools: 0,
        unmonitoredServers: 0,
        unassignedServers: 0,
        unassignedChassis: 0,

        fsmInitialSetup: false,
        
        // True if user-defined patterns exist
        serverPatternExists: false,
        chassisPatternExists: false,

        end: null,

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/SuggestedTasks.css",

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {

            this.inherited(arguments);

        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);

            var panel = this;
            var currentURL = window.location.hash;

            if(currentURL.indexOf("initialsetup=true") != -1){
                this.fsmInitialSetup = true;
            }

            dojo.xhrGet({
                url: this.REST_PATH + "server?inclSrvrsWithProfile=false&numberOfNotMonitoredServers=true",
                handleAs: "json",
                sync: true,
                load: function(response) {
                    panel.unmonitoredServers = parseInt(response);
                    console.log(response);
                },
                error: function(error) {
                    //alert(panel.resources.UNEXPECTED_ERROR.replace("%1", error));
                }
            });

            dojo.xhrGet({
                url: this.REST_PATH + "deploy/status",
                handleAs: "json",
                sync: true,
                load: function(response) {
                    panel.unassignedServers = parseInt(response.unassignedComputeNodes);
                    panel.unassignedChassis = parseInt(response.unassignedCmms);
                    console.log(response);
                },
                error: function(error) {
                    //alert(panel.resources.UNEXPECTED_ERROR.replace("%1", error));
                }
            });

            dojo.xhrGet({
                url: this.REST_PATH + "pool/standby?numberOfPools=true",
                handleAs: "json",
                sync: true,
                load: function(response) {
                    panel.standbyServerPools = parseInt(response);
                    console.log(response);
                },
                error: function(error) {
                    //alert(panel.resources.UNEXPECTED_ERROR.replace("%1", error));
                }
            });
            
            dojo.xhrGet({
                url: this.REST_PATH + "template/list/Server",
                handleAs: "json",
                sync: true,
                load: dojo.hitch(this, function(data) {
                    // Determine if there is an existing user defined server pattern
                    if (data.items[0]) {
                        this.serverPatternExists = true;
                    }
                    else {
                        this.serverPatternExists = false;
                    }
                })
            });
            
            dojo.xhrGet({
                url: this.REST_PATH + "template/list/CMM",
                handleAs: "json",
                sync: true,
                load: dojo.hitch(this, function(data) {
                    // Determine if there is an existing user defined chassis pattern
                    if (data.items[0]) {
                        this.chassisPatternExists = true;
                    }
                    else {
                        this.chassisPatternExists = false;
                    }
                })
            });

            // Check to see if defaults already exist
            dojo.xhrGet({
                url: this.REST_PATH + "tasks",
                handleAs: "json",
                sync: true,
                load: dojo.hitch(this,
                        function(response) {

							//If in Augusta framework then remove any reference for chassis and failover from suggested task
                            if(this.CP_RUN_ENVIRONMENT == "augusta"){

                                this.taskCount = 2;

                                this.chooseConfigComplete = (response.tasks.chooseConfig.status == "completed");
                                this.createTemplateComplete = (response.tasks.createTemplate.status == "completed");
                                this.deployTemplateComplete = (response.tasks.deployTemplate.status == "completed");

                                if (this.chooseConfigComplete) {
                                    if (!this.fsmInitialSetup) {
                                        this.taskCount--;
                                    }
                                    dojo.replaceClass(this._chooseDefaultsImg, "spSvrTmplCompleteIcon", "spSvrTmplReadyIcon");
                                }

                                if (this.unassignedServers == 0) {
                                    this.taskCount--;
                                    this._createServerTemplateLink.style.display = "none";
                                    this._deployServerTemplateLink.style.display = "none";
                                    dojo.replaceClass(this._createServerTemplateImg, "spCreateSvrTmplCompleteIcon",
                                            "spCreateSvrTmplReadyIcon");
                                }
                                
                                if (this.unassignedServer == 0 || !this.serverPatternExists) {
                                    this._deployServerTemplateLink.style.display = "none";
                                }
                                

                            }else{

                                if (this.fsmInitialSetup) {
                                    this.taskCount = 2;
                                } else {
                                    this.taskCount = 5;
                                }

                                this.chooseConfigComplete = (response.tasks.chooseConfig.status == "completed");
                                this.createTemplateComplete = (response.tasks.createTemplate.status == "completed");
                                this.deployTemplateComplete = (response.tasks.deployTemplate.status == "completed");
                                this.createChassisTemplateComplete = (response.tasks.createChassisTemplate.status == "completed");
                                //this.deployChassisTemplateComplete = (response.tasks.deployChassisTemplate.status == "completed");
                                this.standbyPoolComplete = (response.tasks.createStandbyPool.status == "completed");
                                this.failoverMonitoringComplete = (response.tasks.startFailoverMonitoring.status == "completed");

                                if (this.chooseConfigComplete) {
                                    if (!this.fsmInitialSetup) {
                                        this.taskCount--;
                                    }
                                    dojo.replaceClass(this._chooseDefaultsImg, "spSvrTmplCompleteIcon", "spSvrTmplReadyIcon");
                                }

                                if (this.unassignedServers == 0) {
                                    this.taskCount--;
                                    this._createServerTemplateLink.style.display = "none";
                                    this._deployServerTemplateLink.style.display = "none";
                                    dojo.replaceClass(this._createServerTemplateImg, "spCreateSvrTmplCompleteIcon",
                                            "spCreateSvrTmplReadyIcon");
                                }
                                
                                if (this.unassignedServer == 0 || !this.serverPatternExists) {
                                    this._deployServerTemplateLink.style.display = "none";
                                }
                                
                                if (this.unassignedChassis == 0) {
                                    this.taskCount--;
                                    this._createChassisTemplateLink.style.display = "none";
                                    this._deployChassisTemplateLink.style.display = "none";
                                    dojo.replaceClass(this._createChassisTemplateImg, "spCreateChassisTmplCompleteIcon",
                                            "spCreateChassisTmplReadyIcon");
                                }
                                
                                if (this.unassignedChassis == 0 || !this.chassisPatternExists) {
                                    this._deployChassisTemplateLink.style.display = "none";
                                }

                                console.log("standby server pools: " + this.standbyServerPools);
                                console.log("unmonitored servers: " + this.unmonitoredServers);
                                console.log("unassigned servers: " + this.unassignedServers);
                                console.log("unassigned chassis: " + this.unassignedChassis);
                                
                                if (this.standbyPoolComplete) {
                                    if (!this.fsmInitialSetup) {
                                        this.taskCount--;
                                    }
                                    dojo.replaceClass(this._createStandbyServerPoolImg, "spAddrPoolCompleteIcon",
                                            "spAddrPoolReadyIcon");
                                }
                                /*else if (this.unassignedServers == 0) {
                                    this.taskCount--;
                                    this.standbyPoolContainer.style.display = "none";
                                }*/

                                if (this.failoverMonitoringComplete) {
                                    if (!this.fsmInitialSetup) {
                                        this.taskCount--;
                                    }
                                    dojo.replaceClass(this._startFailoverMonitoringImg, "spStartMonitoringCompleteIcon",
                                            "spStartMonitoringReadyIcon");
                                }
                                /*else if (this.standbyServerPools == 0 && this.unassignedServers == 0) {
                                    this.taskCount--;
                                    this.failoverMonitoringContainer.style.display = "none";
                                }*/

                            }
                            
                            

                            // Setup Suggested Task pane
                            this.set("title", this.locResources.TASK_COUNT.replace("%1", this.taskCount));

                        }),
                error: function(error) {
                    //alert(this.resources.UNEXPECTED_ERROR.replace("%1", error));
                }

            });

            // Apply env specific text for "Continue Initial Setup" task
            this.continueInitialSetupTask.title = this.locResources.CONTINUE_TITLE.replace("%1", this.PRODUCT_NAME_SHORT);
            this.continueInitialSetupTask.innerHTML = this.locResources.CONTINUE_TASK;
            this.continueInitialSetupDesc.innerHTML = this.locResources.CONTINUE_DESC.replace("%1", this.PRODUCT_NAME_SHORT);

            // Config Patterns Preferences set
            this.subscribe("ConfigPatternsComplete", function(data) {
                if (data != null && !this.chooseConfigComplete) {
                    dojo.replaceClass(this._chooseDefaultsImg, "spSvrTmplCompleteIcon", "spSvrTmplReadyIcon");
                    this.setTaskComplete("chooseConfig");
                    this.chooseConfigComplete = true;
                    dojo.publish("taskComplete", []);
                }
            });

            // Server Pattern created
            this.subscribe("ServerTemplateComplete", function(data) {
                this.getInfo();
                this.serverPatternExists = true;
                if (this.fsmInitialSetup) {
                    this._editServerTemplateLink.style.display = "";
                }
                if (this.unassignedServers == 0) {
                    this._createServerTemplateLink.style.display = "none";
                    this._deployServerTemplateLink.style.display = "none";
                } else {
                    this._createServerTemplateLink.style.display = "";
                    this._deployServerTemplateLink.style.display = "";
                }
                if (!this.createTemplateComplete) {
                    if (this.unassignedServers == 0) {
                        dojo.replaceClass(this._createServerTemplateImg, "spCreateSvrTmplCompleteIcon", "spCreateSvrTmplReadyIcon");
                        this.setTaskComplete("createTemplate");
                        this.createTemplateComplete = true;
                        dojo.publish("taskComplete", []);
                    }
                }
            });
            
            // Server Profile unassigned
            this.subscribe("UnassignServerProfileComplete", function(data) {
                this.getInfo();
                dojo.replaceClass(this._createServerTemplateImg, "spCreateSvrTmplReadyIcon", "spCreateSvrTmplCompleteIcon");
                this._createServerTemplateLink.style.display = "";
                this._deployServerTemplateLink.style.display = "";
                if (this.unassignedServers == 1) {
                    dojo.publish("taskIncomplete", []);
                }
            });
            
            // Server Profile deployed
            this.subscribe("DeployServerTemplateComplete", function(data) {
                this.getInfo();
                if (this.unassignedServers == 0) {
                    dojo.replaceClass(this._createServerTemplateImg, "spCreateServerTmplCompleteIcon", "spCreateServerTmplReadyIcon");
                    this.setTaskComplete("createTemplate");
                    this._createServerTemplateLink.style.display = "none";
                    this._deployServerTemplateLink.style.display = "none";
                    this.createServerTemplateComplete = true;
                    dojo.publish("taskComplete", []);
                }
            });

            // Chassis Pattern created
            this.subscribe("ChassisTemplateComplete", function(data) {
                this.getInfo();
                this.chassisPatternExists = true;
                if (this.fsmInitialSetup) {
                    this._editChassisTemplateLink.style.display = "";
                }
                if (this.unassignedChassis == 0) {
                    this._createChassisTemplateLink.style.display = "none";
                    this._deployChassisTemplateLink.style.display = "none";
                } else {
                    this._createChassisTemplateLink.style.display = "";
                    this._deployChassisTemplateLink.style.display = "";
                }
                if (!this.createChassisTemplateComplete) {
                    if (this.unassignedChassis == 0) {
                        dojo.replaceClass(this._createChassisTemplateImg, "spCreateChassisTmplCompleteIcon", "spCreateChassisTmplReadyIcon");
                        this.setTaskComplete("createChassisTemplate");
                        this._createChassisTemplateLink.style.display = "none";
                        this._deployChassisTemplateLink.style.display = "none";
                        this.createChassisTemplateComplete = true;
                        dojo.publish("taskComplete", []);
                    }
                }
            });
            
            // Chassis Profile unassigned
            this.subscribe("UnassignChassisProfileComplete", function(data) {
                this.getInfo();
                dojo.replaceClass(this._createChassisTemplateImg, "spCreateChassisTmplReadyIcon", "spCreateChassisTmplCompleteIcon");
                this._createChassisTemplateLink.style.display = "";
                this._deployChassisTemplateLink.style.display = "";
                if (this.unassignedChassis == 1) {
                    dojo.publish("taskIncomplete", []);
                }
            });
            
            // Chassis Profile deployed
            this.subscribe("DeployChassisTemplateComplete", function(data) {
                this.getInfo();
                if (this.unassignedChassis == 0) {
                    dojo.replaceClass(this._createChassisTemplateImg, "spCreateChassisTmplCompleteIcon", "spCreateChassisTmplReadyIcon");
                    this.setTaskComplete("createChassisTemplate");
                    this._createChassisTemplateLink.style.display = "none";
                    this._deployChassisTemplateLink.style.display = "none";
                    this.createChassisTemplateComplete = true;
                    dojo.publish("taskComplete", []);
                }
            });

            // Standby Pool created
            this.subscribe("StandbyPoolComplete", function(data) {
                this.getInfo();
                if (!this.standbyPoolComplete) {
                    if(this.standbyServerPools > 0 || this.unassignedServers == 0) {
                        dojo.replaceClass(this._createStandbyServerPoolImg, "spAddrPoolCompleteIcon", "spAddrPoolReadyIcon");
                        this.setTaskComplete("createStandbyPool");
                        this.standbyPoolComplete = true;
                        dojo.publish("taskComplete", []);
                    }
                }
            });

            // Change in available servers
            this.subscribe("AvailableFailoverServersModified", function() {
                this.getInfo();
                if (!this.failoverMonitoringComplete) {
                    //console.debug("unmonitored servers: " + this.unmonitoredServers);
                    if(this.unmonitoredServers == 0) {
                        dojo.replaceClass(this._startFailoverMonitoringImg, "spStartMonitoringCompleteIcon", "spStartMonitoringReadyIcon");
                        this.setTaskComplete("startFailoverMonitoring");
                        this.failoverMonitoringComplete = true;
                        dojo.publish("taskComplete", []);
                    }
                }
            });

            // Task completed
            this.subscribe("taskComplete", function() {
                (this.taskCount > 0) && this.taskCount--;
                this.set("title", this.locResources.TASK_COUNT.replace("%1", this.taskCount));
            });
            
            // Task now marked incomplete
            this.subscribe("taskIncomplete", function() {
                (this.taskCount < 5) && this.taskCount++;
                this.set("title", this.locResources.TASK_COUNT.replace("%1", this.taskCount));
            });
            
            // Server Pattern deleted
            this.subscribe("ServerTemplateDeleted", function() {
                dojo.xhrGet({
                    url: this.REST_PATH + "template/list/Server",
                    handleAs: "json",
                    sync: true,
                    load: dojo.hitch(this, function(data) {
                        // Determine if there is an existing user defined server pattern
                        if (data.items[0]) {
                            this.serverPatternExists = true;
                        } 
                        else {
                            this.serverPatternExists = false;
                        }
                    })
                });
                
                if (!this.serverPatternExists) {
                    this._editServerTemplateLink.style.display = "none";
                    this._deployServerTemplateLink.style.display = "none";
                }
            });
            
            // Chassis Pattern deleted
            this.subscribe("ChassisTemplateDeleted", function() {
                dojo.xhrGet({
                    url: this.REST_PATH + "template/list/CMM",
                    handleAs: "json",
                    sync: true,
                    load: dojo.hitch(this, function(data) {
                        // Determine if there is an existing user defined server pattern
                        if (data.items[0]) {
                            this.chassisPatternExists = true;
                        } 
                        else {
                            this.chassisPatternExists = false;
                        }
                    })
                });
                
                if (!this.chassisPatternExists) {
                    this._editChassisTemplateLink.style.display = "none";
                    this._deployChassisTemplateLink.style.display = "none";
                }
            });

            this.displayTasks();

			//If in Augusta framework then remove any reference for chassis and failover from suggested task
            if(this.CP_RUN_ENVIRONMENT == "augusta"){
                this.configureCMMContainer.remove();
                this.standbyPoolContainer.remove();
                this.failoverMonitoringContainer.remove();
                this.learnDescription.innerHTML = this.locResources.LEARN_DESC_NO_CHASSIS;
            }else{
                this.learnDescription.innerHTML = this.locResources.LEARN_DESC;                
            }
        },

        getInfo: function() {

            var panel = this;

            dojo.xhrGet({
                url: this.REST_PATH + "server?inclSrvrsWithProfile=false&numberOfNotMonitoredServers=true",
                handleAs: "json",
                sync: true,
                load: function(response) {
                    panel.unmonitoredServers = parseInt(response);
                    //console.debug("unmonitored: " + response);
                },
                error: function(error) {
                    //alert(panel.resources.UNEXPECTED_ERROR.replace("%1", error));
                }
            });

            dojo.xhrGet({
                url: this.REST_PATH + "deploy/status",
                handleAs: "json",
                sync: true,
                load: function(response) {
                    panel.unassignedServers = parseInt(response.unassignedComputeNodes);
                    panel.unassignedChassis = parseInt(response.unassignedCmms);
                },
                error: function(error) {
                    //alert(panel.resources.UNEXPECTED_ERROR.replace("%1", error));
                }
            });

            dojo.xhrGet({
                url: this.REST_PATH + "pool/standby?numberOfPools=true",
                handleAs: "json",
                sync: true,
                load: function(response) {
                    panel.standbyServerPools = parseInt(response);
                    //console.debug(response);
                },
                error: function(error) {
                    //alert(panel.resources.UNEXPECTED_ERROR.replace("%1", error));
                }
            });

            this.cmmInformationMessageText.innerHTML = this.locResources.CMMS_UNASSIGNED.replace("%1", this.unassignedChassis);
            this.sysxInformationMessageText.innerHTML = this.locResources.COMPUTE_NODES_UNASSIGNED.replace("%1", this.unassignedServers);
            this.failoverInformationMessageText.innerHTML = this.locResources.SERVERS_NOT_MONITORED.replace("%1", this.unmonitoredServers);
        },

        displayTasks: function() {

            var taskCounter = 1;

			//If in Augusta framework then remove any reference for chassis and failover from suggested task
            if(this.CP_RUN_ENVIRONMENT == "augusta"){

                this.configureNodesTitle.innerHTML = this.locResources.CONFIGURE_SYSTEM_X_TASK.replace("%1", taskCounter++);
                                
                this.setPreferencesTitle.innerHTML = this.locResources.SET_PREF_TASK.replace("%1", taskCounter++);
                                    
            }else{
                 
                this.configureNodesTitle.innerHTML = this.locResources.CONFIGURE_SYSTEM_X_TASK.replace("%1", taskCounter++);
                
                this.configureCMMSTitle.innerHTML = this.locResources.CONFIGURE_CMM_TASK.replace("%1", taskCounter++);
                
                this.setPreferencesTitle.innerHTML = this.locResources.SET_PREF_TASK.replace("%1", taskCounter++);
                    
                this.createStandbyTitle.innerHTML = this.locResources.CREATE_POOL_TASK.replace("%1", taskCounter++);
                    
                this.startFailoverTitle.innerHTML = this.locResources.START_FAILOVER_TASK.replace("%1", taskCounter++);

            }

            this.cmmInformationMessageText.innerHTML = this.locResources.CMMS_UNASSIGNED.replace("%1", this.unassignedChassis);
            this.sysxInformationMessageText.innerHTML = this.locResources.COMPUTE_NODES_UNASSIGNED.replace("%1", this.unassignedServers);
            this.failoverInformationMessageText.innerHTML = this.locResources.SERVERS_NOT_MONITORED.replace("%1", this.unmonitoredServers);

            if(this.fsmInitialSetup) {
                this.setConfigPatternsContainer.style.display = "none";
                this.standbyPoolContainer.style.display = "none";
                this.failoverMonitoringContainer.style.display = "none";
                if (this.serverPatternExists) {
                    this._editServerTemplateLink.style.display = "";
                }
                if (this.chassisPatternExists) {
                    this._editChassisTemplateLink.style.display = "";
                }
                this.continueInitialSetupContainer.style.display = "";
            }

        },

        onGotoOverview: function() {
            // summary:
            // Override or hitch custom handler.
            // Called when user requests to goto the Concept Overview tab.
        },

        _showLearnConcepts: function(evt) {
            // tags:
            // protected
            this.onGotoOverview();
        },

        _showConfigPatternsPrefs: function(evt) {
            // tags:
            // protected
            var dialog = new support.dialogs.ConfigPatterns({
                //id: this.id + "cp_suggestedtasks_configpatternsprefs"
            });
            dialog.show();
        },

        _showServerTemplateWizard: function(evt) {
            // tags:
            // protected
            require(["support/dialogs/ServerTemplateWizard"], dojo.hitch(this, function() {
                var dialog = new support.dialogs.ServerTemplateWizard({
                    createFromExisting: true
                });
                dialog.show();
            }));
        },
        
        _showEditServerTemplate: function(evt) {
            require(["support/dialogs/SelectExistingPattern"], dojo.hitch(this, function() {
                var dialog = new support.dialogs.SelectExistingPattern({
                    //id: this.id + "cp_suggestedtasks_selectexistingpattern",
                    patternType: "Server"                    
                });
                // dialog.setServer();
                dialog.show();
            }));
        },        

        _showDeployServerTemplate: function(evt) {
            // tags:
            // protected
            var dialog = new support.dialogs.DeployServerTemplate({
                //id: this.id + "cp_suggestedtasks_deployservertemplate"
            });
            dialog.show();
        },

        _showChassisTemplateWizard: function(evt) {
            require(["support/dialogs/ChassisTemplateWizard"], dojo.hitch(this, function() {
                var dialog = new support.dialogs.ChassisTemplateWizard({
                    createFromExisting: true
                });
                dialog.show();
            }));
        },
        
        _showEditChassisTemplate: function(evt) {
            require(["support/dialogs/SelectExistingPattern"], dojo.hitch(this, function() {
                var dialog = new support.dialogs.SelectExistingPattern({
                    //id: this.id + "cp_suggestedtasks_selectexistingcmmpattern",
                    patternType: "CMM"
                });
                //dialog.setChassis();
                dialog.show();
            }));
        },

        _showDeployChassisTemplate: function(evt) {
            var dialog = new support.dialogs.DeployChassisTemplate({
                //id: this.id + "cp_suggestedtasks_deploychassistemplate"
            });
            dialog.show();
        },

        _showStandbyServerPool: function(evt) {
            // tags:
            // protected
            require(["support/dialogs/StandbyPoolTemplate"], dojo.hitch(this, function() {
                var dialog = new support.dialogs.StandbyPoolTemplate({
                    //id: this.id + "cp_suggestedtasks_standbypool"
                });
                dialog.show();
            }));
        },

        _showFailoverMonitoring: function(evt) {
            // tags:
            // protected
            require(["support/dialogs/FailoverMonitorTemplate"], dojo.hitch(this, function() {
                var dialog = new support.dialogs.FailoverMonitorTemplate({
                    //id: this.id + "cp_suggestedtasks_failovermonitor"
                });
                dialog.show();
            }));
        },

        _launchbackLink: function(evt) {
            this.loadFSMHomePage();
            window.close();
        },

        // Set values to indicate preferences have been picked
        setTaskComplete: function(val) {
            dojo.xhrPut({
                url: this.REST_PATH + "tasks/" + val + "/complete",
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                error: function(error) {
                    //alert(this.resources.UNEXPECTED_ERROR.replace("%1", error));
                }
            });
        }
        
        // Set task to incomplete
/*        setTaskIncomplete: function(val) {
            dojo.xhrPut({
                url: this.REST_PATH + "tasks/" + val + "/incomplete",
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                error: function(error) {
                    alert(this.resources.UNEXPECTED_ERROR.replace("%1", error));
                }
            });
        },
        
        configureServersComplete: function() {
            if (this.unassignedServers == 0 && this.serverPatternExists) {
                return true;
            } else {
                return false;
            }
        },
        
        configureChassisComplete: function() {
            if (this.unassignedChassis == 0 && this.chassisPatternExists) {
                return true;
            } else {
                return false;
            }            
        }
*/
    });

});