define([
        "dojo",
        "dijit",
        "support/ConfigPatternsAll",
        "support/panels/Panel",
        "dojo/i18n!./nls/TemplatesTable",
        "dojo/text!./templates/TemplatesTable.html",
        "dijit/layout/LayoutContainer",
        "dijit/layout/ContentPane",
        "support/base/NextGenUIMixin",
        "support/base/TableMixin",
        "dojo/data/ItemFileWriteStore",
        "dijit/form/DropDownButton",
        "dijit/form/Button",
        "dijit/Menu",
        "dijit/MenuItem",
        "dijit/MenuSeparator",
        "dijit/PopupMenuItem",
        "dojo/string",
        "dijit/Toolbar",
        "dijit/ToolbarSeparator",
        "support/dialogs/ProgressDialog"
    ], function(dojo, dijit, layer, panel, mri, template) {

    return dojo.declare("support.panels.TemplatesTable", [ support.panels.Panel, support.base.NextGenUIMixin, support.base.TableMixin ], {

        // summary:
        // A panel widget extending support.panels.Panel, for use in managing templates for
        // Configuration Patterns.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "TemplatesTable",

        // a reference to the function that is responsible for automatically
        pollingFunc: null,

        // a reference to the current Date instance; used to turn off the
        //   automatic refresh mechanism if it runs too long
        pollingStartDate: null,

        _patternTable: null,
        patternStore: null,
        editButton: null,
        deleteButton: null,
        _selectedTemplate: null,

        actionMenu: null,
        _actionButton: null,

        refreshRoot: null,
        
        _iconTitle: "",
        _helpText: "",

        // the properties used to construct the title of this page in the
        // NextGen UI
        cpProductName: null,
        cpPageContext: null,
        cpPageName: null,
        
        // ID prefix used for table
        gridID: "ServerPatternsTable",

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/TemplatesTable.css",

        // Views
        BY_ROOT: 1,
        FLAT: 2,
        // Default View Mode
        viewMode: 1,

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
            this.cpProductName = this.resources.PRODUCT_NAME;
            // this.cpPageContext = this.resources.SERVERS; Must remain null (unspecified), per XB216902
            this.cpPageName = this.resources.SERVER_PATTERNS;
            this._iconTitle = this.locResources.TOOLTIP_TEXT;
            this._helpText = this.locResources.HELP_TEXT;
        },

        // postCreate() is called after
        // buildRendering(). This is useful to override
        // when
        // you need to access and/or manipulate DOM
        // nodes included with your widget.
        // DOM nodes and widgets with the
        // dojoAttachPoint attribute specified can now
        // be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in
        // buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);

            // Setup table widget...
            var patternStore = this._retrieveStore();
            var tableStructure = this._getTableStructure();
            this._patternTable = this.getTable(patternStore, tableStructure, this.resources.NO_DATA, this.gridID, true);
            
            // Set parent rows as "unselectable"                                    
            this._patternTable.disableParentRows();
            this._refreshTree();
            
            setTimeout(dojo.hitch(this, function() {
                this._patternTable.placeAt(this.templateTable);
                this._patternTable.startup();
            }), 500);
                 
            // Connect handlers to reset table capabilities
            dojo.connect(this._patternTable.select.row, "onSelectionChange", this, this.setTableCapabilities); // Fires on row selection
            dojo.connect(this._patternTable, "setStore", this, this.setTableCapabilities); // Fires on store reset
            
            // Invoke the NextGenUIMixin to change the title of the NextGen UI work area
            this.changeNextGenUITitle();

            // reload contents of templates table; by default, switching
            // between links in the details selector does not reload page
            this.subscribeToDetailsSelectorChanges(dojo.hitch(this, function() {
                this._refreshTree();
                this.changeNextGenUITitle();
                this._pollForStatusUpdates();
            }));
            
            this._loadToolBar(this._patternTable); //load table toolbar

            // Register subscribers to topics we care about ...
            this.subscribe("ServerTemplateComplete", function(data) {
                this._refreshTree("Server");
            });
            this.subscribe("PowerTemplateComplete", function(data) {
                this._refreshTree("Power");
            });
            this.subscribe("EthTemplateComplete", function(data) {
                this._refreshTree("Management");
            });
            this.subscribe("PerformanceAndRecoveryComplete", function(data) {
                this._refreshTree("PerformanceRecovery");
            });
            this.subscribe("DevicesAndIOPortsComplete", function(data) {
                this._refreshTree("DevicesAndIOPorts");
            });
            this.subscribe("SystemTemplateComplete", function(data) {
                this._refreshTree("SystemInfo");
            });
            this.subscribe("SASBootTemplateComplete", function(data) {
                this._refreshTree("SAS");
            });
            this.subscribe("FibreBootTemplateComplete", function(data) {
                this._refreshTree("FibreChannel");
            });
            this.subscribe("PXEBootTemplateComplete", function(data) {
                this._refreshTree("PXE");
            });
            this.subscribe("CreateAdapterTemplateComplete", function(data) {
                this._refreshTree("Adapter");
            });
            this.subscribe("EditAdapterTemplateComplete", function(data) {
                this._refreshTree("Adapter");
            });
            this.subscribe("CreatePortVirtualizationTemplateComplete", function(data) {
                this._refreshTree("PortVirtualization");
            });
            this.subscribe("EditPortVirtualizationTemplateComplete", function(data) {
                this._refreshTree("PortVirtualization");
            });
            this.subscribe("CreatePortPhysicalFunctionTemplateComplete", function(data) {
                this._refreshTree("PortPhysicalFunction");
            });
            this.subscribe("EditPortPhysicalFunctionTemplateComplete", function(data) {
                this._refreshTree("PortPhysicalFunction");
            });
            this.subscribe("RefreshTable", function(data) {
                this._refreshTree();
            });
            this.subscribe("ReDeployComplete", function(data) {
                this._refreshTree();
            });
            this.subscribe("DeployServerTemplateComplete", function(data) {
                this._refreshTree();
            });
            
            this.subscribe("CloseDialog", function() {
                if (this.viewMode == this.FLAT) {
                    this._patternTable.showFilter(); // Show filter controls on flat view.
                }
            });

            this.subscribe("EndPollingOnSelectorChange", function() {
                // Used to end polling when ever the CP selector changes                
                this._stopPolling();
             });

            //Start polling on page load 
            this._pollForStatusUpdates();

        },

        _refreshTree: function(/*String?*/expandId) {
            // tags:
            // protected
            // Refresh tree data
            var patternStore = this._retrieveStore();
            this._patternTable.setStore(patternStore);
            this._patternTable.model.clearCache();
            var callback = function(treeView, table, /*String?*/expandId) {
                if (treeView) {
                    table.reexpandRows();
                    expandId && table.tree.expand(expandId);
                    table.disableParentRows();
                }
                else {
                    table.showFilter(); // Show filter controls on flat view.
                }
            };
            this._patternTable.body.refresh().then(callback((this.viewMode == this.BY_ROOT), this._patternTable, expandId));
        },

        _retrieveStore: function() {
            // tags:
            // protected
            // Connect to data source and create store
            var patternData = "";

            dojo.xhrGet({
                url: this.REST_PATH + "template/list/all",
                handleAs: "json",
                preventCache: true,
                sync: true,
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                load: dojo.hitch(this, function(response) {
                    if (response && response.items) {
                        if (this.viewMode == this.BY_ROOT) {
                            patternData = this._flatToRoot(response.items);
                        }
                        else if (this.viewMode == this.FLAT) {
                            patternData = this._rootToFlat(response.items);
                        }
                    }
                })
            });
                
            var patternStore = new dojo.data.ItemFileWriteStore({
                 data: dojo.fromJson(patternData)
            });                        

            patternStore.comparatorMap = {};
            patternStore.comparatorMap["name"] = function (a, b) {
                return a.localeCompare(b);
            };

            if (!("hasChildren" in patternStore)) {
                patternStore.hasChildren = function(id, item) { 
                    return item && this.getValues(item, "children").length;
                };
            };
            
            if (!("getChildren" in patternStore)) {
                patternStore.getChildren = function(item) { 
                    return this.getValues(item, "children");
                };
            };

            return patternStore;
        },
    
        _showConfigPatternsPrefs: function(evt) {
            // tags:
            // protected
            require([ "support/dialogs/ConfigPatterns" ], dojo.hitch(this, function() {
                var dialog = new support.dialogs.ConfigPatterns({
                    id: this.id + "cp_templatestable_configpatternsprefs"
                });
                dialog.show();
            }));
        },

        _loadToolBar: function(grid) {
            // tags:
            // protected
            var toolbar = grid && grid.toolBar && grid.toolBar.widget;
            
            if (toolbar) {
                // Load toolbar buttons/menus
                if (this._patternTable == null) {
                    // Table not created. Nothing to
                    // do but return.
                    return;
                }
    
                if (!this.actionMenu) {
                    this.actionMenu = new dijit.Menu({
                        style: "display: none;"
                    });
                }
                
                // Add custom display config menu
                var displayMenu = new dijit.Menu({
                    style: "display: none;"
                });
    
                // By Type View
                var displayItem1 = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_displayitem1",
                    label: this.resources.BY_TYPE,
                    title: this.locResources.ViewGroup,
                    viewMode: this.BY_ROOT,
                    onClick: dojo.hitch(this, function() {
                        this._setViewByMode(this.BY_ROOT);
                    })
                });
                displayMenu.addChild(displayItem1);
    
                // Flat List View
                var displayItem2 = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_display2",
                    label: this.resources.FLAT,
                    title: this.locResources.ViewList,
                    viewMode: this.FLAT,
                    onClick: dojo.hitch(this, function() {
                        this._setViewByMode(this.FLAT);
                    })
                });
                displayMenu.addChild(displayItem2);
    
                this._displayButton = new dijit.form.DropDownButton({
                    //id: this.id + "cp_templatestable_displaybutton",
                    label: this.resources.BY_TYPE,
                    title: this.resources.TABLE_VIEW,
                    showLabel: true,
                    dropDown: displayMenu
                });
                toolbar.addChild(this._displayButton);
                
                toolbar.addChild(new dijit.ToolbarSeparator());
    
                // Add create menu...
    
                var createMenu = new dijit.Menu({
    //                id: this.id + "_createMenuTemplateTable",
                    //id: "createMenuTemplateTable_"+this.id,
                    style: "display: none;"
                });                                
    
                var newServerTemplate = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newserveritem",
                    label: this.locResources.NewServerTemplate,
                    title: this.locResources.CreateServerTemplate,
                    onClick: dojo.hitch(this, this.createServerTemplate),
                    onKeypress: dojo.hitch(this, this.createServerTemplate)
                });
                createMenu.addChild(newServerTemplate);
    
                var newSystemInformationTemplate = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newsysteminfoitem",
                    label: this.resources.NEW_SYS_INFO_TEMP,
                    title: this.locResources.CreateSystemInfoTemplate,
                    onClick: dojo.hitch(this, this.createSysInfoTemplate),
                    onKeypress: dojo.hitch(this, this.createSysInfoTemplate)
                });
                createMenu.addChild(newSystemInformationTemplate);
    
                var newManagementInterfaceTemplate = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newmanagementinterfaceitem",
                    label: this.resources.NEW_MGMT_TEMP,
                    title: this.locResources.CreateManagementTemplate,
                    onClick: dojo.hitch(this, this.createManagementInterfaceTemplate),
                    onKeypress: dojo.hitch(this, this.createManagementInterfaceTemplate)
                });
                createMenu.addChild(newManagementInterfaceTemplate);
    
                var newPowerStateAndCappingTemplate = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newpowerstateandcappingitem",
                    label: this.locResources.NewPowerTemplate,
                    title: this.locResources.CreatePowerTemplate,
                    onClick: dojo.hitch(this, this.createPowerStateAndCappingTemplate),
                    onKeypress: dojo.hitch(this, this.createPowerStateAndCappingTemplate)
                });
                createMenu.addChild(newPowerStateAndCappingTemplate);
    
                var newPerformanceAndRecoveryTemplate = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newperformanceandrecoveryitem",
                    label: this.locResources.NewPerformanceTemplate,
                    title: this.locResources.CreatePerformanceTemplate,
                    onClick: dojo.hitch(this, this.createPerformanceAndRecoveryTemplate),
                    onKeypress: dojo.hitch(this, this.createPerformanceAndRecoveryTemplate)
                });
                createMenu.addChild(newPerformanceAndRecoveryTemplate);
    
                var newDevicesAndIOPortsTemplate = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newdevicesandioportsitem",
                    label: this.locResources.NewDevicesTemplate,
                    title: this.locResources.CreateDevicesTemplate,
                    onClick: dojo.hitch(this, this.createDevicesAndIOPortsTemplate),
                    onKeypress: dojo.hitch(this, this.createDevicesAndIOPortsTemplate)
                });
                createMenu.addChild(newDevicesAndIOPortsTemplate);
    
                var newBootTargetTemplate = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newboottargetitem",
                    label: this.locResources.NewFibreChannelTemplate,
                    title: this.locResources.CreateFibreChannelTemplate,
                    onClick: dojo.hitch(this, this.createFibreBootTargetTemplate),
                    onKeypress: dojo.hitch(this, this.createFibreBootTargetTemplate)
                });
                createMenu.addChild(newBootTargetTemplate);
    
                var newAdapterTemplate = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newadapteritem",
                    label: this.locResources.NewAdapterTemplate,
                    title: this.locResources.CreateAdapterTemplate,
                    onClick: dojo.hitch(this, this.createAdapterTemplate),
                    onKeypress: dojo.hitch(this, this.createAdapterTemplate)
                });
                createMenu.addChild(newAdapterTemplate);
    
                var newPortVirtualTemplate = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newportvirtualitem",
                    label: this.locResources.NEW_PORT_VIRT_TEMP,
                    title: this.locResources.CreatePortVirtualTemplate,
                    onClick: dojo.hitch(this, this.createPortVirtualTemplate),
                    onKeypress: dojo.hitch(this, this.createPortVirtualTemplate)
                });
                createMenu.addChild(newPortVirtualTemplate);
                                    
                var createButton = new dijit.form.DropDownButton({
                    //id: this.id + "cp_templatestable_createbutton",
                    showLabel: false,
                    iconClass: "spCreateLgIcon",
                    title: this.resources.CREATE_PATTERN_TITLE,
                    dropDown: createMenu
                });
    
                toolbar.addChild(createButton);
                
                var createMenu2 = new dijit.Menu({
                    style: "display: none;"
                });
                
                var newServerTemplate2 = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newserveritem2",
                    label: this.locResources.NewServerTemplate,
                    title: this.locResources.CreateServerTemplate,
                    onClick: dojo.hitch(this, this.createServerTemplate),
                    onKeypress: dojo.hitch(this, this.createServerTemplate)
                });
                createMenu2.addChild(newServerTemplate2);
    
                var newSystemInformationTemplate2 = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newsysteminfoitem2",
                    label: this.resources.NEW_SYS_INFO_TEMP,
                    title: this.locResources.CreateSystemInfoTemplate,
                    onClick: dojo.hitch(this, this.createSysInfoTemplate),
                    onKeypress: dojo.hitch(this, this.createSysInfoTemplate)
                });
                createMenu2.addChild(newSystemInformationTemplate2);
    
                var newManagementInterfaceTemplate2 = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newmanagementinterfaceitem2",
                    label: this.resources.NEW_MGMT_TEMP,
                    title: this.locResources.CreateManagementTemplate,
                    onClick: dojo.hitch(this, this.createManagementInterfaceTemplate),
                    onKeypress: dojo.hitch(this, this.createManagementInterfaceTemplate)
                });
                createMenu2.addChild(newManagementInterfaceTemplate2);
    
                var newPowerStateAndCappingTemplate2 = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newpowerstateandcappingitem2",
                    label: this.locResources.NewPowerTemplate,
                    title: this.locResources.CreatePowerTemplate,
                    onClick: dojo.hitch(this, this.createPowerStateAndCappingTemplate),
                    onKeypress: dojo.hitch(this, this.createPowerStateAndCappingTemplate)
                });
                createMenu2.addChild(newPowerStateAndCappingTemplate2);
    
                var newPerformanceAndRecoveryTemplate2 = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newperformanceandrecoveryitem2",
                    label: this.locResources.NewPerformanceTemplate,
                    title: this.locResources.CreatePerformanceTemplate,
                    onClick: dojo.hitch(this, this.createPerformanceAndRecoveryTemplate),
                    onKeypress: dojo.hitch(this, this.createPerformanceAndRecoveryTemplate)
                });
                createMenu2.addChild(newPerformanceAndRecoveryTemplate2);
    
                var newDevicesAndIOPortsTemplate2 = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newdevicesandioportsitem2",
                    label: this.locResources.NewDevicesTemplate,
                    title: this.locResources.CreateDevicesTemplate,
                    onClick: dojo.hitch(this, this.createDevicesAndIOPortsTemplate),
                    onKeypress: dojo.hitch(this, this.createDevicesAndIOPortsTemplate)
                });
                createMenu2.addChild(newDevicesAndIOPortsTemplate2);
    
                var newBootTargetTemplate2 = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newboottargetitem2",
                    label: this.locResources.NewFibreChannelTemplate,
                    title: this.locResources.CreateFibreChannelTemplate,
                    onClick: dojo.hitch(this, this.createFibreBootTargetTemplate),
                    onKeypress: dojo.hitch(this, this.createFibreBootTargetTemplate)
                });
                createMenu2.addChild(newBootTargetTemplate2);
    
                var newAdapterTemplate2 = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newadapteritem2",
                    label: this.locResources.NewAdapterTemplate,
                    title: this.locResources.CreateAdapterTemplate,
                    onClick: dojo.hitch(this, this.createAdapterTemplate),
                    onKeypress: dojo.hitch(this, this.createAdapterTemplate)
                });
                createMenu2.addChild(newAdapterTemplate2);
    
                var newPortVirtualTemplate2 = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_newportvirtualitem2",
                    label: this.locResources.NEW_PORT_VIRT_TEMP,
                    title: this.locResources.CreatePortVirtualTemplate,
                    onClick: dojo.hitch(this, this.createPortVirtualTemplate),
                    onKeypress: dojo.hitch(this, this.createPortVirtualTemplate)
                });
                createMenu2.addChild(newPortVirtualTemplate2);
    
                // Add custom Edit button...
    
                this.editButton = new dijit.form.Button({
                    //id: this.id + "cp_templatestable_editbutton",
                    showLabel: false,
                    iconClass: "spEditInactiveLgIcon",
                    disabled: true,
                    title: this.resources.EDIT_SELECTED_PATTERN,
                    onClick: dojo.hitch(this, this.editTemplate),
                    onKeypress: dojo.hitch(this, this.editTemplate)
                });
                toolbar.addChild(this.editButton);
    
                // Add custom Delete button...
    
                this.deleteButton = new dijit.form.Button({
                    //id: this.id + "cp_templatestable_deletebutton",
                    showLabel: false,
                    iconClass: "spDeleteInactiveLgIcon",
                    disabled: true,
                    title: this.resources.DELETE_SELECTED_PATTERNS,
                    onClick: dojo.hitch(this, this.deleteTemplates),
                    onKeypress: dojo.hitch(this, this.deleteTemplates)
                });
                toolbar.addChild(this.deleteButton);
                
                toolbar.addChild(new dijit.ToolbarSeparator());
    
                // Add to Actions Menu...
                
                this.expandAllItem = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_expandallitem",
                    label: this.resources.EXPAND_ALL,
                    iconClass: "spExpandAllLgIcon",
                    title: this.resources.EXPAND_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._patternTable.expandAll();
                    }),
                    onKeypress: dojo.hitch(this, function() {
                        this._patternTable.expandAll();
                    })
                });
                this.actionMenu.addChild(this.expandAllItem,0);
    
                this.collapseAllItem = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_collapseallitem",
                    label: this.resources.COLLAPSE_ALL,
                    iconClass: "spCollapseAllLgIcon",
                    title: this.resources.COLLAPSE_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {
                        this._patternTable.collapseAll();
                    }),
                    onKeypress: dojo.hitch(this, function() {
                        this._patternTable.collapseAll();
                    })
                });
                this.actionMenu.addChild(this.collapseAllItem,1);
                
                this.createItem = new dijit.PopupMenuItem({
                    label: this.resources.CREATE,
                    iconClass: "spCreateLgIcon",
                    title: this.resources.CREATE_PATTERN_TITLE,
                    popup: createMenu2
                });
                this.actionMenu.addChild(this.createItem);
                
                var menuSeparator = new dijit.MenuSeparator();
                this.actionMenu.addChild(menuSeparator);
    
                this.detailsItem = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_detailsitem",
                    label: this.resources.DETAILS,
                    title: this.resources.DETAILS_SELECTED_PATTERN,
                    onClick: dojo.hitch(this, this.detailTemplate),
                    onKeypress: dojo.hitch(this, this.detailTemplate)
                });
    
                this.renameItem = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_renameitem",
                    label: this.resources.RENAME,
                    title: this.resources.RENAME_SELECTED_PATTERN,
                    onClick: dojo.hitch(this, this.renameTemplate),
                    onKeypress: dojo.hitch(this, this.renameTemplate)
                });
    
                this.copyItem = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_copyitem",
                    label: this.resources.COPY,
                    title: this.resources.COPY_SELECTED_PATTERN,
                    onClick: dojo.hitch(this, this.copyTemplate),
                    onKeypress: dojo.hitch(this, this.copyTemplate)
                });
    
                this.deployItem = new dijit.MenuItem({
                    //id: this.id + "cp_templatestable_deployitem",
                    label: this.resources.DEPLOY,
                    title: this.resources.DEPLOY_SELECTED_SERVER_PATTERN,
                    iconClass: "spDeployLgIcon",
                    onClick: dojo.hitch(this, this.deployTemplate),
                    onKeypress: dojo.hitch(this, this.deployTemplate)
                });
    
                //Add action menu to action button and create action button if not already created
                if (this._actionButton) {
                    this._actionButton.dropDown = this.actionMenu;
                }
                else {
                    this._actionButton = new dijit.form.DropDownButton({
                        //id: this.id + "cp_templatestable_actionbutton",
                        label: this.resources.ALL_ACTIONS,
                        title: this.resources.ALL_ACTIONS_TITLE,
                        showLabel: true,
                        dropDown: this.actionMenu
                    });   
                    
                    toolbar.addChild(this._actionButton);
                }
                
            }
        },

        // Functions to open dialogs
        createPowerStateAndCappingTemplate: function() {
            require([ "support/dialogs/PowerStateAndCappingTemplate" ], dojo.hitch(this, function() {
                var pscdialog = new support.dialogs.PowerStateAndCappingTemplate({
                    id: this.id + "cp_templatestable_powerstateandcapping"
                });
                pscdialog.show();

                return false;
            }));
        },

        createPXEBootTargetTemplate: function() {
            require([ "support/dialogs/PXENetworkBootTargetTemplate" ], dojo.hitch(this, function() {
                var pxedialog = new support.dialogs.PXENetworkBootTargetTemplate({
                    id: this.id + "cp_templatestable_pxenetworkboottarget"
                });
                pxedialog.show();

                return false;
            }));
        },

        createFibreBootTargetTemplate: function() {
            require([ "support/dialogs/FibreChannelBootTargetTemplate" ], dojo.hitch(this, function() {
                var fibreChanneldialog = new support.dialogs.FibreChannelBootTargetTemplate({
                    id: this.id + "cp_templatestable_fibrechannelboottarget"
                });
                fibreChanneldialog.show();

                return false;
            }));
        },

        createSASBootTargetTemplate: function() {
            require([ "support/dialogs/SASBootTargetTemplate" ], dojo.hitch(this, function() {
                var sasdialog = new support.dialogs.SASBootTargetTemplate({
                    id: this.id + "cp_templatestable_sasboottarget"
                });
                sasdialog.show();

                return false;
            }));
        },

        createSysInfoTemplate: function() {
            require([ "support/dialogs/SystemInfoTemplate" ], dojo.hitch(this, function() {
                var sitdialog = new support.dialogs.SystemInfoTemplate({
                    id: this.id + "cp_templatestable_systeminfo"
                });
                sitdialog.show();

                return false;
            }));
        },

        createPerformanceAndRecoveryTemplate: function() {
            require([ "support/dialogs/PerformanceAndRecoveryTemplate" ], dojo.hitch(this, function() {
                var partdialog = new support.dialogs.PerformanceAndRecoveryTemplate({
                    id: this.id + "cp_templatestable_performanceandrecovery"
                });
                partdialog.show();

                return false;
            }));
        },

        createDevicesAndIOPortsTemplate: function() {
            require([ "support/dialogs/DevicesAndIOPortsTemplate" ], dojo.hitch(this, function() {
                var daiptdialog = new support.dialogs.DevicesAndIOPortsTemplate({
                    id: this.id + "cp_templatestable_devicesandioports"
                });
                daiptdialog.show();

                return false;
            }));
        },

        createManagementInterfaceTemplate: function() {
            require([ "support/dialogs/ManagementInterfaceTemplate" ], dojo.hitch(this, function() {
                var mitdialog = new support.dialogs.ManagementInterfaceTemplate({
                    id: this.id + "cp_templatestable_managementinterface"
                });
                mitdialog.show();

                return false;
            }));
        },

        createAdapterTemplate: function() {
            require([ "support/dialogs/NewAdapterTemplate" ], dojo.hitch(this, function() {
                var natdialog = new support.dialogs.NewAdapterTemplate({
                    id: this.id + "cp_templatestable_newadapter",
                    adapterPorts: 4,
                    adapterType: "unknown"
                });
                natdialog.show();

                return false;
            }));
        },

        createPortVirtualTemplate: function() {
            require([ "support/dialogs/NewPortVirtualizationTemplate" ], dojo.hitch(this, function() {
                var npvtdialog = new support.dialogs.NewPortVirtualizationTemplate({
                    id: this.id + "cp_templatestable_newportvirtualization"
                });
                npvtdialog.show();

                return false;
            }));
        },

        createPortPhysicalTemplate: function() {
            require([ "support/dialogs/NewPortPhysicalFunctionTemplate" ], dojo.hitch(this, function() {
                var nppfdialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                    id: this.id + "cp_templatestable_newportphysicalfunction",
                    nodeType: "sysx"
                });
                nppfdialog.show();

                return false;
            }));
        },

        createServerTemplate: function() {
            require([ "support/dialogs/ServerTemplateWizard" ], dojo.hitch(this, function() {
                var stwdialog = new support.dialogs.ServerTemplateWizard({
                    templateIDNum: null,
                    editMode: false
                });

                stwdialog.show();
            }));
        },

        // Deploy Template
        deployTemplate: function() {

            if (this._selectedTemplate.id.toString() != null && this._selectedTemplate.id.toString() != "") {
                require([ "support/dialogs/DeployServerTemplate" ], dojo.hitch(this, function() {
                    var dst = new support.dialogs.DeployServerTemplate({
                        templateId: this._selectedTemplate.id.toString()
                    });
                    dst.show();
                }));
            }

        },

        // Details Template
        detailTemplate: function() {

            if (this._selectedTemplate.templateType.toString() == "Server") {
                
                require([ "support/panels/ServerTemplateDetails" ], dojo.hitch(this,function() {
                
                    if(this.CP_RUN_ENVIRONMENT == "fsm"){

                        this.loadNGUIWorkAreaWithCPWidget("support/panels/ServerTemplateDetails", null, this._selectedTemplate.id
                                .toString());

                    }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                        var launchParams = {templateIDNum: this._selectedTemplate.id.toString()};
                        
                        this.loadAugustaWorkAreaTask("config.detail.page","support/panels/ServerTemplateDetails",launchParams);
                        
                    }
                    
               }));

            }
            else {

                require([ "support/panels/CategoryTemplateDetails" ], dojo.hitch(this, function() {

                    if(this.CP_RUN_ENVIRONMENT == "fsm"){

                        this.loadNGUIWorkAreaWithCPWidget("support/panels/CategoryTemplateDetails", null, this._selectedTemplate.id
                            .toString());
                    
                    }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                        var launchParams = {templateIDNum: this._selectedTemplate.id.toString()};
                        
                        this.loadAugustaWorkAreaTask("config.detail.page","support/panels/CategoryTemplateDetails",launchParams);
                        
                    }
                    
                }));
            }
        },

        // Edit Template
        editTemplate: function() {

            if (this._selectedTemplate.templateType.toString() == "Server") {
                require([ "support/dialogs/ServerTemplateWizard" ], dojo.hitch(this, function() {
                    var stweditdialog = new support.dialogs.ServerTemplateWizard({
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateInUse: (this._selectedTemplate.inUse.toString() == "true") ? true : false,
                        editMode: true
                    });
                    stweditdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "Power") {
                require([ "support/dialogs/PowerStateAndCappingTemplate" ], dojo.hitch(this, function() {
                    var psceditdialog = new support.dialogs.PowerStateAndCappingTemplate({
                        id: this.id + "cp_templatestable_powerstateandcapping",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit",
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint: "table"
                    });
                    psceditdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "PXE") {
                require([ "support/dialogs/PXENetworkBootTargetTemplate" ], dojo.hitch(this, function() {
                    var pxedialog = new support.dialogs.PXENetworkBootTargetTemplate({
                        id: this.id + "cp_templatestable_pxenetworkboottarget",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit",
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint: "table"
                    });
                    pxedialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "FibreChannel") {
                require([ "support/dialogs/FibreChannelBootTargetTemplate" ], dojo.hitch(this, function() {
                    var fibreChanneldialog = new support.dialogs.FibreChannelBootTargetTemplate({
                        id: this.id + "cp_templatestable_fibrechannelboottarget",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit",
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint: "table"
                    });
                    fibreChanneldialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "SAS") {
                require([ "support/dialogs/SASBootTargetTemplate" ], dojo.hitch(this, function() {
                    var sasdialog = new support.dialogs.SASBootTargetTemplate({
                        id: this.id + "cp_templatestable_sasboottarget",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit",
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint: "table"
                    });
                    sasdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "SystemInfo") {
                require([ "support/dialogs/SystemInfoTemplate" ], dojo.hitch(this, function() {
                    var sitdialog = new support.dialogs.SystemInfoTemplate({
                        id: this.id + "cp_templatestable_systeminfo",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit",
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint: "table"
                    });
                    sitdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "PerformanceRecovery") {
                require([ "support/dialogs/PerformanceAndRecoveryTemplate" ], dojo.hitch(this, function() {
                    var partdialog = new support.dialogs.PerformanceAndRecoveryTemplate({
                        id: this.id + "cp_templatestable_performanceandrecovery",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit",
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint: "table"
                    });
                    partdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "DevicesAndIOPorts") {
                require([ "support/dialogs/DevicesAndIOPortsTemplate" ], dojo.hitch(this, function() {
                    var daiptdialog = new support.dialogs.DevicesAndIOPortsTemplate({
                        id: this.id + "cp_templatestable_devicesandioports",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit",
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint: "table"
                    });
                    daiptdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "Management") {
                require([ "support/dialogs/ManagementInterfaceTemplate" ], dojo.hitch(this, function() {
                    var miteditdialog = new support.dialogs.ManagementInterfaceTemplate({
                        id: this.id + "cp_templatestable_managementinterface",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit",
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint: "table"
                    });
                    miteditdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "Adapter") {
                require([ "support/dialogs/NewAdapterTemplate" ], dojo.hitch(this, function() {
                    var atDialog = new support.dialogs.NewAdapterTemplate({
                        id: this.id + "cp_templatestable_newadapter",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit",
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint: "table"
                    });
                    atDialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "PortVirtualization") {
                require([ "support/dialogs/NewPortVirtualizationTemplate" ], dojo.hitch(this, function() {
                    var pvtDialog = new support.dialogs.NewPortVirtualizationTemplate({
                        id: this.id + "cp_templatestable_newportvirtualization",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit",
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint: "table"
                    });
                    pvtDialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "PortPhysicalFunction") {
                require([ "support/dialogs/NewPortPhysicalFunctionTemplate" ], dojo.hitch(this, function() {
                    var pptDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                        id: this.id + "cp_templatestable_newportphysicalfunction",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "edit",
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint: "table"
                    });
                    pptDialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "ExtendedIMM") {
                require([ "support/dialogs/NewLearnedSettingsTemplate" ], dojo.hitch(this, function() {
                    var eitDialog = new support.dialogs.NewLearnedSettingsTemplate({
                        id: this.id + "cp_templatestable_newlearnedsettings",
                        type: "imm",
                        templateMode: "edit",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateInUse: this._selectedTemplate.inUse.toString().toString(),
                        templateLaunchPoint: "table"
                    });

                    eitDialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "ExtendedUEFI") {
                require([ "support/dialogs/NewLearnedSettingsTemplate" ], dojo.hitch(this, function() {
                    var eutDialog = new support.dialogs.NewLearnedSettingsTemplate({
                        id: this.id + "cp_templatestable_newlearnedsettings",
                        type: "uefi",
                        templateMode: "edit",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint: "table"
                    });

                    eutDialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "ExtendedPort") {
                require([ "support/dialogs/NewLearnedSettingsTemplate" ], dojo.hitch(this, function() {
                    var eutDialog = new support.dialogs.NewLearnedSettingsTemplate({
                        id: this.id + "cp_templatestable_newlearnedsettings",
                        type: "port",
                        templateMode: "edit",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateInUse: this._selectedTemplate.inUse.toString(),
                        templateLaunchPoint: "table"
                    });

                    eutDialog.show();
                }));
            }

            return false;
        },

        // Copy Template
        copyTemplate: function() {

            if (this._selectedTemplate.templateType.toString() == "Server") {
                require([ "support/dialogs/ServerTemplateWizard" ], dojo.hitch(this, function() {
                    var stweditdialog = new support.dialogs.ServerTemplateWizard({
                        templateIDNum: this._selectedTemplate.id.toString(),
                        copyMode: true
                    });
                    stweditdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "Power") {
                require([ "support/dialogs/PowerStateAndCappingTemplate" ], dojo.hitch(this, function() {
                    var psceditdialog = new support.dialogs.PowerStateAndCappingTemplate({
                        id: this.id + "cp_templatestable_powerstateandcapping",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    psceditdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "PXE") {
                require([ "support/dialogs/PXENetworkBootTargetTemplate" ], dojo.hitch(this, function() {
                    var pxedialog = new support.dialogs.PXENetworkBootTargetTemplate({
                        id: this.id + "cp_templatestable_pxenetworkboottarget",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    pxedialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "FibreChannel") {
                require([ "support/dialogs/FibreChannelBootTargetTemplate" ], dojo.hitch(this, function() {
                    var fibreChanneldialog = new support.dialogs.FibreChannelBootTargetTemplate({
                        id: this.id + "cp_templatestable_fibrechannelboottarget",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    fibreChanneldialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "SAS") {
                require([ "support/dialogs/SASBootTargetTemplate" ], dojo.hitch(this, function() {
                    var sasdialog = new support.dialogs.SASBootTargetTemplate({
                        id: this.id + "cp_templatestable_sasboottarget",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    sasdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "SystemInfo") {
                require([ "support/dialogs/SystemInfoTemplate" ], dojo.hitch(this, function() {
                    var sitdialog = new support.dialogs.SystemInfoTemplate({
                        id: this.id + "cp_templatestable_systeminfo",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    sitdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "PerformanceRecovery") {
                require([ "support/dialogs/PerformanceAndRecoveryTemplate" ], dojo.hitch(this, function() {
                    var partdialog = new support.dialogs.PerformanceAndRecoveryTemplate({
                        id: this.id + "cp_templatestable_performanceandrecovery",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    partdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "DevicesAndIOPorts") {
                require([ "support/dialogs/DevicesAndIOPortsTemplate" ], dojo.hitch(this, function() {
                    var daiptdialog = new support.dialogs.DevicesAndIOPortsTemplate({
                        id: this.id + "cp_templatestable_devicesandioports",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    daiptdialog.show();

                    // this.partDialog.set("templateIDNum",this._selectedTemplate.id.toString());
                    // this.partDialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "Management") {
                require([ "support/dialogs/ManagementInterfaceTemplate" ], dojo.hitch(this, function() {
                    var miteditdialog = new support.dialogs.ManagementInterfaceTemplate({
                        id: this.id + "cp_templatestable_managementinterface",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    miteditdialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "Adapter") {
                require([ "support/dialogs/NewAdapterTemplate" ], dojo.hitch(this, function() {
                    var atDialog = new support.dialogs.NewAdapterTemplate({
                        id: this.id + "cp_templatestable_newadapter",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    atDialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "PortVirtualization") {
                require([ "support/dialogs/NewPortVirtualizationTemplate" ], dojo.hitch(this, function() {
                    var pvtDialog = new support.dialogs.NewPortVirtualizationTemplate({
                        id: this.id + "cp_templatestable_newportvirtualization",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    pvtDialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "PortPhysicalFunction") {
                require([ "support/dialogs/NewPortPhysicalFunctionTemplate" ], dojo.hitch(this, function() {
                    var pptDialog = new support.dialogs.NewPortPhysicalFunctionTemplate({
                        id: this.id + "cp_templatestable_newportphysicalfunction",
                        templateIDNum: this._selectedTemplate.id.toString(),
                        templateMode: "copy"
                    });
                    pptDialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "ExtendedIMM") {
                require([ "support/dialogs/NewLearnedSettingsTemplate" ], dojo.hitch(this, function() {
                    var eitDialog = new support.dialogs.NewLearnedSettingsTemplate({
                        id: this.id + "cp_templatestable_newlearnedsettings",
                        type: "imm",
                        templateMode: "copy",
                        templateIDNum: this._selectedTemplate.id.toString()
                    });

                    eitDialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "ExtendedUEFI") {
                require([ "support/dialogs/NewLearnedSettingsTemplate" ], dojo.hitch(this, function() {
                    var eutDialog = new support.dialogs.NewLearnedSettingsTemplate({
                        id: this.id + "cp_templatestable_newlearnedsettings",
                        type: "uefi",
                        templateMode: "copy",
                        templateIDNum: this._selectedTemplate.id.toString()
                    });

                    eutDialog.show();
                }));
            }
            else if (this._selectedTemplate.templateType.toString() == "ExtendedPort") {
                require([ "support/dialogs/NewLearnedSettingsTemplate" ], dojo.hitch(this, function() {
                    var eutDialog = new support.dialogs.NewLearnedSettingsTemplate({
                        id: this.id + "cp_templatestable_newlearnedsettings",
                        type: "port",
                        templateMode: "copy",
                        templateIDNum: this._selectedTemplate.id.toString()
                    });

                    eutDialog.show();
                }));
            }

            return false;
        },

        // Rename Template
        renameTemplate: function() {
            require([ "support/dialogs/Rename" ], dojo.hitch(this, function() {
                var renamedialog = new support.dialogs.Rename({
                    id: this.id + "cp_templatestable_rename",
                    templateIDNum: this._selectedTemplate.id.toString()
                });
                renamedialog.show();
            }));
        },

        // Delete Templates
        deleteTemplates: function() {

            var deleteIds = new Array();
            var deleteTypes = new Array();
            var htmlNames = new Array();
            var ids = this._patternTable.select.row.getSelected();
            for (var loopVar = 0; loopVar < ids.length; ++loopVar) {
                var item = this._patternTable.row(ids[loopVar], true).item();
                deleteIds.push(item.id);
                (dojo.indexOf(deleteTypes, item.type) < 0) && deleteTypes.push(item.type); // don't push dups
                var htmlName = this.encodeForHtml(item.name.toString());
                htmlNames.push(htmlName);
            };
            var processDeleteRequest = function() {
                //console.log("deleteIds=", deleteIds);
                var responses = new Array();
                this.getProgressDialog("delete", false).show(); // Show progress for delete request(s).
                dojo.forEach(deleteIds, dojo.hitch(this, function(id) {
                    // NOTE: The following spawns 1 async request for each of the patterns
                    // selected. This is potentially a multi-threaded process so be sure to
                    // handle accordngly.
                    //console.log("item", idx, id);

                    dojo.xhrDelete({
                        url: this.REST_PATH + "template/" + id,
                        handleAs: "json",
                        headers: {
                            "Content-Type": "application/json; charset=utf-8"
                        },
                        error: dojo.hitch(this, function(response, ioArgs) {
                            var msg = "";
                            if (ioArgs.xhr.status == 400) {
                                msg = this.resources.ERROR_PATTERN_DELETE_INUSE;
                            }
                            else {
                                msg = this.resources.ERROR_PATTERN_DELETE;
                            }
                            this.getModalDialog("Error", msg).show();

                        }),
                        
                        handle: dojo.hitch(this, function(response, ioArgs) {
                            //console.log("Delete template complete: ", response, ioArgs);
                            if (responses.push(response) < deleteIds.length) {
                                // Not the final request. Keep the user waiting.
                                return;
                            }
                            // Proceed...                            
                            dojo.publish("ProgressComplete", response);
                            dojo.publish("ServerTemplateDeleted", response);
                          
                            this._refreshTree();                            
                        })
                    });
                }));
            };



            if(this.CP_RUN_ENVIRONMENT == "fsm"){

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new support.temp.ConfirmationDialog({
                        buttonLabel: this.resources.DELETE,
                        type: "Warning",
                        text: this.resources.CONFIRM_DELETE_TEMP,
                        info: this.locResources.DELETEINFO                    
                    });

                    confirmDialog.set("content", "<p>" + this.locResources.DeleteUnused.replace("%1", htmlNames.join(", ")) + "</p>");
                    confirmDialog.startup();
                    confirmDialog.confirm(processDeleteRequest, this);
                
                }));
 
                
            }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                    var confirmDialog = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources.DELETE,
                        type: "Warning",
                        text: this.resources.CONFIRM_DELETE_TEMP,
                        info: this.locResources.DELETEINFO                    
                    });

                    confirmDialog.set("content", "<p>" + this.locResources.DeleteUnused.replace("%1", htmlNames.join(", ")) + "</p>");
                    confirmDialog.startup();
                    confirmDialog.confirm(processDeleteRequest, this);
                
                }));
                
            }

            
            return false;
        },

        // Set Table Formatters
        _formatFormFactor: function(ff) {
            switch(ff.toString()) {
                case "1sd":
                    return this.resources.FF_BAY_DUAL;
                case "2s":
                    return this.resources.FF_BAY.replace("%1", "1");
                case "2sc":
                    return this.resources.FF_SCALABLE.replace("%1", "2");
                case "4s":
                    return this.resources.FF_BAY.replace("%1", "2");
                case "4sc":
                    return this.resources.FF_SCALABLE.replace("%1", "4");
                case "8sc":
                    return this.resources.FF_SCALABLE.replace("%1", "8");
                case "pxu":
                    return this.resources.FF_PXU;
                default:
                    return "";
            }
        },
        
        _decorateName: function(item, id) {
            if (!item || !id) {
                return "";
            }
                        
            var value = item;
            var rowItem = this._patternTable.row(id, true).item();
            
            if(id == "Server" || id == "Adapter" || id == "PortVirtualization" || id == "SystemInfo" 
                || id == "Management" || id == "Power" || id == "PerformanceRecovery"
                    || id == "DevicesAndIOPorts" || id == "FibreChannel" || id == "ExtendedIMM"
                        || id == "ExtendedUEFI" || id == "ExtendedPort"){                
                return "<span title=\"" + value + "\">" + value + "</span>";
            } 
            if(this.CP_RUN_ENVIRONMENT == "fsm"){
                if (rowItem.type=="Server"){
                    return "<a href=\"#workAreaDijit=support%2Fpanels%2FServerTemplateDetails&path=support:%2Fcom.ibm.stg.sysx.profile." +
                            "navmodel%2Fjs%2Fsupport&cpTarget=" + id + "\" title=\"" + this.resources.LAUNCH_DETAILS + "\">" + value + "</a>";
                } else {
                    return "<a href=\"#workAreaDijit=support%2Fpanels%2FCategoryTemplateDetails&path=support:%2Fcom.ibm.stg.sysx.profile." +
                            "navmodel%2Fjs%2Fsupport&cpTarget=" + id + "\" title=\"" + this.resources.LAUNCH_DETAILS + "\">" + value + "</a>";
                }
            } else if(this.CP_RUN_ENVIRONMENT == "augusta"){
                if (rowItem.type=="Server"){
                    return "<a href=\"#fwWidget=support%2Fpanels%2FServerTemplateDetails&fwTaskId=config.detail.page&templateIDNum=" 
                        + id + "\" title=\"" + this.resources.LAUNCH_DETAILS + "\">" + value + "</a>";
                } else {
                    return "<a href=\"#fwWidget=support%2Fpanels%2FCategoryTemplateDetails&fwTaskId=config.detail.page&templateIDNum=" 
                        + id + "\" title=\"" + this.resources.LAUNCH_DETAILS + "\">" + value + "</a>";
                }
            }
        },

        // Hard-coded path appended to URL, must be changed if path changes
        _formatName: function(item) {
            // tags:
            // protected            
            if (!item || !item.name) {
                return "";
            }
            
            var value = this.decodeSpecialCharacters(item.name);
            value = this.encodeForHtml(value);                        
            
            return value;            
        },
        
        _decorateInUse: function(item) {
            // tags:
            // protected      
            if (item == "") {
                return "";
            }

            if (item == this.resources.DEPLOYED) {
                return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr title=\"" + this.locResources.TemplateInUseText + "\">" +
                    "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spDeployedIcon\"></img></td>" +
                    "<td><span style=\"margin-left:4px; white-space: nowrap;\">" + 
                    this.resources.DEPLOYED + "</span></td></tr></tbody></table>";
            }else if(item == this.resources.REFERENCED){
                return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr title=\"" + this.locResources.TemplateReferenced + "\">" +
                    "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spReferencedIcon\"></img></td>" +
                    "<td><span style=\"margin-left:4px; white-space: nowrap;\">" + 
                    this.resources.REFERENCED + "</span></td></tr></tbody></table>";
            }
            return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr title=\"" + this.locResources.TemplateNotInUseText + "\">" +
                "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spNotInUseIcon\"></img></td>" +
                "<td><span style=\"margin-left:4px; white-space: nowrap;\">" + 
                this.resources.NOT_IN_USE + "</span></td></tr></tbody></table>";
        },

        _formatInUse: function(item) {
            // tags:
            // protected      
            var inUseValue = (item && item.inUse) ? item.inUse : false;
            var refValue = (item && item.referencedBy) ? true : false;                 
            
            if (item.type == "root" && this.viewMode == this.BY_ROOT) {
                return "";
            }

            if (inUseValue) {
                return this.resources.DEPLOYED;
            }else if(refValue){
                return this.resources.REFERENCED;
            }
            return this.resources.NOT_IN_USE;
        },

        _decorateUserDefined: function(item) {
            // tags:
            // protected
            if (item == "") {
                return "";
            }
            
            if (item == this.resources.USER_DEFINED) {
                return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr title=\"" + this.resources.USER_DEFINED_TEMPLATE + "\">" +
                    "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spUserDefinedIcon\"></img></td>" +
                    "<td><span style=\"margin-left:4px; white-space: nowrap;\">" + 
                    this.resources.USER_DEFINED + "</span></td></tr></tbody></table>";
            }
            return "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tbody><tr title=\"" + this.resources.PREDEFINED_TEMPLATE+ "\">" +
                "<td><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spPredefinedIcon\"></img></td>" +
                "<td><span style=\"margin-left:4px; white-space: nowrap;\">" + 
                this.resources.PREDEFINED + "</span></td></tr></tbody></table>";
        },
        
        _formatUserDefined: function(item) {
            var value = (item && item.userDefined) ? item.userDefined : false;
            
            if (item.type == "root" && this.viewMode == this.BY_ROOT) {
                return "";
            }
            
            if (value) {
                return this.resources.USER_DEFINED;
            }
            return this.resources.PREDEFINED;
        },
        
        _decorateDescription: function(item) {
            return "<span title=\"" + item + "\" style=\"white-space: normal;\">" + item + "</span>";
        },

        _formatDescription: function(item) {
            // tags:
            // protected
            if (!item || !item.description) {
                return "";
            }
            var value = this.decodeSpecialCharacters(item.description);
            value = this.encodeForHtml(value);
            return value;
        },
        
        _decorateTemplateType: function(item) {
            return "<span title=\"" + item + "\" style=\"white-space: nowrap;\">" + item + "</span>";
        },

        _formatTemplateType: function(item) {
            // tags:
            // protected
            if (!item || !item.templateType) {
                return "";
            }
            var value = item.templateType;

            switch (value) {
                case "Server":
                    value = this.resources.SERVER;
                    break;
                case "SystemInfo":
                    value = this.resources.SERVER_INFO;
                    break;
                case "Management":
                    value = this.resources.SERVER_MGMT;
                    break;
                case "Power":
                    value = this.resources.SERVER_POWER;
                    break;
                case "PerformanceRecovery":
                    value = this.resources.SERVER_PERFORM;
                    break;
                case "DevicesAndIOPorts":
                    value = this.resources.SERVER_DEVICE;
                    break;
                case "PXE":
                    value = this.resources.SERVER_PXE;
                    break;
                case "FibreChannel":
                    value = this.resources.SERVER_FIBRE;
                    break;
                case "SAS":
                    value = this.resources.SERVER_SAS;
                    break;
                case "Adapter":
                    value = this.resources.SERVER_ADAPTER;
                    break;
                case "PortVirtualization":
                    value = this.resources.SERVER_PORT;
                    break;
                case "PortPhysicalFunction":
                    value = this.resources.SERVER_PORTPF;
                    break;
                case "ExtendedIMM":
                    value = this.resources.SERVER_EXIMM;
                    break;
                case "ExtendedUEFI":
                    value = this.resources.SERVER_EXUEFI;
                    break;
                case "ExtendedPort":
                    value = this.resources.SERVER_EXPORT;
                    break;
                default:
                    value = "";
                    break;
            }
            return value;
        },

        // Set Table Capabilities
        setTableCapabilities: function() {

            if (!this._patternTable) {
                return; // nothing to do. return now
            }            

            this.editButton.set("iconClass", "spEditInactiveLgIcon");
            this.editButton.set("disabled", true);
            this.deleteButton.set("iconClass", "spDeleteInactiveLgIcon");
            this.deleteButton.set("disabled", true);
            
            var ids = this._patternTable.select.row.getSelected();
            
            var menuPosition = 2;
            
            var allowEdit = (ids.length == 1); // assume okay, disprove below
            var allowDelete = (ids.length > 0); // assume okay, disprove below
            var allowDeploy = (ids.length == 1); // assume okay, disprove below
            var allowRename = (ids.length == 1); // assume okay, disprove below
            for (var loopVar = 0; loopVar < ids.length; ++loopVar) {

                var item = this._patternTable.row(ids[loopVar], true).item();
                this._selectedTemplate = item;
                
                if ((item.type == "root") && (this.viewMode != this.FLAT)) {
                    // Root selection not allowed.
                    allowEdit = allowDelete = allowDeploy = allowRename = false;
                }
                if (item.userDefined != "true") {
                    allowDelete = allowRename = false;
                }
                if (item.inUse == "true" || item.referencedBy == null 
                        || item.referencedBy.length > 0) {
                    allowDelete = false;
                }                                
                
            };

            if (allowEdit) {
                this.editButton.set("iconClass", "spEditLgIcon");
                this.editButton.set("disabled", false);

                if (!this.detailsItem) {
                    this.detailsItem = new dijit.MenuItem({
                   //     id: this.id + "cp_templatestable_detailsitem",
                        label: this.resources.DETAILS,
                        title: this.resources.DETAILS_SELECTED_PATTERN,
                        onClick: dojo.hitch(this, this.detailTemplate),
                        onKeypress: dojo.hitch(this, this.detailTemplate)
                    });
                }

                if (!this.copyItem) {
                    this.copyItem = new dijit.MenuItem({
                    //    id: this.id + "cp_templatestable_copyitem",
                        label: this.resources.COPY,
                        title: this.resources.COPY_SELECTED_PATTERN,
                        onClick: dojo.hitch(this, this.copyTemplate),
                        onKeypress: dojo.hitch(this, this.copyTemplate)
                    });
                }

                if (allowRename && !this.renameItem) {
                    this.renameItem = new dijit.MenuItem({
                    //    id: this.id + "cp_templatestable_renameitem",
                        label: this.resources.RENAME,
                        title: this.resources.RENAME_SELECTED_PATTERN,
                        onClick: dojo.hitch(this, this.renameTemplate),
                        onKeypress: dojo.hitch(this, this.renameTemplate)
                    });
                }

                if (!this.editItem) {
                    this.editItem = new dijit.MenuItem({
                     //   id: this.id + "cp_templatestable_edititem",
                        label: this.resources.EDIT,
                        title: this.resources.EDIT_SELECTED_PATTERN,
                        iconClass: "spEditLgIcon",
                        onClick: dojo.hitch(this, this.editTemplate),
                        onKeypress: dojo.hitch(this, this.editTemplate)
                    });
                }

                if (allowDelete && !this.deleteItem) {
                    this.deleteItem = new dijit.MenuItem({
                     //   id: this.id + "cp_templatestable_deleteitem",
                        label: this.resources.DELETE,
                        title: this.resources.DELETE_SELECTED_PATTERNS,
                        iconClass: "spDeleteLgIcon",
                        onClick: dojo.hitch(this, this.deleteTemplates),
                        onKeypress: dojo.hitch(this, this.deleteTemplates)
                    });
                }              
                
                // Add items to actions menu
                this.actionMenu.addChild(this.editItem, ((this.viewMode == this.FLAT)? menuPosition : menuPosition+2));                
                this.actionMenu.addChild(this.copyItem);
                this.actionMenu.addChild(this.detailsItem);
                menuPosition++;
                if (allowDelete) {
                    this.actionMenu.addChild(this.deleteItem, ((this.viewMode == this.FLAT)? menuPosition : menuPosition+2));
                    menuPosition++;
                }
                else if (this.deleteItem) {
                    this.actionMenu.removeChild(this.deleteItem);
                    this.deleteItem = null;
                }
                if (allowRename) {
                    this.actionMenu.addChild(this.renameItem);
                }
                else if (this.renameItem) {
                    this.actionMenu.removeChild(this.renameItem);
                    this.renameItem = null;
                }                

            }
            else {

                this.actionMenu.removeChild(this.detailsItem);
                this.detailsItem = null;

                this.actionMenu.removeChild(this.copyItem);
                this.copyItem = null;

                this.actionMenu.removeChild(this.renameItem);
                this.renameItem = null;
                
                this.actionMenu.removeChild(this.editItem);
                this.editItem = null;
                
                if (!allowDelete && this.deleteItem) {
                    this.actionMenu.removeChild(this.deleteItem);
                    this.deleteItem = null;
                }

            }

            if (allowDelete) {
                this.deleteButton.set("iconClass", "spDeleteLgIcon");
                this.deleteButton.set("disabled", false);
            }

            if (allowDeploy && (this._selectedTemplate.templateType.toString() === "Server")) {

                if (!this.deployItem) {

                    this.deployItem = new dijit.MenuItem({
                      //  id: this.id + "cp_templatestable_deployitem",
                        label: this.resources.DEPLOY,
                        title: this.resources.DEPLOY_SELECTED_SERVER_PATTERN,
                        iconClass: "spDeployLgIcon",
                        onClick: dojo.hitch(this, this.deployTemplate),
                        onKeypress: dojo.hitch(this, this.deployTemplate)
                    });

                }

                this.actionMenu.addChild(this.deployItem, ((this.viewMode == this.FLAT)? menuPosition : menuPosition+2));
                menuPosition++;
            }
            else {
                this.actionMenu.removeChild(this.deployItem);
                this.deployItem = null;
            }

            //Disable Action Button if no children
            if(this.actionMenu.hasChildren()){
                this._actionButton.set("disabled", false);
            }else{
                this._actionButton.set("disabled", true);                
            }

        },

        _setViewByMode: function(menuItem) {
            // tags:
            // protected
            if (this.viewMode == menuItem) {
                return;
            }
                        
            // Clear selected rows when switching views
            this._patternTable.select.row.clear();
            
            // Set Menu Button Label To Current View
            var label = "";
            if (menuItem == this.BY_ROOT) {
                label = this.resources.BY_TYPE;
                this.actionMenu.addChild(this.expandAllItem, 0);
                this.actionMenu.addChild(this.collapseAllItem, 1);
                this._patternTable.hideFilter(); // Hide filter controls on tree view.
            }
            else if (menuItem == this.FLAT) {
                label = this.resources.FLAT;                
                this.actionMenu.removeChild(this.expandAllItem);
                this.actionMenu.removeChild(this.collapseAllItem);
                this._patternTable.showFilter(); // Show filter controls on flat view.
            }
            else { // Invalid view mode
                return;
            }

            // Disable Action Button if no children
            if (this.actionMenu.hasChildren()) {
                this._actionButton.set("disabled", false);
            } else {
                this._actionButton.set("disabled", true);                
            }

            // Reformat Table Data
            this._reRenderTable(menuItem);

            // Set Drop Down Menu Label
            this._displayButton.set("label", label);

            this.viewMode = menuItem;

        },

        _reRenderTable: function(nextViewMode) {
            // tags:
            // protected
            // Re-render Tree table with new columns, column spans, and data
            // presentation
            this.viewMode = nextViewMode;
            
            var patternStore = this._retrieveStore();
            var tableStructure = this._getTableStructure(); // Get structure for current view
            
            this._patternTable.setColumns(tableStructure);
            this._patternTable.setStore(patternStore);
            this._patternTable.model.clearCache();
            this._patternTable.body.refresh();

            if (this.viewMode == this.BY_ROOT) {
                this._patternTable.reexpandRows();
            }

        },

        _getTableStructure: function() {
            // tags:
            // protected            
            var includeType = (this.viewMode == this.FLAT);
            
            var layout = new Array();
            layout.push({
                field: "name",
                name: this.resources.NAME,
                width: "25em",
                formatter: dojo.hitch(this, this._formatName),
                decorator: dojo.hitch(this, this._decorateName),
                title: this.resources.TEMP_NAME
            });
            layout.push({
                field: "inUse",
                name: this.resources.USAGE_STATUS,
                width: "8em",
                formatter: dojo.hitch(this, this._formatInUse),
                decorator: dojo.hitch(this, this._decorateInUse),
                title: this.locResources.TemplateInUseText_Header
            });
            layout.push({
                field: "userDefined",
                name: this.resources.PATTERN_ORIGIN,
                width: "8em",
                formatter: dojo.hitch(this, this._formatUserDefined),
                decorator: dojo.hitch(this, this._decorateUserDefined),
                title: this.locResources.TemplateCC
            });
            (includeType) && layout.push({
                field: "templateType",
                name: this.locResources.TemplateT,
                width: "9em",
                formatter: dojo.hitch(this, this._formatTemplateType),
                decorator: dojo.hitch(this, this._decorateTemplateType),
                title: this.locResources.TemplateT
            });
            layout.push({
                field: "description",
                name: this.resources.DESCRIPTION,
                width: "auto",
                formatter: dojo.hitch(this, this._formatDescription),
                decorator: dojo.hitch(this, this._decorateDescription),
                title: this.locResources.TemplateD
            });

            return layout;
        },

        _flatToRoot: function(dataItems) {
            // tags:
            // protected                       
            var serverChildren = new Array();
            var systemInfoChildren = new Array();
            var managementChildren = new Array();
            var powerChildren = new Array();
            var performanceRecoveryChildren = new Array();
            var devicesAndIOPortsChildren = new Array();
            var pxeChildren = new Array();
            var fibreChannelChildren = new Array();
            var sasChildren = new Array();
            var adapterChildren = new Array();
            var portVirtualizationChildren = new Array();
            var portPhysicalFunctionChildren = new Array();
            var extendedIMMChildren = new Array();
            var extendedUEFIChildren = new Array();
            var extendedPortChildren = new Array();
            
            var newChild = function(item) {
                var child = {
                        id: item.id,
                        name: item.name,
                        type: item.type,
                        templateType: item.type,
                        uri: item.uri,
                        inUse: item.inUse,
                        referencedBy: item.referencedBy,                      
                        userDefined: !(item.userDefined === false),
                        description: item.description,
                        formFactor: item.formFactor ? item.formFactor : ""

                };
                return child;
            };

            dojo.forEach(dataItems, function(item) {
                // console.log(item);

                if (item.type == "Server") {
                    serverChildren.push(newChild(item));
                }
                else if (item.type == "SystemInfo") {
                    systemInfoChildren.push(newChild(item));
                }
                else if (item.type == "Management") {
                    managementChildren.push(newChild(item));
                }
                else if (item.type == "Power") {
                    powerChildren.push(newChild(item));
                }
                else if (item.type == "PerformanceRecovery") {
                    performanceRecoveryChildren.push(newChild(item));
                }
                else if (item.type == "DevicesAndIOPorts") {
                    devicesAndIOPortsChildren.push(newChild(item));
                }
                else if (item.type == "PXE") {
                    pxeChildren.push(newChild(item));
                }
                else if (item.type == "FibreChannel") {
                    fibreChannelChildren.push(newChild(item));
                }
                else if (item.type == "SAS") {
                    // sasChildren.push(newChild(item));
                }
                else if (item.type == "Adapter") {
                    adapterChildren.push(newChild(item));
                }
                else if (item.type == "PortVirtualization") {
                    portVirtualizationChildren.push(newChild(item));
                }
                else if (item.type == "PortPhysicalFunction") {
                    // portPhysicalFunctionChildren.push(newChild(item));
                }
                else if (item.type == "ExtendedIMM") {
                    extendedIMMChildren.push(newChild(item));
                }
                else if (item.type == "ExtendedUEFI") {
                    extendedUEFIChildren.push(newChild(item));
                }
                else if (item.type == "ExtendedPort") {
                    extendedPortChildren.push(newChild(item));
                }

            });

            var items = new Array; // Array that holds only objects with children
            if (serverChildren.length > 0) {
                // Server Templates Objects
                items.push({
                    id: "Server",
                    name: this.locResources.ServerTemplate,
                    type: "root",
                    children: serverChildren
                });
            }
            if (systemInfoChildren.length > 0) {
                // System Information Templates Objects
                items.push({
                    id: "SystemInfo",
                    name: this.locResources.SystemInfoTemplate,
                    type: "root",
                    children: systemInfoChildren
                });
            }
            if (managementChildren.length > 0) {
                // Management Interface Templates Objects
                items.push({
                    id: "Management",
                    name: this.locResources.ManagementTemplate,
                    type: "root",
                    children: managementChildren
                });
            }
            if (powerChildren.length > 0) {
                // Power Schedule and Capping Templates Objects
                items.push({
                    id: "Power",
                    name: this.locResources.PowerTemplate,
                    type: "root",
                    children: powerChildren
                });
            }
            if (performanceRecoveryChildren.length > 0) {
                // Performance and Recovery Templates Objects
                items.push({
                    id: "PerformanceRecovery",
                    name: this.locResources.PerformanceTemplate,
                    type: "root",
                    children: performanceRecoveryChildren
                });
            }
            if (devicesAndIOPortsChildren.length > 0) {
                // Devices And IO Ports Templates Objects
                items.push({
                    id: "DevicesAndIOPorts",
                    name: this.locResources.DevicesTemplate,
                    type: "root",
                    children: devicesAndIOPortsChildren
                });
            }
            if (pxeChildren.length > 0) {
                // PXE Boot Target Templates Objects
                items.push({
                    id: "PXE",
                    name: this.locResources.PXETemplate,
                    type: "root",
                    children: pxeChildren
                });
            }
            if (fibreChannelChildren.length > 0) {
                // Fibre Channel Boot Target Templates Objects
                items.push({
                    id: "FibreChannel",
                    name: this.locResources.FibreChannelTemplate,
                    type: "root",
                    children: fibreChannelChildren
                });
            }
            if (sasChildren.length > 0) {
                // SAS Boot Target Templates Objects
                items.push({
                    id: "SAS",
                    name: this.locResources.SASTemplate,
                    type: "root",
                    children: sasChildren
                });
            }
            if (adapterChildren.length > 0) {
                // Adapter Templates Objects
                items.push({
                    id: "Adapter",
                    name: this.locResources.AdapterTemplate,
                    type: "root",
                    children: adapterChildren
                });
            }
            if (portVirtualizationChildren.length > 0) {
                // Port Virtualization Templates Objects
                items.push({
                    id: "PortVirtualization",
                    name: this.locResources.PortVTemplate,
                    type: "root",
                    children: portVirtualizationChildren
                });
            }

            if (portPhysicalFunctionChildren.length > 0) {
                // Port Physical Templates Objects
                items.push({
                    id: "PortPhysicalFunction",
                    name: this.locResources.PortPTemplate,
                    type: "root",
                    children: portPhysicalFunctionChildren
                });
            }

            if (extendedIMMChildren.length > 0) {
                // Extended IMM Templates Objects
                items.push({
                    id: "ExtendedIMM",
                    name: this.locResources.IMMTemplate,
                    type: "root",
                    children: extendedIMMChildren
                });
            }
            if (extendedUEFIChildren.length > 0) {
                // Extended UEFI Templates Objects
                items.push({
                    id: "ExtendedUEFI",
                    name: this.locResources.UEFITemplate,
                    type: "root",
                    children: extendedUEFIChildren
                });
            }
            if (extendedPortChildren.length > 0) {
                // Extended Port Templates Objects
                items.push({
                    id: "ExtendedPort",
                    name: this.locResources.PORTTemplate,
                    type: "root",
                    children: extendedPortChildren
                });
            }

            var tableData = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": items
            }, true);

            return tableData;

        },

        _rootToFlat: function(dataItems) {
            // tags:
            // protected
            var items = new Array();
            for (var loopVar = 0; loopVar < dataItems.length; ++loopVar) {
                var item = dataItems[loopVar];
                if (!item) {
                    continue;
                };
                switch (item.type) {
                    case "root":
                        // Store expand info since we "lose" the root nodes
                        if (item.id == this.POL_NAME) {
                            this._expandMe.policy = item.expandMe;
                        }
                        else if (item.id == this.SCH_NAME) {
                            this._expandMe.schedule = item.expandMe;
                        }
                        break;
                    case "Server":
                    case "SystemInfo":
                    case "Management":
                    case "Power":
                    case "PerformanceRecovery":
                    case "DevicesAndIOPorts":
                    case "PXE":
                    case "FibreChannel":
                    // case "SAS":
                    case "Adapter":
                    case "PortVirtualization":
                    // case "PortPhysicalFunction":
                    case "ExtendedIMM":
                    case "ExtendedUEFI":
                    case "ExtendedPort":
                        items.push({
                            id: item.id,
                            name: item.name,
                            uri: item.uri,
                            type: "root",
                            templateType: item.type,
                            inUse: item.inUse,
                            referencedBy: item.referencedBy,
                            userDefined: item.userDefined,
                            description: item.description,
                            formFactor: item.formFactor ? item.formFactor : ""
                        });
                        break;
                    default:
                        // Skip unwanted pattern types...
                        break;
                }
            };

            var tableData = dojo.toJson({
                "identifier": "id",
                "label": "name",
                "items": items
            }, true);

            return tableData;

        },
        
        scopedRTFunc : function(){
            if(this._patternTable != null){
                var rowCount = this._patternTable.select.row.getSelected();
                if(rowCount.length == 0){
                    this._refreshTree();
                }
            }else{
                if(this.pollingFunc != null) {
                    this._stopPolling();
                }
            }
        },
        // this function is responsible for continually polling the services
        //   layer for pattern information in order to dynamically update
        _pollForStatusUpdates: function() {
            console.log("Initiating polling for server pattern status updates (Server Patterns page)");
            // call the _retrieveStore(...) function every 10 seconds in
            //   order to obtain the most recent server pattern data
            
            // first, stop the existing polling mechanism (needed in case user
            //   does multiple assignments/deployments in succession)
                console.log("Stopping server pattern status updates before starting again (Server Patterns page)");
                this._stopPolling();
                
            // next, create a bound function so that when
            //   _refreshTree(...) executes, it will execute with the correct
            //   context for "this"
            //var scopedRTFunc = dojo.hitch(this, this._refreshTree);
            
            // next, create a bound function so that when
            //   _checkStopPolling(...) executes, it wille execute with the
            //   correct context for "this"
            var scopedCSPFunc = dojo.hitch(this, this._checkStopPolling);
            
            // set the current date
            this.pollingStartDate = (new Date()).getTime();
            
            // now, simply call the setInterval(...) function to call
            //   _refreshTree(...) followed by _checkStopPolling(...) every 60
            //   seconds
            this.pollingFunc = window.setInterval(dojo.hitch(this, function(){ this.scopedRTFunc();scopedCSPFunc(); }), 60000);
        }, // _pollForStatusUpdates
        
        // this function is responsible for determining whether to stop the
        //   polling for status updates
        _checkStopPolling: function() {
            // if we have been polling for too long, stop the polling
            //   mechanism
            var curDate = (new Date()).getTime();
            var dateDiff = curDate - this.pollingStartDate;
            // if we've been polling for longer than 10 minutes, stop
            if(dateDiff > 600000) {
                console.log("Polling for server pattern status updates has occurred for longer than 10 minutes; stopping polling (Server Patterns page)");
                window.clearInterval(this.pollingFunc);
                this.pollingFunc = null;
            }
        }, // _checkStopPolling
        // this function is responsible for stopping the polling
        _stopPolling: function() {

            if(this.pollingFunc != null) {
                console.log("Stopping server pattern status polling (Server Patterns page)");
                window.clearInterval(this.pollingFunc);
                this.pollingFunc = null;
            }

        },//_stopPolling
        
        goForeground: function(params) { //Page base function that fires when the page is brought to focus
            
            this._refreshTree();
                    
            this._pollForStatusUpdates();

            return true;
        },
        
        goBackground: function(){ //Page base function that fires when the page losses to focus
            
            this._stopPolling();
            
        }
   
    });

});
