/*Example Set-Up:
 * | <link rel="stylesheet" href="dojo/js/widgets/themes/ButtonBar.css" type="text/css">
 * | <script type="text/javascript">
 * |    dojo.registerModulePath("support", "../../js/support");
 * |    dojo.require("support.widgets.ButtonBar");	
 * |    dojo.require("dojo.parser");
 * | </script>
 * | <div data-dojo-type="support.widgets.ButtonBar"
 * |    data-dojo-props="id:'buttonbar', values:'One|Two|Three|Four|Five', selected:'Two',
 * |    titles:'This is one|This is two', onclick:'doSomething()', onkeypress:'doSomething()'">
 * | </div> 
 */

define(["dojo/_base/declare",
        "dojo/_base/array",
        "dojo/dom-class", 
        "dijit/_WidgetBase",
        "dijit/_TemplatedMixin", 
        "dijit/_CssStateMixin", 
        "dijit/form/Button",
        "dojo/i18n!./nls/ButtonBar",
        "dojo/text!../widgets/templates/ButtonBar.html" ], 
       function(modDeclare, 
                modArray,
                modDomClass, 
                mod_WidgetBase, 
                mod_TemplatedMixin, 
                mod_CssStateMixin, 
                modButton, 
                mri, 
                template) {
   modDeclare("support.widgets.ButtonBar", [mod_WidgetBase, mod_TemplatedMixin, mod_CssStateMixin], {
      // templateString: [protected] String
      // This widget's HTML template, provided.
      templateString: template,

      // locResources: [protected] Object
      // This widget's National Language Source (NLS) object.
      locResources: mri,

      baseClass: "ButtonBar",

      // label of the selected button
      selected: null,
      // index of the selected button
      selIndex: 0,
      // String of labels each separated by |
      labels: null,
      // String of values each separated by |
      values: null,
      // hover text value,
      titles: null,

      constructor: function(){
         this._buttonNodes = [];
      },

      postMixInProperties: function() {
         this.inherited(arguments);
            
         if (this.values == null || (this.values != null && this.values.split("|").length < 2)) {
            this.values = this.locResources["DEFAULT_VALUE"].replace("%1", "1") + "|"
               + this.locResources["DEFAULT_VALUE"].replace("%1", "2");
         }

         if (this.labels == null)
            this.labels = this.values;

         this._valuesArr = this.values.split("|");
         this._labelsArr = this.labels.split("|");
         if (this.titles == null)
            this._tooltipsArr = null;
         else
            this._tooltipsArr = this.titles.split("|");
      },

      buildRendering: function() {
         this.inherited(arguments);

         for (var i = 0; i < this._labelsArr.length; i++) {
            var button = new modButton({
               buttonbar: this,
               label: this._labelsArr[i],
               value: this._labelsArr[i],
               title: this._tooltipsArr == null ? "" : this._tooltipsArr[i],
               onClick: function(){
                  return this.buttonbar._onClick(this.label);
               },
               onKeyPress: function(event){
                  return this.buttonbar._onKeyPress(event);
               },
               tabindex: i == 0 ? 0 : -1,
               "class": i == 0 ? "leftbutton" : i == this._labelsArr.length - 1 ? "rightbutton" : "middlebutton"
            }).placeAt(this.buttonsNode);

            this._buttonNodes.push(button);
         }
      },

      _onClick: function(label) {
         this._setActive(modArray.indexOf(this._labelsArr, label));
      },

      _setActive: function(index) {
         this._styleButtons(index);

         if (this.selIndex == index) {
            // Already selected, don't call onClick (which should actually be called onChange).
            return false;
         } else {
            this.selIndex = index;
            var value = this.get("selected");
            return this.onClick(value);
         }
      },

      _styleButtons: function(selectedButton) {
         modArray.forEach(this._buttonNodes, function(item, i) {
            if (i == selectedButton) {
               // selected
               modDomClass.add(item.domNode, "buttonBarButtonActive");
            } else {
               // unselected
               modDomClass.remove(item.domNode, "buttonBarButtonActive");
            }
         }, this);
      },

      onClick: function(value) {
         // override if necessary
         return false;
      },

      _onKeyPress: function(event) {
         var code = event.keyCode;
         if (code == 37 || code == 38 || code == 39 || code == 40) {
            // left arrow, up arrow, right arrow, down arrow
            var index = this._getNextIndex(code);
            this._setActive(index);
            this.onKeyPress(event);
            event.preventDefault();
         }

         this.onKeyPress(event);
      },

      onKeyPress: function(event) {
         // do nothing, override if necessary
      },

      // determines the next index in navigation
      _getNextIndex: function(code) {
         var index = this.selIndex;
         if (code == 37 || code == 38) {
            // left arrow, up arrow
            // navigate focus backward
            index--;
            if (index < 0) 
               index = this._labelsArr.length - 1;
         } else if (code == 39 || code == 40) {
            // right arrow, down arrow, tab
            // navigate focus forward
            index++;
            if (index >= this._labelsArr.length) 
               index = 0;
         }
         return index;
      },

      _getSelectedAttr: function(){
         return this._valuesArr[this.selIndex];
      },
        
      //Called by widget.set("selected", ____); 
      _setSelectedAttr: function(selected) {
         var index;

         if (selected != null) {
            index = modArray.indexOf(this._valuesArr, selected);

            if (index == -1) {
               console.error("ButtonBar.set(\"selected\"): bad value");
               return;
            }
         } else{
            //user has chosen default value to be selected
            //button1 initially selected
            index = 0;
         }
            
         this._setActive(index);
      },

      //Called by widget.set("selIndex", _____);
      //Indicies begin at 0
      _setSelIndexAttr: function(index) {
         if (index < this._labelsArr.length) 
            this._setActive(index);
      }
   });
});