/*Example Set-Up:
 * | <link rel="stylesheet" href="dojo/js/widgets/themes/CapturedSettingsTree.css" type="text/css">
 * | <script type="text/javascript">
 * |    dojo.registerModulePath("support", "../../js/support");
 * |    dojo.require("support.widgets.CapturedSettingsTree");   
 * |    dojo.require("dojo.parser");
 * | </script>
 * | <div data-dojo-type="support.widgets.CapturedSettingsTree"
 * |    data-dojo-props="id:'simple'">
 * | </div> 
 */

define([ "dojo", 
         "dijit", 
         "support/panels/Panel", 
         "dojo/i18n!./nls/CapturedSettingsTree",
         "dojo/text!./templates/CapturedSettingsTree.html", 
         "dojo/data/ItemFileReadStore", 
         "support/base/TableMixin",
         "dijit/Toolbar"
         ], function(dojo, dijit, panel, mri, template) {

    dojo.declare("support.widgets.CapturedSettingsTree", [ support.panels.Panel, dijit._TemplatedMixin, support.base.TableMixin ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        widgetsInTemplate: true,

        label: null,
        jsonData: null,
        width: "100%",
        height: "250px",       
        _treeTable: null,
        _treeStore: null,
        _tableToolbar: null,
        style: null,

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/panels/themes/CapturedSettingsTree.css",

        postMixInProperties: function() {

        },

        postCreate: function() {
            // Setup learned settings table widget...
            var node = document.createElement("div");
            var nodeID = this.id + "capturedSettingsTree";
            var treeID = this.id + "tree";
            node.setAttribute("id", nodeID);
            node.setAttribute("style", "positon:relative; width:100%; height:100%");

            if (this.style == null) {
                this.style = "width:500px; height:250px";
            }
            this._CapturedSettingsTree.setAttribute("style", this.style);

            this._CapturedSettingsTree.appendChild(node);
            
            var settingsStore = this._retrieveStore();       // Get table store information
            var tableStructure = this._generateDetailsLayout();  // Get structure for current view          
            
            this._settingsTable = this.getTable(settingsStore, tableStructure, this.resources.NO_DATA, this.gridID, true);

            this._settingsTable.placeAt(this._CapturedSettingsTree);

            this._settingsTable.startup();
        },

        startup: function() {
        },

        doApplyCellEdit: function(value, rowIndex, fieldIndex) {
            // this.update();
        },

        _setLabelAttr: function(l) {
            this.label = l;
            this.labelSpan.innerHTML = this.label;
        },

        _retrieveStore: function() {
            // store
            if (this._treeStore == null) {
                this._treeStore = new dojo.data.ItemFileReadStore({
                    data: this._getValidJsonData(this.jsonData)
                });
            }
            
            return this._treeStore;
        },
          
        _generateDetailsLayout: function() {
            // layout
            if (this._treeLayout == null) {
                this._treeLayout = [ {
                    field: "array",
                    name: this.locResources.ARRAY,
                    width: "10%",
                    formatter: this._format

                }, {
                    field: "level",
                    name: this.locResources.RAID_LEVEL,
                    formatter: this._format
                }, {
                    field: "drives",
                    name: this.locResources.DRIVES,
                    width: "10%",
                    formatter: this._format
                }, {
                    field: "drive_type",
                    name: this.locResources.DRIVE_TYPE,
                    formatter: this._format
                }, {
                    field: "spare",
                    name: this.locResources.HOT_SPARE,
                    width: "15%",
                    formatter: this._format
                }, {
                    field: "volumes",
                    name: this.locResources.VOLUMES,
                    formatter: this._format
                } ];
            }
            
            return this._treeLayout;
        },

        _format: function(item) {
            // Will probably need a formatter for each field since Gridx passes
            // the item and not field value.  
            // Mark this as a TODO for when this widget is actually used.
            
            if (item == null) {
                return "";
            }
            
            return item;
        },

        getTreeAsJson: function() {
            var store = this._treeStore;
            var items = store._arrayOfAllItems;
            var allObjs = new Array();
            for ( var i = 0; i < items.length; i++) {
                // level, drives, drive_type, spare, volumes
                var item = items[i];
                var type = store.getValue(item, "type");
                var array = store.getValue(item, "array");
                var level = store.getValue(item, "level");
                var drives = store.getValue(item, "drives");
                var driveType = store.getValue(item, "drive_type");
                var spare = store.getValue(item, "spare");
                var volumes = store.getValue(item, "volumes");
                var obj = {
                    type: type,
                    array: array,
                    level: level,
                    drives: drives,
                    driveType: driveType,
                    spare: spare,
                    volumes: volumes
                };
                allObjs.push(obj);
            }
            return allObjs;

        },

        _getValidJsonData: function(jsonData) {
            // if any part of the file is not valid, return the default item
            if (jsonData == null || jsonData.items == null || jsonData.items[0].array == null || jsonData.items[0].level == null
                    || jsonData.items[0].drives == null || jsonData.items[0].drive_type == null || jsonData.items[0].spare == null
                    || jsonData.items[0].volumes == null) {
                jsonData = {
                    "items": [ {
                        "type": "root",
                        "array": "1",
                        "level": this.locResources["RAID"].replace("%1", "1"),
                        "drives": "2",
                        "drive_type": this.locResources.DRIVE_TYPE_HDD,
                        "spare": this.locResources.HOT_SPARE_NO,
                        "volumes": "2",
                        "children": [ {
                            "volumes": "50 " + this.locResources.GB
                        }, {
                            "volumes": "200 " + this.locResources.GB
                        } ]
                    }, {
                        "type": "root",
                        "array": "2",
                        "level": this.locResources["RAID"].replace("%1", "5"),
                        "drives": "5",
                        "drive_type": this.locResources.SSD,
                        "spare": this.locResources.HOT_SPARE_YES,
                        "volumes": "3",
                        "children": [ {
                            "volumes": "500 " + this.locResources.GB
                        }, {
                            "volumes": "1000 " + this.locResources.GB
                        }, {
                            "volumes": "800 " + this.locResources.GB
                        } ]
                    }, {
                        "type": "root",
                        "array": "3",
                        "level": this.locResources["RAID"].replace("%1", "0"),
                        "drives": "3",
                        "drive_type": this.locResources.SSD,
                        "spare": this.locResources.HOT_SPARE_NO,
                        "volumes": "2",
                        "children": [ {
                            "volumes": "100 " + this.locResources.GB
                        }, {
                            "volumes": "100 " + this.locResources.GB
                        } ]
                    } ]
                };
            }
            return jsonData;
        }
    });
});