/*
 * Licensed Materials - Property of IBM
 * 
 * Augusta Source Materials
 * 
 * (C) Copyright IBM Corp. 2014 All Rights Reserved
 * 
 * US Government Users Restricted Rights - Use, duplicate or disclosure restricted by GSA ADP
 * Schedule Contract with IBM Corp.
 */
define([
        "dojo/store/Memory",
        "dojo/i18n!./nls/DetailsSelectorData"
    ], function(memoryStore, mri) {

    // The purpose of this module is to provide necessary data for use by UI Core Framework's
    // Details Selector widget within Augusta/xHMC runtime environments. The Details Selector is used
    // to provide navigation menu and links within the left frame of our UI container.
    //
    // NOTE: Data for the Details Selector provided by NGUI for FSM is handled differently. This
    //       module is currently not used in FSM environments.
    // 
    // Example usage by Details Selector widget:
    //
    //    configPatterns : function ( ) {
    //      var cpTask = new taskObject({
    //          taskId : "config.patterns",
    //          taskURI : "augusta/augustaUICore/widgets/detailsSelector/DetailsSelector",
    //          selector : "Details",
    //          type : gConstants.TASK_TYPE_WORKAREA_UI,
    //          launchParams : {
    //            dataType : "MODULE",
    //            moduleId : "support/widgets/DetailsSelectorData",
    //            contextId : "ConfigPatternsSelectorId" } });
    //        cpTask.invokeTask();
    //      }
    //
    
    var store = new memoryStore();
    var storeItem = {
        id: "ConfigPatternsSelectorId",
        label: mri.PRODUCT_NAME,
        image: "/ibm/augusta/rest/profiles/js/support/common/resources/configPatterns_hdr.png",
        description: mri.PRODUCT_DESCRIPTION,
        properties: [ {
            id: "DisplayName",
            value: mri.PRODUCT_NAME,
            label: "Name"
        } ],
        tasks: [ {
            id: "CommonId",
            label: mri.COMMON_GROUP,
            subTasks: [ {
                id: "GettingStarted",
                label: mri.GETTING_STARTED,
                widget: "support/panels/GettingStarted",
                labelIcon: "/ibm/augusta/rest/profiles/js/support/common/resources/gettingStarted_obj.png"
            }, {
                id: "AddressPools",
                label: mri.ADDRESS_POOLS,
                widget: "support/panels/AddressPools",
                labelIcon: "/ibm/augusta/rest/profiles/js/support/common/resources/ipAddressPool_obj.png"
            } ]
        }, {
            id: "ServersId",
            label: mri.SERVERS_GROUP,
            subTasks: [ {
                id: "ServerPatterns",
                label: mri.SERVER_PATTERNS,
                widget: "support/panels/TemplatesTable",
                labelIcon: "/ibm/augusta/rest/profiles/js/support/common/resources/template_obj.png"
            }, {
                id: "ServerProfiles",
                label: mri.SERVER_PROFILES,
                widget: "support/panels/ServerProfileTable",
                labelIcon: "/ibm/augusta/rest/profiles/js/support/common/resources/profiles_obj.png"
            }
			//If in Augusta framework then remove any reference for failover for Augusta Beta (Will return for Augusta Release)
            /*, {
                id: "ServerPolicies",
                label: mri.SERVER_POLICIES,
                widget: "support/panels/PolicyListTable",
                labelIcon: "/ibm/augusta/rest/profiles/js/support/common/resources/policies_obj.png"
            }, {
                id: "StandbyServerPools",
                label: mri.STANDBY_SERVER_POOLS,
                widget: "support/panels/StandbyListTable",
                labelIcon: "/ibm/augusta/rest/profiles/js/support/common/resources/standbyServerPool_obj.png"
            }*/ ]
        }, {
            id: "ChassisId",
            label: mri.CHASSIS_GROUP,
            subTasks: [ {
                id: "PlaceholderChassis",
                label: mri.PLACEHOLDER_CHASSIS,
                widget: "support/panels/DeployPlaceholderChassis",
                labelIcon: "/ibm/augusta/rest/profiles/js/support/common/resources/placeholderChassis_obj.png"
            } ]
        } ]
    };
    store.add(storeItem, { id: storeItem.id });
    return store;

});