define([ "dojo", "dijit", "dijit/_Widget", "dijit/_TemplatedMixin", "dijit/_WidgetsInTemplateMixin",
        "dojo/text!./templates/DoubleSelects.html", "dijit/form/FilteringSelect", "dojo/store/Memory",
        "support/widgets/OutlinePane", "dijit/form/MultiSelect" ], function(dojo, dijit, _Widget, _TemplatedMixin,
        _WidgetsInTemplateMixin, template) {

    return dojo.declare("support.widgets.DoubleSelects", [ dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
        widgetsInTemplate: true,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "DoubleSelects", // This widget's base CSS classname.

        select1: null,
        select2: null,
        store1: null,
        store2: null,
        value1: null,
        value2: null,
        value: null,
        required: false,
        disabled: false,
        multi: true,

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template
        // HTML file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be
        // directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);

            var ds = this;

            if (this.store1 == null) {
                this.store1 = new dojo.store.Memory({
                    data: {
                        "items": []
                    }
                });
            }

            if (this.store2 == null) {
                if (this.multi) {
                    this.store2 = new Array();
                }
                else {
                    this.store2 = new dojo.store.Memory({
                        data: {
                            "items": []
                        }
                    });
                }
            }

            // select1
            this.select1 = new dijit.form.FilteringSelect({
                required: this.required,
                store: this.store1,
                searchAttr: "name",
                _destroyOnRemove: true,
                onChange: dojo.hitch(this, function(newValue) {
                    this.set("value1", newValue);
                    this.onChange(newValue);
                })
            }).placeAt(this._select1);

            this.select1.textbox.readOnly = true;

            // select2
            if (this.multi) {
                // multi select
                var sel = this._select2;
                for ( var i = 0; i < this.store2.length; i++) {
                    var item = this.store2[i];
                    var label = item.label;
                    var value = item.value;
                    var c = dojo.doc.createElement("option");
                    c.innerHTML = label;
                    c.value = value;
                    sel.appendChild(c);
                }
                this.select2 = new dijit.form.MultiSelect({
                    name: "multiselect"
                }, sel);
                if (this.store2.length == 0) {
                    this.select2.domNode.setAttribute("style", "display:none");
                }
            }
            else {
                // singular select
                this.select2 = new dijit.form.FilteringSelect({
                    required: this.required,
                    store: this.store2,
                    searchAttr: "name",
                    _destroyOnRemove: true,
                    onChange: function(newValue) {
                        ds.set("value2", newValue);
                    },
                    disabled: this.disabled
                }).placeAt(this._select2);
                
                this.select2.textbox.readOnly = true;

            }

        },

        // none
        // custom:one|two|three (multi)
        // custom:one (singular)
        _setValueAttr: function(values) {
            if (values != null) {
                values = values.split(":");
                if (values.length == 1) {
                    var value1 = values;
                    this.set("value1", value1);
                }
                else if (values.length == 2 && this.select1 != null && this.select2 != null) {
                    var value1 = values[0];
                    var value2 = values[1];
                    this.set("value1", value1);
                    this.set("value2", value2); // could be "one" or "one|two|three"
                }
            }
        },

        _getValueAttr: function() {
            //if (this.multi) {
                var values = new Array();
                var value1 = this.get("value1");
                var value2 = this.get("value2");
                if (value2 == "") {
                    return value1;
                }
                else {
                    values.push(value1);
                    values.push(value2);
                    return values.join(":");
                }
            /*}
            else {
                return this.get("value1");
            }*/

        },

        _getValue1Attr: function() {
            return this.select1.get("value");
        },

        _getValue2Attr: function() {
            if (this.multi) {
                return this.select2.get("value").join("|");
            }
            else {
                return this.select2.get("value");
            }
        },

        _setValue1Attr: function(value) {
            this.value1 = value;
            this.select1.set("value", value);
        },

        // could be "one" or "one|two|three"
        _setValue2Attr: function(value) {
            this.value2 = value;
            if (this.multi) {
                // value is an array
                if (value != null) {
                    value = value.split("|");
                }
                this.select2.set("value", value);
            }
            else {
                this.select2.set("value", value);
                this.select2.set("displayValue", value);
            }
        },

        onChange: function(newValue) {

        },

        _setStore2Attr: function(store) {
            this.store2 = store;
            if (store == null) {
                if (this.multi) {
                    store = new Array();
                }
                else {
                    store = new dojo.store.Memory({
                        data: {
                            "items": []
                        }
                    });
                }
            }
            if (this.select2 != null) {
                if (this.multi) {
                    // remove old children
                    while (this.select2.domNode.hasChildNodes()) {
                        this.select2.domNode.removeChild(this.select2.domNode.lastChild);
                    }
                    // append new children
                    for ( var i = 0; i < store.length; i++) {
                        var item = store[i];
                        var label = item.label;
                        var value = item.value;
                        var c = dojo.doc.createElement("option");
                        c.innerHTML = label;
                        c.value = value;
                        this.select2.domNode.appendChild(c);
                    }
                    if (store.length <= 0) {
                        this.select2.domNode.setAttribute("style", "display:none");
                    }
                    else {
                        this.select2.domNode.setAttribute("style", "display:block");
                    }
                }
                else {
                    this.select2.set("store", store);
                }
            }
        },

        _setStore1Attr: function(store) {
            if (this.select1 != null)
                this.select1.set("store", store);
        },

        _setDisabledAttr: function(disabled) {
            if (this.select2 != null)
                this.select2.set("disabled", disabled);
        }
    });
});
