define([
        "dojo",
        "dijit",
        "dijit/_Widget",
        "dijit/_TemplatedMixin",
        "dojo/text!./templates/Icon.html"
    ], function(dojo, dijit, _Widget, _TemplatedMixin, template) {

    dojo.declare("support.widgets.Icon", [ dijit._Widget, dijit._TemplatedMixin ], {

        // summary:
        // A selectable icon widget displaying an image with optional text positioned beneath it.
        //
        // description:
        // Displays an image with optional text beneath it. The following action handlers are
        // supported:
        // - onClick(): fires when the user clicks to select the widget
        // - onDeselect(): if enabled, fires when the user clicks to de-select the widget
        // - onMouseOver(): fires when mouse pointer is moved over the widget
        // - onMouseOut(): fires when mouse pointer is moved off the widget
        //
        // Images for 4 states are managed; "normal" (unselected), "selected", "hover", and
        // "disabled". Image resources and styling must be specified as CSS classnames via
        // "iconClass" parameter.
        //
        // Multiple instances may be grouped (logically) by specifying a common group name via the
        // "groupName" parameter. If specified, groupings are maintained by the widget automatically
        // and are used during onClick() action handing to ensure one and only one instance within
        // the named group is kept in "selected" state.

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "Icon",

        // tabIndex: [protected] String
        // Allows action icon node to receive focus for keyboard input, if enabled.
        tabIndex: "0",

        // iconClass: Object
        // Specifies CSS classnames to be assigned to the instance's image node. Values must be
        // an object with properly labeled elements specifying separate classnames for "normal",
        // "selected", "hover", and "disabled" states.
        //
        // example:
        // iconClass: { normal: "normCls", selected: "selectCls", hover: "hoverCls", disabled:
        // "disabledCls" }
        iconClass: {},

        // label: String
        // Specifies a "string" object for displaying text beneath the image. If provided, action
        // handlers are supported for the instance's text as well as for its image.
        label: "",

        // disabled: Boolean
        // Specifies whether the instance is in disabled state. The value "true" indicates the
        // instance is currently in disabled state, "false" indicates it is not (enabled). If
        // "true", the instance's image node is assigned the "disabled" state classname and all
        // action handlers, including its expand action node, are non-functional. The instance's
        // label text, if any, is also set to display in grey font color. Instances in disabled
        // state cannot be selected. Default is "false" (enabled).
        //
        // Initialize to "true" to set the instance's initial state to "disabled".
        disabled: false,

        // expanderEnabled: boolean
        // This attribute specifies whether this Icon's expand action node is enabled. If "true",
        // the expand action node is enabled for display and set accordingly to the current
        // expanderState value. Default is "false" (disabled).
        //
        // NOTE: The expand action node is non-function while the instance is in "disabled" state.
        expanderEnabled: false,

        // expanderState: boolean
        // If the Icon's expand action node is enabled, this attribute specifies its current state.
        // The value "true" indicates the expand action node is set up with an "expand" icon and
        // will invoke onExpanderClick(true) when clicked. Conversely, the value "false" indicates
        // the expand action node is set up with a "collapse" icon and will invoke
        // onExpanderClick(false) when clicked. The value is meaningless if this Icon's
        // "expanderEnabled" attribute is "false" (disabled).
        //
        // NOTE: The expand action node is non-function while the instance is in "disabled" state.
        expanderState: true,

        // expanderTitle: String|Object
        // If the Icon's expand action node is enabled, this attribute specifies title text to be
        // displayed for mouseover events for the expand action node. Values may be specified as a
        // single "string" object to be applied for both "expand" and "collapse" icons.
        // Alternatively, an object with properly labeled elements may be used to specify unique
        // title text for "expand" and "collapse" icons.
        //
        // NOTE: Title text for the expand action node, if any, will not display while the instance
        // is in "disabled" state.
        //
        // example: (single "string" value)
        // expanderTitle: "Expand/Collapse details"
        //
        // example: (object element values)
        // expanderTitle: { expand: "Expand to see more details", collapse: "Collapse details" }
        expanderTitle: "",

        // groupName: String
        // Specifies a "string" object containing a group name to be used to associate multiple
        // instances within a single group. If specified, the widget automatically ensures one and
        // only one instance within the group is kept in "selected" state during onClick() action
        // handing.
        groupName: "",

        // selected: Boolean
        // Specifies the select state of the instance. The value "true" indicates the instance is
        // currently in "selected" state, "false" indicates it is not (aka: "normal" state).
        //
        // Initialize to "true" to set the instance's initial state to "selected".
        selected: false,

        // explicitDeselection: Boolean
        // Specifies whether the widget is to support explicit de-selection via onClick() action
        // handling. If "true", the instance may be explicitly de-selected by clicking on it when in
        // "selected" state. If "false", the instance may only be de-selected when another instance
        // within the same group is selected (see: "groupName" param).
        //
        // Note: If enabled, clicking on the instance when in "selected" state will cause the
        // onDeselect() action handler to fire. If disabled, the onDeselect() action handler will
        // never fire.
        explicitDeselection: false,

        // Override this method to perform custom behavior during dijit construction.
        // Common operations for constructor:
        // 1) Initialize non-primitive types (i.e. objects and arrays)
        // 2) Add additional properties needed by succeeding lifecycle methods
        constructor: function() {

        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be
        // directly accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);
            this.setDisabled(this.disabled);

            // Register this instance with its icon group, if specified.
            var myGroup = this._getMyGroup();
            if (Object.prototype.toString.apply(myGroup) === "[object Array]") {
                myGroup.push(this);
            }

        },

        _setIconClassAttr: function(value) {
            // tags:
            // protected
            var temp = value;
            // Fix up iconClass assignments.
            if (typeof temp == "string") {
                // Convert to object...
                temp = {
                    normal: value,
                    selected: value,
                    hover: value
                };
            }
            else if ((typeof temp == "object") && (temp.normal)) {
                // Verify it has all necessary items.
                !temp.selected && (temp.selected = temp.normal);
                !temp.disabled && (temp.disabled = temp.normal);
            }
            else {
                return; // Ignore unsupported type.
            }
            this.iconClass = temp;

            // Reset this instance's image node.
            dojo.attr(this.imageNode, "class", (this.disabled) ? this.iconClass.disabled
                    : (this.selected) ? this.iconClass.selected : this.iconClass.normal);

        },

        _setDisabledAttr: function(disable) {
            this.setDisabled(disable);
        },
        
        _setExpanderTitle: function(value) {
            // tags:
            // protected
            var temp = value || "";
            // Fix up expanderTitle assignments.
            if (typeof temp == "string") {
                // Convert to object...
                temp = {
                    expand: value,
                    collapse: value
                };
            }
            else if ((typeof temp == "object") && (temp.expand)) {
                // Verify it has all necessary items.
                !temp.collapse && (temp.collapse = temp.expand);
            }
            else {
                return; // Ignore unsupported type.
            }
            this.expanderTitle = temp;
            
            // Reset this instances's expander title(s).
            this.setExpanderNode(this.expanderEnabled, this.expanderState);

        },

        setDisabled: function(/* boolean */disable) {

            this.disabled = disable;
            dojo.attr(this.focusNode, "tabIndex", (disable) ? -1 : this.tabIndex);

            // Set up this instance's image node.
            dojo.attr(this.imageNode, "class", (disable) ? this.iconClass.disabled : (this.selected) ? this.iconClass.selected
                    : this.iconClass.normal);

            // Set up this instance's label node, if enabled.
            if (this.label) {
                dojo.style(this.labelContainer, "display", "");
                dojo.attr(this.labelNode, "innerHTML", this.label);
                if (disable) {
                    dojo.addClass(this.labelNode, "iconDisabledLabel");
                }
                else {
                    dojo.removeClass(this.labelNode, "iconDisabledLabel");
                }

                // Also set up this instance's expander node.
                this.setExpanderNode(this.expanderEnabled, this.expanderState);

            }
            else {
                dojo.style(this.labelContainer, "display", "none");
            }

        },

        setExpanderNode: function(/* boolean */enable, /* boolean */newState) {

            if (enable) {
                // Set up this instance's expand action node.
                dojo.style(this.expanderNode, "display", "");
                dojo.attr(this.expanderNode, "class", (newState) ? (this.disabled) ? "spExpandDetailIconNoHover"
                        : "spExpandDetailIcon" : (this.disabled) ? "spCollapseDetailIconNoHover" : "spCollapseDetailIcon");
                if (this.expanderTitle != null) {
                    dojo.attr(this.expanderNode, "title", (this.disabled) ? "" : (newState) ? this.expanderTitle.expand
                            : this.expanderTitle.collapse);
                }
            }
            else {
                dojo.style(this.expanderNode, "display", "none");
            }
            this.expanderEnabled = enable;
            this.expanderState = newState;

        },

        setSelected: function(/* boolean */select) {

            if (select) {
                this._select();
                this.onClick(); // As if Icon were clicked!
            }
            else if (this.explicitDeselection) {
                this._deselect();
                this.onDeselect();
            }
        },

        onClick: function() {
            // Override or hitch custom handler.
        },

        onMouseOver: function() {
            // Override or hitch custom handler.
        },

        onMouseOut: function() {
            // Override or hitch custom handler.
        },

        onDeselect: function() {
            // Override or hitch custom handler.
        },

        onKeypress: function(evt) {
            // Override or hitch custom handler.
        },

        onFocus: function() {
            // Override or hitch custom handler.
        },

        onBlur: function() {
            // Override or hitch custom handler.
        },

        onExpanderClick: function(/* boolean */expanding) {
            // Override or hitch custom handler.
            // NOTE: Must return "true" to allow expand action node state change.
            return true;
        },

        _getMyGroup: function() {
            // tags:
            // protected
            var myGroup = null;
            if (typeof this.groupName == "string") {
                myGroup = dojo.getObject(this.groupName);
                if (myGroup == null) {
                    myGroup = new Array();
                    dojo.setObject(this.groupName, myGroup);
                }
            }
            return myGroup;
        },

        _onClick: function(evt) {
            // tags:
            // protected
            if (this.disabled) {
                return;
            }
            if (this.explicitDeselection && this.selected) {
                this._deselect();
                this.onDeselect();
            }
            else {
                this._select();
                this.onClick();
            }
            evt && evt.stopPropagation(); // we've handled it
        },

        _onMouseOver: function(evt) {
            // tags:
            // protected
            if (this.disabled) {
                return;
            }
            (this.iconClass.hover) && dojo.attr(this.imageNode, "class", this.iconClass.hover);
            this.onMouseOver();
        },

        _onMouseOut: function(evt) {
            // tags:
            // protected
            if (this.disabled) {
                return;
            }
            dojo.attr(this.imageNode, "class", (this.selected) ? this.iconClass.selected : this.iconClass.normal);
            this.onMouseOut();
        },

        _onKeypress: function(evt) {
            // tags:
            // protected
            if (this.disabled) {
                return;
            }
            if (evt.charOrCode == dojo.keys.ENTER || evt.charOrCode == " ") {
                if (this.explicitDeselection && this.selected) {
                    this._deselect();
                    this.onDeselect();
                }
                else {
                    this._select();
                    this.onClick();
                }
            }
            this.onKeypress(evt);
        },

        _onFocus: function(evt) {
            // tags:
            // protected
            if (this.disabled) {
                return;
            }
            this.onFocus();
        },

        _onBlur: function(evt) {
            // tags:
            // protected
            if (this.disabled) {
                return;
            }
            this.onBlur();
        },

        _select: function() {
            // tags:
            // protected
            if (this.disabled) {
                return;
            }
            this.selected = true;
            dojo.attr(this.imageNode, "class", this.iconClass.selected);

            // Deselect all other instances in this widget's icon group, if specified.
            var myGroup = this._getMyGroup();
            dojo.forEach(myGroup, dojo.hitch(this, function(widget) {
                if ((widget != this) && widget.selected) {
                    widget._deselect();
                }
            }));
        },

        _deselect: function() {
            // tags:
            // protected
            this.selected = false;
            dojo.attr(this.imageNode, "class", (this.disabled) ? this.iconClass.disabled : this.iconClass.normal);
        },

        _onExpanderClick: function(evt) {
            // tags:
            // protected
            if (this.disabled) {
                return;
            }
            this._onClick(); // As if Icon itself were clicked!
            if (this.onExpanderClick(this.expanderState)) {
                // switch state...
                this.setExpanderNode(this.expanderEnabled, !this.expanderState);
            }
            evt && evt.stopPropagation(); // we've handled it
        }

    });
});