/*Example Set-Up:
 * | <link rel="stylesheet" href="dojo/js/widgets/themes/ToggleButton.css" type="text/css">
 * | <script type="text/javascript">
 * |    dojo.registerModulePath("support", "../../js/support");
 * |    dojo.require("support.widgets.ToggleButton");   
 * |    dojo.require("dojo.parser");
 * |
 * |    dijit.byId("toggle").set("onClick", doSomething());
 * | </script>
 * | <div data-dojo-type="support.widgets.ToggleButton"
 * |    data-dojo-props="id:'toggle', enabled:true, onclick:'doSomething()', onkeypress:'doSomething()'">
 * | </div>
 */

define([
        "dojo",
        "dijit",
        "dijit/_Widget",
        "dijit/_TemplatedMixin",
        "dijit/_WidgetsInTemplateMixin",
        "dojo/i18n!./nls/LabelledSlider",
        "dojo/text!../widgets/templates/LabelledSlider.html",
        "dijit/form/HorizontalSlider"
    ], function(dojo, dijit, _Widget, _TemplatedMixin, _WidgetsInTemplateMixin, mri, template) {

    dojo.declare("support.widgets.LabelledSlider", [ dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        widgetsInTemplate: true,
        
        _sliderValue: null,
        _slider: null,
        initialValue: null,

        postMixInProperties: function() {
            this.inherited(arguments);
        },

        postCreate: function() {
            this.inherited(arguments);
            this._slider = new dijit.form.HorizontalSlider({
                name: "horizontalslider",
                minimum: 1,
                maximum: 100,
                style: "width: 140px;",
                discreteValues: 100,
                intermediateChanges: true,
                showButtons: false
            }).placeAt(this._horizontalslider);
            dojo.connect(this._slider, "onChange", this, function(value) {     
                if(dojo.byId(this.id + "_sliderlabel") != null) {
                    dojo.byId(this.id + "_sliderlabel").innerHTML = value + "%";
                }
                this._sliderValue = value;
                this._onChange(value);
            });
            if (this.initialValue != null && this.initialValue != undefined) {
                this._slider.set("value", this.initialValue);
            } else {
                this._slider.set("value", 25);
            }
        },

        startup: function() {
            this.inherited(arguments);
        },

        // onClick event
        _onClick: function() {
            this.onClick();
        },

        onClick: function() {
            // do nothing, override if necessary
        },
        
        // onChange event
        _onChange: function(newValue) {
            //alert("in _onChange");
            
            this.onChange(newValue);            
        },

        onChange: function(newValue) {            
            //alert("in onChange");
            // do nothing, override if necessary
        },        

        // onmouseover event
        _onMouseOver: function() {
            this.onMouseOver();
        },

        onMouseOver: function() {
            // do nothing, override if necessary
        },

        _onMouseOut: function() {
            this.onMouseOut();
        },

        onMouseOut: function() {
            // do nothing, override if necessary
        },

        // onkeypress event
/*        _onKeyPress: function(e) {
            if (!e) {
                e = window.event;
            }
            var code = e.keyCode;
            // space
            if (code == 0) {
                this.onKeyPress();
                return true;
            }
            // arrow keys
            else if (code == 37 || code == 38 || code == 39 || code == 40) {
                this._onClick();
                this.onKeyPress();
                return true;
            }
            // enter
            else if (code == 13) {
                e.preventDefault();
                this.onKeyPress();
                return false;
            }
            // tab
            else if (code == 9) {
                this.onKeyPress();
                return true;
            }
            this.onKeyPress();
            return false;
        },

        onKeyPress: function() {
            // do nothing, override if necessary
        },*/

        _getValueAttr: function() {            
            return(this._sliderValue);
        },
        
        _setValueAttr: function(value) {
            this._slider.set("value", value);
        },
        
        _setEnabledAttr: function(bool) {
            this.enabled = !bool;
            this._onClick();
        }

    });
});
