define([
        "dojo",
        "dijit",
        "dijit/layout/ContentPane",
        "dijit/_TemplatedMixin",
        "dojo/text!./templates/OutlinePane.html"
    ], function(dojo, dijit, ContentPane, _TemplatedMixin, template) {

    return dojo.declare("support.widgets.OutlinePane", [ dijit.layout.ContentPane, dijit._TemplatedMixin ], {

        // summary:
        // A container widget extending dijit.layout.ContentPane, displaying a border with rounded
        // corners and optional title and action icon node.
        //
        // example:
        // | <div data-dojo-type="support.widgets.OutlinePane" data-dojo-props="title: 'My Title'">
        // | <p>Pane content goes here.</p>
        // | </div>

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "OutlinePane",

        // the name of the CSS stylesheet of this page
        spCssPathAndName: "support/widgets/themes/OutlinePane.css",
        
        // title: String
        // Specifies a "string" object for displaying a title on the container's upper border.
        title: "",

        // _setTitleAttr: [protected]
        _setTitleAttr: {
            node: "titleNode",
            type: "innerHTML"
        }, // override default where title becomes a hover tooltip

        // tabIndex: [protected] String
        // Allows action icon node to receive focus for keyboard input, if enabled.
        tabIndex: "0",

        // enableEditIcon: boolean
        // Specifies whether the container's action icon node is enabled and displays with a
        // standard edit icon provided by the dijit.
        //
        // If "true", the action icon node will be enabled. A standard edit icon will display on
        // the right-hand side of the container's upper border and will support the following
        // action handlers:
        // - onActionClick(): fires when the user clicks to select the container's icon node
        // - onActionMouseOver(): fires when mouse pointer is moved over the container's icon node
        // - onActionMouseOut(): fires when mouse pointer is moved off the container's icon node
        //
        enableEditIcon: false,

        // iconClass: String
        // Specifies one or more CSS classnames to enable and display on the container's action
        // icon node. Multiple classnames may be specified by separating with whitespace.
        //
        // If specified, this setting will override standard icons used for other options such as
        // the icon used for "enableEditIcon". The icon will display on right-hand side of the
        // container's upper border and will support the following action handlers:
        // - onActionClick(): fires when the user clicks to select the container's icon node
        // - onActionMouseOver(): fires when mouse pointer is moved over the container's icon node
        // - onActionMouseOut(): fires when mouse pointer is moved off the container's icon node
        //
        // example: (single "string" value),
        // iconClass: "myIconClass"
        iconClass: "",

        // iconTitle: String|Object
        // If the container's action icon node is visible, this attribute specifies title text to be
        // displayed for mouseover events for the icon node.
        //
        // example: (single "string" value)
        // iconTitle: "Expand section for editing"
        iconTitle: "",

        // actionHidden: boolean
        // If the container's action icon node is enabled, this attribute specifies whether it is
        // hidden or on display at startup. If "true", the action icon node is hidden and must be
        // set to display using showActionIcon(). Default is "false" (on display).
        actionHidden: false,

        // Override this method to perform custom behavior during dijit construction.
        // Common operations for constructor:
        // 1) Initialize non-primitive types (i.e. objects and arrays)
        // 2) Add additional properties needed by succeeding lifecycle methods
        constructor: function() {

        },

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {

            this.inherited(arguments);

        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);

            // Set up this instance's title node, if enabled.
            if (this.title) {
                dojo.addClass(this.outlineNode, "opWithTitleBar");
            }
            else {
                dojo.removeClass(this.titleNode, "opTitle");
            }
            // Set up this instance's action icon node, if enabled.
            if (!this.iconClass && this.enableEditIcon) {
                this.iconClass = "opEditIcon"; // standard edit icon
            }
            if (this.iconClass) {
                dojo.addClass(this.iconNode, this.iconClass);
                dojo.attr(this.iconNode, "title", this.iconTitle);
                dojo.attr(this.iconNode, "tabIndex", this.tabIndex);
                this.showActionIcon(!this.actionHidden);
            }
            else {
                dojo.attr(this.iconNode, "visibility", "hidden");
            }

        },

        showActionIcon: function(/* boolean */newVal) {

            dojo.style(this.iconNode, "display", newVal ? "" : "none");
            this.actionHidden = !newVal;

        },

        _calculateDim: function(input, max) {
            // tags:
            // protected

            // Check for units label and handle accordingly...
            var unit = (input.indexOf("%") >= 0) ? "%" : ((input.indexOf("em") >= 0) ? "em" : "px");
            var val = input.replace(/[^0-9]/g, ""); // strips unit label, and other garbage
            switch (unit) {
                case "%":
                    val = Math.round(max * val / 100);
                    break;
                case "em":
                    val = val * 12; // Approximation, per em
                    break;
                case "px":
                default:
                    break;
            }
            return Math.min(val, max);

        },

        resize: function(newSize) {
            
            var dim = {};
            var pmb = dojo.marginBox(this.getParent().domNode);
            if (this.width && (!this._pmbSave || (this._pmbSave["w"] != pmb["w"]))) {
                dim["w"] = this._calculateDim(this.width, pmb["w"]);
            }
            if (this.height && (!this._pmbSave || (this._pmbSave["h"] != pmb["h"]))) {
                dim["h"] = this._calculateDim(this.height, pmb["h"]);
            }
            dojo.marginBox(this.domNode, dim);
            dojo.marginBox(this.outlineNode, dim);
            
            this.inherited(arguments);
            
            // save the parent's margin box data for reuse; do this after
            //   calling dojo.marginBox(...) on the domNode and outlineNode,
            //   and also after calling this.inherited(...) because IE 8 will
            //   increase the size even more than the dimensions we specified
            //   and fire a resize event, causing infinite resizing and
            //   automatically increasing dialog size
            this._pmbSave = dojo.marginBox(this.getParent().domNode);

        },

       onActionClick: function(evt) {
            // Does nothing.
            // Override to specify custom handler.
        },

        onActionMouseOver: function(evt) {
            // Does nothing.
            // Override to specify custom handler.
        },

        onActionMouseOut: function(evt) {
            // Does nothing.
            // Override to specify custom handler.
        },

        _onActionClick: function(evt) {
            // tags:
            // protected
            this.onActionClick(evt);
        },

        _onActionMouseOver: function(evt) {
            // tags:
            // protected
            this.onActionMouseOver(evt);
        },

        _onActionMouseOut: function(evt) {
            // tags:
            // protected
            this.onActionMouseOut(evt);
        }

    });
});