define([
        "dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/cmmGeneral",
        "dojo/text!./templates/cmmGeneral.html",
        "dojo/dom-geometry",
        "dijit/TooltipDialog",
        "support/widgets/Icon",
        "dijit/form/Select",
        "dojo/data/ItemFileWriteStore",
        "support/base/TableMixin",
        "dijit/form/SimpleTextarea",
        "dijit/TitlePane"
    ], function(dojo, dijit, page, mri, template, domGeometry) {

    return dojo.declare("support.wizard.pages.cmmGeneral", [ support.panels.Panel, support.base.TableMixin ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
        
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // editMode: boolean
        // Specifies whether the wizard is to execute in edit mode. If "true", the "templateIDNum"
        // attribute is required.
        editMode: false,

        // copyMode: boolean
        // Specifies whether the wizard is to execute in copy mode. If "true", the "templateIDNum"
        // attribute is required.
        copyMode: false,

        // inUse: boolean
        // Specifies whether the wizard is to execute in inUse mode.
        inUse: false,
        
        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "nstWizard",

        FROM_CHASSIS: "fromChassis",
        FROM_SCRATCH: "fromScratch",

        _chassisTable: null,
        
        //Variables to hold previously selected type/ff choices
        _formFactorValue: null,
        _formFactorConfirmEnabled: false, // Enabled for selector "onChange" events after initial set.
        _nodeTypeValue: null,
        _nodeTypeConfirmEnabled: false, // Enabled for selector "onChange" events after initial set.
        
        // ID prefix used for table
        gridID: "ChassisTable",

        // Override this method to perform custom behavior during dijit
        // construction.
        // Common operations for constructor:
        // 1) Initialize non-primitive types (i.e. objects and arrays)
        // 2) Add additional properties needed by succeeding lifecycle methods
        constructor: function() {
        },

        // When this method is called, all variables inherited from superclasses
        // are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in
        // the template HTML file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to
        // override when
        // you need to access and/or manipulate DOM nodes included with your
        // widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified
        // can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);

            //Setup Dialog Text
            this._descTitle.innerHTML = this.resources.DESCRIPTION + " (" + this.resources.LIMIT.replace("%500", "500") + ")";
            
            // Set up User Assistance tooltip
            var uaTooltip = new dijit.TooltipDialog({
                style: {
                    width: "550px"
                },
                content: this.locResources.ASSISTANCE_1 + "<p>" + this.locResources.ASSISTANCE_2 + "</p>"
            }); 
            var uaButton = new dijit.form.DropDownButton({
                showLabel: false,
                baseClass: "nstTooltip",
                iconClass: "spTooltipIcon",
                dropDown: uaTooltip
            });
            dojo.connect(uaButton, "onMouseOver", function(e){
                uaButton.openDropDown();
                uaButton._onDropDownMouseUp(e);
            });
            dojo.connect(uaTooltip, "onMouseLeave", uaButton, function(){
                uaButton.closeDropDown();
            });
            uaButton.placeAt(this._uaTooltipButton);

            if (!this.editMode && !this.copyMode) {
                // Setup table widget...
                var chassisStore = this._retrieveStore();       // Get table store information
                var tableStructure = this._generateServerLayout();  // Get structure for current view            
                
                this._chassisTable = this.getTable(chassisStore, tableStructure, this.resources.NO_DATA, this.gridID, true);
                this._chassisTable.column(0).sort(false);

                this._chassisTable.select.row.multiple = false; // set table to single select
                
                this._chassisTable.placeAt(this._chassisTableNode);
                
                dojo.connect(this._chassisTable.select.row, "onSelectionChange", this, this._onChangeChassis);

                this._chassisTable.startup();
            }            
        },

        startup: function() {

            //Set initial field values
            (this.pageParams.templateName) && this._templateName.set("value", this.pageParams.templateName + ((this.copyMode) ? " - " + this.resources.COPY : ""));
            (this.pageParams.templateDesc) && this._templateDescription.set("value", this.pageParams.templateDesc);

            if (this.copyMode || this.editMode) {
                // Hide these containers at startup.
                if (this.editMode) {
                    dojo.style(this._userAssistanceContainer, "display", "none");
                    //Display in use warning
                    if(this.inUse){
                        this.warningMessage.style.display = "block";
                        this.warningMessageText.innerHTML = this.resources.INUSE;
                    }
                }
                dojo.style(this._startingPointContainer, "display", "none");
                dojo.style(this._chassisInputContainer, "display", "none");
                
            }
            else {
                // Hide these containers at startup.
                dojo.style(this._chassisInputContainer, "display", "none");
                dojo.style(this._nameInputContainer, "display", "none");
            }
            
            if (this.existing) {
                this._onSetupFromChassis();
            }
            
            // Start all our children widgets too.
            this.inherited(arguments);

        },

        _chassisSelectMode: function(/* boolean */readOnly) {

            // tags:
            // protected
            //dojo.style(this._chassisTableNode, "display", (readOnly) ? "none" : "");
            //dojo.style(this._chassisDisplayNode, "display", (readOnly) ? "" : "none");

            // Show action icon only in "read-only" mode so user may switch back to "edit mode" if
            // they wish to do so.
            //this._chassisTableContainer.showActionIcon(readOnly);

            this._chassisTable && this._chassisTable.resize();

        },

        _startingPointSelectMode: function(/* boolean */readOnly) {

            // tags:
            // protected
            //dojo.style(this._startingPointIconsNode, "display", (readOnly) ? "none" : "");
            //dojo.style(this._startingPointDisplayNode, "display", (readOnly) ? "" : "none");

            // Show action icon only in "read-only" mode so user may switch back to "edit mode" if
            // they wish to do so.
            //this._startingPointIconsContainer.showActionIcon(readOnly);

        },

        _retrieveStore: function() {

            // summary:
            // Connect to data store and create models for TreeTable widgets.
            // tags:
            // protected
            var chassisData = "";

            // Determine form factor based on given bay information
            // Entries w/o an arch entry, aka empty bays, get no formFactor
            var determineFormFactor = function(bays, arch){
                if(bays && arch && bays.length > 0){
                    if(bays.length == 1 && arch.length > 0){
                        return "2s";
                    }
                    if(bays.length == 2 && arch.length > 0){
                        return "4s";
                        //99300 Determine difference b 2Bay & 2Bay w/ Expansion IO
                    }
                }
                return "";
            };

            // Determine internal usage type string based on arch
            var determineType = function(arch){
                if(arch){
                    if(arch == "x86_32" || arch == "x86_64" || arch == "x86"){
                        return "sysx";
                    }
                    else if(arch == "ppc32" || arch == "ppc64"){
                        return "power";
                    }
                }
                return "";
            };
                
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "chassis",
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: function(response) {

                    var i,j;
                    for (i = 0; i < response.items.length; i++) {
                        if (response.items[i] && response.items[i].children) {
                            for (j = 0; j < response.items[i].children.length; j++) {
                                // Assign formFactors
                                var ff = (response.items[i].children[j].bays && response.items[i].children[j].architecture) 
                                      ? determineFormFactor(response.items[i].children[j].bays,response.items[i].children[j].architecture) 
                                      : "";
                                response.items[i].children[j].formFactor = ff;
                                // Assign Arch Type(sysx or power)
                                var type = (response.items[i].children[j].architecture) ? determineType(response.items[i].children[j].architecture) : "";
                                response.items[i].children[j].archType = type;       
                            }
                        }
                    }
                    chassisData = dojo.toJson({
                        "identifier": "id",
                        "label": "name",
                        "items": response.items
                    }, true); 
                }
            });
            
            var chassisStore = new dojo.data.ItemFileWriteStore({
                data: dojo.fromJson(chassisData)
            });

            if (!("hasChildren" in chassisStore)) {
                chassisStore.hasChildren = function(id, item) { 
                    return item && this.getValues(item, "children").length;
                };
            };
            
            if (!("getChildren" in chassisStore)) {
                chassisStore.getChildren = function(item) { 
                    return this.getValues(item, "children");
                };
            };

            return chassisStore;

        },

        _generateServerLayout: function() {
            var chassisLayout = [{
                field: "name",
                name: this.locResources.NAME,
                decorator: dojo.hitch(this, this._decorateChassisName),
                width: "200px"
            }, {
                field: "ipaddresses",
                name: this.locResources.IPADDR,
                decorator: this._decorateIPAddresses,
                width: "auto"
            }];
            return chassisLayout;
        },

        _decorateChassisName: function(value) {
            // tags:
            // protected
            !value && (value = "");
            return "<span title=\"" + value + "\" style=\"white-space: nowrap;\"><img src=\"" + this._blankGif + "\" alt=\"\" class=\"spChassisIcon\" style=\"vertical-align: middle;\">&nbsp;" + value + "</span>";
        },

        _decorateIPAddresses: function(value) {
            // tags:
            // protected
            !value && (value = "");
            return "<span title=\"" + value + "\">" + value + "</span>";
        },

        AfterPageSelected: function() {

            // Validate page input and enable/disable "Next" button of wizard accordingly.
            dojo.publish("enableNextButton", this.validate());

        },

        validate: function() {
            if (this.copyMode || this.editMode) {
                if (this._templateName.get("value").toString().length <= 0) {
                    // Template name has not been specified.
                    // console.info(this.declaredClass + ": Template name has not been specified.");
                    return false;
                }
            }
            else {
                if (this._startingPoint.value == "") {
                    // A starting point has not been selected.
                    // console.info(this.declaredClass + ": A starting point has not been selected.");
                    return false;
                }
                if (this._startingPoint.value == this.FROM_CHASSIS) {
                    var found = dojo.some(this._chassisTable.select.row.getSelected(), dojo.hitch(this, function(entry, index) {
                        var item = this._chassisTable.row(entry, true).item();
                        return (item.type == "chassis");
                    }));
                    if (!found) {
                        // Start FROM_CHASSIS but none are selected.
                        // console.info(this.declaredClass + ": Start \"" + this.FROM_CHASSIS + "\" but none are selected.");
                        return false;
                    }
                }
                if (this._templateName.get("value").toString().length <= 0) {
                    // Template name has not been specified.
                    // console.info(this.declaredClass + ": Template name has not been specified.");
                    return false;
                }
            }

            // console.info(this.declaredClass + ": Everything checks out on this page!");
            return true;

        },
        
        setTemplateName: function(newValue) {
            this._templateName.set("value", newValue);
        },

        onChangeTemplateName: function(newValue) {
            console.log(this.declaredClass + ": onChangeTemplateName called: ", newValue);
            // Connect handler.
        },

        _onChangeTemplateName: function(newValue) {

            // tags:
            // protected

            // Validate page input and enable/disable "Next" button of wizard accordingly.
            dojo.publish("enableNextButton", this.validate());
            
            // Call connect handler
            this.onChangeTemplateName(newValue);
        },
        
        setTemplateDescription: function(newValue) {
            this._templateDescription.set("value", newValue);
        },

        onChangeTemplateDescription: function(newValue) {
            console.log(this.declaredClass + ": onChangeTemplateDescription called: ", newValue);
            // Connect handler.
        },

        _onChangeTemplateDescription: function(newValue) {
            // tags:
            // protected
            // Call connect handler
            this.onChangeTemplateDescription(newValue);
        },

        onChangeNodeType: function(newValue) {
            console.log(this.declaredClass + ": onChangeNodeType called: ", newValue);
            // Connect handler.
        },

        _onChangeNodeType: function(newValue) {
            // tags:
            // protected
            // Call connect handler
          
        },

        onChangeFormFactor: function(newValue) {
            console.log(this.declaredClass + ": onChangeFormFactor called: ", newValue);
            // Connect handler.
        },

        _onChangeFormFactor: function(newValue) {
            // tags:
            // protected
            // Call connect handler
          
        },

        onChangeChassis: function(newValue) {
            console.log(this.declaredClass + ": onChangeChassis called: ", newValue);
            // Connect handler.
        },

        _onChangeChassis: function() {
            // tags:
            // protected
            var ids = this._chassisTable.select.row.getSelected();
            if (ids.length <= 0) {
                return;
            }
            var row = this._chassisTable.row(ids[0], true);
            var rawData = row.rawData();
            var chassisName = rawData["name"];
            var ipAddrs = rawData["ipaddresses"];
            this._chassisSelectMode(true);
            var newValue = {
                chassisName: chassisName,
                chassisUUID: rawData["uuid"]
            };

            // Validate page input and enable/disable "Next" button of wizard accordingly.
            dojo.publish("enableNextButton", this.validate());
                
            this._chassisTablePane.toggle();
            var title = this.locResources.LEARN_FROM + "&nbsp;&nbsp;" + chassisName + " &mdash; " + ipAddrs;
            var maxWidth = (domGeometry.getContentBox(this._chassisTablePane.domNode).w - 40);
            this._chassisTablePane.set("title", "<div title=\"" + title + "\" style=\"display:inline-block;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;max-width:" + maxWidth + "px;\">" + title + "</div>");

            // Call connect handler
            this.onChangeChassis(newValue);
        },

        onChangeStartingPoint: function(newValue) {
            console.log(this.declaredClass + ": onChangeStartingPoint called ", newValue);
            // Connect handler.
        },

        _onSetupFromChassis: function() {

            // tags:
            // protected
            this._startingPoint.value = this.FROM_CHASSIS;
            this._startingPointSelectMode(true); // set read-only mode
            dojo.style(this._chassisInputContainer, "display", "");
            dojo.style(this._nameInputContainer, "display", "");
            this._chassisTable && this._chassisTable.resize();

            // Validate page input and enable/disable "Next" button of wizard accordingly.
            dojo.publish("enableNextButton", this.validate());
            
            this._startingPointPane.toggle();
            this._startingPointPane.set("title", this.locResources.START_FROM + "&nbsp;&nbsp;<span style=\"font-weight: normal;\"> " + this.locResources.FROM_CHASSIS + "</span>");
            this._startingPointPane.set("tooltip", this.locResources.FROM_CHASSIS);
            
            // Call connect handler
            this.onChangeStartingPoint(this.FROM_CHASSIS);
            this._onChangeChassis(); // Call to inform of pre-selected chassis, if any.

        },

        _onSetupFromScratch: function() {

            // tags:
            // protected
            this._startingPoint.value = this.FROM_SCRATCH;
            this._startingPointSelectMode(true); // set read-only mode
            dojo.style(this._chassisInputContainer, "display", "none");
            dojo.style(this._nameInputContainer, "display", "");

            //Hide header warning just in case
            dojo.style(this.headerWarningMessage, "display", "");
            
            // Validate page input and enable/disable "Next" button of wizard accordingly.
            dojo.publish("enableNextButton", this.validate());
            
            this._startingPointPane.toggle();
            this._startingPointPane.set("title", this.locResources.START_FROM + "&nbsp;&nbsp;<span style=\"font-weight: normal;\"> " + this.locResources.FROM_SCRATCH + "</span>");
            this._startingPointPane.set("tooltip", this.locResources.FROM_SCRATCH);

            // Call connect handler
            this.onChangeStartingPoint(this.FROM_SCRATCH);

        },
        
        setNameAndDesc: function(name, desc){
            this._templateName.set("value", name);
            this._templateDescription.set("value", desc);
        }

    });
});