define([ "dojo",
         "dijit",
         "support/panels/Panel",
         "dojo/i18n!./nls/cmmIOModules",
         "dojo/text!./templates/cmmIOModules.html",
         "dojo/data/ItemFileWriteStore",
         "dijit/form/Select",
         "support/widgets/Select",
         "support/dialogs/ChassisIOModuleTemplate",
         "dijit/form/CheckBox"
         ], function(dojo, dijit, page, mri, template) {

            return dojo.declare("support.wizard.pages.cmmIOModules", [ support.panels.Panel ], {

                // templateString: [protected] String
                // This widget's HTML template, provided.
                templateString: template,

                // locResources: [protected] Object
                // This widget's National Language Source (NLS) object.
                locResources: mri,

                // editMode: boolean
                // Specifies whether the wizard is to execute in edit mode. If "true", the "templateIDNum"
                // attribute is required.
                editMode: false,

                // copyMode: boolean
                // Specifies whether the wizard is to execute in copy mode. If "true", the "templateIDNum"
                // attribute is required.
                copyMode: false,

                // inUse: boolean
                // Specifies whether the wizard is to execute in inUse mode.
                inUse: false,

                // baseClass: [protected] String
                // The root CSS className to be placed on this widget's domNode.
                baseClass: "nstWizard",

                // Internal Selector Names
                SYSTMPL_NAME: "systemInfoTemplate",
                MGMTTMPL_NAME: "managementTemplate",
                PERFORMTMPL_NAME: "performanceTemplate",
                PWRTMPL_NAME: "powerTemplate",

                // Template Type Names - Internal Usage
                IO1: "IO1",
                IO2: "IO2",
                IO3: "IO3",
                IO4: "IO4",
                IOALL: "IOALL",

                SELECTOR_WIDTH: "400px",
                
                allModulesWarning: false,
                module1Warning : false,
                module2Warning : false,
                module3Warning : false,
                module4Warning : false,
                
                selectedIndex:-1,

                // Override this method to perform custom behavior during dijit construction.
                // Common operations for constructor:
                // 1) Initialize non-primitive types (i.e. objects and arrays)
                // 2) Add additional properties needed by succeeding lifecycle methods
                constructor: function() {
                    this.resources = dojo.i18n.getLocalization("support.panels", "Panel", this.lang);
                },

                // When this method is called, all variables inherited from superclasses
                // are "mixed in".
                // Common operations for postMixInProperties
                // 1) Modify or assign values for widget property variables defined in
                // the template HTML file
                postMixInProperties: function() {

                    this.inherited(arguments);

                },

                // postCreate() is called after buildRendering(). This is useful to
                // override when
                // you need to access and/or manipulate DOM nodes included with your
                // widget.
                // DOM nodes and widgets with the dojoAttachPoint attribute specified
                // can now be directly
                // accessed as fields on "this".
                // Common operations for postCreate
                // 1) Access and manipulate DOM nodes created in buildRendering()
                // 2) Add new DOM nodes or widgets
                postCreate: function() {

                    this.inherited(arguments);
                    // Set bay # text
                    this._module1Label.innerHTML = this.resources.BAY_NUM.replace("%1", "1") + ":";
                    this._module2Label.innerHTML = this.resources.BAY_NUM.replace("%1", "2") + ":";
                    this._module3Label.innerHTML = this.resources.BAY_NUM.replace("%1", "3") + ":";
                    this._module4Label.innerHTML = this.resources.BAY_NUM.replace("%1", "4") + ":";

                    this._selectAll = new dijit.form.CheckBox({
                        id: this.id + "_selectallcheckbox",
                        checked: false,
                        onChange: dojo.hitch(this, function(checked) {
                            if (checked) {
                                // dojo.style(this._indivSelectors, "display", "none");
                                dojo.style(this._selectAllWarning,"display", "");
                                dojo.style(this._allModules, "display", "");
                                dojo.style(this._module1, "display", "none");
                                dojo.style(this._module2, "display", "none");
                                dojo.style(this._module3, "display", "none");
                                dojo.style(this._module4, "display", "none");
                            }
                            else {
                                // dojo.style(this._indivSelectors, "display", "");
                                dojo.style(this._allModules, "display", "none");
                                dojo.style(this._selectAllWarning,"display", "none");
                                dojo.style(this._module1, "display", "");
                                dojo.style(this._module2, "display", "");
                                dojo.style(this._module3, "display", "");
                                dojo.style(this._module4, "display", "");
                            }
                        })
                    }).placeAt(this._selectAll);
                    this._setupAllTmplSelector();
                    this._setupModule1Selector();
                    this._setupModule2Selector();
                    this._setupModule3Selector();
                    this._setupModule4Selector();
                    this._setupSubscribers();
                    
                    this.subscribe("ChassisIOTemplateCancelled", function(){
                        this.selectedIndex = -1;
                    });

                },

                _setStartingPointAttr: function(newval) {
                   this._set("startingPoint", newval);
                   if (this.startingPoint == "FROM_SCRATCH") {                   
                      if (this.infoMessage) {
                         this.infoMessage.destroyRecursive();
                      }                      
                   } else {                       
                       
                       if(this.CP_RUN_ENVIRONMENT == "fsm"){

                           require([ "idx/oneui/messaging/SingleMessage" ], dojo.hitch(this, function() {

                                this.infoMessage = new idx.oneui.messaging.SingleMessage({
                                   description: this.resources.LEARNED_CHASSIS_SETTINGS_DETAILS,
                                   type: "information",
                                   dateFormat: { formatLength: "medium", selector: "time" },
                                   showId: false,
                                   showAction: false,
                                   showRefresh: false,
                                   showDetailsLink: false
                                }).placeAt(this.id + "_messagearea");
                                this.infoMessage.set("title", this.resources.LEARNED_CHASSIS_SETTINGS_INFO.replace("%1", this.chassis));
                      
                           }));
                           
                       } else if(this.CP_RUN_ENVIRONMENT == "augusta") {
                           
                           require([ "idx/widget/SingleMessage" ], dojo.hitch(this, function() {

                                this.infoMessage = new idx.widget.SingleMessage({
                                   description: this.resources.LEARNED_CHASSIS_SETTINGS_DETAILS,
                                   type: "information",
                                   dateFormat: { formatLength: "medium", selector: "time" },
                                   showId: false,
                                   showAction: false,
                                   showRefresh: false,
                                   showDetailsLink: false
                                }).placeAt(this.id + "_messagearea");
                                this.infoMessage.set("title", this.resources.LEARNED_CHASSIS_SETTINGS_INFO.replace("%1", this.chassis));
                      
                           }));          
                           
                       }
                       
                   }
                },

                _setChassisAttr: function(chassis) {
                   this._set("chassis", chassis);
                   this.infoMessage.set("title", this.resources.LEARNED_CHASSIS_SETTINGS_INFO.replace("%1", this.chassis));
                },
                
                _presetSelector: function(size, selector) {
                    // tags:
                    // protected
                    // No templates for this category?
                    if (size <= 0) {
                        // Disable selection and allow "emptyLabel" to display.
                        selector.set("disabled", true);
                    }
                    else {
                        var curTemplates = null;
                        var i;
                        if (this.pageParams) {
                            for(i=0;i<this.pageParams.length;i++){
                                // Pick preselected id
                                if (selector.name == this.IO1 && this.pageParams[i].index == "1") {
                                    curTemplates = this.pageParams[i].id;
                                    break;
                                }
                                else if (selector.name == this.IO2 && this.pageParams[i].index == "2") {
                                    curTemplates = this.pageParams[i].id;
                                    break;
                                }
                                else if (selector.name == this.IO3 && this.pageParams[i].index == "3") {
                                    curTemplates = this.pageParams[i].id;
                                    break;
                                }
                                else if (selector.name == this.IO4 && this.pageParams[i].index == "4") {
                                    curTemplates = this.pageParams[i].id;
                                    break;
                                }
                            }                         
                        }

                        // Set drop-down value
                        if (curTemplates) {
                            selector.set("value", curTemplates);
                        }
                        else {
                            selector.set("value", "-1");
                        }
                    }
                    
                    this.allModulesWarning = true;
                    this.module1Warning = true;
                    this.module2Warning = true;
                    this.module3Warning = true;
                    this.module4Warning = true;

                },

                _setupModule1Selector: function() {

                    // summary:
                    // Setup SystemInfo template selector...
                    // tags:
                    // protected
                    var tmplStore = this._regenerateSelectorStore();
                    this._module1Selector = new support.widgets.Select({
                        id: this.get("id") + "_module1TmplSelector",
                        name: this.IO1,
                        required: false,
                        store: tmplStore,
                        emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                        style: {
                            width: this.SELECTOR_WIDTH
                        }
                    }).placeAt(this._module1Selector);
                    // call startup(); crucial for support.widgets.Select Dijits
                    this._module1Selector.startup();
                    // preload selector with saved values
                    this._presetSelector(tmplStore._arrayOfAllItems.length, this._module1Selector);
                    this._module1Selector.onChange = dojo.hitch(this, this._changeModule1Callback);

                },

                _refreshModule1Selector: function() {
                    var tmplStore = this._regenerateSelectorStore();
                    this._module1Selector.setStore(tmplStore);
                },

                _changeModule1Callback: function(newVal) {
                    // tags:
                    // protected
                    dojo.style(this._module1Selector, "display", "none");
                    if (newVal > 0) {
                        dojo.replaceClass(this._module1EditOption, "nstEditIcon", "nstEditIconDisabled");
                        this._module1EditOption.title = this.resources.EDIT_SELECTED_PATTERN;
                    }
                    else {
                        dojo.replaceClass(this._module1EditOption, "nstEditIconDisabled", "nstEditIcon");
                        this._module1EditOption.title = this.resources.EDIT_SELECTED_PATTERN_DISABLED;
                    }

                },

                _createModule1: function() {
                    var dialog = new support.dialogs.ChassisIOModuleTemplate({
                        id: this.id + "cp_cmmiomodules_createchassisiomoduledialog"
                    });
                    this.selectedIndex = 1;
                    dialog.show();
                },

                _editModule1: function() {
                    var templateId = this._module1Selector.value;

                    if (templateId >= 0) {
                        templateId = templateId.toString();
                        var dialog = new support.dialogs.ChassisIOModuleTemplate({
                            id: this.id + "cp_cmmiomodules_editchassisiomoduledialog",
                            templateIDNum: templateId,
                            templateMode: "edit",
                            templateInUse: false,
                            templateLaunchPoint: "wizard"
                        });
                        this.selectedIndex = 1;
                        dialog.show();
                    }
                },

                _module1SelectorOnClick: function(/* Event */evt) {

                    // tags:
                    // protected
                    // 99299: Wade Fode, 8/2/2012. Future consideration to display contents of the
                    // selected template in an expandable section beneath the selector.
                    // if (dojo.style(this._sysTmplContainer, "display") === "block") {
                    // dojo.style(this._sysTmplContainer, "display", "none");
                    // dojo.replaceClass(this._sysTmplSelectorExpander, "expandTwistie", "noTwistie
                    // collapseTwistie");
                    // }
                    // else {
                    // dojo.style(this._sysTmplContainer, "display", "block");
                    // dojo.replaceClass(this._sysTmplSelectorExpander, "collapseTwistie",
                    // "noTwistie
                    // expandTwistie");
                    // }

                },

                _setupModule2Selector: function() {

                    // summary:
                    // Setup SystemInfo template selector...
                    // tags:
                    // protected
                    var tmplStore = this._regenerateSelectorStore();
                    this._module2Selector = new support.widgets.Select({
                        id: this.get("id") + "_module2TmplSelector",
                        name: this.IO2,
                        required: false,
                        store: tmplStore,
                        emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                        style: {
                            width: this.SELECTOR_WIDTH
                        }
                    }).placeAt(this._module2Selector);
                    // call startup(); crucial for support.widgets.Select Dijits
                    this._module2Selector.startup();
                    // preload selector with saved values
                    this._presetSelector(tmplStore._arrayOfAllItems.length, this._module2Selector);
                    this._module2Selector.onChange = dojo.hitch(this, this._changeModule2Callback);

                },

                _refreshModule2Selector: function() {
                    var tmplStore = this._regenerateSelectorStore();
                    this._module2Selector.setStore(tmplStore);
                },

                _changeModule2Callback: function(newVal) {

                    // tags:
                    // protected
                    dojo.style(this._module2Selector, "display", "none");
                    if (newVal > 0) {
                        dojo.replaceClass(this._module2EditOption, "nstEditIcon", "nstEditIconDisabled");
                        this._module2EditOption.title = this.resources.EDIT_SELECTED_PATTERN;
                    }
                    else {
                        dojo.replaceClass(this._module2EditOption, "nstEditIconDisabled", "nstEditIcon");
                        this._module2EditOption.title = this.resources.EDIT_SELECTED_PATTERN_DISABLED;
                    }

                },

                _createModule2: function() {
                    var dialog = new support.dialogs.ChassisIOModuleTemplate({
                        id: this.id + "cp_cmmiomodules_createchassisiomoduledialog2"
                    });
                    this.selectedIndex = 2;
                    dialog.show();
                },

                _editModule2: function() {
                    var templateId = this._module2Selector.value;

                    if (templateId >= 0) {
                        templateId = templateId.toString();
                        var dialog = new support.dialogs.ChassisIOModuleTemplate({
                            id: this.id + "cp_cmmiomodules_editchassisiomoduledialog2",
                            templateIDNum: templateId,
                            templateMode: "edit",
                            templateInUse: false,
                            templateLaunchPoint: "wizard"
                        });
                        this.selectedIndex = 2;
                        dialog.show();
                    }
                },

                _module2SelectorOnClick: function(/* Event */evt) {

                    // tags:
                    // protected
                    // 99299: Wade Fode, 8/2/2012. Future consideration to display contents of the
                    // selected template in an expandable section beneath the selector.
                    // if (dojo.style(this._sysTmplContainer, "display") === "block") {
                    // dojo.style(this._sysTmplContainer, "display", "none");
                    // dojo.replaceClass(this._sysTmplSelectorExpander, "expandTwistie", "noTwistie
                    // collapseTwistie");
                    // }
                    // else {
                    // dojo.style(this._sysTmplContainer, "display", "block");
                    // dojo.replaceClass(this._sysTmplSelectorExpander, "collapseTwistie",
                    // "noTwistie
                    // expandTwistie");
                    // }

                },

                _presetSelectorValues: function() {
                    var i;
                    for (i = 0; i < this.pageParams.length; i++) {
                        if (this.pageParams[i].type == "cmmIOModule") {
                            if (this.pageParams[i].index == "1") {
                                this._module1Selector.set("value", this.pageParams[i].id);
                                this._module1Selector.set("disabled", false);
                            }
                            else if (this.pageParams[i].index == "2") {
                                this._module2Selector.set("value", this.pageParams[i].id);
                                this._module2Selector.set("disabled", false);
                            }
                            else if (this.pageParams[i].index == "3") {
                                this._module3Selector.set("value", this.pageParams[i].id);
                                this._module3Selector.set("disabled", false);
                            }
                            else if (this.pageParams[i].index == "4") {
                                this._module4Selector.set("value", this.pageParams[i].id);
                                this._module4Selector.set("disabled", false);
                            }
                        }

                    }
                },

                refreshAllSelectors: function() {
                    this._refreshModule1Selector();
                    this._refreshModule2Selector();
                    this._refreshModule3Selector();
                    this._refreshModule4Selector();
                    this._refreshAllInfoSelector();

                    this._presetSelectorValues();
                },

                _setupAllTmplSelector: function() {

                    // summary:
                    // Setup SystemInfo template selector...
                    // tags:
                    // protected
                    var tmplStore = this._regenerateSelectorStore();
                    this._allTmplSelector = new support.widgets.Select({
                        id: this.get("id") + "_allTmplSelector",
                        name: this.IOALL,
                        required: false,
                        store: tmplStore,
                        emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                        style: {
                            width: this.SELECTOR_WIDTH
                        },
                        onChange: dojo.hitch(this, this._changeAllTmplCallback)
                    }).placeAt(this._allTmplSelector);
                    // call startup(); crucial for support.widgets.Select Dijits
                    this._allTmplSelector.startup();
                    // preload selector with saved values
                    this._presetSelector(tmplStore._arrayOfAllItems.length, this._allTmplSelector);
                    this._allTmplSelector.onChange = dojo.hitch(this, this._changeAllTmplCallback);
                   
                },

                _refreshAllInfoSelector: function() {
                    var tmplStore = this._regenerateSelectorStore();
                    this._allTmplSelector.setStore(tmplStore);
                },

                _changeAllTmplCallback: function(newVal) {

                    // tags:
                    // protected
                    dojo.style(this._allTmplSelector, "display", "none");
                    if (newVal > 0) {
                        dojo.replaceClass(this._allTmplEditOption, "nstEditIcon", "nstEditIconDisabled");
                        this._allTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN;
                    }
                    else {
                        dojo.replaceClass(this._allTmplEditOption, "nstEditIconDisabled", "nstEditIcon");
                        this._allTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN_DISABLED;
                    }

                },
                _editAllTmpl: function() {
                    var templateId = this._allTmplSelector.get("value");
                    if (templateId >= 0) {
                        templateId = templateId.toString();
                        var dialog = new support.dialogs.ChassisIOModuleTemplate({
                            id: this.id + "cp_cmmiomodules_editchassisiomoduledialogall",
                            templateIDNum: templateId,
                            templateMode: "edit",
                            templateInUse: false,
                            templateLaunchPoint: "wizard"
                        });
                        this.selectedIndex = 0;
                        dialog.show();
                    }
                },
                _createAllTmpl: function() {
                    var dialog = new support.dialogs.ChassisIOModuleTemplate({
                        id: this.id + "cp_cmmiomodules_createchassisiomoduledialogall"
                    });
                    this.selectedIndex = 0;
                    dialog.show();
                },

                _setupModule4Selector: function() {

                    // summary:
                    // Setup SystemInfo template selector...
                    // tags:
                    // protected
                    var tmplStore = this._regenerateSelectorStore();
                    this._module4Selector = new support.widgets.Select({
                        id: this.get("id") + "_module4TmplSelector",
                        name: this.IO4,
                        required: false,
                        store: tmplStore,
                        emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                        style: {
                            width: this.SELECTOR_WIDTH
                        }
                    }).placeAt(this._module4Selector);
                    // call startup(); crucial for support.widgets.Select Dijits
                    this._module4Selector.startup();
                    // preload selector with saved values
                    this._presetSelector(tmplStore._arrayOfAllItems.length, this._module4Selector);
                    this._module4Selector.onChange = dojo.hitch(this, this._changeModule4Callback);

                },

                _refreshModule4Selector: function() {
                    var tmplStore = this._regenerateSelectorStore();
                    this._module4Selector.setStore(tmplStore);
                },

                _changeModule4Callback: function(newVal) {

                    // tags:
                    // protected
                    dojo.style(this._module4Selector, "display", "none");
                    if (newVal > 0) {
                        dojo.replaceClass(this._module4EditOption, "nstEditIcon", "nstEditIconDisabled");
                        this._module4EditOption.title = this.resources.EDIT_SELECTED_PATTERN;
                    }
                    else {
                        dojo.replaceClass(this._module4EditOption, "nstEditIconDisabled", "nstEditIcon");
                        this._module4EditOption.title = this.resources.EDIT_SELECTED_PATTERN_DISABLED;
                    }

                },

                _createModule4: function() {
                    var dialog = new support.dialogs.ChassisIOModuleTemplate({
                        id: this.id + "cp_cmmiomodules_createchassisiomoduledialog4"
                    });
                    this.selectedIndex = 4;
                    dialog.show();
                },

                _editModule4: function() {
                    var templateId = this._module4Selector.value;

                    if (templateId >= 0) {
                        templateId = templateId.toString();
                        var dialog = new support.dialogs.ChassisIOModuleTemplate({
                            id: this.id + "cp_cmmiomodules_editchassismoduledialog4",
                            templateIDNum: templateId,
                            templateMode: "edit",
                            templateInUse: false,
                            templateLaunchPoint: "wizard"
                        });
                        this.selectedIndex = 4;
                        dialog.show();
                    }
                },

                _module4SelectorOnClick: function(/* Event */evt) {

                    // tags:
                    // protected
                    // 99299: Wade Fode, 8/2/2012. Future consideration to display contents of the
                    // selected template in an expandable section beneath the selector.
                    // if (dojo.style(this._sysTmplContainer, "display") === "block") {
                    // dojo.style(this._sysTmplContainer, "display", "none");
                    // dojo.replaceClass(this._sysTmplSelectorExpander, "expandTwistie", "noTwistie
                    // collapseTwistie");
                    // }
                    // else {
                    // dojo.style(this._sysTmplContainer, "display", "block");
                    // dojo.replaceClass(this._sysTmplSelectorExpander, "collapseTwistie",
                    // "noTwistie
                    // expandTwistie");
                    // }

                },
                _setupModule3Selector: function() {

                    // summary:
                    // Setup SystemInfo template selector...
                    // tags:
                    // protected
                    var tmplStore = this._regenerateSelectorStore();
                    this._module3Selector = new support.widgets.Select({
                        id: this.get("id") + "_module3TmplSelector",
                        name: this.IO3,
                        required: false,
                        store: tmplStore,
                        emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                        style: {
                            width: this.SELECTOR_WIDTH
                        }
                    }).placeAt(this._module3Selector);
                    // call startup(); crucial for support.widgets.Select Dijits
                    this._module3Selector.startup();
                    // preload selector with saved values
                    this._presetSelector(tmplStore._arrayOfAllItems.length, this._module3Selector);
                    this._module3Selector.onChange = dojo.hitch(this, this._changeModule3Callback);

                },

                _refreshModule3Selector: function() {
                    var tmplStore = this._regenerateSelectorStore();
                    this._module3Selector.setStore(tmplStore);
                },

                _changeModule3Callback: function(newVal) {

                    // tags:
                    // protected
                    dojo.style(this._module3Selector, "display", "none");
                    if (newVal > 0) {
                        dojo.replaceClass(this._module3EditOption, "nstEditIcon", "nstEditIconDisabled");
                        this._module3EditOption.title = this.resources.EDIT_SELECTED_PATTERN;
                    }
                    else {
                        dojo.replaceClass(this._module3EditOption, "nstEditIconDisabled", "nstEditIcon");
                        this._module3EditOption.title = this.resources.EDIT_SELECTED_PATTERN_DISABLED;
                    }

                },

                _createModule3: function() {
                    var dialog = new support.dialogs.ChassisIOModuleTemplate({
                        id: this.id + "cp_cmmiomodules_createchassisiomoduledialog3"
                    });
                    this.selectedIndex = 3;
                    dialog.show();
                },

                _editModule3: function() {
                    var templateId = this._module3Selector.value;
                    if (templateId >= 0) {
                        templateId = templateId.toString();
                        var dialog = new support.dialogs.ChassisIOModuleTemplate({
                            id: this.id + "cp_cmmiomodules_editchassisiomodule3",
                            templateIDNum: templateId,
                            templateMode: "edit",
                            templateInUse: false,
                            templateLaunchPoint: "wizard"
                        });
                        this.selectedIndex = 3;
                        dialog.show();
                    }
                },

                _module3SelectorOnClick: function(/* Event */evt) {

                    // tags:
                    // protected
                    // 99299: Wade Fode, 8/2/2012. Future consideration to display contents of the
                    // selected template in an expandable section beneath the selector.
                    // if (dojo.style(this._sysTmplContainer, "display") === "block") {
                    // dojo.style(this._sysTmplContainer, "display", "none");
                    // dojo.replaceClass(this._sysTmplSelectorExpander, "expandTwistie", "noTwistie
                    // collapseTwistie");
                    // }
                    // else {
                    // dojo.style(this._sysTmplContainer, "display", "block");
                    // dojo.replaceClass(this._sysTmplSelectorExpander, "collapseTwistie",
                    // "noTwistie
                    // expandTwistie");
                    // }

                },

                _generateTemplateData: function() {
                    var io1,io2,io3,io4;
                    if(this._selectAll.checked == false){
                        io1 = this._module1Selector.get("value");
                        io2 = this._module2Selector.get("value");
                        io3 = this._module3Selector.get("value");
                        io4 = this._module4Selector.get("value");
                    }
                    else{
                        var all = this._allTmplSelector.get("value");
                        io1 = all;
                        io2 = all;
                        io3 = all;
                        io4 = all;
                            
                    }
                    
                    return [{index:"1",type:"cmmIOModule",id:io1},
                            {index:"2",type:"cmmIOModule",id:io2},
                            {index:"3",type:"cmmIOModule",id:io3},
                            {index:"4",type:"cmmIOModule",id:io4}];
                },
                
                _setupSubscribers: function(){
                    // Also set up a subscriber function to be notified when a new SystemInfo
                    // template is created.
                    this._ioSub0 = this.subscribe("ChassisIOTemplateComplete", function(data) {
                        var preValue = this._allTmplSelector.get("value");
                        this._refreshAllInfoSelector();
                        // Set drop-down value
                        if (this.selectedIndex == 0 && data) {
                            this._allTmplSelector.set("value", data);
                        }
                        else if(this.selectedIndex != 0 && preValue){
                            this._allTmplSelector.set("value", preValue);
                        }
                        else {
                            this._allTmplSelector.set("value", "-1");
                        }
                        this._allTmplSelector.set("disabled", false);
                    });


                    // Also set up a subscriber function to be notified when a new SystemInfo
                    // template is created.
                    this._ioSub1 = this.subscribe("ChassisIOTemplateComplete", function(data) {
                        var preValue = this._module1Selector.get("value");
                        this._refreshModule1Selector();
                        // Set drop-down value
                        if (this.selectedIndex == 1 && data) {
                            this._module1Selector.set("value", data);
                        }
                        else if(this.selectedIndex != 1 && preValue){
                            this._module1Selector.set("value", preValue);
                        }
                        else {
                            this._module1Selector.set("value", "-1");
                        }
                        this._module1Selector.set("disabled", false);
                    });

                    // Also set up a subscriber function to be notified when a new SystemInfo
                    // template is created.
                    this._ioSub2 = this.subscribe("ChassisIOTemplateComplete", function(data) {
                        var preValue = this._module2Selector.get("value");
                        this._refreshModule2Selector();
                        // Set drop-down value
                        if (this.selectedIndex == 2 && data) {
                            this._module2Selector.set("value", data);
                        }
                        else if(this.selectedIndex != 2 && preValue){
                            this._module2Selector.set("value", preValue);
                        }
                        else {
                            this._module2Selector.set("value", "-1");
                        }
                        this._module2Selector.set("disabled", false);
                    });

                    // Also set up a subscriber function to be notified when a new SystemInfo
                    // template is created.
                    this._ioSub3 = this.subscribe("ChassisIOTemplateComplete", function(data) {
                        var preValue = this._module3Selector.get("value");
                        this._refreshModule3Selector();
                        // Set drop-down value
                        if (this.selectedIndex == 3 && data) {
                            this._module3Selector.set("value", data);
                        }
                        else if(this.selectedIndex != 3 && preValue){
                            this._module3Selector.set("value", preValue);
                        }
                        else {
                            this._module3Selector.set("value", "-1");
                        }
                        this._module3Selector.set("disabled", false);
                    });

                    // Also set up a subscriber function to be notified when a new SystemInfo
                    // template is created.
                    this._ioSub4 = this.subscribe("ChassisIOTemplateComplete", function(data) {
                        var preValue = this._module4Selector.get("value");
                        this._refreshModule4Selector();
                        // Set drop-down value
                        if (this.selectedIndex == 4 && data) {
                            this._module4Selector.set("value", data);
                        }
                        else if(this.selectedIndex != 4 && preValue){
                            this._module4Selector.set("value", preValue);
                        }
                        else {
                            this._module4Selector.set("value", "-1");
                        }
                        this._module4Selector.set("disabled", false);
                    });
                },
                
                _regenerateSelectorStore: function() {
                    var tmplStore = new dojo.data.ItemFileWriteStore({
                        url: this.REST_PATH + "template/list/cmmIOModule"
                    });
                    var item = {id: "-1", name: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"};
                    tmplStore.newItem(item);
                    tmplStore.save();
                    // fetch the store
                    tmplStore.fetch();
                    // if the store only has one item in it, and it's the one with id
                    //   "-1", then remove that item
                    var storeVals = tmplStore._arrayOfAllItems;
                    if(storeVals != null && storeVals.length == 1 && storeVals[0].id == "-1") {
                        // remove by redefining an empty store
                        tmplStore = new dojo.data.ItemFileWriteStore({
                            data: {items: ""}
                        });
                    }
                    
                    return tmplStore;
                },
                
                unsubscribeSelectors: function(){
                    if(this._ioSub0){
                        this.unsubscribe(this._ioSub0);
                    }
                    if(this._ioSub1){
                        this.unsubscribe(this._ioSub1);
                    }
                    if(this._ioSub2){
                        this.unsubscribe(this._ioSub2);
                    }
                    if(this._ioSub3){
                        this.unsubscribe(this._ioSub3);
                    }
                    if(this._ioSub4){
                        this.unsubscribe(this._ioSub4);
                    }
                },
                
                AfterPageSelected: function() {

                    // Validate page input and enable/disable "Next" button of wizard accordingly.
                    dojo.publish("enableNextButton", true);

                }


            });

        });