define([
        "dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/cmmMgmtModules",
        "dojo/text!./templates/cmmMgmtModules.html",
        "dojo/DeferredList",
        "dojo/_base/lang",
        "dojo/data/ItemFileWriteStore",
        "dijit/form/Select",
        "dijit/Tooltip",
        "support/widgets/Select"
    ], function(dojo, dijit, page, mri, template, modDeferredList, modLang) {

    return dojo.declare("support.wizard.pages.cmmMgmtModules", [ support.panels.Panel ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        // editMode: boolean
        // Specifies whether the wizard is to execute in edit mode. If "true", the "templateIDNum"
        // attribute is required.
        editMode: false,

        // copyMode: boolean
        // Specifies whether the wizard is to execute in copy mode. If "true", the "templateIDNum"
        // attribute is required.
        copyMode: false,

        // inUse: boolean
        // Specifies whether the wizard is to execute in inUse mode.
        inUse: false,
        
        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "nstWizard",

        // Internal Selector Names
        SYSTMPL_NAME: "chassisInfoTemplate",
        MGMTTMPL_NAME: "chassisManagementTemplate",
        PERFORMTMPL_NAME: "performanceTemplate",
        PWRTMPL_NAME: "powerTemplate",

        // Template Type Names - Internal Usage
        SYS_INFO: "cmmInfo",
        MGMT: "cmmManagement",
        POWER: "cmmPower",
        PERFORM: "Advanced",

        SELECTOR_WIDTH: "400px",
        
        sysWarning: false,
        mgmtWarning: false,
        userWarning: false,

        // Override this method to perform custom behavior during dijit construction.
        // Common operations for constructor:
        // 1) Initialize non-primitive types (i.e. objects and arrays)
        // 2) Add additional properties needed by succeeding lifecycle methods
        constructor: function() {
            this.resources = dojo.i18n.getLocalization("support.panels", "Panel", this.lang);
        },

        // When this method is called, all variables inherited from superclasses
        // are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in
        // the template HTML file
        postMixInProperties: function() {

            this.inherited(arguments);

        },

        // postCreate() is called after buildRendering(). This is useful to
        // override when
        // you need to access and/or manipulate DOM nodes included with your
        // widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified
        // can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);
           
            this._setupSysTmplSelector();
            this._setupEthTmplSelector();
            this._setupPwrTmplSelector();
            this._setupPerformTmplSelector();

            this._checkIfHostNameAndSystemNameSame();
            
            //this.sysWarning = false;
            //this.mgmtWarning = false;

        },

        _setStartingPointAttr: function(newval) {
           this._set("startingPoint", newval);
                      
           if (this.startingPoint == "FROM_SCRATCH") {
              if (this.infoMessage) {
                  this.infoMessage.destroyRecursive();
              }
           } else{
               
               if(this.CP_RUN_ENVIRONMENT == "fsm"){

                   require([ "idx/oneui/messaging/SingleMessage" ], dojo.hitch(this, function() {

                        this.infoMessage = new idx.oneui.messaging.SingleMessage({
                           description: this.resources.LEARNED_CHASSIS_SETTINGS_DETAILS,
                           type: "information",
                           dateFormat: { formatLength: "medium", selector: "time" },
                           showId: false,
                           showAction: false,
                           showRefresh: false,
                           showDetailsLink: false
                        }).placeAt(this.id + "_messagearea");
                        this.infoMessage.set("title", this.resources.LEARNED_CHASSIS_SETTINGS_INFO.replace("%1", this.chassis));
                        
                   }));

               } else if(this.CP_RUN_ENVIRONMENT == "augusta") {

                   require([ "idx/widget/SingleMessage" ], dojo.hitch(this, function() {

                        this.infoMessage = new idx.widget.SingleMessage({
                           description: this.LEARNED_CHASSIS_SETTINGS_DETAILS,
                           type: "information",
                           dateFormat: { formatLength: "medium", selector: "time" },
                           showId: false,
                           showAction: false,
                           showRefresh: false,
                           showDetailsLink: false
                        }).placeAt(this.id + "_messagearea");
                        this.infoMessage.set("title", this.resources.LEARNED_CHASSIS_SETTINGS_INFO.replace("%1", this.chassis));
                        
                   }));

               }
               
           }
        },

        _setChassisAttr: function(chassis) {
           this._set("chassis", chassis);
           this.infoMessage.set("title", this.resources.LEARNED_CHASSIS_SETTINGS_INFO.replace("%1", this.chassis));
        },
        
        _presetSelector: function(size, selector) {
            // tags:
            // protected
            // No templates for this category?
            if (size <= 0) {
                // Disable selection and allow "emptyLabel" to display.
                selector.set("disabled", true);
            }
            else {
                var curTemplates = [];
                if (this.pageParams.templates) {
                    // Pick preselected id
                    switch (selector.name) {
                        case this.SYSTMPL_NAME:
                            //this.sysWarning = true;
                            curTemplates = dojo.filter(this.pageParams.templates, dojo.hitch(this,function(item) {
                                return item.type == "cmmInfo";
                            }));
                            break;
                        case this.MGMTTMPL_NAME:
                            //this.mgmtWarning = true;
                            curTemplates = dojo.filter(this.pageParams.templates, dojo.hitch(this,function(item) {
                                return item.type == "cmmManagement";
                            }));
                            break;
                        case this.PERFORMTMPL_NAME:
                            curTemplates = dojo.filter(this.pageParams.templates, dojo.hitch(this,function(item) {
                                return item.type == "cmmUsers";
                            }));
                            break;
                        case this.PWRTMPL_NAME:
                            curTemplates = dojo.filter(this.pageParams.templates, dojo.hitch(this,function(item) {
                                return item.type == "cmmPower";
                            }));
                            break;
                    }
                }

                // Set drop-down value
                if (curTemplates[0]) {
                    selector.set("value", curTemplates[0].id);
                }
                else {
                    selector.set("value", "-1");
                }
            }
            
            this.sysWarning = true;
            this.mgmtWarning = true;
            this.userWarning = true;

        },

        _setupSysTmplSelector: function() {

            // summary:
            // Setup SystemInfo template selector...
            // tags:
            // protected
            var tmplStore = this._regenerateSelectorStore("template/list/cmmInfo");
            this._sysTmplSelector = new support.widgets.Select({
                id: this.get("id") + "_sysTmplSelector",
                name: this.SYSTMPL_NAME,
                required: false,
                store: tmplStore,
                emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                style: {
                    width: this.SELECTOR_WIDTH
                }
            }).placeAt(this._sysTmplSelector);
            // call startup(); crucial for support.widgets.Select Dijits
            this._sysTmplSelector.startup();
            // preload selector with saved values
            this._presetSelector(tmplStore._arrayOfAllItems.length, this._sysTmplSelector);
            this._sysTmplSelector.onChange = dojo.hitch(this, this._changeSysTmplCallback);
                      
            // Also set up a subscriber function to be notified when a new SystemInfo
            // template is created.
            this._infoSub = this.subscribe("ChassisInfoTemplateComplete", function(data) {
                var tmplStore = this._regenerateSelectorStore("template/list/cmmInfo");
                this._sysTmplSelector.setStore(tmplStore);
                // Set drop-down value
                if (data) {
                    this._sysTmplSelector.set("value", data);
                }
                else {
                    this._sysTmplSelector.set("value", "-1");
                }
                this._sysTmplSelector.set("disabled", false);

                this._checkIfHostNameAndSystemNameSame();
                this._checkIfInfoPatternRequired();
            });

        },

        _refreshSysInfoSelector: function() {
            var tmplStore = this._regenerateSelectorStore("template/list/cmmInfo");
            this._sysTmplSelector.setStore(tmplStore);
            this._checkIfHostNameAndSystemNameSame();
        },
        
        _checkIfInfoPatternRequired: function() {
            var sysTemplateId = this._sysTmplSelector.get("value");
            var mgmtTemplateId = this._ethTmplSelector.get("value");
            var hostname = "";
            var infoPatternRequired = false;
            
            if (mgmtTemplateId != "-1" && mgmtTemplateId != "") {
                var mgmtTemplate = dojo.xhrGet({
                    url: this.REST_PATH + "template/" + mgmtTemplateId,
                    handleAs: "json",
                    sync: true,
                    preventCache: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: dojo.hitch(this, function(response) {
                        hostname = response.items[0].CMM__NetworkSettings_autogen_hostname;
                    })
                });
                
                for (var i = 0; i < hostname.pattern.length; i++) {
                    if (hostname.pattern[i].value == "RM" || hostname.pattern[i].value == "SL") {
                        infoPatternRequired = true;
                        break;
                    }
                }
            }
            
            if (sysTemplateId == "-1" && infoPatternRequired) {
                dojo.style(this._sysTmplWarningRequired, "visibility", "visible");
                if (!this.warningMessage1) {
                    if(this.CP_RUN_ENVIRONMENT == "fsm"){

                        require([ "idx/oneui/messaging/SingleMessage" ], dojo.hitch(this, function() {

                            this.warningMessage1 = new idx.oneui.messaging.SingleMessage({
                                title: this.locResources.CHASSIS_INFO_REQUIRED,
                                description: this.locResources.CHASSIS_INFO_MESSAGE,
                                type: "warning",
                                dateFormat: { formatLength: "medium", selector: "time" },
                                showId: false,
                                showAction: false,
                                showRefresh: false,
                                showDetailsLink: false
                             }).placeAt(this.id + "_messagearea");
                    
                        }));
                        
                    }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                        require([ "idx/widget/SingleMessage" ], dojo.hitch(this, function() {

                            this.warningMessage1 = new idx.widget.SingleMessage({
                                title: this.locResources.CHASSIS_INFO_REQUIRED,
                                description: this.locResources.CHASSIS_INFO_MESSAGE,
                                type: "warning",
                                dateFormat: { formatLength: "medium", selector: "time" },
                                showId: false,
                                showAction: false,
                                showRefresh: false,
                                showDetailsLink: false
                             }).placeAt(this.id + "_messagearea");
                    
                        }));

                    }
                }
            } else {
                dojo.style(this._sysTmplWarningRequired, "visibility", "hidden");
                if (this.warningMessage1) {
                    this.warningMessage1.destroyRecursive();
                    this.warningMessage1 = null;
                }                
            }
        },

        _checkIfHostNameAndSystemNameSame: function() {
           var sysTemplateId = this._sysTmplSelector.get("value");
           var ethTemplateId = this._ethTmplSelector.get("value");
           if (typeof sysTemplateId == "string" && sysTemplateId != "" && sysTemplateId != "-1" &&
               typeof ethTemplateId == "string" && ethTemplateId != "" && ethTemplateId != "-1") {
              // Both selectors refer to a template.  Go get the 2 templates.
              var sysTmplDeferred = dojo.xhrGet({
                  url: this.REST_PATH + "template/" + sysTemplateId,
                  handleAs: "json",
                  sync: false,
                  preventCache: true
              });

              var ethTmplDeferred = dojo.xhrGet({
                 url: this.REST_PATH + "template/" + ethTemplateId,
                 handleAs: "json",
                 sync: false,
                 preventCache: true
              });

              // Would be nice if Dojo provided this utility function, but alas...
              function compareObjects(a, b) {
                 if (modLang.isObject(a) && modLang.isObject(b)) {
                    for (var it in a) {
                       if (!compareObjects(a[it], b[it]))
                          return false;
                    }

                    for (var it in b) {
                       if(!(it in a))
                          return false;
                    }

                    return true;
                 } else {
                    return a === b;
                 }
              }

              // When both templates finish loading, go compare the hostname and systemname.
              var templateLoadedDeferred = new modDeferredList([sysTmplDeferred, ethTmplDeferred]);
              templateLoadedDeferred.then(modLang.hitch(this, function(results) {
                 var systemNamePattern = results[0][1].items[0].CMM__GeneralSettings__info__autogen.pattern;
                 var hostNamePattern = results[1][1].items[0].CMM__NetworkSettings_autogen_hostname.pattern;
                 var generateSystemNameEnabled = results[0][1].items[0].CMM__GeneralSettings__info__autogen.CMM__GeneralSettings__info__autogen == 0;
                 var generateHostNameEnabled = results[1][1].items[0].autogenhostnamebuttonvalue == 0;
                 //console.log(systemNamePattern, hostNamePattern);

                 // If the hostname/systemname autogen states are different, or 
                 // autogen is enabled and the patterns are different, then show the warning.
                 // If autogen is disabled, be sure to not compare the patterns because they
                 // could have data the user previously typed in (but it's now being ignored
                 // because it's disabled).
                 var isDifferent = (generateSystemNameEnabled != generateHostNameEnabled) ||
                                   (generateHostNameEnabled && !compareObjects(systemNamePattern, hostNamePattern));
                 if (!isDifferent) {
                    dojo.style(this._sysTmplHostNameSystemNameSame, "visibility", "hidden");
                    dojo.style(this._ethTmplHostNameSystemNameSame, "visibility", "hidden");
                 } else {
                    dojo.style(this._sysTmplHostNameSystemNameSame, "visibility", "visible");
                    dojo.style(this._ethTmplHostNameSystemNameSame, "visibility", "visible");
                 }
              }));
           } else {
              // One of the selections is "No Pattern Selected", so hide the info icon.
              dojo.style(this._sysTmplHostNameSystemNameSame, "visibility", "hidden");
              dojo.style(this._ethTmplHostNameSystemNameSame, "visibility", "hidden");
           }
        },

        _changeSysTmplCallback: function(newVal) {

            // tags:
            // protected
            dojo.style(this._sysTmplContainer, "display", "none");
            // 99299: Wade Fode, 8/2/2012. Future consideration to display contents of the
            // selected template in an expandable section beneath the selector.
            if (newVal > 0) {
                // dojo.replaceClass(this._sysTmplSelectorExpander, "expandTwistie", "noTwistie
                // collapseTwistie");
                dojo.replaceClass(this._sysTmplEditOption, "nstEditIcon", "nstEditIconDisabled");
                this._sysTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN;
            }
            else {
                // dojo.replaceClass(this._sysTmplSelectorExpander, "noTwistie", "expandTwistie
                // collapseTwistie");
                dojo.replaceClass(this._sysTmplEditOption, "nstEditIconDisabled", "nstEditIcon");
                this._sysTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN_DISABLED;
            }

            this._checkIfHostNameAndSystemNameSame();
            this._checkIfInfoPatternRequired();
        },

        _createSysTmpl: function() {
            require([ "support/dialogs/ChassisInfoTemplate" ], dojo.hitch(this, function() {
                var dialog = new support.dialogs.ChassisInfoTemplate({
                    id: this.id + "cp_cmmmgmtmodules_createchassisinfodialog"
                });
                dialog.show();

            }));

        },

        _editSysTmpl: function() {
            var templateId = this._sysTmplSelector.value;

            if (templateId >= 0) {
                templateId = templateId.toString();
                
                require([ "support/dialogs/ChassisInfoTemplate" ], dojo.hitch(this, function() {
                    var dialog = new support.dialogs.ChassisInfoTemplate({
                        id: this.id + "cp_cmmmgmtmodules_editchassisinfodialog",
                        templateIDNum: templateId,
                        templateMode: "edit",
                        // templateInUse: "",
                        templateLaunchPoint: "wizard"

                    });
                    dialog.show();

                }));

            }
        },

        _sysTmplSelectorOnClick: function(/* Event */evt) {

            // tags:
            // protected
            // 99299: Wade Fode, 8/2/2012. Future consideration to display contents of the
            // selected template in an expandable section beneath the selector.
            // if (dojo.style(this._sysTmplContainer, "display") === "block") {
            // dojo.style(this._sysTmplContainer, "display", "none");
            // dojo.replaceClass(this._sysTmplSelectorExpander, "expandTwistie", "noTwistie
            // collapseTwistie");
            // }
            // else {
            // dojo.style(this._sysTmplContainer, "display", "block");
            // dojo.replaceClass(this._sysTmplSelectorExpander, "collapseTwistie", "noTwistie
            // expandTwistie");
            // }

        },

        _setupEthTmplSelector: function() {

            // summary:
            // Setup Management Interface Template selector...
            // tags:
            // protected
            var tmplStore = this._regenerateSelectorStore("template/list/cmmManagement");
            this._ethTmplSelector = new support.widgets.Select({
                id: this.get("id") + "_ethTmplSelector",
                name: this.MGMTTMPL_NAME,
                required: false,
                store: tmplStore,
                emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                style: {
                    width: this.SELECTOR_WIDTH
                }
            }).placeAt(this._ethTmplSelector);
            // call startup(); crucial for support.widgets.Select Dijits
            this._ethTmplSelector.startup();
            // preload selector with saved values
            this._presetSelector(tmplStore._arrayOfAllItems.length, this._ethTmplSelector);
            this._ethTmplSelector.onChange = dojo.hitch(this, this._changeEthTmplCallback);

            // Also set up a subscriber function to be notified when a new Management
            // template is created.
            this._mgmtSub = this.subscribe("ChassisManagementComplete", function(data) {
                var tmplStore = this._regenerateSelectorStore("template/list/cmmManagement");
                this._ethTmplSelector.setStore(tmplStore);
                // Set drop-down value
                if (data) {
                    this._ethTmplSelector.set("value", data);
                }
                else {
                    this._ethTmplSelector.set("value", "-1");
                }
                this._ethTmplSelector.set("disabled", false);

                this._checkIfHostNameAndSystemNameSame();
            });

        },

        _refreshEthSelector: function() {
            var tmplStore = this._regenerateSelectorStore("template/list/cmmManagement");
            this._ethTmplSelector.setStore(tmplStore);
            this._checkIfHostNameAndSystemNameSame();
        },

        _changeEthTmplCallback: function(newVal) {

            // tags:
            // protected
            dojo.style(this._ethTmplContainer, "display", "none");
            // 99299: Wade Fode, 8/2/2012. Future consideration to display contents of the
            // selected template in an expandable section beneath the selector.
            if (newVal > 0) {
                // dojo.replaceClass(this._ethTmplSelectorExpander, "expandTwistie", "noTwistie
                // collapseTwistie");
                dojo.replaceClass(this._ethTmplEditOption, "nstEditIcon", "nstEditIconDisabled");
                this._ethTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN;
            }
            else {
                // dojo.replaceClass(this._ethTmplSelectorExpander, "noTwistie", "expandTwistie
                // collapseTwistie");
                dojo.replaceClass(this._ethTmplEditOption, "nstEditIconDisabled", "nstEditIcon");
                this._ethTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN_DISABLED;
            }

            this._checkIfHostNameAndSystemNameSame();
            this._checkIfInfoPatternRequired();
        },

        _createEthTmpl: function() {
            // Launch Ethernet Interface Template dialog
            require([ "support/dialogs/ChassisManagementInterfaceTemplate" ], dojo.hitch(this, function() {
           
                var dialog = new support.dialogs.ChassisManagementInterfaceTemplate({
                    id: this.id + "cp_cmmmgmtmodules_createchassismgmtintdialog"
                });
                dialog.show();
                
            }));
            
        },

        _editEthTmpl: function() {
            var templateId = this._ethTmplSelector.value;

            if (templateId >= 0) {
                templateId = templateId.toString();
                // Launch Ethenet Interface Template dialog, passing the ID of the newly created
                // template.
                require([ "support/dialogs/ChassisManagementInterfaceTemplate" ], dojo.hitch(this, function() {
               
                    var dialog = new support.dialogs.ChassisManagementInterfaceTemplate({
                        id: this.id + "cp_cmmmgmtmodules_editchassismgmtintdialog",
                        templateIDNum: templateId,
                        templateMode: "edit",
                        // templateInUse: "",
                        templateLaunchPoint: "wizard"

                    });
                    dialog.show();
                
                }));

                
            }
        },

        _setupPwrTmplSelector: function() {

            // summary:
            // Setup Power State Template selector...
            // tags:
            // protected
            var tmplStore = this._regenerateSelectorStore("template/list/cmmPower");
            this._pwrTmplSelector = new support.widgets.Select({
                id: this.get("id") + "_pwrTmplSelector",
                name: this.PWRTMPL_NAME,
                required: false,
                store: tmplStore,
                emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                style: {
                    width: this.SELECTOR_WIDTH
                }
            }).placeAt(this._pwrTmplSelector);
            // call startup(); crucial for support.widgets.Select Dijits
            this._pwrTmplSelector.startup();
            // preload selector with saved values
            this._presetSelector(tmplStore._arrayOfAllItems.length, this._pwrTmplSelector);
            this._pwrTmplSelector.onChange = dojo.hitch(this, this._changePwrTmplCallback);

            // Also set up a subscriber function to be notified when a new
            // template is created.
            this._pwrSub = this.subscribe("ChassisPowerComplete", function(data) {
                var tmplStore = this._regenerateSelectorStore("template/list/cmmPower");
                this._pwrTmplSelector.setStore(tmplStore);
                // Set drop-down value
                if (data) {
                    this._pwrTmplSelector.set("value", data);
                }
                else {
                    this._pwrTmplSelector.set("value", "-1");
                }

                this._pwrTmplSelector.set("disabled", false);
            });

        },

        _refreshPowerSelector: function() {
            var tmplStore = this._regenerateSelectorStore("template/list/cmmPower");
            this._pwrTmplSelector.setStore(tmplStore);
        },

        _changePwrTmplCallback: function(newVal) {

            // tags:
            // protected
            dojo.style(this._pwrTmplContainer, "display", "none");
            // 99299: Wade Fode, 8/2/2012. Future consideration to display contents of the
            // selected template in an expandable section beneath the selector.
            if (newVal > 0) {
                // dojo.replaceClass(this._pwrTmplSelectorExpander, "expandTwistie", "noTwistie
                // collapseTwistie");
                dojo.replaceClass(this._pwrTmplEditOption, "nstEditIcon", "nstEditIconDisabled");
                this._pwrTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN;
            }
            else {
                // dojo.replaceClass(this._pwrTmplSelectorExpander, "noTwistie", "expandTwistie
                // collapseTwistie");
                dojo.replaceClass(this._pwrTmplEditOption, "nstEditIconDisabled", "nstEditIcon");
                this._pwrTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN_DISABLED;
            }

        },

        _createPwrTmpl: function() {
            require([ "support/dialogs/ChassisPowerAndAcousticsTemplate" ], dojo.hitch(this, function() {

                var dialog = new support.dialogs.ChassisPowerAndAcousticsTemplate({
                    id: this.id + "cp_cmmmgmtmodules_createchassispowerandacousticdialog"
                });
                dialog.show();

            }));
           
        },

        _editPwrTmpl: function() {
            var templateId = this._pwrTmplSelector.value;

            if (templateId >= 0) {
                templateId = templateId.toString();

                require([ "support/dialogs/ChassisPowerAndAcousticsTemplate" ], dojo.hitch(this, function() {

                    var dialog = new support.dialogs.ChassisPowerAndAcousticsTemplate({
                        id: this.id + "cp_cmmmgmtmodules_editchassispowerandacousticdialog",
                        templateIDNum: templateId,
                        templateMode: "edit",
                        // templateInUse: "",
                        templateLaunchPoint: "wizard"
                    });
                    dialog.show();

                }));

            }
        },

        _setupPerformTmplSelector: function() {

            // summary:
            // Setup Performance and Recovery Template selector...
            // tags:
            // protected
            var tmplStore = this._regenerateSelectorStore("template/list/cmmUsers");
            this._performTmplSelector = new support.widgets.Select({
                id: this.get("id") + "_performTmplSelector",
                name: this.PERFORMTMPL_NAME,
                required: false,
                store: tmplStore,
                emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                style: {
                    width: this.SELECTOR_WIDTH
                }
            }).placeAt(this._performTmplSelector);
            // call startup(); crucial for support.widgets.Select Dijits
            this._performTmplSelector.startup();
            // preload selector with saved values
            this._presetSelector(tmplStore._arrayOfAllItems.length, this._performTmplSelector);
            this._performTmplSelector.onChange = dojo.hitch(this, this._changePerformTmplCallback);

            // Also set up a subscriber function to be notified when a new Performance and Recovery
            // template is created.
            this._userSub = this.subscribe("ChassisUserTemplateComplete", function(data) {
                var tmplStore = this._regenerateSelectorStore("template/list/cmmUsers");
                this._performTmplSelector.setStore(tmplStore);
                // Set drop-down value
                if (data) {
                    this._performTmplSelector.set("value", data);
                }
                else {
                    this._performTmplSelector.set("value", "-1");
                }
                this._performTmplSelector.set("disabled", false);
            });

        },

        _refreshPerformSelector: function() {
            var tmplStore = this._regenerateSelectorStore("template/list/cmmUsers");
            this._performTmplSelector.setStore(tmplStore);
        },

        _changePerformTmplCallback: function(newVal) {

            // tags:
            // protected
            dojo.style(this._performTmplContainer, "display", "none");
            // 99299: Wade Fode, 8/2/2012. Future consideration to display contents of the
            // selected template in an expandable section beneath the selector.
            if (newVal > 0) {
                // dojo.replaceClass(this._performTmplSelectorExpander, "expandTwistie", "noTwistie
                // collapseTwistie");
                dojo.replaceClass(this._performTmplEditOption, "nstEditIcon", "nstEditIconDisabled");
                this._performTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN;                   
            }
            else {
                // dojo.replaceClass(this._performTmplSelectorExpander, "noTwistie", "expandTwistie
                // collapseTwistie");
                dojo.replaceClass(this._performTmplEditOption, "nstEditIconDisabled", "nstEditIcon");
                this._performTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN_DISABLED;
            }

        },

        _createPerformTmpl: function() {
            require([ "support/dialogs/ChassisUserAndSecurityTemplate" ], dojo.hitch(this, function() {
            
                var dialog = new support.dialogs.ChassisUserAndSecurityTemplate({
                    id: this.id + "cp_cmmmgmtmodules_createchassisuserandsecuritydialog"
                });
                dialog.show();
                
            }));

            
        },

        _editPerformTmpl: function() {
            var templateId = this._performTmplSelector.value;

            if (templateId >= 0) {
                templateId = templateId.toString();
              
                require([ "support/dialogs/ChassisUserAndSecurityTemplate" ], dojo.hitch(this, function() {

                    var dialog = new support.dialogs.ChassisUserAndSecurityTemplate({
                        id: this.id + "cp_cmmmgmtmodules_chassisuserandsecuritydialog",
                        templateIDNum: templateId,
                        templateMode: "edit",
                        // templateInUse: "",
                        templateLaunchPoint: "wizard"
                    });
                    dialog.show();

                }));

            }
        },

        _presetSelectorValues: function() {
            var i;
            for(i=0;i<this.pageParams.length;i++){
                if (this.pageParams[i].type == "cmmInfo") {
                    console.log(this.declaredClass + ": found sys");
                    console.log(this.declaredClass + ": ", this.pageParams[i].id);
                    this._sysTmplSelector.set("value", this.pageParams[i].id);
                    this._sysTmplSelector.set("disabled", false);
                }
                if (this.pageParams[i].type == "cmmManagement") {
                    console.log(this.declaredClass + ": found mgmt");
                    console.log(this.declaredClass + ": ", this.pageParams[i].id);
                    this._ethTmplSelector.set("value", this.pageParams[i].id);
                    this._ethTmplSelector.set("disabled", false);
                }
                if (this.pageParams[i].type == "cmmPower") {
                    console.log(this.declaredClass + ": found pwr");
                    console.log(this.declaredClass + ": ", this.pageParams[i].id);
                    this._pwrTmplSelector.set("value", this.pageParams[i].id);
                    this._pwrTmplSelector.set("disabled", false);
                }
                if (this.pageParams[i].type == "cmmUsers") {
                    console.log(this.declaredClass + ": found usr");
                    console.log(this.declaredClass + ": ", this.pageParams[i].id);
                    this._performTmplSelector.set("value", this.pageParams[i].id);
                    this._performTmplSelector.set("disabled", false);
                }
            }
        },

        refreshAllSelectors: function() {
            this._refreshEthSelector();
            this._refreshPerformSelector();
            this._refreshPowerSelector();
            this._refreshSysInfoSelector();

            this._presetSelectorValues();
        },

        _generateTemplateData: function() {
            var info = this._sysTmplSelector.get("value");
            var mgmt = this._ethTmplSelector.get("value");
            var pwr = this._pwrTmplSelector.get("value");
            var user = this._performTmplSelector.get("value");

            // Append FW Settings information...
            var templates = [];
            templates.push({
                id: (info != null && info != "-1" && info != "") ? info : null,
                type: "cmmInfo"
            });
            templates.push({
                id: (mgmt != null && mgmt != "-1" && mgmt != "") ? mgmt : null,
                type: "cmmManagement"
            });
            templates.push({
                id: (pwr != null && pwr != "-1" && pwr != "") ? pwr : null,
                type: "cmmPower"
            });
            templates.push({
                id: (user != null && user != "-1" && user != "") ? user : null,
                type: "cmmUsers"
            });
            return templates;
        },
        
        unsubscribeSelectors: function(){
            if(this._infoSub){
                this.unsubscribe(this._infoSub);
            }
            if(this._mgmtSub){
                this.unsubscribe(this._mgmtSub);
            }
            if(this._pwrSub){
                this.unsubscribe(this._pwrSub);
            }
            if(this._userSub){
                this.unsubscribe(this._userSub);
            }
        },
        
        _regenerateSelectorStore: function(urlSuffix) {
            var tmplStore = new dojo.data.ItemFileWriteStore({
                url: this.REST_PATH + urlSuffix
            });
            var item = {id: "-1", name: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"};
            tmplStore.newItem(item);
            tmplStore.save();
            // fetch the store
            tmplStore.fetch();
            // if the store only has one item in it, and it's the one with id
            //   "-1", then remove that item
            var storeVals = tmplStore._arrayOfAllItems;
            if(storeVals != null && storeVals.length == 1 && storeVals[0].id == "-1") {
                // remove by redefining an empty store
                tmplStore = new dojo.data.ItemFileWriteStore({
                    data: {items: ""}
                });
            }
            
            return tmplStore;
        },
        
        AfterPageSelected: function(container) {

            // Validate page input and enable/disable "Next" button of wizard accordingly.
            dojo.publish("enableNextButton", true);

        }

    });

});