define([ 
        "dojo", 
        "dijit", 
        "support/panels/Panel",
        "dojo/i18n!./nls/nstBoot", 
        "dojo/text!./templates/nstBoot.html",
        "dijit/form/RadioButton", 
        "support/widgets/ButtonBar", 
        "support/panels/BootOrderTable", 
        "dojo/parser",
        "dijit/layout/TabContainer", 
        "dijit/layout/ContentPane" 
        ], function(dojo, dijit, page, mri, template) {

    return dojo.declare("support.wizard.pages.nstBoot", [ support.panels.Panel ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        // editMode: boolean
        // Specifies whether the wizard is to execute in edit mode. If "true", the "templateIDNum"
        // attribute is required.
        editMode: false,

        // copyMode: boolean
        // Specifies whether the wizard is to execute in copy mode. If "true", the "templateIDNum"
        // attribute is required.
        copyMode: false,

        // inUse: boolean
        // Specifies whether the wizard is to execute in inUse mode.
        inUse: false,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "nstWizard",

        // existing or specify
        // bootConfig: "specify",

        // uefi then legacy, uefi only, legacy only
        bootOrder: null,
        BOOT_ORDER_UEFI_ONLY: "UEFI Only",
        BOOT_ORDER_UEFI_MODE: "UEFI Mode",
        BOOT_ORDER_UEFI_LEGACY: "UEFI and Legacy",
        BOOT_ORDER_LEGACY_ONLY: "Legacy Only",
        BOOT_ORDER_LEGACY_MODE: "Legacy Mode",
        BOOT_ORDER_SECURE_MODE: "Secure Mode",
        BOOT_ORDER_KEEP_EXISTING: "Keep existing",

        // uefi then legacy, uefi only, legacy only
        bootWoL: null,
        
        // Boot tables
        _table1: null,   // Primary boot order table
        _table2: null,   // Wake on LAN table
        _table3: null,   // SAN boot table

        // Override this method to perform custom behavior during dijit
        // construction.
        // Common operations for constructor:
        // 1) Initialize non-primitive types (i.e. objects and arrays)
        // 2) Add additional properties needed by succeeding lifecycle methods
        constructor: function() {
            // Borrow Wizard General Page Resources
            this.genResources = dojo.i18n.getLocalization("support.wizard.pages", "nstGeneral", this.lang);
        },

        // When this method is called, all variables inherited from superclasses
        // are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in
        // the template HTML file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to
        // override when
        // you need to access and/or manipulate DOM nodes included with your
        // widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified
        // can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            
            // The boot mode values used to be translated, but they should not be.
            switch (this.pageParams.bootOrder) {
                case this.BOOT_ORDER_UEFI_ONLY:
                case this.BOOT_ORDER_UEFI_MODE:
                    this.bootOrder_eufi.set("checked", true);
                    break;
                case this.BOOT_ORDER_UEFI_LEGACY:
                    this.bootOrder_both.set("checked", true);
                    break;
                case this.BOOT_ORDER_LEGACY_ONLY:
                case this.BOOT_ORDER_LEGACY_MODE:
                    this.bootOrder_legacy.set("checked", true);
                    break;
                case this.BOOT_ORDER_SECURE_MODE:  // "Secure Mode" is currently not supported. Fall though to "Keep existing".
                case this.BOOT_ORDER_KEEP_EXISTING:
                    // Pattern data uses empty string (""), not "Keep existing", so just
                    // convert and fall through...
                    this.pageParams.bootOrder = "";
                case "":
                    this.bootOrder_keep.set("checked", true);
                    break;
                default:  // Anything else defaults to keep existing
                    this.pageParams.bootOrder = "";
                    this.bootOrder_keep.set("checked", true);
                    break;
            }
            this.set("bootOrder", this.pageParams.bootOrder);
           
                       
            this._table1 = new support.panels.BootOrderTable({
                id: this.id + "_bootordertable1a",
                SANboot: false,
                jsonData: this.pageParams.bootOrderTable,
                learnedOptions: this.pageParams.learnedBootOptions,
                pageParams: this.pageParams,
                style:"width:100%; height:500px;"
            }).placeAt(this._table1placement);
            
            this._table2 = new support.panels.BootOrderTable({
                id: this.id + "_bootordertable2a",
                SANboot: false,
                jsonData: this.pageParams.bootWoLTable,
                learnedOptions: this.pageParams.learnedWoLBootOptions,
                pageParams: this.pageParams,
                style:"width:100%; height:500px;"
            }).placeAt(this._table2placement);
            
            this._table3 = new support.panels.BootOrderTable({
                id: this.id + "_bootordertable3a",
                SANboot: true,
                jsonData: this.pageParams.bootOrderTable,
                learnedOptions: this.pageParams.learnedBootOptions,
                pageParams: this.pageParams,
                style:"width:100%; height:500px;"
            }).placeAt(this._table3placement);
            
            this._bootPageTabContainer.watch("selectedChildWidget", dojo.hitch(this, function(name, oval, nval){
                (this._table1) && this._table1.systemBootModeChanged(this.bootOrder);
                (this._table2) && this._table2.systemBootModeChanged(this.bootOrder);                
                // System boot mode doesn't affect this._table3 (SAN boot tab)

            }));

        },

        _selectBootOrder: function(button) {
            // tags:
            // protected
            if (button.checked) {
                var bootOrder = (button.value != this.BOOT_ORDER_KEEP_EXISTING) ? button.value : "";
                this.set("bootOrder", bootOrder);
                this.pageParams.bootOrder = bootOrder;
                this._table1.systemBootModeChanged(bootOrder);
                this._table2.systemBootModeChanged(bootOrder);
                // System boot mode doesn't affect this._table3 (SAN boot tab)
            }
        },

        _getBootOrderTable: function() {
            // tags:
            // protected
            var allObjs = new Array();
            
            // The primary boot order table and SAN boot table data will be combined together.
            // The services code separates them out.
            if ((this._table1 != null) && (this.bootOrder == this.BOOT_ORDER_LEGACY_ONLY) && (this._table1.bootOrderRadioValue != "keep")) {
                allObjs = this._getTableData(this._table1, allObjs);
            }
            if (this._table3 != null) {
                allObjs = this._getTableData(this._table3, allObjs);
            }            
            return allObjs;

        },

        _getBootWoLTable: function() {
            // tags:
            // protected
            var allObjs = new Array();

            if ((this._table2 != null) && (this.bootOrder == this.BOOT_ORDER_LEGACY_ONLY) && (this._table2.bootOrderRadioValue != "keep")) {
                allObjs = this._getTableData(this._table2, allObjs);
            }
            return allObjs;

        },

        _getTableData: function(bootTableWidget, allObjs) {
            // tags:
            // protected

            var tableItems = bootTableWidget.getTableItems();
            var isHBA = bootTableWidget.isHBA;

            for (var idx in tableItems) {
                var item = tableItems[idx];
                var order = (item.order != undefined) ? item.order : null;
                var option = (item.option != undefined) ? item.option : null;
                
                if((order == null) || (option == null)) {
                    continue;
                }
                var device = (item.device) ? item.device : "*NONE";
                var deviceType = (item.deviceType) ? item.deviceType : "*NONE";
                var devicePort = (isNaN(item.devicePort)) ? "*NONE" : item.devicePort;
                var deviceSlot = (isNaN(item.deviceSlot)) ? "*NONE" : item.deviceSlot;
                var functionNumber = (isNaN(item.functionNumber)) ? "*NONE" : item.functionNumber;
                var template = (item.template) ? item.template : "*NONE";
                var target = (item.target) ? item.target : "*NONE";

                if ((option == bootTableWidget.DEVICE_FC) && isHBA) {
                    allObjs.push({
                            order: order,
                            option: option,
                            device: device,
                            deviceType: deviceType,
                            devicePort: devicePort,
                            deviceSlot: deviceSlot,
                            functionNumber: functionNumber,
                            template: template,
                            target: target
                    });
                } else {
                    allObjs.push({
                            order: order,
                            option: option,
                            device: device,
                            deviceType: "*NONE",
                            devicePort: "*NONE",
                            deviceSlot: "*NONE",
                            functionNumber: "*NONE",
                            template: "*NONE",
                            target: "*NONE"
                    });
                }
            }
            return allObjs;

        },

        _getLearnedBootOptions: function() {
            // tags:
            // protected
            var bootOrderTable = this._table1;
            return bootOrderTable.learnedOptions;            
        },

        _getLearnedWoLBootOptions: function() {
            // tags:
            // protected
            var bootWoLTable = this._table2;
            return bootWoLTable.learnedOptions;
        },

        getBootJsonObject: function() {
            var bootOrderTable = this._getBootOrderTable();
            var bootWoLTable = this._getBootWoLTable();
            var learnedBootOptions = this._getLearnedBootOptions();
            var learnedWoLBootOptions = this._getLearnedWoLBootOptions();
            
            return {
                "bootConfig": "specify",
                "bootOrder": this.bootOrder,
                "bootOrderTable": bootOrderTable,
                "bootWoLTable": bootWoLTable,
                "learnedBootOptions": learnedBootOptions,
                "learnedWoLBootOptions": learnedWoLBootOptions
            };
        },

        processBootUpdates : function(){
            if (this._table1) {
                this._table1.updateGridStoreData();
            }
            if (this._table2) {
                this._table2.updateGridStoreData();
            }
            if (this._table3) {
                this._table3.updateGridStoreData();
            }
        },

        reloadData: function(){
            //  The boot mode values used to be translated.  They should not be.
            switch (this.pageParams.bootOrder) {
                case this.BOOT_ORDER_UEFI_ONLY:
                case this.BOOT_ORDER_UEFI_MODE:
                    this.bootOrder_eufi.set("checked", true);
                    break;
                case this.BOOT_ORDER_UEFI_LEGACY:
                    this.bootOrder_both.set("checked", true);
                    break;
                case this.BOOT_ORDER_LEGACY_ONLY:
                case this.BOOT_ORDER_LEGACY_MODE:
                    this.bootOrder_legacy.set("checked", true);
                    break;
                case this.BOOT_ORDER_SECURE_MODE:  // "Secure Mode" is currently not supported. Fall though to "Keep existing".
                case this.BOOT_ORDER_KEEP_EXISTING:
                    // Pattern data uses empty string (""), not "Keep existing", so just
                    // convert and fall through...
                    this.pageParams.bootOrder = "";
                case "":
                    this.bootOrder_keep.set("checked", true);
                    break;
                default:  // Anything else defaults to keep existing
                    this.pageParams.bootOrder = "";
                    this.bootOrder_keep.set("checked", true);
                    break;
            }
            this.set("bootOrder", this.pageParams.bootOrder);
            
            if (this._table1 == null) {
                this._table1 = new support.panels.BootOrderTable({
                    id: this.id + "_bootordertable1b",
                    SANboot: false,
                    jsonData: this.pageParams.bootOrderTable,
                    learnedOptions: this.pageParams.learnedBootOptions,
                    pageParams: this.pageParams,
                    style:"width:100%; height:500px;"
                }).placeAt(this._table1placement);
            } else {
                this._table1.reloadBootTableData(this.pageParams);
            }

            if (this._table2 == null) {
                this._table2 = new support.panels.BootOrderTable({
                    id: this.id + "_bootordertable2b",
                    SANboot: false,
                    jsonData: this.pageParams.bootWoLTable,
                    learnedOptions: this.pageParams.learnedWoLBootOptions,
                    pageParams: this.pageParams,
                    style:"width:100%; height:500px;"
                }).placeAt(this._table2placement);
            } else {
                this._table2.reloadBootTableData(this.pageParams);
            }

            if (this._table3 == null) {
                this._table3 = new support.panels.BootOrderTable({
                    id: this.id + "_bootordertable3b",
                    SANboot: true,
                    jsonData: this.pageParams.bootOrderTable,
                    learnedOptions: this.pageParams.learnedBootOptions,
                    pageParams: this.pageParams,
                    style:"width:100%; height:500px;"
                }).placeAt(this._table3placement);
            } else {
                this._table3.reloadBootTableData(this.pageParams);
            }

        },

        setBootMode: function(formFactor) {
            switch (formFactor) {
                case "2sc":
                case "4sc":
                case "8sc": 
                    this.both_boot_button.style.display = "none";
                    // Until it is determined if this mode is supported, hide this radio button
                    //this.secure_boot_button.style.display = "block";
                    this.secure_boot_button.style.display = "none";
                    this.space_after_secure_button.style.width = "0px";
                    if (this.bootOrder == this.BOOT_ORDER_UEFI_LEGACY) {  // This mode is not supported for Newport so set to Keep existing
                        this.pageParams.bootOrder = "";
                        this.bootOrder_keep.set("checked", true);
                        this.set("bootOrder", this.pageParams.bootOrder);
                    }
                    break;
                default:
                    this.both_boot_button.style.display = "block";
                    this.secure_boot_button.style.display = "none";
                    this.space_after_secure_button.style.width = "20px";
                    break;
            }
        },
        
        AfterPageSelected: function() {

            // Validate page input and enable/disable "Next" button of wizard accordingly.
            dojo.publish("enableNextButton", true);


        }
    });
});