define([
        "dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/nstFwSettings",
        "dojo/text!./templates/nstFwSettings.html",
        "dojo/DeferredList",
        "dojo/_base/lang",
        "dojo/data/ItemFileReadStore",
        "dijit/form/Select",
        "dijit/Tooltip",
        "support/widgets/Select"
   ], function(dojo, dijit, page, mri, template, modDeferredList, modLang) {

    return dojo.declare("support.wizard.pages.nstFwSettings", [ support.panels.Panel ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        // editMode: boolean
        // Specifies whether the wizard is to execute in edit mode. If "true", the "templateIDNum"
        // attribute is required.
        editMode: false,

        // copyMode: boolean
        // Specifies whether the wizard is to execute in copy mode. If "true", the "templateIDNum"
        // attribute is required.
        copyMode: false,

        // inUse: boolean
        // Specifies whether the wizard is to execute in inUse mode.
        inUse: false,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "nstWizard",

        // Set by the wizard.  Allowed values are FROM_SCRATCH | FROM_SERVER.
        startingPoint: "",

        // Name of chassis settings were learned from.
        chassis: "",

        // Name of server settings were learned from.
        server: "",

        // Internal Selector Names
        SYSTMPL_NAME: "systemInfoTemplate",
        MGMTTMPL_NAME: "managementTemplate",
        PERFORMTMPL_NAME: "performanceTemplate",
        DEVICETMPL_NAME: "deviceTemplate",
        PWRTMPL_NAME: "powerTemplate",
        EXUEFI_NAME: "extendedUEFI",
        EXIMM_NAME: "extendedIMM",
        
        //Template Type Names - Internal Usage
        SYS_INFO: "SystemInfo",
        MGMT: "Management",
        POWER: "Power",
        LEARNED: "Learned",
        DEVICES: "DevicesAndIOPorts",
        PERFORM: "PerformanceRecovery",
        EXIMM: "ExtendedIMM",
        EXUEFI: "ExtendedUEFI",

        SELECTOR_WIDTH: "400px",
        
        sysWarning: false,
        mgmtWarning: false,

        // Override this method to perform custom behavior during dijit construction.
        // Common operations for constructor:
        // 1) Initialize non-primitive types (i.e. objects and arrays)
        // 2) Add additional properties needed by succeeding lifecycle methods
        constructor: function() {
        },

        // When this method is called, all variables inherited from superclasses
        // are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in
        // the template HTML file
        postMixInProperties: function() {

            this.inherited(arguments);

        },

        // postCreate() is called after buildRendering(). This is useful to
        // override when
        // you need to access and/or manipulate DOM nodes included with your
        // widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified
        // can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {

            this.inherited(arguments);
            this._setupSysTmplSelector();
            this._setupEthTmplSelector();
            this._setupPwrTmplSelector();
            this._setupPerformTmplSelector();
            this._setupDeviceTmplSelector();
            this._setupExImmTmplSelector();
            this._setupExUefiTmplSelector();

            this._checkIfHostNameAndSystemNameSame();
            this._validateChannelMode();
            this._scalableWarning();
            
            this.sysWarning = false;
            this.mgmtWarning = false;
        },

        _setStartingPointAttr: function(newval) {
            this._set("startingPoint", newval);
            if (this.startingPoint == "FROM_SCRATCH") {
                if (this.infoMessage) {
                    this.infoMessage.destroyRecursive();
                }
            } else{
                                
                if(this.CP_RUN_ENVIRONMENT == "fsm"){

                    require([ "idx/oneui/messaging/SingleMessage" ], dojo.hitch(this, function() {

                        this.infoMessage = new idx.oneui.messaging.SingleMessage({
                            description: this.locResources.LEARNED_SETTINGS_DETAILS,
                            type: "information",
                            dateFormat: { formatLength: "medium", selector: "time" },
                            showId: false,
                            showAction: false,
                            showRefresh: false,
                            showDetailsLink: false
                        }).placeAt(this.id + "_messagearea");
                        this.infoMessage.set("title", this.locResources.LEARNED_SETTINGS_INFO.replace("%1", this.chassis).replace("%2", this.server));
                    
                    }));
                
                }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                    require([ "idx/widget/SingleMessage" ], dojo.hitch(this, function() {

                        this.infoMessage = new idx.widget.SingleMessage({
                            description: this.locResources.LEARNED_SETTINGS_DETAILS,
                            type: "information",
                            dateFormat: { formatLength: "medium", selector: "time" },
                            showId: false,
                            showAction: false,
                            showRefresh: false,
                            showDetailsLink: false
                        }).placeAt(this.id + "_messagearea");
                        this.infoMessage.set("title", this.locResources.LEARNED_SETTINGS_INFO.replace("%1", this.chassis).replace("%2", this.server));
                    
                    }));
                }
            }          
            this._exImmTmplEditOption.title=this.resources.EDIT_SELECTED_PATTERN_DISABLED;
            this._exUefiTmplEditOption.title=this.resources.EDIT_SELECTED_PATTERN_DISABLED;
        },

        _setChassisAttr: function(chassis) {
           this._set("chassis", chassis);
           this.infoMessage.set("title", this.locResources.LEARNED_SETTINGS_INFO.replace("%1", this.chassis).replace("%2", this.server));
        },

        _setServerAttr: function(server) {
           this._set("server", server);
           this.infoMessage.set("title", this.locResources.LEARNED_SETTINGS_INFO.replace("%1", this.chassis).replace("%2", this.server));
        },

        _presetTemplateCallback: function(size, request) {
            // tags:
            // protected
            // No templates for this category?
            if (size <= 0) {
                // Disable selector and allow "emptyLabel" to display.
                request.selector.set("disabled", true);
                return;
            }
            request.selector.set("disabled", false);
            
            // Add option to select no template and pre-select it.
            // NOTE: Pre-selection may change if template is found below.
            request.selector.addOption({
                value: -1,
                label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
            });
            console.log(this.declaredClass + ": _presetTemplateCallback() pre-setting " + request.selector.name + " to default value");
            request.selector.set("value", -1);
            
            if (this.pageParams.templates) {
                // Pick pre-selected id
                var matches = [];
                switch (request.selector.name) {
                    case this.SYSTMPL_NAME:
                        matches = dojo.filter(this.pageParams.templates, dojo.hitch(this,function(item) {
                            return item.type == this.SYS_INFO;
                        }));
                        break;
                    case this.MGMTTMPL_NAME:
                        matches = dojo.filter(this.pageParams.templates, dojo.hitch(this,function(item) {
                            return item.type == this.MGMT;
                        }));
                        break;
                    case this.PERFORMTMPL_NAME:
                        matches = dojo.filter(this.pageParams.templates, dojo.hitch(this,function(item) {
                            return item.type == this.PERFORM;
                        }));
                        break;
                    case this.DEVICETMPL_NAME:
                        matches = dojo.filter(this.pageParams.templates, dojo.hitch(this,function(item) {
                            return item.type == this.DEVICES;
                        }));
                        break;
                    case this.PWRTMPL_NAME:
                        matches = dojo.filter(this.pageParams.templates, dojo.hitch(this,function(item) {
                            return item.type == this.POWER;
                        }));
                        break;
                    case this.EXIMM_NAME:
                        matches = dojo.filter(this.pageParams.templates, dojo.hitch(this,function(item) {
                            return item.type == this.EXIMM;
                        }));
                        break;
                    case this.EXUEFI_NAME:
                        matches = dojo.filter(this.pageParams.templates, dojo.hitch(this,function(item) {
                            return item.type == this.EXUEFI;
                        }));
                        break;
                }
                if (matches.length > 0) {
                    console.log(this.declaredClass + ": _presetTemplateCallback() setting " + request.selector.name + " to " + matches[0].id);
                    // Change selection to matched item.
                    request.selector.set("value", matches[0].id);
                }
            }
            
            this.sysWarning = true;
            this.mgmtWarning = true;

        },

        _setupSysTmplSelector: function() {

            // summary:
            // Setup SystemInfo template selector...
            // tags:
            // protected
            var tmplStore = new dojo.data.ItemFileReadStore({
                url: this.REST_PATH + "template/list/SystemInfo"
            });
            this._sysTmplSelector = new support.widgets.Select({
                id: this.get("id") + "_sysTmplSelector",
                name: this.SYSTMPL_NAME,
                required: false,
                store: tmplStore,
                emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                maxHeight: 200,
                style: {
                    width: this.SELECTOR_WIDTH
                },
                onChange: dojo.hitch(this, this._changeSysTmplCallback)
            }).placeAt(this._sysTmplSelector);
            tmplStore.fetch({
                query: {},
                onBegin: dojo.hitch(this, this._presetTemplateCallback),
                start: 0,
                count: 0,
                selector: this._sysTmplSelector
            });

            // Also set up a subscriber function to be notified when a new SystemInfo
            // template is created.
            this.subscribe("SystemTemplateComplete", function(data) {
                this._refreshSysTmplSelector(data);
            });
        },
        
        _refreshSysTmplSelector : function(/*anything?*/ selectedValue){
            var tmplStore = new dojo.data.ItemFileReadStore({
                url: this.REST_PATH + "template/list/SystemInfo"
            });
            if (selectedValue == undefined) {
                var preSelect = dojo.filter(this.pageParams.templates, dojo.hitch(this, function(item) {
                    return item.type == this.SYS_INFO;
                }));
                if (preSelect.length > 0) {
                    selectedValue = preSelect[0].id;
                }
            }
            this._sysTmplSelector.setStore(tmplStore, selectedValue);
            this._sysTmplSelector.addOption({
                value: -1,
                label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
            });
            if (selectedValue) {
                this._sysTmplSelector.set("disabled", false);
            }

            this._checkIfHostNameAndSystemNameSame();
        },
        
        _scalableWarning: function() {
            var formFactor = this.pageParams.formFactor;
            var performTemplateId = this._performTmplSelector.get("value");
            var scalable = false;
            
            if (formFactor == "2sc" || formFactor == "4sc" || formFactor == "8sc") {
                scalable = true;                
            }
            
            if (scalable && performTemplateId != -1 && performTemplateId != "") {
                dojo.style(this._performTmplScalableWarning, "visibility", "visible");
                if (!this.warningMessage3) {

                    if(this.CP_RUN_ENVIRONMENT == "fsm"){

                        require([ "idx/oneui/messaging/SingleMessage" ], dojo.hitch(this, function() {

                            this.warningMessage3 = new idx.oneui.messaging.SingleMessage({                                
                                title: this.locResources.PERF_SCALABLE_WARNING_MSG,
                                description: this.locResources.PERF_SCALABLE_WARNING_DETAILS,
                                type: "information",
                                dateFormat: { formatLength: "medium", selector: "time" },
                                showId: false,
                                showAction: false,
                                showRefresh: false,
                                showDetailsLink: false
                            }).placeAt(this.id + "_messagearea");

                        }));

                    }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                        require([ "idx/widget/SingleMessage" ], dojo.hitch(this, function() {

                            this.warningMessage3 = new idx.widget.SingleMessage({
                                title: this.locResources.PERF_SCALABLE_WARNING_MSG,
                                description: this.locResources.PERF_SCALABLE_WARNING_DETAILS,
                                type: "information",
                                dateFormat: { formatLength: "medium", selector: "time" },
                                showId: false,
                                showAction: false,
                                showRefresh: false,
                                showDetailsLink: false
                            }).placeAt(this.id + "_messagearea");

                        }));

                    }
                }

            } else {
                dojo.style(this._performTmplScalableWarning, "visibility", "hidden");
                if (this.warningMessage3) {
                    this.warningMessage3.destroyRecursive();
                    this.warningMessage3 = null;
                }                
            }
        },
        
        _validateChannelMode: function() {
            var channelMode = null;
            //var postWatchdog, rebootOnNMI, haltOnError;
            var invalidMode = false;
            var performTemplateId = this._performTmplSelector.get("value");
            
            if (performTemplateId != -1 && performTemplateId != "") {
                dojo.xhrGet({
                    url: this.REST_PATH + "template/" + performTemplateId,
                    handleAs: "json",
                    sync: true,
                    preventCache: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: dojo.hitch(this, function(response) {
                        channelMode = response.items[0].memory.combobox3_2;
                    })
                });
            
            
                switch (this.pageParams.formFactor) {
                    case "1sd":
                    case "2s":
                    case "pxu":
                    case "4s":
                        if (channelMode != "Independent" && channelMode != "Mirroring" && channelMode != "Sparing") {
                            invalidMode = true;
                        } else {
                            invalidMode = false;
                        }   
                        break;
                    case "2sc":
                    case "4sc":
                    case "8sc":
                        if (channelMode != "Independent" && channelMode != "Lockstep") {
                            invalidMode = true;                        
                        } else {
                            invalidMode = false;
                        }
                        break;
                }
            }
                
            if (invalidMode) {
                dojo.style(this._performTmplWarningRequired, "visibility", "visible");
                if (!this.warningMessage2) {

                    if(this.CP_RUN_ENVIRONMENT == "fsm"){

                        require([ "idx/oneui/messaging/SingleMessage" ], dojo.hitch(this, function() {

                            this.warningMessage2 = new idx.oneui.messaging.SingleMessage({                                
                                title: this.locResources.PERF_WARNING_MSG,
                                description: this.locResources.PERF_WARNING_DETAILS,
                                type: "warning",
                                dateFormat: { formatLength: "medium", selector: "time" },
                                showId: false,
                                showAction: false,
                                showRefresh: false,
                                showDetailsLink: false
                            }).placeAt(this.id + "_messagearea");

                        }));

                    }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                        require([ "idx/widget/SingleMessage" ], dojo.hitch(this, function() {

                            this.warningMessage2 = new idx.widget.SingleMessage({
                                title: this.locResources.PERF_WARNING_MSG,
                                description: this.locResources.PERF_WARNING_DETAILS,
                                type: "warning",
                                dateFormat: { formatLength: "medium", selector: "time" },
                                showId: false,
                                showAction: false,
                                showRefresh: false,
                                showDetailsLink: false
                            }).placeAt(this.id + "_messagearea");

                        }));

                    }
                }

            } else {
                dojo.style(this._performTmplWarningRequired, "visibility", "hidden");
                if (this.warningMessage2) {
                    this.warningMessage2.destroyRecursive();
                    this.warningMessage2 = null;
                }                
            }
        },

        _checkIfInfoPatternRequired: function() {
            var sysTemplateId = this._sysTmplSelector.get("value");
            var mgmtTemplateId = this._ethTmplSelector.get("value");
            var hostname = null;
            var infoPatternRequired = false;
            
            if (mgmtTemplateId != -1 && mgmtTemplateId != "") {
                dojo.xhrGet({
                    url: this.REST_PATH + "template/" + mgmtTemplateId,
                    handleAs: "json",
                    sync: true,
                    preventCache: true,
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    load: dojo.hitch(this, function(response) {
                        hostname = response.items[0].IMM__NetworkSettings_autogen_hostname;
                    })
                });
                
                if (hostname && hostname.pattern) {
                    for (var i = 0; i < hostname.pattern.length; i++) {
                        if (hostname.pattern[i].value == "SL") {
                            infoPatternRequired = true;
                            break;
                        }
                    }
                }
            }
            
            if ((sysTemplateId == -1) && infoPatternRequired) {
                dojo.style(this._sysTmplWarningRequired, "visibility", "visible");
                if (!this.warningMessage1) {
                    
                    if(this.CP_RUN_ENVIRONMENT == "fsm"){

                        require([ "idx/oneui/messaging/SingleMessage" ], dojo.hitch(this, function() {

                            this.warningMessage1 = new idx.oneui.messaging.SingleMessage({                                
                                title: this.locResources.SYS_INFO_REQUIRED,
                                description: this.locResources.SYS_INFO_MESSAGE,
                                type: "warning",
                                dateFormat: { formatLength: "medium", selector: "time" },
                                showId: false,
                                showAction: false,
                                showRefresh: false,
                                showDetailsLink: false
                             }).placeAt(this.id + "_messagearea");
                        
                        }));

                    }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                        require([ "idx/widget/SingleMessage" ], dojo.hitch(this, function() {

                            this.warningMessage1 = new idx.widget.SingleMessage({
                                title: this.locResources.SYS_INFO_REQUIRED,
                                description: this.locResources.SYS_INFO_MESSAGE,
                                type: "warning",
                                dateFormat: { formatLength: "medium", selector: "time" },
                                showId: false,
                                showAction: false,
                                showRefresh: false,
                                showDetailsLink: false
                             }).placeAt(this.id + "_messagearea");
                        
                        }));

                    }
                    
                }
            } else {
                dojo.style(this._sysTmplWarningRequired, "visibility", "hidden");
                if (this.warningMessage1) {
                    this.warningMessage1.destroyRecursive();
                    this.warningMessage1 = null;
                }                
            }
        },
        
        _checkIfHostNameAndSystemNameSame: function() {
           var sysTemplateId = this._sysTmplSelector.get("value");
           var ethTemplateId = this._ethTmplSelector.get("value");
           if (typeof sysTemplateId == "string" && sysTemplateId != "" &&
               typeof ethTemplateId == "string" && ethTemplateId != "") {
              // Both selectors refer to a template.  Go get the 2 templates.
              var sysTmplDeferred = dojo.xhrGet({
                  url: this.REST_PATH + "template/" + sysTemplateId,
                  handleAs: "json",
                  sync: false,
                  preventCache: true
              });

              var ethTmplDeferred = dojo.xhrGet({
                 url: this.REST_PATH + "template/" + ethTemplateId,
                 handleAs: "json",
                 sync: false,
                 preventCache: true
              });

              // Would be nice if Dojo provided this utility function, but alas...
              function compareObjects(a, b) {
                 if (modLang.isObject(a) && modLang.isObject(b)) {
                    for (var it in a) {
                       if (!compareObjects(a[it], b[it]))
                          return false;
                    }

                    for (var it in b) {
                       if(!(it in a))
                          return false;
                    }

                    return true;
                 } else {
                    return a === b;
                 }
              }

              // When both templates finish loading, go compare the hostname and systemname.
              var templateLoadedDeferred = new modDeferredList([sysTmplDeferred, ethTmplDeferred]);
              templateLoadedDeferred.then(modLang.hitch(this, function(results) {
                 var systemNamePattern = results[0][1].items[0].IMM__GeneralSettings__info__autogen.pattern;
                 var hostNamePattern = results[1][1].items[0].IMM__NetworkSettings_autogen_hostname.pattern;
                 var generateSystemNameEnabled = results[0][1].items[0].IMM__GeneralSettings__info__autogen.IMM__GeneralSettings__info__autogen == "Custom";
                 var generateHostNameEnabled = results[1][1].items[0].autogenhostnamebuttonvalue == 0;
                 //console.log(systemNamePattern, hostNamePattern);

                 // If the hostname/systemname autogen states are different, or 
                 // autogen is enabled and the patterns are different, then show the warning.
                 // If autogen is disabled, be sure to not compare the patterns because they
                 // could have data the user previously typed in (but it's now being ignored
                 // because it's disabled).
                 var isDifferent = (generateSystemNameEnabled != generateHostNameEnabled) ||
                                   (generateHostNameEnabled && !compareObjects(systemNamePattern, hostNamePattern));
                 if (!isDifferent) {
                    dojo.style(this._sysTmplHostNameSystemNameSame, "visibility", "hidden");
                    dojo.style(this._ethTmplHostNameSystemNameSame, "visibility", "hidden");
                 } else {
                    dojo.style(this._sysTmplHostNameSystemNameSame, "visibility", "visible");
                    dojo.style(this._ethTmplHostNameSystemNameSame, "visibility", "visible");
                 }
              }));
           } else {
              // One of the selections is "No Pattern Selected", so hide the info icon.
              dojo.style(this._sysTmplHostNameSystemNameSame, "visibility", "hidden");
              dojo.style(this._ethTmplHostNameSystemNameSame, "visibility", "hidden");
           }
        },

        _changeSysTmplCallback: function(newVal) {

            // tags:
            // protected
            dojo.style(this._sysTmplContainer, "display", "none");
            if (newVal > 0) {
                // 99299: Wade Fode, 8/2/2012. Future consideration to display contents of the
                // selected template in an expandable section beneath the selector.
                // dojo.replaceClass(this._sysTmplSelectorExpander, "expandTwistie", "noTwistie collapseTwistie");
                dojo.replaceClass(this._sysTmplEditOption, "nstEditIcon", "nstEditIconDisabled");
                this._sysTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN;
            }
            else {
                // dojo.replaceClass(this._sysTmplSelectorExpander, "noTwistie", "expandTwistie collapseTwistie");
                dojo.replaceClass(this._sysTmplEditOption, "nstEditIconDisabled", "nstEditIcon");
                this._sysTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN_DISABLED;
            }

            this._checkIfHostNameAndSystemNameSame();
            this._checkIfInfoPatternRequired();
        },

        _createSysTmpl: function() {
            require(["support/dialogs/SystemInfoTemplate"], dojo.hitch(this, function() {
                var dialog = new support.dialogs.SystemInfoTemplate({
                    id: this.id + "cp_nstfwsettings_createsysteminfodialog"
                });
                dialog.show();
            }));
        },
        
        _editSysTmpl: function() {
            var templateId = this._sysTmplSelector.get("value");
            if ((typeof templateId == "string") && (templateId != "")) {
                templateId = templateId.toString();
                require(["support/dialogs/SystemInfoTemplate"], dojo.hitch(this, function() {
                    var dialog = new support.dialogs.SystemInfoTemplate({
                        id: this.id + "cp_nstfwsettings_editsysteminfodialog",
                        templateIDNum: templateId,
                        templateMode: "edit",
                        templateLaunchPoint : "wizard"
                    });
                    dialog.show();
                }));
            }
        },

        _sysTmplSelectorOnClick: function(/* Event */evt) {

            // tags:
            // protected
            // 99299: Wade Fode, 8/2/2012. Future consideration to display contents of the
            // selected template in an expandable section beneath the selector.
            // if (dojo.style(this._sysTmplContainer, "display") === "block") {
            // dojo.style(this._sysTmplContainer, "display", "none");
            // dojo.replaceClass(this._sysTmplSelectorExpander, "expandTwistie", "noTwistie
            // collapseTwistie");
            // }
            // else {
            // dojo.style(this._sysTmplContainer, "display", "block");
            // dojo.replaceClass(this._sysTmplSelectorExpander, "collapseTwistie", "noTwistie
            // expandTwistie");
            // }

        },

        _setupEthTmplSelector: function() {

            // summary:
            // Setup Management Interface Template selector...
            // tags:
            // protected
            var tmplStore = new dojo.data.ItemFileReadStore({
                url: this.REST_PATH + "template/list/Management"
            });
            this._ethTmplSelector = new support.widgets.Select({
                id: this.get("id") + "_ethTmplSelector",
                name: this.MGMTTMPL_NAME,
                required: false,
                store: tmplStore,
                emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                maxHeight: 200,
                style: {
                    width: this.SELECTOR_WIDTH
                },
                onChange: dojo.hitch(this, this._changeEthTmplCallback)
            }).placeAt(this._ethTmplSelector);
            tmplStore.fetch({
                query: {},
                onBegin: dojo.hitch(this, this._presetTemplateCallback),
                start: 0,
                count: 0,
                selector: this._ethTmplSelector
            });

            // Also set up a subscriber function to be notified when a new Management
            // template is created.
            this.subscribe("EthTemplateComplete", function(data) {
                this._refreshEthTmplSelector(data);
            });

        },
        
        _refreshEthTmplSelector : function(/*anything?*/ selectedValue){
            var tmplStore = new dojo.data.ItemFileReadStore({
                url: this.REST_PATH + "template/list/Management"
            });
            if (selectedValue == undefined) {
                var preSelect = dojo.filter(this.pageParams.templates, dojo.hitch(this, function(item) {
                    return item.type == this.MGMT;
                }));
                if (preSelect.length > 0) {
                    selectedValue = preSelect[0].id;
                }
            }
            this._ethTmplSelector.setStore(tmplStore, selectedValue);
            this._ethTmplSelector.addOption({
                value: -1,
                label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
            });
            if (selectedValue) {
                this._ethTmplSelector.set("disabled", false);
            }

            this._checkIfHostNameAndSystemNameSame();
        },

        _changeEthTmplCallback: function(newVal) {

            // tags:
            // protected
            dojo.style(this._ethTmplContainer, "display", "none");
            if (newVal > 0) {
                // 99299: Wade Fode, 8/2/2012. Future consideration to display contents of the
                // selected template in an expandable section beneath the selector.
                // dojo.replaceClass(this._ethTmplSelectorExpander, "expandTwistie", "noTwistie collapseTwistie");
                dojo.replaceClass(this._ethTmplEditOption, "nstEditIcon", "nstEditIconDisabled");
                this._ethTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN;
            }
            else {
                // dojo.replaceClass(this._ethTmplSelectorExpander, "noTwistie", "expandTwistie collapseTwistie");
                dojo.replaceClass(this._ethTmplEditOption, "nstEditIconDisabled", "nstEditIcon");
                this._ethTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN_DISABLED;
            }

            this._checkIfHostNameAndSystemNameSame();
            this._checkIfInfoPatternRequired();
        },

        _createEthTmpl: function() {
            require(["support/dialogs/ManagementInterfaceTemplate"], dojo.hitch(this, function() {
                var dialog = new support.dialogs.ManagementInterfaceTemplate({
                    id: this.id + "cp_nstfwsettings_createmgmtintdialog"
                });
                dialog.show();
            }));
        },

        _editEthTmpl: function() {
            var templateId = this._ethTmplSelector.get("value");
            if ((typeof templateId == "string") && (templateId != "")) {
                templateId = templateId.toString();
                require(["support/dialogs/ManagementInterfaceTemplate"], dojo.hitch(this, function() {
                    var dialog = new support.dialogs.ManagementInterfaceTemplate({
                        id: this.id + "cp_nstfwsettings_editmgmtintdialog",
                        templateIDNum: templateId,
                        templateMode: "edit",
                        templateLaunchPoint : "wizard"
                    });
                    dialog.show();
                }));
            }
        },

        _setupPwrTmplSelector: function() {

            // summary:
            // Setup Power State Template selector...
            // tags:
            // protected
            var tmplStore = new dojo.data.ItemFileReadStore({
                url: this.REST_PATH + "template/list/Power"
            });
            this._pwrTmplSelector = new support.widgets.Select({
                id: this.get("id") + "_pwrTmplSelector",
                name: this.PWRTMPL_NAME,
                required: false,
                store: tmplStore,
                emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                maxHeight: 200,
                style: {
                    width: this.SELECTOR_WIDTH
                },
                onChange: dojo.hitch(this, this._changePwrTmplCallback)
            }).placeAt(this._pwrTmplSelector);
            tmplStore.fetch({
                query: {},
                onBegin: dojo.hitch(this, this._presetTemplateCallback),
                start: 0,
                count: 0,
                selector: this._pwrTmplSelector
            });

            // Also set up a subscriber function to be notified when a new Power
            // template is created.
            this.subscribe("PowerTemplateComplete", function(data) {
                this._refreshPwrTmplSelector(data);
            });

        },
        
        _refreshPwrTmplSelector : function(/*anything?*/ selectedValue){
            var tmplStore = new dojo.data.ItemFileReadStore({
                url: this.REST_PATH + "template/list/Power"
            });
            if (selectedValue == undefined) {
                var preSelect = dojo.filter(this.pageParams.templates, dojo.hitch(this, function(item) {
                    return item.type == this.POWER;
                }));
                if (preSelect.length > 0) {
                    selectedValue = preSelect[0].id;
                }
            }
            this._pwrTmplSelector.setStore(tmplStore, selectedValue);
            this._pwrTmplSelector.addOption({
                value: -1,
                label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
            });
            if (selectedValue) {
                this._pwrTmplSelector.set("disabled", false);
            }
        },

        _changePwrTmplCallback: function(newVal) {

            // tags:
            // protected
            dojo.style(this._pwrTmplContainer, "display", "none");
            if (newVal > 0) {
                // 99299: Wade Fode, 8/2/2012. Future consideration to display contents of the
                // selected template in an expandable section beneath the selector.
                // dojo.replaceClass(this._pwrTmplSelectorExpander, "expandTwistie", "noTwistie collapseTwistie");
                dojo.replaceClass(this._pwrTmplEditOption, "nstEditIcon", "nstEditIconDisabled");
                this._pwrTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN;
            }
            else {
                // dojo.replaceClass(this._pwrTmplSelectorExpander, "noTwistie", "expandTwistie collapseTwistie");
                dojo.replaceClass(this._pwrTmplEditOption, "nstEditIconDisabled", "nstEditIcon");
                this._pwrTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN_DISABLED;
            }

        },

        _createPwrTmpl: function() {
            require(["support/dialogs/PowerStateAndCappingTemplate"], dojo.hitch(this, function() {
                var dialog = new support.dialogs.PowerStateAndCappingTemplate({
                    id: this.id + "cp_nstfwsettings_createpowerstatedialog"
                });
                dialog.show();
            }));
        },

        _editPwrTmpl: function() {
            var templateId = this._pwrTmplSelector.get("value");
            if ((typeof templateId == "string") && (templateId != "")) {
                templateId = templateId.toString();
                require(["support/dialogs/PowerStateAndCappingTemplate"], dojo.hitch(this, function() {
                    var dialog = new support.dialogs.PowerStateAndCappingTemplate({
                        id: this.id + "cp_nstfwsettings_editpowerstatedialog",
                        templateIDNum: templateId,
                        templateMode: "edit",
                        templateLaunchPoint : "wizard"
                    });
                    dialog.show();
                }));
            }
        },

        _setupPerformTmplSelector: function() {

            // summary:
            // Setup Performance and Recovery Template selector...
            // tags:
            // protected
            var tmplStore = new dojo.data.ItemFileReadStore({
                url: this.REST_PATH + "template/list/PerformanceRecovery"
            });
            this._performTmplSelector = new support.widgets.Select({
                id: this.get("id") + "_performTmplSelector",
                name: this.PERFORMTMPL_NAME,
                required: false,
                store: tmplStore,
                emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                maxHeight: 200,
                style: {
                    width: this.SELECTOR_WIDTH
                },
                onChange: dojo.hitch(this, this._changePerformTmplCallback)
            }).placeAt(this._performTmplSelector);
            tmplStore.fetch({
                query: {},
                onBegin: dojo.hitch(this, this._presetTemplateCallback),
                start: 0,
                count: 0,
                selector: this._performTmplSelector
            });

            // Also set up a subscriber function to be notified when a new Performance and Recovery
            // template is created.
            this.subscribe("PerformanceAndRecoveryComplete", function(data) {
                this._refreshPerformTmplSelector(data);
            });                        

        },
        
        _refreshPerformTmplSelector : function(/*anything?*/ selectedValue){
            var tmplStore = new dojo.data.ItemFileReadStore({
                url: this.REST_PATH + "template/list/PerformanceRecovery"
            });
            if (selectedValue == undefined) {
                var preSelect = dojo.filter(this.pageParams.templates, dojo.hitch(this, function(item) {
                    return item.type == this.PERFORM;
                }));
                if (preSelect.length > 0) {
                    selectedValue = preSelect[0].id;
                }
            }            
            this._performTmplSelector.setStore(tmplStore, selectedValue);
            this._performTmplSelector.addOption({
                value: -1,
                label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
            });
            if (selectedValue) {
                this._performTmplSelector.set("disabled", false);
            }
        },

        _changePerformTmplCallback: function(newVal) {

            // tags:
            // protected
            dojo.style(this._performTmplContainer, "display", "none");
            if (newVal > 0) {
                // 99299: Wade Fode, 8/2/2012. Future consideration to display contents of the
                // selected template in an expandable section beneath the selector.
                // dojo.replaceClass(this._performTmplSelectorExpander, "expandTwistie", "noTwistie collapseTwistie");
                dojo.replaceClass(this._performTmplEditOption, "nstEditIcon", "nstEditIconDisabled");
                this._performTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN;
            }
            else {
                // dojo.replaceClass(this._performTmplSelectorExpander, "noTwistie", "expandTwistie collapseTwistie");
                dojo.replaceClass(this._performTmplEditOption, "nstEditIconDisabled", "nstEditIcon");
                this._performTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN_DISABLED;
            }
            
            this._validateChannelMode();
            this._scalableWarning();

        },

        _createPerformTmpl: function() {
            require(["support/dialogs/PerformanceAndRecoveryTemplate"], dojo.hitch(this, function() {
                var dialog = new support.dialogs.PerformanceAndRecoveryTemplate({
                    id: this.id + "cp_nstfwsettings_createperformancedialog"
                });
                dialog.show();
            }));
        },

        _editPerformTmpl: function() {
            var templateId = this._performTmplSelector.get("value");
            if ((typeof templateId == "string") && (templateId != "")) {
                templateId = templateId.toString();
                require(["support/dialogs/PerformanceAndRecoveryTemplate"], dojo.hitch(this, function() {
                    var dialog = new support.dialogs.PerformanceAndRecoveryTemplate({
                        id: this.id + "cp_nstfwsettings_editperformancedialog",
                        templateIDNum: templateId,
                        templateMode: "edit",
                        templateLaunchPoint : "wizard"
                    });
                    dialog.show();
                }));
            }
        },

        _setupDeviceTmplSelector: function() {

            // summary:
            // Setup Device and I/O Ports Template selector...
            // tags:
            // protected
            var tmplStore = new dojo.data.ItemFileReadStore({
                url: this.REST_PATH + "template/list/DevicesAndIOPorts"
            });
            this._deviceTmplSelector = new support.widgets.Select({
                id: this.get("id") + "_deviceTmplSelector",
                name: this.DEVICETMPL_NAME,
                required: false,
                store: tmplStore,
                emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                maxHeight: 200,
                style: {
                    width: this.SELECTOR_WIDTH
                },
                onChange: dojo.hitch(this, this._changeDeviceTmplCallback)
            }).placeAt(this._deviceTmplSelector);
            tmplStore.fetch({
                query: {},
                onBegin: dojo.hitch(this, this._presetTemplateCallback),
                start: 0,
                count: 0,
                selector: this._deviceTmplSelector
            });

            // Also set up a subscriber function to be notified when a new Performance and Recovery
            // template is created.
            this.subscribe("DevicesAndIOPortsComplete", function(data) {
                this._refreshDeviceTmplSelector(data);
            });

        },
        
        _refreshDeviceTmplSelector : function(/*anything?*/ selectedValue) {
            var tmplStore = new dojo.data.ItemFileReadStore({
                url: this.REST_PATH + "template/list/DevicesAndIOPorts"
            });
            if (selectedValue == undefined) {
                var preSelect = dojo.filter(this.pageParams.templates, dojo.hitch(this, function(item) {
                    return item.type == this.DEVICES;
                }));
                if (preSelect.length > 0) {
                    selectedValue = preSelect[0].id;
                }
            }
            this._deviceTmplSelector.setStore(tmplStore, selectedValue);
            this._deviceTmplSelector.addOption({
                value: -1,
                label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
            });
            if (selectedValue) {
                this._deviceTmplSelector.set("disabled", false);
            }
        },

        _changeDeviceTmplCallback: function(newVal) {

            // tags:
            // protected
            dojo.style(this._deviceTmplContainer, "display", "none");
            if (newVal > 0) {
                // 99299: Wade Fode, 8/2/2012. Future consideration to display contents of the
                // selected template in an expandable section beneath the selector.
                // dojo.replaceClass(this._deviceTmplSelectorExpander, "expandTwistie", "noTwistie collapseTwistie");
                dojo.replaceClass(this._deviceTmplEditOption, "nstEditIcon", "nstEditIconDisabled");
                this._deviceTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN;
            }
            else {
                // dojo.replaceClass(this._deviceTmplSelectorExpander, "noTwistie", "expandTwistie collapseTwistie");
                dojo.replaceClass(this._deviceTmplEditOption, "nstEditIconDisabled", "nstEditIcon");
                this._deviceTmplEditOption.title = this.resources.EDIT_SELECTED_PATTERN_DISABLED;
            }

        },

        _createDeviceTmpl: function() {
            require(["support/dialogs/DevicesAndIOPortsTemplate"], dojo.hitch(this, function() {
                var dialog = new support.dialogs.DevicesAndIOPortsTemplate({
                    id: this.id + "cp_nstfwsettings_createdevicesandiodialog"
                });
                dialog.show();
            }));
        },

        _editDeviceTmpl: function() {
            var templateId = this._deviceTmplSelector.get("value");
            if ((typeof templateId == "string") && (templateId != "")) {
                templateId = templateId.toString();
                require(["support/dialogs/DevicesAndIOPortsTemplate"], dojo.hitch(this, function() {
                    var dialog = new support.dialogs.DevicesAndIOPortsTemplate({
                        id: this.id + "cp_nstfwsettings_editdevicesanddialog",
                        templateIDNum: templateId,
                        templateMode: "edit",
                        templateLaunchPoint : "wizard"
                    });
                    dialog.show();
                }));
            }
        },

        _setupExImmTmplSelector: function() {

            // summary:
            // Setup Learned Settings Template selector...
            // tags:
            // protected
            var tmplStore = new dojo.data.ItemFileReadStore({
                url: this.REST_PATH + "template/list/ExtendedIMM"
            });
            this._exImmTmplSelector = new support.widgets.Select({
                id: this.get("id") + "_exIMMTmplSelector",
                name: this.EXIMM_NAME,
                required: false,
                store: tmplStore,
                emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                maxHeight: 200,
                style: {
                    width: this.SELECTOR_WIDTH
                },
                onChange: dojo.hitch(this, this._changeExImmTmplCallback)
            }).placeAt(this._exImmTmplSelector);
            tmplStore.fetch({
                query: {},
                onBegin: dojo.hitch(this, this._presetTemplateCallback),
                start: 0,
                count: 0,
                selector: this._exImmTmplSelector
            });

            
            this.subscribe("ExImmTemplateComplete", function(data) {
                this._refreshExImmTmplSelector(data);
            });


        },
        
        _refreshExImmTmplSelector : function(/*anything?*/ selectedValue) {
            var tmplStore = new dojo.data.ItemFileReadStore({
                url: this.REST_PATH + "template/list/ExtendedIMM"
            });
            if (selectedValue == undefined) {
                var preSelect = dojo.filter(this.pageParams.templates, dojo.hitch(this, function(item) {
                    return item.type == this.EXIMM;
                }));
                if (preSelect.length > 0) {
                    selectedValue = preSelect[0].id;
                }
            }
            this._exImmTmplSelector.setStore(tmplStore, selectedValue);
            this._exImmTmplSelector.addOption({
                value: -1,
                label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
            });
            if (selectedValue) {
                this._exImmTmplSelector.set("disabled", false);
                this._exImmTmplEditOption.title=this.locResources.EDIT_DISABLED_MSG_EXT;
            }
        },

        _changeExImmTmplCallback: function(newVal) {

            // tags:
            // protected
            dojo.style(this._exImmTmplContainer, "display", "none");
            if (newVal > 0) {
                // 99299: Wade Fode, 8/2/2012. Future consideration to display contents of the
                // selected template in an expandable section beneath the selector.
                // dojo.replaceClass(this._exImmTmplSelectorExpander, "expandTwistie", "noTwistie collapseTwistie");
                dojo.replaceClass(this._exImmTmplEditOption, "nstEditIcon", "nstEditIconDisabled");
                this._exImmTmplEditOption.title= this.resources.EDIT_SELECTED_PATTERN;
            }
            else {
                // dojo.replaceClass(this._exImmTmplSelectorExpander, "noTwistie", "expandTwistie collapseTwistie");
                dojo.replaceClass(this._exImmTmplEditOption, "nstEditIconDisabled", "nstEditIcon");
                this._exImmTmplEditOption.title=this.resources.EDIT_SELECTED_PATTERN_DISABLED;
            }

        },

        _createExImmTmpl: function() {
            console.log(this.declaredClass + ": Create Extended IMM");
        },

        _editExImmTmpl: function() {
            var templateId = this._exImmTmplSelector.get("value");
            if ((typeof templateId == "string") && (templateId != "")) {
                templateId = templateId.toString();
                require(["support/dialogs/NewLearnedSettingsTemplate"], dojo.hitch(this, function() {
                    var dialog = new support.dialogs.NewLearnedSettingsTemplate({
                        id: this.id + "cp_nstfwsettings_editimmlearnedsettingsdialog",
                        type: "imm",
                        templateMode: "edit",
                        templateIDNum: templateId,
                        templateLaunchPoint : "wizard"                    
                    });                
                    dialog.show();
                }));
            }
        },

        _setupExUefiTmplSelector: function() {

            // summary:
            // Setup Learned Settings Template selector...
            // tags:
            // protected
            var tmplStore = new dojo.data.ItemFileReadStore({
                url: this.REST_PATH + "template/list/ExtendedUEFI"
            });
            this._exUefiTmplSelector = new support.widgets.Select({
                id: this.get("id") + "_exUefiTmplSelector",
                name: this.EXUEFI_NAME,
                required: false,
                store: tmplStore,
                emptyLabel: "&mdash; " + this.resources.NO_TEMP_DEFINED + " &mdash;",
                maxHeight: 200,
                style: {
                    width: this.SELECTOR_WIDTH
                },
                onChange: dojo.hitch(this, this._changeExUefiTmplCallback)
            }).placeAt(this._exUefiTmplSelector);
            tmplStore.fetch({
                query: {},
                onBegin: dojo.hitch(this, this._presetTemplateCallback),
                start: 0,
                count: 0,
                selector: this._exUefiTmplSelector
            });
            
            this.subscribe("ExUefiTemplateComplete", function(data) {
                this._refreshExUefiTmplSelector(data);
            });

        },
        
        _refreshExUefiTmplSelector : function(/*anything?*/ selectedValue) {
            var tmplStore = new dojo.data.ItemFileReadStore({
                url: this.REST_PATH + "template/list/ExtendedUEFI"
            });
            if (selectedValue == undefined) {
                var preSelect = dojo.filter(this.pageParams.templates, dojo.hitch(this, function(item) {
                    return item.type == this.EXUEFI;
                }));
                if (preSelect.length > 0) {
                    selectedValue = preSelect[0].id;
                }
            }
            this._exUefiTmplSelector.setStore(tmplStore, selectedValue);
            this._exUefiTmplSelector.addOption({
                value: -1,
                label: "&mdash; " + this.resources.NO_TEMP_SELECTED + " &mdash;"
            });
            if (selectedValue) {
                this._exUefiTmplSelector.set("disabled", false);
                this._exUefiTmplEditOption.title=this.locResources.EDIT_DISABLED_MSG_EXT;
            }
        },

        _changeExUefiTmplCallback: function(newVal) {

            // tags:
            // protected
            dojo.style(this._exUefiTmplContainer, "display", "none");
            if (newVal > 0) {
                // 99299: Wade Fode, 8/2/2012. Future consideration to display contents of the
                // selected template in an expandable section beneath the selector.
                // dojo.replaceClass(this._exUefiTmplSelectorExpander, "expandTwistie", "noTwistie collapseTwistie");
                dojo.replaceClass(this._exUefiTmplEditOption, "nstEditIcon", "nstEditIconDisabled");
                this._exUefiTmplEditOption.title= this.resources.EDIT_SELECTED_PATTERN;
            }
            else {
                // dojo.replaceClass(this._exUefiTmplSelectorExpander, "noTwistie", "expandTwistie collapseTwistie");
                dojo.replaceClass(this._exUefiTmplEditOption, "nstEditIconDisabled", "nstEditIcon");
                this._exUefiTmplEditOption.title=this.resources.EDIT_SELECTED_PATTERN_DISABLED;
            }

        },
        
        _createExUefiTmpl: function() {
            console.log(this.declaredClass + ": Create Extended UEFI");
        },

        _editExUefiTmpl: function() {
            var templateId = this._exUefiTmplSelector.get("value");
            if ((typeof templateId == "string") && (templateId != "")) {
                templateId = templateId.toString();
                require(["support/dialogs/NewLearnedSettingsTemplate"], dojo.hitch(this, function() {
                    var dialog = new support.dialogs.NewLearnedSettingsTemplate({
                        id: this.id + "cp_nstfwsettings_edituefilearnedsettingsdialog",
                        type: "uefi",
                        templateMode: "edit",
                        templateIDNum: templateId,
                        templateLaunchPoint : "wizard",
                        performanceRelated: this._performTmplSelector.get("value")
                    });
                    dialog.show();
                }));
            }
        },
        
        refreshAllSelectors : function(){
            this._refreshSysTmplSelector();
            this._refreshEthTmplSelector();
            this._refreshPwrTmplSelector();
            this._refreshPerformTmplSelector();
            this._refreshDeviceTmplSelector();
            this._refreshExImmTmplSelector();
            this._refreshExUefiTmplSelector();
        },
        getFWSettings: function(){
          
            var templates = new Array();

            var SystemInfoVal = this._sysTmplSelector.get("value");                        
            if(SystemInfoVal != -1 && SystemInfoVal != null){
               templates.push({
                   id: SystemInfoVal,
                   type: "SystemInfo"
               });
            }

            var ManagementVal = this._ethTmplSelector.get("value");                        
            if(ManagementVal != -1 && ManagementVal != null){
               templates.push({
                   id: ManagementVal,
                   type: "Management"
               });
            }

            var DevicesAndIOPortsVal = this._deviceTmplSelector.get("value");                        
            if(DevicesAndIOPortsVal != -1 && DevicesAndIOPortsVal != null){
               templates.push({
                   id: DevicesAndIOPortsVal,
                   type: "DevicesAndIOPorts"
               });
            }

            var PerformanceRecoveryVal = this._performTmplSelector.get("value");                        
            if(PerformanceRecoveryVal != -1 && PerformanceRecoveryVal != null){
               templates.push({
                   id: PerformanceRecoveryVal,
                   type: "PerformanceRecovery"
               });
            }

            var PowerVal = this._pwrTmplSelector.get("value");                        
            if(PowerVal != -1 && PowerVal != null){
               templates.push({
                   id: PowerVal,
                   type: "Power"
               });
            }

            var ExtendedUEFIVal = this._exUefiTmplSelector.get("value");                        
            if(ExtendedUEFIVal != -1 && ExtendedUEFIVal != null){
               templates.push({
                   id: ExtendedUEFIVal,
                   type: "ExtendedUEFI"
               });
            }

            var ExtendedIMMVal = this._exImmTmplSelector.get("value");                        
            if(ExtendedIMMVal != -1 && ExtendedIMMVal != null){
               templates.push({
                   id: ExtendedIMMVal,
                   type: "ExtendedIMM"
               });
            }
            
            return templates;
            
        },
        AfterPageSelected: function() {
			//!important
			//Last page of wizard so nothing to do here but function still needed for wizard    
      }

    });

});