define([
        "dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/nstGeneral",
        "dojo/text!./templates/nstGeneral.html",
        "dojo/dom-geometry",
        "dijit/TooltipDialog",
        "support/widgets/Icon",
        "dijit/form/Select",
        "dojo/data/ItemFileReadStore",
        "support/base/TableMixin",
        "dijit/form/SimpleTextarea",
        "dijit/TitlePane"
    ], function(dojo, dijit, page, mri, template, domGeometry) {

    return dojo.declare("support.wizard.pages.nstGeneral", [ support.panels.Panel, support.base.TableMixin ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,
        
        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri, 

        // editMode: boolean
        // Specifies whether the wizard is to execute in edit mode. If "true", the "templateIDNum"
        // attribute is required.
        editMode: false,

        // copyMode: boolean
        // Specifies whether the wizard is to execute in copy mode. If "true", the "templateIDNum"
        // attribute is required.
        copyMode: false,

        // inUse: boolean
        // Specifies whether the wizard is to execute in inUse mode.
        inUse: false,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "nstWizard",

        FROM_SERVER: "fromServer",
        FROM_SCRATCH: "fromScratch",

        _serverTable: null,
        
        //Variables to hold previously selected type/ff choices
        _formFactorValue: null,
        _formFactorConfirmEnabled: false, // Enabled for selector "onChange" events after initial set.
        _nodeTypeValue: null,
        _nodeTypeConfirmEnabled: false, // Enabled for selector "onChange" events after initial set.
        
        // ID prefix used for table
        gridID: "ServerTable",

        // Override this method to perform custom behavior during dijit
        // construction.
        // Common operations for constructor:
        // 1) Initialize non-primitive types (i.e. objects and arrays)
        // 2) Add additional properties needed by succeeding lifecycle methods
        constructor: function() {
        },

        // When this method is called, all variables inherited from superclasses
        // are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in
        // the template HTML file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to
        // override when
        // you need to access and/or manipulate DOM nodes included with your
        // widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified
        // can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);

            //Setup Dialog Text
            this._descTitle.innerHTML = this.resources.DESCRIPTION + " (" + this.resources.LIMIT.replace("%500", "500") + ")";
            
            // Set up User Assistance tooltip
            var uaTooltip = new dijit.TooltipDialog({
                style: {
                    width: "550px"
                },
                content: this.locResources.ASSISTANCE_1 + "<p>" + this.locResources.ASSISTANCE_2 + "</p><p>" + this.locResources.ASSISTANCE_3 + "</p>"
            }); 
            var uaButton = new dijit.form.DropDownButton({
                showLabel: false,
                baseClass: "nstTooltip",
                iconClass: "spTooltipIcon",
                dropDown: uaTooltip
            });
            dojo.connect(uaButton, "onMouseOver", function(e){
                uaButton.openDropDown();
                uaButton._onDropDownMouseUp(e);
            });
            dojo.connect(uaTooltip, "onMouseLeave", uaButton, function(){
                uaButton.closeDropDown();
            });
            uaButton.placeAt(this._uaTooltipButton);
            
            var ffMsg = [""];
            ffMsg.push(this.resources.FF_HELP_2S + "<br /><br />");
            ffMsg.push(this.resources.FF_HELP_PXE + "<br /><br />");
            ffMsg.push(this.resources.FF_HELP_1SD_1 + "<ul style=\"list-style-type: none; margin-top: 0px;\">" + 
                "<li>" + this.resources.FF_HELP_1SD_2 + "</li></ul>");
            ffMsg.push(this.resources.FF_HELP_4S + "<br /><br />");
            ffMsg.push(this.resources.FF_HELP_SCAL + "<ul style=\"list-style-type: none; margin-top: 0px;\">" +
                "<li>" +this.resources.FF_HELP_2SC + "</li>" + 
                "<li>" + this.resources.FF_HELP_4SC + "</li>" +
                "<li>" + this.resources.FF_HELP_8SC + "</li></ul>");
            
            // Set up form factor User Assistance tooltip
            var ffTooltip = new dijit.TooltipDialog({
                style: {
                    width: "550px"
                },
                content: ffMsg.join("")
            }); 
            var ffButton = new dijit.form.DropDownButton({
                showLabel: false,
                baseClass: "nstTooltip",
                iconClass: "spTooltipIcon",
                dropDown: ffTooltip
            });
            dojo.connect(ffButton, "onMouseOver", function(e){
                ffButton.openDropDown();
                ffButton._onDropDownMouseUp(e);
            });
            dojo.connect(ffTooltip, "onMouseLeave", ffButton, function(){
                ffButton.closeDropDown();
            });
            ffButton.placeAt(this._ffTooltipButton);
            
            // Add options to the Node Type selector and connect to change handler.
            this._nodeType.addOption([{
                label: this.resources["SYSX"],
                value: "sysx"
            }
            //  Removed "power" option on 9/19/2012. Control has been hidden in HTML template as well.
            //       Will be considered for future support of "power" systems.
            //,
            // {
            //     label: this.resources["POWER"],
            //     value: "power"
            // }
            ]);
            dojo.connect(this._nodeType, "onChange", this, this._onChangeNodeType);

            // Add options to the Form Factor selector and connect to change handler.
            this._formFactor.addOption([{
                label: this.resources["FF_BAY"].replace("%1", "1"),
                value: "2s"
            },
            {
                label: this.resources["FF_BAY_DUAL"].replace("%1", "1"),
                value: "1sd"
            },
            {
                label: this.resources["FF_BAY"].replace("%1", "2"),
                value: "4s"
            },
            {
                label: this.resources["FF_PXU"].replace("%1", "2"),
                value: "pxu"
            },
            {
                label: this.resources["FF_SCALABLE"].replace("%1", "2"),
                value: "2sc"
            },
            {
                label: this.resources["FF_SCALABLE"].replace("%1", "4"),
                value: "4sc"
            },
            {
                label: this.resources["FF_SCALABLE"].replace("%1", "8"),
                value: "8sc"
            }]);
            dojo.connect(this._formFactor, "onChange", this, this._onChangeFormFactor);
            
            if (!this.editMode && !this.copyMode) {
                // Setup table widget...
                var serverStore = this._retrieveStore();       // Get table store information
                var tableStructure = this._generateServerLayout();  // Get structure for current view            
                
                this._serverTable = this.getTable(serverStore, tableStructure, this.resources.NO_DATA, this.gridID, true);
                this._serverTable.column(0).sort(false);
                
                // Set parent rows as "unselectable"
                this._serverTable.disableParentRows();
                
                this._serverTable.select.row.multiple = false; // set table to single select

                this._serverTable.placeAt(this._serverTableNode);
                
                dojo.connect(this._serverTable.select.row, "onSelectionChange", this, this._onChangeServer);

                this._serverTable.startup();
            }            
        },

        startup: function() {
                       
            //Set initial field values
            (this.pageParams.templateName) && this._templateName.set("value", this.pageParams.templateName + ((this.copyMode) ? " - " + this.resources.COPY : ""));
            (this.pageParams.templateDesc) && this._templateDescription.set("value", this.pageParams.templateDesc);
            
            // Disabled on 9/19/2012. Control has been hidden in HTML template as well.
            //       Will be considered for future support of "power" systems.
            // this._nodeType.set("value", this.pageParams.nodeType);
            this._nodeType.set("value", "sysx");
            this._formFactor.set("value", this.pageParams.formFactor);
            this._nodeType.set("disabled", this.editMode && this.inUse);
            this._formFactor.set("disabled", this.editMode && this.inUse);

            if (this.copyMode || this.editMode) {
                // Hide these containers at startup.
                if (this.editMode) {
                    dojo.style(this._userAssistanceContainer, "display", "none");
                    //Display in use warning
                    if(this.inUse){
                        this.warningMessage.style.display = "block";
                        this.warningMessageText.innerHTML = this.resources.INUSE;
                    }
                }
                dojo.style(this._startingPointContainer, "display", "none");
                dojo.style(this._serverInputContainer, "display", "none");
            }
            else {
                // Hide these containers at startup.
                dojo.style(this._serverInputContainer, "display", "none");
                dojo.style(this._nodeTypeInputContainer, "display", "none");
                dojo.style(this._nameInputContainer, "display", "none");
            }
            
            if (this.existing) {
                this._onSetupFromServer();
            }
            
            // Start all our children widgets too.
            this.inherited(arguments);

        },

        _serverSelectMode: function(/* boolean */readOnly) {

            // tags:
            // protected
            //dojo.style(this._serverTableNode, "display", (readOnly) ? "none" : "");
            //dojo.style(this._serverDisplayNode, "display", (readOnly) ? "" : "none");

            // Show action icon only in "read-only" mode so user may switch back to "edit mode" if
            // they wish to do so.
            //this._serverTableContainer.showActionIcon(readOnly);
            
			this._serverTable && this._serverTable.resize();
        },

        _startingPointSelectMode: function(/* boolean */readOnly) {

            // tags:
            // protected
            //this._startingPointPane.toggle();
            //this._startingPointPane.set("title", "Starting point: Create a new pattern from...");
            //dojo.style(this._startingPointIconsNode, "display", (readOnly) ? "none" : "");
            //dojo.style(this._startingPointDisplayNode, "display", (readOnly) ? "" : "none");

            // Show action icon only in "read-only" mode so user may switch back to "edit mode" if
            // they wish to do so.
            //this._startingPointIconsContainer.showActionIcon(readOnly);

        },

        _retrieveStore: function() {

            // summary:
            // Connect to data store and create models for TreeTable widgets.
            // tags:
            // protected            
            var serverData = "";

            // Determine internal usage type string based on arch
            var determineType = function(arch){
                if(arch){
                    if(arch == "x86_32" || arch == "x86_64" || arch == "x86"){
                        return "sysx";
                    }
                    else if(arch == "ppc32" || arch == "ppc64"){
                        return "power";
                    }
                }
                return "";
            };
                
            var xhrGet = dojo.xhrGet({
                url: this.REST_PATH + "learn/list",
                handleAs: "json",
                preventCache: true,
                sync: true,
                load: function(response) {
  
                    var i, j, k;
                    var rootArray = new Array();      // Loop vars
                    var childrenArray = new Array();  // Hold bays for UI display
                    if (response && response.items) {
                        for (i = 0; i < response.items.length; i++) {  // Loop through Chassis
                            var rootObject = response.items[i];
                            
                            if (rootObject.children) {  // Loop through Servers in Chassis
                                for (j = 0; j < rootObject.children.length; j++) {
                                    
                                    // If one bay has more then one compute node then loop through compute nodes (Caracara)
                                    if (rootObject.children[j].ites.length > 1) { 
                                        for (k = 0; k < rootObject.children[j].ites.length; k++) { //Loop compute nodes

                                            var ff = rootObject.children[j].ites[k].formFactor;
                                            var numBays = rootObject.children[j].ites[0].bays.length;

                                            // Set bay label with subnode indication if 1sd
                                            if (ff == "1sd") {
                                                rootObject.children[j].ites[k].bays[0] = rootObject.children[j].ites[k].bays[0] + ":" + rootObject.children[j].ites[k].subBays[0];
                                            }
                                            else if (numBays > 1) {
                                                rootObject.children[j].ites[k].bays[0] = rootObject.children[j].ites[k].bays[0] + "-" + rootObject.children[j].ites[k].bays[numBays-1];
                                            }

                                            // Assign Arch Type(sysx or power)
                                            var type = (rootObject.children[j].ites[k].architecture) ? determineType(rootObject.children[j].ites[k].architecture) : "";
                                                
                                            rootObject.children[j].ites[k].archType = type;       
                                                
                                            // Choose IP
                                            for (var z=0;z < rootObject.children[j].ites[k].ips.length;z++) {
                                                if (rootObject.children[j].ites[k].ips.length == 1) {
                                                    rootObject.children[j].ites[k].ipToDisplay = rootObject.children[j].ites[k].ips[0];       
                                                } else {
                                                    rootObject.children[j].ites[k].ipToDisplay = rootObject.children[j].ites[k].ips[1];                                                                   
                                                }
                                            } 
                                            // Add to list
                                            childrenArray.push(rootObject.children[j].ites[k]);                                            
                                        } 
                                            
                                    } else if (rootObject.children[j].ites.length == 1){ //If bay has only one compute node (Everything except Caracara)

                                        // (Unless one node of the Caracara has already been deployed to)
                                        var ff = rootObject.children[j].ites[0].formFactor;
                                        var numBays = rootObject.children[j].ites[0].bays.length;

                                        // Set bay label with subnode indication if 1sd
                                        if (ff == "1sd") {
                                            rootObject.children[j].ites[0].bays[0] = rootObject.children[j].ites[0].bays[0] + ":" + rootObject.children[j].ites[0].subBays[0];
                                        }
                                        else if (numBays > 1) {
                                            rootObject.children[j].ites[0].bays[0] = rootObject.children[j].ites[0].bays[0] + "-" + rootObject.children[j].ites[0].bays[numBays-1];
                                        }

                                        // Assign Arch Type(sysx or power)
                                        var type = (rootObject.children[j].ites[0].architecture) ? determineType(rootObject.children[j].ites[0].architecture) : "";
                                                        
                                        rootObject.children[j].ites[0].archType = type;       

                                        // Choose IP
                                        for (var z=0;z < rootObject.children[j].ites[0].ips.length;z++) {
                                            if (rootObject.children[j].ites[0].ips.length == 1) {
                                                rootObject.children[j].ites[0].ipToDisplay = rootObject.children[j].ites[0].ips[0];       
                                            } else {
                                                rootObject.children[j].ites[0].ipToDisplay = rootObject.children[j].ites[0].ips[1];                                                                   
                                            }
                                        }
                                        childrenArray.push(rootObject.children[j].ites[0]);                                            
                                    }
                                }
                                rootObject.children = childrenArray;

                            }
                                
                            rootArray.push(rootObject);  // Add Chassis to json
                            childrenArray = new Array(); // Reset Bay info for next chassis
                        }
                    }
                        
                    serverData = dojo.toJson({
                        "identifier": "id",
                        "label": "name",
                        "items": rootArray
                    }, true); 
                }
            });
                
            var serverStore = new dojo.data.ItemFileWriteStore({
                data: dojo.fromJson(serverData)
            });
            
            if (!("hasChildren" in serverStore)) {
                serverStore.hasChildren = function(id, item) { 
                    return item && this.getValues(item, "children").length;
                };
            };
            
            if (!("getChildren" in serverStore)) {
                serverStore.getChildren = function(item) { 
                    return this.getValues(item, "children");
                };
            };

            return serverStore;
        },
        
        _generateServerLayout: function() {
            var serverLayout = [ {
                field: "name",
                name: this.locResources.NAME,
                formatter: dojo.hitch(this, this._formatServerName),
                width: "200px"
            }, {
                field: "bays",
                name: this.resources.BAYS,
                decorator: this._decorateNoWrap,
                width: "45px"
            }, {
                field: "ipToDisplay",
                name: this.locResources.IPADDR,
                decorator: this._decorateNoWrap,
                width: "auto"
            },
            {
                field: "archType",
                name: this.resources.TYPE,
                resources: this.resources, // provide NLS resources to field formatter
                formatter: this._formatArchType,
                decorator: this._decorateNoWrap,
                width: "100px"
            },
            {
                field: "formFactor",
                name: this.resources.FORM_FACTOR,
                resources: this.resources, // provide NLS resources to field formatter
                formatter: this._formatFormFactor,
                decorator: this._decorateNoWrap,
                width: "140px"
            }
            ];
            
            return serverLayout;
        },

        _formatServerName: function(item) {
            // tags:
            // protected
            var value = item.name;
            var imgTag = "";
            !value && (value = "");
            if (value != "") {
                if (item.type == "chassis") {
                    imgTag = "<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spChassisIcon\" style=\"vertical-align: middle;\">&nbsp;";   
                }
                else if (item.type == "imm") {
                    imgTag = "<img src=\"" + this._blankGif + "\" alt=\"\" class=\"spServerIcon\" style=\"vertical-align: middle;\">&nbsp;";
                }
            }
            return "<span title=\"" + value + "\" style=\"white-space: nowrap;\">" + imgTag + value + "</span>";
        },
        
        _formatArchType: function(item) {
            // tags:
            // protected
            var value = item.archType;
            !value && (value = "");
            switch (value) {
                case "sysx":
                    value = this.resources["SYSX"];
                    break;
                case "power":
                    value = this.resources["POWER"];
                    break;
            }
            return value;
        },
        
        _formatFormFactor: function(item) {
            // tags:
            // protected
            var value = item.formFactor;
            !value && (value = "");
            switch (value) {
                case "2s":
                    value = this.resources["FF_BAY"].replace("%1", "1");
                    break;
                case "1sd":
                    value = this.resources["FF_BAY_DUAL"].replace("%1", "1");
                    break;
                case "4s":
                    value = this.resources["FF_BAY"].replace("%1", "2");
                    break;
                case "pxu":
                    value = this.resources["FF_PXU"].replace("%1", "2");
                    break;
                case "2sc":
                    value = this.resources["FF_SCALABLE"].replace("%1", "2");
                    break;
                case "4sc":
                    value = this.resources["FF_SCALABLE"].replace("%1", "4");
                    break;
                case "8sc":
                    value = this.resources["FF_SCALABLE"].replace("%1", "8");
                    break;
            }
            return value;
        },
                
        _decorateNoWrap: function(value) {
            // tags:
            // protected
            !value && (value = "");
            return "<span title=\"" + value + "\" style=\"white-space: nowrap;\">" + value + "</span>";
        },

        AfterPageSelected: function() {

            // Validate page input and enable/disable "Next" button of wizard accordingly.
            dojo.publish("enableNextButton", this.validate());

        },

        validate: function() {

            if (this._nodeType.get("value").toString().length <= 0) {
                // Node type has not been specified.
                // console.info(this.declaredClass + ": Node type has not been specified.");
                return false;
            }
            if (this._formFactor.get("value").toString().length <= 0) {
                // Form factor has not been specified.
                // console.info(this.declaredClass + ": Form factor has not been specified.");
                return false;
            }            
            if (this.copyMode || this.editMode) {
                if (this._templateName.get("value").toString().length <= 0) {
                    // Template name has not been specified.
                    // console.info(this.declaredClass + ": Template name has not been specified.");
                    return false;
                }
            }
            else {
                if (this._startingPoint.value == "") {
                    // A starting point has not been selected.
                    // console.info(this.declaredClass + ": A starting point has not been selected.");
                    return false;
                }
                if (this._startingPoint.value == this.FROM_SERVER) {
                    var found = dojo.some(this._serverTable.select.row.getSelected(), dojo.hitch(this, function(entry, index) {
                    	var item = this._serverTable.row(entry, true).item();
                        return (item.type == "imm");
                    }));
                    if (!found) {
                        // Start FROM_SERVER but none are selected.
                        // console.info(this.declaredClass + ": Start \"" + this.FROM_SERVER + "\" but none are selected.");
                        return false;
                    }
                }
                if (this._templateName.get("value").toString().length <= 0) {
                    // Template name has not been specified.
                    // console.info(this.declaredClass + ": Template name has not been specified.");
                    return false;
                }
            }

            // console.info(this.declaredClass + ": Everything checks out on this page!");
            return true;

        },

        setTemplateName: function(newValue) {
            this._templateName.set("value", newValue);
        },

        onChangeTemplateName: function(newValue) {
            console.log(this.declaredClass + ": onChangeTemplateName called: ", newValue);
            // Connect handler.
        },

        _onChangeTemplateName: function(newValue) {
            // tags:
            // protected

            // Validate page input and enable/disable "Next" button of wizard accordingly.
            var valid = this.validate();
            dojo.publish("enableNextButton", valid);

            // Call connect handler
            this.onChangeTemplateName(newValue);
        },
        
        setTemplateDescription: function(newValue) {
            this._templateDescription.set("value", newValue);
        },

        onChangeTemplateDescription: function(newValue) {
            console.log(this.declaredClass + ": onChangeTemplateDescription called: ", newValue);
            // Connect handler.
        },

        _onChangeTemplateDescription: function(newValue) {
            // tags:
            // protected
            // Call connect handler
            this.onChangeTemplateDescription(newValue);
        },
        
        setNodeType: function(newValue, /*boolean*/confirm) {
            (typeof confirm == "boolean") && (this._nodeTypeConfirmEnabled = confirm);
            this._nodeType.set("value", newValue);
        },

        onChangeNodeType: function(newValue) {
            console.log(this.declaredClass + ": onChangeNodeType called: ", newValue);
            // Connect handler.
        },

        _onChangeNodeType: function(newValue) {
            // tags:
            // protected
            // Call connect handler
           
           if(!(this.editMode && this.inUse)){
               var commitChange = function() {
                   this._nodeTypeValue = newValue;
                   this.onChangeNodeType(this._nodeTypeValue);
               };
               var cancelChange = function() {
                   this._nodeTypeConfirmEnabled = false;
                   this._nodeType.set("value", this._nodeTypeValue);
               };
               if(this._nodeTypeConfirmEnabled){
                   
                   if(this.CP_RUN_ENVIRONMENT == "fsm"){

                       require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                           var prompt = new support.temp.ConfirmationDialog({
                               buttonLabel: "Change",
                               type: "Warning",
                               text: this.locResources.NODE_TYPE_CONFIRM_WARNING,
                               info: "</p>" + this.locResources.NODE_TYPE_CONFIRM_INFO + "</p>"
                           });

                		   prompt.startup();
        		           prompt.confirm(commitChange, this); 
		                   dojo.connect(prompt, "onCancel", this, cancelChange);

                   
                       }));
                       
                   }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                       require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                           var prompt = new idx.widget.ConfirmationDialog({
                               buttonLabel: "Change",
                               type: "Warning",
                               text: this.locResources.NODE_TYPE_CONFIRM_WARNING,
                               info: "</p>" + this.locResources.NODE_TYPE_CONFIRM_INFO + "</p>"
                           });

                		   prompt.startup();
        		           prompt.confirm(commitChange, this); 
		                   dojo.connect(prompt, "onCancel", this, cancelChange);

                   
                       }));
                       
                   }
                   
               }
               else {
                   commitChange.call(this);
                   this._nodeTypeConfirmEnabled = true;
               }
           }
        },
        
        setFormFactor: function(newValue, /*boolean*/confirm) {
            (typeof confirm == "boolean") && (this._formFactorConfirmEnabled = confirm);
            this._formFactor.set("value", newValue);
        },

        onChangeFormFactor: function(newValue) {
            console.log(this.declaredClass + ": onChangeFormFactor called: ", newValue);
            // Connect handler.
        },

        _onChangeFormFactor: function(newValue) {
            // tags:
            // protected
            // Call connect handler
            if(!(this.editMode && this.inUse)){
                var commitChange = function() {
                    this._formFactorValue = newValue;
                    this.onChangeFormFactor(this._formFactorValue); 
                };
                var cancelChange = function() {
                    // Revert back to old value.
                    this._formFactorConfirmEnabled = false; // No confirmation when reverting to old value
                    this._formFactor.set("value", this._formFactorValue);
                };
                if(this._formFactorConfirmEnabled){
                    if(this.CP_RUN_ENVIRONMENT == "fsm"){

                        require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {

                            var prompt = new support.temp.ConfirmationDialog({
                                buttonLabel: "Change",
                                type: "Warning",
                                text: this.locResources.FORM_CONFIRM_WARNING,
                                info: "<p>" + this.locResources.FORM_CONFIRM_INFO + "</p>"
                            });

                		    prompt.startup();
        		            prompt.confirm(commitChange, this); 
		                    dojo.connect(prompt, "onCancel", this, cancelChange);


                        }));
                        
                    }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                        require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {

                            var prompt = new idx.widget.ConfirmationDialog({
                                buttonLabel: "Change",
                                type: "Warning",
                                text: this.locResources.FORM_CONFIRM_WARNING,
                                info: "<p>" + this.locResources.FORM_CONFIRM_INFO + "</p>"
                            });

                		    prompt.startup();
        		            prompt.confirm(commitChange, this); 
		                    dojo.connect(prompt, "onCancel", this, cancelChange);

                        }));
                        
                    }
                    
                }
                else {
                    // Commit without confirmation.
                    commitChange.call(this);
                    this._formFactorConfirmEnabled = true;
                }
            }
        },

        onChangeServer: function(newValue) {
            console.log(this.declaredClass + ": onChangeServer called: ", newValue);
            // Connect handler.
        },

        _onChangeServer: function() {
            // tags:
            // protected
			var ids = this._serverTable.select.row.getSelected();
			if (ids.length != 1) {
			    return;
			}
            var row = this._serverTable.row(ids[0], true);
            var rawData = row.rawData();
            if (rawData["type"] == "imm") {
                var chassisName = row.parent().rawData()["name"];
                var serverName = rawData["name"];
                var formFactor = this._formatFormFactor(rawData); // formatted!
                this._serverSelectMode(true);
                var newValue = {
                    chassisName: chassisName,
                    serverName: serverName, 
                    serverId: rawData["id"],
                    type: rawData["archType"],
                    formFactor: rawData["formFactor"]
                };
    
                // Validate page input and enable/disable "Next" button of wizard accordingly.
                dojo.publish("enableNextButton", this.validate());
    
                this._serverTablePane.toggle();
                var title = this.locResources.LEARN_FROM + "&nbsp;&nbsp;" + chassisName + " &mdash; " + serverName + " &mdash; " + formFactor;
                var maxWidth = (domGeometry.getContentBox(this._serverTablePane.domNode).w - 40);
                this._serverTablePane.set("title", "<div title=\"" + title + "\" style=\"display:inline-block;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;max-width:" + maxWidth + "px;\">" + title + "</div>");
    
                // Call connect handler
                this.onChangeServer(newValue);
            }
        },

        onChangeStartingPoint: function(newValue) {
            console.log(this.declaredClass + ": onChangeStartingPoint called: ", newValue);
            // Connect handler.
        },

        _onSetupFromServer: function() {

            // tags:
            // protected
            this._startingPoint.value = this.FROM_SERVER;
            dojo.style(this._serverInputContainer, "display", "");
            dojo.style(this._nodeTypeInputContainer, "display", "none");
            dojo.style(this._nameInputContainer, "display", "");
            this._serverTable && this._serverTable.resize();

            // Validate page input and enable/disable "Next" button accordingly.
            dojo.publish("enableNextButton", this.validate());
            
            this._startingPointPane.toggle();
            this._startingPointPane.set("title", this.locResources.START_FROM + "&nbsp;&nbsp;<span style=\"font-weight: normal;\">" + this.locResources.FROM_SERVER + "</span>");
            this._startingPointPane.set("tooltip", this.locResources.FROM_SERVER);

            // Call connect handler
            this.onChangeStartingPoint(this.FROM_SERVER);
            this._onChangeServer(); // Call to inform of pre-selected server, if any.

        },

        _onSetupFromScratch: function() {

            // tags:
            // protected
            this._startingPoint.value = this.FROM_SCRATCH;
            dojo.style(this._serverInputContainer, "display", "none");
            dojo.style(this._nodeTypeInputContainer, "display", "");
            dojo.style(this._nameInputContainer, "display", "");

            // Validate page input and enable/disable "Next" button accordingly.
            dojo.publish("enableNextButton", this.validate());
            
            this._startingPointPane.toggle();
            this._startingPointPane.set("title", this.locResources.START_FROM + "&nbsp;&nbsp;<span style=\"font-weight: normal;\">" + this.locResources.FROM_SCRATCH + "</span>");
            this._startingPointPane.set("tooltip", this.locResources.FROM_SCRATCH);

            // Call connect handler
            this.onChangeStartingPoint(this.FROM_SCRATCH);

        }

    });
});