define([
        "dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/nstIOAdapters",
        "dojo/text!./templates/nstIOAdapters.html",
        "support/widgets/Icon",
        "support/widgets/ButtonBar",
        "dijit/form/Select",
        "dijit/form/DropDownButton",
        "dojo/data/ItemFileWriteStore",
        "support/base/TableMixin",
        "dijit/TooltipDialog",
        "dijit/Tooltip",
        "dijit/MenuItem",
        "dijit/form/Button",
        "dijit/ToolbarSeparator",
        "support/panels/CheckButton"
    ], function(dojo, dijit, page, mri, template) {

    return dojo.declare("support.wizard.pages.nstIOAdapters", [ support.panels.Panel, support.base.TableMixin ], {

        // templateString: [protected] String
        // This panel's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        // editMode: boolean
        // Specifies whether the wizard is to execute in edit mode. If "true", the "templateIDNum"
        // attribute is required.
        editMode: false,

        // copyMode: boolean
        // Specifies whether the wizard is to execute in copy mode. If "true", the "templateIDNum"
        // attribute is required.
        copyMode: false,

        // inUse: boolean
        // Specifies whether the wizard is to execute in inUse mode.
        inUse: false,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "nstWizard",
        
        // ID prefix used for table        
        gridID: "AdapterTable",

        _ioAddressingMode: null,
        _addressPoolEditIcon: null,
        _addressPoolEditIconTooltip: null,
        _ethAddressPool: null,
        _ethAddressDomain: null,
        _fcAddressPool: null,
        _fcAddressDomain: null,
        _ethAddressPoolDefault: "IBM",
        _ethAddressDomainDefault: "1",
        _fcAddressPoolDefault: "IBM",
        _fcAddressDomainDefault: "1",

        _adapterStore: null,
        _adapterTable: null,
        _adapterTableTooltip: null,
        _adapterTableActionList: null,

        _advancedSettingButton: null,
        _editPatternButton: null,
        _createPatternButton: null,
        _assignPatternButton: null,
        _removeAdapterMenuItem: null,
        _moreMenu: null,

        _computeNodeItem: null,
        _expansionUnitItem: null,
        _scaledNodeItem: null,
        _pendingAddItem: null,
        _pendingRemoveItem: null,
        _pendingAssignPatternItems: null,
        _pendingCreatePatternItems: null,
        _pendingEditPatternItems: null,
        _ioAddressingTextCache: null,
                
        _adapterArray: null,
        
        _itemCount: 0,
        
        adapterPageChanged: null,
        
        _childAttr: "items",  // or "children"

        // Override this method to perform custom behavior during dijit construction.
        // Common operations for constructor:
        // 1) Initialize non-primitive types (i.e. objects and arrays)
        // 2) Add additional properties needed by succeeding lifecycle methods
        constructor: function() {
        },

        // When this method is called, all variables inherited from superclasses are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in the template HTML
        // file
        postMixInProperties: function() {
            this.inherited(arguments);
        },

        // postCreate() is called after buildRendering(). This is useful to override when
        // you need to access and/or manipulate DOM nodes included with your widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            
            this._adapterArray = new Array();
            this._scaledNodeItem = new Array();

            this._prepareAddressingModeControls();
            this._prepareVirtualAddressingSettings();

            // Setup adapter table widget...
            this._prepareTreeTable();
            this._setupAllNodeItems(true); // force reset
            this._prepareToolbar(this._adapterTable);

            this._moduleBay1Icon.set("title", this.resources["IOMODULE_BAY_TITLE"].replace("%1", 1)); 
            this._moduleBay2Icon.set("title", this.resources["IOMODULE_BAY_TITLE"].replace("%1", 2)); 
            this._moduleBay3Icon.set("title", this.resources["IOMODULE_BAY_TITLE"].replace("%1", 3)); 
            this._moduleBay4Icon.set("title", this.resources["IOMODULE_BAY_TITLE"].replace("%1", 4)); 
                
            // Close graphic view, by default.
            this._closeGraphicView();
            
            // Set up User Assistance tooltip
            var topic = (this.CP_RUN_ENVIRONMENT == "augusta") ? "provisioning_with_configuration_patterns.html" : "fsm_provisioning_with_configuration_patterns.html";
            var spanAttrs = "class=\"linkText\" onclick=\"" + this.genHelpDocLaunchScript(topic) + "\"";
            var uaTooltip = new dijit.TooltipDialog({
                style: {
                    width: "500px"
                },
                content: this.locResources.USER_ASSISTANCE_TOOLTIP.replace("%1", spanAttrs)
            }); 
            var uaButton = new dijit.form.DropDownButton({
                showLabel: false,
                baseClass: "nstTooltip",
                iconClass: "spTooltipIcon",
                dropDown: uaTooltip
            });            
            dojo.connect(uaButton, "onMouseOver", function(e){
                uaButton.openDropDown();
                uaButton._onDropDownMouseUp(e);
            });
            dojo.connect(uaTooltip, "onMouseLeave", uaButton, function(){
                uaButton.closeDropDown();
            });
            uaButton.placeAt(this._uaTooltipButton);
            
            // Set up subscriber service for Edit Virtual Address dialog
            this.subscribe("EditVirtualAddressingComplete", this._editVirtualAddressingHandler);

            // Set up subscriber service for Add IO Adapter dialog
            this.subscribe("IOAdapterTemplateComplete", this._addAdapterHandler);
            
            // Set up subscriber service for Edit Adapter pattern dialog
            this.subscribe("EditAdapterTemplateComplete", this._editAdapterPatternHandler);
            
            // Set up subscriber service for Edit PortVirtualization pattern dialog
            this.subscribe("EditPortVirtualizationTemplateComplete", this._editPortVirtualizationPatternHandler);
            // The SaveAs widget (from Edit) returns CreatePortVirtualizationTemplateComplete, not EditPortVirtualizationTemplateComplete
            this.subscribe("CreatePortVirtualizationTemplateComplete", this._editPortVirtualizationPatternHandler);            
            
            // Set up subscriber service for Edit PortPhysicalFunction pattern dialog
            // this.subscribe("CreatePortPhysicalFunctionTemplateComplete", this._createPortPhysicalFunctionPatternHandler);

            // Set up subscriber service for Assign Adapter pattern dialog
            this.subscribe("ChooseAdapterComplete", this._assignAdapterPatternHandler);

            // Set up subscriber service for Assign PortVirtualization pattern dialog
            this.subscribe("ChoosePortVirtualizationComplete", this._assignPortVirtualizationPatternHandler);

            // Set up subscriber service for Assign PortPhysicalFunction pattern dialog
            //this.subscribe("ChoosePortPhysicalFunctionTemplateComplete", this._assignPortPhysicalFunctionPatternHandler);
            
            // Set up subscriber service for Assign ExtendedPort pattern dialog
            this.subscribe("ChoosePortExtendedSettingsComplete", this._assignExtendedPortPatternHandler);

        },
        
        _prepareAddressingModeControls: function() {
            // tags:
            // protected
            // Prepare I/O Addressing mode button bar
            var virtualAddrMode = (this.pageParams.adapterSettings.ioAddressingMode == "virtual");
            this._addressPoolEditIcon = new support.widgets.Icon({
                id: this.id + "_editvirtualaddrmodeicon",
                iconClass: {
                    normal: "spEditIcon_v2",
                    selected: "spEditIcon_v2",
                    hover: "spEditIcon_v2",
                    disabled: "spEditInactiveIcon_v2"
                },
                disabled: !virtualAddrMode,
                onClick: dojo.hitch(this, this._editVirtualAddressingOptions),
                style: {
                    width: "22px",
                    height: "22px"
                }
            }).placeAt(this._addressPoolEditIconNode);
            this._ioAddressingMode = new support.widgets.ButtonBar({
                values: "burned|virtual",
                selected: (virtualAddrMode) ? "virtual" : "burned",
                labels: this.locResources["ADDRESSING_BURNED_IN_OPT"] + "|" + this.locResources["ADDRESSING_VIRTUAL_OPT"],
                titles: this.locResources["ADDRESSING_BURNED_IN_OPT_TITLE"] + "|" + this.locResources["ADDRESSING_VIRTUAL_OPT_TITLE"]
            }).placeAt(this._ioAddressingModeNode);
            dojo.connect(this._ioAddressingMode, "onClick", this, this._onChangeIoAddressingMode);
            if (this.editMode && this.inUse) {
                dojo.style(this._ioAddressingModeNode, "display", "none");
                dojo.style(this._ioAddressingModeReadOnly, "display", "");
                if (virtualAddrMode) {
                    dojo.attr(this._ioAddressingModeReadOnly, "innerHTML", this.locResources["ADDRESSING_VIRTUAL_OPT"]);
                }
                else {
                    dojo.attr(this._ioAddressingModeReadOnly, "innerHTML", this.locResources["ADDRESSING_BURNED_IN_OPT"]);
                    dojo.style(this._addressPoolEditIconNode, "display", "none");
                }
            }
            else {
                dojo.style(this._ioAddressingModeNode, "display", "");
                dojo.style(this._addressPoolEditIconNode, "display", "");
                dojo.style(this._ioAddressingModeReadOnly, "display", "none");
            }

            // Set up I/O Addressing tooltip
            var ioaTooltip = new dijit.TooltipDialog({
                style: {
                    width: "500px"
                },
                content: this.locResources["IO_ADAPTER_ADDRESSING_TOOLTIP"].replace("%1", this.PRODUCT_NAME_LONG)
            }); 
            var ioaButton = new dijit.form.DropDownButton({
                showLabel: false,
                baseClass: "nstTooltip",
                iconClass: "spTooltipIcon",
                dropDown: ioaTooltip
            });            
            dojo.connect(ioaButton, "onMouseOver", function(e){
                ioaButton.openDropDown();
                ioaButton._onDropDownMouseUp(e);
            });
            dojo.connect(ioaTooltip, "onMouseLeave", ioaButton, function(){
                ioaButton.closeDropDown();
            });
            ioaButton.placeAt(this._ioaTooltipButton);

        },

        _prepareVirtualAddressingSettings: function() {
            // tags:
            // protected
            var test = (this.pageParams.adapterSettings.ethAddressPool);
            this._ethAddressPool = (test) ? this.pageParams.adapterSettings.ethAddressPool : this._ethAddressPoolDefault;
            
            test = (this.pageParams.adapterSettings.ethAddressDomain);
            this._ethAddressDomain = (test) ? this.pageParams.adapterSettings.ethAddressDomain : this._ethAddressDomainDefault;
            
            test = (this.pageParams.adapterSettings.fcAddressPool);
            this._fcAddressPool = (test) ? this.pageParams.adapterSettings.fcAddressPool : this._fcAddressPoolDefault;
            
            test = (this.pageParams.adapterSettings.fcAddressDomain);
            this._fcAddressDomain = (test) ? this.pageParams.adapterSettings.fcAddressDomain : this._fcAddressDomainDefault;
        },

        _prepareTreeTable: function() {
            // tags:
            // protected
            this._retrieveStore();  // Get table store information
            
            var tableStructure = this._getTableLayout();  // Get structure for current view            
            
            this._adapterTable = this.getTable(this._adapterStore, tableStructure, this.resources.NO_DATA, this.gridID, true);

            this._adapterTable.placeAt(this._adapterTableNode);
            
            dojo.connect(this._adapterTable.select.row, "onSelectionChange", this, this._onSelectionChange);
            
            /* Connect to onCellClick to launch add i/o adapter dialog */
            dojo.connect(this._adapterTable, "onCellClick", this, function(evt) {
                if (evt.columnId != 1) {
                    return;
                }
                var cell = this._adapterTable.cell(evt.rowId, evt.columnId, true);
                var cellData = cell.data();
                if (cellData.type == "addIoAdapter") {
                    this._addAdapterFromLink(cellData);
                }
            });
            
            this._adapterTable.startup();      
        },

        _editVirtualAddressingOptions: function() {
            // tags:
            // protected
            console.log(this.declaredClass + ": _editVirtualAddressingOptions called");
            require(["support/dialogs/EditVirtualAddressing"], dojo.hitch(this, function() {
                var dialog = new support.dialogs.EditVirtualAddressing({
                    id: this.id + "cp_nstioadapters_editvirtualaddrdialog",
                    serverPatternInUse: this.inUse,
                    ethernetPoolId: this._ethAddressPool,
                    ethernetPoolSubrangeId:  this._ethAddressDomain,
                    fibreChannelPoolId: this._fcAddressPool,
                    fibreChannelPoolSubrangeId:  this._fcAddressDomain
                });
                dialog.startup();
                dialog.show();
            }));
        },
        
        _getAdapterData: function(adapterId) {
            // tags:
            // protected
            var adapterData = null;
            dojo.xhrGet({
                url: this.REST_PATH + "supportedDevices/ioAdapter/list",
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                sync: true,
                load: function(data) {
                    var matches = dojo.filter(data.items, function(item) {
                        return (item.id == adapterId);
                    });
                    adapterData = matches ? matches[0] : null;
                },
                error: function(error) {
                    console.error(this.declaredClass + ": Error (", error, ") occurred when retrieving adapter data.");
                    alert(this.resources.UNEXPECTED_ERROR.replace("%1", error));
                    return;
                }
            });
            return adapterData;
        },
        
        _getAdapterPattern: function(patternId) {
            // tags:
            // protected
            var patternData = null;
            dojo.xhrGet({
                url: this.REST_PATH + "template/" + patternId,
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                sync: true,
                load: function(data) {
                    if (data.items && (data.items.length > 0) && (data.items[0].type == "Adapter")) {
                        patternData = data.items[0];
                    }
                    else {
                        console.error(this.declaredClass + ": REST service succeeded but \"Adapter\" pattern not returned.");
                    }
                },
                error: function(error) {
                    console.error(this.declaredClass + ": Error (", error, ") occurred when retrieving adapter pattern.");
                    alert(this.resources.UNEXPECTED_ERROR.replace("%1", error));
                    return null;
                }
            });
            return patternData;
        },
        
        _assembleUserData: function(slotNumber, nodeNumber, adapterData, rawData) {
            // tags:
            // protected
            
            var userData = new Object();
            
            userData.slotNumber = slotNumber;
            userData.nodeNumber = nodeNumber;
            userData.adapterId = rawData.adapterId;
            
            // Pull Adapter pattern ID from raw data, if any.
            var matches = dojo.filter(rawData.templates, function(item) {
                return (item.type == "Adapter");
            });
            userData.templateId = (matches.length > 0) ? matches[0].templateId: "*NONE";
            
            // Process adapter's controller data and map to user value.
            userData.controllers = new Array();
            for (var ctrlIdx = 0; ctrlIdx < adapterData.controllers.length; ++ctrlIdx) {

                var ctrlData = adapterData.controllers[ctrlIdx];
                console.log(this.declaredClass + ": Processing ctrlData: ", ctrlData);

                var controller = new Object();
                controller.controllerNumber = ctrlIdx + 1;
                
                var ctrlCfg = this._getControllerConfig(ctrlData);
                if (ctrlCfg == null) {
                    console.info(this.declaredClass + ": Moving on to next controller...");
                    return;
                }
                controller.ports = new Array();
                for (var portIdx = 0; portIdx < ctrlCfg.ports.length; ++portIdx) {
                    var portData = ctrlCfg.ports[portIdx];

                    // Pull ExtendedPort or PortVirtualization pattern ID from user data, if any.
                    var matches = dojo.filter(rawData.templates, function(item) {
                        return (item.portNumber == portData.portNumber);
                    });
                    for (var itemIdx = 0; itemIdx < matches.length; ++itemIdx) {
                        var item = matches[itemIdx];
                        var port = new Object();
                        port.portNumber = item.portNumber;
                        port.templateId = (item.templateId.length > 0) ? item.templateId : "*NONE";
                        port.templateType = (port.templateId != "*NONE") ? item.type : "";
                        controller.ports.push(port);
                    }
                }
                userData.controllers.push(controller);
            }
            return userData;
            
        },
        
        _editVirtualAddressingHandler: function(userData) {
            // tags:
            // protected

            console.log(this.declaredClass + ": Edit virtual addressing invoked. rawData: ", userData);
            this._ethAddressPool = userData.ethAddressPool;
            this._ethAddressDomain = userData.ethAddressDomain;
            this._fcAddressPool = userData.fcAddressPool;
            this._fcAddressDomain = userData.fcAddressDomain;

            this._ioAddressingTextCache && (delete this._ioAddressingTextCache); // destroy cached information
            this._prepareVirtualAddressingIconSummary();
            this._resetIoAddressingText();

        },
        
        _addAdapterHandler: function(rawData) {
            // tags:
            // protected
            // This method is callback handler invoked by AddIOAdapter dialog. Its purpose is to
            // convert information received from AddIOAdapter into a format identical to that
            // found in Server patterns. Once converted the _addAdapterItem function is used
            // to process the user data and add adapter, port, and vport/function details to
            // the adapter table.
            console.log(this.declaredClass + ": Add adapter handler invoked. rawData: ", rawData); 
            if (this._pendingAddItem && rawData) {
                var adapterData = this._getAdapterData(rawData.adapterId);
                console.log(this.declaredClass + ": Processing adapterData: ", adapterData);
                
                var slotNumber = this._pendingAddItem.slotNumber[0];
                var nodeNumber = this._pendingAddItem.nodeNumber[0];
                var userData = this._assembleUserData(slotNumber, nodeNumber, adapterData, rawData);

                this._addAdapterItem(adapterData, userData);
                console.log(this.declaredClass + ": Boot Devices: ", this.getIoBootDevices());
                this.onChangeIoBootDevices(); // call connect handler to notify listeners
                
                this._adapterTable.model.clearCache();
                this._adapterTable.body.refresh();
            }
        },
        
        _addAdapterItem: function(adapterData, userData) {
            // tags:
            // protected
            // This method is used to process user data provided from Server patterns and add
            // the necessary adapter, port, and vport/function details to the adapter table.
            console.log(this.declaredClass + ": Applying userData: ", userData);
            if (this._pendingAddItem && adapterData) {

                var store = this._adapterStore;
                this._setStoreToEmpty();
                switch (adapterData.type) {
                    case "lom":
                        switch (this.pageParams.formFactor) {
                            case "1sd":
                            case "2s":
                            case "pxu":
                            // 4/16/2014: LOM support for 2sc, 4sc and 8sc scaled systems is descoped for GA5.
                            // Commenting out the following lines effectively prevents LOM configuration from being set for
                            // these types of systems. If/When LOM is supported in future simply remove the comment marks.
                            // case "2sc":
                            // case "4sc":
                            // case "8sc":
                                store.setValue(this._pendingAddItem, "name", this.resources["IO_LOM"].replace("%1", ""));
                                break;
                            case "4s":
                                if (this._pendingAddItem.slotNumber[0] == 0) {
                                    store.setValue(this._pendingAddItem, "name", this.resources["IO_LOM"].replace("%1", "1"));
                                }
                                else if (this._pendingAddItem.slotNumber[0] == 2) {
                                    store.setValue(this._pendingAddItem, "name", this.resources["IO_LOM"].replace("%1", "2"));
                                }
                                break;
                            default:
                                console.error(this.declaredClass + ": Invalid adapter type value (" + adapterData.type + "). Must exit.");
                                return;
                        }                                
                        break;
                    case "adapter":
                        var cardNumber = (this._pendingAddItem.slotNumber[0] + 1);
                        store.setValue(this._pendingAddItem, "name", this.resources["IO_ADAPTER"].replace("%1", cardNumber));
                        break;
                    default:
                        console.error(this.declaredClass + ": Invalid adapter type value (" + adapterData.type + "). Must exit.");
                        return;
                }
                store.setValue(this._pendingAddItem, "adapterId", adapterData.id);
                store.setValue(this._pendingAddItem, "type", adapterData.type);
                store.setValue(this._pendingAddItem, "adapterType", adapterData.adapterType);
                store.setValue(this._pendingAddItem, "supportedNodeTypes", adapterData.supportedNodeTypes);
                store.setValue(this._pendingAddItem, "description", adapterData.description);                
                
                var adapterPattern = null;
                if (adapterData.adapterType == "vfa") {
                    if (!userData.templateId || (userData.templateId == "*NONE")) {
                        console.error(this.declaredClass + ": Adapter pattern ID for VFA not found in user data. Must exit.");
                        return;
                    }
                    adapterPattern = this._getAdapterPattern(userData.templateId);
                    console.log(this.declaredClass + ": Adapter pattern: ", adapterPattern);
                    if (adapterPattern == null) {
                        console.error(this.declaredClass + ": Adapter pattern for VFA not available. Must exit.");
                        return;
                    }
                }
                store.setValue(this._pendingAddItem, "template", (adapterPattern) ? adapterPattern.name : "");
                store.setValue(this._pendingAddItem, "templateId", (adapterPattern) ? adapterPattern.id : "*NONE");
                store.setValue(this._pendingAddItem, "templateType", (adapterPattern) ? adapterPattern.type : "");
                store.setValue(this._pendingAddItem, "adapterData", adapterData);
                store.setValue(this._pendingAddItem, "userData", userData);
                this._updateGridStore();
                
                // Update graphic view with adapter details.
                var cardNumber = (this._pendingAddItem.slotNumber[0] + 1);
                
                if(this.pageParams.formFactor == "4sc" || this.pageParams.formFactor == "8sc") {
                    switch(this._pendingAddItem.slotNumber[0]) {
                        case 0:
                            cardNumber = this._pendingAddItem.nodeNumber[0]*4 - 3;
                            break;
                        case 1:
                            cardNumber = this._pendingAddItem.nodeNumber[0]*4 - 2;
                            break;
                        case 2:
                            cardNumber = this._pendingAddItem.nodeNumber[0]*4 - 1;
                            break;
                        case 3:
                            cardNumber = this._pendingAddItem.nodeNumber[0]*4;
                            break;
                    }
                }
                this._prepareCardGraphic(cardNumber, true, adapterData.type, adapterData.description, adapterData.portCount);

                // Add the adapter's physical port items as children of the newly updated item.
                var parentInfo = {
                    parent: this._pendingAddItem,
                    attribute: this._childAttr
                };
                this._addPhysicalPortItems(adapterPattern, parentInfo, store);

                this._pendingAddItem = null;

                // 4/14/2014: Wade Fode
                // LOMs for scaled systems consume TWO mezz slots - Slot 0 and Slot 2. If a LOM configuration was
                // successfully added for a scaled system pattern above, the pending removal of the second adapter
                // slot configuration must be processed now.
                var isScaled = ((this.pageParams.formFactor == "2sc") || (this.pageParams.formFactor == "4sc") || (this.pageParams.formFactor == "8sc"));
                if (isScaled && (adapterData.type == "lom")) {
                    if (this._pendingRemoveItem != null) {
                        var nodeNumber = this._pendingRemoveItem.nodeNumber[0];
                        var slotNumber = this._pendingRemoveItem.slotNumber[0];
                        console.info(this.declaredClass + ": LOM configuration for scaled system added. Removing 2nd adapter item now. nodeNumber=", nodeNumber, "slotNumber=", slotNumber);
                        switch (slotNumber) {
                            case 0:
                                this._removeIoaItem0(nodeNumber);
                                this._prepareCardGraphic(1, false);
                                break;
                            case 1:
                                this._removeIoaItem1(nodeNumber);
                                this._prepareCardGraphic(2, false);
                                break;
                            case 2:
                                this._removeIoaItem2(nodeNumber);
                                this._prepareCardGraphic(3, false);
                                break;
                            case 3:
                                this._removeIoaItem3(nodeNumber);
                                this._prepareCardGraphic(4, false);
                                break;
                            default:
                                console.error(this.declaredClass + ": Invalid slot number ", slotNumber);
                        }
                    } else {
                        console.info(this.declaredClass + ": LOM configuration for scaled system added, however 2nd adapter item was not provided for removal.");
                    }
                }
                this._pendingRemoveItem = null;

            }

            // Re-validate page input.
            this.validate();
            console.log(this.declaredClass + ": Add adapter handler complete.");

        },
        
        _removeChildItems: function(item, store) {
            // tags:
            // protected
            // Need to be careful when calling this function.
            // The _setStoreToEmpty() function needs to be called before
            // The _updateGridStore() function needs to be called after
            var childItems = store.getChildren(item);
            
            console.log("removeChildItems items=", childItems);
            dojo.forEach(childItems, function(item, i) {
                store.deleteItem(item);                
            });
        },
        
        _assignAdapterPatternHandler: function(rawData) {
            // tags:
            // protected

          console.log(this.declaredClass + ": Assign Adapter pattern handler invoked. rawData: ", rawData);
          if (this._pendingAssignPatternItems) {
              console.log(this.declaredClass + ": Selected items: ", this._pendingAssignPatternItems);
              
              // Pull Adapter pattern ID from raw data and retrieve the pattern.
              var matches = dojo.filter(rawData.templates, function(item) {
                  return (item.type == "Adapter");
              });
              if (matches.length == 0) {
                  console.error(this.declaredClass + ": Adapter pattern ID for VFA not found in raw data. Must exit.");
                  return;
              }
              var adapterPattern = this._getAdapterPattern(matches[0].templateId);
              if (!adapterPattern) {
                  console.error(this.declaredClass + ": Adapter pattern for VFA not available. Must exit.");
                  return;
              }
              console.log(this.declaredClass + ": Applying adapterPattern: ", adapterPattern);
			  var theGridStore = this._adapterTable.store;
			  this._setStoreToEmpty();
              for (var i in this._pendingAssignPatternItems) {
                  var adapterItem = this._pendingAssignPatternItems[i];
                  console.log("adapterItem=", adapterItem);
                  
                  this._removeChildItems(adapterItem, theGridStore);
                  
                  // Add new physical port items as children of the adapter item.
                  var userData = this._assembleUserData(adapterItem.slotNumber[0], adapterItem.nodeNumber[0], adapterItem.adapterData[0], rawData);                 
                  this._adapterStore.setValue(adapterItem, "template", adapterPattern.name);
                  this._adapterStore.setValue(adapterItem, "templateId", adapterPattern.id);
                  this._adapterStore.setValue(adapterItem, "templateType", adapterPattern.type);
                  this._adapterStore.setValue(adapterItem, "userData", userData);
                  this._updateGridStore();
                  
                  var adapterInfo = {
                      parent: adapterItem,
                      attribute: this._childAttr
                  };
                  this._addPhysicalPortItems(adapterPattern, adapterInfo, this._adapterStore);
              }
              
          }
          this._pendingAssignPatternItems = null;
          
          // Re-validate page input.
          this.validate();
          console.log(this.declaredClass + ": Assign Adapter pattern handler complete.");

      },
      
      _editAdapterPatternHandler: function(rawData) {
          // tags:
          // protected

        console.log(this.declaredClass + ": Edit Adapter pattern handler invoked. rawData: ", rawData);
        if (this._pendingEditPatternItems) {
            console.log(this.declaredClass + ": Selected items: ", this._pendingEditPatternItems);
            
            var adapterPattern = this._getAdapterPattern(rawData.id);
            if (!adapterPattern) {
                console.error(this.declaredClass + ": Adapter pattern for VFA not available. Must exit.");
                return;
            }
            console.log(this.declaredClass + ": Applying adapterPattern: ", adapterPattern);
            var theGridStore = this._adapterStore;
            this._setStoreToEmpty();
            for (var i in this._pendingEditPatternItems) {
                var adapterItem = this._pendingEditPatternItems[i];
                console.log("adapterItem=", adapterItem);
                
                if (adapterItem.templateId != adapterPattern.id) {
                    console.info(this.declaredClass + ": Pattern mismatch. Skipping to next adapter.");
                    continue;
                }
                
                // Remove all current physical port (child) items to prepare for new configuration.
                this._removeChildItems(adapterItem, theGridStore);
                
                // Add new physical port items as children of the adapter item.                
                theGridStore.setValue(adapterItem, "template", adapterPattern.name);
                
                var adapterInfo = {
                    parent: adapterItem,
                    attribute: this._childAttr
                };
                this._addPhysicalPortItems(adapterPattern, adapterInfo, theGridStore);
            }
            this._updateGridStore();
        }
        this._pendingEditPatternItems = null;        
        console.log(this.declaredClass + ": Edit PortVirtualization pattern handler complete.");
      },
      
      _assignPortVirtualizationPatternHandler: function(userData) {
          // tags:
          // protected

        console.log(this.declaredClass + ": Assign PortVirtualization pattern handler invoked. userData: ", userData);
        if (this._pendingAssignPatternItems) {
            console.log(this.declaredClass + ": Selected items: ", this._pendingAssignPatternItems);
            
            // Get support data for user's pattern selection.
            var portPattern = this._getPortVirtualizationPattern(userData.id);
            if (!portPattern) {
                console.error(this.declaredClass + ": PortVirtualization pattern for VFA not available. Must exit.");
                return;
            }
            console.log(this.declaredClass + ": Applying portPattern: ", portPattern);
            var theGridStore = this._adapterStore;
            this._setStoreToEmpty();
            for (var i in this._pendingAssignPatternItems) {
                var portItem = this._pendingAssignPatternItems[i];
                console.log("portItem=", portItem);
                theGridStore.setValue(portItem, "template", portPattern.name);
                theGridStore.setValue(portItem, "templateId", portPattern.id);
                theGridStore.setValue(portItem, "templateType", portPattern.type);

                // Remove all current virtual port (child) items.
                this._removeChildItems(portItem, theGridStore);
                
                // Add new virtual port items as children of the port item.
                var portInfo = {
                    parent: portItem,
                    attribute: this._childAttr
                };
                this._addVirtualPortItems(portPattern, portInfo, theGridStore);
            }
            this._updateGridStore();
        }
        this._pendingAssignPatternItems = null;
        
        // Re-validate page input.
        this.validate();
        console.log(this.declaredClass + ": Assign PortVirtualization pattern handler complete.");

      },
      
      _editPortVirtualizationPatternHandler: function(userData) {
          // tags:
          // protected

        console.log(this.declaredClass + ": Edit PortVirtualization pattern handler invoked. userData: ", userData);
        if (this._pendingEditPatternItems) {
            console.log(this.declaredClass + ": Selected items: ", this._pendingEditPatternItems);
            
            // Pass the pending items over to the _assignPortVirtualizationPatternHandler queue and let 
            // it do all the work or reassigning the edited pattern.
            this._pendingAssignPatternItems = this._pendingEditPatternItems;
            this._assignPortVirtualizationPatternHandler(userData);

        }
        this._pendingEditPatternItems = null;        
        console.log(this.declaredClass + ": Edit PortVirtualization pattern handler complete.");

    },

      _assignExtendedPortPatternHandler: function(userData) {
          // tags:
          // protected

          console.log(this.declaredClass + ": Assign ExtendedPort pattern handler invoked. userData: ", userData);
          if (this._pendingAssignPatternItems) {
              console.log(this.declaredClass + ": Selected items: ", this._pendingAssignPatternItems);
              
              // Get support data for user's adapter selection.
              var portPattern = null;
              if (userData.id != "*NONE") {
                  portPattern = this._getExtendedPortPattern(userData.id);
              }
              console.log(this.declaredClass + ": Applying portPattern: ", portPattern);
              var store = this._adapterStore;
              this._setStoreToEmpty();
              for (var i in this._pendingAssignPatternItems) {
                  var item = this._pendingAssignPatternItems[i];
                  console.log("item=", item);
                  store.setValue(item, "template", (portPattern) ? portPattern.name : [""]);
                  store.setValue(item, "templateId", (portPattern) ? portPattern.id : ["*NONE"]);
                  store.setValue(item, "templateType", (portPattern) ? portPattern.type : [""]);
                  this._updateGridStore();
              }
              
          }
          this._pendingAssignPatternItems = null;
          
          // Re-validate page input.
          this.validate();
          console.log(this.declaredClass + ": Assign ExtendedPort pattern handler complete.");

      },
        
        _getControllerConfig: function(ctrlData, adapterPattern) {
            // tags:
            // protected
            if (!ctrlData.configurations || (ctrlData.configurations.length == 0)) {
                console.error(this.declaredClass + ": No controller configurations detected for " + ctrlData.id + ".");
                return null;
            }
            if (ctrlData.configurations.length == 1) {
                // Only 1 supported configuration for this controller so use it.
                var config = ctrlData.configurations[0];
                console.log(this.declaredClass + ": Controller configuration: ", config);
                return config;
            }
            // Get configuration chosen by user.
            // Future support for multi-config ASICs. Current design suggests
            // ASIC configuration would be accomplished using Adapter patterns.
            console.log(this.declaredClass + ": Multiple controller configurations detected. Not supported.");
            return null;
        },
        
        _getModuleBayNumbers: function(connectors, ids, start) {
            // tags:
            // protected
            if ((Object.prototype.toString.apply(start) !== "[object Number]") || (start == NaN)) {
                return [];
            }
            var bayNumbers = [];
            for (var i in connectors) {
                var pcData = connectors[i];
                if (dojo.indexOf(ids, pcData.id) >= 0) {
                    var offset = new Number(pcData.moduleBayOffset);
                    (offset != NaN) && bayNumbers.push(start + offset);
                }
            }
            return bayNumbers;
        },
        
        _getPortConfig: function(portData, adapterPattern, adapterData) {
            // tags:
            // protected
            if (!portData.supportedConfigurations || (portData.supportedConfigurations.length == 0)) {
                console.error(this.declaredClass + ": No supported port config IDs detected for " + portData.id + ".");
                return null;                
            }
            var configId = null;
            if (portData.supportedConfigurations.length == 1) {
                // Only 1 supported configuration for this port so use it.
                configId = portData.supportedConfigurations[0];
                console.log(this.declaredClass + ": Using only supported port config ID (" + configId + ") for port " + portData.id + ".");
            }
            else {
                // Get port configuration chosen by user. Its found in the Adapter pattern.
                console.log(this.declaredClass + ": Multiple supported port config IDs detected. Processing Adapter pattern to determine which to use...");
                dojo.some(adapterPattern.controllers, function(ccfg) {
                    return dojo.some(ccfg.ports, function(pcfg) {
                        var found = (pcfg.portNumber == portData.portNumber);
                        if (found) {
                            // Found matching port ID. Get the port config ID and break out.
                            configId = pcfg.portCfgId;
                            console.log(this.declaredClass + ": Found port config ID (" + configId + ") for port " + portData.id + ".");
                        }
                        return found; // breaks out when true
                    });
                });
                if (configId == null) {
                    console.error(this.declaredClass + ": Port config ID for port " + portData.id + " not found in Adapter pattern. Must exit.");
                    return null;
                }
            }
            
            var matches = dojo.filter(adapterData.portConfigurations, function(item) {
                return (item.id == configId);
            });
            if (matches.length == 0) {
                console.error(this.declaredClass + ": Invalid adapter data. Port configuation (" + configId + ") not found.");
                return null;
            }
            var config = matches[0];
            console.log(this.declaredClass + ": Port configuration: ", config);
            return config;
            
        },
        
        _getPortConfigOption: function(portCfg, portData, adapterPattern) {
            // tags:
            // protected
            if (!portCfg.options || (portCfg.options.length == 0)) {
                console.error(this.declaredClass + ": No configuration options detected for " + portCfg.id + ".");
                return null;
            };
            if (portCfg.options.length == 1) {
                // Only 1 option for this port config so use it.
                var option = portCfg.options[0];
                console.log(this.declaredClass + ": Port configuration option: ", option);
                return option;
            }
            // Get port config option chosen by user. Its found in the Adapter pattern.
            console.log(this.declaredClass + ": Multiple port configuration options detected. Processing Adapter pattern to determine which to use...");
            var configOptionId = null;
            dojo.some(adapterPattern.controllers, function(ccfg) {
                return dojo.some(ccfg.ports, function(pcfg) {
                    var found = (pcfg.portNumber == portData.portNumber);
                    if (found) {
                        // Found matching port ID. Get the port config option ID and break out.
                        configOptionId = pcfg.portCfgOptionId;
                        console.log(this.declaredClass + ": Found port config option ID (" + configOptionId + ") for port " + portData.id + ".");
                    }
                    return found; // breaks out when true
                });
            });
            if (configOptionId == null) {
                console.error(this.declaredClass + ": Port config option ID for port " + portData.id + " not found in Adapter pattern. Must exit.");
                return null;
            }
            
            var matches = dojo.filter(portCfg.options, function(item) {
                return (item.id == configOptionId);
            });
            if (matches.length == 0) {
                console.error(this.declaredClass + ": Invalid adapter data. Port configuation option (" + configOptionId + ") not found.");
                return null;
            }
            var option = matches[0];
            console.log(this.declaredClass + ": Port configuration option: ", option);
            return option;
        },
        
//        _getPortPhysicalFunctionPattern: function(patternId) {
//            var patternData = null;
//            dojo.xhrGet({
//                url: this.REST_PATH + "template/" + patternId,
//                handleAs: "json",
//                headers: {
//                    "Content-Type": "application/json; charset=utf-8"
//                },
//                sync: true,
//                load: function(data) {
//                    if (data.items && (data.items.length > 0) && (data.items[0].type == "PortPhysicalFunction")) {
//                        patternData = data.items[0];
//                    }
//                    else {
//                        console.error(this.declaredClass + ": Request succeeded but \"PortPhysicalFunction\" pattern not found.");
//                    }
//                },
//                error: function(error) {
//                    console.error(this.declaredClass + ": Error (", error, ") occurred when retrieving port physical function pattern.");
//                    alert(this.resources.UNEXPECTED_ERROR.replace("%1", error));
//                    patternData = null;
//                }
//            });
//            return patternData;
//        },
        
        _getExtendedPortPattern: function(patternId) {
            // tags:
            // protected
            var patternData = null;
            dojo.xhrGet({
                url: this.REST_PATH + "template/" + patternId,
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                sync: true,
                load: function(data) {
                    if (data.items && (data.items.length > 0) && (data.items[0].type == "ExtendedPort")) {
                        patternData = data.items[0];
                    }
                    else {
                        console.error(this.declaredClass + ": Request succeeded but \"ExtendedPort\" pattern not found.");
                    }
                },
                error: function(error) {
                    console.error(this.declaredClass + ": Error (", error, ") occurred when retrieving extended port settings pattern.");
                    alert(this.resources.UNEXPECTED_ERROR.replace("%1", error));
                    patternData = null;
                }
            });
            return patternData;
        },
        
        _getPortVirtualizationPattern: function(patternId) {
            // tags:
            // protected
            var patternData = null;
            dojo.xhrGet({
                url: this.REST_PATH + "template/" + patternId,
                handleAs: "json",
                headers: {
                    "Content-Type": "application/json; charset=utf-8"
                },
                sync: true,
                load: function(data) {
                    if (data.items && (data.items.length > 0) && (data.items[0].type == "PortVirtualization")) {
                        patternData = data.items[0];
                    }
                    else {
                        console.error(this.declaredClass + ": Request succeeded but \"PortVirtualization\" pattern not found.");
                    }
                },
                error: function(error) {
                    console.error(this.declaredClass + ": Error (", error, ") occurred when retrieving port virtualization pattern.");
                    alert(this.resources.UNEXPECTED_ERROR.replace("%1", error));
                    patternData = null;
                }
            });
            return patternData;
        },
        
        _getEthernetAddressPool: function(vendor) {
            // tags:
            // protected
            var poolData = null;
            if (vendor) {
                dojo.xhrGet({
                    url: this.REST_PATH + "addresspool/vendordefaults/Ethernet/" + vendor,
                    handleAs: "json",
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    sync: true,
                    load: function(data) {
                        poolData = data;
                    },
                    error: function(error) {
                        console.error(this.declaredClass + ": Error (", error, ") occurred when retrieving Ethernet address pool.");
                        alert(this.resources.UNEXPECTED_ERROR.replace("%1", error));
                        poolData = null;
                    }
                });
            }
            return poolData;
        },
        
        _getFibreAddressPool: function(vendor) {
            // tags:
            // protected
            var poolData = null;
            if (vendor) {
                dojo.xhrGet({
                    url: this.REST_PATH + "addresspool/vendordefaults/Fibre/" + vendor,
                    handleAs: "json",
                    headers: {
                        "Content-Type": "application/json; charset=utf-8"
                    },
                    sync: true,
                    load: function(data) {
                        poolData = data;
                    },
                    error: function(error) {
                        console.error(this.declaredClass + ": Error (", error, ") occurred when retrieving port virtualization pattern.");
                        alert(this.resources.UNEXPECTED_ERROR.replace("%1", error));
                        poolData = null;
                    }
                });
            }
            return poolData;
        },
        
        _getAddressingText: function(physicalFunction) {
            // tags:
            // protected

            !this._ioAddressingTextCache && (this._ioAddressingTextCache = new Object());
            var mode = this._ioAddressingMode.get("selected");
            var key = (mode == "burned") ? mode : ("pff:" + physicalFunction + ":eth:" + this._ethAddressPool + ":fc:" + this._fcAddressPool);
            var text = this._ioAddressingTextCache[key]; // lookup cached text
            if (!text) {
                var pending = new Array();
            
                if (mode == "burned") {
                    pending.push(this.locResources["ADDRESSING_BURNED_IN"]);
                }
                else if (mode == "virtual") {
                    switch (physicalFunction) {
                        case "nic":
                        case "fcoe":
                        case "iscsi":
                            if (this._ethAddressPool == "*NONE") {
                                pending.push(this.locResources["ADDRESSING_BURNED_IN"]);
                                break;
                            }
                            var addressPool = this._getEthernetAddressPool(this._ethAddressPool);
                            console.log(this.declaredClass + ": Ethernet address pool: ", addressPool);
                            if (addressPool != null) {
                                pending.push(this.locResources["ADDRESS_POOL_DOMAIN"].replace("%1", addressPool.name).replace("%2", this._ethAddressDomain));
                            }
                            break;
                    }
                    switch (physicalFunction) {
                        case "fc":
                        case "fcoe":
                            if (this._fcAddressPool == "*NONE") {
                                pending.push(this.locResources["ADDRESSING_BURNED_IN"]);
                                break;
                            }
                            var addressPool = this._getFibreAddressPool(this._fcAddressPool);
                            console.log(this.declaredClass + ": Fibre address pool: ", addressPool);
                            if (addressPool != null) {
                                pending.push(this.locResources["ADDRESS_POOL_DOMAIN"].replace("%1", addressPool.name).replace("%2", this._fcAddressDomain));
                            }
                            break;
                    }
                }
                this._ioAddressingTextCache[key] = text = pending.join("<br>");
            }
            return text;

        },
        
        _getControllerUserData: function(userData, controllerNumber) {
            // tags:
            // protected
            var matches = dojo.filter(userData.controllers, function(item) {
                return (item.controllerNumber == controllerNumber);
            });
            if (matches.length == 0) {
                console.error(this.declaredClass + ": Data for controller " + controllerNumber + " not found in user data.");
                return null;
            }
            return matches[0];
        },
        
        _getPortUserData: function(ctrlUserData, portNumber, dftPortUserData) {
            // tags:
            // protected
            var matches = dojo.filter(ctrlUserData.ports, function(item) {
                return (item.portNumber == portNumber);
            });
            if (matches.length == 0) {
                if (dftPortUserData == null) {
                    console.error(this.declaredClass + ": Data for port " + portNumber + " (controller " + ctrlUserData.controllerNumber + ") not found in user data.");
                    return null;
                }
                console.info(this.declaredClass + ": Data for port " + portNumber + " (controller " + ctrlUserData.controllerNumber + ") not found in user data.");
                console.info(this.declaredClass + ": Using user data provided for port " + dftPortUserData.portNumber + ".");
                matches.push(dftPortUserData);
            }
            return matches[0];
        },
        
        _addPhysicalPortItems: function(adapterPattern, parentInfo, store) {
            // tags:
            // protected
            // This method is used to process user data provided from Server patterns and add
            // the necessary port and vport/function details to adapter table item identified
            // via parentInfo param.

            var firstModuleBay = new Number(parentInfo.parent.moduleBay[0].charAt(0));
            
            // Process controller data for this adapter.
            for (var ctrlIdx = 0; ctrlIdx < parentInfo.parent.adapterData[0].controllers.length; ++ctrlIdx) {
                var ctrlData = parentInfo.parent.adapterData[0].controllers[ctrlIdx];
                console.log(this.declaredClass + ": Processing ctrlData: ", ctrlData);

                var ctrlUserData = this._getControllerUserData(parentInfo.parent.userData[0], ctrlIdx + 1);  
                if (ctrlUserData == null) {
                    console.info(this.declaredClass + ": Moving on to next controller...");
                    return;
                }

                var ctrlCfg = this._getControllerConfig(ctrlData, adapterPattern);
                if (ctrlCfg == null) {
                    console.info(this.declaredClass + ": Moving on to next controller...");
                    return;
                }
              
                // Process physical port data for this controller configuration.
                var initPortUserData = null;
                var mutualPortCfg = null; // used for "mutual" portConfigDependency support
                var mutualPortCfgOpt = null; // used for "mutual" portConfigDependency support
                var firstPortNumber = null;
                for (var portIdx = 0; portIdx < ctrlCfg.ports.length; ++portIdx) {
                    var portData = ctrlCfg.ports[portIdx];
                    console.log(this.declaredClass + ": Processing portData: ", portData);

                    // Get I/O Module bay number for this port
                    var moduleBays = this._getModuleBayNumbers(parentInfo.parent.adapterData[0].physicalConnectors, portData.connectors,
                            firstModuleBay);

                    var portUserData = this._getPortUserData(ctrlUserData, portData.portNumber, initPortUserData);  
                    if (portUserData == null) {
                        console.info(this.declaredClass + ": Moving on to next port...");
                        return;
                    }
                    if (!initPortUserData) {
                        initPortUserData = portUserData;
                    }

                    // Get port configuration for this port
                    var portCfg = (mutualPortCfg) ? mutualPortCfg : this._getPortConfig(portData, adapterPattern, parentInfo.parent.adapterData[0]);
                    if (portCfg == null) {
                        console.info(this.declaredClass + ": Moving on to next port...");
                        return;
                    }

                    // Get port configuration option for this port
                    var portCfgOpt = (mutualPortCfgOpt) ? mutualPortCfgOpt : this._getPortConfigOption(portCfg, portData, adapterPattern);
                    if (portCfgOpt == null) {
                        console.info(this.declaredClass + ": Moving on to next port...");
                        return;
                    }

                    // If this controller requires mutual port configuration save it for reuse.
                    if ((ctrlCfg.portConfigDependency == "mutual") && !mutualPortCfg) {
                        mutualPortCfg = portCfg;
                        mutualPortCfgOpt = portCfgOpt;
                        console.info(this.declaredClass + ": Mutual portConfigDependency detected. Port configuration and option data saved for reuse.");
                    }
                    (!firstPortNumber) && (firstPortNumber = portData.portNumber); 
                    
                    var portName = "";
                    var portDescription = "";
                    var portPattern = null;
                    // 4/10/2014: by Wade Fode
                    // Currently, virtual addressing does NOT apply to adapters placed in secondary nodes of scaled servers.
                    // If support changes in the future the following must change as well...
                    var addressingText = (parentInfo.parent.nodeNumber[0] == 1) ? this._getAddressingText(portCfgOpt.physicalFunction) : this.locResources["ADDRESSING_BURNED_IN"];
                    switch (portCfgOpt.physicalFunction) {
                        case "nic":
                            portName = this.locResources["PORT_NAME_NIC"].replace("%1", portData.portLabel);
                            portDescription = this.locResources["PORT_DESCRIPTION_NIC"];
                            if (portUserData.templateId && (portUserData.templateId != "*NONE")) {
                                portPattern = this._getExtendedPortPattern(portUserData.templateId);
                            }
                            break;
                        case "fcoe":
                            portName = this.locResources["PORT_NAME_NIC"].replace("%1", portData.portLabel);
                            portDescription = this.locResources["PORT_DESCRIPTION_NIC"];
                            if (portUserData.templateId && (portUserData.templateId != "*NONE")) {
                                portPattern = this._getExtendedPortPattern(portUserData.templateId);
                            }
                            break;
                        case "fc":
                            portName = this.locResources["PORT_NAME_HBA"].replace("%1", portData.portLabel);
                            portDescription = this.locResources["PORT_DESCRIPTION_HBA"];
                            if (portUserData.templateId && (portUserData.templateId != "*NONE")) {
                                portPattern = this._getExtendedPortPattern(portUserData.templateId);
                            }
                            break;
                        case "virtual":
                            portName = this.locResources["PORT_NAME"].replace("%1", portData.portLabel);
                            portDescription = this.locResources["PORT_DESCRIPTION_VFA"];
                            if ((portCfg.mode == "vNIC_IBMvf") || (portCfg.mode == "vNIC_independent") || (portCfg.mode == "vNIC_IBMufp")) {
                                portPattern = this._getPortVirtualizationPattern(portUserData.templateId);
                                if (!portPattern) {
                                    console.error(this.declaredClass + ": Missing port virtualization pattern for port "+ portData.portNumber + ". Moving on to next port...");
                                    return;
                                }
                            }
                            // else, pNIC mode...
                            break;
                    }
                    console.log(this.declaredClass + ": Port pattern: ", portPattern);
                    var portItem = this._adapterStore.newItem({
                        id: "item" + this._itemCount++,
                        name: portName,
                        type: "port",
                        portConfigMode: portCfg.mode,
                        portPhysicalFunction: portCfgOpt.physicalFunction,
                        portConfigOptionCode: portCfgOpt.optionCode,
                        moduleBay: moduleBays.join("-"),
                        template: (portPattern) ? portPattern.name : "",
                        templateId: (portPattern) ? portPattern.id : "*NONE",
                        ioAddressing: addressingText,
                        description: portDescription,
                        // extra items...
                        nodeNumber: parentInfo.parent.nodeNumber,
                        adapterId: parentInfo.parent.adapterData[0].id,
                        adapterType: parentInfo.parent.adapterData[0].adapterType,
                        controllerNumber: ctrlIdx + 1,
                        controllerVendor: ctrlData.vendor,
                        controllerChipset: ctrlData.chipset,
                        portCfg: portCfg,
                        portCfgOpt: portCfgOpt,
                        portNumber: portData.portNumber,
                        portLabel: portData.portLabel,
                        portProtocols: portCfgOpt.protocols,
                        firstPortNumber: firstPortNumber,
                        templateType: (portPattern) ? portPattern.type : ""
                    }, parentInfo);
                    this._updateGridStore();
                    
                    if (portCfgOpt.physicalFunction == "virtual") {
                        // Add virtual port items as children of the newly added port item.
                        var portInfo = {
                            parent: portItem,
                            attribute: this._childAttr
                        };
                        this._addVirtualPortItems(portPattern, portInfo, store);
                    }
                        
                }
            }
            
        },
        
        _addVirtualPortItems: function(portPattern, parentInfo, store) {
            // tags:
            // protected
            // This method is used to process user data provided from Server patterns and add
            // the necessary vport/function details to adapter table item identified via
            // parentInfo param.

            var parentPortNumber = parentInfo.parent.portNumber[0];

            // Process virtual port data for this port.
            for (var vportIdx = 0; vportIdx < parentInfo.parent.portCfgOpt[0].vports.length; ++vportIdx) {
                var vportData = parentInfo.parent.portCfgOpt[0].vports[vportIdx];
                console.log(this.declaredClass + ": Processing vportData: ", vportData);

                var functionLabel = null;
                var vportName = "";
                var vportEnabled = true;
                var vportMinBandwidth = null;
                var vportMaxBandwidth = null;
                var vportVlan = null;
                var vportDescription = "";
                var vportPattern = null;
                // 4/10/2014: by Wade Fode
                // Currently, virtual addressing does NOT apply to adapters placed in secondary nodes of scaled servers.
                // If support changes in the future the following must change as well...
                var addressingText = (parentInfo.parent.nodeNumber[0] == 1) ? this._getAddressingText(vportData.physicalFunction) : this.locResources["ADDRESSING_BURNED_IN"];
                if ((parentInfo.parent.portCfg[0].mode == "vNIC_IBMvf") || (parentInfo.parent.portCfg[0].mode == "vNIC_independent") || (parentInfo.parent.portCfg[0].mode == "vNIC_IBMufp")) {
                    functionLabel = (vportData.physicalFunctionLabels) ? vportData.physicalFunctionLabels[parentPortNumber - 1] : "";
                    switch (vportData.physicalFunction) {
                        case "nic":
                            vportName = this.locResources["VPORT_NAME_NIC_FUNCTION"].replace("%1", functionLabel);
                            vportDescription = this.locResources["VPORT_DESCRIPTION_NIC"];
                            break;
                        case "fcoe":
                            vportName = this.locResources["VPORT_NAME_HBA_FUNCTION"].replace("%1", functionLabel);
                            vportDescription = this.locResources["VPORT_DESCRIPTION_NIC_WITH_FCOE"];
                            break;
                        case "iscsi":
                            vportName = this.locResources["VPORT_NAME_HBA_FUNCTION"].replace("%1", functionLabel);
                            vportDescription = this.locResources["VPORT_DESCRIPTION_NIC_WITH_ISCSI"];
                            break;
                    }
                    if (portPattern) {
                        dojo.some(portPattern.vports, function(item, i) {
                            var match = (item.portNumber == vportData.portNumber);
                            if (match) {
                                if (parentInfo.parent.portCfg[0].mode == "vNIC_independent") {
                                    vportEnabled = item.enabled;
                                    vportMinBandwidth = (vportEnabled) ? item.minSpeed : null;
                                    vportMaxBandwidth = (vportEnabled) ? item.maxSpeed : null;
                                    vportVlan = (vportEnabled) ? ((item.vlanId == "*NONE") ? 0 : item.vlanId) : null;
                                    !vportEnabled && (addressingText = null);
                                }
                                else {
                                    vportVlan = (item.vlanId == "*NONE") ? 0 : item.vlanId;
                                }
                            }
                            return match;
                        });
                    }
                }
                else { // pNIC mode...
                    switch (vportData.physicalFunction) {
                        case "nic":
                            vportName = this.locResources["VPORT_NAME_NIC"];
                            vportDescription = this.locResources["VPORT_DESCRIPTION_NIC"];
                            break;
                        case "fcoe":
                            vportName = this.locResources["VPORT_NAME_HBA"];
                            vportDescription = this.locResources["VPORT_DESCRIPTION_NIC_WITH_FCOE"];
                            break;
                        case "iscsi":
                            vportName = this.locResources["VPORT_NAME_HBA"];
                            vportDescription = this.locResources["VPORT_DESCRIPTION_NIC_WITH_ISCSI"];
                            break;
                    }
                    // ExtendedPort pattern MAY BE specified from PortVirtualization pattern. Get details...
                    if (portPattern && (portPattern.extendedPortTemplateId != "*NONE")) {
                        vportPattern = this._getExtendedPortPattern(portPattern.extendedPortTemplateId);
                    }
                }
                console.log(this.declaredClass + ": VPort pattern: ", vportPattern);
                this._adapterStore.newItem({
                    id: "item" + this._itemCount++,
                    name: vportName,
                    type: "vport",
                    portPhysicalFunction: vportData.physicalFunction,
                    moduleBay: parentInfo.parent.moduleBay,
                    template: (vportPattern) ? vportPattern.name : "",
                    templateId: (vportPattern) ? vportPattern.id : "*NONE",
                    disabled: !vportEnabled,
                    minBandwidth: vportMinBandwidth,
                    maxBandwidth: vportMaxBandwidth,
                    vlan: vportVlan,
                    ioAddressing: addressingText,
                    description: vportDescription,
                    // extra items...
                    nodeNumber: parentInfo.parent.nodeNumber,
                    portPhysicalFunctionLabel: functionLabel,
                    portNumber: vportData.portNumber,
                    parentPortNumber: parentPortNumber,
                    parentPortLabel: parentInfo.parent.portLabel,
                    parentPattern: parentInfo.parent.template,
                    templateType: (vportPattern) ? vportPattern.type : ""
                }, parentInfo);   
                this._updateGridStore();
            }
        },

        _getSelectedAdapterRows: function() {
            // tags:
            // protected
            // summary:
            // Get selected Adapter rows from the adapters table.
            var selectedRows = this._getSelectedRowItems();  //this._adapterTable.select.row.getSelected();
            return dojo.filter(selectedRows, function(item) {
                return (item.type && ((item.type.toString() == "adapter") || (item.type.toString() == "lom")));
            });
        },

        _getSelectedPortRows: function() {
            // tags:
            // protected
            // summary:
            // Get selected Port rows from the adapters table.
            var selectedRows = this._getSelectedRowItems(); //this._adapterTable.select.row.getSelected();
            return dojo.filter(selectedRows, function(item) {
                return (item.type && (item.type.toString() == "port"));
            });
        },
        
        _removeTableGuidance: function() {
            // tags:
            // protected
            // Remove guidance from adapter table.
            if (this._adapterTableTooltip) {
                this._adapterTableTooltip.destroyRecursive(false);
                this._adapterTableTooltip = null;
            }
        },
        
        _addTableGuidance: function() {
            // tags:
            // protected
            // Add guidance to adapter table if no adapters are listed, otherwise
            // do nothing.
            // returns: true if enabled
            if (this._adapterTableTooltip == null) {
                var found = dojo.some(this._computeNodeItem.items, function(item) {
                    return (item.type && (item.type.toString().indexOf("add") != 0));
                });
                if (!found) {
                    this._adapterTableTooltip = new dijit.Tooltip({
                        connectId: this._adapterTable.get("id"),
                        label: "<p>" + this.locResources["GUIDANCE_FROM_SCRATCH"] + "</p>",
                        position: [ "above-centered" ]
                    });
                }
            }
            return (this._adapterTableTooltip != null);
        },
        
        _prepareCardGraphic: function(/*number*/cardNumber, /*boolean*/show, /*boolean?*/type, /*string?*/desc, /*number?*/numPorts) {
            // tags:
            // protected
            var targetIcon = this["_card" + cardNumber + "Icon"];
            console.log(((show) ? "Showing card " : "Hiding card ") + cardNumber + ", type \"" + type + "\", with " + numPorts + " ports.");
            if (targetIcon) {
                dojo.style(targetIcon.domNode, "display", (show) ? "" : "none");
                if (type) {
                    targetIcon.set("iconClass", (type == "lom") ? "ioFabricConnectorIcon" : "ioAdapterCardIcon");
                    var title = (type == "lom") ? this.resources["FABRIC_CONNECTOR"] : this.resources["IO_ADAPTER"].replace("%1", cardNumber);
                    (desc) && (title += " - " + desc);
                    targetIcon.set("title", title);
                }
                for (var portNumber = 1; portNumber <= 8; ++portNumber) {
                    this._preparePortGraphic(cardNumber, portNumber, (show && (portNumber <= numPorts)));
                }
            }
        },
        
        _preparePortGraphic: function(/*number*/cardNumber, /*number*/portNumber, /*boolean*/show) {
            // tags:
            // protected
            var targetIcon = this["_card" + cardNumber + "Port" + portNumber + "Icon"];
            if (targetIcon) {
                dojo.style(targetIcon.domNode, "display", (show) ? "" : "none");
                targetIcon.set("title", this.locResources["PORT_NAME"].replace("%1", portNumber));
            }
        },
        
        _prepareVirtualAddressingIconSummary: function() {
            // tags:
            // protected

            // Prepare/Update summary for Edit Virtual Addressing icon.
            if (this._ioAddressingMode && this._ioAddressingMode.get("selected") == "virtual") {
                var tooltip = this.locResources.ADDRESSING_VIRTUAL_SUMMARY_TITLE;
                if (this._ethAddressPool != "*NONE") {
                    tooltip = tooltip.replace("%1", this._ethAddressPool);
                } else {
                    tooltip = tooltip.replace("%1", this.resources.DISABLED);
                }
                
                if (this._fcAddressPool != "*NONE") {
                    tooltip = tooltip.replace("%2", this._fcAddressPool);
                    tooltip = tooltip.replace("%3", this._fcAddressDomain);
                } else {
                    tooltip = tooltip.replace("%2", this.resources.DISABLED);
                    tooltip = tooltip.replace("%3", this._ethAddressDomain);
                }
                
                if (this._addressPoolEditIconTooltip) {
                    this._addressPoolEditIconTooltip.set("label", tooltip);
                }
                else {
                    this._addressPoolEditIconTooltip = new dijit.Tooltip({
                        connectId: this._addressPoolEditIcon.get("id"),
                        label: tooltip,
                        position: [ "below" ]
                    });
                }
            }
            else {
                this._addressPoolEditIconTooltip && this._addressPoolEditIconTooltip.destroyRecursive(false);
                this._addressPoolEditIconTooltip = null;    
            }

        },

        AfterPageSelected: function() {

            // Prepare graphic view
            switch (this.pageParams.nodeType) {
                case "power":
                    switch (this.pageParams.formFactor) {
                        case "2s":
                            dojo.replaceClass(this._graphicViewContainer, "graphicViewContainer powerIte2sBackground");
                            break;
                        case "pxu":
                            dojo.replaceClass(this._graphicViewContainer, "graphicViewContainer powerPxuBackground");
                            break;
                        case "4s":
                            dojo.replaceClass(this._graphicViewContainer, "graphicViewContainer powerIte4sBackground");
                            break;
                        default:
                            // Undefined - no graphic
                            dojo.replaceClass(this._graphicViewContainer, "");
                    }
                    break;
                case "sysx":
                    switch (this.pageParams.formFactor) {
                        case "1sd":
                            dojo.replaceClass(this._graphicViewContainer, "graphicViewContainer sysxIte1sdBackground");
                            break;
                        case "2s":
                            dojo.replaceClass(this._graphicViewContainer, "graphicViewContainer sysxIte2sBackground");
                            break;
                        case "pxu":
                            dojo.replaceClass(this._graphicViewContainer, "graphicViewContainer sysxPxuBackground");
                            break;
                        case "4s":
                            dojo.replaceClass(this._graphicViewContainer, "graphicViewContainer sysxIte4sBackground");
                            break;
                        case "2sc":
                            dojo.replaceClass(this._graphicViewContainer, "graphicViewContainer sysxIte2scBackground");
                            break;
                        case "4sc":
                            dojo.replaceClass(this._graphicViewContainer, "graphicViewContainer sysxIte4scBackground");
                            break;
                        case "8sc":
                            dojo.replaceClass(this._graphicViewContainer, "graphicViewContainer sysxIte8scBackground");
                            break;
                        default:
                            // Undefined - no graphic
                            dojo.replaceClass(this._graphicViewContainer, "");
                    }
                    break;
                default:
                    // Undefined - no graphic
                    dojo.replaceClass(this._graphicViewContainer, "");

            }
            
            // Prepare summary for Edit Virtual Addressing icon.
            this._prepareVirtualAddressingIconSummary();

            // Prepare adapter table items
            this._setupAllNodeItems(false);

            // Also expand the Compute Node item.
            if (!this.get("visited")) {
                var id = this._adapterStore.getIdentity(this._computeNodeItem); //this._adapterTable.store.getIdentity(this._computeNodeItem);
                this._adapterTable.tree.expand(id);
            }

            // Set adapter table toolbar capabilities.
            this._setToolbarCapabilities();

            // Validate page input.
            this.validate();
            console.log(this.declaredClass + ": Page preparation complete.");

        },

        _onSelectionChange: function() {
            // tags:
            // protected
            // summary:
            // Handles adapters table selection events.
            this._setToolbarCapabilities();
        },
        
        onChangeIoAddressingMode: function(newValue) {
            console.log(this.declaredClass + ": onChangeIoAddressingMode called: ", newValue);
            // Connect handler.
        },
        
        onChangeIoBootDevices: function() {
            console.log(this.declaredClass + ": onChangeIoBootDevices called");
            // Connect handler.
        },
        
        _onChangeIoAddressingMode: function(newValue) {
            // tags:
            // protected
            this._addressPoolEditIcon.setDisabled(newValue != "virtual");
            this._prepareVirtualAddressingIconSummary();
            this._resetIoAddressingText();
            
            this._resetInfoMessage(newValue);
            
            this._adapterTable.model.clearCache();
            this._adapterTable.body.refresh();
            
            this._setToolbarCapabilities();
            
            this.onChangeIoAddressingMode(newValue); // call connect handler to notify listeners
            return false; // prevent submit
        },
        
        _resetInfoMessage: function(addressingMode) {
            // tags:
            // protected
            if(!this.infoMessageVirtual && (this.pageParams.formFactor == "4sc" 
                || this.pageParams.formFactor == "8sc")){
                
                if(this.CP_RUN_ENVIRONMENT == "fsm"){

                    require([ "idx/oneui/messaging/SingleMessage" ], dojo.hitch(this, function() {

                        this.infoMessageVirtual = new idx.oneui.messaging.SingleMessage({
                            title: this.locResources.VIRTUAL_INFO_MSG,
                            description: this.locResources.VIRTUAL_MSG_DETAILS,
                            type: "information",
                            dateFormat: { formatLength: "medium", selector: "time" },
                            showId: false,
                            showAction: false,
                            showRefresh: false,
                            showDetailsLink: false
                        });
                    
                    }));
                
                }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                    require([ "idx/widget/SingleMessage" ], dojo.hitch(this, function() {

                        this.infoMessageVirtual = new idx.widget.SingleMessage({
                            title: this.locResources.VIRTUAL_INFO_MSG,
                            description: this.locResources.VIRTUAL_MSG_DETAILS,
                            type: "information",
                            dateFormat: { formatLength: "medium", selector: "time" },
                            showId: false,
                            showAction: false,
                            showRefresh: false,
                            showDetailsLink: false
                        });
                    
                    }));

                }
            }
            
            if(this.infoMessageVirtual){
                if((this.pageParams.formFactor == "4sc" || this.pageParams.formFactor == "8sc") 
                        && addressingMode == "virtual") {
                    this.infoMessageVirtual.placeAt(this._infoMsgVirtual);
                    this._infoMsgVirtual.style.display = "block";
                    this.infoMessageVirtual.resize();
                } else {
                    this._infoMsgVirtual.style.display = "none";
                }
            }  
        },
        
        _resetIoAddressingText: function() {
            // tags:
            // protected
            // Reset I/O Addressing text for all ports (including vports).
            var theStore = (this._adapterTable) ? this._adapterStore : null;
            this._setStoreToEmpty();
            if (theStore) {
                var items = theStore._arrayOfAllItems;
                // console.log("items: ", items);
                for (var i in items) {
                    var item = items[i];
                    // console.log("item: ", i, item);
                    if (!item) {
                        continue;
                    }
                    var ppf = theStore.getValue(item, "portPhysicalFunction");
                    var ioa = theStore.getValue(item, "ioAddressing");
                    if (ppf && ioa && (ioa.length > 0)) {
                        // 4/10/2014: by Wade Fode
                        // Currently, virtual addressing does NOT apply to adapters placed in secondary nodes of scaled servers.
                        // If support changes in the future the following must change as well...
                        var nodeNumber = theStore.getValue(item, "nodeNumber");
                        var addressingText = (nodeNumber == 1) ? this._getAddressingText(ppf) : this.locResources["ADDRESSING_BURNED_IN"];
                        theStore.setValue(item, "ioAddressing", addressingText);
                    }
                }
                this._updateGridStore();
            }
        },
        
        getAddapterSettings: function() {
            var adapterSettings = new Object();
            var mode = this._ioAddressingMode.get("selected");
            adapterSettings.ioAddressingMode = mode;
            adapterSettings.ethAddressPool = (mode == "virtual") ? this._ethAddressPool : "*NONE";
            adapterSettings.ethAddressDomain = (mode == "virtual") ? this._ethAddressDomain : "*NONE";
            adapterSettings.fcAddressPool = (mode == "virtual") ? this._fcAddressPool : "*NONE";
            adapterSettings.fcAddressDomain = (mode == "virtual") ? this._fcAddressDomain : "*NONE";
            adapterSettings.adapters = this.getAdapters();
            console.log(this.declaredClass + ": adapterSettings: ", adapterSettings);
            return adapterSettings;
        },
        
        getIoAddressingMode: function() {
           return this._ioAddressingMode.get("selected");
        },
        
        getFcAddressPool: function() {
            return this._fcAddressPool;
        },
        
        getIoBootDevices: function() {
            var adapterRows = dojo.filter(this._computeNodeItem.items, function(item) {
                return (item.type && (item.type.toString().indexOf("add") != 0));
            });
            if (this.pageParams.formFactor == "pxu") {
                // Add Expansion Unit adapters...
                var moreAdapters = dojo.filter(this._expansionUnitItem.items, function(item) {
                    return (item.type && (item.type.toString().indexOf("add") != 0));
                });
                adapterRows = adapterRows.concat(moreAdapters);
            }
            // 4/10/2014: by Wade Fode
            // Currently, virtual addressing does NOT apply to adapters placed in secondary nodes of
            // scaled servers. Therefore they cannot be Boot Devices.
            // If this changes in the future the following may be uncommented...
            //    if (this.pageParams.formFactor == "4sc" || this.pageParams.formFactor == "8sc") {
            //        for(var i = 0; i < this._scaledNodeItem.length; i++){  
            //            if(this._scaledNodeItem[i] != null){
            //                var moreAdapters = dojo.filter(this._scaledNodeItem[i].items, function(item) {
            //                    return (item.type && (item.type.toString().indexOf("add") != 0));
            //                });
            //                adapterRows = adapterRows.concat(moreAdapters);
            //            }
            //        }
            //    }
            // Prepare Boot Device information.
            var bootDevices = new Array();
            var theGridStore = this._adapterStore;
            for (var aIdx in adapterRows) {
                var adapterItem = adapterRows[aIdx];
                var onComplete = function(items) { //, request) {
                    for (var pIdx in items) {
                        var portItem = items[pIdx];
                        if (portItem.portPhysicalFunction == "virtual") {
                            // In this case boot options are virtual functions.
                            var onVPortComplete = function(items) { //, request) {
                                for (var vpIdx in items) {
                                    var vportItem = items[vpIdx];
                                    if (vportItem.disabled[0]) {
                                        return; // skip disabled functions
                                    }
                                    bootDevices.push({
                                        slotNumber: adapterItem.slotNumber[0],
                                        nodeNumber: adapterItem.nodeNumber[0],
                                        cardNumber: (adapterItem.slotNumber[0] + 1),
                                        type: adapterItem.type.toString(),
                                        adapterId: adapterItem.adapterId.toString(),
                                        adapterType: adapterItem.adapterType.toString(),
                                        portNumber: vportItem.parentPortNumber[0],
                                        portLabel: vportItem.parentPortLabel.toString(),
                                        physicalFunction: vportItem.portPhysicalFunction.toString(),
                                        vportNumber: vportItem.portNumber[0],
                                        physicalFunctionLabel: vportItem.portPhysicalFunctionLabel.toString()
                                    });
                                }
                            };
                            onVPortComplete(theGridStore.getChildren(portItem));
                        }
                        else {
                            bootDevices.push({
                                slotNumber: adapterItem.slotNumber[0],
                                nodeNumber: adapterItem.nodeNumber[0],
                                cardNumber: (adapterItem.slotNumber[0] + 1),
                                type: adapterItem.type.toString(),
                                adapterId: adapterItem.adapterId.toString(),
                                adapterType: adapterItem.adapterType.toString(),
                                portNumber: portItem.portNumber[0],
                                portLabel: portItem.portLabel.toString(),
                                physicalFunction: portItem.portPhysicalFunction.toString(),
                                vportNumber: 0, // not a vport
                                physicalFunctionLabel: null // no vport function
                            });
                        }
                    }
                };
                onComplete(theGridStore.getChildren(adapterItem));
            }
            return bootDevices;

        },
        
        getAdapters: function() {
            
            var adapterRows = dojo.filter(this._computeNodeItem.items, function(item) {
                return (item.type && (item.type.toString().indexOf("add") != 0));
            });
            if (this.pageParams.formFactor == "pxu") {
                // Add Expansion Unit adapters...
                var moreAdapters = dojo.filter(this._expansionUnitItem.items, function(item) {
                    return (item.type && (item.type.toString().indexOf("add") != 0));
                });
                adapterRows = adapterRows.concat(moreAdapters);
            }
            if(this.pageParams.formFactor == "4sc" || this.pageParams.formFactor == "8sc") {
                for(var i = 0; i < this._scaledNodeItem.length; i++){  
                    if(this._scaledNodeItem[i] != null){
                        var moreAdapters = dojo.filter(this._scaledNodeItem[i].items, function(item) {
                            return (item.type && (item.type.toString().indexOf("add") != 0));
                        });
                        adapterRows = adapterRows.concat(moreAdapters);
                    }
                }
            }

            // Prepare "adapterSettings" data for Server pattern save operation...
            var adapters = new Array();
            var theGridStore = this._adapterStore;  //this._adapterTable.store;
            var includeId = (this.editMode || this.copyMode);

            for (var aIdx in adapterRows) {
                var adapterItem = adapterRows[aIdx];
                console.log("getAdapters: adapterItem=", adapterItem.slotNumber, adapterItem.nodeNumber, adapterItem.adapterId, adapterItem.templateId);
                
                var adapterObject = new Object();
                adapterObject.id = (includeId) ? adapterItem.id.toString() : "";
                adapterObject.slotNumber = adapterItem.slotNumber[0];
                adapterObject.nodeNumber = adapterItem.nodeNumber[0];
                adapterObject.adapterId = adapterItem.adapterId.toString();
                adapterObject.templateId = (adapterItem.templateId) ? adapterItem.templateId.toString() : "*NONE";
           
                // add child objects to adapter object ...
                adapterObject.controllers = new Array();
                var onComplete = function(items) { //, request) {
                    for (var pIdx in items) {
                        var portItem = items[pIdx];
                        console.log("getAdapters: portItem=", portItem.controllerNumber, portItem.portNumber, portItem.portConfigMode, portItem.templateId, portItem.templateType);
                        var ports = new Array();
                        
//                        if ((portItem.portConfigMode == "pNIC") && (portItem.portPhysicalFunction == "virtual")) {
//                            // Special case for virtual function ports in pNIC mode.
//                            // In this case multiple patterns are recorded.
//                            // NOTE: This is true for for PortPhysicalFunction pattern support, but not for ExtendedPort
//                            // pattern support. In this latter case the pattern is still applied to the physical port -
//                            // virtual ports are not a concern.
//                            var portItemId = theGrid.store.getIdentity(portItem);
//                            var onVPortComplete = function(items, request) {
//                                for (var vpIdx in items) {
//                                    var vportItem = items[vpIdx];
//                                    console.log("getAdapters: vportItem=", vportItem.portNumber, vportItem.templateId, vportItem.templateType);
//                                    var vportObject = new Object();
//                                    vportObject.id = "";
//                                    vportObject.portNumber = new Number(portItem.portNumber[0]).valueOf();
//                                    vportObject.templateId = (vportItem.templateId) ? vportItem.templateId.toString() : "*NONE";
//                                    vportObject.templateType = (vportItem.templateType) ? vportItem.templateType.toString() : "";
//                                    ports.push(vportObject);
//                                }
//                            };
//                            theGrid.treeModel.getChildren(portItem, onVPortComplete, null, {
//                                parentId: portItemId
//                            });
//                            
//                        }
//                        else {
                            var portObject = new Object();
                            portObject.id = "";
                            portObject.portNumber = new Number(portItem.portNumber[0]).valueOf();
                            portObject.templateId = (portItem.templateId) ? portItem.templateId.toString() : "*NONE";
                            portObject.templateType = (portItem.templateType) ? portItem.templateType.toString() : "";
                            ports.push(portObject);
//                        }
                        
                        // Find/Build the appropriate controller object for this port
                        // and update the adapter object accordingly ...
                        var controllerNumber = new Number(portItem.controllerNumber[0]).valueOf();
                        if (!dojo.some(adapterObject.controllers, function(item) {
                            if (item.controllerNumber == controllerNumber) {
                                // Found! Add port objects to it.
                                item.ports = item.ports.concat(ports);
                                return true;
                            }
                            return false;
                        })) {
                            // Not found! Create one and add port object to it.
                            var controllerObject = new Object();
                            controllerObject.id = "";
                            controllerObject.controllerNumber = controllerNumber;
                            controllerObject.ports = ports;
                            adapterObject.controllers.push(controllerObject);
                        };
                    }
                };
                onComplete(theGridStore.getChildren(adapterItem));
                
                adapters.push(adapterObject);
            }
            return adapters;
            
        },
        
        reloadData: function() {
            console.log(this.declaredClass + ": reloadData() called");
            var virtualAddrMode = (this.pageParams.adapterSettings.ioAddressingMode == "virtual");
            this._ioAddressingMode.set("selected", (virtualAddrMode) ? "virtual" : "burned");
            this._addressPoolEditIcon.setDisabled(!virtualAddrMode);
            this._prepareVirtualAddressingSettings();
            this._prepareVirtualAddressingIconSummary();
            this._setupAllNodeItems(true); // force reset
            this._resetInfoMessage(this._ioAddressingMode.get("selected"));
            this._closeGraphicView();
        },

        validate: function() {
            
            var isValid = true;
//            var adapterRows = dojo.filter(this._computeNodeItem.items, function(item) {
//                return (item.type && (item.type.toString().indexOf("add") != 0));
//            });
//            if (this.pageParams.formFactor == "pxu") {
//                // Add Expansion Unit adapters...
//                var moreAdapters = dojo.filter(this._expansionUnitItem.items, function(item) {
//                    return (item.type && (item.type.toString().indexOf("add") != 0));
//                });
//                adapterRows = adapterRows.concat(moreAdapters);
//            }
//            isValid = (adapterRows.length > 0);

            // Enable/Disable "Next" button accordingly.
            switch (this.pageParams.nodeType) {
                case "power":
                    // Validate page input and enable/disable "Next" button of wizard accordingly.
                    dojo.publish("enableNextButton", false);
                    break;
                default:
                    // Validate page input and enable/disable "Next" button of wizard accordingly.
                    dojo.publish("enableNextButton", isValid);
                    break;
            }

            return isValid;

        },

        _retrieveStore: function() {
            // tags:
            // protected
            // summary:
            // Prepare data source and create the model for the adapter table.
            var emptyData = {
                identifier: "id",
                label: "name",
                items: []
            };

            this._adapterStore = new dojo.data.ItemFileWriteStore({
                data: emptyData            
            });
            
            this._computeNodeItem = this._adapterStore.newItem({
                id: "item" + this._itemCount++,
                name: this.locResources["COMPUTE_NODE_ITEM_NAME"],
                type: "root"
            });
            
            var childAttribute = this._childAttr;
            if (!("hasChildren" in this._adapterStore)) {
                this._adapterStore.hasChildren = function(id, item) { 
                    return item && this.getValues(item, childAttribute).length;
                };
            };
            
            if (!("getChildren" in this._adapterStore)) {
                this._adapterStore.getChildren = function(item) { 
                    return this.getValues(item, childAttribute);
                };
            };
        },
                
        onSave: function(itemId) {
            console.log("fetched the items");
            function getItems(items, request) {
                if(items){
                    var i;
                    for(i = 0; i < items.length; i++){
                        var item = items[i];
                        var dirty = this._adapterStore.isDirty(item);
                        if (dirty) {
                            this._adapterStore.save({onComplete: dojo.hitch(this, this.onSave(item.id))});
                        }                        
                    }
                }
                console.log("fetched the items");
            }            
            this._adapterStore.fetch({query:{id: itemId}, onComplete: dojo.hitch(this, getItems), queryOptions: {deep:true}});
            console.log("fetched the items");
        },
        
        _deleteItem: function(item) {
            // tags:
            // protected
            /* This is a workaround for a weird issue with the Gridx table */
            /* For some reason, when the store is set on the Gridx table,  */
            /* the deleteItem will fail.  If the store is set to another   */
            /* dummy store, then the delete will work and the store can be */
            /* set back. */         
            this._setStoreToEmpty();
            this._adapterStore.deleteItem(item);
            this._updateGridStore();
        },
        
        _setStoreToEmpty: function() {
            // tags:
            // protected
            var emptyData = {
                    identifier: "id",
                    label: "name",
                    items: []
                };
            var newStore = new dojo.data.ItemFileWriteStore({data: emptyData});
            this._adapterTable.setStore(newStore);
        },
        
        _openGraphicView: function() {
            // tags:
            // protected
            dojo.style(this._graphicViewContainer, "display", "");
            this._graphicViewIcon.set("title", this.locResources["GRAPHIC_VIEW_CLOSE_TITLE"]);
            
            if(this.pageParams.formFactor == "4sc" || this.pageParams.formFactor == "8sc") {
                dojo.style(this._scaledNode2, "display", "");
                dojo.style(this["primaryNodeLabel"], "display", "inline");
                this._scaledNode2Label.innerHTML = "<label>" + 
                    this.locResources["COMPUTE_NODE_SCALED"].replace("%1", "2") + ":</label>";
                
                if(this.pageParams.formFactor == "8sc") {
                    dojo.style(this._scaledNode3, "display", "");
                    this._scaledNode3Label.innerHTML = "<label>" + 
                        this.locResources["COMPUTE_NODE_SCALED"].replace("%1", "3") + ":</label>";
                    
                    dojo.style(this._scaledNode4, "display", "");
                    this._scaledNode4Label.innerHTML = "<label>" + 
                        this.locResources["COMPUTE_NODE_SCALED"].replace("%1", "4") + ":</label>";
                } else {
                    dojo.style(this._scaledNode3, "display", "none");
                    dojo.style(this._scaledNode4, "display", "none");
                }
            } else {
                dojo.style(this._scaledNode2, "display", "none");
                dojo.style(this._scaledNode3, "display", "none");
                dojo.style(this_scaledNode4, "display", "none");
            }
        },

        _closeGraphicView: function() {
            // tags:
            // protected
            dojo.style(this._graphicViewContainer, "display", "none");
            this._graphicViewIcon.set("title", this.locResources["GRAPHIC_VIEW_OPEN_TITLE"]);
            dojo.style(this._scaledNode2, "display", "none");
            dojo.style(this._scaledNode3, "display", "none");
            dojo.style(this._scaledNode4, "display", "none");
            dojo.style(this["primaryNodeLabel"], "display", "none");
        },

        _getTableLayout: function() {
            // tags:
            // protected

            var layout = [ {
                field: "_item", // Pass entire data item, not just one value.
                name: this.locResources["COLUMN_LOCATION"],
                width: "20em",
                formatter: dojo.hitch(this, this._formatLocation),
                decorator: this._decorateLocation                
            }, {
                field: "_item", // Pass entire data item, not just one value.
                name: this.resources["TYPE"],
                width: "8em",
                formatter: dojo.hitch(this, this._formatType)
            }, {
                field: "moduleBay",
                name: this.locResources["COLUMN_IOMODULE_BAY"],
                width: "5em",
                formatter: this._formatBay
            }, {
                fields: ["type", "template", "parentPattern"],
                name: this.locResources["COLUMN_CONFIG_TEMPLATE"],
                width: "15em",
                formatter: dojo.hitch(this, this._formatConfigPattern) //ForVPort)
            } ];

            // Add Advanced Settings fields, if enabled...
            if (this._advancedSettingButton && this._advancedSettingButton.get("checked")) {
                layout.push({
                    field: "minBandwidth",
                    name: this.locResources["COLUMN_MIN_BANDWIDTH"],
                    width: "7em",
                    formatter: dojo.hitch(this, this._formatMinBandwidth)
                });
                layout.push({
                    field: "maxBandwidth",
                    name: this.locResources["COLUMN_MAX_BANDWIDTH"],
                    width: "7em",
                    formatter: dojo.hitch(this, this._formatMaxBandwidth)
                });
                layout.push({
                    field: "vlan",
                    name: this.locResources["COLUMN_VLAN"],
                    width: "5em",
                    formatter: this._formatVlan
                });
            }

            layout.push({
                fields: ["type", "ioAddressing", "parentPattern"],
                name: this.locResources["COLUMN_IOADDRESSING"],
                width: "10em",
                formatter: dojo.hitch(this, this._formatIOAddressing)
            }, {
                field: "description",
                name: this.resources["DESCRIPTION"],
                width: (this._advancedSettingButton && this._advancedSettingButton.get("checked")) ? "25em" : "auto",
                formatter: this._formatDescription
            });

            return layout;

        },
        
        _addAdapterFromLink: function(item) {
            // tags:
            // protected
            switch (item.slotNumber) {
                case 0:
                    switch (item.type.toString()) {
                        case "addIoAdapter":
                            this._addAdapterSlot0(this._adapterArray[item.nodeNumber*4 - 4], this._adapterArray[item.nodeNumber*4 - 2]);
                            break;
                    }
                    break;
                case 1:
                    switch (item.type.toString()) {
                        case "addIoAdapter":
                            this._addAdapterSlot1(this._adapterArray[item.nodeNumber*4 - 3]);
                            break;
                    }
                    break;
                case 2:
                    switch (item.type.toString()) {
                        case "addIoAdapter":
                            this._addAdapterSlot2(this._adapterArray[item.nodeNumber*4 - 2]);
                            break;
                    }
                    break;
                case 3:
                    switch (item.type.toString()) {
                        case "addIoAdapter":
                            this._addAdapterSlot3(this._adapterArray[item.nodeNumber*4 - 1]);
                            break;
                    }
                    break;
            }
        },        

		_decorateLocation: function(item) {
            // tags:
            // protected
            var hyperlink = "";
            if (item.type.toString() != "addIoAdapter") {
                hyperlink = "<label>" + item.name + "</label>";
            } else {
                hyperlink = "<span title=\"" + item.title + "\" class=\"linkText\">" + item.name + "</span>";
            }
            return hyperlink;
		},
        

        _formatLocation: function(item) {
            // tags:
            // protected
            return item;
        },

        _formatType: function(item) {
            // tags:
            // protected
            var rv = [ "" ];
            
            if (!item.adapterType) {
                return rv.join("");
            }
            
            switch (item.type && item.type.toString()) {
                case "lom":
                case "adapter":
                    switch (item.adapterType.toString()) {
                        case "vfa":
                            rv.push("<span title=\"" + this.resources["VIRTUAL_FABRIC"] + "\" style=\"white-space: nowrap;\">" + this.resources["VIRTUAL_FABRIC"] + "</span>");
                            break;
                        case "ethernet":
                            rv.push("<span title=\"" + this.resources["ETHERNET"] + "\" style=\"white-space: nowrap;\">" + this.resources["ETHERNET"] + "</span>");
                            break;
                        case "fc":
                            rv.push("<span title=\"" + this.resources["FIBRE_CHANNEL"] + "\" style=\"white-space: nowrap;\">" + this.resources["FIBRE_CHANNEL"] + "</span>");
                            break;
                        case "fcoe":
                            rv.push("<span title=\"" + this.resources["FCOE_TITLE"] + "\" style=\"white-space: nowrap;\">" + this.resources["FCOE"] + "</span>");
                            break;
                    }
                    break;
                case "port":
                case "vport":
                    var disabled = (item.disabled && item.disabled[0]);
                    var disabledTitle = disabled ? this.locResources["CHANGE_VIA_PORT_TEMPLATE_DISABLED"].replace("%1", item.parentPattern.toString()) : "";
                    switch (item.portPhysicalFunction.toString()) {
                        case "nic":
                            rv.push("<span title=\"" + (disabled ? disabledTitle : this.resources["ETHERNET"]) + "\" style=\"white-space: nowrap;\">");
                            rv.push("<img src=\"" + this._blankGif + "\" alt=\"\" class=\"" + (disabled ? "spInactiveIcon" : "spEthPortIcon") + "\" align=\"left\" style=\"vertical-align: text-top; margin-right: 4px;\">");
                            rv.push(this.resources["ETHERNET"]);
                            rv.push("</span>");
                            break;
                        case "fc":
                            rv.push("<span title=\"" + (disabled ? disabledTitle : this.resources["FIBRE_CHANNEL"]) + "\" style=\"white-space: nowrap;\">");
                            rv.push("<img src=\"" + this._blankGif + "\" alt=\"\" class=\"" + (disabled ? "spInactiveIcon" : "spFcPortIcon") + "\" align=\"left\" style=\"vertical-align: text-top; margin-right: 4px;\">");
                            rv.push(this.resources["FIBRE_CHANNEL"]);
                            rv.push("</span>");
                            break;
                        case "fcoe":
                            rv.push("<span title=\"" + (disabled ? disabledTitle : this.resources["FCOE_TITLE"]) + "\" style=\"white-space: nowrap;\">");
                            rv.push("<img src=\"" + this._blankGif + "\" alt=\"\" class=\"" + (disabled ? "spInactiveIcon" : "spFcoePortIcon") + "\" align=\"left\" style=\"vertical-align: text-top; margin-right: 4px;\">");
                            rv.push(this.resources["FCOE"]);
                            rv.push("</span>");
                            break;
                        case "iscsi":
                            rv.push("<span title=\"" + (disabled ? disabledTitle : this.resources["ISCSI_TITLE"]) + "\" style=\"white-space: nowrap;\">");
                            rv.push("<img src=\"" + this._blankGif + "\" alt=\"\" class=\"" + (disabled ? "spInactiveIcon" : "spEthPortIcon") + "\" align=\"left\" style=\"vertical-align: text-top; margin-right: 4px;\">");
                            rv.push(this.resources["ISCSI"]);
                            rv.push("</span>");
                            break;
                        case "virtual":
                            var title = this.resources["VIRTUAL_FABRIC"];
                            if (item.portConfigMode == "pNIC") {
                                switch (item.portConfigOptionCode.toString()) {
                                    case "ethOnly":
                                        title = this.locResources["PORT_PNIC_ETHONLY_TITLE"];
                                        break;
                                    case "eth+fcoe":
                                        title = this.locResources["PORT_PNIC_WITH_FCOE_TITLE"];
                                        break;
                                    case "eth+iscsi":
                                        title = this.locResources["PORT_PNIC_WITH_ISCSI_TITLE"];
                                        break;
                                }
                            }
                            else if ((item.portConfigMode == "vNIC_IBMvf") || (item.portConfigMode == "vNIC_independent") || (item.portConfigMode == "vNIC_IBMufp")) {
                                switch (item.portConfigOptionCode.toString()) {
                                    case "ethOnly":
                                        title = this.locResources["PORT_VNIC_ETHONLY_TITLE"];
                                        break;
                                    case "eth+fcoe":
                                        title = this.locResources["PORT_VNIC_WITH_FCOE_TITLE"];
                                        break;
                                    case "eth+iscsi":
                                        title = this.locResources["PORT_VNIC_WITH_ISCSI_TITLE"];
                                        break;
                                }
                            }
                            rv.push("<span title=\"" + title + "\" style=\"white-space: nowrap;\">" + this.resources["VIRTUAL_FABRIC"] + "</span>");
                            break;
                    }
                    break;
            }
            return rv.join("");
        },
        
        _formatConfigPattern: function(item) {
            // tags:
            // protected
            if (!item.template) {
                return;
            }
            
            var rv = [ "" ];
            
            if (item.type.toString() == "vport") {
                rv.push("<i>");
                rv.push("<span title=\"" + this.locResources["CHANGE_VIA_PORT_TEMPLATE"].replace("%1", item.parentPattern.toString()) + "\" style=\"white-space: nowrap;\">" + item.template.toString() + "</span>");
                rv.push("</i>");                
            } else {
                rv.push("<span title=\"" + item.template.replace("<br>", ", ") + "\" style=\"white-space: nowrap;\">" + item.template.toString() + "</span>");   
            }
            
            return rv.join("");
        },
        
        _formatIOAddressing: function(item) {
            // tags:
            // protected
            if (!item.ioAddressing) {
                return;
            }
            
            var rv = [ "" ];
            
            if (item.type.toString() == "vport") {
                rv.push("<i>");
                rv.push("<span title=\"" + this.locResources["CHANGE_VIA_PORT_TEMPLATE"].replace("%1", item.parentPattern.toString()) + "\" style=\"white-space: nowrap;\">" + item.ioAddressing.toString() + "</span>");
                rv.push("</i>");                
            } else {
                rv.push("<span title=\"" + item.ioAddressing.replace("<br>", ", ") + "\" style=\"white-space: nowrap;\">" + item.ioAddressing.toString() + "</span>");   
            }
            
            return rv.join("");
        },        
        
        _formatMinBandwidth: function(item) {
            // tags:
            // protected
            var rv = [ "" ];
            if (item.minBandwidth) {
                var pctg = this.locResources["BANDWIDTH_PERCENTAGE"].replace("%1", item.minBandwidth);
                rv.push("<span title=\"" + pctg + "\" style=\"white-space: nowrap; float:right;\">" + pctg + "</span>");
            }
            return rv.join("");
        },
        
        _formatMaxBandwidth: function(item) {
            // tags:
            // protected
            var rv = [ "" ];
            if (item.maxBandwidth) {
                var pctg = this.locResources["BANDWIDTH_PERCENTAGE"].replace("%1", item.maxBandwidth);
                rv.push("<span title=\"" + pctg + "\" style=\"white-space: nowrap; float:right;\">" + pctg + "</span>");
            }
            return rv.join("");
        },        

        _formatBay: function(item) {
            // tags:
            // protected
            var rv = [ "" ];
            if (item.moduleBay && item.moduleBay !== "") {
                rv.push("<span title=\"" + item.moduleBay + "\" style=\"white-space: nowrap; float:right;\">" + item.moduleBay + "</span>");
            }
            return rv.join("");
        },
        
        _formatVlan: function(item) {
            // tags:
            // protected
            var rv = [ "" ];
            if (item.vlan && item.vlan !== "") {
                rv.push("<span title=\"" + item.vlan + "\" style=\"white-space: nowrap; float:right;\">" + item.vlan + "</span>");
            }
            return rv.join("");
        },        

        _formatDescription: function(item) {
            // tags:
            // protected
            var rv = [ "" ];
            if ((item.description != null) && (item.description !== "")) {
                rv.push("<span title=\"" + item.description.toString() + "\" style=\"white-space: normal;\">" + item.description.toString() + "</span>");
            }
            return rv.join("");
        },
        
        _addAdapterSlot0: function(addItem, removeItem) {
            // tags:
            // protected
            var isScaled = ((this.pageParams.formFactor == "2sc") || (this.pageParams.formFactor == "4sc") || (this.pageParams.formFactor == "8sc"));
            // 4/16/2014: LOM support for 2sc, 4sc and 8sc scaled systems is descoped for GA5.
            // The following line effectively prevents LOM configuration from being an option on Add I/O Adapter dialog for
            // these types of systems. If/When LOM is supported in future simply replace the following with the commented line after it.
            var lomOkay = (!isScaled);
            // var lomOkay = (!isScaled || (removeItem.type[0] == "addIoAdapter")); // Check if 2nd slot is empty for scaled system LOM.
            console.log(this.declaredClass + ": _addAdapterSlot0 called. isScaled=", isScaled, "lomOkay=", lomOkay);
            require(["support/dialogs/AddIOAdapter"], dojo.hitch(this, function() {
                var dialog = new support.dialogs.AddIOAdapter({
                    id: this.id + "cp_nstioadapters_addioadapterslot0",
                    dialogMode:  "add",
                    addType: (lomOkay) ? "lom" : "adapter",
                    nodeType: this.pageParams.nodeType,
                    formFactor: this.pageParams.formFactor,
                    cardNumber: 1
                });
                dialog.startup();
                this._pendingAddItem = addItem;
                this._pendingRemoveItem = (isScaled) ? removeItem : null;
                this._removeTableGuidance();
                dialog.show();
            }));
        },

        _addAdapterSlot1: function(addItem) {
            // tags:
            // protected
            var isScaled = ((this.pageParams.formFactor == "2sc") || (this.pageParams.formFactor == "4sc") || (this.pageParams.formFactor == "8sc"));
            console.log(this.declaredClass + ": _addAdapterSlot1 called. isScaled=", isScaled);
            require(["support/dialogs/AddIOAdapter"], dojo.hitch(this, function() {
                var dialog = new support.dialogs.AddIOAdapter({
                    id: this.id + "cp_nstioadapters_addioadapterslot1",
                    dialogMode:  "add",
                    addType: "adapter",
                    nodeType: this.pageParams.nodeType,
                    formFactor: this.pageParams.formFactor,
                    cardNumber: 2
                });
                dialog.startup();
                this._pendingAddItem = addItem;
                this._pendingRemoveItem = null;
                this._removeTableGuidance();
                dialog.show();
            }));
        },

        _addAdapterSlot2: function(addItem) {
            // tags:
            // protected
            var isScaled = ((this.pageParams.formFactor == "2sc") || (this.pageParams.formFactor == "4sc") || (this.pageParams.formFactor == "8sc"));
            console.log(this.declaredClass + ": _addAdapterSlot2 called. isScaled=", isScaled);
            require(["support/dialogs/AddIOAdapter"], dojo.hitch(this, function() {
                var dialog = new support.dialogs.AddIOAdapter({
                    id: this.id + "cp_nstioadapters_addioadapterslot2",
                    dialogMode:  "add",
                    addType: (this.pageParams.formFactor == "4s") ? "lom" : "adapter",
                    nodeType: this.pageParams.nodeType,
                    formFactor: this.pageParams.formFactor,
                    cardNumber: 3
                });
                dialog.startup();
                this._pendingAddItem = addItem;
                this._pendingRemoveItem = null;
                this._removeTableGuidance();
                dialog.show();
            }));
        },

        _addAdapterSlot3: function(addItem) {
            // tags:
            // protected
            var isScaled = ((this.pageParams.formFactor == "2sc") || (this.pageParams.formFactor == "4sc") || (this.pageParams.formFactor == "8sc"));
            console.log(this.declaredClass + ": _addAdapterSlot3 called. isScaled=", isScaled);
            require(["support/dialogs/AddIOAdapter"], dojo.hitch(this, function() {
                var dialog = new support.dialogs.AddIOAdapter({
                    id: this.id + "cp_nstioadapters_addioadapterslot3",
                    dialogMode:  "add",
                    addType: "adapter",
                    nodeType: this.pageParams.nodeType,
                    formFactor: this.pageParams.formFactor,
                    cardNumber: 4
                });
                dialog.startup();
                this._pendingAddItem = addItem;
                this._pendingRemoveItem = null;
                this._removeTableGuidance();
                dialog.show();
            }));
        },

        _setupAdvancedSettings: function() {
            // tags:
            // protected
            // summary:
            // Reset table layout according to advanced settings option value.
            var newLayout = this._getTableLayout();
            this._adapterTable.setColumns(newLayout);
            this._adapterTable.model.clearCache();
            this._adapterTable.body.refresh();
        },

        _removeSelectedAdapters: function() {
            // tags:
            // protected
            // summary:
            // Remove selected adapter from adapters table.
            console.log(this.declaredClass + ": _removeSelectedAdapters fired!");
            var selectedAdapters = this._getSelectedAdapterRows();
            var commitChange = function() {
                for (var idx in selectedAdapters) {
                    var item = selectedAdapters[idx];
                    switch (item.slotNumber[0]) {
                        case 0:
                            this._removeIoaItem0(item.nodeNumber[0]);
                            this._prepareCardGraphic((item.nodeNumber[0]*4-3), false);
                            break;
                        case 1:
                            this._removeIoaItem1(item.nodeNumber[0]);
                            this._prepareCardGraphic((item.nodeNumber[0]*4-2), false);
                            break;
                        case 2:
                            this._removeIoaItem2(item.nodeNumber[0]);
                            this._prepareCardGraphic((item.nodeNumber[0]*4-1), false);
                            break;
                        case 3:
                            this._removeIoaItem3(item.nodeNumber[0]);
                            this._prepareCardGraphic((item.nodeNumber[0]*4), false);
                            break;
                        default:
                            console.error(this.declaredClass + ": Invalid slot number ", item.slotNumber[0]);
                    }
                    this._setupAllNodeItems(false);
                    
                    // Set this property to indicate a change was made.  
                    // This will force the redeploy dialog on save when editing an in use Server pattern.
                    this.adapterPageChanged = true;
                }
                
                // Re-validate page input.
                this.validate();
                console.log(this.declaredClass + ": Remove adapter handler complete.");

                console.log(this.declaredClass + ": Boot Devices: ", this.getIoBootDevices());
                this.onChangeIoBootDevices(); // call connect handler to notify listeners
                
                // Disable buttons/remove menu items
                this._removeAdapterButton.set("disabled", true);
                this._removeAdapterButton.set("iconClass", "spDeleteInactiveLgIcon");
                this._assignPatternButton.set("disabled", true);
                this._assignPatternButton.set("iconClass", "spEditInactiveLgIcon");
                this.displayMenu.removeChild(this._removeAdapterMenuItem);
                this.displayMenu.removeChild(this._assignPatternItem);

            };
            var adapterList = new Array();
            for (var i in selectedAdapters) {
                var item = selectedAdapters[i];
                adapterList.push(item.name);
            }

            if (this.CP_RUN_ENVIRONMENT == "fsm") {

                require([ "support/temp/ConfirmationDialog" ], dojo.hitch(this, function() {
                    var prompt = new support.temp.ConfirmationDialog({                        
                        buttonLabel: this.resources["REMOVE"],
                        type: "Warning",
                        text: this.locResources["REMOVE_TEXT"],
                        info: "</p>" + this.locResources["REMOVE_INFO"].replace("%1", adapterList.join(", ")) + "</p>"
                    });  
                    
                    prompt.startup();
                    prompt.confirm(commitChange, this);                    
                    
                }));	            
                
            }
            else if (this.CP_RUN_ENVIRONMENT == "augusta") {

                require([ "idx/widget/ConfirmationDialog" ], dojo.hitch(this, function() {
                    var prompt = new idx.widget.ConfirmationDialog({
                        buttonLabel: this.resources["REMOVE"],
                        type: "Warning",
                        text: this.locResources["REMOVE_TEXT"],
                        info: "</p>" + this.locResources["REMOVE_INFO"].replace("%1", adapterList.join(", ")) + "</p>"
                    });
                    
                    prompt.startup();
                    prompt.confirm(commitChange, this);                    
                    
                }));
                
            }


        },

        _editPatternForSelected: function() {
            // tags:
            // protected
            // summary:
            // Detect type of selections made from adapters table and launch the
            // appropriate edit dialog.
            console.log(this.declaredClass + ": _editPatternForSelected fired!");
            
            var selectedPorts = this._getSelectedPortRows();
            if (selectedPorts.length > 0) {
                var initPatternId = selectedPorts[0].templateId.toString();
                var samePattern = dojo.every(selectedPorts, function(item, i) {
                    return (item.templateId == initPatternId);
                });
                console.log(this.declaredClass + ": initPatternId: ", initPatternId, " samePattern: ", samePattern);
                if (!samePattern) {
                    console.info(this.declaredClass + ": Cannot edit multiple patterns at the same time. Must exit.");
                    return;
                }
                
                var templateType = selectedPorts[0].templateType.toString();
                if (templateType == "PortVirtualization") {
                    require(["support/dialogs/NewPortVirtualizationTemplate"], dojo.hitch(this, function() {
                        // Launch dialog to edit the specified PortVirtualization pattern.
                        var dialog = new support.dialogs.NewPortVirtualizationTemplate({
                            id: this.id + "cp_nstioadapters_portvirtualizationdialog1",
                            templateIDNum: initPatternId
                        });
                        dialog.startup();
                        this._pendingEditPatternItems = selectedPorts;
                        dialog.show();
                    }));
                }
                else if (templateType == "ExtendedPort") {
                    require(["support/dialogs/NewLearnedSettingsTemplate"], dojo.hitch(this, function() {
                        // Launch dialog to edit the specified ExtendedPort pattern.
                        var dialog = new support.dialogs.NewLearnedSettingsTemplate({
                            id: this.id + "cp_nstioadapters_learnedsettingsdialog",
                            type: "port",
                            templateIDNum: initPatternId,
                            templateMode: "edit",
                            templateLaunchPoint : "wizard"
                        });
                        dialog.startup();
                        this._pendingEditPatternItems = selectedPorts;
                        dialog.show();
                    }));
                }
                else {
                    console.info(this.declaredClass + ": Unexpected pattern type. Cannot edit pattern type " + templateType + ". Must exit.");
                    return;
                }
            }

            var selectedAdapters = this._getSelectedAdapterRows();
            if (selectedAdapters.length > 0) {
                var initPatternId = selectedAdapters[0].templateId.toString();
                var samePattern = dojo.every(selectedAdapters, function(item, i) {
                    return (item.templateId == initPatternId);
                });
                console.log(this.declaredClass + ": initPatternId: ", initPatternId, " samePattern: ", samePattern);
                if (!samePattern) {
                    console.info(this.declaredClass + ": Cannot edit multiple patterns at the same time. Must exit.");
                    return;
                }
                
                var templateType = selectedAdapters[0].templateType.toString();
                if (templateType == "Adapter") {
                    require(["support/dialogs/NewAdapterTemplate"], dojo.hitch(this, function() {
                        // Launch dialog to edit the specified Adapter pattern.
                        var dialog = new support.dialogs.NewAdapterTemplate({
                            id: this.id + "cp_nstioadapters_newadapterdialog",
                            templateIDNum: initPatternId
                        });
                        dialog.startup();
                        this._pendingEditPatternItems = selectedAdapters;
                        dialog.show();
                    }));
                }
            }

        },

        _createPatternForSelected: function() {
            // tags:
            // protected
            // summary:
            // Detect type of selections made from adapters table and launch the appropriate create dialog.
            console.log(this.declaredClass + ": _createPatternForSelected fired!");
            
            var selectedPorts = this._getSelectedPortRows();
            if (selectedPorts.length > 0) {
                var initAdapterId = selectedPorts[0].adapterId.toString();
                var sameAdapter = dojo.every(selectedPorts, function(item, i) {
                    return (item.adapterId == initAdapterId);
                });
                console.log(this.declaredClass + ": initAdapterId: ", initAdapterId, " sameAdapter: ", sameAdapter);
                if (!sameAdapter) {
                    console.info(this.declaredClass + ": Selection mismatch. Cannot create pattern for uncommon adapters. Must exit.");
                    return;
                }

                var initFunction = selectedPorts[0].portPhysicalFunction.toString();
                if (initFunction == "virtual") {
                    var initMode = selectedPorts[0].portConfigMode.toString();
                    var initOptionCode = selectedPorts[0].portConfigOptionCode.toString();
                    var samePortConfig = dojo.every(selectedPorts, function(item, i) {
                        return ((item.portPhysicalFunction == initFunction) && (item.portConfigMode == initMode) && (item.portConfigOptionCode == initOptionCode));
                    });
                    console.log(this.declaredClass + ": initFunction: ", initFunction, " initMode: ", initMode, " initOptionCode: ", initOptionCode, " samePortConfig: ", samePortConfig);
                    if (samePortConfig) {
                        require(["support/dialogs/NewPortVirtualizationTemplate"], dojo.hitch(this, function() {
                            // Launch dialog for creation of PortVirtualization pattern.
                            // NOTE: Only supported for ports of same adapter type/model, mode, and option code.
                            var dialog = new support.dialogs.NewPortVirtualizationTemplate({
                                id: this.id + "cp_nstioadapters_portvirtualizationdialog2",
                                adapterId: initAdapterId,
                                mode: initMode,
                                protocol: initOptionCode
                            });
                            dialog.startup();
                            this._pendingCreatePatternItems = selectedPorts;
                            dialog.show();
                        }));
                    }
                    else {
                        console.info(this.declaredClass + ": Selection mismatch. Cannot create pattern for virtual ports with unlike configurations. Must exit.");
                        return;
                    }
                }
                else {
                    console.info(this.declaredClass + ": Unexpected function type. Cannot create patterns for " + initFunction + " function. Must exit.");
                    return;
                }
            }
            
            var selectedAdapters = this._getSelectedAdapterRows();
            if (selectedAdapters.length > 0) {
                // 99302: Launch create Adapter pattern dialog
                console.log(this.declaredClass + ": Launch create Adapter pattern dialog");
            }

        },

        _assignPatternForSelected: function() {
            // tags:
            // protected
            // summary:
            // Detect type of selections made from adapters table and launch the appropriate assign dialog.
            console.log(this.declaredClass + ": _assignPatternForSelected fired!");
            
            // Setting this flag to force a redeploy when saving an in use Server pattern.
            // This assumes that if the edit/assign icon is clicked that a change was made.  That may not be
            // true, but it's probably not worth the effort to analyze all possible changes that could be made.
            this.adapterPageChanged = true;
            
            var selectedRows = this._getSelectedRowItems();
            var selectedAdapters = this._getSelectedAdapterRows();
            var selectedPorts = this._getSelectedPortRows();
            var initPatternId = selectedRows[0].templateId.toString();
            var initAdapterId = selectedRows[0].adapterId.toString();
            var initAdapterType = selectedRows[0].adapterType.toString();
            var sameAdapter = dojo.every(selectedRows, function(item, i) {
                return (item.adapterId == initAdapterId);
            });
            console.log(this.declaredClass + ": initAdapterId: ", initAdapterId, " sameAdapter: ", sameAdapter);
            if (!sameAdapter) {
                console.info(this.declaredClass + ": Selection mismatch. Cannot assign pattern for uncommon adapters. Must exit.");
                return;
            }

            if (selectedAdapters.length > 0) {
                console.log(this.declaredClass + ": Launch assign Adapter pattern dialog");
                var initialPortPatternId1 = null;
                var initialPortPatternId2 = null;
                var initialPortExtendedPatternId1 = null;
                var initialPortExtendedPatternId2 = null;
                var grabChildData = function(items) {
                    dojo.forEach(items, function(item, i) {
                        if (item.controllerNumber[0] == 1) {
                            (initialPortPatternId1 == null) && (item.templateType[0] == "PortVirtualization") && (initialPortPatternId1 = item.templateId.toString());
                            (initialPortExtendedPatternId1 == null) && (item.templateType[0] == "ExtendedPort") && (initialPortExtendedPatternId1 = item.templateId.toString());
                        }
                        else if (item.controllerNumber[0] == 2) {
                            (initialPortPatternId2 == null) && (item.templateType[0] == "PortVirtualization") && (initialPortPatternId2 = item.templateId.toString());
                            (initialPortExtendedPatternId2 == null) && (item.templateType[0] == "ExtendedPort") && (initialPortExtendedPatternId2 = item.templateId.toString());
                        }
                    });
                };
                var childItems = this._adapterStore.getChildren(selectedRows[0]);
                grabChildData(childItems);
                require(["support/dialogs/AddIOAdapter"], dojo.hitch(this, function() {
                    var dialog = new support.dialogs.AddIOAdapter({
                        id: this.id + "cp_nstioadapters_addioadapterdialog",
                        dialogMode:  "assign",
                        adapterType:  initAdapterType,
                        adapterId: initAdapterId,
                        adapterPatternId: initPatternId,
                        initialPortPatternId1: initialPortPatternId1,
                        initialPortPatternId2: initialPortPatternId2,
                        initialPortExtendedPatternId1: initialPortExtendedPatternId1,
                        initialPortExtendedPatternId2: initialPortExtendedPatternId2
                    });
                    dialog.startup();
                    this._pendingAssignPatternItems = selectedAdapters;
                    dialog.show();
                }));
            }
            else if (selectedPorts.length > 0) {
                
                var initFunction = selectedPorts[0].portPhysicalFunction.toString();
                if (initFunction == "virtual") {
                    var initMode = selectedPorts[0].portConfigMode.toString();
                    var initOptionCode = selectedPorts[0].portConfigOptionCode.toString();
                    var samePortConfig = dojo.every(selectedPorts, function(item, i) {
                        return ((item.portPhysicalFunction == initFunction) && (item.portConfigMode == initMode) && (item.portConfigOptionCode == initOptionCode));
                    });
                    console.log(this.declaredClass + ": initFunction: ", initFunction, " initMode: ", initMode, " initOptionCode: ", initOptionCode, " samePortConfig: ", samePortConfig);
                    if (samePortConfig) {
                        if (initMode == "pNIC") {
                            var initVendor = selectedPorts[0].controllerVendor.toString();
                            var initChipset = selectedPorts[0].controllerChipset.toString();
                            var initProtocols = selectedPorts[0].portProtocols.toString();
                            console.log(this.declaredClass + ": Launch assign ExtendedPort pattern dialog");
                            require(["support/dialogs/ChooseNewPattern"], dojo.hitch(this, function() {
                                // For "pNIC" mode, launch dialog for assign of ExtendedPort pattern.
                                var dialog = new support.dialogs.ChooseNewPattern({
                                    id: this.id + "cp_nstioadapters_choosenewportextended1",
                                    templateIDNum: initPatternId,
                                    templateType: "portextended",
                                    vendor: initVendor,
                                    chipset: initChipset,
                                    protocol: initProtocols
                                });
                                dialog.startup();
                                this._pendingAssignPatternItems = selectedPorts;
                                dialog.show();
                            }));
                        }
                        else {
                            console.log(this.declaredClass + ": Launch assign PortVirtualization pattern dialog");
                            require(["support/dialogs/ChooseNewPattern"], dojo.hitch(this, function() {
                                // Launch dialog for assign of PortVirtualization pattern.
                                var dialog = new support.dialogs.ChooseNewPattern({
                                    id: this.id + "cp_nstioadapters_choosenewportvirt1",
                                    templateIDNum: initPatternId,
                                    templateType: "portvirt",
                                    adapterId: initAdapterId,
                                    portCfgMode: initMode,
                                    portCfgOptionCode: initOptionCode
                                });
                                dialog.startup();
                                this._pendingAssignPatternItems = selectedPorts;
                                dialog.show();
                            }));
                        }
                    }
                    else {
                        console.info(this.declaredClass + ": Selection mismatch. Cannot assign pattern for virtual ports with unlike configurations. Must exit.");
                        return;
                    }
                }
                else {
                    var initVendor = selectedPorts[0].controllerVendor.toString();
                    var initChipset = selectedPorts[0].controllerChipset.toString();
                    var initProtocols = selectedPorts[0].portProtocols.toString();
                    var noVirtual = dojo.every(selectedPorts, function(item, i) {
                        return (item.portPhysicalFunction != "virtual");
                    });
                    console.log(this.declaredClass + ": initFunction: ", initFunction, " noVirtual: ", noVirtual);
                    // Launch dialog for assign of ExtendedPort pattern.
                    if (noVirtual) {
                        require(["support/dialogs/ChooseNewPattern"], dojo.hitch(this, function() {
                            var dialog = new support.dialogs.ChooseNewPattern({
                                id: this.id + "cp_nstioadapters_choosenewportextended2",
                                templateIDNum: initPatternId,
                                templateType: "portextended",
                                vendor: initVendor,
                                chipset: initChipset,
                                protocol: initProtocols
                            });
                            dialog.startup();
                            this._pendingAssignPatternItems = selectedPorts;
                            dialog.show();
                        }));
                    }
                    else {
                        console.info(this.declaredClass + ": Selection mismatch. Cannot assign pattern for virtual and non-virtual ports alike. Must exit.");
                        return;
                    }
                }
            }

        },

        _prepareToolbar: function(grid) {
            // tags:
            // protected
            // summary:
            // Customize the toolbar for the adapters table.

            var toolbar = grid && grid.toolBar && grid.toolBar.widget;
            
            if (toolbar) {
                // Add custom Collapse All button...
                var collapseAllButton = new dijit.form.Button({
                    id: this.id + "_collapseallbutton",
                    showLabel: false,
                    iconClass: "spCollapseAllIcon",
                    title: this.resources.COLLAPSE_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {this._adapterTable.collapseAll();}),
                    onKeypress: dojo.hitch(this, function() {this._adapterTable.collapseAll();})
                });
                toolbar.addChild(collapseAllButton);
    
                // Add custom Expand All button...
                var expandAllButton = new dijit.form.Button({
                    id: this.id + "_expandallbutton",                      
                    showLabel: false,
                    iconClass: "spExpandAllIcon",
                    title: this.resources.EXPAND_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {this._adapterTable.expandAll();}),
                    onKeypress: dojo.hitch(this, function() {this._adapterTable.expandAll();})
                });
                toolbar.addChild(expandAllButton);
    
                toolbar.addChild(new dijit.ToolbarSeparator({}));
    
                // Add custom Advanced Settings button...
                this._advancedSettingButton = new support.panels.CheckButton({
                    id: this.id + "_advancedsettingscheckbutton",
                    label: this.locResources["BUTTON_ADV_SETTINGS"],
                    title: this.locResources["BUTTON_ADV_SETTINGS_TITLE"],
                    onClick: dojo.hitch(this, this._setupAdvancedSettings),
                    onKeypress: dojo.hitch(this, this._setupAdvancedSettings)
                });
                toolbar.addChild(this._advancedSettingButton);
    
                toolbar.addChild(new dijit.ToolbarSeparator({}));
    
                // Add custom Create Pattern button...
                this._createPatternButton = new dijit.form.Button({
                    //id: "cp_nstioadapters_createpatternbutton",
                    showLabel: false,
                    iconClass: "spCreateLgIcon",
                    title: this.locResources["BUTTON_CREATE_TITLE"],
                    onClick: dojo.hitch(this, this._createPatternForSelected),
                    onKeypress: dojo.hitch(this, this._createPatternForSelected)
                });
                // Related to 99302?  If not need another task.: Uncomment when create patterns are supported.
                // toolbar.addChild(this._createPatternButton);
    
                // Add custom Advanced Settings button...
                // Merge edit and assign functions.  Edit icon will now launch Edit/Assign dialog.
                this._assignPatternButton = new dijit.form.Button({
                    id: this.id + "_assignpatternbutton",
                    showLabel: false,
                    iconClass: "spEditLgIcon",                
                    title: this.locResources["BUTTON_EDIT_TITLE"],
                    onClick: dojo.hitch(this, this._assignPatternForSelected),
                    onKeypress: dojo.hitch(this, this._assignPatternForSelected)
                });
                toolbar.addChild(this._assignPatternButton);
                
                this._removeAdapterButton = new dijit.form.Button({
                    id: this.id + "_removeadapterbutton",
                    showLabel: false,
                    iconClass: "spDeleteLgIcon",
                    title: this.locResources["MENU_ITEM_REMOVE_TITLE"],
                    onClick: dojo.hitch(this, this._removeSelectedAdapters),
                    onKeypress: dojo.hitch(this, this._removeSelectedAdapters)
                });
                toolbar.addChild(this._removeAdapterButton);
    
                toolbar.addChild(new dijit.ToolbarSeparator({}));
    
                // Set up Action List items.
                // Add custom Remove Adapter menu item...
                this.displayMenu = new dijit.Menu({
                    style: "display: none;"
                });
                
                this._removeAdapterMenuItem = new dijit.MenuItem({
                    id: this.id + "_removeadaptermenuitem",
                    label: this.locResources["MENU_ITEM_REMOVE"],
                    title: this.locResources["MENU_ITEM_REMOVE_TITLE"],
                    iconClass: "spDeleteLgIcon",
                    onClick: dojo.hitch(this, this._removeSelectedAdapters),
                    onKeypress: dojo.hitch(this, this._removeSelectedAdapters)
                });                
                
                this._assignPatternItem = new dijit.MenuItem({
                    id: this.id + "_assignpatternitem",
                    label: this.locResources["EDIT_ASSIGN"],
                    title: this.locResources["BUTTON_EDIT_TITLE"],
                    iconClass: "spEditLgIcon",
                    onClick: dojo.hitch(this, this._assignPatternForSelected),
                    onKeypress: dojo.hitch(this, this._assignPatternForSelected)
                });
                
                this._expandAllItem = new dijit.MenuItem({
                    id: this.id + "_expandallitem",
                    label: this.resources.EXPAND_ALL,
                    iconClass: "spExpandAllLgIcon",
                    title: this.resources.EXPAND_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {this._adapterTable.expandAll();}),
                    onKeypress: dojo.hitch(this, function() {this._adapterTable.expandAll();})
                });
                this.displayMenu.addChild(this._expandAllItem,0);
                
                this._collapseAllItem = new dijit.MenuItem({
                    id: this.id + "_collapseallitem",
                    label: this.resources.COLLAPSE_ALL,
                    iconClass: "spCollapseAllLgIcon",
                    title: this.resources.COLLAPSE_ALL_TITLE,
                    onClick: dojo.hitch(this, function() {this._adapterTable.collapseAll();}),
                    onKeypress: dojo.hitch(this, function() {this._adapterTable.collapseAll();})
                });
                this.displayMenu.addChild(this._collapseAllItem,1);
                
                this._moreMenu = new dijit.form.DropDownButton({
                    id: this.id + "_moremenudropdownbutton",
                    label: this.resources.ALL_ACTIONS,
                    title: this.resources.ALL_ACTIONS_TITLE,
                    showLabel: true,
                    dropDown: this.displayMenu
                });
                
                toolbar.addChild(this._moreMenu);
            }
        },

        _getSelectedRowItems: function() {
            // tags:
            // protected
            var selectedRowIds = this._adapterTable.select.row.getSelected();
            var selectedRowItems = [];

            dojo.forEach(selectedRowIds, dojo.hitch(this, function(id) {
                selectedRowItems.push(this._adapterTable.row(id).item());
            }));
            
            return selectedRowItems;
        },
        
        _setToolbarCapabilities: function() {
            // tags:
            // protected
            // summary:
            // Enable/Disable toolbar capabilities for the adapters table.

            var enableCreate = false;
            var enableAssign = false;
            var enableRemove = false;           
            var selectedRows = this._getSelectedRowItems();
            var selectedAdapters = this._getSelectedAdapterRows();
            var selectedPorts = this._getSelectedPortRows();
            
            var unknownSelected = (selectedRows.length == 0) || (selectedRows.length > (selectedAdapters.length + selectedPorts.length));
            if (!unknownSelected) {
                // Determine if "Create" and/or "Assign" buttons should be enabled.
                var initAdapterId = (selectedRows.length > 0) ? selectedRows[0].adapterId.toString() : "*NONE";
                var sameAdapter = (initAdapterId != "*NONE") && dojo.every(selectedRows, function(item, i) {
                    return (item.adapterId == initAdapterId);
                });
                console.log(this.declaredClass + ": initAdapterId: ", initAdapterId, " sameAdapter: ", sameAdapter);
                if (sameAdapter) {
                    var initFunction = (selectedPorts.length > 0) ? selectedPorts[0].portPhysicalFunction.toString() : "*NONE";
                    var allVirtual = (initFunction == "virtual") && dojo.every(selectedPorts, function(item, i) {
                        return (item.portPhysicalFunction == initFunction);
                    });
                    var noVirtual = dojo.every(selectedPorts, function(item, i) {
                        return (item.portPhysicalFunction != "virtual");
                    });
                    console.log(this.declaredClass + ": initFunction: ", initFunction, " allVirtual: ", allVirtual, " noVirtual: ", noVirtual);
                    if (allVirtual) {
                        var initMode = selectedPorts[0].portConfigMode.toString();
                        var initOptionCode = selectedPorts[0].portConfigOptionCode.toString();
                        var samePortConfig = dojo.every(selectedPorts, function(item, i) {
                            return ((item.portConfigMode == initMode) && (item.portConfigOptionCode == initOptionCode));
                        });
                        console.log(this.declaredClass + ": initMode: ", initMode, " initOptionCode: ", initOptionCode, " samePortConfig: ", samePortConfig);
                        enableCreate = ((selectedAdapters.length == 0) && samePortConfig);
                        enableAssign = ((selectedAdapters.length == 0) && samePortConfig);
                    }
                    else if (noVirtual) {
                        var initAdapterType = (selectedAdapters.length > 0) ? selectedAdapters[0].adapterType.toString() : "*NONE";
                        enableAssign = ((selectedAdapters.length == 0) || (initAdapterType == "vfa"));
                    }
                }
                
                // Determine if "Remove" adapter option should be enabled.
                enableRemove = ((selectedPorts.length == 0) && (selectedAdapters.length > 0));
            }
            
            this._createPatternButton.set("disabled", !enableCreate);
            this._createPatternButton.set("iconClass", enableCreate ? "spCreateLgIcon" : "spCreateInactiveLgIcon");
            this._assignPatternButton.set("disabled", !enableAssign);
            this._assignPatternButton.set("iconClass", enableAssign ? "spEditLgIcon" : "spEditInactiveLgIcon");
            if (enableAssign) {
                this.displayMenu.addChild(this._assignPatternItem);
            } else {
                this.displayMenu.removeChild(this._assignPatternItem);
            }
            this._removeAdapterButton.set("disabled", !enableRemove);
            this._removeAdapterButton.set("iconClass", enableRemove ? "spDeleteLgIcon" : "spDeleteInactiveLgIcon");
            if (enableRemove) {
                this.displayMenu.addChild(this._removeAdapterMenuItem);
            } else {
                this.displayMenu.removeChild(this._removeAdapterMenuItem);
            }
        },

        _setupAllNodeItems: function(/*boolean*/ reset) {
            // tags:
            // protected
            this._setupScaledNodeItems(reset);
            this._setupExpansionUnitItems(reset);
            this._setupComputeNodeItems(reset); // NOTE: Must be called last since it refreshes the grid!
        },

        _setupComputeNodeItems: function(/*boolean*/ reset) {
            // tags:
            // protected

            // Remove adapters if "reset" flagged or not supported for the node type.
            dojo.forEach(this._computeNodeItem.items, dojo.hitch(this, function(item) {
                if (reset || item.supportedNodeTypes && (dojo.indexOf(item.supportedNodeTypes, this.pageParams.nodeType) < 0)) {
                    switch (item.slotNumber[0]) {
                        case 0:
                            this._removeIoaItem0(1);
                            this._prepareCardGraphic(1, false);
                            break;
                        case 1:
                            this._removeIoaItem1(1);
                            this._prepareCardGraphic(2, false);
                            break;
                        case 2:
                            this._removeIoaItem2(1);
                            this._prepareCardGraphic(3, false);
                            break;
                        case 3:
                            this._removeIoaItem3(1);
                            this._prepareCardGraphic(4, false);
                            break;
                        default:
                            console.error(this.declaredClass + ": Invalid slot number ", item.slotNumber[0]);
                    }
                }
            }));

            var description = [ "" ];
            //var slot0Full = dojo.some(this._computeNodeItem.children, function(item) {
            var slot0Full = dojo.some(this._computeNodeItem.items, function(item) {
                return (item.slotNumber[0] == 0);
            });
            //var slot1Full = dojo.some(this._computeNodeItem.children, function(item) {
            var slot1Full = dojo.some(this._computeNodeItem.items, function(item) {
                return (item.slotNumber[0] == 1);
            });
            //var slot2Full = dojo.some(this._computeNodeItem.children, function(item) {
            var slot2Full = dojo.some(this._computeNodeItem.items, function(item) {
                return (item.slotNumber[0] == 2);
            });
            //var slot3Full = dojo.some(this._computeNodeItem.children, function(item) {
            var slot3Full = dojo.some(this._computeNodeItem.items, function(item) {
                return (item.slotNumber[0] == 3);
            });
            switch (this.pageParams.nodeType) {
                case "sysx":
                    description.push(this.resources["SYSX"]);
                    break;
                case "power":
                    description.push(this.resources["POWER"]);
                    break;
            }
            switch (this.pageParams.formFactor) {
                case "2s":
                    description.push("&ndash; " + this.locResources["FORM_FACTOR_2S"]);
                    if (!slot0Full) {
                        var setupComplete = this._setupIoaItem0(this._computeNodeItem, true, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 0 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if ((adapterData.type == "lom" || adapterData.type == "adapter")  && (dojo.indexOf(adapterData.supportedFormFactors, "2s") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[0];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot1Full) {
                        var setupComplete = this._setupIoaItem1(this._computeNodeItem, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 1 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "2s") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[1];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    this._removeIoaItem2(1);
                    this._removeIoaItem3(1);
                    this._prepareCardGraphic(3, false);
                    this._prepareCardGraphic(4, false);
                    break;
                case "pxu":
                    description.push("&ndash; " + this.locResources["FORM_FACTOR_PXU"]);
                    if (!slot0Full) {
                        var setupComplete = this._setupIoaItem0(this._computeNodeItem, true, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 0 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if ((adapterData.type == "lom" || adapterData.type == "adapter") && (dojo.indexOf(adapterData.supportedFormFactors,"pxu") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[0];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot1Full) {
                        var setupComplete = this._setupIoaItem1(this._computeNodeItem, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 1 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "pxu") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[1];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    break;
                case "1sd":
                    description.push("&ndash; " + this.locResources["FORM_FACTOR_1SD"]);
                    if (!slot0Full) {
                        var setupComplete = this._setupIoaItem0(this._computeNodeItem, false, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 0 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if ((adapterData.type == "lom") && (dojo.indexOf(adapterData.supportedFormFactors, "1sd") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[0];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot1Full) {
                        var setupComplete = this._setupIoaItem1(this._computeNodeItem, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 1 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && dojo.indexOf(adapterData.supportedFormFactors, "1sd") >= 0) {
                                    this._pendingAddItem = this._adapterArray[1];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    this._prepareCardGraphic(3, false);
                    this._prepareCardGraphic(4, false);
                    this._removeIoaItem2(1);
                    this._removeIoaItem3(1);
                    break;
                case "4s":
                    description.push("&ndash; " + this.locResources["FORM_FACTOR_4S"]);
                    if (!slot0Full) {
                        var setupComplete = this._setupIoaItem0(this._computeNodeItem, true, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 0 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if ((adapterData.type == "lom" || adapterData.type == "adapter") && (dojo.indexOf(adapterData.supportedFormFactors, "4s") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[0];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot1Full) {
                        var setupComplete = this._setupIoaItem1(this._computeNodeItem, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 1 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "4s") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[1];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot2Full) {
                        var setupComplete = this._setupIoaItem2(this._computeNodeItem, true, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 2 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if ((adapterData.type == "lom" || adapterData.type == "adapter") && (dojo.indexOf(adapterData.supportedFormFactors, "4s") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[2];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot3Full) {
                        var setupComplete = this._setupIoaItem3(this._computeNodeItem, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 3 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "4s") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[3];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    break;
                case "2sc":
                    description.push("&ndash; " + this.locResources["FORM_FACTOR_2SC"]);
                    if (!slot0Full) {
                        // 4/16/2014: LOM support for 2sc, 4sc and 8sc scaled systems is descoped for GA5.
                        // If/When LOM is supported in future change the second param in following line to true.
                        var setupComplete = this._setupIoaItem0(this._computeNodeItem, false, 1); 
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 0 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if ((adapterData.type == "lom" || adapterData.type == "adapter") && (dojo.indexOf(adapterData.supportedFormFactors, "2sc") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[0];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot1Full) {
                        var setupComplete = this._setupIoaItem1(this._computeNodeItem, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 1 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "2sc") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[1];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot2Full) {
                        var setupComplete = this._setupIoaItem2(this._computeNodeItem, false, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 2 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "2sc") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[2];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot3Full) {
                        var setupComplete = this._setupIoaItem3(this._computeNodeItem, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 3 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "2sc") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[3];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    break;
                case "4sc":
                    description.push("&ndash; " + this.locResources["FORM_FACTOR_2SC"]);
                    if (!slot0Full) {
                        // 4/16/2014: LOM support for 2sc, 4sc and 8sc scaled systems is descoped for GA5.
                        // If/When LOM is supported in future change the second param in following line to true.
                        var setupComplete = this._setupIoaItem0(this._computeNodeItem, false, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 0 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if ((adapterData.type == "lom" || adapterData.type == "adapter") && (dojo.indexOf(adapterData.supportedFormFactors, "4sc") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[0];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot1Full) {
                        var setupComplete = this._setupIoaItem1(this._computeNodeItem, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 1 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "4sc") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[1];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot2Full) {
                        var setupComplete = this._setupIoaItem2(this._computeNodeItem, false, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 2 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "4sc") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[2];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot3Full) {
                        var setupComplete = this._setupIoaItem3(this._computeNodeItem, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 3 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "4sc") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[3];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    break;
                case "8sc":
                    description.push("&ndash; " + this.locResources["FORM_FACTOR_2SC"]);
                    if (!slot0Full) {
                        // 4/16/2014: LOM support for 2sc, 4sc and 8sc scaled systems is descoped for GA5.
                        // If/When LOM is supported in future change the second param in following line to true.
                        var setupComplete = this._setupIoaItem0(this._computeNodeItem, false, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 0 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if ((adapterData.type == "lom" || adapterData.type == "adapter") && (dojo.indexOf(adapterData.supportedFormFactors, "8sc") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[0];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot1Full) {
                        var setupComplete = this._setupIoaItem1(this._computeNodeItem, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 1 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "8sc") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[1];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot2Full) {
                        var setupComplete = this._setupIoaItem2(this._computeNodeItem, false, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 2 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "8sc") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[2];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot3Full) {
                        var setupComplete = this._setupIoaItem3(this._computeNodeItem, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 3 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "8sc") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[3];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    break;
            }
            this._setStoreToEmpty();
            this._adapterStore.setValue(this._computeNodeItem, "description", description.join(" "));
            var isScaled = ((this.pageParams.formFactor == "2sc") || (this.pageParams.formFactor == "4sc") || (this.pageParams.formFactor == "8sc"));
            var computeNodeName = isScaled ? this.locResources.COMPUTE_NODE_SCALED_PRIMARY : this.locResources.COMPUTE_NODE_ITEM_NAME;
            this._adapterStore.setValue(this._computeNodeItem, "name", computeNodeName);
            this._updateGridStore();
        },

        _setupExpansionUnitItems: function(/*boolean*/ reset) {
            // tags:
            // protected
            switch (this.pageParams.formFactor) {
                case "1sd":
                case "2s":
                case "2sc":
                case "4sc":
                case "8sc":
                case "4s":
                    this._removeExpansionUnit(); // Removes children too, if any.
                    break;
                case "pxu":
                    var parentItem = this._getExpansionUnitItem();
                    var slot2Full = dojo.some(parentItem.items, function(item) {
                        return (item.slotNumber[0] == 2);
                    });
                    var slot3Full = dojo.some(parentItem.items, function(item) {
                        return (item.slotNumber[0] == 3);
                    });
                    if (!slot2Full) {
                        var setupComplete = this._setupIoaItem2(parentItem, false, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 2 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "pxu") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[2];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot3Full) {
                        var setupComplete = this._setupIoaItem3(parentItem, 1);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 3 && item.nodeNumber == 1);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "pxu") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[3];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    break;
            }

        },

        _getExpansionUnitItem: function() {
            // tags:
            // protected
            if (!this._expansionUnitItem) {
                this._expansionUnitItem = this._adapterStore.newItem({  //this._adapterTable.store.newItem({
                    id: "item" + this._itemCount++,
                    name: this.locResources["EXPANSION_UNIT_ITEM_NAME"],
                    type: "root",
                    description: this.locResources["EXPANSION_UNIT_ITEM_DESCRIPTION"]
                });
                this._updateGridStore();
            }
            return this._expansionUnitItem;
        },

        _removeExpansionUnit: function() {
            // tags:
            // protected
            if (this._expansionUnitItem) {
                this._deleteItem(this._expansionUnitItem);
                this._expansionUnitItem = null;
            }
        },
        
        _setupScaledNodeItems: function(/*boolean*/ reset) {
            // tags:
            // protected
            switch (this.pageParams.formFactor) {
             
                case "1sd":
                case "2s":
                case "pxu":
                case "4s":
                    this._removeScaledNodeItem(); // Removes children too, if any.
                    break;
                case "4sc":
                    if (reset) {
                        this._removeScaledNodeItem();
                        this._adapterArray[4] = null;
                        this._adapterArray[5] = null;
                        this._adapterArray[6] = null;
                        this._adapterArray[7] = null;
                        this._adapterStore.save();
                    }
                    var parentItem = this._getScaledNodeItem(2);
                    var slot0Full = dojo.some(parentItem.items, function(item) {
                        return (item.slotNumber[0] == 0);
                    });
                    var slot1Full = dojo.some(parentItem.items, function(item) {
                        return (item.slotNumber[0] == 1);
                    });
                    var slot2Full = dojo.some(parentItem.items, function(item) {
                        return (item.slotNumber[0] == 2);
                    });
                    var slot3Full = dojo.some(parentItem.items, function(item) {
                        return (item.slotNumber[0] == 3);
                    });
                    if (!slot0Full) {
                        // 4/16/2014: LOM support for 2sc, 4sc and 8sc scaled systems is descoped for GA5.
                        // If/When LOM is supported in future change the second param in following line to true.
                        var setupComplete = this._setupIoaItem0(parentItem, false, 2);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 0 && item.nodeNumber == 2);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if ((adapterData.type == "lom" || adapterData.type == "adapter") && (dojo.indexOf(adapterData.supportedFormFactors, "4sc") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[4];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot1Full) {
                        var setupComplete = this._setupIoaItem1(parentItem, 2);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 1 && item.nodeNumber == 2);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "4sc") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[5];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot2Full) {
                        var setupComplete = this._setupIoaItem2(parentItem, false, 2);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 2 && item.nodeNumber == 2);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "4sc") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[6];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    if (!slot3Full) {
                        var setupComplete = this._setupIoaItem3(parentItem, 2);
                        if (setupComplete && reset) {
                            var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                return (item.slotNumber == 3 && item.nodeNumber == 2);
                            });
                            if (matches.length > 0) {
                                var userData = matches[0];
                                var adapterData = this._getAdapterData(userData.adapterId);
                                if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "4sc") >= 0)) {
                                    this._pendingAddItem = this._adapterArray[7];
                                    this._addAdapterItem(adapterData, userData);
                                }
                            }
                        }
                    }
                    break;
                case "8sc":
                    for(var i=2; i<5; i++) {
                    	if (reset) {
                	        this._removeScaledNodeItem();
            	            this._adapterArray[i*4-4] = null;
        	                this._adapterArray[i*4-3] = null;
    	                    this._adapterArray[i*4-2] = null;
	                        this._adapterArray[i*4-1] = null;
                        	this._adapterStore.save();
                    	}
                        var parentItem = this._getScaledNodeItem(i);
                        var slot0Full = dojo.some(parentItem.items, function(item) {
                            return (item.slotNumber[0] == 0);
                        });
                        var slot1Full = dojo.some(parentItem.items, function(item) {
                            return (item.slotNumber[0] == 1);
                        });
                        var slot2Full = dojo.some(parentItem.items, function(item) {
                            return (item.slotNumber[0] == 2);
                        });
                        var slot3Full = dojo.some(parentItem.items, function(item) {
                            return (item.slotNumber[0] == 3);
                        });
                        if (!slot0Full) {
                            // 4/16/2014: LOM support for 2sc, 4sc and 8sc scaled systems is descoped for GA5.
                            // If/When LOM is supported in future change the second param in following line to true.
                            var setupComplete = this._setupIoaItem0(parentItem, false, i);
                            if (setupComplete && reset) {
                                var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                    return (item.slotNumber == 0 && item.nodeNumber == i);
                                });
                                if (matches.length > 0) {
                                    var userData = matches[0];
                                    var adapterData = this._getAdapterData(userData.adapterId);
                                    if ((adapterData.type == "lom" || adapterData.type == "adapter") && (dojo.indexOf(adapterData.supportedFormFactors, "8sc") >= 0)) {
                                        this._pendingAddItem = this._adapterArray[i*4-4];
                                        this._addAdapterItem(adapterData, userData);
                                    }
                                }
                            }
                        }
                        if (!slot1Full) {
                            var setupComplete = this._setupIoaItem1(parentItem, i);
                            if (setupComplete && reset) {
                                var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                    return (item.slotNumber == 1 && item.nodeNumber == i);
                                });
                                if (matches.length > 0) {
                                    var userData = matches[0];
                                    var adapterData = this._getAdapterData(userData.adapterId);
                                    if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "8sc") >= 0)) {
                                        this._pendingAddItem = this._adapterArray[i*4-3];
                                        this._addAdapterItem(adapterData, userData);
                                    }
                                }
                            }
                        }
                        if (!slot2Full) {
                            var setupComplete = this._setupIoaItem2(parentItem, false, i);
                            if (setupComplete && reset) {
                                var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                    return (item.slotNumber == 2 && item.nodeNumber == i);
                                });
                                if (matches.length > 0) {
                                    var userData = matches[0];
                                    var adapterData = this._getAdapterData(userData.adapterId);
                                    if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "8sc") >= 0)) {
                                        this._pendingAddItem = this._adapterArray[i*4-2];
                                        this._addAdapterItem(adapterData, userData);
                                    }
                                }
                            }
                        }
                        if (!slot3Full) {
                            var setupComplete = this._setupIoaItem3(parentItem, i);
                            if (setupComplete && reset) {
                                var matches = dojo.filter(this.pageParams.adapterSettings.adapters, function(item) {
                                    return (item.slotNumber == 3 && item.nodeNumber == i);
                                });
                                if (matches.length > 0) {
                                    var userData = matches[0];
                                    var adapterData = this._getAdapterData(userData.adapterId);
                                    if (adapterData.type == "adapter" && (dojo.indexOf(adapterData.supportedFormFactors, "8sc") >= 0)) {
                                        this._pendingAddItem = this._adapterArray[i*4-1];
                                        this._addAdapterItem(adapterData, userData);
                                    }
                                }
                            }
                        }
                    }
                    break;
            }

        },
        
        _getScaledNodeItem: function(nodeNumber) {
            // tags:
            // protected
            if (this._scaledNodeItem[nodeNumber-2] == null) {
                var scaledDesc = this.resources["SYSX"] + " &ndash; " + this.locResources["FORM_FACTOR_2SC"];
                this._scaledNodeItem[nodeNumber-2] = this._adapterStore.newItem({  //this._adapterTable.store.newItem({
                    id: "item_scaled_" + nodeNumber,
                    name: this.locResources["COMPUTE_NODE_SCALED"].replace("%1", nodeNumber),
                    type: "root",
                    nodeNumber: nodeNumber,
                    description: scaledDesc
                });
                this._updateGridStore();
            }
            return this._scaledNodeItem[nodeNumber-2];
        },

        _removeScaledNodeItem: function() {
            // tags:
            // protected
            for(var i=0; i < this._scaledNodeItem.length; i++) {
                if (this._scaledNodeItem[i] != null) {
                    dojo.forEach(this._scaledNodeItem[i].items, dojo.hitch(this, function(item) {
                        this._deleteItem(item);
                    }));
                    this._deleteItem(this._scaledNodeItem[i]);
                    this._scaledNodeItem[i] = null;
                }
            }
        },

        _setupIoaItem0: function(parentItem, /*boolean*/either, nodeNumber) {
            // tags:
            // protected
            var location = parseInt(nodeNumber)*4 - 4;
            if (this._adapterArray[location] != null) {
                // Verify its under the correct parent.
                var myId = this._adapterArray[location].id;
                if (dojo.some(parentItem.items, function(item) {
                    return item.id == myId;
                })) {
                    return true; // assume all is good
                }
                this._removeIoaItem0(nodeNumber); // Remove now to create under correct parent below.
            }
            var is1sd = this.pageParams.formFactor == "1sd";
            // Set up "Add button" for IO Adapter 1
            var rowData = {
                id: "item" + this._itemCount++,
                slotNumber: 0,
                nodeNumber: nodeNumber,
                type: "addIoAdapter",
                name: (is1sd) ? this.locResources["BUTTON_ADD_LOM"].replace("%1", "") : ((either) ? this.locResources["BUTTON_ADD_ADAPTER_OR_LOM"].replace("%1", "1") : this.locResources["BUTTON_ADD_ADAPTER"].replace("%1", "1")),
                title: (is1sd) ? this.locResources["BUTTON_ADD_LOM_1SD"] : ((either) ? this.locResources["BUTTON_ADD_ADAPTER1_OR_LOM_TITLE"] : this.locResources["BUTTON_ADD_ADAPTER1_TITLE"]),
                moduleBay: "1-2",
                description: this.locResources["EMPTY_SLOT_DESCRIPTION"]
            };
            var parentInfo = {
                parent: parentItem,
                attribute: this._childAttr
            };
            this._adapterArray[location] = this._adapterStore.newItem(rowData, parentInfo);  //this._adapterTable.store.newItem(rowData, parentInfo);
            this._updateGridStore();                
            this._prepareCardGraphic(location+1, false);
            return true;
        },

        _removeIoaItem0: function(nodeNumber) {
            // tags:
            // protected
            var location = parseInt(nodeNumber)*4 - 4;
            if (this._adapterArray[location] != null) {
                this._deleteItem(this._adapterArray[location]);                
                this._adapterArray[location] = null;
            }
        },

        _setupIoaItem1: function(parentItem, nodeNumber) {
            // tags:
            // protected
            var location = parseInt(nodeNumber)*4 - 3;
            if (this._adapterArray[location] != null) { // tags:
                // Verify its under the correct parent.
                var myId = this._adapterArray[location].id;
                if (dojo.some(parentItem.items, function(item) {
                    return item.id == myId;
                })) {
                    return true; // assume all is good
                }
                this._removeIoaItem1(nodeNumber); // Remove now to create under correct parent below.
            }
            var is1sd = this.pageParams.formFactor == "1sd";
                // Set up "Add button" for IO Adapter 2
            var rowData = {
                id: "item" + this._itemCount++,
                slotNumber: 1,
                nodeNumber: nodeNumber,
                type: "addIoAdapter",
                name: (is1sd) ? this.locResources["BUTTON_ADD_ADAPTER"].replace("%1", "") : this.locResources["BUTTON_ADD_ADAPTER"].replace("%1", "2"),
                title: (is1sd) ? this.locResources["BUTTON_ADD_ADAPTER_1SD"] : this.locResources["BUTTON_ADD_ADAPTER2_TITLE"],
                moduleBay: "3-4",
                description: this.locResources["EMPTY_SLOT_DESCRIPTION"]
            };
            var parentInfo = {
                parent: parentItem,
                attribute: this._childAttr
            };
            this._adapterArray[location] = this._adapterStore.newItem(rowData, parentInfo);
            this._updateGridStore();
            this._prepareCardGraphic(location+1, false);
            return true;
        },

        _removeIoaItem1: function(nodeNumber) {
            // tags:
            // protected
            var location = parseInt(nodeNumber)*4 - 3;
            if (this._adapterArray[location] != null) {
                this._deleteItem(this._adapterArray[location]);                
                this._adapterArray[location] = null;
            }
        },

        _setupIoaItem2: function(parentItem, /*boolean*/either, nodeNumber) {
            // tags:
            // protected

            // 4/14/2014: Wade Fode
            // LOMs for scaled systems consume TWO mezz slots - Slot 0 and Slot 2. If a LOM configuration was
            // successfully added for a scaled system pattern then omit setup for Slot 2 and just return.
            var isScaled = ((this.pageParams.formFactor == "2sc") || (this.pageParams.formFactor == "4sc") || (this.pageParams.formFactor == "8sc"));
            if (isScaled) {
                var lomLocation = parseInt(nodeNumber)*4 - 4;
                var checkItem =  this._adapterArray[lomLocation];
                if (checkItem && checkItem.type.toString() == "lom") {
                    // LOM configuration for scaled system pattern exists. Remove Item 2 (if any) and return now.
                    console.info(this.declaredClass + ": LOM configuration for scaled system exists. Discarding Slot 2 configuration. nodeNumber=", nodeNumber, "slotNumber=", 2);
                    this._removeIoaItem2(nodeNumber);
                    return false;
                }
            }
            
            // Proceed to set up Item 2.
            var location = parseInt(nodeNumber)*4 - 2;
            if (this._adapterArray[location]) {
                // Verify its under the correct parent.
                var myId = this._adapterArray[location].id;
                if (dojo.some(parentItem.items, function(item) {
                    return item.id == myId;
                })) {
                    return true; // assume all is good
                }
                this._removeIoaItem2(nodeNumber); // Remove now to create under correct parent below.
            }
            if (!this._adapterArray[location]) {
                // Set up "Add button" for IO Adapter 3
                var rowData = {
                    id: "item" + this._itemCount++,
                    slotNumber: 2,
                    nodeNumber: nodeNumber,
                    type: "addIoAdapter",
                    name: (either) ? this.locResources["BUTTON_ADD_ADAPTER_OR_LOM"].replace("%1", "3") : this.locResources["BUTTON_ADD_ADAPTER"].replace("%1", "3"),
                    title: (either) ? this.locResources["BUTTON_ADD_ADAPTER3_OR_LOM_TITLE"] : this.locResources["BUTTON_ADD_ADAPTER3_TITLE"],
                    moduleBay: "1-2",
                    description: this.locResources["EMPTY_SLOT_DESCRIPTION"]
                };
                var parentInfo = {
                    parent: parentItem,
                    attribute: this._childAttr
                };
                this._adapterArray[location] = this._adapterStore.newItem(rowData, parentInfo);
                this._updateGridStore();
                this._prepareCardGraphic(location+1, false);
            }
            return true;
        },

        _removeIoaItem2: function(nodeNumber) {
            // tags:
            // protected
            var location = parseInt(nodeNumber)*4 - 2;
            if (this._adapterArray[location]) {
                this._deleteItem(this._adapterArray[location]);
                this._adapterArray[location] = null;
            }
        },

        _setupIoaItem3: function(parentItem, nodeNumber) {
            // tags:
            // protected
            var location = parseInt(nodeNumber)*4 - 1;
            if (this._adapterArray[location]) {
                // Verify its under the correct parent.
                var myId = this._adapterArray[location].id;
                if (dojo.some(parentItem.items, function(item) {
                    return item.id == myId;
                })) {
                    return true; // assume all is good
                }
                this._removeIoaItem3(nodeNumber); // Remove now to create under correct parent below.
            }
            if (!this._adapterArray[location]) {
                // Set up "Add button" for IO Adapter 4
                var rowData = {
                    id: "item" + this._itemCount++,
                    slotNumber: 3,
                    nodeNumber: nodeNumber,
                    type: "addIoAdapter",
                    name: this.locResources["BUTTON_ADD_ADAPTER"].replace("%1", "4"),
                    title: this.locResources["BUTTON_ADD_ADAPTER4_TITLE"],
                    moduleBay: "3-4",
                    description: this.locResources["EMPTY_SLOT_DESCRIPTION"]
                };
                var parentInfo = {
                    parent: parentItem,
                    attribute: this._childAttr
                };
                this._adapterArray[location] = this._adapterStore.newItem(rowData, parentInfo);
                this._updateGridStore();
                this._prepareCardGraphic(location+1, false);
            }
            return true;
        },

        _removeIoaItem3: function(nodeNumber) {
            // tags:
            // protected
            var location = parseInt(nodeNumber)*4 - 1;
            if (this._adapterArray[location]) {
                this._deleteItem(this._adapterArray[location]);
                this._adapterArray[location] = null;
            }
        },
        
        _updateGridStore: function() {
            // tags:
            // protected
            this._adapterTable.setStore(this._adapterStore);
            this._adapterTable.model.clearCache();
            this._adapterTable.body.refresh().then(this._adapterTable.reexpandRows);
        }

    });
});
