define([ 
        "dojo",
        "dijit",
        "support/panels/Panel",
        "dojo/i18n!./nls/nstStorage",
        "dojo/text!./templates/nstStorage.html",
        "support/widgets/Icon",
        "dojo/data/ItemFileReadStore",
        "dojox/grid/LazyTreeGridStoreModel",
        "dijit/form/SimpleTextarea",
        "dijit/form/Button",
        "dijit/form/NumberSpinner",
        "support/widgets/CapturedSettingsTree"
    ], function(dojo, dijit, page, mri, template) {

    return dojo.declare("support.wizard.pages.nstStorage", [ support.panels.Panel ], {

        // templateString: [protected] String
        // This widget's HTML template, provided.
        templateString: template,

        // locResources: [protected] Object
        // This widget's National Language Source (NLS) object.
        locResources: mri,

        // editMode: boolean
        // Specifies whether the wizard is to execute in edit mode. If "true", the "templateIDNum"
        // attribute is required.
        editMode: false,

        // copyMode: boolean
        // Specifies whether the wizard is to execute in copy mode. If "true", the "templateIDNum"
        // attribute is required.
        copyMode: false,

        // inUse: boolean
        // Specifies whether the wizard is to execute in inUse mode.
        inUse: false,

        // baseClass: [protected] String
        // The root CSS className to be placed on this widget's domNode.
        baseClass: "nstWizard",

        storage: null,
        _storageLabel: null,
        _storageIcon: null,

        // Default Menu Options, Override To Preload Saved Settings
        numDrivesValue: 2,
        diskTypeValue: "None",
        raidLevelValue: "RAID 1",
        storageVolumes: null,

        editTableInit: false, // Only need to load the CapStorage table once in edit/copy mode

        pageParams : null,
        
        // Override this method to perform custom behavior during dijit
        // construction.
        // Common operations for constructor:
        // 1) Initialize non-primitive types (i.e. objects and arrays)
        // 2) Add additional properties needed by succeeding lifecycle methods
        constructor: function() {
            // Borrow Wizard General Page Resources
            this.genResources = dojo.i18n.getLocalization("support.wizard.pages", "nstGeneral", this.lang);
        },

        // When this method is called, all variables inherited from superclasses
        // are "mixed in".
        // Common operations for postMixInProperties
        // 1) Modify or assign values for widget property variables defined in
        // the template HTML file
        postMixInProperties: function() {

            this.inherited(arguments);

        },

        // postCreate() is called after buildRendering(). This is useful to
        // override when
        // you need to access and/or manipulate DOM nodes included with your
        // widget.
        // DOM nodes and widgets with the dojoAttachPoint attribute specified
        // can now be directly
        // accessed as fields on "this".
        // Common operations for postCreate
        // 1) Access and manipulate DOM nodes created in buildRendering()
        // 2) Add new DOM nodes or widgets
        postCreate: function() {
            this.inherited(arguments);
            
            //Connect storage type change to set function
            dojo.connect(this._Specify, "onClick", this, function(){
                this.storage = "Specify";
                this.set("storage", this);
            });

            //Connect storage type change to set function
            dojo.connect(this._KeepExisting, "onClick", this, function(){
                this.storage = "Keep Existing";
                this.set("storage", this);
            });

            //Connect storage type change to set function
            dojo.connect(this._DisableDisk, "onClick", this, function(){
                this.storage = "Disable Disk";
                this.set("storage", this);
            });

            //Connect storage type change to set function
            dojo.connect(this._Clone, "onClick", this, function(){
                this.storage = "Clone";
                this.set("storage", this);
            });

            this._loadTemplateData();            
        },

        _setStorageAttr: function(s) {

            this._storageLabel = s.label;
            this._storageIcon = s.iconClass.normal;
            var tooltip = new dijit.Tooltip({
                connectId: [ "nstStorage" ],
                label: this._getTooltipLabel(),
                position: [ "below" ],
                iconClass: this._storageIcon
            });
            if (this.storage == "Specify") {
                this.curStorageID = "Specify";
                this.specifySettings.setAttribute("style", "display:block");
                this.keepSettings.setAttribute("style", "display:none");
                this.disableSettings.setAttribute("style", "display:none");
                this.capturedSettings.setAttribute("style", "display:none");
                this._specifyStorageConfigSettings(this.copyMode || this.editMode);
                if(this.infoMessage){
                    this.infoMessage.resize();
                }
            }
            else if (this.storage == "Clone") {
                this.curStorageID = "Clone";
                this.specifySettings.setAttribute("style", "display:none");
                this.keepSettings.setAttribute("style", "display:none");
                this.disableSettings.setAttribute("style", "display:none");
                this.capturedSettings.setAttribute("style", "display:block");
                // Go to table loader if in create mode, in create/edit mode it only gets loaded
                // once
                if (!this.editMode && !this.copyMode) {
                    this._capturedStorageConfigSettings();
                }
            } 
            else if (this.storage == "Keep Existing") {
                this.curStorageID = "Keep Existing";
                this.specifySettings.setAttribute("style", "display:none");
                this.keepSettings.setAttribute("style", "display:block");
                this.disableSettings.setAttribute("style", "display:none");
                this.capturedSettings.setAttribute("style", "display:none");
            }
            else if (this.storage == "Disable Disk") {
                this.curStorageID = "Disable Disk";
                this.specifySettings.setAttribute("style", "display:none");
                this.keepSettings.setAttribute("style", "display:none");
                this.disableSettings.setAttribute("style", "display:block");
                this.capturedSettings.setAttribute("style", "display:none");
                
                this._resetInfoMsgDisable();
            }

        },

        _setSelectedConfigAttr: function(c) {
            this._storageLabel = c;
            // console.log(this.declaredClass + ": ", this._storageLabel);
        },

        /*
         * _getStorageIcon : function(){ return "nst" + this.storage + "Icon"; },
         */

        _getTooltipLabel: function() {
            // console.log(this.declaredClass + ": ", this._storageIcon);
            var tooltip = "<table><tr>" +
                "<td style=\"width:75px;height:58px\"></td>" +
                "<td><span style=\"font-weight: bold;\">Select local storage configuration</span><br/>" +
                "<span>" + this.locResources.SELECTED_CONFIG + ": " + this.get("_storageLabel") + "</span></td>" +
                "</tr></table>";
            return tooltip;
        },

        _getTooltipIcon: function() {
            if (this.storage == "KeepExisting") {
                return;
            }
        },

        _setNumDrivesConstraints: function() {
           var constraints = {};
           var disabled = false;

           if(this.pageParams.formFactor == "2sc" || this.pageParams.formFactor == "4sc"
               || this.pageParams.formFactor == "8sc") {
               if (this.raidLevelValue == "RAID 0") {
                   constraints = {
                           min: 2,
                           max: 8
                   };
                   disabled = false;
               }
               else if (this.raidLevelValue == "RAID 1") {
                   constraints = {
                           min: 2,
                           max: 8
                   };
                   disabled = false;
               }
               else if (this.raidLevelValue == "RAID 5") {
                   constraints = {
                           min: 3,
                           max: 8
                   };
                   disabled = false;
               }
               else if (this.raidLevelValue == "RAID 10") {
                   constraints = {
                           min: 4,
                           max: 8
                   };
                   disabled = false;
               }
           }else{
               if (this.raidLevelValue == "RAID 0") {
                   constraints = {
                           min: 2,
                           max: 2
                   };
                   disabled = true;
               }
               else if (this.raidLevelValue == "RAID 1") {
                   constraints = {
                           min: 2,
                           max: 2
                   };
                   disabled = true;
               }
           }
           this.numDrives.set("constraints", constraints);
           this.numDrives.set("disabled", disabled);
       },

       _specifyStorageConfigSettings: function() {
           if(!this.infoMessage){
               
               if(this.CP_RUN_ENVIRONMENT == "fsm"){

                   require([ "idx/oneui/messaging/SingleMessage" ], dojo.hitch(this, function() {

                       this.infoMessage = new idx.oneui.messaging.SingleMessage({
                           title: this.locResources.INFO_MSG,
                           description: this.locResources.INFO_MSG_DETAILS,
                           type: "information",
                           dateFormat: { formatLength: "medium", selector: "time" },
                           showId: false,
                           showAction: false,
                           showRefresh: false,
                           showDetailsLink: false
                       }, this._infoMsg);

                   }));
               
               }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                   require([ "idx/widget/SingleMessage" ], dojo.hitch(this, function() {

                       this.infoMessage = new idx.widget.SingleMessage({
                           title: this.locResources.INFO_MSG,
                           description: this.locResources.INFO_MSG_DETAILS,
                           type: "information",
                           dateFormat: { formatLength: "medium", selector: "time" },
                           showId: false,
                           showAction: false,
                           showRefresh: false,
                           showDetailsLink: false
                       }, this._infoMsg);

                   }));

               }
           }
           if(!this.diskType){
                this.diskType = new dijit.form.Select({
                    id: this.id + "cp_nststorage_disktypeselector",
                    disabled: this.inUse,
                    value: this.diskTypeValue,
                    options: [{
                        label: this.locResources.ANY,
                        value: "None"
                    }, {
                        label: this.locResources.SAS_HDD,
                        value: "SAS-HDD"
                    }, {
                        label: this.locResources.SAS_SDD,
                        value: "SAS-SSD"
                    }, {
                        label: this.locResources.SATA_HDD,
                        value: "SATA-HDD"
                    }, {
                        label: this.locResources.SATA_SDD,
                        value: "SATA-SSD"
                    } ]
                }, this._DiskType);
                dojo.connect(this.diskType, "onChange", this, function(newValue) {
                    this.diskTypeValue = newValue;
                });
            }
            else {
                this.diskType.value = this.diskTypeValue;
            }
            if(!this.raidLevel){
                var raidOptions;
                if(this.pageParams.formFactor == "2sc" || this.pageParams.formFactor == "4sc" 
                    || this.pageParams.formFactor == "8sc") {
                    raidOptions = [
                                   {
                                       label: this.locResources.RAID_0,
                                       value: "RAID 0"
                                   },
                                   {
                                       label: this.locResources.RAID_1,
                                       value: "RAID 1"
                                   }
                                   /* Disabling for GA5 release
                                   {
                                       label: this.locResources.RAID_5,
                                       value: "RAID 5"
                                   },
                                   {
                                       label: this.locResources.RAID_10,
                                       value: "RAID 10"
                                   }*/
                                   
                                ];
                } else {
                    raidOptions = [
                                   {
                                       label: this.locResources.RAID_0,
                                       value: "RAID 0"
                                   },
                                   {
                                       label: this.locResources.RAID_1,
                                       value: "RAID 1"
                                   }                                   
                                ];
                }
                this.raidLevel = new dijit.form.Select({
                    id: this.id + "cp_nststorage_raidlevelselector",
                    disabled: this.inUse,
                    value: this.raidLevelValue,
                    options: raidOptions
                }, this._RAIDLevel);
            
                dojo.connect(this.raidLevel, "onChange", this, function(newValue) {
                    this.raidLevelValue = newValue;

                    // Set the default number of drives value based on the new RAID level.
                    var value = 0;
                    if (this.raidLevelValue == "RAID 0") {
                        value = 2;
                    }
                    else if (this.raidLevelValue == "RAID 1") {
                        value = 2;
                    }
                    else if (this.raidLevelValue == "RAID 5") {
                        value = 3;
                    } else if (this.raidLevelValue == "RAID 10") {
                        value = 4;
                    }
                    this.numDrives.set("value", value);

                    // Set the constraints on number of drives based on the RAID level.
                    this._setNumDrivesConstraints();
                });
            }
            else {
                this.raidLevel.set("value", this.raidLevelValue);
            }
               
            if(!this.numDrives){
                this.numDrives = new dijit.form.NumberSpinner({
                    id: this.id + "cp_nststorage_numdrivespinner",
                    intermediateChanges: true,
                    required: true
                },  this._NumberOfDrives);
                dojo.connect(this.numDrives, "onChange", this, function(newValue) {
                    this.numDrivesValue = newValue;
                    dojo.publish("enableNextButton", this.validate());
                });
            }

            this.numDrives.set("value", this.numDrivesValue);

            // Set the constraints on number of drives based on the RAID level.
            this._setNumDrivesConstraints();
        },

        _capturedStorageConfigSettings: function() {
            /*
             * 99301 In create mode load Storage Volume Info From Backend Based On Server
             * Selected(this.pageParams.baseServer) Or if in edit/copy mode, this.storageVolumes has
             * already been populated with the saved data in _loadTemplateData()
             */
            // Only create new table if name of server used has changed
            if (this.pageParams && this.pageParams.baseServerChanged && !this._Clone.disabled) {
                if (this.pageParams.baseServer && this.pageParams.baseServer.chassisName && this.pageParams.baseServer.serverName) {
                    var chassis = this.pageParams.baseServer.chassisName;
                    var server = this.pageParams.baseServer.serverName;
                    var ff = this._translateFormFactor(this.pageParams.baseServer.formFactor);
                    var type = this._translateTypeName(this.pageParams.baseServer.type);
                    var title = "";
                    if (type == "" || ff == "") {
                        title = this.locResources["CAPTURED_TABLE_TITLE_1"].replace("%1", chassis).replace("%2", server);
                    }
                    else {
                        title = this.locResources["CAPTURED_TABLE_TITLE_2"].replace("%1", chassis).replace("%2", server).replace(
                                "%3", type).replace("%4", ff);
                    }

                    this._capturedStorageTitle.innerHTML = "<b>" + title + "</b>";
                }

                if (!this.capturedGrid) {
                    this.capturedGrid = new support.widgets.CapturedSettingsTree({
                        id: this.id + "cp_nststorage_capturedsettingstree",
                        width: "500px",
                        height: "200px",
                        jsonData: this.storageVolumes
                    }, this._CapturedGrid);
                }
                else {
                    var newStore = new dojo.data.ItemFileReadStore({
                        data: this.capturedGrid._getValidJsonData(this.storageVolumes)
                    });
                    this.capturedGrid._treeTable.grid.setStore(newStore);
                }
                this.pageParams.baseServerChanged = false;
            }
        },

        _loadTemplateData: function() {
            console.log(this.declaredClass + ": ", this.pageParams);

            // Load in storage Selection
            var id = this.pageParams.storageID ? this.pageParams.storageID : "";
            if (id == "Specify") {
                if (this.pageParams.storageAttrs.numDrives) {
                    this.numDrivesValue = this.pageParams.storageAttrs.numDrives;
                }
                if (this.pageParams.storageAttrs.diskType) {
                    this.diskTypeValue = this.pageParams.storageAttrs.diskType;
                }
                if (this.pageParams.storageAttrs.raidLevel) {
                    this.raidLevelValue = this.pageParams.storageAttrs.raidLevel;
                }
                this._Specify.setDisabled(false);
                this._Specify.setSelected(true);
                if(this.editMode){
                    // if in edit mode and the server template is in use,
                    //   disable the other storage options from being clicked
                    if(this.inUse){
                        this._Clone.setDisabled(true);
                        this._KeepExisting.setDisabled(true);
                        this._DisableDisk.setDisabled(true);
                        // this._Reset.setDisabled(true);
                    }
                    else{
                        this._Clone.setDisabled(false);
                        this._KeepExisting.setDisabled(false);
                        this._DisableDisk.setDisabled(false);
                        // this._Reset.setDisabled(false);
                    }
                }
                this.curStorageID = id;
            }
            else if (id == "Clone") {
                if (this.pageParams.storageAttrs.storageVolumes) {
                    this.storageVolumes = this.pageParams.storageAttrs.storageVolumes;
                }
                this._Clone.setDisabled(false);
                this._Clone.setSelected(true);
                if(this.editMode){
                    // if in edit mode and the server template is in use,
                    //   disable the other storage options from being clicked
                    if(this.inUse){
                        this._KeepExisting.setDisabled(true);
                        this._Specify.setDisabled(true);
                        this._DisableDisk.setDisabled(true);
                        // this._Reset.setDisabled(true);
                    }
                    else{
                        this._KeepExisting.setDisabled(false);
                        this._Specify.setDisabled(false);
                        this._DisableDisk.setDisabled(false);
                        // this._Reset.setDisabled(false);
                    }
                }
                this.curStorageID = id;

            }
            else if (id == "Keep Existing") {
               this._KeepExisting.setDisabled(false);
               this._KeepExisting.setSelected(true);
               if(this.editMode){
                   // if in edit mode and the server template is in use,
                   //   disable the other storage options from being clicked
                   if(this.inUse){
                       this._Clone.setDisabled(true); 
                       this._Specify.setDisabled(true);
                       this._DisableDisk.setDisabled(true);
                       //this._Reset.setDisabled(true);
                   }
                   else {
                       this._Clone.setDisabled(false); 
                       this._Specify.setDisabled(false);
                       this._DisableDisk.setDisabled(false);
                       //this._Reset.setDisabled(false);
                   }
               }
               this.curStorageID = id; 
            }
            else if (id == "Disable Disk") {
                this._DisableDisk.setDisabled(false);
                this._DisableDisk.setSelected(true);
                if(this.editMode){
                    // if in edit mode and the server template is in use,
                    //   disable the other storage options from being clicked
                    if(this.inUse){
                        this._Clone.setDisabled(true);
                        // this._KeepExisting.setDisabled(true);
                        this._Specify.setDisabled(true);
                        // this._Reset.setDisabled(true);
                    }
                    else{
                        this._Clone.setDisabled(false);
                        // this._KeepExisting.setDisabled(false);
                        this._Specify.setDisabled(false);
                        // this._Reset.setDisabled(false);
                    }
                    this.curStorageID = id;
                }
            }
            /*
             * else if (id == "Reset") { this._Reset.setSelected(true);
             * this._Clone.setDisabled(true); this._KeepExisting.setDisabled(true);
             * this._Specify.setDisabled(true); this._DisableDisk.setDisabled(true);
             * this.curStorageID = id; }
             */
            else {
                return;
            }

        },

        AfterPageSelected: function() {

            // Pre-load storage volume settings if clone is pre-selected
            // Otherwise table is loaded upon clicking clone
            if (this._Clone.selected) {
                if ((this.editMode || this.copyMode) && !this.editTableInit) {
                    this.editTableInit = true;
                    this.pageParams.baseServerChanged = true; // Set true, so table loads
                }
                this._capturedStorageConfigSettings();
            }
            if (this._Clone.get("disabled"))
                this._Clone.set("title", this.locResources.CLONE_TITLE_DISABLED);
            else
                this._Clone.set("title", this.locResources.CLONE_TITLE);
            
            // Validate page input and enable/disable "Next" button of wizard accordingly.
            dojo.publish("enableNextButton", this.validate());

        },

        // "Translate" from internal naming conventions to translation safe, properly named strings
        _translateTypeName: function(type) {
            if (type) {
                if (type == "sysx") {
                    return this.genResources.SYSX;
                }
                else if (type == "power") {
                    return this.genResources.POWER;
                }
            }
            return "";
        },

        // "Translate" from internal naming conventions to translation safe, properly named strings
        _translateFormFactor: function(ff) {
            if (ff) {
                if (ff == "2s") {
                    return this.resources["FF_BAY"].replace("%1", "1");
                }
                else if (ff == "4s") {
                    return this.resources["FF_BAY"].replace("%1", "2");
                }
                else if (ff == "pxu") {
                    return this.resources["FF_PXU"].replace("%1", "2");
                }
            }
            return "";
        },
        
        _resetInfoMsgDisable: function() {
            if(!this.infoMessageDisable && this.pageParams.formFactor == "2s"){
                
                if(this.CP_RUN_ENVIRONMENT == "fsm"){

                    require([ "idx/oneui/messaging/SingleMessage" ], dojo.hitch(this, function() {

                        this.infoMessageDisable = new idx.oneui.messaging.SingleMessage({
                            title: this.locResources.DISABLE_INFO_MSG,
                            description: this.locResources.DISABLE_MSG_DETAILS,
                            type: "information",
                            dateFormat: { formatLength: "medium", selector: "time" },
                            showId: false,
                            showAction: false,
                            showRefresh: false,
                            showDetailsLink: false
                        });
                    
                    }));
                
                }else if(this.CP_RUN_ENVIRONMENT == "augusta"){

                    require([ "idx/widget/SingleMessage" ], dojo.hitch(this, function() {

                        this.infoMessageDisable = new idx.widget.SingleMessage({
                            title: this.locResources.DISABLE_INFO_MSG,
                            description: this.locResources.DISABLE_MSG_DETAILS,
                            type: "information",
                            dateFormat: { formatLength: "medium", selector: "time" },
                            showId: false,
                            showAction: false,
                            showRefresh: false,
                            showDetailsLink: false
                        });
                    
                    }));

                }
            }
            
            if(this.infoMessageDisable){
                if(this.pageParams.formFactor == "2s") {
                    this.infoMessageDisable.placeAt(this._infoMsgDisable);
                    this._infoMsgDisable.style.display = "block";
                    this.infoMessageDisable.resize();
                } else {
                    this._infoMsgDisable.style.display = "none";
                }
            }
        },
        
        refreshRAIDOptions: function(){
            this._resetInfoMsgDisable();
            if(this.raidLevel) {
                var raidOptions;
                if(this.pageParams.formFactor == "2sc" || this.pageParams.formFactor == "4sc"
                    || this.pageParams.formFactor == "8sc") {
                    //this.raidLevel.addOption(["RAID 5", "RAID 10"]);
                    raidOptions = [
                                   {
                                       label: this.locResources.RAID_0,
                                       value: "RAID 0"
                                   },
                                   {
                                       label: this.locResources.RAID_1,
                                       value: "RAID 1"
                                   }
                                   /*Disabling for GA5 release
                                   {
                                       label: this.locResources.RAID_5,
                                       value: "RAID 5"
                                   },
                                   {
                                       label: this.locResources.RAID_10,
                                       value: "RAID 10"
                                   }*/
                                   
                                ];
                } else {
                    //this.raidLevel.removeOption(["RAID 5", "RAID 10"]);
                    raidOptions = [
                                   {
                                       label: this.locResources.RAID_0,
                                       value: "RAID 0"
                                   },
                                   {
                                       label: this.locResources.RAID_1,
                                       value: "RAID 1"
                                   }
                                   
                                ];
                }
                this.raidLevel.set("options", raidOptions);
                this._setNumDrivesConstraints();
            } else {
                this._specifyStorageConfigSettings();
            }
            
        },
        
        validate: function() {
            if(this.storage == "Specify") {
                if(this.numDrives.isValid()) {
                    return true;
                } else {
                    return false;
                }
            } else {
                return true;
            }
        }
    });
});